/*  Prototype JavaScript framework, version 1.7
 *  (c) 2005-2010 Sam Stephenson
 *
 *  Prototype is freely distributable under the terms of an MIT-style license.
 *  For details, see the Prototype web site: http://www.prototypejs.org/
 *
 *--------------------------------------------------------------------------*/

var Prototype = {

  Version: '1.7',

  Browser: (function(){
    var ua = navigator.userAgent;
    var isOpera = Object.prototype.toString.call(window.opera) == '[object Opera]';
    return {
      IE:             !!window.attachEvent && !isOpera,
      Opera:          isOpera,
      WebKit:         ua.indexOf('AppleWebKit/') > -1,
      Gecko:          ua.indexOf('Gecko') > -1 && ua.indexOf('KHTML') === -1,
      MobileSafari:   /Apple.*Mobile/.test(ua)
    }
  })(),

  BrowserFeatures: {
    XPath: !!document.evaluate,

    SelectorsAPI: !!document.querySelector,

    ElementExtensions: (function() {
      var constructor = window.Element || window.HTMLElement;
      return !!(constructor && constructor.prototype);
    })(),
    SpecificElementExtensions: (function() {
      if (typeof window.HTMLDivElement !== 'undefined')
        return true;

      var div = document.createElement('div'),
          form = document.createElement('form'),
          isSupported = false;

      if (div['__proto__'] && (div['__proto__'] !== form['__proto__'])) {
        isSupported = true;
      }

      div = form = null;

      return isSupported;
    })()
  },

  ScriptFragment: '<script[^>]*>([\\S\\s]*?)<\/script>',
  JSONFilter: /^\/\*-secure-([\s\S]*)\*\/\s*$/,

  emptyFunction: function() { },

  K: function(x) { return x }
};

if (Prototype.Browser.MobileSafari)
  Prototype.BrowserFeatures.SpecificElementExtensions = false;
/* Based on Alex Arnell's inheritance implementation. */

var Class = (function() {

  var IS_DONTENUM_BUGGY = (function(){
    for (var p in { toString: 1 }) {
      if (p === 'toString') return false;
    }
    return true;
  })();

  function subclass() {};
  function create() {
    var parent = null, properties = $A(arguments);
    if (Object.isFunction(properties[0]))
      parent = properties.shift();

    function klass() {
      this.initialize.apply(this, arguments);
    }

    Object.extend(klass, Class.Methods);
    klass.superclass = parent;
    klass.subclasses = [];

    if (parent) {
      subclass.prototype = parent.prototype;
      klass.prototype = new subclass;
      parent.subclasses.push(klass);
    }

    for (var i = 0, length = properties.length; i < length; i++)
      klass.addMethods(properties[i]);

    if (!klass.prototype.initialize)
      klass.prototype.initialize = Prototype.emptyFunction;

    klass.prototype.constructor = klass;
    return klass;
  }

  function addMethods(source) {
    var ancestor   = this.superclass && this.superclass.prototype,
        properties = Object.keys(source);

    if (IS_DONTENUM_BUGGY) {
      if (source.toString != Object.prototype.toString)
        properties.push("toString");
      if (source.valueOf != Object.prototype.valueOf)
        properties.push("valueOf");
    }

    for (var i = 0, length = properties.length; i < length; i++) {
      var property = properties[i], value = source[property];
      if (ancestor && Object.isFunction(value) &&
          value.argumentNames()[0] == "$super") {
        var method = value;
        value = (function(m) {
          return function() { return ancestor[m].apply(this, arguments); };
        })(property).wrap(method);

        value.valueOf = method.valueOf.bind(method);
        value.toString = method.toString.bind(method);
      }
      this.prototype[property] = value;
    }

    return this;
  }

  return {
    create: create,
    Methods: {
      addMethods: addMethods
    }
  };
})();
(function() {

  var _toString = Object.prototype.toString,
      NULL_TYPE = 'Null',
      UNDEFINED_TYPE = 'Undefined',
      BOOLEAN_TYPE = 'Boolean',
      NUMBER_TYPE = 'Number',
      STRING_TYPE = 'String',
      OBJECT_TYPE = 'Object',
      FUNCTION_CLASS = '[object Function]',
      BOOLEAN_CLASS = '[object Boolean]',
      NUMBER_CLASS = '[object Number]',
      STRING_CLASS = '[object String]',
      ARRAY_CLASS = '[object Array]',
      DATE_CLASS = '[object Date]',
      NATIVE_JSON_STRINGIFY_SUPPORT = window.JSON &&
        typeof JSON.stringify === 'function' &&
        JSON.stringify(0) === '0' &&
        typeof JSON.stringify(Prototype.K) === 'undefined';

  function Type(o) {
    switch(o) {
      case null: return NULL_TYPE;
      case (void 0): return UNDEFINED_TYPE;
    }
    var type = typeof o;
    switch(type) {
      case 'boolean': return BOOLEAN_TYPE;
      case 'number':  return NUMBER_TYPE;
      case 'string':  return STRING_TYPE;
    }
    return OBJECT_TYPE;
  }

  function extend(destination, source) {
    for (var property in source)
      destination[property] = source[property];
    return destination;
  }

  function inspect(object) {
    try {
      if (isUndefined(object)) return 'undefined';
      if (object === null) return 'null';
      return object.inspect ? object.inspect() : String(object);
    } catch (e) {
      if (e instanceof RangeError) return '...';
      throw e;
    }
  }

  function toJSON(value) {
    return Str('', { '': value }, []);
  }

  function Str(key, holder, stack) {
    var value = holder[key],
        type = typeof value;

    if (Type(value) === OBJECT_TYPE && typeof value.toJSON === 'function') {
      value = value.toJSON(key);
    }

    var _class = _toString.call(value);

    switch (_class) {
      case NUMBER_CLASS:
      case BOOLEAN_CLASS:
      case STRING_CLASS:
        value = value.valueOf();
    }

    switch (value) {
      case null: return 'null';
      case true: return 'true';
      case false: return 'false';
    }

    type = typeof value;
    switch (type) {
      case 'string':
        return value.inspect(true);
      case 'number':
        return isFinite(value) ? String(value) : 'null';
      case 'object':

        for (var i = 0, length = stack.length; i < length; i++) {
          if (stack[i] === value) { throw new TypeError(); }
        }
        stack.push(value);

        var partial = [];
        if (_class === ARRAY_CLASS) {
          for (var i = 0, length = value.length; i < length; i++) {
            var str = Str(i, value, stack);
            partial.push(typeof str === 'undefined' ? 'null' : str);
          }
          partial = '[' + partial.join(',') + ']';
        } else {
          var keys = Object.keys(value);
          for (var i = 0, length = keys.length; i < length; i++) {
            var key = keys[i], str = Str(key, value, stack);
            if (typeof str !== "undefined") {
               partial.push(key.inspect(true)+ ':' + str);
             }
          }
          partial = '{' + partial.join(',') + '}';
        }
        stack.pop();
        return partial;
    }
  }

  function stringify(object) {
    return JSON.stringify(object);
  }

  function toQueryString(object) {
    return $H(object).toQueryString();
  }

  function toHTML(object) {
    return object && object.toHTML ? object.toHTML() : String.interpret(object);
  }

  function keys(object) {
    if (Type(object) !== OBJECT_TYPE) { throw new TypeError(); }
    var results = [];
    for (var property in object) {
      if (object.hasOwnProperty(property)) {
        results.push(property);
      }
    }
    return results;
  }

  function values(object) {
    var results = [];
    for (var property in object)
      results.push(object[property]);
    return results;
  }

  function clone(object) {
    return extend({ }, object);
  }

  function isElement(object) {
    return !!(object && object.nodeType == 1);
  }

  function isArray(object) {
    return _toString.call(object) === ARRAY_CLASS;
  }

  var hasNativeIsArray = (typeof Array.isArray == 'function')
    && Array.isArray([]) && !Array.isArray({});

  if (hasNativeIsArray) {
    isArray = Array.isArray;
  }

  function isHash(object) {
    return object instanceof Hash;
  }

  function isFunction(object) {
    return _toString.call(object) === FUNCTION_CLASS;
  }

  function isString(object) {
    return _toString.call(object) === STRING_CLASS;
  }

  function isNumber(object) {
    return _toString.call(object) === NUMBER_CLASS;
  }

  function isDate(object) {
    return _toString.call(object) === DATE_CLASS;
  }

  function isUndefined(object) {
    return typeof object === "undefined";
  }

  extend(Object, {
    extend:        extend,
    inspect:       inspect,
    toJSON:        NATIVE_JSON_STRINGIFY_SUPPORT ? stringify : toJSON,
    toQueryString: toQueryString,
    toHTML:        toHTML,
    keys:          Object.keys || keys,
    values:        values,
    clone:         clone,
    isElement:     isElement,
    isArray:       isArray,
    isHash:        isHash,
    isFunction:    isFunction,
    isString:      isString,
    isNumber:      isNumber,
    isDate:        isDate,
    isUndefined:   isUndefined
  });
})();
Object.extend(Function.prototype, (function() {
  var slice = Array.prototype.slice;

  function update(array, args) {
    var arrayLength = array.length, length = args.length;
    while (length--) array[arrayLength + length] = args[length];
    return array;
  }

  function merge(array, args) {
    array = slice.call(array, 0);
    return update(array, args);
  }

  function argumentNames() {
    var names = this.toString().match(/^[\s\(]*function[^(]*\(([^)]*)\)/)[1]
      .replace(/\/\/.*?[\r\n]|\/\*(?:.|[\r\n])*?\*\//g, '')
      .replace(/\s+/g, '').split(',');
    return names.length == 1 && !names[0] ? [] : names;
  }

  function bind(context) {
    if (arguments.length < 2 && Object.isUndefined(arguments[0])) return this;
    var __method = this, args = slice.call(arguments, 1);
    return function() {
      var a = merge(args, arguments);
      return __method.apply(context, a);
    }
  }

  function bindAsEventListener(context) {
    var __method = this, args = slice.call(arguments, 1);
    return function(event) {
      var a = update([event || window.event], args);
      return __method.apply(context, a);
    }
  }

  function curry() {
    if (!arguments.length) return this;
    var __method = this, args = slice.call(arguments, 0);
    return function() {
      var a = merge(args, arguments);
      return __method.apply(this, a);
    }
  }

  function delay(timeout) {
    var __method = this, args = slice.call(arguments, 1);
    timeout = timeout * 1000;
    return window.setTimeout(function() {
      return __method.apply(__method, args);
    }, timeout);
  }

  function defer() {
    var args = update([0.01], arguments);
    return this.delay.apply(this, args);
  }

  function wrap(wrapper) {
    var __method = this;
    return function() {
      var a = update([__method.bind(this)], arguments);
      return wrapper.apply(this, a);
    }
  }

  function methodize() {
    if (this._methodized) return this._methodized;
    var __method = this;
    return this._methodized = function() {
      var a = update([this], arguments);
      return __method.apply(null, a);
    };
  }

  return {
    argumentNames:       argumentNames,
    bind:                bind,
    bindAsEventListener: bindAsEventListener,
    curry:               curry,
    delay:               delay,
    defer:               defer,
    wrap:                wrap,
    methodize:           methodize
  }
})());



(function(proto) {


  function toISOString() {
    return this.getUTCFullYear() + '-' +
      (this.getUTCMonth() + 1).toPaddedString(2) + '-' +
      this.getUTCDate().toPaddedString(2) + 'T' +
      this.getUTCHours().toPaddedString(2) + ':' +
      this.getUTCMinutes().toPaddedString(2) + ':' +
      this.getUTCSeconds().toPaddedString(2) + 'Z';
  }


  function toJSON() {
    return this.toISOString();
  }

  if (!proto.toISOString) proto.toISOString = toISOString;
  if (!proto.toJSON) proto.toJSON = toJSON;

})(Date.prototype);


RegExp.prototype.match = RegExp.prototype.test;

RegExp.escape = function(str) {
  return String(str).replace(/([.*+?^=!:${}()|[\]\/\\])/g, '\\$1');
};
var PeriodicalExecuter = Class.create({
  initialize: function(callback, frequency) {
    this.callback = callback;
    this.frequency = frequency;
    this.currentlyExecuting = false;

    this.registerCallback();
  },

  registerCallback: function() {
    this.timer = setInterval(this.onTimerEvent.bind(this), this.frequency * 1000);
  },

  execute: function() {
    this.callback(this);
  },

  stop: function() {
    if (!this.timer) return;
    clearInterval(this.timer);
    this.timer = null;
  },

  onTimerEvent: function() {
    if (!this.currentlyExecuting) {
      try {
        this.currentlyExecuting = true;
        this.execute();
        this.currentlyExecuting = false;
      } catch(e) {
        this.currentlyExecuting = false;
        throw e;
      }
    }
  }
});
Object.extend(String, {
  interpret: function(value) {
    return value == null ? '' : String(value);
  },
  specialChar: {
    '\b': '\\b',
    '\t': '\\t',
    '\n': '\\n',
    '\f': '\\f',
    '\r': '\\r',
    '\\': '\\\\'
  }
});

Object.extend(String.prototype, (function() {
  var NATIVE_JSON_PARSE_SUPPORT = window.JSON &&
    typeof JSON.parse === 'function' &&
    JSON.parse('{"test": true}').test;

  function prepareReplacement(replacement) {
    if (Object.isFunction(replacement)) return replacement;
    var template = new Template(replacement);
    return function(match) { return template.evaluate(match) };
  }

  function gsub(pattern, replacement) {
    var result = '', source = this, match;
    replacement = prepareReplacement(replacement);

    if (Object.isString(pattern))
      pattern = RegExp.escape(pattern);

    if (!(pattern.length || pattern.source)) {
      replacement = replacement('');
      return replacement + source.split('').join(replacement) + replacement;
    }

    while (source.length > 0) {
      if (match = source.match(pattern)) {
        result += source.slice(0, match.index);
        result += String.interpret(replacement(match));
        source  = source.slice(match.index + match[0].length);
      } else {
        result += source, source = '';
      }
    }
    return result;
  }

  function sub(pattern, replacement, count) {
    replacement = prepareReplacement(replacement);
    count = Object.isUndefined(count) ? 1 : count;

    return this.gsub(pattern, function(match) {
      if (--count < 0) return match[0];
      return replacement(match);
    });
  }

  function scan(pattern, iterator) {
    this.gsub(pattern, iterator);
    return String(this);
  }

  function truncate(length, truncation) {
    length = length || 30;
    truncation = Object.isUndefined(truncation) ? '...' : truncation;
    return this.length > length ?
      this.slice(0, length - truncation.length) + truncation : String(this);
  }

  function strip() {
    return this.replace(/^\s+/, '').replace(/\s+$/, '');
  }

  function stripTags() {
    return this.replace(/<\w+(\s+("[^"]*"|'[^']*'|[^>])+)?>|<\/\w+>/gi, '');
  }

  function stripScripts() {
    return this.replace(new RegExp(Prototype.ScriptFragment, 'img'), '');
  }

  function extractScripts() {
    var matchAll = new RegExp(Prototype.ScriptFragment, 'img'),
        matchOne = new RegExp(Prototype.ScriptFragment, 'im');
    return (this.match(matchAll) || []).map(function(scriptTag) {
      return (scriptTag.match(matchOne) || ['', ''])[1];
    });
  }

  function evalScripts() {
    return this.extractScripts().map(function(script) { return eval(script) });
  }

  function escapeHTML() {
    return this.replace(/&/g,'&amp;').replace(/</g,'&lt;').replace(/>/g,'&gt;');
  }

  function unescapeHTML() {
    return this.stripTags().replace(/&lt;/g,'<').replace(/&gt;/g,'>').replace(/&amp;/g,'&');
  }


  function toQueryParams(separator) {
    var match = this.strip().match(/([^?#]*)(#.*)?$/);
    if (!match) return { };

    return match[1].split(separator || '&').inject({ }, function(hash, pair) {
      if ((pair = pair.split('='))[0]) {
        var key = decodeURIComponent(pair.shift()),
            value = pair.length > 1 ? pair.join('=') : pair[0];

        if (value != undefined) value = decodeURIComponent(value);

        if (key in hash) {
          if (!Object.isArray(hash[key])) hash[key] = [hash[key]];
          hash[key].push(value);
        }
        else hash[key] = value;
      }
      return hash;
    });
  }

  function toArray() {
    return this.split('');
  }

  function succ() {
    return this.slice(0, this.length - 1) +
      String.fromCharCode(this.charCodeAt(this.length - 1) + 1);
  }

  function times(count) {
    return count < 1 ? '' : new Array(count + 1).join(this);
  }

  function camelize() {
    return this.replace(/-+(.)?/g, function(match, chr) {
      return chr ? chr.toUpperCase() : '';
    });
  }

  function capitalize() {
    return this.charAt(0).toUpperCase() + this.substring(1).toLowerCase();
  }

  function underscore() {
    return this.replace(/::/g, '/')
               .replace(/([A-Z]+)([A-Z][a-z])/g, '$1_$2')
               .replace(/([a-z\d])([A-Z])/g, '$1_$2')
               .replace(/-/g, '_')
               .toLowerCase();
  }

  function dasherize() {
    return this.replace(/_/g, '-');
  }

  function inspect(useDoubleQuotes) {
    var escapedString = this.replace(/[\x00-\x1f\\]/g, function(character) {
      if (character in String.specialChar) {
        return String.specialChar[character];
      }
      return '\\u00' + character.charCodeAt().toPaddedString(2, 16);
    });
    if (useDoubleQuotes) return '"' + escapedString.replace(/"/g, '\\"') + '"';
    return "'" + escapedString.replace(/'/g, '\\\'') + "'";
  }

  function unfilterJSON(filter) {
    return this.replace(filter || Prototype.JSONFilter, '$1');
  }

  function isJSON() {
    var str = this;
    if (str.blank()) return false;
    str = str.replace(/\\(?:["\\\/bfnrt]|u[0-9a-fA-F]{4})/g, '@');
    str = str.replace(/"[^"\\\n\r]*"|true|false|null|-?\d+(?:\.\d*)?(?:[eE][+\-]?\d+)?/g, ']');
    str = str.replace(/(?:^|:|,)(?:\s*\[)+/g, '');
    return (/^[\],:{}\s]*$/).test(str);
  }

  function evalJSON(sanitize) {
    var json = this.unfilterJSON(),
        cx = /[\u0000\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/g;
    if (cx.test(json)) {
      json = json.replace(cx, function (a) {
        return '\\u' + ('0000' + a.charCodeAt(0).toString(16)).slice(-4);
      });
    }
    try {
      if (!sanitize || json.isJSON()) return eval('(' + json + ')');
    } catch (e) { }
    throw new SyntaxError('Badly formed JSON string: ' + this.inspect());
  }

  function parseJSON() {
    var json = this.unfilterJSON();
    return JSON.parse(json);
  }

  function include(pattern) {
    return this.indexOf(pattern) > -1;
  }

  function startsWith(pattern) {
    return this.lastIndexOf(pattern, 0) === 0;
  }

  function endsWith(pattern) {
    var d = this.length - pattern.length;
    return d >= 0 && this.indexOf(pattern, d) === d;
  }

  function empty() {
    return this == '';
  }

  function blank() {
    return /^\s*$/.test(this);
  }

  function interpolate(object, pattern) {
    return new Template(this, pattern).evaluate(object);
  }

  return {
    gsub:           gsub,
    sub:            sub,
    scan:           scan,
    truncate:       truncate,
    strip:          String.prototype.trim || strip,
    stripTags:      stripTags,
    stripScripts:   stripScripts,
    extractScripts: extractScripts,
    evalScripts:    evalScripts,
    escapeHTML:     escapeHTML,
    unescapeHTML:   unescapeHTML,
    toQueryParams:  toQueryParams,
    parseQuery:     toQueryParams,
    toArray:        toArray,
    succ:           succ,
    times:          times,
    camelize:       camelize,
    capitalize:     capitalize,
    underscore:     underscore,
    dasherize:      dasherize,
    inspect:        inspect,
    unfilterJSON:   unfilterJSON,
    isJSON:         isJSON,
    evalJSON:       NATIVE_JSON_PARSE_SUPPORT ? parseJSON : evalJSON,
    include:        include,
    startsWith:     startsWith,
    endsWith:       endsWith,
    empty:          empty,
    blank:          blank,
    interpolate:    interpolate
  };
})());

var Template = Class.create({
  initialize: function(template, pattern) {
    this.template = template.toString();
    this.pattern = pattern || Template.Pattern;
  },

  evaluate: function(object) {
    if (object && Object.isFunction(object.toTemplateReplacements))
      object = object.toTemplateReplacements();

    return this.template.gsub(this.pattern, function(match) {
      if (object == null) return (match[1] + '');

      var before = match[1] || '';
      if (before == '\\') return match[2];

      var ctx = object, expr = match[3],
          pattern = /^([^.[]+|\[((?:.*?[^\\])?)\])(\.|\[|$)/;

      match = pattern.exec(expr);
      if (match == null) return before;

      while (match != null) {
        var comp = match[1].startsWith('[') ? match[2].replace(/\\\\]/g, ']') : match[1];
        ctx = ctx[comp];
        if (null == ctx || '' == match[3]) break;
        expr = expr.substring('[' == match[3] ? match[1].length : match[0].length);
        match = pattern.exec(expr);
      }

      return before + String.interpret(ctx);
    });
  }
});
Template.Pattern = /(^|.|\r|\n)(#\{(.*?)\})/;

var $break = { };

var Enumerable = (function() {
  function each(iterator, context) {
    var index = 0;
    try {
      this._each(function(value) {
        iterator.call(context, value, index++);
      });
    } catch (e) {
      if (e != $break) throw e;
    }
    return this;
  }

  function eachSlice(number, iterator, context) {
    var index = -number, slices = [], array = this.toArray();
    if (number < 1) return array;
    while ((index += number) < array.length)
      slices.push(array.slice(index, index+number));
    return slices.collect(iterator, context);
  }

  function all(iterator, context) {
    iterator = iterator || Prototype.K;
    var result = true;
    this.each(function(value, index) {
      result = result && !!iterator.call(context, value, index);
      if (!result) throw $break;
    });
    return result;
  }

  function any(iterator, context) {
    iterator = iterator || Prototype.K;
    var result = false;
    this.each(function(value, index) {
      if (result = !!iterator.call(context, value, index))
        throw $break;
    });
    return result;
  }

  function collect(iterator, context) {
    iterator = iterator || Prototype.K;
    var results = [];
    this.each(function(value, index) {
      results.push(iterator.call(context, value, index));
    });
    return results;
  }

  function detect(iterator, context) {
    var result;
    this.each(function(value, index) {
      if (iterator.call(context, value, index)) {
        result = value;
        throw $break;
      }
    });
    return result;
  }

  function findAll(iterator, context) {
    var results = [];
    this.each(function(value, index) {
      if (iterator.call(context, value, index))
        results.push(value);
    });
    return results;
  }

  function grep(filter, iterator, context) {
    iterator = iterator || Prototype.K;
    var results = [];

    if (Object.isString(filter))
      filter = new RegExp(RegExp.escape(filter));

    this.each(function(value, index) {
      if (filter.match(value))
        results.push(iterator.call(context, value, index));
    });
    return results;
  }

  function include(object) {
    if (Object.isFunction(this.indexOf))
      if (this.indexOf(object) != -1) return true;

    var found = false;
    this.each(function(value) {
      if (value == object) {
        found = true;
        throw $break;
      }
    });
    return found;
  }

  function inGroupsOf(number, fillWith) {
    fillWith = Object.isUndefined(fillWith) ? null : fillWith;
    return this.eachSlice(number, function(slice) {
      while(slice.length < number) slice.push(fillWith);
      return slice;
    });
  }

  function inject(memo, iterator, context) {
    this.each(function(value, index) {
      memo = iterator.call(context, memo, value, index);
    });
    return memo;
  }

  function invoke(method) {
    var args = $A(arguments).slice(1);
    return this.map(function(value) {
      return value[method].apply(value, args);
    });
  }

  function max(iterator, context) {
    iterator = iterator || Prototype.K;
    var result;
    this.each(function(value, index) {
      value = iterator.call(context, value, index);
      if (result == null || value >= result)
        result = value;
    });
    return result;
  }

  function min(iterator, context) {
    iterator = iterator || Prototype.K;
    var result;
    this.each(function(value, index) {
      value = iterator.call(context, value, index);
      if (result == null || value < result)
        result = value;
    });
    return result;
  }

  function partition(iterator, context) {
    iterator = iterator || Prototype.K;
    var trues = [], falses = [];
    this.each(function(value, index) {
      (iterator.call(context, value, index) ?
        trues : falses).push(value);
    });
    return [trues, falses];
  }

  function pluck(property) {
    var results = [];
    this.each(function(value) {
      results.push(value[property]);
    });
    return results;
  }

  function reject(iterator, context) {
    var results = [];
    this.each(function(value, index) {
      if (!iterator.call(context, value, index))
        results.push(value);
    });
    return results;
  }

  function sortBy(iterator, context) {
    return this.map(function(value, index) {
      return {
        value: value,
        criteria: iterator.call(context, value, index)
      };
    }).sort(function(left, right) {
      var a = left.criteria, b = right.criteria;
      return a < b ? -1 : a > b ? 1 : 0;
    }).pluck('value');
  }

  function toArray() {
    return this.map();
  }

  function zip() {
    var iterator = Prototype.K, args = $A(arguments);
    if (Object.isFunction(args.last()))
      iterator = args.pop();

    var collections = [this].concat(args).map($A);
    return this.map(function(value, index) {
      return iterator(collections.pluck(index));
    });
  }

  function size() {
    return this.toArray().length;
  }

  function inspect() {
    return '#<Enumerable:' + this.toArray().inspect() + '>';
  }









  return {
    each:       each,
    eachSlice:  eachSlice,
    all:        all,
    every:      all,
    any:        any,
    some:       any,
    collect:    collect,
    map:        collect,
    detect:     detect,
    findAll:    findAll,
    select:     findAll,
    filter:     findAll,
    grep:       grep,
    include:    include,
    member:     include,
    inGroupsOf: inGroupsOf,
    inject:     inject,
    invoke:     invoke,
    max:        max,
    min:        min,
    partition:  partition,
    pluck:      pluck,
    reject:     reject,
    sortBy:     sortBy,
    toArray:    toArray,
    entries:    toArray,
    zip:        zip,
    size:       size,
    inspect:    inspect,
    find:       detect
  };
})();

function $A(iterable) {
  if (!iterable) return [];
  if ('toArray' in Object(iterable)) return iterable.toArray();
  var length = iterable.length || 0, results = new Array(length);
  while (length--) results[length] = iterable[length];
  return results;
}


function $w(string) {
  if (!Object.isString(string)) return [];
  string = string.strip();
  return string ? string.split(/\s+/) : [];
}

Array.from = $A;


(function() {
  var arrayProto = Array.prototype,
      slice = arrayProto.slice,
      _each = arrayProto.forEach; // use native browser JS 1.6 implementation if available

  function each(iterator, context) {
    for (var i = 0, length = this.length >>> 0; i < length; i++) {
      if (i in this) iterator.call(context, this[i], i, this);
    }
  }
  if (!_each) _each = each;

  function clear() {
    this.length = 0;
    return this;
  }

  function first() {
    return this[0];
  }

  function last() {
    return this[this.length - 1];
  }

  function compact() {
    return this.select(function(value) {
      return value != null;
    });
  }

  function flatten() {
    return this.inject([], function(array, value) {
      if (Object.isArray(value))
        return array.concat(value.flatten());
      array.push(value);
      return array;
    });
  }

  function without() {
    var values = slice.call(arguments, 0);
    return this.select(function(value) {
      return !values.include(value);
    });
  }

  function reverse(inline) {
    return (inline === false ? this.toArray() : this)._reverse();
  }

  function uniq(sorted) {
    return this.inject([], function(array, value, index) {
      if (0 == index || (sorted ? array.last() != value : !array.include(value)))
        array.push(value);
      return array;
    });
  }

  function intersect(array) {
    return this.uniq().findAll(function(item) {
      return array.detect(function(value) { return item === value });
    });
  }


  function clone() {
    return slice.call(this, 0);
  }

  function size() {
    return this.length;
  }

  function inspect() {
    return '[' + this.map(Object.inspect).join(', ') + ']';
  }

  function indexOf(item, i) {
    i || (i = 0);
    var length = this.length;
    if (i < 0) i = length + i;
    for (; i < length; i++)
      if (this[i] === item) return i;
    return -1;
  }

  function lastIndexOf(item, i) {
    i = isNaN(i) ? this.length : (i < 0 ? this.length + i : i) + 1;
    var n = this.slice(0, i).reverse().indexOf(item);
    return (n < 0) ? n : i - n - 1;
  }

  function concat() {
    var array = slice.call(this, 0), item;
    for (var i = 0, length = arguments.length; i < length; i++) {
      item = arguments[i];
      if (Object.isArray(item) && !('callee' in item)) {
        for (var j = 0, arrayLength = item.length; j < arrayLength; j++)
          array.push(item[j]);
      } else {
        array.push(item);
      }
    }
    return array;
  }

  Object.extend(arrayProto, Enumerable);

  if (!arrayProto._reverse)
    arrayProto._reverse = arrayProto.reverse;

  Object.extend(arrayProto, {
    _each:     _each,
    clear:     clear,
    first:     first,
    last:      last,
    compact:   compact,
    flatten:   flatten,
    without:   without,
    reverse:   reverse,
    uniq:      uniq,
    intersect: intersect,
    clone:     clone,
    toArray:   clone,
    size:      size,
    inspect:   inspect
  });

  var CONCAT_ARGUMENTS_BUGGY = (function() {
    return [].concat(arguments)[0][0] !== 1;
  })(1,2)

  if (CONCAT_ARGUMENTS_BUGGY) arrayProto.concat = concat;

  if (!arrayProto.indexOf) arrayProto.indexOf = indexOf;
  if (!arrayProto.lastIndexOf) arrayProto.lastIndexOf = lastIndexOf;
})();
function $H(object) {
  return new Hash(object);
};

var Hash = Class.create(Enumerable, (function() {
  function initialize(object) {
    this._object = Object.isHash(object) ? object.toObject() : Object.clone(object);
  }


  function _each(iterator) {
    for (var key in this._object) {
      var value = this._object[key], pair = [key, value];
      pair.key = key;
      pair.value = value;
      iterator(pair);
    }
  }

  function set(key, value) {
    return this._object[key] = value;
  }

  function get(key) {
    if (this._object[key] !== Object.prototype[key])
      return this._object[key];
  }

  function unset(key) {
    var value = this._object[key];
    delete this._object[key];
    return value;
  }

  function toObject() {
    return Object.clone(this._object);
  }



  function keys() {
    return this.pluck('key');
  }

  function values() {
    return this.pluck('value');
  }

  function index(value) {
    var match = this.detect(function(pair) {
      return pair.value === value;
    });
    return match && match.key;
  }

  function merge(object) {
    return this.clone().update(object);
  }

  function update(object) {
    return new Hash(object).inject(this, function(result, pair) {
      result.set(pair.key, pair.value);
      return result;
    });
  }

  function toQueryPair(key, value) {
    if (Object.isUndefined(value)) return key;
    return key + '=' + encodeURIComponent(String.interpret(value));
  }

  function toQueryString() {
    return this.inject([], function(results, pair) {
      var key = encodeURIComponent(pair.key), values = pair.value;

      if (values && typeof values == 'object') {
        if (Object.isArray(values)) {
          var queryValues = [];
          for (var i = 0, len = values.length, value; i < len; i++) {
            value = values[i];
            queryValues.push(toQueryPair(key, value));
          }
          return results.concat(queryValues);
        }
      } else results.push(toQueryPair(key, values));
      return results;
    }).join('&');
  }

  function inspect() {
    return '#<Hash:{' + this.map(function(pair) {
      return pair.map(Object.inspect).join(': ');
    }).join(', ') + '}>';
  }

  function clone() {
    return new Hash(this);
  }

  return {
    initialize:             initialize,
    _each:                  _each,
    set:                    set,
    get:                    get,
    unset:                  unset,
    toObject:               toObject,
    toTemplateReplacements: toObject,
    keys:                   keys,
    values:                 values,
    index:                  index,
    merge:                  merge,
    update:                 update,
    toQueryString:          toQueryString,
    inspect:                inspect,
    toJSON:                 toObject,
    clone:                  clone
  };
})());

Hash.from = $H;
Object.extend(Number.prototype, (function() {
  function toColorPart() {
    return this.toPaddedString(2, 16);
  }

  function succ() {
    return this + 1;
  }

  function times(iterator, context) {
    $R(0, this, true).each(iterator, context);
    return this;
  }

  function toPaddedString(length, radix) {
    var string = this.toString(radix || 10);
    return '0'.times(length - string.length) + string;
  }

  function abs() {
    return Math.abs(this);
  }

  function round() {
    return Math.round(this);
  }

  function ceil() {
    return Math.ceil(this);
  }

  function floor() {
    return Math.floor(this);
  }

  return {
    toColorPart:    toColorPart,
    succ:           succ,
    times:          times,
    toPaddedString: toPaddedString,
    abs:            abs,
    round:          round,
    ceil:           ceil,
    floor:          floor
  };
})());

function $R(start, end, exclusive) {
  return new ObjectRange(start, end, exclusive);
}

var ObjectRange = Class.create(Enumerable, (function() {
  function initialize(start, end, exclusive) {
    this.start = start;
    this.end = end;
    this.exclusive = exclusive;
  }

  function _each(iterator) {
    var value = this.start;
    while (this.include(value)) {
      iterator(value);
      value = value.succ();
    }
  }

  function include(value) {
    if (value < this.start)
      return false;
    if (this.exclusive)
      return value < this.end;
    return value <= this.end;
  }

  return {
    initialize: initialize,
    _each:      _each,
    include:    include
  };
})());



var Abstract = { };


var Try = {
  these: function() {
    var returnValue;

    for (var i = 0, length = arguments.length; i < length; i++) {
      var lambda = arguments[i];
      try {
        returnValue = lambda();
        break;
      } catch (e) { }
    }

    return returnValue;
  }
};

var Ajax = {
  getTransport: function() {
    return Try.these(
      function() {return new XMLHttpRequest()},
      function() {return new ActiveXObject('Msxml2.XMLHTTP')},
      function() {return new ActiveXObject('Microsoft.XMLHTTP')}
    ) || false;
  },

  activeRequestCount: 0
};

Ajax.Responders = {
  responders: [],

  _each: function(iterator) {
    this.responders._each(iterator);
  },

  register: function(responder) {
    if (!this.include(responder))
      this.responders.push(responder);
  },

  unregister: function(responder) {
    this.responders = this.responders.without(responder);
  },

  dispatch: function(callback, request, transport, json) {
    this.each(function(responder) {
      if (Object.isFunction(responder[callback])) {
        try {
          responder[callback].apply(responder, [request, transport, json]);
        } catch (e) { }
      }
    });
  }
};

Object.extend(Ajax.Responders, Enumerable);

Ajax.Responders.register({
  onCreate:   function() { Ajax.activeRequestCount++ },
  onComplete: function() { Ajax.activeRequestCount-- }
});
Ajax.Base = Class.create({
  initialize: function(options) {
    this.options = {
      method:       'post',
      asynchronous: true,
      contentType:  'application/x-www-form-urlencoded',
      encoding:     'UTF-8',
      parameters:   '',
      evalJSON:     true,
      evalJS:       true
    };
    Object.extend(this.options, options || { });

    this.options.method = this.options.method.toLowerCase();

    if (Object.isHash(this.options.parameters))
      this.options.parameters = this.options.parameters.toObject();
  }
});
Ajax.Request = Class.create(Ajax.Base, {
  _complete: false,

  initialize: function($super, url, options) {
    $super(options);
    this.transport = Ajax.getTransport();
    this.request(url);
  },

  request: function(url) {
    this.url = url;
    this.method = this.options.method;
    var params = Object.isString(this.options.parameters) ?
          this.options.parameters :
          Object.toQueryString(this.options.parameters);

    if (!['get', 'post'].include(this.method)) {
      params += (params ? '&' : '') + "_method=" + this.method;
      this.method = 'post';
    }

    if (params && this.method === 'get') {
      this.url += (this.url.include('?') ? '&' : '?') + params;
    }

    this.parameters = params.toQueryParams();

    try {
      var response = new Ajax.Response(this);
      if (this.options.onCreate) this.options.onCreate(response);
      Ajax.Responders.dispatch('onCreate', this, response);

      this.transport.open(this.method.toUpperCase(), this.url,
        this.options.asynchronous);

      if (this.options.asynchronous) this.respondToReadyState.bind(this).defer(1);

      this.transport.onreadystatechange = this.onStateChange.bind(this);
      this.setRequestHeaders();

      this.body = this.method == 'post' ? (this.options.postBody || params) : null;
      this.transport.send(this.body);

      /* Force Firefox to handle ready state 4 for synchronous requests */
      if (!this.options.asynchronous && this.transport.overrideMimeType)
        this.onStateChange();

    }
    catch (e) {
      this.dispatchException(e);
    }
  },

  onStateChange: function() {
    var readyState = this.transport.readyState;
    if (readyState > 1 && !((readyState == 4) && this._complete))
      this.respondToReadyState(this.transport.readyState);
  },

  setRequestHeaders: function() {
    var headers = {
      'X-Requested-With': 'XMLHttpRequest',
      'X-Prototype-Version': Prototype.Version,
      'Accept': 'text/javascript, text/html, application/xml, text/xml, */*'
    };

    if (this.method == 'post') {
      headers['Content-type'] = this.options.contentType +
        (this.options.encoding ? '; charset=' + this.options.encoding : '');

      /* Force "Connection: close" for older Mozilla browsers to work
       * around a bug where XMLHttpRequest sends an incorrect
       * Content-length header. See Mozilla Bugzilla #246651.
       */
      if (this.transport.overrideMimeType &&
          (navigator.userAgent.match(/Gecko\/(\d{4})/) || [0,2005])[1] < 2005)
            headers['Connection'] = 'close';
    }

    if (typeof this.options.requestHeaders == 'object') {
      var extras = this.options.requestHeaders;

      if (Object.isFunction(extras.push))
        for (var i = 0, length = extras.length; i < length; i += 2)
          headers[extras[i]] = extras[i+1];
      else
        $H(extras).each(function(pair) { headers[pair.key] = pair.value });
    }

    for (var name in headers)
      this.transport.setRequestHeader(name, headers[name]);
  },

  success: function() {
    var status = this.getStatus();
    return !status || (status >= 200 && status < 300) || status == 304;
  },

  getStatus: function() {
    try {
      if (this.transport.status === 1223) return 204;
      return this.transport.status || 0;
    } catch (e) { return 0 }
  },

  respondToReadyState: function(readyState) {
    var state = Ajax.Request.Events[readyState], response = new Ajax.Response(this);

    if (state == 'Complete') {
      try {
        this._complete = true;
        (this.options['on' + response.status]
         || this.options['on' + (this.success() ? 'Success' : 'Failure')]
         || Prototype.emptyFunction)(response, response.headerJSON);
      } catch (e) {
        this.dispatchException(e);
      }

      var contentType = response.getHeader('Content-type');
      if (this.options.evalJS == 'force'
          || (this.options.evalJS && this.isSameOrigin() && contentType
          && contentType.match(/^\s*(text|application)\/(x-)?(java|ecma)script(;.*)?\s*$/i)))
        this.evalResponse();
    }

    try {
      (this.options['on' + state] || Prototype.emptyFunction)(response, response.headerJSON);
      Ajax.Responders.dispatch('on' + state, this, response, response.headerJSON);
    } catch (e) {
      this.dispatchException(e);
    }

    if (state == 'Complete') {
      this.transport.onreadystatechange = Prototype.emptyFunction;
    }
  },

  isSameOrigin: function() {
    var m = this.url.match(/^\s*https?:\/\/[^\/]*/);
    return !m || (m[0] == '#{protocol}//#{domain}#{port}'.interpolate({
      protocol: location.protocol,
      domain: document.domain,
      port: location.port ? ':' + location.port : ''
    }));
  },

  getHeader: function(name) {
    try {
      return this.transport.getResponseHeader(name) || null;
    } catch (e) { return null; }
  },

  evalResponse: function() {
    try {
      return eval((this.transport.responseText || '').unfilterJSON());
    } catch (e) {
      this.dispatchException(e);
    }
  },

  dispatchException: function(exception) {
    (this.options.onException || Prototype.emptyFunction)(this, exception);
    Ajax.Responders.dispatch('onException', this, exception);
  }
});

Ajax.Request.Events =
  ['Uninitialized', 'Loading', 'Loaded', 'Interactive', 'Complete'];








Ajax.Response = Class.create({
  initialize: function(request){
    this.request = request;
    var transport  = this.transport  = request.transport,
        readyState = this.readyState = transport.readyState;

    if ((readyState > 2 && !Prototype.Browser.IE) || readyState == 4) {
      this.status       = this.getStatus();
      this.statusText   = this.getStatusText();
      this.responseText = String.interpret(transport.responseText);
      this.headerJSON   = this._getHeaderJSON();
    }

    if (readyState == 4) {
      var xml = transport.responseXML;
      this.responseXML  = Object.isUndefined(xml) ? null : xml;
      this.responseJSON = this._getResponseJSON();
    }
  },

  status:      0,

  statusText: '',

  getStatus: Ajax.Request.prototype.getStatus,

  getStatusText: function() {
    try {
      return this.transport.statusText || '';
    } catch (e) { return '' }
  },

  getHeader: Ajax.Request.prototype.getHeader,

  getAllHeaders: function() {
    try {
      return this.getAllResponseHeaders();
    } catch (e) { return null }
  },

  getResponseHeader: function(name) {
    return this.transport.getResponseHeader(name);
  },

  getAllResponseHeaders: function() {
    return this.transport.getAllResponseHeaders();
  },

  _getHeaderJSON: function() {
    var json = this.getHeader('X-JSON');
    if (!json) return null;
    json = decodeURIComponent(escape(json));
    try {
      return json.evalJSON(this.request.options.sanitizeJSON ||
        !this.request.isSameOrigin());
    } catch (e) {
      this.request.dispatchException(e);
    }
  },

  _getResponseJSON: function() {
    var options = this.request.options;
    if (!options.evalJSON || (options.evalJSON != 'force' &&
      !(this.getHeader('Content-type') || '').include('application/json')) ||
        this.responseText.blank())
          return null;
    try {
      return this.responseText.evalJSON(options.sanitizeJSON ||
        !this.request.isSameOrigin());
    } catch (e) {
      this.request.dispatchException(e);
    }
  }
});

Ajax.Updater = Class.create(Ajax.Request, {
  initialize: function($super, container, url, options) {
    this.container = {
      success: (container.success || container),
      failure: (container.failure || (container.success ? null : container))
    };

    options = Object.clone(options);
    var onComplete = options.onComplete;
    options.onComplete = (function(response, json) {
      this.updateContent(response.responseText);
      if (Object.isFunction(onComplete)) onComplete(response, json);
    }).bind(this);

    $super(url, options);
  },

  updateContent: function(responseText) {
    var receiver = this.container[this.success() ? 'success' : 'failure'],
        options = this.options;

    if (!options.evalScripts) responseText = responseText.stripScripts();

    if (receiver = $(receiver)) {
      if (options.insertion) {
        if (Object.isString(options.insertion)) {
          var insertion = { }; insertion[options.insertion] = responseText;
          receiver.insert(insertion);
        }
        else options.insertion(receiver, responseText);
      }
      else receiver.update(responseText);
    }
  }
});

Ajax.PeriodicalUpdater = Class.create(Ajax.Base, {
  initialize: function($super, container, url, options) {
    $super(options);
    this.onComplete = this.options.onComplete;

    this.frequency = (this.options.frequency || 2);
    this.decay = (this.options.decay || 1);

    this.updater = { };
    this.container = container;
    this.url = url;

    this.start();
  },

  start: function() {
    this.options.onComplete = this.updateComplete.bind(this);
    this.onTimerEvent();
  },

  stop: function() {
    this.updater.options.onComplete = undefined;
    clearTimeout(this.timer);
    (this.onComplete || Prototype.emptyFunction).apply(this, arguments);
  },

  updateComplete: function(response) {
    if (this.options.decay) {
      this.decay = (response.responseText == this.lastText ?
        this.decay * this.options.decay : 1);

      this.lastText = response.responseText;
    }
    this.timer = this.onTimerEvent.bind(this).delay(this.decay * this.frequency);
  },

  onTimerEvent: function() {
    this.updater = new Ajax.Updater(this.container, this.url, this.options);
  }
});


function $(element) {
  if (arguments.length > 1) {
    for (var i = 0, elements = [], length = arguments.length; i < length; i++)
      elements.push($(arguments[i]));
    return elements;
  }
  if (Object.isString(element))
    element = document.getElementById(element);
  return Element.extend(element);
}

if (Prototype.BrowserFeatures.XPath) {
  document._getElementsByXPath = function(expression, parentElement) {
    var results = [];
    var query = document.evaluate(expression, $(parentElement) || document,
      null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
    for (var i = 0, length = query.snapshotLength; i < length; i++)
      results.push(Element.extend(query.snapshotItem(i)));
    return results;
  };
}

/*--------------------------------------------------------------------------*/

if (!Node) var Node = { };

if (!Node.ELEMENT_NODE) {
  Object.extend(Node, {
    ELEMENT_NODE: 1,
    ATTRIBUTE_NODE: 2,
    TEXT_NODE: 3,
    CDATA_SECTION_NODE: 4,
    ENTITY_REFERENCE_NODE: 5,
    ENTITY_NODE: 6,
    PROCESSING_INSTRUCTION_NODE: 7,
    COMMENT_NODE: 8,
    DOCUMENT_NODE: 9,
    DOCUMENT_TYPE_NODE: 10,
    DOCUMENT_FRAGMENT_NODE: 11,
    NOTATION_NODE: 12
  });
}



(function(global) {
  function shouldUseCache(tagName, attributes) {
    if (tagName === 'select') return false;
    if ('type' in attributes) return false;
    return true;
  }

  var HAS_EXTENDED_CREATE_ELEMENT_SYNTAX = (function(){
    try {
      var el = document.createElement('<input name="x">');
      return el.tagName.toLowerCase() === 'input' && el.name === 'x';
    }
    catch(err) {
      return false;
    }
  })();

  var element = global.Element;

  global.Element = function(tagName, attributes) {
    attributes = attributes || { };
    tagName = tagName.toLowerCase();
    var cache = Element.cache;

    if (HAS_EXTENDED_CREATE_ELEMENT_SYNTAX && attributes.name) {
      tagName = '<' + tagName + ' name="' + attributes.name + '">';
      delete attributes.name;
      return Element.writeAttribute(document.createElement(tagName), attributes);
    }

    if (!cache[tagName]) cache[tagName] = Element.extend(document.createElement(tagName));

    var node = shouldUseCache(tagName, attributes) ?
     cache[tagName].cloneNode(false) : document.createElement(tagName);

    return Element.writeAttribute(node, attributes);
  };

  Object.extend(global.Element, element || { });
  if (element) global.Element.prototype = element.prototype;

})(this);

Element.idCounter = 1;
Element.cache = { };

Element._purgeElement = function(element) {
  var uid = element._prototypeUID;
  if (uid) {
    Element.stopObserving(element);
    element._prototypeUID = void 0;
    delete Element.Storage[uid];
  }
}

Element.Methods = {
  visible: function(element) {
    return $(element).style.display != 'none';
  },

  toggle: function(element) {
    element = $(element);
    Element[Element.visible(element) ? 'hide' : 'show'](element);
    return element;
  },

  hide: function(element) {
    element = $(element);
    element.style.display = 'none';
    return element;
  },

  show: function(element) {
    element = $(element);
    element.style.display = '';
    return element;
  },

  remove: function(element) {
    element = $(element);
    element.parentNode.removeChild(element);
    return element;
  },

  update: (function(){

    var SELECT_ELEMENT_INNERHTML_BUGGY = (function(){
      var el = document.createElement("select"),
          isBuggy = true;
      el.innerHTML = "<option value=\"test\">test</option>";
      if (el.options && el.options[0]) {
        isBuggy = el.options[0].nodeName.toUpperCase() !== "OPTION";
      }
      el = null;
      return isBuggy;
    })();

    var TABLE_ELEMENT_INNERHTML_BUGGY = (function(){
      try {
        var el = document.createElement("table");
        if (el && el.tBodies) {
          el.innerHTML = "<tbody><tr><td>test</td></tr></tbody>";
          var isBuggy = typeof el.tBodies[0] == "undefined";
          el = null;
          return isBuggy;
        }
      } catch (e) {
        return true;
      }
    })();

    var LINK_ELEMENT_INNERHTML_BUGGY = (function() {
      try {
        var el = document.createElement('div');
        el.innerHTML = "<link>";
        var isBuggy = (el.childNodes.length === 0);
        el = null;
        return isBuggy;
      } catch(e) {
        return true;
      }
    })();

    var ANY_INNERHTML_BUGGY = SELECT_ELEMENT_INNERHTML_BUGGY ||
     TABLE_ELEMENT_INNERHTML_BUGGY || LINK_ELEMENT_INNERHTML_BUGGY;

    var SCRIPT_ELEMENT_REJECTS_TEXTNODE_APPENDING = (function () {
      var s = document.createElement("script"),
          isBuggy = false;
      try {
        s.appendChild(document.createTextNode(""));
        isBuggy = !s.firstChild ||
          s.firstChild && s.firstChild.nodeType !== 3;
      } catch (e) {
        isBuggy = true;
      }
      s = null;
      return isBuggy;
    })();


    function update(element, content) {
      element = $(element);
      var purgeElement = Element._purgeElement;

      var descendants = element.getElementsByTagName('*'),
       i = descendants.length;
      while (i--) purgeElement(descendants[i]);

      if (content && content.toElement)
        content = content.toElement();

      if (Object.isElement(content))
        return element.update().insert(content);

      content = Object.toHTML(content);

      var tagName = element.tagName.toUpperCase();

      if (tagName === 'SCRIPT' && SCRIPT_ELEMENT_REJECTS_TEXTNODE_APPENDING) {
        element.text = content;
        return element;
      }

      if (ANY_INNERHTML_BUGGY) {
        if (tagName in Element._insertionTranslations.tags) {
          while (element.firstChild) {
            element.removeChild(element.firstChild);
          }
          Element._getContentFromAnonymousElement(tagName, content.stripScripts())
            .each(function(node) {
              element.appendChild(node)
            });
        } else if (LINK_ELEMENT_INNERHTML_BUGGY && Object.isString(content) && content.indexOf('<link') > -1) {
          while (element.firstChild) {
            element.removeChild(element.firstChild);
          }
          var nodes = Element._getContentFromAnonymousElement(tagName, content.stripScripts(), true);
          nodes.each(function(node) { element.appendChild(node) });
        }
        else {
          element.innerHTML = content.stripScripts();
        }
      }
      else {
        element.innerHTML = content.stripScripts();
      }

      content.evalScripts.bind(content).defer();
      return element;
    }

    return update;
  })(),

  replace: function(element, content) {
    element = $(element);
    if (content && content.toElement) content = content.toElement();
    else if (!Object.isElement(content)) {
      content = Object.toHTML(content);
      var range = element.ownerDocument.createRange();
      range.selectNode(element);
      content.evalScripts.bind(content).defer();
      content = range.createContextualFragment(content.stripScripts());
    }
    element.parentNode.replaceChild(content, element);
    return element;
  },

  insert: function(element, insertions) {
    element = $(element);

    if (Object.isString(insertions) || Object.isNumber(insertions) ||
        Object.isElement(insertions) || (insertions && (insertions.toElement || insertions.toHTML)))
          insertions = {bottom:insertions};

    var content, insert, tagName, childNodes;

    for (var position in insertions) {
      content  = insertions[position];
      position = position.toLowerCase();
      insert = Element._insertionTranslations[position];

      if (content && content.toElement) content = content.toElement();
      if (Object.isElement(content)) {
        insert(element, content);
        continue;
      }

      content = Object.toHTML(content);

      tagName = ((position == 'before' || position == 'after')
        ? element.parentNode : element).tagName.toUpperCase();

      childNodes = Element._getContentFromAnonymousElement(tagName, content.stripScripts());

      if (position == 'top' || position == 'after') childNodes.reverse();
      childNodes.each(insert.curry(element));

      content.evalScripts.bind(content).defer();
    }

    return element;
  },

  wrap: function(element, wrapper, attributes) {
    element = $(element);
    if (Object.isElement(wrapper))
      $(wrapper).writeAttribute(attributes || { });
    else if (Object.isString(wrapper)) wrapper = new Element(wrapper, attributes);
    else wrapper = new Element('div', wrapper);
    if (element.parentNode)
      element.parentNode.replaceChild(wrapper, element);
    wrapper.appendChild(element);
    return wrapper;
  },

  inspect: function(element) {
    element = $(element);
    var result = '<' + element.tagName.toLowerCase();
    $H({'id': 'id', 'className': 'class'}).each(function(pair) {
      var property = pair.first(),
          attribute = pair.last(),
          value = (element[property] || '').toString();
      if (value) result += ' ' + attribute + '=' + value.inspect(true);
    });
    return result + '>';
  },

  recursivelyCollect: function(element, property, maximumLength) {
    element = $(element);
    maximumLength = maximumLength || -1;
    var elements = [];

    while (element = element[property]) {
      if (element.nodeType == 1)
        elements.push(Element.extend(element));
      if (elements.length == maximumLength)
        break;
    }

    return elements;
  },

  ancestors: function(element) {
    return Element.recursivelyCollect(element, 'parentNode');
  },

  descendants: function(element) {
    return Element.select(element, "*");
  },

  firstDescendant: function(element) {
    element = $(element).firstChild;
    while (element && element.nodeType != 1) element = element.nextSibling;
    return $(element);
  },

  immediateDescendants: function(element) {
    var results = [], child = $(element).firstChild;
    while (child) {
      if (child.nodeType === 1) {
        results.push(Element.extend(child));
      }
      child = child.nextSibling;
    }
    return results;
  },

  previousSiblings: function(element, maximumLength) {
    return Element.recursivelyCollect(element, 'previousSibling');
  },

  nextSiblings: function(element) {
    return Element.recursivelyCollect(element, 'nextSibling');
  },

  siblings: function(element) {
    element = $(element);
    return Element.previousSiblings(element).reverse()
      .concat(Element.nextSiblings(element));
  },

  match: function(element, selector) {
    element = $(element);
    if (Object.isString(selector))
      return Prototype.Selector.match(element, selector);
    return selector.match(element);
  },

  up: function(element, expression, index) {
    element = $(element);
    if (arguments.length == 1) return $(element.parentNode);
    var ancestors = Element.ancestors(element);
    return Object.isNumber(expression) ? ancestors[expression] :
      Prototype.Selector.find(ancestors, expression, index);
  },

  down: function(element, expression, index) {
    element = $(element);
    if (arguments.length == 1) return Element.firstDescendant(element);
    return Object.isNumber(expression) ? Element.descendants(element)[expression] :
      Element.select(element, expression)[index || 0];
  },

  previous: function(element, expression, index) {
    element = $(element);
    if (Object.isNumber(expression)) index = expression, expression = false;
    if (!Object.isNumber(index)) index = 0;

    if (expression) {
      return Prototype.Selector.find(element.previousSiblings(), expression, index);
    } else {
      return element.recursivelyCollect("previousSibling", index + 1)[index];
    }
  },

  next: function(element, expression, index) {
    element = $(element);
    if (Object.isNumber(expression)) index = expression, expression = false;
    if (!Object.isNumber(index)) index = 0;

    if (expression) {
      return Prototype.Selector.find(element.nextSiblings(), expression, index);
    } else {
      var maximumLength = Object.isNumber(index) ? index + 1 : 1;
      return element.recursivelyCollect("nextSibling", index + 1)[index];
    }
  },


  select: function(element) {
    element = $(element);
    var expressions = Array.prototype.slice.call(arguments, 1).join(', ');
    return Prototype.Selector.select(expressions, element);
  },

  adjacent: function(element) {
    element = $(element);
    var expressions = Array.prototype.slice.call(arguments, 1).join(', ');
    return Prototype.Selector.select(expressions, element.parentNode).without(element);
  },

  identify: function(element) {
    element = $(element);
    var id = Element.readAttribute(element, 'id');
    if (id) return id;
    do { id = 'anonymous_element_' + Element.idCounter++ } while ($(id));
    Element.writeAttribute(element, 'id', id);
    return id;
  },

  readAttribute: function(element, name) {
    element = $(element);
    if (Prototype.Browser.IE) {
      var t = Element._attributeTranslations.read;
      if (t.values[name]) return t.values[name](element, name);
      if (t.names[name]) name = t.names[name];
      if (name.include(':')) {
        return (!element.attributes || !element.attributes[name]) ? null :
         element.attributes[name].value;
      }
    }
    return element.getAttribute(name);
  },

  writeAttribute: function(element, name, value) {
    element = $(element);
    var attributes = { }, t = Element._attributeTranslations.write;

    if (typeof name == 'object') attributes = name;
    else attributes[name] = Object.isUndefined(value) ? true : value;

    for (var attr in attributes) {
      name = t.names[attr] || attr;
      value = attributes[attr];
      if (t.values[attr]) name = t.values[attr](element, value);
      if (value === false || value === null)
        element.removeAttribute(name);
      else if (value === true)
        element.setAttribute(name, name);
      else element.setAttribute(name, value);
    }
    return element;
  },

  getHeight: function(element) {
    return Element.getDimensions(element).height;
  },

  getWidth: function(element) {
    return Element.getDimensions(element).width;
  },

  classNames: function(element) {
    return new Element.ClassNames(element);
  },

  hasClassName: function(element, className) {
    if (!(element = $(element))) return;
    var elementClassName = element.className;
    return (elementClassName.length > 0 && (elementClassName == className ||
      new RegExp("(^|\\s)" + className + "(\\s|$)").test(elementClassName)));
  },

  addClassName: function(element, className) {
    if (!(element = $(element))) return;
    if (!Element.hasClassName(element, className))
      element.className += (element.className ? ' ' : '') + className;
    return element;
  },

  removeClassName: function(element, className) {
    if (!(element = $(element))) return;
    element.className = element.className.replace(
      new RegExp("(^|\\s+)" + className + "(\\s+|$)"), ' ').strip();
    return element;
  },

  toggleClassName: function(element, className) {
    if (!(element = $(element))) return;
    return Element[Element.hasClassName(element, className) ?
      'removeClassName' : 'addClassName'](element, className);
  },

  cleanWhitespace: function(element) {
    element = $(element);
    var node = element.firstChild;
    while (node) {
      var nextNode = node.nextSibling;
      if (node.nodeType == 3 && !/\S/.test(node.nodeValue))
        element.removeChild(node);
      node = nextNode;
    }
    return element;
  },

  empty: function(element) {
    return $(element).innerHTML.blank();
  },

  descendantOf: function(element, ancestor) {
    element = $(element), ancestor = $(ancestor);

    if (element.compareDocumentPosition)
      return (element.compareDocumentPosition(ancestor) & 8) === 8;

    if (ancestor.contains)
      return ancestor.contains(element) && ancestor !== element;

    while (element = element.parentNode)
      if (element == ancestor) return true;

    return false;
  },

  scrollTo: function(element) {
    element = $(element);
    var pos = Element.cumulativeOffset(element);
    window.scrollTo(pos[0], pos[1]);
    return element;
  },

  getStyle: function(element, style) {
    element = $(element);
    style = style == 'float' ? 'cssFloat' : style.camelize();
    var value = element.style[style];
    if (!value || value == 'auto') {
      var css = document.defaultView.getComputedStyle(element, null);
      value = css ? css[style] : null;
    }
    if (style == 'opacity') return value ? parseFloat(value) : 1.0;
    return value == 'auto' ? null : value;
  },

  getOpacity: function(element) {
    return $(element).getStyle('opacity');
  },

  setStyle: function(element, styles) {
    element = $(element);
    var elementStyle = element.style, match;
    if (Object.isString(styles)) {
      element.style.cssText += ';' + styles;
      return styles.include('opacity') ?
        element.setOpacity(styles.match(/opacity:\s*(\d?\.?\d*)/)[1]) : element;
    }
    for (var property in styles)
      if (property == 'opacity') element.setOpacity(styles[property]);
      else
        elementStyle[(property == 'float' || property == 'cssFloat') ?
          (Object.isUndefined(elementStyle.styleFloat) ? 'cssFloat' : 'styleFloat') :
            property] = styles[property];

    return element;
  },

  setOpacity: function(element, value) {
    element = $(element);
    element.style.opacity = (value == 1 || value === '') ? '' :
      (value < 0.00001) ? 0 : value;
    return element;
  },

  makePositioned: function(element) {
    element = $(element);
    var pos = Element.getStyle(element, 'position');
    if (pos == 'static' || !pos) {
      element._madePositioned = true;
      element.style.position = 'relative';
      if (Prototype.Browser.Opera) {
        element.style.top = 0;
        element.style.left = 0;
      }
    }
    return element;
  },

  undoPositioned: function(element) {
    element = $(element);
    if (element._madePositioned) {
      element._madePositioned = undefined;
      element.style.position =
        element.style.top =
        element.style.left =
        element.style.bottom =
        element.style.right = '';
    }
    return element;
  },

  makeClipping: function(element) {
    element = $(element);
    if (element._overflow) return element;
    element._overflow = Element.getStyle(element, 'overflow') || 'auto';
    if (element._overflow !== 'hidden')
      element.style.overflow = 'hidden';
    return element;
  },

  undoClipping: function(element) {
    element = $(element);
    if (!element._overflow) return element;
    element.style.overflow = element._overflow == 'auto' ? '' : element._overflow;
    element._overflow = null;
    return element;
  },

  clonePosition: function(element, source) {
    var options = Object.extend({
      setLeft:    true,
      setTop:     true,
      setWidth:   true,
      setHeight:  true,
      offsetTop:  0,
      offsetLeft: 0
    }, arguments[2] || { });

    source = $(source);
    var p = Element.viewportOffset(source), delta = [0, 0], parent = null;

    element = $(element);

    if (Element.getStyle(element, 'position') == 'absolute') {
      parent = Element.getOffsetParent(element);
      delta = Element.viewportOffset(parent);
    }

    if (parent == document.body) {
      delta[0] -= document.body.offsetLeft;
      delta[1] -= document.body.offsetTop;
    }

    if (options.setLeft)   element.style.left  = (p[0] - delta[0] + options.offsetLeft) + 'px';
    if (options.setTop)    element.style.top   = (p[1] - delta[1] + options.offsetTop) + 'px';
    if (options.setWidth)  element.style.width = source.offsetWidth + 'px';
    if (options.setHeight) element.style.height = source.offsetHeight + 'px';
    return element;
  }
};

Object.extend(Element.Methods, {
  getElementsBySelector: Element.Methods.select,

  childElements: Element.Methods.immediateDescendants
});

Element._attributeTranslations = {
  write: {
    names: {
      className: 'class',
      htmlFor:   'for'
    },
    values: { }
  }
};

if (Prototype.Browser.Opera) {
  Element.Methods.getStyle = Element.Methods.getStyle.wrap(
    function(proceed, element, style) {
      switch (style) {
        case 'height': case 'width':
          if (!Element.visible(element)) return null;

          var dim = parseInt(proceed(element, style), 10);

          if (dim !== element['offset' + style.capitalize()])
            return dim + 'px';

          var properties;
          if (style === 'height') {
            properties = ['border-top-width', 'padding-top',
             'padding-bottom', 'border-bottom-width'];
          }
          else {
            properties = ['border-left-width', 'padding-left',
             'padding-right', 'border-right-width'];
          }
          return properties.inject(dim, function(memo, property) {
            var val = proceed(element, property);
            return val === null ? memo : memo - parseInt(val, 10);
          }) + 'px';
        default: return proceed(element, style);
      }
    }
  );

  Element.Methods.readAttribute = Element.Methods.readAttribute.wrap(
    function(proceed, element, attribute) {
      if (attribute === 'title') return element.title;
      return proceed(element, attribute);
    }
  );
}

else if (Prototype.Browser.IE) {
  Element.Methods.getStyle = function(element, style) {
    element = $(element);
    style = (style == 'float' || style == 'cssFloat') ? 'styleFloat' : style.camelize();
    var value = element.style[style];
    if (!value && element.currentStyle) value = element.currentStyle[style];

    if (style == 'opacity') {
      if (value = (element.getStyle('filter') || '').match(/alpha\(opacity=(.*)\)/))
        if (value[1]) return parseFloat(value[1]) / 100;
      return 1.0;
    }

    if (value == 'auto') {
      if ((style == 'width' || style == 'height') && (element.getStyle('display') != 'none'))
        return element['offset' + style.capitalize()] + 'px';
      return null;
    }
    return value;
  };

  Element.Methods.setOpacity = function(element, value) {
    function stripAlpha(filter){
      return filter.replace(/alpha\([^\)]*\)/gi,'');
    }
    element = $(element);
    var currentStyle = element.currentStyle;
    if ((currentStyle && !currentStyle.hasLayout) ||
      (!currentStyle && element.style.zoom == 'normal'))
        element.style.zoom = 1;

    var filter = element.getStyle('filter'), style = element.style;
    if (value == 1 || value === '') {
      (filter = stripAlpha(filter)) ?
        style.filter = filter : style.removeAttribute('filter');
      return element;
    } else if (value < 0.00001) value = 0;
    style.filter = stripAlpha(filter) +
      'alpha(opacity=' + (value * 100) + ')';
    return element;
  };

  Element._attributeTranslations = (function(){

    var classProp = 'className',
        forProp = 'for',
        el = document.createElement('div');

    el.setAttribute(classProp, 'x');

    if (el.className !== 'x') {
      el.setAttribute('class', 'x');
      if (el.className === 'x') {
        classProp = 'class';
      }
    }
    el = null;

    el = document.createElement('label');
    el.setAttribute(forProp, 'x');
    if (el.htmlFor !== 'x') {
      el.setAttribute('htmlFor', 'x');
      if (el.htmlFor === 'x') {
        forProp = 'htmlFor';
      }
    }
    el = null;

    return {
      read: {
        names: {
          'class':      classProp,
          'className':  classProp,
          'for':        forProp,
          'htmlFor':    forProp
        },
        values: {
          _getAttr: function(element, attribute) {
            return element.getAttribute(attribute);
          },
          _getAttr2: function(element, attribute) {
            return element.getAttribute(attribute, 2);
          },
          _getAttrNode: function(element, attribute) {
            var node = element.getAttributeNode(attribute);
            return node ? node.value : "";
          },
          _getEv: (function(){

            var el = document.createElement('div'), f;
            el.onclick = Prototype.emptyFunction;
            var value = el.getAttribute('onclick');

            if (String(value).indexOf('{') > -1) {
              f = function(element, attribute) {
                attribute = element.getAttribute(attribute);
                if (!attribute) return null;
                attribute = attribute.toString();
                attribute = attribute.split('{')[1];
                attribute = attribute.split('}')[0];
                return attribute.strip();
              };
            }
            else if (value === '') {
              f = function(element, attribute) {
                attribute = element.getAttribute(attribute);
                if (!attribute) return null;
                return attribute.strip();
              };
            }
            el = null;
            return f;
          })(),
          _flag: function(element, attribute) {
            return $(element).hasAttribute(attribute) ? attribute : null;
          },
          style: function(element) {
            return element.style.cssText.toLowerCase();
          },
          title: function(element) {
            return element.title;
          }
        }
      }
    }
  })();

  Element._attributeTranslations.write = {
    names: Object.extend({
      cellpadding: 'cellPadding',
      cellspacing: 'cellSpacing'
    }, Element._attributeTranslations.read.names),
    values: {
      checked: function(element, value) {
        element.checked = !!value;
      },

      style: function(element, value) {
        element.style.cssText = value ? value : '';
      }
    }
  };

  Element._attributeTranslations.has = {};

  $w('colSpan rowSpan vAlign dateTime accessKey tabIndex ' +
      'encType maxLength readOnly longDesc frameBorder').each(function(attr) {
    Element._attributeTranslations.write.names[attr.toLowerCase()] = attr;
    Element._attributeTranslations.has[attr.toLowerCase()] = attr;
  });

  (function(v) {
    Object.extend(v, {
      href:        v._getAttr2,
      src:         v._getAttr2,
      type:        v._getAttr,
      action:      v._getAttrNode,
      disabled:    v._flag,
      checked:     v._flag,
      readonly:    v._flag,
      multiple:    v._flag,
      onload:      v._getEv,
      onunload:    v._getEv,
      onclick:     v._getEv,
      ondblclick:  v._getEv,
      onmousedown: v._getEv,
      onmouseup:   v._getEv,
      onmouseover: v._getEv,
      onmousemove: v._getEv,
      onmouseout:  v._getEv,
      onfocus:     v._getEv,
      onblur:      v._getEv,
      onkeypress:  v._getEv,
      onkeydown:   v._getEv,
      onkeyup:     v._getEv,
      onsubmit:    v._getEv,
      onreset:     v._getEv,
      onselect:    v._getEv,
      onchange:    v._getEv
    });
  })(Element._attributeTranslations.read.values);

  if (Prototype.BrowserFeatures.ElementExtensions) {
    (function() {
      function _descendants(element) {
        var nodes = element.getElementsByTagName('*'), results = [];
        for (var i = 0, node; node = nodes[i]; i++)
          if (node.tagName !== "!") // Filter out comment nodes.
            results.push(node);
        return results;
      }

      Element.Methods.down = function(element, expression, index) {
        element = $(element);
        if (arguments.length == 1) return element.firstDescendant();
        return Object.isNumber(expression) ? _descendants(element)[expression] :
          Element.select(element, expression)[index || 0];
      }
    })();
  }

}

else if (Prototype.Browser.Gecko && /rv:1\.8\.0/.test(navigator.userAgent)) {
  Element.Methods.setOpacity = function(element, value) {
    element = $(element);
    element.style.opacity = (value == 1) ? 0.999999 :
      (value === '') ? '' : (value < 0.00001) ? 0 : value;
    return element;
  };
}

else if (Prototype.Browser.WebKit) {
  Element.Methods.setOpacity = function(element, value) {
    element = $(element);
    element.style.opacity = (value == 1 || value === '') ? '' :
      (value < 0.00001) ? 0 : value;

    if (value == 1)
      if (element.tagName.toUpperCase() == 'IMG' && element.width) {
        element.width++; element.width--;
      } else try {
        var n = document.createTextNode(' ');
        element.appendChild(n);
        element.removeChild(n);
      } catch (e) { }

    return element;
  };
}

if ('outerHTML' in document.documentElement) {
  Element.Methods.replace = function(element, content) {
    element = $(element);

    if (content && content.toElement) content = content.toElement();
    if (Object.isElement(content)) {
      element.parentNode.replaceChild(content, element);
      return element;
    }

    content = Object.toHTML(content);
    var parent = element.parentNode, tagName = parent.tagName.toUpperCase();

    if (Element._insertionTranslations.tags[tagName]) {
      var nextSibling = element.next(),
          fragments = Element._getContentFromAnonymousElement(tagName, content.stripScripts());
      parent.removeChild(element);
      if (nextSibling)
        fragments.each(function(node) { parent.insertBefore(node, nextSibling) });
      else
        fragments.each(function(node) { parent.appendChild(node) });
    }
    else element.outerHTML = content.stripScripts();

    content.evalScripts.bind(content).defer();
    return element;
  };
}

Element._returnOffset = function(l, t) {
  var result = [l, t];
  result.left = l;
  result.top = t;
  return result;
};

Element._getContentFromAnonymousElement = function(tagName, html, force) {
  var div = new Element('div'),
      t = Element._insertionTranslations.tags[tagName];

  var workaround = false;
  if (t) workaround = true;
  else if (force) {
    workaround = true;
    t = ['', '', 0];
  }

  if (workaround) {
    div.innerHTML = '&nbsp;' + t[0] + html + t[1];
    div.removeChild(div.firstChild);
    for (var i = t[2]; i--; ) {
      div = div.firstChild;
    }
  }
  else {
    div.innerHTML = html;
  }
  return $A(div.childNodes);
};

Element._insertionTranslations = {
  before: function(element, node) {
    element.parentNode.insertBefore(node, element);
  },
  top: function(element, node) {
    element.insertBefore(node, element.firstChild);
  },
  bottom: function(element, node) {
    element.appendChild(node);
  },
  after: function(element, node) {
    element.parentNode.insertBefore(node, element.nextSibling);
  },
  tags: {
    TABLE:  ['<table>',                '</table>',                   1],
    TBODY:  ['<table><tbody>',         '</tbody></table>',           2],
    TR:     ['<table><tbody><tr>',     '</tr></tbody></table>',      3],
    TD:     ['<table><tbody><tr><td>', '</td></tr></tbody></table>', 4],
    SELECT: ['<select>',               '</select>',                  1]
  }
};

(function() {
  var tags = Element._insertionTranslations.tags;
  Object.extend(tags, {
    THEAD: tags.TBODY,
    TFOOT: tags.TBODY,
    TH:    tags.TD
  });
})();

Element.Methods.Simulated = {
  hasAttribute: function(element, attribute) {
    attribute = Element._attributeTranslations.has[attribute] || attribute;
    var node = $(element).getAttributeNode(attribute);
    return !!(node && node.specified);
  }
};

Element.Methods.ByTag = { };

Object.extend(Element, Element.Methods);

(function(div) {

  if (!Prototype.BrowserFeatures.ElementExtensions && div['__proto__']) {
    window.HTMLElement = { };
    window.HTMLElement.prototype = div['__proto__'];
    Prototype.BrowserFeatures.ElementExtensions = true;
  }

  div = null;

})(document.createElement('div'));

Element.extend = (function() {

  function checkDeficiency(tagName) {
    if (typeof window.Element != 'undefined') {
      var proto = window.Element.prototype;
      if (proto) {
        var id = '_' + (Math.random()+'').slice(2),
            el = document.createElement(tagName);
        proto[id] = 'x';
        var isBuggy = (el[id] !== 'x');
        delete proto[id];
        el = null;
        return isBuggy;
      }
    }
    return false;
  }

  function extendElementWith(element, methods) {
    for (var property in methods) {
      var value = methods[property];
      if (Object.isFunction(value) && !(property in element))
        element[property] = value.methodize();
    }
  }

  var HTMLOBJECTELEMENT_PROTOTYPE_BUGGY = checkDeficiency('object');

  if (Prototype.BrowserFeatures.SpecificElementExtensions) {
    if (HTMLOBJECTELEMENT_PROTOTYPE_BUGGY) {
      return function(element) {
        if (element && typeof element._extendedByPrototype == 'undefined') {
          var t = element.tagName;
          if (t && (/^(?:object|applet|embed)$/i.test(t))) {
            extendElementWith(element, Element.Methods);
            extendElementWith(element, Element.Methods.Simulated);
            extendElementWith(element, Element.Methods.ByTag[t.toUpperCase()]);
          }
        }
        return element;
      }
    }
    return Prototype.K;
  }

  var Methods = { }, ByTag = Element.Methods.ByTag;

  var extend = Object.extend(function(element) {
    if (!element || typeof element._extendedByPrototype != 'undefined' ||
        element.nodeType != 1 || element == window) return element;

    var methods = Object.clone(Methods),
        tagName = element.tagName.toUpperCase();

    if (ByTag[tagName]) Object.extend(methods, ByTag[tagName]);

    extendElementWith(element, methods);

    element._extendedByPrototype = Prototype.emptyFunction;
    return element;

  }, {
    refresh: function() {
      if (!Prototype.BrowserFeatures.ElementExtensions) {
        Object.extend(Methods, Element.Methods);
        Object.extend(Methods, Element.Methods.Simulated);
      }
    }
  });

  extend.refresh();
  return extend;
})();

if (document.documentElement.hasAttribute) {
  Element.hasAttribute = function(element, attribute) {
    return element.hasAttribute(attribute);
  };
}
else {
  Element.hasAttribute = Element.Methods.Simulated.hasAttribute;
}

Element.addMethods = function(methods) {
  var F = Prototype.BrowserFeatures, T = Element.Methods.ByTag;

  if (!methods) {
    Object.extend(Form, Form.Methods);
    Object.extend(Form.Element, Form.Element.Methods);
    Object.extend(Element.Methods.ByTag, {
      "FORM":     Object.clone(Form.Methods),
      "INPUT":    Object.clone(Form.Element.Methods),
      "SELECT":   Object.clone(Form.Element.Methods),
      "TEXTAREA": Object.clone(Form.Element.Methods),
      "BUTTON":   Object.clone(Form.Element.Methods)
    });
  }

  if (arguments.length == 2) {
    var tagName = methods;
    methods = arguments[1];
  }

  if (!tagName) Object.extend(Element.Methods, methods || { });
  else {
    if (Object.isArray(tagName)) tagName.each(extend);
    else extend(tagName);
  }

  function extend(tagName) {
    tagName = tagName.toUpperCase();
    if (!Element.Methods.ByTag[tagName])
      Element.Methods.ByTag[tagName] = { };
    Object.extend(Element.Methods.ByTag[tagName], methods);
  }

  function copy(methods, destination, onlyIfAbsent) {
    onlyIfAbsent = onlyIfAbsent || false;
    for (var property in methods) {
      var value = methods[property];
      if (!Object.isFunction(value)) continue;
      if (!onlyIfAbsent || !(property in destination))
        destination[property] = value.methodize();
    }
  }

  function findDOMClass(tagName) {
    var klass;
    var trans = {
      "OPTGROUP": "OptGroup", "TEXTAREA": "TextArea", "P": "Paragraph",
      "FIELDSET": "FieldSet", "UL": "UList", "OL": "OList", "DL": "DList",
      "DIR": "Directory", "H1": "Heading", "H2": "Heading", "H3": "Heading",
      "H4": "Heading", "H5": "Heading", "H6": "Heading", "Q": "Quote",
      "INS": "Mod", "DEL": "Mod", "A": "Anchor", "IMG": "Image", "CAPTION":
      "TableCaption", "COL": "TableCol", "COLGROUP": "TableCol", "THEAD":
      "TableSection", "TFOOT": "TableSection", "TBODY": "TableSection", "TR":
      "TableRow", "TH": "TableCell", "TD": "TableCell", "FRAMESET":
      "FrameSet", "IFRAME": "IFrame"
    };
    if (trans[tagName]) klass = 'HTML' + trans[tagName] + 'Element';
    if (window[klass]) return window[klass];
    klass = 'HTML' + tagName + 'Element';
    if (window[klass]) return window[klass];
    klass = 'HTML' + tagName.capitalize() + 'Element';
    if (window[klass]) return window[klass];

    var element = document.createElement(tagName),
        proto = element['__proto__'] || element.constructor.prototype;

    element = null;
    return proto;
  }

  var elementPrototype = window.HTMLElement ? HTMLElement.prototype :
   Element.prototype;

  if (F.ElementExtensions) {
    copy(Element.Methods, elementPrototype);
    copy(Element.Methods.Simulated, elementPrototype, true);
  }

  if (F.SpecificElementExtensions) {
    for (var tag in Element.Methods.ByTag) {
      var klass = findDOMClass(tag);
      if (Object.isUndefined(klass)) continue;
      copy(T[tag], klass.prototype);
    }
  }

  Object.extend(Element, Element.Methods);
  delete Element.ByTag;

  if (Element.extend.refresh) Element.extend.refresh();
  Element.cache = { };
};


document.viewport = {

  getDimensions: function() {
    return { width: this.getWidth(), height: this.getHeight() };
  },

  getScrollOffsets: function() {
    return Element._returnOffset(
      window.pageXOffset || document.documentElement.scrollLeft || document.body.scrollLeft,
      window.pageYOffset || document.documentElement.scrollTop  || document.body.scrollTop);
  }
};

(function(viewport) {
  var B = Prototype.Browser, doc = document, element, property = {};

  function getRootElement() {
    if (B.WebKit && !doc.evaluate)
      return document;

    if (B.Opera && window.parseFloat(window.opera.version()) < 9.5)
      return document.body;

    return document.documentElement;
  }

  function define(D) {
    if (!element) element = getRootElement();

    property[D] = 'client' + D;

    viewport['get' + D] = function() { return element[property[D]] };
    return viewport['get' + D]();
  }

  viewport.getWidth  = define.curry('Width');

  viewport.getHeight = define.curry('Height');
})(document.viewport);


Element.Storage = {
  UID: 1
};

Element.addMethods({
  getStorage: function(element) {
    if (!(element = $(element))) return;

    var uid;
    if (element === window) {
      uid = 0;
    } else {
      if (typeof element._prototypeUID === "undefined")
        element._prototypeUID = Element.Storage.UID++;
      uid = element._prototypeUID;
    }

    if (!Element.Storage[uid])
      Element.Storage[uid] = $H();

    return Element.Storage[uid];
  },

  store: function(element, key, value) {
    if (!(element = $(element))) return;

    if (arguments.length === 2) {
      Element.getStorage(element).update(key);
    } else {
      Element.getStorage(element).set(key, value);
    }

    return element;
  },

  retrieve: function(element, key, defaultValue) {
    if (!(element = $(element))) return;
    var hash = Element.getStorage(element), value = hash.get(key);

    if (Object.isUndefined(value)) {
      hash.set(key, defaultValue);
      value = defaultValue;
    }

    return value;
  },

  clone: function(element, deep) {
    if (!(element = $(element))) return;
    var clone = element.cloneNode(deep);
    clone._prototypeUID = void 0;
    if (deep) {
      var descendants = Element.select(clone, '*'),
          i = descendants.length;
      while (i--) {
        descendants[i]._prototypeUID = void 0;
      }
    }
    return Element.extend(clone);
  },

  purge: function(element) {
    if (!(element = $(element))) return;
    var purgeElement = Element._purgeElement;

    purgeElement(element);

    var descendants = element.getElementsByTagName('*'),
     i = descendants.length;

    while (i--) purgeElement(descendants[i]);

    return null;
  }
});

(function() {

  function toDecimal(pctString) {
    var match = pctString.match(/^(\d+)%?$/i);
    if (!match) return null;
    return (Number(match[1]) / 100);
  }

  function getPixelValue(value, property, context) {
    var element = null;
    if (Object.isElement(value)) {
      element = value;
      value = element.getStyle(property);
    }

    if (value === null) {
      return null;
    }

    if ((/^(?:-)?\d+(\.\d+)?(px)?$/i).test(value)) {
      return window.parseFloat(value);
    }

    var isPercentage = value.include('%'), isViewport = (context === document.viewport);

    if (/\d/.test(value) && element && element.runtimeStyle && !(isPercentage && isViewport)) {
      var style = element.style.left, rStyle = element.runtimeStyle.left;
      element.runtimeStyle.left = element.currentStyle.left;
      element.style.left = value || 0;
      value = element.style.pixelLeft;
      element.style.left = style;
      element.runtimeStyle.left = rStyle;

      return value;
    }

    if (element && isPercentage) {
      context = context || element.parentNode;
      var decimal = toDecimal(value);
      var whole = null;
      var position = element.getStyle('position');

      var isHorizontal = property.include('left') || property.include('right') ||
       property.include('width');

      var isVertical =  property.include('top') || property.include('bottom') ||
        property.include('height');

      if (context === document.viewport) {
        if (isHorizontal) {
          whole = document.viewport.getWidth();
        } else if (isVertical) {
          whole = document.viewport.getHeight();
        }
      } else {
        if (isHorizontal) {
          whole = $(context).measure('width');
        } else if (isVertical) {
          whole = $(context).measure('height');
        }
      }

      return (whole === null) ? 0 : whole * decimal;
    }

    return 0;
  }

  function toCSSPixels(number) {
    if (Object.isString(number) && number.endsWith('px')) {
      return number;
    }
    return number + 'px';
  }

  function isDisplayed(element) {
    var originalElement = element;
    while (element && element.parentNode) {
      var display = element.getStyle('display');
      if (display === 'none') {
        return false;
      }
      element = $(element.parentNode);
    }
    return true;
  }

  var hasLayout = Prototype.K;
  if ('currentStyle' in document.documentElement) {
    hasLayout = function(element) {
      if (!element.currentStyle.hasLayout) {
        element.style.zoom = 1;
      }
      return element;
    };
  }

  function cssNameFor(key) {
    if (key.include('border')) key = key + '-width';
    return key.camelize();
  }

  Element.Layout = Class.create(Hash, {
    initialize: function($super, element, preCompute) {
      $super();
      this.element = $(element);

      Element.Layout.PROPERTIES.each( function(property) {
        this._set(property, null);
      }, this);

      if (preCompute) {
        this._preComputing = true;
        this._begin();
        Element.Layout.PROPERTIES.each( this._compute, this );
        this._end();
        this._preComputing = false;
      }
    },

    _set: function(property, value) {
      return Hash.prototype.set.call(this, property, value);
    },

    set: function(property, value) {
      throw "Properties of Element.Layout are read-only.";
    },

    get: function($super, property) {
      var value = $super(property);
      return value === null ? this._compute(property) : value;
    },

    _begin: function() {
      if (this._prepared) return;

      var element = this.element;
      if (isDisplayed(element)) {
        this._prepared = true;
        return;
      }

      var originalStyles = {
        position:   element.style.position   || '',
        width:      element.style.width      || '',
        visibility: element.style.visibility || '',
        display:    element.style.display    || ''
      };

      element.store('prototype_original_styles', originalStyles);

      var position = element.getStyle('position'),
       width = element.getStyle('width');

      if (width === "0px" || width === null) {
        element.style.display = 'block';
        width = element.getStyle('width');
      }

      var context = (position === 'fixed') ? document.viewport :
       element.parentNode;

      element.setStyle({
        position:   'absolute',
        visibility: 'hidden',
        display:    'block'
      });

      var positionedWidth = element.getStyle('width');

      var newWidth;
      if (width && (positionedWidth === width)) {
        newWidth = getPixelValue(element, 'width', context);
      } else if (position === 'absolute' || position === 'fixed') {
        newWidth = getPixelValue(element, 'width', context);
      } else {
        var parent = element.parentNode, pLayout = $(parent).getLayout();

        newWidth = pLayout.get('width') -
         this.get('margin-left') -
         this.get('border-left') -
         this.get('padding-left') -
         this.get('padding-right') -
         this.get('border-right') -
         this.get('margin-right');
      }

      element.setStyle({ width: newWidth + 'px' });

      this._prepared = true;
    },

    _end: function() {
      var element = this.element;
      var originalStyles = element.retrieve('prototype_original_styles');
      element.store('prototype_original_styles', null);
      element.setStyle(originalStyles);
      this._prepared = false;
    },

    _compute: function(property) {
      var COMPUTATIONS = Element.Layout.COMPUTATIONS;
      if (!(property in COMPUTATIONS)) {
        throw "Property not found.";
      }

      return this._set(property, COMPUTATIONS[property].call(this, this.element));
    },

    toObject: function() {
      var args = $A(arguments);
      var keys = (args.length === 0) ? Element.Layout.PROPERTIES :
       args.join(' ').split(' ');
      var obj = {};
      keys.each( function(key) {
        if (!Element.Layout.PROPERTIES.include(key)) return;
        var value = this.get(key);
        if (value != null) obj[key] = value;
      }, this);
      return obj;
    },

    toHash: function() {
      var obj = this.toObject.apply(this, arguments);
      return new Hash(obj);
    },

    toCSS: function() {
      var args = $A(arguments);
      var keys = (args.length === 0) ? Element.Layout.PROPERTIES :
       args.join(' ').split(' ');
      var css = {};

      keys.each( function(key) {
        if (!Element.Layout.PROPERTIES.include(key)) return;
        if (Element.Layout.COMPOSITE_PROPERTIES.include(key)) return;

        var value = this.get(key);
        if (value != null) css[cssNameFor(key)] = value + 'px';
      }, this);
      return css;
    },

    inspect: function() {
      return "#<Element.Layout>";
    }
  });

  Object.extend(Element.Layout, {
    PROPERTIES: $w('height width top left right bottom border-left border-right border-top border-bottom padding-left padding-right padding-top padding-bottom margin-top margin-bottom margin-left margin-right padding-box-width padding-box-height border-box-width border-box-height margin-box-width margin-box-height'),

    COMPOSITE_PROPERTIES: $w('padding-box-width padding-box-height margin-box-width margin-box-height border-box-width border-box-height'),

    COMPUTATIONS: {
      'height': function(element) {
        if (!this._preComputing) this._begin();

        var bHeight = this.get('border-box-height');
        if (bHeight <= 0) {
          if (!this._preComputing) this._end();
          return 0;
        }

        var bTop = this.get('border-top'),
         bBottom = this.get('border-bottom');

        var pTop = this.get('padding-top'),
         pBottom = this.get('padding-bottom');

        if (!this._preComputing) this._end();

        return bHeight - bTop - bBottom - pTop - pBottom;
      },

      'width': function(element) {
        if (!this._preComputing) this._begin();

        var bWidth = this.get('border-box-width');
        if (bWidth <= 0) {
          if (!this._preComputing) this._end();
          return 0;
        }

        var bLeft = this.get('border-left'),
         bRight = this.get('border-right');

        var pLeft = this.get('padding-left'),
         pRight = this.get('padding-right');

        if (!this._preComputing) this._end();

        return bWidth - bLeft - bRight - pLeft - pRight;
      },

      'padding-box-height': function(element) {
        var height = this.get('height'),
         pTop = this.get('padding-top'),
         pBottom = this.get('padding-bottom');

        return height + pTop + pBottom;
      },

      'padding-box-width': function(element) {
        var width = this.get('width'),
         pLeft = this.get('padding-left'),
         pRight = this.get('padding-right');

        return width + pLeft + pRight;
      },

      'border-box-height': function(element) {
        if (!this._preComputing) this._begin();
        var height = element.offsetHeight;
        if (!this._preComputing) this._end();
        return height;
      },

      'border-box-width': function(element) {
        if (!this._preComputing) this._begin();
        var width = element.offsetWidth;
        if (!this._preComputing) this._end();
        return width;
      },

      'margin-box-height': function(element) {
        var bHeight = this.get('border-box-height'),
         mTop = this.get('margin-top'),
         mBottom = this.get('margin-bottom');

        if (bHeight <= 0) return 0;

        return bHeight + mTop + mBottom;
      },

      'margin-box-width': function(element) {
        var bWidth = this.get('border-box-width'),
         mLeft = this.get('margin-left'),
         mRight = this.get('margin-right');

        if (bWidth <= 0) return 0;

        return bWidth + mLeft + mRight;
      },

      'top': function(element) {
        var offset = element.positionedOffset();
        return offset.top;
      },

      'bottom': function(element) {
        var offset = element.positionedOffset(),
         parent = element.getOffsetParent(),
         pHeight = parent.measure('height');

        var mHeight = this.get('border-box-height');

        return pHeight - mHeight - offset.top;
      },

      'left': function(element) {
        var offset = element.positionedOffset();
        return offset.left;
      },

      'right': function(element) {
        var offset = element.positionedOffset(),
         parent = element.getOffsetParent(),
         pWidth = parent.measure('width');

        var mWidth = this.get('border-box-width');

        return pWidth - mWidth - offset.left;
      },

      'padding-top': function(element) {
        return getPixelValue(element, 'paddingTop');
      },

      'padding-bottom': function(element) {
        return getPixelValue(element, 'paddingBottom');
      },

      'padding-left': function(element) {
        return getPixelValue(element, 'paddingLeft');
      },

      'padding-right': function(element) {
        return getPixelValue(element, 'paddingRight');
      },

      'border-top': function(element) {
        return getPixelValue(element, 'borderTopWidth');
      },

      'border-bottom': function(element) {
        return getPixelValue(element, 'borderBottomWidth');
      },

      'border-left': function(element) {
        return getPixelValue(element, 'borderLeftWidth');
      },

      'border-right': function(element) {
        return getPixelValue(element, 'borderRightWidth');
      },

      'margin-top': function(element) {
        return getPixelValue(element, 'marginTop');
      },

      'margin-bottom': function(element) {
        return getPixelValue(element, 'marginBottom');
      },

      'margin-left': function(element) {
        return getPixelValue(element, 'marginLeft');
      },

      'margin-right': function(element) {
        return getPixelValue(element, 'marginRight');
      }
    }
  });

  if ('getBoundingClientRect' in document.documentElement) {
    Object.extend(Element.Layout.COMPUTATIONS, {
      'right': function(element) {
        var parent = hasLayout(element.getOffsetParent());
        var rect = element.getBoundingClientRect(),
         pRect = parent.getBoundingClientRect();

        return (pRect.right - rect.right).round();
      },

      'bottom': function(element) {
        var parent = hasLayout(element.getOffsetParent());
        var rect = element.getBoundingClientRect(),
         pRect = parent.getBoundingClientRect();

        return (pRect.bottom - rect.bottom).round();
      }
    });
  }

  Element.Offset = Class.create({
    initialize: function(left, top) {
      this.left = left.round();
      this.top  = top.round();

      this[0] = this.left;
      this[1] = this.top;
    },

    relativeTo: function(offset) {
      return new Element.Offset(
        this.left - offset.left,
        this.top  - offset.top
      );
    },

    inspect: function() {
      return "#<Element.Offset left: #{left} top: #{top}>".interpolate(this);
    },

    toString: function() {
      return "[#{left}, #{top}]".interpolate(this);
    },

    toArray: function() {
      return [this.left, this.top];
    }
  });

  function getLayout(element, preCompute) {
    return new Element.Layout(element, preCompute);
  }

  function measure(element, property) {
    return $(element).getLayout().get(property);
  }

  function getDimensions(element) {
    element = $(element);
    var display = Element.getStyle(element, 'display');

    if (display && display !== 'none') {
      return { width: element.offsetWidth, height: element.offsetHeight };
    }

    var style = element.style;
    var originalStyles = {
      visibility: style.visibility,
      position:   style.position,
      display:    style.display
    };

    var newStyles = {
      visibility: 'hidden',
      display:    'block'
    };

    if (originalStyles.position !== 'fixed')
      newStyles.position = 'absolute';

    Element.setStyle(element, newStyles);

    var dimensions = {
      width:  element.offsetWidth,
      height: element.offsetHeight
    };

    Element.setStyle(element, originalStyles);

    return dimensions;
  }

  function getOffsetParent(element) {
    element = $(element);

    if (isDocument(element) || isDetached(element) || isBody(element) || isHtml(element))
      return $(document.body);

    var isInline = (Element.getStyle(element, 'display') === 'inline');
    if (!isInline && element.offsetParent) return $(element.offsetParent);

    while ((element = element.parentNode) && element !== document.body) {
      if (Element.getStyle(element, 'position') !== 'static') {
        return isHtml(element) ? $(document.body) : $(element);
      }
    }

    return $(document.body);
  }


  function cumulativeOffset(element) {
    element = $(element);
    var valueT = 0, valueL = 0;
    if (element.parentNode) {
      do {
        valueT += element.offsetTop  || 0;
        valueL += element.offsetLeft || 0;
        element = element.offsetParent;
      } while (element);
    }
    return new Element.Offset(valueL, valueT);
  }

  function positionedOffset(element) {
    element = $(element);

    var layout = element.getLayout();

    var valueT = 0, valueL = 0;
    do {
      valueT += element.offsetTop  || 0;
      valueL += element.offsetLeft || 0;
      element = element.offsetParent;
      if (element) {
        if (isBody(element)) break;
        var p = Element.getStyle(element, 'position');
        if (p !== 'static') break;
      }
    } while (element);

    valueL -= layout.get('margin-top');
    valueT -= layout.get('margin-left');

    return new Element.Offset(valueL, valueT);
  }

  function cumulativeScrollOffset(element) {
    var valueT = 0, valueL = 0;
    do {
      valueT += element.scrollTop  || 0;
      valueL += element.scrollLeft || 0;
      element = element.parentNode;
    } while (element);
    return new Element.Offset(valueL, valueT);
  }

  function viewportOffset(forElement) {
    element = $(element);
    var valueT = 0, valueL = 0, docBody = document.body;

    var element = forElement;
    do {
      valueT += element.offsetTop  || 0;
      valueL += element.offsetLeft || 0;
      if (element.offsetParent == docBody &&
        Element.getStyle(element, 'position') == 'absolute') break;
    } while (element = element.offsetParent);

    element = forElement;
    do {
      if (element != docBody) {
        valueT -= element.scrollTop  || 0;
        valueL -= element.scrollLeft || 0;
      }
    } while (element = element.parentNode);
    return new Element.Offset(valueL, valueT);
  }

  function absolutize(element) {
    element = $(element);

    if (Element.getStyle(element, 'position') === 'absolute') {
      return element;
    }

    var offsetParent = getOffsetParent(element);
    var eOffset = element.viewportOffset(),
     pOffset = offsetParent.viewportOffset();

    var offset = eOffset.relativeTo(pOffset);
    var layout = element.getLayout();

    element.store('prototype_absolutize_original_styles', {
      left:   element.getStyle('left'),
      top:    element.getStyle('top'),
      width:  element.getStyle('width'),
      height: element.getStyle('height')
    });

    element.setStyle({
      position: 'absolute',
      top:    offset.top + 'px',
      left:   offset.left + 'px',
      width:  layout.get('width') + 'px',
      height: layout.get('height') + 'px'
    });

    return element;
  }

  function relativize(element) {
    element = $(element);
    if (Element.getStyle(element, 'position') === 'relative') {
      return element;
    }

    var originalStyles =
     element.retrieve('prototype_absolutize_original_styles');

    if (originalStyles) element.setStyle(originalStyles);
    return element;
  }

  if (Prototype.Browser.IE) {
    getOffsetParent = getOffsetParent.wrap(
      function(proceed, element) {
        element = $(element);

        if (isDocument(element) || isDetached(element) || isBody(element) || isHtml(element))
          return $(document.body);

        var position = element.getStyle('position');
        if (position !== 'static') return proceed(element);

        element.setStyle({ position: 'relative' });
        var value = proceed(element);
        element.setStyle({ position: position });
        return value;
      }
    );

    positionedOffset = positionedOffset.wrap(function(proceed, element) {
      element = $(element);
      if (!element.parentNode) return new Element.Offset(0, 0);
      var position = element.getStyle('position');
      if (position !== 'static') return proceed(element);

      var offsetParent = element.getOffsetParent();
      if (offsetParent && offsetParent.getStyle('position') === 'fixed')
        hasLayout(offsetParent);

      element.setStyle({ position: 'relative' });
      var value = proceed(element);
      element.setStyle({ position: position });
      return value;
    });
  } else if (Prototype.Browser.Webkit) {
    cumulativeOffset = function(element) {
      element = $(element);
      var valueT = 0, valueL = 0;
      do {
        valueT += element.offsetTop  || 0;
        valueL += element.offsetLeft || 0;
        if (element.offsetParent == document.body)
          if (Element.getStyle(element, 'position') == 'absolute') break;

        element = element.offsetParent;
      } while (element);

      return new Element.Offset(valueL, valueT);
    };
  }


  Element.addMethods({
    getLayout:              getLayout,
    measure:                measure,
    getDimensions:          getDimensions,
    getOffsetParent:        getOffsetParent,
    cumulativeOffset:       cumulativeOffset,
    positionedOffset:       positionedOffset,
    cumulativeScrollOffset: cumulativeScrollOffset,
    viewportOffset:         viewportOffset,
    absolutize:             absolutize,
    relativize:             relativize
  });

  function isBody(element) {
    return element.nodeName.toUpperCase() === 'BODY';
  }

  function isHtml(element) {
    return element.nodeName.toUpperCase() === 'HTML';
  }

  function isDocument(element) {
    return element.nodeType === Node.DOCUMENT_NODE;
  }

  function isDetached(element) {
    return element !== document.body &&
     !Element.descendantOf(element, document.body);
  }

  if ('getBoundingClientRect' in document.documentElement) {
    Element.addMethods({
      viewportOffset: function(element) {
        element = $(element);
        if (isDetached(element)) return new Element.Offset(0, 0);

        var rect = element.getBoundingClientRect(),
         docEl = document.documentElement;
        return new Element.Offset(rect.left - docEl.clientLeft,
         rect.top - docEl.clientTop);
      }
    });
  }
})();
window.$$ = function() {
  var expression = $A(arguments).join(', ');
  return Prototype.Selector.select(expression, document);
};

Prototype.Selector = (function() {

  function select() {
    throw new Error('Method "Prototype.Selector.select" must be defined.');
  }

  function match() {
    throw new Error('Method "Prototype.Selector.match" must be defined.');
  }

  function find(elements, expression, index) {
    index = index || 0;
    var match = Prototype.Selector.match, length = elements.length, matchIndex = 0, i;

    for (i = 0; i < length; i++) {
      if (match(elements[i], expression) && index == matchIndex++) {
        return Element.extend(elements[i]);
      }
    }
  }

  function extendElements(elements) {
    for (var i = 0, length = elements.length; i < length; i++) {
      Element.extend(elements[i]);
    }
    return elements;
  }


  var K = Prototype.K;

  return {
    select: select,
    match: match,
    find: find,
    extendElements: (Element.extend === K) ? K : extendElements,
    extendElement: Element.extend
  };
})();
/*!
 * Sizzle CSS Selector Engine - v1.0
 *  Copyright 2009, The Dojo Foundation
 *  Released under the MIT, BSD, and GPL Licenses.
 *  More information: http://sizzlejs.com/
 */
(function(){

var chunker = /((?:\((?:\([^()]+\)|[^()]+)+\)|\[(?:\[[^[\]]*\]|['"][^'"]*['"]|[^[\]'"]+)+\]|\\.|[^ >+~,(\[\\]+)+|[>+~])(\s*,\s*)?((?:.|\r|\n)*)/g,
	done = 0,
	toString = Object.prototype.toString,
	hasDuplicate = false,
	baseHasDuplicate = true;

[0, 0].sort(function(){
	baseHasDuplicate = false;
	return 0;
});

var Sizzle = function(selector, context, results, seed) {
	results = results || [];
	var origContext = context = context || document;

	if ( context.nodeType !== 1 && context.nodeType !== 9 ) {
		return [];
	}

	if ( !selector || typeof selector !== "string" ) {
		return results;
	}

	var parts = [], m, set, checkSet, check, mode, extra, prune = true, contextXML = isXML(context),
		soFar = selector;

	while ( (chunker.exec(""), m = chunker.exec(soFar)) !== null ) {
		soFar = m[3];

		parts.push( m[1] );

		if ( m[2] ) {
			extra = m[3];
			break;
		}
	}

	if ( parts.length > 1 && origPOS.exec( selector ) ) {
		if ( parts.length === 2 && Expr.relative[ parts[0] ] ) {
			set = posProcess( parts[0] + parts[1], context );
		} else {
			set = Expr.relative[ parts[0] ] ?
				[ context ] :
				Sizzle( parts.shift(), context );

			while ( parts.length ) {
				selector = parts.shift();

				if ( Expr.relative[ selector ] )
					selector += parts.shift();

				set = posProcess( selector, set );
			}
		}
	} else {
		if ( !seed && parts.length > 1 && context.nodeType === 9 && !contextXML &&
				Expr.match.ID.test(parts[0]) && !Expr.match.ID.test(parts[parts.length - 1]) ) {
			var ret = Sizzle.find( parts.shift(), context, contextXML );
			context = ret.expr ? Sizzle.filter( ret.expr, ret.set )[0] : ret.set[0];
		}

		if ( context ) {
			var ret = seed ?
				{ expr: parts.pop(), set: makeArray(seed) } :
				Sizzle.find( parts.pop(), parts.length === 1 && (parts[0] === "~" || parts[0] === "+") && context.parentNode ? context.parentNode : context, contextXML );
			set = ret.expr ? Sizzle.filter( ret.expr, ret.set ) : ret.set;

			if ( parts.length > 0 ) {
				checkSet = makeArray(set);
			} else {
				prune = false;
			}

			while ( parts.length ) {
				var cur = parts.pop(), pop = cur;

				if ( !Expr.relative[ cur ] ) {
					cur = "";
				} else {
					pop = parts.pop();
				}

				if ( pop == null ) {
					pop = context;
				}

				Expr.relative[ cur ]( checkSet, pop, contextXML );
			}
		} else {
			checkSet = parts = [];
		}
	}

	if ( !checkSet ) {
		checkSet = set;
	}

	if ( !checkSet ) {
		throw "Syntax error, unrecognized expression: " + (cur || selector);
	}

	if ( toString.call(checkSet) === "[object Array]" ) {
		if ( !prune ) {
			results.push.apply( results, checkSet );
		} else if ( context && context.nodeType === 1 ) {
			for ( var i = 0; checkSet[i] != null; i++ ) {
				if ( checkSet[i] && (checkSet[i] === true || checkSet[i].nodeType === 1 && contains(context, checkSet[i])) ) {
					results.push( set[i] );
				}
			}
		} else {
			for ( var i = 0; checkSet[i] != null; i++ ) {
				if ( checkSet[i] && checkSet[i].nodeType === 1 ) {
					results.push( set[i] );
				}
			}
		}
	} else {
		makeArray( checkSet, results );
	}

	if ( extra ) {
		Sizzle( extra, origContext, results, seed );
		Sizzle.uniqueSort( results );
	}

	return results;
};

Sizzle.uniqueSort = function(results){
	if ( sortOrder ) {
		hasDuplicate = baseHasDuplicate;
		results.sort(sortOrder);

		if ( hasDuplicate ) {
			for ( var i = 1; i < results.length; i++ ) {
				if ( results[i] === results[i-1] ) {
					results.splice(i--, 1);
				}
			}
		}
	}

	return results;
};

Sizzle.matches = function(expr, set){
	return Sizzle(expr, null, null, set);
};

Sizzle.find = function(expr, context, isXML){
	var set, match;

	if ( !expr ) {
		return [];
	}

	for ( var i = 0, l = Expr.order.length; i < l; i++ ) {
		var type = Expr.order[i], match;

		if ( (match = Expr.leftMatch[ type ].exec( expr )) ) {
			var left = match[1];
			match.splice(1,1);

			if ( left.substr( left.length - 1 ) !== "\\" ) {
				match[1] = (match[1] || "").replace(/\\/g, "");
				set = Expr.find[ type ]( match, context, isXML );
				if ( set != null ) {
					expr = expr.replace( Expr.match[ type ], "" );
					break;
				}
			}
		}
	}

	if ( !set ) {
		set = context.getElementsByTagName("*");
	}

	return {set: set, expr: expr};
};

Sizzle.filter = function(expr, set, inplace, not){
	var old = expr, result = [], curLoop = set, match, anyFound,
		isXMLFilter = set && set[0] && isXML(set[0]);

	while ( expr && set.length ) {
		for ( var type in Expr.filter ) {
			if ( (match = Expr.match[ type ].exec( expr )) != null ) {
				var filter = Expr.filter[ type ], found, item;
				anyFound = false;

				if ( curLoop == result ) {
					result = [];
				}

				if ( Expr.preFilter[ type ] ) {
					match = Expr.preFilter[ type ]( match, curLoop, inplace, result, not, isXMLFilter );

					if ( !match ) {
						anyFound = found = true;
					} else if ( match === true ) {
						continue;
					}
				}

				if ( match ) {
					for ( var i = 0; (item = curLoop[i]) != null; i++ ) {
						if ( item ) {
							found = filter( item, match, i, curLoop );
							var pass = not ^ !!found;

							if ( inplace && found != null ) {
								if ( pass ) {
									anyFound = true;
								} else {
									curLoop[i] = false;
								}
							} else if ( pass ) {
								result.push( item );
								anyFound = true;
							}
						}
					}
				}

				if ( found !== undefined ) {
					if ( !inplace ) {
						curLoop = result;
					}

					expr = expr.replace( Expr.match[ type ], "" );

					if ( !anyFound ) {
						return [];
					}

					break;
				}
			}
		}

		if ( expr == old ) {
			if ( anyFound == null ) {
				throw "Syntax error, unrecognized expression: " + expr;
			} else {
				break;
			}
		}

		old = expr;
	}

	return curLoop;
};

var Expr = Sizzle.selectors = {
	order: [ "ID", "NAME", "TAG" ],
	match: {
		ID: /#((?:[\w\u00c0-\uFFFF-]|\\.)+)/,
		CLASS: /\.((?:[\w\u00c0-\uFFFF-]|\\.)+)/,
		NAME: /\[name=['"]*((?:[\w\u00c0-\uFFFF-]|\\.)+)['"]*\]/,
		ATTR: /\[\s*((?:[\w\u00c0-\uFFFF-]|\\.)+)\s*(?:(\S?=)\s*(['"]*)(.*?)\3|)\s*\]/,
		TAG: /^((?:[\w\u00c0-\uFFFF\*-]|\\.)+)/,
		CHILD: /:(only|nth|last|first)-child(?:\((even|odd|[\dn+-]*)\))?/,
		POS: /:(nth|eq|gt|lt|first|last|even|odd)(?:\((\d*)\))?(?=[^-]|$)/,
		PSEUDO: /:((?:[\w\u00c0-\uFFFF-]|\\.)+)(?:\((['"]*)((?:\([^\)]+\)|[^\2\(\)]*)+)\2\))?/
	},
	leftMatch: {},
	attrMap: {
		"class": "className",
		"for": "htmlFor"
	},
	attrHandle: {
		href: function(elem){
			return elem.getAttribute("href");
		}
	},
	relative: {
		"+": function(checkSet, part, isXML){
			var isPartStr = typeof part === "string",
				isTag = isPartStr && !/\W/.test(part),
				isPartStrNotTag = isPartStr && !isTag;

			if ( isTag && !isXML ) {
				part = part.toUpperCase();
			}

			for ( var i = 0, l = checkSet.length, elem; i < l; i++ ) {
				if ( (elem = checkSet[i]) ) {
					while ( (elem = elem.previousSibling) && elem.nodeType !== 1 ) {}

					checkSet[i] = isPartStrNotTag || elem && elem.nodeName === part ?
						elem || false :
						elem === part;
				}
			}

			if ( isPartStrNotTag ) {
				Sizzle.filter( part, checkSet, true );
			}
		},
		">": function(checkSet, part, isXML){
			var isPartStr = typeof part === "string";

			if ( isPartStr && !/\W/.test(part) ) {
				part = isXML ? part : part.toUpperCase();

				for ( var i = 0, l = checkSet.length; i < l; i++ ) {
					var elem = checkSet[i];
					if ( elem ) {
						var parent = elem.parentNode;
						checkSet[i] = parent.nodeName === part ? parent : false;
					}
				}
			} else {
				for ( var i = 0, l = checkSet.length; i < l; i++ ) {
					var elem = checkSet[i];
					if ( elem ) {
						checkSet[i] = isPartStr ?
							elem.parentNode :
							elem.parentNode === part;
					}
				}

				if ( isPartStr ) {
					Sizzle.filter( part, checkSet, true );
				}
			}
		},
		"": function(checkSet, part, isXML){
			var doneName = done++, checkFn = dirCheck;

			if ( !/\W/.test(part) ) {
				var nodeCheck = part = isXML ? part : part.toUpperCase();
				checkFn = dirNodeCheck;
			}

			checkFn("parentNode", part, doneName, checkSet, nodeCheck, isXML);
		},
		"~": function(checkSet, part, isXML){
			var doneName = done++, checkFn = dirCheck;

			if ( typeof part === "string" && !/\W/.test(part) ) {
				var nodeCheck = part = isXML ? part : part.toUpperCase();
				checkFn = dirNodeCheck;
			}

			checkFn("previousSibling", part, doneName, checkSet, nodeCheck, isXML);
		}
	},
	find: {
		ID: function(match, context, isXML){
			if ( typeof context.getElementById !== "undefined" && !isXML ) {
				var m = context.getElementById(match[1]);
				return m ? [m] : [];
			}
		},
		NAME: function(match, context, isXML){
			if ( typeof context.getElementsByName !== "undefined" ) {
				var ret = [], results = context.getElementsByName(match[1]);

				for ( var i = 0, l = results.length; i < l; i++ ) {
					if ( results[i].getAttribute("name") === match[1] ) {
						ret.push( results[i] );
					}
				}

				return ret.length === 0 ? null : ret;
			}
		},
		TAG: function(match, context){
			return context.getElementsByTagName(match[1]);
		}
	},
	preFilter: {
		CLASS: function(match, curLoop, inplace, result, not, isXML){
			match = " " + match[1].replace(/\\/g, "") + " ";

			if ( isXML ) {
				return match;
			}

			for ( var i = 0, elem; (elem = curLoop[i]) != null; i++ ) {
				if ( elem ) {
					if ( not ^ (elem.className && (" " + elem.className + " ").indexOf(match) >= 0) ) {
						if ( !inplace )
							result.push( elem );
					} else if ( inplace ) {
						curLoop[i] = false;
					}
				}
			}

			return false;
		},
		ID: function(match){
			return match[1].replace(/\\/g, "");
		},
		TAG: function(match, curLoop){
			for ( var i = 0; curLoop[i] === false; i++ ){}
			return curLoop[i] && isXML(curLoop[i]) ? match[1] : match[1].toUpperCase();
		},
		CHILD: function(match){
			if ( match[1] == "nth" ) {
				var test = /(-?)(\d*)n((?:\+|-)?\d*)/.exec(
					match[2] == "even" && "2n" || match[2] == "odd" && "2n+1" ||
					!/\D/.test( match[2] ) && "0n+" + match[2] || match[2]);

				match[2] = (test[1] + (test[2] || 1)) - 0;
				match[3] = test[3] - 0;
			}

			match[0] = done++;

			return match;
		},
		ATTR: function(match, curLoop, inplace, result, not, isXML){
			var name = match[1].replace(/\\/g, "");

			if ( !isXML && Expr.attrMap[name] ) {
				match[1] = Expr.attrMap[name];
			}

			if ( match[2] === "~=" ) {
				match[4] = " " + match[4] + " ";
			}

			return match;
		},
		PSEUDO: function(match, curLoop, inplace, result, not){
			if ( match[1] === "not" ) {
				if ( ( chunker.exec(match[3]) || "" ).length > 1 || /^\w/.test(match[3]) ) {
					match[3] = Sizzle(match[3], null, null, curLoop);
				} else {
					var ret = Sizzle.filter(match[3], curLoop, inplace, true ^ not);
					if ( !inplace ) {
						result.push.apply( result, ret );
					}
					return false;
				}
			} else if ( Expr.match.POS.test( match[0] ) || Expr.match.CHILD.test( match[0] ) ) {
				return true;
			}

			return match;
		},
		POS: function(match){
			match.unshift( true );
			return match;
		}
	},
	filters: {
		enabled: function(elem){
			return elem.disabled === false && elem.type !== "hidden";
		},
		disabled: function(elem){
			return elem.disabled === true;
		},
		checked: function(elem){
			return elem.checked === true;
		},
		selected: function(elem){
			elem.parentNode.selectedIndex;
			return elem.selected === true;
		},
		parent: function(elem){
			return !!elem.firstChild;
		},
		empty: function(elem){
			return !elem.firstChild;
		},
		has: function(elem, i, match){
			return !!Sizzle( match[3], elem ).length;
		},
		header: function(elem){
			return /h\d/i.test( elem.nodeName );
		},
		text: function(elem){
			return "text" === elem.type;
		},
		radio: function(elem){
			return "radio" === elem.type;
		},
		checkbox: function(elem){
			return "checkbox" === elem.type;
		},
		file: function(elem){
			return "file" === elem.type;
		},
		password: function(elem){
			return "password" === elem.type;
		},
		submit: function(elem){
			return "submit" === elem.type;
		},
		image: function(elem){
			return "image" === elem.type;
		},
		reset: function(elem){
			return "reset" === elem.type;
		},
		button: function(elem){
			return "button" === elem.type || elem.nodeName.toUpperCase() === "BUTTON";
		},
		input: function(elem){
			return /input|select|textarea|button/i.test(elem.nodeName);
		}
	},
	setFilters: {
		first: function(elem, i){
			return i === 0;
		},
		last: function(elem, i, match, array){
			return i === array.length - 1;
		},
		even: function(elem, i){
			return i % 2 === 0;
		},
		odd: function(elem, i){
			return i % 2 === 1;
		},
		lt: function(elem, i, match){
			return i < match[3] - 0;
		},
		gt: function(elem, i, match){
			return i > match[3] - 0;
		},
		nth: function(elem, i, match){
			return match[3] - 0 == i;
		},
		eq: function(elem, i, match){
			return match[3] - 0 == i;
		}
	},
	filter: {
		PSEUDO: function(elem, match, i, array){
			var name = match[1], filter = Expr.filters[ name ];

			if ( filter ) {
				return filter( elem, i, match, array );
			} else if ( name === "contains" ) {
				return (elem.textContent || elem.innerText || "").indexOf(match[3]) >= 0;
			} else if ( name === "not" ) {
				var not = match[3];

				for ( var i = 0, l = not.length; i < l; i++ ) {
					if ( not[i] === elem ) {
						return false;
					}
				}

				return true;
			}
		},
		CHILD: function(elem, match){
			var type = match[1], node = elem;
			switch (type) {
				case 'only':
				case 'first':
					while ( (node = node.previousSibling) )  {
						if ( node.nodeType === 1 ) return false;
					}
					if ( type == 'first') return true;
					node = elem;
				case 'last':
					while ( (node = node.nextSibling) )  {
						if ( node.nodeType === 1 ) return false;
					}
					return true;
				case 'nth':
					var first = match[2], last = match[3];

					if ( first == 1 && last == 0 ) {
						return true;
					}

					var doneName = match[0],
						parent = elem.parentNode;

					if ( parent && (parent.sizcache !== doneName || !elem.nodeIndex) ) {
						var count = 0;
						for ( node = parent.firstChild; node; node = node.nextSibling ) {
							if ( node.nodeType === 1 ) {
								node.nodeIndex = ++count;
							}
						}
						parent.sizcache = doneName;
					}

					var diff = elem.nodeIndex - last;
					if ( first == 0 ) {
						return diff == 0;
					} else {
						return ( diff % first == 0 && diff / first >= 0 );
					}
			}
		},
		ID: function(elem, match){
			return elem.nodeType === 1 && elem.getAttribute("id") === match;
		},
		TAG: function(elem, match){
			return (match === "*" && elem.nodeType === 1) || elem.nodeName === match;
		},
		CLASS: function(elem, match){
			return (" " + (elem.className || elem.getAttribute("class")) + " ")
				.indexOf( match ) > -1;
		},
		ATTR: function(elem, match){
			var name = match[1],
				result = Expr.attrHandle[ name ] ?
					Expr.attrHandle[ name ]( elem ) :
					elem[ name ] != null ?
						elem[ name ] :
						elem.getAttribute( name ),
				value = result + "",
				type = match[2],
				check = match[4];

			return result == null ?
				type === "!=" :
				type === "=" ?
				value === check :
				type === "*=" ?
				value.indexOf(check) >= 0 :
				type === "~=" ?
				(" " + value + " ").indexOf(check) >= 0 :
				!check ?
				value && result !== false :
				type === "!=" ?
				value != check :
				type === "^=" ?
				value.indexOf(check) === 0 :
				type === "$=" ?
				value.substr(value.length - check.length) === check :
				type === "|=" ?
				value === check || value.substr(0, check.length + 1) === check + "-" :
				false;
		},
		POS: function(elem, match, i, array){
			var name = match[2], filter = Expr.setFilters[ name ];

			if ( filter ) {
				return filter( elem, i, match, array );
			}
		}
	}
};

var origPOS = Expr.match.POS;

for ( var type in Expr.match ) {
	Expr.match[ type ] = new RegExp( Expr.match[ type ].source + /(?![^\[]*\])(?![^\(]*\))/.source );
	Expr.leftMatch[ type ] = new RegExp( /(^(?:.|\r|\n)*?)/.source + Expr.match[ type ].source );
}

var makeArray = function(array, results) {
	array = Array.prototype.slice.call( array, 0 );

	if ( results ) {
		results.push.apply( results, array );
		return results;
	}

	return array;
};

try {
	Array.prototype.slice.call( document.documentElement.childNodes, 0 );

} catch(e){
	makeArray = function(array, results) {
		var ret = results || [];

		if ( toString.call(array) === "[object Array]" ) {
			Array.prototype.push.apply( ret, array );
		} else {
			if ( typeof array.length === "number" ) {
				for ( var i = 0, l = array.length; i < l; i++ ) {
					ret.push( array[i] );
				}
			} else {
				for ( var i = 0; array[i]; i++ ) {
					ret.push( array[i] );
				}
			}
		}

		return ret;
	};
}

var sortOrder;

if ( document.documentElement.compareDocumentPosition ) {
	sortOrder = function( a, b ) {
		if ( !a.compareDocumentPosition || !b.compareDocumentPosition ) {
			if ( a == b ) {
				hasDuplicate = true;
			}
			return 0;
		}

		var ret = a.compareDocumentPosition(b) & 4 ? -1 : a === b ? 0 : 1;
		if ( ret === 0 ) {
			hasDuplicate = true;
		}
		return ret;
	};
} else if ( "sourceIndex" in document.documentElement ) {
	sortOrder = function( a, b ) {
		if ( !a.sourceIndex || !b.sourceIndex ) {
			if ( a == b ) {
				hasDuplicate = true;
			}
			return 0;
		}

		var ret = a.sourceIndex - b.sourceIndex;
		if ( ret === 0 ) {
			hasDuplicate = true;
		}
		return ret;
	};
} else if ( document.createRange ) {
	sortOrder = function( a, b ) {
		if ( !a.ownerDocument || !b.ownerDocument ) {
			if ( a == b ) {
				hasDuplicate = true;
			}
			return 0;
		}

		var aRange = a.ownerDocument.createRange(), bRange = b.ownerDocument.createRange();
		aRange.setStart(a, 0);
		aRange.setEnd(a, 0);
		bRange.setStart(b, 0);
		bRange.setEnd(b, 0);
		var ret = aRange.compareBoundaryPoints(Range.START_TO_END, bRange);
		if ( ret === 0 ) {
			hasDuplicate = true;
		}
		return ret;
	};
}

(function(){
	var form = document.createElement("div"),
		id = "script" + (new Date).getTime();
	form.innerHTML = "<a name='" + id + "'/>";

	var root = document.documentElement;
	root.insertBefore( form, root.firstChild );

	if ( !!document.getElementById( id ) ) {
		Expr.find.ID = function(match, context, isXML){
			if ( typeof context.getElementById !== "undefined" && !isXML ) {
				var m = context.getElementById(match[1]);
				return m ? m.id === match[1] || typeof m.getAttributeNode !== "undefined" && m.getAttributeNode("id").nodeValue === match[1] ? [m] : undefined : [];
			}
		};

		Expr.filter.ID = function(elem, match){
			var node = typeof elem.getAttributeNode !== "undefined" && elem.getAttributeNode("id");
			return elem.nodeType === 1 && node && node.nodeValue === match;
		};
	}

	root.removeChild( form );
	root = form = null; // release memory in IE
})();

(function(){

	var div = document.createElement("div");
	div.appendChild( document.createComment("") );

	if ( div.getElementsByTagName("*").length > 0 ) {
		Expr.find.TAG = function(match, context){
			var results = context.getElementsByTagName(match[1]);

			if ( match[1] === "*" ) {
				var tmp = [];

				for ( var i = 0; results[i]; i++ ) {
					if ( results[i].nodeType === 1 ) {
						tmp.push( results[i] );
					}
				}

				results = tmp;
			}

			return results;
		};
	}

	div.innerHTML = "<a href='#'></a>";
	if ( div.firstChild && typeof div.firstChild.getAttribute !== "undefined" &&
			div.firstChild.getAttribute("href") !== "#" ) {
		Expr.attrHandle.href = function(elem){
			return elem.getAttribute("href", 2);
		};
	}

	div = null; // release memory in IE
})();

if ( document.querySelectorAll ) (function(){
	var oldSizzle = Sizzle, div = document.createElement("div");
	div.innerHTML = "<p class='TEST'></p>";

	if ( div.querySelectorAll && div.querySelectorAll(".TEST").length === 0 ) {
		return;
	}

	Sizzle = function(query, context, extra, seed){
		context = context || document;

		if ( !seed && context.nodeType === 9 && !isXML(context) ) {
			try {
				return makeArray( context.querySelectorAll(query), extra );
			} catch(e){}
		}

		return oldSizzle(query, context, extra, seed);
	};

	for ( var prop in oldSizzle ) {
		Sizzle[ prop ] = oldSizzle[ prop ];
	}

	div = null; // release memory in IE
})();

if ( document.getElementsByClassName && document.documentElement.getElementsByClassName ) (function(){
	var div = document.createElement("div");
	div.innerHTML = "<div class='test e'></div><div class='test'></div>";

	if ( div.getElementsByClassName("e").length === 0 )
		return;

	div.lastChild.className = "e";

	if ( div.getElementsByClassName("e").length === 1 )
		return;

	Expr.order.splice(1, 0, "CLASS");
	Expr.find.CLASS = function(match, context, isXML) {
		if ( typeof context.getElementsByClassName !== "undefined" && !isXML ) {
			return context.getElementsByClassName(match[1]);
		}
	};

	div = null; // release memory in IE
})();

function dirNodeCheck( dir, cur, doneName, checkSet, nodeCheck, isXML ) {
	var sibDir = dir == "previousSibling" && !isXML;
	for ( var i = 0, l = checkSet.length; i < l; i++ ) {
		var elem = checkSet[i];
		if ( elem ) {
			if ( sibDir && elem.nodeType === 1 ){
				elem.sizcache = doneName;
				elem.sizset = i;
			}
			elem = elem[dir];
			var match = false;

			while ( elem ) {
				if ( elem.sizcache === doneName ) {
					match = checkSet[elem.sizset];
					break;
				}

				if ( elem.nodeType === 1 && !isXML ){
					elem.sizcache = doneName;
					elem.sizset = i;
				}

				if ( elem.nodeName === cur ) {
					match = elem;
					break;
				}

				elem = elem[dir];
			}

			checkSet[i] = match;
		}
	}
}

function dirCheck( dir, cur, doneName, checkSet, nodeCheck, isXML ) {
	var sibDir = dir == "previousSibling" && !isXML;
	for ( var i = 0, l = checkSet.length; i < l; i++ ) {
		var elem = checkSet[i];
		if ( elem ) {
			if ( sibDir && elem.nodeType === 1 ) {
				elem.sizcache = doneName;
				elem.sizset = i;
			}
			elem = elem[dir];
			var match = false;

			while ( elem ) {
				if ( elem.sizcache === doneName ) {
					match = checkSet[elem.sizset];
					break;
				}

				if ( elem.nodeType === 1 ) {
					if ( !isXML ) {
						elem.sizcache = doneName;
						elem.sizset = i;
					}
					if ( typeof cur !== "string" ) {
						if ( elem === cur ) {
							match = true;
							break;
						}

					} else if ( Sizzle.filter( cur, [elem] ).length > 0 ) {
						match = elem;
						break;
					}
				}

				elem = elem[dir];
			}

			checkSet[i] = match;
		}
	}
}

var contains = document.compareDocumentPosition ?  function(a, b){
	return a.compareDocumentPosition(b) & 16;
} : function(a, b){
	return a !== b && (a.contains ? a.contains(b) : true);
};

var isXML = function(elem){
	return elem.nodeType === 9 && elem.documentElement.nodeName !== "HTML" ||
		!!elem.ownerDocument && elem.ownerDocument.documentElement.nodeName !== "HTML";
};

var posProcess = function(selector, context){
	var tmpSet = [], later = "", match,
		root = context.nodeType ? [context] : context;

	while ( (match = Expr.match.PSEUDO.exec( selector )) ) {
		later += match[0];
		selector = selector.replace( Expr.match.PSEUDO, "" );
	}

	selector = Expr.relative[selector] ? selector + "*" : selector;

	for ( var i = 0, l = root.length; i < l; i++ ) {
		Sizzle( selector, root[i], tmpSet );
	}

	return Sizzle.filter( later, tmpSet );
};


window.Sizzle = Sizzle;

})();

Prototype._original_property = window.Sizzle;

;(function(engine) {
  var extendElements = Prototype.Selector.extendElements;

  function select(selector, scope) {
    return extendElements(engine(selector, scope || document));
  }

  function match(element, selector) {
    return engine.matches(selector, [element]).length == 1;
  }

  Prototype.Selector.engine = engine;
  Prototype.Selector.select = select;
  Prototype.Selector.match = match;
})(Sizzle);

window.Sizzle = Prototype._original_property;
delete Prototype._original_property;

var Form = {
  reset: function(form) {
    form = $(form);
    form.reset();
    return form;
  },

  serializeElements: function(elements, options) {
    if (typeof options != 'object') options = { hash: !!options };
    else if (Object.isUndefined(options.hash)) options.hash = true;
    var key, value, submitted = false, submit = options.submit, accumulator, initial;

    if (options.hash) {
      initial = {};
      accumulator = function(result, key, value) {
        if (key in result) {
          if (!Object.isArray(result[key])) result[key] = [result[key]];
          result[key].push(value);
        } else result[key] = value;
        return result;
      };
    } else {
      initial = '';
      accumulator = function(result, key, value) {
        return result + (result ? '&' : '') + encodeURIComponent(key) + '=' + encodeURIComponent(value);
      }
    }

    return elements.inject(initial, function(result, element) {
      if (!element.disabled && element.name) {
        key = element.name; value = $(element).getValue();
        if (value != null && element.type != 'file' && (element.type != 'submit' || (!submitted &&
            submit !== false && (!submit || key == submit) && (submitted = true)))) {
          result = accumulator(result, key, value);
        }
      }
      return result;
    });
  }
};

Form.Methods = {
  serialize: function(form, options) {
    return Form.serializeElements(Form.getElements(form), options);
  },

  getElements: function(form) {
    var elements = $(form).getElementsByTagName('*'),
        element,
        arr = [ ],
        serializers = Form.Element.Serializers;
    for (var i = 0; element = elements[i]; i++) {
      arr.push(element);
    }
    return arr.inject([], function(elements, child) {
      if (serializers[child.tagName.toLowerCase()])
        elements.push(Element.extend(child));
      return elements;
    })
  },

  getInputs: function(form, typeName, name) {
    form = $(form);
    var inputs = form.getElementsByTagName('input');

    if (!typeName && !name) return $A(inputs).map(Element.extend);

    for (var i = 0, matchingInputs = [], length = inputs.length; i < length; i++) {
      var input = inputs[i];
      if ((typeName && input.type != typeName) || (name && input.name != name))
        continue;
      matchingInputs.push(Element.extend(input));
    }

    return matchingInputs;
  },

  disable: function(form) {
    form = $(form);
    Form.getElements(form).invoke('disable');
    return form;
  },

  enable: function(form) {
    form = $(form);
    Form.getElements(form).invoke('enable');
    return form;
  },

  findFirstElement: function(form) {
    var elements = $(form).getElements().findAll(function(element) {
      return 'hidden' != element.type && !element.disabled;
    });
    var firstByIndex = elements.findAll(function(element) {
      return element.hasAttribute('tabIndex') && element.tabIndex >= 0;
    }).sortBy(function(element) { return element.tabIndex }).first();

    return firstByIndex ? firstByIndex : elements.find(function(element) {
      return /^(?:input|select|textarea)$/i.test(element.tagName);
    });
  },

  focusFirstElement: function(form) {
    form = $(form);
    var element = form.findFirstElement();
    if (element) element.activate();
    return form;
  },

  request: function(form, options) {
    form = $(form), options = Object.clone(options || { });

    var params = options.parameters, action = form.readAttribute('action') || '';
    if (action.blank()) action = window.location.href;
    options.parameters = form.serialize(true);

    if (params) {
      if (Object.isString(params)) params = params.toQueryParams();
      Object.extend(options.parameters, params);
    }

    if (form.hasAttribute('method') && !options.method)
      options.method = form.method;

    return new Ajax.Request(action, options);
  }
};

/*--------------------------------------------------------------------------*/


Form.Element = {
  focus: function(element) {
    $(element).focus();
    return element;
  },

  select: function(element) {
    $(element).select();
    return element;
  }
};

Form.Element.Methods = {

  serialize: function(element) {
    element = $(element);
    if (!element.disabled && element.name) {
      var value = element.getValue();
      if (value != undefined) {
        var pair = { };
        pair[element.name] = value;
        return Object.toQueryString(pair);
      }
    }
    return '';
  },

  getValue: function(element) {
    element = $(element);
    var method = element.tagName.toLowerCase();
    return Form.Element.Serializers[method](element);
  },

  setValue: function(element, value) {
    element = $(element);
    var method = element.tagName.toLowerCase();
    Form.Element.Serializers[method](element, value);
    return element;
  },

  clear: function(element) {
    $(element).value = '';
    return element;
  },

  present: function(element) {
    return $(element).value != '';
  },

  activate: function(element) {
    element = $(element);
    try {
      element.focus();
      if (element.select && (element.tagName.toLowerCase() != 'input' ||
          !(/^(?:button|reset|submit)$/i.test(element.type))))
        element.select();
    } catch (e) { }
    return element;
  },

  disable: function(element) {
    element = $(element);
    element.disabled = true;
    return element;
  },

  enable: function(element) {
    element = $(element);
    element.disabled = false;
    return element;
  }
};

/*--------------------------------------------------------------------------*/

var Field = Form.Element;

var $F = Form.Element.Methods.getValue;

/*--------------------------------------------------------------------------*/

Form.Element.Serializers = (function() {
  function input(element, value) {
    switch (element.type.toLowerCase()) {
      case 'checkbox':
      case 'radio':
        return inputSelector(element, value);
      default:
        return valueSelector(element, value);
    }
  }

  function inputSelector(element, value) {
    if (Object.isUndefined(value))
      return element.checked ? element.value : null;
    else element.checked = !!value;
  }

  function valueSelector(element, value) {
    if (Object.isUndefined(value)) return element.value;
    else element.value = value;
  }

  function select(element, value) {
    if (Object.isUndefined(value))
      return (element.type === 'select-one' ? selectOne : selectMany)(element);

    var opt, currentValue, single = !Object.isArray(value);
    for (var i = 0, length = element.length; i < length; i++) {
      opt = element.options[i];
      currentValue = this.optionValue(opt);
      if (single) {
        if (currentValue == value) {
          opt.selected = true;
          return;
        }
      }
      else opt.selected = value.include(currentValue);
    }
  }

  function selectOne(element) {
    var index = element.selectedIndex;
    return index >= 0 ? optionValue(element.options[index]) : null;
  }

  function selectMany(element) {
    var values, length = element.length;
    if (!length) return null;

    for (var i = 0, values = []; i < length; i++) {
      var opt = element.options[i];
      if (opt.selected) values.push(optionValue(opt));
    }
    return values;
  }

  function optionValue(opt) {
    return Element.hasAttribute(opt, 'value') ? opt.value : opt.text;
  }

  return {
    input:         input,
    inputSelector: inputSelector,
    textarea:      valueSelector,
    select:        select,
    selectOne:     selectOne,
    selectMany:    selectMany,
    optionValue:   optionValue,
    button:        valueSelector
  };
})();

/*--------------------------------------------------------------------------*/


Abstract.TimedObserver = Class.create(PeriodicalExecuter, {
  initialize: function($super, element, frequency, callback) {
    $super(callback, frequency);
    this.element   = $(element);
    this.lastValue = this.getValue();
  },

  execute: function() {
    var value = this.getValue();
    if (Object.isString(this.lastValue) && Object.isString(value) ?
        this.lastValue != value : String(this.lastValue) != String(value)) {
      this.callback(this.element, value);
      this.lastValue = value;
    }
  }
});

Form.Element.Observer = Class.create(Abstract.TimedObserver, {
  getValue: function() {
    return Form.Element.getValue(this.element);
  }
});

Form.Observer = Class.create(Abstract.TimedObserver, {
  getValue: function() {
    return Form.serialize(this.element);
  }
});

/*--------------------------------------------------------------------------*/

Abstract.EventObserver = Class.create({
  initialize: function(element, callback) {
    this.element  = $(element);
    this.callback = callback;

    this.lastValue = this.getValue();
    if (this.element.tagName.toLowerCase() == 'form')
      this.registerFormCallbacks();
    else
      this.registerCallback(this.element);
  },

  onElementEvent: function() {
    var value = this.getValue();
    if (this.lastValue != value) {
      this.callback(this.element, value);
      this.lastValue = value;
    }
  },

  registerFormCallbacks: function() {
    Form.getElements(this.element).each(this.registerCallback, this);
  },

  registerCallback: function(element) {
    if (element.type) {
      switch (element.type.toLowerCase()) {
        case 'checkbox':
        case 'radio':
          Event.observe(element, 'click', this.onElementEvent.bind(this));
          break;
        default:
          Event.observe(element, 'change', this.onElementEvent.bind(this));
          break;
      }
    }
  }
});

Form.Element.EventObserver = Class.create(Abstract.EventObserver, {
  getValue: function() {
    return Form.Element.getValue(this.element);
  }
});

Form.EventObserver = Class.create(Abstract.EventObserver, {
  getValue: function() {
    return Form.serialize(this.element);
  }
});
(function() {

  var Event = {
    KEY_BACKSPACE: 8,
    KEY_TAB:       9,
    KEY_RETURN:   13,
    KEY_ESC:      27,
    KEY_LEFT:     37,
    KEY_UP:       38,
    KEY_RIGHT:    39,
    KEY_DOWN:     40,
    KEY_DELETE:   46,
    KEY_HOME:     36,
    KEY_END:      35,
    KEY_PAGEUP:   33,
    KEY_PAGEDOWN: 34,
    KEY_INSERT:   45,

    cache: {}
  };

  var docEl = document.documentElement;
  var MOUSEENTER_MOUSELEAVE_EVENTS_SUPPORTED = 'onmouseenter' in docEl
    && 'onmouseleave' in docEl;



  var isIELegacyEvent = function(event) { return false; };

  if (window.attachEvent) {
    if (window.addEventListener) {
      isIELegacyEvent = function(event) {
        return !(event instanceof window.Event);
      };
    } else {
      isIELegacyEvent = function(event) { return true; };
    }
  }

  var _isButton;

  function _isButtonForDOMEvents(event, code) {
    return event.which ? (event.which === code + 1) : (event.button === code);
  }

  var legacyButtonMap = { 0: 1, 1: 4, 2: 2 };
  function _isButtonForLegacyEvents(event, code) {
    return event.button === legacyButtonMap[code];
  }

  function _isButtonForWebKit(event, code) {
    switch (code) {
      case 0: return event.which == 1 && !event.metaKey;
      case 1: return event.which == 2 || (event.which == 1 && event.metaKey);
      case 2: return event.which == 3;
      default: return false;
    }
  }

  if (window.attachEvent) {
    if (!window.addEventListener) {
      _isButton = _isButtonForLegacyEvents;
    } else {
      _isButton = function(event, code) {
        return isIELegacyEvent(event) ? _isButtonForLegacyEvents(event, code) :
         _isButtonForDOMEvents(event, code);
      }
    }
  } else if (Prototype.Browser.WebKit) {
    _isButton = _isButtonForWebKit;
  } else {
    _isButton = _isButtonForDOMEvents;
  }

  function isLeftClick(event)   { return _isButton(event, 0) }

  function isMiddleClick(event) { return _isButton(event, 1) }

  function isRightClick(event)  { return _isButton(event, 2) }

  function element(event) {
    event = Event.extend(event);

    var node = event.target, type = event.type,
     currentTarget = event.currentTarget;

    if (currentTarget && currentTarget.tagName) {
      if (type === 'load' || type === 'error' ||
        (type === 'click' && currentTarget.tagName.toLowerCase() === 'input'
          && currentTarget.type === 'radio'))
            node = currentTarget;
    }

    if (node.nodeType == Node.TEXT_NODE)
      node = node.parentNode;

    return Element.extend(node);
  }

  function findElement(event, expression) {
    var element = Event.element(event);

    if (!expression) return element;
    while (element) {
      if (Object.isElement(element) && Prototype.Selector.match(element, expression)) {
        return Element.extend(element);
      }
      element = element.parentNode;
    }
  }

  function pointer(event) {
    return { x: pointerX(event), y: pointerY(event) };
  }

  function pointerX(event) {
    var docElement = document.documentElement,
     body = document.body || { scrollLeft: 0 };

    return event.pageX || (event.clientX +
      (docElement.scrollLeft || body.scrollLeft) -
      (docElement.clientLeft || 0));
  }

  function pointerY(event) {
    var docElement = document.documentElement,
     body = document.body || { scrollTop: 0 };

    return  event.pageY || (event.clientY +
       (docElement.scrollTop || body.scrollTop) -
       (docElement.clientTop || 0));
  }


  function stop(event) {
    Event.extend(event);
    event.preventDefault();
    event.stopPropagation();

    event.stopped = true;
  }


  Event.Methods = {
    isLeftClick:   isLeftClick,
    isMiddleClick: isMiddleClick,
    isRightClick:  isRightClick,

    element:     element,
    findElement: findElement,

    pointer:  pointer,
    pointerX: pointerX,
    pointerY: pointerY,

    stop: stop
  };

  var methods = Object.keys(Event.Methods).inject({ }, function(m, name) {
    m[name] = Event.Methods[name].methodize();
    return m;
  });

  if (window.attachEvent) {
    function _relatedTarget(event) {
      var element;
      switch (event.type) {
        case 'mouseover':
        case 'mouseenter':
          element = event.fromElement;
          break;
        case 'mouseout':
        case 'mouseleave':
          element = event.toElement;
          break;
        default:
          return null;
      }
      return Element.extend(element);
    }

    var additionalMethods = {
      stopPropagation: function() { this.cancelBubble = true },
      preventDefault:  function() { this.returnValue = false },
      inspect: function() { return '[object Event]' }
    };

    Event.extend = function(event, element) {
      if (!event) return false;

      if (!isIELegacyEvent(event)) return event;

      if (event._extendedByPrototype) return event;
      event._extendedByPrototype = Prototype.emptyFunction;

      var pointer = Event.pointer(event);

      Object.extend(event, {
        target: event.srcElement || element,
        relatedTarget: _relatedTarget(event),
        pageX:  pointer.x,
        pageY:  pointer.y
      });

      Object.extend(event, methods);
      Object.extend(event, additionalMethods);

      return event;
    };
  } else {
    Event.extend = Prototype.K;
  }

  if (window.addEventListener) {
    Event.prototype = window.Event.prototype || document.createEvent('HTMLEvents').__proto__;
    Object.extend(Event.prototype, methods);
  }

  function _createResponder(element, eventName, handler) {
    var registry = Element.retrieve(element, 'prototype_event_registry');

    if (Object.isUndefined(registry)) {
      CACHE.push(element);
      registry = Element.retrieve(element, 'prototype_event_registry', $H());
    }

    var respondersForEvent = registry.get(eventName);
    if (Object.isUndefined(respondersForEvent)) {
      respondersForEvent = [];
      registry.set(eventName, respondersForEvent);
    }

    if (respondersForEvent.pluck('handler').include(handler)) return false;

    var responder;
    if (eventName.include(":")) {
      responder = function(event) {
        if (Object.isUndefined(event.eventName))
          return false;

        if (event.eventName !== eventName)
          return false;

        Event.extend(event, element);
        handler.call(element, event);
      };
    } else {
      if (!MOUSEENTER_MOUSELEAVE_EVENTS_SUPPORTED &&
       (eventName === "mouseenter" || eventName === "mouseleave")) {
        if (eventName === "mouseenter" || eventName === "mouseleave") {
          responder = function(event) {
            Event.extend(event, element);

            var parent = event.relatedTarget;
            while (parent && parent !== element) {
              try { parent = parent.parentNode; }
              catch(e) { parent = element; }
            }

            if (parent === element) return;

            handler.call(element, event);
          };
        }
      } else {
        responder = function(event) {
          Event.extend(event, element);
          handler.call(element, event);
        };
      }
    }

    responder.handler = handler;
    respondersForEvent.push(responder);
    return responder;
  }

  function _destroyCache() {
    for (var i = 0, length = CACHE.length; i < length; i++) {
      Event.stopObserving(CACHE[i]);
      CACHE[i] = null;
    }
  }

  var CACHE = [];

  if (Prototype.Browser.IE)
    window.attachEvent('onunload', _destroyCache);

  if (Prototype.Browser.WebKit)
    window.addEventListener('unload', Prototype.emptyFunction, false);


  var _getDOMEventName = Prototype.K,
      translations = { mouseenter: "mouseover", mouseleave: "mouseout" };

  if (!MOUSEENTER_MOUSELEAVE_EVENTS_SUPPORTED) {
    _getDOMEventName = function(eventName) {
      return (translations[eventName] || eventName);
    };
  }

  function observe(element, eventName, handler) {
    element = $(element);

    var responder = _createResponder(element, eventName, handler);

    if (!responder) return element;

    if (eventName.include(':')) {
      if (element.addEventListener)
        element.addEventListener("dataavailable", responder, false);
      else {
        element.attachEvent("ondataavailable", responder);
        element.attachEvent("onlosecapture", responder);
      }
    } else {
      var actualEventName = _getDOMEventName(eventName);

      if (element.addEventListener)
        element.addEventListener(actualEventName, responder, false);
      else
        element.attachEvent("on" + actualEventName, responder);
    }

    return element;
  }

  function stopObserving(element, eventName, handler) {
    element = $(element);

    var registry = Element.retrieve(element, 'prototype_event_registry');
    if (!registry) return element;

    if (!eventName) {
      registry.each( function(pair) {
        var eventName = pair.key;
        stopObserving(element, eventName);
      });
      return element;
    }

    var responders = registry.get(eventName);
    if (!responders) return element;

    if (!handler) {
      responders.each(function(r) {
        stopObserving(element, eventName, r.handler);
      });
      return element;
    }

    var i = responders.length, responder;
    while (i--) {
      if (responders[i].handler === handler) {
        responder = responders[i];
        break;
      }
    }
    if (!responder) return element;

    if (eventName.include(':')) {
      if (element.removeEventListener)
        element.removeEventListener("dataavailable", responder, false);
      else {
        element.detachEvent("ondataavailable", responder);
        element.detachEvent("onlosecapture", responder);
      }
    } else {
      var actualEventName = _getDOMEventName(eventName);
      if (element.removeEventListener)
        element.removeEventListener(actualEventName, responder, false);
      else
        element.detachEvent('on' + actualEventName, responder);
    }

    registry.set(eventName, responders.without(responder));

    return element;
  }

  function fire(element, eventName, memo, bubble) {
    element = $(element);

    if (Object.isUndefined(bubble))
      bubble = true;

    if (element == document && document.createEvent && !element.dispatchEvent)
      element = document.documentElement;

    var event;
    if (document.createEvent) {
      event = document.createEvent('HTMLEvents');
      event.initEvent('dataavailable', bubble, true);
    } else {
      event = document.createEventObject();
      event.eventType = bubble ? 'ondataavailable' : 'onlosecapture';
    }

    event.eventName = eventName;
    event.memo = memo || { };

    if (document.createEvent)
      element.dispatchEvent(event);
    else
      element.fireEvent(event.eventType, event);

    return Event.extend(event);
  }

  Event.Handler = Class.create({
    initialize: function(element, eventName, selector, callback) {
      this.element   = $(element);
      this.eventName = eventName;
      this.selector  = selector;
      this.callback  = callback;
      this.handler   = this.handleEvent.bind(this);
    },

    start: function() {
      Event.observe(this.element, this.eventName, this.handler);
      return this;
    },

    stop: function() {
      Event.stopObserving(this.element, this.eventName, this.handler);
      return this;
    },

    handleEvent: function(event) {
      var element = Event.findElement(event, this.selector);
      if (element) this.callback.call(this.element, event, element);
    }
  });

  function on(element, eventName, selector, callback) {
    element = $(element);
    if (Object.isFunction(selector) && Object.isUndefined(callback)) {
      callback = selector, selector = null;
    }

    return new Event.Handler(element, eventName, selector, callback).start();
  }

  Object.extend(Event, Event.Methods);

  Object.extend(Event, {
    fire:          fire,
    observe:       observe,
    stopObserving: stopObserving,
    on:            on
  });

  Element.addMethods({
    fire:          fire,

    observe:       observe,

    stopObserving: stopObserving,

    on:            on
  });

  Object.extend(document, {
    fire:          fire.methodize(),

    observe:       observe.methodize(),

    stopObserving: stopObserving.methodize(),

    on:            on.methodize(),

    loaded:        false
  });

  if (window.Event) Object.extend(window.Event, Event);
  else window.Event = Event;
})();

(function() {
  /* Support for the DOMContentLoaded event is based on work by Dan Webb,
     Matthias Miller, Dean Edwards, John Resig, and Diego Perini. */

  var timer;

  function fireContentLoadedEvent() {
    if (document.loaded) return;
    if (timer) window.clearTimeout(timer);
    document.loaded = true;
    document.fire('dom:loaded');
  }

  function checkReadyState() {
    if (document.readyState === 'complete') {
      document.stopObserving('readystatechange', checkReadyState);
      fireContentLoadedEvent();
    }
  }

  function pollDoScroll() {
    try { document.documentElement.doScroll('left'); }
    catch(e) {
      timer = pollDoScroll.defer();
      return;
    }
    fireContentLoadedEvent();
  }

  if (document.addEventListener) {
    document.addEventListener('DOMContentLoaded', fireContentLoadedEvent, false);
  } else {
    document.observe('readystatechange', checkReadyState);
    if (window == top)
      timer = pollDoScroll.defer();
  }

  Event.observe(window, 'load', fireContentLoadedEvent);
})();


Element.addMethods();
/*------------------------------- DEPRECATED -------------------------------*/

Hash.toQueryString = Object.toQueryString;

var Toggle = { display: Element.toggle };

Element.Methods.childOf = Element.Methods.descendantOf;

var Insertion = {
  Before: function(element, content) {
    return Element.insert(element, {before:content});
  },

  Top: function(element, content) {
    return Element.insert(element, {top:content});
  },

  Bottom: function(element, content) {
    return Element.insert(element, {bottom:content});
  },

  After: function(element, content) {
    return Element.insert(element, {after:content});
  }
};

var $continue = new Error('"throw $continue" is deprecated, use "return" instead');

var Position = {
  includeScrollOffsets: false,

  prepare: function() {
    this.deltaX =  window.pageXOffset
                || document.documentElement.scrollLeft
                || document.body.scrollLeft
                || 0;
    this.deltaY =  window.pageYOffset
                || document.documentElement.scrollTop
                || document.body.scrollTop
                || 0;
  },

  within: function(element, x, y) {
    if (this.includeScrollOffsets)
      return this.withinIncludingScrolloffsets(element, x, y);
    this.xcomp = x;
    this.ycomp = y;
    this.offset = Element.cumulativeOffset(element);

    return (y >= this.offset[1] &&
            y <  this.offset[1] + element.offsetHeight &&
            x >= this.offset[0] &&
            x <  this.offset[0] + element.offsetWidth);
  },

  withinIncludingScrolloffsets: function(element, x, y) {
    var offsetcache = Element.cumulativeScrollOffset(element);

    this.xcomp = x + offsetcache[0] - this.deltaX;
    this.ycomp = y + offsetcache[1] - this.deltaY;
    this.offset = Element.cumulativeOffset(element);

    return (this.ycomp >= this.offset[1] &&
            this.ycomp <  this.offset[1] + element.offsetHeight &&
            this.xcomp >= this.offset[0] &&
            this.xcomp <  this.offset[0] + element.offsetWidth);
  },

  overlap: function(mode, element) {
    if (!mode) return 0;
    if (mode == 'vertical')
      return ((this.offset[1] + element.offsetHeight) - this.ycomp) /
        element.offsetHeight;
    if (mode == 'horizontal')
      return ((this.offset[0] + element.offsetWidth) - this.xcomp) /
        element.offsetWidth;
  },


  cumulativeOffset: Element.Methods.cumulativeOffset,

  positionedOffset: Element.Methods.positionedOffset,

  absolutize: function(element) {
    Position.prepare();
    return Element.absolutize(element);
  },

  relativize: function(element) {
    Position.prepare();
    return Element.relativize(element);
  },

  realOffset: Element.Methods.cumulativeScrollOffset,

  offsetParent: Element.Methods.getOffsetParent,

  page: Element.Methods.viewportOffset,

  clone: function(source, target, options) {
    options = options || { };
    return Element.clonePosition(target, source, options);
  }
};

/*--------------------------------------------------------------------------*/

if (!document.getElementsByClassName) document.getElementsByClassName = function(instanceMethods){
  function iter(name) {
    return name.blank() ? null : "[contains(concat(' ', @class, ' '), ' " + name + " ')]";
  }

  instanceMethods.getElementsByClassName = Prototype.BrowserFeatures.XPath ?
  function(element, className) {
    className = className.toString().strip();
    var cond = /\s/.test(className) ? $w(className).map(iter).join('') : iter(className);
    return cond ? document._getElementsByXPath('.//*' + cond, element) : [];
  } : function(element, className) {
    className = className.toString().strip();
    var elements = [], classNames = (/\s/.test(className) ? $w(className) : null);
    if (!classNames && !className) return elements;

    var nodes = $(element).getElementsByTagName('*');
    className = ' ' + className + ' ';

    for (var i = 0, child, cn; child = nodes[i]; i++) {
      if (child.className && (cn = ' ' + child.className + ' ') && (cn.include(className) ||
          (classNames && classNames.all(function(name) {
            return !name.toString().blank() && cn.include(' ' + name + ' ');
          }))))
        elements.push(Element.extend(child));
    }
    return elements;
  };

  return function(className, parentElement) {
    return $(parentElement || document.body).getElementsByClassName(className);
  };
}(Element.Methods);

/*--------------------------------------------------------------------------*/

Element.ClassNames = Class.create();
Element.ClassNames.prototype = {
  initialize: function(element) {
    this.element = $(element);
  },

  _each: function(iterator) {
    this.element.className.split(/\s+/).select(function(name) {
      return name.length > 0;
    })._each(iterator);
  },

  set: function(className) {
    this.element.className = className;
  },

  add: function(classNameToAdd) {
    if (this.include(classNameToAdd)) return;
    this.set($A(this).concat(classNameToAdd).join(' '));
  },

  remove: function(classNameToRemove) {
    if (!this.include(classNameToRemove)) return;
    this.set($A(this).without(classNameToRemove).join(' '));
  },

  toString: function() {
    return $A(this).join(' ');
  }
};

Object.extend(Element.ClassNames.prototype, Enumerable);

/*--------------------------------------------------------------------------*/

(function() {
  window.Selector = Class.create({
    initialize: function(expression) {
      this.expression = expression.strip();
    },

    findElements: function(rootElement) {
      return Prototype.Selector.select(this.expression, rootElement);
    },

    match: function(element) {
      return Prototype.Selector.match(element, this.expression);
    },

    toString: function() {
      return this.expression;
    },

    inspect: function() {
      return "#<Selector: " + this.expression + ">";
    }
  });

  Object.extend(Selector, {
    matchElements: function(elements, expression) {
      var match = Prototype.Selector.match,
          results = [];

      for (var i = 0, length = elements.length; i < length; i++) {
        var element = elements[i];
        if (match(element, expression)) {
          results.push(Element.extend(element));
        }
      }
      return results;
    },

    findElement: function(elements, expression, index) {
      index = index || 0;
      var matchIndex = 0, element;
      for (var i = 0, length = elements.length; i < length; i++) {
        element = elements[i];
        if (Prototype.Selector.match(element, expression) && index === matchIndex++) {
          return Element.extend(element);
        }
      }
    },

    findChildElements: function(element, expressions) {
      var selector = expressions.toArray().join(', ');
      return Prototype.Selector.select(selector, element || document);
    }
  });
})();

/*!
 * jQuery JavaScript Library v1.7.2
 * http://jquery.com/
 *
 * Copyright 2011, John Resig
 * Dual licensed under the MIT or GPL Version 2 licenses.
 * http://jquery.org/license
 *
 * Includes Sizzle.js
 * http://sizzlejs.com/
 * Copyright 2011, The Dojo Foundation
 * Released under the MIT, BSD, and GPL Licenses.
 *
 * Date: Wed Mar 21 12:46:34 2012 -0700
 */
(function( window, undefined ) {

// Use the correct document accordingly with window argument (sandbox)
var document = window.document,
	navigator = window.navigator,
	location = window.location;
var jQuery = (function() {

// Define a local copy of jQuery
var jQuery = function( selector, context ) {
		// The jQuery object is actually just the init constructor 'enhanced'
		return new jQuery.fn.init( selector, context, rootjQuery );
	},

	// Map over jQuery in case of overwrite
	_jQuery = window.jQuery,

	// Map over the $ in case of overwrite
	_$ = window.$,

	// A central reference to the root jQuery(document)
	rootjQuery,

	// A simple way to check for HTML strings or ID strings
	// Prioritize #id over <tag> to avoid XSS via location.hash (#9521)
	quickExpr = /^(?:[^#<]*(<[\w\W]+>)[^>]*$|#([\w\-]*)$)/,

	// Check if a string has a non-whitespace character in it
	rnotwhite = /\S/,

	// Used for trimming whitespace
	trimLeft = /^\s+/,
	trimRight = /\s+$/,

	// Match a standalone tag
	rsingleTag = /^<(\w+)\s*\/?>(?:<\/\1>)?$/,

	// JSON RegExp
	rvalidchars = /^[\],:{}\s]*$/,
	rvalidescape = /\\(?:["\\\/bfnrt]|u[0-9a-fA-F]{4})/g,
	rvalidtokens = /"[^"\\\n\r]*"|true|false|null|-?\d+(?:\.\d*)?(?:[eE][+\-]?\d+)?/g,
	rvalidbraces = /(?:^|:|,)(?:\s*\[)+/g,

	// Useragent RegExp
	rwebkit = /(webkit)[ \/]([\w.]+)/,
	ropera = /(opera)(?:.*version)?[ \/]([\w.]+)/,
	rmsie = /(msie) ([\w.]+)/,
	rmozilla = /(mozilla)(?:.*? rv:([\w.]+))?/,

	// Matches dashed string for camelizing
	rdashAlpha = /-([a-z]|[0-9])/ig,
	rmsPrefix = /^-ms-/,

	// Used by jQuery.camelCase as callback to replace()
	fcamelCase = function( all, letter ) {
		return ( letter + "" ).toUpperCase();
	},

	// Keep a UserAgent string for use with jQuery.browser
	userAgent = navigator.userAgent,

	// For matching the engine and version of the browser
	browserMatch,

	// The deferred used on DOM ready
	readyList,

	// The ready event handler
	DOMContentLoaded,

	// Save a reference to some core methods
	toString = Object.prototype.toString,
	hasOwn = Object.prototype.hasOwnProperty,
	push = Array.prototype.push,
	slice = Array.prototype.slice,
	trim = String.prototype.trim,
	indexOf = Array.prototype.indexOf,

	// [[Class]] -> type pairs
	class2type = {};

jQuery.fn = jQuery.prototype = {
	constructor: jQuery,
	init: function( selector, context, rootjQuery ) {
		var match, elem, ret, doc;

		// Handle $(""), $(null), or $(undefined)
		if ( !selector ) {
			return this;
		}

		// Handle $(DOMElement)
		if ( selector.nodeType ) {
			this.context = this[0] = selector;
			this.length = 1;
			return this;
		}

		// The body element only exists once, optimize finding it
		if ( selector === "body" && !context && document.body ) {
			this.context = document;
			this[0] = document.body;
			this.selector = selector;
			this.length = 1;
			return this;
		}

		// Handle HTML strings
		if ( typeof selector === "string" ) {
			// Are we dealing with HTML string or an ID?
			if ( selector.charAt(0) === "<" && selector.charAt( selector.length - 1 ) === ">" && selector.length >= 3 ) {
				// Assume that strings that start and end with <> are HTML and skip the regex check
				match = [ null, selector, null ];

			} else {
				match = quickExpr.exec( selector );
			}

			// Verify a match, and that no context was specified for #id
			if ( match && (match[1] || !context) ) {

				// HANDLE: $(html) -> $(array)
				if ( match[1] ) {
					context = context instanceof jQuery ? context[0] : context;
					doc = ( context ? context.ownerDocument || context : document );

					// If a single string is passed in and it's a single tag
					// just do a createElement and skip the rest
					ret = rsingleTag.exec( selector );

					if ( ret ) {
						if ( jQuery.isPlainObject( context ) ) {
							selector = [ document.createElement( ret[1] ) ];
							jQuery.fn.attr.call( selector, context, true );

						} else {
							selector = [ doc.createElement( ret[1] ) ];
						}

					} else {
						ret = jQuery.buildFragment( [ match[1] ], [ doc ] );
						selector = ( ret.cacheable ? jQuery.clone(ret.fragment) : ret.fragment ).childNodes;
					}

					return jQuery.merge( this, selector );

				// HANDLE: $("#id")
				} else {
					elem = document.getElementById( match[2] );

					// Check parentNode to catch when Blackberry 4.6 returns
					// nodes that are no longer in the document #6963
					if ( elem && elem.parentNode ) {
						// Handle the case where IE and Opera return items
						// by name instead of ID
						if ( elem.id !== match[2] ) {
							return rootjQuery.find( selector );
						}

						// Otherwise, we inject the element directly into the jQuery object
						this.length = 1;
						this[0] = elem;
					}

					this.context = document;
					this.selector = selector;
					return this;
				}

			// HANDLE: $(expr, $(...))
			} else if ( !context || context.jquery ) {
				return ( context || rootjQuery ).find( selector );

			// HANDLE: $(expr, context)
			// (which is just equivalent to: $(context).find(expr)
			} else {
				return this.constructor( context ).find( selector );
			}

		// HANDLE: $(function)
		// Shortcut for document ready
		} else if ( jQuery.isFunction( selector ) ) {
			return rootjQuery.ready( selector );
		}

		if ( selector.selector !== undefined ) {
			this.selector = selector.selector;
			this.context = selector.context;
		}

		return jQuery.makeArray( selector, this );
	},

	// Start with an empty selector
	selector: "",

	// The current version of jQuery being used
	jquery: "1.7.2",

	// The default length of a jQuery object is 0
	length: 0,

	// The number of elements contained in the matched element set
	size: function() {
		return this.length;
	},

	toArray: function() {
		return slice.call( this, 0 );
	},

	// Get the Nth element in the matched element set OR
	// Get the whole matched element set as a clean array
	get: function( num ) {
		return num == null ?

			// Return a 'clean' array
			this.toArray() :

			// Return just the object
			( num < 0 ? this[ this.length + num ] : this[ num ] );
	},

	// Take an array of elements and push it onto the stack
	// (returning the new matched element set)
	pushStack: function( elems, name, selector ) {
		// Build a new jQuery matched element set
		var ret = this.constructor();

		if ( jQuery.isArray( elems ) ) {
			push.apply( ret, elems );

		} else {
			jQuery.merge( ret, elems );
		}

		// Add the old object onto the stack (as a reference)
		ret.prevObject = this;

		ret.context = this.context;

		if ( name === "find" ) {
			ret.selector = this.selector + ( this.selector ? " " : "" ) + selector;
		} else if ( name ) {
			ret.selector = this.selector + "." + name + "(" + selector + ")";
		}

		// Return the newly-formed element set
		return ret;
	},

	// Execute a callback for every element in the matched set.
	// (You can seed the arguments with an array of args, but this is
	// only used internally.)
	each: function( callback, args ) {
		return jQuery.each( this, callback, args );
	},

	ready: function( fn ) {
		// Attach the listeners
		jQuery.bindReady();

		// Add the callback
		readyList.add( fn );

		return this;
	},

	eq: function( i ) {
		i = +i;
		return i === -1 ?
			this.slice( i ) :
			this.slice( i, i + 1 );
	},

	first: function() {
		return this.eq( 0 );
	},

	last: function() {
		return this.eq( -1 );
	},

	slice: function() {
		return this.pushStack( slice.apply( this, arguments ),
			"slice", slice.call(arguments).join(",") );
	},

	map: function( callback ) {
		return this.pushStack( jQuery.map(this, function( elem, i ) {
			return callback.call( elem, i, elem );
		}));
	},

	end: function() {
		return this.prevObject || this.constructor(null);
	},

	// For internal use only.
	// Behaves like an Array's method, not like a jQuery method.
	push: push,
	sort: [].sort,
	splice: [].splice
};

// Give the init function the jQuery prototype for later instantiation
jQuery.fn.init.prototype = jQuery.fn;

jQuery.extend = jQuery.fn.extend = function() {
	var options, name, src, copy, copyIsArray, clone,
		target = arguments[0] || {},
		i = 1,
		length = arguments.length,
		deep = false;

	// Handle a deep copy situation
	if ( typeof target === "boolean" ) {
		deep = target;
		target = arguments[1] || {};
		// skip the boolean and the target
		i = 2;
	}

	// Handle case when target is a string or something (possible in deep copy)
	if ( typeof target !== "object" && !jQuery.isFunction(target) ) {
		target = {};
	}

	// extend jQuery itself if only one argument is passed
	if ( length === i ) {
		target = this;
		--i;
	}

	for ( ; i < length; i++ ) {
		// Only deal with non-null/undefined values
		if ( (options = arguments[ i ]) != null ) {
			// Extend the base object
			for ( name in options ) {
				src = target[ name ];
				copy = options[ name ];

				// Prevent never-ending loop
				if ( target === copy ) {
					continue;
				}

				// Recurse if we're merging plain objects or arrays
				if ( deep && copy && ( jQuery.isPlainObject(copy) || (copyIsArray = jQuery.isArray(copy)) ) ) {
					if ( copyIsArray ) {
						copyIsArray = false;
						clone = src && jQuery.isArray(src) ? src : [];

					} else {
						clone = src && jQuery.isPlainObject(src) ? src : {};
					}

					// Never move original objects, clone them
					target[ name ] = jQuery.extend( deep, clone, copy );

				// Don't bring in undefined values
				} else if ( copy !== undefined ) {
					target[ name ] = copy;
				}
			}
		}
	}

	// Return the modified object
	return target;
};

jQuery.extend({
	noConflict: function( deep ) {
		if ( window.$ === jQuery ) {
			window.$ = _$;
		}

		if ( deep && window.jQuery === jQuery ) {
			window.jQuery = _jQuery;
		}

		return jQuery;
	},

	// Is the DOM ready to be used? Set to true once it occurs.
	isReady: false,

	// A counter to track how many items to wait for before
	// the ready event fires. See #6781
	readyWait: 1,

	// Hold (or release) the ready event
	holdReady: function( hold ) {
		if ( hold ) {
			jQuery.readyWait++;
		} else {
			jQuery.ready( true );
		}
	},

	// Handle when the DOM is ready
	ready: function( wait ) {
		// Either a released hold or an DOMready/load event and not yet ready
		if ( (wait === true && !--jQuery.readyWait) || (wait !== true && !jQuery.isReady) ) {
			// Make sure body exists, at least, in case IE gets a little overzealous (ticket #5443).
			if ( !document.body ) {
				return setTimeout( jQuery.ready, 1 );
			}

			// Remember that the DOM is ready
			jQuery.isReady = true;

			// If a normal DOM Ready event fired, decrement, and wait if need be
			if ( wait !== true && --jQuery.readyWait > 0 ) {
				return;
			}

			// If there are functions bound, to execute
			readyList.fireWith( document, [ jQuery ] );

			// Trigger any bound ready events
			if ( jQuery.fn.trigger ) {
				jQuery( document ).trigger( "ready" ).off( "ready" );
			}
		}
	},

	bindReady: function() {
		if ( readyList ) {
			return;
		}

		readyList = jQuery.Callbacks( "once memory" );

		// Catch cases where $(document).ready() is called after the
		// browser event has already occurred.
		if ( document.readyState === "complete" ) {
			// Handle it asynchronously to allow scripts the opportunity to delay ready
			return setTimeout( jQuery.ready, 1 );
		}

		// Mozilla, Opera and webkit nightlies currently support this event
		if ( document.addEventListener ) {
			// Use the handy event callback
			document.addEventListener( "DOMContentLoaded", DOMContentLoaded, false );

			// A fallback to window.onload, that will always work
			window.addEventListener( "load", jQuery.ready, false );

		// If IE event model is used
		} else if ( document.attachEvent ) {
			// ensure firing before onload,
			// maybe late but safe also for iframes
			document.attachEvent( "onreadystatechange", DOMContentLoaded );

			// A fallback to window.onload, that will always work
			window.attachEvent( "onload", jQuery.ready );

			// If IE and not a frame
			// continually check to see if the document is ready
			var toplevel = false;

			try {
				toplevel = window.frameElement == null;
			} catch(e) {}

			if ( document.documentElement.doScroll && toplevel ) {
				doScrollCheck();
			}
		}
	},

	// See test/unit/core.js for details concerning isFunction.
	// Since version 1.3, DOM methods and functions like alert
	// aren't supported. They return false on IE (#2968).
	isFunction: function( obj ) {
		return jQuery.type(obj) === "function";
	},

	isArray: Array.isArray || function( obj ) {
		return jQuery.type(obj) === "array";
	},

	isWindow: function( obj ) {
		return obj != null && obj == obj.window;
	},

	isNumeric: function( obj ) {
		return !isNaN( parseFloat(obj) ) && isFinite( obj );
	},

	type: function( obj ) {
		return obj == null ?
			String( obj ) :
			class2type[ toString.call(obj) ] || "object";
	},

	isPlainObject: function( obj ) {
		// Must be an Object.
		// Because of IE, we also have to check the presence of the constructor property.
		// Make sure that DOM nodes and window objects don't pass through, as well
		if ( !obj || jQuery.type(obj) !== "object" || obj.nodeType || jQuery.isWindow( obj ) ) {
			return false;
		}

		try {
			// Not own constructor property must be Object
			if ( obj.constructor &&
				!hasOwn.call(obj, "constructor") &&
				!hasOwn.call(obj.constructor.prototype, "isPrototypeOf") ) {
				return false;
			}
		} catch ( e ) {
			// IE8,9 Will throw exceptions on certain host objects #9897
			return false;
		}

		// Own properties are enumerated firstly, so to speed up,
		// if last one is own, then all properties are own.

		var key;
		for ( key in obj ) {}

		return key === undefined || hasOwn.call( obj, key );
	},

	isEmptyObject: function( obj ) {
		for ( var name in obj ) {
			return false;
		}
		return true;
	},

	error: function( msg ) {
		throw new Error( msg );
	},

	parseJSON: function( data ) {
		if ( typeof data !== "string" || !data ) {
			return null;
		}

		// Make sure leading/trailing whitespace is removed (IE can't handle it)
		data = jQuery.trim( data );

		// Attempt to parse using the native JSON parser first
		if ( window.JSON && window.JSON.parse ) {
			return window.JSON.parse( data );
		}

		// Make sure the incoming data is actual JSON
		// Logic borrowed from http://json.org/json2.js
		if ( rvalidchars.test( data.replace( rvalidescape, "@" )
			.replace( rvalidtokens, "]" )
			.replace( rvalidbraces, "")) ) {

			return ( new Function( "return " + data ) )();

		}
		jQuery.error( "Invalid JSON: " + data );
	},

	// Cross-browser xml parsing
	parseXML: function( data ) {
		if ( typeof data !== "string" || !data ) {
			return null;
		}
		var xml, tmp;
		try {
			if ( window.DOMParser ) { // Standard
				tmp = new DOMParser();
				xml = tmp.parseFromString( data , "text/xml" );
			} else { // IE
				xml = new ActiveXObject( "Microsoft.XMLDOM" );
				xml.async = "false";
				xml.loadXML( data );
			}
		} catch( e ) {
			xml = undefined;
		}
		if ( !xml || !xml.documentElement || xml.getElementsByTagName( "parsererror" ).length ) {
			jQuery.error( "Invalid XML: " + data );
		}
		return xml;
	},

	noop: function() {},

	// Evaluates a script in a global context
	// Workarounds based on findings by Jim Driscoll
	// http://weblogs.java.net/blog/driscoll/archive/2009/09/08/eval-javascript-global-context
	globalEval: function( data ) {
		if ( data && rnotwhite.test( data ) ) {
			// We use execScript on Internet Explorer
			// We use an anonymous function so that context is window
			// rather than jQuery in Firefox
			( window.execScript || function( data ) {
				window[ "eval" ].call( window, data );
			} )( data );
		}
	},

	// Convert dashed to camelCase; used by the css and data modules
	// Microsoft forgot to hump their vendor prefix (#9572)
	camelCase: function( string ) {
		return string.replace( rmsPrefix, "ms-" ).replace( rdashAlpha, fcamelCase );
	},

	nodeName: function( elem, name ) {
		return elem.nodeName && elem.nodeName.toUpperCase() === name.toUpperCase();
	},

	// args is for internal usage only
	each: function( object, callback, args ) {
		var name, i = 0,
			length = object.length,
			isObj = length === undefined || jQuery.isFunction( object );

		if ( args ) {
			if ( isObj ) {
				for ( name in object ) {
					if ( callback.apply( object[ name ], args ) === false ) {
						break;
					}
				}
			} else {
				for ( ; i < length; ) {
					if ( callback.apply( object[ i++ ], args ) === false ) {
						break;
					}
				}
			}

		// A special, fast, case for the most common use of each
		} else {
			if ( isObj ) {
				for ( name in object ) {
					if ( callback.call( object[ name ], name, object[ name ] ) === false ) {
						break;
					}
				}
			} else {
				for ( ; i < length; ) {
					if ( callback.call( object[ i ], i, object[ i++ ] ) === false ) {
						break;
					}
				}
			}
		}

		return object;
	},

	// Use native String.trim function wherever possible
	trim: trim ?
		function( text ) {
			return text == null ?
				"" :
				trim.call( text );
		} :

		// Otherwise use our own trimming functionality
		function( text ) {
			return text == null ?
				"" :
				text.toString().replace( trimLeft, "" ).replace( trimRight, "" );
		},

	// results is for internal usage only
	makeArray: function( array, results ) {
		var ret = results || [];

		if ( array != null ) {
			// The window, strings (and functions) also have 'length'
			// Tweaked logic slightly to handle Blackberry 4.7 RegExp issues #6930
			var type = jQuery.type( array );

			if ( array.length == null || type === "string" || type === "function" || type === "regexp" || jQuery.isWindow( array ) ) {
				push.call( ret, array );
			} else {
				jQuery.merge( ret, array );
			}
		}

		return ret;
	},

	inArray: function( elem, array, i ) {
		var len;

		if ( array ) {
			if ( indexOf ) {
				return indexOf.call( array, elem, i );
			}

			len = array.length;
			i = i ? i < 0 ? Math.max( 0, len + i ) : i : 0;

			for ( ; i < len; i++ ) {
				// Skip accessing in sparse arrays
				if ( i in array && array[ i ] === elem ) {
					return i;
				}
			}
		}

		return -1;
	},

	merge: function( first, second ) {
		var i = first.length,
			j = 0;

		if ( typeof second.length === "number" ) {
			for ( var l = second.length; j < l; j++ ) {
				first[ i++ ] = second[ j ];
			}

		} else {
			while ( second[j] !== undefined ) {
				first[ i++ ] = second[ j++ ];
			}
		}

		first.length = i;

		return first;
	},

	grep: function( elems, callback, inv ) {
		var ret = [], retVal;
		inv = !!inv;

		// Go through the array, only saving the items
		// that pass the validator function
		for ( var i = 0, length = elems.length; i < length; i++ ) {
			retVal = !!callback( elems[ i ], i );
			if ( inv !== retVal ) {
				ret.push( elems[ i ] );
			}
		}

		return ret;
	},

	// arg is for internal usage only
	map: function( elems, callback, arg ) {
		var value, key, ret = [],
			i = 0,
			length = elems.length,
			// jquery objects are treated as arrays
			isArray = elems instanceof jQuery || length !== undefined && typeof length === "number" && ( ( length > 0 && elems[ 0 ] && elems[ length -1 ] ) || length === 0 || jQuery.isArray( elems ) ) ;

		// Go through the array, translating each of the items to their
		if ( isArray ) {
			for ( ; i < length; i++ ) {
				value = callback( elems[ i ], i, arg );

				if ( value != null ) {
					ret[ ret.length ] = value;
				}
			}

		// Go through every key on the object,
		} else {
			for ( key in elems ) {
				value = callback( elems[ key ], key, arg );

				if ( value != null ) {
					ret[ ret.length ] = value;
				}
			}
		}

		// Flatten any nested arrays
		return ret.concat.apply( [], ret );
	},

	// A global GUID counter for objects
	guid: 1,

	// Bind a function to a context, optionally partially applying any
	// arguments.
	proxy: function( fn, context ) {
		if ( typeof context === "string" ) {
			var tmp = fn[ context ];
			context = fn;
			fn = tmp;
		}

		// Quick check to determine if target is callable, in the spec
		// this throws a TypeError, but we will just return undefined.
		if ( !jQuery.isFunction( fn ) ) {
			return undefined;
		}

		// Simulated bind
		var args = slice.call( arguments, 2 ),
			proxy = function() {
				return fn.apply( context, args.concat( slice.call( arguments ) ) );
			};

		// Set the guid of unique handler to the same of original handler, so it can be removed
		proxy.guid = fn.guid = fn.guid || proxy.guid || jQuery.guid++;

		return proxy;
	},

	// Mutifunctional method to get and set values to a collection
	// The value/s can optionally be executed if it's a function
	access: function( elems, fn, key, value, chainable, emptyGet, pass ) {
		var exec,
			bulk = key == null,
			i = 0,
			length = elems.length;

		// Sets many values
		if ( key && typeof key === "object" ) {
			for ( i in key ) {
				jQuery.access( elems, fn, i, key[i], 1, emptyGet, value );
			}
			chainable = 1;

		// Sets one value
		} else if ( value !== undefined ) {
			// Optionally, function values get executed if exec is true
			exec = pass === undefined && jQuery.isFunction( value );

			if ( bulk ) {
				// Bulk operations only iterate when executing function values
				if ( exec ) {
					exec = fn;
					fn = function( elem, key, value ) {
						return exec.call( jQuery( elem ), value );
					};

				// Otherwise they run against the entire set
				} else {
					fn.call( elems, value );
					fn = null;
				}
			}

			if ( fn ) {
				for (; i < length; i++ ) {
					fn( elems[i], key, exec ? value.call( elems[i], i, fn( elems[i], key ) ) : value, pass );
				}
			}

			chainable = 1;
		}

		return chainable ?
			elems :

			// Gets
			bulk ?
				fn.call( elems ) :
				length ? fn( elems[0], key ) : emptyGet;
	},

	now: function() {
		return ( new Date() ).getTime();
	},

	// Use of jQuery.browser is frowned upon.
	// More details: http://docs.jquery.com/Utilities/jQuery.browser
	uaMatch: function( ua ) {
		ua = ua.toLowerCase();

		var match = rwebkit.exec( ua ) ||
			ropera.exec( ua ) ||
			rmsie.exec( ua ) ||
			ua.indexOf("compatible") < 0 && rmozilla.exec( ua ) ||
			[];

		return { browser: match[1] || "", version: match[2] || "0" };
	},

	sub: function() {
		function jQuerySub( selector, context ) {
			return new jQuerySub.fn.init( selector, context );
		}
		jQuery.extend( true, jQuerySub, this );
		jQuerySub.superclass = this;
		jQuerySub.fn = jQuerySub.prototype = this();
		jQuerySub.fn.constructor = jQuerySub;
		jQuerySub.sub = this.sub;
		jQuerySub.fn.init = function init( selector, context ) {
			if ( context && context instanceof jQuery && !(context instanceof jQuerySub) ) {
				context = jQuerySub( context );
			}

			return jQuery.fn.init.call( this, selector, context, rootjQuerySub );
		};
		jQuerySub.fn.init.prototype = jQuerySub.fn;
		var rootjQuerySub = jQuerySub(document);
		return jQuerySub;
	},

	browser: {}
});

// Populate the class2type map
jQuery.each("Boolean Number String Function Array Date RegExp Object".split(" "), function(i, name) {
	class2type[ "[object " + name + "]" ] = name.toLowerCase();
});

browserMatch = jQuery.uaMatch( userAgent );
if ( browserMatch.browser ) {
	jQuery.browser[ browserMatch.browser ] = true;
	jQuery.browser.version = browserMatch.version;
}

// Deprecated, use jQuery.browser.webkit instead
if ( jQuery.browser.webkit ) {
	jQuery.browser.safari = true;
}

// IE doesn't match non-breaking spaces with \s
if ( rnotwhite.test( "\xA0" ) ) {
	trimLeft = /^[\s\xA0]+/;
	trimRight = /[\s\xA0]+$/;
}

// All jQuery objects should point back to these
rootjQuery = jQuery(document);

// Cleanup functions for the document ready method
if ( document.addEventListener ) {
	DOMContentLoaded = function() {
		document.removeEventListener( "DOMContentLoaded", DOMContentLoaded, false );
		jQuery.ready();
	};

} else if ( document.attachEvent ) {
	DOMContentLoaded = function() {
		// Make sure body exists, at least, in case IE gets a little overzealous (ticket #5443).
		if ( document.readyState === "complete" ) {
			document.detachEvent( "onreadystatechange", DOMContentLoaded );
			jQuery.ready();
		}
	};
}

// The DOM ready check for Internet Explorer
function doScrollCheck() {
	if ( jQuery.isReady ) {
		return;
	}

	try {
		// If IE is used, use the trick by Diego Perini
		// http://javascript.nwbox.com/IEContentLoaded/
		document.documentElement.doScroll("left");
	} catch(e) {
		setTimeout( doScrollCheck, 1 );
		return;
	}

	// and execute any waiting functions
	jQuery.ready();
}

return jQuery;

})();


// String to Object flags format cache
var flagsCache = {};

// Convert String-formatted flags into Object-formatted ones and store in cache
function createFlags( flags ) {
	var object = flagsCache[ flags ] = {},
		i, length;
	flags = flags.split( /\s+/ );
	for ( i = 0, length = flags.length; i < length; i++ ) {
		object[ flags[i] ] = true;
	}
	return object;
}

/*
 * Create a callback list using the following parameters:
 *
 *	flags:	an optional list of space-separated flags that will change how
 *			the callback list behaves
 *
 * By default a callback list will act like an event callback list and can be
 * "fired" multiple times.
 *
 * Possible flags:
 *
 *	once:			will ensure the callback list can only be fired once (like a Deferred)
 *
 *	memory:			will keep track of previous values and will call any callback added
 *					after the list has been fired right away with the latest "memorized"
 *					values (like a Deferred)
 *
 *	unique:			will ensure a callback can only be added once (no duplicate in the list)
 *
 *	stopOnFalse:	interrupt callings when a callback returns false
 *
 */
jQuery.Callbacks = function( flags ) {

	// Convert flags from String-formatted to Object-formatted
	// (we check in cache first)
	flags = flags ? ( flagsCache[ flags ] || createFlags( flags ) ) : {};

	var // Actual callback list
		list = [],
		// Stack of fire calls for repeatable lists
		stack = [],
		// Last fire value (for non-forgettable lists)
		memory,
		// Flag to know if list was already fired
		fired,
		// Flag to know if list is currently firing
		firing,
		// First callback to fire (used internally by add and fireWith)
		firingStart,
		// End of the loop when firing
		firingLength,
		// Index of currently firing callback (modified by remove if needed)
		firingIndex,
		// Add one or several callbacks to the list
		add = function( args ) {
			var i,
				length,
				elem,
				type,
				actual;
			for ( i = 0, length = args.length; i < length; i++ ) {
				elem = args[ i ];
				type = jQuery.type( elem );
				if ( type === "array" ) {
					// Inspect recursively
					add( elem );
				} else if ( type === "function" ) {
					// Add if not in unique mode and callback is not in
					if ( !flags.unique || !self.has( elem ) ) {
						list.push( elem );
					}
				}
			}
		},
		// Fire callbacks
		fire = function( context, args ) {
			args = args || [];
			memory = !flags.memory || [ context, args ];
			fired = true;
			firing = true;
			firingIndex = firingStart || 0;
			firingStart = 0;
			firingLength = list.length;
			for ( ; list && firingIndex < firingLength; firingIndex++ ) {
				if ( list[ firingIndex ].apply( context, args ) === false && flags.stopOnFalse ) {
					memory = true; // Mark as halted
					break;
				}
			}
			firing = false;
			if ( list ) {
				if ( !flags.once ) {
					if ( stack && stack.length ) {
						memory = stack.shift();
						self.fireWith( memory[ 0 ], memory[ 1 ] );
					}
				} else if ( memory === true ) {
					self.disable();
				} else {
					list = [];
				}
			}
		},
		// Actual Callbacks object
		self = {
			// Add a callback or a collection of callbacks to the list
			add: function() {
				if ( list ) {
					var length = list.length;
					add( arguments );
					// Do we need to add the callbacks to the
					// current firing batch?
					if ( firing ) {
						firingLength = list.length;
					// With memory, if we're not firing then
					// we should call right away, unless previous
					// firing was halted (stopOnFalse)
					} else if ( memory && memory !== true ) {
						firingStart = length;
						fire( memory[ 0 ], memory[ 1 ] );
					}
				}
				return this;
			},
			// Remove a callback from the list
			remove: function() {
				if ( list ) {
					var args = arguments,
						argIndex = 0,
						argLength = args.length;
					for ( ; argIndex < argLength ; argIndex++ ) {
						for ( var i = 0; i < list.length; i++ ) {
							if ( args[ argIndex ] === list[ i ] ) {
								// Handle firingIndex and firingLength
								if ( firing ) {
									if ( i <= firingLength ) {
										firingLength--;
										if ( i <= firingIndex ) {
											firingIndex--;
										}
									}
								}
								// Remove the element
								list.splice( i--, 1 );
								// If we have some unicity property then
								// we only need to do this once
								if ( flags.unique ) {
									break;
								}
							}
						}
					}
				}
				return this;
			},
			// Control if a given callback is in the list
			has: function( fn ) {
				if ( list ) {
					var i = 0,
						length = list.length;
					for ( ; i < length; i++ ) {
						if ( fn === list[ i ] ) {
							return true;
						}
					}
				}
				return false;
			},
			// Remove all callbacks from the list
			empty: function() {
				list = [];
				return this;
			},
			// Have the list do nothing anymore
			disable: function() {
				list = stack = memory = undefined;
				return this;
			},
			// Is it disabled?
			disabled: function() {
				return !list;
			},
			// Lock the list in its current state
			lock: function() {
				stack = undefined;
				if ( !memory || memory === true ) {
					self.disable();
				}
				return this;
			},
			// Is it locked?
			locked: function() {
				return !stack;
			},
			// Call all callbacks with the given context and arguments
			fireWith: function( context, args ) {
				if ( stack ) {
					if ( firing ) {
						if ( !flags.once ) {
							stack.push( [ context, args ] );
						}
					} else if ( !( flags.once && memory ) ) {
						fire( context, args );
					}
				}
				return this;
			},
			// Call all the callbacks with the given arguments
			fire: function() {
				self.fireWith( this, arguments );
				return this;
			},
			// To know if the callbacks have already been called at least once
			fired: function() {
				return !!fired;
			}
		};

	return self;
};




var // Static reference to slice
	sliceDeferred = [].slice;

jQuery.extend({

	Deferred: function( func ) {
		var doneList = jQuery.Callbacks( "once memory" ),
			failList = jQuery.Callbacks( "once memory" ),
			progressList = jQuery.Callbacks( "memory" ),
			state = "pending",
			lists = {
				resolve: doneList,
				reject: failList,
				notify: progressList
			},
			promise = {
				done: doneList.add,
				fail: failList.add,
				progress: progressList.add,

				state: function() {
					return state;
				},

				// Deprecated
				isResolved: doneList.fired,
				isRejected: failList.fired,

				then: function( doneCallbacks, failCallbacks, progressCallbacks ) {
					deferred.done( doneCallbacks ).fail( failCallbacks ).progress( progressCallbacks );
					return this;
				},
				always: function() {
					deferred.done.apply( deferred, arguments ).fail.apply( deferred, arguments );
					return this;
				},
				pipe: function( fnDone, fnFail, fnProgress ) {
					return jQuery.Deferred(function( newDefer ) {
						jQuery.each( {
							done: [ fnDone, "resolve" ],
							fail: [ fnFail, "reject" ],
							progress: [ fnProgress, "notify" ]
						}, function( handler, data ) {
							var fn = data[ 0 ],
								action = data[ 1 ],
								returned;
							if ( jQuery.isFunction( fn ) ) {
								deferred[ handler ](function() {
									returned = fn.apply( this, arguments );
									if ( returned && jQuery.isFunction( returned.promise ) ) {
										returned.promise().then( newDefer.resolve, newDefer.reject, newDefer.notify );
									} else {
										newDefer[ action + "With" ]( this === deferred ? newDefer : this, [ returned ] );
									}
								});
							} else {
								deferred[ handler ]( newDefer[ action ] );
							}
						});
					}).promise();
				},
				// Get a promise for this deferred
				// If obj is provided, the promise aspect is added to the object
				promise: function( obj ) {
					if ( obj == null ) {
						obj = promise;
					} else {
						for ( var key in promise ) {
							obj[ key ] = promise[ key ];
						}
					}
					return obj;
				}
			},
			deferred = promise.promise({}),
			key;

		for ( key in lists ) {
			deferred[ key ] = lists[ key ].fire;
			deferred[ key + "With" ] = lists[ key ].fireWith;
		}

		// Handle state
		deferred.done( function() {
			state = "resolved";
		}, failList.disable, progressList.lock ).fail( function() {
			state = "rejected";
		}, doneList.disable, progressList.lock );

		// Call given func if any
		if ( func ) {
			func.call( deferred, deferred );
		}

		// All done!
		return deferred;
	},

	// Deferred helper
	when: function( firstParam ) {
		var args = sliceDeferred.call( arguments, 0 ),
			i = 0,
			length = args.length,
			pValues = new Array( length ),
			count = length,
			pCount = length,
			deferred = length <= 1 && firstParam && jQuery.isFunction( firstParam.promise ) ?
				firstParam :
				jQuery.Deferred(),
			promise = deferred.promise();
		function resolveFunc( i ) {
			return function( value ) {
				args[ i ] = arguments.length > 1 ? sliceDeferred.call( arguments, 0 ) : value;
				if ( !( --count ) ) {
					deferred.resolveWith( deferred, args );
				}
			};
		}
		function progressFunc( i ) {
			return function( value ) {
				pValues[ i ] = arguments.length > 1 ? sliceDeferred.call( arguments, 0 ) : value;
				deferred.notifyWith( promise, pValues );
			};
		}
		if ( length > 1 ) {
			for ( ; i < length; i++ ) {
				if ( args[ i ] && args[ i ].promise && jQuery.isFunction( args[ i ].promise ) ) {
					args[ i ].promise().then( resolveFunc(i), deferred.reject, progressFunc(i) );
				} else {
					--count;
				}
			}
			if ( !count ) {
				deferred.resolveWith( deferred, args );
			}
		} else if ( deferred !== firstParam ) {
			deferred.resolveWith( deferred, length ? [ firstParam ] : [] );
		}
		return promise;
	}
});




jQuery.support = (function() {

	var support,
		all,
		a,
		select,
		opt,
		input,
		fragment,
		tds,
		events,
		eventName,
		i,
		isSupported,
		div = document.createElement( "div" ),
		documentElement = document.documentElement;

	// Preliminary tests
	div.setAttribute("className", "t");
	div.innerHTML = "   <link/><table></table><a href='/a' style='top:1px;float:left;opacity:.55;'>a</a><input type='checkbox'/>";

	all = div.getElementsByTagName( "*" );
	a = div.getElementsByTagName( "a" )[ 0 ];

	// Can't get basic test support
	if ( !all || !all.length || !a ) {
		return {};
	}

	// First batch of supports tests
	select = document.createElement( "select" );
	opt = select.appendChild( document.createElement("option") );
	input = div.getElementsByTagName( "input" )[ 0 ];

	support = {
		// IE strips leading whitespace when .innerHTML is used
		leadingWhitespace: ( div.firstChild.nodeType === 3 ),

		// Make sure that tbody elements aren't automatically inserted
		// IE will insert them into empty tables
		tbody: !div.getElementsByTagName("tbody").length,

		// Make sure that link elements get serialized correctly by innerHTML
		// This requires a wrapper element in IE
		htmlSerialize: !!div.getElementsByTagName("link").length,

		// Get the style information from getAttribute
		// (IE uses .cssText instead)
		style: /top/.test( a.getAttribute("style") ),

		// Make sure that URLs aren't manipulated
		// (IE normalizes it by default)
		hrefNormalized: ( a.getAttribute("href") === "/a" ),

		// Make sure that element opacity exists
		// (IE uses filter instead)
		// Use a regex to work around a WebKit issue. See #5145
		opacity: /^0.55/.test( a.style.opacity ),

		// Verify style float existence
		// (IE uses styleFloat instead of cssFloat)
		cssFloat: !!a.style.cssFloat,

		// Make sure that if no value is specified for a checkbox
		// that it defaults to "on".
		// (WebKit defaults to "" instead)
		checkOn: ( input.value === "on" ),

		// Make sure that a selected-by-default option has a working selected property.
		// (WebKit defaults to false instead of true, IE too, if it's in an optgroup)
		optSelected: opt.selected,

		// Test setAttribute on camelCase class. If it works, we need attrFixes when doing get/setAttribute (ie6/7)
		getSetAttribute: div.className !== "t",

		// Tests for enctype support on a form(#6743)
		enctype: !!document.createElement("form").enctype,

		// Makes sure cloning an html5 element does not cause problems
		// Where outerHTML is undefined, this still works
		html5Clone: document.createElement("nav").cloneNode( true ).outerHTML !== "<:nav></:nav>",

		// Will be defined later
		submitBubbles: true,
		changeBubbles: true,
		focusinBubbles: false,
		deleteExpando: true,
		noCloneEvent: true,
		inlineBlockNeedsLayout: false,
		shrinkWrapBlocks: false,
		reliableMarginRight: true,
		pixelMargin: true
	};

	// jQuery.boxModel DEPRECATED in 1.3, use jQuery.support.boxModel instead
	jQuery.boxModel = support.boxModel = (document.compatMode === "CSS1Compat");

	// Make sure checked status is properly cloned
	input.checked = true;
	support.noCloneChecked = input.cloneNode( true ).checked;

	// Make sure that the options inside disabled selects aren't marked as disabled
	// (WebKit marks them as disabled)
	select.disabled = true;
	support.optDisabled = !opt.disabled;

	// Test to see if it's possible to delete an expando from an element
	// Fails in Internet Explorer
	try {
		delete div.test;
	} catch( e ) {
		support.deleteExpando = false;
	}

	if ( !div.addEventListener && div.attachEvent && div.fireEvent ) {
		div.attachEvent( "onclick", function() {
			// Cloning a node shouldn't copy over any
			// bound event handlers (IE does this)
			support.noCloneEvent = false;
		});
		div.cloneNode( true ).fireEvent( "onclick" );
	}

	// Check if a radio maintains its value
	// after being appended to the DOM
	input = document.createElement("input");
	input.value = "t";
	input.setAttribute("type", "radio");
	support.radioValue = input.value === "t";

	input.setAttribute("checked", "checked");

	// #11217 - WebKit loses check when the name is after the checked attribute
	input.setAttribute( "name", "t" );

	div.appendChild( input );
	fragment = document.createDocumentFragment();
	fragment.appendChild( div.lastChild );

	// WebKit doesn't clone checked state correctly in fragments
	support.checkClone = fragment.cloneNode( true ).cloneNode( true ).lastChild.checked;

	// Check if a disconnected checkbox will retain its checked
	// value of true after appended to the DOM (IE6/7)
	support.appendChecked = input.checked;

	fragment.removeChild( input );
	fragment.appendChild( div );

	// Technique from Juriy Zaytsev
	// http://perfectionkills.com/detecting-event-support-without-browser-sniffing/
	// We only care about the case where non-standard event systems
	// are used, namely in IE. Short-circuiting here helps us to
	// avoid an eval call (in setAttribute) which can cause CSP
	// to go haywire. See: https://developer.mozilla.org/en/Security/CSP
	if ( div.attachEvent ) {
		for ( i in {
			submit: 1,
			change: 1,
			focusin: 1
		}) {
			eventName = "on" + i;
			isSupported = ( eventName in div );
			if ( !isSupported ) {
				div.setAttribute( eventName, "return;" );
				isSupported = ( typeof div[ eventName ] === "function" );
			}
			support[ i + "Bubbles" ] = isSupported;
		}
	}

	fragment.removeChild( div );

	// Null elements to avoid leaks in IE
	fragment = select = opt = div = input = null;

	// Run tests that need a body at doc ready
	jQuery(function() {
		var container, outer, inner, table, td, offsetSupport,
			marginDiv, conMarginTop, style, html, positionTopLeftWidthHeight,
			paddingMarginBorderVisibility, paddingMarginBorder,
			body = document.getElementsByTagName("body")[0];

		if ( !body ) {
			// Return for frameset docs that don't have a body
			return;
		}

		conMarginTop = 1;
		paddingMarginBorder = "padding:0;margin:0;border:";
		positionTopLeftWidthHeight = "position:absolute;top:0;left:0;width:1px;height:1px;";
		paddingMarginBorderVisibility = paddingMarginBorder + "0;visibility:hidden;";
		style = "style='" + positionTopLeftWidthHeight + paddingMarginBorder + "5px solid #000;";
		html = "<div " + style + "display:block;'><div style='" + paddingMarginBorder + "0;display:block;overflow:hidden;'></div></div>" +
			"<table " + style + "' cellpadding='0' cellspacing='0'>" +
			"<tr><td></td></tr></table>";

		container = document.createElement("div");
		container.style.cssText = paddingMarginBorderVisibility + "width:0;height:0;position:static;top:0;margin-top:" + conMarginTop + "px";
		body.insertBefore( container, body.firstChild );

		// Construct the test element
		div = document.createElement("div");
		container.appendChild( div );

		// Check if table cells still have offsetWidth/Height when they are set
		// to display:none and there are still other visible table cells in a
		// table row; if so, offsetWidth/Height are not reliable for use when
		// determining if an element has been hidden directly using
		// display:none (it is still safe to use offsets if a parent element is
		// hidden; don safety goggles and see bug #4512 for more information).
		// (only IE 8 fails this test)
		div.innerHTML = "<table><tr><td style='" + paddingMarginBorder + "0;display:none'></td><td>t</td></tr></table>";
		tds = div.getElementsByTagName( "td" );
		isSupported = ( tds[ 0 ].offsetHeight === 0 );

		tds[ 0 ].style.display = "";
		tds[ 1 ].style.display = "none";

		// Check if empty table cells still have offsetWidth/Height
		// (IE <= 8 fail this test)
		support.reliableHiddenOffsets = isSupported && ( tds[ 0 ].offsetHeight === 0 );

		// Check if div with explicit width and no margin-right incorrectly
		// gets computed margin-right based on width of container. For more
		// info see bug #3333
		// Fails in WebKit before Feb 2011 nightlies
		// WebKit Bug 13343 - getComputedStyle returns wrong value for margin-right
		if ( window.getComputedStyle ) {
			div.innerHTML = "";
			marginDiv = document.createElement( "div" );
			marginDiv.style.width = "0";
			marginDiv.style.marginRight = "0";
			div.style.width = "2px";
			div.appendChild( marginDiv );
			support.reliableMarginRight =
				( parseInt( ( window.getComputedStyle( marginDiv, null ) || { marginRight: 0 } ).marginRight, 10 ) || 0 ) === 0;
		}

		if ( typeof div.style.zoom !== "undefined" ) {
			// Check if natively block-level elements act like inline-block
			// elements when setting their display to 'inline' and giving
			// them layout
			// (IE < 8 does this)
			div.innerHTML = "";
			div.style.width = div.style.padding = "1px";
			div.style.border = 0;
			div.style.overflow = "hidden";
			div.style.display = "inline";
			div.style.zoom = 1;
			support.inlineBlockNeedsLayout = ( div.offsetWidth === 3 );

			// Check if elements with layout shrink-wrap their children
			// (IE 6 does this)
			div.style.display = "block";
			div.style.overflow = "visible";
			div.innerHTML = "<div style='width:5px;'></div>";
			support.shrinkWrapBlocks = ( div.offsetWidth !== 3 );
		}

		div.style.cssText = positionTopLeftWidthHeight + paddingMarginBorderVisibility;
		div.innerHTML = html;

		outer = div.firstChild;
		inner = outer.firstChild;
		td = outer.nextSibling.firstChild.firstChild;

		offsetSupport = {
			doesNotAddBorder: ( inner.offsetTop !== 5 ),
			doesAddBorderForTableAndCells: ( td.offsetTop === 5 )
		};

		inner.style.position = "fixed";
		inner.style.top = "20px";

		// safari subtracts parent border width here which is 5px
		offsetSupport.fixedPosition = ( inner.offsetTop === 20 || inner.offsetTop === 15 );
		inner.style.position = inner.style.top = "";

		outer.style.overflow = "hidden";
		outer.style.position = "relative";

		offsetSupport.subtractsBorderForOverflowNotVisible = ( inner.offsetTop === -5 );
		offsetSupport.doesNotIncludeMarginInBodyOffset = ( body.offsetTop !== conMarginTop );

		if ( window.getComputedStyle ) {
			div.style.marginTop = "1%";
			support.pixelMargin = ( window.getComputedStyle( div, null ) || { marginTop: 0 } ).marginTop !== "1%";
		}

		if ( typeof container.style.zoom !== "undefined" ) {
			container.style.zoom = 1;
		}

		body.removeChild( container );
		marginDiv = div = container = null;

		jQuery.extend( support, offsetSupport );
	});

	return support;
})();




var rbrace = /^(?:\{.*\}|\[.*\])$/,
	rmultiDash = /([A-Z])/g;

jQuery.extend({
	cache: {},

	// Please use with caution
	uuid: 0,

	// Unique for each copy of jQuery on the page
	// Non-digits removed to match rinlinejQuery
	expando: "jQuery" + ( jQuery.fn.jquery + Math.random() ).replace( /\D/g, "" ),

	// The following elements throw uncatchable exceptions if you
	// attempt to add expando properties to them.
	noData: {
		"embed": true,
		// Ban all objects except for Flash (which handle expandos)
		"object": "clsid:D27CDB6E-AE6D-11cf-96B8-444553540000",
		"applet": true
	},

	hasData: function( elem ) {
		elem = elem.nodeType ? jQuery.cache[ elem[jQuery.expando] ] : elem[ jQuery.expando ];
		return !!elem && !isEmptyDataObject( elem );
	},

	data: function( elem, name, data, pvt /* Internal Use Only */ ) {
		if ( !jQuery.acceptData( elem ) ) {
			return;
		}

		var privateCache, thisCache, ret,
			internalKey = jQuery.expando,
			getByName = typeof name === "string",

			// We have to handle DOM nodes and JS objects differently because IE6-7
			// can't GC object references properly across the DOM-JS boundary
			isNode = elem.nodeType,

			// Only DOM nodes need the global jQuery cache; JS object data is
			// attached directly to the object so GC can occur automatically
			cache = isNode ? jQuery.cache : elem,

			// Only defining an ID for JS objects if its cache already exists allows
			// the code to shortcut on the same path as a DOM node with no cache
			id = isNode ? elem[ internalKey ] : elem[ internalKey ] && internalKey,
			isEvents = name === "events";

		// Avoid doing any more work than we need to when trying to get data on an
		// object that has no data at all
		if ( (!id || !cache[id] || (!isEvents && !pvt && !cache[id].data)) && getByName && data === undefined ) {
			return;
		}

		if ( !id ) {
			// Only DOM nodes need a new unique ID for each element since their data
			// ends up in the global cache
			if ( isNode ) {
				elem[ internalKey ] = id = ++jQuery.uuid;
			} else {
				id = internalKey;
			}
		}

		if ( !cache[ id ] ) {
			cache[ id ] = {};

			// Avoids exposing jQuery metadata on plain JS objects when the object
			// is serialized using JSON.stringify
			if ( !isNode ) {
				cache[ id ].toJSON = jQuery.noop;
			}
		}

		// An object can be passed to jQuery.data instead of a key/value pair; this gets
		// shallow copied over onto the existing cache
		if ( typeof name === "object" || typeof name === "function" ) {
			if ( pvt ) {
				cache[ id ] = jQuery.extend( cache[ id ], name );
			} else {
				cache[ id ].data = jQuery.extend( cache[ id ].data, name );
			}
		}

		privateCache = thisCache = cache[ id ];

		// jQuery data() is stored in a separate object inside the object's internal data
		// cache in order to avoid key collisions between internal data and user-defined
		// data.
		if ( !pvt ) {
			if ( !thisCache.data ) {
				thisCache.data = {};
			}

			thisCache = thisCache.data;
		}

		if ( data !== undefined ) {
			thisCache[ jQuery.camelCase( name ) ] = data;
		}

		// Users should not attempt to inspect the internal events object using jQuery.data,
		// it is undocumented and subject to change. But does anyone listen? No.
		if ( isEvents && !thisCache[ name ] ) {
			return privateCache.events;
		}

		// Check for both converted-to-camel and non-converted data property names
		// If a data property was specified
		if ( getByName ) {

			// First Try to find as-is property data
			ret = thisCache[ name ];

			// Test for null|undefined property data
			if ( ret == null ) {

				// Try to find the camelCased property
				ret = thisCache[ jQuery.camelCase( name ) ];
			}
		} else {
			ret = thisCache;
		}

		return ret;
	},

	removeData: function( elem, name, pvt /* Internal Use Only */ ) {
		if ( !jQuery.acceptData( elem ) ) {
			return;
		}

		var thisCache, i, l,

			// Reference to internal data cache key
			internalKey = jQuery.expando,

			isNode = elem.nodeType,

			// See jQuery.data for more information
			cache = isNode ? jQuery.cache : elem,

			// See jQuery.data for more information
			id = isNode ? elem[ internalKey ] : internalKey;

		// If there is already no cache entry for this object, there is no
		// purpose in continuing
		if ( !cache[ id ] ) {
			return;
		}

		if ( name ) {

			thisCache = pvt ? cache[ id ] : cache[ id ].data;

			if ( thisCache ) {

				// Support array or space separated string names for data keys
				if ( !jQuery.isArray( name ) ) {

					// try the string as a key before any manipulation
					if ( name in thisCache ) {
						name = [ name ];
					} else {

						// split the camel cased version by spaces unless a key with the spaces exists
						name = jQuery.camelCase( name );
						if ( name in thisCache ) {
							name = [ name ];
						} else {
							name = name.split( " " );
						}
					}
				}

				for ( i = 0, l = name.length; i < l; i++ ) {
					delete thisCache[ name[i] ];
				}

				// If there is no data left in the cache, we want to continue
				// and let the cache object itself get destroyed
				if ( !( pvt ? isEmptyDataObject : jQuery.isEmptyObject )( thisCache ) ) {
					return;
				}
			}
		}

		// See jQuery.data for more information
		if ( !pvt ) {
			delete cache[ id ].data;

			// Don't destroy the parent cache unless the internal data object
			// had been the only thing left in it
			if ( !isEmptyDataObject(cache[ id ]) ) {
				return;
			}
		}

		// Browsers that fail expando deletion also refuse to delete expandos on
		// the window, but it will allow it on all other JS objects; other browsers
		// don't care
		// Ensure that `cache` is not a window object #10080
		if ( jQuery.support.deleteExpando || !cache.setInterval ) {
			delete cache[ id ];
		} else {
			cache[ id ] = null;
		}

		// We destroyed the cache and need to eliminate the expando on the node to avoid
		// false lookups in the cache for entries that no longer exist
		if ( isNode ) {
			// IE does not allow us to delete expando properties from nodes,
			// nor does it have a removeAttribute function on Document nodes;
			// we must handle all of these cases
			if ( jQuery.support.deleteExpando ) {
				delete elem[ internalKey ];
			} else if ( elem.removeAttribute ) {
				elem.removeAttribute( internalKey );
			} else {
				elem[ internalKey ] = null;
			}
		}
	},

	// For internal use only.
	_data: function( elem, name, data ) {
		return jQuery.data( elem, name, data, true );
	},

	// A method for determining if a DOM node can handle the data expando
	acceptData: function( elem ) {
		if ( elem.nodeName ) {
			var match = jQuery.noData[ elem.nodeName.toLowerCase() ];

			if ( match ) {
				return !(match === true || elem.getAttribute("classid") !== match);
			}
		}

		return true;
	}
});

jQuery.fn.extend({
	data: function( key, value ) {
		var parts, part, attr, name, l,
			elem = this[0],
			i = 0,
			data = null;

		// Gets all values
		if ( key === undefined ) {
			if ( this.length ) {
				data = jQuery.data( elem );

				if ( elem.nodeType === 1 && !jQuery._data( elem, "parsedAttrs" ) ) {
					attr = elem.attributes;
					for ( l = attr.length; i < l; i++ ) {
						name = attr[i].name;

						if ( name.indexOf( "data-" ) === 0 ) {
							name = jQuery.camelCase( name.substring(5) );

							dataAttr( elem, name, data[ name ] );
						}
					}
					jQuery._data( elem, "parsedAttrs", true );
				}
			}

			return data;
		}

		// Sets multiple values
		if ( typeof key === "object" ) {
			return this.each(function() {
				jQuery.data( this, key );
			});
		}

		parts = key.split( ".", 2 );
		parts[1] = parts[1] ? "." + parts[1] : "";
		part = parts[1] + "!";

		return jQuery.access( this, function( value ) {

			if ( value === undefined ) {
				data = this.triggerHandler( "getData" + part, [ parts[0] ] );

				// Try to fetch any internally stored data first
				if ( data === undefined && elem ) {
					data = jQuery.data( elem, key );
					data = dataAttr( elem, key, data );
				}

				return data === undefined && parts[1] ?
					this.data( parts[0] ) :
					data;
			}

			parts[1] = value;
			this.each(function() {
				var self = jQuery( this );

				self.triggerHandler( "setData" + part, parts );
				jQuery.data( this, key, value );
				self.triggerHandler( "changeData" + part, parts );
			});
		}, null, value, arguments.length > 1, null, false );
	},

	removeData: function( key ) {
		return this.each(function() {
			jQuery.removeData( this, key );
		});
	}
});

function dataAttr( elem, key, data ) {
	// If nothing was found internally, try to fetch any
	// data from the HTML5 data-* attribute
	if ( data === undefined && elem.nodeType === 1 ) {

		var name = "data-" + key.replace( rmultiDash, "-$1" ).toLowerCase();

		data = elem.getAttribute( name );

		if ( typeof data === "string" ) {
			try {
				data = data === "true" ? true :
				data === "false" ? false :
				data === "null" ? null :
				jQuery.isNumeric( data ) ? +data :
					rbrace.test( data ) ? jQuery.parseJSON( data ) :
					data;
			} catch( e ) {}

			// Make sure we set the data so it isn't changed later
			jQuery.data( elem, key, data );

		} else {
			data = undefined;
		}
	}

	return data;
}

// checks a cache object for emptiness
function isEmptyDataObject( obj ) {
	for ( var name in obj ) {

		// if the public data object is empty, the private is still empty
		if ( name === "data" && jQuery.isEmptyObject( obj[name] ) ) {
			continue;
		}
		if ( name !== "toJSON" ) {
			return false;
		}
	}

	return true;
}




function handleQueueMarkDefer( elem, type, src ) {
	var deferDataKey = type + "defer",
		queueDataKey = type + "queue",
		markDataKey = type + "mark",
		defer = jQuery._data( elem, deferDataKey );
	if ( defer &&
		( src === "queue" || !jQuery._data(elem, queueDataKey) ) &&
		( src === "mark" || !jQuery._data(elem, markDataKey) ) ) {
		// Give room for hard-coded callbacks to fire first
		// and eventually mark/queue something else on the element
		setTimeout( function() {
			if ( !jQuery._data( elem, queueDataKey ) &&
				!jQuery._data( elem, markDataKey ) ) {
				jQuery.removeData( elem, deferDataKey, true );
				defer.fire();
			}
		}, 0 );
	}
}

jQuery.extend({

	_mark: function( elem, type ) {
		if ( elem ) {
			type = ( type || "fx" ) + "mark";
			jQuery._data( elem, type, (jQuery._data( elem, type ) || 0) + 1 );
		}
	},

	_unmark: function( force, elem, type ) {
		if ( force !== true ) {
			type = elem;
			elem = force;
			force = false;
		}
		if ( elem ) {
			type = type || "fx";
			var key = type + "mark",
				count = force ? 0 : ( (jQuery._data( elem, key ) || 1) - 1 );
			if ( count ) {
				jQuery._data( elem, key, count );
			} else {
				jQuery.removeData( elem, key, true );
				handleQueueMarkDefer( elem, type, "mark" );
			}
		}
	},

	queue: function( elem, type, data ) {
		var q;
		if ( elem ) {
			type = ( type || "fx" ) + "queue";
			q = jQuery._data( elem, type );

			// Speed up dequeue by getting out quickly if this is just a lookup
			if ( data ) {
				if ( !q || jQuery.isArray(data) ) {
					q = jQuery._data( elem, type, jQuery.makeArray(data) );
				} else {
					q.push( data );
				}
			}
			return q || [];
		}
	},

	dequeue: function( elem, type ) {
		type = type || "fx";

		var queue = jQuery.queue( elem, type ),
			fn = queue.shift(),
			hooks = {};

		// If the fx queue is dequeued, always remove the progress sentinel
		if ( fn === "inprogress" ) {
			fn = queue.shift();
		}

		if ( fn ) {
			// Add a progress sentinel to prevent the fx queue from being
			// automatically dequeued
			if ( type === "fx" ) {
				queue.unshift( "inprogress" );
			}

			jQuery._data( elem, type + ".run", hooks );
			fn.call( elem, function() {
				jQuery.dequeue( elem, type );
			}, hooks );
		}

		if ( !queue.length ) {
			jQuery.removeData( elem, type + "queue " + type + ".run", true );
			handleQueueMarkDefer( elem, type, "queue" );
		}
	}
});

jQuery.fn.extend({
	queue: function( type, data ) {
		var setter = 2;

		if ( typeof type !== "string" ) {
			data = type;
			type = "fx";
			setter--;
		}

		if ( arguments.length < setter ) {
			return jQuery.queue( this[0], type );
		}

		return data === undefined ?
			this :
			this.each(function() {
				var queue = jQuery.queue( this, type, data );

				if ( type === "fx" && queue[0] !== "inprogress" ) {
					jQuery.dequeue( this, type );
				}
			});
	},
	dequeue: function( type ) {
		return this.each(function() {
			jQuery.dequeue( this, type );
		});
	},
	// Based off of the plugin by Clint Helfers, with permission.
	// http://blindsignals.com/index.php/2009/07/jquery-delay/
	delay: function( time, type ) {
		time = jQuery.fx ? jQuery.fx.speeds[ time ] || time : time;
		type = type || "fx";

		return this.queue( type, function( next, hooks ) {
			var timeout = setTimeout( next, time );
			hooks.stop = function() {
				clearTimeout( timeout );
			};
		});
	},
	clearQueue: function( type ) {
		return this.queue( type || "fx", [] );
	},
	// Get a promise resolved when queues of a certain type
	// are emptied (fx is the type by default)
	promise: function( type, object ) {
		if ( typeof type !== "string" ) {
			object = type;
			type = undefined;
		}
		type = type || "fx";
		var defer = jQuery.Deferred(),
			elements = this,
			i = elements.length,
			count = 1,
			deferDataKey = type + "defer",
			queueDataKey = type + "queue",
			markDataKey = type + "mark",
			tmp;
		function resolve() {
			if ( !( --count ) ) {
				defer.resolveWith( elements, [ elements ] );
			}
		}
		while( i-- ) {
			if (( tmp = jQuery.data( elements[ i ], deferDataKey, undefined, true ) ||
					( jQuery.data( elements[ i ], queueDataKey, undefined, true ) ||
						jQuery.data( elements[ i ], markDataKey, undefined, true ) ) &&
					jQuery.data( elements[ i ], deferDataKey, jQuery.Callbacks( "once memory" ), true ) )) {
				count++;
				tmp.add( resolve );
			}
		}
		resolve();
		return defer.promise( object );
	}
});




var rclass = /[\n\t\r]/g,
	rspace = /\s+/,
	rreturn = /\r/g,
	rtype = /^(?:button|input)$/i,
	rfocusable = /^(?:button|input|object|select|textarea)$/i,
	rclickable = /^a(?:rea)?$/i,
	rboolean = /^(?:autofocus|autoplay|async|checked|controls|defer|disabled|hidden|loop|multiple|open|readonly|required|scoped|selected)$/i,
	getSetAttribute = jQuery.support.getSetAttribute,
	nodeHook, boolHook, fixSpecified;

jQuery.fn.extend({
	attr: function( name, value ) {
		return jQuery.access( this, jQuery.attr, name, value, arguments.length > 1 );
	},

	removeAttr: function( name ) {
		return this.each(function() {
			jQuery.removeAttr( this, name );
		});
	},

	prop: function( name, value ) {
		return jQuery.access( this, jQuery.prop, name, value, arguments.length > 1 );
	},

	removeProp: function( name ) {
		name = jQuery.propFix[ name ] || name;
		return this.each(function() {
			// try/catch handles cases where IE balks (such as removing a property on window)
			try {
				this[ name ] = undefined;
				delete this[ name ];
			} catch( e ) {}
		});
	},

	addClass: function( value ) {
		var classNames, i, l, elem,
			setClass, c, cl;

		if ( jQuery.isFunction( value ) ) {
			return this.each(function( j ) {
				jQuery( this ).addClass( value.call(this, j, this.className) );
			});
		}

		if ( value && typeof value === "string" ) {
			classNames = value.split( rspace );

			for ( i = 0, l = this.length; i < l; i++ ) {
				elem = this[ i ];

				if ( elem.nodeType === 1 ) {
					if ( !elem.className && classNames.length === 1 ) {
						elem.className = value;

					} else {
						setClass = " " + elem.className + " ";

						for ( c = 0, cl = classNames.length; c < cl; c++ ) {
							if ( !~setClass.indexOf( " " + classNames[ c ] + " " ) ) {
								setClass += classNames[ c ] + " ";
							}
						}
						elem.className = jQuery.trim( setClass );
					}
				}
			}
		}

		return this;
	},

	removeClass: function( value ) {
		var classNames, i, l, elem, className, c, cl;

		if ( jQuery.isFunction( value ) ) {
			return this.each(function( j ) {
				jQuery( this ).removeClass( value.call(this, j, this.className) );
			});
		}

		if ( (value && typeof value === "string") || value === undefined ) {
			classNames = ( value || "" ).split( rspace );

			for ( i = 0, l = this.length; i < l; i++ ) {
				elem = this[ i ];

				if ( elem.nodeType === 1 && elem.className ) {
					if ( value ) {
						className = (" " + elem.className + " ").replace( rclass, " " );
						for ( c = 0, cl = classNames.length; c < cl; c++ ) {
							className = className.replace(" " + classNames[ c ] + " ", " ");
						}
						elem.className = jQuery.trim( className );

					} else {
						elem.className = "";
					}
				}
			}
		}

		return this;
	},

	toggleClass: function( value, stateVal ) {
		var type = typeof value,
			isBool = typeof stateVal === "boolean";

		if ( jQuery.isFunction( value ) ) {
			return this.each(function( i ) {
				jQuery( this ).toggleClass( value.call(this, i, this.className, stateVal), stateVal );
			});
		}

		return this.each(function() {
			if ( type === "string" ) {
				// toggle individual class names
				var className,
					i = 0,
					self = jQuery( this ),
					state = stateVal,
					classNames = value.split( rspace );

				while ( (className = classNames[ i++ ]) ) {
					// check each className given, space seperated list
					state = isBool ? state : !self.hasClass( className );
					self[ state ? "addClass" : "removeClass" ]( className );
				}

			} else if ( type === "undefined" || type === "boolean" ) {
				if ( this.className ) {
					// store className if set
					jQuery._data( this, "__className__", this.className );
				}

				// toggle whole className
				this.className = this.className || value === false ? "" : jQuery._data( this, "__className__" ) || "";
			}
		});
	},

	hasClass: function( selector ) {
		var className = " " + selector + " ",
			i = 0,
			l = this.length;
		for ( ; i < l; i++ ) {
			if ( this[i].nodeType === 1 && (" " + this[i].className + " ").replace(rclass, " ").indexOf( className ) > -1 ) {
				return true;
			}
		}

		return false;
	},

	val: function( value ) {
		var hooks, ret, isFunction,
			elem = this[0];

		if ( !arguments.length ) {
			if ( elem ) {
				hooks = jQuery.valHooks[ elem.type ] || jQuery.valHooks[ elem.nodeName.toLowerCase() ];

				if ( hooks && "get" in hooks && (ret = hooks.get( elem, "value" )) !== undefined ) {
					return ret;
				}

				ret = elem.value;

				return typeof ret === "string" ?
					// handle most common string cases
					ret.replace(rreturn, "") :
					// handle cases where value is null/undef or number
					ret == null ? "" : ret;
			}

			return;
		}

		isFunction = jQuery.isFunction( value );

		return this.each(function( i ) {
			var self = jQuery(this), val;

			if ( this.nodeType !== 1 ) {
				return;
			}

			if ( isFunction ) {
				val = value.call( this, i, self.val() );
			} else {
				val = value;
			}

			// Treat null/undefined as ""; convert numbers to string
			if ( val == null ) {
				val = "";
			} else if ( typeof val === "number" ) {
				val += "";
			} else if ( jQuery.isArray( val ) ) {
				val = jQuery.map(val, function ( value ) {
					return value == null ? "" : value + "";
				});
			}

			hooks = jQuery.valHooks[ this.type ] || jQuery.valHooks[ this.nodeName.toLowerCase() ];

			// If set returns undefined, fall back to normal setting
			if ( !hooks || !("set" in hooks) || hooks.set( this, val, "value" ) === undefined ) {
				this.value = val;
			}
		});
	}
});

jQuery.extend({
	valHooks: {
		option: {
			get: function( elem ) {
				// attributes.value is undefined in Blackberry 4.7 but
				// uses .value. See #6932
				var val = elem.attributes.value;
				return !val || val.specified ? elem.value : elem.text;
			}
		},
		select: {
			get: function( elem ) {
				var value, i, max, option,
					index = elem.selectedIndex,
					values = [],
					options = elem.options,
					one = elem.type === "select-one";

				// Nothing was selected
				if ( index < 0 ) {
					return null;
				}

				// Loop through all the selected options
				i = one ? index : 0;
				max = one ? index + 1 : options.length;
				for ( ; i < max; i++ ) {
					option = options[ i ];

					// Don't return options that are disabled or in a disabled optgroup
					if ( option.selected && (jQuery.support.optDisabled ? !option.disabled : option.getAttribute("disabled") === null) &&
							(!option.parentNode.disabled || !jQuery.nodeName( option.parentNode, "optgroup" )) ) {

						// Get the specific value for the option
						value = jQuery( option ).val();

						// We don't need an array for one selects
						if ( one ) {
							return value;
						}

						// Multi-Selects return an array
						values.push( value );
					}
				}

				// Fixes Bug #2551 -- select.val() broken in IE after form.reset()
				if ( one && !values.length && options.length ) {
					return jQuery( options[ index ] ).val();
				}

				return values;
			},

			set: function( elem, value ) {
				var values = jQuery.makeArray( value );

				jQuery(elem).find("option").each(function() {
					this.selected = jQuery.inArray( jQuery(this).val(), values ) >= 0;
				});

				if ( !values.length ) {
					elem.selectedIndex = -1;
				}
				return values;
			}
		}
	},

	attrFn: {
		val: true,
		css: true,
		html: true,
		text: true,
		data: true,
		width: true,
		height: true,
		offset: true
	},

	attr: function( elem, name, value, pass ) {
		var ret, hooks, notxml,
			nType = elem.nodeType;

		// don't get/set attributes on text, comment and attribute nodes
		if ( !elem || nType === 3 || nType === 8 || nType === 2 ) {
			return;
		}

		if ( pass && name in jQuery.attrFn ) {
			return jQuery( elem )[ name ]( value );
		}

		// Fallback to prop when attributes are not supported
		if ( typeof elem.getAttribute === "undefined" ) {
			return jQuery.prop( elem, name, value );
		}

		notxml = nType !== 1 || !jQuery.isXMLDoc( elem );

		// All attributes are lowercase
		// Grab necessary hook if one is defined
		if ( notxml ) {
			name = name.toLowerCase();
			hooks = jQuery.attrHooks[ name ] || ( rboolean.test( name ) ? boolHook : nodeHook );
		}

		if ( value !== undefined ) {

			if ( value === null ) {
				jQuery.removeAttr( elem, name );
				return;

			} else if ( hooks && "set" in hooks && notxml && (ret = hooks.set( elem, value, name )) !== undefined ) {
				return ret;

			} else {
				elem.setAttribute( name, "" + value );
				return value;
			}

		} else if ( hooks && "get" in hooks && notxml && (ret = hooks.get( elem, name )) !== null ) {
			return ret;

		} else {

			ret = elem.getAttribute( name );

			// Non-existent attributes return null, we normalize to undefined
			return ret === null ?
				undefined :
				ret;
		}
	},

	removeAttr: function( elem, value ) {
		var propName, attrNames, name, l, isBool,
			i = 0;

		if ( value && elem.nodeType === 1 ) {
			attrNames = value.toLowerCase().split( rspace );
			l = attrNames.length;

			for ( ; i < l; i++ ) {
				name = attrNames[ i ];

				if ( name ) {
					propName = jQuery.propFix[ name ] || name;
					isBool = rboolean.test( name );

					// See #9699 for explanation of this approach (setting first, then removal)
					// Do not do this for boolean attributes (see #10870)
					if ( !isBool ) {
						jQuery.attr( elem, name, "" );
					}
					elem.removeAttribute( getSetAttribute ? name : propName );

					// Set corresponding property to false for boolean attributes
					if ( isBool && propName in elem ) {
						elem[ propName ] = false;
					}
				}
			}
		}
	},

	attrHooks: {
		type: {
			set: function( elem, value ) {
				// We can't allow the type property to be changed (since it causes problems in IE)
				if ( rtype.test( elem.nodeName ) && elem.parentNode ) {
					jQuery.error( "type property can't be changed" );
				} else if ( !jQuery.support.radioValue && value === "radio" && jQuery.nodeName(elem, "input") ) {
					// Setting the type on a radio button after the value resets the value in IE6-9
					// Reset value to it's default in case type is set after value
					// This is for element creation
					var val = elem.value;
					elem.setAttribute( "type", value );
					if ( val ) {
						elem.value = val;
					}
					return value;
				}
			}
		},
		// Use the value property for back compat
		// Use the nodeHook for button elements in IE6/7 (#1954)
		value: {
			get: function( elem, name ) {
				if ( nodeHook && jQuery.nodeName( elem, "button" ) ) {
					return nodeHook.get( elem, name );
				}
				return name in elem ?
					elem.value :
					null;
			},
			set: function( elem, value, name ) {
				if ( nodeHook && jQuery.nodeName( elem, "button" ) ) {
					return nodeHook.set( elem, value, name );
				}
				// Does not return so that setAttribute is also used
				elem.value = value;
			}
		}
	},

	propFix: {
		tabindex: "tabIndex",
		readonly: "readOnly",
		"for": "htmlFor",
		"class": "className",
		maxlength: "maxLength",
		cellspacing: "cellSpacing",
		cellpadding: "cellPadding",
		rowspan: "rowSpan",
		colspan: "colSpan",
		usemap: "useMap",
		frameborder: "frameBorder",
		contenteditable: "contentEditable"
	},

	prop: function( elem, name, value ) {
		var ret, hooks, notxml,
			nType = elem.nodeType;

		// don't get/set properties on text, comment and attribute nodes
		if ( !elem || nType === 3 || nType === 8 || nType === 2 ) {
			return;
		}

		notxml = nType !== 1 || !jQuery.isXMLDoc( elem );

		if ( notxml ) {
			// Fix name and attach hooks
			name = jQuery.propFix[ name ] || name;
			hooks = jQuery.propHooks[ name ];
		}

		if ( value !== undefined ) {
			if ( hooks && "set" in hooks && (ret = hooks.set( elem, value, name )) !== undefined ) {
				return ret;

			} else {
				return ( elem[ name ] = value );
			}

		} else {
			if ( hooks && "get" in hooks && (ret = hooks.get( elem, name )) !== null ) {
				return ret;

			} else {
				return elem[ name ];
			}
		}
	},

	propHooks: {
		tabIndex: {
			get: function( elem ) {
				// elem.tabIndex doesn't always return the correct value when it hasn't been explicitly set
				// http://fluidproject.org/blog/2008/01/09/getting-setting-and-removing-tabindex-values-with-javascript/
				var attributeNode = elem.getAttributeNode("tabindex");

				return attributeNode && attributeNode.specified ?
					parseInt( attributeNode.value, 10 ) :
					rfocusable.test( elem.nodeName ) || rclickable.test( elem.nodeName ) && elem.href ?
						0 :
						undefined;
			}
		}
	}
});

// Add the tabIndex propHook to attrHooks for back-compat (different case is intentional)
jQuery.attrHooks.tabindex = jQuery.propHooks.tabIndex;

// Hook for boolean attributes
boolHook = {
	get: function( elem, name ) {
		// Align boolean attributes with corresponding properties
		// Fall back to attribute presence where some booleans are not supported
		var attrNode,
			property = jQuery.prop( elem, name );
		return property === true || typeof property !== "boolean" && ( attrNode = elem.getAttributeNode(name) ) && attrNode.nodeValue !== false ?
			name.toLowerCase() :
			undefined;
	},
	set: function( elem, value, name ) {
		var propName;
		if ( value === false ) {
			// Remove boolean attributes when set to false
			jQuery.removeAttr( elem, name );
		} else {
			// value is true since we know at this point it's type boolean and not false
			// Set boolean attributes to the same name and set the DOM property
			propName = jQuery.propFix[ name ] || name;
			if ( propName in elem ) {
				// Only set the IDL specifically if it already exists on the element
				elem[ propName ] = true;
			}

			elem.setAttribute( name, name.toLowerCase() );
		}
		return name;
	}
};

// IE6/7 do not support getting/setting some attributes with get/setAttribute
if ( !getSetAttribute ) {

	fixSpecified = {
		name: true,
		id: true,
		coords: true
	};

	// Use this for any attribute in IE6/7
	// This fixes almost every IE6/7 issue
	nodeHook = jQuery.valHooks.button = {
		get: function( elem, name ) {
			var ret;
			ret = elem.getAttributeNode( name );
			return ret && ( fixSpecified[ name ] ? ret.nodeValue !== "" : ret.specified ) ?
				ret.nodeValue :
				undefined;
		},
		set: function( elem, value, name ) {
			// Set the existing or create a new attribute node
			var ret = elem.getAttributeNode( name );
			if ( !ret ) {
				ret = document.createAttribute( name );
				elem.setAttributeNode( ret );
			}
			return ( ret.nodeValue = value + "" );
		}
	};

	// Apply the nodeHook to tabindex
	jQuery.attrHooks.tabindex.set = nodeHook.set;

	// Set width and height to auto instead of 0 on empty string( Bug #8150 )
	// This is for removals
	jQuery.each([ "width", "height" ], function( i, name ) {
		jQuery.attrHooks[ name ] = jQuery.extend( jQuery.attrHooks[ name ], {
			set: function( elem, value ) {
				if ( value === "" ) {
					elem.setAttribute( name, "auto" );
					return value;
				}
			}
		});
	});

	// Set contenteditable to false on removals(#10429)
	// Setting to empty string throws an error as an invalid value
	jQuery.attrHooks.contenteditable = {
		get: nodeHook.get,
		set: function( elem, value, name ) {
			if ( value === "" ) {
				value = "false";
			}
			nodeHook.set( elem, value, name );
		}
	};
}


// Some attributes require a special call on IE
if ( !jQuery.support.hrefNormalized ) {
	jQuery.each([ "href", "src", "width", "height" ], function( i, name ) {
		jQuery.attrHooks[ name ] = jQuery.extend( jQuery.attrHooks[ name ], {
			get: function( elem ) {
				var ret = elem.getAttribute( name, 2 );
				return ret === null ? undefined : ret;
			}
		});
	});
}

if ( !jQuery.support.style ) {
	jQuery.attrHooks.style = {
		get: function( elem ) {
			// Return undefined in the case of empty string
			// Normalize to lowercase since IE uppercases css property names
			return elem.style.cssText.toLowerCase() || undefined;
		},
		set: function( elem, value ) {
			return ( elem.style.cssText = "" + value );
		}
	};
}

// Safari mis-reports the default selected property of an option
// Accessing the parent's selectedIndex property fixes it
if ( !jQuery.support.optSelected ) {
	jQuery.propHooks.selected = jQuery.extend( jQuery.propHooks.selected, {
		get: function( elem ) {
			var parent = elem.parentNode;

			if ( parent ) {
				parent.selectedIndex;

				// Make sure that it also works with optgroups, see #5701
				if ( parent.parentNode ) {
					parent.parentNode.selectedIndex;
				}
			}
			return null;
		}
	});
}

// IE6/7 call enctype encoding
if ( !jQuery.support.enctype ) {
	jQuery.propFix.enctype = "encoding";
}

// Radios and checkboxes getter/setter
if ( !jQuery.support.checkOn ) {
	jQuery.each([ "radio", "checkbox" ], function() {
		jQuery.valHooks[ this ] = {
			get: function( elem ) {
				// Handle the case where in Webkit "" is returned instead of "on" if a value isn't specified
				return elem.getAttribute("value") === null ? "on" : elem.value;
			}
		};
	});
}
jQuery.each([ "radio", "checkbox" ], function() {
	jQuery.valHooks[ this ] = jQuery.extend( jQuery.valHooks[ this ], {
		set: function( elem, value ) {
			if ( jQuery.isArray( value ) ) {
				return ( elem.checked = jQuery.inArray( jQuery(elem).val(), value ) >= 0 );
			}
		}
	});
});




var rformElems = /^(?:textarea|input|select)$/i,
	rtypenamespace = /^([^\.]*)?(?:\.(.+))?$/,
	rhoverHack = /(?:^|\s)hover(\.\S+)?\b/,
	rkeyEvent = /^key/,
	rmouseEvent = /^(?:mouse|contextmenu)|click/,
	rfocusMorph = /^(?:focusinfocus|focusoutblur)$/,
	rquickIs = /^(\w*)(?:#([\w\-]+))?(?:\.([\w\-]+))?$/,
	quickParse = function( selector ) {
		var quick = rquickIs.exec( selector );
		if ( quick ) {
			//   0  1    2   3
			// [ _, tag, id, class ]
			quick[1] = ( quick[1] || "" ).toLowerCase();
			quick[3] = quick[3] && new RegExp( "(?:^|\\s)" + quick[3] + "(?:\\s|$)" );
		}
		return quick;
	},
	quickIs = function( elem, m ) {
		var attrs = elem.attributes || {};
		return (
			(!m[1] || elem.nodeName.toLowerCase() === m[1]) &&
			(!m[2] || (attrs.id || {}).value === m[2]) &&
			(!m[3] || m[3].test( (attrs[ "class" ] || {}).value ))
		);
	},
	hoverHack = function( events ) {
		return jQuery.event.special.hover ? events : events.replace( rhoverHack, "mouseenter$1 mouseleave$1" );
	};

/*
 * Helper functions for managing events -- not part of the public interface.
 * Props to Dean Edwards' addEvent library for many of the ideas.
 */
jQuery.event = {

	add: function( elem, types, handler, data, selector ) {

		var elemData, eventHandle, events,
			t, tns, type, namespaces, handleObj,
			handleObjIn, quick, handlers, special;

		// Don't attach events to noData or text/comment nodes (allow plain objects tho)
		if ( elem.nodeType === 3 || elem.nodeType === 8 || !types || !handler || !(elemData = jQuery._data( elem )) ) {
			return;
		}

		// Caller can pass in an object of custom data in lieu of the handler
		if ( handler.handler ) {
			handleObjIn = handler;
			handler = handleObjIn.handler;
			selector = handleObjIn.selector;
		}

		// Make sure that the handler has a unique ID, used to find/remove it later
		if ( !handler.guid ) {
			handler.guid = jQuery.guid++;
		}

		// Init the element's event structure and main handler, if this is the first
		events = elemData.events;
		if ( !events ) {
			elemData.events = events = {};
		}
		eventHandle = elemData.handle;
		if ( !eventHandle ) {
			elemData.handle = eventHandle = function( e ) {
				// Discard the second event of a jQuery.event.trigger() and
				// when an event is called after a page has unloaded
				return typeof jQuery !== "undefined" && (!e || jQuery.event.triggered !== e.type) ?
					jQuery.event.dispatch.apply( eventHandle.elem, arguments ) :
					undefined;
			};
			// Add elem as a property of the handle fn to prevent a memory leak with IE non-native events
			eventHandle.elem = elem;
		}

		// Handle multiple events separated by a space
		// jQuery(...).bind("mouseover mouseout", fn);
		types = jQuery.trim( hoverHack(types) ).split( " " );
		for ( t = 0; t < types.length; t++ ) {

			tns = rtypenamespace.exec( types[t] ) || [];
			type = tns[1];
			namespaces = ( tns[2] || "" ).split( "." ).sort();

			// If event changes its type, use the special event handlers for the changed type
			special = jQuery.event.special[ type ] || {};

			// If selector defined, determine special event api type, otherwise given type
			type = ( selector ? special.delegateType : special.bindType ) || type;

			// Update special based on newly reset type
			special = jQuery.event.special[ type ] || {};

			// handleObj is passed to all event handlers
			handleObj = jQuery.extend({
				type: type,
				origType: tns[1],
				data: data,
				handler: handler,
				guid: handler.guid,
				selector: selector,
				quick: selector && quickParse( selector ),
				namespace: namespaces.join(".")
			}, handleObjIn );

			// Init the event handler queue if we're the first
			handlers = events[ type ];
			if ( !handlers ) {
				handlers = events[ type ] = [];
				handlers.delegateCount = 0;

				// Only use addEventListener/attachEvent if the special events handler returns false
				if ( !special.setup || special.setup.call( elem, data, namespaces, eventHandle ) === false ) {
					// Bind the global event handler to the element
					if ( elem.addEventListener ) {
						elem.addEventListener( type, eventHandle, false );

					} else if ( elem.attachEvent ) {
						elem.attachEvent( "on" + type, eventHandle );
					}
				}
			}

			if ( special.add ) {
				special.add.call( elem, handleObj );

				if ( !handleObj.handler.guid ) {
					handleObj.handler.guid = handler.guid;
				}
			}

			// Add to the element's handler list, delegates in front
			if ( selector ) {
				handlers.splice( handlers.delegateCount++, 0, handleObj );
			} else {
				handlers.push( handleObj );
			}

			// Keep track of which events have ever been used, for event optimization
			jQuery.event.global[ type ] = true;
		}

		// Nullify elem to prevent memory leaks in IE
		elem = null;
	},

	global: {},

	// Detach an event or set of events from an element
	remove: function( elem, types, handler, selector, mappedTypes ) {

		var elemData = jQuery.hasData( elem ) && jQuery._data( elem ),
			t, tns, type, origType, namespaces, origCount,
			j, events, special, handle, eventType, handleObj;

		if ( !elemData || !(events = elemData.events) ) {
			return;
		}

		// Once for each type.namespace in types; type may be omitted
		types = jQuery.trim( hoverHack( types || "" ) ).split(" ");
		for ( t = 0; t < types.length; t++ ) {
			tns = rtypenamespace.exec( types[t] ) || [];
			type = origType = tns[1];
			namespaces = tns[2];

			// Unbind all events (on this namespace, if provided) for the element
			if ( !type ) {
				for ( type in events ) {
					jQuery.event.remove( elem, type + types[ t ], handler, selector, true );
				}
				continue;
			}

			special = jQuery.event.special[ type ] || {};
			type = ( selector? special.delegateType : special.bindType ) || type;
			eventType = events[ type ] || [];
			origCount = eventType.length;
			namespaces = namespaces ? new RegExp("(^|\\.)" + namespaces.split(".").sort().join("\\.(?:.*\\.)?") + "(\\.|$)") : null;

			// Remove matching events
			for ( j = 0; j < eventType.length; j++ ) {
				handleObj = eventType[ j ];

				if ( ( mappedTypes || origType === handleObj.origType ) &&
					 ( !handler || handler.guid === handleObj.guid ) &&
					 ( !namespaces || namespaces.test( handleObj.namespace ) ) &&
					 ( !selector || selector === handleObj.selector || selector === "**" && handleObj.selector ) ) {
					eventType.splice( j--, 1 );

					if ( handleObj.selector ) {
						eventType.delegateCount--;
					}
					if ( special.remove ) {
						special.remove.call( elem, handleObj );
					}
				}
			}

			// Remove generic event handler if we removed something and no more handlers exist
			// (avoids potential for endless recursion during removal of special event handlers)
			if ( eventType.length === 0 && origCount !== eventType.length ) {
				if ( !special.teardown || special.teardown.call( elem, namespaces ) === false ) {
					jQuery.removeEvent( elem, type, elemData.handle );
				}

				delete events[ type ];
			}
		}

		// Remove the expando if it's no longer used
		if ( jQuery.isEmptyObject( events ) ) {
			handle = elemData.handle;
			if ( handle ) {
				handle.elem = null;
			}

			// removeData also checks for emptiness and clears the expando if empty
			// so use it instead of delete
			jQuery.removeData( elem, [ "events", "handle" ], true );
		}
	},

	// Events that are safe to short-circuit if no handlers are attached.
	// Native DOM events should not be added, they may have inline handlers.
	customEvent: {
		"getData": true,
		"setData": true,
		"changeData": true
	},

	trigger: function( event, data, elem, onlyHandlers ) {
		// Don't do events on text and comment nodes
		if ( elem && (elem.nodeType === 3 || elem.nodeType === 8) ) {
			return;
		}

		// Event object or event type
		var type = event.type || event,
			namespaces = [],
			cache, exclusive, i, cur, old, ontype, special, handle, eventPath, bubbleType;

		// focus/blur morphs to focusin/out; ensure we're not firing them right now
		if ( rfocusMorph.test( type + jQuery.event.triggered ) ) {
			return;
		}

		if ( type.indexOf( "!" ) >= 0 ) {
			// Exclusive events trigger only for the exact event (no namespaces)
			type = type.slice(0, -1);
			exclusive = true;
		}

		if ( type.indexOf( "." ) >= 0 ) {
			// Namespaced trigger; create a regexp to match event type in handle()
			namespaces = type.split(".");
			type = namespaces.shift();
			namespaces.sort();
		}

		if ( (!elem || jQuery.event.customEvent[ type ]) && !jQuery.event.global[ type ] ) {
			// No jQuery handlers for this event type, and it can't have inline handlers
			return;
		}

		// Caller can pass in an Event, Object, or just an event type string
		event = typeof event === "object" ?
			// jQuery.Event object
			event[ jQuery.expando ] ? event :
			// Object literal
			new jQuery.Event( type, event ) :
			// Just the event type (string)
			new jQuery.Event( type );

		event.type = type;
		event.isTrigger = true;
		event.exclusive = exclusive;
		event.namespace = namespaces.join( "." );
		event.namespace_re = event.namespace? new RegExp("(^|\\.)" + namespaces.join("\\.(?:.*\\.)?") + "(\\.|$)") : null;
		ontype = type.indexOf( ":" ) < 0 ? "on" + type : "";

		// Handle a global trigger
		if ( !elem ) {

			// TODO: Stop taunting the data cache; remove global events and always attach to document
			cache = jQuery.cache;
			for ( i in cache ) {
				if ( cache[ i ].events && cache[ i ].events[ type ] ) {
					jQuery.event.trigger( event, data, cache[ i ].handle.elem, true );
				}
			}
			return;
		}

		// Clean up the event in case it is being reused
		event.result = undefined;
		if ( !event.target ) {
			event.target = elem;
		}

		// Clone any incoming data and prepend the event, creating the handler arg list
		data = data != null ? jQuery.makeArray( data ) : [];
		data.unshift( event );

		// Allow special events to draw outside the lines
		special = jQuery.event.special[ type ] || {};
		if ( special.trigger && special.trigger.apply( elem, data ) === false ) {
			return;
		}

		// Determine event propagation path in advance, per W3C events spec (#9951)
		// Bubble up to document, then to window; watch for a global ownerDocument var (#9724)
		eventPath = [[ elem, special.bindType || type ]];
		if ( !onlyHandlers && !special.noBubble && !jQuery.isWindow( elem ) ) {

			bubbleType = special.delegateType || type;
			cur = rfocusMorph.test( bubbleType + type ) ? elem : elem.parentNode;
			old = null;
			for ( ; cur; cur = cur.parentNode ) {
				eventPath.push([ cur, bubbleType ]);
				old = cur;
			}

			// Only add window if we got to document (e.g., not plain obj or detached DOM)
			if ( old && old === elem.ownerDocument ) {
				eventPath.push([ old.defaultView || old.parentWindow || window, bubbleType ]);
			}
		}

		// Fire handlers on the event path
		for ( i = 0; i < eventPath.length && !event.isPropagationStopped(); i++ ) {

			cur = eventPath[i][0];
			event.type = eventPath[i][1];

			handle = ( jQuery._data( cur, "events" ) || {} )[ event.type ] && jQuery._data( cur, "handle" );
			if ( handle ) {
				handle.apply( cur, data );
			}
			// Note that this is a bare JS function and not a jQuery handler
			handle = ontype && cur[ ontype ];
			if ( handle && jQuery.acceptData( cur ) && handle.apply( cur, data ) === false ) {
				event.preventDefault();
			}
		}
		event.type = type;

		// If nobody prevented the default action, do it now
		if ( !onlyHandlers && !event.isDefaultPrevented() ) {

			if ( (!special._default || special._default.apply( elem.ownerDocument, data ) === false) &&
				!(type === "click" && jQuery.nodeName( elem, "a" )) && jQuery.acceptData( elem ) ) {

				// Call a native DOM method on the target with the same name name as the event.
				// Can't use an .isFunction() check here because IE6/7 fails that test.
				// Don't do default actions on window, that's where global variables be (#6170)
				// IE<9 dies on focus/blur to hidden element (#1486)
				if ( ontype && elem[ type ] && ((type !== "focus" && type !== "blur") || event.target.offsetWidth !== 0) && !jQuery.isWindow( elem ) ) {

					// Don't re-trigger an onFOO event when we call its FOO() method
					old = elem[ ontype ];

					if ( old ) {
						elem[ ontype ] = null;
					}

					// Prevent re-triggering of the same event, since we already bubbled it above
					jQuery.event.triggered = type;
					elem[ type ]();
					jQuery.event.triggered = undefined;

					if ( old ) {
						elem[ ontype ] = old;
					}
				}
			}
		}

		return event.result;
	},

	dispatch: function( event ) {

		// Make a writable jQuery.Event from the native event object
		event = jQuery.event.fix( event || window.event );

		var handlers = ( (jQuery._data( this, "events" ) || {} )[ event.type ] || []),
			delegateCount = handlers.delegateCount,
			args = [].slice.call( arguments, 0 ),
			run_all = !event.exclusive && !event.namespace,
			special = jQuery.event.special[ event.type ] || {},
			handlerQueue = [],
			i, j, cur, jqcur, ret, selMatch, matched, matches, handleObj, sel, related;

		// Use the fix-ed jQuery.Event rather than the (read-only) native event
		args[0] = event;
		event.delegateTarget = this;

		// Call the preDispatch hook for the mapped type, and let it bail if desired
		if ( special.preDispatch && special.preDispatch.call( this, event ) === false ) {
			return;
		}

		// Determine handlers that should run if there are delegated events
		// Avoid non-left-click bubbling in Firefox (#3861)
		if ( delegateCount && !(event.button && event.type === "click") ) {

			// Pregenerate a single jQuery object for reuse with .is()
			jqcur = jQuery(this);
			jqcur.context = this.ownerDocument || this;

			for ( cur = event.target; cur != this; cur = cur.parentNode || this ) {

				// Don't process events on disabled elements (#6911, #8165)
				if ( cur.disabled !== true ) {
					selMatch = {};
					matches = [];
					jqcur[0] = cur;
					for ( i = 0; i < delegateCount; i++ ) {
						handleObj = handlers[ i ];
						sel = handleObj.selector;

						if ( selMatch[ sel ] === undefined ) {
							selMatch[ sel ] = (
								handleObj.quick ? quickIs( cur, handleObj.quick ) : jqcur.is( sel )
							);
						}
						if ( selMatch[ sel ] ) {
							matches.push( handleObj );
						}
					}
					if ( matches.length ) {
						handlerQueue.push({ elem: cur, matches: matches });
					}
				}
			}
		}

		// Add the remaining (directly-bound) handlers
		if ( handlers.length > delegateCount ) {
			handlerQueue.push({ elem: this, matches: handlers.slice( delegateCount ) });
		}

		// Run delegates first; they may want to stop propagation beneath us
		for ( i = 0; i < handlerQueue.length && !event.isPropagationStopped(); i++ ) {
			matched = handlerQueue[ i ];
			event.currentTarget = matched.elem;

			for ( j = 0; j < matched.matches.length && !event.isImmediatePropagationStopped(); j++ ) {
				handleObj = matched.matches[ j ];

				// Triggered event must either 1) be non-exclusive and have no namespace, or
				// 2) have namespace(s) a subset or equal to those in the bound event (both can have no namespace).
				if ( run_all || (!event.namespace && !handleObj.namespace) || event.namespace_re && event.namespace_re.test( handleObj.namespace ) ) {

					event.data = handleObj.data;
					event.handleObj = handleObj;

					ret = ( (jQuery.event.special[ handleObj.origType ] || {}).handle || handleObj.handler )
							.apply( matched.elem, args );

					if ( ret !== undefined ) {
						event.result = ret;
						if ( ret === false ) {
							event.preventDefault();
							event.stopPropagation();
						}
					}
				}
			}
		}

		// Call the postDispatch hook for the mapped type
		if ( special.postDispatch ) {
			special.postDispatch.call( this, event );
		}

		return event.result;
	},

	// Includes some event props shared by KeyEvent and MouseEvent
	// *** attrChange attrName relatedNode srcElement  are not normalized, non-W3C, deprecated, will be removed in 1.8 ***
	props: "attrChange attrName relatedNode srcElement altKey bubbles cancelable ctrlKey currentTarget eventPhase metaKey relatedTarget shiftKey target timeStamp view which".split(" "),

	fixHooks: {},

	keyHooks: {
		props: "char charCode key keyCode".split(" "),
		filter: function( event, original ) {

			// Add which for key events
			if ( event.which == null ) {
				event.which = original.charCode != null ? original.charCode : original.keyCode;
			}

			return event;
		}
	},

	mouseHooks: {
		props: "button buttons clientX clientY fromElement offsetX offsetY pageX pageY screenX screenY toElement".split(" "),
		filter: function( event, original ) {
			var eventDoc, doc, body,
				button = original.button,
				fromElement = original.fromElement;

			// Calculate pageX/Y if missing and clientX/Y available
			if ( event.pageX == null && original.clientX != null ) {
				eventDoc = event.target.ownerDocument || document;
				doc = eventDoc.documentElement;
				body = eventDoc.body;

				event.pageX = original.clientX + ( doc && doc.scrollLeft || body && body.scrollLeft || 0 ) - ( doc && doc.clientLeft || body && body.clientLeft || 0 );
				event.pageY = original.clientY + ( doc && doc.scrollTop  || body && body.scrollTop  || 0 ) - ( doc && doc.clientTop  || body && body.clientTop  || 0 );
			}

			// Add relatedTarget, if necessary
			if ( !event.relatedTarget && fromElement ) {
				event.relatedTarget = fromElement === event.target ? original.toElement : fromElement;
			}

			// Add which for click: 1 === left; 2 === middle; 3 === right
			// Note: button is not normalized, so don't use it
			if ( !event.which && button !== undefined ) {
				event.which = ( button & 1 ? 1 : ( button & 2 ? 3 : ( button & 4 ? 2 : 0 ) ) );
			}

			return event;
		}
	},

	fix: function( event ) {
		if ( event[ jQuery.expando ] ) {
			return event;
		}

		// Create a writable copy of the event object and normalize some properties
		var i, prop,
			originalEvent = event,
			fixHook = jQuery.event.fixHooks[ event.type ] || {},
			copy = fixHook.props ? this.props.concat( fixHook.props ) : this.props;

		event = jQuery.Event( originalEvent );

		for ( i = copy.length; i; ) {
			prop = copy[ --i ];
			event[ prop ] = originalEvent[ prop ];
		}

		// Fix target property, if necessary (#1925, IE 6/7/8 & Safari2)
		if ( !event.target ) {
			event.target = originalEvent.srcElement || document;
		}

		// Target should not be a text node (#504, Safari)
		if ( event.target.nodeType === 3 ) {
			event.target = event.target.parentNode;
		}

		// For mouse/key events; add metaKey if it's not there (#3368, IE6/7/8)
		if ( event.metaKey === undefined ) {
			event.metaKey = event.ctrlKey;
		}

		return fixHook.filter? fixHook.filter( event, originalEvent ) : event;
	},

	special: {
		ready: {
			// Make sure the ready event is setup
			setup: jQuery.bindReady
		},

		load: {
			// Prevent triggered image.load events from bubbling to window.load
			noBubble: true
		},

		focus: {
			delegateType: "focusin"
		},
		blur: {
			delegateType: "focusout"
		},

		beforeunload: {
			setup: function( data, namespaces, eventHandle ) {
				// We only want to do this special case on windows
				if ( jQuery.isWindow( this ) ) {
					this.onbeforeunload = eventHandle;
				}
			},

			teardown: function( namespaces, eventHandle ) {
				if ( this.onbeforeunload === eventHandle ) {
					this.onbeforeunload = null;
				}
			}
		}
	},

	simulate: function( type, elem, event, bubble ) {
		// Piggyback on a donor event to simulate a different one.
		// Fake originalEvent to avoid donor's stopPropagation, but if the
		// simulated event prevents default then we do the same on the donor.
		var e = jQuery.extend(
			new jQuery.Event(),
			event,
			{ type: type,
				isSimulated: true,
				originalEvent: {}
			}
		);
		if ( bubble ) {
			jQuery.event.trigger( e, null, elem );
		} else {
			jQuery.event.dispatch.call( elem, e );
		}
		if ( e.isDefaultPrevented() ) {
			event.preventDefault();
		}
	}
};

// Some plugins are using, but it's undocumented/deprecated and will be removed.
// The 1.7 special event interface should provide all the hooks needed now.
jQuery.event.handle = jQuery.event.dispatch;

jQuery.removeEvent = document.removeEventListener ?
	function( elem, type, handle ) {
		if ( elem.removeEventListener ) {
			elem.removeEventListener( type, handle, false );
		}
	} :
	function( elem, type, handle ) {
		if ( elem.detachEvent ) {
			elem.detachEvent( "on" + type, handle );
		}
	};

jQuery.Event = function( src, props ) {
	// Allow instantiation without the 'new' keyword
	if ( !(this instanceof jQuery.Event) ) {
		return new jQuery.Event( src, props );
	}

	// Event object
	if ( src && src.type ) {
		this.originalEvent = src;
		this.type = src.type;

		// Events bubbling up the document may have been marked as prevented
		// by a handler lower down the tree; reflect the correct value.
		this.isDefaultPrevented = ( src.defaultPrevented || src.returnValue === false ||
			src.getPreventDefault && src.getPreventDefault() ) ? returnTrue : returnFalse;

	// Event type
	} else {
		this.type = src;
	}

	// Put explicitly provided properties onto the event object
	if ( props ) {
		jQuery.extend( this, props );
	}

	// Create a timestamp if incoming event doesn't have one
	this.timeStamp = src && src.timeStamp || jQuery.now();

	// Mark it as fixed
	this[ jQuery.expando ] = true;
};

function returnFalse() {
	return false;
}
function returnTrue() {
	return true;
}

// jQuery.Event is based on DOM3 Events as specified by the ECMAScript Language Binding
// http://www.w3.org/TR/2003/WD-DOM-Level-3-Events-20030331/ecma-script-binding.html
jQuery.Event.prototype = {
	preventDefault: function() {
		this.isDefaultPrevented = returnTrue;

		var e = this.originalEvent;
		if ( !e ) {
			return;
		}

		// if preventDefault exists run it on the original event
		if ( e.preventDefault ) {
			e.preventDefault();

		// otherwise set the returnValue property of the original event to false (IE)
		} else {
			e.returnValue = false;
		}
	},
	stopPropagation: function() {
		this.isPropagationStopped = returnTrue;

		var e = this.originalEvent;
		if ( !e ) {
			return;
		}
		// if stopPropagation exists run it on the original event
		if ( e.stopPropagation ) {
			e.stopPropagation();
		}
		// otherwise set the cancelBubble property of the original event to true (IE)
		e.cancelBubble = true;
	},
	stopImmediatePropagation: function() {
		this.isImmediatePropagationStopped = returnTrue;
		this.stopPropagation();
	},
	isDefaultPrevented: returnFalse,
	isPropagationStopped: returnFalse,
	isImmediatePropagationStopped: returnFalse
};

// Create mouseenter/leave events using mouseover/out and event-time checks
jQuery.each({
	mouseenter: "mouseover",
	mouseleave: "mouseout"
}, function( orig, fix ) {
	jQuery.event.special[ orig ] = {
		delegateType: fix,
		bindType: fix,

		handle: function( event ) {
			var target = this,
				related = event.relatedTarget,
				handleObj = event.handleObj,
				selector = handleObj.selector,
				ret;

			// For mousenter/leave call the handler if related is outside the target.
			// NB: No relatedTarget if the mouse left/entered the browser window
			if ( !related || (related !== target && !jQuery.contains( target, related )) ) {
				event.type = handleObj.origType;
				ret = handleObj.handler.apply( this, arguments );
				event.type = fix;
			}
			return ret;
		}
	};
});

// IE submit delegation
if ( !jQuery.support.submitBubbles ) {

	jQuery.event.special.submit = {
		setup: function() {
			// Only need this for delegated form submit events
			if ( jQuery.nodeName( this, "form" ) ) {
				return false;
			}

			// Lazy-add a submit handler when a descendant form may potentially be submitted
			jQuery.event.add( this, "click._submit keypress._submit", function( e ) {
				// Node name check avoids a VML-related crash in IE (#9807)
				var elem = e.target,
					form = jQuery.nodeName( elem, "input" ) || jQuery.nodeName( elem, "button" ) ? elem.form : undefined;
				if ( form && !form._submit_attached ) {
					jQuery.event.add( form, "submit._submit", function( event ) {
						event._submit_bubble = true;
					});
					form._submit_attached = true;
				}
			});
			// return undefined since we don't need an event listener
		},
		
		postDispatch: function( event ) {
			// If form was submitted by the user, bubble the event up the tree
			if ( event._submit_bubble ) {
				delete event._submit_bubble;
				if ( this.parentNode && !event.isTrigger ) {
					jQuery.event.simulate( "submit", this.parentNode, event, true );
				}
			}
		},

		teardown: function() {
			// Only need this for delegated form submit events
			if ( jQuery.nodeName( this, "form" ) ) {
				return false;
			}

			// Remove delegated handlers; cleanData eventually reaps submit handlers attached above
			jQuery.event.remove( this, "._submit" );
		}
	};
}

// IE change delegation and checkbox/radio fix
if ( !jQuery.support.changeBubbles ) {

	jQuery.event.special.change = {

		setup: function() {

			if ( rformElems.test( this.nodeName ) ) {
				// IE doesn't fire change on a check/radio until blur; trigger it on click
				// after a propertychange. Eat the blur-change in special.change.handle.
				// This still fires onchange a second time for check/radio after blur.
				if ( this.type === "checkbox" || this.type === "radio" ) {
					jQuery.event.add( this, "propertychange._change", function( event ) {
						if ( event.originalEvent.propertyName === "checked" ) {
							this._just_changed = true;
						}
					});
					jQuery.event.add( this, "click._change", function( event ) {
						if ( this._just_changed && !event.isTrigger ) {
							this._just_changed = false;
							jQuery.event.simulate( "change", this, event, true );
						}
					});
				}
				return false;
			}
			// Delegated event; lazy-add a change handler on descendant inputs
			jQuery.event.add( this, "beforeactivate._change", function( e ) {
				var elem = e.target;

				if ( rformElems.test( elem.nodeName ) && !elem._change_attached ) {
					jQuery.event.add( elem, "change._change", function( event ) {
						if ( this.parentNode && !event.isSimulated && !event.isTrigger ) {
							jQuery.event.simulate( "change", this.parentNode, event, true );
						}
					});
					elem._change_attached = true;
				}
			});
		},

		handle: function( event ) {
			var elem = event.target;

			// Swallow native change events from checkbox/radio, we already triggered them above
			if ( this !== elem || event.isSimulated || event.isTrigger || (elem.type !== "radio" && elem.type !== "checkbox") ) {
				return event.handleObj.handler.apply( this, arguments );
			}
		},

		teardown: function() {
			jQuery.event.remove( this, "._change" );

			return rformElems.test( this.nodeName );
		}
	};
}

// Create "bubbling" focus and blur events
if ( !jQuery.support.focusinBubbles ) {
	jQuery.each({ focus: "focusin", blur: "focusout" }, function( orig, fix ) {

		// Attach a single capturing handler while someone wants focusin/focusout
		var attaches = 0,
			handler = function( event ) {
				jQuery.event.simulate( fix, event.target, jQuery.event.fix( event ), true );
			};

		jQuery.event.special[ fix ] = {
			setup: function() {
				if ( attaches++ === 0 ) {
					document.addEventListener( orig, handler, true );
				}
			},
			teardown: function() {
				if ( --attaches === 0 ) {
					document.removeEventListener( orig, handler, true );
				}
			}
		};
	});
}

jQuery.fn.extend({

	on: function( types, selector, data, fn, /*INTERNAL*/ one ) {
		var origFn, type;

		// Types can be a map of types/handlers
		if ( typeof types === "object" ) {
			// ( types-Object, selector, data )
			if ( typeof selector !== "string" ) { // && selector != null
				// ( types-Object, data )
				data = data || selector;
				selector = undefined;
			}
			for ( type in types ) {
				this.on( type, selector, data, types[ type ], one );
			}
			return this;
		}

		if ( data == null && fn == null ) {
			// ( types, fn )
			fn = selector;
			data = selector = undefined;
		} else if ( fn == null ) {
			if ( typeof selector === "string" ) {
				// ( types, selector, fn )
				fn = data;
				data = undefined;
			} else {
				// ( types, data, fn )
				fn = data;
				data = selector;
				selector = undefined;
			}
		}
		if ( fn === false ) {
			fn = returnFalse;
		} else if ( !fn ) {
			return this;
		}

		if ( one === 1 ) {
			origFn = fn;
			fn = function( event ) {
				// Can use an empty set, since event contains the info
				jQuery().off( event );
				return origFn.apply( this, arguments );
			};
			// Use same guid so caller can remove using origFn
			fn.guid = origFn.guid || ( origFn.guid = jQuery.guid++ );
		}
		return this.each( function() {
			jQuery.event.add( this, types, fn, data, selector );
		});
	},
	one: function( types, selector, data, fn ) {
		return this.on( types, selector, data, fn, 1 );
	},
	off: function( types, selector, fn ) {
		if ( types && types.preventDefault && types.handleObj ) {
			// ( event )  dispatched jQuery.Event
			var handleObj = types.handleObj;
			jQuery( types.delegateTarget ).off(
				handleObj.namespace ? handleObj.origType + "." + handleObj.namespace : handleObj.origType,
				handleObj.selector,
				handleObj.handler
			);
			return this;
		}
		if ( typeof types === "object" ) {
			// ( types-object [, selector] )
			for ( var type in types ) {
				this.off( type, selector, types[ type ] );
			}
			return this;
		}
		if ( selector === false || typeof selector === "function" ) {
			// ( types [, fn] )
			fn = selector;
			selector = undefined;
		}
		if ( fn === false ) {
			fn = returnFalse;
		}
		return this.each(function() {
			jQuery.event.remove( this, types, fn, selector );
		});
	},

	bind: function( types, data, fn ) {
		return this.on( types, null, data, fn );
	},
	unbind: function( types, fn ) {
		return this.off( types, null, fn );
	},

	live: function( types, data, fn ) {
		jQuery( this.context ).on( types, this.selector, data, fn );
		return this;
	},
	die: function( types, fn ) {
		jQuery( this.context ).off( types, this.selector || "**", fn );
		return this;
	},

	delegate: function( selector, types, data, fn ) {
		return this.on( types, selector, data, fn );
	},
	undelegate: function( selector, types, fn ) {
		// ( namespace ) or ( selector, types [, fn] )
		return arguments.length == 1? this.off( selector, "**" ) : this.off( types, selector, fn );
	},

	trigger: function( type, data ) {
		return this.each(function() {
			jQuery.event.trigger( type, data, this );
		});
	},
	triggerHandler: function( type, data ) {
		if ( this[0] ) {
			return jQuery.event.trigger( type, data, this[0], true );
		}
	},

	toggle: function( fn ) {
		// Save reference to arguments for access in closure
		var args = arguments,
			guid = fn.guid || jQuery.guid++,
			i = 0,
			toggler = function( event ) {
				// Figure out which function to execute
				var lastToggle = ( jQuery._data( this, "lastToggle" + fn.guid ) || 0 ) % i;
				jQuery._data( this, "lastToggle" + fn.guid, lastToggle + 1 );

				// Make sure that clicks stop
				event.preventDefault();

				// and execute the function
				return args[ lastToggle ].apply( this, arguments ) || false;
			};

		// link all the functions, so any of them can unbind this click handler
		toggler.guid = guid;
		while ( i < args.length ) {
			args[ i++ ].guid = guid;
		}

		return this.click( toggler );
	},

	hover: function( fnOver, fnOut ) {
		return this.mouseenter( fnOver ).mouseleave( fnOut || fnOver );
	}
});

jQuery.each( ("blur focus focusin focusout load resize scroll unload click dblclick " +
	"mousedown mouseup mousemove mouseover mouseout mouseenter mouseleave " +
	"change select submit keydown keypress keyup error contextmenu").split(" "), function( i, name ) {

	// Handle event binding
	jQuery.fn[ name ] = function( data, fn ) {
		if ( fn == null ) {
			fn = data;
			data = null;
		}

		return arguments.length > 0 ?
			this.on( name, null, data, fn ) :
			this.trigger( name );
	};

	if ( jQuery.attrFn ) {
		jQuery.attrFn[ name ] = true;
	}

	if ( rkeyEvent.test( name ) ) {
		jQuery.event.fixHooks[ name ] = jQuery.event.keyHooks;
	}

	if ( rmouseEvent.test( name ) ) {
		jQuery.event.fixHooks[ name ] = jQuery.event.mouseHooks;
	}
});



/*!
 * Sizzle CSS Selector Engine
 *  Copyright 2011, The Dojo Foundation
 *  Released under the MIT, BSD, and GPL Licenses.
 *  More information: http://sizzlejs.com/
 */
(function(){

var chunker = /((?:\((?:\([^()]+\)|[^()]+)+\)|\[(?:\[[^\[\]]*\]|['"][^'"]*['"]|[^\[\]'"]+)+\]|\\.|[^ >+~,(\[\\]+)+|[>+~])(\s*,\s*)?((?:.|\r|\n)*)/g,
	expando = "sizcache" + (Math.random() + '').replace('.', ''),
	done = 0,
	toString = Object.prototype.toString,
	hasDuplicate = false,
	baseHasDuplicate = true,
	rBackslash = /\\/g,
	rReturn = /\r\n/g,
	rNonWord = /\W/;

// Here we check if the JavaScript engine is using some sort of
// optimization where it does not always call our comparision
// function. If that is the case, discard the hasDuplicate value.
//   Thus far that includes Google Chrome.
[0, 0].sort(function() {
	baseHasDuplicate = false;
	return 0;
});

var Sizzle = function( selector, context, results, seed ) {
	results = results || [];
	context = context || document;

	var origContext = context;

	if ( context.nodeType !== 1 && context.nodeType !== 9 ) {
		return [];
	}

	if ( !selector || typeof selector !== "string" ) {
		return results;
	}

	var m, set, checkSet, extra, ret, cur, pop, i,
		prune = true,
		contextXML = Sizzle.isXML( context ),
		parts = [],
		soFar = selector;

	// Reset the position of the chunker regexp (start from head)
	do {
		chunker.exec( "" );
		m = chunker.exec( soFar );

		if ( m ) {
			soFar = m[3];

			parts.push( m[1] );

			if ( m[2] ) {
				extra = m[3];
				break;
			}
		}
	} while ( m );

	if ( parts.length > 1 && origPOS.exec( selector ) ) {

		if ( parts.length === 2 && Expr.relative[ parts[0] ] ) {
			set = posProcess( parts[0] + parts[1], context, seed );

		} else {
			set = Expr.relative[ parts[0] ] ?
				[ context ] :
				Sizzle( parts.shift(), context );

			while ( parts.length ) {
				selector = parts.shift();

				if ( Expr.relative[ selector ] ) {
					selector += parts.shift();
				}

				set = posProcess( selector, set, seed );
			}
		}

	} else {
		// Take a shortcut and set the context if the root selector is an ID
		// (but not if it'll be faster if the inner selector is an ID)
		if ( !seed && parts.length > 1 && context.nodeType === 9 && !contextXML &&
				Expr.match.ID.test(parts[0]) && !Expr.match.ID.test(parts[parts.length - 1]) ) {

			ret = Sizzle.find( parts.shift(), context, contextXML );
			context = ret.expr ?
				Sizzle.filter( ret.expr, ret.set )[0] :
				ret.set[0];
		}

		if ( context ) {
			ret = seed ?
				{ expr: parts.pop(), set: makeArray(seed) } :
				Sizzle.find( parts.pop(), parts.length === 1 && (parts[0] === "~" || parts[0] === "+") && context.parentNode ? context.parentNode : context, contextXML );

			set = ret.expr ?
				Sizzle.filter( ret.expr, ret.set ) :
				ret.set;

			if ( parts.length > 0 ) {
				checkSet = makeArray( set );

			} else {
				prune = false;
			}

			while ( parts.length ) {
				cur = parts.pop();
				pop = cur;

				if ( !Expr.relative[ cur ] ) {
					cur = "";
				} else {
					pop = parts.pop();
				}

				if ( pop == null ) {
					pop = context;
				}

				Expr.relative[ cur ]( checkSet, pop, contextXML );
			}

		} else {
			checkSet = parts = [];
		}
	}

	if ( !checkSet ) {
		checkSet = set;
	}

	if ( !checkSet ) {
		Sizzle.error( cur || selector );
	}

	if ( toString.call(checkSet) === "[object Array]" ) {
		if ( !prune ) {
			results.push.apply( results, checkSet );

		} else if ( context && context.nodeType === 1 ) {
			for ( i = 0; checkSet[i] != null; i++ ) {
				if ( checkSet[i] && (checkSet[i] === true || checkSet[i].nodeType === 1 && Sizzle.contains(context, checkSet[i])) ) {
					results.push( set[i] );
				}
			}

		} else {
			for ( i = 0; checkSet[i] != null; i++ ) {
				if ( checkSet[i] && checkSet[i].nodeType === 1 ) {
					results.push( set[i] );
				}
			}
		}

	} else {
		makeArray( checkSet, results );
	}

	if ( extra ) {
		Sizzle( extra, origContext, results, seed );
		Sizzle.uniqueSort( results );
	}

	return results;
};

Sizzle.uniqueSort = function( results ) {
	if ( sortOrder ) {
		hasDuplicate = baseHasDuplicate;
		results.sort( sortOrder );

		if ( hasDuplicate ) {
			for ( var i = 1; i < results.length; i++ ) {
				if ( results[i] === results[ i - 1 ] ) {
					results.splice( i--, 1 );
				}
			}
		}
	}

	return results;
};

Sizzle.matches = function( expr, set ) {
	return Sizzle( expr, null, null, set );
};

Sizzle.matchesSelector = function( node, expr ) {
	return Sizzle( expr, null, null, [node] ).length > 0;
};

Sizzle.find = function( expr, context, isXML ) {
	var set, i, len, match, type, left;

	if ( !expr ) {
		return [];
	}

	for ( i = 0, len = Expr.order.length; i < len; i++ ) {
		type = Expr.order[i];

		if ( (match = Expr.leftMatch[ type ].exec( expr )) ) {
			left = match[1];
			match.splice( 1, 1 );

			if ( left.substr( left.length - 1 ) !== "\\" ) {
				match[1] = (match[1] || "").replace( rBackslash, "" );
				set = Expr.find[ type ]( match, context, isXML );

				if ( set != null ) {
					expr = expr.replace( Expr.match[ type ], "" );
					break;
				}
			}
		}
	}

	if ( !set ) {
		set = typeof context.getElementsByTagName !== "undefined" ?
			context.getElementsByTagName( "*" ) :
			[];
	}

	return { set: set, expr: expr };
};

Sizzle.filter = function( expr, set, inplace, not ) {
	var match, anyFound,
		type, found, item, filter, left,
		i, pass,
		old = expr,
		result = [],
		curLoop = set,
		isXMLFilter = set && set[0] && Sizzle.isXML( set[0] );

	while ( expr && set.length ) {
		for ( type in Expr.filter ) {
			if ( (match = Expr.leftMatch[ type ].exec( expr )) != null && match[2] ) {
				filter = Expr.filter[ type ];
				left = match[1];

				anyFound = false;

				match.splice(1,1);

				if ( left.substr( left.length - 1 ) === "\\" ) {
					continue;
				}

				if ( curLoop === result ) {
					result = [];
				}

				if ( Expr.preFilter[ type ] ) {
					match = Expr.preFilter[ type ]( match, curLoop, inplace, result, not, isXMLFilter );

					if ( !match ) {
						anyFound = found = true;

					} else if ( match === true ) {
						continue;
					}
				}

				if ( match ) {
					for ( i = 0; (item = curLoop[i]) != null; i++ ) {
						if ( item ) {
							found = filter( item, match, i, curLoop );
							pass = not ^ found;

							if ( inplace && found != null ) {
								if ( pass ) {
									anyFound = true;

								} else {
									curLoop[i] = false;
								}

							} else if ( pass ) {
								result.push( item );
								anyFound = true;
							}
						}
					}
				}

				if ( found !== undefined ) {
					if ( !inplace ) {
						curLoop = result;
					}

					expr = expr.replace( Expr.match[ type ], "" );

					if ( !anyFound ) {
						return [];
					}

					break;
				}
			}
		}

		// Improper expression
		if ( expr === old ) {
			if ( anyFound == null ) {
				Sizzle.error( expr );

			} else {
				break;
			}
		}

		old = expr;
	}

	return curLoop;
};

Sizzle.error = function( msg ) {
	throw new Error( "Syntax error, unrecognized expression: " + msg );
};

/**
 * Utility function for retreiving the text value of an array of DOM nodes
 * @param {Array|Element} elem
 */
var getText = Sizzle.getText = function( elem ) {
    var i, node,
		nodeType = elem.nodeType,
		ret = "";

	if ( nodeType ) {
		if ( nodeType === 1 || nodeType === 9 || nodeType === 11 ) {
			// Use textContent || innerText for elements
			if ( typeof elem.textContent === 'string' ) {
				return elem.textContent;
			} else if ( typeof elem.innerText === 'string' ) {
				// Replace IE's carriage returns
				return elem.innerText.replace( rReturn, '' );
			} else {
				// Traverse it's children
				for ( elem = elem.firstChild; elem; elem = elem.nextSibling) {
					ret += getText( elem );
				}
			}
		} else if ( nodeType === 3 || nodeType === 4 ) {
			return elem.nodeValue;
		}
	} else {

		// If no nodeType, this is expected to be an array
		for ( i = 0; (node = elem[i]); i++ ) {
			// Do not traverse comment nodes
			if ( node.nodeType !== 8 ) {
				ret += getText( node );
			}
		}
	}
	return ret;
};

var Expr = Sizzle.selectors = {
	order: [ "ID", "NAME", "TAG" ],

	match: {
		ID: /#((?:[\w\u00c0-\uFFFF\-]|\\.)+)/,
		CLASS: /\.((?:[\w\u00c0-\uFFFF\-]|\\.)+)/,
		NAME: /\[name=['"]*((?:[\w\u00c0-\uFFFF\-]|\\.)+)['"]*\]/,
		ATTR: /\[\s*((?:[\w\u00c0-\uFFFF\-]|\\.)+)\s*(?:(\S?=)\s*(?:(['"])(.*?)\3|(#?(?:[\w\u00c0-\uFFFF\-]|\\.)*)|)|)\s*\]/,
		TAG: /^((?:[\w\u00c0-\uFFFF\*\-]|\\.)+)/,
		CHILD: /:(only|nth|last|first)-child(?:\(\s*(even|odd|(?:[+\-]?\d+|(?:[+\-]?\d*)?n\s*(?:[+\-]\s*\d+)?))\s*\))?/,
		POS: /:(nth|eq|gt|lt|first|last|even|odd)(?:\((\d*)\))?(?=[^\-]|$)/,
		PSEUDO: /:((?:[\w\u00c0-\uFFFF\-]|\\.)+)(?:\((['"]?)((?:\([^\)]+\)|[^\(\)]*)+)\2\))?/
	},

	leftMatch: {},

	attrMap: {
		"class": "className",
		"for": "htmlFor"
	},

	attrHandle: {
		href: function( elem ) {
			return elem.getAttribute( "href" );
		},
		type: function( elem ) {
			return elem.getAttribute( "type" );
		}
	},

	relative: {
		"+": function(checkSet, part){
			var isPartStr = typeof part === "string",
				isTag = isPartStr && !rNonWord.test( part ),
				isPartStrNotTag = isPartStr && !isTag;

			if ( isTag ) {
				part = part.toLowerCase();
			}

			for ( var i = 0, l = checkSet.length, elem; i < l; i++ ) {
				if ( (elem = checkSet[i]) ) {
					while ( (elem = elem.previousSibling) && elem.nodeType !== 1 ) {}

					checkSet[i] = isPartStrNotTag || elem && elem.nodeName.toLowerCase() === part ?
						elem || false :
						elem === part;
				}
			}

			if ( isPartStrNotTag ) {
				Sizzle.filter( part, checkSet, true );
			}
		},

		">": function( checkSet, part ) {
			var elem,
				isPartStr = typeof part === "string",
				i = 0,
				l = checkSet.length;

			if ( isPartStr && !rNonWord.test( part ) ) {
				part = part.toLowerCase();

				for ( ; i < l; i++ ) {
					elem = checkSet[i];

					if ( elem ) {
						var parent = elem.parentNode;
						checkSet[i] = parent.nodeName.toLowerCase() === part ? parent : false;
					}
				}

			} else {
				for ( ; i < l; i++ ) {
					elem = checkSet[i];

					if ( elem ) {
						checkSet[i] = isPartStr ?
							elem.parentNode :
							elem.parentNode === part;
					}
				}

				if ( isPartStr ) {
					Sizzle.filter( part, checkSet, true );
				}
			}
		},

		"": function(checkSet, part, isXML){
			var nodeCheck,
				doneName = done++,
				checkFn = dirCheck;

			if ( typeof part === "string" && !rNonWord.test( part ) ) {
				part = part.toLowerCase();
				nodeCheck = part;
				checkFn = dirNodeCheck;
			}

			checkFn( "parentNode", part, doneName, checkSet, nodeCheck, isXML );
		},

		"~": function( checkSet, part, isXML ) {
			var nodeCheck,
				doneName = done++,
				checkFn = dirCheck;

			if ( typeof part === "string" && !rNonWord.test( part ) ) {
				part = part.toLowerCase();
				nodeCheck = part;
				checkFn = dirNodeCheck;
			}

			checkFn( "previousSibling", part, doneName, checkSet, nodeCheck, isXML );
		}
	},

	find: {
		ID: function( match, context, isXML ) {
			if ( typeof context.getElementById !== "undefined" && !isXML ) {
				var m = context.getElementById(match[1]);
				// Check parentNode to catch when Blackberry 4.6 returns
				// nodes that are no longer in the document #6963
				return m && m.parentNode ? [m] : [];
			}
		},

		NAME: function( match, context ) {
			if ( typeof context.getElementsByName !== "undefined" ) {
				var ret = [],
					results = context.getElementsByName( match[1] );

				for ( var i = 0, l = results.length; i < l; i++ ) {
					if ( results[i].getAttribute("name") === match[1] ) {
						ret.push( results[i] );
					}
				}

				return ret.length === 0 ? null : ret;
			}
		},

		TAG: function( match, context ) {
			if ( typeof context.getElementsByTagName !== "undefined" ) {
				return context.getElementsByTagName( match[1] );
			}
		}
	},
	preFilter: {
		CLASS: function( match, curLoop, inplace, result, not, isXML ) {
			match = " " + match[1].replace( rBackslash, "" ) + " ";

			if ( isXML ) {
				return match;
			}

			for ( var i = 0, elem; (elem = curLoop[i]) != null; i++ ) {
				if ( elem ) {
					if ( not ^ (elem.className && (" " + elem.className + " ").replace(/[\t\n\r]/g, " ").indexOf(match) >= 0) ) {
						if ( !inplace ) {
							result.push( elem );
						}

					} else if ( inplace ) {
						curLoop[i] = false;
					}
				}
			}

			return false;
		},

		ID: function( match ) {
			return match[1].replace( rBackslash, "" );
		},

		TAG: function( match, curLoop ) {
			return match[1].replace( rBackslash, "" ).toLowerCase();
		},

		CHILD: function( match ) {
			if ( match[1] === "nth" ) {
				if ( !match[2] ) {
					Sizzle.error( match[0] );
				}

				match[2] = match[2].replace(/^\+|\s*/g, '');

				// parse equations like 'even', 'odd', '5', '2n', '3n+2', '4n-1', '-n+6'
				var test = /(-?)(\d*)(?:n([+\-]?\d*))?/.exec(
					match[2] === "even" && "2n" || match[2] === "odd" && "2n+1" ||
					!/\D/.test( match[2] ) && "0n+" + match[2] || match[2]);

				// calculate the numbers (first)n+(last) including if they are negative
				match[2] = (test[1] + (test[2] || 1)) - 0;
				match[3] = test[3] - 0;
			}
			else if ( match[2] ) {
				Sizzle.error( match[0] );
			}

			// TODO: Move to normal caching system
			match[0] = done++;

			return match;
		},

		ATTR: function( match, curLoop, inplace, result, not, isXML ) {
			var name = match[1] = match[1].replace( rBackslash, "" );

			if ( !isXML && Expr.attrMap[name] ) {
				match[1] = Expr.attrMap[name];
			}

			// Handle if an un-quoted value was used
			match[4] = ( match[4] || match[5] || "" ).replace( rBackslash, "" );

			if ( match[2] === "~=" ) {
				match[4] = " " + match[4] + " ";
			}

			return match;
		},

		PSEUDO: function( match, curLoop, inplace, result, not ) {
			if ( match[1] === "not" ) {
				// If we're dealing with a complex expression, or a simple one
				if ( ( chunker.exec(match[3]) || "" ).length > 1 || /^\w/.test(match[3]) ) {
					match[3] = Sizzle(match[3], null, null, curLoop);

				} else {
					var ret = Sizzle.filter(match[3], curLoop, inplace, true ^ not);

					if ( !inplace ) {
						result.push.apply( result, ret );
					}

					return false;
				}

			} else if ( Expr.match.POS.test( match[0] ) || Expr.match.CHILD.test( match[0] ) ) {
				return true;
			}

			return match;
		},

		POS: function( match ) {
			match.unshift( true );

			return match;
		}
	},

	filters: {
		enabled: function( elem ) {
			return elem.disabled === false && elem.type !== "hidden";
		},

		disabled: function( elem ) {
			return elem.disabled === true;
		},

		checked: function( elem ) {
			return elem.checked === true;
		},

		selected: function( elem ) {
			// Accessing this property makes selected-by-default
			// options in Safari work properly
			if ( elem.parentNode ) {
				elem.parentNode.selectedIndex;
			}

			return elem.selected === true;
		},

		parent: function( elem ) {
			return !!elem.firstChild;
		},

		empty: function( elem ) {
			return !elem.firstChild;
		},

		has: function( elem, i, match ) {
			return !!Sizzle( match[3], elem ).length;
		},

		header: function( elem ) {
			return (/h\d/i).test( elem.nodeName );
		},

		text: function( elem ) {
			var attr = elem.getAttribute( "type" ), type = elem.type;
			// IE6 and 7 will map elem.type to 'text' for new HTML5 types (search, etc)
			// use getAttribute instead to test this case
			return elem.nodeName.toLowerCase() === "input" && "text" === type && ( attr === type || attr === null );
		},

		radio: function( elem ) {
			return elem.nodeName.toLowerCase() === "input" && "radio" === elem.type;
		},

		checkbox: function( elem ) {
			return elem.nodeName.toLowerCase() === "input" && "checkbox" === elem.type;
		},

		file: function( elem ) {
			return elem.nodeName.toLowerCase() === "input" && "file" === elem.type;
		},

		password: function( elem ) {
			return elem.nodeName.toLowerCase() === "input" && "password" === elem.type;
		},

		submit: function( elem ) {
			var name = elem.nodeName.toLowerCase();
			return (name === "input" || name === "button") && "submit" === elem.type;
		},

		image: function( elem ) {
			return elem.nodeName.toLowerCase() === "input" && "image" === elem.type;
		},

		reset: function( elem ) {
			var name = elem.nodeName.toLowerCase();
			return (name === "input" || name === "button") && "reset" === elem.type;
		},

		button: function( elem ) {
			var name = elem.nodeName.toLowerCase();
			return name === "input" && "button" === elem.type || name === "button";
		},

		input: function( elem ) {
			return (/input|select|textarea|button/i).test( elem.nodeName );
		},

		focus: function( elem ) {
			return elem === elem.ownerDocument.activeElement;
		}
	},
	setFilters: {
		first: function( elem, i ) {
			return i === 0;
		},

		last: function( elem, i, match, array ) {
			return i === array.length - 1;
		},

		even: function( elem, i ) {
			return i % 2 === 0;
		},

		odd: function( elem, i ) {
			return i % 2 === 1;
		},

		lt: function( elem, i, match ) {
			return i < match[3] - 0;
		},

		gt: function( elem, i, match ) {
			return i > match[3] - 0;
		},

		nth: function( elem, i, match ) {
			return match[3] - 0 === i;
		},

		eq: function( elem, i, match ) {
			return match[3] - 0 === i;
		}
	},
	filter: {
		PSEUDO: function( elem, match, i, array ) {
			var name = match[1],
				filter = Expr.filters[ name ];

			if ( filter ) {
				return filter( elem, i, match, array );

			} else if ( name === "contains" ) {
				return (elem.textContent || elem.innerText || getText([ elem ]) || "").indexOf(match[3]) >= 0;

			} else if ( name === "not" ) {
				var not = match[3];

				for ( var j = 0, l = not.length; j < l; j++ ) {
					if ( not[j] === elem ) {
						return false;
					}
				}

				return true;

			} else {
				Sizzle.error( name );
			}
		},

		CHILD: function( elem, match ) {
			var first, last,
				doneName, parent, cache,
				count, diff,
				type = match[1],
				node = elem;

			switch ( type ) {
				case "only":
				case "first":
					while ( (node = node.previousSibling) ) {
						if ( node.nodeType === 1 ) {
							return false;
						}
					}

					if ( type === "first" ) {
						return true;
					}

					node = elem;

					/* falls through */
				case "last":
					while ( (node = node.nextSibling) ) {
						if ( node.nodeType === 1 ) {
							return false;
						}
					}

					return true;

				case "nth":
					first = match[2];
					last = match[3];

					if ( first === 1 && last === 0 ) {
						return true;
					}

					doneName = match[0];
					parent = elem.parentNode;

					if ( parent && (parent[ expando ] !== doneName || !elem.nodeIndex) ) {
						count = 0;

						for ( node = parent.firstChild; node; node = node.nextSibling ) {
							if ( node.nodeType === 1 ) {
								node.nodeIndex = ++count;
							}
						}

						parent[ expando ] = doneName;
					}

					diff = elem.nodeIndex - last;

					if ( first === 0 ) {
						return diff === 0;

					} else {
						return ( diff % first === 0 && diff / first >= 0 );
					}
			}
		},

		ID: function( elem, match ) {
			return elem.nodeType === 1 && elem.getAttribute("id") === match;
		},

		TAG: function( elem, match ) {
			return (match === "*" && elem.nodeType === 1) || !!elem.nodeName && elem.nodeName.toLowerCase() === match;
		},

		CLASS: function( elem, match ) {
			return (" " + (elem.className || elem.getAttribute("class")) + " ")
				.indexOf( match ) > -1;
		},

		ATTR: function( elem, match ) {
			var name = match[1],
				result = Sizzle.attr ?
					Sizzle.attr( elem, name ) :
					Expr.attrHandle[ name ] ?
					Expr.attrHandle[ name ]( elem ) :
					elem[ name ] != null ?
						elem[ name ] :
						elem.getAttribute( name ),
				value = result + "",
				type = match[2],
				check = match[4];

			return result == null ?
				type === "!=" :
				!type && Sizzle.attr ?
				result != null :
				type === "=" ?
				value === check :
				type === "*=" ?
				value.indexOf(check) >= 0 :
				type === "~=" ?
				(" " + value + " ").indexOf(check) >= 0 :
				!check ?
				value && result !== false :
				type === "!=" ?
				value !== check :
				type === "^=" ?
				value.indexOf(check) === 0 :
				type === "$=" ?
				value.substr(value.length - check.length) === check :
				type === "|=" ?
				value === check || value.substr(0, check.length + 1) === check + "-" :
				false;
		},

		POS: function( elem, match, i, array ) {
			var name = match[2],
				filter = Expr.setFilters[ name ];

			if ( filter ) {
				return filter( elem, i, match, array );
			}
		}
	}
};

var origPOS = Expr.match.POS,
	fescape = function(all, num){
		return "\\" + (num - 0 + 1);
	};

for ( var type in Expr.match ) {
	Expr.match[ type ] = new RegExp( Expr.match[ type ].source + (/(?![^\[]*\])(?![^\(]*\))/.source) );
	Expr.leftMatch[ type ] = new RegExp( /(^(?:.|\r|\n)*?)/.source + Expr.match[ type ].source.replace(/\\(\d+)/g, fescape) );
}
// Expose origPOS
// "global" as in regardless of relation to brackets/parens
Expr.match.globalPOS = origPOS;

var makeArray = function( array, results ) {
	array = Array.prototype.slice.call( array, 0 );

	if ( results ) {
		results.push.apply( results, array );
		return results;
	}

	return array;
};

// Perform a simple check to determine if the browser is capable of
// converting a NodeList to an array using builtin methods.
// Also verifies that the returned array holds DOM nodes
// (which is not the case in the Blackberry browser)
try {
	Array.prototype.slice.call( document.documentElement.childNodes, 0 )[0].nodeType;

// Provide a fallback method if it does not work
} catch( e ) {
	makeArray = function( array, results ) {
		var i = 0,
			ret = results || [];

		if ( toString.call(array) === "[object Array]" ) {
			Array.prototype.push.apply( ret, array );

		} else {
			if ( typeof array.length === "number" ) {
				for ( var l = array.length; i < l; i++ ) {
					ret.push( array[i] );
				}

			} else {
				for ( ; array[i]; i++ ) {
					ret.push( array[i] );
				}
			}
		}

		return ret;
	};
}

var sortOrder, siblingCheck;

if ( document.documentElement.compareDocumentPosition ) {
	sortOrder = function( a, b ) {
		if ( a === b ) {
			hasDuplicate = true;
			return 0;
		}

		if ( !a.compareDocumentPosition || !b.compareDocumentPosition ) {
			return a.compareDocumentPosition ? -1 : 1;
		}

		return a.compareDocumentPosition(b) & 4 ? -1 : 1;
	};

} else {
	sortOrder = function( a, b ) {
		// The nodes are identical, we can exit early
		if ( a === b ) {
			hasDuplicate = true;
			return 0;

		// Fallback to using sourceIndex (in IE) if it's available on both nodes
		} else if ( a.sourceIndex && b.sourceIndex ) {
			return a.sourceIndex - b.sourceIndex;
		}

		var al, bl,
			ap = [],
			bp = [],
			aup = a.parentNode,
			bup = b.parentNode,
			cur = aup;

		// If the nodes are siblings (or identical) we can do a quick check
		if ( aup === bup ) {
			return siblingCheck( a, b );

		// If no parents were found then the nodes are disconnected
		} else if ( !aup ) {
			return -1;

		} else if ( !bup ) {
			return 1;
		}

		// Otherwise they're somewhere else in the tree so we need
		// to build up a full list of the parentNodes for comparison
		while ( cur ) {
			ap.unshift( cur );
			cur = cur.parentNode;
		}

		cur = bup;

		while ( cur ) {
			bp.unshift( cur );
			cur = cur.parentNode;
		}

		al = ap.length;
		bl = bp.length;

		// Start walking down the tree looking for a discrepancy
		for ( var i = 0; i < al && i < bl; i++ ) {
			if ( ap[i] !== bp[i] ) {
				return siblingCheck( ap[i], bp[i] );
			}
		}

		// We ended someplace up the tree so do a sibling check
		return i === al ?
			siblingCheck( a, bp[i], -1 ) :
			siblingCheck( ap[i], b, 1 );
	};

	siblingCheck = function( a, b, ret ) {
		if ( a === b ) {
			return ret;
		}

		var cur = a.nextSibling;

		while ( cur ) {
			if ( cur === b ) {
				return -1;
			}

			cur = cur.nextSibling;
		}

		return 1;
	};
}

// Check to see if the browser returns elements by name when
// querying by getElementById (and provide a workaround)
(function(){
	// We're going to inject a fake input element with a specified name
	var form = document.createElement("div"),
		id = "script" + (new Date()).getTime(),
		root = document.documentElement;

	form.innerHTML = "<a name='" + id + "'/>";

	// Inject it into the root element, check its status, and remove it quickly
	root.insertBefore( form, root.firstChild );

	// The workaround has to do additional checks after a getElementById
	// Which slows things down for other browsers (hence the branching)
	if ( document.getElementById( id ) ) {
		Expr.find.ID = function( match, context, isXML ) {
			if ( typeof context.getElementById !== "undefined" && !isXML ) {
				var m = context.getElementById(match[1]);

				return m ?
					m.id === match[1] || typeof m.getAttributeNode !== "undefined" && m.getAttributeNode("id").nodeValue === match[1] ?
						[m] :
						undefined :
					[];
			}
		};

		Expr.filter.ID = function( elem, match ) {
			var node = typeof elem.getAttributeNode !== "undefined" && elem.getAttributeNode("id");

			return elem.nodeType === 1 && node && node.nodeValue === match;
		};
	}

	root.removeChild( form );

	// release memory in IE
	root = form = null;
})();

(function(){
	// Check to see if the browser returns only elements
	// when doing getElementsByTagName("*")

	// Create a fake element
	var div = document.createElement("div");
	div.appendChild( document.createComment("") );

	// Make sure no comments are found
	if ( div.getElementsByTagName("*").length > 0 ) {
		Expr.find.TAG = function( match, context ) {
			var results = context.getElementsByTagName( match[1] );

			// Filter out possible comments
			if ( match[1] === "*" ) {
				var tmp = [];

				for ( var i = 0; results[i]; i++ ) {
					if ( results[i].nodeType === 1 ) {
						tmp.push( results[i] );
					}
				}

				results = tmp;
			}

			return results;
		};
	}

	// Check to see if an attribute returns normalized href attributes
	div.innerHTML = "<a href='#'></a>";

	if ( div.firstChild && typeof div.firstChild.getAttribute !== "undefined" &&
			div.firstChild.getAttribute("href") !== "#" ) {

		Expr.attrHandle.href = function( elem ) {
			return elem.getAttribute( "href", 2 );
		};
	}

	// release memory in IE
	div = null;
})();

if ( document.querySelectorAll ) {
	(function(){
		var oldSizzle = Sizzle,
			div = document.createElement("div"),
			id = "__sizzle__";

		div.innerHTML = "<p class='TEST'></p>";

		// Safari can't handle uppercase or unicode characters when
		// in quirks mode.
		if ( div.querySelectorAll && div.querySelectorAll(".TEST").length === 0 ) {
			return;
		}

		Sizzle = function( query, context, extra, seed ) {
			context = context || document;

			// Only use querySelectorAll on non-XML documents
			// (ID selectors don't work in non-HTML documents)
			if ( !seed && !Sizzle.isXML(context) ) {
				// See if we find a selector to speed up
				var match = /^(\w+$)|^\.([\w\-]+$)|^#([\w\-]+$)/.exec( query );

				if ( match && (context.nodeType === 1 || context.nodeType === 9) ) {
					// Speed-up: Sizzle("TAG")
					if ( match[1] ) {
						return makeArray( context.getElementsByTagName( query ), extra );

					// Speed-up: Sizzle(".CLASS")
					} else if ( match[2] && Expr.find.CLASS && context.getElementsByClassName ) {
						return makeArray( context.getElementsByClassName( match[2] ), extra );
					}
				}

				if ( context.nodeType === 9 ) {
					// Speed-up: Sizzle("body")
					// The body element only exists once, optimize finding it
					if ( query === "body" && context.body ) {
						return makeArray( [ context.body ], extra );

					// Speed-up: Sizzle("#ID")
					} else if ( match && match[3] ) {
						var elem = context.getElementById( match[3] );

						// Check parentNode to catch when Blackberry 4.6 returns
						// nodes that are no longer in the document #6963
						if ( elem && elem.parentNode ) {
							// Handle the case where IE and Opera return items
							// by name instead of ID
							if ( elem.id === match[3] ) {
								return makeArray( [ elem ], extra );
							}

						} else {
							return makeArray( [], extra );
						}
					}

					try {
						return makeArray( context.querySelectorAll(query), extra );
					} catch(qsaError) {}

				// qSA works strangely on Element-rooted queries
				// We can work around this by specifying an extra ID on the root
				// and working up from there (Thanks to Andrew Dupont for the technique)
				// IE 8 doesn't work on object elements
				} else if ( context.nodeType === 1 && context.nodeName.toLowerCase() !== "object" ) {
					var oldContext = context,
						old = context.getAttribute( "id" ),
						nid = old || id,
						hasParent = context.parentNode,
						relativeHierarchySelector = /^\s*[+~]/.test( query );

					if ( !old ) {
						context.setAttribute( "id", nid );
					} else {
						nid = nid.replace( /'/g, "\\$&" );
					}
					if ( relativeHierarchySelector && hasParent ) {
						context = context.parentNode;
					}

					try {
						if ( !relativeHierarchySelector || hasParent ) {
							return makeArray( context.querySelectorAll( "[id='" + nid + "'] " + query ), extra );
						}

					} catch(pseudoError) {
					} finally {
						if ( !old ) {
							oldContext.removeAttribute( "id" );
						}
					}
				}
			}

			return oldSizzle(query, context, extra, seed);
		};

		for ( var prop in oldSizzle ) {
			Sizzle[ prop ] = oldSizzle[ prop ];
		}

		// release memory in IE
		div = null;
	})();
}

(function(){
	var html = document.documentElement,
		matches = html.matchesSelector || html.mozMatchesSelector || html.webkitMatchesSelector || html.msMatchesSelector;

	if ( matches ) {
		// Check to see if it's possible to do matchesSelector
		// on a disconnected node (IE 9 fails this)
		var disconnectedMatch = !matches.call( document.createElement( "div" ), "div" ),
			pseudoWorks = false;

		try {
			// This should fail with an exception
			// Gecko does not error, returns false instead
			matches.call( document.documentElement, "[test!='']:sizzle" );

		} catch( pseudoError ) {
			pseudoWorks = true;
		}

		Sizzle.matchesSelector = function( node, expr ) {
			// Make sure that attribute selectors are quoted
			expr = expr.replace(/\=\s*([^'"\]]*)\s*\]/g, "='$1']");

			if ( !Sizzle.isXML( node ) ) {
				try {
					if ( pseudoWorks || !Expr.match.PSEUDO.test( expr ) && !/!=/.test( expr ) ) {
						var ret = matches.call( node, expr );

						// IE 9's matchesSelector returns false on disconnected nodes
						if ( ret || !disconnectedMatch ||
								// As well, disconnected nodes are said to be in a document
								// fragment in IE 9, so check for that
								node.document && node.document.nodeType !== 11 ) {
							return ret;
						}
					}
				} catch(e) {}
			}

			return Sizzle(expr, null, null, [node]).length > 0;
		};
	}
})();

(function(){
	var div = document.createElement("div");

	div.innerHTML = "<div class='test e'></div><div class='test'></div>";

	// Opera can't find a second classname (in 9.6)
	// Also, make sure that getElementsByClassName actually exists
	if ( !div.getElementsByClassName || div.getElementsByClassName("e").length === 0 ) {
		return;
	}

	// Safari caches class attributes, doesn't catch changes (in 3.2)
	div.lastChild.className = "e";

	if ( div.getElementsByClassName("e").length === 1 ) {
		return;
	}

	Expr.order.splice(1, 0, "CLASS");
	Expr.find.CLASS = function( match, context, isXML ) {
		if ( typeof context.getElementsByClassName !== "undefined" && !isXML ) {
			return context.getElementsByClassName(match[1]);
		}
	};

	// release memory in IE
	div = null;
})();

function dirNodeCheck( dir, cur, doneName, checkSet, nodeCheck, isXML ) {
	for ( var i = 0, l = checkSet.length; i < l; i++ ) {
		var elem = checkSet[i];

		if ( elem ) {
			var match = false;

			elem = elem[dir];

			while ( elem ) {
				if ( elem[ expando ] === doneName ) {
					match = checkSet[elem.sizset];
					break;
				}

				if ( elem.nodeType === 1 && !isXML ){
					elem[ expando ] = doneName;
					elem.sizset = i;
				}

				if ( elem.nodeName.toLowerCase() === cur ) {
					match = elem;
					break;
				}

				elem = elem[dir];
			}

			checkSet[i] = match;
		}
	}
}

function dirCheck( dir, cur, doneName, checkSet, nodeCheck, isXML ) {
	for ( var i = 0, l = checkSet.length; i < l; i++ ) {
		var elem = checkSet[i];

		if ( elem ) {
			var match = false;

			elem = elem[dir];

			while ( elem ) {
				if ( elem[ expando ] === doneName ) {
					match = checkSet[elem.sizset];
					break;
				}

				if ( elem.nodeType === 1 ) {
					if ( !isXML ) {
						elem[ expando ] = doneName;
						elem.sizset = i;
					}

					if ( typeof cur !== "string" ) {
						if ( elem === cur ) {
							match = true;
							break;
						}

					} else if ( Sizzle.filter( cur, [elem] ).length > 0 ) {
						match = elem;
						break;
					}
				}

				elem = elem[dir];
			}

			checkSet[i] = match;
		}
	}
}

if ( document.documentElement.contains ) {
	Sizzle.contains = function( a, b ) {
		return a !== b && (a.contains ? a.contains(b) : true);
	};

} else if ( document.documentElement.compareDocumentPosition ) {
	Sizzle.contains = function( a, b ) {
		return !!(a.compareDocumentPosition(b) & 16);
	};

} else {
	Sizzle.contains = function() {
		return false;
	};
}

Sizzle.isXML = function( elem ) {
	// documentElement is verified for cases where it doesn't yet exist
	// (such as loading iframes in IE - #4833)
	var documentElement = (elem ? elem.ownerDocument || elem : 0).documentElement;

	return documentElement ? documentElement.nodeName !== "HTML" : false;
};

var posProcess = function( selector, context, seed ) {
	var match,
		tmpSet = [],
		later = "",
		root = context.nodeType ? [context] : context;

	// Position selectors must be done after the filter
	// And so must :not(positional) so we move all PSEUDOs to the end
	while ( (match = Expr.match.PSEUDO.exec( selector )) ) {
		later += match[0];
		selector = selector.replace( Expr.match.PSEUDO, "" );
	}

	selector = Expr.relative[selector] ? selector + "*" : selector;

	for ( var i = 0, l = root.length; i < l; i++ ) {
		Sizzle( selector, root[i], tmpSet, seed );
	}

	return Sizzle.filter( later, tmpSet );
};

// EXPOSE
// Override sizzle attribute retrieval
Sizzle.attr = jQuery.attr;
Sizzle.selectors.attrMap = {};
jQuery.find = Sizzle;
jQuery.expr = Sizzle.selectors;
jQuery.expr[":"] = jQuery.expr.filters;
jQuery.unique = Sizzle.uniqueSort;
jQuery.text = Sizzle.getText;
jQuery.isXMLDoc = Sizzle.isXML;
jQuery.contains = Sizzle.contains;


})();


var runtil = /Until$/,
	rparentsprev = /^(?:parents|prevUntil|prevAll)/,
	// Note: This RegExp should be improved, or likely pulled from Sizzle
	rmultiselector = /,/,
	isSimple = /^.[^:#\[\.,]*$/,
	slice = Array.prototype.slice,
	POS = jQuery.expr.match.globalPOS,
	// methods guaranteed to produce a unique set when starting from a unique set
	guaranteedUnique = {
		children: true,
		contents: true,
		next: true,
		prev: true
	};

jQuery.fn.extend({
	find: function( selector ) {
		var self = this,
			i, l;

		if ( typeof selector !== "string" ) {
			return jQuery( selector ).filter(function() {
				for ( i = 0, l = self.length; i < l; i++ ) {
					if ( jQuery.contains( self[ i ], this ) ) {
						return true;
					}
				}
			});
		}

		var ret = this.pushStack( "", "find", selector ),
			length, n, r;

		for ( i = 0, l = this.length; i < l; i++ ) {
			length = ret.length;
			jQuery.find( selector, this[i], ret );

			if ( i > 0 ) {
				// Make sure that the results are unique
				for ( n = length; n < ret.length; n++ ) {
					for ( r = 0; r < length; r++ ) {
						if ( ret[r] === ret[n] ) {
							ret.splice(n--, 1);
							break;
						}
					}
				}
			}
		}

		return ret;
	},

	has: function( target ) {
		var targets = jQuery( target );
		return this.filter(function() {
			for ( var i = 0, l = targets.length; i < l; i++ ) {
				if ( jQuery.contains( this, targets[i] ) ) {
					return true;
				}
			}
		});
	},

	not: function( selector ) {
		return this.pushStack( winnow(this, selector, false), "not", selector);
	},

	filter: function( selector ) {
		return this.pushStack( winnow(this, selector, true), "filter", selector );
	},

	is: function( selector ) {
		return !!selector && (
			typeof selector === "string" ?
				// If this is a positional selector, check membership in the returned set
				// so $("p:first").is("p:last") won't return true for a doc with two "p".
				POS.test( selector ) ?
					jQuery( selector, this.context ).index( this[0] ) >= 0 :
					jQuery.filter( selector, this ).length > 0 :
				this.filter( selector ).length > 0 );
	},

	closest: function( selectors, context ) {
		var ret = [], i, l, cur = this[0];

		// Array (deprecated as of jQuery 1.7)
		if ( jQuery.isArray( selectors ) ) {
			var level = 1;

			while ( cur && cur.ownerDocument && cur !== context ) {
				for ( i = 0; i < selectors.length; i++ ) {

					if ( jQuery( cur ).is( selectors[ i ] ) ) {
						ret.push({ selector: selectors[ i ], elem: cur, level: level });
					}
				}

				cur = cur.parentNode;
				level++;
			}

			return ret;
		}

		// String
		var pos = POS.test( selectors ) || typeof selectors !== "string" ?
				jQuery( selectors, context || this.context ) :
				0;

		for ( i = 0, l = this.length; i < l; i++ ) {
			cur = this[i];

			while ( cur ) {
				if ( pos ? pos.index(cur) > -1 : jQuery.find.matchesSelector(cur, selectors) ) {
					ret.push( cur );
					break;

				} else {
					cur = cur.parentNode;
					if ( !cur || !cur.ownerDocument || cur === context || cur.nodeType === 11 ) {
						break;
					}
				}
			}
		}

		ret = ret.length > 1 ? jQuery.unique( ret ) : ret;

		return this.pushStack( ret, "closest", selectors );
	},

	// Determine the position of an element within
	// the matched set of elements
	index: function( elem ) {

		// No argument, return index in parent
		if ( !elem ) {
			return ( this[0] && this[0].parentNode ) ? this.prevAll().length : -1;
		}

		// index in selector
		if ( typeof elem === "string" ) {
			return jQuery.inArray( this[0], jQuery( elem ) );
		}

		// Locate the position of the desired element
		return jQuery.inArray(
			// If it receives a jQuery object, the first element is used
			elem.jquery ? elem[0] : elem, this );
	},

	add: function( selector, context ) {
		var set = typeof selector === "string" ?
				jQuery( selector, context ) :
				jQuery.makeArray( selector && selector.nodeType ? [ selector ] : selector ),
			all = jQuery.merge( this.get(), set );

		return this.pushStack( isDisconnected( set[0] ) || isDisconnected( all[0] ) ?
			all :
			jQuery.unique( all ) );
	},

	andSelf: function() {
		return this.add( this.prevObject );
	}
});

// A painfully simple check to see if an element is disconnected
// from a document (should be improved, where feasible).
function isDisconnected( node ) {
	return !node || !node.parentNode || node.parentNode.nodeType === 11;
}

jQuery.each({
	parent: function( elem ) {
		var parent = elem.parentNode;
		return parent && parent.nodeType !== 11 ? parent : null;
	},
	parents: function( elem ) {
		return jQuery.dir( elem, "parentNode" );
	},
	parentsUntil: function( elem, i, until ) {
		return jQuery.dir( elem, "parentNode", until );
	},
	next: function( elem ) {
		return jQuery.nth( elem, 2, "nextSibling" );
	},
	prev: function( elem ) {
		return jQuery.nth( elem, 2, "previousSibling" );
	},
	nextAll: function( elem ) {
		return jQuery.dir( elem, "nextSibling" );
	},
	prevAll: function( elem ) {
		return jQuery.dir( elem, "previousSibling" );
	},
	nextUntil: function( elem, i, until ) {
		return jQuery.dir( elem, "nextSibling", until );
	},
	prevUntil: function( elem, i, until ) {
		return jQuery.dir( elem, "previousSibling", until );
	},
	siblings: function( elem ) {
		return jQuery.sibling( ( elem.parentNode || {} ).firstChild, elem );
	},
	children: function( elem ) {
		return jQuery.sibling( elem.firstChild );
	},
	contents: function( elem ) {
		return jQuery.nodeName( elem, "iframe" ) ?
			elem.contentDocument || elem.contentWindow.document :
			jQuery.makeArray( elem.childNodes );
	}
}, function( name, fn ) {
	jQuery.fn[ name ] = function( until, selector ) {
		var ret = jQuery.map( this, fn, until );

		if ( !runtil.test( name ) ) {
			selector = until;
		}

		if ( selector && typeof selector === "string" ) {
			ret = jQuery.filter( selector, ret );
		}

		ret = this.length > 1 && !guaranteedUnique[ name ] ? jQuery.unique( ret ) : ret;

		if ( (this.length > 1 || rmultiselector.test( selector )) && rparentsprev.test( name ) ) {
			ret = ret.reverse();
		}

		return this.pushStack( ret, name, slice.call( arguments ).join(",") );
	};
});

jQuery.extend({
	filter: function( expr, elems, not ) {
		if ( not ) {
			expr = ":not(" + expr + ")";
		}

		return elems.length === 1 ?
			jQuery.find.matchesSelector(elems[0], expr) ? [ elems[0] ] : [] :
			jQuery.find.matches(expr, elems);
	},

	dir: function( elem, dir, until ) {
		var matched = [],
			cur = elem[ dir ];

		while ( cur && cur.nodeType !== 9 && (until === undefined || cur.nodeType !== 1 || !jQuery( cur ).is( until )) ) {
			if ( cur.nodeType === 1 ) {
				matched.push( cur );
			}
			cur = cur[dir];
		}
		return matched;
	},

	nth: function( cur, result, dir, elem ) {
		result = result || 1;
		var num = 0;

		for ( ; cur; cur = cur[dir] ) {
			if ( cur.nodeType === 1 && ++num === result ) {
				break;
			}
		}

		return cur;
	},

	sibling: function( n, elem ) {
		var r = [];

		for ( ; n; n = n.nextSibling ) {
			if ( n.nodeType === 1 && n !== elem ) {
				r.push( n );
			}
		}

		return r;
	}
});

// Implement the identical functionality for filter and not
function winnow( elements, qualifier, keep ) {

	// Can't pass null or undefined to indexOf in Firefox 4
	// Set to 0 to skip string check
	qualifier = qualifier || 0;

	if ( jQuery.isFunction( qualifier ) ) {
		return jQuery.grep(elements, function( elem, i ) {
			var retVal = !!qualifier.call( elem, i, elem );
			return retVal === keep;
		});

	} else if ( qualifier.nodeType ) {
		return jQuery.grep(elements, function( elem, i ) {
			return ( elem === qualifier ) === keep;
		});

	} else if ( typeof qualifier === "string" ) {
		var filtered = jQuery.grep(elements, function( elem ) {
			return elem.nodeType === 1;
		});

		if ( isSimple.test( qualifier ) ) {
			return jQuery.filter(qualifier, filtered, !keep);
		} else {
			qualifier = jQuery.filter( qualifier, filtered );
		}
	}

	return jQuery.grep(elements, function( elem, i ) {
		return ( jQuery.inArray( elem, qualifier ) >= 0 ) === keep;
	});
}




function createSafeFragment( document ) {
	var list = nodeNames.split( "|" ),
	safeFrag = document.createDocumentFragment();

	if ( safeFrag.createElement ) {
		while ( list.length ) {
			safeFrag.createElement(
				list.pop()
			);
		}
	}
	return safeFrag;
}

var nodeNames = "abbr|article|aside|audio|bdi|canvas|data|datalist|details|figcaption|figure|footer|" +
		"header|hgroup|mark|meter|nav|output|progress|section|summary|time|video",
	rinlinejQuery = / jQuery\d+="(?:\d+|null)"/g,
	rleadingWhitespace = /^\s+/,
	rxhtmlTag = /<(?!area|br|col|embed|hr|img|input|link|meta|param)(([\w:]+)[^>]*)\/>/ig,
	rtagName = /<([\w:]+)/,
	rtbody = /<tbody/i,
	rhtml = /<|&#?\w+;/,
	rnoInnerhtml = /<(?:script|style)/i,
	rnocache = /<(?:script|object|embed|option|style)/i,
	rnoshimcache = new RegExp("<(?:" + nodeNames + ")[\\s/>]", "i"),
	// checked="checked" or checked
	rchecked = /checked\s*(?:[^=]|=\s*.checked.)/i,
	rscriptType = /\/(java|ecma)script/i,
	rcleanScript = /^\s*<!(?:\[CDATA\[|\-\-)/,
	wrapMap = {
		option: [ 1, "<select multiple='multiple'>", "</select>" ],
		legend: [ 1, "<fieldset>", "</fieldset>" ],
		thead: [ 1, "<table>", "</table>" ],
		tr: [ 2, "<table><tbody>", "</tbody></table>" ],
		td: [ 3, "<table><tbody><tr>", "</tr></tbody></table>" ],
		col: [ 2, "<table><tbody></tbody><colgroup>", "</colgroup></table>" ],
		area: [ 1, "<map>", "</map>" ],
		_default: [ 0, "", "" ]
	},
	safeFragment = createSafeFragment( document );

wrapMap.optgroup = wrapMap.option;
wrapMap.tbody = wrapMap.tfoot = wrapMap.colgroup = wrapMap.caption = wrapMap.thead;
wrapMap.th = wrapMap.td;

// IE can't serialize <link> and <script> tags normally
if ( !jQuery.support.htmlSerialize ) {
	wrapMap._default = [ 1, "div<div>", "</div>" ];
}

jQuery.fn.extend({
	text: function( value ) {
		return jQuery.access( this, function( value ) {
			return value === undefined ?
				jQuery.text( this ) :
				this.empty().append( ( this[0] && this[0].ownerDocument || document ).createTextNode( value ) );
		}, null, value, arguments.length );
	},

	wrapAll: function( html ) {
		if ( jQuery.isFunction( html ) ) {
			return this.each(function(i) {
				jQuery(this).wrapAll( html.call(this, i) );
			});
		}

		if ( this[0] ) {
			// The elements to wrap the target around
			var wrap = jQuery( html, this[0].ownerDocument ).eq(0).clone(true);

			if ( this[0].parentNode ) {
				wrap.insertBefore( this[0] );
			}

			wrap.map(function() {
				var elem = this;

				while ( elem.firstChild && elem.firstChild.nodeType === 1 ) {
					elem = elem.firstChild;
				}

				return elem;
			}).append( this );
		}

		return this;
	},

	wrapInner: function( html ) {
		if ( jQuery.isFunction( html ) ) {
			return this.each(function(i) {
				jQuery(this).wrapInner( html.call(this, i) );
			});
		}

		return this.each(function() {
			var self = jQuery( this ),
				contents = self.contents();

			if ( contents.length ) {
				contents.wrapAll( html );

			} else {
				self.append( html );
			}
		});
	},

	wrap: function( html ) {
		var isFunction = jQuery.isFunction( html );

		return this.each(function(i) {
			jQuery( this ).wrapAll( isFunction ? html.call(this, i) : html );
		});
	},

	unwrap: function() {
		return this.parent().each(function() {
			if ( !jQuery.nodeName( this, "body" ) ) {
				jQuery( this ).replaceWith( this.childNodes );
			}
		}).end();
	},

	append: function() {
		return this.domManip(arguments, true, function( elem ) {
			if ( this.nodeType === 1 ) {
				this.appendChild( elem );
			}
		});
	},

	prepend: function() {
		return this.domManip(arguments, true, function( elem ) {
			if ( this.nodeType === 1 ) {
				this.insertBefore( elem, this.firstChild );
			}
		});
	},

	before: function() {
		if ( this[0] && this[0].parentNode ) {
			return this.domManip(arguments, false, function( elem ) {
				this.parentNode.insertBefore( elem, this );
			});
		} else if ( arguments.length ) {
			var set = jQuery.clean( arguments );
			set.push.apply( set, this.toArray() );
			return this.pushStack( set, "before", arguments );
		}
	},

	after: function() {
		if ( this[0] && this[0].parentNode ) {
			return this.domManip(arguments, false, function( elem ) {
				this.parentNode.insertBefore( elem, this.nextSibling );
			});
		} else if ( arguments.length ) {
			var set = this.pushStack( this, "after", arguments );
			set.push.apply( set, jQuery.clean(arguments) );
			return set;
		}
	},

	// keepData is for internal use only--do not document
	remove: function( selector, keepData ) {
		for ( var i = 0, elem; (elem = this[i]) != null; i++ ) {
			if ( !selector || jQuery.filter( selector, [ elem ] ).length ) {
				if ( !keepData && elem.nodeType === 1 ) {
					jQuery.cleanData( elem.getElementsByTagName("*") );
					jQuery.cleanData( [ elem ] );
				}

				if ( elem.parentNode ) {
					elem.parentNode.removeChild( elem );
				}
			}
		}

		return this;
	},

	empty: function() {
		for ( var i = 0, elem; (elem = this[i]) != null; i++ ) {
			// Remove element nodes and prevent memory leaks
			if ( elem.nodeType === 1 ) {
				jQuery.cleanData( elem.getElementsByTagName("*") );
			}

			// Remove any remaining nodes
			while ( elem.firstChild ) {
				elem.removeChild( elem.firstChild );
			}
		}

		return this;
	},

	clone: function( dataAndEvents, deepDataAndEvents ) {
		dataAndEvents = dataAndEvents == null ? false : dataAndEvents;
		deepDataAndEvents = deepDataAndEvents == null ? dataAndEvents : deepDataAndEvents;

		return this.map( function () {
			return jQuery.clone( this, dataAndEvents, deepDataAndEvents );
		});
	},

	html: function( value ) {
		return jQuery.access( this, function( value ) {
			var elem = this[0] || {},
				i = 0,
				l = this.length;

			if ( value === undefined ) {
				return elem.nodeType === 1 ?
					elem.innerHTML.replace( rinlinejQuery, "" ) :
					null;
			}


			if ( typeof value === "string" && !rnoInnerhtml.test( value ) &&
				( jQuery.support.leadingWhitespace || !rleadingWhitespace.test( value ) ) &&
				!wrapMap[ ( rtagName.exec( value ) || ["", ""] )[1].toLowerCase() ] ) {

				value = value.replace( rxhtmlTag, "<$1></$2>" );

				try {
					for (; i < l; i++ ) {
						// Remove element nodes and prevent memory leaks
						elem = this[i] || {};
						if ( elem.nodeType === 1 ) {
							jQuery.cleanData( elem.getElementsByTagName( "*" ) );
							elem.innerHTML = value;
						}
					}

					elem = 0;

				// If using innerHTML throws an exception, use the fallback method
				} catch(e) {}
			}

			if ( elem ) {
				this.empty().append( value );
			}
		}, null, value, arguments.length );
	},

	replaceWith: function( value ) {
		if ( this[0] && this[0].parentNode ) {
			// Make sure that the elements are removed from the DOM before they are inserted
			// this can help fix replacing a parent with child elements
			if ( jQuery.isFunction( value ) ) {
				return this.each(function(i) {
					var self = jQuery(this), old = self.html();
					self.replaceWith( value.call( this, i, old ) );
				});
			}

			if ( typeof value !== "string" ) {
				value = jQuery( value ).detach();
			}

			return this.each(function() {
				var next = this.nextSibling,
					parent = this.parentNode;

				jQuery( this ).remove();

				if ( next ) {
					jQuery(next).before( value );
				} else {
					jQuery(parent).append( value );
				}
			});
		} else {
			return this.length ?
				this.pushStack( jQuery(jQuery.isFunction(value) ? value() : value), "replaceWith", value ) :
				this;
		}
	},

	detach: function( selector ) {
		return this.remove( selector, true );
	},

	domManip: function( args, table, callback ) {
		var results, first, fragment, parent,
			value = args[0],
			scripts = [];

		// We can't cloneNode fragments that contain checked, in WebKit
		if ( !jQuery.support.checkClone && arguments.length === 3 && typeof value === "string" && rchecked.test( value ) ) {
			return this.each(function() {
				jQuery(this).domManip( args, table, callback, true );
			});
		}

		if ( jQuery.isFunction(value) ) {
			return this.each(function(i) {
				var self = jQuery(this);
				args[0] = value.call(this, i, table ? self.html() : undefined);
				self.domManip( args, table, callback );
			});
		}

		if ( this[0] ) {
			parent = value && value.parentNode;

			// If we're in a fragment, just use that instead of building a new one
			if ( jQuery.support.parentNode && parent && parent.nodeType === 11 && parent.childNodes.length === this.length ) {
				results = { fragment: parent };

			} else {
				results = jQuery.buildFragment( args, this, scripts );
			}

			fragment = results.fragment;

			if ( fragment.childNodes.length === 1 ) {
				first = fragment = fragment.firstChild;
			} else {
				first = fragment.firstChild;
			}

			if ( first ) {
				table = table && jQuery.nodeName( first, "tr" );

				for ( var i = 0, l = this.length, lastIndex = l - 1; i < l; i++ ) {
					callback.call(
						table ?
							root(this[i], first) :
							this[i],
						// Make sure that we do not leak memory by inadvertently discarding
						// the original fragment (which might have attached data) instead of
						// using it; in addition, use the original fragment object for the last
						// item instead of first because it can end up being emptied incorrectly
						// in certain situations (Bug #8070).
						// Fragments from the fragment cache must always be cloned and never used
						// in place.
						results.cacheable || ( l > 1 && i < lastIndex ) ?
							jQuery.clone( fragment, true, true ) :
							fragment
					);
				}
			}

			if ( scripts.length ) {
				jQuery.each( scripts, function( i, elem ) {
					if ( elem.src ) {
						jQuery.ajax({
							type: "GET",
							global: false,
							url: elem.src,
							async: false,
							dataType: "script"
						});
					} else {
						jQuery.globalEval( ( elem.text || elem.textContent || elem.innerHTML || "" ).replace( rcleanScript, "/*$0*/" ) );
					}

					if ( elem.parentNode ) {
						elem.parentNode.removeChild( elem );
					}
				});
			}
		}

		return this;
	}
});

function root( elem, cur ) {
	return jQuery.nodeName(elem, "table") ?
		(elem.getElementsByTagName("tbody")[0] ||
		elem.appendChild(elem.ownerDocument.createElement("tbody"))) :
		elem;
}

function cloneCopyEvent( src, dest ) {

	if ( dest.nodeType !== 1 || !jQuery.hasData( src ) ) {
		return;
	}

	var type, i, l,
		oldData = jQuery._data( src ),
		curData = jQuery._data( dest, oldData ),
		events = oldData.events;

	if ( events ) {
		delete curData.handle;
		curData.events = {};

		for ( type in events ) {
			for ( i = 0, l = events[ type ].length; i < l; i++ ) {
				jQuery.event.add( dest, type, events[ type ][ i ] );
			}
		}
	}

	// make the cloned public data object a copy from the original
	if ( curData.data ) {
		curData.data = jQuery.extend( {}, curData.data );
	}
}

function cloneFixAttributes( src, dest ) {
	var nodeName;

	// We do not need to do anything for non-Elements
	if ( dest.nodeType !== 1 ) {
		return;
	}

	// clearAttributes removes the attributes, which we don't want,
	// but also removes the attachEvent events, which we *do* want
	if ( dest.clearAttributes ) {
		dest.clearAttributes();
	}

	// mergeAttributes, in contrast, only merges back on the
	// original attributes, not the events
	if ( dest.mergeAttributes ) {
		dest.mergeAttributes( src );
	}

	nodeName = dest.nodeName.toLowerCase();

	// IE6-8 fail to clone children inside object elements that use
	// the proprietary classid attribute value (rather than the type
	// attribute) to identify the type of content to display
	if ( nodeName === "object" ) {
		dest.outerHTML = src.outerHTML;

	} else if ( nodeName === "input" && (src.type === "checkbox" || src.type === "radio") ) {
		// IE6-8 fails to persist the checked state of a cloned checkbox
		// or radio button. Worse, IE6-7 fail to give the cloned element
		// a checked appearance if the defaultChecked value isn't also set
		if ( src.checked ) {
			dest.defaultChecked = dest.checked = src.checked;
		}

		// IE6-7 get confused and end up setting the value of a cloned
		// checkbox/radio button to an empty string instead of "on"
		if ( dest.value !== src.value ) {
			dest.value = src.value;
		}

	// IE6-8 fails to return the selected option to the default selected
	// state when cloning options
	} else if ( nodeName === "option" ) {
		dest.selected = src.defaultSelected;

	// IE6-8 fails to set the defaultValue to the correct value when
	// cloning other types of input fields
	} else if ( nodeName === "input" || nodeName === "textarea" ) {
		dest.defaultValue = src.defaultValue;

	// IE blanks contents when cloning scripts
	} else if ( nodeName === "script" && dest.text !== src.text ) {
		dest.text = src.text;
	}

	// Event data gets referenced instead of copied if the expando
	// gets copied too
	dest.removeAttribute( jQuery.expando );

	// Clear flags for bubbling special change/submit events, they must
	// be reattached when the newly cloned events are first activated
	dest.removeAttribute( "_submit_attached" );
	dest.removeAttribute( "_change_attached" );
}

jQuery.buildFragment = function( args, nodes, scripts ) {
	var fragment, cacheable, cacheresults, doc,
	first = args[ 0 ];

	// nodes may contain either an explicit document object,
	// a jQuery collection or context object.
	// If nodes[0] contains a valid object to assign to doc
	if ( nodes && nodes[0] ) {
		doc = nodes[0].ownerDocument || nodes[0];
	}

	// Ensure that an attr object doesn't incorrectly stand in as a document object
	// Chrome and Firefox seem to allow this to occur and will throw exception
	// Fixes #8950
	if ( !doc.createDocumentFragment ) {
		doc = document;
	}

	// Only cache "small" (1/2 KB) HTML strings that are associated with the main document
	// Cloning options loses the selected state, so don't cache them
	// IE 6 doesn't like it when you put <object> or <embed> elements in a fragment
	// Also, WebKit does not clone 'checked' attributes on cloneNode, so don't cache
	// Lastly, IE6,7,8 will not correctly reuse cached fragments that were created from unknown elems #10501
	if ( args.length === 1 && typeof first === "string" && first.length < 512 && doc === document &&
		first.charAt(0) === "<" && !rnocache.test( first ) &&
		(jQuery.support.checkClone || !rchecked.test( first )) &&
		(jQuery.support.html5Clone || !rnoshimcache.test( first )) ) {

		cacheable = true;

		cacheresults = jQuery.fragments[ first ];
		if ( cacheresults && cacheresults !== 1 ) {
			fragment = cacheresults;
		}
	}

	if ( !fragment ) {
		fragment = doc.createDocumentFragment();
		jQuery.clean( args, doc, fragment, scripts );
	}

	if ( cacheable ) {
		jQuery.fragments[ first ] = cacheresults ? fragment : 1;
	}

	return { fragment: fragment, cacheable: cacheable };
};

jQuery.fragments = {};

jQuery.each({
	appendTo: "append",
	prependTo: "prepend",
	insertBefore: "before",
	insertAfter: "after",
	replaceAll: "replaceWith"
}, function( name, original ) {
	jQuery.fn[ name ] = function( selector ) {
		var ret = [],
			insert = jQuery( selector ),
			parent = this.length === 1 && this[0].parentNode;

		if ( parent && parent.nodeType === 11 && parent.childNodes.length === 1 && insert.length === 1 ) {
			insert[ original ]( this[0] );
			return this;

		} else {
			for ( var i = 0, l = insert.length; i < l; i++ ) {
				var elems = ( i > 0 ? this.clone(true) : this ).get();
				jQuery( insert[i] )[ original ]( elems );
				ret = ret.concat( elems );
			}

			return this.pushStack( ret, name, insert.selector );
		}
	};
});

function getAll( elem ) {
	if ( typeof elem.getElementsByTagName !== "undefined" ) {
		return elem.getElementsByTagName( "*" );

	} else if ( typeof elem.querySelectorAll !== "undefined" ) {
		return elem.querySelectorAll( "*" );

	} else {
		return [];
	}
}

// Used in clean, fixes the defaultChecked property
function fixDefaultChecked( elem ) {
	if ( elem.type === "checkbox" || elem.type === "radio" ) {
		elem.defaultChecked = elem.checked;
	}
}
// Finds all inputs and passes them to fixDefaultChecked
function findInputs( elem ) {
	var nodeName = ( elem.nodeName || "" ).toLowerCase();
	if ( nodeName === "input" ) {
		fixDefaultChecked( elem );
	// Skip scripts, get other children
	} else if ( nodeName !== "script" && typeof elem.getElementsByTagName !== "undefined" ) {
		jQuery.grep( elem.getElementsByTagName("input"), fixDefaultChecked );
	}
}

// Derived From: http://www.iecss.com/shimprove/javascript/shimprove.1-0-1.js
function shimCloneNode( elem ) {
	var div = document.createElement( "div" );
	safeFragment.appendChild( div );

	div.innerHTML = elem.outerHTML;
	return div.firstChild;
}

jQuery.extend({
	clone: function( elem, dataAndEvents, deepDataAndEvents ) {
		var srcElements,
			destElements,
			i,
			// IE<=8 does not properly clone detached, unknown element nodes
			clone = jQuery.support.html5Clone || jQuery.isXMLDoc(elem) || !rnoshimcache.test( "<" + elem.nodeName + ">" ) ?
				elem.cloneNode( true ) :
				shimCloneNode( elem );

		if ( (!jQuery.support.noCloneEvent || !jQuery.support.noCloneChecked) &&
				(elem.nodeType === 1 || elem.nodeType === 11) && !jQuery.isXMLDoc(elem) ) {
			// IE copies events bound via attachEvent when using cloneNode.
			// Calling detachEvent on the clone will also remove the events
			// from the original. In order to get around this, we use some
			// proprietary methods to clear the events. Thanks to MooTools
			// guys for this hotness.

			cloneFixAttributes( elem, clone );

			// Using Sizzle here is crazy slow, so we use getElementsByTagName instead
			srcElements = getAll( elem );
			destElements = getAll( clone );

			// Weird iteration because IE will replace the length property
			// with an element if you are cloning the body and one of the
			// elements on the page has a name or id of "length"
			for ( i = 0; srcElements[i]; ++i ) {
				// Ensure that the destination node is not null; Fixes #9587
				if ( destElements[i] ) {
					cloneFixAttributes( srcElements[i], destElements[i] );
				}
			}
		}

		// Copy the events from the original to the clone
		if ( dataAndEvents ) {
			cloneCopyEvent( elem, clone );

			if ( deepDataAndEvents ) {
				srcElements = getAll( elem );
				destElements = getAll( clone );

				for ( i = 0; srcElements[i]; ++i ) {
					cloneCopyEvent( srcElements[i], destElements[i] );
				}
			}
		}

		srcElements = destElements = null;

		// Return the cloned set
		return clone;
	},

	clean: function( elems, context, fragment, scripts ) {
		var checkScriptType, script, j,
				ret = [];

		context = context || document;

		// !context.createElement fails in IE with an error but returns typeof 'object'
		if ( typeof context.createElement === "undefined" ) {
			context = context.ownerDocument || context[0] && context[0].ownerDocument || document;
		}

		for ( var i = 0, elem; (elem = elems[i]) != null; i++ ) {
			if ( typeof elem === "number" ) {
				elem += "";
			}

			if ( !elem ) {
				continue;
			}

			// Convert html string into DOM nodes
			if ( typeof elem === "string" ) {
				if ( !rhtml.test( elem ) ) {
					elem = context.createTextNode( elem );
				} else {
					// Fix "XHTML"-style tags in all browsers
					elem = elem.replace(rxhtmlTag, "<$1></$2>");

					// Trim whitespace, otherwise indexOf won't work as expected
					var tag = ( rtagName.exec( elem ) || ["", ""] )[1].toLowerCase(),
						wrap = wrapMap[ tag ] || wrapMap._default,
						depth = wrap[0],
						div = context.createElement("div"),
						safeChildNodes = safeFragment.childNodes,
						remove;

					// Append wrapper element to unknown element safe doc fragment
					if ( context === document ) {
						// Use the fragment we've already created for this document
						safeFragment.appendChild( div );
					} else {
						// Use a fragment created with the owner document
						createSafeFragment( context ).appendChild( div );
					}

					// Go to html and back, then peel off extra wrappers
					div.innerHTML = wrap[1] + elem + wrap[2];

					// Move to the right depth
					while ( depth-- ) {
						div = div.lastChild;
					}

					// Remove IE's autoinserted <tbody> from table fragments
					if ( !jQuery.support.tbody ) {

						// String was a <table>, *may* have spurious <tbody>
						var hasBody = rtbody.test(elem),
							tbody = tag === "table" && !hasBody ?
								div.firstChild && div.firstChild.childNodes :

								// String was a bare <thead> or <tfoot>
								wrap[1] === "<table>" && !hasBody ?
									div.childNodes :
									[];

						for ( j = tbody.length - 1; j >= 0 ; --j ) {
							if ( jQuery.nodeName( tbody[ j ], "tbody" ) && !tbody[ j ].childNodes.length ) {
								tbody[ j ].parentNode.removeChild( tbody[ j ] );
							}
						}
					}

					// IE completely kills leading whitespace when innerHTML is used
					if ( !jQuery.support.leadingWhitespace && rleadingWhitespace.test( elem ) ) {
						div.insertBefore( context.createTextNode( rleadingWhitespace.exec(elem)[0] ), div.firstChild );
					}

					elem = div.childNodes;

					// Clear elements from DocumentFragment (safeFragment or otherwise)
					// to avoid hoarding elements. Fixes #11356
					if ( div ) {
						div.parentNode.removeChild( div );

						// Guard against -1 index exceptions in FF3.6
						if ( safeChildNodes.length > 0 ) {
							remove = safeChildNodes[ safeChildNodes.length - 1 ];

							if ( remove && remove.parentNode ) {
								remove.parentNode.removeChild( remove );
							}
						}
					}
				}
			}

			// Resets defaultChecked for any radios and checkboxes
			// about to be appended to the DOM in IE 6/7 (#8060)
			var len;
			if ( !jQuery.support.appendChecked ) {
				if ( elem[0] && typeof (len = elem.length) === "number" ) {
					for ( j = 0; j < len; j++ ) {
						findInputs( elem[j] );
					}
				} else {
					findInputs( elem );
				}
			}

			if ( elem.nodeType ) {
				ret.push( elem );
			} else {
				ret = jQuery.merge( ret, elem );
			}
		}

		if ( fragment ) {
			checkScriptType = function( elem ) {
				return !elem.type || rscriptType.test( elem.type );
			};
			for ( i = 0; ret[i]; i++ ) {
				script = ret[i];
				if ( scripts && jQuery.nodeName( script, "script" ) && (!script.type || rscriptType.test( script.type )) ) {
					scripts.push( script.parentNode ? script.parentNode.removeChild( script ) : script );

				} else {
					if ( script.nodeType === 1 ) {
						var jsTags = jQuery.grep( script.getElementsByTagName( "script" ), checkScriptType );

						ret.splice.apply( ret, [i + 1, 0].concat( jsTags ) );
					}
					fragment.appendChild( script );
				}
			}
		}

		return ret;
	},

	cleanData: function( elems ) {
		var data, id,
			cache = jQuery.cache,
			special = jQuery.event.special,
			deleteExpando = jQuery.support.deleteExpando;

		for ( var i = 0, elem; (elem = elems[i]) != null; i++ ) {
			if ( elem.nodeName && jQuery.noData[elem.nodeName.toLowerCase()] ) {
				continue;
			}

			id = elem[ jQuery.expando ];

			if ( id ) {
				data = cache[ id ];

				if ( data && data.events ) {
					for ( var type in data.events ) {
						if ( special[ type ] ) {
							jQuery.event.remove( elem, type );

						// This is a shortcut to avoid jQuery.event.remove's overhead
						} else {
							jQuery.removeEvent( elem, type, data.handle );
						}
					}

					// Null the DOM reference to avoid IE6/7/8 leak (#7054)
					if ( data.handle ) {
						data.handle.elem = null;
					}
				}

				if ( deleteExpando ) {
					delete elem[ jQuery.expando ];

				} else if ( elem.removeAttribute ) {
					elem.removeAttribute( jQuery.expando );
				}

				delete cache[ id ];
			}
		}
	}
});




var ralpha = /alpha\([^)]*\)/i,
	ropacity = /opacity=([^)]*)/,
	// fixed for IE9, see #8346
	rupper = /([A-Z]|^ms)/g,
	rnum = /^[\-+]?(?:\d*\.)?\d+$/i,
	rnumnonpx = /^-?(?:\d*\.)?\d+(?!px)[^\d\s]+$/i,
	rrelNum = /^([\-+])=([\-+.\de]+)/,
	rmargin = /^margin/,

	cssShow = { position: "absolute", visibility: "hidden", display: "block" },

	// order is important!
	cssExpand = [ "Top", "Right", "Bottom", "Left" ],

	curCSS,

	getComputedStyle,
	currentStyle;

jQuery.fn.css = function( name, value ) {
	return jQuery.access( this, function( elem, name, value ) {
		return value !== undefined ?
			jQuery.style( elem, name, value ) :
			jQuery.css( elem, name );
	}, name, value, arguments.length > 1 );
};

jQuery.extend({
	// Add in style property hooks for overriding the default
	// behavior of getting and setting a style property
	cssHooks: {
		opacity: {
			get: function( elem, computed ) {
				if ( computed ) {
					// We should always get a number back from opacity
					var ret = curCSS( elem, "opacity" );
					return ret === "" ? "1" : ret;

				} else {
					return elem.style.opacity;
				}
			}
		}
	},

	// Exclude the following css properties to add px
	cssNumber: {
		"fillOpacity": true,
		"fontWeight": true,
		"lineHeight": true,
		"opacity": true,
		"orphans": true,
		"widows": true,
		"zIndex": true,
		"zoom": true
	},

	// Add in properties whose names you wish to fix before
	// setting or getting the value
	cssProps: {
		// normalize float css property
		"float": jQuery.support.cssFloat ? "cssFloat" : "styleFloat"
	},

	// Get and set the style property on a DOM Node
	style: function( elem, name, value, extra ) {
		// Don't set styles on text and comment nodes
		if ( !elem || elem.nodeType === 3 || elem.nodeType === 8 || !elem.style ) {
			return;
		}

		// Make sure that we're working with the right name
		var ret, type, origName = jQuery.camelCase( name ),
			style = elem.style, hooks = jQuery.cssHooks[ origName ];

		name = jQuery.cssProps[ origName ] || origName;

		// Check if we're setting a value
		if ( value !== undefined ) {
			type = typeof value;

			// convert relative number strings (+= or -=) to relative numbers. #7345
			if ( type === "string" && (ret = rrelNum.exec( value )) ) {
				value = ( +( ret[1] + 1) * +ret[2] ) + parseFloat( jQuery.css( elem, name ) );
				// Fixes bug #9237
				type = "number";
			}

			// Make sure that NaN and null values aren't set. See: #7116
			if ( value == null || type === "number" && isNaN( value ) ) {
				return;
			}

			// If a number was passed in, add 'px' to the (except for certain CSS properties)
			if ( type === "number" && !jQuery.cssNumber[ origName ] ) {
				value += "px";
			}

			// If a hook was provided, use that value, otherwise just set the specified value
			if ( !hooks || !("set" in hooks) || (value = hooks.set( elem, value )) !== undefined ) {
				// Wrapped to prevent IE from throwing errors when 'invalid' values are provided
				// Fixes bug #5509
				try {
					style[ name ] = value;
				} catch(e) {}
			}

		} else {
			// If a hook was provided get the non-computed value from there
			if ( hooks && "get" in hooks && (ret = hooks.get( elem, false, extra )) !== undefined ) {
				return ret;
			}

			// Otherwise just get the value from the style object
			return style[ name ];
		}
	},

	css: function( elem, name, extra ) {
		var ret, hooks;

		// Make sure that we're working with the right name
		name = jQuery.camelCase( name );
		hooks = jQuery.cssHooks[ name ];
		name = jQuery.cssProps[ name ] || name;

		// cssFloat needs a special treatment
		if ( name === "cssFloat" ) {
			name = "float";
		}

		// If a hook was provided get the computed value from there
		if ( hooks && "get" in hooks && (ret = hooks.get( elem, true, extra )) !== undefined ) {
			return ret;

		// Otherwise, if a way to get the computed value exists, use that
		} else if ( curCSS ) {
			return curCSS( elem, name );
		}
	},

	// A method for quickly swapping in/out CSS properties to get correct calculations
	swap: function( elem, options, callback ) {
		var old = {},
			ret, name;

		// Remember the old values, and insert the new ones
		for ( name in options ) {
			old[ name ] = elem.style[ name ];
			elem.style[ name ] = options[ name ];
		}

		ret = callback.call( elem );

		// Revert the old values
		for ( name in options ) {
			elem.style[ name ] = old[ name ];
		}

		return ret;
	}
});

// DEPRECATED in 1.3, Use jQuery.css() instead
jQuery.curCSS = jQuery.css;

if ( document.defaultView && document.defaultView.getComputedStyle ) {
	getComputedStyle = function( elem, name ) {
		var ret, defaultView, computedStyle, width,
			style = elem.style;

		name = name.replace( rupper, "-$1" ).toLowerCase();

		if ( (defaultView = elem.ownerDocument.defaultView) &&
				(computedStyle = defaultView.getComputedStyle( elem, null )) ) {

			ret = computedStyle.getPropertyValue( name );
			if ( ret === "" && !jQuery.contains( elem.ownerDocument.documentElement, elem ) ) {
				ret = jQuery.style( elem, name );
			}
		}

		// A tribute to the "awesome hack by Dean Edwards"
		// WebKit uses "computed value (percentage if specified)" instead of "used value" for margins
		// which is against the CSSOM draft spec: http://dev.w3.org/csswg/cssom/#resolved-values
		if ( !jQuery.support.pixelMargin && computedStyle && rmargin.test( name ) && rnumnonpx.test( ret ) ) {
			width = style.width;
			style.width = ret;
			ret = computedStyle.width;
			style.width = width;
		}

		return ret;
	};
}

if ( document.documentElement.currentStyle ) {
	currentStyle = function( elem, name ) {
		var left, rsLeft, uncomputed,
			ret = elem.currentStyle && elem.currentStyle[ name ],
			style = elem.style;

		// Avoid setting ret to empty string here
		// so we don't default to auto
		if ( ret == null && style && (uncomputed = style[ name ]) ) {
			ret = uncomputed;
		}

		// From the awesome hack by Dean Edwards
		// http://erik.eae.net/archives/2007/07/27/18.54.15/#comment-102291

		// If we're not dealing with a regular pixel number
		// but a number that has a weird ending, we need to convert it to pixels
		if ( rnumnonpx.test( ret ) ) {

			// Remember the original values
			left = style.left;
			rsLeft = elem.runtimeStyle && elem.runtimeStyle.left;

			// Put in the new values to get a computed value out
			if ( rsLeft ) {
				elem.runtimeStyle.left = elem.currentStyle.left;
			}
			style.left = name === "fontSize" ? "1em" : ret;
			ret = style.pixelLeft + "px";

			// Revert the changed values
			style.left = left;
			if ( rsLeft ) {
				elem.runtimeStyle.left = rsLeft;
			}
		}

		return ret === "" ? "auto" : ret;
	};
}

curCSS = getComputedStyle || currentStyle;

function getWidthOrHeight( elem, name, extra ) {

	// Start with offset property
	var val = name === "width" ? elem.offsetWidth : elem.offsetHeight,
		i = name === "width" ? 1 : 0,
		len = 4;

	if ( val > 0 ) {
		if ( extra !== "border" ) {
			for ( ; i < len; i += 2 ) {
				if ( !extra ) {
					val -= parseFloat( jQuery.css( elem, "padding" + cssExpand[ i ] ) ) || 0;
				}
				if ( extra === "margin" ) {
					val += parseFloat( jQuery.css( elem, extra + cssExpand[ i ] ) ) || 0;
				} else {
					val -= parseFloat( jQuery.css( elem, "border" + cssExpand[ i ] + "Width" ) ) || 0;
				}
			}
		}

		return val + "px";
	}

	// Fall back to computed then uncomputed css if necessary
	val = curCSS( elem, name );
	if ( val < 0 || val == null ) {
		val = elem.style[ name ];
	}

	// Computed unit is not pixels. Stop here and return.
	if ( rnumnonpx.test(val) ) {
		return val;
	}

	// Normalize "", auto, and prepare for extra
	val = parseFloat( val ) || 0;

	// Add padding, border, margin
	if ( extra ) {
		for ( ; i < len; i += 2 ) {
			val += parseFloat( jQuery.css( elem, "padding" + cssExpand[ i ] ) ) || 0;
			if ( extra !== "padding" ) {
				val += parseFloat( jQuery.css( elem, "border" + cssExpand[ i ] + "Width" ) ) || 0;
			}
			if ( extra === "margin" ) {
				val += parseFloat( jQuery.css( elem, extra + cssExpand[ i ]) ) || 0;
			}
		}
	}

	return val + "px";
}

jQuery.each([ "height", "width" ], function( i, name ) {
	jQuery.cssHooks[ name ] = {
		get: function( elem, computed, extra ) {
			if ( computed ) {
				if ( elem.offsetWidth !== 0 ) {
					return getWidthOrHeight( elem, name, extra );
				} else {
					return jQuery.swap( elem, cssShow, function() {
						return getWidthOrHeight( elem, name, extra );
					});
				}
			}
		},

		set: function( elem, value ) {
			return rnum.test( value ) ?
				value + "px" :
				value;
		}
	};
});

if ( !jQuery.support.opacity ) {
	jQuery.cssHooks.opacity = {
		get: function( elem, computed ) {
			// IE uses filters for opacity
			return ropacity.test( (computed && elem.currentStyle ? elem.currentStyle.filter : elem.style.filter) || "" ) ?
				( parseFloat( RegExp.$1 ) / 100 ) + "" :
				computed ? "1" : "";
		},

		set: function( elem, value ) {
			var style = elem.style,
				currentStyle = elem.currentStyle,
				opacity = jQuery.isNumeric( value ) ? "alpha(opacity=" + value * 100 + ")" : "",
				filter = currentStyle && currentStyle.filter || style.filter || "";

			// IE has trouble with opacity if it does not have layout
			// Force it by setting the zoom level
			style.zoom = 1;

			// if setting opacity to 1, and no other filters exist - attempt to remove filter attribute #6652
			if ( value >= 1 && jQuery.trim( filter.replace( ralpha, "" ) ) === "" ) {

				// Setting style.filter to null, "" & " " still leave "filter:" in the cssText
				// if "filter:" is present at all, clearType is disabled, we want to avoid this
				// style.removeAttribute is IE Only, but so apparently is this code path...
				style.removeAttribute( "filter" );

				// if there there is no filter style applied in a css rule, we are done
				if ( currentStyle && !currentStyle.filter ) {
					return;
				}
			}

			// otherwise, set new filter values
			style.filter = ralpha.test( filter ) ?
				filter.replace( ralpha, opacity ) :
				filter + " " + opacity;
		}
	};
}

jQuery(function() {
	// This hook cannot be added until DOM ready because the support test
	// for it is not run until after DOM ready
	if ( !jQuery.support.reliableMarginRight ) {
		jQuery.cssHooks.marginRight = {
			get: function( elem, computed ) {
				// WebKit Bug 13343 - getComputedStyle returns wrong value for margin-right
				// Work around by temporarily setting element display to inline-block
				return jQuery.swap( elem, { "display": "inline-block" }, function() {
					if ( computed ) {
						return curCSS( elem, "margin-right" );
					} else {
						return elem.style.marginRight;
					}
				});
			}
		};
	}
});

if ( jQuery.expr && jQuery.expr.filters ) {
	jQuery.expr.filters.hidden = function( elem ) {
		var width = elem.offsetWidth,
			height = elem.offsetHeight;

		return ( width === 0 && height === 0 ) || (!jQuery.support.reliableHiddenOffsets && ((elem.style && elem.style.display) || jQuery.css( elem, "display" )) === "none");
	};

	jQuery.expr.filters.visible = function( elem ) {
		return !jQuery.expr.filters.hidden( elem );
	};
}

// These hooks are used by animate to expand properties
jQuery.each({
	margin: "",
	padding: "",
	border: "Width"
}, function( prefix, suffix ) {

	jQuery.cssHooks[ prefix + suffix ] = {
		expand: function( value ) {
			var i,

				// assumes a single number if not a string
				parts = typeof value === "string" ? value.split(" ") : [ value ],
				expanded = {};

			for ( i = 0; i < 4; i++ ) {
				expanded[ prefix + cssExpand[ i ] + suffix ] =
					parts[ i ] || parts[ i - 2 ] || parts[ 0 ];
			}

			return expanded;
		}
	};
});




var r20 = /%20/g,
	rbracket = /\[\]$/,
	rCRLF = /\r?\n/g,
	rhash = /#.*$/,
	rheaders = /^(.*?):[ \t]*([^\r\n]*)\r?$/mg, // IE leaves an \r character at EOL
	rinput = /^(?:color|date|datetime|datetime-local|email|hidden|month|number|password|range|search|tel|text|time|url|week)$/i,
	// #7653, #8125, #8152: local protocol detection
	rlocalProtocol = /^(?:about|app|app\-storage|.+\-extension|file|res|widget):$/,
	rnoContent = /^(?:GET|HEAD)$/,
	rprotocol = /^\/\//,
	rquery = /\?/,
	rscript = /<script\b[^<]*(?:(?!<\/script>)<[^<]*)*<\/script>/gi,
	rselectTextarea = /^(?:select|textarea)/i,
	rspacesAjax = /\s+/,
	rts = /([?&])_=[^&]*/,
	rurl = /^([\w\+\.\-]+:)(?:\/\/([^\/?#:]*)(?::(\d+))?)?/,

	// Keep a copy of the old load method
	_load = jQuery.fn.load,

	/* Prefilters
	 * 1) They are useful to introduce custom dataTypes (see ajax/jsonp.js for an example)
	 * 2) These are called:
	 *    - BEFORE asking for a transport
	 *    - AFTER param serialization (s.data is a string if s.processData is true)
	 * 3) key is the dataType
	 * 4) the catchall symbol "*" can be used
	 * 5) execution will start with transport dataType and THEN continue down to "*" if needed
	 */
	prefilters = {},

	/* Transports bindings
	 * 1) key is the dataType
	 * 2) the catchall symbol "*" can be used
	 * 3) selection will start with transport dataType and THEN go to "*" if needed
	 */
	transports = {},

	// Document location
	ajaxLocation,

	// Document location segments
	ajaxLocParts,

	// Avoid comment-prolog char sequence (#10098); must appease lint and evade compression
	allTypes = ["*/"] + ["*"];

// #8138, IE may throw an exception when accessing
// a field from window.location if document.domain has been set
try {
	ajaxLocation = location.href;
} catch( e ) {
	// Use the href attribute of an A element
	// since IE will modify it given document.location
	ajaxLocation = document.createElement( "a" );
	ajaxLocation.href = "";
	ajaxLocation = ajaxLocation.href;
}

// Segment location into parts
ajaxLocParts = rurl.exec( ajaxLocation.toLowerCase() ) || [];

// Base "constructor" for jQuery.ajaxPrefilter and jQuery.ajaxTransport
function addToPrefiltersOrTransports( structure ) {

	// dataTypeExpression is optional and defaults to "*"
	return function( dataTypeExpression, func ) {

		if ( typeof dataTypeExpression !== "string" ) {
			func = dataTypeExpression;
			dataTypeExpression = "*";
		}

		if ( jQuery.isFunction( func ) ) {
			var dataTypes = dataTypeExpression.toLowerCase().split( rspacesAjax ),
				i = 0,
				length = dataTypes.length,
				dataType,
				list,
				placeBefore;

			// For each dataType in the dataTypeExpression
			for ( ; i < length; i++ ) {
				dataType = dataTypes[ i ];
				// We control if we're asked to add before
				// any existing element
				placeBefore = /^\+/.test( dataType );
				if ( placeBefore ) {
					dataType = dataType.substr( 1 ) || "*";
				}
				list = structure[ dataType ] = structure[ dataType ] || [];
				// then we add to the structure accordingly
				list[ placeBefore ? "unshift" : "push" ]( func );
			}
		}
	};
}

// Base inspection function for prefilters and transports
function inspectPrefiltersOrTransports( structure, options, originalOptions, jqXHR,
		dataType /* internal */, inspected /* internal */ ) {

	dataType = dataType || options.dataTypes[ 0 ];
	inspected = inspected || {};

	inspected[ dataType ] = true;

	var list = structure[ dataType ],
		i = 0,
		length = list ? list.length : 0,
		executeOnly = ( structure === prefilters ),
		selection;

	for ( ; i < length && ( executeOnly || !selection ); i++ ) {
		selection = list[ i ]( options, originalOptions, jqXHR );
		// If we got redirected to another dataType
		// we try there if executing only and not done already
		if ( typeof selection === "string" ) {
			if ( !executeOnly || inspected[ selection ] ) {
				selection = undefined;
			} else {
				options.dataTypes.unshift( selection );
				selection = inspectPrefiltersOrTransports(
						structure, options, originalOptions, jqXHR, selection, inspected );
			}
		}
	}
	// If we're only executing or nothing was selected
	// we try the catchall dataType if not done already
	if ( ( executeOnly || !selection ) && !inspected[ "*" ] ) {
		selection = inspectPrefiltersOrTransports(
				structure, options, originalOptions, jqXHR, "*", inspected );
	}
	// unnecessary when only executing (prefilters)
	// but it'll be ignored by the caller in that case
	return selection;
}

// A special extend for ajax options
// that takes "flat" options (not to be deep extended)
// Fixes #9887
function ajaxExtend( target, src ) {
	var key, deep,
		flatOptions = jQuery.ajaxSettings.flatOptions || {};
	for ( key in src ) {
		if ( src[ key ] !== undefined ) {
			( flatOptions[ key ] ? target : ( deep || ( deep = {} ) ) )[ key ] = src[ key ];
		}
	}
	if ( deep ) {
		jQuery.extend( true, target, deep );
	}
}

jQuery.fn.extend({
	load: function( url, params, callback ) {
		if ( typeof url !== "string" && _load ) {
			return _load.apply( this, arguments );

		// Don't do a request if no elements are being requested
		} else if ( !this.length ) {
			return this;
		}

		var off = url.indexOf( " " );
		if ( off >= 0 ) {
			var selector = url.slice( off, url.length );
			url = url.slice( 0, off );
		}

		// Default to a GET request
		var type = "GET";

		// If the second parameter was provided
		if ( params ) {
			// If it's a function
			if ( jQuery.isFunction( params ) ) {
				// We assume that it's the callback
				callback = params;
				params = undefined;

			// Otherwise, build a param string
			} else if ( typeof params === "object" ) {
				params = jQuery.param( params, jQuery.ajaxSettings.traditional );
				type = "POST";
			}
		}

		var self = this;

		// Request the remote document
		jQuery.ajax({
			url: url,
			type: type,
			dataType: "html",
			data: params,
			// Complete callback (responseText is used internally)
			complete: function( jqXHR, status, responseText ) {
				// Store the response as specified by the jqXHR object
				responseText = jqXHR.responseText;
				// If successful, inject the HTML into all the matched elements
				if ( jqXHR.isResolved() ) {
					// #4825: Get the actual response in case
					// a dataFilter is present in ajaxSettings
					jqXHR.done(function( r ) {
						responseText = r;
					});
					// See if a selector was specified
					self.html( selector ?
						// Create a dummy div to hold the results
						jQuery("<div>")
							// inject the contents of the document in, removing the scripts
							// to avoid any 'Permission Denied' errors in IE
							.append(responseText.replace(rscript, ""))

							// Locate the specified elements
							.find(selector) :

						// If not, just inject the full result
						responseText );
				}

				if ( callback ) {
					self.each( callback, [ responseText, status, jqXHR ] );
				}
			}
		});

		return this;
	},

	serialize: function() {
		return jQuery.param( this.serializeArray() );
	},

	serializeArray: function() {
		return this.map(function(){
			return this.elements ? jQuery.makeArray( this.elements ) : this;
		})
		.filter(function(){
			return this.name && !this.disabled &&
				( this.checked || rselectTextarea.test( this.nodeName ) ||
					rinput.test( this.type ) );
		})
		.map(function( i, elem ){
			var val = jQuery( this ).val();

			return val == null ?
				null :
				jQuery.isArray( val ) ?
					jQuery.map( val, function( val, i ){
						return { name: elem.name, value: val.replace( rCRLF, "\r\n" ) };
					}) :
					{ name: elem.name, value: val.replace( rCRLF, "\r\n" ) };
		}).get();
	}
});

// Attach a bunch of functions for handling common AJAX events
jQuery.each( "ajaxStart ajaxStop ajaxComplete ajaxError ajaxSuccess ajaxSend".split( " " ), function( i, o ){
	jQuery.fn[ o ] = function( f ){
		return this.on( o, f );
	};
});

jQuery.each( [ "get", "post" ], function( i, method ) {
	jQuery[ method ] = function( url, data, callback, type ) {
		// shift arguments if data argument was omitted
		if ( jQuery.isFunction( data ) ) {
			type = type || callback;
			callback = data;
			data = undefined;
		}

		return jQuery.ajax({
			type: method,
			url: url,
			data: data,
			success: callback,
			dataType: type
		});
	};
});

jQuery.extend({

	getScript: function( url, callback ) {
		return jQuery.get( url, undefined, callback, "script" );
	},

	getJSON: function( url, data, callback ) {
		return jQuery.get( url, data, callback, "json" );
	},

	// Creates a full fledged settings object into target
	// with both ajaxSettings and settings fields.
	// If target is omitted, writes into ajaxSettings.
	ajaxSetup: function( target, settings ) {
		if ( settings ) {
			// Building a settings object
			ajaxExtend( target, jQuery.ajaxSettings );
		} else {
			// Extending ajaxSettings
			settings = target;
			target = jQuery.ajaxSettings;
		}
		ajaxExtend( target, settings );
		return target;
	},

	ajaxSettings: {
		url: ajaxLocation,
		isLocal: rlocalProtocol.test( ajaxLocParts[ 1 ] ),
		global: true,
		type: "GET",
		contentType: "application/x-www-form-urlencoded; charset=UTF-8",
		processData: true,
		async: true,
		/*
		timeout: 0,
		data: null,
		dataType: null,
		username: null,
		password: null,
		cache: null,
		traditional: false,
		headers: {},
		*/

		accepts: {
			xml: "application/xml, text/xml",
			html: "text/html",
			text: "text/plain",
			json: "application/json, text/javascript",
			"*": allTypes
		},

		contents: {
			xml: /xml/,
			html: /html/,
			json: /json/
		},

		responseFields: {
			xml: "responseXML",
			text: "responseText"
		},

		// List of data converters
		// 1) key format is "source_type destination_type" (a single space in-between)
		// 2) the catchall symbol "*" can be used for source_type
		converters: {

			// Convert anything to text
			"* text": window.String,

			// Text to html (true = no transformation)
			"text html": true,

			// Evaluate text as a json expression
			"text json": jQuery.parseJSON,

			// Parse text as xml
			"text xml": jQuery.parseXML
		},

		// For options that shouldn't be deep extended:
		// you can add your own custom options here if
		// and when you create one that shouldn't be
		// deep extended (see ajaxExtend)
		flatOptions: {
			context: true,
			url: true
		}
	},

	ajaxPrefilter: addToPrefiltersOrTransports( prefilters ),
	ajaxTransport: addToPrefiltersOrTransports( transports ),

	// Main method
	ajax: function( url, options ) {

		// If url is an object, simulate pre-1.5 signature
		if ( typeof url === "object" ) {
			options = url;
			url = undefined;
		}

		// Force options to be an object
		options = options || {};

		var // Create the final options object
			s = jQuery.ajaxSetup( {}, options ),
			// Callbacks context
			callbackContext = s.context || s,
			// Context for global events
			// It's the callbackContext if one was provided in the options
			// and if it's a DOM node or a jQuery collection
			globalEventContext = callbackContext !== s &&
				( callbackContext.nodeType || callbackContext instanceof jQuery ) ?
						jQuery( callbackContext ) : jQuery.event,
			// Deferreds
			deferred = jQuery.Deferred(),
			completeDeferred = jQuery.Callbacks( "once memory" ),
			// Status-dependent callbacks
			statusCode = s.statusCode || {},
			// ifModified key
			ifModifiedKey,
			// Headers (they are sent all at once)
			requestHeaders = {},
			requestHeadersNames = {},
			// Response headers
			responseHeadersString,
			responseHeaders,
			// transport
			transport,
			// timeout handle
			timeoutTimer,
			// Cross-domain detection vars
			parts,
			// The jqXHR state
			state = 0,
			// To know if global events are to be dispatched
			fireGlobals,
			// Loop variable
			i,
			// Fake xhr
			jqXHR = {

				readyState: 0,

				// Caches the header
				setRequestHeader: function( name, value ) {
					if ( !state ) {
						var lname = name.toLowerCase();
						name = requestHeadersNames[ lname ] = requestHeadersNames[ lname ] || name;
						requestHeaders[ name ] = value;
					}
					return this;
				},

				// Raw string
				getAllResponseHeaders: function() {
					return state === 2 ? responseHeadersString : null;
				},

				// Builds headers hashtable if needed
				getResponseHeader: function( key ) {
					var match;
					if ( state === 2 ) {
						if ( !responseHeaders ) {
							responseHeaders = {};
							while( ( match = rheaders.exec( responseHeadersString ) ) ) {
								responseHeaders[ match[1].toLowerCase() ] = match[ 2 ];
							}
						}
						match = responseHeaders[ key.toLowerCase() ];
					}
					return match === undefined ? null : match;
				},

				// Overrides response content-type header
				overrideMimeType: function( type ) {
					if ( !state ) {
						s.mimeType = type;
					}
					return this;
				},

				// Cancel the request
				abort: function( statusText ) {
					statusText = statusText || "abort";
					if ( transport ) {
						transport.abort( statusText );
					}
					done( 0, statusText );
					return this;
				}
			};

		// Callback for when everything is done
		// It is defined here because jslint complains if it is declared
		// at the end of the function (which would be more logical and readable)
		function done( status, nativeStatusText, responses, headers ) {

			// Called once
			if ( state === 2 ) {
				return;
			}

			// State is "done" now
			state = 2;

			// Clear timeout if it exists
			if ( timeoutTimer ) {
				clearTimeout( timeoutTimer );
			}

			// Dereference transport for early garbage collection
			// (no matter how long the jqXHR object will be used)
			transport = undefined;

			// Cache response headers
			responseHeadersString = headers || "";

			// Set readyState
			jqXHR.readyState = status > 0 ? 4 : 0;

			var isSuccess,
				success,
				error,
				statusText = nativeStatusText,
				response = responses ? ajaxHandleResponses( s, jqXHR, responses ) : undefined,
				lastModified,
				etag;

			// If successful, handle type chaining
			if ( status >= 200 && status < 300 || status === 304 ) {

				// Set the If-Modified-Since and/or If-None-Match header, if in ifModified mode.
				if ( s.ifModified ) {

					if ( ( lastModified = jqXHR.getResponseHeader( "Last-Modified" ) ) ) {
						jQuery.lastModified[ ifModifiedKey ] = lastModified;
					}
					if ( ( etag = jqXHR.getResponseHeader( "Etag" ) ) ) {
						jQuery.etag[ ifModifiedKey ] = etag;
					}
				}

				// If not modified
				if ( status === 304 ) {

					statusText = "notmodified";
					isSuccess = true;

				// If we have data
				} else {

					try {
						success = ajaxConvert( s, response );
						statusText = "success";
						isSuccess = true;
					} catch(e) {
						// We have a parsererror
						statusText = "parsererror";
						error = e;
					}
				}
			} else {
				// We extract error from statusText
				// then normalize statusText and status for non-aborts
				error = statusText;
				if ( !statusText || status ) {
					statusText = "error";
					if ( status < 0 ) {
						status = 0;
					}
				}
			}

			// Set data for the fake xhr object
			jqXHR.status = status;
			jqXHR.statusText = "" + ( nativeStatusText || statusText );

			// Success/Error
			if ( isSuccess ) {
				deferred.resolveWith( callbackContext, [ success, statusText, jqXHR ] );
			} else {
				deferred.rejectWith( callbackContext, [ jqXHR, statusText, error ] );
			}

			// Status-dependent callbacks
			jqXHR.statusCode( statusCode );
			statusCode = undefined;

			if ( fireGlobals ) {
				globalEventContext.trigger( "ajax" + ( isSuccess ? "Success" : "Error" ),
						[ jqXHR, s, isSuccess ? success : error ] );
			}

			// Complete
			completeDeferred.fireWith( callbackContext, [ jqXHR, statusText ] );

			if ( fireGlobals ) {
				globalEventContext.trigger( "ajaxComplete", [ jqXHR, s ] );
				// Handle the global AJAX counter
				if ( !( --jQuery.active ) ) {
					jQuery.event.trigger( "ajaxStop" );
				}
			}
		}

		// Attach deferreds
		deferred.promise( jqXHR );
		jqXHR.success = jqXHR.done;
		jqXHR.error = jqXHR.fail;
		jqXHR.complete = completeDeferred.add;

		// Status-dependent callbacks
		jqXHR.statusCode = function( map ) {
			if ( map ) {
				var tmp;
				if ( state < 2 ) {
					for ( tmp in map ) {
						statusCode[ tmp ] = [ statusCode[tmp], map[tmp] ];
					}
				} else {
					tmp = map[ jqXHR.status ];
					jqXHR.then( tmp, tmp );
				}
			}
			return this;
		};

		// Remove hash character (#7531: and string promotion)
		// Add protocol if not provided (#5866: IE7 issue with protocol-less urls)
		// We also use the url parameter if available
		s.url = ( ( url || s.url ) + "" ).replace( rhash, "" ).replace( rprotocol, ajaxLocParts[ 1 ] + "//" );

		// Extract dataTypes list
		s.dataTypes = jQuery.trim( s.dataType || "*" ).toLowerCase().split( rspacesAjax );

		// Determine if a cross-domain request is in order
		if ( s.crossDomain == null ) {
			parts = rurl.exec( s.url.toLowerCase() );
			s.crossDomain = !!( parts &&
				( parts[ 1 ] != ajaxLocParts[ 1 ] || parts[ 2 ] != ajaxLocParts[ 2 ] ||
					( parts[ 3 ] || ( parts[ 1 ] === "http:" ? 80 : 443 ) ) !=
						( ajaxLocParts[ 3 ] || ( ajaxLocParts[ 1 ] === "http:" ? 80 : 443 ) ) )
			);
		}

		// Convert data if not already a string
		if ( s.data && s.processData && typeof s.data !== "string" ) {
			s.data = jQuery.param( s.data, s.traditional );
		}

		// Apply prefilters
		inspectPrefiltersOrTransports( prefilters, s, options, jqXHR );

		// If request was aborted inside a prefilter, stop there
		if ( state === 2 ) {
			return false;
		}

		// We can fire global events as of now if asked to
		fireGlobals = s.global;

		// Uppercase the type
		s.type = s.type.toUpperCase();

		// Determine if request has content
		s.hasContent = !rnoContent.test( s.type );

		// Watch for a new set of requests
		if ( fireGlobals && jQuery.active++ === 0 ) {
			jQuery.event.trigger( "ajaxStart" );
		}

		// More options handling for requests with no content
		if ( !s.hasContent ) {

			// If data is available, append data to url
			if ( s.data ) {
				s.url += ( rquery.test( s.url ) ? "&" : "?" ) + s.data;
				// #9682: remove data so that it's not used in an eventual retry
				delete s.data;
			}

			// Get ifModifiedKey before adding the anti-cache parameter
			ifModifiedKey = s.url;

			// Add anti-cache in url if needed
			if ( s.cache === false ) {

				var ts = jQuery.now(),
					// try replacing _= if it is there
					ret = s.url.replace( rts, "$1_=" + ts );

				// if nothing was replaced, add timestamp to the end
				s.url = ret + ( ( ret === s.url ) ? ( rquery.test( s.url ) ? "&" : "?" ) + "_=" + ts : "" );
			}
		}

		// Set the correct header, if data is being sent
		if ( s.data && s.hasContent && s.contentType !== false || options.contentType ) {
			jqXHR.setRequestHeader( "Content-Type", s.contentType );
		}

		// Set the If-Modified-Since and/or If-None-Match header, if in ifModified mode.
		if ( s.ifModified ) {
			ifModifiedKey = ifModifiedKey || s.url;
			if ( jQuery.lastModified[ ifModifiedKey ] ) {
				jqXHR.setRequestHeader( "If-Modified-Since", jQuery.lastModified[ ifModifiedKey ] );
			}
			if ( jQuery.etag[ ifModifiedKey ] ) {
				jqXHR.setRequestHeader( "If-None-Match", jQuery.etag[ ifModifiedKey ] );
			}
		}

		// Set the Accepts header for the server, depending on the dataType
		jqXHR.setRequestHeader(
			"Accept",
			s.dataTypes[ 0 ] && s.accepts[ s.dataTypes[0] ] ?
				s.accepts[ s.dataTypes[0] ] + ( s.dataTypes[ 0 ] !== "*" ? ", " + allTypes + "; q=0.01" : "" ) :
				s.accepts[ "*" ]
		);

		// Check for headers option
		for ( i in s.headers ) {
			jqXHR.setRequestHeader( i, s.headers[ i ] );
		}

		// Allow custom headers/mimetypes and early abort
		if ( s.beforeSend && ( s.beforeSend.call( callbackContext, jqXHR, s ) === false || state === 2 ) ) {
				// Abort if not done already
				jqXHR.abort();
				return false;

		}

		// Install callbacks on deferreds
		for ( i in { success: 1, error: 1, complete: 1 } ) {
			jqXHR[ i ]( s[ i ] );
		}

		// Get transport
		transport = inspectPrefiltersOrTransports( transports, s, options, jqXHR );

		// If no transport, we auto-abort
		if ( !transport ) {
			done( -1, "No Transport" );
		} else {
			jqXHR.readyState = 1;
			// Send global event
			if ( fireGlobals ) {
				globalEventContext.trigger( "ajaxSend", [ jqXHR, s ] );
			}
			// Timeout
			if ( s.async && s.timeout > 0 ) {
				timeoutTimer = setTimeout( function(){
					jqXHR.abort( "timeout" );
				}, s.timeout );
			}

			try {
				state = 1;
				transport.send( requestHeaders, done );
			} catch (e) {
				// Propagate exception as error if not done
				if ( state < 2 ) {
					done( -1, e );
				// Simply rethrow otherwise
				} else {
					throw e;
				}
			}
		}

		return jqXHR;
	},

	// Serialize an array of form elements or a set of
	// key/values into a query string
	param: function( a, traditional ) {
		var s = [],
			add = function( key, value ) {
				// If value is a function, invoke it and return its value
				value = jQuery.isFunction( value ) ? value() : value;
				s[ s.length ] = encodeURIComponent( key ) + "=" + encodeURIComponent( value );
			};

		// Set traditional to true for jQuery <= 1.3.2 behavior.
		if ( traditional === undefined ) {
			traditional = jQuery.ajaxSettings.traditional;
		}

		// If an array was passed in, assume that it is an array of form elements.
		if ( jQuery.isArray( a ) || ( a.jquery && !jQuery.isPlainObject( a ) ) ) {
			// Serialize the form elements
			jQuery.each( a, function() {
				add( this.name, this.value );
			});

		} else {
			// If traditional, encode the "old" way (the way 1.3.2 or older
			// did it), otherwise encode params recursively.
			for ( var prefix in a ) {
				buildParams( prefix, a[ prefix ], traditional, add );
			}
		}

		// Return the resulting serialization
		return s.join( "&" ).replace( r20, "+" );
	}
});

function buildParams( prefix, obj, traditional, add ) {
	if ( jQuery.isArray( obj ) ) {
		// Serialize array item.
		jQuery.each( obj, function( i, v ) {
			if ( traditional || rbracket.test( prefix ) ) {
				// Treat each array item as a scalar.
				add( prefix, v );

			} else {
				// If array item is non-scalar (array or object), encode its
				// numeric index to resolve deserialization ambiguity issues.
				// Note that rack (as of 1.0.0) can't currently deserialize
				// nested arrays properly, and attempting to do so may cause
				// a server error. Possible fixes are to modify rack's
				// deserialization algorithm or to provide an option or flag
				// to force array serialization to be shallow.
				buildParams( prefix + "[" + ( typeof v === "object" ? i : "" ) + "]", v, traditional, add );
			}
		});

	} else if ( !traditional && jQuery.type( obj ) === "object" ) {
		// Serialize object item.
		for ( var name in obj ) {
			buildParams( prefix + "[" + name + "]", obj[ name ], traditional, add );
		}

	} else {
		// Serialize scalar item.
		add( prefix, obj );
	}
}

// This is still on the jQuery object... for now
// Want to move this to jQuery.ajax some day
jQuery.extend({

	// Counter for holding the number of active queries
	active: 0,

	// Last-Modified header cache for next request
	lastModified: {},
	etag: {}

});

/* Handles responses to an ajax request:
 * - sets all responseXXX fields accordingly
 * - finds the right dataType (mediates between content-type and expected dataType)
 * - returns the corresponding response
 */
function ajaxHandleResponses( s, jqXHR, responses ) {

	var contents = s.contents,
		dataTypes = s.dataTypes,
		responseFields = s.responseFields,
		ct,
		type,
		finalDataType,
		firstDataType;

	// Fill responseXXX fields
	for ( type in responseFields ) {
		if ( type in responses ) {
			jqXHR[ responseFields[type] ] = responses[ type ];
		}
	}

	// Remove auto dataType and get content-type in the process
	while( dataTypes[ 0 ] === "*" ) {
		dataTypes.shift();
		if ( ct === undefined ) {
			ct = s.mimeType || jqXHR.getResponseHeader( "content-type" );
		}
	}

	// Check if we're dealing with a known content-type
	if ( ct ) {
		for ( type in contents ) {
			if ( contents[ type ] && contents[ type ].test( ct ) ) {
				dataTypes.unshift( type );
				break;
			}
		}
	}

	// Check to see if we have a response for the expected dataType
	if ( dataTypes[ 0 ] in responses ) {
		finalDataType = dataTypes[ 0 ];
	} else {
		// Try convertible dataTypes
		for ( type in responses ) {
			if ( !dataTypes[ 0 ] || s.converters[ type + " " + dataTypes[0] ] ) {
				finalDataType = type;
				break;
			}
			if ( !firstDataType ) {
				firstDataType = type;
			}
		}
		// Or just use first one
		finalDataType = finalDataType || firstDataType;
	}

	// If we found a dataType
	// We add the dataType to the list if needed
	// and return the corresponding response
	if ( finalDataType ) {
		if ( finalDataType !== dataTypes[ 0 ] ) {
			dataTypes.unshift( finalDataType );
		}
		return responses[ finalDataType ];
	}
}

// Chain conversions given the request and the original response
function ajaxConvert( s, response ) {

	// Apply the dataFilter if provided
	if ( s.dataFilter ) {
		response = s.dataFilter( response, s.dataType );
	}

	var dataTypes = s.dataTypes,
		converters = {},
		i,
		key,
		length = dataTypes.length,
		tmp,
		// Current and previous dataTypes
		current = dataTypes[ 0 ],
		prev,
		// Conversion expression
		conversion,
		// Conversion function
		conv,
		// Conversion functions (transitive conversion)
		conv1,
		conv2;

	// For each dataType in the chain
	for ( i = 1; i < length; i++ ) {

		// Create converters map
		// with lowercased keys
		if ( i === 1 ) {
			for ( key in s.converters ) {
				if ( typeof key === "string" ) {
					converters[ key.toLowerCase() ] = s.converters[ key ];
				}
			}
		}

		// Get the dataTypes
		prev = current;
		current = dataTypes[ i ];

		// If current is auto dataType, update it to prev
		if ( current === "*" ) {
			current = prev;
		// If no auto and dataTypes are actually different
		} else if ( prev !== "*" && prev !== current ) {

			// Get the converter
			conversion = prev + " " + current;
			conv = converters[ conversion ] || converters[ "* " + current ];

			// If there is no direct converter, search transitively
			if ( !conv ) {
				conv2 = undefined;
				for ( conv1 in converters ) {
					tmp = conv1.split( " " );
					if ( tmp[ 0 ] === prev || tmp[ 0 ] === "*" ) {
						conv2 = converters[ tmp[1] + " " + current ];
						if ( conv2 ) {
							conv1 = converters[ conv1 ];
							if ( conv1 === true ) {
								conv = conv2;
							} else if ( conv2 === true ) {
								conv = conv1;
							}
							break;
						}
					}
				}
			}
			// If we found no converter, dispatch an error
			if ( !( conv || conv2 ) ) {
				jQuery.error( "No conversion from " + conversion.replace(" "," to ") );
			}
			// If found converter is not an equivalence
			if ( conv !== true ) {
				// Convert with 1 or 2 converters accordingly
				response = conv ? conv( response ) : conv2( conv1(response) );
			}
		}
	}
	return response;
}




var jsc = jQuery.now(),
	jsre = /(\=)\?(&|$)|\?\?/i;

// Default jsonp settings
jQuery.ajaxSetup({
	jsonp: "callback",
	jsonpCallback: function() {
		return jQuery.expando + "_" + ( jsc++ );
	}
});

// Detect, normalize options and install callbacks for jsonp requests
jQuery.ajaxPrefilter( "json jsonp", function( s, originalSettings, jqXHR ) {

	var inspectData = ( typeof s.data === "string" ) && /^application\/x\-www\-form\-urlencoded/.test( s.contentType );

	if ( s.dataTypes[ 0 ] === "jsonp" ||
		s.jsonp !== false && ( jsre.test( s.url ) ||
				inspectData && jsre.test( s.data ) ) ) {

		var responseContainer,
			jsonpCallback = s.jsonpCallback =
				jQuery.isFunction( s.jsonpCallback ) ? s.jsonpCallback() : s.jsonpCallback,
			previous = window[ jsonpCallback ],
			url = s.url,
			data = s.data,
			replace = "$1" + jsonpCallback + "$2";

		if ( s.jsonp !== false ) {
			url = url.replace( jsre, replace );
			if ( s.url === url ) {
				if ( inspectData ) {
					data = data.replace( jsre, replace );
				}
				if ( s.data === data ) {
					// Add callback manually
					url += (/\?/.test( url ) ? "&" : "?") + s.jsonp + "=" + jsonpCallback;
				}
			}
		}

		s.url = url;
		s.data = data;

		// Install callback
		window[ jsonpCallback ] = function( response ) {
			responseContainer = [ response ];
		};

		// Clean-up function
		jqXHR.always(function() {
			// Set callback back to previous value
			window[ jsonpCallback ] = previous;
			// Call if it was a function and we have a response
			if ( responseContainer && jQuery.isFunction( previous ) ) {
				window[ jsonpCallback ]( responseContainer[ 0 ] );
			}
		});

		// Use data converter to retrieve json after script execution
		s.converters["script json"] = function() {
			if ( !responseContainer ) {
				jQuery.error( jsonpCallback + " was not called" );
			}
			return responseContainer[ 0 ];
		};

		// force json dataType
		s.dataTypes[ 0 ] = "json";

		// Delegate to script
		return "script";
	}
});




// Install script dataType
jQuery.ajaxSetup({
	accepts: {
		script: "text/javascript, application/javascript, application/ecmascript, application/x-ecmascript"
	},
	contents: {
		script: /javascript|ecmascript/
	},
	converters: {
		"text script": function( text ) {
			jQuery.globalEval( text );
			return text;
		}
	}
});

// Handle cache's special case and global
jQuery.ajaxPrefilter( "script", function( s ) {
	if ( s.cache === undefined ) {
		s.cache = false;
	}
	if ( s.crossDomain ) {
		s.type = "GET";
		s.global = false;
	}
});

// Bind script tag hack transport
jQuery.ajaxTransport( "script", function(s) {

	// This transport only deals with cross domain requests
	if ( s.crossDomain ) {

		var script,
			head = document.head || document.getElementsByTagName( "head" )[0] || document.documentElement;

		return {

			send: function( _, callback ) {

				script = document.createElement( "script" );

				script.async = "async";

				if ( s.scriptCharset ) {
					script.charset = s.scriptCharset;
				}

				script.src = s.url;

				// Attach handlers for all browsers
				script.onload = script.onreadystatechange = function( _, isAbort ) {

					if ( isAbort || !script.readyState || /loaded|complete/.test( script.readyState ) ) {

						// Handle memory leak in IE
						script.onload = script.onreadystatechange = null;

						// Remove the script
						if ( head && script.parentNode ) {
							head.removeChild( script );
						}

						// Dereference the script
						script = undefined;

						// Callback if not abort
						if ( !isAbort ) {
							callback( 200, "success" );
						}
					}
				};
				// Use insertBefore instead of appendChild  to circumvent an IE6 bug.
				// This arises when a base node is used (#2709 and #4378).
				head.insertBefore( script, head.firstChild );
			},

			abort: function() {
				if ( script ) {
					script.onload( 0, 1 );
				}
			}
		};
	}
});




var // #5280: Internet Explorer will keep connections alive if we don't abort on unload
	xhrOnUnloadAbort = window.ActiveXObject ? function() {
		// Abort all pending requests
		for ( var key in xhrCallbacks ) {
			xhrCallbacks[ key ]( 0, 1 );
		}
	} : false,
	xhrId = 0,
	xhrCallbacks;

// Functions to create xhrs
function createStandardXHR() {
	try {
		return new window.XMLHttpRequest();
	} catch( e ) {}
}

function createActiveXHR() {
	try {
		return new window.ActiveXObject( "Microsoft.XMLHTTP" );
	} catch( e ) {}
}

// Create the request object
// (This is still attached to ajaxSettings for backward compatibility)
jQuery.ajaxSettings.xhr = window.ActiveXObject ?
	/* Microsoft failed to properly
	 * implement the XMLHttpRequest in IE7 (can't request local files),
	 * so we use the ActiveXObject when it is available
	 * Additionally XMLHttpRequest can be disabled in IE7/IE8 so
	 * we need a fallback.
	 */
	function() {
		return !this.isLocal && createStandardXHR() || createActiveXHR();
	} :
	// For all other browsers, use the standard XMLHttpRequest object
	createStandardXHR;

// Determine support properties
(function( xhr ) {
	jQuery.extend( jQuery.support, {
		ajax: !!xhr,
		cors: !!xhr && ( "withCredentials" in xhr )
	});
})( jQuery.ajaxSettings.xhr() );

// Create transport if the browser can provide an xhr
if ( jQuery.support.ajax ) {

	jQuery.ajaxTransport(function( s ) {
		// Cross domain only allowed if supported through XMLHttpRequest
		if ( !s.crossDomain || jQuery.support.cors ) {

			var callback;

			return {
				send: function( headers, complete ) {

					// Get a new xhr
					var xhr = s.xhr(),
						handle,
						i;

					// Open the socket
					// Passing null username, generates a login popup on Opera (#2865)
					if ( s.username ) {
						xhr.open( s.type, s.url, s.async, s.username, s.password );
					} else {
						xhr.open( s.type, s.url, s.async );
					}

					// Apply custom fields if provided
					if ( s.xhrFields ) {
						for ( i in s.xhrFields ) {
							xhr[ i ] = s.xhrFields[ i ];
						}
					}

					// Override mime type if needed
					if ( s.mimeType && xhr.overrideMimeType ) {
						xhr.overrideMimeType( s.mimeType );
					}

					// X-Requested-With header
					// For cross-domain requests, seeing as conditions for a preflight are
					// akin to a jigsaw puzzle, we simply never set it to be sure.
					// (it can always be set on a per-request basis or even using ajaxSetup)
					// For same-domain requests, won't change header if already provided.
					if ( !s.crossDomain && !headers["X-Requested-With"] ) {
						headers[ "X-Requested-With" ] = "XMLHttpRequest";
					}

					// Need an extra try/catch for cross domain requests in Firefox 3
					try {
						for ( i in headers ) {
							xhr.setRequestHeader( i, headers[ i ] );
						}
					} catch( _ ) {}

					// Do send the request
					// This may raise an exception which is actually
					// handled in jQuery.ajax (so no try/catch here)
					xhr.send( ( s.hasContent && s.data ) || null );

					// Listener
					callback = function( _, isAbort ) {

						var status,
							statusText,
							responseHeaders,
							responses,
							xml;

						// Firefox throws exceptions when accessing properties
						// of an xhr when a network error occured
						// http://helpful.knobs-dials.com/index.php/Component_returned_failure_code:_0x80040111_(NS_ERROR_NOT_AVAILABLE)
						try {

							// Was never called and is aborted or complete
							if ( callback && ( isAbort || xhr.readyState === 4 ) ) {

								// Only called once
								callback = undefined;

								// Do not keep as active anymore
								if ( handle ) {
									xhr.onreadystatechange = jQuery.noop;
									if ( xhrOnUnloadAbort ) {
										delete xhrCallbacks[ handle ];
									}
								}

								// If it's an abort
								if ( isAbort ) {
									// Abort it manually if needed
									if ( xhr.readyState !== 4 ) {
										xhr.abort();
									}
								} else {
									status = xhr.status;
									responseHeaders = xhr.getAllResponseHeaders();
									responses = {};
									xml = xhr.responseXML;

									// Construct response list
									if ( xml && xml.documentElement /* #4958 */ ) {
										responses.xml = xml;
									}

									// When requesting binary data, IE6-9 will throw an exception
									// on any attempt to access responseText (#11426)
									try {
										responses.text = xhr.responseText;
									} catch( _ ) {
									}

									// Firefox throws an exception when accessing
									// statusText for faulty cross-domain requests
									try {
										statusText = xhr.statusText;
									} catch( e ) {
										// We normalize with Webkit giving an empty statusText
										statusText = "";
									}

									// Filter status for non standard behaviors

									// If the request is local and we have data: assume a success
									// (success with no data won't get notified, that's the best we
									// can do given current implementations)
									if ( !status && s.isLocal && !s.crossDomain ) {
										status = responses.text ? 200 : 404;
									// IE - #1450: sometimes returns 1223 when it should be 204
									} else if ( status === 1223 ) {
										status = 204;
									}
								}
							}
						} catch( firefoxAccessException ) {
							if ( !isAbort ) {
								complete( -1, firefoxAccessException );
							}
						}

						// Call complete if needed
						if ( responses ) {
							complete( status, statusText, responses, responseHeaders );
						}
					};

					// if we're in sync mode or it's in cache
					// and has been retrieved directly (IE6 & IE7)
					// we need to manually fire the callback
					if ( !s.async || xhr.readyState === 4 ) {
						callback();
					} else {
						handle = ++xhrId;
						if ( xhrOnUnloadAbort ) {
							// Create the active xhrs callbacks list if needed
							// and attach the unload handler
							if ( !xhrCallbacks ) {
								xhrCallbacks = {};
								jQuery( window ).unload( xhrOnUnloadAbort );
							}
							// Add to list of active xhrs callbacks
							xhrCallbacks[ handle ] = callback;
						}
						xhr.onreadystatechange = callback;
					}
				},

				abort: function() {
					if ( callback ) {
						callback(0,1);
					}
				}
			};
		}
	});
}




var elemdisplay = {},
	iframe, iframeDoc,
	rfxtypes = /^(?:toggle|show|hide)$/,
	rfxnum = /^([+\-]=)?([\d+.\-]+)([a-z%]*)$/i,
	timerId,
	fxAttrs = [
		// height animations
		[ "height", "marginTop", "marginBottom", "paddingTop", "paddingBottom" ],
		// width animations
		[ "width", "marginLeft", "marginRight", "paddingLeft", "paddingRight" ],
		// opacity animations
		[ "opacity" ]
	],
	fxNow;

jQuery.fn.extend({
	show: function( speed, easing, callback ) {
		var elem, display;

		if ( speed || speed === 0 ) {
			return this.animate( genFx("show", 3), speed, easing, callback );

		} else {
			for ( var i = 0, j = this.length; i < j; i++ ) {
				elem = this[ i ];

				if ( elem.style ) {
					display = elem.style.display;

					// Reset the inline display of this element to learn if it is
					// being hidden by cascaded rules or not
					if ( !jQuery._data(elem, "olddisplay") && display === "none" ) {
						display = elem.style.display = "";
					}

					// Set elements which have been overridden with display: none
					// in a stylesheet to whatever the default browser style is
					// for such an element
					if ( (display === "" && jQuery.css(elem, "display") === "none") ||
						!jQuery.contains( elem.ownerDocument.documentElement, elem ) ) {
						jQuery._data( elem, "olddisplay", defaultDisplay(elem.nodeName) );
					}
				}
			}

			// Set the display of most of the elements in a second loop
			// to avoid the constant reflow
			for ( i = 0; i < j; i++ ) {
				elem = this[ i ];

				if ( elem.style ) {
					display = elem.style.display;

					if ( display === "" || display === "none" ) {
						elem.style.display = jQuery._data( elem, "olddisplay" ) || "";
					}
				}
			}

			return this;
		}
	},

	hide: function( speed, easing, callback ) {
		if ( speed || speed === 0 ) {
			return this.animate( genFx("hide", 3), speed, easing, callback);

		} else {
			var elem, display,
				i = 0,
				j = this.length;

			for ( ; i < j; i++ ) {
				elem = this[i];
				if ( elem.style ) {
					display = jQuery.css( elem, "display" );

					if ( display !== "none" && !jQuery._data( elem, "olddisplay" ) ) {
						jQuery._data( elem, "olddisplay", display );
					}
				}
			}

			// Set the display of the elements in a second loop
			// to avoid the constant reflow
			for ( i = 0; i < j; i++ ) {
				if ( this[i].style ) {
					this[i].style.display = "none";
				}
			}

			return this;
		}
	},

	// Save the old toggle function
	_toggle: jQuery.fn.toggle,

	toggle: function( fn, fn2, callback ) {
		var bool = typeof fn === "boolean";

		if ( jQuery.isFunction(fn) && jQuery.isFunction(fn2) ) {
			this._toggle.apply( this, arguments );

		} else if ( fn == null || bool ) {
			this.each(function() {
				var state = bool ? fn : jQuery(this).is(":hidden");
				jQuery(this)[ state ? "show" : "hide" ]();
			});

		} else {
			this.animate(genFx("toggle", 3), fn, fn2, callback);
		}

		return this;
	},

	fadeTo: function( speed, to, easing, callback ) {
		return this.filter(":hidden").css("opacity", 0).show().end()
					.animate({opacity: to}, speed, easing, callback);
	},

	animate: function( prop, speed, easing, callback ) {
		var optall = jQuery.speed( speed, easing, callback );

		if ( jQuery.isEmptyObject( prop ) ) {
			return this.each( optall.complete, [ false ] );
		}

		// Do not change referenced properties as per-property easing will be lost
		prop = jQuery.extend( {}, prop );

		function doAnimation() {
			// XXX 'this' does not always have a nodeName when running the
			// test suite

			if ( optall.queue === false ) {
				jQuery._mark( this );
			}

			var opt = jQuery.extend( {}, optall ),
				isElement = this.nodeType === 1,
				hidden = isElement && jQuery(this).is(":hidden"),
				name, val, p, e, hooks, replace,
				parts, start, end, unit,
				method;

			// will store per property easing and be used to determine when an animation is complete
			opt.animatedProperties = {};

			// first pass over propertys to expand / normalize
			for ( p in prop ) {
				name = jQuery.camelCase( p );
				if ( p !== name ) {
					prop[ name ] = prop[ p ];
					delete prop[ p ];
				}

				if ( ( hooks = jQuery.cssHooks[ name ] ) && "expand" in hooks ) {
					replace = hooks.expand( prop[ name ] );
					delete prop[ name ];

					// not quite $.extend, this wont overwrite keys already present.
					// also - reusing 'p' from above because we have the correct "name"
					for ( p in replace ) {
						if ( ! ( p in prop ) ) {
							prop[ p ] = replace[ p ];
						}
					}
				}
			}

			for ( name in prop ) {
				val = prop[ name ];
				// easing resolution: per property > opt.specialEasing > opt.easing > 'swing' (default)
				if ( jQuery.isArray( val ) ) {
					opt.animatedProperties[ name ] = val[ 1 ];
					val = prop[ name ] = val[ 0 ];
				} else {
					opt.animatedProperties[ name ] = opt.specialEasing && opt.specialEasing[ name ] || opt.easing || 'swing';
				}

				if ( val === "hide" && hidden || val === "show" && !hidden ) {
					return opt.complete.call( this );
				}

				if ( isElement && ( name === "height" || name === "width" ) ) {
					// Make sure that nothing sneaks out
					// Record all 3 overflow attributes because IE does not
					// change the overflow attribute when overflowX and
					// overflowY are set to the same value
					opt.overflow = [ this.style.overflow, this.style.overflowX, this.style.overflowY ];

					// Set display property to inline-block for height/width
					// animations on inline elements that are having width/height animated
					if ( jQuery.css( this, "display" ) === "inline" &&
							jQuery.css( this, "float" ) === "none" ) {

						// inline-level elements accept inline-block;
						// block-level elements need to be inline with layout
						if ( !jQuery.support.inlineBlockNeedsLayout || defaultDisplay( this.nodeName ) === "inline" ) {
							this.style.display = "inline-block";

						} else {
							this.style.zoom = 1;
						}
					}
				}
			}

			if ( opt.overflow != null ) {
				this.style.overflow = "hidden";
			}

			for ( p in prop ) {
				e = new jQuery.fx( this, opt, p );
				val = prop[ p ];

				if ( rfxtypes.test( val ) ) {

					// Tracks whether to show or hide based on private
					// data attached to the element
					method = jQuery._data( this, "toggle" + p ) || ( val === "toggle" ? hidden ? "show" : "hide" : 0 );
					if ( method ) {
						jQuery._data( this, "toggle" + p, method === "show" ? "hide" : "show" );
						e[ method ]();
					} else {
						e[ val ]();
					}

				} else {
					parts = rfxnum.exec( val.toString() );
					start = e.cur();

					if ( parts ) {
						end = parseFloat( parts[2] );
						unit = parts[3] || ( jQuery.cssNumber[ p ] ? "" : "px" );

						// We need to compute starting value
						if ( unit !== "px" ) {
							jQuery.style( this, p, (end || 1) + unit);
							start = ( (end || 1) / e.cur() ) * start;
							jQuery.style( this, p, start + unit);
						}

						// If a +=/-= token was provided, we're doing a relative animation
						if ( parts[1] ) {
							end = ( (parts[ 1 ] === "-=" ? -1 : 1) * end ) + start;
						}

						e.custom( start, end, unit );

					} else {
						e.custom( start, val, "" );
					}
				}
			}

			// For JS strict compliance
			return true;
		}

		return optall.queue === false ?
			this.each( doAnimation ) :
			this.queue( optall.queue, doAnimation );
	},

	stop: function( type, clearQueue, gotoEnd ) {
		if ( typeof type !== "string" ) {
			gotoEnd = clearQueue;
			clearQueue = type;
			type = undefined;
		}
		if ( clearQueue && type !== false ) {
			this.queue( type || "fx", [] );
		}

		return this.each(function() {
			var index,
				hadTimers = false,
				timers = jQuery.timers,
				data = jQuery._data( this );

			// clear marker counters if we know they won't be
			if ( !gotoEnd ) {
				jQuery._unmark( true, this );
			}

			function stopQueue( elem, data, index ) {
				var hooks = data[ index ];
				jQuery.removeData( elem, index, true );
				hooks.stop( gotoEnd );
			}

			if ( type == null ) {
				for ( index in data ) {
					if ( data[ index ] && data[ index ].stop && index.indexOf(".run") === index.length - 4 ) {
						stopQueue( this, data, index );
					}
				}
			} else if ( data[ index = type + ".run" ] && data[ index ].stop ){
				stopQueue( this, data, index );
			}

			for ( index = timers.length; index--; ) {
				if ( timers[ index ].elem === this && (type == null || timers[ index ].queue === type) ) {
					if ( gotoEnd ) {

						// force the next step to be the last
						timers[ index ]( true );
					} else {
						timers[ index ].saveState();
					}
					hadTimers = true;
					timers.splice( index, 1 );
				}
			}

			// start the next in the queue if the last step wasn't forced
			// timers currently will call their complete callbacks, which will dequeue
			// but only if they were gotoEnd
			if ( !( gotoEnd && hadTimers ) ) {
				jQuery.dequeue( this, type );
			}
		});
	}

});

// Animations created synchronously will run synchronously
function createFxNow() {
	setTimeout( clearFxNow, 0 );
	return ( fxNow = jQuery.now() );
}

function clearFxNow() {
	fxNow = undefined;
}

// Generate parameters to create a standard animation
function genFx( type, num ) {
	var obj = {};

	jQuery.each( fxAttrs.concat.apply([], fxAttrs.slice( 0, num )), function() {
		obj[ this ] = type;
	});

	return obj;
}

// Generate shortcuts for custom animations
jQuery.each({
	slideDown: genFx( "show", 1 ),
	slideUp: genFx( "hide", 1 ),
	slideToggle: genFx( "toggle", 1 ),
	fadeIn: { opacity: "show" },
	fadeOut: { opacity: "hide" },
	fadeToggle: { opacity: "toggle" }
}, function( name, props ) {
	jQuery.fn[ name ] = function( speed, easing, callback ) {
		return this.animate( props, speed, easing, callback );
	};
});

jQuery.extend({
	speed: function( speed, easing, fn ) {
		var opt = speed && typeof speed === "object" ? jQuery.extend( {}, speed ) : {
			complete: fn || !fn && easing ||
				jQuery.isFunction( speed ) && speed,
			duration: speed,
			easing: fn && easing || easing && !jQuery.isFunction( easing ) && easing
		};

		opt.duration = jQuery.fx.off ? 0 : typeof opt.duration === "number" ? opt.duration :
			opt.duration in jQuery.fx.speeds ? jQuery.fx.speeds[ opt.duration ] : jQuery.fx.speeds._default;

		// normalize opt.queue - true/undefined/null -> "fx"
		if ( opt.queue == null || opt.queue === true ) {
			opt.queue = "fx";
		}

		// Queueing
		opt.old = opt.complete;

		opt.complete = function( noUnmark ) {
			if ( jQuery.isFunction( opt.old ) ) {
				opt.old.call( this );
			}

			if ( opt.queue ) {
				jQuery.dequeue( this, opt.queue );
			} else if ( noUnmark !== false ) {
				jQuery._unmark( this );
			}
		};

		return opt;
	},

	easing: {
		linear: function( p ) {
			return p;
		},
		swing: function( p ) {
			return ( -Math.cos( p*Math.PI ) / 2 ) + 0.5;
		}
	},

	timers: [],

	fx: function( elem, options, prop ) {
		this.options = options;
		this.elem = elem;
		this.prop = prop;

		options.orig = options.orig || {};
	}

});

jQuery.fx.prototype = {
	// Simple function for setting a style value
	update: function() {
		if ( this.options.step ) {
			this.options.step.call( this.elem, this.now, this );
		}

		( jQuery.fx.step[ this.prop ] || jQuery.fx.step._default )( this );
	},

	// Get the current size
	cur: function() {
		if ( this.elem[ this.prop ] != null && (!this.elem.style || this.elem.style[ this.prop ] == null) ) {
			return this.elem[ this.prop ];
		}

		var parsed,
			r = jQuery.css( this.elem, this.prop );
		// Empty strings, null, undefined and "auto" are converted to 0,
		// complex values such as "rotate(1rad)" are returned as is,
		// simple values such as "10px" are parsed to Float.
		return isNaN( parsed = parseFloat( r ) ) ? !r || r === "auto" ? 0 : r : parsed;
	},

	// Start an animation from one number to another
	custom: function( from, to, unit ) {
		var self = this,
			fx = jQuery.fx;

		this.startTime = fxNow || createFxNow();
		this.end = to;
		this.now = this.start = from;
		this.pos = this.state = 0;
		this.unit = unit || this.unit || ( jQuery.cssNumber[ this.prop ] ? "" : "px" );

		function t( gotoEnd ) {
			return self.step( gotoEnd );
		}

		t.queue = this.options.queue;
		t.elem = this.elem;
		t.saveState = function() {
			if ( jQuery._data( self.elem, "fxshow" + self.prop ) === undefined ) {
				if ( self.options.hide ) {
					jQuery._data( self.elem, "fxshow" + self.prop, self.start );
				} else if ( self.options.show ) {
					jQuery._data( self.elem, "fxshow" + self.prop, self.end );
				}
			}
		};

		if ( t() && jQuery.timers.push(t) && !timerId ) {
			timerId = setInterval( fx.tick, fx.interval );
		}
	},

	// Simple 'show' function
	show: function() {
		var dataShow = jQuery._data( this.elem, "fxshow" + this.prop );

		// Remember where we started, so that we can go back to it later
		this.options.orig[ this.prop ] = dataShow || jQuery.style( this.elem, this.prop );
		this.options.show = true;

		// Begin the animation
		// Make sure that we start at a small width/height to avoid any flash of content
		if ( dataShow !== undefined ) {
			// This show is picking up where a previous hide or show left off
			this.custom( this.cur(), dataShow );
		} else {
			this.custom( this.prop === "width" || this.prop === "height" ? 1 : 0, this.cur() );
		}

		// Start by showing the element
		jQuery( this.elem ).show();
	},

	// Simple 'hide' function
	hide: function() {
		// Remember where we started, so that we can go back to it later
		this.options.orig[ this.prop ] = jQuery._data( this.elem, "fxshow" + this.prop ) || jQuery.style( this.elem, this.prop );
		this.options.hide = true;

		// Begin the animation
		this.custom( this.cur(), 0 );
	},

	// Each step of an animation
	step: function( gotoEnd ) {
		var p, n, complete,
			t = fxNow || createFxNow(),
			done = true,
			elem = this.elem,
			options = this.options;

		if ( gotoEnd || t >= options.duration + this.startTime ) {
			this.now = this.end;
			this.pos = this.state = 1;
			this.update();

			options.animatedProperties[ this.prop ] = true;

			for ( p in options.animatedProperties ) {
				if ( options.animatedProperties[ p ] !== true ) {
					done = false;
				}
			}

			if ( done ) {
				// Reset the overflow
				if ( options.overflow != null && !jQuery.support.shrinkWrapBlocks ) {

					jQuery.each( [ "", "X", "Y" ], function( index, value ) {
						elem.style[ "overflow" + value ] = options.overflow[ index ];
					});
				}

				// Hide the element if the "hide" operation was done
				if ( options.hide ) {
					jQuery( elem ).hide();
				}

				// Reset the properties, if the item has been hidden or shown
				if ( options.hide || options.show ) {
					for ( p in options.animatedProperties ) {
						jQuery.style( elem, p, options.orig[ p ] );
						jQuery.removeData( elem, "fxshow" + p, true );
						// Toggle data is no longer needed
						jQuery.removeData( elem, "toggle" + p, true );
					}
				}

				// Execute the complete function
				// in the event that the complete function throws an exception
				// we must ensure it won't be called twice. #5684

				complete = options.complete;
				if ( complete ) {

					options.complete = false;
					complete.call( elem );
				}
			}

			return false;

		} else {
			// classical easing cannot be used with an Infinity duration
			if ( options.duration == Infinity ) {
				this.now = t;
			} else {
				n = t - this.startTime;
				this.state = n / options.duration;

				// Perform the easing function, defaults to swing
				this.pos = jQuery.easing[ options.animatedProperties[this.prop] ]( this.state, n, 0, 1, options.duration );
				this.now = this.start + ( (this.end - this.start) * this.pos );
			}
			// Perform the next step of the animation
			this.update();
		}

		return true;
	}
};

jQuery.extend( jQuery.fx, {
	tick: function() {
		var timer,
			timers = jQuery.timers,
			i = 0;

		for ( ; i < timers.length; i++ ) {
			timer = timers[ i ];
			// Checks the timer has not already been removed
			if ( !timer() && timers[ i ] === timer ) {
				timers.splice( i--, 1 );
			}
		}

		if ( !timers.length ) {
			jQuery.fx.stop();
		}
	},

	interval: 13,

	stop: function() {
		clearInterval( timerId );
		timerId = null;
	},

	speeds: {
		slow: 600,
		fast: 200,
		// Default speed
		_default: 400
	},

	step: {
		opacity: function( fx ) {
			jQuery.style( fx.elem, "opacity", fx.now );
		},

		_default: function( fx ) {
			if ( fx.elem.style && fx.elem.style[ fx.prop ] != null ) {
				fx.elem.style[ fx.prop ] = fx.now + fx.unit;
			} else {
				fx.elem[ fx.prop ] = fx.now;
			}
		}
	}
});

// Ensure props that can't be negative don't go there on undershoot easing
jQuery.each( fxAttrs.concat.apply( [], fxAttrs ), function( i, prop ) {
	// exclude marginTop, marginLeft, marginBottom and marginRight from this list
	if ( prop.indexOf( "margin" ) ) {
		jQuery.fx.step[ prop ] = function( fx ) {
			jQuery.style( fx.elem, prop, Math.max(0, fx.now) + fx.unit );
		};
	}
});

if ( jQuery.expr && jQuery.expr.filters ) {
	jQuery.expr.filters.animated = function( elem ) {
		return jQuery.grep(jQuery.timers, function( fn ) {
			return elem === fn.elem;
		}).length;
	};
}

// Try to restore the default display value of an element
function defaultDisplay( nodeName ) {

	if ( !elemdisplay[ nodeName ] ) {

		var body = document.body,
			elem = jQuery( "<" + nodeName + ">" ).appendTo( body ),
			display = elem.css( "display" );
		elem.remove();

		// If the simple way fails,
		// get element's real default display by attaching it to a temp iframe
		if ( display === "none" || display === "" ) {
			// No iframe to use yet, so create it
			if ( !iframe ) {
				iframe = document.createElement( "iframe" );
				iframe.frameBorder = iframe.width = iframe.height = 0;
			}

			body.appendChild( iframe );

			// Create a cacheable copy of the iframe document on first call.
			// IE and Opera will allow us to reuse the iframeDoc without re-writing the fake HTML
			// document to it; WebKit & Firefox won't allow reusing the iframe document.
			if ( !iframeDoc || !iframe.createElement ) {
				iframeDoc = ( iframe.contentWindow || iframe.contentDocument ).document;
				iframeDoc.write( ( jQuery.support.boxModel ? "<!doctype html>" : "" ) + "<html><body>" );
				iframeDoc.close();
			}

			elem = iframeDoc.createElement( nodeName );

			iframeDoc.body.appendChild( elem );

			display = jQuery.css( elem, "display" );
			body.removeChild( iframe );
		}

		// Store the correct default display
		elemdisplay[ nodeName ] = display;
	}

	return elemdisplay[ nodeName ];
}




var getOffset,
	rtable = /^t(?:able|d|h)$/i,
	rroot = /^(?:body|html)$/i;

if ( "getBoundingClientRect" in document.documentElement ) {
	getOffset = function( elem, doc, docElem, box ) {
		try {
			box = elem.getBoundingClientRect();
		} catch(e) {}

		// Make sure we're not dealing with a disconnected DOM node
		if ( !box || !jQuery.contains( docElem, elem ) ) {
			return box ? { top: box.top, left: box.left } : { top: 0, left: 0 };
		}

		var body = doc.body,
			win = getWindow( doc ),
			clientTop  = docElem.clientTop  || body.clientTop  || 0,
			clientLeft = docElem.clientLeft || body.clientLeft || 0,
			scrollTop  = win.pageYOffset || jQuery.support.boxModel && docElem.scrollTop  || body.scrollTop,
			scrollLeft = win.pageXOffset || jQuery.support.boxModel && docElem.scrollLeft || body.scrollLeft,
			top  = box.top  + scrollTop  - clientTop,
			left = box.left + scrollLeft - clientLeft;

		return { top: top, left: left };
	};

} else {
	getOffset = function( elem, doc, docElem ) {
		var computedStyle,
			offsetParent = elem.offsetParent,
			prevOffsetParent = elem,
			body = doc.body,
			defaultView = doc.defaultView,
			prevComputedStyle = defaultView ? defaultView.getComputedStyle( elem, null ) : elem.currentStyle,
			top = elem.offsetTop,
			left = elem.offsetLeft;

		while ( (elem = elem.parentNode) && elem !== body && elem !== docElem ) {
			if ( jQuery.support.fixedPosition && prevComputedStyle.position === "fixed" ) {
				break;
			}

			computedStyle = defaultView ? defaultView.getComputedStyle(elem, null) : elem.currentStyle;
			top  -= elem.scrollTop;
			left -= elem.scrollLeft;

			if ( elem === offsetParent ) {
				top  += elem.offsetTop;
				left += elem.offsetLeft;

				if ( jQuery.support.doesNotAddBorder && !(jQuery.support.doesAddBorderForTableAndCells && rtable.test(elem.nodeName)) ) {
					top  += parseFloat( computedStyle.borderTopWidth  ) || 0;
					left += parseFloat( computedStyle.borderLeftWidth ) || 0;
				}

				prevOffsetParent = offsetParent;
				offsetParent = elem.offsetParent;
			}

			if ( jQuery.support.subtractsBorderForOverflowNotVisible && computedStyle.overflow !== "visible" ) {
				top  += parseFloat( computedStyle.borderTopWidth  ) || 0;
				left += parseFloat( computedStyle.borderLeftWidth ) || 0;
			}

			prevComputedStyle = computedStyle;
		}

		if ( prevComputedStyle.position === "relative" || prevComputedStyle.position === "static" ) {
			top  += body.offsetTop;
			left += body.offsetLeft;
		}

		if ( jQuery.support.fixedPosition && prevComputedStyle.position === "fixed" ) {
			top  += Math.max( docElem.scrollTop, body.scrollTop );
			left += Math.max( docElem.scrollLeft, body.scrollLeft );
		}

		return { top: top, left: left };
	};
}

jQuery.fn.offset = function( options ) {
	if ( arguments.length ) {
		return options === undefined ?
			this :
			this.each(function( i ) {
				jQuery.offset.setOffset( this, options, i );
			});
	}

	var elem = this[0],
		doc = elem && elem.ownerDocument;

	if ( !doc ) {
		return null;
	}

	if ( elem === doc.body ) {
		return jQuery.offset.bodyOffset( elem );
	}

	return getOffset( elem, doc, doc.documentElement );
};

jQuery.offset = {

	bodyOffset: function( body ) {
		var top = body.offsetTop,
			left = body.offsetLeft;

		if ( jQuery.support.doesNotIncludeMarginInBodyOffset ) {
			top  += parseFloat( jQuery.css(body, "marginTop") ) || 0;
			left += parseFloat( jQuery.css(body, "marginLeft") ) || 0;
		}

		return { top: top, left: left };
	},

	setOffset: function( elem, options, i ) {
		var position = jQuery.css( elem, "position" );

		// set position first, in-case top/left are set even on static elem
		if ( position === "static" ) {
			elem.style.position = "relative";
		}

		var curElem = jQuery( elem ),
			curOffset = curElem.offset(),
			curCSSTop = jQuery.css( elem, "top" ),
			curCSSLeft = jQuery.css( elem, "left" ),
			calculatePosition = ( position === "absolute" || position === "fixed" ) && jQuery.inArray("auto", [curCSSTop, curCSSLeft]) > -1,
			props = {}, curPosition = {}, curTop, curLeft;

		// need to be able to calculate position if either top or left is auto and position is either absolute or fixed
		if ( calculatePosition ) {
			curPosition = curElem.position();
			curTop = curPosition.top;
			curLeft = curPosition.left;
		} else {
			curTop = parseFloat( curCSSTop ) || 0;
			curLeft = parseFloat( curCSSLeft ) || 0;
		}

		if ( jQuery.isFunction( options ) ) {
			options = options.call( elem, i, curOffset );
		}

		if ( options.top != null ) {
			props.top = ( options.top - curOffset.top ) + curTop;
		}
		if ( options.left != null ) {
			props.left = ( options.left - curOffset.left ) + curLeft;
		}

		if ( "using" in options ) {
			options.using.call( elem, props );
		} else {
			curElem.css( props );
		}
	}
};


jQuery.fn.extend({

	position: function() {
		if ( !this[0] ) {
			return null;
		}

		var elem = this[0],

		// Get *real* offsetParent
		offsetParent = this.offsetParent(),

		// Get correct offsets
		offset       = this.offset(),
		parentOffset = rroot.test(offsetParent[0].nodeName) ? { top: 0, left: 0 } : offsetParent.offset();

		// Subtract element margins
		// note: when an element has margin: auto the offsetLeft and marginLeft
		// are the same in Safari causing offset.left to incorrectly be 0
		offset.top  -= parseFloat( jQuery.css(elem, "marginTop") ) || 0;
		offset.left -= parseFloat( jQuery.css(elem, "marginLeft") ) || 0;

		// Add offsetParent borders
		parentOffset.top  += parseFloat( jQuery.css(offsetParent[0], "borderTopWidth") ) || 0;
		parentOffset.left += parseFloat( jQuery.css(offsetParent[0], "borderLeftWidth") ) || 0;

		// Subtract the two offsets
		return {
			top:  offset.top  - parentOffset.top,
			left: offset.left - parentOffset.left
		};
	},

	offsetParent: function() {
		return this.map(function() {
			var offsetParent = this.offsetParent || document.body;
			while ( offsetParent && (!rroot.test(offsetParent.nodeName) && jQuery.css(offsetParent, "position") === "static") ) {
				offsetParent = offsetParent.offsetParent;
			}
			return offsetParent;
		});
	}
});


// Create scrollLeft and scrollTop methods
jQuery.each( {scrollLeft: "pageXOffset", scrollTop: "pageYOffset"}, function( method, prop ) {
	var top = /Y/.test( prop );

	jQuery.fn[ method ] = function( val ) {
		return jQuery.access( this, function( elem, method, val ) {
			var win = getWindow( elem );

			if ( val === undefined ) {
				return win ? (prop in win) ? win[ prop ] :
					jQuery.support.boxModel && win.document.documentElement[ method ] ||
						win.document.body[ method ] :
					elem[ method ];
			}

			if ( win ) {
				win.scrollTo(
					!top ? val : jQuery( win ).scrollLeft(),
					 top ? val : jQuery( win ).scrollTop()
				);

			} else {
				elem[ method ] = val;
			}
		}, method, val, arguments.length, null );
	};
});

function getWindow( elem ) {
	return jQuery.isWindow( elem ) ?
		elem :
		elem.nodeType === 9 ?
			elem.defaultView || elem.parentWindow :
			false;
}




// Create width, height, innerHeight, innerWidth, outerHeight and outerWidth methods
jQuery.each( { Height: "height", Width: "width" }, function( name, type ) {
	var clientProp = "client" + name,
		scrollProp = "scroll" + name,
		offsetProp = "offset" + name;

	// innerHeight and innerWidth
	jQuery.fn[ "inner" + name ] = function() {
		var elem = this[0];
		return elem ?
			elem.style ?
			parseFloat( jQuery.css( elem, type, "padding" ) ) :
			this[ type ]() :
			null;
	};

	// outerHeight and outerWidth
	jQuery.fn[ "outer" + name ] = function( margin ) {
		var elem = this[0];
		return elem ?
			elem.style ?
			parseFloat( jQuery.css( elem, type, margin ? "margin" : "border" ) ) :
			this[ type ]() :
			null;
	};

	jQuery.fn[ type ] = function( value ) {
		return jQuery.access( this, function( elem, type, value ) {
			var doc, docElemProp, orig, ret;

			if ( jQuery.isWindow( elem ) ) {
				// 3rd condition allows Nokia support, as it supports the docElem prop but not CSS1Compat
				doc = elem.document;
				docElemProp = doc.documentElement[ clientProp ];
				return jQuery.support.boxModel && docElemProp ||
					doc.body && doc.body[ clientProp ] || docElemProp;
			}

			// Get document width or height
			if ( elem.nodeType === 9 ) {
				// Either scroll[Width/Height] or offset[Width/Height], whichever is greater
				doc = elem.documentElement;

				// when a window > document, IE6 reports a offset[Width/Height] > client[Width/Height]
				// so we can't use max, as it'll choose the incorrect offset[Width/Height]
				// instead we use the correct client[Width/Height]
				// support:IE6
				if ( doc[ clientProp ] >= doc[ scrollProp ] ) {
					return doc[ clientProp ];
				}

				return Math.max(
					elem.body[ scrollProp ], doc[ scrollProp ],
					elem.body[ offsetProp ], doc[ offsetProp ]
				);
			}

			// Get width or height on the element
			if ( value === undefined ) {
				orig = jQuery.css( elem, type );
				ret = parseFloat( orig );
				return jQuery.isNumeric( ret ) ? ret : orig;
			}

			// Set the width or height on the element
			jQuery( elem ).css( type, value );
		}, type, value, arguments.length, null );
	};
});




// Expose jQuery to the global object
window.jQuery = window.$ = jQuery;

// Expose jQuery as an AMD module, but only for AMD loaders that
// understand the issues with loading multiple versions of jQuery
// in a page that all might call define(). The loader will indicate
// they have special allowances for multiple jQuery versions by
// specifying define.amd.jQuery = true. Register as a named module,
// since jQuery can be concatenated with other files that may use define,
// but not use a proper concatenation script that understands anonymous
// AMD modules. A named AMD is safest and most robust way to register.
// Lowercase jquery is used because AMD module names are derived from
// file names, and jQuery is normally delivered in a lowercase file name.
// Do this after creating the global so that if an AMD module wants to call
// noConflict to hide this version of jQuery, it will work.
if ( typeof define === "function" && define.amd && define.amd.jQuery ) {
	define( "jquery", [], function () { return jQuery; } );
}



})( window );
// ColorBox v1.3.17.1 - a full featured, light-weight, customizable lightbox based on jQuery 1.3+
// Copyright (c) 2011 Jack Moore - jack@colorpowered.com
// Licensed under the MIT license: http://www.opensource.org/licenses/mit-license.php
// + minWidth&minHeight support
(function ($, document, window) {
	var
	// ColorBox Default Settings.	
	// See http://colorpowered.com/colorbox for details.
	defaults = {
		transition: "elastic",
		speed: 300,
		width: false,
		initialWidth: "600",
		innerWidth: false,
		maxWidth: false,
		minWidth:false,
		height: false,
		initialHeight: "450",
		innerHeight: false,
		maxHeight: false,
		minHeight:false,
		scalePhotos: true,
		scrolling: true,
		inline: false,
		html: false,
		iframe: false,
		fastIframe: true,
		photo: false,
		href: false,
		title: false,
		rel: false,
		opacity: 0.9,
		preloading: true,
		current: "image {current} of {total}",
		previous: "previous",
		next: "next",
		close: "close",
		open: false,
		returnFocus: true,
		loop: true,
		slideshow: false,
		slideshowAuto: true,
		slideshowSpeed: 2500,
		slideshowStart: "start slideshow",
		slideshowStop: "stop slideshow",
		onOpen: false,
		onLoad: false,
		onComplete: false,
		onCleanup: false,
		onClosed: false,
		overlayClose: true,		
		escKey: true,
		arrowKey: true,
        top: false,
        bottom: false,
        left: false,
        right: false,
        fixed: false,
        data: false
	},
	
	// Abstracting the HTML and event identifiers for easy rebranding
	colorbox = 'colorbox',
	prefix = 'cbox',
	
	// Events	
	event_open = prefix + '_open',
	event_load = prefix + '_load',
	event_complete = prefix + '_complete',
	event_cleanup = prefix + '_cleanup',
	event_closed = prefix + '_closed',
	event_purge = prefix + '_purge',
	
	// Special Handling for IE
	isIE = $.browser.msie && !$.support.opacity, // Detects IE6,7,8.  IE9 supports opacity.  Feature detection alone gave a false positive on at least one phone browser and on some development versions of Chrome, hence the user-agent test.
	isIE6 = isIE && $.browser.version < 7,
	event_ie6 = prefix + '_IE6',

	// Cached jQuery Object Variables
	$overlay,
	$box,
	$wrap,
	$content,
	$topBorder,
	$leftBorder,
	$rightBorder,
	$bottomBorder,
	$related,
	$window,
	$loaded,
	$loadingBay,
	$loadingOverlay,
	$title,
	$current,
	$slideshow,
	$next,
	$prev,
	$close,
	$groupControls,

	// Variables for cached values or use across multiple functions
	settings = {},
	interfaceHeight,
	interfaceWidth,
	loadedHeight,
	loadedWidth,
	element,
	index,
	photo,
	open,
	active,
	closing,
    handler,
    loadingTimer,
	
	publicMethod,
	boxElement = prefix + 'Element';
	
	// ****************
	// HELPER FUNCTIONS
	// ****************

	// jQuery object generator to reduce code size
	function $div(id, cssText) { 
		var div = document.createElement('div');
		if (id) {
            div.id = prefix + id;
        }
		div.style.cssText = cssText || '';
		return $(div);
	}

	// Convert % values to pixels
	function setSize(size, dimension) {
		dimension = dimension === 'x' ? $window.width() : $window.height();
		return (typeof size === 'string') ? Math.round((/%/.test(size) ? (dimension / 100) * parseInt(size, 10) : parseInt(size, 10))) : size;
	}
	
	// Checks an href to see if it is a photo.
	// There is a force photo option (photo: true) for hrefs that cannot be matched by this regex.
	function isImage(url) {
		return settings.photo || /\.(gif|png|jpg|jpeg|bmp)(?:\?([^#]*))?(?:#(\.*))?$/i.test(url);
	}
	
	// Assigns function results to their respective settings.  This allows functions to be used as values.
	function process(settings) {
		for (var i in settings) {
			if ($.isFunction(settings[i]) && i.substring(0, 2) !== 'on') { // checks to make sure the function isn't one of the callbacks, they will be handled at the appropriate time.
			    settings[i] = settings[i].call(element);
			}
		}
        
		settings.rel = settings.rel || element.rel || 'nofollow';
		settings.href = settings.href || $(element).attr('href');
		settings.title = settings.title || element.title;
        
        if (typeof settings.href === "string") {
            settings.href = $.trim(settings.href);
        }
	}

	function trigger(event, callback) {
		if (callback) {
			callback.call(element);
		}
		$.event.trigger(event);
	}

	// Slideshow functionality
	function slideshow() {
		var
		timeOut,
		className = prefix + "Slideshow_",
		click = "click." + prefix,
		start,
		stop,
		clear;
		
		if (settings.slideshow && $related[1]) {
			start = function () {
				$slideshow
					.text(settings.slideshowStop)
					.unbind(click)
					.bind(event_complete, function () {
						if (index < $related.length - 1 || settings.loop) {
							timeOut = setTimeout(publicMethod.next, settings.slideshowSpeed);
						}
					})
					.bind(event_load, function () {
						clearTimeout(timeOut);
					})
					.one(click + ' ' + event_cleanup, stop);
				$box.removeClass(className + "off").addClass(className + "on");
				timeOut = setTimeout(publicMethod.next, settings.slideshowSpeed);
			};
			
			stop = function () {
				clearTimeout(timeOut);
				$slideshow
					.text(settings.slideshowStart)
					.unbind([event_complete, event_load, event_cleanup, click].join(' '))
					.one(click, start);
				$box.removeClass(className + "on").addClass(className + "off");
			};
			
			if (settings.slideshowAuto) {
				start();
			} else {
				stop();
			}
		} else {
            $box.removeClass(className + "off " + className + "on");
        }
	}

	function launch(elem) {
		if (!closing) {
			
			element = elem;
			
			process($.extend(settings, $.data(element, colorbox)));
			
			$related = $(element);
			
			index = 0;
			
			if (settings.rel !== 'nofollow') {
				$related = $('.' + boxElement).filter(function () {
					var relRelated = $.data(this, colorbox).rel || this.rel;
					return (relRelated === settings.rel);
				});
				index = $related.index(element);
				
				// Check direct calls to ColorBox.
				if (index === -1) {
					$related = $related.add(element);
					index = $related.length - 1;
				}
			}
			
			if (!open) {
				open = active = true; // Prevents the page-change action from queuing up if the visitor holds down the left or right keys.
				
				$box.show();
				
				if (settings.returnFocus) {
					try {
						element.blur();
						$(element).one(event_closed, function () {
							try {
								this.focus();
							} catch (e) {
								// do nothing
							}
						});
					} catch (e) {
						// do nothing
					}
				}
				
				// +settings.opacity avoids a problem in IE when using non-zero-prefixed-string-values, like '.5'
				$overlay.css({"opacity": +settings.opacity, "cursor": settings.overlayClose ? "pointer" : "auto"}).show();
				
				// Opens inital empty ColorBox prior to content being loaded.
				settings.w = setSize(settings.initialWidth, 'x');
				settings.h = setSize(settings.initialHeight, 'y');
				publicMethod.position(0);
				
				if (isIE6) {
					$window.bind('resize.' + event_ie6 + ' scroll.' + event_ie6, function () {
						$overlay.css({width: $window.width(), height: $window.height(), top: $window.scrollTop(), left: $window.scrollLeft()});
					}).trigger('resize.' + event_ie6);
				}
				
				trigger(event_open, settings.onOpen);
				
				$groupControls.add($title).hide();
				
				$close.html(settings.close).show();
			}
			
			publicMethod.load(true);
		}
	}

	// ****************
	// PUBLIC FUNCTIONS
	// Usage format: $.fn.colorbox.close();
	// Usage from within an iframe: parent.$.fn.colorbox.close();
	// ****************
	
	publicMethod = $.fn[colorbox] = $[colorbox] = function (options, callback) {
		var $this = this, autoOpen;
		
		if (!$this[0] && $this.selector) { // if a selector was given and it didn't match any elements, go ahead and exit.
			return $this;
		}
		
		options = options || {};
		
		if (callback) {
			options.onComplete = callback;
		}
		
		if (!$this[0] || $this.selector === undefined) { // detects $.colorbox() and $.fn.colorbox()
			$this = $('<a/>');
			options.open = true; // assume an immediate open
		}
		
		$this.each(function () {
			$.data(this, colorbox, $.extend({}, $.data(this, colorbox) || defaults, options));
			$(this).addClass(boxElement);
		});
		
		autoOpen = options.open;
		
		if ($.isFunction(autoOpen)) {
			autoOpen = autoOpen.call($this);
		}
		
		if (autoOpen) {
			launch($this[0]);
		}
		
		return $this;
	};

	// Initialize ColorBox: store common calculations, preload the interface graphics, append the html.
	// This preps colorbox for a speedy open when clicked, and lightens the burdon on the browser by only
	// having to run once, instead of each time colorbox is opened.
	publicMethod.init = function () {
		// Create & Append jQuery Objects
		$window = $(window);
		$box = $div().attr({id: colorbox, 'class': isIE ? prefix + (isIE6 ? 'IE6' : 'IE') : ''});
		$overlay = $div("Overlay", isIE6 ? 'position:absolute' : '').hide();
		
		$wrap = $div("Wrapper");
		$content = $div("Content").append(
			$loaded = $div("LoadedContent", 'width:0; height:0; overflow:hidden'),
			$loadingOverlay = $div("LoadingOverlay").add($div("LoadingGraphic")),
			$title = $div("Title"),
			$current = $div("Current"),
			$next = $div("Next"),
			$prev = $div("Previous"),
			$slideshow = $div("Slideshow").bind(event_open, slideshow),
			$close = $div("Close")
		);
		$wrap.append( // The 3x3 Grid that makes up ColorBox
			$div().append(
				$div("TopLeft"),
				$topBorder = $div("TopCenter"),
				$div("TopRight")
			),
			$div(false, 'clear:left').append(
				$leftBorder = $div("MiddleLeft"),
				$content,
				$rightBorder = $div("MiddleRight")
			),
			$div(false, 'clear:left').append(
				$div("BottomLeft"),
				$bottomBorder = $div("BottomCenter"),
				$div("BottomRight")
			)
		).children().children().css({'float': 'left'});
		
		$loadingBay = $div(false, 'position:absolute; width:9999px; visibility:hidden; display:none');
		
		$('body').prepend($overlay, $box.append($wrap, $loadingBay));
		
		$content.children()
		.hover(function () {
			$(this).addClass('hover');
		}, function () {
			$(this).removeClass('hover');
		}).addClass('hover');
		
		// Cache values needed for size calculations
		interfaceHeight = $topBorder.height() + $bottomBorder.height() + $content.outerHeight(true) - $content.height();//Subtraction needed for IE6
		interfaceWidth = $leftBorder.width() + $rightBorder.width() + $content.outerWidth(true) - $content.width();
		loadedHeight = $loaded.outerHeight(true);
		loadedWidth = $loaded.outerWidth(true);
		
		// Setting padding to remove the need to do size conversions during the animation step.
		$box.css({"padding-bottom": interfaceHeight, "padding-right": interfaceWidth}).hide();
		
        // Setup button events.
        $next.click(function () {
            publicMethod.next();
        });
        $prev.click(function () {
            publicMethod.prev();
        });
        $close.click(function () {
            publicMethod.close();
        });
		
		$groupControls = $next.add($prev).add($current).add($slideshow);
		
		// Adding the 'hover' class allowed the browser to load the hover-state
		// background graphics.  The class can now can be removed.
		$content.children().removeClass('hover');
		


        
		$overlay.click(function () {
			if (settings.overlayClose) {
				publicMethod.close();
			}
		});
		
		// Set Navigation Key Bindings
		$(document).bind('keydown.' + prefix, function (e) {
            var key = e.keyCode;
			if (open && settings.escKey && key === 27) {
				e.preventDefault();
				publicMethod.close();
			}
			if (open && settings.arrowKey && $related[1]) {
				if (key === 37) {
					e.preventDefault();
					$prev.click();
				} else if (key === 39) {
					e.preventDefault();
					$next.click();
				}
			}
		});
	};
	
	publicMethod.remove = function () {
		$box.add($overlay).remove();
		$('.' + boxElement).removeData(colorbox).removeClass(boxElement);
	};

	publicMethod.position = function (speed, loadedCallback) {
        var animate_speed, top = 0, left = 0;
        
        // remove the modal so that it doesn't influence the document width/height        
        $box.hide();
        
        if (settings.fixed && !isIE6) {
            $box.css({position: 'fixed'});
        } else {
            top = $window.scrollTop();
            left = $window.scrollLeft();
            $box.css({position: 'absolute'});
        }
        
		// keeps the top and left positions within the browser's viewport.
        if (settings.right !== false) {
            left += Math.max($window.width() - settings.w - loadedWidth - interfaceWidth - setSize(settings.right, 'x'), 0);
        } else if (settings.left !== false) {
            left += setSize(settings.left, 'x');
        } else {
            left += Math.max($window.width() - settings.w - loadedWidth - interfaceWidth, 0) / 2;
        }
        
        if (settings.bottom !== false) {
            top += Math.max(document.documentElement.clientHeight - settings.h - loadedHeight - interfaceHeight - setSize(settings.bottom, 'y'), 0);
        } else if (settings.top !== false) {
            top += setSize(settings.top, 'y');
        } else {
            top += Math.max(document.documentElement.clientHeight - settings.h - loadedHeight - interfaceHeight, 0) / 2;
        }
        
        $box.show();
        
		// setting the speed to 0 to reduce the delay between same-sized content.
		animate_speed = ($box.width() === settings.w + loadedWidth && $box.height() === settings.h + loadedHeight) ? 0 : speed;
        
		// this gives the wrapper plenty of breathing room so it's floated contents can move around smoothly,
		// but it has to be shrank down around the size of div#colorbox when it's done.  If not,
		// it can invoke an obscure IE bug when using iframes.
		$wrap[0].style.width = $wrap[0].style.height = "9999px";
		
		function modalDimensions(that) {
			// loading overlay height has to be explicitly set for IE6.
			$topBorder[0].style.width = $bottomBorder[0].style.width = $content[0].style.width = that.style.width;
			$loadingOverlay[0].style.height = $loadingOverlay[1].style.height = $content[0].style.height = $leftBorder[0].style.height = $rightBorder[0].style.height = that.style.height;
		}
		
		$box.dequeue().animate({width: settings.w + loadedWidth, height: settings.h + loadedHeight, top: top, left: left}, {
			duration: animate_speed,
			complete: function () {
				modalDimensions(this);
				
				active = false;
				
				// shrink the wrapper down to exactly the size of colorbox to avoid a bug in IE's iframe implementation.
				$wrap[0].style.width = (settings.w + loadedWidth + interfaceWidth) + "px";
				$wrap[0].style.height = (settings.h + loadedHeight + interfaceHeight) + "px";
				
				if (loadedCallback) {
					loadedCallback();
				}
			},
			step: function () {
				modalDimensions(this);
			}
		});
	};

	publicMethod.resize = function (options) {
		if (open) {
			options = options || {};
			
			if (options.width) {
				settings.w = setSize(options.width, 'x') - loadedWidth - interfaceWidth;
			}
			if (options.innerWidth) {
				settings.w = setSize(options.innerWidth, 'x');
			}
			$loaded.css({width: settings.w});
			
			if (options.height) {
				settings.h = setSize(options.height, 'y') - loadedHeight - interfaceHeight;
			}
			if (options.innerHeight) {
				settings.h = setSize(options.innerHeight, 'y');
			}
			if (!options.innerHeight && !options.height) {				
				var $child = $loaded.wrapInner("<div style='overflow:auto'></div>").children(); // temporary wrapper to get an accurate estimate of just how high the total content should be.
				settings.h = $child.height();
				$child.replaceWith($child.children()); // ditch the temporary wrapper div used in height calculation
			}
			$loaded.css({height: settings.h});
			
			publicMethod.position(settings.transition === "none" ? 0 : settings.speed);
		}
	};

	publicMethod.prep = function (object) {
		if (!open) {
			return;
		}
		
		var speed = settings.transition === "none" ? 0 : settings.speed;
		
		$window.unbind('resize.' + prefix);
		$loaded.remove();
		$loaded = $div('LoadedContent').html(object);
		
		function getWidth() {
			settings.w = settings.w || $loaded.width();
			settings.w = settings.minw && settings.minw > settings.w ? settings.minw : settings.w;
			settings.w = settings.mw && settings.mw < settings.w ? settings.mw : settings.w;
			return settings.w;
		}
		function getHeight() {
			settings.h = settings.h || $loaded.height();
			settings.h = settings.minh && settings.minh > settings.h ? settings.minh : settings.h;
			settings.h = settings.mh && settings.mh < settings.h ? settings.mh : settings.h;
			return settings.h;
		}
		
		$loaded.hide()
		.appendTo($loadingBay.show())// content has to be appended to the DOM for accurate size calculations.
		.css({width: getWidth(), overflow: settings.scrolling ? 'auto' : 'hidden'})
		.css({height: getHeight()})// sets the height independently from the width in case the new width influences the value of height.
		.prependTo($content);
		
		$loadingBay.hide();
		
		// floating the IMG removes the bottom line-height and fixed a problem where IE miscalculates the width of the parent element as 100% of the document width.
		//$(photo).css({'float': 'none', marginLeft: 'auto', marginRight: 'auto'});
		
        $(photo).css({'float': 'none'});
        
		// Hides SELECT elements in IE6 because they would otherwise sit on top of the overlay.
		if (isIE6) {
			$('select').not($box.find('select')).filter(function () {
				return this.style.visibility !== 'hidden';
			}).css({'visibility': 'hidden'}).one(event_cleanup, function () {
				this.style.visibility = 'inherit';
			});
		}
		
		function setPosition(s) {
			publicMethod.position(s, function () {
				var prev, prevSrc, next, nextSrc, total = $related.length, iframe, complete;
				
				if (!open) {
					return;
				}
				
                function removeFilter() {
                    if (isIE) {
                        $box[0].style.removeAttribute('filter');
                    }
                }
                
				complete = function () {
                    clearTimeout(loadingTimer);
					$loadingOverlay.hide();
					trigger(event_complete, settings.onComplete);
				};
				
				if (isIE) {
					//This fadeIn helps the bicubic resampling to kick-in.
					if (photo) {
						$loaded.fadeIn(100);
					}
				}
				
				$title.html(settings.title).add($loaded).show();
				
				if (total > 1) { // handle grouping
					if (typeof settings.current === "string") {
						$current.html(settings.current.replace(/\{current\}/, index + 1).replace(/\{total\}/, total)).show();
					}
					
					$next[(settings.loop || index < total - 1) ? "show" : "hide"]().html(settings.next);
					$prev[(settings.loop || index) ? "show" : "hide"]().html(settings.previous);
					
					prev = index ? $related[index - 1] : $related[total - 1];
					next = index < total - 1 ? $related[index + 1] : $related[0];
					
					if (settings.slideshow) {
						$slideshow.show();
					}
					
					// Preloads images within a rel group
					if (settings.preloading) {
						nextSrc = $.data(next, colorbox).href || next.href;
						prevSrc = $.data(prev, colorbox).href || prev.href;
						
						nextSrc = $.isFunction(nextSrc) ? nextSrc.call(next) : nextSrc;
						prevSrc = $.isFunction(prevSrc) ? prevSrc.call(prev) : prevSrc;
						
						if (isImage(nextSrc)) {
							$('<img/>')[0].src = nextSrc;
						}
						
						if (isImage(prevSrc)) {
							$('<img/>')[0].src = prevSrc;
						}
					}
				} else {
					$groupControls.hide();
				}
				
				if (settings.iframe) {
					iframe = $('<iframe/>').addClass(prefix + 'Iframe')[0];
					
					if (settings.fastIframe) {
						complete();
					} else {
						$(iframe).one('load', complete);
					}
					iframe.name = prefix + (+new Date());
					iframe.src = settings.href;
					
					if (!settings.scrolling) {
						iframe.scrolling = "no";
					}
					
					if (isIE) {
                        iframe.frameBorder = 0;
						iframe.allowTransparency = "true";
					}
					
					$(iframe).appendTo($loaded).one(event_purge, function () {
						iframe.src = "//about:blank";
					});
				} else {
					complete();
				}
				
				if (settings.transition === 'fade') {
					$box.fadeTo(speed, 1, removeFilter);
				} else {
                    removeFilter();
				}
				
				$window.bind('resize.' + prefix, function () {
					publicMethod.position(0);
				});
			});
		}
		
		if (settings.transition === 'fade') {
			$box.fadeTo(speed, 0, function () {
				setPosition(0);
			});
		} else {
			setPosition(speed);
		}
	};

	publicMethod.load = function (launched) {
		var href, setResize, prep = publicMethod.prep;
		
		active = true;
		
		photo = false;
		
		element = $related[index];
		
		if (!launched) {
			process($.extend(settings, $.data(element, colorbox)));
		}
		
		trigger(event_purge);
		
		trigger(event_load, settings.onLoad);
		
		settings.h = settings.height ?
				setSize(settings.height, 'y') - loadedHeight - interfaceHeight :
				settings.innerHeight && setSize(settings.innerHeight, 'y');
		
		settings.w = settings.width ?
				setSize(settings.width, 'x') - loadedWidth - interfaceWidth :
				settings.innerWidth && setSize(settings.innerWidth, 'x');
		
		// Sets the minimum dimensions for use in image scaling
		settings.mw = settings.w;
		settings.mh = settings.h;
		settings.minw = settings.w;
		settings.minh = settings.h;

		// Re-evaluate the minimum width and height based on maxWidth and maxHeight values.
		// If the width or height exceed the maxWidth or maxHeight, use the maximum values instead.
		if (settings.maxWidth) {
			settings.mw = setSize(settings.maxWidth, 'x') - loadedWidth - interfaceWidth;
			settings.mw = settings.w && settings.w < settings.mw ? settings.w : settings.mw;
		}
		if(settings.minWidth){
			settings.minw = setSize(settings.minWidth, 'x') - loadedWidth - interfaceWidth;
		 	settings.minw = settings.w && settings.w > settings.minw ? settings.w : settings.minw;
		}
		if (settings.maxHeight) {
			settings.mh = setSize(settings.maxHeight, 'y') - loadedHeight - interfaceHeight;
			settings.mh = settings.h && settings.h < settings.mh ? settings.h : settings.mh;
		}
		if(settings.minHeight){
			settings.minh = setSize(settings.minHeight, 'y') - loadedHeight - interfaceHeight;
		 	settings.minh = settings.h && settings.h > settings.minh ? settings.h : settings.minh;
		}

		href = settings.href;
		
        loadingTimer = setTimeout(function () {
            $loadingOverlay.show();
        }, 100);
        
		if (settings.inline) {
			// Inserts an empty placeholder where inline content is being pulled from.
			// An event is bound to put inline content back when ColorBox closes or loads new content.
			$div().hide().insertBefore($(href)[0]).one(event_purge, function () {
				$(this).replaceWith($loaded.children());
			});
			prep($(href));
		} else if (settings.iframe) {
			// IFrame element won't be added to the DOM until it is ready to be displayed,
			// to avoid problems with DOM-ready JS that might be trying to run in that iframe.
			prep(" ");
		} else if (settings.html) {
			prep(settings.html);
		} else if (isImage(href)) {
			$(photo = new Image())
			.addClass(prefix + 'Photo')
			.error(function () {
				settings.title = false;
				prep($div('Error').text('This image could not be loaded'));
			})
			.load(function () {
				var percent;
				photo.onload = null; //stops animated gifs from firing the onload repeatedly.
				
				if (settings.scalePhotos) {
					setResize = function () {
						photo.height -= photo.height * percent;
						photo.width -= photo.width * percent;	
					};
					if (settings.mw && photo.width > settings.mw) {
						percent = (photo.width - settings.mw) / photo.width;
						setResize();
					}
					if (settings.mh && photo.height > settings.mh) {
						percent = (photo.height - settings.mh) / photo.height;
						setResize();
					}
				}
				
				if (settings.h) {
					photo.style.marginTop = Math.max(settings.h - photo.height, 0) / 2 + 'px';
				}
				
				if ($related[1] && (index < $related.length - 1 || settings.loop)) {
					photo.style.cursor = 'pointer';
					photo.onclick = function () {
                        publicMethod.next();
                    };
				}
				
				if (isIE) {
					photo.style.msInterpolationMode = 'bicubic';
				}
				
				setTimeout(function () { // A pause because Chrome will sometimes report a 0 by 0 size otherwise.
					prep(photo);
				}, 1);
			});
			
			setTimeout(function () { // A pause because Opera 10.6+ will sometimes not run the onload function otherwise.
				photo.src = href;
			}, 1);
		} else if (href) {
			$loadingBay.load(href, settings.data, function (data, status, xhr) {
				prep(status === 'error' ? $div('Error').text('Request unsuccessful: ' + xhr.statusText) : $(this).contents());
			});
		}
	};
        
	// Navigates to the next page/image in a set.
	publicMethod.next = function () {
		if (!active && $related[1] && (index < $related.length - 1 || settings.loop)) {
			index = index < $related.length - 1 ? index + 1 : 0;
			publicMethod.load();
		}
	};
	
	publicMethod.prev = function () {
		if (!active && $related[1] && (index || settings.loop)) {
			index = index ? index - 1 : $related.length - 1;
			publicMethod.load();
		}
	};

	// Note: to use this within an iframe use the following format: parent.$.fn.colorbox.close();
	publicMethod.close = function () {
		if (open && !closing) {
			
			closing = true;
			
			open = false;
			
			trigger(event_cleanup, settings.onCleanup);
			
			$window.unbind('.' + prefix + ' .' + event_ie6);
			
			$overlay.fadeTo(200, 0);
			
			$box.stop().fadeTo(300, 0, function () {
                 
				$box.add($overlay).css({'opacity': 1, cursor: 'auto'}).hide();
				
				trigger(event_purge);
				
				$loaded.remove();
				
				setTimeout(function () {
					closing = false;
					trigger(event_closed, settings.onClosed);
				}, 1);
			});
		}
	};

	// A method for fetching the current element ColorBox is referencing.
	// returns a jQuery object.
	publicMethod.element = function () {
		return $(element);
	};

	publicMethod.settings = defaults;
    
	// Bind the live event before DOM-ready for maximum performance in IE6 & 7.
    handler = function (e) {
        // checks to see if it was a non-left mouse-click and for clicks modified with ctrl, shift, or alt.
        if (!((e.button !== 0 && typeof e.button !== 'undefined') || e.ctrlKey || e.shiftKey || e.altKey)) {
            e.preventDefault();
            launch(this);
        }
    };
    
    if ($.fn.delegate) {
        $(document).delegate('.' + boxElement, 'click', handler);
    } else {
        $('.' + boxElement).live('click', handler);
    }
    
	// Initializes ColorBox when the DOM has loaded
	$(publicMethod.init);

}(jQuery, document, this));
/**
* hoverIntent is similar to jQuery's built-in "hover" function except that
* instead of firing the onMouseOver event immediately, hoverIntent checks
* to see if the user's mouse has slowed down (beneath the sensitivity
* threshold) before firing the onMouseOver event.
* 
* hoverIntent r6 // 2011.02.26 // jQuery 1.5.1+
* <http://cherne.net/brian/resources/jquery.hoverIntent.html>
* 
* hoverIntent is currently available for use in all personal or commercial 
* projects under both MIT and GPL licenses. This means that you can choose 
* the license that best suits your project, and use it accordingly.
* 
* // basic usage (just like .hover) receives onMouseOver and onMouseOut functions
* $("ul li").hoverIntent( showNav , hideNav );
* 
* // advanced usage receives configuration object only
* $("ul li").hoverIntent({
*	sensitivity: 7, // number = sensitivity threshold (must be 1 or higher)
*	interval: 100,   // number = milliseconds of polling interval
*	over: showNav,  // function = onMouseOver callback (required)
*	timeout: 0,   // number = milliseconds delay before onMouseOut function call
*	out: hideNav    // function = onMouseOut callback (required)
* });
* 
* @param  f  onMouseOver function || An object with configuration options
* @param  g  onMouseOut function  || Nothing (use configuration options object)
* @author    Brian Cherne brian(at)cherne(dot)net
*/
(function($) {
	$.fn.hoverIntent = function(f,g) {
		// default configuration options
		var cfg = {
			sensitivity: 7,
			interval: 100,
			timeout: 0
		};
		// override configuration options with user supplied object
		cfg = $.extend(cfg, g ? { over: f, out: g } : f );

		// instantiate variables
		// cX, cY = current X and Y position of mouse, updated by mousemove event
		// pX, pY = previous X and Y position of mouse, set by mouseover and polling interval
		var cX, cY, pX, pY;

		// A private function for getting mouse position
		var track = function(ev) {
			cX = ev.pageX;
			cY = ev.pageY;
		};

		// A private function for comparing current and previous mouse position
		var compare = function(ev,ob) {
			ob.hoverIntent_t = clearTimeout(ob.hoverIntent_t);
			// compare mouse positions to see if they've crossed the threshold
			if ( ( Math.abs(pX-cX) + Math.abs(pY-cY) ) < cfg.sensitivity ) {
				$(ob).unbind("mousemove",track);
				// set hoverIntent state to true (so mouseOut can be called)
				ob.hoverIntent_s = 1;
				return cfg.over.apply(ob,[ev]);
			} else {
				// set previous coordinates for next time
				pX = cX; pY = cY;
				// use self-calling timeout, guarantees intervals are spaced out properly (avoids JavaScript timer bugs)
				ob.hoverIntent_t = setTimeout( function(){compare(ev, ob);} , cfg.interval );
			}
		};

		// A private function for delaying the mouseOut function
		var delay = function(ev,ob) {
			ob.hoverIntent_t = clearTimeout(ob.hoverIntent_t);
			ob.hoverIntent_s = 0;
			return cfg.out.apply(ob,[ev]);
		};

		// A private function for handling mouse 'hovering'
		var handleHover = function(e) {
			// copy objects to be passed into t (required for event object to be passed in IE)
			var ev = jQuery.extend({},e);
			var ob = this;

			// cancel hoverIntent timer if it exists
			if (ob.hoverIntent_t) { ob.hoverIntent_t = clearTimeout(ob.hoverIntent_t); }

			// if e.type == "mouseenter"
			if (e.type == "mouseenter") {
				// set "previous" X and Y position based on initial entry point
				pX = ev.pageX; pY = ev.pageY;
				// update "current" X and Y position based on mousemove
				$(ob).bind("mousemove",track);
				// start polling interval (self-calling timeout) to compare mouse coordinates over time
				if (ob.hoverIntent_s != 1) { ob.hoverIntent_t = setTimeout( function(){compare(ev,ob);} , cfg.interval );}

			// else e.type == "mouseleave"
			} else {
				// unbind expensive mousemove event
				$(ob).unbind("mousemove",track);
				// if hoverIntent state is true, then call the mouseOut function after the specified delay
				if (ob.hoverIntent_s == 1) { ob.hoverIntent_t = setTimeout( function(){delay(ev,ob);} , cfg.timeout );}
			}
		};

		// bind the function to the two event listeners
		return this.bind('mouseenter',handleHover).bind('mouseleave',handleHover);
	};
})(jQuery);
/**
 * Cookie plugin
 *
 * Copyright (c) 2006 Klaus Hartl (stilbuero.de)
 * Dual licensed under the MIT and GPL licenses:
 * http://www.opensource.org/licenses/mit-license.php
 * http://www.gnu.org/licenses/gpl.html
 *
 */

/**
 * Create a cookie with the given name and value and other optional parameters.
 *
 * @example $.cookie('the_cookie', 'the_value');
 * @desc Set the value of a cookie.
 * @example $.cookie('the_cookie', 'the_value', { expires: 7, path: '/', domain: 'jquery.com', secure: true });
 * @desc Create a cookie with all available options.
 * @example $.cookie('the_cookie', 'the_value');
 * @desc Create a session cookie.
 * @example $.cookie('the_cookie', null);
 * @desc Delete a cookie by passing null as value. Keep in mind that you have to use the same path and domain
 *       used when the cookie was set.
 *
 * @param String name The name of the cookie.
 * @param String value The value of the cookie.
 * @param Object options An object literal containing key/value pairs to provide optional cookie attributes.
 * @option Number|Date expires Either an integer specifying the expiration date from now on in days or a Date object.
 *                             If a negative value is specified (e.g. a date in the past), the cookie will be deleted.
 *                             If set to null or omitted, the cookie will be a session cookie and will not be retained
 *                             when the the browser exits.
 * @option String path The value of the path atribute of the cookie (default: path of page that created the cookie).
 * @option String domain The value of the domain attribute of the cookie (default: domain of page that created the cookie).
 * @option Boolean secure If true, the secure attribute of the cookie will be set and the cookie transmission will
 *                        require a secure protocol (like HTTPS).
 * @type undefined
 *
 * @name $.cookie
 * @cat Plugins/Cookie
 * @author Klaus Hartl/klaus.hartl@stilbuero.de
 */

/**
 * Get the value of a cookie with the given name.
 *
 * @example $.cookie('the_cookie');
 * @desc Get the value of a cookie.
 *
 * @param String name The name of the cookie.
 * @return The value of the cookie.
 * @type String
 *
 * @name $.cookie
 * @cat Plugins/Cookie
 * @author Klaus Hartl/klaus.hartl@stilbuero.de
 */
jQuery.cookie = function(name, value, options) {
    if (typeof value != 'undefined') { // name and value given, set cookie
        options = options || {};
        if (value === null) {
            value = '';
            options.expires = -1;
        }
        var expires = '';
        if (options.expires && (typeof options.expires == 'number' || options.expires.toUTCString)) {
            var date;
            if (typeof options.expires == 'number') {
                date = new Date();
                date.setTime(date.getTime() + (options.expires * 24 * 60 * 60 * 1000));
            } else {
                date = options.expires;
            }
            expires = '; expires=' + date.toUTCString(); // use expires attribute, max-age is not supported by IE
        }
        // CAUTION: Needed to parenthesize options.path and options.domain
        // in the following expressions, otherwise they evaluate to undefined
        // in the packed version for some reason...
        var path = options.path ? '; path=' + (options.path) : '';
        var domain = options.domain ? '; domain=' + (options.domain) : '';
        var secure = options.secure ? '; secure' : '';
        document.cookie = [name, '=', encodeURIComponent(value), expires, path, domain, secure].join('');
    } else { // only name given, get cookie
        var cookieValue = null;
        if (document.cookie && document.cookie != '') {
            var cookies = document.cookie.split(';');
            for (var i = 0; i < cookies.length; i++) {
                var cookie = jQuery.trim(cookies[i]);
                // Does this cookie string begin with the name we want?
                if (cookie.substring(0, name.length + 1) == (name + '=')) {
                    cookieValue = decodeURIComponent(cookie.substring(name.length + 1));
                    break;
                }
            }
        }
        return cookieValue;
    }
};
/* jshint browser: true */
/* global define: false */
/* https://github.com/andris9/simpleStorage */

// AMD shim
(function(root, factory) {

    'use strict';

    if (typeof define === 'function' && define.amd) {
        define(factory);
    } else if (typeof exports !== 'undefined') {
        module.exports = factory();
    } else {
        root.simpleStorage = factory();
    }

}(this, function() {

    'use strict';

    var
        VERSION = '0.1.3',

    /* This is the object, that holds the cached values */
        _storage = false,

    /* How much space does the storage take */
        _storage_size = 0,

        _storage_available = false,

        _ttl_timeout = null;

    // This method might throw as it touches localStorage and doing so
    // can be prohibited in some environments
    function _init() {

        // If localStorage does not exist, the following throws
        // This is intentional
        window.localStorage.setItem('__simpleStorageInitTest', 'tmpval');
        window.localStorage.removeItem('__simpleStorageInitTest');

        // Load data from storage
        _load_storage();

        // remove dead keys
        _handleTTL();

        // start listening for changes
        _setupUpdateObserver();

        // handle cached navigation
        if ('addEventListener' in window) {
            window.addEventListener('pageshow', function(event) {
                if (event.persisted) {
                    _reloadData();
                }
            }, false);
        }

        _storage_available = true;
    }

    /**
     * Sets up a storage change observer
     */
    function _setupUpdateObserver() {
        if ('addEventListener' in window) {
            window.addEventListener('storage', _reloadData, false);
        } else {
            document.attachEvent('onstorage', _reloadData);
        }
    }

    /**
     * Reload data from storage when needed
     */
    function _reloadData() {
        try {
            _load_storage();
        } catch (E) {
            _storage_available = false;
            return;
        }
        _handleTTL();
    }

    function _load_storage() {
        var source = localStorage.getItem('simpleStorage');

        try {
            _storage = JSON.parse(source) || {};
        } catch (E) {
            _storage = {};
        }

        _storage_size = _get_storage_size();
    }

    function _save() {
        try {
            localStorage.setItem('simpleStorage', JSON.stringify(_storage));
            _storage_size = _get_storage_size();
        } catch (E) {
            return E;
        }
        return true;
    }

    function _get_storage_size() {
        var source = localStorage.getItem('simpleStorage');
        return source ? String(source).length : 0;
    }

    function _handleTTL() {
        var curtime, i, len, expire, keys, nextExpire = Infinity,
            expiredKeysCount = 0;

        clearTimeout(_ttl_timeout);

        if (!_storage || !_storage.__simpleStorage_meta || !_storage.__simpleStorage_meta.TTL) {
            return;
        }

        curtime = +new Date();
        keys = _storage.__simpleStorage_meta.TTL.keys || [];
        expire = _storage.__simpleStorage_meta.TTL.expire || {};

        for (i = 0, len = keys.length; i < len; i++) {
            if (expire[keys[i]] <= curtime) {
                expiredKeysCount++;
                delete _storage[keys[i]];
                delete expire[keys[i]];
            } else {
                if (expire[keys[i]] < nextExpire) {
                    nextExpire = expire[keys[i]];
                }
                break;
            }
        }

        // set next check
        if (nextExpire != Infinity) {
            _ttl_timeout = setTimeout(_handleTTL, Math.min(nextExpire - curtime, 0x7FFFFFFF));
        }

        // remove expired from TTL list and save changes
        if (expiredKeysCount) {
            keys.splice(0, expiredKeysCount);

            _cleanMetaObject();
            _save();
        }
    }

    function _setTTL(key, ttl) {
        var curtime = +new Date(),
            i, len, added = false;

        ttl = Number(ttl) || 0;

        // Set TTL value for the key
        if (ttl !== 0) {
            // If key exists, set TTL
            if (_storage.hasOwnProperty(key)) {

                if (!_storage.__simpleStorage_meta) {
                    _storage.__simpleStorage_meta = {};
                }

                if (!_storage.__simpleStorage_meta.TTL) {
                    _storage.__simpleStorage_meta.TTL = {
                        expire: {},
                        keys: []
                    };
                }

                _storage.__simpleStorage_meta.TTL.expire[key] = curtime + ttl;

                // find the expiring key in the array and remove it and all before it (because of sort)
                if (_storage.__simpleStorage_meta.TTL.expire.hasOwnProperty(key)) {
                    for (i = 0, len = _storage.__simpleStorage_meta.TTL.keys.length; i < len; i++) {
                        if (_storage.__simpleStorage_meta.TTL.keys[i] == key) {
                            _storage.__simpleStorage_meta.TTL.keys.splice(i);
                        }
                    }
                }

                // add key to keys array preserving sort (soonest first)
                for (i = 0, len = _storage.__simpleStorage_meta.TTL.keys.length; i < len; i++) {
                    if (_storage.__simpleStorage_meta.TTL.expire[_storage.__simpleStorage_meta.TTL.keys[i]] > (curtime + ttl)) {
                        _storage.__simpleStorage_meta.TTL.keys.splice(i, 0, key);
                        added = true;
                        break;
                    }
                }

                // if not added in previous loop, add here
                if (!added) {
                    _storage.__simpleStorage_meta.TTL.keys.push(key);
                }
            } else {
                return false;
            }
        } else {
            // Remove TTL if set
            if (_storage && _storage.__simpleStorage_meta && _storage.__simpleStorage_meta.TTL) {

                if (_storage.__simpleStorage_meta.TTL.expire.hasOwnProperty(key)) {
                    delete _storage.__simpleStorage_meta.TTL.expire[key];
                    for (i = 0, len = _storage.__simpleStorage_meta.TTL.keys.length; i < len; i++) {
                        if (_storage.__simpleStorage_meta.TTL.keys[i] == key) {
                            _storage.__simpleStorage_meta.TTL.keys.splice(i, 1);
                            break;
                        }
                    }
                }

                _cleanMetaObject();
            }
        }

        // schedule next TTL check
        clearTimeout(_ttl_timeout);
        if (_storage && _storage.__simpleStorage_meta && _storage.__simpleStorage_meta.TTL && _storage.__simpleStorage_meta.TTL.keys.length) {
            _ttl_timeout = setTimeout(_handleTTL, Math.min(Math.max(_storage.__simpleStorage_meta.TTL.expire[_storage.__simpleStorage_meta.TTL.keys[0]] - curtime, 0), 0x7FFFFFFF));
        }

        return true;
    }

    function _cleanMetaObject() {
        var updated = false,
            hasProperties = false,
            i;

        if (!_storage || !_storage.__simpleStorage_meta) {
            return updated;
        }

        // If nothing to TTL, remove the object
        if (_storage.__simpleStorage_meta.TTL && !_storage.__simpleStorage_meta.TTL.keys.length) {
            delete _storage.__simpleStorage_meta.TTL;
            updated = true;
        }

        // If meta object is empty, remove it
        for (i in _storage.__simpleStorage_meta) {
            if (_storage.__simpleStorage_meta.hasOwnProperty(i)) {
                hasProperties = true;
                break;
            }
        }

        if (!hasProperties) {
            delete _storage.__simpleStorage_meta;
            updated = true;
        }

        return updated;
    }

    ////////////////////////// PUBLIC INTERFACE /////////////////////////

    try {
        _init();
    } catch (E) {}

    return {

        version: VERSION,

        canUse: function() {
            return !!_storage_available;
        },

        set: function(key, value, options) {
            if (key == '__simpleStorage_meta') {
                return false;
            }

            if (!_storage) {
                return false;
            }

            // undefined values are deleted automatically
            if (typeof value == 'undefined') {
                return this.deleteKey(key);
            }

            options = options || {};

            // Check if the value is JSON compatible (and remove reference to existing objects/arrays)
            try {
                value = JSON.parse(JSON.stringify(value));
            } catch (E) {
                return E;
            }

            _storage[key] = value;

            _setTTL(key, options.TTL || 0);

            return _save();
        },

        get: function(key) {
            if (!_storage) {
                return false;
            }

            if (_storage.hasOwnProperty(key) && key != '__simpleStorage_meta') {
                // TTL value for an existing key is either a positive number or an Infinity
                if (this.getTTL(key)) {
                    return _storage[key];
                }
            }
        },

        deleteKey: function(key) {

            if (!_storage) {
                return false;
            }

            if (key in _storage) {
                delete _storage[key];

                _setTTL(key, 0);

                return _save();
            }

            return false;
        },

        setTTL: function(key, ttl) {
            if (!_storage) {
                return false;
            }

            _setTTL(key, ttl);

            return _save();
        },

        getTTL: function(key) {
            var ttl;

            if (!_storage) {
                return false;
            }

            if (_storage.hasOwnProperty(key)) {
                if (_storage.__simpleStorage_meta &&
                    _storage.__simpleStorage_meta.TTL &&
                    _storage.__simpleStorage_meta.TTL.expire &&
                    _storage.__simpleStorage_meta.TTL.expire.hasOwnProperty(key)) {

                    ttl = Math.max(_storage.__simpleStorage_meta.TTL.expire[key] - (+new Date()) || 0, 0);

                    return ttl || false;
                } else {
                    return Infinity;
                }
            }

            return false;
        },

        flush: function() {
            if (!_storage) {
                return false;
            }

            _storage = {};
            try {
                localStorage.removeItem('simpleStorage');
                return true;
            } catch (E) {
                return E;
            }
        },

        index: function() {
            if (!_storage) {
                return false;
            }

            var index = [],
                i;
            for (i in _storage) {
                if (_storage.hasOwnProperty(i) && i != '__simpleStorage_meta') {
                    index.push(i);
                }
            }
            return index;
        },

        storageSize: function() {
            return _storage_size;
        }
    };

}));

/*
 Textbox.io JavaScript Library
 Copyright (c) 2013-2017 Ephox Corp. All rights reserved.
 This software is provided "AS IS," without a warranty of any kind.

 Version: 2.4.0.49
 Client Build: 2.4.0.224
*/
(function(){function Hu(){return{major:"2",minor:"4",full:"2.4.0.49"}}function ZN(){var a=h.none(),b=function(b){a=h.some(b.api)};return{add:function(c){var d=f.curry(b,c);a.isNone()&&b(c);c.api.events.focus.addListener(d)},get:function(){return a},remove:function(b){a.is(b.api)&&(a=h.none())}}}function $N(a,b,c){var d=h.from(a.ui.editorName).map(H.error);c=c.map(H.value);d=d.or(c).getOrThunk(function(){return H.value(aO())});c=Iu.toolbar(a.platform.toolbar,b);b=Iu.mentions(a.mentions,b);return Z.deepMerge(a,
{ui:{editorName:d},platform:{toolbar:c},mentions:b})}function bO(){var a=[];return{get:function(b){return k.find(a,function(a){return a.is(b)})},findAll:function(b){var c=ra.all(b);return k.filter(a,function(a){return k.exists(c,a.is)})},add:function(b){a.push(b)},remove:function(b){k.indexOf(a,b).each(function(b){a.splice(b,1)})}}}function Ju(a,b,c,d,e){return a(c,d)?h.some(c):R.isFunction(e)&&e(c)?h.none():b(c,d,e)}function ob(a,b){k.each(b,function(b,d){if(d>a.length-1){if(b.required())throw Error('missing parameter "'+
b.name()+'"');}else b.validate(a[d])})}function cO(a){var b=Lj.create(),c=W.map(a,function(a){var c=dO.create();b.bind(a,c.eventTrigger);return c});a=W.map(c,function(a){return a.eventRegistry});c=W.map(c,function(a){return a.eventTrigger});return{registry:a,trigger:c,destroy:function(){b.unbindAll()}}}function eO(){var a={};return{getOrSetIndexed:function(b,c){if(void 0!==a[b])return a[b];var d=c();return a[b]=d},waitForLoad:function(){var b=W.values(a);return Nc.par(b)}}}function Ku(){var a=eO(),
b=function(){return a.waitForLoad()};return{preload:function(){b().get(f.identity)},addStylesheet:function(b,d){return a.getOrSetIndexed(b,function(){return fO.download(b,d)})},addScript:function(b,d){return a.getOrSetIndexed(b,function(){return gO.download(b).map(d)})},waitForLoad:b}}function hO(a){var b=function(b,c,g){ob(arguments,[V.func("filter"),V.string("selector"),V.func("callback")]);var l=b(c,Lu(g));a.filter().customFilters().addFilter(l)},c=function(b,c,g){ob(arguments,[V.func("filter"),
V.func("predicate"),V.func("callback")]);var l=b(iO(c),Lu(g));a.filter().customFilters().addFilter(l)};return{selector:{addInput:f.curry(b,xh.inputSelector),addOutput:f.curry(b,xh.outputSelector)},predicate:{addInput:f.curry(c,xh.inputPredicate),addOutput:f.curry(c,xh.outputPredicate)}}}function Zn(a,b,c,d){return{term:function(){return RegExp(a,d.getOr("g"))},prefix:b,suffix:c}}function jO(a,b,c){var d=db.sanitise(a),e=db.sanitise(b);return{patterns:function(){return[RegExp(d+"(.*?)"+e,"gi")]},useCursor:f.always,
generate:function(a,b){return c.generate(a,b.key(),b.value())},derive:function(a){var b=a[1];return c.get(b).map(function(c){return{startIndex:f.constant(a.index),finishIndex:f.constant(a.index+a[0].length),key:f.constant(b),value:f.constant(c)}})}}}function Mu(a,b,c){return jO(a,b,{get:function(a){a=c(a);return h.some(a)},generate:function(a,b,c){a=Od.fromHtml(c,a.dom());return hd.species({elements:f.constant(a),cursor:h.none,dictation:h.none,notify:h.none})}})}function Nu(a,b){var c=function(a){try{return d(a)}catch(b){return h.none()}},
d=function(b){return a(b)?h.from(b.dom().nodeValue):h.none()},e=Y.detect().browser,g=e.isIE()&&10===e.version.major?c:d;return{get:function(c){if(!a(c))throw Error("Can only get "+b+" value of a "+b+" node");return g(c).getOr("")},getOption:g,set:function(c,d){if(!a(c))throw Error("Can only set raw "+b+" value of a "+b+" node");c.dom().nodeValue=d}}}function Sa(){return{up:f.constant({selector:J.ancestor,closest:J.closest,predicate:sa.ancestor,all:y.parents}),down:f.constant({selector:X.descendants,
predicate:pc.descendants}),styles:f.constant({get:t.get,getRaw:t.getRaw,set:t.set,remove:t.remove}),attrs:f.constant({get:m.get,set:m.set,remove:m.remove,copyTo:function(a,b){var c=m.clone(a);m.setAll(b,c)}}),insert:f.constant({before:v.before,after:v.after,afterAll:K.after,append:v.append,appendAll:K.append,prepend:v.prepend,wrap:v.wrap}),remove:f.constant({unwrap:B.unwrap,remove:B.remove}),create:f.constant({nu:q.fromTag,clone:function(a){return q.fromDom(a.dom().cloneNode(!1))},text:q.fromText}),
query:f.constant({comparePosition:function(a,b){return a.dom().compareDocumentPosition(b.dom())},prevSibling:y.prevSibling,nextSibling:y.nextSibling}),property:f.constant({children:y.children,name:A.name,parent:y.parent,isText:A.isText,isComment:A.isComment,isElement:A.isElement,getText:Da.get,setText:Da.set,isBoundary:function(a){return A.isElement(a)?"body"===A.name(a)?!0:k.contains(kO,A.name(a)):!1},isEmptyTag:function(a){return A.isElement(a)?k.contains(["br","img","hr","input"],A.name(a)):!1}}),
eq:L.eq,is:L.is}}function lO(a,b,c){var d=cO({loaded:a.events.loaded,focus:a.events.focus,change:a.events.change,dirty:a.events.dirty,internal:a.events.internal}),e=function(a,b){var c=y.owner(a),d=y.defaultView(a),c=Ea.get(c),e=b();d.dom().scrollTo(c.left(),c.top());return e},g=f.curry($n.getText,a),l=Mj.makeReadOnly({content:{documentElement:f.curry(Oc.documentElement,a),insertHtmlAtCursor:f.curry(Oc.insertHtmlAtCursor,a),uploadImages:f.curry(Oc.uploadImages,a),get:f.curry(Oc.get,a),set:f.curry(Oc.set,
a),isDirty:f.curry(Oc.isDirty,a),setDirty:f.curry(Oc.setDirty,a),selection:{findTagAtCursor:f.curry($n.findTagAtCursor,a),getText:g,getHtml:f.curry($n.getHtml,a)},getSelectedText:g},mode:{set:f.curry(Ou.set,a),get:f.curry(Ou.get,a)},filters:hO(a),macros:{addSimpleMacro:f.curry(Pu.addSimpleMacro,a),removeMacro:f.curry(Pu.removeMacro,a)},message:f.curry(mO.message,a),events:d.registry,element:function(){return a.element().dom()},focus:a.focus,restore:hb.cached(function(){return e(a.element(),function(){nO.blur();
var b=Oc.get(a);d.destroy();a.destroy();return c.restore(b)})})});k.each(b,function(a){ao.hookup(l,a)});return l}function oO(){var a=ZN(),b=bO(),c=function(a){return a.api},d=function(c,d){var l=$N(d.options(),function(){return p.getOrDie("Internal Error")},h.from(m.get(c,"id"))),p=pO.one(c,d.constructors(),l).map(function(d){var g=lO(d.polish,l&&l.plugins?l.plugins:[],{restore:function(c){b.remove(p);a.remove(p);return d.restore(c).dom()}}),p={is:function(a){return L.eq(c,a)},api:g};b.add(p);a.add(p);
return g});return p};return{getOrCreate:function(a,g){return b.get(a).map(c).orThunk(function(){return d(a,g)})},get:function(a){a=b.findAll(a);return k.map(a,c)},getActiveEditor:a.get}}function bo(a,b,c){var d=c||!1,e=function(){b();d=!0},g=function(){a();d=!1};return{on:e,off:g,toggle:function(){(d?g:e)()},isOn:function(){return d}}}function s(a){var b=C.immutable.apply(null,a),c=[];return{bind:function(a){if(void 0===a)throw"Event bind error: undefined handler";c.push(a)},unbind:function(a){c=
k.filter(c,function(b){return b!==a})},trigger:function(){var a=b.apply(null,arguments);k.each(c,function(b){b(a)})}}}function Qu(a,b){var c=function(c){var d=b(c);return 0>=d||null===d?(c=t.get(c,a),parseFloat(c)||0):d},d=function(a,b){return k.foldl(b,function(b,c){var d=t.get(a,c),d=void 0===d?0:parseInt(d,10);return isNaN(d)?b:b+d},0)};return{set:function(b,c){if(!R.isNumber(c)&&!c.match(/^[0-9]+$/))throw a+".set accepts only positive integer values. Value was "+c;var d=b.dom();Pd.isSupported(d)&&
(d.style[a]=c+"px")},get:c,getOuter:c,aggregate:d,max:function(a,b,c){a=d(a,c);return b>a?b-a:0}}}function Pc(a){return k.mapToObject(a,function(){return{bind:f.noop,unbind:f.noop}})}function Ru(a){a=wa(a.dirty);var b=a.get,c=a.set,d=D.create({dirty:s([]),clean:s([])}),e=Qc.tap(function(a){var l=b();a!==l&&(c(a),!0===a?d.trigger.dirty():(e.control.block(),d.trigger.clean(),e.control.unblock()))});return{events:d.registry,isDirty:b,setDirty:e.instance,state:function(){return{dirty:b()}}}}function Su(a,
b,c){var d=[],e=f.curry(b,"input"),g=f.curry(b,"output"),l=f.curry(c,"input"),p=f.curry(c,"output");b=function(a){xh.cata(a,e,g,l,p);d.push(a)};k.each(a.customs,b);return{state:function(){return{customs:d.slice(0)}},addFilter:b}}function co(a){var b={};W.each(a,function(a,d){b[d]=a});return{get:function(){return W.values(b)},add:function(a){var d=La.generate("macro");b[d]=a;return d},remove:function(a){delete b[a]},state:function(){return Z.merge({},b)}}}function qO(){var a=f.noop;return eo({setAutocorrect:f.noop,
setSpelling:f.noop,events:Pc(["analytics"]),init:function(b){b.body().each(function(b){m.set(b,"spellcheck","false");a=function(){m.remove(b,"spellcheck")}})},ui:h.none,check:f.noop,scanView:f.noop,situation:nf.noSituation,hasDropdown:f.constant(!1),hasAutocorrect:f.constant(!1),handle:f.noop,assign:f.noop,lock:f.apply,destroy:function(){a()},state:f.constant({}),lookup:f.constant(h.none()),name:f.constant("spelling-disabled")})}function M(a,b){var c=s(a),d=0,e=function(b){var d=k.map(a,function(a){return b[a]()});
c.trigger.apply(null,d)};return{bind:function(a){c.bind(a);d++;1===d&&b.bind(e)},unbind:function(a){c.unbind(a);d--;0===d&&b.unbind(e)},trigger:f.die("Cannot trigger a source event.")}}function Tu(a,b,c){var d=Aa.state(c),e=h.none(),g=h.none(),l=D.create({loaded:s([]),focus:s([]),dirty:M([],d.laundry.events.dirty),change:s([]),internal:s(["command","data"]),paint:s(["dimensions"])}),p=P.div();ea.add(p,[x.resolve("source-container"),"ephox-textbox-font",x.resolve("layer-editor")]);c=function(){return p};
var r=function(){return g.getOrDie("CodeMirror not added to DOM")},z=function(a){return O()?Uu.beautify(a):a},w=function(a){var b=r().doc;a=z(a);b.setValue(a);b.clearHistory()},G=function(a){var b=q.fromDom(a.getWrapperElement()),c=-1;a.on("changes",function(){var d=a.doc.lineCount();d!=c&&(c=d,d=a.getScrollInfo(),d=Vu({width:qa.get(b),docWidth:d.width,height:d.height,docHeight:d.height}),l.trigger.paint(d))})},k=function(){r().focus()},O=function(){return!0===a.content.prettyprint};return zh.internal({activateInline:Rc.activateInline,
blur:f.die("blur is not supported in the API."),communication:Aa.communication,connect:function(c){if(a.content&&!0===a.content.autoresize){var d=c.dom().clientHeight;ia.set(p,d)}v.append(c,p);b.updateUi(p);Ah.detect(p);c=Nj.use.codemirror()(p.dom(),{mode:"text/html",extraKeys:{"Ctrl-Space":"autocomplete"},lineWrapping:!0});c.on("focus",function(){l.trigger.focus()});g=h.some(c);l.trigger.loaded();G(c);return!0},content:f.constant({body:f.constant(h.none()),doc:f.die("content.doc is not supported in code view"),
execCommand:f.die("content.execCommand is not supported in code view"),getSelectedText:function(){var a=r().getSelection();return void 0!==a&&null!==a&&0<a.length?h.some(a):h.none()},focus:k,isInline:f.constant(!1)}),deactivateInline:Rc.deactivateInline,destroy:function(){B.remove(p);B.empty(p);g=h.none()},element:c,events:l.registry,filter:d.filter,focus:k,fromTransfer:function(a){var b=function(a){w(a);e=h.some(a);g.each(function(a){a.on("changes",function(a,b){d.laundry.setDirty(!0);l.trigger.change()})})};
xe.cata(a,b,b)},fullscreen:Aa.fullscreen,getContent:function(){return r().doc.getValue()},getMode:Ab.code,insertHtmlAtCursor:function(a){r().replaceSelection(a)},isDirty:d.laundry.isDirty,isPrettyPrinting:O,keyboard:Rc.keyboard,macros:d.macros,operations:Rc.operations,setContent:function(a){w(a);d.laundry.setDirty(!1)},setDirty:d.laundry.setDirty,smartview:Rc.smartview,state:function(){return d.getState({content:{}})},toTransfer:function(){var a=r().doc.getValue();return e.filter(function(b){return z(b)===
a}).fold(function(){return xe.changed(a)},xe.unchanged)},ui:Rc.ui,undo:f.constant(Aa.undo()),uploader:f.constant(Aa.uploader()),viewport:c})}function rO(a){var b=P.span();n.add(b,ja.resolve("toolbar-group"));var c=k.map(a,function(a){return a.component()});K.append(b,c);return Wu.group({component:function(){return b},isVisible:function(){return k.exists(c,Bh.isShowing)},refreshUi:function(){k.each(a,function(a){a.refreshUi()})},destroy:function(){k.each(a,function(a){a.destroy()})}})}function fo(a,
b,c){n.add(a,Ta.resolve("button"));var d=sO.disable(a);b=b.getOr(a);var e=null,g=n.toggler(a,Ta.resolve("button-disabled")),l=function(){g.isOn()||f.trigger.click()},p=E.bind(b,"click",l),f=D.create({click:s([])});return{element:function(){return a},name:function(){return c},destroy:function(){p.unbind();d.unbind();B.remove(a)},content:function(b){null!==e&&B.remove(e.element());v.append(a,b.element());e=b},enable:g.off,disable:g.on,disabled:g.isOn,click:l,events:f.registry}}function ac(a){var b=
q.fromTag("span");n.add(b,Ta.resolve("independent-button"));return fo(b,h.none(),a)}function Qd(a){var b=P.span();m.set(b,"aria-hidden",!0);ea.add(b,[Ta.resolve("icon-content")].concat(a));return{element:function(){return b},destroy:function(){B.remove(b)}}}function go(a){var b=P.span();n.add(b,Ta.resolve("statebutton"));var c=fo(P.span(),h.some(b),a);n.add(c.element(),Ta.resolve("dependent-button"));v.append(b,c.element());a=n.toggler(b,Ta.resolve("statebutton-selected"));var d=n.toggler(b,Ta.resolve("statebutton-disabled"));
return{element:function(){return b},destroy:function(){c.destroy();B.remove(b)},name:c.name,content:c.content,enable:function(){d.off();c.enable()},disable:function(){d.on();c.disable()},disabled:function(){return d.isOn()&&c.disabled()},click:c.click,events:c.events,select:a.on,deselect:a.off,selected:a.isOn}}function id(a){var b=go(a),c=D.create({click:s([])}),d=function(){b.selected()?b.deselect():b.select();c.trigger.click()};b.events.click.bind(d);return{element:b.element,name:b.name,content:b.content,
destroy:function(){b.events.click.unbind(d);b.destroy()},enable:b.enable,disable:b.disable,disabled:b.disabled,click:b.click,events:c.registry,select:b.select,deselect:b.deselect,selected:b.selected}}function tO(a){var b=P.span();n.add(b,Ta.resolve("dropbutton"));var c=P.span();n.add(c,Ta.resolve("dropbutton-button"));var d=P.span();n.add(d,Ta.resolve("dropbutton-arrow"));K.append(b,[c,d]);var e=ac(),g=id();g.content(Qd(["ephox-pastry-arrow-icon"]));v.append(c,e.element());v.append(d,g.element());
var l=D.create({click:s([]),open:s([]),close:s([])}),p=function(){l.trigger.click()},f=function(){(g.selected()?l.trigger.open:l.trigger.close)()};e.events.click.bind(p);g.events.click.bind(f);return{element:function(){return b},name:function(){return a},content:e.content,events:l.registry,open:function(){g.select();l.trigger.open()},close:function(){g.deselect();l.trigger.close()},destroy:function(){e.events.click.unbind(p);g.events.click.unbind(f);e.destroy();g.destroy();B.remove(b)},isOpen:function(){return g.selected()}}}
function uO(a){var b=P.span();n.add(b,Ta.resolve("dropbutton"));var c=P.span();n.add(c,Ta.resolve("dropbutton-button"));var d=P.span();n.add(d,Ta.resolve("dropbutton-arrow"));K.append(b,[c,d]);var e=id(),g=id();g.content(Qd(["ephox-pastry-arrow-icon"]));v.append(c,e.element());v.append(d,g.element());var l=D.create({click:s([]),open:s([]),close:s([])}),p=function(){l.trigger.click()},r=function(){(g.selected()?l.trigger.open:l.trigger.close)()};e.events.click.bind(p);g.events.click.bind(r);return{element:f.constant(b),
name:f.constant(a),content:e.content,events:l.registry,open:function(){g.select();l.trigger.open()},close:function(){g.deselect();l.trigger.close()},destroy:function(){e.events.click.unbind(p);g.events.click.unbind(r);e.destroy();g.destroy();B.remove(b)},isOpen:function(){return g.selected()},click:e.click,enable:e.enable,disable:e.disable,disabled:e.disabled,select:e.select,deselect:e.deselect,selected:e.selected}}function qc(a){var b=P.span();n.add(b,Ta.resolve("text-content"));v.append(b,q.fromText(a));
return{element:function(){return b},destroy:function(){B.remove(b)}}}function Oj(a){var b=qd.resolve(a);return{set:function(a){n.add(a,b)},up:function(a){return n.has(a,b)?h.some(a):J.ancestor(a,"."+b)},all:function(a){return X.descendants(a,"."+b)},first:function(a){return J.descendant(a,"."+b)},is:function(a){return n.has(a,b)}}}function Xu(a,b){var c=void 0!==b?b:"info",d=P.div();ea.add(d,[x.resolve("banner"),x.resolve("banner-"+c)]);c=P.div();n.add(c,x.resolve("banner-message"));v.append(c,q.fromText(a));
var e=P.div();n.add(e,x.resolve("banner-buttons"));var g=q.fromTag("span");n.add(g,x.resolve("banner-close"));v.append(e,g);Q.button(g,u.translate("banner.close"));K.append(d,[c,e]);var l=E.bind(g,"click",function(){p.trigger.close()}),p=D.create({close:s([])});return{element:function(){return d},action:function(a){v.prepend(e,a.element())},destroy:function(){l.unbind();B.remove(d)},events:p.registry}}function Ch(a,b){var c=void 0!==b?b:"error",d=a.description(),e=a.link(),g=Xu(d,c),l=h.none(),p=
function(a,b){return E.bind(a,"click",function(a){a.kill();window.open(b.url())})},c=function(a){var b=q.fromTag("a");m.set(b,"href",a.url());m.set(b,"target","_blank");var c=q.fromText(a.text());l=h.some(p(b,a));v.append(b,c);g.action({element:f.constant(b)})};null!==e&&e.url()&&e.text()&&c(e);return{element:g.element,action:g.action,destroy:function(){g.destroy();l.each(function(a){a.unbind()})},events:g.events}}function vO(a,b){var c=D.create({populate:s(["items"])}),d=function(b){var c=a(b.textkey),
c=bb.item(wO,b.name+"-menu-trigger",c);n.add(c,ja.resolve("menu-more"));b=Dh.generate("ephox-chameleon-"+b.name);K.prepend(c,b);Q.menuItem(c,h.none(),h.some(!0));return c}(b),e=function(a){a=q.fromText(a);g(a)},g=function(a){J.child(d,".ephox-gopher-item-text").each(function(b){B.empty(b);v.append(b,a);n.remove(b,Yu)})},l=eg.control(d);return{element:f.constant(d),populate:function(a){c.trigger.populate(a)},select:function(c){J.child(d,Oa.tokenSelector()).each(B.remove);var g=Oa.describe(d,b.describe(c));
v.append(d,g);c=a(c.textkey());e(c)},selectElement:g,disable:l.disable,enable:l.enable,clear:function(){e(a(b.textkey));J.child(d,".ephox-gopher-item-text").each(function(a){n.add(a,Yu)})},destroy:function(){B.remove(d)},events:c.registry}}function xO(a,b){var c=b?a.slice(0):[],d=b,e=b?k.map(W.values(N),function(a){return a()}):[];return{swallowShortcuts:function(a){c=a.slice(0)},yieldShortcuts:function(a){c=k.difference(c,a)},swallowChars:function(){d=!0},yieldChars:function(){d=!1},swallowKeys:function(a){e=
a.slice(0)},yieldKeys:function(a){e=k.difference(e,a)},process:function(a,b){a.fold(function(a){k.contains(c,a)&&b()},function(a){k.contains(e,a)&&b()},function(){d&&b()})}}}function yO(a,b){var c=!1,d=xO(a,b);return{on:function(){c=!0},off:function(){c=!1},isOn:function(){return c},handle:function(a){return function(b){c&&a(b.raw())}},stop:function(b){var c=zO.identify(b,a);d.process(c,f.curry(Zu.stop,b))},swallowShortcuts:d.swallowShortcuts,yieldShortcuts:d.yieldShortcuts,swallowChars:d.swallowChars,
yieldChars:d.yieldChars,swallowKeys:d.swallowKeys,yieldKeys:d.yieldKeys}}function Eh(a){var b=Z.merge({shortcuts:[],swallow:!1},a),c=D.create({character:s(["character"]),space:s(["shift","stopper"]),left:s(["shift","stopper"]),right:s(["shift","stopper"]),backspace:s(["shift","stopper"]),tab:s(["shift","stopper"]),enter:s(["shift","stopper"]),capslock:s(["shift","stopper"]),escape:s(["shift","stopper"]),pageup:s(["shift","stopper"]),pagedown:s(["shift","stopper"]),end:s(["shift","stopper"]),home:s(["shift",
"stopper"]),down:s(["shift","stopper"]),up:s(["shift","stopper"]),insert:s(["shift","stopper"]),del:s(["shift","stopper"]),command:s(["command"])}),d=yO(b.shortcuts,b.swallow),e=function(a){return k.find(b.shortcuts,function(b){return $u.isCommand(a,b)})},g=function(a,b,d){p(a);c.trigger[b].apply(void 0,d)},l=function(a){return function(){d.isOn()&&Zu.stop(a)}};a=d.handle(function(a){e(a).fold(function(){var b=pf.getSpecial(a.keyCode);pf.noModifiers(a)&&null!==b&&(b=b.toLowerCase(),g(a,b,[a.shiftKey,
l(a)]))},function(b){g(a,"command",[b])})});var p=function(a){d.isOn()&&d.stop(a)},f=d.handle(function(a){if(pf.noModifiers(a))pf.isCharCode(a)||null===pf.getSpecial(a.keyCode)?e(a).isSome()?p(a):a.which&&g(a,"character",[String.fromCharCode(a.which)]):p(a);else return!1});return{off:d.off,on:d.on,events:c.registry,swallowShortcuts:d.swallowShortcuts,yieldShortcuts:d.yieldShortcuts,swallowChars:d.swallowChars,yieldChars:d.yieldChars,swallowKeys:d.swallowKeys,yieldKeys:d.yieldKeys,keypress:f,keydown:a}}
function AO(a){var b=D.create({mouseover:s(["item"]),click:s(["item"])}),c=[],d=Eh({shortcuts:a.shortcuts,swallow:a.swallow});a=a.tabs?[N.TAB()]:[];d.swallowKeys([N.UP(),N.DOWN(),N.ENTER(),N.SPACE(),N.ESCAPE()].concat(a));var e=function(){k.each(c,function(a){a.mouseover().unbind();a.click().unbind();a.keydown().unbind()});c=[];d.off()};return{destroy:function(){e()},attach:function(a){e();c=k.map(a,function(a){var c=E.bind(a,"mouseover",function(a){Sb.up(a.target()).each(b.trigger.mouseover)}),e=
E.bind(a,"click",function(a){Sb.up(a.target()).each(b.trigger.click)});a=E.bind(a,"keydown",function(a){d.keydown(a)});return{mouseover:f.constant(c),click:f.constant(e),keydown:f.constant(a)}});d.on()},detach:e,simulateKey:function(a){d.keydown(a)},events:b.registry,keys:d.events}}function BO(a,b){var c=D.create({select:s(["item"]),active:s(["item"]),show:s(["menu"]),hide:s(["menu"]),collapse:s(["menu"]),expand:s(["menu","trigger"]),escaped:s([])}),d=AO({shortcuts:[],swallow:[],tabs:b.tabs}),e={},
g=function(b){Sc.activate(a,b)},l=function(){var b=Sc.inactive(a);k.each(b,function(a){ye.submenu().is(a)&&c.trigger.hide(a)})},p=function(b){Sb.isEnabled(b)&&Sc.submenu(a,e,b).each(function(a){c.trigger.expand(a,b)})},r=function(a){Sc.up(a).each(function(b){g(b);Tc.selectAll(b,[a]);c.trigger.active(a)})},z=function(b){Sc.active(a).bind(function(a){Tc.selected(a).orThunk(function(){return ho.first(a)}).each(function(c){b(a,c)})})},w=function(a){z(function(b,c){Sb.isEnabled(c)&&a(b,c)})},h=function(b){z(function(d,
g){b(d,g).each(function(b){L.eq(g,b)||(Sc.submenu(a,e,g).each(c.trigger.collapse),c.trigger.active(b))})})},I=f.curry(h,ho.prev),O=f.curry(h,ho.next),Va=function(){ye.main().first(a).each(function(a){g(a);Tc.selected(a).each(c.trigger.active)})},m=function(b){T.active().fold(b.stopper(),function(a){!1===k.contains(["input"],A.name(a))&&b.stopper()()});w(function(b,d){Sc.submenu(a,e,d).fold(function(){ye.submenu().is(b)&&(c.trigger.collapse(b),Va())},function(a){c.trigger.expand(a,d);Tc.none(a);Tc.first(a).each(c.trigger.active);
g(a)})})},h=function(a){T.active().fold(a.stopper(),function(b){!1===k.contains(["input"],A.name(b))&&a.stopper()()});w(function(a,b){ye.submenu().is(a)&&(c.trigger.collapse(a),Va())})},$b=function(a){w(function(b,d){var g=Sb.get(d);e.hasOwnProperty(g)&&void 0!==e[g]?m(a):c.trigger.select(d)})},n=function(a){a.shift()?I(a):O(a)};d.keys.up.bind(I);d.keys.down.bind(O);b.tabs&&d.keys.tab.bind(n);d.keys.enter.bind($b);d.keys.space.bind($b);d.keys.escape.bind(function(a){a.stopper()();z(function(a,b){ye.submenu().is(a)?
(c.trigger.collapse(a),Va()):c.trigger.escaped()})});"ltr"===rd.getDirection(a)?(d.keys.left.bind(h),d.keys.right.bind(m)):(d.keys.right.bind(h),d.keys.left.bind(m));d.events.mouseover.bind(function(b){b=b.item();Tc.is(b)&&Sc.submenu(a,e,b).forall(Tb.isVisible)||(r(b),l(),p(b))});d.events.click.bind(function(a){a=a.item();var b=Sb.get(a);e.hasOwnProperty(b)&&void 0!==e[b]||!Sb.isEnabled(a)||c.trigger.select(a)});return{destroy:function(){d.destroy()},populate:function(a){e=Z.merge({},a)},on:function(a){a=
void 0!==a?a:[q.fromDom(document)];d.attach(a)},off:function(){d.detach()},selectFirst:function(){Sc.active(a).each(function(a){Tc.none(a);Tc.first(a).each(c.trigger.active)})},select:function(a){r(a);l()},setActiveMenu:g,simulateKey:d.simulateKey,state:function(){return Sc.active(a).bind(function(a){return Tc.selected(a).map(function(b){return{menu:f.constant(a),item:f.constant(b)}})})},events:c.registry}}function io(a,b,c){var d=P.div();n.add(d,ja.resolve("menu-"+b()+"-sandbox"));m.set(d,"tabindex",
-1);v.append(a(),d);var e=h.none(),g=BO(d,{tabs:!0}),l=wa({}),p=function(){g.off();w();z.trigger.close()},r=function(a){var b=W.mapToArray(l.get(),C.immutable("submenu","item")),c=m.get(a,"data-ephox-gopher-menu-value");return k.find(b,function(a){return a.submenu()===c}).bind(function(b){return J.descendant(d,"[data-ephox-gopher-item-value="+b.item()+"]").map(function(b){return{menu:f.constant(a),item:f.constant(b)}})})};a=function(a){var b=r(a.menu()).map(function(a){return a.item()});z.trigger.collapse(a.menu(),
b)};var z=D.create({open:s([]),close:s([]),itemAction:s(["value","item"]),itemHover:M(["item"],g.events.active),reveal:M(["menu","trigger"],g.events.expand),collapse:s(["menu","item"]),escaped:M([],g.events.escaped)});g.events.hide.bind(a);g.events.collapse.bind(a);g.events.escaped.bind(function(){g.state().fold(p,function(a){r(a.menu()).fold(function(){p()},function(a){z.trigger.collapse(a.menu(),h.some(a.item()));g.select(a.item())})})});g.events.select.bind(function(a){a=a.item();var b=m.get(a,
"data-ephox-gopher-item-value");z.trigger.itemAction(b,a)});g.events.active.bind(function(a){c.focusActive()&&T.focusInside(a.item())});var w=function(){W.each(l.get(),function(a,b){J.descendant(d,'[data-ephox-gopher-menu-value="'+a+'"]').each(function(a){var c=J.descendant(d,'[data-ephox-gopher-item-value="'+b+'"]');z.trigger.collapse(a,c)})})};return{update:function(a){l.set(a);g.populate(a)},setActiveMenu:function(a){e=h.from(a);g.setActiveMenu(a);g.selectFirst()},getActiveMenu:function(){return e},
structure:l,destroy:function(){p();g.destroy()},open:function(a,b){T.focus(b);g.on(a);z.trigger.open()},close:p,events:z.registry,sandbox:f.constant(d),select:g.select,simulateKey:g.simulateKey}}function fg(a,b){return{southeast:f.constant(Ba(-a,b)),southwest:f.constant(Ba(a,b)),northeast:f.constant(Ba(-a,-b)),northwest:f.constant(Ba(a,-b))}}function Pj(){return{northwest:function(){return Ba(-2,1)},northeast:function(){return Ba(0,1)},southwest:function(){return Ba(-2,-2)},southeast:function(){return Ba(0,
-2)}}}function av(a,b,c){c=c||[q.fromDom(document)];var d=[],e=function(c){var d=b();c=c.target();L.member(c,d)||CO.anyAncestor(c,d)||a(c)};return{on:function(){k.each(c,function(a){d.push(E.bind(a,"mousedown",e).unbind)})},off:function(){k.each(d,f.apply)},mousedown:e}}function DO(a){var b=Z.merge({layerClass:null,zindex:null},a),c=P.div();n.add(c,Ta.resolve("panel"));t.setAll(c,{position:"fixed",display:"none",top:"0px"});b.zindex&&t.set(c,"z-index",b.zindex);b.layerClass&&n.add(c,b.layerClass);
a=void 0!==a.container?a.container:J.first("body").getOrDie();v.append(a,c);a=Tb.displayToggler(c,"block");return{element:function(){return c},destroy:function(){B.remove(c)},content:function(a){B.empty(c);v.append(c,a)},show:a.on,hide:a.off,toggle:a.toggle}}function Fh(a,b,c,d,e){var g=[],l=function(){var a=k.map(g,function(a){return a.popup().panel()}).concat(d);k.each(g,function(b){b.popup().setComponents(a)})},p=function(a){var b=Qj.get(g,a,Rj(a)).fold(function(){return z(a)},function(a){return a.popup()});
l();g=Qj.put(g,a,b,bv,Rj(a));return b},r=function(a){Qj.get(g,a,Rj(a)).each(function(a){a.popup().close()});t.set(a,"display","none")},z=function(b){var c=EO.create(a,e);c.setContent(b);g=Qj.put(g,b,c,bv,Rj(b));l();return c};return{initMenu:z,registerMenus:function(a){g=FO.triage(g);return k.map(a,p)},hideAll:function(){k.each(g,function(a){r(a.menu())})},hideMenu:r,expandMenu:function(d,e){var g=p(d),l=eb.descendant(d,"."+Sj.styles())?Rd.expandable():Rd.anchored(),l=rf.submenu(a,b,f.constant(e),
c.uiDirection(),l);t.set(d,"display","block");g.open();l.recalculate(g);return{popup:g,anchorage:l}}}}function cv(a,b){var c=P.span();ea.add(c,[Ta.resolve("righticon-content"),b]);v.append(c,a);return{element:function(){return c},destroy:function(){B.remove(c)}}}function GO(a,b){var c=qc(a),d=cv(c.element(),b);return{element:d.element,destroy:function(){c.destroy();d.destroy()}}}function HO(a,b,c,d,e,g,l,p,r,z){var w=id(c);l=jo.control(w);ea.add(w.element(),[ja.resolve("dropdown-button"),ja.resolve(c+
"-dropdown-button")]);m.set(w.element(),"title",p(d));Q.toolbarButton(w.element(),p(d),!0,r);var h=io(a.uiContainer,f.constant("dropdown-"+c),Gh.menu({focusActive:!0})),I=q.fromDom(document),O=bb.mainmenu(I,c+"-menu",[]),Va=Fh(h.sandbox,a.bounds,b,[w.element()],z).initMenu(O);c=Pj();var ga=rf.main(a.uiContainer,a.bounds,w.element,c,b.uiDirection(),Rd.anchored());Q.menu(O,p(e));var $b=La.generate("menu-control");m.set(O,"id",$b);Q.controls(w.element(),$b);var yh=P.span();n.add(yh,ja.resolve("dropdown-viewport"));
w.content({element:f.constant(yh)});var of=function(a,b){var c=b.map(Dh.generate).getOr([]);K.prepend(a,c)};b=function(){t(p(d))};var t=function(a){B.empty(yh);v.append(yh,GO(a,"ephox-pastry-arrow-righticon").element())},u=function(){w.deselect();r&&ha.pressed(w);ha.collapsed(w.element());Q.disown(w.element());Va.close();h.close()},x=function(){w.select();r&&ha.pressed(w);ha.expanded(w.element());Q.owns(w.element(),$b);Va.open();var b=[a.uiContainer()];h.open(b,w.element())},y=D.create({select:M(["value",
"item"],h.events.itemAction),itemHover:M(["item"],h.events.itemHover),escaped:M([],h.events.escaped),open:s(["menu","popup","anchor"]),close:M([],h.events.close)});w.events.click.bind(function(){(w.selected()?x:u)()});Va.events.close.bind(u);h.events.open.bind(function(){y.trigger.open(O,Va,ga);h.update({});JO()||h.setActiveMenu(O)});h.events.escaped.bind(u);b();return{element:f.constant(w.element()),menu:Va.element,list:f.constant(O),click:w.click,populate:function(a){B.empty(O);k.each(a,function(a){a.label().each(function(a){a=
Tj.generateSeparator(I,a,p);v.append(O,a)});k.each(a.items(),function(a){var b=Tj.generateItem(I,a,g,p);Q.menuItemCheckbox(b);of(b,a.icon());v.append(O,b)})})},enable:l.enable,disable:l.disable,close:u,uiStale:Va.uiStale,recalculate:function(){ga.recalculate(Va)},clear:b,select:function(a){t(p(a.textkey()));of(yh,a.icon())},selectElement:function(a){a=cv(a,"ephox-pastry-arrow-righticon");w.content({element:a.element,destroy:f.noop})},tick:f.curry(gg.tick,O),untick:f.curry(gg.untick,O),activate:f.curry(gg.activate,
w.element()),deactivate:f.curry(gg.deactivate,w.element()),destroy:function(){h.destroy();w.destroy()},events:y.registry}}function KO(a){var b=dv.menu(a);return{widget:b.widget,menu:b.menu,general:b.general,"inline-widget":function(a){return b["inline-widget"]({instance:function(){return a.instance(ko.menu())}})},"split-widget":b["split-widget"],dropdown:b.dropdown,palette:b.palette}}function LO(a,b,c,d,e,g,l){var p=dv.toolbar(g,c,l,sf.editorContainer()),r=function(r){return f.curry(r,p,a,b,c,d,e,
g,l)};return{widget:r(lo.widget),"inline-widget":r(lo.inlineWidget),"split-widget":r(lo.splitWidget),cards:p.cards,general:r(MO.init),dropdown:r(NO.init),menu:r(OO.init),palette:p.palette}}function PO(a){var b=go("overflower");n.add(b.element(),ja.resolve("more-button"));b.content(qc(a("toolbar.more")));var c=ze.onButton(b);m.set(b.element(),"tabindex","-1");a=a("aria.label.more");m.set(b.element(),"title",a);var d=tf.aria(b,a,!0,!0),e=mo.toolbar([{component:b.element,destroy:b.destroy,refreshUi:f.noop}]);
return{group:e.component,select:function(){b.select();d.press()},deselect:function(){d.release();b.deselect()},focus:function(){T.focus(b.element())},destroy:function(){c.unbind();d.destroy();e.destroy()},events:b.events}}function ev(a){var b=[];return{add:function(c,d){var e=d?a.indexOf(b,d).getOrDie():b.length;b.splice(e,0,c);return fv(c,e)},remove:function(c){var d=a.resolve(b,c).getOrDie();return k.indexOf(b,d).bind(function(a){b.splice(a,1);return fv(d,a)})},get:function(){return b.slice(0)},
itemAt:function(c){return a.resolve(b,c)}}}function gv(a){var b=function(a,b){return R.isNumber(b)?c(a,b):R.isString(b)?d(a,b):R.isObject(b)?e(a,b):H.error("Unsupported specifier type: "+b)},c=function(a,b){return 0===a.length?H.error("Items is empty"):0>b||b>=a.length?H.error("Item index: "+b+" is out of range (0-"+(a.length-1)+")"):H.value(a[b])},d=function(b,c){return k.find(b,function(b){return a(b)===c}).fold(function(){return H.error("No item with value: ["+c+"]")},function(a){return H.value(a)})},
e=function(a,b){return k.indexOf(a,b).fold(function(){return H.error("Specified object is not a registered item: "+b)},function(a){return H.value(b)})};return{resolve:b,indexOf:function(a,c){return b(a,c).bind(function(b){return k.indexOf(a,b).fold(function(){return H.error("Item not found: "+c)},function(a){return H.value(a)})})},index:c,value:d,item:e}}function QO(a,b){var c=D.create({grow:s([]),shrink:s([]),exit:s([])}),d=ev(gv(f.identity)),e=q.fromTag("div");t.set(e,"overflow-x","auto");n.add(e,
ja.resolve("toolstrip"));Q.toolbarGroup(e);var g=q.fromTag("div");ea.add(g,[ja.resolve("toolbar"),ja.resolve("toolbar-primary")]);var l=q.fromTag("div");ea.add(l,[ja.resolve("toolbar"),ja.resolve("toolbar-more")]);v.append(e,g);v.append(e,l);var p=function(){var a=d.get();return k.bind(a,function(a){return a.isVisible()?[a.component()]:[]})},r=function(){var b=p(),b=a.build(b);B.empty(g);B.empty(l);K.append(g,b);t.reflow(g)};return{element:function(){return e},main:f.constant(g),mores:f.constant(l),
destroy:function(){var a=d.get();k.each(a,function(a){a.destroy()})},focused:f.never,enter:f.noop,restore:f.noop,refocus:f.noop,refresh:f.curry(hg.preserve,r,e),repaint:f.curry(hg.preserve,r,e),add:function(a){d.add(a)},events:c.registry}}function hv(a){no(a);var b=!1,c=E.bind(a,"transitionend",function(c){"height"===c.raw().propertyName&&(ea.remove(a,[iv,jv]),b&&t.remove(a,"height"))});return{visible:function(){return b},hide:function(){b=!1;oo(a,ia.get(a));n.add(a,jv);no(a)},show:function(){kv(a);
var c=ia.get(a);no(a);n.add(a,iv);kv(a);oo(a,c);b=!0},destroy:function(){c.unbind()}}}function RO(a,b){var c=D.create({grow:s([]),shrink:s([]),exit:s([])}),d=ev(gv(f.identity)),e=q.fromTag("div");Q.toolbarGroup(e);n.add(e,ja.resolve("toolstrip"));var g=q.fromTag("div");ea.add(g,[ja.resolve("toolbar"),ja.resolve("toolbar-primary")]);var l=q.fromTag("div");ea.add(l,[ja.resolve("toolbar"),ja.resolve("toolbar-more")]);v.append(e,g);v.append(e,l);var p=hv(l);b.events.click.bind(function(){p.visible()?
(p.hide(),b.deselect(),c.trigger.shrink()):(p.show(),b.select(),c.trigger.grow())});var r=function(){var a=d.get();return k.bind(a,function(a){return a.isVisible()?[a.component()]:[]})},z=function(){a.update();var c=r(),c=a.build(c);t.set(g,"visibility","hidden");a.clearWidth();B.empty(g);K.append(g,c);var d=a.width(g);v.append(g,b.group());var c=SO.partition(d,c,qa.getOuter,b.group()),d=c.extra(),e=c.within();B.empty(g);B.empty(l);K.append(g,e);K.append(l,d);a.updateWidth(b,c);t.remove(g,"visibility")},
w=function(){hg.preserve(z,e)};return{element:function(){return e},main:f.constant(g),mores:f.constant(l),destroy:function(){var a=d.get();k.each(a,function(a){a.destroy()});b.destroy();p.destroy()},focused:f.never,enter:f.noop,restore:f.noop,refocus:f.noop,refresh:w,repaint:function(){a.shouldRefresh()&&w()},add:function(a){d.add(a)},events:c.registry}}function Ae(a,b,c){var d=E.bind(a,"keydown",c.getOr(function(a){k.find(b,function(b){return b.matches(a)}).each(function(b){b.run(a).each(function(b){a.kill()})})})),
e=D.create({enter:s([]),exit:s([])});return rc({isFocused:f.curry(Ma.isFocused,a),element:f.constant(a),clear:f.noop,enter:function(){Ma.set(a);e.trigger.enter()},refresh:f.noop,isEnabled:f.always,exit:function(){e.trigger.exit()},events:e.registry,destroy:function(){d.unbind()}})}function po(a,b){var c=Ae(a,[U.rule(U.inSet(N.SPACE().concat(N.ENTER())),function(){b.execute()(a);return h.some(!0)})],b.process());return rc({element:c.element,destroy:c.destroy,clear:c.clear,isFocused:c.isFocused,isEnabled:f.always,
refresh:f.noop,enter:c.enter,exit:c.exit})}function Hh(a,b){var c=f.curry(b,-1,0),d=f.curry(b,1,0),e=rd.onDirection(c,d),g=rd.onDirection(d,c),c=f.curry(b,0,-1),d=f.curry(b,0,1),l=function(b){return b(a)()},e=f.curry(l,e),g=f.curry(l,g);return{west:e,east:g,north:c,south:d}}function lv(a,b){var c=E.bind(a,"keydown",function(a){var c=la.locate(b,a);if(U.is(N.ESCAPE()[0])(a))d(),a.kill();else if(void 0!==c&&null!==c&&U.is(N.TAB()[0])(a)){var e=(U.isShift(a)?la.prev:la.next)(b,c);la.progress(c,e);a.kill()}}),
d=function(){la.exit(b);e.trigger.exit()},e=D.create({enter:s([]),exit:s([])});return rc({enter:function(){la.first(b).each(function(a){a.enter()})},exit:d,clear:f.curry(la.clear,b),refresh:f.curry(la.refresh,b),isEnabled:f.curry(la.isEnabled,b),isFocused:f.curry(Ma.isFocused,a),destroy:function(){la.destroy(b);c.unbind()},events:e.registry})}function TO(a,b){var c=PO(b),d=Y.detect().deviceType.isTouch()?QO(a,c):RO(a,c);Q.toolbar(d.main(),b("aria.label.toolbar"));Q.group(d.mores());var e=D.create({drawer:M(["delta",
"total"],d.events.drawer),grow:M([],d.events.grow),shrink:s([]),transitionend:s(["property"]),exit:s([])}),g=E.bind(d.element(),"transitionend",function(a){e.trigger.transitionend(a.raw().propertyName)}),l=h.none(),p=pa.sectors({first:h.none,trigger:ze.trigger}),f=ba.sectors(d.element(),UO,p);f.events.exit.bind(function(){l=h.none();e.trigger.exit()});f.events.active.bind(function(a){l=h.some(a.element())});var z=function(){return f.isFocused()},w=function(){l.each(T.focus)};d.events.shrink.bind(function(){T.search(d.mores()).isSome()&&
c.focus();e.trigger.shrink()});return{element:d.element,refresh:d.refresh,repaint:d.repaint,add:d.add,enter:function(){f.enter()},destroy:function(){f.exit();l=h.none();g.unbind();d.destroy();f.destroy()},focused:z,restore:w,refocus:function(){z()&&w()},events:e.registry}}function VO(){var a=P.div();n.add(a,x.resolve("bannerarium"));n.add(a,x.resolve("layer-above-editor"));var b=h.some([]),c=Lj.create(),d=function(a){b=b.bind(function(b){b=k.filter(b,function(b){return b!==a});return h.some(b)});
e(a)},e=function(a){a.destroy();c.unbind(a.events.close);B.remove(a.element())},g=D.create({close:s(["banner"])});return{element:function(){return a},display:function(e){b.fold(function(){e.destroy()},function(b){b.push(e);c.bind(e.events.close,function(){d(e);g.trigger.close(e)});v.append(a,e.element())})},hide:d,destroy:function(){B.remove(a);b=b.bind(function(a){k.each(a,e);return h.none()})},events:g.registry}}function mv(){var a=VO();a.events.close.bind(function(){b.trigger.resize()});var b=
D.create({resize:s([])});return{element:a.element,display:function(c){a.display(c);b.trigger.resize()},hide:function(c){a.hide(c);b.trigger.resize()},destroy:a.destroy,events:b.registry}}function nv(a,b,c,d){var e=-1;return{shouldRefresh:function(){return qa.getOuter(b)!==e},updateWidth:f.curry(function(a,b,c){e=qa.getOuter(a)},b),updateHeight:f.noop,clearWidth:f.noop,build:f.identity,update:f.noop,width:qa.getOuter,init:function(a,b,c,d,e){a.refresh()},state:f.constant({toolbar:{drag:h.none()}}),
integrate:function(a,b,c,d,e){var f=P.div();ea.add(f,[x.resolve("sticky-container"),x.resolve("layer-above-editor")]);K.append(f,[b.element(),c.element(),e]);K.append(a,[f,d])},destroy:f.noop,events:Pc(["changed"])}}function WO(a){a=Z.merge({layerClass:ov.resolve("blocker")},a);var b=q.fromTag("div");m.set(b,"role","presentation");t.setAll(b,{position:"fixed",left:"0px",top:"0px",width:"100%",height:"100%"});n.add(b,ov.resolve("blocker"));n.add(b,a.layerClass);return{element:function(){return b},
destroy:function(){B.remove(b)}}}function XO(){var a=h.none(),b=function(b,c){var g=a.map(function(a){return b.compare(a,c)});a=h.some(c);return g},c=D.create({move:s(["info"])});return{onEvent:function(a,e){e.extract(a).each(function(a){b(e,a).each(function(a){c.trigger.move(a)})})},reset:function(){a=h.none()},events:c.registry}}function YO(a){return{onEvent:function(a,c){},reset:f.noop}}function ZO(){var a=YO(),b=XO(),c=a;return{on:function(){c.reset();c=b},off:function(){c.reset();c=a},isOn:function(){return c===
b},onEvent:function(a,b){c.onEvent(a,b)},events:b.events}}function $O(a,b){var c=D.create({relocate:s(["x","y","deltaX","deltaY"]),dragend:s([])}),d=aP.both(a),e=Ih.transform(d),g=E.bind(a,"mousedown",function(a){0!==a.raw().button||eb.closest(a.target(),b)||(e.on(),e.go(q.fromDom(document.body)))});e.events.stop.bind(function(){c.trigger.dragend();e.off()});d.events.relocate.bind(function(a){c.trigger.relocate(a.x(),a.y(),a.deltaX(),a.deltaY())});return{blocker:e.element,destroy:function(){g.unbind();
e.destroy()},events:c.registry}}function bP(a){var b=function(a,b,c){a=y.owner(a);a=Ea.get(a);b-=a.left();c-=a.top();return Ba(b,c)},c=function(a,b,c){a=y.owner(a);a=Ea.get(a);b+=a.left();c+=a.top();return Ba(b,c)};return{locationToFixed:b,fixedToLocation:c,absoluteToFixed:function(b,c,g){var l=a();b=y.owner(b);b=Ea.get(b);c-=b.left();return Ba(c,l.top()-g)},locationToAbsolute:function(b,c,g){b=a();return Ba(c,g).translate(-b.left(),-b.top())},absoluteToLocation:function(b,c,g){b=a();return Ba(c,
g).translate(b.left(),b.top())},fromFixed:function(a,b,g){var l=Ba(b,g);a=c(a,b,g);return pv(l,a)},fromLocation:function(a,c,g){a=b(a,c,g);c=Ba(c,g);return pv(a,c)},boundToFixed:function(a,b,c,l,p,f,z){b=c<a.x()?a.fx():c>a.x()+a.w()?a.fx()+a.w():p;a=l<a.y()?a.fy():l+z>a.y()+a.h()?a.fy()+a.h()-z:f;return Ba(b,a)}}}function qv(a,b){var c=a.isInline()?a.element():b;return cP(function(){var a=Ea.get();return{x:a.left,y:a.top,w:f.constant(window.innerWidth),h:f.constant(window.innerHeight),fx:f.constant(0),
fy:f.constant(0)}},function(){return a.isInline()?Ba(0,0):Na.absolute(c)},function(){var a=J.descendant(b,".ephox-chameleon-toolbar").getOr(b);return ia.getOuter(a)},c)}function dP(a,b,c,d){var e=P.div();t.set(e,"position","relative");n.add(e,x.resolve("dragging-container"));var g=P.div();n.add(g,x.resolve("draggable-container"));var l=a.isInline()?b:g;c=qv(a,l);var p=rv.transitions,r=sv.except([".ephox-pastry-button","."+x.resolve("banner-buttons")+" *",".ephox-polish-ui-container",".ephox-polish-socket"]),
z=tv.register(c,l,r,p);bc.initFade(l,g);z.events.appear.bind(f.curry(bc.fadeIn,l,g));z.events.disappear.bind(f.curry(bc.fadeOut,l,g));n.add(g,x.resolve("layer-above-editor"));ea.add(z.blocker(),[x.resolve("above-outside"),x.resolve("dragging")]);var w=P.span();Q.hidden(w,!0);ea.add(w,[x.resolve("toolbar-drag-handle"),x.resolve("general-drag-handle"),"ephox-textbox-font",x.resolve("icon")]);c=Y.detect();c.browser.isIE()&&10===c.browser.version.major&&n.add(w,x.resolve("ie10"));return{shouldRefresh:f.constant(!0),
updateWidth:f.curry(bc.updateWidth,l),updateHeight:f.noop,clearWidth:f.curry(bc.clearWidth,l),build:function(a){return[w].concat(a)},width:f.curry(bc.width,l),init:function(b,c,e,p,f){var r=t.preserve(l,function(){t.set(l,"visibility","hidden");t.remove(l,"display");b.refresh();return ia.get(b.element())});f.toolbar.drag.fold(function(){var b=bc.offsetFrom(a,l,r,p,d);z.init(b.left,b.top,h.from(p.toolbar.offset).getOr({top:-1*r,left:0}))},z.setState);K.append(g,[b.element(),c.element()])},update:function(){z.update()},
state:function(){return{toolbar:{drag:z.state()}}},integrate:function(c,d,l,p,f){v.append(g,d.element());a.isInline()?(K.append(e,[g,f,p]),v.append(b,e)):K.append(b,[g,f,p])},destroy:function(){z.destroy()},events:D.create({changed:M([],z.events.dragend)}).registry}}function eP(a,b,c,d){var e=P.div();t.set(e,"position","relative");n.add(e,x.resolve("floating-container"));var g=P.div();n.add(g,x.resolve("draggable-container"));var l=a.isInline()?b:g;c=qv(a,b);var p=tv.register(c,l,sv.none,rv.transitions);
bc.initFade(l,g);p.events.appear.bind(f.curry(bc.fadeIn,l,g));p.events.disappear.bind(f.curry(bc.fadeOut,l,g));n.add(g,x.resolve("layer-above-editor"));var r=function(){p.update()},z=function(){if(t.getRaw(l,"position").is("fixed")){var b=Na.absolute(a.element());t.getRaw(l,"left").is(b.left())||t.set(l,"left",b.left()+"px")}b=qa.get(a.element());b!==qa.get(g)&&t.set(g,"width",b+"px");return b},w=function(){var b=ia.get(g)+"px";t.getRaw(a.element(),"margin-top").is(b)||t.set(a.element(),"margin-top",
b)};return{shouldRefresh:function(){return!0},updateWidth:function(b,c){a.isInline()?bc.updateWidth(l,b,c):(z(),w(),r())},updateHeight:function(){a.isInline()||w()},clearWidth:function(){a.isInline()&&bc.clearWidth(l)},build:f.identity,width:function(b){return a.isInline()?bc.width(l,b):qa.get(a.element())},init:function(b,c,e,f,r){r=t.preserve(l,function(){t.set(l,"visibility","hidden");t.remove(l,"display");b.refresh();return ia.get(b.element())});e=bc.offsetFrom(a,l,r,f,d);a.isInline()||(z(),w());
f=d?{top:0,left:0}:h.from(f.toolbar.offset).getOr({top:-1*r,left:0});p.init(e.left,e.top,f);K.append(g,[b.element(),c.element()])},update:r,state:f.constant({toolbar:{drag:h.none()}}),integrate:function(c,d,l,p,f){v.append(g,d.element());a.isInline()?(K.append(e,[g,f,p]),v.append(b,e)):K.append(b,[g,f,p])},destroy:function(){p.destroy()},events:D.create({changed:M([],p.events.dragend)}).registry}}function fP(a,b,c,d,e,g){var l=Y.detect(),p=q.fromTag("div"),r=c.uiContainer(),z=P.div();uv(d,p);n.add(p,
x.resolve("editor-main"));!0===e.autoresize&&n.add(p,jd.autoresize());var w=void 0!==e.toolbar&&!0===e.toolbar.sticky,h=Uj.isIE()||Uj.isEdge()&&(16>Uj.version.major||16===Uj.version.major&&"rtl"===rd.getDirection(p));w&&!h&&n.add(p,x.resolve("toolbar-sticky"));w=w&&h;a.isInline()||l.deviceType.isTouch()?v.append(p,r):(uv(d,z),n.add(z,x.resolve("external-ui")),d=ca.body(),v.append(d,z),v.append(z,r));var k=gP.setup(a,p,e,g,w),O=Sd.toolbar(k,u.translate);!1===e.toolbar.visible&&n.add(p,"ephox-polish-no-toolbar");
b.events.toolbar.bind(O.enter);O.events.exit.bind(a.focus);var m=mv(),ga=P.div();n.add(ga,x.resolve("socket"));b=P.div();n.add(b,x.resolve("widget-container"));k.integrate(p,O,m,ga,b);vv.build(p,e.sideBar,e.bottomBar);var $b=a.isInline()?function(){t.set(m.element(),"min-width",qa.getOuter(a.element())+"px")}:f.noop;a.events.paint.bind(function(a){O.repaint();c.refresh();$b()});k.events.changed.bind(function(){O.refresh()});O.events.transitionend.bind(k.updateHeight);return{element:f.constant(p),
socket:function(a){a.connect(ga)},viewport:f.constant(ga),uiContainerParent:f.constant(z),connect:function(a){v.append(a,p);Ah.detect(p);return!0},destroy:function(){O.destroy();k.destroy();m.destroy();B.remove(p);B.remove(z)},toolbar:O,position:function(){$b();k.init(O,m,ga,e,g)},refresh:function(){k.update();O.refresh()},banners:m.element,displayBanner:m.display,hideBanner:m.hide,state:k.state,dialogContainer:f.constant(b)}}function hP(a){var b=h.none(),c=function(b){return function(){for(var c=
[],d=0;d<arguments.length;d++)c[d]=arguments[d];d=a.get()[b];if(null===d||void 0===d)throw"can't find instance method: "+b;return d.apply(null,c)}},d=function(){var a=b.getOrDie("Cannot reconnect the editor before it has been connected the first time.");c("connect")(a)&&e.trigger.connected()},e=D.create({connected:s([])});return zh.external({activateInline:c("activateInline"),deactivateInline:c("deactivateInline"),content:c("content"),keyboard:c("keyboard"),smartview:c("smartview"),macros:c("macros"),
ui:c("ui"),filter:c("filter"),undo:c("undo"),operations:c("operations"),getMode:c("getMode"),element:c("element"),connect:function(a){b=h.some(a);d()},reconnect:function(){var c=b.getOrDie("Cannot reconnect the editor before it has been connected the first time."),e=t.getRaw(c,"height");a.get().events.loaded.bind(function(){e.fold(function(){t.remove(c,"height")},function(a){t.set(c,"height",a)})});var p=t.get(c,"height");t.set(c,"height",p);d()},focus:c("focus"),getContent:c("getContent"),setContent:c("setContent"),
insertHtmlAtCursor:c("insertHtmlAtCursor"),state:c("state"),communication:c("communication"),uploader:c("uploader"),isDirty:c("isDirty"),setDirty:c("setDirty"),isPrettyPrinting:c("isPrettyPrinting"),events:e.registry})}function wv(a){var b=f.compose(H.error,a);return{invalidError:b,different:function(a,d,e,g,l){return 0===d.length?b(g):d===e?b(l):H.value(a)}}}function iP(a){var b=wv(a);return{changeAlt:function(a,d,e,g){var l=m.get(a,"src");return b.different(a,d,l,e,g).map(function(a){m.set(a,"alt",
d);return a})}}}function cc(a,b,c,d){var e=L.eq(a,c)&&b===d;return{startContainer:f.constant(a),startOffset:f.constant(b),endContainer:f.constant(c),endOffset:f.constant(d),collapsed:f.constant(e)}}function xv(a){var b=function(b){return a(b).isRtl()?Vj.rtl:Vj.ltr};return{delta:function(a,d){return b(d).delta(a,d)},edge:function(a){return b(a).edge(a)},positions:function(a,d){return b(d).positions(a,d)}}}function jP(a){var b=wv(a);return{changeCaption:function(a,d,e,g){var l=m.get(a,"summary");return b.different(a,
d,l,e,g).map(function(a){Be.setCaption(a,d);return a})},changeSummary:function(a,d,e,g){var l=J.descendant(a,"caption").map(ua.get).getOr("");return b.different(a,d,l,e,g).map(function(a){Be.setSummary(a,d);return a})},setHeaders:function(a,d,e,g){return(!0===e&&!0===g?Be.addBothHeaders:!0===e?Be.addRowHeader:!0===g?Be.addColumnHeader:f.constant(b.invalidError("a11y.t4b.repair.errors.none")))(a,d)},setScope:function(a,b){m.set(a,"scope",b);return H.value(a)}}}function kP(a){var b=C.immutableBag("id description prompts element severity url repair".split(" "),
[]),c=C.immutableBag(["ui","repair"],[]),d=iP(a),e=jP(a),g={I1:h.some(c({ui:[Bb.field("image-alt","a11y.i1.repair.info",f.constant(""))],repair:function(a,b,c){return d.changeAlt(b.element(),c["image-alt"](),"a11y.i1.repair.errors.empty","a11y.i1.repair.errors.duplicate")}})),I2:h.some(c({ui:[Bb.field("image-alt","a11y.i2.repair.info",function(a){return m.get(a.element(),"alt")})],repair:function(a,b,c){return d.changeAlt(b.element(),c["image-alt"](),"a11y.i2.repair.errors.empty","a11y.i2.repair.errors.duplicate")}})),
T1:h.some(c({ui:[Bb.field("table-caption","a11y.t1.repair.info",f.constant(""))],repair:function(a,b,c){return e.changeCaption(b.element(),c["table-caption"](),"a11y.t1.repair.errors.empty","a11y.t1.repair.errors.duplicate")}})),T2:h.some(c({ui:[Bb.field("table-summary","a11y.t2.repair.info",f.constant(""))],repair:function(a,b,c){return e.changeSummary(b.element(),c["table-summary"](),"a11y.t2.repair.errors.empty","a11y.t2.repair.errors.duplicate")}})),T3:h.some(c({ui:[Bb.field("table-caption","a11y.t3.repair.info",
function(a){return m.get(a.element(),"summary")})],repair:function(a,b,c){return e.changeCaption(b.element(),c["table-caption"](),"a11y.t3.repair.errors.empty","a11y.t3.repair.errors.duplicate")}})),T4A:h.none(),T4B:h.some(c({ui:[Bb.toggles("table-headers","a11y.t4b.repair.info",[Bb.toggle("row-header","ephox-polish-row-header","a11y.t4b.repair.row"),Bb.toggle("col-header","ephox-polish-column-header","a11y.t4b.repair.column")])],repair:function(a,b,c){var d=c["row-header"]();c=c["col-header"]();
return e.setHeaders(a,b.element(),d,c)}})),T4C:h.some(c({ui:[Bb.select("scope-type","a11y.t4c.repair.info",[Bb.pair("row","a11y.t4c.repair.scope.row"),Bb.pair("col","a11y.t4c.repair.scope.col"),Bb.pair("rowgroup","a11y.t4c.repair.scope.rowgroup"),Bb.pair("colgroup","a11y.t4c.repair.scope.colgroup")])],repair:function(a,b,c){return e.setScope(b.element(),c["scope-type"]())}})),D1:h.some(c({ui:[Bb.select("heading-type","a11y.d1.repair.info",[Bb.pair("h1","a11y.d1.repair.block.h1"),Bb.pair("h2","a11y.d1.repair.block.h2"),
Bb.pair("h3","a11y.d1.repair.block.h3"),Bb.pair("h4","a11y.d1.repair.block.h4"),Bb.pair("h5","a11y.d1.repair.block.h5"),Bb.pair("h6","a11y.d1.repair.block.h6")])],repair:function(a,b,c){return yv.convertToHeading(b.element(),c["heading-type"]())}})),D2:h.none(),D3:h.some(c({ui:[],repair:function(a,b,c){return yv.joinLinks(a.content().isRoot,b.element())}})),D4:h.none(),D5:h.none(),O1:h.none(),O2:h.none()};return{convert:function(a,c){var d=function(b){var c=b.repair();return function(b,d){return a.undo().transaction(function(){return c(a,
b,d)})}};return k.map(c,function(a){var c=a.rule(),e=c.id().toUpperCase(),l=void 0!==g[e]?g[e]:h.none(),p=c.desckey();return b({id:e,description:p,prompts:l.map(function(a){return a.ui()}),element:q.fromDom(a.element()),url:c.url(),severity:c.severity(),repair:l.map(d).getOr(function(){return H.error("no repair")})})})}}}function lP(a){return{audit:function(b){return mP.audit(b,a.rules())}}}function nP(){var a=wa([]),b=wa(0),c=C.immutable("index","total","detail"),d=function(d){var g=a.get();if(0===
g.length)return h.none();var l,p=b.get();l=g.length-1;d=p+d;l=d>l?0:0>d?l:d;b.set(l);return h.some(c(l,g.length,g[l]))};return{load:function(d){if(0===d.length)return h.none();b.set(0);a.set(d);return h.some(c(b.get(),d.length,d[0]))},backwards:function(){return d(-1)},current:function(){var d=a.get();return 0===d.length?h.none():h.some(c(b.get(),d.length,d[b.get()]))},resolve:function(){var d=a.get();if(0===d.length)return h.none();var d=d.slice(0,b.get()).concat(d.slice(b.get()+1)),g=Math.min(d.length-
1,b.get());b.set(g);a.set(d);return 0===d.length?h.none():h.some(c(b.get(),d.length,d[b.get()]))},forwards:function(){return d(1)}}}function oP(a,b){var c=P.div();n.add(c,Ha.resolve(b,"assistance"));var d=Ha.resolve(b,"assistance-invalid"),e=h.none(),g=q.fromTag("p");n.add(g,Ha.resolve(b,"repair-error"));m.set(g,"aria-live","polite");var l=function(){n.remove(c,d);B.remove(g)},p=D.create({execute:s([])}),r=ba.dynamic(c,pa.dynamic({})),z=[r];return{element:f.constant(c),keylayouts:f.constant(z),markInvalid:function(a){n.add(c,
d);Oa.shout(c,a);ua.set(g,a);v.append(c,g)},markValid:l,getChange:function(){return e.map(function(a){return a.get()})},destroy:function(){e.each(function(a){a.destroy()})},update:function(d){l();d=d.detail();var g=d.prompts();e.each(function(a){a.destroy()});d=pP.generate(a,g,d,b);e=h.some(d);B.empty(c);K.prepend(c,d.elements());d.events.execute.bind(function(){p.trigger.execute()});r.set(d.keylayouts())},events:p.registry}}function qP(a,b){var c=P.div();n.add(c,Ha.resolve(b,"commands"));var d=P.div();
n.add(d,Ha.resolve(b,"commands-buttons"));var e=Td.defaultActionButton(a,[Ha.resolve(b,"repair")],dc.repairLabel(),dc.repairText(),b),g=Td.normalButton(a,[Ha.resolve(b,"ignore")],dc.ignoreLabel(),dc.ignoreText(),b),l=Td.defaultActionButton(a,[Ha.resolve(b,"ok")],dc.okLabel(),dc.okText(),b),p=function(){K.append(d,[g.element(),e.element()])};p();var r=pa.button({execute:function(a){a.dom().click()}}),r=[ba.button(g.element(),r),ba.button(e.element(),r),ba.button(l.element(),r)];v.append(c,d);var z=
D.create({repair:M([],e.events.click),ignore:M([],g.events.click),done:M([],l.events.click)});return{element:f.constant(c),destroy:function(){e.destroy();g.destroy();l.destroy()},keylayouts:f.constant(r),focus:function(){var a=e.disabled()?e.element():g.element();T.focus(a)},update:function(a){B.remove(l.element());a=a.detail();Tb.isVisible(g.element())||p();a.prompts().fold(function(){e.disable();ha.disable(e.element())},function(a){e.enable();ha.enable(e.element());Oa.speak(ca.body(),Ha.translate(b,
dc.repairAvailable()))})},zero:function(a){B.empty(d);v.append(d,l.element());T.focus(l.element())},events:z.registry}}function rP(a,b){var c=P.div();n.add(c,Ha.resolve(b,"information"));m.set(c,"data-severity","info");var d=P.div();n.add(d,Ha.resolve(b,"icon"));var e=P.div();n.add(e,Ha.resolve(b,"description"));v.append(e,q.fromText(""));var g=q.fromTag("a");n.add(g,Ha.resolve(b,"help-icon"));m.set(g,"href","javascript:void(0)");K.append(c,[d,e,g]);var l=E.bind(g,"click",function(){var a=m.get(g,
"data-url");window.open(a,"","height=700,width=820,resizable=yes,scrollbars=yes")}),d=[ba.single(g,[],h.none())];return{element:f.constant(c),destroy:function(){l.unbind()},keylayouts:f.constant(d),update:function(a){a=a.detail();var d=a.severity();m.set(c,"data-severity",d);Oa.speak(ca.body(),Ha.translate(b,dc.severity(d)));d=Ha.translate(b,a.description());ua.set(e,d);Oa.speak(ca.body(),d);m.set(g,"title",Ha.translate(b,dc.issueHelp()));m.set(g,"data-url",a.url());v.after(e,g)},zero:function(){m.set(c,
"data-severity","success");var a=Ha.translate(b,dc.noIssue());ua.set(e,a);Oa.speak(ca.body(),a);B.remove(g)}}}function sP(a,b){var c=P.div(),d=P.span();n.add(d,Ha.resolve(b,"spin"));n.add(c,Ha.resolve(b,"loading"));v.append(c,d);return{element:f.constant(c),destroy:f.noop}}function tP(a,b){var c=Wj("titlebar",h.none(),b),d=Wj("title",h.none(),b),e=Wj("title-text",h.some(dc.title()),b),g=Wj("issue",h.some(dc.running()),b);K.append(d,[e,g]);var l=P.div();n.add(l,Ha.resolve(b,"navigation"));var e=Td.icon(a,
Ha.resolve(b,"previous"),dc.previous(),b),p=Td.icon(a,Ha.resolve(b,"next"),dc.next(),b),r=[e,p],e=D.create({previous:M([],e.events.click),next:M([],p.events.click)});K.append(l,k.map(r,function(a){return a.element()}));v.append(c,d);var z=pa.button({execute:function(a){a.dom().click()}}),p=k.map(r,function(a){return ba.button(a.element(),z)}),w=function(a){a.disable();ha.disable(a.element())},G=function(a){a.enable();ha.enable(a.element())},I=function(){k.each(r,w)},O=function(){k.each(r,G)},m=function(){hg.preserve(function(){v.after(d,
l)},l)};return{element:f.constant(c),update:function(a){var c=a.index()+1;a=a.total();c=Ha.translate(b,dc.issueCounter(),c,a);ua.set(g,c);Oa.speak(ca.body(),c);(1===a?I:O)();m()},zero:function(){ua.set(g,"&nbsp;");m();I()},keylayouts:f.constant(p),destroy:function(){k.each(r,function(a){a.destroy()})},events:e.registry}}function uP(a,b){var c=P.div();n.add(c,Ha.resolve(b,"widget"));var d=Ha.resolve(b,"loading");n.add(c,d);var e=tP(a,b),g=rP(a,b),l=oP(a,b),p=qP(a,b),r=sP(a,b);K.append(c,[e.element(),
r.element()]);var z=D.create({showcase:s(["detail"]),ignored:s(["detail"]),solved:s(["detail"]),done:s([])}),h=function(a){k.each([e,g,l,p],function(b){b.update(a)});z.trigger.showcase(a.detail());hg.preserve(function(){v.after(e.element(),g.element());v.after(g.element(),l.element());v.after(l.element(),p.element())},c)},G=function(a){a.fold(function(){v.after(e.element(),g.element());v.after(g.element(),p.element());B.remove(l.element());e.zero();g.zero();p.zero()},h)},I=wa(nP());e.events.previous.bind(function(){I.get().backwards().each(h)});
e.events.next.bind(function(){I.get().forwards().each(h)});var O=function(){var a=l.getChange().getOr({});I.get().current().each(function(b){var c=b.detail();c.repair()(c,a).fold(function(a){l.markInvalid(a)},function(a){l.markValid();a=I.get().resolve();z.trigger.solved(c);G(a);p.focus()})})};l.events.execute.bind(O);p.events.repair.bind(O);p.events.ignore.bind(function(){I.get().current().each(function(a){var b=I.get().resolve();z.trigger.ignored(a.detail());G(b)})});p.events.done.bind(function(){z.trigger.done()});
O=k.flatten([e.keylayouts(),g.keylayouts(),l.keylayouts(),p.keylayouts()]);return{element:f.constant(c),destroy:function(){k.each([e,g,l,p],function(a){a.destroy()})},load:function(a){B.remove(r.element());n.remove(c,d);a=I.get().load(a);G(a)},open:function(){v.append(c,r.element());n.add(c,d)},skip:function(){var a=I.get().resolve();G(a)},keylayouts:f.constant(O),events:z.registry}}function vP(a){var b=qo.ignoreAttr(),c={prefix:f.constant("accessibility-check"),translate:zv.translate,tnames:f.constant("a11y")},
d=uP(wP,c),c=xP.all(),c=yP({rules:c}),e=lP(c),g=kP(zv.translate),l=qo.manager(a.content()),p=function(c){c=e.audit(c.dom());c=g.convert(a,c);return k.filter(c,function(a){var c=a.element(),c=Ud.read(c,b);return!k.contains(c,a.id())})};d.events.showcase.bind(function(b){a.content().body().each(function(c){c=b.detail().element();ca.inBody(c)?(l.moveTo(c,b.detail().severity()),uf.elementIntoView(a.content(),c)):d.skip()})});d.events.ignored.bind(function(a){a=a.detail();l.ignore(a.element(),a.id(),a.severity())});
d.events.solved.bind(function(a){a=a.detail();l.solve(a.element(),a.id(),a.severity())});c=D.create({done:M([],d.events.done)});return{element:d.element,anchorName:f.constant("a11y-widget-anchor"),textkey:f.constant("action.a11ycheck"),dialogTextkey:f.constant("a11y.widget.title"),destroy:function(){d.destroy()},keylayouts:d.keylayouts,openWidget:function(){a.content().hideInlineSelection();d.open();a.content().body().each(function(a){a=p(a);l.start(a);d.load(a)})},closeWidget:function(){l.stop()},
events:c.registry}}function zP(a,b){var c=D.create({open:s([]),close:s([])}),d=q.fromTag("div");n.add(d,ja.resolve("layer-menus"));n.add(d,ja.resolve("toolstrip"));v.append(a(),d);var e=function(){t.getRaw(d,"opacity").isNone()&&(g.cancel(),t.remove(d,"display"),c.trigger.close())},g=zc.last(e,1E3),l=E.bind(d,"transitionend",e);return{element:function(){return d},destroy:function(){l.unbind();B.remove(d)},open:function(){t.set(d,"display","block");t.reflow(d);c.trigger.open();t.set(d,"opacity","1")},
close:function(){t.remove(d,"opacity");g.throttle()},setContent:function(a){v.append(d,a)},events:c.registry}}function Av(a,b,c,d){var e=a.content(),g=a.ui().tabstrip.element(),l=[b.element()],p="."+x.resolve(c)+", ."+ja.resolve("more-button"),f=function(){var a=X.descendants(g,p);return l.concat(a)},z=sc.api(),h=D.create({abort:s([]),open:s([]),close:s([])}),k=function(){e.events.mousedown.unbind(I);z.clear();O.off()},I=function(){z.isSet()&&(b.closeWidget(),z.run(function(a){a.close()}))},O=av(I,
f);b.events.done.bind(function(){I()});var m=ba.multiple(b.element(),b.keylayouts());m.events.exit.bind(function(){h.trigger.abort()});return{destroy:function(){k();m.destroy();b.destroy();O.off()},events:h.registry,open:function(){O.on();b.openWidget();var c=zP(a.ui().dialogContainer,f);c.setContent(b.element());n.add(c.element(),d);var g=c.element();ea.add(g,[x.resolve(b.anchorName()),x.resolve("side-widget-anchor")]);Q.dialog(g,u.translate(b.dialogTextkey()));c.open();c.events.close.bind(function(){e.focus();
h.trigger.close();k()});z.set(c);m.enter();h.trigger.open();e.events.mousedown.bind(I)},close:I,isOpen:z.isSet}}function AP(a){var b=a.filter();qo.configure(b);var c=sc.api(),d=function(){var b;b=vP(a);b=Av(a,b,ro,BP);b.events.abort.bind(b.close);b.events.close.bind(g);c.set(b);b.open()},e=function(){c.run(function(a){a.close()})},g=function(){c.clear()};return{ui:$a.ui({"a11ycheck.general":{toggle:!1,hasPopup:!1,value:ro,textkey:u.key("action.a11ycheck"),action:function(){(c.isSet()?e:d)()},updateGeneral:function(a){n.add(a.element(),
x.resolve(ro))},icon:!0,tooltip:Wa.get}}),destroy:g}}function so(a){return{change:function(b,c){return(a.directionAt(b).isRtl()?Bv.rtl:Bv.ltr).change(b,c)}}}function CP(a){var b=a.property(),c=function(a,c){return b.parent(a).map(b.name).map(function(a){return!k.contains(c,a)}).getOr(!1)};return{validateText:function(a){return b.isText(a)&&c(a,DP)}}}function EP(a,b){return{is:function(c){return t.get(c,a)===b},remove:function(b){t.remove(b,a)},set:function(c){t.set(c,a,b)}}}function FP(a){var b=a.content(),
c=a.keyboard(),d=a.shortcuts(),e=a.context(),g=D.create({analytics:s(["message","properties"])}),l=f.curry(kd.indent,a),p=f.curry(kd.outdent,a),r=f.curry(kd.blockquote,a);d.events.indent.bind(l);d.events.outdent.bind(p);d.events.blockquote.bind(r);var z=function(a){var c=a.stopper();Xj.context(b).bind(function(b){Xj.cata(b,f.noop,function(b){(a.shift()?p:l)();c()})})},w=c.editor();w.on();w.events.tab.bind(z);e.events.preLinebreak.bind(function(b){to.atQuoteBreak(a.content()).each(function(c){kd.breakQuote(a,
c.quote(),c.element());b.handler().handle()})});var k=C.immutable("value","icon","textkey"),c=k("justifyleft",h.some(ja.resolve("justifyleft")),u.key("alignment.left")),d=k("justifycenter",h.some(ja.resolve("justifycenter")),u.key("alignment.center")),e=k("justifyright",h.some(ja.resolve("justifyright")),u.key("alignment.right")),I=k("justifyfull",h.some(ja.resolve("justifyfull")),u.key("alignment.justify")),O=k("justifyleft",h.some(ja.resolve("justifyleft")),u.key("alignment.left")),m=k("justifycenter",
h.some(ja.resolve("justifycenter")),u.key("alignment.center")),k=k("justifyright",h.some(ja.resolve("justifyright")),u.key("alignment.right")),ga=[c,d,e,I],$b=[O,m,k],n=function(){return b.getSelection().bind(ib.detect).fold(function(){return ga},function(a){return $b})},q={justifyleft:f.curry(kd.imageLeft),justifycenter:f.curry(kd.imageCentre),justifyright:f.curry(kd.imageRight)},t={justifyleft:kd.justifyLeft,justifycenter:kd.justifyCentre,justifyright:kd.justifyRight,justifyfull:kd.justifyFull},
v=function(a){return a.toLowerCase()},x=h.none();return{ui:$a.ui({"indent.general":{value:"indent",textkey:u.key("action.indent"),action:l,updateGeneral:f.identity,tooltip:Wa.get,toggle:!1},"outdent.general":{value:"outdent",textkey:u.key("action.outdent"),action:p,updateGeneral:f.identity,tooltip:Wa.get,toggle:!1},"blockquote.general":{value:"blockquote",textkey:u.key("action.blockquote"),action:r,updateGeneral:function(a){to.read(b).quoted().fold(a.deselect,f.noop,a.select)},tooltip:Wa.get,toggle:!0},
"alignment.dropdown":{name:"alignment",textkey:u.key("alignment.toolbar"),menuTextkey:u.key("alignment.menu"),action:function(c){var d=b.getSelection().bind(ib.detect).fold(f.constant(t),f.constant(q));(void 0!==d[c]?d[c]:f.noop)(a)},updateDropdown:function(a){var c=n(),d=to.read(b).alignment().fold(function(){return 0<c.length?oa.on(c[0].value()):oa.off()},oa.unchanged,oa.on);a.populate([{label:h.none,items:f.constant(c)}]);Yj.updateMenu(a,d.map(v),oa.on(),c,x)},values:ga,render:x,tooltip:Wa.get,
toggle:!1,describe:function(a){return u.translate("alignment.describe",u.get(a.textkey()))}}}),events:g.registry,destroy:function(){w.off();w.events.tab.unbind(z);w.destroy()}}}function GP(a,b){var c=function(a,b,c){var d=q.fromText(b);a(c,d);return h.some({element:f.constant(d),cleanup:function(){B.remove(d)},splits:f.constant([])})},d=function(a){return{none:h.none,start:f.curry(c,v.before,a),end:f.curry(c,v.after,a),middle:function(b,d,e){return c(v.after,a,d)}}},e=function(a){var b=function(a){return function(b,
c){v.appendAt(b,c,a)}};return{prepend:f.curry(c,v.prepend,a),before:f.curry(c,v.before,a),after:f.curry(c,v.after,a),insertAt:function(d,e){return c(b(e),a,d)},append:f.curry(c,v.append,a)}};return{before:{text:d(a),element:{hasPrevious:e(a).insertAt,first:e(a).prepend,empty:e(a).before,otherwise:e(a).append}},after:{text:d(b),element:{hasNext:e(b).insertAt,rightEdge:e(b).append,empty:e(b).after,otherwise:e(b).append}},general:{insertBefore:e(a).before,insertAfter:e(b).after}}}function Cv(a,b,c,d){c=
[b].concat(c);var e=f.curry(uo.nuTag,b),g=f.curry(vf.relatedTags,c),l=Ub.tagList(c),p=vf.queriesTag(b),r={nu:e,matches:g,purifiers:f.constant(l)},z=function(a){var b=q.fromText(Ua.zeroWidth());Zj.atStartOf(a.startContainer(),a.startOffset(),b);return sb.hare(b,0,b,1)},w=function(a,b,c){return A.isText(b)&&c===Da.get(b).length?Lb.after(b,a).fold(function(){return S.point(b,c)},function(a){return S.point(a,0)}):S.point(b,c)},G=function(a,b,c){return A.isText(b)&&0===c?Lb.before(b,a).fold(function(){return S.point(b,
c)},function(a){return S.point(a,ka.getEnd(a))}):S.point(b,c)},I=function(a,b){return b.map(function(b){var c=w(a,b.start(),b.soffset());b=G(a,b.finish(),b.foffset());return sb.fussy(c.element(),c.offset(),b.element(),b.offset())})};c={nu:e,purifiers:f.constant(l),isStyled:f.constant(!0)};var O=["font-size"],m=function(a){return a.getSelection().map(function(a){var c=W.keys(d),c=k.difference(c,O);return k.forall(c,function(c){return vf.queryStyleOn(c,d[c],[b],a)})}).getOr(!1)};return{run:function(a,
c){var d=c.doc();return vo.detect(c,p.queryOn).fold(function(){return h.none()},function(b){b=z(b);Vd.doRemoveUpto(a,l,b,g);return h.some(sb.fromHare(b))},function(a){a=z(a);var c=q.fromTag(b,d.dom());n.add(c,jb.styleSpan());v.wrap(a.startContainer(),c);return h.some(sb.fussy(a.startContainer(),1,a.startContainer(),1))},function(b){b=Vd.doRemoveUpto(a,l,b,g);return I(a,b)},function(b){return wo.run(c.doc(),a,b,r)})},query:function(a){return p.query(a)||m(a)},deferred:f.constant(c),name:f.constant(a)}}
function Dv(a,b){return{element:f.constant(b),wrap:function(c){a.insert().append(b,c)}}}function ak(a,b,c,d,e,g){var l=f.curry(uo.nu,d,e,b),p=f.curry(vf.relatedStyle,d,e,c),r=Ub.cssAndTag(c,d,e),z=vf.queriesStyle(d,e,c),w=f.constant(!0),k=function(b,c){Wd.generic(b,f.noop,w,function(b){n.add(b,jb.styleSpan());t.set(b,d,c);m.set(b,jb.overrideSpan(),a);t.reflow(b)});return h.none()};b={nu:l,purifiers:f.constant(r),isStyled:function(a){return"span"===A.name(a)&&Jh.rawState(a,d,e)},matches:p};return{run:function(a,
b){return vo.detect(b,z.queryOn).fold(function(){return h.none()},function(a){return k(b,g)},function(a){return k(b,e)},function(b){return Vd.doRemoveUpto(a,r,b,p)},function(c){return wo.run(b.doc(),a,c,{nu:l,matches:p,purifiers:f.constant(r)})})},query:function(b){var c="["+jb.overrideSpan()+'="'+a+'"]';return b.getSelection().bind(function(a){return J.closest(a.startContainer(),c).bind(function(b){return J.closest(a.endContainer(),c).bind(function(a){return L.eq(b,a)?t.getRaw(b,d).map(function(a){return a===
e}):h.none()})})}).getOrThunk(function(){return z.query(b)})},deferred:f.constant(b),name:f.constant(a)}}function HP(a){var b=a.content(),c=a.shortcuts(),d=a.undo(),e=D.create({analytics:s(["message","properties"])}),g=function(a,c){var d=IP.read(b);(void 0!==d[a]?d[a]():oa.unchanged()).fold(c.deselect,f.noop,c.select)},l=function(c){var e=function(a){return c(b.isRoot,a)};return function(){tb.cata(a.selections().get(),f.noop,bk.multiple(b,d,e),bk.single(b,d,e));b.uiChanged()}},p=l(Ac.bold),r=l(Ac.italic),
z=l(Ac.underline),h=l(Ac.strikethrough),k=l(Ac.subscript),I=l(Ac.superscript),l=l(Ac.removeformat);c.events.bold.bind(p);c.events.italic.bind(r);c.events.underline.bind(z);return{ui:$a.ui({"bold.general":{toggle:!0,value:"bold",textkey:u.key("action.bold"),action:p,updateGeneral:f.curry(g,"bold"),icon:!0,tooltip:Wa.get},"italic.general":{toggle:!0,value:"italic",textkey:u.key("action.italic"),action:r,updateGeneral:f.curry(g,"italic"),icon:!0,tooltip:Wa.get},"underline.general":{toggle:!0,value:"underline",
textkey:u.key("action.underline"),action:z,updateGeneral:f.curry(g,"underline"),icon:!0,tooltip:Wa.get},"strikethrough.general":{toggle:!0,value:"strikethrough",textkey:u.key("action.strikethrough"),action:h,updateGeneral:f.curry(g,"strikethrough"),icon:!0,tooltip:Wa.get},"removeformat.general":{toggle:!1,value:"removeformat",textkey:u.key("action.removeformat"),action:l,updateGeneral:f.identity,icon:!0,tooltip:Wa.get},"subscript.general":{toggle:!0,value:"subscript",textkey:u.key("action.subscript"),
action:k,updateGeneral:f.curry(g,"subscript"),icon:!0,tooltip:Wa.get},"superscript.general":{toggle:!0,value:"superscript",textkey:u.key("action.superscript"),action:I,updateGeneral:f.curry(g,"superscript"),icon:!0,tooltip:Wa.get}}),events:e.registry,destroy:f.noop}}function Ev(a){var b=function(b,d,e,g){var l=e.join(",");b=a.up().closest(b,l,g);d=a.up().closest(d,l,g);return b.isSome()&&d.isSome()?(g=b.getOrDie("Just checked it was some."),d=d.getOrDie("Just checked it was some."),a.eq(g,d)?k.find(e,
f.curry(a.is,g)):h.none()):h.none()};return{query:function(a,d,e){d=d.concat(JP.tags());return b(a.startContainer(),a.endContainer(),d,e)},queryAll:b}}function KP(a){var b=C.immutable("base","value","textkey","icon","styles","tag");a=k.map(a.styles,function(a){return{name:a.rule,text:h.from(a.text).map(u.raw),style:void 0===a.style?{}:a.style}});var c={};k.each(a,function(a){LP.decipher(a).each(function(b){c[a.name]=b})});a=f.constant(!0);var d=f.constant(!1),e=function(a,b,c,d){return function(e){return Ce.cata(e,
a,b,c,d)}},g=W.values(c),l=k.filter(g,e(d,d,d,a)),p=k.filter(g,e(d,d,a,d)),r=k.filter(g,e(d,a,d,d)),z=k.filter(g,e(a,d,d,d));a=function(){var a=l.concat(r).concat(p).concat(z);return k.map(a,function(a){return Ce.cata(a,function(a,b,c){return"."+a},function(a,b,c,d){return a+"."+b},function(a,b,c){return a},function(a,b,c,d){return a+"."+b})})}();var w=function(a){return void 0!==c[a]?h.some(c[a]):h.none()},G=function(a){return Ce.cata(a,function(a,c,d){return b("span","."+a,c,h.none(),d,"span")},
function(a,c,d,e){return b(a,a+"."+c,d,h.none(),e,"span")},function(a,c,d){return b(a,a,c,h.none(),d,a)},function(a,c,d,e){return b(a,a+"."+c,d,h.none(),e,a)})},d=function(){var a=W.values(c);return k.map(a,G)}();return{candidateByName:function(a){return w(a).map(G)},toCandidate:G,ruleByName:w,divisions:function(){return{classblocks:f.constant(l),blocks:f.constant(p),objects:f.constant(r),inlines:f.constant(z)}},backup:function(a){return b(a,a,u.key("block."+a),h.none(),{},"span")},selectors:f.constant(a),
everything:f.constant(d)}}function MP(a,b,c){var d=c(void 0===b||"default"===b?h.none():h.some(b)),e=q.fromTag("div");n.add(e,x.resolve("list-style-option"));c=void 0===b||"default"===b?NP(e,"ul"===a?"list.unordered.default":"list.ordered.default"):OP(a,b);v.append(e,c);a=u.translate(("ul"===a?"list.unordered.":"list.ordered.")+b);m.set(e,"title",a);Q.label(e,a);Q.menuItemCheckbox(e);var g=Oa.describe(e,u.translate("aria.widget.navigation"));v.append(e,g);var l=D.create({action:s(["type","element"])}),
p=function(){ha.checked(e,!0);n.add(e,x.resolve("list-style-selected"))},r=function(){ha.checked(e,!1);n.remove(e,x.resolve("list-style-selected"))},z=E.bind(e,"click",function(a){l.trigger.action(d,h.some(e))});return{element:f.constant(e),instance:d,style:b,update:function(a){(d.query(a)&&d.queryStyle(a)?oa.on(!0):oa.off()).fold(r,f.noop,p)},select:p,deselect:r,events:l.registry,destroy:function(){z.unbind();B.remove(e);B.remove(g)}}}function PP(a,b,c){var d=D.create({action:s(["type"])}),e=P.div();
n.add(e,x.resolve("list-style-palette"));var g=k.map(a,function(a){return MP("bullist"===b?"ul":"ol",a,c)});k.each(g,function(a){a.events.action.bind(function(a){a.element().each(r);d.trigger.action(a.type())});v.append(e,a.element())});var l=function(){var a=Math.floor(g.length/4)+(0<g.length%4?1:0);return ba.grid(e,{cell:f.constant("."+x.resolve("list-style-option"))},a,4,pa.grid({trigger:function(a){a.dom().click();return h.some(a)}}))}(),p=function(a){return k.find(g,a)},r=function(a){J.sibling(a,
"["+ck+'="default"]').filter(function(b){return b!==a}).bind(function(b){m.remove(b,ck);m.set(a,ck,"default")})};return{enter:l.enter,element:f.constant(e),update:function(a){k.each(g,function(b){b.update(a)})},events:d.registry,destroy:function(a){l.isFocused()&&l.exit();l.destroy();k.each(g,function(a){a.destroy()});B.remove(e)},findDefault:function(){return p(function(a){return m.has(a.element(),ck)}).map(function(a){return a.instance})}}}function Fv(a){var b=a.region(),c=a.content(),d=a.undo(),
e=a.keyboard(),g=a.listConstructor(),l=a.type(),p=a.styles(),r=a.textkey(),z=a.menuTextkey(),h=g("default"),k=Y.detect(),I=e.all();I.swallowKeys([N.ESCAPE()]);var O=D.create({abort:s([]),done:s([]),close:s([]),analytics:s(["message"])});I.events.escape.bind(function(a){O.trigger.abort()});var m=function(e,g){var l=ig.list(function(a,c,d,l,f){return e.apply(a,c,d,l,f,so(b),g)},b,c,d,a.selections());O.trigger.done();c.focus();return l},ga=function(){J.descendant(n.element(),'[data-ephox-polish-list-style="default"]').each(function(a){T.focus(a)})},
n=PP(p,l,g);n.events.action.bind(function(a){m(a.type(),!1)});return{action:function(){var a=n.findDefault().getOr(h);m(a,!0)},element:n.element,shortcut:!1,textkey:f.constant(r),menuTextkey:f.constant(z),label:f.constant(!1),name:f.constant(l),tooltip:Wa.get,toggle:!0,value:l,updateWidget:function(a){c.getSelection().each(function(b){var c=h.query(b)?oa.on(!0):oa.off();c.fold(a.deselect,f.noop,a.select);c.fold(a.release,f.noop,a.press);n.update(b)})},open:function(){k.deviceType.isTouch()||ga();
I.on()},close:function(){I.off();O.trigger.close()},icon:!0,events:O.registry,enter:n.enter,destroy:function(){I.off();I.destroy();n.destroy()}}}function QP(a,b,c){var d=function(b){if("p"===A.name(b))return Gv.replace(a,b,"li");var c=q.fromTag("li");v.before(b,c);v.append(c,b);return c},e=function(a){var d=q.fromTag(b);c.each(function(a){t.set(d,"list-style-type",a)});v.before(a[0],d);K.append(d,a);h.from(m.get(a[0],"dir")).each(function(a){m.set(d,"dir",a)});return d},g=function(a){var d=ec.mutate(a,
b);c.fold(function(){t.remove(d,"list-style-type")},function(a){t.set(d,"list-style-type",a)});return d},l=function(a){var b=f.constant(!0),c=f.constant(!1);return a.fold(b,c,c)},p=function(a,b){return function(c){return Kh.cata(c,function(c){var d=X.descendants(c,"li");return 0<d.length?(c=d[0],d=d[d.length-1],c=Cb.first(c).getOr(c),d=Cb.last(d).getOr(d),xo.outdent(a,c,0,d,ka.getEnd(d),b).map(function(a){return a.all()})):h.none()},h.none,h.none).getOr([])}},r=function(a){return Kh.cata(a,function(a){return[a]},
function(a){return[g(a)]},function(a){a=k.map(a,d);e(a);return a})};return{apply:function(a,d,e){e=RP.categorise(e,b,c);a=k.forall(e,l)?p(a,d):r;return k.bind(e,a)}}}function SP(a,b){var c=D.create({analytics:s(["message"])}),d=KP(b),e=a.region(),g=a.content(),l=a.undo(),p=h.some(Hv.format),r=g.isRoot,z=function(a){TP.deviceType.isiOS()&&k.each(k.groupBy(k.filter(k.map(a,function(a){return J.ancestor(a,"ul,ol",r).getOr(!1)}),f.identity),function(a){return a.dom()}),function(a){var b=t.get(a[0],"list-style-type");
t.set(a[0],"list-style-type","none");setTimeout(function(){t.set(a[0],"list-style-type",b)},0)})},w=function(){return tb.cata(a.selections().get(),f.constant([]),function(a){return 0<a.length?Iv.groups(d,a[0]):[]},function(a,b){return Iv.groups(d,a.startContainer(),r)})},G=function(a){var b=De.hasSelection(g),c=UP.read(g,d),e=w();a.populate(e);Yj.updateMenu(a,c.tag(),b.content(),d.everything(),h.none())},I=function(b){d.ruleByName(b).each(function(b){Ce.cata(b,function(b,c,d){ig.inline(e,g,l,a.selections(),
b)},function(b,c,d,f){ig.setClass(e,g,l,a.selections(),b,c)},function(b,c,d){ig.transform(e,g,l,a.selections(),b).each(function(a){k.each(a,function(a){m.remove(a,"class")});z(a)})},function(b,c,d,f){ig.transform(e,g,l,a.selections(),b).each(function(a){k.each(a,function(a){m.set(a,"class",c)});z(a)})});g.uiChanged()})},O=C.immutableBag("region content undo selections keyboard listConstructor type styles textkey menuTextkey".split(" "),[]),Va=O({region:e,content:g,undo:l,selections:a.selections(),
keyboard:a.keyboard(),listConstructor:f.curry(dk.unordered,r),type:"bullist",styles:["default","disc","circle","square"],textkey:u.key("action.bullist"),menuTextkey:u.key("list.unordered.menu")}),O=O({region:e,content:g,undo:l,selections:a.selections(),keyboard:a.keyboard(),listConstructor:f.curry(dk.ordered,r),type:"numlist",styles:"default decimal upper-alpha lower-alpha upper-roman lower-roman lower-greek".split(" "),textkey:u.key("action.numlist"),menuTextkey:u.key("list.ordered.menu")}),ga=function(a){return u.translate("block.describe",
u.get(a.textkey()))},Va=f.curry(Fv,Va),O=f.curry(Fv,O);return{ui:$a.ui({"blocktag.dropdown":function(){return{name:"blocktag",textkey:u.key("block.styles"),menuTextkey:u.key("block.menu"),updateDropdown:G,action:I,values:d.everything(),render:p,tooltip:Wa.get,toggle:!1,describe:ga}},"bullist.widget":Va,"numlist.widget":O,"bullist.general":f.constant(Jv.ul(a)),"numlist.general":f.constant(Jv.ol(a))},f.identity),events:c.registry,destroy:f.noop}}function Qa(a,b,c,d){return{context:a,profile:b,action:c,
label:f.constant(d)}}function VP(a){var b=a.content(),c=sc.destroyable();return{ui:$a.ui({"bookmark.general":{toggle:!1,value:"bookmark",textkey:u.key("insert.bookmark"),action:function(){b.getSelection().each(function(a){var c=q.fromTag("span"),g=b.getSelectedText().fold(function(){return ek.generate("textbox-bookmark")},function(a){a=50<a.length?a.substr(0,50):a;return ek.generate(Uc.getAnchorValue(a,"textbox-bookmark"))});m.set(c,"id",g);a=Bc.after(a.startContainer(),a.startOffset(),a.endContainer(),
a.endOffset())?cc(a.endContainer(),a.endOffset(),a.endContainer(),a.endOffset()):cc(a.startContainer(),a.startOffset(),a.startContainer(),a.startOffset());var l=q.fromText(Ua.zeroWidth());Cc.insertAt(b,a,[c,l]).each(function(a){var c=y.nextSibling(a.element()).filter(function(a){return ka.isCursorPosition(a)&&!L.eq(l,a)}).getOrThunk(function(){var b=q.fromText(" ");v.after(a.element(),b);return b});b.win().each(function(a){Fa.setExact(a,c,0,c,0)})})})},updateGeneral:f.noop,icon:!0,shortcut:!1,hasPopup:!1,
hasDialog:!0}}),destroy:function(){c.clear()}}}function WP(a,b){return{is:function(c){return m.get(c,a)===b},remove:function(b){m.remove(b,a)},set:function(c){m.set(c,a,b)}}}function XP(a){var b=a.content(),c=D.create({analytics:s(["message","properties"])}),d=function(a,c){Kv.readDir(b).map(function(b){return b[a]}).fold(oa.unchanged,f.apply).fold(c.deselect,f.noop,c.select)};return{ui:$a.ui({"ltrdir.general":{value:"ltrdir",textkey:u.key("directionality.ltrdir"),action:f.curry(Lv.dirLtr,a),updateGeneral:f.curry(d,
"ltrdir"),toggle:!0},"rtldir.general":{value:"rtldir",textkey:u.key("directionality.rtldir"),action:f.curry(Lv.dirRtl,a),updateGeneral:f.curry(d,"rtldir"),toggle:!0}}),events:c.registry,destroy:f.noop}}function Mv(a,b){return Nv(document.createElement("canvas"),a,b)}function Ov(a){return a.getContext("2d")}function Nv(a,b,c){a.width=b;a.height=c;return a}function Pv(a,b){return new (Vb.getOrDie("Blob"))(a,b)}function yo(){return new (Vb.getOrDie("FileReader"))}function Qv(a){return new (Vb.getOrDie("Uint8Array"))(a)}
function Rv(a){return new Xd(function(b,c){function d(){f();b(l)}function e(){f();c("Unable to load data of type "+a.type+": "+g)}var g=URL.createObjectURL(a),l=new Image,f=function(){l.removeEventListener("load",d);l.removeEventListener("error",e)};l.addEventListener("load",d);l.addEventListener("error",e);l.src=g;l.complete&&d()})}function Sv(a){return new Xd(function(b,c){var d=new XMLHttpRequest;d.open("GET",a,!0);d.responseType="blob";d.onload=function(){200==this.status&&b(this.response)};d.onerror=
function(){var a;0===this.status?(a=Error("No access to download image"),a.code=18,a.name="SecurityError"):a=Error("Error "+this.status+" downloading image");c(a)};d.send()})}function Tv(a){var b=a.split(",");a=/data:([^;]+)/.exec(b[0]);if(!a)return h.none();a=a[1];for(var b=fk.atob(b[1]),c=b.length,d=Math.ceil(c/1024),e=Array(d),g=0;g<d;++g){for(var l=1024*g,f=Math.min(l+1024,c),r=Array(f-l),z=0;l<f;++z,++l)r[z]=b[l].charCodeAt(0);e[g]=Qv(r)}return h.some(Pv(e,{type:a}))}function zo(a){return new Xd(function(b,
c){Tv(a).fold(function(){c("uri is not base64: "+a)},b)})}function Uv(a){return new Xd(function(b){var c=new yo;c.onloadend=function(){b(c.result)};c.readAsDataURL(a)})}function Ao(a,b,c){function d(b,c){return a.then(function(a){return Dc.canvasToDataURL(a,b,c)})}return{getType:f.constant(b.type),toBlob:function(){return Xd.resolve(b)},toDataURL:function(){return c},toBase64:function(){return c.split(",")[1]},toAdjustedBlob:function(b,c){return a.then(function(a){return Dc.canvasToBlob(a,b,c)})},
toAdjustedDataURL:d,toAdjustedBase64:function(a,b){return d(a,b).then(function(a){return a.split(",")[1]})},toCanvas:function(){return a.then(fc.clone)}}}function Vv(a){return Dc.blobToDataUri(a).then(function(b){return Ao(Dc.blobToCanvas(a),a,b)})}function YP(a){var b=D.create({insertAssets:s(["assets"]),insertHtml:s(["html"])}),c=a.content(),d=h.none(),e=function(a){c.win().each(function(b){Fa.getAtPoint(b,a.x(),a.y()).each(function(a){Fa.setExact(b,a.start(),a.soffset(),a.finish(),a.foffset())})})},
g=function(a){e(a);b.trigger.insertAssets(a.assets())},l=function(b){e(b);b=b.html();a.undo().save();b=Od.fromHtml(b);Cc.insertAtCursor(c,b)};c.events.ready.bind(function(a){var b=h.none();d=h.some(ZP.forImages(a.api().canvas(),b,!0));d.each(function(a){a.events.dragndropimages.bind(g);a.events.dragndrophtml.bind(l)})});return{events:b.registry,destroy:function(a){d.each(function(a){a.destroy()})}}}function gk(a,b){var c=void 0!==b?b:$P,d=C.immutable("result","timestamp"),e={},g=function(a,b,c){e[a]=
d(b,c)},l=function(a){var b=Lh.now();W.map(a,function(a,d){b-a.timestamp<c&&g(d,a.result,a.timestamp)})};R.isObject(a)&&l(a);return{set:g,get:function(a,b){return h.from(e[b]).filter(function(b){return a-b.timestamp()<c}).map(function(a){return a.result()})},dump:function(){return W.map(e,function(a){return{result:a.result(),timestamp:a.timestamp()}})}}}function aQ(a){return za.nu(function(b){var c=yo();c.onload=function(a){b(a.target.result)};c.readAsText(a)})}function bQ(a,b,c){var d=gk(c),e=hk.get("ephox.media.service.one",
Bo.requestOne(),a+"/1/embed?url=${url}&maxWidth=${maxWidth}",b);return{checkOne:function(a,b,c){var f=Lh.now();c=ik.single(function(c,d){return e.execute({url:encodeURIComponent(a),maxWidth:b,fresh:d})},d,b+"&"+a,f,void 0!==c?c:!1);return Yd.foldSync(c,Wv.failed,Wv.success)},dumpCache:d.dump}}function cQ(a,b,c){var d=Co.embedChecking(),e=a.internalevents(),g=a.communication(),l=bQ(c.url,g.newHeaders(),b.cache),f=a.undo(),r=a.content();a=a.filter();Co.configure(a);var z=["img","video","audio"],h=function(a,
b){v.after(a,b);B.remove(a)},G=function(a,b){n.add(a,d);var e=l.checkOne(b,400);Yd.handle(e,function(b){n.remove(a,d)},function(e){n.remove(a,d);f.save();var g=q.fromHtml(e.recommended().html().trim());k.contains(z,A.name(g))||m.set(g,"data-ephox-embed-iri",b);var l=q.fromTag("div");v.append(l,g);c.inlineStyles&&r.doc().each(function(a){Xv.inlinePrefixedStyles(a,l,".ephox-summary-card")});r.getSelection().fold(function(){h(a,g)},function(b){var c=L.eq(b.startContainer(),a)||L.contains(a,b.startContainer());
h(a,g);var d=y.nextSibling(g).filter(ka.isCursorPosition).getOrThunk(function(){var a=q.fromTag("br");v.after(g,a);return a});(c||y.parent(g).filter(function(a){return b.collapsed()&&(L.eq(b.startContainer(),a)||L.contains(a,b.startContainer()))}).isSome())&&r.setSelection(d,0,d,0)})})},I=function(a){var b=a.command(),c=a.data();m.has(c,Xa.bookmarkAnchorDataAttr())||"insertLink"!==b&&"updateLink"!==b||y.parent(c).each(function(a){var b=pb.get(a).replace(Ua.zeroWidth(),""),d=m.get(c,"href");if(b===
d||"http://"+b===d)G(c,d);else{var e=y.children(a),g=k.find(k.reverse(e),function(a){return"br"===A.name(a)&&jk.documentPositionPreceding(c.dom(),a.dom())}).map(function(a){a=k.indexOf(e,a).getOr(-1);return e.slice(a+1,e.length)}).getOr(e);a=k.find(e,function(a){return"br"===A.name(a)&&jk.documentPositionPreceding(a.dom(),c.dom())}).map(function(a){a=k.indexOf(g,a).getOr(0);return g.slice(0,a)}).getOr(g);a=k.foldl(a,function(a,b){return a+pb.get(b)},"").replace(Ua.zeroWidth(),"");a!==d&&"http://"+
a!==d||G(c,d)}})};e.events.internal.bind(I);return{destroy:function(){e.events.internal.unbind(I)},state:function(){return{cache:l.dumpCache()}}}}function Yv(a){var b=function(){return null===a.project.apiKey?{}:{"textbox-api-key":a.project.apiKey}},c=function(a){var b=a+(-1===a.indexOf("?")?"?":"&")+"url=";return function(a){return h.some(b+encodeURIComponent(a))}},c=void 0===a.imageEdit.proxy?h.none:c(a.imageEdit.proxy);return{headers:function(){return{headers:b(),xhrFields:{withCredentials:!0}}},
newHeaders:b,proxy:c,destroy:f.noop}}function dQ(a){var b="true"===m.get(a,"contenteditable");m.set(a,"contenteditable","true");var c=E.bind(a,"keydown",function(a){u.trigger.keydown(a)}),d=E.bind(a,"keyup",function(a){u.trigger.keyup(a)}),e=E.bind(a,"keypress",function(a){u.trigger.keypress(a)}),g=E.bind(a,"mousedown",function(a){u.trigger.mousedown(a)}),l=E.bind(a,"mouseout",function(a){u.trigger.mouseout(a)}),p=E.bind(a,"mouseup",function(a){u.trigger.mouseup(a)}),r=E.bind(a,"contextmenu",function(a){u.trigger.contextmenu(a)}),
z=E.bind(a,"cut",function(a){u.trigger.cut(a)}),h=E.bind(a,"copy",function(a){u.trigger.copy(a)}),G=E.bind(a,"paste",function(a){u.trigger.paste(a)}),I=E.bind(q.fromDom(document),"selectionchange",function(a){u.trigger.selectionchange(a)}),O=function(){u.trigger.paint()},Va=E.capture(a,"load",O);Zv.bind(a,O);var ga=$v.browser.isFirefox()?"focus":"focusin",$b=E.bind(a,ga,function(a){u.trigger.focus(a)}),t=E.bind(a,"focusout",function(a){u.trigger.focusOut(a)}),of=E.bind(a,"dblclick",function(a){u.trigger.dblclick(a)}),
v=function(b){return L.eq(a,b)},ga={loaded:za.pure(H.error("no iframe in inline mode"))},u=D.create({keydown:s(["original"]),keyup:s(["original"]),keypress:s(["original"]),mousedown:s(["original"]),mouseup:s(["original"]),mouseout:s(["original"]),contextmenu:s(["original"]),cut:s(["original"]),copy:s(["original"]),paste:s(["original"]),focus:s(["original"]),dblclick:s(["original"]),focusOut:s(["original"]),selectionchange:s(["original"]),paint:s([])});return{win:f.constant(window),doc:f.constant(q.fromDom(document)),
body:f.constant(a),get:function(){return ua.get(a)},set:function(b){ua.set(a,b)},focus:function(){$v.browser.isIE()?a.dom().setActive(!0):T.focus(a)},focused:function(){return document.activeElement===a.dom()},blur:function(){a.dom().blur()},destroy:function(){b||m.remove(a,"contenteditable");k.each([c,d,e,g,p,l,r,z,h,G,$b,t,of,I,Va],function(a){a.unbind()});Zv.unbind(a,O)},events:u.registry,futures:ga,background:f.noop,foreground:f.noop,hasRange:function(a){return sa.closest(a.startContainer(),v).isSome()&&
sa.closest(a.endContainer(),v).isSome()},isActive:function(){return n.has(a,jd.active())}}}function aw(a,b,c){var d=f.curry(L.eq,b),e=function(c){var e=Ja.freefallRtl(b).element();return J.ancestor(e,"table",d).filter(eQ(a,c)).map(bw.above)},g=function(b,d,e,g,f){b.kill();Fa.setExact(a,d,e,g,f);c()};return function(c){var f=c.y();e(f).orThunk(function(){var c=Ja.freefallLtr(b).element();return J.ancestor(c,"table",d).filter(fQ(a,f)).map(bw.below)}).fold(function(){if("html"===A.name(c.target())){var e=
c.x(),f=c.y();Fa.getAtPoint(a,e,f).each(function(a){var e=function(a){return J.closest(a,'[contenteditable="false"]',d).isSome()},f=e(a.start()),e=e(a.finish());(f||e)&&kk.browser.isIE()||T.focus(b);if(kk.browser.isIE()&&1===y.childNodesCount(b)&&(f=y.children(b)[0],"p"===A.name(f)&&0===y.childNodesCount(f))){v.append(f,q.fromTag("br"));g(c,f,0,f,0);return}g(c,a.start(),a.soffset(),a.finish(),a.foffset())})}},function(a){var b=q.fromHtml("<p><br /></p>");a.fold(function(a){v.after(a,b)},function(a){v.before(a,
b)});g(c,b,0,b,0)})}}function cw(a){var b=null,c=function(){b=null;ub.get(a.win()).each(function(b){b.collapsed()&&a.focused()&&d.trigger.loiter(b.startContainer(),b.startOffset())})},d=D.create({loiter:s(["node","offset"])});return{reset:function(){null!==b&&(clearTimeout(b),b=null);b=setTimeout(c,1E3)},destroy:function(){null!==b&&(clearTimeout(b),b=null)},events:d.registry}}function dw(a,b){var c=h.none(),d=Qc.tap(function(){d.control.block();var g=a();g.filter(function(a){return ca.inBody(a.startContainer())}).each(function(a){var d=
function(){e.trigger.nodeChanged(a.startContainer(),a.startOffset(),c.map(function(a){return a.startContainer()}))};c.fold(d,function(c){var e=L.eq,g=a.startContainer(),f=a.endContainer(),h=c.startContainer(),k=c.endContainer(),m=c.startOffset()!==a.startOffset();c=c.endOffset()!==a.endOffset();e(h,g)&&e(k,f)&&!(!A.isText(g)&&m||!A.isText(f)&&c)||!1!==gQ([a.startContainer()],b)||d()})});c=g;d.control.unblock()}),e=D.create({nodeChanged:s(["node","offset","prev"])});return{update:function(){d.instance()},
clear:function(){c=h.none()},destroy:function(){c=h.none();d.control.block()},events:e.registry}}function hQ(){var a=h.none();return{get:function(){return a},restore:function(b){a.each(b)},update:function(b){a=b.or(a)},clear:function(){a=h.none()}}}function iQ(a,b){var c=function(){return O().orThunk(g.get)},d=E.bind(a.body(),"click",aw(a.win(),a.body(),f.noop)),e=dw(c,b),g=hQ(),l=function(){e.update();I()},p=function(){g.restore(function(a){m(a.startContainer(),a.startOffset(),a.endContainer(),a.endOffset())})};
e.events.nodeChanged.bind(function(a){y.trigger.nodeChanged(a.node(),a.offset(),a.prev());y.trigger.uiStale()});a.events.keyup.bind(function(a){y.trigger.keyup(a.original());l();r.reset();y.trigger.interaction()});a.events.mouseout.bind(function(a){I()});a.events.mouseup.bind(function(a){y.trigger.mouseup(a.original());l();r.reset();y.trigger.interaction()});var r=cw(a);r.events.loiter.bind(function(a){y.trigger.loiter(a.node(),a.offset())});var z=function(){O().isNone()?p():I()},h=function(){a.isActive()&&
(z(),ew.focus(a,c,n))},k=function(){y.trigger.domChanged()},I=function(){var a=O();g.update(a)},O=function(){return ub.get(window).filter(a.hasRange)},m=function(a,b,c,d){try{ub.setExact(window,a,b,c,d)}catch(e){console.log("Skipping restoring invalid cursor position")}},ga=function(b){of(function(c){a.isActive()&&b();l()},Do.build(a.focused,z,h,q.fromDom(document)))},n=function(a,b,c,d){ga(function(){m(a,b,c,d)})},t=function(a,b){var c=ub.get(window);p();var d=a(b);b.toOld();c.each(function(a){m(a.startContainer(),
a.startOffset(),a.endContainer(),a.endOffset());jQ.browser.isEdge()&&(b.isSet()||b.toOld())});return d},of=function(a,b){var c=a(b);b.toOld();return c},u=function(b){var c=Do.build(a.focused,z,h,q.fromDom(document));return(O().isSome()?of:t)(b,c)},v=function(a){return function(){var b=arguments;return u(function(c){c.toEditor();return a(b)})}}(function(a){document.execCommand.apply(document,a);I()}),x=function(a,b){return Ba(a,b)},y=D.create({nodeChanged:s(["node","offset","prev"]),loiter:s(["node",
"offset"]),uiStale:s([]),domChanged:s([]),interaction:s([]),keyup:s(["original"]),mouseup:s(["original"]),keydown:M(["original"],a.events.keydown),keypress:M(["original"],a.events.keypress),mousedown:M(["original"],a.events.mousedown),contextmenu:M(["original"],a.events.contextmenu),cut:M(["original"],a.events.cut),copy:M(["original"],a.events.copy),paste:M(["original"],a.events.paste),focus:M(["original"],a.events.focus),focusOut:M(["original"],a.events.focusOut),dblclick:M(["original"],a.events.dblclick),
selectionchange:s(["original"]),paint:M([],a.events.paint)});a.events.selectionchange.bind(function(a){O().each(function(){y.trigger.selectionchange(a.original())})});return{win:a.win,doc:a.doc,body:a.body,canvas:a.body,getContent:a.get,setContent:a.set,resetContent:function(b){a.set(b);g.clear();e.clear();b=Ja.freefallLtr(a.body());n(b.element(),b.offset(),b.element(),b.offset());k()},blur:a.blur,focus:function(){Do.toEditor(a.focused,z,h,q.fromDom(document))},uiChanged:function(){u(function(){y.trigger.uiStale()})},
domChanged:k,clearSelection:function(){ub.clearAll(window)},getSelection:c,setSelection:n,setRelativeSelection:function(a,b){ga(function(){ub.setRelative(window,a,b)})},backupSelection:I,replaceSelection:function(a){O().fold(function(){p();O().each(function(){ub.replace(window,a);l()})},function(b){ub.replace(window,a);l()})},execCommand:v,destroy:function(){a.destroy();r.destroy();d.unbind();e.destroy()},events:y.registry,futures:a.futures,background:a.background,foreground:a.foreground,translateScreen:x,
translateLocation:x}}function Eo(){var a=null,b=[],c=function(){return null!==a},d=function(){k.each(b,function(a){e(a)})},e=function(b){setTimeout(function(){b.apply(null,a)},0)};return{get:function(a){c()?e(a):b.push(a)},set:function(){a=Array.prototype.slice.call(arguments);d();b=[]},isSet:c}}function kQ(a,b,c){var d=q.fromTag("div");Q.presentation(d);t.set(d,"display","none");var e=ua.get(a);(function(b){var c=y.owner(a);k.each(b,function(a){lQ.addLinkIfMissing(c,a)})})(b);b=Eo();var g=dQ(a),
l=iQ(g,c);b.set(l);return{element:function(){return a},onCanvas:function(a){a(l.canvas())},initial:f.constant(e),placeholder:function(){return d},tabindex:function(b){m.set(a,"tabindex",b)},destroy:function(){B.remove(d);l.destroy()},loaded:b.get,isInline:f.constant(!0)}}function mQ(a,b){var c=a.doc();return{execCommand:function(a){var e=arguments,g=a.toLowerCase(),l=f.die("This should not be in hare."),l={fontsize:l,forecolor:l,fontname:l,bgcolor:l,inserthtml:l,blockquote:l};(void 0!==l[g]?l[g]:
function(){c.dom().execCommand.apply(c.dom(),e)})();b.update()}}}function nQ(a){var b=function(){return ub.get(a.win())},c=function(b,c,g,l){b=cc(b,c,g,l);ub.set(a.win(),b)};return{clearSelection:function(){ub.clearAll(a.win())},getSelection:b,replaceSelection:function(b){ub.replace(a.win(),b)},setSelection:c,setRelativeSelection:function(b,c){ub.setRelative(a.win(),b,c)},restore:f.noop,backup:f.noop,nativeCall:f.apply,focus:function(){ew.focus(a,b,c)},destroy:f.noop,clear:f.noop}}function oQ(a,b){var c=
h.none(),d=E.bind(b,"blur",function(a){g()}),e=function(){c=h.none()},g=function(){ub.get(a.win()).fold(f.noop,function(a){c=h.some(a)})},l=function(b,d){try{d(a.win(),b),c=h.some(b)}catch(e){fw()}},p=function(a){l(a,ub.set)},r=function(){c.each(p)},z=function(a){var b=document.activeElement;a=a();null!==b&&void 0!==b&&b!==document.activeElement&&void 0!==b.focus&&b.focus();return a};return{clearSelection:function(){T.hasFocus(b)&&ub.clearAll(a.win())},getSelection:function(){return T.hasFocus(b)?
ub.get(a.win()):c},replaceSelection:function(c){T.hasFocus(b)||r();ub.replace(a.win(),c)},setSelection:function(a,c,d,e){z(function(){var g=cc(a,c,d,e);T.hasFocus(b)?p(g):l(g,f.noop)})},setRelativeSelection:function(b,c){z(function(){e();try{ub.setRelative(a.win(),b,c)}catch(d){fw()}g()})},restore:r,backup:g,nativeCall:function(a){return z(function(){T.hasFocus(b)||r();return a()})},focus:function(){if(document.activeElement!==b.dom()){var d=c;a.focus();d.each(p)}else ub.get(a.win()).fold(r,f.noop)},
destroy:function(){d.unbind()},clear:e}}function pQ(a,b,c){var d=Y.detect(),e=d.browser.isIE()||d.browser.isEdge()?oQ(a,b):nQ(a),g=E.bind(a.doc(),"click",aw(a.win(),a.body(),e.backup)),l=-1,f=function(){return e.getSelection()},r=dw(f,c),z=mQ(a,r);(d.browser.isChrome()||d.browser.isSafari())&&z.execCommand("defaultParagraphSeparator",!1,"p");r.events.nodeChanged.bind(function(a){n.trigger.nodeChanged(a.node(),a.offset(),a.prev());n.trigger.uiStale()});a.events.keyup.bind(function(a){e.backup();r.update(a)});
var h=zc.adaptable(r.update,300);a.events.mouseup.bind(function(a){e.backup();h.throttle(a)});var k=cw(a);k.events.loiter.bind(function(a){n.trigger.loiter(a.node(),a.offset())});c=function(){k.reset();n.trigger.interaction()};a.events.keyup.bind(c);a.events.mouseup.bind(c);var I=function(a){n.trigger.paint()};c=function(b){b=y.documentElement(a.body()).dom().scrollHeight;l!==b&&(l=b,I())};a.events.paint.bind(I);a.events.keyup.bind(c);a.events.mouseup.bind(c);var O=function(){n.trigger.domChanged()},
m=function(a,b,c,d){e.setSelection(a,b,c,d);r.update()},ga=function(a,c){return Na.absolute(b).translate(a,c)},n=D.create({nodeChanged:s(["node","offset","prev"]),loiter:s(["node","offset"]),uiStale:s([]),domChanged:s([]),interaction:s([]),keydown:M(["original"],a.events.keydown),keyup:M(["original"],a.events.keyup),keypress:M(["original"],a.events.keypress),mousedown:M(["original"],a.events.mousedown),mouseup:M(["original"],a.events.mouseup),contextmenu:M(["original"],a.events.contextmenu),cut:M(["original"],
a.events.cut),copy:M(["original"],a.events.copy),paste:M(["original"],a.events.paste),focus:M(["original"],a.events.focus),focusOut:M(["original"],a.events.focusOut),dblclick:M(["original"],a.events.dblclick),selectionchange:M(["original"],a.events.selectionchange),paint:s([])});return{win:a.win,doc:a.doc,body:a.body,canvas:a.doc,getContent:a.get,setContent:a.set,resetContent:function(b){a.set(b);l=-1;e.clear();r.clear();b=Ja.freefallLtr(a.body());m(b.element(),b.offset(),b.element(),b.offset());
O()},blur:a.blur,focus:function(){e.focus()},uiChanged:function(){n.trigger.uiStale()},domChanged:O,clearSelection:function(){e.clearSelection()},getSelection:f,setSelection:m,setRelativeSelection:function(a,b){e.setRelativeSelection(a,b);r.update()},backupSelection:function(){e.backup()},replaceSelection:function(a){e.replaceSelection(a);r.update()},execCommand:function(){var a=arguments;e.nativeCall(function(){z.execCommand.apply(null,a);n.trigger.uiStale()})},destroy:function(){e.destroy();a.destroy();
k.destroy();g.unbind();r.destroy()},events:n.registry,futures:a.futures,background:a.background,foreground:a.foreground,translateScreen:ga,translateLocation:function(b,c){var d=Ea.get(a.doc());return ga(b,c).translate(-d.left(),-d.top())}}}function qQ(a){return{background:f.noop,foreground:f.noop,focus:f.noop,keydown:f.noop,mousedown:h.some,mouseup:h.some,selectionchange:function(a){return zc.adaptable(a,300).throttle}}}function rQ(a,b){return{background:f.noop,foreground:f.noop,focus:function(a,
d,e){Ea.preserve(q.fromDom(document),function(){Ea.preserve(y.owner(e),function(){b.browser.isEdge()&&window.getSelection().removeAllRanges();e.dom().focus()})})},keydown:f.noop,mousedown:h.some,mouseup:h.some,selectionchange:f.constant(f.noop)}}function sQ(a){var b=function(a){return"body"===A.name(a.target())||"html"===A.name(a.target())?h.none():h.some(a)};return{background:function(a,b,e){y.owner(b).dom().defaultView.focus()},foreground:function(a,b,e){void 0!==a.focus&&a.focus()},focus:f.noop,
keydown:function(a){void 0!==a.focus&&a.focus()},mousedown:function(a){return b(a)},mouseup:function(a){return b(a)},selectionchange:f.constant(f.noop)}}function tQ(a,b){var c=a.dom().contentWindow,d=gw.doc(a),e=J.descendant(d,"body").getOrDie();ea.add(e,[jd.base(),jd.active()]);b.autoresize&&n.add(y.documentElement(e),jd.autoresize());var g=uQ.browser.isFirefox(),l=D.create({keydown:s(["original"]),keyup:s(["original"]),keypress:s(["original"]),mousedown:s(["original"]),mouseup:s(["original"]),contextmenu:s(["original"]),
cut:s(["original"]),copy:s(["original"]),paste:s(["original"]),focus:s(["original"]),dblclick:s(["original"]),focusOut:s(["original"]),paint:s([]),selectionchange:s(["original"])}),p=vQ.get(l),r={loaded:hw.cWaitFor(a,"load",1E3)},z=E.bind(d,"keydown",function(a){p.keydown(c);l.trigger.keydown(a)}),h=E.bind(d,"keyup",function(a){l.trigger.keyup(a)}),G=E.bind(d,"keypress",function(a){l.trigger.keypress(a)}),I=E.bind(d,"mousedown",function(a){p.mousedown(a).fold(function(){a.kill()},l.trigger.mousedown)}),
O=E.bind(d,"mouseup",function(a){p.mouseup(a).fold(function(){a.kill()},l.trigger.mouseup)}),m=E.bind(d,"contextmenu",function(a){l.trigger.contextmenu(a)}),ga=E.bind(d,"cut",function(a){l.trigger.cut(a)}),q=E.bind(d,"copy",function(a){l.trigger.copy(a)}),t=E.bind(d,"paste",function(a){l.trigger.paste(a)}),u=E.bind(d,g?"focus":"focusin",function(a){l.trigger.focus(a)}),v=E.bind(d,"focusout",function(a){l.trigger.focusOut(a)}),x=E.bind(d,"dblclick",function(a){l.trigger.dblclick(a)}),A=p.selectionchange(function(a){l.trigger.mouseup(a)}),
IO=E.bind(d,"selectionchange",function(a){A(a);l.trigger.selectionchange(a)}),B=E.bind(d,"touchstart",function(a){l.trigger.mousedown(a)}),C=E.bind(d,"touchend",function(a){l.trigger.mouseup(a);K()}),F=function(){l.trigger.paint()},H=E.capture(d,"load",F),L=function(){c.addEventListener("resize",F,!1);return{unbind:function(){c.removeEventListener&&c.removeEventListener("resize",F,!1)}}}(),K=function(){p.foreground(c,a,e)};return{win:f.constant(c),doc:f.constant(d),body:f.constant(e),get:function(){return ua.get(e)},
set:function(a){ua.set(e,a)},focus:function(){p.focus(c,a,e)},focused:function(){return document.activeElement===a.dom()},blur:function(){e.dom().blur()},destroy:function(){k.each([z,h,G,I,O,m,ga,q,t,u,v,IO,x,B,C,H,L],function(a){a.unbind()})},events:l.registry,futures:r,background:function(){p.background(c,a,e)},foreground:K}}function wQ(a){var b=q.fromTag("iframe");n.add(b,xQ.resolve("content-iframe"));var c=h.none(),d=Eo(),e=E.bind(b,"load",function(){e.unbind();var g=yQ(a);iw.write(b,g);g=jw.onShow(b,
function(){c.each(f.apply);c=h.none();ca.inBody(b)&&d.set(tQ(b,a))});c=h.some(g)});return{element:function(){return b},tabindex:function(a){m.set(b,"tabindex",a)},destroy:function(){B.remove(b);e.unbind();c.each(f.apply)},loaded:d.get}}function zQ(a,b){var c=wQ(a),d=Eo(),e=f.noop;c.loaded(function(g){var l=pQ(g,c.element(),b);g=function(a){a=ia.get(y.documentElement(l.body()));var b=ia.get(c.element());a!==b&&ia.set(c.element(),a)};var p=function(a){a=ia.get(l.body());var b=ia.get(c.element());a!==
b&&ia.set(l.body(),b)};a.autoresize?(l.events.uiStale.bind(f.curry(g,"uiStale")),l.events.domChanged.bind(f.curry(g,"domChanged")),l.events.paint.bind(f.curry(g,"paint"))):(l.events.uiStale.bind(f.curry(p,"uiStale")),l.events.domChanged.bind(f.curry(p,"domChanged")),l.events.paint.bind(f.curry(p,"paint")));setTimeout(function(){d.set(l);e=l.destroy},0)});return{element:c.element,onCanvas:function(a){d.get(function(b){a(b.canvas())})},initial:f.constant(""),placeholder:c.element,tabindex:c.tabindex,
destroy:function(){e();c.destroy()},loaded:d.get,isInline:f.constant(!1)}}function AQ(a){var b=h.none(),c=h.none(),d=function(){b.each(function(b){a.setSelection(b.startContainer(),b.startOffset(),b.endContainer(),b.endOffset())});c.each(function(a){a.restore()})},e=function(){b=h.none();c=h.none()};return{restore:function(){d();e()},save:function(){b=a.getSelection();c=a.doc().map(Ea.capture)},get:function(){return b},clear:e,attempt:function(a){b=a.or(b)}}}function Fo(){var a=D.create({fail:s(["url",
"image"]),success:s(["url","image"]),start:s(["url"])}),b=function(a,b,c){var l=a.dom();l.addEventListener(b,c,!1);return{unbind:function(){l.removeEventListener(b,c,!1)}}},c=function(c){var e=m.get(c,"src"),g=b(c,"load",function(){g.unbind();l.unbind();(!0===c.dom().complete?a.trigger.success:a.trigger.fail)(e,c)}),l=b(c,"error",function(){g.unbind();l.unbind();a.trigger.fail(e,c)})};return{go:function(b){a.trigger.start(b);var e=new Image;e.src=b;c(q.fromDom(e))},monitor:c,events:a.registry}}function BQ(a,
b,c,d,e){lk.configure(a);var g=h.none(),l={inline:c.inline,base:h.from(c.base),editorStyles:[c.css+"/content.css"],stylesheets:c.stylesheets,documentStyles:c.documentStyles,autoresize:c.autoresize};d=CQ.build(d,l);var p=d.editable();null!==c.tabindex&&p.tabindex(c.tabindex);void 0!==e.raw&&p.setContent(e.raw);p.events.ready.bind(function(){if(void 0!==e.raw){var a=p.getContent();e.raw===a&&e.marked!==e.raw&&z(e.marked,!1)}a=p.body().getOrDie();g=h.some(E.bind(a,"dblclick",function(a){kw.correct(p)}))});
p.events.ready.bind(function(a){Go.addClasses(a.api().body())});var r=function(){return p.getSelection().bind(function(a){var b=p.body().getOrDie("content should not return a selection if body is not yet available");return mk.serialise(b,a)}).getOr(p.getContent())},z=function(a,b){p.body().each(function(c){p.setContent(a);mk.extract(c).fold(function(){var a=Ja.freefallLtr(c);p.setSelection(a.element(),a.offset(),a.element(),a.offset())},function(a){var b=mk.getSelection(a);mk.remove(a);p.setRelativeSelection(b.start(),
b.finish())});if(!lw.deviceType.isiOS()){var d=X.descendants(c,"img");b?Nc.mapM(d,nk.loadIfNeeded).get(function(){Va()}):Va()}})},w=D.create({nodeChanged:s(["node","offset","prev"]),undoChanged:s([]),loiter:M(["node","offset"],p.events.loiter),uiStale:M([],p.events.uiStale),domChanged:M([],p.events.domChanged),interaction:M([],p.events.interaction),keydown:M(["original"],p.events.keydown),keyup:M(["original"],p.events.keyup),keypress:M(["original"],p.events.keypress),mousedown:M(["original"],p.events.mousedown),
mouseup:M(["original"],p.events.mouseup),contextmenu:M(["original"],p.events.contextmenu),copy:M(["original"],p.events.copy),paste:M(["original"],p.events.paste),focus:M(["original"],p.events.focus),focusOut:M(["original"],p.events.focusOut),dblclick:M(["original"],p.events.dblclick),selectionchange:M(["original"],p.events.selectionchange),paint:M([],p.events.paint),ready:M(["api"],p.events.ready),loaded:M([],p.events.loaded),background:s([]),foreground:s([]),scrollToCursor:s([]),scrollToElement:s(["element"]),
toEditing:s([]),toReading:s([])});p.events.nodeChanged.bind(function(a){w.trigger.nodeChanged(a.node(),a.offset(),a.prev());a.prev().fold(I,function(b){sa.ancestor(b,fa.isBlock).each(function(b){sa.ancestor(a.node(),fa.isBlock).each(function(a){L.eq(a,b)||I()})})})});var k=Qc.tap(function(){w.trigger.undoChanged()}),I=function(){k.instance()},O=function(a){k.instance();var b=k.control.isBlocked();k.control.block();a=a();b||k.control.unblock();k.instance();return a},Va=function(){w.trigger.scrollToCursor()},
ga=Qc.tap(function(){p.focus()});c=function(a){return function(){var b=arguments;y.parent(p.element()).fold(function(){a.apply(null,b)},function(c){var d=c.dom().scrollTop;a.apply(null,b);c.dom().scrollTop=d})}}(p.execCommand);var $b=function(a){return A.isText(a)&&(lw.browser.isSafari()||0===Da.get(a).length)||A.isElement(a)&&!fa.isEmptyTag(a)&&!fa.isBlock(a)},t=function(a){var b=q.fromText(Ua.zeroWidth());v.after(a,b);return b},u=function(a,b){var c=y.children(a);if(b>=c.length)return S.point(a,
b);var d=c[b];return A.isElement(d)&&!fa.isEmptyTag(d)?u(c[b],0):S.point(a,b)},x=function(a){a=$b(a)?t(a):a;p.setRelativeSelection(da.after(a),da.after(a));p.getSelection().each(function(a){a=u(a.startContainer(),a.startOffset());p.setSelection(a.element(),a.offset(),a.element(),a.offset())})},C=function(){return qf.getSelection().bind(function(a){return qf.win().bind(function(b){return jg.fragment(b,a)})})},qf={onCanvas:p.onCanvas,element:p.element,isInline:p.isInline,win:p.win,doc:p.doc,body:p.body,
getContent:p.getContent,setContent:p.setContent,resetContent:p.resetContent,focus:function(){ga.instance()},blur:p.blur,uiChanged:p.uiChanged,domChanged:p.domChanged,undoChanged:I,clearSelection:p.clearSelection,getSelection:p.getSelection,getSelectionStart:function(){return p.getSelection().map(function(a){return a.startContainer()})},setSelection:p.setSelection,setRelativeSelection:p.setRelativeSelection,backupSelection:p.backupSelection,replaceSelection:p.replaceSelection,replaceElement:function(a,
b){var c=qf.getSelection().filter(function(b){var c=f.curry(L.eq,a);return sa.closest(b.startContainer(),c)||sa.closest(b.endContainer(),c)}).isSome();v.before(a,b);B.remove(a);c&&x(b);qf.domChanged();I()},findTagAtCursor:function(b){var c=function(b,c,d){b=y.child(b,c).getOr(b);return sa.closest(b,function(b){return ra.is(b,d)&&a.includeContext(b)})};return qf.getSelection().bind(function(a){return c(a.startContainer(),a.startOffset(),b).orThunk(function(){return c(a.endContainer(),a.endOffset(),
b)})})},getFilteredAttributes:function(b){b=ec.shallow(b);var c=q.fromTag("div");v.append(c,b);a.filterDom(a.output(),c);return m.clone(b)},execCommand:c,insertHtmlAtCursor:function(a){var c=Od.fromHtml(a);O(function(){Cc.insertAtCursorWithInstructions(Ee.doNotMerge,qf,c).each(function(a){if(0===c.length)p.setSelection(a.element(),a.offset(),a.element(),a.offset());else{var d=a.element();y.parent(d).each(function(a){x(d);b.input(c)})}})})},on:p.on,destroy:function(){g.each(function(a){a.unbind()});
p.destroy()},events:w.registry,state:function(){return{raw:p.getContent(),marked:r()}},serialize:r,deserialize:function(a){z(a,!0)},background:function(){p.background();w.trigger.background()},foreground:function(){p.foreground();w.trigger.foreground()},isRoot:function(a){return p.body().fold(f.constant(!1),function(b){return L.eq(b,a)})},selectWord:function(){kw.expand(p)},cursor:function(){return AQ(p)},activate:function(){ga.control.unblock()},deactivate:function(){ga.control.block()},cursorIntoView:Va,
elementIntoView:function(a){w.trigger.scrollToElement(a)},placeholder:p.placeholder,connect:d.connect,translateScreen:p.translateScreen,translateLocation:p.translateLocation,runOrOnLoaded:function(a){p.body().fold(function(){p.events.ready.bind(function(b){a(b.api().body())})},function(b){a(b)})},getSelectedText:function(){return C().map(jg.text)},getSelectedHtml:function(){return C().map(function(b){return jg.html(b,a)})},getFilteredContents:function(b){return Mh.output(a,b)},toEditing:function(){w.trigger.toEditing()},
toReading:function(){w.trigger.toReading()},undoOperation:O,markAsEphemeral:function(a){n.add(a,lk.markerGeneral())},hideInlineSelection:function(){p.isInline()&&(p.backupSelection(),p.clearSelection())}};return qf}function DQ(a){var b=wa({}),c=wa([]),d={},e=h.none();return{structure:b.get,all:c.get,main:function(){return c.get()[0]},update:function(g){k.each(c.get(),function(a){B.remove(a)});e=h.some(g);var l=W.keys(d);W.each(d,function(a){a.assign(e)});var f=W.tupleMap(d,function(a,b){return{k:b,
v:a.situation(g)}}),r=EQ.derive(a,l,f),z=FQ.derive(a,l,f),l=k.foldr(l,function(a,b){return Z.merge(a,f[b].submenus())},{});if(0===r.length)c.set([]);else{var w=bb.mainmenu(a,x.resolve("main-context-menu"),[]);K.append(w,r);Q.menu(w,u.translate("aria.context.menu.generic"));r=[w].concat(z);k.each(r,function(a){n.add(a,x.resolve("layer-menus"))});c.set(r);b.set(l)}},handle:function(a){var b=Ho.type(a),b=d[b];void 0!==b&&b.handle(a)},register:function(a,b){d[a]=b}}}function GQ(a,b,c,d,e,g){var l=[],
p={register:function(a,b){l=l.concat([{name:a,feature:b}])},destroy:f.noop};b.events.ready.bind(function(f){p=HQ(a,b,c,d,e,g);k.each(l,function(a){p.register(a.name,a.feature)})});return{register:function(a,b){p.register(a,b)},destroy:function(){p.destroy()}}}function IQ(a,b,c,d){var e=function(c){b.get().fold(f.noop,function(b){c.stopper();d.transaction(function(){k.each(b,JQ)});a.domChanged();a.toEditing();k.find(b,function(a){return ra.is(a,sd.lastSelectedSelector())}).bind(Cb.first).each(function(b){a.setSelection(b,
0,b,0)})},f.noop)},g=c.editor();g.events.del.bind(e);g.events.backspace.bind(e);g.on();return{destroy:function(){g.off()}}}function KQ(){var a=!0;return{isHandled:function(){return a},handle:function(){a=!1}}}function LQ(a,b){var c=D.create({preLinebreak:s(["handler"]),postLinebreak:s([])});MQ.listen(a);var d=function(d){b.save();a.getSelection().each(function(b){var e=KQ();c.trigger.preLinebreak(e);a.getSelection().each(function(b){e.isHandled()&&(mw.preLinebreak(a),(d?nw.soft:nw.hard)(b,a.isRoot).each(function(b){a.setRelativeSelection(b.start(),
b.finish());a.domChanged();a.cursorIntoView()}));c.trigger.postLinebreak()})})},e=De.isTextArea;a.events.keydown.bind(function(a){ld.isEnter(a)&&!e(a.original().target())&&(a.original().raw().preventDefault(),d(a.original().raw().shiftKey))});return{linebreak:d,destroy:f.noop,events:c.registry}}function Io(a,b){return{find:function(b){return X.descendants(b,a)},cleaner:b}}function NQ(){var a=[];return{add:function(b){a.push(b)},get:function(){return a.slice(0)}}}function OQ(){var a={},b=function(b){return h.from(a[b])};
return{get:b,getOrInit:function(b){var d=a[b];void 0===d&&(a[b]=NQ(),d=a[b]);return d},filterDom:function(a,d){b(a).each(function(a){ok.filterDom(d,a.get())})},filterHtml:function(a,d){return b(a).fold(function(){return d},function(a){return ok.filterHtml(d,a.get())})}}}function PQ(a,b){return{find:function(b){return pc.descendants(b,a)},cleaner:b}}function QQ(a){var b=OQ(),c=f.constant("output"),d=f.constant("input"),e=f.constant("history"),g=f.constant("internal"),l=f.constant("paste"),p=[],r=function(a,
c,d){a=b.getOrInit(a);c=Io(c,d);a.add(c)},z=function(a,c,d){a=b.getOrInit(a);c=PQ(c,d);a.add(c)};a=Su(a,r,z);return{add:function(a,b,c){r(a,"."+b,c)},addSelector:r,addPredicate:z,filterHtml:b.filterHtml,filterDom:b.filterDom,internal:g,output:c,input:d,paste:l,history:e,customFilters:f.constant(a),state:a.state,destroy:f.noop,addExcludedContext:function(a){p=p.concat(a)},includeContext:function(a){return!k.exists(p,f.curry(ra.is,a))}}}function RQ(a,b,c){var d=!1,e=D.create({maximize:s(["layout"]),
restore:s(["layout"]),analytics:s(["message"]),select:s([]),deselect:s([])}),g=function(){e.trigger.maximize(c.alternate)},l=function(){e.trigger.restore(c.alternate)},p=function(a){d?a.select():a.deselect()},r=function(){e.trigger.analytics("fullscreen_"+(d?"off":"on"));a.focus();(d?l:g)()},z=function(a,b){var c=u.key(d?"action.toggle.fullscreen":"action.fullscreen");return Wa.get(c,b)};b.events.fullscreen.bind(r);b=$a.ui({"fullscreen.general":function(){return{toggle:!0,value:"fullscreen",textkey:u.key("action.fullscreen"),
tooltip:z,action:r,updateGeneral:p,icon:!0,shortcut:!0,selected:d}}},f.identity);var h=$a.actions({maximize:g,restore:l});return{ui:b,actions:h,select:function(){d=!0},deselect:function(){d=!1},destroy:f.noop,events:e.registry}}function SQ(){var a={inputLinks:"insertLink"},b=D.create({internal:s(["command","data"])}),c=function(c,e){h.from(a[c]).fold(function(){throw Error("invalid internal event: "+c);},function(a){k.each(e,function(c){b.trigger.internal(a,c)})})};return{input:function(a){a=Uc.many(a);
c("inputLinks",a)},destroy:f.noop,events:b.registry}}function TQ(a,b,c){var d=Eh(),e=[E.bind(b,"keydown",d.keydown),E.bind(b,"keypress",d.keypress),E.bind(c,"keydown",d.keydown),E.bind(c,"keypress",d.keypress)],g=function(a){d.keydown(a.original())},l=function(a){d.keypress(a.original())};a.events.keydown.bind(g);a.events.keypress.bind(l);return{off:d.off,on:d.on,swallowShortcuts:d.swallowShortcuts,yieldShortcuts:d.yieldShortcuts,swallowChars:d.swallowChars,yieldChars:d.yieldChars,swallowKeys:d.swallowKeys,
yieldKeys:d.yieldKeys,destroy:function(){k.each(e,function(a){a.unbind()});a.events.keydown.unbind(g);a.events.keypress.unbind(l)},events:d.events}}function UQ(a){var b=Eh(),c=function(a){b.keydown(a.original())},d=function(a){b.keypress(a.original())};a.events.keydown.bind(c);a.events.keypress.bind(d);return{off:b.off,on:b.on,swallowShortcuts:b.swallowShortcuts,yieldShortcuts:b.yieldShortcuts,swallowChars:b.swallowChars,yieldChars:b.yieldChars,swallowKeys:b.swallowKeys,yieldKeys:b.yieldKeys,events:b.events,
destroy:function(){a.events.keydown.unbind(c);a.events.keypress.unbind(d)}}}function VQ(a){var b=Eh(),c=E.bind(a,"keydown",b.keydown),d=E.bind(a,"keypress",b.keypress);return{off:b.off,on:b.on,swallowShortcuts:b.swallowShortcuts,yieldShortcuts:b.yieldShortcuts,swallowChars:b.swallowChars,yieldChars:b.yieldChars,swallowKeys:b.swallowKeys,yieldKeys:b.yieldKeys,destroy:function(){c.unbind();d.unbind()},events:b.events}}function WQ(a,b){return{all:function(){return TQ(a,b.element(),b.uiContainerParent())},
container:function(a){return VQ(a)},editor:function(){return UQ(a)},destroy:f.noop}}function ow(a){return{backup:f.identity,restore:f.constant(h.none()),element:f.constant(a)}}function XQ(a,b){var c=x.resolve("ops-eph-cursor");a.add(a.internal(),c,Ca.remove);a.add(a.output(),c,Ca.remove);a.addExcludedContext(["."+c]);var d=function(a){var d=function(d){d=pw.diagnose(b.isRoot,d)(c,a);d.cursor().each(function(a){ca.inBody(a.startContainer())&&ca.inBody(a.endContainer())&&b.getSelection().each(function(c){b.setSelection(a.startContainer(),
a.startOffset(),a.endContainer(),a.endOffset())})});return d.result()};return function(){return b.getSelection().fold(a,d)}};return{destroy:f.noop,run:function(a){return d(a)()}}}function YQ(a,b){var c=XQ(a,b),d=function(a,c){var d=a(),e=qw.handle(c);b.getSelection().forall(function(a){return!L.eq(a.startContainer(),e.startContainer())||!L.eq(a.endContainer(),e.endContainer())||a.startOffset()!==e.startOffset()||a.endOffset()!==e.endOffset()})&&b.setSelection(e.startContainer(),e.startOffset(),e.endContainer(),
e.endOffset());return d},e=function(a){return A.isText(a)||"br"===A.name(a)};return{destroy:function(){},run:c.run,preserve:f.curry(uf.preserve,b),lightweightWith:function(a,b){return e(b.startContainer())&&e(b.endContainer())||ib.detect(b).isSome()?d(a,b):c.run(a)},adapt:function(a){b.getSelection().each(function(b){c.run(function(){a(b.startContainer(),b.endContainer())})})},pure:function(a){b.getSelection().each(function(b){a(b.startContainer(),b.endContainer())})},atomic:function(a){return c.run(a)}}}
function rw(){var a=D.create({back:s([]),abort:s([])});return{ui:$a.ui({"back.general":{toggle:!1,value:"mobile-back",textkey:u.key("toolbar.backbutton"),action:function(){a.trigger.back()},updateGeneral:f.noop,icon:!0,label:!0}}),events:a.registry,destroy:f.noop}}function Zd(a,b){return H.error({small:f.constant(a),big:f.constant(b)})}function ZQ(a,b,c,d,e){var g=function(a,b){return a()?b:{feature:h.none()}};c=g(function(){return!a&&Mb.isLargeDesktop()},{feature:c.ui(),command:"fullscreen",type:"general",
unsupported:"Fullscreen is not supported in inline mode"});e=Mb.choice([Nh(Mb.isLargeDesktop,{feature:d.fileupload().ui(),command:"fileupload",type:"general"}),Nh(Mb.isLargeTouch,{feature:d.fileupload().ui(),command:"fileupload",type:"menu"})],Jo.make({feature:d.fileupload().ui(),command:"fileupload"},e.allowLocalImages?["camera"]:[]));var l=g(Mb.isLargeDesktop,{feature:d.accessibility().ui(),command:"a11ycheck",type:"general"}),g=g(Mb.isLarge,{feature:d.specialchar().ui(),command:"specialchar",type:"general"}),
f=Mb.choice([Nh(Mb.isLarge,{feature:d.blocktag().ui(),command:"bullist",type:"split-widget"})],{feature:d.blocktag().ui(),command:"bullist",type:"general"}),r=Mb.choice([Nh(Mb.isLarge,{feature:d.blocktag().ui(),command:"numlist",type:"split-widget"})],{feature:d.blocktag().ui(),command:"numlist",type:"general"}),z={feature:d.formatting().ui(),command:"superscript",type:"general"},w={feature:d.formatting().ui(),command:"subscript",type:"general"},k={feature:d.formatting().ui(),command:"strikethrough",
type:"general"},I={feature:d.font().ui(),command:"fontface",type:"dropdown"},O=Mb.choice([Nh(Mb.isTouch,{feature:d.font().ui(),command:"fontsize",type:"general"})],{feature:d.font().ui(),command:"fontsize",type:"inline-widget"}),m={feature:d.font().ui(),command:"fontcolor",type:"widget"},ga={feature:d.spelling().hasAutocorrect()?d.macros().ui():h.none(),command:"autocorrect",type:"general"},n={feature:d.spelling().ui(),command:"spelling",type:d.spelling().hasDropdown()?"dropdown":"general"},q={feature:d.macros().ui(),
command:"capitalisation",type:"general"},s={feature:d.help().ui(),command:"help",type:"general"},t={feature:d.wordcount().ui(),command:"wordcount",type:"general"};return{link:{feature:d.link().ui(),command:"link",type:"general"},wordcount:t,media:{feature:d.media().ui(),command:"media",type:"general"},hr:{feature:d.horizontalrule().ui(),command:"horizontalrule",type:"general"},specialchar:g,fileupload:e,table:{feature:d.table().ui(),command:"table",type:"widget"},undo:{feature:b.ui(),command:"undo",
type:"general"},redo:{feature:b.ui(),command:"redo",type:"general"},styles:{feature:d.blocktag().ui(),command:"blocktag",type:"dropdown"},language:{feature:d.parlance().ui(),command:"parlance",type:"dropdown"},ltrdir:{feature:d.directionality().ui(),command:"ltrdir",type:"general"},rtldir:{feature:d.directionality().ui(),command:"rtldir",type:"general"},bold:{feature:d.formatting().ui(),command:"bold",type:"general"},italic:{feature:d.formatting().ui(),command:"italic",type:"general"},underline:{feature:d.formatting().ui(),
command:"underline",type:"general"},bookmark:{feature:d.bookmark().ui(),command:"bookmark",type:"general"},strikethrough:k,superscript:z,subscript:w,ul:f,ol:r,removeformat:{feature:d.formatting().ui(),command:"removeformat",type:"general"},alignment:{feature:d.alignindent().ui(),command:"alignment",type:"dropdown"},outdent:{feature:d.alignindent().ui(),command:"outdent",type:"general"},indent:{feature:d.alignindent().ui(),command:"indent",type:"general"},blockquote:{feature:d.alignindent().ui(),command:"blockquote",
type:"general"},"font-menu":{name:"font",textkey:u.key("font.menu-button"),menuTextkey:u.key("font.menu"),type:"submenu",contentFocus:!0,commands:[{category:"Font-menu",items:[I,O,m,z,w,k]}]},"font-face":I,"font-size":O,"font-color":m,fullscreen:c,find:{feature:d.findreplace().ui(),command:"find",type:"general"},accessibility:l,usersettings:{name:"usersettings",textkey:u.key("cog.menu-button"),menuTextkey:u.key("cog.menu"),type:"submenu",flatten:Jo.never(),contentFocus:!0,commands:[{category:"a",
items:[t]},{category:"b",items:[n,ga,q]},{caetgory:"",items:[s]}]}}}function sw(){var a={id:"insert",label:u.key("insert.menu-button"),menuLabel:u.key("insert.menu"),items:[["link","bookmark","fileupload","media","table"],["specialchar","hr"]]};return{undo:{label:u.key("category.undo"),items:["undo","redo"]},insert:{label:u.key("category.insert"),items:[a]},style:{label:u.key("category.style"),items:["styles"]},language:{label:u.key("category.language"),items:["language","ltrdir","rtldir"]},emphasis:{label:u.key("category.emphasis"),
items:["bold","italic","underline"]},align:{label:u.key("category.align"),items:["alignment"]},listindent:{label:u.key("category.listindent"),items:["ul","ol","outdent","indent","blockquote"]},format:{label:u.key("category.format"),items:["font-menu","removeformat"]},tools:{label:u.key("category.tools"),items:["find","accessibility","fullscreen","usersettings"]},"table-tools":{label:u.key("category.table"),items:["table-tools"]},"image-tools":{label:u.key("category.image"),items:["image-tools","crop-tools"]}}}
function pk(a){return{closeMenus:function(b){b=b?b.target():a;tw.target(b,"mousedown")}}}function $Q(a){return function(b,c){var d=E.bind(a,"touchmove",b.move),e=E.bind(a,"touchend",b.drop);return{element:f.constant(a),start:f.noop,stop:f.noop,destroy:function(){d.unbind();e.unbind()}}}}function uw(a,b,c,d){var e=wa(h.none());c=qk.mode({compare:Ko.compare,extract:Ko.extract,sink:c,mutate:function(a,c){e.get().each(function(a){Ko.resize(b,a,c)})}});var g=Ih.transform({},{mode:c}),l=E.bind(a,"touchstart",
function(a){"img"===A.name(a.target())&&2===a.raw().touches.length&&(e.set(h.some(a.target())),g.on(),g.go(b))});g.events.stop.bind(function(){e.get().each(function(a){t.remove(a,"transform")});d()});return{destroy:function(){l.unbind();g.destroy()}}}function aR(a,b,c){var d=c.toolbar.element(),e=vw.tag(),g=[],l=q.fromDom(document),p=pk(d),r=null,z=sc.api(),w=h.none(),G=function(){z.run(function(a){a.toEditing()})},I=function(){p.closeMenus()},O=function(){a.getSelection().each(function(a){a=a.collapsed()?
"0px":"23px";t.set(d,"margin-top",a)})},m=function(a){t.set(d,"margin-top","0px")},ga=function(){var c=[E.bind(d,"touchmove",p.closeMenus)];g=a.doc().map(function(a){return b||!0===bR?[]:[E.bind(a,"selectionchange",O),E.bind(l,"focusin",m)]}).getOr([]).concat(c)},s=function(){k.each(g,function(a){a.unbind()})};return{enter:function(){a.body().each(function(b){a.doc().each(function(c){a.win().each(function(d){T.focus(b);r=kg.onChange();var e=$Q(c),e=uw(c,b,e,f.noop);w=h.some(e);d=cR.setup(window,d);
z.set(d);a.events.toEditing.bind(G);r.events.change.bind(I)})})});n.add(d,"ephox-polish-mobile-show");c.toolbar.refresh();e.maximize();ga()},exit:function(){z.clear();a.events.toEditing.unbind(G);r.events.change.unbind(I);null!==r&&r.destroy();e.restore();s()},destroy:function(){w.each(function(a){a.destroy()});s()}}}function ww(){var a=q.fromTag("div");n.add(a,x.resolve("disabled-mask"));var b=!1,c=P.div();n.add(c,x.resolve("content-container"));var d=P.div();Q.hidden(d,!0);n.add(d,x.resolve("tap-icon"));
v.append(c,d);var e=P.div();n.add(e,x.resolve("disclosure"));v.append(e,q.fromText(u.translate("taptoedit")));v.append(c,e);v.append(a,c);Q.textButton(a,c);var g=D.create({fullscreen:s([])}),l=function(b){var c=a.dom().getClientRects()[0];return k.forall(b,function(a){return a.pageY>=c.top&&a.pageY<=c.top+c.height&&a.pageX>=c.left&&a.pageX<=c.left+c.width})},p=[E.bind(a,"touchmove",function(a){b=!0}),E.bind(a,"touchend",function(a){var c=a.raw().changedTouches;l(c)&&!b&&(a.kill(),g.trigger.fullscreen())}),
E.bind(a,"touchstart",function(a){b=!1}),E.bind(a,"click",function(a){a.kill();g.trigger.fullscreen()})];return{element:f.constant(a),events:g.registry,destroy:function(){k.each(p,function(a){a.unbind()});B.remove(d);B.remove(e);B.remove(c);B.remove(a)},show:function(){t.remove(a,"display")},hide:function(){t.set(a,"display","none")}}}function xw(a,b,c,d,e,g){var l=g.element(),f=aR(a,e.fillscreen,g),r=function(){f.enter();d.actions().maximize()},h=function(){var a=ww();v.append(g.element(),a.element());
a.events.fullscreen.bind(function(a){d.actions().maximize()})};return{init:function(){g.socket(a);a.events.scrollToCursor.bind(function(){uf.cursorIntoView(a)})},loaded:function(){var a=!0===e.fillscreen;(n.has(l,x.resolve("fullscreen-maximized"))||a?r:h)()},restoreEditor:function(){d.actions().restore();f.exit()},destroy:function(){a.body().each(T.blur)}}}function dR(a,b,c,d,e){var g=d.toolbar.items,l=function(l){var f=xw(a,b,c,e,d,l),h=rw();l=Oh.config({items:[["back"]].concat(g),contextual:[]},
{back:{feature:h.ui(),command:"back",type:"general"}},{allowLocalImages:d.allowLocalImages});h.events.back.bind(function(){f.restoreEditor()});return{init:f.init,loaded:f.loaded,config:l,destroy:function(){f.destroy();h.destroy()}}};return{tabbed:l,full:l,destroy:f.noop}}function eR(a,b,c,d,e){var g=d.toolbar.items,l=function(l){var f=xw(a,b,c,e,d,l);l=Oh.config({items:g,contextual:[]},{},{allowLocalImages:d.allowLocalImages});return{init:f.init,loaded:f.loaded,config:l,destroy:function(){f.destroy()}}};
return{tabbed:l,full:l,destroy:f.noop}}function fR(a,b,c,d){b=function(b){return{init:function(){b.socket(a);a.events.scrollToCursor.bind(function(){uf.cursorIntoView(a)});a.events.toEditing.bind(a.focus)},loaded:f.noop,config:Oh.config(d.toolbar,{},{allowLocalImages:d.allowLocalImages}),destroy:f.noop}};return{tabbed:b,full:b,destroy:f.noop}}function gR(a,b){var c=vw.tag(),d=h.none(),e=ww();return{maximize:function(){d=h.some(document.body.scrollTop);t.set(e.element(),"display","none");n.add(a,x.resolve("fullscreen-maximized"));
yw.clobberStyles(a,b);c.maximize()},restore:function(){yw.restoreStyles();n.remove(a,x.resolve("fullscreen-maximized"));c.restore();t.remove(e.element(),"display");d=d.bind(function(a){document.body.scrollTop=a;return h.none()})},destroy:function(){e.destroy()},mask:f.constant(e)}}function hR(a,b){var c=a.document,d=q.fromTag("div");n.add(d,td.resolve("unfocused-selections"));v.append(q.fromDom(c.documentElement),d);var e=E.bind(d,"touchstart",function(c){c.prevent();Lo.resume(a,b);l()}),g=function(a){var b=
q.fromTag("span");ea.add(b,[td.resolve("layer-editor"),td.resolve("unfocused-selection")]);t.setAll(b,{left:a.left+"px",top:a.top+"px",width:a.width+"px",height:a.height+"px"});return b},l=function(){B.empty(d)};return{update:function(){l();var b=Mo.getRectangles(a),b=k.map(b,g);K.append(d,b)},isActive:function(){return 0<y.children(d).length},destroy:function(){e.unbind();B.remove(d)},clear:l}}function iR(a){var b=wa($d.pure({}));return{start:function(c){var d=$d.nu(function(b){return a(c).get(b)});
d.get(f.identity);b.set(d)},idle:function(a){b.get().get(function(){a()})}}}function jR(a){return function(b,c){var d=E.bind(a,"touchmove",b.move),e=E.bind(a,"touchend",b.drop),g=E.bind(a,"gesturestart",function(a){"img"===A.name(a.target())&&a.prevent()});return{element:f.constant(a),start:f.noop,stop:f.noop,destroy:function(){d.unbind();e.unbind();g.unbind()}}}}function kR(a,b,c,d,e,g,l,f){var r=[],z=g.toolbar.element(),w=g.banners(),G=g.viewport(),I=h.none(),O=pk(z),m=null,ga=sc.api(),s=h.none(),
u=gc.event(G),v=C.immutableBag("cWin ceBody socket toolstrip toolbar contentElement cursor keyboardType isScrolling".split(" "),[]),y=function(){ga.run(function(a){a.highlightSelection()})},A=function(){ga.run(function(a){a.refreshSelection()})},B=function(a){var b=Na.absolute(a).top()-G.dom().scrollTop;ga.run(function(c){c.scrollIntoView(b,b+ia.get(a))})},D=function(){zw.insertAtCursor(d,a,function(a){var b=a.absolute().top()-G.dom().scrollTop;ga.run(function(c){c.scrollIntoView(b,b+a.height())})})},
F=function(a){D();ga.run(function(a){a.syncHeight()})},H=function(a){var b=ia.get(z);ga.run(function(a){a.setViewportOffset(b)});A();F()},K=function(a){B(a.element())},M=zc.adaptable(F,300),N=function(){O.closeMenus()},P=function(){U();var a=setInterval(function(){0===window.pageYOffset&&(clearInterval(a),F())},100)},Q=function(){ga.run(function(a){a.clearSelection()})},R=function(){Q();M.throttle()},S=function(){ga.run(function(a){a.toEditing()})},W=function(){ga.run(function(a){a.toReading()})},
V=function(a){ga.run(function(b){b.onToolbarTouch(a)})},U=function(){(kg.get().isPortrait()?n.remove:n.add)(g.element(),x.resolve("mobile-landscape"))},X=function(a){var b=t.get(a,"background-color");void 0!==b&&""!==b&&t.set(a,"-webkit-tap-highlight-color",b)},Z=function(b){var c=a.win().getOrDie(),d=function(a){T.focus(b);a=a.raw().changedTouches[0];Fa.getAtPoint(c,a.pageX,a.pageY).each(function(a){Fa.setExact(c,a.start(),a.soffset(),a.finish(),a.foffset())})},e=a.doc().map(function(a){var c=jR(a),
c=uw(a,b,c,F);s=h.some(c);var e=q.fromDom(a.dom().documentElement);return[E.bind(a,"touchend",function(a){(L.eq(e,a.target())||L.eq(b,a.target()))&&d(a)})]}).getOr([]),g=[E.capture(z,"touchstart",function(a){y();V(a)}),E.bind(b,"touchstart",Q),rk.handle(b,Y),E.bind(z,"touchmove",O.closeMenus)];r=e.concat(g)},Y=function(a){a.kill()};return{exit:function(){a.body().each(function(b){t.remove(w,"display");t.set(g.viewport(),"overflow","hidden");I=I.bind(function(a){a.unbind();return h.none()});n.remove(z,
x.resolve("mobile-show"));ga.clear();T.blur(b);t.remove(b,"padding-top");k.each(r,function(a){a.unbind()});l.restore();a.events.keyup.unbind(R);a.events.domChanged.unbind(M.throttle);a.events.scrollToCursor.unbind(M.throttle);a.events.scrollToElement.unbind(K);a.events.domChanged.unbind(A);a.events.nodeChanged.unbind(A);a.events.toEditing.unbind(S);a.events.toReading.unbind(W);m.events.change.unbind(N);m.events.ready.unbind(P);g.toolbar.events.transitionend.unbind(H);null!==m&&m.destroy()})},enter:function(b){a.body().each(function(b){a.win().each(function(d){m=
kg.onChange();l.maximize();Z(b);T.focus(b);I=h.some(Aw.exclusive(c,"."+gc.scrollable()));t.getRaw(z,"overflow-x").is("auto")&&J.descendant(z,".ephox-chameleon-toolbar").each(function(a){Aw.markAsHorizontal(a);gc.register(a)});X(b);U();document.body.scrollTop=0;n.add(z,x.resolve("mobile-show"));g.toolbar.refresh();t.set(w,"display","block");J.descendant(z,".ephox-chameleon-toolbar").each(function(c){ga.set(lR.setup(v({cWin:d,ceBody:b,socket:G,toolstrip:z,toolbar:c,contentElement:a.element(),cursor:a.cursor(),
keyboardType:mR.stubborn,isScrolling:u.isScrolling})))});a.events.keyup.bind(R);a.events.domChanged.bind(M.throttle);a.events.scrollToCursor.bind(M.throttle);a.events.scrollToElement.bind(K);a.events.domChanged.bind(A);a.events.nodeChanged.bind(A);a.events.toEditing.bind(S);a.events.toReading.bind(W);m.events.change.bind(N);m.events.ready.bind(P);g.toolbar.events.transitionend.bind(H);F()})})},destroy:function(){s.each(function(a){a.destroy()});I.each(function(a){a.unbind()});u.destroy()}}}function Bw(a,
b,c,d,e){var g=gR(e.element(),a),l=q.fromDom(document),p=a.element(),r=f.constant(!0===d.fillscreen),h=kR(a,b,l,c,p,e,g,r),w=function(a){var b=y.children(a),c;if(!(c=0===b.length)){if(b=1===b.length)b=ua.get(a),b=10===b.charCodeAt(0)&&10===b.charCodeAt(1);c=b}c&&(b=q.fromHtml("<p><br></p>"),v.append(a,b));h.enter()},k=function(a){v.append(e.element(),g.mask().element());g.mask().events.fullscreen.bind(function(a){h.enter()})},I=function(a){(r()?w:k)(a)};return{init:function(){e.socket(a)},loaded:function(){a.body().each(I)},
restoreEditor:function(){a.body().each(T.blur);a.element().dom().focus();a.element().dom().blur();h.exit();T.focus(g.mask().element())},destroy:function(){a.body().each(T.blur);h.destroy();g.destroy()}}}function nR(a,b,c,d){var e=d.toolbar.items;return{tabbed:function(g){var l=Bw(a,b,c,d,g),f=rw();g=Oh.config({items:[["back"]].concat(e),contextual:[]},{back:{feature:f.ui(),command:"back",type:"general"}},{allowLocalImages:d.allowLocalImages});f.events.back.bind(function(){l.restoreEditor()});return{init:l.init,
loaded:l.loaded,config:g,destroy:function(){l.destroy();f.destroy()}}},full:f.die("mobile does not support fullscreen layout"),destroy:f.noop}}function oR(a,b,c,d){var e=d.toolbar.items;return{tabbed:function(g){var l=Bw(a,b,c,d,g);g=Oh.config({items:e,contextual:[]},{},{allowLocalImages:d.allowLocalImages});return{init:l.init,loaded:l.loaded,config:g,destroy:function(){l.restoreEditor();l.destroy()}}},full:f.die("mobile does not support fullscreen layout"),destroy:f.noop}}function pR(a,b,c,d,e){return(Cw.deviceType.isiOS()?
!0===e.fillscreen?oR:nR:Cw.deviceType.isAndroid()?!0===e.fillscreen?eR:dR:fR)(a,b,c,e,d)}function Ph(a,b){var c=No.readOr(a,b),d=wa(c),e=function(a){d.set(a)};return{setAndCache:function(b){e(b);No.write(a,b)},set:e,get:function(){return d.get()}}}function qR(a){var b=Ph("native.spelling",a.nativeSpelling),c=Ph("spelling",a.spelling),d=Ph("autocorrect",a.autocorrect),e=Ph("linkpreviews",a.linkpreviews),g=Ph("capitalisation",a.capitalisation);return{spelling:f.constant(c),nativeSpelling:f.constant(b),
autocorrect:f.constant(d),linkpreviews:f.constant(e),capitalisation:f.constant(g),state:function(){return{nativeSpelling:b.get(),spelling:c.get(),autocorrect:d.get(),linkpreviews:e.get(),capitalisation:g.get()}},destroy:f.noop}}function rR(a,b,c){var d=b.all();d.on();var e=[];d.events.backspace.bind(function(){Oo.get(a,e,Qh.left).each(B.remove)});b=function(b){var d=b.stopper();Oo.get(a,e,Qh.left).each(function(b){d();Dw.upOrLeft(a,c,b)})};d.events.right.bind(function(b){var d=b.stopper();Oo.get(a,
e,Qh.right).each(function(b){d();Dw.right(a,c,b)})});d.events.left.bind(b);d.events.up.bind(b);return{registerSkip:function(a){e.push(a)},destroy:function(){d.destroy()}}}function sR(a,b){(b.error?h.some(b.error):h.none()).map(function(a){return Ch(ae.get(a.name,[a]))}).each(a.displayBanner);return{destroy:f.noop}}function Ew(a){return{elementFromPoint:function(b,c){return h.from(a.document.elementFromPoint(b,c)).map(q.fromDom)},getRect:function(a){return a.dom().getBoundingClientRect()},getRangedRect:function(b,
c,d,e){b=Wb.exact(b,c,d,e);return Fa.getFirstRect(a,b).map(function(a){return W.map(a,f.apply)})},getSelection:function(){return Fa.get(a).map(function(b){return lg.convertToRange(a,b)})},fromSitus:function(b){b=Wb.relative(b.start(),b.finish());return lg.convertToRange(a,b)},situsFromPoint:function(b,c){return Fa.getAtPoint(a,b,c).map(function(a){return{start:f.constant(da.on(a.start(),a.soffset())),finish:f.constant(da.on(a.finish(),a.foffset()))}})},clearSelection:function(){Fa.clear(a)},setSelection:function(b){Fa.setExact(a,
b.start(),b.soffset(),b.finish(),b.foffset())},setRelativeSelection:function(b,c){Fa.setRelative(a,b,c)},selectContents:function(b){Fa.setToElement(a,b)},getInnerHeight:function(){return a.innerHeight},getScrollY:function(){return Ea.get(q.fromDom(a.document)).top()},scrollBy:function(b,c){Ea.by(b,c,q.fromDom(a.document))}}}function tR(a,b,c,d){var e=h.none(),g=function(){e=h.none()};return{mousedown:function(a){d.clear(b);e=Fw(a.target(),c)},mouseover:function(g){e.each(function(e){d.clear(b);Fw(g.target(),
c).each(function(g){Fe.identify(e,g,c).each(function(c){var l=c.boxes().getOr([]);if(1<l.length||1===l.length&&!L.eq(e,g))d.selectRange(b,l,c.start(),c.finish()),a.selectContents(g)})})})},mouseup:function(){e.each(g)}}}function uR(a){var b=function(b){return!a.isRoot(b)&&(Rh.isBlock(b)||"li"===A.name(b))},c=function(a){return A.isText(a)&&0===Da.get(a).trim().length},d=function(b,c,d,e){return sa.closest(b,function(b){return y.parent(b).exists(a.isRoot)},a.isRoot).fold(function(){return S.point(b,
c)},function(a){return d(a,b,c)?e(a):S.point(b,c)})},e=function(e,g){var l=function(b){return Lb.before(b,a.isRoot).fold(function(){return S.point(e,g)},function(a){return c(a)?l(a):S.point(a,ka.getEnd(a))})};return sa.closest(e,function(a){return b(a)&&y.prevSibling(a).isSome()},a.isRoot).fold(function(){return d(e,g,wf.isAtLeftEdge,l)},function(a){return b(e)||wf.isAtLeftEdge(a,e,g)?l(a):S.point(e,g)})},g=function(e,g){var l=function(b){return Lb.after(b,a.isRoot).fold(function(){return S.point(e,
g)},function(a){return c(a)?l(a):S.point(a,0)})};return sa.closest(e,function(a){return b(a)&&y.nextSibling(a).isSome()},a.isRoot).fold(function(){return d(e,g,wf.isAtRightEdge,l)},function(a){return b(e)||wf.isAtRightEdge(a,e,g)?l(a):S.point(e,g)})},l=function(a,b){return L.eq(a.element(),b.element())?a.offset()<=b.offset():Bc.after(a.element(),a.offset(),b.element(),b.offset())};return{getSelection:function(a){return a.getSelection().bind(function(a){var b;b=a.startContainer();var c=a.endContainer();
L.eq(b,c)?(c=y.child(b,a.startOffset()),b=y.child(b,a.endOffset()),b=c.filter(fa.isList).isSome()||b.filter(fa.isList).isSome()):b=!0;if(b){if(Bc.after(a.startContainer(),a.startOffset(),a.endContainer(),a.endOffset()))return b=Ja.toLeaf(a.startContainer(),a.startOffset()),a=Ja.toLeaf(a.endContainer(),a.endOffset()),b=ka.isStart(b.element(),b.offset())?e(b.element(),b.offset()):S.point(b.element(),b.offset()),a=ka.isEnd(a.element(),a.offset())?g(a.element(),a.offset()):S.point(a.element(),a.offset()),
a=l(b,a)?h.some(cc(b.element(),b.offset(),a.element(),a.offset())):h.some(cc(a.element(),a.offset(),b.element(),b.offset())),a;b=Ja.toLeaf(a.startContainer(),a.startOffset());a=Ja.toLeaf(a.endContainer(),a.endOffset());b=ka.isEnd(b.element(),b.offset())?g(b.element(),b.offset()):S.point(b.element(),b.offset());a=ka.isStart(a.element(),a.offset())?e(a.element(),a.offset()):S.point(a.element(),a.offset());a=l(b,a)?h.some(cc(a.element(),a.offset(),b.element(),b.offset())):h.some(cc(b.element(),b.offset(),
a.element(),a.offset()));return a}return h.some(a)})}}}function vR(a,b,c){wR.configure(b);var d=uR(c),e=D.create({beforeScan:s([]),afterScan:s([])}),g=Gw.byClass(sd);b=function(){c.body().each(function(a){c.getSelection().each(function(b){be.sharedOne(Ya.table,[b.startContainer(),b.endContainer()]).fold(function(){g.clear(a)},f.noop)})})};(Hw.browser.isChrome()||Hw.browser.isSafari())&&c.events.contextmenu.bind(b);c.events.nodeChanged.bind(b);var l=wa([]);c.runOrOnLoaded(function(b){var d=y.defaultView(b).dom(),
p=Iw.mouse(d,b,c.isRoot,g),r=Iw.keyboard(d,b,c.isRoot,g);l.set([sk.leftDown.bind(b,p.mousedown),sk.leftPressedOver.bind(b,p.mouseover),sk.leftUp.bind(b,p.mouseup),E.bind(b,"keyup",function(a){a.raw().shiftKey&&Vc.isNavigation(a.raw().which)&&c.getSelection().each(function(b){r.keyup(a,b.startContainer(),b.startOffset(),b.endContainer(),b.endOffset()).each(function(b){h(a,b)})},f.noop)}),E.bind(b,"keydown",function(b){c.getSelection().each(function(c){e.trigger.beforeScan();var d=a.directionAt(c.endContainer()).isRtl()?
Vc.rtl:Vc.ltr;r.keydown(b,c.startContainer(),c.startOffset(),c.endContainer(),c.endOffset(),d).each(function(a){h(b,a)});e.trigger.afterScan()})})]);var h=function(a,b){b.kill()&&a.kill();b.selection().each(function(a){c.setRelativeSelection(a.start(),a.finish())})}});var p=function(a){return{win:a.win,doc:a.doc,execCommand:a.execCommand,getSelection:function(){return d.getSelection(a)},setSelection:a.setSelection,setRelativeSelection:a.setRelativeSelection,focus:a.focus}},r=function(a,b){return{doc:b.doc,
win:b.win,execCommand:b.execCommand,focus:b.focus,setSelection:f.noop,setRelativeSelection:f.noop,getSelection:function(){return h.some(z(a))}}},z=function(a){return{startContainer:f.constant(a),startOffset:f.constant(0),endContainer:f.constant(a),endOffset:f.constant(ka.getEnd(a))}};return{get:function(){return c.on(function(a){return c.body().fold(tb.none,function(b){return Jw.retrieve(b,sd.selectedSelector()).fold(function(){return a.getSelection().fold(tb.none,function(b){b=d.getSelection(a).getOr(b);
return tb.single(b,p(a))})},function(b){var c=k.map(b,function(b){return r(b,a)});return tb.multiple(b,c)})})}).getOrThunk(tb.none)},surround:z,events:e.registry,destroy:function(){var a=l.get();k.each(a,function(a){a.unbind()})}}}function xR(a){a=Z.merge({scopes:[q.fromDom(document)],shortcuts:[],swallow:!1},a);var b=Eh(a),c=[],d=[];k.each(a.scopes,function(a){c.push(E.bind(a,"keydown",b.keydown).unbind);d.push(E.bind(a,"keypress",b.keypress).unbind)});return{off:b.off,on:b.on,destroy:function(){k.each(c,
f.apply);k.each(d,f.apply)},events:b.events,swallowShortcuts:b.swallowShortcuts,yieldShortcuts:b.yieldShortcuts,swallowChars:b.swallowChars,yieldChars:b.yieldChars,swallowKeys:b.swallowKeys,yieldKeys:b.yieldKeys}}function yR(a,b){var c=D.create({undo:s([]),redo:s([]),context:s([]),toolbar:s([]),plaincode:s([]),bold:s([]),italic:s([]),underline:s([]),indent:s([]),outdent:s([]),link:s([]),blockquote:s([]),search:s([]),help:s([]),fullscreen:s([])}),d=zR(c),e=[d.undo,d.redo,d.context,d.toolbar],g=[d.plaincode,
d.bold,d.italic,d.underline,d.indent,d.outdent,d.link,d.search,d.blockquote,d.help,d.fullscreen],e=(d=!0===b.features)?e.concat(g):e,l=k.bind(e,function(a){return a.keys()}),p=k.map(e,function(a){return AR(a.keys(),a.trigger())}),r=h.none();a.onCanvas(function(a){a=xR({scopes:[a],shortcuts:l});r=h.some(a);a.swallowShortcuts(l);a.events.command.bind(function(a){var b=a.command();ab.findMap(p,function(a){return a(b)}).each(f.apply)});a.on()});return{isHelpShortcutEnabled:f.constant(d),events:c.registry,
destroy:function(){r.each(function(a){a.destroy()})}}}function BR(){var a=[],b=function(){a=k.filter(a,function(a){return Tb.isVisible(a.popup().panel())})};return{recalculate:function(){b();k.map(a,function(a){a.anchor().recalculate(a.popup())})},dismiss:function(){b();k.map(a,function(a){a.anchor().isAnchored()||a.popup().close()})},update:function(b,d){a=CR.put(a,b,d,DR,function(a){return a.popup()===b})},destroy:function(){}}}function ER(a,b,c,d){var e=BR(),g=a.isInline()?f.noop:hb.cached(function(){l=
FR.bind(a.doc().getOrDie(),function(b){e.dismiss();a.focus()})}),l=null,p=P.div();ea.add(p,[x.resolve("ui-container"),x.resolve("layer-above-editor")]);d.each(function(a){m.set(p,"data-ephox-editor",a)});var r=E.bind(p,"keydown",function(a){var b;if(b=k.contains(N.BACKSPACE(),a.raw().which))b=a.target(),b=f.curry(ra.is,b),b=!k.exists(["textarea","input",'[contenteditable="true"]'],b);b&&a.kill()});return{uiContainer:f.constant(p),uiOrigin:function(){return mg.relativeOrigin(p)},refresh:e.recalculate,
bounds:b,show:function(a,b){g();e.update(a,b);b.place(a)},destroy:function(){null!==l&&l.unbind();e.destroy();r.unbind();B.remove(p)}}}function GR(a,b,c){a=Xu(c,a);c=function(){var a=setTimeout(function(){clearTimeout(a);d.trigger.expired()},b);return{clear:function(){clearTimeout(a)}}};var d=D.create({expired:s([]),close:s([])}),e=0<b?c():{clear:f.noop};a.events.close.bind(function(){e.clear();d.trigger.close()});return{element:a.element,action:a.action,destroy:a.destroy,events:d.registry}}function HR(a,
b,c,d,e,g,l,p){var r=l.layout.build(a,b,c,d,e,g,l,p);b=function(a){eb.closest(a.target(),'input,div[contenteditable="true"],textarea,.ephox-polish-dialog.ephox-polish-help-modal,select,.ephox-polish-bottom-bar,.ephox-polish-side-bar')||a.prevent()};var h=[E.bind(r.element(),"mousedown",b),E.bind(c.uiContainer(),"mousedown",b)];(function(){var b=n.toggler(r.element(),x.resolve("content-ltr")),c=n.toggler(r.element(),x.resolve("content-rtl")),d=n.toggler(r.uiContainerParent(),x.resolve("content-ltr")),
e=n.toggler(r.uiContainerParent(),x.resolve("content-rtl")),l=function(){b.off();d.off()},p=function(){b.on();d.on()},h=function(){c.off();e.off()},z=function(){c.on();e.on()},k=function(){var a=g.uiDirection().isRtl(),b=a?z:h;(a?l:p)();b()};a.events.uiStale.bind(function(){Kv.readDir(a).fold(k,function(a){a.ltrdir().fold(l,f.noop,p);a.rtldir().fold(h,f.noop,z);b.isOn()===c.isOn()&&k()})})})();return{element:r.element,connect:r.connect,viewport:r.viewport,uiContainerParent:r.uiContainerParent,socket:r.socket,
showMessage:function(a,b,c){a=GR(a,b,c);b=f.curry(r.hideBanner,a);a.events.expired.bind(b);r.displayBanner(a);return b},displayBanner:r.displayBanner,hideBanner:r.hideBanner,init:r.init,refocus:r.tabstrip.refocus,tabstrip:r.tabstrip,refresh:r.refresh,position:r.position,state:r.state,destroy:function(){r.destroy();k.each(h,function(a){a.unbind()})},dialogContainer:r.dialogContainer}}function Sh(a){var b=void 0!==a?a.slice(0):[];return{pop:function(){return h.from(b.pop())},push:function(a){b.push(a)},
peek:function(){return h.from(b[b.length-1])},pushAll:function(a){b=b.concat(a)},clear:function(){b=[]},isEmpty:function(){return 0===b.length},size:function(){return b.length},state:function(){return b.slice(0)}}}function Kw(a){var b=D.create({changed:s([]),reset:s([])}),c=a.initial,d=Sh(a.undo),e=Sh(a.redo),g=function(a){d.isEmpty()&&d.push(c);d.push(a);b.trigger.changed()},l=function(a,b){b(d.peek().getOr(c),a)?(d.pop(),d.push(a)):(g(a),e.clear())};return{save:l,checkChanged:function(a,b){return!b(d.peek().getOr(c),
a)},undo:function(a,c){l(a,c);return d.pop().bind(function(a){d.isEmpty()||(e.push(a),b.trigger.changed());return d.peek()})},redo:function(a,b){b(d.peek().getOr(c),a)||e.clear();return e.pop().bind(function(a){g(a);return d.peek()})},reset:function(a){c=a;d.clear();e.clear();b.trigger.reset()},state:function(){return{initial:c,undo:d.state(),redo:e.state()}},rewrite:function(a){var g=a(c),l=k.map(d.state(),a);a=k.map(e.state(),a);c=g;d=Sh(l);e=Sh(a);d.isEmpty()&&e.isEmpty()||b.trigger.changed()},
events:b.registry}}function IR(a,b,c,d,e){var g=JR.fromState(Lw(e,KR)),l=D.create({changed:M([],g.events.changed),reset:M([],g.events.reset),undo:s([]),redo:s([])}),p=function(b,c){var d;if(!(d=""===b.raw()&&LR.isDefaultEmpty(c.raw()))){d=a.filterHtml(a.internal(),b.raw());var e=a.filterHtml(a.internal(),c.raw());d=d===e}return d},r=function(){var c=a.filterHtml(a.history(),b.getContent()),d=a.filterHtml(a.history(),b.serialize());return tk(c,d)},h=function(a){return b.deserialize(a.marked())};e=
function(){var a=r();g.undo(a,p).each(h);b.domChanged();b.toEditing();l.trigger.undo()};var w=function(){var a=r();g.redo(a,p).each(h);b.domChanged();b.toEditing();l.trigger.redo()},k=Qc.tap(function(a,b){if(a()){var c=r();g.save(c,b)}}),I=function(){k.instance(function(){return b.getSelectionStart().exists(ca.inBody)},p)},O=function(a){var b=k.control.isBlocked();k.control.block();a=a();b||k.control.unblock();return a},m=c.editor();m.events.enter.bind(I);m.on();d.events.undo.bind(e);d.events.redo.bind(w);
b.events.undoChanged.bind(I);b.events.loiter.bind(I);c=$a.ui({"undo.general":{toggle:!1,value:"undo",textkey:u.key("action.undo"),action:e,shortcut:!0,updateGeneral:f.identity,tooltip:Wa.get},"redo.general":{toggle:!1,value:"redo",textkey:u.key("action.redo"),action:w,shortcut:!0,updateGeneral:f.identity,tooltip:Wa.get}});return{undo:e,redo:w,save:I,checkChanged:function(){var a=r();return g.checkChanged(a,p)},reset:function(a){g.reset(tk(a,a))},destroy:function(){m.off();m.destroy()},state:function(){return Lw(g.state(),
MR)},rewrite:function(a,b){var c=function(c){return c.replace(a,b)};g.rewrite(function(a){return tk(c(a.raw()),c(a.marked()))})},transaction:function(a){I();a=O(a);I();return a},disable:O,replaceTop:function(){k.instance(f.constant(!0),f.constant(!0))},ui:c,events:l.registry}}function NR(a){var b=sc.unbindable(),c=D.create({onChange:s([])}),d=zc.last(function(){c.trigger.onChange()},1E3),e=function(){a.runOrOnLoaded(function(c){a.doc().each(function(a){b.set(E.bind(a,"scroll",function(a){d.throttle()}));
d.throttle()})})};a.isInline()||e();var g=function(){return a.body().bind(function(b){return a.win().map(function(a){var c=a.innerHeight,d=y.children(b);return k.findIndex(d,function(a){if(!A.isElement(a))return!1;a=a.dom().getBoundingClientRect();return a.top<c&&0<a.bottom}).map(function(a){var b=k.findIndex(d.slice(a+1),function(a){if(!A.isElement(a))return!1;a=a.dom().getBoundingClientRect();return!(a.top<c&&0<a.bottom)}).map(function(b){return a+1+b}).getOr(d.length);return d.slice(a,b)}).getOr([b])})}).getOr([])};
return{sections:function(){return g()},setOn:e,setOff:function(){b.clear()},viewport:function(){return{assess:function(b){if(!A.isElement(b))return xf.inView(b);var c=a.win().map(function(a){return a.innerHeight}).getOr(0),d=b.dom().getBoundingClientRect();c=0>d.bottom?xf.aboveView:d.top>c?xf.belowView:xf.inView;return c(b)}}},destroy:function(){b.clear();d.cancel()},events:c.registry}}function OR(a,b,c){var d=Yv(a),e=qR(c.profile),g=QQ(c.filters),l=SQ(),p=BQ(g,l,a.content,b,c.content),r=vR(b,g,p),
z=yR(p,a.shortcuts),w=a.ui.layout.bounds(a.ui),G=ER(p,w,b,a.ui.editorName),I=!1!==a.fullscreen.bundled?RQ(p,z,a.fullscreen):Aa.fullscreen(),O=pR(p,G,b,I,a.platform),m=HR(p,z,G,O,I,b,a.ui,c.ui),n=WQ(p,m),q=IR(g,p,n,z,c.undo),s=IQ(p,r,n,q),t=rR(p,n,q),v=YQ(g,p),x=LQ(p,q,a.editorcontext),y=NR(p),A=sR(m,a.registration),B=GQ(b,p,n,z,G,a.contextmenu),C=mw.onNodeChange(p,v),D=Ru(c.laundry);q.events.changed.bind(function(){D.setDirty(!0)});D.events.clean.bind(function(){p.undoChanged()});p.events.interaction.bind(function(a){!D.isDirty()&&
q.checkChanged()&&D.setDirty(!0)});var E=Y.detect(),F=E.deviceType.isTouch(),w=z.isHelpShortcutEnabled();c=!p.isInline()&&!F&&a.ui.showHelpHint;a=PR.editor(a.ui.editorName,a.ui.isFullScreen);var E=u.translate(E.os.isOSX()?"aria.help.mac":"aria.help.ctrl"),w=w?h.some(E):h.none(),H=(p.isInline()?Q.inline:Q.editor)(m.element(),p.element(),a,w,c),K=h.none();p.runOrOnLoaded(function(a){var b=rk.handle(a,function(a){J.closest(a.target(),"a").each(function(b){J.closest(b,'[contenteditable="false"]').each(a.kill)})});
K=h.some(b);F&&(Q.label(a,u.translate("aria.label.content")),Q.region(m.viewport()),Q.region(m.tabstrip.element()))});return{region:f.constant(b),communication:f.constant(d),profile:f.constant(e),filter:f.constant(g),internalevents:f.constant(l),content:f.constant(p),keyboard:f.constant(n),shortcuts:f.constant(z),smartview:f.constant(G),undo:f.constant(q),del:f.constant(s),proximity:f.constant(t),operations:f.constant(v),context:f.constant(x),fullscreen:f.constant(I),platform:f.constant(O),registration:f.constant(A),
contextmenu:f.constant(B),ui:f.constant(m),selections:f.constant(r),insertFilteredHtmlAtCursor:function(a){a=g.filterHtml(g.input(),a);p.insertHtmlAtCursor(a)},viewablearea:f.constant(y),laundry:f.constant(D),state:function(){return{ui:m.state(),undo:q.state(),content:p.state(),profile:e.state(),laundry:D.state()}},destroy:function(){K.each(function(a){a.unbind()});k.each([p,y,C,B,A,O,I,x,v,t,q,s,G,z,n,r,g,e,d,m],function(a){a.destroy?a.destroy():console.log("Feature missing destroy method: ",a)});
H.destroy()}}}function Mw(){var a=Nw.single(),b=D.create({upload:s(["assets"])});a.events.upload.bind(function(a){ud.multiple(a.files()).get(function(a){b.trigger.upload(a)})});return{element:a.element,open:a.open,destroy:a.destroy,events:b.registry}}function Po(a){var b=h.none(),c=function(c){d();c=Ch(c);c.events.close.bind(function(){b=h.none()});b=h.some(c);a().displayBanner(c)},d=function(){b.each(a().hideBanner);b=h.none()};return{show:c,showKey:function(a){var b=ae.get.apply(void 0,arguments);
c(b)},hide:d,reset:function(){b=h.none()}}}function Ow(a,b){var c=Z.merge({layerClass:void 0,zindex:void 0,padding:"0px"},a),d=D.create({exiting:s([])}),e=q.fromTag("div");Q.presentation(e);n.add(e,Db.resolve("modal"));n.add(e,Db.resolve("modal-hidden"));t.setAll(e,{position:"fixed",left:c.padding,top:c.padding,right:c.padding,bottom:c.padding});void 0!==c.background&&t.set(e,"background",c.background);void 0!==c.zindex&&t.set(e,"z-index",c.zindex);void 0!==c.layerClass&&n.add(e,c.layerClass);var g=
void 0!==b?b:J.first("body").getOrDie();v.append(g,e);return{element:function(){return e},content:function(a){B.empty(e);v.append(e,a)},show:function(){n.add(g,Db.resolve("modal-active"));n.remove(e,Db.resolve("modal-hidden"));n.add(e,Db.resolve("modal-show"))},hide:function(){n.remove(g,Db.resolve("modal-active"));n.add(e,Db.resolve("modal-hidden"));n.remove(e,Db.resolve("modal-show"))},destroy:function(){B.remove(e);n.remove(g,Db.resolve("modal-active"))},events:d.registry}}function QR(a,b){var c=
kg.onChange(),d=D.create({exiting:s([])}),e=Z.merge({layerClass:null,zindex:null,padding:"0px",background:"rgba(0, 0, 0, 0.5)"},a),g=q.fromTag("div");Q.presentation(g);n.add(g,Db.resolve("moodal"));n.add(g,Db.resolve("moodal-hidden"));t.setAll(g,{left:e.padding,top:e.padding,right:e.padding,bottom:e.padding});e.background&&t.set(g,"background",e.background);e.zindex&&t.set(g,"z-index",e.zindex);e.layerClass&&n.add(g,e.layerClass);var l=void 0!==b?b:J.first("body").getOrDie();v.append(l,g);var p=q.fromTag("div");
Q.presentation(p);n.add(p,Db.resolve("moodal-flexor"));v.append(g,p);var r=function(){t.setAll(p,{top:window.scrollY+"px",height:window.innerHeight+"px"})},h=function(){w.unbind();c.destroy();B.remove(g);n.remove(l,Db.resolve("modal-active"))},w=E.bind(g,"click",function(){d.trigger.exiting();h()});c.events.ready.bind(function(){r()});return{element:f.constant(g),content:function(a){B.empty(p);v.append(p,a)},show:function(){n.add(l,Db.resolve("moodal-active"));n.remove(g,Db.resolve("moodal-hidden"));
n.add(g,Db.resolve("moodal-show"));r()},hide:function(){n.remove(l,Db.resolve("moodal-active"));n.add(g,Db.resolve("moodal-hidden"));n.remove(g,Db.resolve("moodal-show"))},destroy:h,events:d.registry}}function RR(a){var b=Z.deepMerge({layerClass:void 0,zindex:void 0,modal:{}},a);a=void 0!==a.container?a.container:J.first("body").getOrDie();var c=(Y.detect().os.isiOS()?QR:Ow)(b.modal,a),d=c.element(),e=D.create({exiting:M([],c.events.exiting)});void 0!==b.zindex&&t.set(d,"z-index",b.zindex);void 0!==
b.layerClass&&n.add(d,b.layerClass);n.add(d,Db.resolve("centered-dialog"));v.append(a,d);return{content:c.content,show:function(){c.show()},hide:function(){c.hide()},destroy:function(){B.remove(d);c.destroy()},events:e.registry}}function SR(){var a=function(a,b){n.add(a,x.resolve(b));return a},b=a(P.div(),"dialog"),c=a(P.div(),"top"),d=a(P.span(),"dialog-title"),e=a(P.span(),"dialog-close"),g=a(P.div(),"dialog-content"),l=a(P.div(),"dialog-controls");m.set(b,"tabindex",-1);m.set(e,"tabindex",-1);
Q.button(e,u.translate("button.close"));var f=E.bind(e,"click",function(){r.trigger.close()});K.append(b,[c,g,l]);K.append(c,[d,e]);var r=D.create({close:s([])});return{element:function(){return b},focus:function(){T.focus(b)},setTitle:function(a){B.empty(d);v.append(d,q.fromText(a));TR||(Q.dialog(b,a),a=La.generate("ephox-aria"),m.set(d,"id",a),Q.labelledBy(b,a))},setContent:function(a){B.empty(g);v.append(g,a);a=La.generate("ephox-aria");m.set(g,"id",a);Q.describedBy(b,a)},setControls:function(a){B.empty(l);
v.append(l,a)},close:function(){r.trigger.close()},destroy:function(){B.remove(b);f.unbind()},events:r.registry}}function ce(a){var b=SR(),c=h.none(),d=RR({container:a,layerClass:x.resolve("layer-dialog"),modal:{layerClass:x.resolve("layer-below-dialog")}});d.content(b.element());d.events.exiting.bind(function(){gc.unlock([ca.body()]);b.close()});var e=function(){d.hide();gc.unlock([ca.body()])};return{element:b.element,destroy:function(){b.destroy();c.each(function(a){a.unbind()});e();d.destroy()},
setTitle:b.setTitle,setContent:b.setContent,setControls:b.setControls,show:function(){d.content(b.element());d.show();b.focus();UR.deviceType.isTouch()&&(c=h.some(E.bind(b.element(),"click",function(a){a.stop()})));gc.lock([ca.body()])},hide:e,events:b.events}}function Pw(a,b,c,d,e,g){var l=Wc.okCancel("file-upload"),f=d(e),r=null,h=Y.detect(),w=D.create({close:s([]),analytics:s(["message"]),escaped:s([]),insertAssets:s(["assets"]),opened:s([])}),k=function(){r.unbind();m.hide();w.trigger.close();
w.trigger.analytics("insert_image_closed");I()},I=function(){f.reset();l.ok().disable()},m=ce(c);m.setTitle(u.translate("fileupload.title"));m.setContent(f.element());m.setControls(l.element());m.events.close.bind(k);n.add(m.element(),x.resolve("imageupload-dialog"));var Va=g(m.element(),f.keylayouts(),pa.dialog({cancel:l.cancel().element(),ok:l.ok().element(),execute:qb.click,index:0}));f.events.upload.bind(function(a){w.trigger.insertAssets(a.assets());k()});f.events.dragndrop.bind(function(a){w.trigger.insertAssets(a.assets());
k()});f.events.success.bind(function(a){l.ok().enable()});f.events.lookup.bind(function(a){l.ok().disable()});f.events.setActive.bind(function(a){I()});l.events.ok.bind(function(){f.selected().each(function(a){w.trigger.insertAssets([a]);k()})});l.events.cancel.bind(k);I();return{show:function(){h.deviceType.isiOS()&&(Q.hidden(b.element(),!0),setTimeout(function(){f.focus();Q.hidden(b.element(),!1)},250));m.show();Va.enter();w.trigger.analytics("insert_image_opened");w.trigger.opened();var c=q.fromDom(window);
r=E.bind(c,"orientationchange",function(){a.foreground()})},element:m.element,events:w.registry,destroy:function(){Va.exit();Va.destroy();m.hide();m.destroy();f.destroy();l.destroy()}}}function VR(a){var b=D.create({success:s(["assets"]),invalidUrl:s(["url"]),empty:s([]),lookup:s([])}),c=[],d=P.div(),e=n.toggler(d,kb.resolve("url-indicator-loading")),g=Fo();g.events.start.bind(function(){e.on()});g.events.fail.bind(function(a){e.off();b.trigger.invalidUrl(a.url());c=[]});g.events.success.bind(function(a){e.off();
var d=$.get(z);a.url()===d?ud.fromImages([a.image()]).get(function(a){c=a;b.trigger.success(c)}):g.go(d)});var l=P.div();n.add(l,kb.resolve("url-selector"));var p=q.fromTag("label");a=q.fromText(a("fileupload.web.url"));var r=La.generate(kb.resolve("input-id"));m.set(p,"for",r);n.add(p,kb.resolve("url-input-label"));v.append(p,a);var z=q.fromTag("input");m.setAll(z,{id:r,type:"url",placeholder:"http://www.example.com/image.jpg"});n.add(z,kb.resolve("url-input"));Qw.os.isAndroid()&&m.set(z,"autofocus",
"true");Qw.browser.isSafari()&&m.setAll(z,{autocomplete:"off",autocorrect:"off",autocapitalize:"off",spellcheck:"false"});Q.required(z);a=q.fromHtml("<span role=presentation>*</span>");n.add(a,kb.resolve("url-input-required"));v.append(p,a);K.append(l,[p,z,d]);var w=zc.adaptable(function(a){""!==a&&g.go(a)},1E3),k=E.bind(z,"input",function(){var a=$.get(z);""===a.trim()?(b.trigger.empty(),w.throttle("")):(b.trigger.lookup(),w.throttle(a))});return{element:f.constant(l),input:f.constant(z),events:b.registry,
reset:function(){e.off();$.set(z,"")},focus:function(){T.focus(z)},selected:function(){return h.from(c[0])},destroy:function(){k.unbind()}}}function Rw(a){var b=D.create({lookup:s([]),success:s([])}),c=P.span(),d=q.fromHtml('<div role=presentation><span role=presentation class="'+kb.resolve("url-input-required")+'">*</span> '+a("fileupload.weburlhelp")+"</div>"),e=La.generate(kb.resolve("help-message"));m.set(d,"id",e);n.add(d,kb.resolve("fileupload-weburl-help"));var g=q.fromHtml("<div role=presentation><div role=presentation>"+
a("fileupload.invalid1")+"</div><div role=presentation>"+a("fileupload.invalid2")+"</div><div role=presentation>"+a("fileupload.invalid3")+"</div></div>");n.add(g,kb.resolve("fileupload-weburl-help"));n.add(g,kb.resolve("fileupload-weburl-help-invalid"));var l=function(a){B.empty(w);v.append(w,a)},p=function(a){var b=q.fromTag("img");v.append(w,b);WR.previewAsset(a,b)},r=P.div();n.add(r,kb.resolve("web-url-container"));var h=VR(a),w=P.div();n.add(w,kb.resolve("image-preview-container"));a=La.generate("ephox-hermes-url-state");
Q.live(w,a,"polite");Q.controls(h.input(),a);Q.describedBy(h.input(),e);e=P.div();n.add(e,kb.resolve("hbox"));K.append(e,[h.element(),w,d]);v.append(r,e);h.events.lookup.bind(function(a){b.trigger.lookup();B.empty(w)});h.events.success.bind(function(a){k.each(a.assets(),p);b.trigger.success()});h.events.invalidUrl.bind(function(){l(g)});h.events.empty.bind(function(){l(c)});l(c);return{element:f.constant(r),selected:h.selected,input:h.input,events:b.registry,focus:h.focus,reset:function(){l(c);h.reset()},
destroy:function(){h.destroy()}}}function XR(a){a=P.div();var b=Rw(u.translate),c=D.create({upload:s(["assets"]),dragndrop:s(["assets"]),dragover:s([]),dragleave:s([]),drop:s([]),setActive:s(["item"]),lookup:M([],b.events.lookup),success:M([],b.events.success)}),d=[function(){var a=b.input();return ba.input(a,pa.input({}))}()];v.append(a,b.element());return{element:f.constant(a),events:c.registry,keylayouts:f.constant(d),reset:function(){b.reset()},focus:b.focus,selected:b.selected,destroy:function(){b.destroy()}}}
function YR(a,b,c,d,e){var g=D.create({insertAsset:s(["asset"]),open:s([]),done:s([]),abort:s([])}),l=q.fromDom(document);d=bb.mainmenu(l,"file-upload-menu",[]);var p=function(a,b){var c=bb.item(l,a,u.get(b)+"...");Q.menuItem(c,h.some(u.get(b)),h.none());return c},r=p("from-url",u.key("image.from-url")),p=p("from-camera",u.key("image.from-camera")),r=!0===e.allowLocalImages?[r,p]:[r],z=Mw();z.events.upload.bind(function(a){h.from(a.assets()[0]).each(function(a){G();g.trigger.insertAsset(a)})});var w=
function(a){a=a.assets();k.each(a,function(a){g.trigger.insertAsset(a)})},G=function(){tw.target(c.uiContainer(),"mousedown")};K.append(d,r);d={name:"image",textkey:u.key("image.menu-button"),menuTextkey:u.key("image.menu"),icon:h.none,tooltip:Wa.get,contentFocus:!1,toggle:!1};r=[[{handlers:f.constant({"from-url":function(d){g.trigger.open();var l=Pw(a,b,c.uiContainer(),XR,e,ba.dialog);l.events.insertAssets.bind(w);d=function(){a.focus();l.destroy()};l.events.opened.bind(G);l.events.close.bind(d);
l.events.escaped.bind(d);l.show()},"from-camera":function(){g.trigger.open();z.open()}}),destroyer:f.noop,refreshMenu:f.noop,structure:f.constant({}),submenus:f.constant([]),items:f.constant(r),events:g.registry}]];d=Sd.menu(d,r,u.get,Gh.menu({focusActive:!0}));r={label:f.constant(!1),hasDialog:!0};return Z.merge(d,r)}function ZR(a){var b=q.fromTag("button");m.set(b,"type","button");n.add(b,Ta.resolve("independent-button"));return fo(b,h.none(),a)}function $R(a){var b=Nw.multiple(),c=P.div();n.add(c,
kb.resolve("drop-container"));var d=Sw.register(c,h.some(kb.resolve("drop-active")),!1),e=ZR();e.content(qc(a("fileupload.chooseimage")));n.add(e.element(),kb.resolve("default-action"));n.add(e.element(),kb.resolve("file-selector-trigger"));var g=q.fromTag("p");n.add(g,kb.resolve("help-text-drop-icon"));var l=q.fromTag("p");n.add(l,kb.resolve("help-text-drop"));ha.disable(l);v.append(l,q.fromText(a("fileupload.dropimages")));K.append(c,[g,l,e.element(),b.element()]);return{container:f.constant(c),
dropper:f.constant(d),picker:f.constant(b),button:f.constant(e),focus:function(){T.focus(e.element())},destroy:function(){b.destroy();d.destroy();e.destroy()}}}function aS(a){var b=$R(a);a=b.picker();var c=b.container(),d=b.dropper(),e=b.button(),g=D.create({upload:s(["assets"]),dragndrop:s(["assets"]),dragover:M([],d.events.dragover),dragleave:M([],d.events.dragleave),drop:M([],d.events.drop)}),l=function(a){return function(b){ud.multiple(b.files()).get(function(b){a(b)})}};a.events.upload.bind(l(g.trigger.upload));
d.events.dragndropimages.bind(l(g.trigger.dragndrop));e.events.click.bind(a.open);return{element:f.constant(c),button:f.constant(e),input:a.element,focus:b.focus,events:g.registry,destroy:function(){b.destroy()}}}function Tw(a,b,c){var d=[],e=C.immutable("view","item","container"),g=D.create({setActive:s(["item"])}),l=P.div();n.add(l,Ta.resolve("tablayout-wrapper"));n.add(l,Ta.resolve("tablayout-wrapper-"+a));void 0!==c&&n.add(l,c);var p=P.div();n.add(p,Ta.resolve("tablayout-body"));var r=q.fromTag("div");
n.add(r,Ta.resolve("tablayout-menu"));K.append(l,[r,p]);var h=function(a){var b=go(a);b.content(qc(a));n.add(b.element(),Ta.resolve("tablayout-item"));return b},w=[],G=k.map(b,function(a){var b=h(a.heading());m.set(b.element(),"data-ephox-pastry-tablayout-tab",a.heading());v.append(r,b.element());var c=q.fromTag("div");n.add(c,Ta.resolve("tablayout-content"));m.set(c,"data-ephox-pastry-tablayout-view",a.heading());v.append(p,c);w.push(c);var g=e(a,b,c);d.push(g);b.events.click.bind(function(){I(g)});
return b}),I=function(a){k.each(d,function(b){var c=b.view(),d=b.item(),e=b.container();b===a?(n.add(e,Ta.resolve("tablayout-active")),d.select(),c.renderer().render(e),g.trigger.setActive(d)):(n.remove(e,Ta.resolve("tablayout-active")),d.deselect())})};I(d[0]);return{element:function(){return l},tabs:f.constant(G),body:f.constant(p),menu:f.constant(r),tabContent:f.constant(w),destroy:function(){k.each(G,function(a){a.destroy()})},events:g.registry}}function Uw(a){return{render:function(b){B.empty(b);
v.append(b,a)}}}function bS(a){var b=P.div(),c=aS(u.translate),d=Rw(u.translate),e=function(a,b){var c=u.translate(a),d=Uw(b);return yf(c,d)},g=e("fileupload.tabweburl",d.element());a=(a.allowLocalImages?[e("fileupload.tablocal",c.element())]:[]).concat(g);var l=Tw(Qo.top(),a,"file-upload");Q.hidden(c.input(),!0);e=function(){var a=c.button();n.add(a.element(),x.resolve("dialog-button-default"));var b=Y.detect().browser.isFirefox(),d={execute:qb.button},b=b?Z.merge(d,{process:f.noop}):d,b=pa.button(b);
return ba.button(a.element(),b)}();Q.tabList(l.menu());var g=function(){var a=d.input();return ba.input(a,pa.input({}))}(),p=ba.flow(l.menu(),{item:f.constant(".ephox-pastry-statebutton")},pa.flow({trigger:qb.button,executeOnArrow:!0})),e=[p].concat(e).concat(g),r=l.tabs();k.each(l.tabContent(),function(a){Q.tabPanel(a)});k.each(a,function(a,b){var c=r[b];Q.tabButton(c.element(),a.heading());n.add(c.element(),x.resolve("tab-button"));c.events.click.bind(function(){p.clickFocus(c.element())})});k.each(l.tabContent(),
function(a,b){var c=r[b],d=La.generate("ephox-aria");m.set(c.element(),"id",d);Q.labelledBy(a,d)});a=D.create({upload:M(["assets"],c.events.upload),dragndrop:M(["assets"],c.events.dragndrop),dragover:M([],c.events.dragover),dragleave:M([],c.events.dragleave),drop:M([],c.events.drop),lookup:M([],d.events.lookup),success:M([],d.events.success),setActive:M(["item"],l.events.setActive)});l.events.setActive.bind(function(a){a=m.get(a.item().element(),"data-ephox-pastry-tablayout-tab");a===u.translate("fileupload.tablocal")?
c.focus():a===u.translate("fileupload.tabweburl")&&d.focus()});v.append(b,l.element());return{element:f.constant(b),events:a.registry,focus:d.focus,keylayouts:f.constant(e),reset:function(){d.reset()},selected:d.selected,destroy:function(){l.destroy();c.destroy();d.destroy()}}}function md(a,b,c){a=parseFloat(a);a>c?a=c:a<b&&(a=b);return a}function Ge(a,b){var c,d,e,g,l=[],f=Array(10);for(c=0;5>c;c++){for(d=0;5>d;d++)l[d]=b[d+5*c];for(d=0;5>d;d++){for(e=g=0;5>e;e++)g+=a[d+5*e]*l[e];f[d+5*c]=g}}return f}
function Ro(a,b){b=md(b,0,1);return a.map(function(a,d){a=0===d%6?1-(1-a)*b:a*b;return md(a,0,1)})}function uk(a,b){return a.toCanvas().then(function(c){var d=a.getType(),e=fc.get2dContext(c),g=e.getImageData(0,0,c.width,c.height),l=g.data,f,r,h,k,G,I=b[0],m=b[1],n=b[2],q=b[3],s=b[4],t=b[5],u=b[6],v=b[7],x=b[8],y=b[9],A=b[10],B=b[11],D=b[12],C=b[13],E=b[14],F=b[15],J=b[16],H=b[17],K=b[18],L=b[19];for(G=0;G<l.length;G+=4)f=l[G],r=l[G+1],h=l[G+2],k=l[G+3],l[G]=f*I+r*m+h*n+k*q+s,l[G+1]=f*t+r*u+h*v+k*
x+y,l[G+2]=f*A+r*B+h*D+k*C+E,l[G+3]=f*F+r*J+h*H+k*K+L;e.putImageData(g,0,0);return vd.fromCanvas(c,d)})}function Vw(a,b){return a.toCanvas().then(function(c){return cS(c,a.getType(),b)})}function cS(a,b,c){var d=fc.get2dContext(a),e,g;e=d.getImageData(0,0,a.width,a.height);g=d.getImageData(0,0,a.width,a.height);g=function(a,b,c){function d(a,b,c){a>c?a=c:a<b&&(a=b);return a}var e,g,f,h,k,m,n,q,s,t,u,v,x,y;f=Math.round(Math.sqrt(c.length));h=Math.floor(f/2);e=a.data;g=b.data;x=a.width;y=a.height;for(k=
0;k<y;k++)for(a=0;a<x;a++){for(t=m=n=q=0;t<f;t++)for(s=0;s<f;s++)u=d(a+s-h,0,x-1),v=d(k+t-h,0,y-1),u=4*(v*x+u),v=c[t*f+s],m+=e[u]*v,n+=e[u+1]*v,q+=e[u+2]*v;u=4*(k*x+a);g[u]=d(m,0,255);g[u+1]=d(n,0,255);g[u+2]=d(q,0,255)}return b}(e,g,c);d.putImageData(g,0,0);return vd.fromCanvas(a,b)}function Ww(a){return function(b,c){return b.toCanvas().then(function(d){var e=b.getType(),g=fc.get2dContext(d),l,f=Array(256);for(l=0;l<f.length;l++)f[l]=a(l,c);l=g.getImageData(0,0,d.width,d.height);var r=l.data,h;
for(h=0;h<r.length;h+=4)r[h]=f[r[h]],r[h+1]=f[r[h+1]],r[h+2]=f[r[h+2]];g.putImageData(l,0,0);return vd.fromCanvas(d,e)})}}function ng(a){return function(b,c){return uk(b,a(de.identity(),c))}}function Xw(a){return function(b){return Vw(b,a)}}function Yw(a,b,c){var d=og.getWidth(a),e=og.getHeight(a),d=b/d,g=c/e,e=!1;if(0.5>d||2<d)d=0.5>d?0.5:2,e=!0;if(0.5>g||2<g)g=0.5>g?0.5:2,e=!0;a=dS(a,d,g);return e?a.then(function(a){return Yw(a,b,c)}):a}function dS(a,b,c){return new Xd(function(d){var e=og.getWidth(a),
g=og.getHeight(a),l=Math.floor(e*b),f=Math.floor(g*c),r=fc.create(l,f);fc.get2dContext(r).drawImage(a,0,0,e,g,0,0,l,f);d(r)})}function eS(a,b,c,d,e){var g=a.content(),l=a.smartview(),p=a.ui().tabstrip,r=Po(a.ui),h=Th.processResults(r.showKey),k=function(d){a.undo().save();d=fS.validate(d);(d.failure()?f.curry(r.showKey,"fileupload.invalidimages"):r.hide)();c.assets(d.assets()).map(h).get(function(a){So.insertAssets(g,b,a)})},G=function(a){k(a.assets())};d.events.insertAssets.bind(function(a){e.allowLocalImages?
k(a.assets()):ae.getOrLog("errors.dragdrop.images.disallowed")});var I=g.cursor();d=D.create({analytics:s(["message"])});return{ui:$a.ui({"fileupload.general":f.constant({value:"image",textkey:u.key("insert.image"),icon:!0,updateGeneral:f.identity,action:function(){var b=Pw(g,p,l.uiContainer(),bS,e,ba.dialog);b.events.insertAssets.bind(G);b.events.close.bind(function(){g.focus();b.destroy()});b.events.escaped.bind(function(){g.focus();b.destroy()});a.content().hideInlineSelection();b.show()},hasDialog:!0}),
"fileupload.menu":function(){var c=YR(g,p,l,a.ui(),e);c.events.insertAsset.bind(function(a){I.get().bind(function(c){So.insertMobileAssets(g,c,b,[a.asset()])})});c.events.open.bind(function(a){g.getSelection().each(I.save)});return c},"camera.general":function(){var a=Mw();a.events.upload.bind(function(a){I.get().bind(function(c){So.insertMobileAssets(g,c,b,a.assets())})});return{value:"image",textkey:u.key("image.from-camera"),icon:!0,updateGeneral:f.identity,action:function(){g.getSelection().each(I.save);
g.toReading();a.open()},hasDialog:!0}}},f.identity),events:d.registry,destroy:function(){}}}function Zw(a,b,c){var d=function(){b.counter.update(0,0)},e=function(e){e.all.fold(d,function(d){b.counter.update(e.pos+1,d.length);d=d[e.pos][0];a.setSelection(d,0,d,0);c.element(d)})},g=function(){c.cursor();d()},l=function(a,b,c){ee.clear(a);ee.search(a,b,c).fold(g,e);return h.some(b)},p=function(a,b){var c=(!0===b?ee.prev:ee.next)(a);c.fold(g,e);return c},r=function(b){a.body().each(function(c){ee.replace(c,
b).fold(g,e);a.domChanged();a.undoChanged()})},z=h.none(),k=function(){z=h.none()};b.events.search.bind(function(b){a.body().each(function(a){l(a,b.term(),b.matchcase())})});b.events.next.bind(function(){a.body().each(function(a){p(a,!1)})});b.events.prev.bind(function(){a.body().each(function(a){p(a,!0)})});b.events.replace.bind(function(a){r(a.value())});b.events.replaceAll.bind(function(b){a.body().each(function(c){ee.replaceAll(c,b.value()).fold(g,e);a.undoChanged()})});b.events.interactive.bind(function(b){a.body().each(function(a){z=
l(a,b.term(),b.matchcase())})});b.events.encase.bind(function(b){a.body().each(function(a){z.each(function(c){l(a,c,b.matchcase())})})});b.events.clear.bind(function(b){a.body().each(function(a){ee.clear(a);k()})});return{zeroCount:d,updateMatch:e,enterSearchAction:function(c,d){a.body().each(function(a){var e=b.getCase();z=z.filter(function(a){return c===a}).fold(function(){return l(a,c,e)},function(b){return p(a,d.isShift()).fold(function(){return l(a,c,e)},f.constant(z))})});(0===c.length?b.counter.hide:
b.counter.show)()},enterReplaceAction:r,clearLastSearch:k}}function $w(a){var b=a.content();a=function(a){b.elementIntoView(a)};var c=function(){b.cursorIntoView()};a={element:ax?a:f.curry(uf.elementIntoView,b),cursor:ax?c:f.curry(uf.cursorIntoView,b)};return{get:f.constant(a)}}function bx(){var a=P.div();n.add(a,x.resolve("search-widget"));var b=vk.searchField(),c=(Mb.isSmallTouch()?vk.dummyCounter:vk.counter)(b.input()),d=vk.replaceDrawer(),e=d.element(),g=D.create({next:M([],b.events.next),prev:M([],
b.events.previous),replace:s(["value"]),replaceAll:s(["value"]),search:s(["term","matchcase","isShift"]),interactive:s(["term","matchcase"]),encase:s(["matchcase"]),clear:s([])});ha.setPressed(d.matchcase.element(),d.matchcase.selected());var l=hv(e);Q.linkElements(b.moreToggle.element(),e);cx(l.visible(),b.moreToggle.element(),e);b.moreToggle.events.click.bind(function(){(l.visible()?l.hide:l.show)()});b.events.input.bind(function(){var a=$.get(b.input()),d=ia.get(b.element());t.set(c.element(),
"height",d+"px");a.length>=gS?r.throttle(a):(g.trigger.clear(),c.hide(),c.update(0,0))});d.replace.events.click.bind(function(){var a=$.get(d.input.element());g.trigger.replace(a)});d.replaceAll.events.click.bind(function(){var a=$.get(d.input.element());g.trigger.replaceAll(a)});d.matchcase.events.click.bind(function(){ha.setPressed(d.matchcase.element(),d.matchcase.selected());g.trigger.encase(p())});var p=function(){return d.matchcase.selected()},r=zc.adaptable(function(a){c.show();g.trigger.interactive(a,
p())},500),h=E.bind(e,"transitionend",function(a){"opacity"===a.raw().propertyName&&n.remove(e,"ephox-chameleon-toolbar-more-open");cx(l.visible(),b.moreToggle.element(),e)});K.append(a,[c.element(),b.element(),d.element()]);return{element:f.constant(a),search:b,focus:function(){b.focus()},drawer:d,counter:c,getCase:p,destroy:function(){h.unbind();b.destroy();d.destroy();c.destroy();l.destroy();B.remove(a)},events:g.registry}}function hS(a){var b=a.content(),c=bx();a=$w(a);var d=Zw(b,c,a.get()),e=
function(a){var c=a.endContainer();a=a.endOffset();b.setSelection(c,a,c,a)};a=pa.button({execute:qb.button});a=[ba.input(c.search.input(),pa.input({execute:function(a,b){d.enterSearchAction(a,b)}})),ba.button(c.search.previous(),a),ba.button(c.search.next(),a),ba.button(c.search.moreToggle.element(),a),ba.input(c.drawer.input.element(),pa.input({execute:function(a,b){d.enterReplaceAction(a)}})),ba.button(c.drawer.matchcase.element(),a),ba.button(c.drawer.replace.element(),a),ba.button(c.drawer.replaceAll.element(),
a)];var g=D.create({done:s([])});return{element:c.element,anchorName:f.constant("search-widget-anchor"),textkey:f.constant("action.search"),dialogTextkey:f.constant("search.menu"),destroy:function(){c.destroy()},keylayouts:f.constant(a),openWidget:function(){d.enterSearchAction($.get(c.search.input()),Xc.shiftKeyState(!1))},closeWidget:function(){b.body().each(function(a){b.getSelection().each(e);ee.clear(a)});d.clearLastSearch()},events:g.registry}}function iS(a,b){var c=a.content(),d=bx(),e=$w(a),
g=Zw(c,d,e.get()),e=D.create({toEditor:s([])}),l=Uh.text(d.search.element(),d.search.input());return{element:d.element,addEscape:function(a){v.append(d.search.element(),a.element())},open:function(){g.enterSearchAction($.get(d.search.input()),Xc.shiftKeyState(!1));setTimeout(d.focus,250)},close:function(){c.body().each(function(a){ee.clear(a).fold(g.zeroCount,g.updateMatch)});g.clearLastSearch()},execute:function(a){if(T.hasFocus(d.search.input())){a=Xc.shiftKeyState(!0===a.raw().shiftKey);var b=
$.get(d.search.input());g.enterSearchAction(b,a)}else a=$.get(d.drawer.input.element()),g.enterReplaceAction(a)},type:f.constant(l),events:e.registry,destroy:function(){d.destroy()}}}function wk(a,b){jS(b);var c=a.content(),d=a.ui().tabstrip.element();n.add(b.element(),x.resolve("slick-widget"));kS.browser.isSafari()&&k.each(X.descendants(b.element(),"input,textarea"),function(a){m.setAll(a,{autocomplete:"off",autocorrect:"off",autocapitalize:"off",spellcheck:"false"})});var e=function(){var a=X.descendants(b.element(),
"input,textarea");k.findIndex(a,T.hasFocus).each(function(b){b=(b+1)%a.length;0<=b&&b<a.length&&T.focus(a[b])})},g=Uh.cata(b.type(),function(a,c){n.add(a,x.resolve("slick-widget-field"));n.add(c,x.resolve("slick-widget-input"));var d=E.bind(b.element(),"keydown",function(a){var c={original:f.constant(a)};ld.isEnter(c)?(b.execute(a),a.kill()):ld.isTab(c)?(e(),a.kill()):ld.isEscape(c)&&(G(),a.kill())}),g=E.bind(c,"blur",function(a){h.from(a.raw().relatedTarget).each(function(a){!0===sa.ancestor(q.fromDom(a),
function(a){return n.has(a,"ephox-polish-mobile-toolbar-hide")}).isSome()&&To.refreshInput(c)})});return{destroy:function(){d.unbind();g.unbind()}}}),l=ac("shizniz"),p=u.translate("toolbar.backbutton"),r=Qd([ja.resolve("icon")]);l.content(r);m.set(l.element(),"title",p);Q.toolbarButton(l.element(),p,!1,!1);b.addEscape(l);var z=wa(!1);v.prepend(d,b.element());var w=D.create({done:s([]),abort:s([]),open:s([]),close:s([])}),G=function(){O();c.toEditing()};b.events.toEditor.bind(G);var I=hb.cached(function(a){return X.children(d,
"."+ja.resolve("toolbar"))}),O=function(){b.close();z.set(!1);k.each(I(),function(a){n.remove(a,x.resolve("mobile-toolbar-hide"))});n.remove(b.element(),x.resolve("mobile-toolbar-show"));n.add(b.element(),x.resolve("mobile-toolbar-hide"));w.trigger.close()};c.events.mousedown.bind(O);O();l.events.click.bind(function(){G();w.trigger.abort()});return{element:b.element,destroy:function(){c.events.mousedown.unbind(O);g.destroy();b.destroy()},events:w.registry,open:function(){var a=E.bind(d,"transitionend",
function(){b.open();w.trigger.open();a.unbind()});z.set(!0);k.each(I(),function(a){n.remove(a,x.resolve("mobile-toolbar-show"));n.add(a,x.resolve("mobile-toolbar-hide"))});n.remove(b.element(),x.resolve("mobile-toolbar-hide"));n.add(b.element(),x.resolve("mobile-toolbar-show"))},close:O,isOpen:function(){return z.get()}}}function lS(a){var b=a.filter();Xc.configure(b);var c=D.create({analytics:s(["message"])}),d=hb.cached(function(){var b=mS(a);b.events.open.bind(function(){c.trigger.analytics("search_tool_open")});
b.events.abort.bind(b.close);return b}),b=function(){var a=d();(a.isOpen()?a.close:a.open)()};a.shortcuts().events.search.bind(b);return{ui:$a.ui({"find.general":{toggle:!1,hasPopup:!1,value:Uo,textkey:u.key("action.search"),action:b,updateGeneral:function(a){n.add(a.element(),x.resolve(Uo))},icon:!0,tooltip:Wa.get}}),events:c.registry,destroy:function(){d().destroy()}}}function Vh(a,b){var c=ac();ea.add(c.element(),a);Q.button(c.element(),u.translate(b));m.set(c.element(),"title",u.translate(b));
return c}function nS(a,b){var c=oS(xk.configuration()),d=c.i18nNamespace(),e=c.cssNamespace(),g=P.div();n.add(g,x.resolve("fontsize-widget"));var l=P.span();n.add(l,x.resolve("fontsize-widget-field"));var p=P.div();n.add(p,x.resolve("fontsize-widget-exit"));var r=q.fromTag("input");n.add(r,x.resolve("fontsize-widget-input"));Q.label(r,u.translate(d+".stepper.input"));var h=function(b){n.remove(r,x.resolve("slick-widget-invalid-input"));Q.invalid(r,!1);$.set(r,b);He.partialSize(a.content(),a.undo(),
a.selections(),b);Oa.speak(ca.body(),b)},k=function(){n.add(r,x.resolve("slick-widget-invalid-input"));Q.invalid(r,!0);Oa.speak(ca.body(),u.translate("aria.input.invalid"));T.focus(r)},G=function(a){return function(){var b=$.get(r),b=a(c.specials(),b);Ie.validate(b,c.tag(),c.property()).fold(k,h)}},I=G(yk.decrease),G=G(yk.increase),m=function(a,b){return Vh([x.resolve(e+a),x.resolve("stepper-button")],d+b)},Va=m("-stepper-decrease",".stepper.decrease"),m=m("-stepper-increase",".stepper.increase");
Va.events.click.bind(I);m.events.click.bind(G);K.append(l,[Va.element(),r,m.element()]);v.append(g,l);v.append(g,p);var l=Uh.text(l,r),ga=D.create({toEditor:s([]),size:s(["value"])}),t=function(){$.set(r,"")},y=function(a){a=Ie.sanitise(a);Ie.validate(a,c.tag(),c.property()).fold(k,function(a){n.remove(r,x.resolve("slick-widget-invalid-input"));Q.invalid(r,!1)});$.set(r,a)};return{element:f.constant(g),addEscape:function(a){v.append(p,a.element())},open:function(){T.focus(r)},close:f.noop,execute:function(){var a=
$.get(r);Ie.validate(a,c.tag(),c.property()).fold(k,function(a){h(a);ga.trigger.toEditor()})},destroy:f.noop,type:f.constant(l),represent:function(a){a.fold(t,y,y)},events:ga.registry}}function pS(a,b){var c=a.content(),d=wa(h.none()),e=function(){var c=nS(a,b),e=wk(a,c),c={represent:c.represent,refreshValue:c.refreshValue,open:e.open,close:e.close,isOpen:e.isOpen};d.set(h.some(c));return c},g={toggle:!1,value:"size",textkey:u.key("font.size"),action:function(){var b=d.get().getOrThunk(e);if(b.isOpen())b.close();
else{var g=zk.getState(a.selections(),c);b.open();b.represent(g.readState.size())}},updateGeneral:f.noop,icon:!0,shortcut:!1,hasPopup:!1,hasDialog:!1};return{general:f.constant(g)}}function qS(a){var b=a.hex(),c=q.fromTag("span");n.add(c,x.resolve("palette-detail-action"));m.set(c,"title",u.get(a.name()));Q.option(c,u.get(a.name()),!1,!1);var d=D.create({selected:s(["value"])}),e=E.bind(c,"click",function(){T.focus(c);d.trigger.selected(b)});return{element:function(){return c},value:f.constant(b),
destroy:function(){e.unbind();B.remove(c)},events:d.registry}}function rS(){var a=[],b=P.div(),c=q.fromTag("span");Q.listBox(c);n.add(c,x.resolve("palette-detail-actions"));n.add(c,x.resolve("palette-color-options"));Q.label(c,u.translate("aria.font.palette"));v.append(b,c);var d=Lj.create(),e=D.create({selected:s(["value"])});return{element:function(){return b},add:function(b){a.push(b.destroy);v.append(c,b.element());d.bind(b.events.selected,function(a){e.trigger.selected(a.value())})},remove:function(a){B.remove(a.element());
d.unbind(a.events.selected)},show:function(){n.add(b,x.resolve("palette-detail-active"));ha.showPanel(b);t.remove(b,"display")},hide:function(){n.remove(b,x.resolve("palette-detail-active"));ha.hidePanel(b);t.set(b,"display","none")},destroy:function(){k.each(a,f.apply);B.remove(b);a=[]},events:e.registry}}function dx(a,b){var c=Wh.makeLayout(b,sS,{isValid:tS}),d=c.layout();return{isFocused:d.isFocused,enterGrid:d.enter,grid:d,refresh:c.refresh,destroy:d.destroy,clear:d.clear}}function uS(){var a=
q.fromTag("div");vS()?Q.presentation(a):Q.menuItem(a,h.none(),h.none());n.add(a,x.resolve("color-widget"));var b=q.fromTag("div");n.add(b,x.resolve("color-widget-controls"));Q.tabList(b);var c=q.fromTag("div");n.add(c,x.resolve("color-widget-palettes"));Q.toolbarGroup(c);var d=id("text-color");n.add(d.element(),x.resolve("tab-button"));d.content(qc(u.translate("font.color")));Q.tabButton(d.element(),u.translate("aria.font.color"));var e=id("highlight-color");n.add(e.element(),x.resolve("tab-button"));
e.content(qc(u.translate("font.highlight")));Q.tabButton(e.element(),u.translate("aria.font.highlight"));v.append(b,d.element());v.append(b,e.element());v.append(a,b);v.append(a,c);return{element:f.constant(a),colorToggle:f.constant(d),highlightToggle:f.constant(e),content:f.constant(c),controls:f.constant(b),destroy:function(){d.destroy();e.destroy();B.remove(c);B.remove(b)}}}function wS(){var a=Wh.create(Ak.colors()),b=Wh.create(Ak.colors()),c=D.create({color:M(["value"],a.events.selected),highlight:M(["value"],
b.events.selected),abort:s([])}),d=uS(),e=d.colorToggle(),g=d.highlightToggle();Q.linkTabToPanel(e.element(),a.element());Q.linkTabToPanel(g.element(),b.element());v.append(d.content(),a.element());v.append(d.content(),b.element());var l=dx(d,a.element()),p=dx(d,b.element()),r=ba.flow(d.controls(),{item:f.constant(".ephox-pastry-statebutton")},pa.flow({trigger:function(a,b){return J.descendant(a,".ephox-pastry-button").map(function(a){a.dom().click();return a})},executeOnArrow:!0})),h=ba.multiple(d.element(),
[r,l.grid,p.grid]);h.events.exit.bind(function(a){c.trigger.abort()});var k=function(){b.hide();e.select();g.deselect();ha.tabSelected(e.element(),[g.element()]);a.show()};e.events.click.bind(function(){r.clickFocus(e.element());k()});g.events.click.bind(function(){r.clickFocus(g.element());a.hide();e.deselect();ha.tabSelected(g.element(),[e.element()]);b.show()});return{element:d.element,destroy:function(){a.destroy();h.destroy();b.destroy();d.destroy()},open:function(){h.clear();h.refresh();e.click();
k();h.enter()},close:function(){h.exit()},events:c.registry}}function xS(){var a=h.none(),b=function(b){a.each(function(a){a[b]()})};return{enter:f.curry(b,"enter"),exit:f.curry(b,"exit"),destroy:f.curry(b,"destroy"),set:function(b){a=h.some(b)}}}function yS(a,b){var c=b.i18nNamespace(),d=b.cssNamespace(),e=D.create({plus:s([]),minus:s([]),validate:s([]),abort:s([])}),g=ac(),l=ac(),p=P.div(),r=P.div(),z=q.fromTag("input");Q.label(z,u.translate(c+".stepper.input"));Q.toolbarButton(g.element(),u.translate(c+
".stepper.increase"),!1,!1);Q.toolbarButton(l.element(),u.translate(c+".stepper.decrease"),!1,!1);m.set(z,"tabindex","-1");m.set(g.element(),"tabindex","-1");m.set(l.element(),"tabindex","-1");n.add(r,x.resolve(d+"-stepper-input"));n.add(p,x.resolve(d+"-stepper-options"));n.add(g.element(),x.resolve("stepper-button"));g.content(Qd([x.resolve(d+"-stepper-increase")]));n.add(l.element(),x.resolve("stepper-button"));l.content(Qd([x.resolve(d+"-stepper-decrease")]));v.append(r,z);K.append(p,[g.element(),
l.element()]);K.append(a,[r,p]);g.events.click.bind(function(){T.focus(g.element());e.trigger.plus()});l.events.click.bind(function(){T.focus(l.element());e.trigger.minus()});var k=function(){z.dom().setSelectionRange(0,$.get(z).length)},G=E.capture(a,"click",function(a){L.eq(a.target(),r)&&I.enter()}),I=xS(),O=n.toggler(z,x.resolve("invalid-"+d+"-size"));return{destroy:function(){g.destroy();l.destroy();I.destroy();G.unbind();B.remove(z)},highlight:function(){O.on();m.set(z,"title",u.translate("aria.stepper.input.invalid"));
Oa.shout(z,u.translate("aria.stepper.input.invalid"))},dehighlight:function(){O.off();m.remove(z,"title")},input:f.constant(z),focus:function(){T.focus(z);k()},enter:I.enter,exit:I.exit,initLayout:function(){var b=ba.single(z,[U.basic(N.ENTER()[0],function(){e.trigger.validate();return h.some(!0)})],h.none());b.events.enter.bind(k);var c=pa.button({execute:function(a){a.dom().click();T.focus(a);return h.some(a)}}),b=ba.multiple(a,[b,ba.button(g.element(),c),ba.button(l.element(),c)]);b.events.exit.bind(function(){y.parent(r).each(T.focus);
e.trigger.abort()});I.set(b)},events:e.registry}}function Bk(a){var b=zS(a),c=P.div(),d=yS(c,b);n.add(c,x.resolve(b.cssNamespace()+"-size-widget"));var e=function(){return $.get(d.input())},g=function(a){var b=e();a=a(b);a=Ie.sanitise(a);$.set(d.input(),a);l.trigger.action(a);d.dehighlight()};d.events.validate.bind(function(a){a=e();Ie.validate(a,b.tag(),b.property()).fold(function(){d.highlight()},function(a){$.set(d.input(),a);l.trigger.action(a);d.dehighlight();l.trigger.close()})});d.events.plus.bind(function(a){g(f.curry(yk.increase,
b.specials()))});d.events.minus.bind(function(a){g(f.curry(yk.decrease,b.specials()))});var l=D.create({close:s([]),action:s(["value"]),abort:M([],d.events.abort)}),p=E.bind(c,"click",function(a){(L.eq(a.target(),c)||L.eq(a.target(),d.input()))&&d.enter()});return{element:f.constant(c),input:d.input,getCurrentValue:e,focus:d.focus,enter:d.enter,initLayout:d.initLayout,destroy:function(){p.unbind();d.destroy();B.remove(c)},updateInlineWidget:function(a,e){var g=function(a){Ie.validate(a,b.tag(),b.property()).each(function(a){$.set(d.input(),
a);d.dehighlight()})};a.size().fold(function(){g(e);n.add(c,x.resolve("default-"+b.cssNamespace()+"-size"))},f.noop,function(a){g(a);n.remove(c,x.resolve("default-"+b.cssNamespace()+"-size"))})},open:f.noop,close:f.noop,events:l.registry}}function AS(a,b,c,d){var e=c();c=Wc.okCancel("font-size");var g=ce(b),l=null,f=D.create({close:s([]),open:s([]),analytics:s(["message"]),ok:s(["value"])});b=function(){l.unbind();g.hide();f.trigger.close()};g.setTitle(u.translate("font.size"));g.setContent(e.element());
g.setControls(c.element());g.events.close.bind(b);c.events.ok.bind(function(){f.trigger.ok(e.getCurrentValue())});c.events.cancel.bind(b);n.add(g.element(),x.resolve("widget-dialog"));return{show:function(){g.show();var b=q.fromDom(window);l=E.bind(b,"orientationchange",function(){a.foreground()});f.trigger.open()},element:g.element,contents:e,events:f.registry,destroy:function(){g.hide();g.destroy();e.destroy()}}}function BS(a,b){var c=a.content(),d=a.undo(),e={},g=function(a){return k.map(a.split(/,\s?/g),
function(a){return a.trim().replace(/^[\'\"](.*)[\'\"]$/,"$1")}).join(",")},l=(0===b.faces.length?h.none():h.some(b.faces)).fold(Ak.faces,function(a){return k.map(a,function(a){var b=g(a.value);e[b]=a.value;return Xh.face(u.raw(a.text),b,h.none())})}),p=Y.detect().deviceType.isiOS(),r=D.create({analytics:s(["message","properties"])}),z=h.some(Hv.face),w=function(a){var b=g(a);a=k.find(l,function(a){return a.value()===b});var c=Xh.face(u.raw(b),b,h.none());return a.getOr(c)},p=p?CS:DS,G=pS(a,b);return{ui:$a.ui({"fontface.dropdown":f.constant({name:"fontface",
textkey:u.key("font.face"),menuTextkey:u.key("font.menu"),updateDropdown:function(b){var c=ES.face(a.selections()).map(w);b.populate([{label:h.none,items:f.constant(l)}]);Yj.updateMenu(b,c,oa.on(),l,z)},action:function(b){b=e.hasOwnProperty(b)?e[b]:b;He.face(c,d,a.selections(),b)},values:l,render:z,describe:function(a){return u.translate("font.describe",u.get(a.textkey()))},hasPopup:!0}),"fontcolor.widget":f.curry(FS.widget,a),"fontsize.widget":f.curry(p.widget,a),"fontsize.general":G.general},f.identity),
events:r.registry,destroy:f.noop}}function ex(a,b,c){var d=Tw(a,b,c);Q.tabList(d.menu());k.map(d.tabs(),function(a){var b='.ephox-pastry-tablayout-content[data-ephox-pastry-tablayout-view="'+a.name()+'"]';m.set(a.element(),"tabindex","-1");n.add(a.element(),x.resolve("tab-button"));J.descendant(d.body(),b).each(function(b){var c=La.generate("ephox-aria");m.set(a.element(),"id",c);Q.tabButton(a.element(),a.name());Q.tabPanel(b);Q.linkTabToPanel(a.element(),b);Q.labelledBy(b,c)})});return{element:d.element,
tabs:d.tabs,body:d.body,menu:d.menu,destroy:d.destroy,events:d.events}}function GS(a){var b=2===W.keys(a.integration).length,c=Je.aboutTags(b,a.integration);return{content:function(a){return k.foldl(c,function(a,b){return a.replace(b.str(),b.replace())},a)},dom:b?function(a){J.descendant(a,".ephox-polish-help-integration").each(function(a){t.set(a,"display","block")})}:f.noop}}function Yh(a,b,c){var d=h.none(),e=h.none(),g=HS(c),l=function(a,c){var e=b.content(c.trim());d=h.some(e);p(a,e)},p=function(a,
c){var d=q.fromHtml(c.trim());b.dom(d);B.empty(a);n.remove(a,x.resolve("tablayout-loading"));r(d);var e=q.fromTag("div");Q.document(e);v.append(e,d);v.append(a,e)},r=function(a){k();e=J.descendant(a,".ephox-license-link").map(function(a){return E.bind(a,"click",function(a){window.open(g,"","height=700,width=620,resizable=yes,scrollbars=yes")})})},z=function(b){a.get(function(a){a.fold(function(a){b("&nbsp;");console.error("Unable to download help article",a)},b)})},k=function(){e.each(function(a){a.unbind()})};
return{render:function(a){d.fold(function(){n.add(a,x.resolve("tablayout-loading"));z(f.curry(l,a))},function(b){p(a,b)})},destroy:function(){k()}}}function IS(a){var b={content:f.identity,dom:f.noop},c=Y.detect(),d;!0===a.source?(d=a.baseUrl+"/help/"+fe.get()+"/",d={about:wd.get(d+"about.html",xd.text(),nd.xhr()),accessibility:wd.get(d+"accessibility.html",xd.text(),nd.xhr()),a11ycheck:wd.get(d+"a11ycheck.html",xd.text(),nd.xhr()),markdown:wd.get(d+"markdown.html",xd.text(),nd.xhr()),shortcuts:wd.get(d+
"shortcuts.html",xd.text(),nd.xhr())}):d=JS(a.baseUrl+"/help_"+fe.get()+".js?"+Je.resourceParam());c=!c.deviceType.isTouch();a=[Zh(c,yf(u.translate("help.dialog.shortcuts"),Yh(d.shortcuts,b,a))),Zh(c,yf(u.translate("help.dialog.accessibility"),Yh(d.accessibility,b,a))),Zh(!0,yf(u.translate("help.dialog.markdown"),Yh(d.markdown,b,a))),Zh(c,yf(u.translate("help.dialog.a11ycheck"),Yh(d.a11ycheck,b,a))),Zh(!0,yf(u.translate("help.dialog.about"),Yh(d.about,GS(a),a)))];return ab.cat(a)}function KS(a,b){var c=
D.create({close:s([])}),d=ce(a),e=IS(b),g=ex(Qo.left(),e,"help");m.set(g.body(),"tabindex","-1");n.add(d.element(),x.resolve("help-modal"));var l=y.children(g.element());k.each(l,gc.register);d.setTitle(u.translate("cog.help"));d.setContent(g.element());var f=LS(g),l=MS(g),r=ba.multiple(d.element(),[f,l]);g.events.setActive.bind(function(a){f.clickFocus(a.item().element())});var h=function(){d.hide()};d.events.close.bind(function(){r.exit();h()});r.events.exit.bind(function(){h();c.trigger.close()});
return{element:d.element,open:function(){d.show();r.enter()},destroy:function(){g.destroy();d.destroy();r.destroy();k.each(e,function(a){a.renderer().destroy()})},hide:h,events:c.registry}}function NS(a,b){var c=KS(a,b),d=function(){e.trigger.analytics("help_screen_hide");e.trigger.close();g()},e=D.create({close:s([]),analytics:s(["message"])});c.events.close.bind(d);var g=function(){c.events.close.unbind(d);c.destroy()};return{show:function(){e.trigger.analytics("help_screen_show");c.open()},destroy:g,
events:e.registry}}function OS(a,b){var c=a.content(),d=a.smartview(),e=a.shortcuts(),g=D.create({analytics:s(["message"])}),l=sc.destroyable(),p=function(){var e=NS(d.uiContainer(),b);l.set(e);e.events.close.bind(function(){c.toEditing();l.clear()});e.events.analytics.bind(function(a){g.trigger.analytics(a.message())});c.toReading();a.content().hideInlineSelection();e.show()};e.events.help.bind(p);return{ui:$a.ui({"help.general":{toggle:!1,value:"help",textkey:u.key("cog.help"),action:p,updateGeneral:f.noop,
icon:!0,hasPopup:!1,hasDialog:!0,tooltip:Wa.get}}),events:g.registry,destroy:function(){l.clear()}}}function PS(a,b,c){var d=a.filter(),e=a.content(),g=a.operations();ge.configure(d);var l=void 0===c.external?[]:k.map(c.external,function(a){return QS(a.prefix,a.suffix,a.style)});k.each(l,function(a){var c=Mu(a.prefix(),a.suffix(),function(b){var c=q.fromTag("span");ea.add(c,[ge.highlightClass()]);t.setAll(c,a.style());var d=q.fromTag("div");v.append(c,q.fromText(a.prefix()+b+a.suffix()));v.append(d,
c);return ua.get(d)});b.customMacros().add(c)});a=function(a){e.getSelection().each(function(b){a.prev().each(function(a){J.ancestor(a,"."+ge.placeholderClass()).each(function(a){fx.rehighlight(g,l,a)})});J.ancestor(b.startContainer(),"."+ge.highlightClass()).each(function(a){fx.unhighlight(g,a)})})};return 0<l.length?(e.events.nodeChanged.bind(a),{run:function(){RS.run(e,l)},destroy:f.noop}):{run:f.noop,destroy:f.noop}}function SS(a){var b=a.content();return{ui:$a.ui({"horizontalrule.general":{toggle:!1,
value:"horizontalrule",textkey:u.key("insert.horizontalrule"),action:function(){Cc.insertAtCursor(b,[q.fromTag("hr")]).each(function(a){a=da.after(a.element());b.setRelativeSelection(a,a);b.toEditing()})},updateGeneral:f.noop,icon:!0,shortcut:!1,hasPopup:!1}}),destroy:f.noop}}function gx(a){var b=x.resolve("dialog-float-selected"),c=q.fromTag("fieldset");m.set(c,"role","radiogroup");a=q.fromHtml("<legend>"+u.translate(a)+"</legend>");var d=La.generate("ephox-aria");m.set(a,"id",d);m.set(c,"aria-labelledby",
d);var e=P.div();n.add(e,x.resolve("dialog-float-options"));K.append(c,[a,e]);var g=function(){k.each(l,function(a){n.remove(a.element(),b);ha.checked(a.element(),!1)})},l=k.map(["none","center","left","right"],function(a){var c=ac(),d=u.translate("align."+a);m.setAll(c.element(),{tabindex:"-1","data-float-value":a,title:d,"aria-label":d,role:"radio"});a=P.div();var d=P.div(),e=P.div(),l=P.div();Q.hidden(a,!0);n.add(d,x.resolve("dialog-float-icon-inner-left"));n.add(e,x.resolve("dialog-float-icon-inner-center"));
n.add(l,x.resolve("dialog-float-icon-inner-right"));K.append(a,[d,e,l]);c.events.click.bind(function(){g();n.add(c.element(),b);ha.checked(c.element(),!0);T.focus(c.element())});n.add(c.element(),x.resolve("dialog-float-container"));n.add(a,x.resolve("dialog-float-icon"));v.append(c.element(),a);return c});K.append(e,k.map(l,function(a){return a.element()}));var p=qb.radio(c,"."+x.resolve("dialog-float-container"),"."+x.resolve("dialog-float-selected"));return{element:function(){return c},get:function(){return J.child(e,
"."+b).map(function(a){return m.get(a,"data-float-value")})},select:function(a){g();a.each(function(a){J.child(e,'span[data-float-value="'+a+'"]').or(y.firstChild(e)).each(function(a){n.add(a,b);ha.checked(a,!0)})});p.refresh()},keylayout:f.constant(p),destroy:function(){k.each(l,function(a){a.destroy()})}}}function TS(){var a=P.div(),b=P.div(),c=P.div(),d=P.div();n.add(c,x.resolve("image-url-controls"));n.add(d,x.resolve("image-size-controls"));var e=gx("image.dialog.fields.align"),g=hc.input("image-url",
"image.dialog.fields.url"),l=hc.input("image-alt","image.dialog.fields.alt"),p=hc.input("image-width","image.dialog.fields.width"),r=hc.input("image-height","image.dialog.fields.height"),z=q.fromTag("input");m.set(z,"type","hidden");var k=id("lock");n.add(k.element(),x.resolve("dialog-button"));var G=Qd([x.resolve("icon"),x.resolve("image-lock-button")]);m.set(k.element(),"title",u.translate("image.dialog.fields.constrain.on"));Q.toolbarButton(k.element(),u.translate("image.dialog.fields.constrain.label"),
!1,!0);ha.press(k.element());k.select();k.content(G);k.events.click.bind(function(){T.focus(k.element());var a=k.selected()?"on":"off";m.set(k.element(),"title",u.translate("image.dialog.fields.constrain."+a));(k.selected()?ha.press:ha.release)(k.element())});G=P.div();n.add(G,x.resolve("form-group"));n.add(G,x.resolve("form-group-lock"));var I=q.fromHtml("<label>"+u.translate("image.dialog.fields.constrain.label")+"</label>"),O="constrain"+La.generate("");m.set(k.element(),"id",O);m.set(I,"for",
O);var s=function(a){a.isLocal()?(m.set(g.input(),"disabled","disabled"),$.set(g.input(),u.translate("image.dialog.fields.urllocal"))):(m.remove(g.input(),"disabled"),$.set(g.input(),a.url()))},ga=function(a,b,c){var d=parseFloat($.get(z));a=$.get(a);$.set(b,c(d,a).getOrThunk(function(){return $.get(b)}));Xb.dehighlight(b)},O=function(a){Xb.validateField(a.target(),"width",h.some("px")).fold(function(){Xb.highlight(a.target())},function(b){Xb.dehighlight(a.target());b=a.target();k.selected()&&(L.eq(b,
p.input())?ga(p.input(),r.input(),Vo.fromWidth):L.eq(b,r.input())&&ga(r.input(),p.input(),Vo.fromHeight))})},t=function(a){if(m.has(a.target(),"placeholder")){var b=m.get(a.target(),"placeholder");""===$.get(a.target())&&$.set(a.target(),b)}},v=E.bind(p.input(),"input",O),y=E.bind(r.input(),"input",O),A=E.bind(p.input(),"focus",t),B=E.bind(r.input(),"focus",t);K.append(G,[I,k.element()]);K.append(b,[e.element()]);K.append(c,[g.element(),l.element()]);K.append(d,[p.element(),r.element(),G]);K.append(a,
[c,d,b]);b=[ba.input(g.input(),pa.input({})),ba.input(l.input(),pa.input({})),ba.input(p.input(),pa.input({})),ba.input(r.input(),pa.input({})),ba.button(k.element(),pa.button({execute:qb.button})),e.keylayout()];return{element:f.constant(a),destroy:function(){k.destroy();v.unbind();y.unbind();A.unbind();B.unbind();e.destroy()},dehighlight:f.curry(Xb.clearHighlights,a),highlight:Xb.highlight,keylayouts:f.constant(b),set:function(a){s(a);$.set(l.input(),a.alt());$.set(p.input(),a.width());$.set(r.input(),
a.height());var b=a.computedWidth().bind(function(b){return a.computedHeight().bind(function(a){return Vo.calculate(b,a)})}).getOr(1);a.computedWidth().each(f.curry(m.set,p.input(),"placeholder"));a.computedHeight().each(f.curry(m.set,r.input(),"placeholder"));$.set(z,b);e.select(a.align())},get:function(){return Xb.validate([Xb.rule(p.input(),"width",h.some("px")),Xb.rule(r.input(),"height",h.some("px"))]).map(function(a){return Ck.properties({url:$.get(g.input()),alt:$.get(l.input()),width:a.width(),
height:a.height(),isLocal:m.has(g.input(),"disabled"),align:e.get()})})}}}function US(a){var b=D.create({ok:s(["image","properties"]),close:s([])}),c=wa(h.none()),d=ce(a),e=Wc.okCancel("image-properties"),g=TS();d.setContent(g.element());n.add(d.element(),x.resolve("dialog-narrow"));a=function(){var a=g.get();g.dehighlight();a.fold(function(a){g.highlight(a)},function(a){c.get().each(function(c){b.trigger.ok(c,a)})})};e.events.ok.bind(a);e.events.cancel.bind(function(){d.hide();b.trigger.close()});
d.setTitle(u.translate("image.dialog.title"));d.setControls(e.element());d.events.close.bind(function(){b.trigger.close()});var l=qb.setup(d,e,g.keylayouts());return{element:d.element,open:function(){d.show();l.enter()},hide:function(){l.exit();d.hide()},destroy:function(){d.destroy();l.destroy();e.destroy();g.destroy()},represent:function(a){var b=Ck.divine(a);c.set(h.some(a));g.set(b)},execute:a,events:b.registry}}function VS(a,b){var c=hx.image(a.content()),d=D.create({changed:s(["image"]),contextmenu:M(["image"],
c.events.contextmenu)}),e=function(b){var c=a.content(),e=US(a.smartview().uiContainer()),f=ix.generate(a.keyboard(),e,["image","properties"]);f.events.ok.bind(function(a){Ck.apply(a.image(),a.properties());d.trigger.changed(a.image());f.destroy();c.domChanged();c.focus();c.uiChanged()});f.events.close.bind(function(a){f.destroy();c.focus()});f.represent(b);f.open()};c.events.properties.bind(function(a){e(a.image())});return{destroy:f.noop,situation:c.situation,handle:c.handle,assign:c.assign,events:d.registry}}
function WS(a){var b=function(b){return wd.get(b,xd.blob(),nd.xhr(),a.newHeaders())},c=function(a){return!(Wo.local(a)||Wo.sameDomain(a,String(Ke.location)))};return{getImage:function(d){var e=m.get(d,"src");return(c(e)?a.proxy(e):h.none()).fold(function(){return Pa.fromPromise($h.imageToBlob(d.dom()))},b)}}}function XS(a,b,c){var d=h.none();return{setTarget:function(a){d=h.some(a)},clearTarget:function(){d=h.none()},mutate:function(e,g){d.each(function(d){var f=fb.delta({top:g,left:e}),r=zf.updatePosition(c,
function(b){return d(a,b,f)});zf.drawAtOrigin(c,a);b.update(r)})}}}function YS(a){var b=D.create({reposition:s(["direction","delta"])}),c=Le("top","n",h.none()),d=Le("bottom","s",h.none()),e=Le("right","e",h.none()),g=Le("left","w",h.none()),l=Le("top-left","nw",h.some(a("image.crop.handle.nw"))),p=Le("top-right","ne",h.some(a("image.crop.handle.ne"))),r=Le("bottom-right","se",h.some(a("image.crop.handle.se"))),k=Le("bottom-left","sw",h.some(a("image.crop.handle.sw"))),w=$d.nu(function(a){setTimeout(function(){t.reflow(c);
var b=qa.get(c),d=ia.get(c);a(f.constant(3*b+2*d))},0)}),G=b.trigger.reposition;a=[c,d,e,g,p,l,r,k];G=[ba.single(l,ai.navigation("nw",G),h.none()),ba.single(p,ai.navigation("ne",G),h.none()),ba.single(r,ai.navigation("se",G),h.none()),ba.single(k,ai.navigation("sw",G),h.none())];return{elements:f.constant(a),keylayouts:f.constant(G),events:b.registry,refresh:function(a,b){var f=jx.calculate(a,b);Me(c,f.n());Me(d,f.s());Me(e,f.e());Me(g,f.w());Me(l,f.nw());Me(k,f.sw());Me(r,f.se());Me(p,f.ne());w.get(function(a){kx([c,
d],b.width,a);kx([e,g],b.height,a)})}}}function ZS(){var a=Dk("left"),b=Dk("top"),c=Dk("right"),d=Dk("bottom");return{elements:f.constant([a,b,c,d]),refresh:function(e,g){var l=$S.derive(e,g);Ek(a,l.leftRim());Ek(b,l.topRim());Ek(c,l.rightRim());Ek(d,l.bottomRim())}}}function aT(a,b,c,d){var e=q.fromTag("div");n.add(e,lx.magicBox());Af.base(e,d("image.crop.begin"));Fk.effect(e,"execute");var g=q.fromTag("div");m.set(g,"role","row");var l=ZS(),p=YS(d),r=function(a){t.setAll(e,{top:a.top()+"px",left:a.left()+
"px"})};d=function(b){var c=a();r(c);l.refresh(c,b);p.refresh(c,b);z.trigger.updated(b)};K.append(g,k.flatten([l.elements(),[b],p.elements()]));v.append(e,g);var z=D.create({updated:s(["newBounds"]),reposition:M(["direction","delta"],p.events.reposition)});d(c);b=p.keylayouts();return{element:f.constant(e),bitsbox:f.constant(g),keylayouts:f.constant(b),move:r,update:d,destroy:function(){B.remove(e)},events:z.registry,tryMovement:function(a){return J.closest(a,"."+Za.resolve("corner-wrapper")).bind(function(a){a=
m.get(a,"data-"+Za.resolve("direction"));return void 0!==Gk[a]?h.some(Gk[a]):h.none()})}}}function bT(a,b,c,d){var e=q.fromTag("div");ea.add(e,[Za.resolve("panel-central"),Za.resolve("side-panel")]);m.set(e,"tabindex","-1");Af.cell(e,d("image.crop.handle.shade"));Fk.grab(e,!1);var g=cT(a());zf.writePosition(e,g);zf.drawAtOrigin(e);var l=aT(a,e,g,d);v.append(b,l.element());var p=XS(a(),l,e),r=Ih.transform(p),z=E.bind(e,"dblclick",function(){t.trigger.apply()}),w=dT.bind(l.element(),function(a){l.tryMovement(a).orThunk(function(){return J.closest(a,
"."+Za.resolve("panel-central")).map(function(){return Gk.translate})}).each(G)}),G=function(a){r.off();p.clearTarget();p.setTarget(a);r.on();r.go(c)},I=function(b,c){var d=zf.updatePosition(e,function(d){return Gk[b](a(),d,c)});zf.drawAtOrigin(e);l.update(d)};d=k.flatten([l.keylayouts(),[ba.single(e,ai.navigation("translate",I),h.none())]]);var O=function(){var c=a();return E.bind(y.defaultView(b),"resize",function(){var b=a();if(b.height()!==c.height()||b.width()!==c.width())t.trigger.cancel();
else if(b.top()!==c.top()||b.left()!==c.left())l.move(b),c=b})}(),g=function(a){return function(){T.active(y.owner(b)).each(function(b){Fk.grab(b,a)})}},n=g("true"),ga=g("false");k.each(d,function(a){a.events.enter.bind(n);a.events.exit.bind(ga)});l.events.reposition.bind(function(a){I(a.direction(),a.delta())});var t=D.create({updated:M(["newBounds"],l.events.updated),apply:s([]),cancel:s([])});return{element:l.element,keylayouts:f.constant(d),destroy:function(){r.destroy();O.unbind();w.unbind();
z.unbind();l.destroy()},boxes:function(){return{box:a,magic:l.element,shade:f.constant(e)}},get:function(){return zf.readPosition(e)},events:t.registry}}function eT(a,b,c,d,e){var g=f.curry(fT,a),l=bT(g,b,c,e);return{element:l.element,keylayouts:l.keylayouts,crop:function(){var b;b=l.get();var c=qa.get(a),e=ia.get(a);b=gT({cropX:b.left(),cropY:b.top(),cropWidth:b.width(),cropHeight:b.height(),displayWidth:c,displayHeight:e});return d.animate(l.boxes().magic(),a,l.boxes(),b)},destroy:function(){l.destroy()},
events:l.events}}function hT(a){var b=a.content(),c=D.create({ok:s(["cropAction","image","cleanup"])}),d=sc.value(),e=sc.value(),g=function(a){var g=b.body().getOrDie();Oa.speak(ca.body(),u.translate("image.crop.announce"));var f=y.documentElement(a),h=eT(a,f,f,iT.slide(),u.translate),k=zc.adaptable(function(a,b){Oa.speak(ca.body(),u.translate("aria.image.crop.size",a,b))},500);h.events.updated.bind(function(a){a=a.newBounds();k.throttle(a.width(),a.height())});var G=function(){d.clear();t.exit();
t.destroy();v.unbindAll();s.clear()},m=function(){G();e.clear();h.destroy()},O=function(){m();ib.select(b,a)},n=function(){G();h.crop().get(function(b){s.clear();Oa.speak(ca.body(),u.translate("image.crop.apply"));var d=pg.crop(b.cropX(),b.cropY(),b.cropWidth(),b.cropHeight()),e=Xo.changedBounds(a);c.trigger.ok(function(a){return Th.resampleAction(e,a,d)},a,function(a){m();return a})})};d.set(n);e.set(m);jT(b,a);var q=function(){Oa.speak(ca.body(),u.translate("image.crop.cancel"));O()},s=sc.unbindable();
setTimeout(function(){s.set(E.bind(g,"focus",function(a){q()}))});var t=ba.inlineDialog(f,h.keylayouts(),pa.inlineDialog({ok:n,cancel:q}));t.enter();var v=Lj.create();v.bind(b.events.domChanged,O);v.bind(b.events.nodeChanged,function(a){ca.inBody(a.node())&&O()});v.bind(h.events.apply,n);v.bind(h.events.cancel,O)};return{apply:function(){d.on(f.apply)},open:function(){a.content().getSelection().bind(ib.detect).each(function(a){g(a.image())})},destroy:function(){e.on(f.apply)},isActive:d.isSet,events:c.registry}}
function kT(a){a=lT.parse(a.preferredWidth,a.preferredHeight);var b=a.fold(mT,Th.instance),c=a.fold(nT,function(){return oT(b)});return{assets:function(a){return Nc.mapM(a,b)},correlated:function(a){return Nc.mapM(a,c)}}}function pT(a,b,c){var d=WS(a.communication()),e=Po(a.ui),g=a.content(),l=function(c){return function(d){d.fold(function(a){a=Yo.diagnoseImageEdit(a,m.get(c,"src"));e.show(a.bannerMessage());console.error("Image edit operation failed: ",a.consoleMessage())},function(c){b.uploadAssets([c]);
g.focus();a.undo().save();g.domChanged()});ib.select(g,c);return d}},p=function(b,c){a.undo().save();var e=Zo.applyEdit(b);return Zo.run(d,c,e)(b)},r=hT(a);r.events.ok.bind(function(a){var b=a.image(),c=a.cropAction();a=a.cleanup();var d=he.updater(he.keepCrop,b);p(b,c).map(d).map(a).get(l(b))});var h=function(){return g.getSelection().bind(ib.detect)},k=function(a,b){return function(){h().each(function(c){c=c.image();var d=f.identity,e=he.updater(b,c);p(c,a).map(e).map(d).get(l(c))})}},G=[qg.make("crop-begin",
"image.crop.begin",r.open),qg.make("rotate-left","image.toolbar.rotateleft",k(pg.rotateLeft,he.keepRotate)),qg.make("rotate-right","image.toolbar.rotateright",k(pg.rotateRight,he.keepRotate)),qg.make("flip-horizontal","image.toolbar.fliphorizontal",k(pg.flipHorizontal,he.keepFlip)),qg.make("flip-vertical","image.toolbar.flipvertical",k(pg.flipVertical,he.keepFlip))],I=[qg.make("crop-apply","image.crop.apply",r.apply)],k=$a.ui({"edit.palette":f.curry(mx.edit,a.ui().refresh,h,r.isActive,G),"crop.palette":f.curry(mx.crop,
a.ui().refresh,r.isActive,I)},f.identity),G=hx.imageEdit(r.isActive,I,G);c=kT(c);return{destroy:function(){r.destroy()},ui:k,insertResample:f.constant(c),situation:G.situation,handle:G.handle,assign:G.assign,name:f.constant("imageEdit")}}function qT(a,b){var c=x.resolve("cursor-pointer"),d=b.os.isOSX(),e=h.none(),g=function(b){return a.isInline()?Na.absolute(b).top():0},l=function(b){var c=m.get(b,"href");if("top"===m.get(b,Xa.bookmarkAnchorDataAttr()))a.doc().each(function(b){a.body().each(function(a){var c=
Ea.get(b);Ea.to(c.left(),g(a),b)})});else if("bottom"===m.get(b,Xa.bookmarkAnchorDataAttr()))a.win().each(function(b){a.doc().each(function(c){a.body().each(function(a){var d=g(a),e=Ea.get(c);a=d+ia.get(a)-b.innerHeight;Ea.to(e.left(),a,c)})})});else if(/^#/.test(c)){var d=c.substr(1);a.body().each(function(b){Uc.anchorTarget(b,d).each(function(b){a.win().each(function(a){Ea.setToElement(a,b)})})})}else window.open(c)},f=function(a,b){var c=d?N.META():N.CTRL();k.contains(c,a)&&b()},r=q.fromDom(window),
z=h.none();a.events.ready.bind(function(a){a=a.api().body();var b=rk.handle(a,function(a){var b=a.target();(d?a.raw().metaKey:a.raw().ctrlKey)&&J.closest(b,"a[href]").bind(function(b){a.prevent();l(b)})});z=h.some(b);e=h.some(n.toggler(a,c))});var w=function(){e.each(function(a){a.on()})},G=function(){e.each(function(a){a.off()})},I=function(a){f(a.original().raw().which,G)};a.events.keyup.bind(I);var O=E.bind(r,"keyup",function(a){f(a.raw().which,G)}),s=function(a){f(a.original().raw().which,w)};
a.events.keydown.bind(s);var t=E.bind(r,"keydown",function(a){f(a.raw().which,w)});return{follow:l,destroy:function(){z.each(function(a){a.unbind()});a.events.keyup.unbind(I);a.events.keydown.unbind(s);O.unbind();t.unbind()}}}function rT(a,b,c,d){var e=q.fromTag("div");Q.dialog(e,u.get(d));ea.add(e,[x.resolve("expandable-inline-popup")]);d=P.div();n.add(d,x.resolve("inline-popup-content"));var g=P.div();ea.add(g,[x.resolve("inline-popup-content-expand"),bi]);var l=P.div();n.add(l,x.resolve("inline-popup-controls"));
var p=P.div();n.add(p,x.resolve("inline-popup-expand-button-wrapper"));var r=q.fromTag("span");ea.add(r,[x.resolve("inline-popup-expand-button"),x.resolve("icon")]);var h=u.translate("aria.label.more");m.set(r,"title",h);Q.button(r,h);Q.group(g);Q.linkElements(r,g);v.append(p,r);h=P.div();n.add(h,x.resolve("inline-popup-body"));K.append(h,[d,g,p]);K.append(e,[h,l]);K.append(d,a);K.append(g,b);K.append(l,c);n.add(e,bi);var k="0px",G=function(){n.remove(e,bi);n.add(g,x.resolve("expand-close"));Q.hidden(g,
!0);t.setAll(g,{opacity:"0",height:"0px",visibility:"hidden"});ha.collapsed(r)},I=E.bind(g,"transitionend",function(a){ea.remove(g,[x.resolve("expand-close"),x.resolve("expand-open")])}),O=E.bind(r,"click",function(a){n.has(e,bi)?G():(n.add(e,bi),n.add(g,x.resolve("expand-open")),Q.hidden(g,!1),t.setAll(g,{opacity:"1",height:k,visibility:"visible"}),ha.expanded(r))});a=[ba.button(r,pa.button({execute:qb.click}))];return{element:f.constant(e),keylayouts:f.constant(a),destroy:function(){B.remove(e);
O.unbind();I.unbind()},show:function(){k=ia.getOuter(g)+"px";G();t.reflow(e);n.add(e,x.resolve("enable-transition"))}}}function sT(a){var b=tT();a.url().each(function(a){$.set(b.input(),a)});var c=hc.input("text-to-display","link.properties.text.label"),d=wa(!1);n.add(c.element(),x.resolve("text-to-display"));m.setAll(c.input(),{placeholder:u.translate("link.properties.text.placeholder"),autocomplete:"off",autocorrect:"off"});a.text().each(function(a){$.set(c.input(),a)});var e=hc.input("title-text",
"link.properties.title.label");n.add(e.element(),x.resolve("title-container"));m.setAll(e.input(),{placeholder:u.translate("link.properties.title.placeholder"),autocomplete:"off",autocorrect:"off"});a.title().each(function(a){$.set(e.input(),a)});var g=uT.target(a),l=E.bind(c.input(),"input",function(){d.set(!0)});return{url:f.constant(b),text:f.constant(c),textChanged:d.get,title:f.constant(e),target:f.constant(g),keylayouts:function(){return[ba.input(b.input(),pa.input({})),ba.input(c.input(),pa.input({})),
ba.input(e.input(),pa.input({})),ba.button(g.input(),pa.button({execute:qb.click,process:f.noop}))]},destroy:function(){l.unbind()}}}function vT(a){var b=R.isNumber(a)?a:wT,c=No.access("local.linkCache");a=c.read();var d=function(a){return 0>b-a.length?a.slice(0,b):a}(R.isArray(a)?a:[]),e=function(a){d.splice(a,1)},g=function(){return d.slice(0)};return{add:function(a){"#"!==a.charAt(0)&&(d.length>=b&&d.pop(),k.indexOf(d,a).each(e),d.unshift(a))},state:g,destroy:function(){c.write(g())}}}function xT(a,
b){return{position:function(c,d){var e=c.isRtl()?rg.allRtl():rg.all(),g=fg(0,0),l=mg.element(a,d);Hk.simple(l,b,g,e,{})}}}function yT(a,b,c,d){var e=D.create({change:s([])}),g=function(){var a=b.body().map(Uc.all).getOr([]),a=k.bind(a,function(a){var b=m.get(a,"href");return aa.startsWith(b,"#")||m.has(a,Xa.bookmarkAnchorDataAttr())?[]:[b]}),a=d.state().concat(a),a=zT.stringArray(a);return k.map(a,function(a){var b=h.none();return AT(a,f.constant(!1),b,f.noop)})},l=function(a,b,c){var d=Uc.getAnchorValue(a,
ek.generate("textbox-heading"));return h.some(nx("#"+d,c,function(){m.set(b,"id",d)}))},p=function(a){var b="h1"!==A.name(a),c=pb.get(a);return m.has(a,"id")||m.has(a,"name")?(a=m.has(a,"id")?m.get(a,"id"):m.get(a,"name"),h.some(nx("#"+a,b,f.noop))):0===c.trim().length?h.none():l(c,a,b)},r=function(a){m.remove(a,Xa.bookmarkAnchorDataAttr())},z=function(a){var b=Xa.reservedUrls()[a.toLowerCase()];return function(a){m.set(a,Xa.bookmarkAnchorDataAttr(),b)}},w=function(){var a=b.body().map(Uc.findLinkTargets).getOr([]),
c=a.length>BT,a=k.map(a,function(a){return p(a).map(function(b){var d=(c?Uc.linkTargetFastText:Uc.linkTargetText)($o,a);return ap(b.url(),b.indent(),b.action(),d.before(),d.innerText(),d.after())})}),a=ab.cat(a),d=function(a){return"\u2014 "+a+" \u2014"},e=z(Xa.topAnchorUrl()),e=ap(Xa.topAnchorUrl(),!1,e,"",d(Xa.topAnchorUrl()),""),g=z(Xa.bottomAnchorUrl()),d=ap(Xa.bottomAnchorUrl(),!1,g,"",d(Xa.bottomAnchorUrl()),"");return[e].concat(a).concat([d])};return{predict:function(b,d,l){var f=hb.cached(function(){var a=
g(),d=w(),a=Ik.config({container:c,translate:u.translate,sections:[Ik.section({id:"recent",highlight:"strong",contentLimit:$o,items:a,itemLimit:10,render:ox.anchors,extractors:[ci.title,ci.url]}),Ik.section({id:"anchors",highlight:"strong",contentLimit:$o,items:d,render:ox.standard,extractors:[ci.innerText,ci.url]})]}),e=Ik.init(b,a);ea.add(e.element(),[x.resolve("layer-menus"),x.resolve("tarot-inline")]);e.events.click.bind(function(a){e.hide();m(b,a.item())});a=qa.get(b);t.setAll(e.element(),{"min-width":a+
"px","box-sizing":"border-box"});return e}),p=wa(h.none()),m=function(a,b){var c=b.url();$.set(a,c);CT&&a.dom().click();p.set(h.some(b));e.trigger.change()},n=l.container(b);n.swallowKeys([N.UP(),N.DOWN()]);n.events.up.bind(function(a){a.stopper()();a=f();q(a);a.moveup()});n.events.down.bind(function(a){a.stopper()();a=f();q(a);a.movedown()});n.events.escape.bind(function(a){var b=f();b.isShowing()&&(b.hide(),a.stopper()())});n.events.enter.bind(function(a){f().getSelectedData().each(function(c){a.stopper()();
var d=f();d.hide();d.reset();m(b,c)})});n.on();var q=function(c){if(!c.isShowing()){var d=$.get(b);""===d?c.resetFull():c.filter(d);c.isShowing()||c.show(a.uiDirection())}},s=[E.bind(b,"input",function(c){e.trigger.change();c=f();p.set(h.none());var d=$.get(b);0<d.length?(c.filter(d),c.isShowing()||c.show(a.uiDirection())):(c.reset(),c.hide())}),E.bind(b,"focus",function(a){f()}),E.bind(b,"blur",function(a){var c=f().hide;T.active().fold(c,function(a){L.eq(a,b)||c()})}),E.bind(c,"click",function(a){f().hide()}),
E.bind(d,"click",function(a){a.stop();a=f();a.isShowing()?a.hide():q(a)})];return{destroy:function(){f().destroy();n.destroy();k.each(s,function(a){a.unbind()})},getSelectedValue:function(){return p.get().fold(function(){var a=$.get(b),c=Xa.isReservedUrl(a)?z(a):r;return px(a,c)},function(a){return px(a.url(),a.action())})}}},events:e.registry}}function DT(a,b,c,d,e,g,l){var p=vT(1337),r=sT(g),z=l();l=[r.url().element()];var w=[r.text().element(),r.title().element(),r.target().element()],G=rT(l,w,
z.elements(),z.dialogTitle());n.add(G.element(),x.resolve("inline-link-dialog"));var I=Sd.popup(d.uiContainer,d.bounds,a,[],sf.editorContainer()).initMenu(G.element());n.add(I.element(),x.resolve("bubble-box"));l=I.element();l=yT(a,c,l,p);var O=l.predict(r.url().input(),r.url().dropArrow(),b);g.invalid()&&(r.url().showInvalid(),l.events.change.bind(function(){r.url().hideInvalid()}));var q={before:function(){t.set(I.panel(),"visibility","hidden");I.open()},after:function(){t.remove(I.panel(),"visibility");
t.remove(I.panel(),"display")}},ga=D.create({remove:M([],z.events().remove),cancel:s([]),execute:M([],z.events().execute)}),u=function(a){var b=a.command();a=a.data();m.has(a,Xa.bookmarkAnchorDataAttr())||"insertLink"!==b&&"updateLink"!==b||(b=m.get(a,"href"),p.add(b))};e.events.internal.bind(u);b=function(){ga.trigger.cancel()};z.events().cancel.bind(b);I.events.close.bind(b);b=k.flatten([r.keylayouts(),G.keylayouts(),z.keylayouts()]);var v=ba.inlineDialog(G.element(),b,pa.inlineDialog({ok:z.execute.click,
cancel:f.noop}));v.events.exit.bind(function(){I.close();ga.trigger.cancel()});return{showPopup:function(b){q.before();hg.preserve(function(){b.showExpandable(a,c,d,I,G);q.after();G.show();v.enter()},G.element())},hide:function(){I.closed()||v.exit()},getValues:function(){var a=O.getSelectedValue(),b=$.get(r.text().input()).trim(),c=$.get(r.title().input());return sg.properties({url:""===a.url()?h.none():h.some(a.url()),textChanged:r.textChanged(),text:h.some(b),title:h.some(c),target:bp.getValue(r.target().input()),
action:h.some(a.action()),invalid:!1})},destroy:function(){p.destroy();e.events.internal.unbind(u);O.destroy();r.destroy();v.destroy();z.destroy();I.destroy();G.destroy()},events:ga.registry}}function ET(a,b){var c=a.filter(),d=a.content(),e=a.smartview(),g=qx.enabled(d),l=a.internalevents(),p=Y.detect();sg.configure(c);var r=qT(d,p),c=function(){d.getSelection().each(function(a){d.setSelection(a.startContainer(),a.startOffset(),a.endContainer(),a.endOffset())})},k=p.browser.isIE()?c:f.noop,w=D.create({insert:s(["properties"]),
update:s(["properties"]),unlink:s([])}),G=sc.destroyable(),I=function(b,c){var g=DT(a.region(),a.keyboard(),d,e,l,b,c);g.events.remove.bind(function(a){w.trigger.unlink();O()});g.events.cancel.bind(function(a){d.focus();O();k()});G.set(g);d.events.mousedown.bind(O);return g},O=function(){G.clear();d.events.mousedown.unbind(O)},q=function(a){var b=I(a,rx.update);b.events.execute.bind(function(a){w.trigger.update(b.getValues());O()});b.showPopup(cp)};g.events.properties.bind(function(a){a=a.link();
var b=m.has(a,Xa.bookmarkAnchorDataAttr())?Xa.getReservedUrl(m.get(a,Xa.bookmarkAnchorDataAttr())):m.get(a,"href"),c=h.from(pb.get(a).trim());a=sg.properties({url:h.from(b),textChanged:!1,text:c,title:h.from(m.get(a,"title")),target:h.from(m.get(a,"target")),action:h.none(),invalid:n.has(a,yd.validationHighlight())});q(a)});g.events.follow.bind(function(a){d.focus();setTimeout(function(){r.follow(a.link())},0)});g.events.ignore.bind(function(a){b.ignore(a.link())});g.events.remove.bind(function(){w.trigger.unlink()});
return{showContext:q,showInsert:function(a){var b=I(a,rx.insert);b.events.execute.bind(function(a){w.trigger.insert(b.getValues());O()});b.showPopup(cp)},situation:g.situation,handle:g.handle,assign:g.assign,events:w.registry,destroy:function(){O();r.destroy()}}}function FT(a){var b=wa(h.none());a=P.div();n.add(a,x.resolve("link-widget"));var c=P.span();n.add(c,x.resolve("link-widget-field"));var d=q.fromTag("input");n.add(d,x.resolve("link-widget-input"));Q.label(d,u.translate("link.properties.url.placeholder"));
v.append(c,d);m.set(d,"type","url");m.set(d,"placeholder",u.translate("link.dialog.tooltip"));var e=Uh.text(c,d),g=Vh([x.resolve("link-widget-insert")],"insert.link"),l=Vh([x.resolve("link-widget-update")],"link.dialog.update"),p=Vh([x.resolve("link-widget-remove")],"link.dialog.remove");v.append(a,c);var r=D.create({insert:s(["properties"]),update:s(["properties"]),unlink:s([]),toEditor:s([])}),z=function(a,b){return sg.properties({url:h.some(a),textChanged:b.isSome(),text:b,title:h.none(),target:h.none(),
action:h.none(),invalid:!1})},w=function(){var a=$.get(d),c=b.get().fold(function(){return z(a,a)},function(b){return b.text().fold(function(){return z(a,h.some(a))},function(b){b=h.from(b).filter(function(a){return""!==a.trim()});return z(a,b)})});r.trigger.insert(c);b.set(h.none());r.trigger.toEditor()},G=function(){var a=$.get(d);r.trigger.update(z(a,h.none()));b.set(h.none());r.trigger.toEditor()};g.events.click.bind(w);l.events.click.bind(G);p.events.click.bind(function(){r.trigger.unlink();
r.trigger.toEditor()});return{element:f.constant(a),addEscape:function(a){v.append(c,a.element())},type:f.constant(e),execute:function(){Tb.isVisible(g.element())?w():Tb.isVisible(l.element())&&G()},open:function(){T.focus(d)},close:f.noop,destroy:f.noop,represent:function(a){k.each([g.element(),l.element(),p.element()],B.remove);a.url().fold(function(){$.set(d,"http://");v.after(d,g.element())},function(a){K.after(d,[l.element(),p.element()]);$.set(d,a)});b.set(h.some(a))},events:r.registry}}function GT(a){var b=
FT(a),c=wk(a,b);a=qx.disabled();var d=D.create({insert:s(["properties"]),update:s(["properties"]),unlink:s([])});b.events.insert.bind(function(a){d.trigger.insert(a.properties())});b.events.update.bind(function(a){d.trigger.update(a.properties())});b.events.unlink.bind(function(a){d.trigger.unlink()});return{showContext:function(a){b.represent(a);c.open()},showInsert:function(a){b.represent(a);c.open()},situation:a.situation,handle:a.handle,assign:a.assign,events:d.registry,destroy:function(){c.close();
c.destroy()}}}function HT(a,b){var c=a.content(),d=a.undo(),e=a.shortcuts(),g=Y.detect().deviceType.isTouch()?GT(a):ET(a,b),l=D.create({analytics:s(["message"])});g.events.insert.bind(function(b){d.transaction(function(){var d=dp.insert(c,b.properties());a.internalevents().input(d)});c.domChanged();c.cursorIntoView();l.trigger.analytics("link_inserted")});g.events.update.bind(function(a){d.transaction(function(){var b=a.properties();dp.update(c,b)});c.domChanged();c.cursorIntoView()});g.events.unlink.bind(function(a){d.save();
dp.unlink(c);c.domChanged();c.cursorIntoView()});var p=function(){c.getSelection().each(function(a){Uc.sharedLinkOrClosest(a,c.isRoot).fold(function(){c.selectWord();var a=c.getSelectedText().map(function(a){return a.trim()});g.showInsert(sg.properties({url:h.none(),textChanged:!1,text:a,title:h.none(),target:h.none(),action:h.none(),invalid:!1}))},function(a){var b=h.from(pb.get(a).trim());g.showContext(sg.properties({url:h.from(m.get(a,"href")),textChanged:!1,text:b,title:h.from(m.get(a,"title")),
target:h.from(m.get(a,"target")),action:h.none(),invalid:n.has(a,yd.validationHighlight())}))})})},r=$a.ui({"link.general":{toggle:!1,value:"link",textkey:u.key("insert.link"),action:p,updateGeneral:f.noop,icon:!0,shortcut:!0,hasPopup:!1,hasDialog:!0}});e.events.link.bind(p);return{ui:r,situation:g.situation,handle:g.handle,assign:g.assign,events:l.registry,destroy:function(){g.destroy()}}}function IT(a,b,c){var d=gk(c),e=hk.get("ephox.link.service.one",sx.one(),a+"/1/check?url=${url}&fresh=${fresh}",
b),g=hk.post("ephox.link.service.many",sx.many(),a+"/1/check",b),l=function(a){return{url:a.url,result:tx.derive(a.url,a.result)}};return{checkOne:function(a,b){var c=Lh.now(),c=ik.single(function(b,c){return e.execute({url:encodeURIComponent(a),fresh:c})},d,a,c,void 0!==b?b:!1);return Yd.mapSync(c,l)},checkMany:function(a,b){var c=Lh.now(),c=ik.multiple(function(a,b){return g.execute({urls:a})},d,a,function(a){return ie.indexOnKey(a.results,"url")},function(a){return a.url},c,void 0!==b?b:!1);return Yd.mapSync(c,
function(a){return W.map(a,l)})},dumpCache:d.dump}}function JT(a){var b=a.slice(0),c=function(a){var c=m.get(a,"href");if(aa.startsWith(c,"#")||m.has(a,Xa.bookmarkAnchorDataAttr()))a=H.error(a);else{var g;g=y.owner(a).dom().baseURI;g=h.from(g).getOr(window.location);g=ux.resolve(g,a.dom().href);if(k.contains(b,g))a=H.error(a);else{var l=tg.parse(g).protocol;a="https"===l||"http"===l?H.value(KT(c,g)):H.error(a)}}return a};return{extract:c,ignore:function(a){c(a).each(function(a){b.push(a.resolved())})},
state:function(){return b.slice(0)}}}function LT(a,b,c){var d=a.content(),e=a.filter(),g=a.communication(),l=a.undo(),p=a.viewablearea(),r=p.viewport(),z=JT(b.ignore),w=yd.validationHighlight(),G=yd.invalidLink(),m=yd.unknownLink();yd.configure(e);var O=IT(c.url,g.newHeaders(),b.cache),q=function(a){a=k.map(a,function(a){return xf.cata(r.assess(a),h.none,f.compose(h.some,z.extract),h.none)});a=ab.cat(a);a=di.partition(a);k.each(a.errors,x);var b=MT.generate(a.values);a=W.keys(b);a=k.map(a,function(a){return{url:a}});
a=O.checkMany(a);t(a,function(a){W.each(a,function(a){var c=b[a.url];void 0===c?console.warn("Link validator response from server was unknown: ",a.url,a.result):k.each(c,function(b){A(a.result,b)})})})};a=function(){var a=p.sections(),a=Uc.many(a),a=k.filter(a,function(a){return!n.has(a,Co.embedChecking())});q(a)};var s=zc.adaptable(a,500).throttle;p.events.onChange.bind(s);d.events.domChanged.bind(s);var t=function(a,b){Yd.handle(a,function(a){console.error("Link Validation server failure: ",a)},
function(a){b(a)})},v=function(a){return d.body().map(function(b){return Uc.hrefUrl(b,a)})},u=function(a,b){v(a).each(function(a){k.each(a,function(a){n.has(a,yd.validationHighlight())||ea.add(a,[w,b])})})},x=function(a){n.has(a,yd.validationHighlight())&&ea.remove(a,yd.allClasses())},y=function(a){v(a).each(function(a){k.each(a,x)})},A=function(a,b){tx.cata(a,function(a){u(b,G)},function(a){u(b,m)},function(a){y(b)})};l.events.undo.bind(a);l.events.redo.bind(a);return{destroy:f.noop,refresh:s,ignore:function(a){l.save();
z.ignore(a);s();d.focus();d.domChanged();d.cursorIntoView()},state:function(){return{ignore:z.state(),cache:O.dumpCache()}}}}function NT(a,b,c){return{patterns:function(){return[RegExp("(\\W)(\\w+)(\\W*)$","gi"),RegExp("(^)(\\w+)(\\W*)$","gi")]},useCursor:f.constant(!0),generate:function(a,b){var g=q.fromText(b.text());return hd.species({elements:f.constant([g]),cursor:h.none,dictation:f.constant(h.some(function(){return c(b.text())})),notify:h.none})},derive:function(a){var c=a.index+a[1].length,
g=c+a[2].length;return b(a[2]).map(function(a){return{startIndex:f.constant(c),finishIndex:f.constant(g),text:f.constant(a)}})},label:a}}function ep(a,b){var c=db.sanitise(a);return{patterns:function(){return[RegExp("("+c+")","gi")]},useCursor:f.constant(!0),generate:function(a,c){var g=q.fromTag("span",a.dom());g.dom().innerHTML=b;return hd.species({elements:f.constant([g]),cursor:h.none,dictation:h.none,notify:h.none})},derive:function(a){var b=a.index;a=b+a[0].length;return h.some({startIndex:f.constant(b),
finishIndex:f.constant(a)})}}}function Bf(a,b,c){var d=db.sanitise(b);return{patterns:function(){return[RegExp("^("+d+"\\s)(.+)","gi")]},useCursor:f.never,generate:function(b,d){var l=q.fromTag(a,b.dom());t.set(l,"list-style-type",c);var p=q.fromTag("li",b.dom()),r=q.fromTag("li",b.dom());v.append(r,q.fromTag("br",b.dom()));K.append(l,[p,r]);K.append(p,d.clones());return hd.species({elements:f.constant([l]),cursor:f.constant(h.some(S.point(r,0))),dictation:h.none,notify:h.none})},derive:function(a){var b=
a[1];return h.some({startIndex:f.constant(a.index),finishIndex:f.constant(a.index+b.length)})}}}function Jk(a,b,c,d){var e=db.sanitise(b),g=db.sanitise(c);return{patterns:function(){return[RegExp(e+"([^\\s].*?[^\\s])"+g,"gi"),RegExp(e+"([^\\s])"+g,"gi")]},useCursor:f.always,generate:function(a,b){var c=q.fromTag(d,a.dom()),e=q.fromText(b.text());v.append(c,e);return hd.species({elements:f.constant([c]),cursor:h.none,dictation:h.none,notify:h.none})},derive:function(a){return h.some({startIndex:f.constant(a.index),
finishIndex:f.constant(a.index+a[0].length),text:f.constant(a[1])})}}}function OT(a,b){var c=db.sanitise(a);return{patterns:function(){return[RegExp("("+c+")","gi")]},useCursor:f.always,generate:function(a,c){return hd.species({elements:f.constant([q.fromTag(b,a.dom())]),cursor:h.none,dictation:h.none,notify:h.none})},derive:function(a){return h.some({startIndex:f.constant(a.index),finishIndex:f.constant(a.index+a[0].length)})}}}function PT(a,b,c){var d=c.whitelist.fold(QT,function(a){return function(b,
c){return k.contains(a,b)?c:[]}}),e=function(a){return{derive:a.derive,generate:a.generate,patterns:a.patterns,useCursor:a.useCursor,allowed:f.constant(!0)}},g=function(a,b){return{derive:a.derive,generate:a.generate,patterns:a.patterns,useCursor:a.useCursor,allowed:function(a){return!1===eb.closest(a,b)}}};c=d("headings",[e(ug.spaced("######","h6")),e(ug.spaced("#####","h5")),e(ug.spaced("####","h4")),e(ug.spaced("###","h3")),e(ug.spaced("##","h2")),e(ug.spaced("#","h1"))]);var l=d("lists",[e(Bf("ul",
"*","disc")),e(Bf("ol","1.","decimal")),e(Bf("ol","a.","lower-alpha")),e(Bf("ol","i.","lower-roman")),e(Bf("ol","1)","decimal")),e(Bf("ol","a)","lower-alpha")),e(Bf("ol","i)","lower-roman"))]),p=d("semantics",[e(Jk("bold","**","**","strong")),e(Jk("bold","__","__","strong")),e(Jk("italic","*","*","em")),e(Jk("italic","_","_","em"))]),r=d("entities",[e(ep("(c)","&copy;")),e(ep("--","&mdash;")),e(ep("-","&mdash;"))]),h=d("capitalisation",[e(vx.normalise()),e(vx.capitalise())]),w=function(a){return u.translate("aria.autocorrect.announce",
a)};a=a.hasAutocorrect()?d("autocorrect",[e(NT("autocorrect",a.lookup,w))]):[];var w=d("hr",[e(OT("---","hr"))]),G=hb.cached(function(a){a.runOrOnLoaded(function(b){var c=hb.cached(function(){y.owner(b).dom().execCommand("AutoUrlDetect",!1,!1)});a.events.focus.bind(function(){c()})});return d("link",[g(RT,"a")])});return{headings:f.constant(c),semantics:f.constant(p),entities:f.constant(r),cases:f.constant(h),lists:f.constant(l),dynCustoms:function(){var a=b.get();return k.map(a,e)},link:G,hr:f.constant(w),
corrections:f.constant(a)}}function ST(a,b,c,d,e,g,l,p){var r=h.none(),z=h.none(),w=PT(e,g,p),G=k.flatten([w.link(b),w.hr(),w.semantics(),w.entities()]),m=function(){return k.flatten([G,a.autocorrect().get()?w.corrections():[],a.capitalisation().get()?w.cases():[],w.dynCustoms()])},n=w.headings().concat(w.lists()),q=function(a,b,c,d){var e=Kk.descend(b,c),g=TT.get(e.element(),e.offset());c=k.filter(d,function(a){return a.allowed(b)});return ab.findMap(c,function(b){return a(g,b,e.element(),e.offset())})},
s=function(a,c){b.undoOperation(function(){a(function(){return c.execute(b,l)}).each(function(a){b.setSelection(a.element(),a.offset(),a.element(),a.offset())});b.domChanged()})};return{spaceDown:function(){b.getSelection().map(function(a){a.collapsed()&&(r=q(fp.inline,a.startContainer(),a.startOffset(),m()))})},spaceUp:function(){var a=f.curry(s,d.run);r.each(a);r=h.none()},enterDown:function(){b.getSelection().map(function(a){a.collapsed()&&(z=q(fp.block,a.startContainer(),a.startOffset(),n.concat()));
z=z.orThunk(function(){return q(fp.inline,a.startContainer(),a.startOffset(),m())})})},enterUp:function(){var a=f.curry(s,f.apply);z.each(a);z=h.none()}}}function UT(a,b,c,d){var e=a.profile(),g=a.content(),l=a.undo(),p=a.context(),r=a.operations(),h=a.internalevents(),k=D.create({analytics:s(["message"])}),G=co(c.customs),m=ST(e,g,l,r,b,G,h,d);g.events.keydown.bind(function(a){ld.isSpace(a)&&m.spaceDown()});g.events.keyup.bind(function(a){ld.isSpace(a)&&m.spaceUp()});p.events.preLinebreak.bind(function(a){m.enterDown()});
p.events.postLinebreak.bind(function(a){m.enterUp()});var n=function(a){return function(){var b=a.get();a.setAndCache(!b)}},q=function(a){return function(b){(!0===a.get()?b.select:b.deselect)()}};b=function(a,b,c){return{toggle:!0,value:a,textkey:u.key(b),action:n(c),updateGeneral:q(c),icon:!1,tooltip:Wa.get}};return{ui:$a.ui({"capitalisation.general":b("capitalisation","cog.capitalisation",a.profile().capitalisation()),"autocorrect.general":b("autocorrect","cog.autocorrect",a.profile().autocorrect())}),
destroy:f.noop,events:k.registry,customMacros:f.constant(G),state:function(){return{customs:G.state()}}}}function VT(){var a=D.create({insert:s(["embed"]),input:s(["raw"])}),b=q.fromTag("span"),c=hc.textarea("media-embed","media.embed"),d=u.translate("media.placeholder"),e=Oa.describe(c.textarea(),d);v.append(c.element(),e);e=P.span();Q.hidden(e,!0);n.add(e,x.resolve("media-icon"));m.set(e,"alt",u.translate("media.insert"));var g=ac(),l=u.translate("media.insert");g.content(qc(l));n.add(g.element(),
[x.resolve("media-insert")]);Wc.makeDefault(g.element());Q.button(g.element(),l);m.set(c.textarea(),"placeholder",d);wx(g);a.registry.input.bind(function(){var a=$.get(c.textarea());g.disabled()?""!==a&&(g.enable(),ha.enable(g.element())):""===a&&wx(g)});var p=E.bind(c.textarea(),"input",a.trigger.input);K.append(b,[e,c.element(),g.element()]);var r=function(){var b=$.get(c.textarea());a.trigger.insert(b);WT&&T.blur(c.textarea())},d=[ba.input(c.textarea(),pa.input({})),ba.button(g.element(),pa.button({execute:qb.click}))];
g.events.click.bind(function(){r()});return{element:f.constant(b),execute:r,empty:function(){$.set(c.textarea(),"")},keylayouts:f.constant(d),events:a.registry,destroy:function(){p.unbind();g.destroy();B.remove(b)}}}function XT(a){var b=a.region(),c=a.content(),d=a.smartview(),e=VT(),g=ba.inlineDialog(e.element(),e.keylayouts(),pa.inlineDialog({ok:e.execute,cancel:f.noop}));Q.dialog(e.element(),u.translate("insert.media"));var l=Sd.popup(d.uiContainer,d.bounds,b,[],sf.editorContainer()).initMenu(e.element());
ea.add(e.element(),[x.resolve("media-embed-dialog"),x.resolve("inline-dialog")]);g.events.exit.bind(function(){p()});var p=function(){l.closed()||(l.close(),e.empty(),c.focus(),g.exit())};a=D.create({insert:M(["embed"],e.events.insert)});return{element:e.element,showPopup:function(a){l.open();a.show(b,c,d,l,e);g.enter()},isClosed:l.closed,events:a.registry,destroy:function(){p();g.destroy();e.destroy();l.destroy()}}}function YT(a){a=P.div();n.add(a,x.resolve("media-widget"));var b=P.span();n.add(b,
x.resolve("media-widget-field"));var c=q.fromTag(Mb.isLarge()?"textarea":"input");n.add(c,x.resolve("media-widget-input"));var d=u.translate("media.placeholder");m.set(c,"placeholder",d);m.set(c,"title",d);v.append(b,c);d=Vh([x.resolve("media-widget-insert")],"insert.media");v.append(b,d.element());v.append(a,b);var e=Uh.text(b,c),g=D.create({insert:s(["embed"]),toEditor:s([])}),l=function(){var a=$.get(c),a=ZT?a.replace(/=\u201c([^\u201d]*)\u201d/g,'="$1"'):a;g.trigger.insert(a);g.trigger.toEditor()};
d.events.click.bind(function(){l()});return{element:f.constant(a),addEscape:function(a){v.append(b,a.element())},open:function(){$.set(c,"");T.focus(c)},close:f.noop,execute:function(){l()},destroy:f.noop,type:f.constant(e),events:g.registry}}function $T(a){var b=a.content(),c=function(){var c=sc.destroyable(),d=function(){c.clear();b.events.mousedown.unbind(d)};return{open:function(){var e=XT(a);e.events.insert.bind(function(c){c=c.embed();a.insertFilteredHtmlAtCursor(c);b.cursorIntoView();d()});
e.showPopup(cp);b.events.mousedown.bind(d);c.set(e)},destroy:function(){d()}}},d=function(){var c=YT(a);c.events.insert.bind(function(c){c=c.embed();a.insertFilteredHtmlAtCursor(c);b.cursorIntoView()});return wk(a,c)},e=(Y.detect().deviceType.isTouch()?d:c)();return{ui:$a.ui({"media.general":{toggle:!1,value:"media",textkey:u.key("insert.media"),action:e.open,updateGeneral:f.noop,icon:!0,shortcut:!1,hasPopup:!1,hasDialog:!0}}),destroy:function(){e.destroy()}}}function aU(a,b){var c=q.fromTag("div");
n.add(c,x.resolve("mentions-business-card"));var d=a.initMenu(c),e=null,g=function(a,b,e){B.empty(c);v.append(c,e);d.open();a(b,d)};return{showFull:function(a){return function(b,c){clearTimeout(e);e=setTimeout(function(){g(a,b,c)},bU)}}(b.showUnder),showMini:f.curry(g,b.showAtSide),hide:function(){clearTimeout(e);d.close()},destroy:function(){clearTimeout(e);d.destroy()}}}function cU(a){var b=h.none();return{fetch:function(b,d){a.fetch(b,function(a,b){var c=xx("fetch()",a,b);d(c)})},fetchExtra:function(b,
d,e){a.menuExtra(b,d,function(a,b){var c=xx("menuExtra()",a,b);e(c)})},menuHover:function(c,d){b=h.some(c.userid);a.menuHover(c,function(a){a=gp("menuHover() callback",a);b.is(c.userid)&&d(a)})},hover:function(b,d){a.hover(b.dom(),function(a){a=gp("hover() callback",a);d(a)})},menuComplete:function(b){b=a.menuComplete(b);return gp("menuComplete result",b)},hoverSelector:function(){return a.hoverSelector},isTrigger:function(a){return 64===a}}}function dU(a){var b=bb.mainmenu(q.fromDom(document),x.resolve("mentions-list"),
[]),c=wa({}),d=function(a){return a.userid},e=function(a){return a.name},g=function(a,b){J.child(a,".ephox-gopher-item-text").each(function(a){ua.set(a,b)})},l=function(a){return k.map(a,function(a){var b=bb.item(q.fromDom(document),a.userid,a.name);g(b,a.name);return b})},p=function(a){return k.foldr(a,function(a,b){var c={};c[b.userid]=b;return Z.merge(a,c)},{})},r=function(a,b){var c=l(a),d=k.map(b,function(a){return bb.item(q.fromDom(document),a.key,a.value)});return c.concat(d)},z=function(a){var b=
m.get(a,"data-ephox-gopher-item-value"),d=c.get();return void 0!==d[b]?hp.fetched(d[b].userid,d[b].name,d[b].rawname,d[b],a):k.contains([yx,zx],b)?hp.none(a):hp.extra(b,a)};return{element:f.constant(b),selected:function(){return J.descendant(b,".ephox-gopher-item-selected")},setData:function(d,e){var g=p(d);c.set(Z.merge(c.get(),g));g=0===d.length?[{userid:zx,name:a.get("mentions.lookahead.noresults")}]:d;g=r(g,e);B.empty(b);K.append(b,g)},getInfo:z,getUserId:function(a){return z(a).fold(h.none,h.none,
f.compose(h.some,d))},setLoading:function(){var c=[{userid:yx,name:a.get("mentions.lookahead.searching")}],c=r(c,[]);B.empty(b);K.append(b,c)},getName:function(a){return z(a).fold(h.none,h.none,f.compose(h.some,e))},refind:function(a){a=m.get(a,"data-ephox-gopher-item-value");return void 0!==a&&null!==a?J.descendant(b,'[data-ephox-gopher-item-value="'+a+'"]'):h.none()},isOpen:function(){return Tb.isVisible(b)}}}function eU(a,b,c){var d=function(a){Oa.speak(ca.body(),a)},e=dU(c);a.setView(e.element());
var g=wa(""),l=function(b){r(b.rawData(),b.extra());a.reposition(h.none())},p=function(a){var b=e.selected();a();return b.bind(e.refind)},r=function(b,g){d(c.get("mentions.lookahead.open"));a.updateItems(p,function(){e.setData(b,g)})},k=function(a){return e.isOpen()&&2<=a.length?ip.refetch(a):2<=a.length?ip.open(a):ip.nothing()},w=function(a){a.fold(m,l)},G=D.create({accept:s(["information"]),menuHover:s(["element","info"]),none:s([])}),m=function(){a.cancel()};return{setData:r,isOpen:function(){return e.isOpen()},
lookup:function(c,d){var l=aa.startsWith(c,"@")?c.substring(1):c;g.set(l);var p=f.noop;k(l).fold(p,function(a){b.fetch(a,w)},function(c){e.setLoading();a.open(d);b.fetch(c,w)})},close:function(){a.close()},onClick:function(a){e.getInfo(a).fold(function(){G.trigger.none()},function(a,c){b.fetchExtra(a,g.get(),w)},function(a,b,d,e,g){Oa.speak(ca.body(),c.get("mentions.lookahead.selected",d));G.trigger.accept(e)})},onHover:function(a){d(pb.get(a));e.getInfo(a).fold(f.noop,f.noop,function(a,b,c,d,e){G.trigger.menuHover(e,
d)})},events:G.registry}}function fU(a,b,c,d){var e=q.fromTag("div");n.add(e,x.resolve("mentions-lookahead-container"));var g=c.initMenu(e),l=wa(h.none()),p=D.create({close:s([])}),r=function(){return Tb.isVisible(e)};c=function(){l.set(h.none());g.close()};g.events.close.bind(function(){b.close();p.trigger.close()});a.events.mousedown.bind(c);return{element:f.constant(e),destroy:function(){l.set(h.none());g.destroy()},setView:function(a){B.empty(e);v.append(e,a)},updateItems:function(a,c){a(c).fold(function(){b.setActiveMenu(e)},
function(a){b.select(a)})},reposition:function(a){a.isSome()&&l.set(a);r()&&l.get().each(function(a){d.showUnder(a,g)})},open:function(c){a.body().each(function(a){g.open();d.showUnder(c,g);l.set(h.some(c));b.open([b.sandbox()],a)})},isOpen:r,cancel:function(){l.set(h.none());p.trigger.close()},close:c,events:p.registry}}function gU(a,b,c,d,e,g){var l=fU(a,b,c,d),f=eU(l,e,g);a=D.create({menuHover:M(["element","info"],f.events.menuHover),accept:M(["information"],f.events.accept),close:M([],l.events.close)});
f.events.none.bind(l.close);return{onHover:f.onHover,onClick:f.onClick,close:function(){f.isOpen()&&f.close();l.isOpen()&&l.cancel()},isOpen:l.isOpen,lookup:function(a,b){l.reposition(h.some(b));f.lookup(a,b)},destroy:function(){l.destroy()},events:a.registry}}function hU(a,b){var c=wa([]),d=null,e=null,g=D.create({contentHover:s(["target"]),stopHover:s([]),menuHover:s(["info"])});a.runOrOnLoaded(function(a){c.set([E.bind(a,"mouseover",function(a){z(a)}),E.bind(ca.body(),"mouseover",function(a){J.closest(a.target(),
jp).each(f)}),E.bind(a,"mouseout",function(a){w(a)}),E.bind(ca.body(),"mouseout",function(a){m(a)})])});var l=function(a){clearTimeout(d);clearTimeout(e);d=setTimeout(function(){g.trigger.contentHover(a)},300)},f=function(){clearTimeout(e)},r=function(){clearTimeout(d);e=setTimeout(function(){g.trigger.stopHover()},300)},z=function(a){J.closest(a.target(),b).each(function(a){l(a);g.trigger.contentHover(a)})},w=function(a){if(!eb.closest(a.target(),b))return!1;h.from(a.raw().relatedTarget||a.raw().toElement).map(q.fromDom).filter(function(a){return eb.closest(a,
b)}).isNone()&&r()},m=function(a){if(!eb.closest(a.target(),jp))return!1;h.from(a.raw().relatedTarget||a.raw().toElement).map(q.fromDom).filter(function(a){return eb.closest(a,jp)}).isNone()&&r()};return{destroy:function(){k.each(c.get(),function(a){a.unbind()})},events:g.registry}}function iU(a){return{get:function(b){var c=a[b];return R.isString(c)?jU(c,Array.prototype.slice.call(arguments,1)):u.translate.apply(void 0,arguments)}}}function Ax(a){return{show:function(b,c,d,e,g){c=fg(0,0);b=b.directionAt(a);
b=Lk.mainmenu(d,f.constant(a),c,b);d.show(e,b);e.setComponents([a,g.element()])},showWith:function(b,c,d,e,g){b=b.directionAt(a);b=Lk.submenu(d,f.constant(a),b);d.show(e,b);e.setComponents([a,g.element()])}}}function kU(a,b,c,d){return{showAtSide:function(d,g){Ax(d).showWith(a,b,c,g,{element:f.constant(g.element())})},showUnder:function(d,g){Ax(d).show(a,b,c,g,{element:f.constant(g.element())})}}}function lU(a,b){var c=iU(b.aria),d=a.content(),e=a.smartview(),g=a.region(),l=a.operations(),p=cU(b),
r=Sd.menuSystem(e.uiContainer,f.constant("mentions"),Gh.menu({focusActive:!1}));t.set(r.sandbox(),"z-index",String(void 0!==b.zIndex?b.zIndex:mU));var h=Sd.popup(r.sandbox,e.bounds,g,[],sf.editorContainer()),e=kU(g,d,e,r.sandbox()),w=gU(d,r,h,e,p,c),m=aU(h,e);nU.configure(a.filter());var I=hU(d,p.hoverSelector());w.events.menuHover.bind(function(a){p.menuHover(a.info(),function(b){b.fold(f.noop,function(b){Tb.isVisible(a.element())&&m.showMini(a.element(),b)})})});I.events.contentHover.bind(function(a){p.hover(a.target(),
function(b){b.fold(f.noop,function(b){w.close();m.showFull(a.target(),b)})})});I.events.stopHover.bind(function(a){m.hide()});r.events.itemHover.bind(function(a){a=a.item();w.onHover(a);d.backupSelection();T.focus(a);d.focus()});r.events.itemAction.bind(function(a){w.onClick(a.item())});d.events.nodeChanged.bind(function(a){d.getSelection().each(function(b){b.collapsed()&&(vg.detectOn(b,p).fold(w.close,w.close,f.noop,w.close),a.prev().each(function(a){kp.validatePrevious(a,b.startContainer())}))})});
d.events.keyup.bind(function(a){d.getSelection().each(function(b){b.collapsed()&&(k.contains(n,a.original().raw().which)||vg.withinIncomplete(b.startContainer()).each(function(a){var b=pc.descendants(a,A.isText),b=k.map(b,Da.get).join("");w.lookup(b,a);m.hide()}))})});d.events.keypress.bind(function(a){var b=a.original();p.isTrigger(b.raw().which)&&d.getSelection().each(function(a){Bx.isValidMention(a.startContainer(),a.startOffset())&&(a=function(){var a=kp.create();Cc.insertAtCursor(d,[a]).each(function(a){d.setSelection(a.element(),
a.offset(),a.element(),a.offset())});b.kill();Oa.speak(ca.body(),c.get("mentions.initiated"))},vg.detect(d,p).fold(a,a,f.noop,f.noop))})});var n=k.flatten([N.ENTER(),N.DOWN(),N.UP()]);a.context().events.preLinebreak.bind(function(a){s(function(b,c){w.isOpen()?a.handler().handle():y()},f.noop)});d.events.keydown.bind(function(a){var b=a.original().raw().which;k.contains(N.ESCAPE(),b)?s(function(){y();a.original().kill()},f.noop):k.contains(n,b)&&w.isOpen()&&(r.simulateKey(a.original()),a.original().kill())});
var s=function(a,b){vg.detect(d,p).fold(b,b,a,b)},u=function(a){s(function(b,c){p.menuComplete(a).each(function(a){v.after(c,a);B.remove(c);var b=q.fromText(Ua.zeroWidth());v.after(a,b);d.setRelativeSelection(da.after(b),da.after(b));x()})},x)},x=function(){m.hide();w.close()},y=function(){s(function(a,b){Oa.speak(ca.body(),c.get("mentions.lookahead.cancelled"));l.atomic(function(){kp.cancel(b)});x()},x)};w.events.close.bind(y);w.events.accept.bind(function(a){u(a.information())});return{destroy:function(){w.destroy();
m.destroy();I.destroy();r.destroy()}}}function oU(a){var b=function(a){var b;b={zh:"zh_cn",pt:"pt_br"};b=void 0===b[a]?a:b[a];b=u.has("parlance."+b)?u.key("parlance."+b):u.raw(b);return pU(b,a,h.none())},c=k.map(a.languages,b);return{build:function(d){return d.filter(function(b){return!k.contains(a.languages,b)}).fold(function(){return c},function(a){return c.concat([b(a)])})}}}function qU(a,b,c){var d=a.content(),e=a.undo(),g=oU(c),l=function(){return d.on(function(a){return Cf.language(a).fold(oa.off,
oa.on)}).getOrThunk(oa.unchanged)};return{ui:$a.ui({"parlance.dropdown":f.constant({name:"parlance",textkey:u.key("parlance.set"),menuTextkey:u.key("parlance.menu"),updateDropdown:function(a){var b=l(),c=b.fold(h.none,h.none,h.some),c=g.build(c);a.populate([{label:h.none,items:f.constant(c)}]);b.fold(a.deactivate,f.noop,a.activate);b.fold(a.untick,f.noop,a.tick)},action:function(c){var g=function(a){return Ac.language(d.isRoot,a,c)};b.lock(function(){tb.cata(a.selections().get(),f.noop,bk.multiple(d,
e,g),bk.single(d,e,g));d.getSelection().each(function(a){b.check(a.startContainer(),a.startOffset(),a.endContainer(),a.endOffset())})});setTimeout(function(){d.uiChanged()},0)},values:g.build(h.none()),render:h.none(),toggle:!1,describe:function(){return u.key("parlance.set")}})},f.identity),destroy:f.noop}}function rU(){var a=q.fromTag("textarea");Q.label(a,u.translate("plaincode.dialog.editor"));Q.textarea(a);return{element:f.constant(a)}}function sU(a){var b=D.create({execute:s(["html"]),close:s([])}),
c=ce(a);n.add(c.element(),x.resolve("plaincode-modal"));c.setTitle(u.translate("plaincode.dialog.title"));var d=function(){e();b.trigger.close()},e=function(){c.hide()},g=rU();c.setContent(g.element());a=Wc.okCancel("plaincode");a.events.ok.bind(function(){b.trigger.execute($.get(g.element()));d()});a.events.cancel.bind(function(){d()});c.events.close.bind(function(){l.exit();d()});c.setControls(a.element());var l=qb.setup(c,a,[ba.single(g.element(),[U.rule(U.is(N.ENTER()[0]),function(a){a.raw().stopPropagation();
return h.none()})],h.none())]);return{element:c.element,open:function(){c.show();l.enter()},setHtml:function(a){$.set(g.element(),a)},destroy:function(){c.destroy();l.destroy()},hide:e,events:b.registry}}function tU(a){var b=sU(a),c=function(){d.trigger.close();b.events.close.unbind(c);b.events.execute.unbind(e);b.destroy()},d=D.create({execute:s(["html"]),close:s([])}),e=function(a){d.trigger.execute(a.html())};b.events.close.bind(c);b.events.execute.bind(e);return{show:function(){b.open()},setHtml:b.setHtml,
events:d.registry}}function uU(a,b){var c=a.content(),d=a.shortcuts(),e=a.smartview(),g=b.setContent,l=b.getContent;d.events.plaincode.bind(function(){p()});var p=function(){var a=tU(e.uiContainer());a.events.close.bind(function(){c.toEditing()});a.events.execute.bind(function(a){g(a.html())});c.toReading();a.setHtml(l());a.show()};return{destroy:f.noop}}function vU(){var a=D.create({grow:s(["x","y"])}),b=1,c=1,d=h.none(),e=function(a){return t.getRaw(a,"width").bind(function(a){return aa.endsWith(a,
"px")?h.some(parseInt(a,10)):h.none()}).getOrThunk(function(){return qa.get(a)})};return{setTarget:function(a){d=h.some(a);c=qa.get(a)/(ia.get(a)+0)},setMultiplier:function(a){b=a},clearTarget:function(){d=h.none()},mutate:function(g,l){d.each(function(d){var f=e(d)+b*g,h=1===f?1:Math.floor(f/c);5<=f&&5<=h&&(qa.set(d,f),ia.set(d,h));a.trigger.grow(g,l)})},events:a.registry,destroy:f.noop}}function wU(a){var b=h.none(),c=vU();c.events.grow.bind(function(){g()});var d=Ih.transform(c,{layerClass:x.resolve("layer-below-dialog")});
d.events.stop.bind(function(){b.each(function(a){l.trigger.stop(a)})});n.add(d.element(),x.resolve("resize-eph-block"));var e=xU.create();e.events.click.bind(function(b){c.setMultiplier(b.multiplier());a.body().each(function(a){d.go(y.documentElement(a))})});var g=function(){e.clear();b.each(lp.clean)},l=D.create({stop:s(["target"])});return{showHandles:function(a){var g=y.documentElement(a);c.setTarget(a);b=h.some(a);d.on();e.refresh(g,a)},hideHandles:g,events:l.registry,destroy:function(){e.destroy();
c.destroy();d.destroy();b=h.none()}}}function yU(a){var b=a.content(),c=a.undo();a=a.filter();lp.configure(a);var d=wU(b),e=function(a){b.getSelection().bind(ib.detect).each(function(d){c.save();ib.remove(b,d.image());b.domChanged();c.save();a.original().kill()})},g=function(a){b.getSelection().bind(ib.detect).each(function(c){b.setRelativeSelection(da.before(c.image()),da.before(c.image()));a.original().kill()})},l=function(a){b.getSelection().bind(ib.detect).each(function(c){b.setRelativeSelection(da.after(c.image()),
da.after(c.image()));a.original().kill()})},p=function(){b.getSelection().bind(ib.detect).fold(function(){d.hideHandles()},function(a){a=a.image();d.showHandles(a);ib.reselect(b,a)})};b.events.loaded.bind(p);b.events.domChanged.bind(p);b.events.paint.bind(p);b.events.nodeChanged.bind(p);b.events.selectionchange.bind(function(a){b.getSelection().bind(ib.detect).fold(function(){p()},function(){f.noop()})});var r=h.none();b.events.ready.bind(function(a){a=a.api().body();r=h.some(rk.handle(a,function(a){"img"===
A.name(a.target())&&ib.select(b,a.target())}))});b.events.mousedown.bind(function(a){a=a.original().target();"img"===A.name(a)&&lp.disableNativeResize(a)});b.events.keydown.bind(function(a){ld.isDel(a)||ld.isBackspace(a)?e(a):ld.isLeftOrUp(a)?g(a):ld.isRightOrDown(a)&&l(a)});d.events.stop.bind(function(a){a=a.target();d.showHandles(a);ib.reselect(b,a)});return{destroy:function(){d.destroy();r.each(function(a){a.unbind()})}}}function zU(a,b){var c=a.content(),d=a.filter(),e=a.selections(),g=f.curry(d.filterDom,
d.output());c.events.ready.bind(function(a){a=a.api();var b=AU(a,d),r=function(a,l){tb.cata(e.get(),f.noop,function(c){BU(c,g).each(function(d){b(a,d.text,d.html,f.curry(l,function(){CU(c)}))})},function(e,g){DU(e,g,d).each(function(d){b(a,d.text,d.html,f.curry(l,function(){c.insertHtmlAtCursor("")}))})})};a.events.cut.bind(function(a){r(a.original(),f.apply)});a.events.copy.bind(function(a){r(a.original(),f.noop)})});return{destroy:f.noop}}function EU(a,b){var c=function(a,b,c){a(c,b);return h.some({element:f.constant(b),
cleanup:function(){B.remove(b)},splits:f.constant([])})},d=function(a){return{none:h.none,start:f.curry(c,v.before,a),end:f.curry(c,v.after,a),middle:function(b,d,e){return c(v.after,a,d)}}},e=function(a){var b=function(a){return function(b,c){v.appendAt(b,c,a)}};return{prepend:f.curry(c,v.prepend,a),before:f.curry(c,v.before,a),after:f.curry(c,v.after,a),insertAt:function(d,e){return c(b(e),a,d)},append:f.curry(c,v.append,a)}};return{before:{text:d(a),element:{hasPrevious:e(a).insertAt,first:e(a).prepend,
empty:e(a).before,otherwise:e(a).append}},after:{text:d(b),element:{hasNext:e(b).insertAt,rightEdge:e(b).append,empty:e(b).after,otherwise:e(b).append}},general:{insertBefore:e(a).before,insertAfter:e(b).after}}}function FU(a){var b=Mk.fromUnsafe(a),c=q.fromTag("span"),d=q.fromTag("span");k.each([c,d],a.markAsEphemeral);var e=EU(c,d);return{backup:function(){a.getSelection().each(function(g){g.collapsed()?(Cc.insertAtCursor(a,[c]),B.unwrap(d),b.begin()):Cx.both(e,g.startContainer(),g.startOffset(),
g.endContainer(),g.endOffset())})},insert:function(e,l,f){a.setRelativeSelection(da.after(c),ca.inBody(d)?da.before(d):da.after(c));Cc.insertAtCursorWith(a,e,l,f).each(function(b){!eb.ancestor(b.element(),"table")||"td"!==A.name(b.element())&&"th"!==A.name(b.element())?a.setRelativeSelection(da.after(b.element()),da.after(b.element())):a.setSelection(b.element(),0,b.element(),0)});b.end();uf.cursorIntoView(a);k.each([c,d],B.unwrap)},cancel:function(){k.each([c,d],B.unwrap)}}}function Dx(a,b){var c=
D.create({cancel:s([]),error:s(["message"]),insert:s(["elements","assets","correlated","isInternal"])}),d=function(d,e,f){d=Ex.choose(a,b,d);d.capture()&&f();f=k.map(d.steps(),function(a){return a(e)});Ex.run(f,d.input()).get(function(a){var b=a.bundle().isInternal().getOr(!1);Ka.cata(a.response(),function(a){c.trigger.error(a)},function(a,d){c.trigger.insert(a,Fx(d),d,b)},function(){c.trigger.cancel()},function(a,d,e){c.trigger.insert(a,Fx(d),d,b);c.trigger.error(e)})})},e=Qc.tap(function(a){var b=
GU.fromEvent(a);Gx.willBlock()&&(e.control.block(),a.preventDefault());d(b,e.control,function(){a.preventDefault()})});return{paste:e.instance,pasteCustom:function(a){var b=Qc.tap(f.noop);d(a,b.control,f.noop)},isBlocked:e.control.isBlocked,destroy:f.noop,events:c.registry}}function Nk(){return function(a,b){return va.sync(function(a){var d=function(){va.call(a,{response:b.response(),bundle:b.bundle()})},e=function(d,e){Nc.mapM(d,HU).get(function(d){va.call(a,{response:e(d),bundle:b.bundle()})})};
Ka.cata(b.response(),d,function(a,b){e(b,function(b){return Ka.paste(a,b)})},d,function(a,b,c){e(b,function(b){return Ka.incomplete(a,b,c)})})})}}function Hx(a){return function(b,c){return va.sync(function(b){var e=function(){va.call(b,{response:c.response(),bundle:c.bundle()})},g=function(a){var e=IU(a),g=Nc.mapM(e,function(a){return a.asset()}),l=k.map(e,function(a){return a.image()});g.get(function(e){e=je.updateSources(e,l);va.call(b,{response:Ka.paste(a,e),bundle:c.bundle()})})},l=function(l,
f,h){!1===a.allowLocalImages?(f=k.filter(Ix(l),Jx),k.each(f,B.remove),va.call(b,{response:0<f.length?JU(l):c.response(),bundle:c.bundle()})):0===f.length?g(l):e()};Ka.cata(c.response(),e,l,e,l)})}}function Kx(a){var b=a.translations,c=D.create({insert:s(["elements","correlated"]),incomplete:s(["elements","correlated","message"])});return{events:c.registry,processRtf:function(a,e,g,l){g=wg.images(g);var f=ei.find(a);KU.convert(f,g,function(b,g){var f=y.children(a),p=b.concat(e);g?c.trigger.incomplete(f,
p,"errors.imageimport.failed"):c.trigger.insert(f,p);l()},b)}}}function LU(a,b){var c=D.create({insert:s(["elements","correlated"]),incomplete:s(["elements","correlated","message"])}),d=Kx(b);d.events.incomplete.bind(function(a){c.trigger.incomplete(a.elements(),a.correlated(),a.message())});d.events.insert.bind(function(a){c.trigger.insert(a.elements(),a.correlated())});return{events:c.registry,gordon:function(e,g){var l=function(a){d.processRtf(e,g,a.rtf(),a.hide())},f=function(a){var b=ei.find(e);
k.each(b,B.remove);c.trigger.incomplete(y.children(e),g,a)},h=function(){var a=ei.find(e);k.each(a,B.remove);c.trigger.insert(y.children(e),g)},z=function(a){f(a.message())};if(!0===b.allowLocalImages&&!0===b.enableFlashImport){var w=a(b);w.events.response.bind(l);w.events.cancel.bind(h);w.events.failed.bind(z);w.events.error.bind(z);w.open()}else f("errors.local.images.disallowed")}}}function Lx(a,b){var c=LU(a,b),d=wa(h.none()),e=function(a){d.get().each(function(b){va.call(b,{response:a,bundle:rb.nu({})})})};
c.events.insert.bind(function(a){e(Ka.paste(a.elements(),a.correlated()))});c.events.incomplete.bind(function(a){e(Ka.incomplete(a.elements(),a.correlated(),a.message()))});return function(a,b){return va.sync(function(a){var e=function(){va.call(a,{response:b.response(),bundle:b.bundle()})},g=function(b,g,l){d.set(h.some(a));var f=q.fromTag("div");K.append(f,b);ei.exists(f)?c.gordon(f,g,l):e()};Ka.cata(b.response(),e,g,e,g)})}}function MU(a,b){var c=b.translations,d=function(a,b,c){c(h.some(Z.merge(b,
{officeStyles:a,htmlStyles:a})))},e=function(b,e){var f=q.fromTag("div");n.add(f,Ec.resolve("styles-dialog-content"));var r=q.fromTag("p"),k=Od.fromHtml(c("cement.dialog.paste.instructions"));K.append(r,k);v.append(f,r);var r={text:c("cement.dialog.paste.clean"),tabindex:0,className:Ec.resolve("clean-styles"),click:function(){w.destroy();d(!1,b,e)}},k={text:c("cement.dialog.paste.merge"),tabindex:1,className:Ec.resolve("merge-styles"),click:function(){w.destroy();d(!0,b,e)}},w=a(!0);w.setTitle(c("cement.dialog.paste.title"));
w.setContent(f);w.setButtons([r,k]);w.events.close.bind(function(){e(h.none());w.destroy()});w.show()};return{get:function(a,c){var f=b[a?"officeStyles":"htmlStyles"];"clean"===f?d(!1,b,c):"merge"===f?d(!0,b,c):e(b,c)},destroy:f.noop}}function NU(a){return{play:function(b,c,d){var e=q.fromTag("div"),g=q.fromTag("iframe");t.setAll(e,{display:"none"});var l=E.bind(g,"load",function(){l.unbind();iw.write(g,b);var a=g.dom().contentWindow.document;if(void 0===a)throw"sandbox iframe load event did not fire correctly";
var h=q.fromDom(a),a=a.body;if(void 0===a)throw"sandbox iframe does not have a body";a=q.fromDom(a);h=c(h,a);B.remove(e);setTimeout(f.curry(d,h),0)});v.append(e,g);v.append(a,e)}}}function Mx(a,b,c){return function(c,e,g){var l=function(a){c.receive(a)},f=function(a,b,c){c=void 0!==c?c:a.type()===cb.END_ELEMENT_TYPE;return cb.token(b,c,{})},h={emit:l,emitTokens:function(a){k.each(a,l)},receive:function(b){a(h,b,f)},document:window.document};b(h);return h}}function mp(a,b,c){return{pred:a,action:b,
label:f.constant(c)}}function Nx(a,b,c){var d=Ox(a+" :: FALLBACK --- ",c);c=function(c,g,l){k.find(b,function(a){return a.pred(g,l)}).fold(f.constant(d),function(b){var c=b.label();return void 0===c?b.action:Ox(a+" :: "+c,b.action)})(c,g,l)};c.toString=function(){return"Handlers for "+a};return c}function OU(a,b){var c=Eb.state(0,void 0,[],[]),d=function(c){k.each(c.result(),function(c){c=c(b);a.emit(c)})};return{closeAllLists:function(){var a=Px.closeAllLists(c,0);c=a.state();d(a)},openItem:function(a,
b,l){l&&("UL"===l.tag&&PU[a-1]===l.type&&(l={tag:"UL"}),a=Px.openItem(c,a,b,l),c=a.state(),d(a))},getCurrentListType:function(){return c.type()},getCurrentLevel:function(){return c.level()}}}function QU(a,b,c,d,e){return function(g,l){var f=ic.asEvent(g).getOrDie("Must pass through event type").nativeEvent(),h=e(),k=l.response();return va.sync(function(e){var g=a(c);g.events.after.bind(function(a){a=a.container();if(np.browser.isSafari()&&eb.descendant(a,'img[src^="webkit-fake-url"]'))a=np.deviceType.isWebView()?
"webview.imagepaste":"safari.imagepaste",va.call(e,{response:Ka.error(a),bundle:rb.nu({})});else{b(a);n.add(a,Qx.binStyle());var c=RU.isWord(np,a),g=y.children(a),g=d.findClipboardTags(g).isSome();va.call(e,{response:k,bundle:rb.nu({isWord:c,isInternal:g,proxyBin:a,backgroundAssets:h})})}});h.convert(f);g.run()})}}function SU(a,b){var c=Kx(a),d=wa(h.none()),e=function(a){d.get().each(function(b){va.call(b,{response:a,bundle:rb.nu({})})})};c.events.insert.bind(function(a){e(Ka.paste(a.elements(),a.correlated()))});
c.events.incomplete.bind(function(a){e(Ka.incomplete(a.elements(),a.correlated(),a.message()))});return function(g,l){var p=ic.asWord(g).getOrDie("Word input required for rtf data"),r=function(a){return va.sync(function(b){var g=function(){va.call(b,{response:l.response(),bundle:l.bundle()})},p=function(g,l){d.set(h.some(b));var p=q.fromTag("div");K.append(p,g);a.fold(function(){var a=function(){k.each(c,B.remove);var a=y.children(p);e(Ka.incomplete(a,l,"errors.local.images.disallowed"))},b=function(){var a=
y.children(p);e(Ka.paste(a,l))},c=ei.find(p);return 0<c.length?a():b()},function(a){c.processRtf(p,l,a,f.noop)})};Ka.cata(l.response(),g,p,g,p)})};return Ok.match(p.rtf(),{getFromFlash:function(a){return b(g,l)},disabled:function(){return r(h.none())},fromClipboard:function(b){return r(!0===a.allowLocalImages?h.some(b.rtf()):h.none())}})}}function TU(a,b,c){var d=q.fromTag("div"),e="flashbin-wrapper-"+(UU.os.isOSX()?"cmd":"ctrl");n.add(d,Ec.resolve(e));var g;try{g=void 0!==navigator.plugins["Shockwave Flash"]}catch(l){g=
!1}a=(g?VU:WU)(d,a,b,c.translations);return{element:f.constant(d),reset:a.reset}}function XU(a,b,c){var d=function(a){return k.forall(b,function(b){return R.isFunction(a[b])})},e=!0,g=setInterval(function(){var b=a.dom();R.isFunction(b.PercentLoaded)&&100===b.PercentLoaded()&&d(b)&&(l(),c())},500),l=function(){e&&(clearInterval(g),e=!1)};return{stop:l}}function YU(a){var b=D.create({response:s(["rtf"]),spin:s([]),cancel:s([]),error:s(["message"]),reset:s([]),hide:s([]),failed:s(["message"])}),c=!1,
d=q.fromTag("div");n.add(d,Ec.resolve("flashbin-target"));var e=ZU.responsive();e.events.crashed.bind(function(){b.trigger.failed("flash.crashed")});e.events.timeout.bind(function(){b.trigger.failed("flash.crashed")});var g=function(){I.stop();if(!c){c=!0;try{var a=w.dom();W.each(z,function(b,c){var d=a[c];if(void 0===d)throw'Flash object does not have the method "'+c+'"';d.call(a,b)});b.trigger.reset();clearTimeout(u);n.remove(d,Ec.resolve("flash-activate"));m();O()}catch(e){console.log("Flash dialog - Error during load: ",
e)}}},l=Ne.permanent(g),p=function(){return!ca.inBody(w)},h=function(){return!w.dom().SetVariable},z={setSpinCallback:Ne.permanent(function(){e.start(1E3,10,p,h);b.trigger.spin()}),setPasteCallback:Ne.permanent(function(a){e.stop();setTimeout(function(){""===a?b.trigger.error("flash.crashed"):b.trigger.response(unescape(a))},0)}),setEscapeCallback:Ne.permanent(b.trigger.cancel),setErrorCallback:Ne.permanent(function(a){e.stop();b.trigger.error(a)}),setStartPasteCallback:Ne.permanent(f.noop)},w=function(){var b=
a.replace(/^https?:\/\//,"//"),c='    <param name="allowscriptaccess" value="always">    <param name="wmode" value="opaque">    <param name="FlashVars" value="onLoad='+l+'">';if(op.browser.isIE()&&10===op.browser.version.major){var d=La.generate("flash-bin");return q.fromHtml('<object classid="clsid:d27cdb6e-ae6d-11cf-96b8-444553540000" id="'+d+'"><param name="movie" value="'+b+'">'+c+"</object>")}return q.fromHtml('<object type="application/x-shockwave-flash" data="'+b+'">'+c+"</object>")}(),m=function(){t.setAll(w,
{width:"2px",height:"2px"})};m();var I=XU(w,W.keys(z),g);v.append(d,w);var O=function(){op.browser.isFirefox()&&window.getSelection().removeAllRanges();T.focus(w)},u=null,x=function(){n.add(d,Ec.resolve("flash-activate"));t.remove(w,"height");t.remove(w,"width");b.trigger.hide()},y=function(){t.set(d,"display","none");$U.each(function(a){k.each(a,function(a){a.unbind()})})};return{focus:O,element:function(){return d},activate:function(){u=setTimeout(x,3E3);b.trigger.spin();t.set(d,"display","block");
O()},deactivate:y,destroy:function(){y();k.each(W.values(z),function(a){Ne.unregister(a)});Ne.unregister(l);I.stop()},events:b.registry}}function aV(a,b){var c=b.translations,d=D.create({response:s(["rtf","hide"]),cancel:s([]),error:s(["message"]),failed:s(["message"])});return{open:function(){var e=YU(b.swf);e.deactivate();var g=q.fromDom(window),l=E.bind(g,"mouseup",e.focus),f=function(){w()},g=function(){w();d.trigger.cancel()};e.events.cancel.bind(g);e.events.response.bind(function(a){d.trigger.response(a.rtf(),
f)});e.events.error.bind(function(a){w();d.trigger.error(a.message())});e.events.failed.bind(function(a){w();d.trigger.failed(a.message())});var h=a();h.setTitle(c("cement.dialog.flash.title"));var k=TU(e,h.reflow,b);k.reset();h.setContent(k.element());h.events.close.bind(g);h.show();e.activate();var w=function(){l.unbind();e.destroy();h.destroy()}},events:d.registry}}function Rx(a,b,c,d){var e=L.eq(a,c)&&b===d;return{startContainer:f.constant(a),startOffset:f.constant(b),endContainer:f.constant(c),
endOffset:f.constant(d),collapsed:f.constant(e)}}function bV(a){var b=q.fromTag("br"),c=h.none();return{cleanup:function(){B.remove(b)},toOn:function(b,e){var g=y.owner(e).dom().defaultView;c.each(function(b){a.set(g,b)})},toOff:function(d,e){var g=y.owner(e).dom().defaultView;v.append(e,b);c=a.get(g,Rx);a.set(g,Rx(b,0,b,0))}}}function cV(a,b){var c=q.fromTag("div");m.setAll(c,b);m.setAll(c,{contenteditable:"true","aria-hidden":"true"});t.setAll(c,{position:"fixed",top:"0px",width:"100px",height:"100px",
overflow:"hidden",opacity:"0"});ea.add(c,[Sx,Tx]);var d=function(a){return n.has(a,Tx)};return{attach:function(a){B.empty(c);t.set(c,"left",dV(a));v.append(a,c)},focus:function(){J.ancestor(c,"body").each(function(b){a.toOff(b,c)})},contents:function(){eV.consolidate(c,d);var a=C.immutable("elements","html","container"),b=y.children(c),l=ua.get(c);return a(b,l,c)},container:function(){return c},detach:function(){B.remove(c)}}}function fV(a,b,c){var d=cV(a,c),e=function(){a.cleanup();var b=d.contents();
d.detach();l.trigger.after(b.elements(),b.html(),d.container())},g=Qc.tap(function(){l.trigger.before();d.attach(b);d.focus();Gx.run(y.owner(b),g,e)}),l=D.create({before:s([]),after:s(["elements","html","container"])});c=f.noop;return{instance:f.constant(function(){g.instance()}),destroy:c,events:l.registry}}function gV(a){return function(b){var c=D.create({after:s(["container"])}),d=bV(hV),e=fV(d,b,a);e.events.after.bind(function(a){d.toOn(b,a.container());c.trigger.after(a.container())});return{run:function(){e.instance()()},
events:c.registry}}}function iV(){return Dx([Df.text()],Df.none())}function jV(a,b){return function(c){var d=D.create({close:s([])}),e=ce(a);e.events.close.bind(function(){d.trigger.close()});n.add(e.element(),x.resolve("external-dialog"));c&&n.add(e.element(),x.resolve("dialog-vertical-controls"));var g=q.fromTag("div"),l=wa([]),p=function(a){var b=ac();if(void 0===a.text)throw"dialog button did not specify text";b.content(qc(a.text));Q.button(b.element(),a.text);void 0!==a.tabindex&&m.set(b.element(),
"tabindex",a.tabindex);void 0!==a.className&&n.add(b.element(),a.className);void 0!==a.click&&b.events.click.bind(a.click);n.add(b.element(),x.resolve("dialog-button"));return b};return{element:e.element,events:d.registry,setTitle:e.setTitle,setContent:e.setContent,setButtons:function(a){B.empty(g);a=k.map(a,p);K.append(g,k.map(a,function(a){return a.element()}));l.set(a);e.setControls(g)},reflow:f.noop,show:function(){b.hideInlineSelection();e.show();var a=k.map(l.get(),function(a){return ba.button(a.element(),
pa.button({execute:qb.click}))}),c=ba.multipleAll(e.element(),a);c.events.exit.bind(function(){d.trigger.close()});setTimeout(function(){c.enter()},0)},destroy:function(){k.each(l.get(),function(a){a.destroy()});l.set([]);e.destroy()}}}}function kV(a,b,c){var d={"errors.imageimport.unsupported":"errors.images.import.unsupported","errors.imageimport.invalid":"errors.images.import.invalid"};return{baseUrl:c.baseUrl,enableFlashImport:c.enableFlashImport,swf:c.swf,officeStyles:c.officeStyles,htmlStyles:c.htmlStyles,
translations:function(b){var c=Array.prototype.slice.call(arguments,1);return a.translate.apply(void 0,[void 0!==d[b]?d[b]:b].concat(c))},allowLocalImages:c.allowLocalImages,intraFlag:b}}function lV(a,b,c,d){var e=a.filter(),g=a.content(),l=a.undo(),f=a.smartview(),r=a.region(),z=a.internalevents(),w=!0===d.forceText?iV:mV(f,e,d,g),G=h.none(),I=D.create({analytics:s(["message"])}),n=function(a){var b=k.contains(["flash.clipboard.no.rtf","safari.imagepaste"],a.message())?"info":"error";t(a.message(),
b)},t=function(b,c){ae.getOrLog(b,[]).each(function(b){b=Ch(b,c);a.ui().displayBanner(b)});u()},u=function(){gc.unblock();g.focus()};e.add(e.history(),Qx.binStyle(),Ca.remove);e.addSelector(e.history(),"[data-ephox-polish-cement-inserted]",Ca.removeAttr("data-ephox-polish-cement-inserted"));g.events.ready.bind(function(a){var d=pp.get(g,f,a.api().doc()),s=nV.rules(g,r,d),v=FU(g),x=a.api(),B=w(x.body());G=h.some(B);x.events.paste.bind(function(a){x.getSelection().filter(function(a){return k.forall(fi.ignoreSelectors(),
function(b){return!1===eb.ancestor(a.startContainer(),b,g.isRoot)})}).each(function(b){B.isBlocked()||(v.backup(),gc.block());B.paste(a.original().raw())})});B.events.cancel.bind(function(){u();v.cancel()});var D=Th.processResults(t);B.events.error.bind(n);B.events.insert.bind(function(a){c.correlated(a.correlated()).map(D).get(function(c){u();l.save();var d=q.fromTag("div");K.append(d,a.elements());e.filterDom(e.input(),d);d=y.children(d);k.each(d,function(a){A.isElement(a)&&m.set(a,"data-ephox-polish-cement-inserted",
!0)});v.insert(d,[s.collapsed()],[s.ranged()]);g.body().each(function(a){a=X.descendants(a,"[data-ephox-polish-cement-inserted]");k.each(a,function(a){m.remove(a,"data-ephox-polish-cement-inserted")});z.input(a)});g.domChanged();I.trigger.analytics("paste_action");b.pasteUpload();b.uploadAssets(c)})})});return{destroy:function(){G.each(function(a){a.destroy()})},events:I.registry}}function oV(a,b,c,d){var e=zU(a,d.copy),g=lV(a,b,c,d.paste);return{destroy:function(){e.destroy();g.destroy()}}}function pV(){var a=
{latin:[{start:"A",finish:"Z"},{start:"a",finish:"z"},{start:"\u00c0",finish:"\u00ff"}],punctuation:[{start:"!",finish:"/"},{start:":",finish:"@"},{start:"[",finish:"`"},{start:"{",finish:"~"},{start:"\u00a1",finish:"\u00bf"},{start:"\u2010",finish:"\u2027"},{start:"\u2030",finish:"\u205e"}],currency:[Oe("$"),{start:"\u00a2",finish:"\u00a5"},{start:"\u20a0",finish:"\u20bd"},Oe("\u5186"),Oe("\u5143"),Oe("\u5713"),Oe("\u5706")],"extended-latin-a":[{start:"\u0100",finish:"\u017f"}],"extended-latin-b":[{start:"\u0180",
finish:"\u024f"}],arrows:[{start:"\u2190",finish:"\u21ff"},Oe("\u2794"),{start:"\u2798",finish:"\u27af"},{start:"\u27b1",finish:"\u27be"}],mathematical:[{start:"\u2795",finish:"\u2797"},{start:"\u2200",finish:"\u22ff"}],miscellaneous:[{start:"\u2701",finish:"\u275e"},{start:"\u2761",finish:"\u2793"},Oe("\u27b0"),Oe("\u27bf")]},b={};W.each(a,function(a,d,e){b[d]=k.bind(a,qV)});return b}function rV(a){var b=D.create({insert:s(["string"]),close:s([])}),c=ce(a);a=pV();a=sV.generateItems(a);var d=hc.input("specialchar",
"specialchar.selects"),e=ex(Qo.left(),a,x.resolve("specialchars"));n.add(e.body(),x.resolve("specialchar-body"));n.add(c.element(),x.resolve("specialchar-dialog"));k.each(y.children(e.element()),gc.register);var g=E.bind(c.element(),"click",function(a){a=a.target();J.closest(a,"[data-ephox-polish-special-char]").each(function(a){T.focus(a);var b=m.get(a,"data-ephox-polish-special-char");a=d.input();b=$.get(a)+b;$.set(a,b)})}),l=tV.controls("specialchar");c.setTitle(u.translate("specialchar.dialog"));
c.setContent(e.element());c.setControls(l.element());v.after(e.element(),d.element());a=function(){c.hide();$.set(d.input(),"");b.trigger.close()};var f=uV(e),h=vV(e),z=qb.setup(c,l,[f,h,ba.input(d.input(),pa.input({}))]);e.events.setActive.bind(function(a){f.clickFocus(a.item().element())});c.events.close.bind(function(){z.exit();b.trigger.close()});l.events.cancel.bind(a);c.events.close.bind(a);l.events.insert.bind(function(){b.trigger.insert($.get(d.input()))});return{element:c.element(),events:b.registry,
open:function(){c.show();z.enter()},destroy:function(){g.unbind();c.destroy();e.destroy();l.destroy();z.destroy()},close:function(){c.hide();$.set(d.input(),"")}}}function wV(a,b){var c=rV(b),d=D.create({insert:M(["string"],c.events.insert),close:s([])}),e=function(){d.trigger.close();c.close()};c.events.close.bind(e);return{show:function(){a.toReading();c.open()},close:e,destroy:function(){c.events.close.unbind(e);c.destroy()},events:d.registry}}function xV(a){var b=a.content(),c=a.smartview(),d=
function(a){a=q.fromText(a.string());Cc.insertAtCursor(b,[a]).each(function(a){a=da.after(a.element());b.setRelativeSelection(a,a);b.focus();b.toEditing()})},e=sc.destroyable(),g=function(){e.clear()};return{ui:$a.ui({"specialchar.general":{toggle:!1,value:"specialchar",textkey:u.key("specialchar.open"),action:function(){var l=wV(b,c.uiContainer());e.set(l);l.events.close.bind(function(){b.focus();g()});l.events.insert.bind(function(a){d(a);g()});a.content().hideInlineSelection();l.show()},updateGeneral:f.noop,
icon:!0,shortcut:!1,hasPopup:!1,hasDialog:!0}}),destroy:g}}function yV(a,b,c,d,e){var g=a.content(),l=a.profile().nativeSpelling(),p=function(a){m.set(a,"spellcheck","false")},r=function(a){m.set(a,"spellcheck","true")};g.runOrOnLoaded(function(){var a=l.get()?r:p;g.body().each(a)});a=$a.ui({"spelling.general":{toggle:!0,value:"spellcheck",textkey:u.key("cog.spellcheck"),action:function(){var a=!l.get();l.setAndCache(a);a=a?r:p;g.body().each(a)},updateGeneral:function(a){(l.get()?a.select:a.deselect)()},
icon:!0,tooltip:Wa.get}});return eo({events:Pc(["analytics"]),init:f.noop,hasDropdown:f.constant(!1),hasAutocorrect:f.constant(!1),check:f.noop,scanView:f.noop,lock:f.apply,situation:nf.noSituation,handle:f.noop,assign:f.noop,destroy:function(){g.body().each(function(a){m.remove(a,"spellcheck")})},state:f.constant({}),ui:a,lookup:f.constant(h.none()),name:f.constant("spelling-disabled")})}function zV(a){var b={},c=function(a){return b.hasOwnProperty(a)&&void 0!==b[a]?b[a]:{}},d=function(a,d){var e=
c(a);d(e);b[a]=e},e=function(a,b){var d=c(a);return b(d)},g=function(a,b,c){d(a,function(a){var d=xg.clean(b);a.hasOwnProperty(d)&&void 0!==a[d]||(a[d]=Ux(h.some(c),!1))})},l=function(a,b){d(a,function(a){var c=xg.clean(b),d=a.hasOwnProperty(c)?a[c]:void 0,d=void 0!==d?d.suggestions():h.none(),d=Ux(d,!0);a[c]=d})};W.each(void 0!==a?a:{},function(a,b){W.each(a,function(a,c){!0===a.ignore&&l(b,c);void 0!==a.suggestions&&g(b,c,a.suggestions)})});return{add:g,get:function(a,b){return e(a,function(a){var c=
xg.clean(b);return a.hasOwnProperty(c)?h.from(a[c]):h.none()})},ignore:l,suggestions:function(a,b){return e(a,function(a){var c=xg.clean(b);return(a.hasOwnProperty(c)?h.from(a[c]):h.none()).bind(function(a){return a.suggestions()})})},state:function(){return W.map(b,function(a,b){return W.map(a,function(a,b){return{ignore:a.ignore(),suggestions:a.suggestions().getOr([])}})})},destroy:function(){b={}}}}function AV(a,b,c){var d=D.create({change:s(["language"])});a=c?BV(a,d):CV(a,b,d);return{ui:$a.ui(a),
hasDropdown:f.constant(c),events:d.registry}}function DV(){var a=[],b=!1,c=function(){b=!0;h.from(a.shift()).fold(function(){b=!1},function(a){a.f()(a.area(),c)})};return{enqueue:function(d,e){a.push(EV(d,e));b||c()},destroy:function(){a=[]}}}function FV(a,b){var c=wa(a),d=DV(),e=function(a,d,e,f){e=Pk.cata(e,function(a){return ca.inBody(a)?Qk.single(a,c.get(),b):Qk.empty()},function(a,d,e,g){return ca.inBody(a)&&ca.inBody(e)?Qk.range(a,d,e,g,c.get(),b):Qk.empty()});d(a,e,c.get(),f)};return{run:function(a,
b,c){e(a,b,c,f.identity)},setLang:c.set,schedule:function(a,b,c,f){d.enqueue(c,function(c,d){void 0!==f&&f()?d():e(a,b,c,d)})},destroy:function(){d.destroy()}}}function GV(a,b){var c=function(b){return k.map(b,function(b){var c=Vx.getInCache(a,b.lang(),b.words());return{zone:f.constant(b),words:f.constant(c),lang:b.lang}})},d=function(a,b,c){k.each(b,function(b){HV.highlight(a,b.zone(),b.words(),c)})};return{local:function(a,b,l,f){b=b.zones();b=c(b);d(a,b,l);f()},remote:function(c,g,l,f){g=g.zones();
IV.scan(a,b,g).get(function(a){d(c,a,l);f()})},destroy:f.noop}}function JV(a,b,c){var d=Wx.initCaches(c),e=hk.post("ephox.spelling.service.many",KV.many(),a,b);return{checkMany:function(a,b,c){c=Lh.now();return Wx.execute(e,d,a,b,c)},dumpCache:function(){return W.map(d,function(a){return a.dump()})}}}function LV(a,b,c,d,e){var g=MV.spelling(b),l=JV(g+"correction",a.newHeaders(),e),p=D.create({failed:s(["resource"])}),h=wd.get(g+"autocorrect?language="+c,xd.json(),nd.xhr(),a.newHeaders()),k=$d.nu(function(a){h.get(function(b){b.fold(function(){a(b)},
function(b){l.checkMany(c,["startup"],!0).get(a)})})});k.get(function(a){a.fold(function(a){console.error("Unable to communicate with the spelling service:",Yo.consoleMessage(a));p.trigger.failed(b)},f.noop)});return{correction:function(a,b,e,g){k.get(function(f){f.fold(g,function(){l.checkMany(d?b:c,a.words).get(function(a){a.fold(function(a){console.error("Error response from the spelling service:",Yo.consoleMessage(a));g(a)},function(a){e(a)})})})})},autocorrect:f.constant(h),events:p.registry,
state:function(){return l.dumpCache()}}}function NV(a){var b=a.content(),c=a.profile(),d=LV(a.communication(),a.url(),a.language(),a.multilingual(),a.fromServiceState()),e=a.autocorrectOn()?Xx.active(d):Xx.inactive(),g=D.create({failed:s(["resource"])}),l=function(a){g.trigger.failed(a.resource())};d.events.failed.bind(l);var f=FV(a.language(),a.viewable().viewport()),h=GV(a.cache(),d),k=function(a){b.body().each(function(b){ok.filterDom(b,[Io("."+a,Ca.unwrap)])})},w=function(){return!ke.isOn(c.spelling().get())},
m=function(){k("ephox-cram-annotation-wrap")};return{run:function(a,b){var d=c.spelling().get();ke.isOn(d)&&(f.setLang(d),f.run(a,h.local,b),f.schedule(a,h.remote,b,w))},refresh:function(){var a=c.spelling().get();ke.isOn(a)||m()},clear:m,lookup:e.lookup,destroy:function(){d.events.failed.unbind(l);f.destroy();h.destroy()},state:d.state,events:g.registry}}function OV(){var a=D.create({incorrect:s(["substitution"])}),b=h.none(),c={};c[zd.spelling()]=function(c){b.each(function(b){b=qp.substitute(b,
m.get(c,zd.value()));a.trigger.incorrect(b)})};return{assign:function(a){b=a},run:function(a){var b=m.get(a,zd.type()),b=c[b];void 0!==b&&b(a)},events:a.registry}}function PV(a,b,c){var d=OV();d.events.incorrect.bind(function(c){b.save();a.focus();a.domChanged();c.substitution().each(function(b){Yx.selectEnd(a,b)})});var e=f.curry(function(a,b,c,d){return a.marker(d).bind(a.detail).bind(function(a){return c(a).map(function(a){return k.map(a,function(a){var c={};c[zd.type()]=b;c[zd.value()]=a;return QV(a,
a,c)})})})},qp,zd.spelling(),function(a){return c.suggestions(a.lang(),a.word())});return{assign:d.assign,handle:function(a){d.run(a)},situation:function(a){a=e(a).getOr([]);return RV.generate(a,SV)},destroy:f.noop}}function TV(a,b){var c=D.create({lookup:s(["element"])}),d=function(b){a.getSelection().each(b)},e=function(){d(function(a){c.trigger.lookup(a.startContainer())})};a.events.keyup.bind(function(a){ld.isSpace(a)&&e()});a.events.nodeChanged.bind(function(a){a.prev().each(c.trigger.lookup)});
return{events:c.registry,destroy:f.noop}}function UV(a,b,c,d,e,g,l,p,h){var k=PV(b,e,p),w=TV(b,c),m=function(a){b.getSelection().each(function(b){var c=Ja.toLeaf(b.endContainer(),b.endOffset()),e=Ja.toLeaf(b.startContainer(),b.startOffset()),g;(g=b.collapsed())||(g=!Zx.isIE())||(g=Bc.after(b.endContainer(),b.endOffset(),b.startContainer(),b.startOffset()));b=g&&!Zx.isFirefox()?cc(e.element(),e.offset(),c.element(),c.offset()):b;return(g?d.lightweightWith:f.apply)(a,b)})},I=Qc.tap(function(a){a=a.element();
a=Pk.single(a);n(a)});w.events.lookup.bind(function(a){I.instance(a)});var n=function(a){h.run(m,a)};return{destroy:function(){k.destroy();w.destroy()},scan:n,situation:k.situation,lock:function(a){var b=I.control.isBlocked();I.control.block();a();b||I.control.unblock()},assign:k.assign,handle:k.handle}}function VV(a,b,c,d,e){e=b.language.getOr(e);var g=a.communication(),l=a.profile(),p=a.filter(),r=a.content(),z=a.operations(),w=a.undo(),G=a.smartview(),I=a.shortcuts(),n=a.context(),q=a.viewablearea();
rp.configure(p);a.profile().spelling().set(e);var t=zV(b.words),u=NV(WV({communication:g,profile:l,content:r,undo:w,viewable:q,cache:t,url:c.url,autocorrectOn:c.autocorrect,language:d,fromServiceState:b.service,multilingual:c.multilingual})),v=h.none();b=D.create({analytics:s(["message"])});u.events.failed.bind(function(b){b=ae.get("error.code.spelling.not.found",[b.resource()]);b=Ch(b);a.ui().displayBanner(b)});r.events.ready.bind(function(a){r.body().each(function(a){m.set(a,"spellcheck",!1)});
a=a.api().doc();a=UV(a,r,n,z,w,I,G,t,u);u.refresh();v=h.some(a)});d=AV(a,d,c.menu);d.events.change.bind(function(a){(ke.isOn(a.language())?A:u.clear)();r.uiChanged();r.domChanged()});var x=function(a){k.each(a,function(a){a=Pk.single(a);B(a)})},A=c.scanView?function(){var a=q.sections();x(a)}:u.clear;c.scanView&&q.events.onChange.bind(A);var B=function(a){v.each(function(b){b.scan(a)})};return eo({events:b.registry,destroy:function(){r.body().each(function(a){m.remove(a,"spellcheck")});v.each(function(a){a.destroy()});
t.destroy();u.destroy()},state:function(){return{words:t.state(),service:u.state(),language:h.some(a.profile().spelling().get())}},init:f.noop,hasDropdown:d.hasDropdown,hasAutocorrect:f.constant(c.autocorrect),ui:d.ui,check:function(a,b,c,d){a=rp.isWrap(a)?Ja.toLeaf(a,b):S.point(a,b);c=rp.isWrap(c)?Ja.toLeaf(c,d):S.point(c,d);c=Pk.range(a.element(),a.offset(),c.element(),c.offset());B(c)},scanView:A,situation:function(a){var b={primary:f.constant([]),secondary:f.constant([]),submenus:f.constant({})};
return v.fold(f.constant(b),function(b){return b.situation(a)})},lock:function(a){return v.fold(function(){return a()},function(b){return b.lock(a)})},assign:function(a){v.each(function(b){b.assign(a.bind(y.firstChild).or(a))})},handle:function(a){v.each(function(b){b.handle(a)})},lookup:u.lookup,name:f.constant("spelling")})}function XV(a,b,c,d,e){return(!0===c.service?VV:yV)(a,b,c,d,e)}function $x(a){var b=id(a);n.add(b.element(),x.resolve("choice-button"));var c=P.span();m.set(b.element(),"title",
u.translate("table."+a));m.set(b.element(),"tabindex","-1");Q.button(b.element(),u.translate("table."+a),!1);var d=Oa.describe(b.element(),u.translate("aria.table.headerdescription"));v.append(b.element(),d);b.events.click.bind(function(){T.focus(b.element());ha.pressed(b)});ea.add(c,[x.resolve(a),"ephox-pastry-icon-content","ephox-polish-icon"]);b.content({element:f.constant(c)});return b}function YV(a,b){var c=!1,d=function(a,b,c){var d=c.full();d.row()===b.rows()&&d.column()===b.columns()||a.setSize(d.row(),
d.column());a=a.setSelection(c.selection().row(),c.selection().column());T.focus(a)};return{on:function(){c=!0},off:function(){c=!1},mousemove:function(e,g,l,f){if(c){var h;h=qa.get(e.element());var k=ia.get(e.element());h=xa.dimensions(h,k);k=Na.absolute(e.element());k=xa.coords(k.left(),k.top());l=xa.coords(l,f);l=a.pickerCell(k,h,g,l);l=ay.resize(l,b);d(e,g,l)}},manual:function(a,g,l,f){c&&(l=ay.grow(g,l,f,b),d(a,g,l))}}}function ZV(a,b,c){var d=D.create({select:s(["rows","cols","rowHeaders","columnHeaders"])}),
e=q.fromTag("table");Q.presentation(e);Q.hidden(e,!0);n.add(e,Ef.table());var g=q.fromTag("tbody");v.append(e,g);var l=0,p=0,h=function(a,b){l=b;p=a;z()},z=function(){B.empty(g);var a=c.ids();od.repeat(p,function(b){var c=q.fromTag("tr");n.add(c,Ef.row());var d=od.repeat(l,function(c){var d=q.fromTag("td");n.add(d,Fc.resolve("cell-"+c+"-"+b));n.add(d,Ef.cell());var e=q.fromTag("button");m.set(e,"type","button");m.set(e,"aria-labelledby",a[b][c]);n.add(e,Ef.button());v.append(d,e);return d});K.append(c,
d);v.append(g,c)})},w=function(a,b){var c=Pe.cells(e);k.each(c,function(a){n.remove(a,Fc.resolve("picker-selected"))});c=Pe.rows(e).slice(0,a);k.each(c,function(a,c){var d=Pe.cells(a).slice(0,b);k.each(d,function(a,b){var d=Pe.grid(e,"data-picker-header-row","data-picker-header-col"),d=c<d.rows()||b<d.columns()?[Fc.resolve("picker-selected"),Fc.resolve("picker-header")]:[Fc.resolve("picker-selected")];ea.add(a,d)})});m.set(e,"data-picker-col",b-1);m.set(e,"data-picker-row",a-1);c=Pe.cells(c[c.length-
1]).slice(0,b);return Pe.button(c[c.length-1])},G=function(){var a=parseInt(m.get(e,"data-picker-col"),10)+1,b=parseInt(m.get(e,"data-picker-row"),10)+1;return xa.grid(b,a)},I={element:f.constant(e),setSelection:w,setSize:h},O=YV(a,b),t=E.bind(e,"mousemove",function(a){O.mousemove(I,xa.grid(p,l),a.raw().pageX,a.raw().pageY)}),u=function(a,b){O.manual(I,G(),a,b)},x=sk.realClick.bind(e,function(a){y();a.raw().preventDefault()}),y=function(){var a=Pe.grid(e,"data-picker-row","data-picker-col"),b=Pe.grid(e,
"data-picker-header-row","data-picker-header-col");d.trigger.select(a.rows()+1,a.columns()+1,b.rows(),b.columns())};return{element:function(){return e},destroy:function(){x.unbind();t.unbind();B.remove(e)},setSize:h,setHeaders:function(a,b){m.set(e,"data-picker-header-row",a);m.set(e,"data-picker-header-col",b)},setSelection:w,on:O.on,off:O.off,reset:function(){h(b.minRows,b.minCols);var a=w(1,1);T.focus(a)},refresh:function(){var a=G();z();return w(a.rows(),a.columns())},events:d.registry,sendLeft:f.curry(u,
a.isRtl()?1:-1,0),sendRight:f.curry(u,a.isRtl()?-1:1,0),sendUp:f.curry(u,0,-1),sendDown:f.curry(u,0,1),sendExecute:y}}function $V(a,b){var c=Af.createHelp(b.maxRows,b.maxCols,{row:function(a){return u.translate("table.picker.rows",a)},col:function(a){return u.translate("table.picker.cols",a)}}),d=ZV(a,b,c);Q.label(d.element(),u.translate("table.picker.label"));var e=Oa.describe(d.element(),u.translate("table.picker.describepicker")),g=q.fromTag("div");m.set(g,"tabIndex","-1");Q.group(g);ea.add(g,
[x.resolve("insert-table"),x.resolve("toolbar-menu")]);var l=q.fromTag("div");n.add(l,x.resolve("table-controls"));Q.toolbarGroup(l,u.translate("table.picker.header"));var p=$x("row-header"),r=$x("column-header");K.append(l,[p.element(),r.element()]);K.append(g,[l,d.element(),e,c.help()]);var k=function(){d.setHeaders(p.selected()?1:0,r.selected()?1:0);var a=T.active(),b=d.refresh();a.each(function(a){T.hasFocus(a)||T.focus(b)})},c=function(a,b){return function(){b();return h.some(!0)}},c=ba.single(d.element(),
[U.basic(N.LEFT()[0],c("left",d.sendLeft)),U.basic(N.RIGHT()[0],c("right",d.sendRight)),U.basic(N.UP()[0],c("up",d.sendUp)),U.basic(N.DOWN()[0],c("down",d.sendDown)),U.rule(U.inSet(N.SPACE().concat(N.ENTER())),c("execute",d.sendExecute))],h.none());c.events.enter.bind(function(){T.focus(d.element())});var w=ba.flow(l,{item:f.constant(".ephox-pastry-statebutton")},pa.flow({trigger:function(a){return J.descendant(a,".ephox-pastry-button").map(function(a){a.dom().click();return a})}})),G=ba.multiple(g,
[c,w]);p.events.click.bind(function(){w.clickFocus(p.element());k()});r.events.click.bind(function(){w.clickFocus(r.element());k()});l=D.create({select:M(["rows","cols","rowHeaders","columnHeaders"],d.events.select),exit:M([],G.events.exit)});return{element:function(){return g},destroy:function(){G.destroy();p.destroy();r.destroy();d.destroy()},setSize:d.setSize,setSelection:d.setSelection,on:function(){d.on();G.enter()},off:function(){d.off();G.isFocused()&&G.exit()},reset:function(){d.setHeaders(0,
0);p.deselect();ha.release(p.element());r.deselect();ha.release(r.element());d.reset()},events:l.registry}}function aW(a,b){var c=D.create({enabled:s(["enabled"]),select:s(["rows","cols","rowHeaders","columnHeaders"]),insert:s(["table"]),done:s([]),abort:s([])}),d=Z.merge({minCols:7,maxCols:10,minRows:4,maxRows:10},b),e=$V(a,d);e.events.select.bind(function(a){Y.detect().browser.isIE()?(c.trigger.select(a.rows(),a.cols(),a.rowHeaders(),a.columnHeaders()),c.trigger.done(),g()):(g(),c.trigger.done(),
c.trigger.select(a.rows(),a.cols(),a.rowHeaders(),a.columnHeaders()))});e.events.exit.bind(function(){g();c.trigger.abort()});var g=function(){e.off()};return{element:e.element,open:function(){e.reset();e.on()},close:g,destroy:function(){e.destroy()},events:c.registry}}function bW(){var a=q.fromTag("fieldset"),b=q.fromHtml("<legend>"+u.translate("table.border")+"</legend>");v.append(a,b);var c=La.generate(""),b=function(a){var b=q.fromTag("input");m.setAll(b,{type:"radio",id:"tableborder"+a+c,value:a,
name:c});return b},d=b("1"),e=b("0");K.append(a,[d,q.fromHtml('<label for="tableborder1'+c+'">'+u.translate("border.on")+"</label>"),e,q.fromHtml('<label for="tableborder0'+c+'">'+u.translate("border.off")+"</label>")]);var g=qb.radio(a,'input[type="radio"]',"input:checked");return{element:function(){return a},get:function(){return gi.find(a).map(function(a){return $.get(a)})},select:function(a){gi.set(e,!1);gi.set(d,!1);a.each(function(a){"0"===a?gi.set(e,!0):"1"===a&&gi.set(d,!0)});g.refresh()},
keylayout:f.constant(g),destroy:function(){B.remove(a)}}}function cW(){var a=P.div();n.add(a,x.resolve("table-size-controls"));var b=hc.input("table-width","table.width"),c=hc.input("table-height","table.height");k.map([b,c],function(a){m.set(a.input(),"placeholder","Auto")});K.append(a,[b.element(),c.element()]);var d=[ba.input(b.input(),pa.input({})),ba.input(c.input(),pa.input({}))],e=function(a){var b=a.target();Xb.validateField(b,"width",h.some("px")).fold(function(){Xb.highlight(b)},function(a){Xb.dehighlight(b)})},
g=[E.bind(b.input(),"input",e),E.bind(c.input(),"input",e)];return{element:function(){return a},get:function(){return Xb.validate([Xb.rule(b.input(),"width",h.some("px")),Xb.rule(c.input(),"height",h.some("px"))])},set:function(a){a.width().bind(f.curry($.set,b.input()));a.height().bind(f.curry($.set,c.input()))},keylayouts:f.constant(d),highlight:Xb.highlight,dehighlight:f.curry(Xb.clearHighlights,a),destroy:function(){B.remove(a);k.each(g,function(a){a.unbind()})}}}function dW(a,b){var c=D.create({ok:s(["properties"]),
close:s([])}),d=ce(b);d.setTitle(u.translate("table.dialog"));n.add(d.element(),x.resolve("dialog-narrow"));var e=P.div(),g=gx("table.float"),l=hc.input("tablesummary","table.summary"),p=hc.input("tablecaption","table.caption"),r=cW(),k=bW();K.append(e,[p.element(),l.element(),k.element(),r.element(),g.element()]);var w=Wc.okCancel("table");w.events.ok.bind(function(){t()});w.events.cancel.bind(function(){d.hide();c.trigger.close()});d.events.close.bind(function(){d.hide();c.trigger.close()});d.setContent(e);
d.setControls(w.element());var m=qb.setup(d,w,[ba.input(p.input(),pa.input({})),ba.input(l.input(),pa.input({})),k.keylayout()].concat(r.keylayouts()).concat([g.keylayout()])),I=function(){return r.get().map(function(a){a={width:f.constant(h.some(a.width())),height:f.constant(h.some(a.height()))};return sp.properties({floatval:g.get(),caption:q(p.input()),summary:q(l.input()),border:k.get(),dimensions:a})})},q=function(a){return $.get(a).trim()},t=function(){I().fold(function(a){r.highlight(a)},function(a){c.trigger.ok(a)})};
return{element:d.element,events:c.registry,open:function(){d.show();m.enter()},represent:function(b){b=sp.divine(a,b);g.select(b.floatval());void 0!==b.summary()&&$.set(l.input(),b.summary());void 0!==b.caption()&&$.set(p.input(),b.caption());r.set(b.dimensions());k.select(b.border())},execute:t,destroy:function(){g.destroy();r.destroy();k.destroy();w.destroy();d.destroy();m.destroy()},hide:function(){m.exit();d.hide()}}}function eW(a,b,c,d,e,g,l,p){var r=h.none(),z={"tablemenu-delete-table":Nb.erase,
"delete-row":Nb.execute(b,Fb.eraseRows,f.noop,p),"delete-column":Nb.execute(b,Fb.eraseColumns,f.noop,p),"make-header-column":Nb.execute(b,Fb.makeColumnHeader,f.noop,p),"make-header-row":Nb.execute(b,Fb.makeRowHeader,f.noop,p),"unmake-header-column":Nb.execute(b,Fb.unmakeColumnHeader,f.noop,p),"unmake-header-row":Nb.execute(b,Fb.unmakeRowHeader,f.noop,p),"insert-row-before":Nb.execute(b,Fb.insertRowBefore,f.noop,p),"insert-row-after":Nb.execute(b,Fb.insertRowAfter,f.noop,p),"insert-column-before":Nb.execute(b,
Fb.insertColumnBefore,tp.halve,p),"insert-column-after":Nb.execute(b,Fb.insertColumnAfter,tp.halve,p),"tablemenu-table-properties":function(b,d,f){b=dW(a,l);var p=ix.generate(e,b,["properties"]);p.events.ok.bind(function(a){c.activate();r.each(function(b){Ya.table(b.element(),c.isRoot).each(function(b){Yc.runTransaction(c,g,function(){sp.apply(b,a.properties())})})});p.destroy()});p.events.close.bind(function(a){p.destroy();c.activate();c.focus()});c.deactivate();p.represent(d);p.open();return h.none()},
"cell-merge":Nb.execute(b,Fb.mergeCells,f.noop,p),"cell-unmerge":Nb.execute(b,Fb.unmergeCells,f.noop,p),"cell-split-cols":Nb.execute(b,Fb.splitCellIntoColumns,tp.halve,p),"cell-split-rows":Nb.execute(b,Fb.splitCellIntoRows,f.noop,p)},w=C.immutable("value","text","attrs"),G=Gw.byClass(sd);return{handle:function(a){c.doc().each(function(b){r.each(function(b){var d=b.element();Ya.table(d,c.isRoot).each(function(d){var e=m.get(a,"data-ephox-gopher-item-value"),l=z[e];void 0!==l&&Yc.runTransaction(c,g,
function(){l(c,d,b).each(function(a){c.setSelection(a.startContainer(),a.startOffset(),a.endContainer(),a.endOffset());G.clear(d)})})})})})},assign:function(a){r=a.bind(function(a){return Ya.cell(a,c.isRoot).bind(function(a){return J.ancestor(a,"table").map(function(b){return Rk.forMenu(d,b,a)})}).orThunk(function(){return Ya.notCell(a,c.isRoot).map(function(a){return Rk.notCell(a)})})})},situation:function(a){a=r.map(function(a){var b=a.element(),b=Ya.cell(b,c.isRoot).isSome();return k.flatten([b?
[w("tablemenu-row",u.translate("table.context.row.menu"),{})]:[],b?[w("tablemenu-column",u.translate("table.context.column.menu"),{})]:[],[w("tablemenu-table",u.translate("table.context.table.menu"),{})],a.mergable().isSome()?[w("cell-merge",u.translate("table.context.cell.merge"),{})]:[],a.unmergable().isSome()?[w("cell-unmerge",u.translate("table.context.cell.unmerge"),{})]:[],b&&1===a.selection().length&&a.mergable().isNone()&&a.unmergable().isNone()?[w("cell-split-cols",u.translate("table.context.cell.split-cols"),
{}),w("cell-split-rows",u.translate("table.context.cell.split-rows"),{})]:[]])}).getOr([]);var b=r.map(function(a){var b=a.element();a=function(a,c,d){var e=a+"-menu",g="th"===A.name(b)?"unmake":"make",l="make"===g?"header":"normal";c=[w("insert-"+a+"-before",u.translate("table.context."+a+"."+c),{}),w("insert-"+a+"-after",u.translate("table.context."+a+"."+d),{}),w("delete-"+a,u.translate("table.context.common.delete"),{}),w(g+"-header-"+a,u.translate("table.context.common."+l),{})];return{textkey:f.constant(u.key("table.context."+
a+".title")),value:f.constant(e),items:f.constant(c)}};var c={textkey:f.constant(u.key("table.context.table.title")),value:f.constant("table-menu"),items:f.constant([w("tablemenu-table-properties",u.translate("table.context.table.properties"),{}),w("tablemenu-delete-table",u.translate("table.context.common.delete"),{})])};return k.flatten([[a("row","insertabove","insertbelow")],[a("column","insertleft","insertright")],[c]])}).getOr([]);return{primary:f.constant(a),secondary:f.constant(b),submenus:f.constant({"tablemenu-row":"row-menu",
"tablemenu-column":"column-menu","tablemenu-table":"table-menu"})}},destroy:f.noop}}function fW(){var a=D.create({drag:s(["xDelta","yDelta"])});return{mutate:function(b,c){a.trigger.drag(b,c)},events:a.registry}}function gW(){var a=D.create({drag:s(["xDelta","yDelta","target"])}),b=h.none(),c=fW();c.events.drag.bind(function(c){b.each(function(b){a.trigger.drag(c.xDelta(),c.yDelta(),b)})});return{assign:function(a){b=h.some(a)},get:function(){return b},mutate:c.mutate,events:a.registry}}function hW(a,
b,c){var d=gW(),e=Ih.transform(d,{}),g=h.none(),l=function(a,b){return h.from(m.get(a,b))};d.events.drag.bind(function(a){l(a.target(),"data-row").each(function(b){b=Zc.getInt(a.target(),"top");t.set(a.target(),"top",b+a.yDelta()+"px")});l(a.target(),"data-column").each(function(b){b=Zc.getInt(a.target(),"left");t.set(a.target(),"left",b+a.xDelta()+"px")})});var p=function(a,b){var c=Zc.getInt(a,b),d=parseInt(m.get(a,"data-initial-"+b),10);return c-d};e.events.stop.bind(function(){d.get().each(function(d){g.each(function(e){l(d,
"data-row").each(function(a){var b=p(d,"top");m.remove(d,"data-initial-top");I.trigger.adjustHeight(e,b,parseInt(a,10))});l(d,"data-column").each(function(a){var b=p(d,"left");m.remove(d,"data-initial-left");I.trigger.adjustWidth(e,b,parseInt(a,10))});Ad.refresh(a,e,c,b)})})});var r=function(b,c){I.trigger.startAdjust();d.assign(b);m.set(b,"data-initial-"+c,parseInt(t.get(b,c),10));n.add(b,iW);t.set(b,"opacity","0.2");e.go(a.parent())},k=E.bind(a.parent(),"mousedown",function(a){Ad.isRowBar(a.target())&&
r(a.target(),"top");Ad.isColBar(a.target())&&r(a.target(),"left")}),w=function(b){return L.eq(b,a.view())},G=E.bind(a.view(),"mouseover",function(d){"table"===A.name(d.target())||eb.closest(d.target(),"table",w)?(g="table"===A.name(d.target())?h.some(d.target()):J.ancestor(d.target(),"table",w),g.each(function(d){Ad.refresh(a,d,c,b)})):ca.inBody(d.target())&&Ad.destroy(a)}),I=D.create({adjustHeight:s(["table","delta","row"]),adjustWidth:s(["table","delta","column"]),startAdjust:s([])});return{destroy:function(){k.unbind();
G.unbind();e.destroy();Ad.destroy(a)},refresh:function(d){Ad.refresh(a,d,c,b)},on:e.on,off:e.off,hideBars:f.curry(Ad.hide,a),showBars:f.curry(Ad.show,a),events:I.registry}}function jW(a,b){var c=hi.height,d=hW(a,b,c),e=D.create({beforeResize:s(["table"]),afterResize:s(["table"]),startDrag:s([])});d.events.adjustHeight.bind(function(a){e.trigger.beforeResize(a.table());var b=c.delta(a.delta(),a.table());up.adjustHeight(a.table(),b,a.row(),c);e.trigger.afterResize(a.table())});d.events.startAdjust.bind(function(a){e.trigger.startDrag()});
d.events.adjustWidth.bind(function(a){e.trigger.beforeResize(a.table());var c=b.delta(a.delta(),a.table());up.adjustWidth(a.table(),c,a.column(),b);e.trigger.afterResize(a.table())});return{on:d.on,off:d.off,hideBars:d.hideBars,showBars:d.showBars,destroy:d.destroy,events:e.registry}}function kW(a,b){var c=a.content(),d=a.keyboard(),e=a.undo(),g=a.smartview(),l=function(){return v.getOr(vp.only(c.element()))},p=eW(a.filter(),a.region(),c,a.selections(),d,e,g.uiContainer(),l),r=function(a,b,d){e.save();
k.each(b,function(b){a(b,d)});e.save();c.focus()},z=f.curry(r,by.setBackgroundColor),r=f.curry(r,by.setBorderColor),z=$a.ui({"table.widget":f.curry(lW.widget,a,b),"cell.palette":f.constant({items:[{type:"widget",build:f.curry(cy.widget,a,{action:z,shortname:"background",name:"cellbackgroundcolor",textkey:"table.cell.color.background",menuTextkey:"table.cell.color.background"})},{type:"widget",build:f.curry(cy.widget,a,{action:r,shortname:"border",name:"cellbordercolor",textkey:"table.cell.color.border",
menuTextkey:"table.cell.color.border"})},{type:"widget",build:f.curry(mW.widget,a)},{type:"dropdown",build:f.curry(nW.widget,a)}],updatePalette:function(b){var d=le.selectedCells(a.selections(),c.isRoot),e=f.curry(b.show,h.some(u.translate("table.context.palette.show")));b=f.curry(b.hide,h.some(u.translate("table.context.palette.hide")));(0<d.length?e:b)()&&a.ui().refresh()}})},f.identity),w=function(b){var d=b.stopper();Xj.context(c).bind(function(e){Xj.cata(e,function(e){Ya.cell(e.startContainer(),
c.isRoot).each(function(e){(b.shift()?dy.backward:dy.forward)(a.region(),c,e,l).fold(d,function(a){d();c.setSelection(a.startContainer(),a.startOffset(),a.endContainer(),a.endOffset())})})},f.noop)})},m=function(a){ey.upOrLeft(c)},n=function(a){ey.right(c)},q=d.editor();q.on();q.events.tab.bind(w);q.events.left.bind(m);q.events.up.bind(m);q.events.right.bind(n);var s=c.cursor(),t=h.none(),v=h.none();c.events.ready.bind(function(b){b=pp.get(c,g,b.api().doc());v=h.some(b);var d=xv(a.region().directionAt),
l=jW(b,d);l.events.beforeResize.bind(e.save);l.events.afterResize.bind(function(){c.focus();e.save();c.domChanged();s.restore()});l.on();l.events.startDrag.bind(s.save);t=h.some(l);b=a.selections();b.events.beforeScan.bind(function(){l.hideBars()});b.events.afterScan.bind(function(){l.showBars()})});return{ui:z,destroy:function(){p.destroy();t.each(function(a){a.destroy()});q.off();q.events.tab.unbind(w);q.events.left.unbind(m);q.events.up.unbind(m);q.events.right.unbind(n);q.destroy()},handle:p.handle,
assign:p.assign,situation:p.situation,name:f.constant("table")}}function oW(){var a=D.create({pasteUpload:s([]),uploadAssets:s(["assets"])});return{destroy:f.noop,events:a.registry,pasteUpload:a.trigger.pasteUpload,uploadAssets:a.trigger.uploadAssets}}function pW(a,b){var c=a().uiContainer(),d=D.create({close:s([])}),e=ce(c);e.setTitle(u.translate("wordcount.dialog"));ea.add(e.element(),[x.resolve("dialog-wordcount"),x.resolve("dialog-narrow")]);var c=b().body().getOrDie(),g=yg.toText(c,f.constant(!1)),
l=pb.get(c),c=b().getSelectedText().getOr(""),g=fy.count(g),l=l.length,c=fy.count(c),p=q.fromHtml(aa.supplant('<table class="ephox-polish-tabular ephox-polish-wordcount-table" tabindex="-1">    <thead>        <tr>            <th scope="col"> ${headerCounts}&nbsp;</th>            <th scope="col"> ${headerDocument}&nbsp;</th>            <th scope="col"> ${headerSelection}&nbsp;</th>        </tr>    </thead>    <tbody>        <tr>            <td>${wordCountLabel}&nbsp;</td>            <td class="ephox-polish-wordcount-document-words">${documentCountWords}&nbsp;</td>            <td class="ephox-polish-wordcount-selection-words">${selectionCountWords}&nbsp;</td>        </tr>        <tr>            <td>${wordCountCharsNoSpacesLabel}&nbsp;</td>            <td class="ephox-polish-wordcount-document-charsnospaces">${documentCharactersNoSpaces}&nbsp;</td>            <td class="ephox-polish-wordcount-selection-charsnospaces">${selectionCharactersNoSpaces}&nbsp;</td>        </tr>        <tr>            <td>${wordcountCharsLabel}&nbsp;</td>            <td class="ephox-polish-wordcount-document-chars">${documentCountCharsWithSpaces}&nbsp;</td>            <td class="ephox-polish-wordcount-selection-chars">${selectionCountCharsWithSpaces}&nbsp;</td>        </tr>    </tbody></table>',
{headerCounts:u.translate("wordcount.counts"),headerDocument:u.translate("wordcount.document"),headerSelection:u.translate("wordcount.selection"),wordCountLabel:u.translate("wordcount.words"),wordCountCharsNoSpacesLabel:u.translate("wordcount.charactersnospaces"),wordcountCharsLabel:u.translate("wordcount.characters"),documentCountWords:g.words(),selectionCountWords:c.words(),documentCharactersNoSpaces:g.characters().withoutSpaces(),selectionCharactersNoSpaces:c.characters().withoutSpaces(),documentCountCharsWithSpaces:l,
selectionCountCharsWithSpaces:c.characters().withSpaces()}));e.setContent(p);var h=ba.multiple(e.element(),[]),c=function(){k();d.trigger.close()},k=function(){e.hide()};e.events.close.bind(c);h.events.exit.bind(c);return{element:e.element,events:d.registry,open:function(){e.show();h.enter();T.focus(p)},destroy:function(){h.destroy();e.destroy()},hide:k}}function qW(a,b){var c=pW(a,b),d=D.create({close:s([])});c.events.close.bind(function(){d.trigger.close();c.destroy()});return{show:function(){c.open()},
events:d.registry}}function rW(a){var b=a.content,c=a.smartview;return{ui:$a.ui({"wordcount.general":{toggle:!1,value:"wordcount",textkey:u.key("wordcount.open"),action:function(){var d=qW(c,b);d.events.close.bind(function(){a.ui().tabstrip.focused()||b().focus()});a.content().hideInlineSelection();d.show()},updateGeneral:f.noop,icon:!0,shortcut:!1,hasPopup:!1,hasDialog:!0}}),destroy:f.noop}}function Sk(a,b,c){var d=function(a,b,c,d){return b.bundled?a.apply(void 0,d.concat([b])):c()},e=OR(a,b,c);
b=d(HP,a.formatting,Aa.basic,[e]);var g=d(FP,a.alignindent,Aa.basic,[e]),l=d(VP,a.bookmark,Aa.basic,[e]),p=d(XP,a.directionality,Aa.basic,[e]),r=d(kW,a.table,Aa.table,[e]),z=d(SP,a.blocktag,Aa.basic,[e]),w=d(BS,a.font,Aa.basic,[e]),G=d(rW,a.wordcount,Aa.basic,[e]),n=d($T,a.media,Aa.basic,[e]),q=d(xV,a.specialchar,Aa.basic,[e]),t=d(LT,a.linkvalidator,Aa.linkvalidator,[e,c.linkvalidator]),u=d(HT,a.link,Aa.basic,[e,t]),v=d(SS,a.horizontalrule,Aa.basic,[e]),x=d(OS,a.help,Aa.dialog,[e]),A=function(b,c,
d,g){var l=a.ui.locale.code,f=void 0!==l?l:fe.get();return(!0===c.bundled?ke.determine(f):h.none()).fold(function(){return d()},function(a){var d=e.profile().spelling().get(),d=void 0!==l||!0!==c.menu&&d!==ke.offKey()||!1===d?a:d;return b.apply(void 0,g.concat([c,a,d]))})}(XV,a.spelling,Aa.spelling,[e,c.spelling]),B=d(qU,a.parlance,Aa.basic,[e,A]),C=d(yU,a.resize,Aa.basic,[e]),E=d(UT,a.macros,Aa.macros,[e,A,c.macros]),F=d(PS,a.highlighter,Aa.highlighter,[e,E]),J=d(YP,a.draganddrop,Aa.draganddrop,
[e]),H=oW(),K=d(pT,a.imageEdit,Aa.imageEdit,[e,H]),L=d(VS,a.image,Aa.image,[e]),N=d(eS,a.fileupload,Aa.dialog,[e,H,K.insertResample(),J]),P=d(oV,a.smartclipboard,Aa.basic,[e,H,K.insertResample()]),Q=d(lU,a.mentions,Aa.basic,[e]),R=d(lS,a.findreplace,Aa.basic,[e]),S=d(AP,a.accessibility,Aa.basic,[e]),T=d(cQ,a.embed,Aa.basic,[e,c.embed]);c=function(){var a=e.content();return a.body().fold(a.getContent,function(a){a=ec.deep(a);return Mh.output(e.filter(),a)})};var U=function(a){var b=e.filter(),c=e.content();
a=b.filterHtml(b.input(),a);c.resetContent(a);F.run();t.refresh();A.scanView();return a},d=d(uU,{getContent:c,setContent:U,bundled:a.plaincodeview.bundled},Aa.basic,[e]);e.contextmenu().register("table",r);e.contextmenu().register("imageEdit",K);e.contextmenu().register("image",L);e.contextmenu().register("spelling",A);e.contextmenu().register("link",u);var V=sW({formatting:b,alignindent:g,bookmark:l,directionality:p,table:r,font:w,imageEdit:K,parlance:B,blocktag:z,wordcount:G,specialchar:q,media:n,
link:u,linkvalidator:t,horizontalrule:v,help:x,fileupload:N,findreplace:R,accessibility:S,plaincodeview:d,macros:E,spelling:A});e.content().events.ready.bind(function(){e.ui().init(e.keyboard(),e.undo(),V);A.init(e.content());F.run();t.refresh()});e.content().events.loaded.bind(function(){setTimeout(function(){X.trigger.loaded();Y()},0)});var X=D.create({loaded:s([]),ready:M([],e.content().events.ready),focus:s([]),change:s([]),dirty:M([],e.laundry().events.dirty),internal:s(["command","data"]),paint:s(["dimensions"])}),
Y=e.content().isInline()?f.noop:function(){e.content().body().each(function(a){a=y.documentElement(a);var b=e.content().element();X.trigger.paint(Vu({width:qa.get(b),docWidth:qa.get(a),height:ia.get(b),docHeight:ia.get(a)}))})};e.undo().events.changed.bind(function(){X.trigger.change()});e.undo().events.reset.bind(function(){X.trigger.change()});e.content().events.paint.bind(Y);e.content().events.nodeChanged.bind(Y);e.content().events.focus.bind(function(){X.trigger.focus()});e.internalevents().events.internal.bind(function(a){var b=
a.command();"insertLink"===b&&(a=m.get(a.data(),"href"),X.trigger.internal(b,a))});d=e.content().isInline()?function(){var a=e.ui(),b=e.content(),c=a.state(),c=void 0!==c.toolbar&&void 0!==c.toolbar.drag?c.toolbar.drag.isSome():!1;b.activate();b.focus();e.viewablearea().setOn();c||a.position();a.refresh()}:f.noop;b=e.content().isInline()?function(){e.content().deactivate();e.content().blur();e.viewablearea().setOff()}:f.noop;return zh.internal({activateInline:d,deactivateInline:b,element:e.ui().element,
viewport:e.ui().viewport,connect:e.ui().connect,focus:e.content().focus,blur:e.content().blur,getContent:c,toTransfer:f.compose(xe.changed,c),setContent:function(a){var b=e.undo();b.disable(function(){var c=U(a);b.reset(c)});e.laundry().setDirty(!1)},insertHtmlAtCursor:e.insertFilteredHtmlAtCursor,isDirty:e.laundry().isDirty,setDirty:e.laundry().setDirty,fromTransfer:function(a){var b=e.undo();xe.cata(a,function(a){b.disable(function(){U(a)});b.replaceTop()},function(a){U(a);b.save()})},uploader:f.constant(H),
state:function(){var a=e.state();return Z.merge(a,{spelling:A.state(),macros:E.state(),filters:e.filter().customFilters().state(),linkvalidator:t.state(),embed:T.state()})},destroy:function(){e.content().deactivate();var a=k.map(W.values(V),f.apply);k.each(a.concat([T,L,K,A,C,E,F,H,P,Q,e,J]),function(a){a.destroy()})},events:X.registry,communication:e.communication,fullscreen:e.fullscreen,content:e.content,ui:e.ui,undo:e.undo,smartview:e.smartview,keyboard:e.keyboard,filter:e.filter,operations:e.operations,
macros:f.constant(E),getMode:Ab.pencil,isPrettyPrinting:function(){return!0===a.content.prettyprint}})}function tW(a,b){var c=Aa.state(b),d=wa(null),e=q.fromTag("div"),g=function(){ua.set(e,'<div class="e-p-s-img"></div>');l()},l=void 0===a.content.inline?f.noop:function(){if(!1===a.ui.toolbar.visible)t.set(e,"display","none");else{var b={isInline:f.constant(!0),element:f.constant(a.content.inline)},b=bc.offsetFrom(b,e,ia.get(e),a.ui,!1);t.setAll(e,{position:"absolute",left:b.left+"px",top:b.top-
4+"px"})}},p=null,r=f.die("API unavailable while the editor is loading"),k={body:f.constant(h.none()),doc:r,execCommand:r,getSelectedText:r,focus:r,deactivate:f.noop,blur:f.noop},w=void 0===a.content.inline?d.get:function(){return ua.get(a.content.inline)};return zh.internal({activateInline:l,blur:r,communication:f.constant(Yv(a)),connect:function(a){p=setTimeout(g,1E3);v.append(a,e);wp.add();return!1},content:f.constant(k),deactivateInline:f.noop,destroy:function(){clearTimeout(p);wp.remove();B.remove(e)},
element:f.constant(e),events:Rc.events(),failure:function(){clearTimeout(p);wp.remove();ua.set(e,"<p>Unable to load editor. See browser console for details.</p>")},filter:c.filter,focus:r,fromTransfer:function(a){xe.cata(a,d.set,d.set)},fullscreen:Aa.fullscreen,getContent:w,getMode:r,insertHtmlAtCursor:r,isDirty:c.laundry.isDirty,isPrettyPrinting:function(){return!0===a.content.prettyprint},keyboard:Rc.keyboard,macros:c.macros,operations:Rc.operations,setContent:function(b){void 0===a.content.inline?
d.set(b):ua.set(a.content.inline,b);c.laundry.setDirty(!1)},setDirty:c.laundry.setDirty,smartview:Rc.smartview,state:function(){var a=Qe.isEmpty(w())?{}:{content:{}};return c.getState(a)},toTransfer:f.compose(xe.unchanged,w),ui:Rc.ui,undo:f.constant(Aa.undo()),uploader:f.constant(Aa.uploader()),viewport:f.constant(e)})}function uW(a,b){return(!0===b.bundled?vW:wW)(a,b)}function xW(a,b){var c=D.create({change:s(["type"])}),d=me.removeClasses([x.resolve(Ab.none()),x.resolve(Ab.wait()),x.resolve(Ab.pencil()),
x.resolve(Ab.code())]),e=P.div();Q.hidden(e,"true");ea.add(e,[x.resolve("html-switch-wrapper"),x.resolve("layer-editor")]);var g=P.div();ea.add(g,[x.resolve("html-switch"),x.resolve("icon")]);v.append(e,g);var l=function(a){d(g);n.add(g,x.resolve(a));m.set(g,"data-icon",a);m.remove(g,"title")},f=function(a,b){var c=m.get(g,"data-icon");a!==c&&(O=c,l(a),m.set(g,"title",b))},h=function(a){b.showButton&&a!==Ab.none()?f(a,u.translate("toolbar.switch-"+a)):l(Ab.none());w.control.unblock()},k=sc.unbindable(),
w=Qc.tap(function(){c.trigger.change(m.get(g,"data-icon"))});b.showButton&&k.set(E.bind(e,"click",w.instance));var G=function(a){v.append(a,e)},q=rd.onDirection("right","left"),O=Ab.none();h(O);return{render:G,destroy:function(){B.remove(e);k.clear()},switchto:h,spin:function(){f(Ab.wait(),u.translate("loading.wait"));w.control.block()},stopSpin:function(){"spin"!==O&&h(O)},init:function(b){var c=a.get();G(c.viewport());c.events.paint.bind(function(a){var b=a.dimensions();a=q(c.element());b="left"===
a&&(gy.isFirefox()||gy.isChrome())?0:b.width()-b.docWidth();t.set(e,a,b+"px")});h(Ab.none())},events:c.registry}}function yW(a,b,c,d,e,g){var l=q.fromTag("div"),p=c.uiContainer(),h=P.div();d.updateUi(l);d=[sf.editorContainer(),"ephox-textbox-font",x.resolve("layer-editor"),x.resolve("editor-main"),x.resolve("fullscreen-maximized")];ea.add(l,d);m.set(l,"lang",fe.get());Go.addClasses(l);v.append(l,p);var z=nv(a,l,{},!1),w=Sd.toolbar(z,u.translate);b.events.toolbar.bind(w.enter);w.events.exit.bind(a.focus);
var G=mv(),I=ca.body(),s=P.div();n.add(s,x.resolve("socket"));var t=k.map(["dragover","drop"],function(a){return E.bind(s,a,function(a){a.kill()})});b=P.div();n.add(b,x.resolve("widget-container"));z.integrate(l,w,G,s,b);vv.build(l,e.sideBar,e.bottomBar);var y=Ow({layerClass:x.resolve("layer-below-editor"),background:"rgba(255, 255, 255, 0.92)"}),A=x.resolve("body-fullscreen");a.events.paint.bind(function(a){w.repaint();c.refresh()});return{element:f.constant(l),socket:function(a){v.append(s,a.element())},
viewport:f.constant(s),uiContainerParent:f.constant(h),connect:function(a){y.show();n.add(I,A);v.append(a,l);Ah.detect(l);return!0},destroy:function(){k.each(t,function(a){a.unbind()});w.destroy();n.remove(I,A);B.remove(l);B.remove(h);y.destroy();G.destroy()},banners:G.element,displayBanner:G.display,hideBanner:G.hide,position:function(){z.init(w,G,s,e,g)},refresh:function(){z.update();w.refresh()},toolbar:w,dialogContainer:f.constant(b)}}function zW(a){var b=h.none(),c=h.none(),d=D.create({maximize:s([]),
restore:s([])}),e=function(c){b=y.parent(a.get().element());b.getOrDie("maximize called on editor that is not in the DOM.");a.swap(Sk,{ui:{layout:AW,isFullScreen:!0,autoresize:!1},fullscreen:{layout:void 0!==c.layout()?c.layout():xp},content:{autoresize:!1}}).destroy();var e=a.get();e.fullscreen().select();e.connect(ca.body());e.events.ready.bind(function(){e.focus();d.trigger.maximize()})},g=function(c){a.swap(Sk,{ui:{layout:void 0!==c.layout()?c.layout():xp,isFullScreen:!1}}).destroy();var e=a.get();
e.fullscreen().deselect();c=b.getOrDie("restore called on editor that was not previously in the DOM.");a.get().connect(c);e.events.ready.bind(function(){e.focus();d.trigger.restore()})};return{init:function(b){if(Tk.goingToDesign(b)){b=a.get();var d=a.get().element();c.each(function(a){if(L.eq(a,d))throw"Fullscreen init called twice on the same instance";});c=h.some(d);b.fullscreen().events.maximize.bind(e);b.fullscreen().events.restore.bind(g)}},destroy:function(){},events:d.registry}}function BW(){var a=
[];return{setCallback:function(b){R.isFunction(b)&&a.push(b)},done:function(b){k.each(a,function(a){a.apply(null,[b])});a=[]}}}function CW(a){var b=function(){var b=a.url,c=b.lastIndexOf("/"),b=0<c?b.substr(0,c):b,b=void 0===a.basePath?b:a.basePath;return aa.endsWith(b,"/")?b:b+"/"}(),c=function(c,e,g){var l=ii.getFilename(c,e);e=!0===a.credentials?nd.xhr():nd.none();c=hy.file(ii.buildData("image",c,l));wd.post(a.url,c,xd.text(),e).map(function(a){a.fold(function(a){g(H.error(ii.failureObject(a.message(),
a.status(),a.responseText())))},function(a){var c;try{var d=$c.parse(a);if(R.isString(d.location))c=d.location;else{g(H.error(ii.failureObject("JSON response did not contain a string location",500,a)));return}}catch(e){c=a}a=c.split(/\s+/);a=ux.resolve(b,1===a.length&&""!==a[0]?a[0]:l);g(H.value({location:a}))})})};return{upload:function(a,b,g){a=a.imageresult();tc.imageResultToBlob(a).then(function(a){c(a,b,g)})}}}function DW(a){var b=function(b,d,e,g){var l=function(a){g(H.error(a))},f=function(a){R.isString(a)?
g(H.value({location:a})):(console.error("Image upload result was not a string"),l(""))},h=ii.getFilename(b,e),k=EW(e,h,b,d);setTimeout(function(){a(k,f,l)},0)};return{upload:function(a,d,e){var g=a.imageresult();tc.imageResultToBlob(g).then(function(a){b(a,tc.imageResultToDataURL(g),d,e)})}}}function FW(){var a={},b=function(a){Re.revokeObjectURL(a.objurl())};return{add:function(b,d,e){d=GW(b,d,e);return a[b]=d},get:function(b){return h.from(a[b])},remove:function(c){var d=a[c];delete a[c];void 0!==
d&&b(d)},lookupByData:function(b){return W.find(a,function(a){return tc.imageResultToDataURL(a.imageresult())===b})},destroy:function(){W.each(a,b);a={}}}}function HW(){var a=[],b=[],c=D.create({complete:s(["response"])}),d=function(d){a=k.filter(a,function(a,b){return a!==d});!1===e()&&(c.trigger.complete(b),b=[])},e=function(){return 0<a.length};return{findById:IW,findAll:JW,register:function(b,c){m.set(b,Uk,c);a.push(c)},report:function(a,c,e){k.each(c,function(a){m.remove(a,Uk);b.push({success:e,
element:a.dom()})});d(a)},inProgress:e,isActive:function(b){return k.contains(a,b)},events:c.registry}}function KW(a,b){var c=FW(),d=BW(),e=HW(),g=Po(a.ui),l=LW.choose(b,g,a.communication()),f=l.uploader(),r=l.errors(),z=MW.instance(b,c),w=D.create({begin:s([]),finish:s([])}),m=function(a){w.trigger.finish();d.done(a)};e.events.complete.bind(function(a){m(a.response())});var n=function(a,b,d,g){ji.handleUpload(f,e,c,a,b,d,function(a){a.fold(function(a){g(a)},t)})},q=function(d){a.content().body().each(function(a){var e=
r.instance(),g=ji.registerAssets(c,a,d.assets()),g=di.partition(g);k.each(g.errors,function(a){console.error(a)});0<g.values.length&&!0===b.immediate&&(w.trigger.begin(),k.each(g.values,function(b){u(a,b,e)}))})},t=function(b,c,d){a.undo().rewrite(d.objurl(),b.location)},u=function(a,c,d){!1!==b.bundled&&ji.prepareForUpload(e,c.blobInfo().id(),c.image()).each(function(b){n(a,b,c.blobInfo(),d)})},v=function(a){k.each(a,function(a){e.report(a,h.none(),!1)})},x=function(a,b){var c=di.partition(b),d=
c.values;v(c.errors);var e=r.instance();z(d).get(function(b){b=di.partition(b);0<b.errors.length&&(g.showKey("imageedit.generic"),console.error("Image resample on upload errors: ",b.errors),v(b.errors));k.each(b.values,function(b){u(a,b,e)})})},y=function(g){R.isFunction(g)&&d.setCallback(g);w.trigger.begin();!0!==b.bundled?m([]):a.content().body().fold(function(){m([])},function(a){var b=ji.findBlobs(e,c,a);0!==b.length||e.inProgress()?x(a,b):m([])})};return{init:function(d){Tk.goingToDesign(d)&&
(d=a.filter(),NW.configure(d,c),a.uploader().events.uploadAssets.bind(q),a.uploader().events.pasteUpload.bind(function(){!0===b.immediate&&y()}));g.reset()},events:w.registry,reconstitute:y,destroy:function(){c.destroy()}}}function OW(a,b,c,d){var e=zW(a),g=KW(b,c.imageupload),l=uW(a,c.codeview),f=xW(a,c.codeview),h=[e,g,f],z=h.concat([l]);e.events.restore.bind(function(){d.restore();m(z,Ab.code())});e.events.maximize.bind(function(){d.save();m(h,Ab.code())});g.events.begin.bind(f.spin);g.events.finish.bind(f.stopSpin);
l.events.change.bind(function(c){a.get().events.loaded.bind(function(){b.focus()});b.reconnect()});f.events.change.bind(function(a){w(a.type())});l.events.ready.bind(function(a){f.switchto(a.type())});var w=function(b){g.reconstitute(function(){a.get().getMode()!==b&&(f.spin(),l.setMode(b))})},m=function(a,b){k.each(a,function(a){a.init(b)})};b.events.connected.bind(function(){m(z,Tk.alternate(b.getMode()))});return{setMode:w,uploadImages:g.reconstitute,destroy:function(){f.destroy();a.destroy();
k.each(z,function(a){a.destroy()})}}}function PW(a){var b=QW(a);return{directionAt:function(a){return"rtl"===(A.isText(a)?y.parent(a):h.some(a)).map(rd.getDirection).getOr("ltr")?iy:jy},uiDirection:function(){return b?iy:jy},updateUi:function(a){m.set(a,"dir",b?"rtl":"ltr")},destroy:f.noop}}function RW(a,b){var c=D.create({create:s(["instance"]),loaded:s([]),focus:s([]),dirty:s([]),change:s([]),internal:s(["command","data"])}),d=hb.cached(function(){c.trigger.loaded()}),e=function(a){a.events.focus.bind(function(){c.trigger.focus()});
a.events.dirty.bind(function(){c.trigger.dirty()});a.events.change.bind(function(){c.trigger.change()});a.events.internal.bind(function(a){c.trigger.internal(a.command(),a.data())});a.events.loaded.bind(d)};e(a);var g=a,l=f.compose(PW,fe.get);return{get:function(){return g},swap:function(a,d){var h=g,k=Z.deepMerge(b,d),m=h.state(),k=a(k,l(),m);e(k);c.trigger.create(k);g=k;return{old:f.constant(h),nu:f.constant(k),destroy:function(){h.destroy()}}},destroy:function(){g.destroy()},events:c.registry}}
function yp(a,b){var c=SW(a);ki.setLanguage(c.ui.locale.code);var d=Z.deepMerge(ky.state(),b),e=gc.capture(q.fromDom(document),window),g=tW(c,d),l=RW(g,c),f=hP(l),d=OW(l,f,c,e),e=D.create({loaded:M([],l.events.loaded),focus:M([],l.events.focus),dirty:M([],l.events.dirty),change:M([],l.events.change),connected:M([],f.events.connected),internal:M(["command","data"],l.events.internal)});ki.waitForLoad().get(function(a){a=ki.getFailures(a);if(0===a.length){a=l.swap("code"===c.codeview.mode?Tu:Sk);f.reconnect();
var b=function(a){Qe.isEmpty(a)||f.setContent(a)},d=a.old().toTransfer();xe.cata(d,b,b);a.destroy()}else k.each(a,function(a){console.error(a)}),console.error("Required resources are not available. Cancelling editor load."),g.failure()});return zh.external({activateInline:f.activateInline,communication:Rc.communication,connect:f.connect,content:f.content,deactivateInline:f.deactivateInline,element:f.element,events:e.registry,filter:f.filter,focus:f.focus,getContent:f.getContent,getMode:f.getMode,
insertHtmlAtCursor:f.insertHtmlAtCursor,isDirty:f.isDirty,isPrettyPrinting:f.isPrettyPrinting,keyboard:f.keyboard,macros:f.macros,operations:f.operations,setContent:f.setContent,setDirty:f.setDirty,smartview:f.smartview,state:f.state,ui:f.ui,undo:f.undo,setMode:d.setMode,uploadImages:d.uploadImages,destroy:d.destroy})}function TW(a){a=void 0!==a?a:{};return Z.deepMerge(UW(a),VW(a))}function WW(a,b,c){n.add(b,jd.base());return{toolbarShow:function(){n.add(b,jd.active());t.set(a(),"display","block")},
toolbarHide:function(){n.remove(b,jd.active());t.set(a(),"display","none")},isVisible:function(){return n.has(b,jd.active())},restore:function(){ea.remove(b,[jd.base(),jd.active()]);m.setAll(b,c)}}}function XW(a){var b=!1;return{submit:function(){b=!0;a.submit[0]?a.submit[0].click():a.submit.click?a.submit.click():a.submit();b=!1},inprogress:function(){return b}}}function YW(a){var b=function(b,d){var e=hb.cached(function(){var b=Z.deepMerge(TW(d),a);return ly.setupResources(b)});return{constructors:f.constant(b),
options:e}};return{classic:function(a){return b(ZW,a)},inline:function(a){a=Z.deepMerge({ui:{toolbar:{draggable:!0}}},a);return b($W,a)}}}function aX(a){var b=oO(),c=YW(a),d=function(a,c){var d;d=R.isString(a)?ra.one(a).getOrThunk(f.die("Selector did not match any elements: "+a)):q.fromDom(a);return b.getOrCreate(d,c)},e=function(a,b){R.isString(a)?console.error("Selector did not match a "+b+": ",a):console.error("Element was not a "+b+": ",a);throw Error("Unable to create textbox.io editor");},g=
Y.detect(),l=function(a){R.isString(a)||ob(a,[V.domElement("element")]);return R.isString(a)?ra.all(a):k.map(a,q.fromDom)},p=function(a,c){var d=l(a),d=k.map(d,function(a){return b.getOrCreate(a,c)});return ab.cat(d)};return Mj.makeReadOnly({replace:function(a,b){ob(arguments,[V.stringOrDom("element"),Vk.object("options")]);return d(a,c.classic(b)).getOrThunk(f.curry(e,a,"div or textarea"))},replaceAll:function(a,b){ob(arguments,[V.stringOrArray("elements"),Vk.object("options")]);return p(a,c.classic(b))},
inline:function(a,b){if(g.deviceType.isTouch())throw Error("Mobile does not not support inline editing");ob(arguments,[V.stringOrDom("element"),Vk.object("options")]);return d(a,c.inline(b)).getOrThunk(f.curry(e,a,"supported tag for inline editing"))},inlineAll:function(a,b){if(g.deviceType.isTouch())throw Error("Mobile does not not support inline editing");ob(arguments,[V.stringOrArray("elements"),Vk.object("options")]);return p(a,c.inline(b))},get:function(a){ob(arguments,[V.string("selector")]);
return b.get(a)},getActiveEditor:function(){return b.getActiveEditor().getOr(null)},definePlugin:function(a,b){ob(arguments,[V.string("namespace"),V.object("code")]);ao.get(a).each(function(){throw Error("namespace "+a+" is already defined");});if(void 0===b.initialize||!R.isFunction(b.initialize))throw Error("plugin does not have an initialize function");return ao.register(a,b)},version:Hu,isSupported:f.always})}var zp=function(a){return function(){return a}},bX=zp(!1),cX=zp(!0),f={noop:function(){},
noarg:function(a){return function(){return a()}},compose:function(a,b){return function(){return a(b.apply(null,arguments))}},constant:zp,identity:function(a){return a},tripleEquals:function(a,b){return a===b},curry:function(a){for(var b=Array(arguments.length-1),c=1;c<arguments.length;c++)b[c-1]=arguments[c];return function(){for(var c=Array(arguments.length),e=0;e<c.length;e++)c[e]=arguments[e];c=b.concat(c);return a.apply(null,c)}},not:function(a){return function(){return!a.apply(null,arguments)}},
die:function(a){return function(){throw Error(a);}},apply:function(a){return a()},call:function(a){a()},never:bX,always:cX},li=f.never,Ap=f.always,Ff=function(){return Bp},Bp=function(){var a=function(a){return a.isNone()},b=function(a){return a()},c=function(a){return a},a={fold:function(a,b){return a()},is:li,isSome:li,isNone:Ap,getOr:c,getOrThunk:b,getOrDie:function(a){throw Error(a||"error: getOrDie called on none.");},or:c,orThunk:b,map:Ff,ap:Ff,each:function(){},bind:Ff,flatten:Ff,exists:li,
forall:Ap,filter:Ff,equals:a,equals_:a,toArray:function(){return[]},toString:f.constant("none()")};Object.freeze&&Object.freeze(a);return a}(),Wk=function(a){var b=function(){return a},c=function(){return e},d=function(b){return b(a)},e={fold:function(b,c){return c(a)},is:function(b){return a===b},isSome:Ap,isNone:li,getOr:b,getOrThunk:b,getOrDie:b,or:c,orThunk:c,map:function(b){return Wk(b(a))},ap:function(b){return b.fold(Ff,function(b){return Wk(b(a))})},each:function(b){b(a)},bind:d,flatten:b,
exists:d,forall:d,filter:function(b){return b(a)?e:Bp},equals:function(b){return b.is(a)},equals_:function(b,c){return b.fold(li,function(b){return c(a,b)})},toArray:function(){return[a]},toString:function(){return"some("+a+")"}};return e},h={some:Wk,none:Ff,from:function(a){return null===a||void 0===a?Bp:Wk(a)}},my=function(){var a=Array.prototype.indexOf,b=function(b,c){return a.call(b,c)},c=function(a,b){var c;a:{c=0;for(var l=a.length;c<l;++c)if(a[c]===b)break a;c=-1}return c};return void 0===
a?c:b}(),ny=function(a,b){return-1<my(a,b)},oy=function(a,b){for(var c=a.length,d=Array(c),e=0;e<c;e++)d[e]=b(a[e],e,a);return d},py=function(a,b){for(var c=0,d=a.length;c<d;c++)b(a[c],c,a)},qy=function(a,b){for(var c=a.length-1;0<=c;c--)b(a[c],c,a)},ry=function(a,b){for(var c=[],d=0,e=a.length;d<e;d++){var g=a[d];b(g,d,a)&&c.push(g)}return c},sy=function(a,b){for(var c=0,d=a.length;c<d;c++)if(b(a[c],c,a))return h.some(c);return h.none()},dX=Array.prototype.push,ty=function(a){for(var b=[],c=0,d=
a.length;c<d;++c){if(!Array.prototype.isPrototypeOf(a[c]))throw Error("Arr.flatten item "+c+" was not an array, input: "+a);dX.apply(b,a[c])}return b},uy=function(a,b){for(var c=0,d=a.length;c<d;++c)if(!0!==b(a[c],c,a))return!1;return!0},vy=Array.prototype.slice,k={map:oy,each:py,eachr:qy,partition:function(a,b){for(var c=[],d=[],e=0,g=a.length;e<g;e++){var l=a[e];(b(l,e,a)?c:d).push(l)}return{pass:c,fail:d}},filter:ry,groupBy:function(a,b){if(0===a.length)return[];for(var c=b(a[0]),d=[],e=[],g=0,
l=a.length;g<l;g++){var f=a[g],h=b(f);h!==c&&(d.push(e),e=[]);c=h;e.push(f)}0!==e.length&&d.push(e);return d},indexOf:function(a,b){var c=my(a,b);return-1===c?h.none():h.some(c)},foldr:function(a,b,c){qy(a,function(a){c=b(c,a)});return c},foldl:function(a,b,c){py(a,function(a){c=b(c,a)});return c},find:function(a,b){for(var c=0,d=a.length;c<d;c++){var e=a[c];if(b(e,c,a))return h.some(e)}return h.none()},findIndex:sy,flatten:ty,bind:function(a,b){var c=oy(a,b);return ty(c)},forall:uy,exists:function(a,
b){return sy(a,b).isSome()},contains:ny,equal:function(a,b){return a.length===b.length&&uy(a,function(a,d){return a===b[d]})},reverse:function(a){a=vy.call(a,0);a.reverse();return a},chunk:function(a,b){for(var c=[],d=0;d<a.length;d+=b){var e=a.slice(d,d+b);c.push(e)}return c},difference:function(a,b){return ry(a,function(a){return!ny(b,a)})},mapToObject:function(a,b){for(var c={},d=0,e=a.length;d<e;d++){var g=a[d];c[String(g)]=b(g,d)}return c},pure:function(a){return[a]},sort:function(a,b){var c=
vy.call(a,0);c.sort(b);return c},range:function(a,b){for(var c=[],d=0;d<a;d++)c.push(b(d));return c},head:function(a){return 0===a.length?h.none():h.some(a[0])},last:function(a){return 0===a.length?h.none():h.some(a[a.length-1])}},Se=function(a){return function(b){if(null===b)b="null";else{var c=typeof b;b="object"===c&&Array.prototype.isPrototypeOf(b)?"array":"object"===c&&String.prototype.isPrototypeOf(b)?"string":c}return b===a}},R={isString:Se("string"),isObject:Se("object"),isArray:Se("array"),
isNull:Se("null"),isBoolean:Se("boolean"),isUndefined:Se("undefined"),isFunction:Se("function"),isNumber:Se("number")},Ke="undefined"!==typeof window?window:Function("return this;")(),wy=function(a,b){for(var c=void 0!==b&&null!==b?b:Ke,d=0;d<a.length&&void 0!==c&&null!==c;++d)c=c[a[d]];return c},xy=function(a,b){for(var c=void 0!==b?b:Ke,d=0;d<a.length;++d){var e=a[d];if(void 0===c[e]||null===c[e])c[e]={};c=c[e]}return c},ne={path:wy,resolve:function(a,b){var c=a.split(".");return wy(c,b)},forge:xy,
namespace:function(a,b){var c=a.split(".");return xy(c,b)}},Vb={getOrDie:function(a,b){var c=ne.resolve(a,b);if(void 0===c||null===c)throw a+" not available on this browser";return c}},mi={isPrototypeOf:function(a){var b=ne.resolve("ownerDocument.defaultView",a);return Vb.getOrDie("HTMLElement",b).prototype.isPrototypeOf(a)}},ni=function(a,b,c,d,e,g){return{name:f.constant(a),output:f.constant(g||a),required:f.constant(d),validate:function(d){if(!c(d))throw d=e(a,b),Error(d);}}},oi=function(a,b){return'value of "'+
a+'" is not a '+b},Gf=function(a,b){return'value of "'+a+'" is not an '+b},oe=function(a,b,c){return function(d,e){return ni(d,a,b,!0,c,e)}},yy=function(a){return mi.isPrototypeOf(a)},eX=function(a){return R.isString(a)||R.isArray(a)||R.isObject(a)},Cp=oe("array",R.isArray,Gf),fX=oe("boolean",R.isBoolean,oi),gX=oe("function",R.isFunction,oi),hX=oe("number",R.isNumber,oi),iX=oe("object",R.isObject,Gf),jX=oe("string",R.isString,oi),kX=oe("DOM element",yy,oi),lX=oe("string or an array",function(a){return R.isString(a)||
R.isArray(a)},Gf),mX=oe("string or a DOM element",function(a){return R.isString(a)||yy(a)},Gf),V={arrayOfStrings:function(a){return ni(a,"array of strings",function(b){Cp(a,void 0).validate(b);return k.forall(b,R.isString)},!0,Gf,void 0)},arrayOfObjects:function(a){return ni(a,"array of objects",function(b){Cp(a,void 0).validate(b);return k.forall(b,R.isObject)},!0,Gf,void 0)},arrayOfToolbarGroups:function(a){return ni(a,"array of toolbar groups",function(b){Cp(a,void 0).validate(b);return k.forall(b,
eX)},!0,Gf,void 0)},bool:fX,domElement:kX,func:gX,number:hX,object:iX,string:jX,stringOrArray:lX,stringOrDom:mX,param:ni},zy={paramList:function(){return[V.object("aria"),V.func("fetch"),V.func("menuComplete"),V.func("menuExtra"),V.func("menuHover"),V.string("hoverSelector"),V.func("hover")]},encodeWithEditorApi:function(){return["menuComplete"]}},Ay=function(a){return function(){for(var b=Array(arguments.length),c=0;c<b.length;c++)b[c]=arguments[c];if(0===b.length)throw Error("Can't merge zero objects");
for(var c={},d=0;d<b.length;d++){var e=b[d],g;for(g in e)e.hasOwnProperty(g)&&(c[g]=a(c[g],e[g]))}return c}},By=Ay(function(a,b){return R.isObject(a)&&R.isObject(b)?By(a,b):b}),nX=Ay(function(a,b){return b}),Z={deepMerge:By,merge:nX},Dp=function(){var a=Object.keys,b=function(a){var b=[],e;for(e in a)a.hasOwnProperty(e)&&b.push(e);return b};return void 0===a?b:a}(),Xk=function(a,b){for(var c=Dp(a),d=0,e=c.length;d<e;d++){var g=c[d];b(a[g],g,a)}},Cy=function(a,b){var c={};Xk(a,function(d,e){var g=
b(d,e,a);c[g.k]=g.v});return c},Dy=function(a,b){var c=[];Xk(a,function(a,e){c.push(b(a,e))});return c},Ey=function(a){return Dy(a,function(a){return a})},W={bifilter:function(a,b){var c={},d={};Xk(a,function(a,g){(b(a,g)?c:d)[g]=a});return{t:c,f:d}},each:Xk,map:function(a,b){return Cy(a,function(a,d,e){return{k:d,v:b(a,d,e)}})},mapToArray:Dy,tupleMap:Cy,find:function(a,b){for(var c=Dp(a),d=0,e=c.length;d<e;d++){var g=c[d],l=a[g];if(b(l,g,a))return h.some(l)}return h.none()},keys:Dp,values:Ey,size:function(a){return Ey(a).length}},
Ep=function(a,b,c){return k.map(a,function(a){return b(a,c)})},Fp=function(a,b){if(R.isString(a))return a;if(R.isArray(a))return Ep(a,Fp,b);if(R.isObject(a)&&void 0!==a.action){var c=f.compose(a.action,b);return Z.merge(a,{action:c})}c=Ep(a.items,Fp,b);return Z.deepMerge(a,{items:c})},Iu={mentions:function(a,b){var c=zy.encodeWithEditorApi();return W.map(a,function(a,e){return k.contains(c,e)?function(c){return a(b(),c)}:a})},toolbar:function(a,b){return void 0===a.items?a:{items:Ep(a.items,Fp,b)}}},
Yk=function(a){return{is:function(b){return a===b},isValue:f.always,isError:f.never,getOr:f.constant(a),getOrThunk:f.constant(a),getOrDie:f.constant(a),or:function(b){return Yk(a)},orThunk:function(b){return Yk(a)},fold:function(b,c){return c(a)},map:function(b){return Yk(b(a))},each:function(b){b(a)},bind:function(b){return b(a)},exists:function(b){return b(a)},forall:function(b){return b(a)},toOption:function(){return h.some(a)}}},Gp=function(a){return{is:f.never,isValue:f.never,isError:f.always,
getOr:f.identity,getOrThunk:function(a){return a()},getOrDie:function(){return f.die(a)()},or:function(a){return a},orThunk:function(a){return a()},fold:function(b,c){return b(a)},map:function(b){return Gp(a)},each:f.noop,bind:function(b){return Gp(a)},exists:f.never,forall:f.always,toOption:h.none}},H={value:Yk,error:Gp},aO=function(){var a=(new Date).getTime();return String(a).slice(-6)},pi=function(a){if(null===a||void 0===a)throw Error("Node cannot be null or undefined");return{dom:f.constant(a)}},
q={fromHtml:function(a,b){var c=(b||document).createElement("div");c.innerHTML=a;if(!c.hasChildNodes()||1<c.childNodes.length)throw console.error("HTML does not have a single root node",a),"HTML must have a single root node";return pi(c.childNodes[0])},fromTag:function(a,b){var c=(b||document).createElement(a);return pi(c)},fromText:function(a,b){var c=(b||document).createTextNode(a);return pi(c)},fromDom:pi,fromPoint:function(a,b,c){return h.from(a.dom().elementFromPoint(b,c)).map(pi)}},ra={all:function(a,
b){var c=void 0===b?document:b.dom();return 1!==c.nodeType&&9!==c.nodeType||0===c.childElementCount?[]:k.map(c.querySelectorAll(a),q.fromDom)},is:function(a,b){var c=a.dom();if(1!==c.nodeType)return!1;if(void 0!==c.matches)return c.matches(b);if(void 0!==c.msMatchesSelector)return c.msMatchesSelector(b);if(void 0!==c.webkitMatchesSelector)return c.webkitMatchesSelector(b);if(void 0!==c.mozMatchesSelector)return c.mozMatchesSelector(b);throw Error("Browser lacks native selectors");},one:function(a,
b){var c=void 0===b?document:b.dom();return 1!==c.nodeType&&9!==c.nodeType||0===c.childElementCount?h.none():h.from(c.querySelector(a)).map(q.fromDom)}},Fy=function(a){return a.dom().nodeName.toLowerCase()},Hp=function(a){return a.dom().nodeType},Ip=function(a){return function(b){return Hp(b)===a}},oX=Ip(1),pX=Ip(3),qX=Ip(9),A={name:Fy,type:Hp,value:function(a){return a.dom().nodeValue},isElement:oX,isText:pX,isDocument:qX,isComment:function(a){return 8===Hp(a)||"#comment"===Fy(a)}},pO={one:function(a,
b,c){var d=A.name(a);return h.from(b[d]).map(function(b){return b.replace(a,c)})}},jk={documentPositionPreceding:function(a,b){var c=Vb.getOrDie("Node").DOCUMENT_POSITION_PRECEDING;return 0!==(a.compareDocumentPosition(b)&c)},documentPositionContainedBy:function(a,b){var c=Vb.getOrDie("Node").DOCUMENT_POSITION_CONTAINED_BY;return 0!==(a.compareDocumentPosition(b)&c)}},hb={cached:function(a){var b=!1,c;return function(){b||(b=!0,c=a.apply(null,arguments));return c}}},Gy=function(){return Jp(0,0)},
Jp=function(a,b){return{major:a,minor:b}},Zk={nu:Jp,detect:function(a,b){var c=String(b).toLowerCase();if(0===a.length)c=Gy();else{var d;a:{for(d=0;d<a.length;d++){var e=a[d];if(e.test(c)){d=e;break a}}d=void 0}c=d?Jp(Number(c.replace(d,"$1")),Number(c.replace(d,"$2"))):{major:0,minor:0}}return c},unknown:Gy},zg=function(a,b){return function(){return b===a}},Hy=function(a){var b=a.current;return{current:b,version:a.version,isEdge:zg("Edge",b),isChrome:zg("Chrome",b),isIE:zg("IE",b),isOpera:zg("Opera",
b),isFirefox:zg("Firefox",b),isSafari:zg("Safari",b)}},rX=function(){return Hy({current:void 0,version:Zk.unknown()})};f.constant("Edge");f.constant("Chrome");f.constant("IE");f.constant("Opera");f.constant("Firefox");f.constant("Safari");var Hf=function(a,b){return function(){return b===a}},Iy=function(a){var b=a.current;return{current:b,version:a.version,isWindows:Hf("Windows",b),isiOS:Hf("iOS",b),isAndroid:Hf("Android",b),isOSX:Hf("OSX",b),isLinux:Hf("Linux",b),isSolaris:Hf("Solaris",b),isFreeBSD:Hf("FreeBSD",
b)}},sX=function(){return Iy({current:void 0,version:Zk.unknown()})};f.constant("Windows");f.constant("iOS");f.constant("Android");f.constant("Linux");f.constant("OSX");f.constant("Solaris");f.constant("FreeBSD");var Jy=function(a,b){var c=String(b).toLowerCase();return k.find(a,function(a){return a.search(c)})},Ky={detectBrowser:function(a,b){return Jy(a,b).map(function(a){var d=Zk.detect(a.versionRegexes,b);return{current:a.name,version:d}})},detectOs:function(a,b){return Jy(a,b).map(function(a){var d=
Zk.detect(a.versionRegexes,b);return{current:a.name,version:d}})}},$k={addToStart:function(a,b){return b+a},addToEnd:function(a,b){return a+b},removeFromStart:function(a,b){return a.substring(b)},removeFromEnd:function(a,b){return a.substring(0,a.length-b)}},Ly={first:function(a,b){return a.substr(0,b)},last:function(a,b){return a.substr(a.length-b,a.length)},head:function(a){return""===a?h.none():h.some(a.substr(0,1))},tail:function(a){return""===a?h.none():h.some(a.substring(1))}},My=function(a,
b,c){return""===b?!0:a.length<b.length?!1:a.substr(c,c+b.length)===b},Kp=function(a,b){return My(a,b,0)},Lp=function(a,b){return My(a,b,a.length-b.length)},aa={supplant:function(a,b){return a.replace(/\${([^{}]*)}/g,function(a,d){var e=b[d],g=typeof e;return"string"===g||"number"===g?e:a})},startsWith:Kp,removeLeading:function(a,b){return Kp(a,b)?$k.removeFromStart(a,b.length):a},removeTrailing:function(a,b){return Lp(a,b)?$k.removeFromEnd(a,b.length):a},ensureLeading:function(a,b){return Kp(a,b)?
a:$k.addToStart(a,b)},ensureTrailing:function(a,b){return Lp(a,b)?a:$k.addToEnd(a,b)},endsWith:Lp,contains:function(a,b){return-1!==a.indexOf(b)},trim:function(a){return a.replace(/^\s+|\s+$/g,"")},lTrim:function(a){return a.replace(/^\s+/g,"")},rTrim:function(a){return a.replace(/\s+$/g,"")},capitalize:function(a){return Ly.head(a).bind(function(b){return Ly.tail(a).map(function(a){return b.toUpperCase()+a})}).getOr(a)}},Mp=/.*?version\/\ ?([0-9]+)\.([0-9]+).*/,Te=function(a){return function(b){return aa.contains(b,
a)}},tX=[{name:"Edge",versionRegexes:[/.*?edge\/ ?([0-9]+)\.([0-9]+)$/],search:function(a){return aa.contains(a,"edge/")&&aa.contains(a,"chrome")&&aa.contains(a,"safari")&&aa.contains(a,"applewebkit")}},{name:"Chrome",versionRegexes:[/.*?chrome\/([0-9]+)\.([0-9]+).*/,Mp],search:function(a){return aa.contains(a,"chrome")&&!aa.contains(a,"chromeframe")}},{name:"IE",versionRegexes:[/.*?msie\ ?([0-9]+)\.([0-9]+).*/,/.*?rv:([0-9]+)\.([0-9]+).*/],search:function(a){return aa.contains(a,"msie")||aa.contains(a,
"trident")}},{name:"Opera",versionRegexes:[Mp,/.*?opera\/([0-9]+)\.([0-9]+).*/],search:Te("opera")},{name:"Firefox",versionRegexes:[/.*?firefox\/\ ?([0-9]+)\.([0-9]+).*/],search:Te("firefox")},{name:"Safari",versionRegexes:[Mp,/.*?cpu os ([0-9]+)_([0-9]+).*/],search:function(a){return(aa.contains(a,"safari")||aa.contains(a,"mobile/"))&&aa.contains(a,"applewebkit")}}],uX=[{name:"Windows",search:Te("win"),versionRegexes:[/.*?windows\ nt\ ?([0-9]+)\.([0-9]+).*/]},{name:"iOS",search:function(a){return aa.contains(a,
"iphone")||aa.contains(a,"ipad")},versionRegexes:[/.*?version\/\ ?([0-9]+)\.([0-9]+).*/,/.*cpu os ([0-9]+)_([0-9]+).*/,/.*cpu iphone os ([0-9]+)_([0-9]+).*/]},{name:"Android",search:Te("android"),versionRegexes:[/.*?android\ ?([0-9]+)\.([0-9]+).*/]},{name:"OSX",search:Te("os x"),versionRegexes:[/.*?os\ x\ ?([0-9]+)_([0-9]+).*/]},{name:"Linux",search:Te("linux"),versionRegexes:[]},{name:"Solaris",search:Te("sunos"),versionRegexes:[]},{name:"FreeBSD",search:Te("freebsd"),versionRegexes:[]}],Ny={browsers:f.constant(tX),
oses:f.constant(uX)},vX={detect:function(a){var b=Ny.browsers(),c=Ny.oses(),b=Ky.detectBrowser(b,a).fold(rX,Hy),c=Ky.detectOs(c,a).fold(sX,Iy),d=c.isiOS()&&!0===/ipad/i.test(a),e=c.isiOS()&&!d,g=c.isAndroid()&&3===c.version.major,l=c.isAndroid()&&4===c.version.major,g=d||g||l&&!0===/mobile/i.test(a),p=(l=c.isiOS()||c.isAndroid())&&!g;a=b.isSafari()&&c.isiOS()&&!1===/safari/i.test(a);a={isiPad:f.constant(d),isiPhone:f.constant(e),isTablet:f.constant(g),isPhone:f.constant(p),isTouch:f.constant(l),isAndroid:c.isAndroid,
isiOS:c.isiOS,isWebView:f.constant(a)};return{browser:b,os:c,deviceType:a}}},Y={detect:hb.cached(function(){return vX.detect(navigator.userAgent)})},Oy=function(a,b){return a.dom()===b.dom()},wX=function(a,b){var c=a.dom(),d=b.dom();return c===d?!1:c.contains(d)},xX=function(a,b){return jk.documentPositionContainedBy(a.dom(),b.dom())},yX=Y.detect().browser.isIE()?xX:wX,L={eq:Oy,isEqualNode:function(a,b){return a.dom().isEqualNode(b.dom())},member:function(a,b){return k.exists(b,f.curry(Oy,a))},contains:yX,
is:ra.is},Py=function(a){a=a.dom().body;if(null===a||void 0===a)throw"Body is not available yet";return q.fromDom(a)},ca={body:hb.cached(function(){return Py(q.fromDom(document))}),getBody:Py,inBody:function(a){a=A.isText(a)?a.dom().parentNode:a.dom();return void 0!==a&&null!==a&&a.ownerDocument.body.contains(a)}},Qy=function(a,b,c){a=a.dom();for(c=R.isFunction(c)?c:f.constant(!1);a.parentNode;){a=a.parentNode;var d=q.fromDom(a);if(b(d))return h.some(d);if(c(d))break}return h.none()},Ry=function(a,
b){return k.find(a.dom().childNodes,f.compose(b,q.fromDom)).map(q.fromDom)},Sy=function(a,b){var c=function(a){for(var e=0;e<a.childNodes.length;e++){if(b(q.fromDom(a.childNodes[e])))return h.some(q.fromDom(a.childNodes[e]));var g=c(a.childNodes[e]);if(g.isSome())return g}return h.none()};return c(a.dom())},sa={first:function(a){return Sy(ca.body(),a)},ancestor:Qy,closest:function(a,b,c){return Ju(function(a){return b(a)},Qy,a,b,c)},sibling:function(a,b){var c=a.dom();return c.parentNode?Ry(q.fromDom(c.parentNode),
function(c){return!L.eq(a,c)&&b(c)}):h.none()},child:Ry,descendant:Sy},Gc={any:function(a){return sa.first(a).isSome()},ancestor:function(a,b,c){return sa.ancestor(a,b,c).isSome()},closest:function(a,b,c){return sa.closest(a,b,c).isSome()},sibling:function(a,b){return sa.sibling(a,b).isSome()},child:function(a,b){return sa.child(a,b).isSome()},descendant:function(a,b){return sa.descendant(a,b).isSome()}},Ag=function(a){return a.slice(0).sort()},Bd={sort:Ag,reqMessage:function(a,b){throw Error("All required keys ("+
Ag(a).join(", ")+") were not specified. Specified keys were: "+Ag(b).join(", ")+".");},unsuppMessage:function(a){throw Error("Unsupported keys for object: "+Ag(a).join(", "));},validateStrArr:function(a,b){if(!R.isArray(b))throw Error("The "+a+" fields must be an array. Was: "+b+".");k.each(b,function(b){if(!R.isString(b))throw Error("The value "+b+" in the "+a+" fields was not a string.");})},invalidTypeMessage:function(a,b){throw Error("All values need to be of type: "+b+". Keys ("+Ag(a).join(", ")+
") were not.");},checkDupes:function(a){var b=Ag(a);k.find(b,function(a,d){return d<b.length-1&&a===b[d+1]}).each(function(a){throw Error("The field: "+a+" occurs more than once in the combined fields: ["+b.join(", ")+"].");})}},C={immutable:function(){var a=arguments;return function(){for(var b=Array(arguments.length),c=0;c<b.length;c++)b[c]=arguments[c];if(a.length!==b.length)throw Error('Wrong number of arguments to struct. Expected "['+a.length+']", got '+b.length+" arguments");var d={};k.each(a,
function(a,c){d[a]=f.constant(b[c])});return d}},immutableBag:function(a,b){var c=a.concat(b);if(0===c.length)throw Error("You must specify at least one required or optional field.");Bd.validateStrArr("required",a);Bd.validateStrArr("optional",b);Bd.checkDupes(c);return function(d){var e=W.keys(d);k.forall(a,function(a){return k.contains(e,a)})||Bd.reqMessage(a,e);var g=k.filter(e,function(a){return!k.contains(c,a)});0<g.length&&Bd.unsuppMessage(g);var l={};k.each(a,function(a){l[a]=f.constant(d[a])});
k.each(b,function(a){l[a]=f.constant(Object.prototype.hasOwnProperty.call(d,a)?h.some(d[a]):h.none())});return l}}},Ty={toArray:function(a,b){var c=[],d=function(a){c.push(a);return b(a)},e=b(a);do e=e.bind(d);while(e.isSome());return c}},Uy=function(a){return q.fromDom(a.dom().ownerDocument)},Np=function(a){a=a.dom();return h.from(a.parentNode).map(q.fromDom)},Vy=function(a){a=a.dom();return h.from(a.previousSibling).map(q.fromDom)},Wy=function(a){a=a.dom();return h.from(a.nextSibling).map(q.fromDom)},
al=function(a){a=a.dom();return k.map(a.childNodes,q.fromDom)},Op=function(a,b){var c=a.dom().childNodes;return h.from(c[b]).map(q.fromDom)},Xy=C.immutable("element","offset"),y={owner:Uy,defaultView:function(a){a=a.dom().ownerDocument.defaultView;return q.fromDom(a)},documentElement:function(a){a=Uy(a);return q.fromDom(a.dom().documentElement)},parent:Np,findIndex:function(a){return Np(a).bind(function(b){b=al(b);return k.findIndex(b,function(b){return L.eq(a,b)})})},parents:function(a,b){for(var c=
R.isFunction(b)?b:f.constant(!1),d=a.dom(),e=[];null!==d.parentNode&&void 0!==d.parentNode;){var d=d.parentNode,g=q.fromDom(d);e.push(g);if(!0===c(g))break}return e},siblings:function(a){return Np(a).map(al).map(function(b){return k.filter(b,function(b){return!L.eq(a,b)})}).getOr([])},prevSibling:Vy,offsetParent:function(a){a=a.dom();return h.from(a.offsetParent).map(q.fromDom)},prevSiblings:function(a){return k.reverse(Ty.toArray(a,Vy))},nextSibling:Wy,nextSiblings:function(a){return Ty.toArray(a,
Wy)},children:al,child:Op,firstChild:function(a){return Op(a,0)},lastChild:function(a){return Op(a,a.dom().childNodes.length-1)},childNodesCount:function(a){return a.dom().childNodes.length},hasChildNodes:function(a){return a.dom().hasChildNodes()},leaf:function(a,b){var c=al(a);return 0<c.length&&b<c.length?Xy(c[b],0):Xy(a,b)}},Yy=function(a){a.dom().focus()},Pp=function(a){a=void 0!==a?a.dom():document;return h.from(a.activeElement).map(q.fromDom)},T={hasFocus:function(a){var b=y.owner(a).dom();
return a.dom()===b.activeElement},focus:Yy,blur:function(a){a.dom().blur()},active:Pp,search:function(a){return Pp(y.owner(a)).filter(function(b){return a.dom().contains(b.dom())})},focusInside:function(a){var b=y.owner(a);Pp(b).filter(function(b){return Gc.closest(b,f.curry(L.eq,a))}).fold(function(){Yy(a)},f.noop)}},bl=function(a,b){y.parent(a).each(function(c){c.dom().insertBefore(b.dom(),a.dom())})},qi=function(a,b){a.dom().appendChild(b.dom())},v={before:bl,after:function(a,b){y.nextSibling(a).fold(function(){y.parent(a).each(function(a){qi(a,
b)})},function(a){bl(a,b)})},prepend:function(a,b){y.firstChild(a).fold(function(){qi(a,b)},function(c){a.dom().insertBefore(b.dom(),c.dom())})},append:qi,appendAt:function(a,b,c){y.child(a,c).fold(function(){qi(a,b)},function(a){bl(a,b)})},wrap:function(a,b){bl(a,b);qi(b,a)}},K={before:function(a,b){k.each(b,function(b){v.before(a,b)})},after:function(a,b){k.each(b,function(c,d){v.after(0===d?a:b[d-1],c)})},prepend:function(a,b){k.each(b.slice().reverse(),function(b){v.prepend(a,b)})},append:function(a,
b){k.each(b,function(b){v.append(a,b)})}},Qp=function(a){a=a.dom();null!==a.parentNode&&a.parentNode.removeChild(a)},B={empty:function(a){a.dom().textContent="";k.each(y.children(a),function(a){Qp(a)})},remove:Qp,unwrap:function(a){var b=y.children(a);0<b.length&&K.before(a,b);Qp(a)}},Zy=function(a,b,c){if(R.isString(c)||R.isBoolean(c)||R.isNumber(c))a.setAttribute(b,c+"");else throw console.error("Invalid call to Attr.set. Key ",b,":: Value ",c,":: Element ",a),Error("Attribute value was not simple");
},$y=function(a,b,c){Zy(a.dom(),b,c)},az=function(a,b){var c=a.dom().getAttribute(b);return null===c?void 0:c},Rp=function(a,b){var c=a.dom();return c&&c.hasAttribute?c.hasAttribute(b):!1},m={clone:function(a){return k.foldl(a.dom().attributes,function(a,c){a[c.name]=c.value;return a},{})},set:$y,setAll:function(a,b){var c=a.dom();W.each(b,function(a,b){Zy(c,b,a)})},get:az,has:Rp,remove:function(a,b){a.dom().removeAttribute(b)},hasNone:function(a){a=a.dom().attributes;return void 0===a||null===a||
0===a.length},transfer:function(a,b,c){A.isElement(a)&&A.isElement(b)&&k.each(c,function(c){Rp(a,c)&&!Rp(b,c)&&$y(b,c,az(a,c))})}},Pd={isSupported:function(a){return void 0!==a.style}},Sp=function(a,b,c){if(!R.isString(c))throw console.error("Invalid call to CSS.set. Property ",b,":: Value ",c,":: Element ",a),Error("CSS value must be a string: "+c);Pd.isSupported(a)&&a.style.setProperty(b,c)},Tp=function(a,b,c){a=a.dom();Sp(a,b,c)},bz=function(a,b){return Pd.isSupported(a)?a.style.getPropertyValue(b):
""},cl=function(a,b){var c=a.dom(),c=bz(c,b);return h.from(c).filter(function(a){return 0<a.length})},zX=function(a,b,c){cl(a,c).each(function(a){cl(b,c).isNone()&&Tp(b,c,a)})},t={copy:function(a,b){var c=a.dom(),d=b.dom();Pd.isSupported(c)&&Pd.isSupported(d)&&(d.style.cssText=c.style.cssText)},set:Tp,preserve:function(a,b){var c=m.get(a,"style"),d=b(a);(void 0===c?m.remove:m.set)(a,"style",c);return d},setAll:function(a,b){var c=a.dom();W.each(b,function(a,b){Sp(c,b,a)})},setOptions:function(a,b){var c=
a.dom();W.each(b,function(a,b){a.fold(function(){Pd.isSupported(c)&&c.style.removeProperty(b)},function(a){Sp(c,b,a)})})},remove:function(a,b){var c=a.dom();Pd.isSupported(c)&&c.style.removeProperty(b);m.has(a,"style")&&""===aa.trim(m.get(a,"style"))&&m.remove(a,"style")},get:function(a,b){var c=a.dom(),d=window.getComputedStyle(c).getPropertyValue(b),c=""!==d||ca.inBody(a)?d:bz(c,b);return null===c?void 0:c},getRaw:cl,getAllRaw:function(a){var b={};a=a.dom();if(Pd.isSupported(a))for(var c=0;c<a.style.length;c++){var d=
a.style.item(c);b[d]=a.style[d]}return b},isValidValue:function(a,b,c){a=q.fromTag(a);Tp(a,b,c);return cl(a,b).isSome()},reflow:function(a){return a.dom().offsetWidth},transfer:function(a,b,c){A.isElement(a)&&A.isElement(b)&&k.each(c,function(c){zX(a,b,c)})}},nO={blur:function(){var a=q.fromHtml('<input type="text" />');t.setAll(a,{"z-index":"-666",background:"transparent",border:"0px",height:"50px",position:"absolute",top:"25%",left:"50%"});v.append(q.fromDom(document.body),a);T.focus(a);B.remove(a)}},
AX=function(a){return[W.map(a,function(a){return a()})]},dO={create:function(){var a=[];return{eventRegistry:{addListener:function(b){ob(arguments,[V.func("handler")]);a.push(b)},removeListener:function(b){k.indexOf(a,b).each(function(b){a.splice(b,1)})}},eventTrigger:function(b){k.each(a,function(a){var d=AX(b);a.apply(null,d)})}}}},Lj={create:function(){var a=[],b=[];return{bind:function(c,d){if(k.contains(a,c))throw"Invalid key, key already exists.";a.push(c);b.push(d);c.bind(d)},unbind:function(c){k.indexOf(a,
c).fold(function(){throw"Invalid key, does not exist.";},function(d){a.splice(d,1);d=b.splice(d,1)[0];c.unbind(d)})},unbindAll:function(){k.each(a,function(a,d){a.unbind(b[d])});a.splice(0,a.length);b.splice(0,b.length)}}}},cz=function(a){Object.freeze(a);W.each(a,function(a){R.isObject(a)&&cz(a)});return a},Mj={makeReadOnly:cz},Up=function(a){var b=h.none(),c=[],d=function(a){e()?g(a):c.push(a)},e=function(){return b.isSome()},g=function(a){b.each(function(b){setTimeout(function(){a(b)},0)})};a(function(a){b=
h.some(a);k.each(c,g);c=[]});return{get:d,map:function(a){return Up(function(b){d(function(c){b(a(c))})})},isReady:e}},$d={nu:Up,pure:function(a){return Up(function(b){b(a)})}},BX={bounce:function(a){return function(){var b=Array.prototype.slice.call(arguments),c=this;setTimeout(function(){a.apply(c,b)},0)}}},ri=function(a){var b=function(b){a(BX.bounce(b))};return{map:function(a){return ri(function(d){b(function(b){b=a(b);d(b)})})},bind:function(a){return ri(function(d){b(function(b){a(b).get(d)})})},
anonBind:function(a){return ri(function(d){b(function(b){a.get(d)})})},toLazy:function(){return $d.nu(b)},get:b}},za={nu:ri,pure:function(a){return ri(function(b){b(a)})}},CX={par:function(a,b){return b(function(b){var d=[],e=0,g=function(g){return function(f){d[g]=f;e++;e>=a.length&&b(d)}};0===a.length?b([]):k.each(a,function(a,b){a.get(g(b))})})}},dz=function(a){return CX.par(a,za.nu)},Nc={par:dz,mapM:function(a,b){var c=k.map(a,b);return dz(c)},compose:function(a,b){return function(c){return b(c).bind(a)}}},
ez={install:function(a){var b=ne.namespace(a);b.callbacks={};var c=0,d=function(){var a="callback_"+c;c++;return a},e=function(c,e){var f=d();b.callbacks[f]=function(){e||g(f);c.apply(null,arguments)};return a+".callbacks."+f},g=function(a){a=a.substring(a.lastIndexOf(".")+1);void 0!==b.callbacks[a]&&delete b.callbacks[a]};b.ephemeral=function(a){return e(a,!1)};b.permanent=function(a){return e(a,!0)};b.unregister=g;return b}},DX=function(a,b,c,d,e,g,l){return{target:f.constant(a),x:f.constant(b),
y:f.constant(c),stop:d,prevent:e,kill:g,raw:f.constant(l)}},EX=function(a,b){return function(c){if(a(c)){var d=q.fromDom(c.target),e=function(){c.stopPropagation()},g=function(){c.preventDefault()},l=f.compose(g,e),d=DX(d,c.clientX,c.clientY,e,g,l,c);b(d)}}},fz=function(a,b,c,d,e){c=EX(c,d);a.dom().addEventListener(b,c,e);return{unbind:f.curry(FX,a,b,c,e)}},FX=function(a,b,c,d){a.dom().removeEventListener(b,c,d)},Vp={bind:function(a,b,c,d){return fz(a,b,c,d,!1)},capture:function(a,b,c,d){return fz(a,
b,c,d,!0)}},gz=f.constant(!0),E={bind:function(a,b,c){return Vp.bind(a,b,gz,c)},capture:function(a,b,c){return Vp.capture(a,b,gz,c)}},GX=ez.install("ephox.henchman.features"),gO={download:function(a){return $d.nu(function(b){var c=function(){e.unbind();g.unbind()},d=q.fromTag("script");m.set(d,"src",a);m.set(d,"type","text/javascript");m.set(d,"async","async");m.set(d,"data-main",GX.ephemeral(function(a){b(H.value(a))}));var e=E.bind(d,"error",function(){c();b(H.error("Error loading external script tag "+
a))}),g=E.bind(d,"load",c);v.append(q.fromDom(document.head),d)})}},hz={addStylesheet:function(a,b){var c=b||q.fromDom(document),d=q.fromTag("link",c.dom());m.setAll(d,{rel:"stylesheet",type:"text/css",href:a});c=q.fromDom(c.dom().head);v.append(c,d);return d}},fO={download:function(a,b){return $d.nu(function(c){var d=function(b){k.each(g,function(a){a.unbind()});c(b.fold(function(b){return H.error(b+'Unable to download editor stylesheets from "'+a+'"')},H.value))},e=hz.addStylesheet(a),g=[E.bind(e,
"load",function(a){var c;a:{try{var e=a.target().dom().sheet.cssRules;c=R.isObject(e)&&0===e.length;break a}catch(g){}c=!1}c?d(H.error("")):b(d)}),E.bind(e,"error",f.curry(d,H.error("")))]})}},iz=Ku(),dl={addStylesheet:function(a,b){return iz.addStylesheet(a,b)},addScript:function(a,b){return iz.addScript(a,b)}},el=C.immutable("str","replace"),jz=function(a,b){return"2.4.0"!==a?h.some(b+(void 0===a?"":" version "+a)+" appears to be cached, as it does not match editor version 2.4.0."):h.none()},Je=
{aboutTags:function(a,b){var c=[el("@@FULL_VERSION@@","2.4.0.49"),el("@@CLIENT_VERSION@@","2.4.0.224")],d=a?[el("@@INTEGRATION_TYPE@@",b.type),el("@@INTEGRATION_VERSION@@",b.version)]:[];return k.flatten([c,d])},strictMatch:jz,weakMatch:function(a,b){return jz(a,b).map(function(a){return a+" This may produce unexpected results."})},resourceParam:f.constant("version=2.4.0"),fullParam:f.constant("version=2.4.0.49")},wa=function(a){var b=a;return{get:function(){return b},set:function(a){b=a},clone:function(){return wa(b)}}},
HX=function(a){a.each(function(a){Je.weakMatch(a.version,"Code view").each(function(a){console.warn(a)});kz.set(a.codemirror());lz.set(a.jsbeautify())});return a},kz=wa(null),lz=wa(null),Nj={use:{codemirror:kz.get,jsbeautify:lz.get},load:{codeview:function(a){a=a+"/codeview.js?"+Je.resourceParam();return dl.addScript(a,HX)}}},Uu={beautify:function(a){return Nj.use.jsbeautify().html(a,{indent_size:"2",indent_char:" ",max_preserve_newlines:"5",preserve_newlines:!0,keep_array_indentation:!1,break_chained_methods:!1,
indent_scripts:"normal",brace_style:"collapse",space_before_conditional:!0,unescape_strings:!1,jslint_happy:!1,end_with_newline:!1,wrap_line_length:0,indent_inner_html:!1,comma_first:!1,e4x:!1})}},IX=function(a){return a.map(function(a){return a.dom()}).getOr(null)},Oc={documentElement:function(a){if(a.content().isInline())throw Error("documentElement called on inline mode");return IX(a.content().doc())},insertHtmlAtCursor:function(a,b){ob(arguments,[V.object("polish"),V.string("html")]);a.content().focus();
a.insertHtmlAtCursor(b)},uploadImages:function(a,b){ob(arguments,[V.object("polish"),V.func("callback")]);return a.uploadImages(b)},get:function(a){var b=null!==Nj.use.jsbeautify()&&a.isPrettyPrinting();a=a.getContent();return b?Uu.beautify(a):a},set:function(a,b){ob(arguments,[V.object("polish"),V.string("html")]);a.setContent(b)},isDirty:function(a){ob(arguments,[V.object("polish")]);return a.isDirty()},setDirty:function(a,b){ob(arguments,[V.object("polish"),V.bool("state")]);a.setDirty(b)}},ma=
{generate:function(a){if(!R.isArray(a))throw Error("cases must be an array");if(0===a.length)throw Error("there must be at least one case");var b=[],c={};k.each(a,function(d,e){var g=W.keys(d);if(1!==g.length)throw Error("one and only one name per case");var l=g[0],f=d[l];if(void 0!==c[l])throw Error("duplicate key detected:"+l);if("cata"===l)throw Error("cannot have a case named cata (sorry)");if(!R.isArray(f))throw Error("case arguments must be an array");b.push(l);c[l]=function(){var c=arguments.length;
if(c!==f.length)throw Error("Wrong number of arguments to case "+l+". Expected "+f.length+" ("+f+"), got "+c);for(var d=Array(c),c=0;c<d.length;c++)d[c]=arguments[c];return{fold:function(){if(arguments.length!==a.length)throw Error("Wrong number of arguments to fold. Expected "+a.length+", got "+arguments.length);return arguments[e].apply(null,d)},match:function(a){var c=W.keys(a);if(b.length!==c.length)throw Error("Wrong number of arguments to match. Expected: "+b.join(",")+"\nActual: "+c.join(","));
if(!k.forall(b,function(a){return k.contains(c,a)}))throw Error("Not all branches were specified when using match. Specified: "+c.join(", ")+"\nRequired: "+b.join(", "));return a[l].apply(null,d)},log:function(a){console.log(a,{constructors:b,constructor:l,params:d})}}}});return c}},JX=ma.generate([{inputSelector:["selector","callback"]},{outputSelector:["selector","callback"]},{inputPredicate:["predicate","callback"]},{outputPredicate:["predicate","callback"]}]),xh=Z.merge(JX,{cata:function(a,b,
c,d,e){return a.fold(b,c,d,e)}}),Lu=function(a){return function(b){b=k.map(b,function(a){return a.dom()});a(b)}},iO=function(a){return function(b){return a(b.dom())}},Ua={zeroWidth:function(){return"\ufeff"}},Wp="\\w'\\-\\u0100-\\u017F\\u00C0-\\u00FF"+Ua.zeroWidth()+"\\u2018\\u2019",KX="[^"+Wp+"]",LX="["+Wp+"]",si={chars:f.constant(Wp),wordbreak:f.constant(KX),wordchar:f.constant(LX)},fl={token:function(a){return Zn(a,f.constant(0),f.constant(0),h.none())},word:function(a){a="((?:^'?)|(?:"+si.wordbreak()+
"+'?))"+a+"((?:'?$)|(?:'?"+si.wordbreak()+"+))";return Zn(a,function(a){return 1<a.length?a[1].length:0},function(a){return 2<a.length?a[2].length:0},h.none())}},Xp=function(a){return a.replace(/[-\[\]{}()*+?.,\\\^$|#\s]/g,"\\$&")},Yp={sanitise:Xp,word:function(a){a=Xp(a);return fl.word(a)},token:function(a){a=Xp(a);return fl.token(a)}},db={safeword:function(a){return Yp.word(a)},safetoken:function(a){return Yp.token(a)},custom:function(a,b,c,d){return Zn(a,b,c,d)},unsafeword:function(a){return fl.word(a)},
unsafetoken:function(a){return fl.token(a)},sanitise:function(a){return Yp.sanitise(a)},chars:function(){return si.chars()},wordbreak:function(){return si.wordbreak()},wordchar:function(){return si.wordchar()}},nz=function(a,b){return mz(a,b,{validate:R.isFunction,label:"function"})},mz=function(a,b,c){if(0===b.length)throw Error("You must specify at least one required field.");Bd.validateStrArr("required",b);Bd.checkDupes(b);return function(d){var e=W.keys(d);k.forall(b,function(a){return k.contains(e,
a)})||Bd.reqMessage(b,e);a(b,e);var g=k.filter(b,function(a){return!c.validate(d[a],a)});0<g.length&&Bd.invalidTypeMessage(g,c.label);return d}},oz=f.noop,Gb={exactly:f.curry(nz,function(a,b){var c=k.filter(b,function(b){return!k.contains(a,b)});0<c.length&&Bd.unsuppMessage(c)}),ensure:f.curry(nz,oz),ensureWith:f.curry(mz,oz)},MX=Gb.ensure(["elements","cursor","dictation","notify"]),NX=Gb.ensure(["elements","cursor","dictation","range","notify"]),hd={species:MX,applicator:NX},Od={fromHtml:function(a,
b){var c=(b||document).createElement("div");c.innerHTML=a;return y.children(q.fromDom(c))},fromTags:function(a,b){return k.map(a,function(a){return q.fromTag(a,b)})},fromText:function(a,b){return k.map(a,function(a){return q.fromText(a,b)})},fromDom:function(a){return k.map(a,q.fromDom)}},Pu={addSimpleMacro:function(a,b,c,d){ob(arguments,[V.object("polish"),V.string("startString"),V.string("endString"),V.func("callback")]);var e=Mu(b,c,d);return a.macros().customMacros().add(e)},removeMacro:function(a,
b){a.macros().customMacros().remove(b)}},pz=["error","info","warning"],mO={message:function(a,b,c,d){if(!k.contains(pz,b))throw'Message level: "'+b+'" is not supported. Supported levels are: ['+pz.join(", ")+"].";ob(arguments,[V.object("polish"),V.string("level"),V.number("timeout"),V.string("msg")]);var e=a.ui().showMessage(b,c,d);return{hide:function(){e()}}}},OX=f.constant("pencil"),PX=f.constant("code"),QX=f.constant("spin"),RX=f.constant("none"),Ab={pencil:OX,code:PX,wait:QX,none:RX},Ou={get:function(a){return a.getMode()===
Ab.pencil()?"design":"code"},set:function(a,b){ob(arguments,[V.object("polish"),V.string("mode")]);var c=b.toLowerCase();if("code"!==c&&"design"!==c)throw Error('Editor mode "'+b+'" not supported. Must be either design or code.');a.setMode("design"===c?Ab.pencil():Ab.code())}},kO="body p div article aside figcaption figure footer header nav section ol ul li table thead tbody tfoot caption tr td th h1 h2 h3 h4 h5 h6 blockquote pre address".split(" "),Zp=function(a,b){var c=[];k.each(y.children(a),
function(a){b(a)&&(c=c.concat([a]));c=c.concat(Zp(a,b))});return c},pc={all:function(a){return Zp(ca.body(),a)},ancestors:function(a,b,c){return k.filter(y.parents(a,c),b)},siblings:function(a,b){return k.filter(y.siblings(a),b)},children:function(a,b){return k.filter(y.children(a),b)},descendants:Zp},X={all:function(a){return ra.all(a)},ancestors:function(a,b,c){return pc.ancestors(a,function(a){return ra.is(a,b)},c)},siblings:function(a,b){return pc.siblings(a,function(a){return ra.is(a,b)})},children:function(a,
b){return pc.children(a,function(a){return ra.is(a,b)})},descendants:function(a,b){return ra.all(b,a)}},qz=function(a,b,c){return sa.ancestor(a,function(a){return ra.is(a,b)},c)},J={first:function(a){return ra.one(a)},ancestor:qz,sibling:function(a,b){return sa.sibling(a,function(a){return ra.is(a,b)})},child:function(a,b){return sa.child(a,function(a){return ra.is(a,b)})},descendant:function(a,b){return ra.one(b,a)},closest:function(a,b,c){return Ju(ra.is,qz,a,b,c)}},$p=Nu(A.isText,"text"),Da={get:function(a){return $p.get(a)},
getOption:function(a){return $p.getOption(a)},set:function(a,b){$p.set(a,b)}},SX=C.immutable("element","offset"),TX=C.immutable("element","deltaOffset"),UX=C.immutable("element","start","finish"),VX=C.immutable("begin","end"),WX=C.immutable("element","text"),S={point:SX,delta:TX,range:UX,points:VX,text:WX},gl=f.constant(!1),XX=f.constant(!0),aq=function(a){return{isBoundary:function(){return a(XX,gl,gl)},fold:a,toText:function(){return a(h.none,h.none,function(a,c){return h.some(a)})},is:function(b){return a(gl,
gl,function(a,d){return d.eq(a,b)})},len:function(){return a(f.constant(0),f.constant(1),function(a,c){return c.property().getText(a).length})}}},hl={text:function(a,b){return aq(function(c,d,e){return e(a,b)})},boundary:function(a,b){return aq(function(c,d,e){return c(a,b)})},empty:function(a,b){return aq(function(c,d,e){return d(a,b)})},cata:function(a,b,c,d){return a.fold(b,c,d)}},YX={boundAt:function(a,b,c,d){b=k.findIndex(a,f.curry(d,b)).getOr(0);c=k.findIndex(a,f.curry(d,c)).map(function(a){return a+
1}).getOr(a.length);return a.slice(b,c)}},ZX={sliceby:function(a,b){var c=k.findIndex(a,b).getOr(-1);return a.slice(0,c)}},bq=ma.generate([{include:["item"]},{excludeWith:["item"]},{excludeWithout:["item"]}]),Bg={include:bq.include,excludeWith:bq.excludeWith,excludeWithout:bq.excludeWithout,cata:function(a,b,c,d){return a.fold(b,c,d)}},rz=function(a,b){var c=[],d=[];k.each(a,function(a){var g=b(a);Bg.cata(g,function(){d.push(a)},function(){0<d.length&&c.push(d);c.push([a]);d=[]},function(){0<d.length&&
c.push(d);d=[]})});0<d.length&&c.push(d);return c},sz={splitby:function(a,b){return rz(a,function(a){return b(a)?Bg.excludeWithout(a):Bg.include(a)})},splitbyAdv:rz},tz={splitby:function(a,b){return sz.splitby(a,b)},splitbyAdv:function(a,b){return sz.splitbyAdv(a,b)},sliceby:function(a,b){return ZX.sliceby(a,b)},boundAt:function(a,b,c,d){return YX.boundAt(a,b,c,d)}},il={count:function(a){return k.foldr(a,function(a,c){return c.len()+a},0)},dropUntil:function(a,b){return tz.sliceby(a,function(a){return a.is(b)})},
gen:function(a,b){return a.fold(h.none,function(a){return h.some(S.range(a,b,b+1))},function(c){return h.some(S.range(c,b,b+a.len()))})},justText:function(a){return k.bind(a,function(a){return a.fold(f.constant([]),f.constant([]),function(a){return[a]})})}},jl=function(a,b,c){if(a.property().isText(b))return[hl.text(b,a)];if(a.property().isEmptyTag(b))return[hl.empty(b,a)];if(a.property().isElement(b)){var d=a.property().children(b),e=a.property().isBoundary(b)?[hl.boundary(b,a)]:[];b=void 0!==c&&
c(b)?[]:k.bind(d,function(b){return jl(a,b,c)});return e.concat(b).concat(e)}return[]},uz=function(a,b,c,d,e){a=jl(a,d,e);b=il.dropUntil(a,b);b=il.count(b);return S.point(d,b+c)},Cg={typed:jl,items:function(a,b,c){a=jl(a,b,c);var d=function(a,b){return a};return k.map(a,function(a){return a.fold(d,d,d)})},extractTo:function(a,b,c,d,e){return a.up().predicate(b,d).fold(function(){return S.point(b,c)},function(d){return uz(a,b,c,d,e)})},extract:function(a,b,c,d){return a.property().parent(b).fold(function(){return S.point(b,
c)},function(e){return uz(a,b,c,e,d)})}},$X=function(a,b){return"img"===b.property().name(a)?" ":"\n"},aY={from:function(a,b,c){b=Cg.typed(a,b,c);return k.map(b,function(b){return b.fold(f.constant("\n"),$X,a.property().getText)}).join("")}},bY={make:function(a,b,c){return k.foldl(a,function(a,c){return b(c,a.len).fold(f.constant(a),function(b){return{len:b.finish(),list:a.list.concat([b])}})},{len:void 0!==c?c:0,list:[]}).list}},vz=function(a,b){return b>=a.start()&&b<=a.finish()},wz=function(a,
b){return k.findIndex(a,function(a){return a.start()===b})},cY=function(a,b){return a[a.length-1]&&a[a.length-1].finish()===b?a.length+1:-1},kl={get:function(a,b){return k.find(a,function(a){return vz(a,b)})},find:function(a,b){return k.find(a,b)},inUnit:vz,sublist:function(a,b,c){b=wz(a,b);var d=wz(a,c);return b.bind(function(b){var g=d.getOr(cY(a,c));return-1<g?h.some(a.slice(b,g)):h.none()}).getOr([])}},xz={translate:function(a,b){return k.map(a,function(a){return Z.merge(a,{start:f.constant(a.start()+
b),finish:f.constant(a.finish()+b)})})}},dY=function(a,b,c){b=c(a,b);return xz.translate(b,a.start())},eY={splits:function(a,b,c){return 0===b.length?a:k.bind(a,function(a){var e=k.bind(b,function(b){return kl.inUnit(a,b)?[b-a.start()]:[]});return 0<e.length?dY(a,e,c):[a]})}},ad={generate:function(a,b,c){return bY.make(a,b,c)},get:function(a,b){return kl.get(a,b)},find:function(a,b){return kl.find(a,b)},splits:function(a,b,c){return eY.splits(a,b,c)},translate:function(a,b){return xz.translate(a,
b)},sublist:function(a,b,c){return kl.sublist(a,b,c)}},fY={find:function(a,b,c,d){a=Cg.typed(a,b,d);a=ad.generate(a,il.gen);return ad.get(a,c).map(function(a){return S.point(a.element(),c-a.start())})}},Ue={extract:function(a,b,c,d){return Cg.extract(a,b,c,d)},extractTo:function(a,b,c,d,e){return Cg.extractTo(a,b,c,d,e)},all:function(a,b,c){return Cg.items(a,b,c)},from:function(a,b,c){return Cg.typed(a,b,c)},find:function(a,b,c,d){return fY.find(a,b,c,d)},toText:function(a,b,c){return aY.from(a,b,
c)}},Dg=Sa(),yg={extract:function(a,b,c){return Ue.extract(Dg,a,b,c)},extractTo:function(a,b,c,d){return Ue.extractTo(Dg,a,b,c,d)},all:function(a,b){return Ue.all(Dg,a,b)},from:function(a,b){return Ue.from(Dg,a,b)},find:function(a,b,c){return Ue.find(Dg,a,b,c)},toText:function(a,b){return Ue.toText(Dg,a,b)}},yz=function(a){var b=Object.create(null);W.each(a,function(a,d){b[d]=a});return b},gY=yz({img:!0,br:!0,hr:!0,input:!0}),hY=yz({script:!0,style:!0,noscript:!0}),iY=/[<>&\u00A0]/g,jY={"<":"&lt;",
">":"&gt;","&":"&amp;","\u00a0":"&nbsp;"},kY=/["<>&]/g,lY={'"':"&quot;","<":"&lt;",">":"&gt;","&":"&amp;"},zz=function(a,b,c){return a.replace(b,function(a){return c[a]||a})},Az=function(a,b){for(var c="<"+a,d="",e=0;e<b.length;e++){var g=b[e];g.specified&&"_moz_dirty"!==g.name&&(d+=" "+g.name+'="'+zz(g.value,kY,lY)+'"')}return c+d},Cz=function(a){var b="";k.each(a,function(a){b+=Bz(a)});return b},mY=function(a){var b="";k.each(a,function(a){b+=a.textContent});return b},Bz=function(a){var b=a.nodeType;
if(1===b){var b=a.nodeName.toLowerCase(),c=a.childNodes;a=a.attributes;0===c.length&&gY[b]?b=Az(b,a)+" />":(c=hY[b]?mY(c):Cz(c),b=Az(b,a)+">"+c+"</"+b+">");return b}if(3===b)return zz(a.textContent,iY,jY);if(8==b)return"\x3c!--"+a.data+"--\x3e";throw"unsupported node type "+a.nodeType;},ll={writeNode:function(a){if(!R.isObject(a))throw"sugarNode parameter must be an object";return Bz(a.dom())},writeChildren:function(a){if(!R.isObject(a))throw"sugarNode parameter must be an object";return Cz(a.dom().childNodes)}},
Dz=function(a){var b=RegExp(Ua.zeroWidth(),"g");return a.replace(b,"")},Mh={output:function(a,b){a.filterDom(a.output(),b);var c=ll.writeChildren(b);return Dz(c)},zeroWidth:Dz},ml={toText:function(a){a=k.map(a,function(a){return yg.toText(a,f.constant(!1))}).join("");return Mh.zeroWidth(a.trim())},toHtml:function(a){a=k.map(a,ll.writeNode).join("");return Mh.zeroWidth(a)}},Ez=function(a){return a.dom().innerHTML},ua={get:Ez,set:function(a,b){var c=y.owner(a).dom(),d=q.fromDom(c.createDocumentFragment()),
c=Od.fromHtml(b,c);K.append(d,c);B.empty(a);v.append(a,d)},getOuter:function(a){var b=q.fromTag("div");a=q.fromDom(a.dom().cloneNode(!0));v.append(b,a);return Ez(b)}},pb={get:function(a){return a.dom().textContent},set:function(a,b){a.dom().textContent=b}},Fz=function(a,b){return Mj.makeReadOnly({getAttributes:function(){return Mj.makeReadOnly(a.getFilteredAttributes(b))},setAttributes:function(c){ob(arguments,[V.object("attributes")]);W.each(c,function(a,b){if(!R.isString(a))throw Error("attribute key "+
b+" does not have a string value: "+a);});m.setAll(b,c);a.domChanged();a.undoChanged()},getText:function(){return ml.toText([b])},setText:function(a){ob(arguments,[V.string("text")]);pb.set(b,a)},getHtml:function(){return a.getFilteredContents(b)},setHtml:function(a){ob(arguments,[V.string("html")]);ua.set(b,a)},replaceElement:function(c){ob(arguments,[V.string("html")]);var d=Od.fromHtml(c);if(1!==d.length)throw Error("replacement html must be a single node");d=d[0];a.replaceElement(b,d);return Fz(a,
d)}})},$n={getText:function(a){return a.content().getSelectedText().getOr("")},getHtml:function(a){return a.content().getSelectedHtml().getOr("")},findTagAtCursor:function(a,b){var c=a.content();return c.findTagAtCursor(b).map(function(a){return Fz(c,a)}).getOr(null)}},Gz={},Hz=function(a){return h.from(Gz[a])},ao={get:Hz,register:function(a,b){return Gz[a]=b},hookup:function(a,b){Hz(b).fold(function(){console&&console.error&&console.error('textbox.io Could not find plugin "'+b+'". Have you declared it?')},
function(b){a.events.loaded.addListener(function(){b.initialize(a)})})}},Iz={attached:function(a,b){var c=b||q.fromDom(document.documentElement);return sa.ancestor(a,f.curry(L.eq,c)).isSome()},windowOf:function(a){var b=a.dom();return b===b.window?a:A.isDocument(a)?b.defaultView||b.parentWindow:null}},Ba=function(a,b){return{left:f.constant(a),top:f.constant(b),translate:function(c,d){return Ba(a+c,b+d)}}},nl=function(a,b){return void 0!==a?a:void 0!==b?b:0},Jz=function(a){var b=a.dom(),c=b.ownerDocument,
d=c.body,c=q.fromDom(c.documentElement);if(d===b)return Ba(d.offsetLeft,d.offsetTop);if(!Iz.attached(a,c))return Ba(0,0);a=b.getBoundingClientRect();return Ba(a.left,a.top)},Na={absolute:function(a){var b=a.dom().ownerDocument,c=b.body,d=Iz.windowOf(q.fromDom(b)),e=b.documentElement,b=nl(d.pageYOffset,e.scrollTop),d=nl(d.pageXOffset,e.scrollLeft),g=nl(e.clientTop,c.clientTop),c=nl(e.clientLeft,c.clientLeft);return Jz(a).translate(d-c,b-g)},relative:function(a){a=a.dom();return Ba(a.offsetLeft,a.offsetTop)},
viewport:Jz},nY=Y.detect().browser.isSafari(),ol=function(a){a=void 0!==a?a.dom():document;return Ba(a.body.scrollLeft||a.documentElement.scrollLeft,a.body.scrollTop||a.documentElement.scrollTop)},pl=function(a,b,c){(void 0!==c?c.dom():document).defaultView.scrollTo(a,b)},cq=function(a,b){nY&&R.isFunction(a.dom().scrollIntoViewIfNeeded)?a.dom().scrollIntoViewIfNeeded(!1):a.dom().scrollIntoView(b)},Ea={get:ol,to:pl,by:function(a,b,c){(void 0!==c?c.dom():document).defaultView.scrollBy(a,b)},preserve:function(a,
b){var c=ol(a);b();var d=ol(a);c.top()===d.top()&&c.left()===d.left()||pl(c.left(),c.top(),a)},capture:function(a){var b=h.none(),c=function(){b=h.some(ol(a))};c();return{save:c,restore:function(){b.each(function(b){pl(b.left(),b.top(),a)})}}},intoView:cq,intoViewIfNeeded:function(a,b){var c=b.dom().getBoundingClientRect(),d=a.dom().getBoundingClientRect();d.top<c.top?cq(a,!0):d.bottom>c.bottom&&cq(a,!1)},setToElement:function(a,b){var c=Na.absolute(b),d=q.fromDom(a.document);pl(c.left(),c.top(),
d)},scrollBarWidth:function(){var a=q.fromHtml('<div style="width: 100px; height: 100px; overflow: scroll; position: absolute; top: -9999px;"></div>');v.after(ca.body(),a);var b=a.dom().offsetWidth-a.dom().clientWidth;B.remove(a);return b}},Kz={withOptions:function(a,b){return m.has(a,"tabindex")?Z.deepMerge(b,{content:{tabindex:m.get(a,"tabindex")}}):b}},Lz=Y.detect(),oY=Lz.browser.isIE()&&10===Lz.browser.version.major,dq=function(a,b){var c=t.getRaw(a,b).getOrThunk(function(){return t.get(a,b)});
return oY&&"0px"===c?"auto":c},ti=function(a,b,c){a=dq(a,b);return void 0===a?c:h.some(a)},pY=function(a){return{width:ti(a,"width",h.none()),"max-width":ti(a,"max-width",h.some("none")),"min-width":ti(a,"min-width",h.some("none")),"max-height":ti(a,"max-height",h.some("none")),"min-height":ti(a,"min-height",h.some("1em"))}},ql=function(a,b,c){var d=t.getRaw(b,"display"),e=dq(b,"height");t.set(b,"display","none");var g=dq(b,"height"),l=pY(b);d.fold(function(){t.remove(b,"display")},function(a){t.set(b,
"display",a)});!0===a?t.set(c,"height","auto"):(a="auto"===g?e:g,void 0!==a&&t.set(c,"height",a));W.each(l,function(a,b){a.each(function(a){t.set(c,b,a)})})},qY=f.curry(ql,!1),rY=f.curry(ql,!0),sY=function(a){ql(!1,a,a)},tY=function(a){ql(!0,a,a)},Mz=function(a){return a.replace(/\./g,"-")},lb={create:function(a){var b=Mz(a);return{resolve:function(a){a=a.split(" ");return k.map(a,function(a){return b+"-"+a}).join(" ")}}},cssNamespace:Mz,cssClass:function(a,b){return a+"-"+b}},Nz=lb.create("ephox-elder"),
rl={resolve:Nz.resolve,scrollable:f.curry(Nz.resolve,"scrollable")},eq=function(a,b){var c=m.get(a,b);return void 0===c||""===c?[]:c.split(" ")},Ud={read:eq,add:function(a,b,c){c=eq(a,b).concat([c]);m.set(a,b,c.join(" "))},remove:function(a,b,c){var d=k.filter(eq(a,b),function(a){return a!==c});0<d.length?m.set(a,b,d.join(" ")):m.remove(a,b)}},Oz=function(a){return Ud.read(a,"class")},Pz=function(a,b){return Ud.add(a,"class",b)},Qz=function(a,b){return Ud.remove(a,"class",b)},uc={get:Oz,add:Pz,remove:Qz,
toggle:function(a,b){k.contains(Oz(a),b)?Qz(a,b):Pz(a,b)},supports:function(a){return void 0!==a.dom().classList}},Rz=function(a,b){return uc.supports(a)&&a.dom().classList.contains(b)},n={add:function(a,b){uc.supports(a)?a.dom().classList.add(b):uc.add(a,b)},remove:function(a,b){uc.supports(a)?a.dom().classList.remove(b):uc.remove(a,b);0===(uc.supports(a)?a.dom().classList:uc.get(a)).length&&m.remove(a,"class")},toggle:function(a,b){return uc.supports(a)?a.dom().classList.toggle(b):uc.toggle(a,b)},
toggler:function(a,b){var c=uc.supports(a),d=a.dom().classList;return bo(function(){c?d.remove(b):uc.remove(a,b)},function(){c?d.add(b):uc.add(a,b)},Rz(a,b))},has:Rz},ea={add:function(a,b){k.each(b,function(b){n.add(a,b)})},remove:function(a,b){k.each(b,function(b){n.remove(a,b)})},toggle:function(a,b){k.each(b,function(b){n.toggle(a,b)})},hasAll:function(a,b){return k.forall(b,function(b){return n.has(a,b)})},hasAny:function(a,b){return k.exists(b,function(b){return n.has(a,b)})},get:function(a){if(uc.supports(a)){a=
a.dom().classList;for(var b=Array(a.length),c=0;c<a.length;c++)b[c]=a.item(c);a=b}else a=uc.get(a);return a}},me={addClass:function(a){return function(b){n.add(b,a)}},removeClass:function(a){return function(b){n.remove(b,a)}},removeClasses:function(a){return function(b){ea.remove(b,a)}},hasClass:function(a){return function(b){return n.has(b,a)}}},sl=h.none(),Sz=rl.resolve("modal-scroll-lock"),Tz=function(a,b){k.each(b,function(b){a(b)})},uY=f.curry(Tz,me.addClass(Sz)),vY=f.curry(Tz,me.removeClass(Sz)),
zc={adaptable:function(a,b){var c=null,d=null;return{cancel:function(){null!==c&&(clearTimeout(c),d=c=null)},throttle:function(){d=arguments;null===c&&(c=setTimeout(function(){a.apply(null,d);d=c=null},b))}}},first:function(a,b){var c=null;return{cancel:function(){null!==c&&(clearTimeout(c),c=null)},throttle:function(){var d=arguments;null===c&&(c=setTimeout(function(){a.apply(null,d);d=c=null},b))}}},last:function(a,b){var c=null;return{cancel:function(){null!==c&&(clearTimeout(c),c=null)},throttle:function(){var d=
arguments;null!==c&&clearTimeout(c);c=setTimeout(function(){a.apply(null,d);d=c=null},b)}}}},D={create:function(a){var b=W.map(a,function(a){return{bind:a.bind,unbind:a.unbind}});a=W.map(a,function(a){return a.trigger});return{registry:b,trigger:a}}},gc={register:function(a){n.add(a,rl.scrollable())},deregister:function(a){n.remove(a,rl.scrollable())},scrollable:rl.scrollable,preserve:Ea.preserve,capture:Ea.capture,block:function(){if(sl.isNone()){var a=Ea.get(),b=E.bind(q.fromDom(window),"scroll",
function(b){b=Ea.get();b.left()===a.left()&&b.top()===a.top()||window.scrollTo(a.left(),a.top())});sl=h.some(b)}},unblock:function(){sl.each(function(a){a.unbind();sl=h.none()})},lock:uY,unlock:vY,event:function(a){var b=!1,c=D.create({started:s([]),stopped:s([])}),d=zc.last(function(){!0===b&&(b=!1,c.trigger.stopped())},250),e=E.bind(a,"scroll",function(){!1===b&&c.trigger.started();b=!0;d.throttle()});return{events:c.registry,destroy:function(){e.unbind()},isScrolling:function(){return b}}}},Qe=
{isEmpty:function(a){return null===a||void 0===a||""===a||R.isArray(a)&&0===a.length}},tl=Ku(),fq={preload:function(){tl.preload()},addStylesheet:function(a,b){return tl.addStylesheet(a,b)},addScript:function(a,b){return tl.addScript(a,b)},waitForLoad:function(){return tl.waitForLoad()}},Uz=function(){return{zh_hk:"zh_tw",iw:"he",no:"nb",nn:"nb",en:"en",ar:"ar",ca:"ca",cs:"cs",da:"da",de:"de",el:"el",es:"es",fa:"fa",fi:"fi",fr:"fr",he:"he",hr:"hr",hu:"hu",it:"it",ja:"ja",kk:"kk",ko:"ko",nl:"nl",nb:"nb",
pl:"pl",pt:"pt",pt_pt:"pt_pt",ro:"ro",ru:"ru",sl:"sl",sk:"sk",sv:"sv",th:"th",tr:"tr",uk:"uk",zh:"zh",zh_tw:"zh_tw"}},Vz=function(a){a=a.toLowerCase().replace("-","_");var b=a.indexOf(";");a=-1<b?a.substring(0,b):a;a=aa.trim(a);b=a.indexOf("_");b=-1<b?a.substring(0,b):a;return{full:a,languageCode:b}},Wz=function(a,b){var c=a.split(","),c=k.map(c,function(a){a=Vz(a);return b[a.full]||b[a.languageCode]||""});return k.find(c,function(a){return""!==a})},gq={extract:Vz,determine:function(a){var b=Uz();
return Wz(a,b)},determineFrom:Wz,localeList:Uz},ab={cat:function(a){for(var b=[],c=function(a){b.push(a)},d=0;d<a.length;d++)a[d].each(c);return b},findMap:function(a,b){for(var c=0;c<a.length;c++){var d=b(a[c],c);if(d.isSome())return d}return h.none()},liftN:function(a,b){for(var c=[],d=0;d<a.length;d++){var e=a[d];if(e.isSome())c.push(e.getOrDie());else return h.none()}return h.some(b.apply(null,c))}},Xz=function(){var a=ne.resolve("navigator")||{};return a.userLanguage||a.language||"en"}(),Yz=
h.none(),fe={determine:function(a){var b=void 0!==a?[a,Xz]:[Xz];return ab.findMap(b,gq.determine).fold(function(){console.warn("WARNING: locales not supported: "+b.join(", "));return"en"},f.identity)},get:function(){return Yz.getOrDie("locale used before it has been set")},set:function(a){Yz=h.from(a)}},wY={font:{arial:"Arial",courier:"Courier New",comicsans:"Comic Sans MS",consolas:"Consolas",georgia:"Georgia",impact:"Impact",tahoma:"Tahoma",timesnr:"Times New Roman",trebuchet:"Trebuchet MS",verdana:"Verdana"},
error:{duplicate:{code:"That snippet code is already taken.",uuid:"That snippet uuid is already taken."},unknown:{uuid:"The snippet uuid cannot be found"},code:{duplicate:"That snippet code is already taken."}},cog:{login:"Log in",logout:"Log out","not-logged-in":"Not logged in",user:"User: {0}"},banner:{"logged-in":"Logged in as: {0}"}},Zz={},$z="",hq={},aA=function(a){return ne.resolve(a,hq)||ne.resolve(a,Zz)||ne.resolve(a,wY)},If={loadEnglish:function(a){Zz=a},load:function(a,b){$z=a;hq=b;fe.set(a)},
isLoaded:function(a){return $z===a&&0<W.keys(hq).length},get:function(a){var b=aA(a);if(R.isString(b))return b;throw Error("key "+a+" does not exist");},has:function(a){if(!R.isString(a))throw a.fold?console.error("object details were",a.fold(f.identity,f.identity,f.identity)):console.error("object key was",a),"key was an object";return R.isString(aA(a))}},iq=h.none(),jq={set:function(a){iq.fold(function(){iq=h.some(a)},function(b){b!==a&&console.error('Only one base URL supported per page. Requested value "'+
a+'", already using "'+b+'"')})},get:function(){return iq.getOrDie("Must begin loading with a base URL")}},xY={execute:function(a){if("complete"===document.readyState||"interactive"===document.readyState)a();else var b=E.bind(q.fromDom(document),"DOMContentLoaded",function(){a();b.unbind()})}},ul=Qu("width",function(a){return a.dom().offsetWidth}),qa={set:function(a,b){ul.set(a,b)},get:function(a){return ul.get(a)},getOuter:function(a){return ul.getOuter(a)},setMax:function(a,b){var c=ul.max(a,b,
"margin-left border-left-width padding-left padding-right border-right-width margin-right".split(" "));t.set(a,"max-width",c+"px")}},bA=function(){var a=q.fromTag("div");n.add(a,"ephox-textbox-font");var b=q.fromTag("span");n.add(b,"ephox-chameleon-icon");v.append(a,b);v.append(ca.body(),a);var c=qa.get(b),b=t.get(b,"font-family");B.remove(a);return 0<c&&void 0!==b&&aa.contains(b,"Textbox Icons")},yY=function(a){var b=0,c=setInterval(function(){b+=250;bA()?(clearInterval(c),a(!0)):2E3<b&&(clearInterval(c),
a(!1))},250)},zY={wait:hb.cached(function(){return $d.nu(function(a){xY.execute(function(){bA()?a(!0):yY(a)})})})},AY=function(a){zY.wait().get(function(b){a(b?H.value(!0):H.error("Stylesheet loaded, but did not contain Textbox.io styles"))})},cA=function(a,b){var c=a+"/i18n/tbio_"+b+".js?"+Je.resourceParam();return fq.addScript(c,function(a){return a.bind(function(a){return Je.strictMatch(a.version,"Language strings").fold(function(){return H.value(a.strings())},H.error)})})},dA=function(a){var b=
"/textboxio.css?"+Je.fullParam();fq.addStylesheet(a+b,AY)},ki={load:function(a){jq.set(a);a=jq.get();dA(a+"/css");cA(a,"en").get(function(a){a.each(If.loadEnglish)});return a},devLoad:dA,setLanguage:function(a){var b=fe.determine(a);If.isLoaded(b)||(a=jq.get(),cA(a,b).get(function(a){a.each(function(a){If.load(b,a)})}))},waitForLoad:function(){return fq.waitForLoad()},getFailures:function(a){return k.bind(a,function(a){return a.fold(function(a){return[a]},f.constant([]))})}},P={span:function(a){a=
q.fromTag("span",a);m.set(a,"role","presentation");return a},div:function(a){a=q.fromTag("div",a);m.set(a,"role","presentation");return a},tag:function(a,b){var c=q.fromTag(a,b);m.set(c,"role","presentation");return c}},x={resolve:lb.create("ephox-polish").resolve},eb={any:function(a){return J.first(a).isSome()},ancestor:function(a,b,c){return J.ancestor(a,b,c).isSome()},sibling:function(a,b){return J.sibling(a,b).isSome()},child:function(a,b){return J.child(a,b).isSome()},descendant:function(a,b){return J.descendant(a,
b).isSome()},closest:function(a,b,c){return J.closest(a,b,c).isSome()}},kq=x.resolve("highcontrast-mode"),BY=function(a){var b=q.fromTag("div");n.add(b,x.resolve("highcontrast-detect"));v.append(a,b);t.reflow(b);(!0===("none"===t.get(b,"background-image")?!0:!1)?n.add:n.remove)(a,kq);B.remove(b)},CY=function(a){var b=q.fromDom(document.documentElement);(!0===(m.has(b,"hc")&&m.has(b,"hcx"))?n.add:n.remove)(a,kq)},DY=f.noop,Ah={detect:function(a){var b=Y.detect();b.os.isWindows()&&(b.browser.isIE()||
b.browser.isEdge()||b.browser.isFirefox()?BY:b.browser.isChrome()?CY:DY)(a)},isHighContrast:function(a){return eb.ancestor(a.uiContainer(),"."+kq)}},Vu=C.immutableBag(["width","docWidth","height","docHeight"],[]),eA=ma.generate([{unchanged:["html"]},{changed:["html"]}]),xe={unchanged:eA.unchanged,changed:eA.changed,cata:function(a,b,c){return a.fold(b,c)}},fA=function(){return{validate:function(a,b){return R.isFunction(a)||"events"===b&&R.isObject(a)},label:"Function (or events)"}}(),gA="activateInline communication connect content deactivateInline element events filter focus getContent getMode insertHtmlAtCursor isDirty isPrettyPrinting keyboard macros operations setContent setDirty smartview state ui undo".split(" "),
EY=Gb.ensureWith(gA.concat([]),fA),FY=Gb.ensureWith(gA.concat("blur destroy fromTransfer fullscreen uploader toTransfer viewport".split(" ")),fA),zh={external:EY,internal:FY},GY={create:function(){var a=!1;return{isBlocked:function(){return a},block:function(){a=!0},unblock:function(){a=!1}}}},HY={create:function(a,b){return{control:a,instance:b}}},Qc={tap:function(a){var b=GY.create();return HY.create(b,function(){b.isBlocked()||a.apply(null,arguments)})}},hA=0,La={generate:function(a){var b=(new Date).getTime(),
c=Math.floor(1E9*Math.random());hA++;return a+"_"+c+hA+String(b)}},iA={isFunctionOrEvents:function(a){var b=function(b){return!0===k.forall(a,function(a){return void 0!==b[a]&&void 0!==b[a].bind&&R.isFunction(b[a].bind)})?!0:!1};return{validate:function(a,d){return R.isFunction(a)||"events"===d&&R.isObject(a)&&b(a)},label:"Function (or events)"}}},eo=Gb.ensureWith("events init ui check scanView situation hasDropdown hasAutocorrect handle assign lock destroy state lookup name".split(" "),iA.isFunctionOrEvents(["analytics"])),
nf={noSituation:function(){return{primary:f.constant([]),secondary:f.constant([]),submenus:f.constant({})}},basic:function(){return{destroy:f.noop,ui:h.none,state:f.constant({}),events:Pc(["analytics"])}}},jA=function(a,b){var c={};W.each(a,function(a,b){var d=b.split(".");if(1>=d.length)c[b]={finalvalue:a};else{var f=c;ne.namespace(b,c);for(var h=0;h<d.length-1;h++)f=f[d[h]];f[d[d.length-1]]={finalvalue:a}}});var d=function(a){var c={},l=W.keys(a);k.each(l,function(l){c[l]=void 0!==a[l].finalvalue?
b(a[l].finalvalue):f.constant(d(a[l]))});return c};return d(c)},$a={ui:function(a,b){var c=jA(a,void 0!==b?b:f.constant);return f.constant(h.some(c))},actions:function(a){a=jA(a,f.identity);return f.constant(a)}},Aa={basic:nf.basic,draganddrop:function(){return{events:Pc(["insertAssets","insertHtml"]),destroy:f.noop}},communication:function(){return{headers:f.constant({}),destroy:f.noop}},fullscreen:function(){return{ui:h.none,actions:$a.actions({maximize:f.noop,restore:f.noop}),select:f.noop,deselect:f.noop,
destroy:f.noop,events:Pc(["analytics","maximize","restore"])}},dialog:function(){return{destroy:f.noop,ui:h.none,events:Pc(["analytics"]),actions:$a.actions({show:f.noop})}},spelling:function(){return qO()},editorcontext:function(){return{events:Pc(["blockChanged","enterDown","enterUp"]),destroy:f.noop}},highlighter:function(){return{destroy:f.noop,run:f.noop}},linkvalidator:function(){return{destroy:f.noop,refresh:f.noop,ignore:f.noop,state:f.constant({ignore:[],cache:{}})}},macros:function(){return{events:Pc(["analytics"]),
state:f.constant({}),customMacros:f.constant(co({})),destroy:f.noop}},table:function(){return{ui:h.none,state:f.constant({}),events:Pc(["analytics"]),situation:nf.noSituation,handle:f.noop,assign:f.noop,destroy:f.noop,name:f.constant("table-disabled")}},image:function(){return{events:Pc(["changed","contextmenu"]),situation:nf.noSituation,handle:f.noop,assign:f.noop,destroy:f.noop,name:f.constant("image-disabled")}},imageEdit:function(){var a=function(a,b){return za.pure(k.map(b,f.compose(H.value,
a)))},b=f.curry(a,f.identity),a=f.curry(a,function(a){return a.asset()});return{situation:nf.noSituation,handle:f.noop,assign:f.noop,ui:h.none,destroy:f.noop,insertResample:f.constant({assets:b,correlated:a,destroy:f.noop}),name:f.constant("imageEdit-disabled")}},mobile:function(){return{ui:h.none,events:Pc(["fullscreen"]),destroy:f.noop}},noAnalytics:function(){return{log:f.noop,register:f.noop}},state:function(a){var b=Ru(a.laundry),c=Su(a.filters,f.noop,f.noop),d=co(a.macros.customs);return{filter:f.constant({customFilters:f.constant(c)}),
macros:f.constant({customMacros:f.constant(d)}),laundry:b,getState:function(e){return Z.merge(a,void 0===e?{}:e,{laundry:b.state(),filters:c.state(),macros:{customs:d.state()}})}}},uploader:function(){var a=D.create({reconstitute:s([]),uploadAssets:s(["assets"])});return{destroy:f.noop,events:a.registry,reconstitute:f.noop,uploadAssets:f.noop}},undo:function(){return{reset:f.noop,disable:f.apply,save:f.noop}}},Rc={activateInline:f.noop,deactivateInline:f.noop,communication:Aa.communication,events:function(){return D.create({loaded:s([]),
focus:s([]),dirty:s([]),change:s([]),internal:s(["command","data"]),paint:s([])}).registry},keyboard:f.die("the keyboard feature is not implemented in this mode"),operations:f.die("the operations feature is not implemented in this mode"),smartview:f.die("the smartview feature is not implemented in this mode"),ui:f.die("the ui feature is not implemented in this mode")},vl=Qu("height",function(a){return ca.inBody(a)?a.dom().getBoundingClientRect().height:a.dom().offsetHeight}),ia={set:function(a,b){vl.set(a,
b)},get:function(a){return vl.get(a)},getOuter:function(a){return vl.getOuter(a)},setMax:function(a,b){var c=vl.max(a,b,"margin-top border-top-width padding-top padding-bottom border-bottom-width margin-bottom".split(" "));t.set(a,"max-height",c+"px")}},IY="undo insert style emphasis align listindent format tools".split(" "),Ve=C.immutable("x","y","width","height"),JY={intersect:function(a,b){var c=Math.max(a.x(),b.x()),d=Math.max(a.y(),b.y()),e=Math.min(a.x()+a.width(),b.x()+b.width()),e=Math.max(0,
e-c),g=Math.min(a.y()+a.height(),b.y()+b.height()),g=Math.max(0,g-d);return Ve(c,d,e,g)}},Cd=ma.generate([{unbundled:[]},{unsupported:["message"]},{item:["info","blueprint"]},{customMenu:["info","groups","blueprint"]},{textboxMenu:["info","blueprint"]},{palette:["info","children","blueprint"]}]),kA={create:function(a){var b=function(a,b){return a.feature.fold(function(){return void 0!==a.unsupported?[Cd.unsupported(a.unsupported)]:[Cd.unbundled()]},function(c){if(void 0===c[a.command])return console.warn("unknown command: "+
a.command),[];c=c[a.command]();return[b(c)]})},c=function(a,c){return b(c,function(b){return k.contains(["widget","inline-widget","split-widget"],c.type)?Cd.item({instance:b.widget},a[c.type]):Cd.item(b[c.type](),a[c.type])})},d={dropdown:c,general:c,widget:c,"split-widget":c,"inline-widget":c,palette:function(a,c){return b(c,function(b){b=b.palette();var c=k.map(b.items,function(b){return k.contains(["widget","inline-widget","split-widget"],b.type)?Cd.item({instance:b.build},a[b.type]):k.contains(["dropdown"],
b.type)?Cd.item(b.build(),a[b.type]):Cd.item(b.spec,a[b.type])});return Cd.palette(b,c,a.palette)})},submenu:function(b,c){var d=k.map(c.commands,a);return[Cd.customMenu(c,d,b.menu)]},menu:function(a,c){return b(c,function(b){return Cd.textboxMenu(b[c.type](),a[c.type])})}};return{classify:function(a,b){var c=d[b.type];return void 0!==c?c(a,b):[]}}},cata:function(a,b,c,d,e,g,l){return a.fold(b,c,d,e,g,l)}},KY={fromConfig:function(a,b,c){var d=function(a,b){var c=f.curry(e.classify,a),c=k.bind(b.items,
c);return{category:b.category,books:c}};a=f.curry(d,a);b=f.curry(d,b);var e=kA.create(b);return k.map(c,a)}},ja={resolve:lb.create("ephox-chameleon").resolve},Bh={show:function(a){t.remove(a,"display")},hide:function(a){t.set(a,"display","none")},isShowing:function(a){return t.getRaw(a,"display").isNone()},isHiding:function(a){return t.getRaw(a,"display").is("none")}},Wu={group:Gb.ensure(["component","isVisible","refreshUi","destroy"])},mo={menu:function(a,b){var c=D.create({done:s([]),abort:s(["item"])}),
d=function(a){a.done.bind(function(){c.trigger.done()});a.abort.bind(function(a){c.trigger.abort(a.item())});return Z.merge(a,{done:c.registry.done,abort:c.registry.abort})},e=k.foldr(b,function(a,b){var c=Z.merge(b.structure(),a.structures),e=b.submenus().concat(a.submenus),f=Z.merge(b.handlers(),a.actions),h=[b.refreshMenu].concat(a.refreshMenu),k=[b.destroyer].concat(a.destroyers),m=d(b.events);return{structures:c,submenus:e,refreshMenu:h,actions:f,destroyers:k,events:m}},{structures:{},submenus:[],
actions:{},refreshMenu:[],destroyers:[]});return{structure:f.constant(e.structures),init:function(a){k.each(e.refreshMenu,f.apply)},name:f.constant(a),handlers:f.constant(e.actions),submenus:f.constant(e.submenus),events:e.events,destroy:function(){k.each(e.destroyers,f.apply)}}},toolbar:function(a){a=rO(a);var b=D.create({escaped:s([]),executed:s([])});return Wu.group({component:a.component,isVisible:a.isVisible,refreshUi:a.refreshUi,destroy:a.destroy,events:b.registry})}},lA=function(a,b){n.add(a,
ja.resolve("user-icon"));t.setAll(a,{"background-image":"url("+b+")"})},wl={setUrl:lA,findAndSetUrl:function(a,b){J.descendant(a,".ephox-pastry-icon-content").each(function(a){lA(a,b)})}},xl=function(a,b){m.set(a,"aria-pressed",b?"true":"false")},ha={expanded:function(a){m.set(a,"aria-expanded","true")},collapsed:function(a){m.set(a,"aria-expanded","false")},checked:function(a,b){m.set(a,"aria-checked",String(b))},setPressed:xl,press:function(a){xl(a,!0)},release:function(a){xl(a,!1)},pressed:function(a){xl(a.element(),
a.selected())},enable:function(a){m.set(a,"aria-disabled","false")},disable:function(a){m.set(a,"aria-disabled","true")},tabSelected:function(a,b){m.setAll(a,{"aria-selected":"true","aria-pressed":"true"});k.each(b,function(a){m.setAll(a,{"aria-selected":"false","aria-pressed":"false"})})},showPanel:function(a){m.set(a,"aria-selected","true");m.set(a,"aria-hidden","false")},hidePanel:function(a){m.set(a,"aria-selected","false");m.set(a,"aria-hidden","true")}},LY=function(a){a.enable();ha.enable(a.element())},
MY=function(a){a.disable();ha.disable(a.element())},jo={control:function(a){return{enable:f.curry(LY,a),disable:f.curry(MY,a)}},ariaPressed:function(a){return{press:f.curry(ha.press,a.element()),release:f.curry(ha.release,a.element())}}},yl={css:function(a){var b=a.replace(/\./g,"-");return{resolve:function(a){return b+"-"+a}}}},ui={resolve:yl.css("ephox-echo").resolve},NY=ui.resolve("aria-help"),OY=ui.resolve("aria-help-visible"),zl=function(a){m.set(a,"role","presentation")},mA=function(a,b){m.setAll(a,
{role:"application","aria-label":b})},nA=function(a){m.set(a,"role","button")},lq=function(a,b){m.set(a,"aria-owns",b)},oA=function(a,b){m.set(a,"aria-controls",b)},pA=function(a,b){var c=La.generate("ephox-aria");m.set(b,"id",c);oA(a,c)},qA=function(a,b){m.set(a,"aria-describedby",b)},rA=function(a,b){m.set(a,"aria-labelledby",b)},Al=function(a,b){m.set(a,"aria-label",b)},Q={document:function(a){m.set(a,"role","document")},presentation:zl,region:function(a){m.set(a,"role","region")},controls:oA,
owns:lq,disown:function(a){m.remove(a,"aria-owns")},editor:function(a,b,c,d,e){mA(a,c);d.each(function(c){var d=q.fromTag("span");zl(d);v.append(d,q.fromText(c));c=La.generate("ephox-aria");m.set(d,"id",c);qA(b,c);n.add(d,NY);!0===e&&n.add(d,OY);v.append(a,d)});return{destroy:function(){m.remove(b,"aria-describedby")}}},inline:function(a,b,c,d,e){mA(a,c);var g=[];m.has(b,"id")?lq(a,m.get(b,"id")):(c=La.generate("ephox-aria-content"),m.set(b,"id",c),lq(a,c),g.push(function(){m.remove(b,"id")}));d.each(function(a){if(m.has(b,
"aria-label")){var c=m.get(b,"aria-label");Al(b,a);g.push(function(){Al(b,c)})}else Al(b,a),g.push(function(){m.remove(b,"aria-label")})});return{destroy:function(){k.each(g,f.apply)}}},group:function(a){m.set(a,"role","group")},toolbar:function(a,b){m.setAll(a,{role:"group","aria-label":b})},toolbarGroup:function(a,b){m.set(a,"role","toolbar");null===b||void 0===b||""===b||R.isArray(b)&&0===b.length||m.set(a,"aria-label",b)},toolbarButton:function(a,b,c,d){m.setAll(a,{role:"button","aria-label":b,
"aria-haspopup":""+c});d&&m.set(a,"aria-pressed","false");c&&m.set(a,"aria-expanded","false")},textButton:function(a,b){nA(a);zl(b)},menu:function(a,b){m.setAll(a,{"aria-label":b,role:"menu"})},menuItem:function(a,b,c){b=b.map(function(a){return{"aria-label":a}}).getOr({});m.setAll(a,Z.merge(b,{role:"menuitem","aria-haspopup":c.getOr(!1)}))},menuItemCheckbox:function(a){m.setAll(a,{role:"menuitemcheckbox","aria-checked":!1})},checkbox:function(a,b){m.setAll(a,{role:"checkbox","aria-label":b,"aria-checked":!1})},
dialog:function(a,b){m.setAll(a,{role:"dialog","aria-label":b})},buttonRole:nA,button:function(a,b){m.setAll(a,{"aria-label":b,role:"button"})},labelledField:function(a,b,c){var d=q.fromTag("div");zl(d);b+=La.generate("");var e=La.generate("label");c=q.fromHtml("<label>"+c+"</label>");m.set(c,"for",b);m.set(c,"id",e);m.set(a,"id",b);rA(a,e);K.append(d,[c,a]);return{element:f.constant(d),field:f.constant(a)}},multiline:function(a){m.set(a,"aria-multiline","true")},textarea:function(a){m.setAll(a,{"aria-multiline":"true",
role:"textbox"})},label:Al,widget:function(a){m.set(a,"role","widget")},option:function(a){m.set(a,"role","option")},listBox:function(a){m.set(a,"role","listbox")},live:function(a,b,c){m.setAll(a,{"aria-live":c?c:"polite",id:b})},tabList:function(a){m.set(a,"role","tablist")},tabButton:function(a,b){m.setAll(a,{"aria-label":b,role:"tab"})},tabPanel:function(a){m.setAll(a,{role:"tabpanel"})},linkElements:pA,linkTabToPanel:function(a,b){pA(a,b)},describedBy:qA,labelledBy:rA,required:function(a){m.set(a,
"aria-required","true")},hidden:function(a,b){b?m.set(a,"aria-hidden",b):m.remove(a,"aria-hidden")},invalid:function(a,b){!0===b?m.set(a,"aria-invalid","true"):m.remove(a,"aria-invalid")}},sO={disable:function(a){m.set(a,"unselectable","on");t.setAll(a,{"-webkit-user-select":"none","user-select":"none","-ms-user-select":"none"});a.dom().style.setProperty("-moz-user-select","-moz-none");return E.bind(a,"onselectstart",function(){return!1})}},Ta={resolve:lb.create("ephox-pastry").resolve},sA=function(a,
b,c,d){Q.toolbarButton(a.element(),b,c,d);b=jo.control(a);c=jo.ariaPressed(a);return{enable:b.enable,disable:b.disable,press:c.press,release:c.release,destroy:function(){a.destroy()}}},mq=function(a,b,c,d,e,g,l,f){l=h.from(l).fold(function(){return f(e)},function(a){return a(e,b)});g=h.from(g).getOr(!1);a=a(b);c=Qd([ja.resolve("icon"),ja.resolve(c)]);g=!0===g?[qc(f(e)).element()]:[];K.append(c.element(),g);a.content(c);a.events.click.bind(function(){d()});m.set(a.element(),"title",l);a.initAria=sA;
return a},tA=function(a,b){J.descendant(a,b).bind(function(a){Q.hidden(a,"true")})},uA=function(a,b,c,d,e,g,l,f){a=mq(a,b,c,d,e,g,l,f);n.add(a.element(),ja.resolve("split-"+b));tA(a.element(),".ephox-pastry-dropbutton-arrow");tA(a.element(),".ephox-pastry-dropbutton-button");J.descendant(a.element(),".ephox-pastry-dropbutton-arrow .ephox-pastry-icon-content").bind(function(a){n.add(a,ja.resolve("icon"));n.add(a,ja.resolve("arrow-"+b))});return a},tf={aria:sA,typical:function(a,b,c,d,e,g,l){return mq(ac,
a,b,c,d,e,g,l)},toggle:function(a,b,c,d,e,g,l){return mq(id,a,b,c,d,e,g,l)},splitToggle:function(a,b,c,d,e,g,l){return uA(uO,a,b,c,d,e,g,l)},split:function(a,b,c,d,e,g,l){return uA(tO,a,b,c,d,e,g,l)}},qd={resolve:lb.create("ephox-gopher").resolve},nq=Oj("item"),oq=qd.resolve("disabled-item"),vA="data-"+qd.resolve("item-value"),Sb={set:function(a,b){nq.set(a);m.set(a,vA,b)},get:function(a){return m.get(a,vA)},up:nq.up,all:nq.all,disable:function(a){n.add(a,oq)},enable:function(a){n.remove(a,oq)},isEnabled:function(a){return!n.has(a,
oq)}},vi=Oj("menu"),wA="data-"+qd.resolve("menu-value"),Bl=qd.resolve("active-menu"),Sc={set:function(a,b){vi.set(a);m.set(a,wA,b)},active:function(a){return J.descendant(a,"."+Bl)},inactive:function(a){a=vi.all(a);return k.filter(a,function(a){return!n.has(a,Bl)})},activate:function(a,b){var c=vi.all(a);k.each(c,function(a){n.remove(a,Bl)});n.add(b,Bl)},up:vi.up,all:vi.all,submenu:function(a,b,c){c=Sb.get(c);return(b.hasOwnProperty(c)?h.from(b[c]):h.none()).bind(function(b){return J.descendant(a,
"["+wA+'="'+b+'"]')})}},PY=Oj("submenu"),QY=Oj("main-menu"),ye={main:f.constant(QY),submenu:f.constant(PY)},xA=function(a,b,c){Sb.set(c,b);m.set(c,"tabindex",-1);return c},zA=function(a,b,c){return yA(a,b,c,!1)},yA=function(a,b,c,d){var e=q.fromTag("li",a.dom()),g=P.span();n.add(g,qd.resolve("item-text"));!0===d&&n.add(g,qd.resolve("item-ellipsis"));v.append(g,q.fromText(c));v.append(e,g);xA(a,b,e);return e},AA=function(a){a=q.fromTag("li",a.dom());n.add(a,qd.resolve("separator"));return a},RY=function(a,
b){return"-"!==b.value()?zA(a,b.value(),b.text()):AA(a)},BA=function(a,b,c){var d=q.fromTag("ol",a.dom());Sc.set(d,b);a=k.map(c,f.curry(RY,a));K.append(d,a);return d},CA=function(a,b){var c=q.fromTag("div",a.dom());Sc.set(c,b);return c},bb={item:zA,ellipsisItem:function(a,b,c){return yA(a,b,c,!0)},mainmenu:function(a,b,c){a=BA(a,b,c);ye.main().set(a);return a},submenu:function(a,b,c){a=BA(a,b,c);ye.submenu().set(a);return a},widget:CA,widgetSubmenu:function(a,b){var c=CA(a,b);ye.submenu().set(c);
return c},separator:AA,custom:xA,enableItem:Sb.enable,disableItem:Sb.disable},DA=function(a,b){if(void 0===a)throw Error("Cannot intersperse undefined");if(1>=a.length)return a;var c=[];c.push(a[0]);for(var d=1;d<a.length;d++)c.push(b()),c.push(a[d]);return c},EA={intersperse:function(a,b){return DA(a,function(){return b})},intersperseThunk:DA},SY={css:function(a){var b=/^[a-zA-Z]/.test(a)?"":"e";a=a.replace(/[^\w]/gi,"-");return b+a}},TY={splits:function(a,b){if(0===b.length)return[a];var c=k.foldl(b,
function(b,c){if(0===c)return b;var d=a.substring(b.prev,c);return{prev:c,values:b.values.concat([d])}},{prev:0,values:[]}),d=b[b.length-1];return d<a.length?c.values.concat(a.substring(d)):c.values}},Cl={cssSanitise:function(a){return SY.css(a)},splits:function(a,b){return TY.splits(a,b)}},UY=function(a,b){var c=k.flatten(b);return mo.menu(a,c)},VY=function(a){return k.map(a,function(a){a=k.map(a,function(a){return a.items()});return k.flatten(a)})},FA={create:function(a,b,c){var d=a.name,e=a.textkey,
g=a.menuTextkey,l=a.tooltip,p=a.label,r=h.from(a.icon),z=f.constant(void 0!==a.contentFocus?a.contentFocus:!1),w=q.fromDom(document),m=R.isString(e)?Cl.cssSanitise(d):d,n=c(e),g=c(g),s=bb.mainmenu(w,m+"-menu",[]),t=tf.toggle(d,m,f.identity,e,p,l,c);r.filter(R.isString).each(function(a){wl.findAndSetUrl(t.element(),a)});var u=t.initAria(t,n,!0,a.toggle);a=VY(b);var v=UY(m,b);b=EA.intersperseThunk(a,function(){return[bb.separator(w)]});Q.menu(s,g);K.append(s,k.flatten(b));return{button:f.constant(t),
contentFocus:z,destroy:function(){t.destroy();v.destroy();u.destroy()},element:f.constant(s),events:v.events,handlers:v.handlers,icon:f.constant(r),init:v.init,label:f.constant(n),name:v.name,structure:v.structure,submenus:v.submenus,textkey:f.constant(e),menuTextkey:f.constant(g),updateMenu:f.noop,enable:u.enable,disable:u.disable}}},WY=function(a,b){return h.some(b(a))},XY=function(a,b){return h.some(b(a,[]))},GA=function(a,b){var c=f.curry(GA,a);return kA.cata(b,h.none,h.none,WY,function(b,e,g){e=
k.map(e,function(a){return k.bind(a.books,c)});b=FA.create(b,e,a);return h.some(g(b,[]))},XY,function(a,b,g){b=k.bind(b,c);return h.some(g(a,b))}).toArray()},YY={build:function(a,b){return{items:k.bind(a.books,function(a){return GA(b,a)}),category:a.category}}},ZY={build:function(a,b,c){var d=c.translations();a=KY.fromConfig(b.toolbar(),b.menu(),a);a=k.map(a,function(a){return YY.build(a,d)});return k.map(a,function(a){var b=mo.toolbar(a.items);a=a.category;Q.toolbarGroup(b.component(),d(a));return b})}},
$Y=C.immutable("toolbar","menu"),aZ=C.immutable("smartview","translations"),HA=function(a){return function(){return a.apply(null,arguments)}},IA={blueprints:HA($Y),env:HA(aZ)},JA=function(){return/(?:(?:[A-Za-z]{3,9}:(?:\/\/))(?:[-.~*+=!&;:'%@?^${}(),\w]+@)?[A-Za-z0-9-]+(?:\.[A-Za-z0-9-]+)*|(?:www\.|[-;:&=+$,.\w]+@)[A-Za-z0-9-]+(?:\.[A-Za-z0-9-]+)*)(?::[0-9]+)?(?:\/[-+~%.()\/\w]*)?(?:\?(?:[-.~*+=!&;:'%@?^${}(),\/\w]+))?(?:#(?:[-.~*+=!&;:'%@?^${}(),\/\w]+))?/g},Dl={tokens:function(a,b){return a.replace(/\{(\d+)\}/g,
function(c,d,e,g){d=parseInt(d,10);if(void 0===b[d])throw"No value for token: "+c+" in translation: "+a;return b[d]})},link:JA,autolink:function(){var a=JA().source;return RegExp("("+a+")[-.~*+=!&;:'%@?#^${}(),]*","g")}},pq=ma.generate([{raw:["text"]},{key:["key"]},{pkey:["key","params"]}]),qq=function(a){var b=If.get(a);if(1<arguments.length||/\{(\d+)\}/.test(b)){for(var c=Array(arguments.length-1),d=1;d<arguments.length;d++)c[d-1]=arguments[d];return Dl.tokens(b,c)}return b},u={get:function(a){if(R.isString(a))throw"string passed to I18n.get:"+
a;return a.fold(f.identity,qq,qq)},has:If.has,translate:qq,raw:pq.raw,key:function(a){If.get(a);return pq.key(a)},pkey:function(a,b){If.get(a);return pq.pkey(a,b)},cata:function(a,b,c,d){return a.fold(b,c,d)}},Dd={dropdown:Gb.exactly("enable disable select clear selectElement populate tick untick activate deactivate".split(" ")),menu:Gb.exactly(["enable","disable"]),general:Gb.exactly("enable disable select deselect selected element".split(" ")),widget:Gb.exactly("enable disable select deselect press release".split(" ")),
palette:Gb.exactly(["show","hide"])},Tj={generateItem:function(a,b,c,d){return c.fold(function(){return bb.item(a,b.value(),d(b.textkey()))},function(c){c=c(b);var d=q.fromTag("li",a.dom());v.append(d,c);return bb.custom(a,b.value(),d)})},generateSeparator:function(a,b,c){var d=bb.separator(a);v.append(d,q.fromText(c(b),a.dom()));return d}},bZ=function(a,b){var c=k.find(a,function(a){return a.value()===b});return h.from(c)},KA=function(a){a=X.children(a,".ephox-pastry-toggle-list-item-selected");
k.each(a,function(a){n.remove(a,"ephox-pastry-toggle-list-item-selected");ha.checked(a,!1)})},gg={updateDropdown:function(a,b,c,d,e){b.fold(a.clear,f.noop,function(b){e.fold(function(){(R.isString(b)?bZ(d,b):h.some(b)).each(a.select)},function(c){c=c(b);var d=Ta.resolve("arrow-righticon");n.add(c,d);a.selectElement(c)})});c.fold(a.disable,f.noop,a.enable)},tick:function(a,b){KA(a);var c=X.descendants(a,".ephox-gopher-item");k.find(c,function(a){return m.get(a,"data-ephox-gopher-item-value")===b}).each(function(a){n.add(a,
"ephox-pastry-toggle-list-item-selected");ha.checked(a,!0)})},untick:KA,activate:function(a){n.add(a,"ephox-pastry-statebutton-selected")},deactivate:function(a){n.remove(a,"ephox-pastry-statebutton-selected")}},rq=Ta.resolve("toggle-list-item-selected"),cZ=function(a){bb.enableItem(a);ha.enable(a)},dZ=function(a){bb.disableItem(a);ha.disable(a)},eZ=function(a){n.remove(a,rq);ha.checked(a,!1)},fZ=function(a){n.add(a,rq);ha.checked(a,!0)},eg={control:function(a){return{enable:f.curry(cZ,a),disable:f.curry(dZ,
a)}},selected:function(a){return{deselect:f.curry(eZ,a),select:f.curry(fZ,a),selected:f.curry(n.has,a,rq)}}},LA=function(a){var b=P.span();Q.hidden(b,!0);ea.add(b,["ephox-pastry-icon-content","ephox-chameleon-icon"].concat([a]));return[b]},gZ=function(a){var b=P.span();Q.hidden(b,!0);ea.add(b,["ephox-pastry-icon-content","ephox-chameleon-"+a.value]);wl.setUrl(b,a.icon);return[b]},hZ=function(a){return LA(!0===a.icon?"ephox-chameleon-"+a.value:a.icon)},Dh={generate:LA,generateFrom:function(a){return void 0===
a.icon?[]:(R.isString(a.icon)?gZ:hZ)(a)}},iZ=Y.detect().browser.isFirefox(),jZ={position:"absolute",left:"-9999px"},MA=function(a,b){var c=q.fromTag("span",a.dom());Q.presentation(c);var d=q.fromText(b,a.dom());v.append(c,d);return c},NA=function(a,b){var c=La.generate("ephox-echo-voice");m.set(b,"id",c);m.set(a,"aria-describedby",c)},OA=function(a,b,c){var d=y.owner(b),e=MA(d,c);iZ&&NA(b,e);m.setAll(e,a(c));t.setAll(e,jZ);v.append(b,e);setTimeout(function(){m.remove(e,"aria-live");B.remove(e)},1E3)},
kZ=f.curry(OA,function(a){return{"aria-live":"polite","aria-atomic":"true","aria-label":a}}),lZ=f.curry(OA,f.constant({"aria-live":"assertive","aria-atomic":"true",role:"alert"})),Oa={describe:function(a,b){var c=y.owner(a),c=MA(c,b);t.set(c,"display","none");Q.hidden(c,!0);NA(a,c);return c},speak:kZ,shout:lZ,tokenSelector:function(){return'span[id^="ephox-echo-voice"]'}},wO=q.fromDom(document),Yu=ja.resolve("label-default"),sq=q.fromDom(document),mZ="data-"+qd.resolve("item-value"),nZ=function(a){a=
bb.submenu(sq,a+"-menu",[]);n.remove(a,"ephox-gopher-main-menu");n.add(a,"ephox-gopher-submenu");return a},oZ=function(a,b){var c=m.get(b,"data-ephox-gopher-menu-value"),d={};d[a.name+"-menu-trigger"]=c;return d},pZ=function(a,b){var c=nZ(b.name);Q.menu(c,a(b.textkey,!1));var d=oZ(b,c),e=vO(a,b),g=function(d){d=k.bind(d,function(c){var d=k.map(c.items(),function(c){c=Tj.generateItem(sq,c,b.render,a);Q.menuItemCheckbox(c);return c});return c.label().fold(f.constant(d),function(b){return[Tj.generateSeparator(sq,
b,a)].concat(d)})});B.empty(c);K.append(c,d)},l=D.create({done:s([]),abort:s(["item"])}),p={};k.each(b.values,function(a){p[a.value()]=function(a){a=m.get(a,mZ);b.action(a);l.trigger.done()}});e.events.populate.bind(function(a){g(a.items())});return{items:f.constant([e.element()]),structure:f.constant(d),submenus:f.constant([c]),refreshMenu:function(){b.updateDropdown(Dd.dropdown({enable:e.enable,disable:e.disable,select:e.select,clear:e.clear,selectElement:e.selectElement,populate:e.populate,tick:f.curry(gg.tick,
c),untick:f.curry(gg.untick,c),activate:f.noop,deactivate:f.noop}))},destroyer:function(){e.destroy()},handlers:f.constant(p),events:l.registry}},qZ=q.fromDom(document),rZ=function(a,b){var c=a(b.textkey),d=(!0===b.hasDialog?bb.ellipsisItem:bb.item)(qZ,b.value,c),e=eg.control(d),g=!0===b.toggle?eg.selected(d):{select:f.noop,deselect:f.noop,selected:f.constant(!1)};!0===b.toggle?Q.menuItemCheckbox(d):Q.menuItem(d,h.some(c),h.some(!1));var l=function(){b.updateGeneral(Dd.general({disable:e.disable,
enable:e.enable,select:g.select,deselect:g.deselect,selected:g.selected,element:f.constant(d)}))},p=D.create({done:s([]),abort:s(["item"])}),c={};c[b.value]=function(){p.trigger.done();b.action();l()};var r=Dh.generateFrom(b);K.prepend(d,r);return{items:f.constant([d]),structure:f.constant({}),submenus:f.constant([]),refreshMenu:l,handlers:f.constant(c),destroyer:f.identity,events:p.registry}},sZ=q.fromDom(document),tZ=function(a){return{submenu:function(b){var c=b.textkey();b=b.element();Q.menu(b,
a(c));n.remove(b,"ephox-gopher-main-menu");n.add(b,"ephox-gopher-submenu");return b},item:function(b){var c=b.textkey(),c=a(c);b=bb.item(sZ,b.name()+"-menu-trigger",c);n.add(b,ja.resolve("menu-more"));Q.menuItem(b,h.some(c),h.some(!0));return b}}},uZ=function(a,b,c){c=tZ(a);a=c.submenu(b);var d=m.get(a,"data-ephox-gopher-menu-value");c=c.item(b);var e={};e[b.name()+"-menu-trigger"]=d;var d=Z.merge(e,b.structure()),g=Dh.generate("ephox-chameleon-"+b.name());K.prepend(c,g);b.icon().each(function(a){0<
g.length&&wl.setUrl(g[0],a)});var l=eg.control(c),e=void 0!==b.destroyer?b.destroyer:void 0!==b.destroy?b.destroy:f.noop;return{items:f.constant([c]),structure:f.constant(d),submenus:f.constant([a].concat(b.submenus())),refreshMenu:function(){b.updateMenu({enable:l.enable,disable:l.disable})},destroyer:e,handlers:b.handlers,events:b.events}},tq=q.fromDom(document),PA=ja.resolve("menu-widget"),vZ=function(a,b){var c=bb.widgetSubmenu(tq,b);v.append(c,a.element());ea.add(a.element(),[PA,"ephox-gopher-item"]);
return c},wZ=function(a,b,c){a=a(b.textkey());a=bb.item(tq,c,a);n.add(a,ja.resolve("menu-more"));c=Dh.generate(ja.resolve(c));K.prepend(a,c);return a},QA=function(a,b){var c=b.instance(),d=c.name(),e=d+"-submenu",g=vZ(c,e),l=wZ(a,c,d);Q.menu(g,a(c.menuTextkey()));Q.menuItem(l,h.some(a(c.textkey())),h.some(!0));var p=eg.control(l),r={};r[d]=e;var k=D.create({done:M([],c.events.done),abort:s(["item"])});c.events.abort.bind(function(){k.trigger.abort(l)});return{items:f.constant([l]),structure:f.constant(r),
submenus:f.constant([g]),refreshMenu:function(){c.updateWidget(Dd.widget({enable:p.enable,disable:p.disable,select:f.noop,deselect:f.noop,press:f.noop,release:f.noop}))},destroyer:c.destroy,handlers:f.constant({}),events:k.registry}},Sj={widget:QA,inline:function(a,b){var c=b.instance(),d=c.name();n.add(c.element(),ja.resolve("menu-inline-widget"));var e=q.fromTag("li");v.append(e,c.element());var g=bb.custom(tq,d,e),l=eg.control(g);Q.menuItem(g,h.some(a(c.textkey())),h.some(!1));e={};e[d]=function(a){var b=
T.active().filter(function(b){return Gc.ancestor(b,f.curry(L.eq,a))});c.element().dom().click();b.each(T.focus)};var p=D.create({done:M([],c.events.done),abort:s(["item"])});c.events.abort.bind(function(){p.trigger.abort(g)});return{items:f.constant([g]),structure:f.constant({}),submenus:f.constant([]),refreshMenu:function(){c.updateInlineWidget(Dd.widget({enable:l.enable,disable:l.disable,select:f.noop,deselect:f.noop,press:f.noop,release:f.noop}))},destroyer:c.destroy,handlers:f.constant(e),events:p.registry}},
split:QA,styles:f.constant(PA)},RA=ja.resolve("keyfocus"),SA=function(a,b){var c=P.span();t.set(c,"display","none");n.add(c,RA);v.append(a,c);return E.bind(c,"click",function(a){b();a.kill()})},ze={onButton:function(a){return SA(a.element(),a.click)},custom:SA,trigger:function(a){var b="."+RA;a=ra.is(a,b)?h.some(a):J.descendant(a,b);a.each(function(a){a.dom().click()});return a}},Gh={menu:C.immutableBag(["focusActive"],[])},Eg=qd.resolve("item-selected"),uq=function(a,b){n.add(b,Eg)},TA=function(a,
b){n.remove(b,Eg)},Tc={is:function(a){return n.has(a,Eg)},none:function(a){var b=Sb.all(a);k.each(b,function(b){TA(a,b)})},selectAll:function(a,b){var c=Sb.all(a);k.each(c,function(a){n.remove(a,Eg)});k.each(b,function(a){n.add(a,Eg)})},select:uq,deselect:TA,selected:function(a){return J.descendant(a,"."+Eg)},first:function(a){var b=Sb.all(a);return h.from(b[0]).map(function(b){uq(a,b);return b})},last:function(a){var b=Sb.all(a);return h.from(b[b.length-1]).map(function(b){uq(a,b);return b})}},xZ=
function(a,b){var c=Sb.all(a);return k.findIndex(c,function(a){return L.eq(b,a)}).map(function(a){return{index:f.constant(a),all:f.constant(c)}})},yZ=function(a,b){return 0===b?a.length-1:b-1},zZ=function(a,b){return b===a.length-1?0:b+1},UA=function(a,b,c){return xZ(a,b).map(function(d){var e=c(d.all(),d.index());d=d.all()[e];Tc.deselect(a,b);Tc.select(a,d);return d})},ho={first:function(a){var b=Sb.all(a);return h.from(b[0]).map(function(b){Tc.select(a,b);return b})},prev:function(a,b){return UA(a,
b,yZ)},next:function(a,b){return UA(a,b,zZ)}},vb=function(a){return function(){return a}},N={BACKSPACE:vb([8]),TAB:vb([9]),ENTER:vb([13]),SHIFT:vb([16]),CTRL:vb([17]),ALT:vb([18]),CAPSLOCK:vb([20]),ESCAPE:vb([27]),SPACE:vb([32]),PAGEUP:vb([33]),PAGEDOWN:vb([34]),END:vb([35]),HOME:vb([36]),LEFT:vb([37]),UP:vb([38]),RIGHT:vb([39]),DOWN:vb([40]),INSERT:vb([45]),DEL:vb([46]),META:vb([91,93,224]),F10:vb([121])},$u={isCommand:function(a,b){var c=[];k.each(b,function(a){a="string"===typeof a?[a.toUpperCase().charCodeAt(0)]:
a;c=c.concat(a)});var d=function(a){return 0<k.filter(c,function(b){return k.contains(N[a](),b)}).length};if(!!a.altKey!==d("ALT")||!!a.shiftKey!==d("SHIFT")||!!a.ctrlKey!==d("CTRL")||!!a.metaKey!==d("META"))return!1;var d=null!==a.charCode&&void 0!==a.charCode&&0!==a.charCode?String.fromCharCode(a.charCode).toUpperCase().charCodeAt(0):a.which,e=N.CTRL().concat(N.ALT()).concat(N.SHIFT()).concat(N.META()),g=k.filter(c,function(a){return!k.contains(e,a)});return 1<g.length?!1:0===g.length&&void 0===
a.which||g[0]===d}},Zu={stop:function(a){a.preventDefault();a.stopPropagation&&a.stopPropagation()}},AZ={BACKSPACE:N.BACKSPACE,TAB:N.TAB,ENTER:N.ENTER,CAPSLOCK:N.CAPSLOCK,ESCAPE:N.ESCAPE,SPACE:N.SPACE,PAGEUP:N.PAGEUP,PAGEDOWN:N.PAGEDOWN,END:N.END,HOME:N.HOME,LEFT:N.LEFT,UP:N.UP,DOWN:N.DOWN,RIGHT:N.RIGHT,INSERT:N.INSERT,DEL:N.DEL},pf={getSpecial:function(a){var b=null;W.each(AZ,function(c,d){k.contains(c(),a)&&(b=d)});return b},noModifiers:function(a){return!(a.metaKey||a.altKey||a.ctrlKey)},isCharCode:function(a){return a.charCode===
a.keyCode}},VA=function(a,b){return k.find(b,function(b){return $u.isCommand(a,b)})},WA=function(a){return pf.isCharCode(a)?h.none():h.from(pf.getSpecial(a.keyCode))},zO={identify:function(a,b){return{fold:function(c,d,e){VA(a,b).fold(function(){WA(a).fold(function(){e(a.which&&String.fromCharCode(a.which))},function(a){d(N[a]())})},c)}}},shortcut:VA,special:WA},XA=function(a){return"rtl"===t.get(a,"direction")?"rtl":"ltr"},rd={onDirection:function(a,b){return function(c){return"rtl"===XA(c)?b:a}},
getDirection:XA},YA=function(a,b,c,d){var e=t.get(a,b);void 0===e&&(e="");c=e===c?d:c;e=f.curry(t.set,a,b,e);a=f.curry(t.set,a,b,c);return bo(e,a,!1)},Tb={toggler:function(a){return YA(a,"visibility","hidden","visible")},displayToggler:function(a,b){return YA(a,"display","none",b)},isVisible:function(a){a=a.dom();return!(0>=a.offsetWidth&&0>=a.offsetHeight)}},El=ma.generate([{southeast:[]},{southwest:[]},{northeast:[]},{northwest:[]}]),Hc={southeast:El.southeast,southwest:El.southwest,northeast:El.northeast,
northwest:El.northwest,cata:function(a,b,c,d,e){return a.fold(b,c,d,e)}},ZA=function(a,b){return a.view(b).fold(f.constant([]),function(b){var d=a.owner(b),d=ZA(a,d);return[b].concat(d)})},BZ={pathTo:function(a,b){var c=b.owner(a),c=ZA(b,c);return h.some(c)}},CZ={view:function(a){return(a.dom()===document?h.none():h.from(a.dom().defaultView.frameElement)).map(q.fromDom)},owner:function(a){return y.owner(a)}},wi={find:function(a){var b=q.fromDom(document),c=Ea.get(b);return BZ.pathTo(a,CZ).fold(f.curry(Na.absolute,
a),function(b){var e=Na.viewport(a);b=k.foldr(b,function(a,b){var c=Na.viewport(b);return{left:a.left+c.left(),top:a.top+c.top()}},{left:0,top:0});return Ba(b.left+e.left()+c.left(),b.top+e.top()+c.top())})}},pe={absolute:function(a){var b=wi.find(a),c=qa.getOuter(a);a=ia.getOuter(a);return Ve(b.left(),b.top(),c,a)},view:function(){return Ve(0,0,window.innerWidth,window.innerHeight)}},$A=function(a,b,c){var d=Ba(b,c);b=function(){var a=Ea.get();return d.translate(-a.left(),-a.top())};return a.fold(f.constant(d),
b,b)},vq={toBox:function(a,b){var c=wi.find(b),c=$A(a,c.left(),c.top()),d=qa.getOuter(b),e=ia.getOuter(b);return Ve(c.left(),c.top(),d,e)},viewport:function(a,b){return b.fold(function(){return a.fold(pe.view,pe.view,Ve)},function(b){return a.fold(b,b,Ve)})},translate:$A},DZ=C.immutableBag("x y width height maxHeight direction classes label candidateYforTest".split(" "),[]),EZ=C.immutable("position","left","top","right","bottom"),Jf={decision:DZ,css:EZ},wq=ma.generate([{none:[]},{relative:["x","y",
"width","height"]},{fixed:["x","y","width","height"]}]),aB=function(a,b,c,d,e,g){c=b.x()-c;d=b.y()-d;var l=b.width(),f=b.height();e-=c+l;g-=d+f;var r=h.some(c),k=h.some(d),w=h.some(e),m=h.some(g),n=h.none();return Hc.cata(b.direction(),function(){return Jf.css(a,r,k,n,n)},function(){return Jf.css(a,n,k,w,n)},function(){return Jf.css(a,r,n,n,m)},function(){return Jf.css(a,n,n,w,m)})},Ed={none:wq.none,relative:wq.relative,fixed:wq.fixed,reposition:function(a,b){return a.fold(function(){return Jf.css("absolute",
h.some(b.x()),h.some(b.y()),h.none(),h.none())},function(a,d,e,g){return aB("absolute",b,a,d,e,g)},function(a,d,e,g){return aB("fixed",b,a,d,e,g)})},viewport:function(a,b){return vq.viewport(a,b)},toBox:function(a,b){return vq.toBox(a,b)},translate:function(a,b,c){return vq.translate(a,b,c)},cata:function(a,b,c,d){return a.fold(b,c,d)}},bB=C.immutable("anchorBox","origin"),mg={box:function(a,b){return bB(a,b)},element:function(a,b){var c=Ed.toBox(b,a);return bB(c,b)},relativeOrigin:function(a){a=
a.dom().getBoundingClientRect();return Ed.relative(a.left,a.top,a.width,a.height)}},xi={resolve:lb.create("ephox-repartee").resolve},cB=xi.resolve("caret-top-align"),dB=xi.resolve("caret-bottom-align"),eB=xi.resolve("caret-right-align"),fB=xi.resolve("caret-left-align"),gB=xi.resolve("caret-middle-align"),wb={east:f.constant(fB),south:f.constant(cB),west:f.constant(eB),north:f.constant(dB),middle:f.constant(gB),all:f.constant([cB,dB,fB,eB,gB])},Fd=C.immutable("x","y","bubble","direction","anchors",
"label"),hB=function(a,b){return a.x()+a.width()-b.width()/2},iB=function(a,b){return a.x()+a.width()-b.width()},xq=function(a,b){return a.y()-b.height()},yq=function(a){return a.y()+a.height()},zq=function(a,b,c){return Fd(a.x(),yq(a),c.southeast(),Hc.southeast(),[wb.south(),wb.east()],"layout-se")},Aq=function(a,b,c){return Fd(iB(a,b),yq(a),c.southwest(),Hc.southwest(),[wb.south(),wb.west()],"layout-sw")},Bq=function(a,b,c){return Fd(a.x(),xq(a,b),c.northeast(),Hc.northeast(),[wb.north(),wb.east()],
"layout-ne")},Cq=function(a,b,c){return Fd(iB(a,b),xq(a,b),c.northwest(),Hc.northwest(),[wb.north(),wb.west()],"layout-nw")},Dq=function(a,b,c){return Fd(hB(a,b),xq(a,b),c.northeast(),Hc.northeast(),[wb.north(),wb.middle()],"layout-nm")},Eq=function(a,b,c){return Fd(hB(a,b),yq(a),c.southeast(),Hc.southeast(),[wb.south(),wb.middle()],"layout-sm")},rg={southeast:zq,northeast:Bq,southwest:Aq,northwest:Cq,southmiddle:Eq,northmiddle:Dq,all:function(){return[zq,Aq,Bq,Cq,Eq,Dq]},allRtl:function(){return[Aq,
zq,Cq,Bq,Eq,Dq]}},jB=function(a){return a.x()+a.width()},kB=function(a,b){return a.y()-b.height()+a.height()},Fq=function(a,b,c){return Fd(jB(a),a.y(),c.southeast(),Hc.southeast(),[wb.south(),wb.east()],"link-layout-se")},Gq=function(a,b,c){b=a.x()-b.width();return Fd(b,a.y(),c.southwest(),Hc.southwest(),[wb.south(),wb.west()],"link-layout-sw")},Hq=function(a,b,c){return Fd(jB(a),kB(a,b),c.northeast(),Hc.northeast(),[wb.north(),wb.east()],"link-layout-ne")},Iq=function(a,b,c){var d;d=a.x()-b.width();
return Fd(d,kB(a,b),c.northwest(),Hc.northwest(),[wb.north(),wb.west()],"link-layout-nw")},FZ={southeast:Fq,northeast:Hq,southwest:Gq,northwest:Iq,all:function(){return[Fq,Gq,Hq,Iq]},allRtl:function(){return[Gq,Fq,Iq,Hq]}},Rd={anchored:f.constant(function(a,b){ia.setMax(a,Math.floor(b));t.setAll(a,{"overflow-x":"hidden","overflow-y":"auto"})}),expandable:f.constant(function(a,b){ia.setMax(a,Math.floor(b))})},Kf=ma.generate([{fit:["reposition"]},{nofit:["reposition","deltaW","deltaH"]}]),GZ=function(a,
b,c,d){var e=a.x(),g=a.y(),l=a.bubble().left(),p=a.bubble().top(),h=d.x(),k=d.y(),w=d.width(),m=d.height(),l=e+l,e=g+p,n=l>=h,q=e>=k,g=n&&q,p=l+b<=h+w&&e+c<=k+m,h=n?Math.min(b,h+w-l):Math.abs(h-(l+b)),w=q?Math.min(c,k+m-e):Math.abs(k-(e+c));b=d.width()-b;d=Math.max(0,l);b=Math.min(d,b);c=q?e:e+(c-w);d=f.constant(c+w-k);k=f.constant(k+m-c);k=Hc.cata(a.direction(),k,k,d,d);a=Jf.decision({x:b,y:c,width:h,height:w,maxHeight:k,direction:a.direction(),classes:a.anchors(),label:a.label(),candidateYforTest:e});
return g&&p?Kf.fit(a):Kf.nofit(a,h,w)},HZ={attempts:function(a,b,c,d,e){var g=c.width(),l=c.height(),p=function(a,f,p,h){a=a(b,c,d);return GZ(a,g,l,e).fold(Kf.fit,function(a,b,c){return b>p||c>h?Kf.nofit(a,b,c):Kf.nofit(f,p,h)})};return k.foldl(a,function(a,b){var c=f.curry(p,b);return a.fold(Kf.fit,c)},Kf.nofit(Jf.decision({x:b.x(),y:b.y(),width:c.width(),height:c.height(),maxHeight:c.height(),direction:Hc.southeast(),classes:[],label:"none",candidateYforTest:b.y()}),-1,-1)).fold(f.identity,f.identity)}},
IZ={measure:function(a){var b=q.fromTag("div");t.setAll(b,{position:"fixed",opacity:"0",top:"0",left:"0",bottom:"0",right:"0"});v.prepend(a,b);a=Na.viewport(b);var c=qa.get(b),d=ia.get(b);B.remove(b);return{top:a.top,left:a.left,width:f.constant(c),height:f.constant(d)}}},Fl={layout:function(a,b,c,d){t.remove(b,"max-height");b={width:f.constant(qa.getOuter(b)),height:f.constant(ia.getOuter(b))};return HZ.attempts(d.preference(),a,b,c,d.bounds())},setClasses:function(a,b){ea.remove(a,wb.all());ea.add(a,
b.classes())},setHeight:function(a,b,c){c.maxHeightFunction()(a,b.maxHeight())},position:function(a,b,c){var d=function(a){return a+"px"};c=c.origin();var e=f.constant(c);c=c.fold(e,e,function(){var b=IZ.measure(a);return Ed.fixed(b.left(),b.top(),b.width(),b.height())});b=Ed.reposition(c,b,a);t.setOptions(a,{position:h.some(b.position()),left:b.left().map(d),top:b.top().map(d),right:b.right().map(d),bottom:b.bottom().map(d)})}},lB=C.immutableBag(["bounds","origin","preference","maxHeightFunction"],
[]),yi=function(a,b,c){return void 0===a[b]?c:a[b]},mB=function(a,b,c,d){a=Fl.layout(a,b,c,d);Fl.position(b,a,d);Fl.setClasses(b,a);Fl.setHeight(b,a,d)},Hk={simple:function(a,b,c,d,e){var g=yi(e,"maxHeightFunction",Rd.anchored());e=a.anchorBox();a=a.origin();d=lB({bounds:Ed.viewport(a,h.none()),origin:a,preference:d,maxHeightFunction:g});mB(e,b,c,d)},detailed:function(a,b,c,d){d=void 0!==d?d:{};d=lB({bounds:yi(d,"bounds",pe.view()),origin:yi(d,"origin",Ed.none()),preference:yi(d,"preference",rg.all()),
maxHeightFunction:yi(d,"maxHeightFunction",Rd.anchored())});mB(a,b,c,d)}},JZ=function(a,b,c,d,e,g,f){return function(a){var r,k,w=Ed.none();r=Ed.viewport(w,h.some(b));k=Ed.toBox(w,c());r={bounds:r,origin:w,preference:e.isRtl()?g.allRtl():g.all(),maxHeightFunction:f};a=a.panel();Hk.detailed(k,a,d,r)}},KZ=function(a,b,c,d,e,g,f){return function(b){var h=e.isRtl()?g.allRtl():g.all(),k={maxHeightFunction:f},w=mg.element(c(),mg.relativeOrigin(a()));b=b.panel();Hk.simple(w,b,d,h,k)}},nB=Y.detect().deviceType.isiOS()?
JZ:KZ,rf={main:function(a,b,c,d,e,g){a=nB(a,b,c,d,e,rg,g);return{recalculate:a,isAnchored:f.always,place:a}},submenu:function(a,b,c,d,e){a=nB(a,b,c,fg(3,-2),d,FZ,e);return{recalculate:a,isAnchored:f.always,place:a}}},oB=function(a){return function(b){return b.key()===a}},LZ=C.immutable("key","value"),Qj={put:function(a,b,c,d,e){var g=void 0!==d?d:LZ,f=void 0!==e?e:oB(b);a=k.foldl(a,function(a,d){var e=f(d)?g(b,c):d;return{updated:f(d)||a.updated,acc:a.acc.concat([e])}},{updated:!1,acc:[]});return a.updated?
a.acc:a.acc.concat([g(b,c)])},get:function(a,b,c){b=void 0!==c?c:oB(b);return k.find(a,b)}},FO={triage:function(a){a=k.foldr(a,function(a,c){var d=ca.inBody(c.menu());return{keep:d?[c].concat(a.keep):a.keep,throwaway:!1===d?[c].concat(a.throwaway):a.throwaway}},{throwaway:[],keep:[]});k.each(a.throwaway,function(a){a.popup().destroy()});return a.keep}},pB=function(a,b){return Gc.ancestor(a,f.curry(L.eq,b))},CO={ancestor:pB,anyAncestor:function(a,b){return k.exists(b,function(b){return pB(a,b)})},
sibling:function(a,b){return Gc.sibling(a,f.curry(L.eq,b))},child:function(a,b){return Gc.child(a,f.curry(L.eq,b))},descendant:function(a,b){return Gc.descendant(a,f.curry(L.eq,b))}},EO={create:function(a,b){var c=D.create({open:s([]),close:s([]),uistale:s([])});if(void 0===a)throw"uicontainer not specified";var d=DO({layerClass:ja.resolve("layer-menus"),container:a()}),e=P.div();n.add(e,ja.resolve("popup-container"));d.content(e);var g=!1,l=function(){g&&(r.off(),d.hide(),g=!1,k.control.block(),
c.trigger.close())},h=[e],r=av(function(a){!1===!g&&(l(),J.ancestor(a,"."+b).fold(function(){T.focus(a)},f.noop))},function(){return h}),k=Qc.tap(function(){c.trigger.uistale()});k.control.block();return{element:function(){return e},panel:d.element,destroy:function(){d.hide();d.destroy();r.off()},open:function(){g||(k.control.unblock(),t.set(d.element(),"opacity","0"),d.show(),t.reflow(d.element()),r.on(),g=!0,c.trigger.open(),t.set(d.element(),"opacity","1"))},close:l,closed:function(){return!g},
mousedown:function(a){r.mousedown(a.original())},uiStale:function(){k.instance()},setContent:function(a){v.append(e,a)},setComponents:function(a){h=a.concat([e])},events:c.registry}}},bv=C.immutable("menu","popup"),Rj=function(a){return function(b){return L.eq(b.menu(),a)}},JO=Y.detect().deviceType.isiOS,MZ=function(a,b,c,d,e){var g=HO(a,b,e.name,e.textkey,e.menuTextkey,e.render,e.layout,c,e.toggle,d);a=function(){e.updateDropdown(Dd.dropdown({enable:g.enable,disable:g.disable,select:g.select,clear:g.clear,
selectElement:g.selectElement,populate:g.populate,tick:g.tick,untick:g.untick,activate:g.activate,deactivate:g.deactivate}))};var f=ze.onButton(g);g.events.select.bind(function(a){e.action(a.value());g.close()});var h=D.create({select:M(["value","item"],g.events.select),itemHover:M(["item"],g.events.itemHover),escaped:M([],g.events.escaped),open:M(["menu","popup","anchor"],g.events.open),close:M([],g.events.close)});a();m.set(g.element(),"tabindex","-1");return{dropdown:function(){return{events:h.registry,
close:g.close,uiStale:g.uiStale,enable:g.enable,disable:g.disable,list:g.list,menu:g.menu,recalculate:g.recalculate}},component:g.element,refreshUi:a,destroy:function(){f.unbind();g.destroy()}}},NZ=function(a,b){var c=tf.typical(a.value,a.value,a.action,a.textkey,a.label,a.tooltip,b);a.setup&&a.setup(c,a);return c},OZ=function(a,b){var c=tf.toggle(a.value,a.value,a.action,a.textkey,a.label,a.tooltip,b);a.setup&&a.setup(c,a);a.selected&&c.select();return c},PZ=function(a,b){var c=!0===b.hasPopup,d=
(!0===b.toggle?OZ:NZ)(b,a);R.isString(b.icon)&&wl.findAndSetUrl(d.element(),b.icon);var e=d.initAria(d,a(b.textkey),c,b.toggle);m.set(d.element(),"tabindex","-1");var g=function(){!0===b.toggle&&ha.pressed(d)},l=ze.onButton(d);d.events.click.bind(function(){g()});var h=!0===b.toggle?d.select:f.noop,k=!0===b.toggle?d.deselect:f.noop,z=!0===b.toggle?d.selected:f.constant(!1);g();var w=D.create({click:M([],d.events.click)});return{generals:function(){return{events:w.registry,enableButton:d.enable,disableButton:d.disable}},
component:d.element,refreshUi:function(){b.updateGeneral(Dd.general({disable:e.disable,enable:e.enable,select:h,deselect:k,selected:z,element:d.element}));g()},destroy:function(){l.unbind();d.destroy();e.destroy()}}},QZ=Y.detect().deviceType.isiOS,RZ=function(a,b,c,d,e,g){var f=e.button(),h=ze.onButton(f),r=io(a.uiContainer,e.name,Gh.menu({focusActive:!0}));m.set(f.element(),"tabindex","-1");var z=P.div();n.add(z,ja.resolve("menu-system-main"));var w=Fh(r.sandbox,a.bounds,b,[f.element()],d),G=w.initMenu(z);
c=Pj();var q=rf.main(a.uiContainer,a.bounds,f.element,c,b.uiDirection(),Rd.anchored()),u=function(){q.recalculate(G)},x=function(){var a=y.children(r.sandbox());k.each(a,function(a){t.set(a,"display","none")})},A=function(a){r.update(a.structure());B.empty(z);v.append(z,a.element());a=w.registerMenus(a.submenus());a=k.map(a,function(a){return a.panel()});B.empty(r.sandbox());K.append(r.sandbox(),[G.panel()].concat(a));x()},C=function(a){e.init(a)},E=D.create({open:s(["menu","popup","anchor"]),close:M([],
r.events.close),itemAction:M(["value","item"],r.events.itemAction),itemHover:M(["item"],r.events.itemHover),reveal:s(["menu","trigger","popup","anchor"]),collapse:M(["menu","item"],r.events.collapse),escaped:M([],r.events.escaped)});f.events.click.bind(function(){return(f.selected()?G.open:G.close)()});e.events.done.bind(function(){r.close()});e.events.abort.bind(function(a){r.select(a.item());T.focus(a.item())});G.events.uistale.bind(function(){r.getActiveMenu().bind(function(a){C(a)})});G.events.close.bind(function(){r.close();
ha.collapsed(f.element())});G.events.open.bind(function(){var a=void 0!==g?g:[],a=[r.sandbox()].concat(a);A(e);C(e);t.set(G.panel(),"display","block");t.set(z,"display","block");r.open(a,z);ha.expanded(f.element())});r.events.open.bind(function(){E.trigger.open(e.element(),G,q);QZ()||r.setActiveMenu(e.element())});r.events.close.bind(function(){w.hideAll();f.deselect()});r.events.itemAction.bind(function(a){var b=e.handlers();if(void 0!==b[a.value()])(0,b[a.value()])(a.item())});r.events.reveal.bind(function(a){ha.expanded(a.trigger());
var b=w.expandMenu(a.menu(),a.trigger(),G);E.trigger.reveal(a.menu(),a.trigger(),b.popup,b.anchorage)});r.events.collapse.bind(function(a){w.hideMenu(a.menu());a.item().each(ha.collapsed)});return{menus:function(){return{events:E.registry,enable:e.enable,disable:e.disable,close:G.close,uiStale:G.uiStale,mousedown:G.mousedown,contentFocus:e.contentFocus,menu:G.element,refreshMenus:C,recalculate:u}},component:f.element,refreshUi:function(){e.updateMenu({enable:e.enable,disable:e.disable})},destroy:function(){h.unbind();
r.destroy();e.destroy();G.destroy()}}},SZ=function(a,b){var c=P.span();n.add(c,ja.resolve("palette-group"));var d=f.curry(Oa.speak,ca.body()),e=function(a){var b=Bh.isShowing(c);Bh.hide(c);b&&a.each(d);return b},g=Dd.palette({show:function(a){var b=Bh.isHiding(c);Bh.show(c);b&&a.each(d);return b},hide:e});e(h.none());K.append(c,k.map(b,function(a){return a.component()}));return{component:f.constant(c),destroy:function(){k.each(b,function(a){a.destroy()})},refreshUi:function(){k.each(b,function(a){a.refreshUi()});
a.updatePalette(g)}}},TZ=function(a,b,c,d,e){var g=e.instance();e=q.fromDom(document);var l=(g.toggle?tf.splitToggle:tf.split)(g.name(),g.name(),f.identity,g.textkey(),g.label(),g.tooltip,c),h=ze.custom(l.element(),function(){l.open();g.enter()});m.set(l.element(),"tabindex","-1");var k=l.initAria(l,c(g.textkey()),!0,g.toggle);d=Fh(a.uiContainer,a.bounds,b,[l.element()],d);var z=Pj(),w=bb.widget(e,g.name());e=La.generate("menu-control");m.set(w,"id",e);Q.controls(l.element(),e);Q.owns(l.element(),
e);Q.menu(w,c(g.menuTextkey()));v.append(w,g.element());var n=d.initMenu(w),I=rf.main(a.uiContainer,a.bounds,l.element,z,b.uiDirection(),Rd.expandable()),t=function(){I.recalculate(n)};n.events.open.bind(function(){ha.expanded(l.element());g.open();u.trigger.open(w,n,I)});n.events.close.bind(function(){ha.collapsed(l.element());g.close();l.close()});g.events.done.bind(function(){n.close();l.close()});g.events.abort.bind(function(){n.close();l.close();u.trigger.abort()});l.events.open.bind(function(){n.open();
I.recalculate(n)});l.events.close.bind(function(){g.close();n.close()});l.events.click.bind(function(){g.action()});var u=D.create({abort:s([]),close:M([],g.events.close),open:s(["menu","popup","anchor"])});return{widgets:function(){return{events:u.registry,open:l.open,isOpen:l.isOpen,close:l.close,recalculate:t}},component:l.element,refreshUi:function(){g.updateWidget(Dd.widget({select:l.select,deselect:l.deselect,enable:l.enable,disable:l.disable,press:k.press,release:k.release}))},destroy:function(){h.unbind();
k.destroy();n.destroy();g.destroy();l.destroy()}}},UZ=function(a,b,c,d,e){var g=e.instance(),l=tf.toggle(g.name(),g.name(),f.identity,g.textkey(),g.label(),g.tooltip,c),h=ze.onButton(l),k=l.initAria(l,c(g.textkey()),!0,!0);c=Fh(a.uiContainer,a.bounds,b,[l.element()],d);d=Pj();var z=c.initMenu(g.element()),w=rf.main(a.uiContainer,a.bounds,l.element,d,b.uiDirection(),Rd.expandable());m.set(l.element(),"tabindex","-1");n.add(z.element(),ja.resolve("toolbar-widget"));var G=function(){w.recalculate(z)};
z.events.open.bind(function(){ha.expanded(l.element());g.open();q.trigger.open(g.element(),z,w)});z.events.close.bind(function(){ha.collapsed(l.element());g.close()});g.events.done.bind(function(){z.close();l.deselect()});g.events.abort.bind(function(){z.close();l.deselect();q.trigger.abort()});l.events.click.bind(function(){l.selected()?(z.open(),w.recalculate(z)):z.close();ha.pressed(l)});var q=D.create({abort:s([]),open:s(["menu","popup","anchor"])}),t=function(){l.deselect();z.close()};return{widgets:function(){return{events:q.registry,
close:t,recalculate:G}},component:l.element,refreshUi:function(){g.updateWidget(Dd.widget({select:f.noop,deselect:f.noop,enable:k.enable,disable:k.disable,press:f.noop,release:f.noop}))},destroy:function(){h.unbind();k.destroy();z.destroy();g.destroy();l.destroy()}}},VZ=function(a,b){var c=b.instance(),d=ze.custom(c.element(),function(){c.element().dom().click()});m.remove(c.element(),"tabindex");n.add(c.element(),ja.resolve("toolbar-inline-widget"));Q.toolbarButton(c.element(),a(c.textkey()),!1);
var e=D.create({abort:s([]),open:s(["menu","popup","anchor"])});return{widgets:function(){return{events:e.registry,recalculate:f.noop,close:f.noop}},component:c.element,refreshUi:function(){c.updateInlineWidget(Dd.widget({select:f.noop,deselect:f.noop,enable:f.curry(ha.enable,c.element()),disable:f.curry(ha.disable,c.element()),press:f.noop,release:f.noop}))},destroy:function(){d.unbind();c.destroy()}}},dv={menu:function(a){return{general:f.curry(rZ,a),menu:f.curry(uZ,a),dropdown:f.curry(pZ,a),widget:f.curry(Sj.widget,
a),"split-widget":f.curry(Sj.split,a),"inline-widget":f.curry(Sj.inline,a),palette:f.die("No palettes for menus yet")}},toolbar:function(a,b,c,d){var e=function(e){return f.curry(e,a,b,c,d)};return{menu:e(RZ),dropdown:e(MZ),widget:e(UZ),"split-widget":e(TZ),"inline-widget":f.curry(VZ,c),general:f.curry(PZ,c),palette:SZ}}},qB=ma.generate([{toolbar:[]},{menu:[]}]),ko={toolbar:qB.toolbar,menu:qB.menu,cata:function(a,b,c){return a.fold(b,c)}},WZ=function(a,b,c,d){a.events.escaped.bind(function(a){c.focused()&&
T.focus(d.component())});a.events.close.bind(function(a){b.focus()})},XZ=function(a,b,c,d){a.events.open.bind(function(a){gc.register(a.popup().panel())})},NO={init:function(a,b,c,d,e,g,f,h,r){var z=a.dropdown(r),w=z.dropdown();a=c.toolbar;(Y.detect().deviceType.isTouch()?XZ:WZ)(w,e,a,z);var m=!1;w.events.open.bind(function(a){e.toReading();f.show(a.popup(),a.anchor());m=!0});w.events.close.bind(function(){m=!1});w.events.escaped.bind(function(){T.focus(z.component())});e.events.mousedown.bind(function(){m&&
w.close()});e.events.uiStale.bind(w.uiStale);c.toolbar.events.transitionend.bind(function(a){k.contains(["top","height"],a.property())&&w.recalculate()});return{component:z.component,refreshUi:z.refreshUi,destroy:z.destroy}}},MO={init:function(a,b,c,d,e,g,f,h,k){a=a.general(k);b=a.generals();a.refreshUi();return{generals:b,component:a.component,refreshUi:a.refreshUi,destroy:a.destroy}}},rB=function(a){var b=h.none(),c=function(){b=h.none()};return{begin:function(){b=h.from(Ea.capture(a))},end:function(){b.each(function(a){a.restore()});
c()},destroy:c}},Mk={fromSafe:function(a){return rB(a.doc())},fromUnsafe:function(a){a=a.doc().getOrDie();return rB(a)}},OO={init:function(a,b,c,d,e,g,l,p,k){var z=a.menu(k,[]),w=z.menus(),m=Mk.fromUnsafe(e);w.events.open.bind(function(a){e.toReading();w.refreshMenus(a.menu());l.show(a.popup(),a.anchor())});w.events.close.bind(function(){e.focus();m.end()});w.events.escaped.bind(function(){T.focus(z.component())});w.events.reveal.bind(function(a){k.init(a.menu());l.show(a.popup(),a.anchor())});w.events.itemAction.bind(function(a){m.end()});
w.events.itemHover.bind(function(a){m.begin();if(null!==document.activeElement&&void 0!==document.activeElement){var b=q.fromDom(document.activeElement),c=y.owner(a.item());"input"!==A.name(b)&&gc.preserve(c,f.curry(T.focusInside,a.item()))}else e.backupSelection(),T.focus(a.item())});e.events.mousedown.bind(w.mousedown);e.events.uiStale.bind(w.uiStale);e.events.focus.bind(w.enable);var n=!e.isInline()&&k.contentFocus();n&&w.disable();return{component:z.component,refreshUi:function(){z.refreshUi();
var a=n?h.none():h.some(!0);e.getSelection().or(a).fold(w.disable,w.enable)},destroy:function(){m.destroy();e.events.mousedown.unbind(w.mousedown);e.events.uiStale.unbind(w.update);z.destroy()}}}},Jq=function(a,b,c,d,e,g,f,h,k,z,w){var m=c[b]({instance:a(w.instance)});a=m.widgets();e=e.toolbar;a.events.open.bind(function(a){f.toReading();k.show(a.popup(),a.anchor())});a.events.abort.bind(function(){T.focus(m.component())});f.events.mousedown.bind(a.close);return(Y.detect().deviceType.isTouch()&&"split-widget"===
b?YZ:ZZ)(a,f,e,m)},ZZ=function(a,b,c,d){return d},YZ=function(a,b,c,d){var e=E.capture(d.component(),"click",function(b){b.kill();n.add(d.component(),Ta.resolve("statebutton-selected"));a.isOpen()?a.close():a.open()});a.events.close.bind(function(){n.remove(d.component(),Ta.resolve("statebutton-selected"))});return{widgets:d.widgets,component:d.component,refreshUi:d.refreshUi,destroy:function(){e.unbind();d.destroy()}}},lo={widget:f.curry(Jq,f.identity,"widget"),inlineWidget:f.curry(Jq,function(a){return function(){return a(ko.toolbar())}},
"inline-widget"),splitWidget:f.curry(Jq,f.identity,"split-widget")},$Z=x.resolve("editor-container"),sf={editorContainer:f.constant($Z)},Gl=C.immutable("key","link"),sB=C.immutable("url","text"),tB=function(a,b,c){return function(d){var e=sB(b,u.translate(c));return Gl(u.pkey(a,d),h.some(e))}},Hb=function(a){return function(b){return Gl(u.pkey(a,b),h.none())}},Kq=function(a){return function(b){var c=a(b);b=sB(b[0].websiteUrl,u.translate("cloud.dashboard"));return Gl(c.key(),h.some(b))}},a_={"no.api.key":Kq(Hb("cloud.error.apikey")),
"domain.not.supported":Kq(function(a){return Gl(u.pkey("cloud.error.domain",[a[0].details.domain]),h.none())}),"plan.exceeded":Kq(Hb("cloud.error.plan")),"error.code.invalid.toolbar":Hb("errors.toolbar.invalid"),"error.code.spelling.not.found":Hb("errors.spelling.missing.service"),"error.code.images.not.found":Hb("errors.images.missing.service"),"safari.imagepaste":tB("errors.safari.image","https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work","errors.safari.url"),
"webview.imagepaste":Hb("errors.webview.image"),"flash.clipboard.no.rtf":function(a){return(Y.detect().browser.isSafari()?tB("errors.safari.browser-settings","https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing","errors.safari.rtf"):Hb("errors.images.missing.flash"))(a)},"fileupload.invalidimages":Hb("errors.images.upload.invalid"),"error.imageupload":Hb("errors.images.upload.failed"),"error.imageupload.generic":Hb("errors.images.upload.unknown"),"error.imageupload.cors":Hb("errors.images.upload.cors"),
"errors.paste.word.notready":Hb("errors.paste.notready"),"errors.paste.process.failure":Hb("errors.paste.generic"),"flash.crashed":Hb("errors.flash.crashed"),"errors.imageimport.failed":Hb("errors.images.import.failed"),"errors.imageimport.unsupported":Hb("errors.images.import.unsupported"),"errors.imageimport.invalid":Hb("errors.images.import.invalid"),"imageedit.needsproxy":Hb("errors.images.edit.needsproxy"),"imageedit.proxyerror":Hb("errors.images.edit.proxyerror"),"imageedit.generic":Hb("errors.images.edit.generic")},
b_={"errors.local.images.disallowed":"Removing pasted images from content. Local image insert has been disabled.","errors.dragdrop.images.disallowed":"Blocking image drop operation. Local image insert has been disabled."},uB=function(a,b){var c=(0,a_[a])(b);return{description:f.constant(u.get(c.key())),link:f.constant(c.link().getOr(null))}},ae={get:uB,getOrLog:function(a,b){return h.from(b_[a]).fold(function(){var c=uB(a,b);return h.some(c)},function(a){console.warn(a);return h.none()})}},c_=function(a,
b){var c=function(b){var c=ae.get("error.code.invalid.toolbar",[b.small()]),c=Ch(c);a.displayBanner(c);k.each(b.big(),function(a){console.error(a)})};return b.fold(function(a){c(a);return[]},f.identity)},vB={setup:function(a,b,c,d,e,g){g=c_(a,g);var f=function(a){a=R.isString(a)?u.has(a)?u.key(a):u.raw(a):a;return u.get(a)};n.add(a.toolbar.element(),x.resolve("layer-above-editor"));b=IA.blueprints(LO(e.uiContainer,a,b,c,d,e,f),KO(f));e=IA.env(e.uiContainer,f);e=ZY.build(g,b,e);k.each(e,a.toolbar.add);
a.position();k.each(e,function(a){c.events.uiStale.bind(function(){a.refreshUi()})})}},hg={preserve:function(a,b){var c=y.owner(b),d=T.active(c).bind(function(a){var c=function(b){return L.eq(a,b)};return c(b)?h.some(b):sa.descendant(b,c)}),e=a(b);d.each(function(a){T.active(c).filter(function(b){return L.eq(b,a)}).orThunk(function(){T.focus(a)})});return e}},fv=C.immutable("component","index"),UO={item:f.constant('.ephox-chameleon-toolbar-group [tabindex="-1"]'),group:f.constant(".ephox-chameleon-toolbar-group"),
sector:f.constant(".ephox-chameleon-toolstrip > *")},wB=ja.resolve("toolbar-more-closed"),xB=ja.resolve("toolbar-more-open"),jv=ja.resolve("toolbar-more-hide"),iv=ja.resolve("toolbar-more-show"),oo=function(a,b){t.set(a,"height",b+"px");t.reflow(a)},no=function(a){n.remove(a,xB);n.add(a,wB);oo(a,0)},kv=function(a){n.remove(a,wB);n.add(a,xB);t.remove(a,"height")},Lq=C.immutable("within","extra","withinWidth"),yB=function(a,b,c){var d=ad.generate(a,function(a,b){var d=c(a);return h.some({element:f.constant(a),
start:f.constant(b),finish:f.constant(b+d),width:f.constant(d)})});a=k.filter(d,function(a){return a.finish()<b});var e=k.foldr(a,function(a,b){return a+b.width()},0),d=d.slice(a.length);return{within:f.constant(a),extra:f.constant(d),withinWidth:f.constant(e)}},Hl=function(a){return k.map(a,function(a){return a.element()})},d_=function(a,b,c){a=Hl(a.concat(b));return Lq(a,[],c)},e_=function(a,b,c,d){a=Hl(a).concat([c]);return Lq(a,Hl(b),d)},f_=function(a,b,c){return Lq(Hl(a),[],c)},g_=function(a,
b,c){a=yB(b,a,c);return 0===a.extra().length?h.some(a):h.none()},SO={partition:function(a,b,c,d){var e=g_(a,b,c).getOrThunk(function(){return yB(b,a-c(d),c)}),g=e.within(),f=e.extra(),e=e.withinWidth();return 1===f.length&&f[0].width()<=c(d)?d_(g,f,e):1<=f.length?e_(g,f,d,e):f_(g,f,e)}},pa={grid:C.immutableBag(["trigger"],["isValid","first"]),flow:C.immutableBag(["trigger"],["first","executeOnArrow"]),sectors:C.immutableBag(["trigger","first"],[]),radio:C.immutableBag(["one"],[]),input:C.immutableBag([],
["execute"]),button:C.immutableBag(["execute"],["process"]),dialog:C.immutableBag(["execute","ok","cancel"],["index"]),inlineDialog:C.immutableBag(["ok","cancel"],["index"]),page:C.immutableBag(["trigger"],["first"]),tabular:C.immutableBag(["trigger"],["first","isValid"]),dynamic:C.immutableBag([],["execute"])},rc=Gb.ensure(["enter","exit","destroy"]),zB=function(a){return function(b){return b.raw().which===a}},U={inSet:function(a){return function(b){return k.contains(a,b.raw().which)}},and:function(a){return function(b){return k.forall(a,
function(a){return a(b)})}},is:zB,isShift:function(a){return!0===a.raw().shiftKey},basic:function(a,b){return{matches:zB(a),run:b}},rule:function(a,b){return{matches:a,run:b}}},Mq=function(a){m.has(a,"tabIndex")||m.set(a,"tabIndex","-1")},Ma={set:function(a){Mq(a);T.focus(a)},find:function(a){return T.search(a)},isFocused:T.hasFocus,mark:Mq,markSelectors:function(a,b){k.each(b,function(b){b=X.descendants(a,b);k.each(b,Mq)})}},Gd={adjust:function(a,b,c,d){a+=b;return a>d?c:a<c?d:a},findPrev:function(a,
b,c){var d=k.reverse(a.slice(0,b));a=k.reverse(a.slice(b+1));return k.find(d.concat(a),c)},findNext:function(a,b,c){var d=a.slice(0,b);a=a.slice(b+1);return k.find(a.concat(d),c)}},AB=function(a){return k.find(a,Il)},Il=function(a){return Tb.isVisible(a.element())&&a.isEnabled()&&"hidden"!==t.get(a.element(),"visibility")},la={exit:function(a){k.each(a,function(a){a.clear();a.exit()})},clear:function(a){k.each(a,function(a){a.clear()})},refresh:function(a){k.each(a,function(a){a.refresh()})},first:AB,
enterFirst:function(a,b){b.index().bind(function(b){return h.from(a[b]).filter(Il)}).orThunk(function(){return AB(a)}).each(function(a){a.enter()})},isEnabled:function(a){return k.exists(a,function(a){return a.isEnabled()})},locate:function(a,b){return k.find(a,function(a){var d=b.target();a=a.element();return Gc.closest(d,f.curry(L.eq,a))}).getOr(void 0)},next:function(a,b){var c=k.indexOf(a,b).getOr(-1);return Gd.findNext(a,c,Il).getOr(b)},prev:function(a,b){var c=k.indexOf(a,b).getOr(-1);return Gd.findPrev(a,
c,Il).getOr(b)},prevNoCycle:function(a,b){var c=k.indexOf(a,b).getOr(-1);return 0<c?h.some(a[c-1]):h.none()},nextNoCycle:function(a,b){var c=k.indexOf(a,b).getOr(-1);return c+1<a.length?h.some(a[c+1]):h.none()},progress:function(a,b){a.exit();b.enter()},destroy:function(a){k.each(a,function(a){a.destroy()})}},h_=C.immutableBag(["index","focused","group","items"],[]),i_=C.immutableBag(["index","sectors","sector"],[]),j_=C.immutableBag(["index","candidates"],[]),Lf={ofFocus:function(a,b,c){return Ma.find(a).bind(function(a){return J.closest(a,
b).bind(function(b){var g=X.descendants(b,c),l=k.filter(g,Tb.isVisible);return k.findIndex(l,f.curry(L.eq,a)).map(function(c){return h_({index:c,focused:a,group:b,items:l})})})})},ofSector:function(a,b,c){var d=X.descendants(a,b);return J.closest(c,b).bind(function(a){return k.findIndex(d,f.curry(L.eq,a)).map(function(b){return i_({index:b,sectors:d,sector:a})})})},within:function(a,b,c){var d=X.descendants(a,c);return k.findIndex(d,f.curry(L.eq,b)).map(function(a){return j_({index:a,candidates:d})})}},
BB={horizontal:function(a,b,c,d){return Lf.within(a,c,b).bind(function(a){var b=a.index();a=a.candidates();b=Gd.adjust(b,d,0,a.length-1);return h.from(a[b])})}},Jl={delegate:function(a,b,c){return Ae(a,[U.basic(N.LEFT()[0],b.west),U.basic(N.RIGHT()[0],b.east),U.basic(N.UP()[0],b.north),U.basic(N.DOWN()[0],b.south),U.rule(U.inSet(N.SPACE().concat(N.ENTER())),function(){return Ma.find(a).bind(c.trigger()).map(f.always)})],h.none())},findRecent:function(a,b,c,d){return d.isValid().fold(f.constant(c),
function(a){return c.filter(a)}).orThunk(function(){return d.first().bind(function(a){return a()}).orThunk(function(){return J.descendant(a,b)})})}},k_=C.immutable("row","column"),$={set:function(a,b){if(void 0===b)throw Error("Value.set was undefined");a.dom().value=b},get:function(a){return a.dom().value}},l_={move:function(a,b,c){var d=C.immutable("row","col"),e=a.length-1,g=a[b.row()]-1,e=Gd.adjust(b.row(),c.dr(),0,e);a=a[e]-1;g=Math.max(g,a);b=Math.min(a,Gd.adjust(b.col(),c.dc(),0,g));return d(e,
b)}},CB={single:Ae,multiple:lv,multipleAll:function(a,b){var c=E.bind(a,"keydown",function(a){var c=la.locate(b,a);if(U.is(N.ESCAPE()[0])(a))d(),a.kill();else if(void 0!==c&&null!==c)if(U.is(N.UP()[0])(a))la.progress(c,la.prev(b,c)),a.kill();else if(U.is(N.DOWN()[0])(a))la.progress(c,la.next(b,c)),a.kill();else if(U.is(N.TAB()[0])(a)){var e=(U.isShift(a)?la.prev:la.next)(b,c);la.progress(c,e);a.kill()}}),d=function(){la.exit(b);e.trigger.exit()},e=D.create({enter:s([]),exit:s([])});return rc({enter:function(){la.first(b).each(function(a){a.enter()})},
exit:d,clear:f.curry(la.clear,b),refresh:f.curry(la.refresh,b),isEnabled:f.curry(la.isEnabled,b),isFocused:f.curry(Ma.isFocused,a),events:e.registry,destroy:function(){la.destroy(b);c.unbind()}})},flow:function(a,b,c){var d=h.none();Ma.markSelectors(a,[b.item()]);var e=function(c){return Ma.find(a).bind(function(d){return BB.horizontal(a,b.item(),d,c)})},g=Hh(a,function(a){return e(a).bind(function(a){k(a);return c.executeOnArrow().fold(function(){return h.some(!0)},l)})}),l=function(){return Ma.find(a).bind(c.trigger()).map(f.always)},
p=Ae(a,[U.rule(U.inSet(N.LEFT().concat(N.UP())),g.west),U.rule(U.inSet(N.RIGHT().concat(N.DOWN())),g.east),U.rule(U.inSet(N.SPACE().concat(N.ENTER())),l)],h.none()),k=function(a){d=h.some(a);Ma.set(a)};return rc({element:p.element,destroy:p.destroy,clickFocus:function(a){ra.is(a,b.item())&&k(a)},clear:function(){d=h.none()},isFocused:p.isFocused,isEnabled:f.constant(!0),refresh:f.noop,enter:function(){p.enter();d.orThunk(function(){return c.first().fold(function(){return J.descendant(a,b.item())},
f.apply)}).each(k)},exit:p.exit})},grid:function(a,b,c,d,e){var g=c,l=d;Ma.markSelectors(a,[b.cell()]);var p=wa(h.none()),k=function(a){return y.findIndex(a).map(function(a){return k_(Math.floor(a/l),a%l)})},z=function(c,d){return Ma.find(a).bind(function(e){return k(e).bind(function(e){var f=Gd.adjust(e.row(),d,0,g-1),p=Gd.adjust(e.column(),c,0,l-1);e=X.descendants(a,b.cell());var k=e.length-1,f=f*l,f=f+p>k?0<c?f:k:f+p;return 0<=f&&f<e.length?h.some(e[f]):h.none()})})};c=Hh(a,function(a,b){return z(a,
b).bind(function(a){Ma.set(a);p.set(h.some(a));return h.some(!0)})});var w=Jl.delegate(a,c,e);return rc({element:w.element,destroy:w.destroy,clear:function(){p.set(h.none())},isFocused:w.isFocused,isEnabled:f.always,refresh:f.noop,enter:function(){w.enter();p.set(Jl.findRecent(a,b.cell(),p.get(),e));p.get().each(T.focus)},exit:w.exit,resize:function(a,b){g=a;l=b}})},sectors:function(a,b,c){Ma.markSelectors(a,[b.sector(),b.group(),b.item()]);var d=function(a){return sa.descendant(a,function(a){return ra.is(a,
b.item())&&r(a)})},e=function(a){return sa.child(a,m)},g=function(a){a=pc.children(a,m);return 0<a.length?h.some(a[a.length-1]):h.none()},l=Hh(a,function(c){return Lf.ofFocus(a,b.group(),b.item()).bind(function(a){var b=a.items();a=Gd.adjust(a.index(),c,0,b.length-1);return h.from(b[a])})}),p=function(c){return Lf.ofFocus(a,b.group(),b.item()).bind(function(a){return c(a.group()).bind(d)})},r=function(a){return 0<ia.get(a)},z=function(c){return Lf.ofSector(a,b.sector(),c).map(function(a){return Gd.findPrev(a.sectors(),
a.index(),r).getOr(a.sector())})},w=function(c){return Lf.ofSector(a,b.sector(),c).map(function(a){return Gd.findNext(a.sectors(),a.index(),r).getOr(a.sector())})},m=function(a){var c;if(c=ra.is(a,b.group()))a=X.descendants(a,b.item()),c=k.exists(a,Tb.isVisible);return c},n=function(a){return function(){return a().map(function(a){Ma.set(a);q.trigger.active(a);return a})}},q=D.create({enter:s([]),exit:s([]),active:s(["element"])}),t=Ae(a,[U.rule(U.inSet(N.ESCAPE()),function(){t.exit();q.trigger.exit();
return h.some(!0)}),U.rule(U.inSet(N.RIGHT().concat(N.DOWN())),n(l.east)),U.rule(U.inSet(N.LEFT().concat(N.UP())),n(l.west)),U.rule(U.and([U.isShift,U.inSet(N.TAB())]),n(f.curry(p,function(a){return y.prevSibling(a).filter(m).orThunk(function(){return z(a).bind(g)})}))),U.rule(U.inSet(N.TAB()),n(f.curry(p,function(a){return y.nextSibling(a).filter(m).orThunk(function(){return w(a).bind(e)})}))),U.rule(U.inSet(N.SPACE().concat(N.ENTER())),function(){return Ma.find(a).bind(c.trigger()).map(f.always)})],
h.none());return rc({element:t.element,clear:f.noop,isFocused:t.isFocused,enter:function(){t.enter();q.trigger.enter();c.first()().orThunk(function(){return d(a)}).each(function(a){Ma.set(a);q.trigger.active(a)})},refresh:f.noop,isEnabled:f.always,exit:t.exit,destroy:t.destroy,events:q.registry})},radio:function(a,b,c){var d=function(c){return Ma.find(a).bind(function(d){return BB.horizontal(a,b.item(),d,c)})},e=Hh(a,function(b){return d(b).bind(function(b){Ma.set(b);c.one()(a,b);return h.some(!0)})}),
g=Ae(a,[U.rule(U.inSet(N.LEFT().concat(N.UP())),e.west),U.rule(U.inSet(N.RIGHT().concat(N.DOWN())),e.east),U.rule(U.inSet(N.SPACE()),f.curry(h.some,!0))],h.none());return rc({element:g.element,destroy:g.destroy,clear:f.noop,refresh:function(){g.isFocused()&&J.descendant(a,b.selected()).each(Ma.set)},isFocused:g.isFocused,isEnabled:f.always,enter:function(){g.enter();J.descendant(a,b.selected()).orThunk(function(){return J.descendant(a,b.item())}).each(function(d){ra.is(d,b.selected())||c.one()(a,
d);Ma.set(d)})},exit:g.exit})},input:function(a,b){var c=Ae(a,[U.basic(N.ENTER()[0],function(c){return b.execute().map(function(b){var g={isShift:f.constant(U.isShift(c))},l=$.get(a);b(l,g);return!0})})],h.none());return rc({element:c.element,destroy:c.destroy,clear:c.clear,isFocused:c.isFocused,isEnabled:function(){return!m.has(a,"disabled")},refresh:f.noop,enter:function(){c.enter();a.dom().setSelectionRange(0,$.get(a).length)},exit:c.exit})},dialog:function(a,b,c){var d=c.execute(),e=pa.button({execute:d}),
g=b.slice(0).concat([po(c.cancel(),e),po(c.ok(),e)]),l=E.bind(a,"keydown",function(a){var b=la.locate(g,a);if(void 0!==b&&null!==b){var e=U.isShift(a);U.is(N.TAB()[0])(a)?(e=(e?la.prev:la.next)(g,b),la.progress(b,e),a.kill()):U.is(N.ESCAPE()[0])(a)?(h(),a.kill(),d(c.cancel())):(b=b.element(),"button"===A.name(b)||e||!U.is(N.ENTER()[0])(a)?(b=[N.UP()[0],N.DOWN()[0]],e=["textarea","select","input"],U.inSet(b)(a)&&!1===k.contains(e,A.name(a.target()))&&a.kill()):(d(c.ok()),a.kill()))}}),h=function(){la.exit(g);
r.trigger.exit()},r=D.create({enter:s([]),exit:s([])});return rc({enter:function(){la.enterFirst(g,c)},exit:h,clear:f.curry(la.clear,g),refresh:f.curry(la.refresh,g),isEnabled:f.curry(la.isEnabled,g),isFocused:f.curry(Ma.isFocused,a),events:r.registry,destroy:function(){la.destroy(g);l.unbind()}})},button:po,inlineDialog:function(a,b,c){var d=c.ok(),e=c.cancel(),g=E.bind(a,"keydown",function(a){var c=la.locate(b,a);if(void 0!==c&&null!==c){var g=U.isShift(a);U.is(N.TAB()[0])(a)?(g=(g?la.prev:la.next)(b,
c),la.progress(c,g),a.kill()):U.is(N.ESCAPE()[0])(a)?(l(),a.kill(),e(a.target())):!g&&U.is(N.ENTER()[0])(a)?(d(a.target()),a.kill()):(c=[N.UP()[0],N.DOWN()[0]],g=["textarea","select","input"],U.inSet(c)(a)&&!1===k.contains(g,A.name(a.target()))&&a.kill())}}),l=function(){la.exit(b);h.trigger.exit()},h=D.create({enter:s([]),exit:s([])});return rc({enter:function(){la.enterFirst(b,c)},exit:l,clear:f.curry(la.clear,b),refresh:f.curry(la.refresh,b),isEnabled:f.curry(la.isEnabled,b),isFocused:f.curry(Ma.isFocused,
a),events:h.registry,destroy:function(){la.destroy(b);g.unbind()}})},tabular:function(a,b,c){var d=b.row(),e=b.cell();Ma.markSelectors(a,[d,e]);var g=wa(h.none()),l=Hh(a,function(a,b){return n(a,b).bind(function(a){Ma.set(a);g.set(h.some(a));return h.some(!0)})}),p=C.immutable("row","col"),r=C.immutable("dc","dr"),z=function(a){a=X.descendants(a,d);return k.map(a,function(a){return X.descendants(a,e).length})},w=function(b){return J.closest(b,d).bind(function(c){return Lf.within(a,c,d).bind(function(a){return Lf.within(c,
b,e).map(function(b){return p(a.index(),b.index())})})})},m=function(b){var c=X.descendants(a,d);return h.from(c[b.row()]).bind(function(a){a=X.descendants(a,e);return h.from(a[b.col()])})},n=function(b,c){return Ma.find(a).bind(function(d){return w(d).bind(function(d){var e=z(a);d=l_.move(e,d,r(b,c));return m(d)})})},q=Jl.delegate(a,l,c);return rc({element:q.element,destroy:q.destroy,clear:function(){g.set(h.none())},isFocused:q.isFocused,isEnabled:f.always,refresh:f.noop,enter:function(){q.enter();
g.set(Jl.findRecent(a,b.cell(),g.get(),c));g.get().each(Ma.set)},exit:q.exit})},page:function(a,b,c){var d=b.stops().join(","),e=function(){return c.first().bind(function(b){return J.descendant(a,b)}).getOr(a)},g=function(a){var b=e();return(T.hasFocus(b)?h.some:Ma.find)(b).bind(function(b){var c;c=e();if(L.eq(c,b)&&0>a)c=h.none();else{var g=X.descendants(c,d);b=k.findIndex(g,f.curry(L.eq,b)).getOr(-1)+a;c=b>=g.length?h.none():0>b?h.some(c):0<=b&&b<g.length?h.some(g[b]):h.none()}return c})};b=function(a){return g(a).bind(function(a){Ma.set(a);
return h.some(!0)})};var l=Ae(a,[U.rule(U.and([U.isShift,U.inSet(N.TAB())]),f.curry(b,-1)),U.rule(U.inSet(N.TAB()),f.curry(b,1)),U.rule(U.inSet(N.SPACE().concat(N.ENTER())),function(){return Ma.find(a).bind(c.trigger()).map(f.always)})],h.none());return rc({element:l.element,destroy:l.destroy,clear:f.noop,isFocused:l.isFocused,isEnabled:f.always,refresh:f.noop,enter:function(){l.enter();Ma.set(e())},exit:l.exit})},dynamic:function(a){var b=wa([]),c=function(a,c){U.is(N.TAB()[0])(c)&&(U.isShift(c)?
la.prevNoCycle:la.nextNoCycle)(b.get(),a).each(function(b){la.progress(a,b);c.kill()})},d=E.bind(a,"keydown",function(a){var d=la.locate(b.get(),a);void 0!==d&&null!==d&&c(d,a)}),e=D.create({enter:s([]),exit:s([])}),g=function(a){return function(){return a(b.get())}};return rc({element:f.constant(a),enter:function(){la.first(b.get()).each(function(a){a.enter()})},exit:function(){la.exit(b.get());e.trigger.exit()},set:function(a){la.destroy(b.get());b.set(a)},clear:g(la.clear),refresh:g(la.refresh),
isEnabled:g(la.isEnabled),isFocused:f.curry(Ma.isFocused,a),destroy:function(){la.destroy(b.get());d.unbind()},events:e.registry})}},m_={grid:function(a,b,c,d,e){Ma.markSelectors(a,[b.cell()])}},n_=function(a,b){var c=h.from(m_[b]).getOr(f.noop);return function(){c.apply(null,arguments);return{element:f.noop,clear:f.noop,isFocused:f.noop,isEnabled:f.noop,refresh:f.noop,enter:f.noop,exit:f.noop,events:D.create({enter:s([]),exit:s([]),active:s([])}).registry,destroy:f.noop,set:f.noop,clickFocus:f.noop,
resize:f.noop}}},ba=Y.detect().deviceType.isTouch()?W.map(CB,n_):CB,Sd={toolbar:function(a,b){return TO(a,b)},menu:function(a,b,c){return FA.create(a,b,c)},popup:function(a,b,c,d,e){return Fh(a,b,c,d,e)},menuSystem:function(a,b,c){return io(a,b,c)}},jd={styleSpan:f.constant("ephox-hare-style-span"),base:f.constant("ephox-candy-mountain"),active:f.constant("ephox-candy-mountain-active"),autoresize:f.constant("ephox-candy-mountain-autoresize")},DB=Y.detect(),Kl=DB.deviceType.isTouch,Ll=function(a,b){return window.screen.width>=
a&&window.screen.height>=b},EB=function(){return!Ll(620,700)&&Kl()},Mb={isTouch:Kl,choice:function(a,b){return k.foldl(a,function(a,b){return a.orThunk(function(){return b.predicate()?h.some(b.value()):h.none()})},h.none()).getOr(b)},isLarge:function(){return Ll(620,700)},isLargeTouch:function(){return Ll(620,700)&&Kl()},isSmallTouch:EB,isLargeDesktop:function(){return Ll(620,700)&&!Kl()},isSmallAndroid:function(){return EB()&&DB.deviceType.isAndroid()}},Mf=Y.detect(),o_={"ephox-platform-desktop":!Mf.deviceType.isTouch(),
"ephox-platform-touch":Mf.deviceType.isTouch(),"ephox-platform-small-touch":Mb.isSmallTouch(),"ephox-platform-android":Mf.deviceType.isAndroid(),"ephox-platform-ios":Mf.deviceType.isiOS(),"ephox-platform-ie":Mf.browser.isIE()||Mf.browser.isEdge(),"ephox-platform-firefox":Mf.browser.isFirefox()},FB=[];W.each(o_,function(a,b){!0===a&&FB.push(b)});var Go={addClasses:function(a){ea.add(a,FB)}},GB=function(a,b){var c=q.fromTag("div");n.add(c,x.resolve(b));v.append(c,a);return c},vv={build:function(a,b,
c){h.from(b).filter(mi.isPrototypeOf).map(q.fromDom).each(function(b){var c=q.fromTag("div");n.add(c,x.resolve("flex-row"));var g=q.fromTag("div");n.add(g,x.resolve("flex-column"));K.append(g,y.children(a));v.append(a,c);v.append(c,g);v.append(c,GB(b,"side-bar"))});h.from(c).filter(mi.isPrototypeOf).map(q.fromDom).each(function(b){v.append(a,GB(b,"bottom-bar"))})}},p_=Gb.exactly(["compare","extract","mutate","sink"]),q_=Gb.exactly(["element","start","stop","destroy"]),r_=Gb.exactly(["forceDrop","drop",
"move","delayDrop"]),qk={mode:p_,sink:q_,api:r_},ov={resolve:yl.css("ephox-dragster").resolve},s_=qk.mode({compare:function(a,b){return Ba(b.left()-a.left(),b.top()-a.top())},extract:function(a){return h.some(Ba(a.x(),a.y()))},sink:function(a,b){var c=WO(b),d=E.bind(c.element(),"mousedown",a.forceDrop),e=E.bind(c.element(),"mouseup",a.drop),g=E.bind(c.element(),"mousemove",a.move),f=E.bind(c.element(),"mouseout",a.delayDrop);return qk.sink({element:c.element,start:function(a){v.append(a,c.element())},
stop:function(){B.remove(c.element())},destroy:function(){c.destroy();e.unbind();g.unbind();f.unbind();d.unbind()}})},mutate:function(a,b){a.mutate(b.left(),b.top())}}),t_={setup:function(a,b,c){var d=!1,e=D.create({start:s([]),stop:s([])}),g=ZO(),f=function(){m.stop();g.isOn()&&(g.off(),e.trigger.stop())},h=zc.last(f,200);g.events.move.bind(function(c){b.mutate(a,c.info())});var k=function(a){return function(){var b=Array.prototype.slice.call(arguments,0);if(d)return a.apply(null,b)}},m=b.sink(qk.api({forceDrop:f,
drop:k(f),move:k(function(a,c){h.cancel();g.onEvent(a,b)}),delayDrop:k(h.throttle)}),c);return{element:m.element,go:function(a){m.start(a);g.on();e.trigger.start()},on:function(){d=!0},off:function(){d=!1},destroy:function(){m.destroy()},events:e.registry}}},Ih={transform:function(a,b){var c=void 0!==b?b:{};return t_.setup(a,void 0!==c.mode?c.mode:s_,b)}},aP={both:function(a){var b=D.create({relocate:s(["x","y","deltaX","deltaY"])});return{mutate:function(c,d){var e=Na.absolute(a),g=e.left()+c,e=
e.top()+d;b.trigger.relocate(g,e,c,d)},events:b.registry}}},HB=C.immutable("hx","hy","position","x","y"),u_=function(a,b,c,d,e,g){g=b.x();b=b.y()-c;return(b>=a.y()&&b<=a.y()+a.h()?[HB(g,b,"absolute",g,b)]:[]).concat([HB(e,a.y(),"fixed",e-d.left(),a.fy())])},v_={detect:function(a,b,c,d,e,g){a=u_(a,b,e,g,c,d);return k.find(a,function(a){return 10>Math.abs(d-a.hy())&&38>Math.abs(c-a.hx())})}},IB={box:function(a){var b=Na.absolute(a),c=qa.getOuter(a);a=ia.getOuter(a);var d=b.left(),b=b.top();return{x:f.constant(d),
y:f.constant(b),width:f.constant(c),height:f.constant(a),right:f.constant(d+c),bottom:f.constant(b+a)}}},Nq={resolve:lb.create("ephox-ego").resolve},Ml=function(a){var b="data-"+Nq.resolve(a+"-left"),c="data-"+Nq.resolve(a+"-top"),d="data-"+Nq.resolve(a+"-position");return{get:function(a){var g=parseInt(m.get(a,b),10),l=parseInt(m.get(a,c),10);a=m.get(a,d);return isNaN(g)||isNaN(l)?h.none():h.some({left:f.constant(g),top:f.constant(l),position:f.constant(a)})},set:function(a,g,f,h){m.set(a,d,g);m.set(a,
b,f+"px");m.set(a,c,h+"px")},clear:function(a){m.remove(a,d);m.remove(a,b);m.remove(a,c)}}}("pre-snap"),w_=function(a,b,c,d,e,g){return Ml.get(b).fold(function(){return a.fromLocation(b,c,d)},function(c){return a.fromFixed(b,c.left()+e,c.top()+g)})},x_=function(a,b,c,d){var e=y.owner(b),e=Ea.get(e);b=IB.box(b);c=c.loc();return v_.detect(a,b,c.left(),c.top(),d,e)},Oq=function(a,b,c,d){return{attrs:{"data-ephox-snap":"absolute"===b?"true":"false"},styles:{position:b,left:a.left+c+"px",top:a.top+d+"px"}}},
zi={move:function(a,b,c,d,e,g){var f=a.graph(),h=a.lazyTheatre(),k=a.lazyHeight();c=w_(f,b,c,d,e,g);var m=c.loc(),w=c.fixed(),n={top:0,left:0};return x_(h,a.container(),c,k).fold(function(){var a=f.boundToFixed(h,b,m.left(),m.top(),w.left(),w.top(),k);Ml.set(b,"fixed",a.left(),a.top());return Oq(n,"fixed",a.left(),a.top())},function(c){var d=f.boundToFixed(h,b,m.left(),m.top(),w.left(),w.top(),k);Ml.set(b,"fixed",d.left(),d.top());d="absolute"===c.position()?f.locationToAbsolute(b,c.x(),c.y()):Ba(c.x(),
c.y());a.resetOffset();return Oq(n,c.position(),d.left(),d.top())})},getSnapAttrs:Oq,forget:function(a){Ml.clear(a)}},sv={none:function(a,b){var c=D.create({dragend:s([]),moved:s([])});return{blocker:f.die("no blocker"),setState:f.noop,getState:f.constant({}),init:function(d,e){var g=zi.getSnapAttrs(a.getOffset(),"absolute",d,e);t.setAll(b,g.styles);m.setAll(b,g.attrs);c.trigger.moved()},update:f.noop,destroy:f.noop,events:c.registry}},except:function(a){return function(b,c){var d=$O(c,a.join(",")),
e=D.create({dragend:M([],d.events.dragend),moved:s([])});d.events.relocate.bind(function(a){var d=a.x(),e=a.y(),f=a.deltaX();a=a.deltaY();d=zi.move(b,c,d,e,f,a);t.setAll(c,d.styles);m.setAll(c,d.attrs)});d.events.dragend.bind(function(){e.trigger.moved()});return{blocker:d.blocker,setState:function(a){t.setAll(c,a.styles);m.setAll(c,a.attrs)},getState:function(){var a=Na.viewport(c);return zi.getSnapAttrs(b.getOffset(),"absolute",a.left(),a.top())},init:function(a,d){var f=zi.getSnapAttrs(b.getOffset(),
"absolute",a,d);t.setAll(c,f.styles);m.setAll(c,f.attrs);e.trigger.moved()},update:function(){zi.forget(c)},destroy:function(){d.destroy()},events:e.registry}}}},JB={isInside:function(a,b,c,d){return!(b>a.y()+a.h()||c-d<a.y())},isAllInside:function(a,b,c){return b>a.y()&&b<a.y()+a.h()&&c>a.y()&&c<a.y()+a.h()}},y_=function(a,b,c,d,e,g){if("true"===m.get(c,"data-ephox-snap")){e=Na.absolute(e);var f=e.top()+g.top;if(JB.isAllInside(b,f,f+d()))return a=a.locationToAbsolute(c,e.left(),f),h.some({position:"absolute",
left:g.left+a.left()+"px",top:a.top()+"px"});a=g.top+e.top()-d()<=b.y()?b.fy():b.fy()+b.h()-d();return h.some({position:"fixed",left:g.left+e.left()+"px",top:a+"px"})}return h.none()},z_={update:function(a,b,c,d,e,g){y_(a,b,c,d,e,g).each(function(a){t.setAll(c,a)})},getState:function(a){return"true"===m.get(a,"data-ephox-snap")}},tv={register:function(a,b,c,d){var e=a.container(),g=c(a,b),f=d(a,b);c=D.create({dragend:M([],g.events.dragend),appear:M(["element"],f.events.appear),disappear:M(["element"],
f.events.disappear)});g.events.moved.bind(function(){k();p=!0});f.events.scrolled.bind(function(){k()});f.events.appear.bind(function(){var c=a.lazyTheatre(),d=a.graph();z_.update(d,c,b,a.lazyHeight,e,a.getOffset())});var p=!1,k=function(){g.update();f.update();var c=a.lazyTheatre(),d=Na.absolute(b),c=Math.max(0,c.x()+c.w()-d.left());t.set(b,"max-width",c+"px")};return{blocker:g.blocker,setState:function(a){g.setState(a);k();p=!0},state:function(){return p?h.some(g.getState()):h.none()},init:function(b,
c,d){a.setOffset(d);g.init(b,c);k()},update:k,destroy:function(){f.destroy();g.destroy()},events:c.registry}}},rv={transitions:function(a,b){var c=a.container(),d=y.defaultView(b),e=D.create({appear:s(["element"]),disappear:s(["element"]),scrolled:s([])}),g=E.bind(d,"scroll",function(){e.trigger.scrolled()});return{update:function(){var d=a.lazyTheatre(),g=a.lazyHeight(),f=IB.box(c);JB.isInside(d,f.y(),f.bottom(),g)?e.trigger.appear(b):e.trigger.disappear(b)},destroy:function(){g.unbind()},events:e.registry}}},
pv=C.immutable("fixed","loc"),cP=function(a,b,c,d){b=bP(b);var e={top:0,left:0},g=function(a){e.top=a.top;e.left=a.left};return{lazyTheatre:a,graph:f.constant(b),lazyHeight:c,container:f.constant(d),setOffset:g,resetOffset:function(){g({top:-1*c(),left:0})},getOffset:function(){return e}}},Pq=x.resolve("drag-show"),KB=x.resolve("fade-container"),bc={width:function(a,b){return t.getRaw(a,"max-width").fold(function(){return y.owner(b).dom().defaultView.innerWidth-Na.absolute(b).left()},function(a){return parseInt(a,
10)})},updateWidth:function(a,b,c){b=0===c.extra().length?c.withinWidth():c.withinWidth()+qa.getOuter(b.group());0<b&&t.set(a,"width",b+"px")},clearWidth:function(a){t.remove(a,"width")},offsetFrom:function(a,b,c,d,e){b=void 0!==d.toolbar.offset?d.toolbar.offset:e?{left:0,top:0}:{left:0,top:-1*c};c=Na.absolute(a.element());a=a.isInline()?{left:c.left(),top:c.top()}:{left:0,top:0};return{left:a.left+b.left,top:a.top+b.top}},initFade:function(a,b){n.add(b,Pq)},fadeIn:function(a,b){n.remove(a,KB);n.add(b,
Pq)},fadeOut:function(a,b){n.remove(b,Pq);n.add(a,KB)}},gP={setup:function(a,b,c,d,e){return(!0===c.toolbar.draggable?dP:e||void 0!==c.toolbar.offset||a.isInline()?eP:nv)(a,b,c,e)}},Uj=Y.detect().browser,uv=function(a,b){a.updateUi(b);var c=[sf.editorContainer(),"ephox-textbox-font",x.resolve("layer-editor")];ea.add(b,c);m.set(b,"lang",fe.get());Go.addClasses(b)},A_=function(a){a=pe.absolute(a);var b=pe.view();a=JY.intersect(a,b);return Ve(a.x(),a.y(),a.width(),a.height())},xp={build:function(a,b,
c,d,e,g,f,h){var k=fP(a,b,c,g,f,h),m=d.tabbed(k);m.init();return{element:k.element,socket:k.socket,viewport:k.viewport,uiContainerParent:k.uiContainerParent,init:function(b,d,f){d=m.config(a.isInline(),d,e,f);vB.setup(k,g,a,b,c,d);m.loaded()},connect:k.connect,destroy:function(){k.destroy();m.destroy()},displayBanner:k.displayBanner,hideBanner:k.hideBanner,position:k.position,refresh:k.refresh,tabstrip:k.toolbar,state:k.state,dialogContainer:k.dialogContainer}},bounds:function(){var a=function(a){var c=
function(){return h.from(a.bounds()).filter(mi.isPrototypeOf).map(q.fromDom).map(A_).getOrThunk(pe.view)};return R.isFunction(a.bounds)?c:pe.view};return Y.detect().deviceType.isTouch()?f.constant(pe.view):a}()},ky={settings:function(a){return{onprem:!0,blocktag:{styles:[{rule:"p"},{rule:"h1"},{rule:"h2"},{rule:"h3"},{rule:"h4"},{rule:"div"},{rule:"pre"}],bundled:!0},image:{bundled:!0},imageEdit:{bundled:!0},imageupload:{immediate:!1,bundled:!1},content:{css:a+"/css",tabindex:null,stylesheets:[],
documentStyles:"",prettyprint:!0,autoresize:!1},ui:{layout:xp,isFullScreen:!1,toolbar:{draggable:!1},locale:{},editorName:H.value("debug"),bounds:null,showHelpHint:!1,autoresize:!1},shortcuts:{features:!0},spelling:{bundled:!0,autocorrect:!0,scanView:!0,multilingual:!0,menu:!0},specialchar:{bundled:!0},fileupload:{bundled:!0,allowLocalImages:!0},horizontalrule:{bundled:!0},media:{bundled:!0},mentions:{bundled:!1},registration:{bundled:!1},help:{baseUrl:a+"/html",integration:{},bundled:!0},draganddrop:{bundled:!0},
wordcount:{bundled:!0},formatting:{bundled:!0},alignindent:{bundled:!0},directionality:{bundled:!0},parlance:{bundled:!0,languages:"en_us en_gb es fr de pt zh".split(" ")},font:{faces:[],bundled:!0},lists:{bundled:!0},link:{bundled:!0},linkvalidator:{bundled:!1},bookmark:{bundled:!0},removeformat:{bundled:!0},fullscreen:{bundled:!0},editorcontext:{bundled:!0},macros:{whitelist:h.none(),bundled:!0},highlighter:{external:[],bundled:!0},table:{bundled:!0},contextmenu:{bundled:!0},project:{apiKey:null},
smartclipboard:{bundled:!0,copy:{},paste:{forceText:!1,baseUrl:a+"/js",swf:a+"/flash/textboxpaste.swf",officeStyles:"prompt",htmlStyles:"merge",allowLocalImages:!0,enableFlashImport:!0}},resize:{bundled:!0},platform:{toolbar:{items:IY,contextual:["table-tools","image-tools"]},allowLocalImages:!0,fillscreen:!1},codeview:{bundled:!0,showButton:!0,mode:"pencil",css:a+"/css",js:a+"/js"},findreplace:{bundled:!0},accessibility:{bundled:!0},plaincodeview:{bundled:!0},embed:{inlineStyles:!0,bundled:!1}}},
state:function(){return{spelling:{words:{},language:h.none()},undo:{initial:{raw:"",marked:""},undo:[],redo:[]},profile:{spelling:!1,nativeSpelling:!0,autocorrect:!0,capitalisation:!1,linkpreviews:!1},content:{raw:"",marked:""},ui:{toolbar:{drag:h.none()}},macros:{customs:{}},filters:{customs:[]},laundry:{dirty:!1},linkvalidator:{ignore:[],cache:{}},embed:{cache:{}}}}},Qq=C.immutable("item","mode"),Rq=function(a,b,c,d){var e=void 0!==d?d:Ai;return a.property().parent(b).map(function(a){return Qq(a,
e)})},Ai=function(a,b,c,d){var e=void 0!==d?d:Fg;return c.sibling(a,b).map(function(a){return Qq(a,e)})},Fg=function(a,b,c,d){var e=void 0!==d?d:Fg;a=a.property().children(b);return c.first(a).map(function(a){return Qq(a,e)})},B_=[{current:Rq,next:Ai,fallback:h.none()},{current:Ai,next:Fg,fallback:h.some(Rq)},{current:Fg,next:Fg,fallback:h.some(Ai)}],LB=function(a,b,c,d,e){e=void 0!==e?e:B_;return k.find(e,function(a){return a.current===c}).bind(function(c){return c.current(a,b,d,c.next).orThunk(function(){return c.fallback.bind(function(c){return LB(a,
b,c,d)})})})},Nf={backtrack:Rq,sidestep:Ai,advance:Fg,go:LB},Nl={left:function(){return{sibling:function(a,b){return a.query().prevSibling(b)},first:function(a){return 0<a.length?h.some(a[a.length-1]):h.none()}}},right:function(){return{sibling:function(a,b){return a.query().nextSibling(b)},first:function(a){return 0<a.length?h.some(a[0]):h.none()}}}},Sq=function(a,b,c,d,e,g){return Nf.go(a,b,d,e).bind(function(b){return g(b.item())?h.none():c(b.item())?h.some(b.item()):Sq(a,b.item(),c,b.mode(),e,
g)})},Ol={left:function(a,b,c,d){return Sq(a,b,c,Nf.sidestep,Nl.left(),d)},right:function(a,b,c,d){return Sq(a,b,c,Nf.sidestep,Nl.right(),d)}},MB=function(a,b){return 0===a.property().children(b).length},NB=function(a,b,c,d){return Ol.left(a,b,c,d)},OB=function(a,b,c,d){return Ol.right(a,b,c,d)},na={before:function(a,b,c){return NB(a,b,f.curry(MB,a),c)},after:function(a,b,c){return OB(a,b,f.curry(MB,a),c)},seekLeft:NB,seekRight:OB,walkers:function(){return{left:Nl.left,right:Nl.right}},walk:function(a,
b,c,d,e){return Nf.go(a,b,c,d,e)},backtrack:Nf.backtrack,sidestep:Nf.sidestep,advance:Nf.advance},Gg=Sa(),Lb={gather:function(a,b,c){return na.gather(Gg,a,b,c)},before:function(a,b){return na.before(Gg,a,b)},after:function(a,b){return na.after(Gg,a,b)},seekLeft:function(a,b,c){return na.seekLeft(Gg,a,b,c)},seekRight:function(a,b,c){return na.seekRight(Gg,a,b,c)},walkers:function(){return na.walkers()},walk:function(a,b,c,d){return na.walk(Gg,a,b,c,d)}},PB=function(a){return q.fromDom(a.dom().cloneNode(!0))},
Tq=function(a,b){var c=q.fromTag(b),d=m.clone(a);m.setAll(c,d);return c},ec={shallow:function(a){return q.fromDom(a.dom().cloneNode(!1))},shallowAs:Tq,deep:PB,copy:function(a,b){var c=Tq(a,b),d=y.children(PB(a));K.append(c,d);return c},mutate:function(a,b){var c=Tq(a,b);v.before(a,c);var d=y.children(a);K.append(c,d);B.remove(a);return c}},yv={convertToHeading:function(a,b){var c=y.owner(a),c=q.fromTag(b,c.dom()),d=y.children(a),d=k.map(d,ec.deep);K.append(c,d);d=X.descendants(c,"b,strong");k.each(d,
B.unwrap);v.before(a,c);B.remove(a);return H.value(c)},joinLinks:function(a,b){return Lb.seekLeft(b,function(a){return"a"===A.name(a)},a).fold(function(){return H.value(b)},function(a){var d=y.children(b);K.append(a,d);B.remove(b);return H.value(a)})}},QB=C.immutable("row","y"),RB=C.immutable("col","x"),SB=function(a){return Na.absolute(a).left()+qa.getOuter(a)},TB=function(a){return Na.absolute(a).left()},UB=function(a,b){return RB(a,TB(b))},VB=function(a,b){return RB(a,SB(b))},Uq=function(a){return Na.absolute(a).top()},
Vq=function(a,b,c){if(0===c.length)return[];var d=k.map(c.slice(1),function(b,c){return b.map(function(b){return a(c,b)})}),e=c[c.length-1].map(function(a){return b(c.length-1,a)});return d.concat([e])},C_={delta:f.identity,positions:f.curry(Vq,function(a,b){return QB(a,Uq(b))},function(a,b){return QB(a,Uq(b)+ia.getOuter(b))}),edge:Uq},D_={delta:f.identity,edge:TB,positions:f.curry(Vq,UB,VB)},E_={delta:function(a,b){return-a},edge:SB,positions:f.curry(Vq,VB,UB)},hi={height:C_,rtl:E_,ltr:D_},Vj={ltr:hi.ltr,
rtl:hi.rtl},WB=function(a){return"img"===A.name(a)?1:Da.getOption(a).fold(function(){return y.children(a).length},function(a){return a.length})},F_=function(a){return Da.getOption(a).filter(function(a){return 0!==a.trim().length||-1<a.indexOf("\u00a0")}).isSome()},G_=["img","br"],ka={getEnd:WB,isEnd:function(a,b){return WB(a)===b},isStart:function(a,b){return 0===b},isCursorPosition:function(a){return F_(a)||k.contains(G_,A.name(a))}},H_=function(a,b){var c=function(a){a=y.children(a);for(var e=a.length-
1;0<=e;e--){var g=a[e];if(b(g))return h.some(g);g=c(g);if(g.isSome())return g}return h.none()};return c(a)},Cb={first:function(a){return sa.descendant(a,ka.isCursorPosition)},last:function(a){return H_(a,ka.isCursorPosition)}},Wq=function(){var a=q.fromTag("td");v.append(a,q.fromTag("br"));return a},I_=function(a,b,c){var d=ec.copy(a,b);W.each(c,function(a,b){null===a?m.remove(d,b):m.set(d,b,a)});return d},J_=function(a){return a},XB=function(a){return function(){return q.fromTag("tr",a.dom())}},
K_=function(a,b,c){return Cb.first(a).map(function(d){var e=c.join(",");d=X.ancestors(d,e,function(b){return L.eq(b,a)});return k.foldr(d,function(a,b){var c=ec.shallow(b);v.append(a,c);return c},b)}).getOr(b)},YB={cellOperations:function(a,b,c){return{row:XB(b),cell:function(b){var e=y.owner(b.element()),e=q.fromTag(A.name(b.element()),e.dom()),g=c.getOr("strong em b i span font h1 h2 h3 h4 h5 h6 p div".split(" ")),g=0<g.length?K_(b.element(),e,g):e;v.append(g,q.fromTag("br"));t.copy(b.element(),
e);t.remove(e,"height");1!==b.colspan()&&t.remove(b.element(),"width");a(b.element(),e);return e},replace:I_,gap:Wq}},paste:function(a){return{row:XB(a),cell:Wq,replace:J_,gap:Wq}}},Nb={erase:function(a,b,c,d){a=q.fromText("");v.after(b,a);B.remove(b);return h.some(cc(a,0,a,0))},execute:function(a,b,c,d){return function(e,g,l){var p=d(),k=e.doc().getOrDie();e=xv(a.directionAt);var m=h.some([]),k=YB.cellOperations(c,k,m);return b(p,g,l,k,e,f.noop,f.noop).bind(function(a){return a.cursor().map(function(a){return cc(a,
0,a,0)})})}}},vp={only:function(a){var b=h.from(a.dom().documentElement).map(q.fromDom).getOr(a);return{parent:f.constant(b),view:f.constant(a),origin:f.constant(Ba(0,0))}},detached:function(a,b){var c=f.curry(Na.absolute,b);return{parent:f.constant(b),view:f.constant(a),origin:c}},body:function(a,b){return{parent:f.constant(b),view:f.constant(a),origin:f.constant(Ba(0,0))}}},ZB=function(a,b,c){return a.isInline()?vp.detached(a.element(),b.uiContainer()):vp.only(c)},pp={get:ZB,fromEngine:function(a){var b=
a.content(),c=a.smartview();return b.doc().map(function(a){return ZB(b,c,a)})}},L_=C.immutable("width","height"),M_=C.immutable("rows","columns"),N_=C.immutable("row","column"),O_=C.immutable("x","y"),P_=C.immutable("element","rowspan","colspan"),Q_=C.immutable("element","rowspan","colspan","isNew"),R_=C.immutable("element","rowspan","colspan","row","column"),S_=C.immutable("element","cells","section"),T_=C.immutable("element","isNew"),U_=C.immutable("element","cells","section","isNew"),V_=C.immutable("cells",
"section"),W_=C.immutable("details","section"),X_=C.immutable("startRow","startCol","finishRow","finishCol"),xa={dimensions:L_,grid:M_,address:N_,coords:O_,extended:R_,detail:P_,detailnew:Q_,rowdata:S_,elementnew:T_,rowdatanew:U_,rowcells:V_,rowdetails:W_,bounds:X_},Xq=function(a,b,c){return k.bind(y.children(a),function(a){return ra.is(a,b)?c(a)?[a]:[]:Xq(a,b,c)})},Bi={firstLayer:function(a,b){return Xq(a,b,f.constant(!0))},filterFirstLayer:Xq},Yq=function(a,b,c){var d=void 0!==c?c:f.constant(!1);
return d(b)?h.none():k.contains(a,A.name(b))?h.some(b):J.ancestor(b,a.join(","),function(a){return ra.is(a,"table")||d(a)})},$B=function(a,b){return y.parent(b).map(function(b){return X.children(b,a)})},Y_=f.curry($B,"th,td"),Z_=f.curry($B,"tr"),Zq=function(a,b){return parseInt(m.get(a,b),10)},Ya={cell:function(a,b){return Yq(["td","th"],a,b)},firstCell:function(a){return J.descendant(a,"th,td")},cells:function(a){return Bi.firstLayer(a,"th,td")},neighbourCells:Y_,table:function(a,b){return J.closest(a,
"table",b)},row:function(a,b){return Yq(["tr"],a,b)},rows:function(a){return Bi.firstLayer(a,"tr")},notCell:function(a,b){return Yq(["caption","tr","tbody","tfoot","thead"],a,b)},neighbourRows:Z_,attr:Zq,grid:function(a,b,c){b=Zq(a,b);a=Zq(a,c);return xa.grid(b,a)}},We={fromTable:function(a){a=Ya.rows(a);return k.map(a,function(a){var c=y.parent(a).bind(function(a){a=A.name(a);return"tfoot"===a||"thead"===a||"tbody"===a?a:"tbody"}),d=k.map(Ya.cells(a),function(a){var b=m.has(a,"rowspan")?parseInt(m.get(a,
"rowspan"),10):1,c=m.has(a,"colspan")?parseInt(m.get(a,"colspan"),10):1;return xa.detail(a,b,c)});return xa.rowdata(a,d,c)})},fromPastedRows:function(a,b){return k.map(a,function(a){var d=k.map(Ya.cells(a),function(a){var b=m.has(a,"rowspan")?parseInt(m.get(a,"rowspan"),10):1,c=m.has(a,"colspan")?parseInt(m.get(a,"colspan"),10):1;return xa.detail(a,b,c)});return xa.rowdata(a,d,b.section())})}},aC=function(a,b){var c=k.bind(a.all(),function(a){return a.cells()});return k.filter(c,b)},gb={generate:function(a){var b=
{},c=[],d=a.length,e=0;k.each(a,function(a,d){var f=[];k.each(a.cells(),function(a,c){for(var g=0;void 0!==b[d+","+g];)g++;for(var h=xa.extended(a.element(),a.rowspan(),a.colspan(),d,g),k=0;k<a.colspan();k++)for(var m=0;m<a.rowspan();m++){var n=g+k;b[d+m+","+n]=h;e=Math.max(e,n+1)}f.push(h)});c.push(xa.rowdata(a.element(),f,a.section()))});a=xa.grid(d,e);return{grid:f.constant(a),access:f.constant(b),all:f.constant(c)}},getAt:function(a,b,c){a=a.access()[b+","+c];return void 0!==a?h.some(a):h.none()},
findItem:function(a,b,c){a=aC(a,function(a){return c(b,a.element())});return 0<a.length?h.some(a[0]):h.none()},filterItems:aC,justCells:function(a){a=k.map(a.all(),function(a){return a.cells()});return k.flatten(a)}},od={repeat:function(a,b){for(var c=[],d=0;d<a;d++)c.push(b(d));return c},range:function(a,b){for(var c=[],d=a;d<b;d++)c.push(d);return c},unique:function(a,b){var c=[];k.each(a,function(d,e){e<a.length-1&&!b(d,a[e+1])?c.push(d):e===a.length-1&&c.push(d)});return c},deduce:function(a,
b){if(0>b||b>=a.length-1)return h.none();var c=a[b].fold(function(){var c=k.reverse(a.slice(0,b));return ab.findMap(c,function(a,b){return a.map(function(a){return{value:a,delta:b+1}})})},function(a){return h.some({value:a,delta:0})}),d=a[b+1].fold(function(){var c=a.slice(b+1);return ab.findMap(c,function(a,b){return a.map(function(a){return{value:a,delta:b+1}})})},function(a){return h.some({value:a,delta:1})});return c.bind(function(a){return d.map(function(b){var c=b.delta+a.delta;return Math.abs(b.value-
a.value)/c})})}},bC=function(a,b,c){var d=a();return k.find(d,b).orThunk(function(){return h.from(d[0]).orThunk(c)}).map(function(a){return a.element()})},Pl={columns:function(a){var b=a.grid(),c=od.range(0,b.columns()),d=od.range(0,b.rows());return k.map(c,function(b){return bC(function(){return k.bind(d,function(c){return gb.getAt(a,c,b).filter(function(a){return a.column()===b}).fold(f.constant([]),function(a){return[a]})})},function(a){return 1===a.colspan()},function(){return gb.getAt(a,0,b)})})},
rows:function(a){var b=a.grid(),c=od.range(0,b.rows()),d=od.range(0,b.columns());return k.map(c,function(b){return bC(function(){return k.bind(d,function(c){return gb.getAt(a,b,c).filter(function(a){return a.row()===b}).fold(f.constant([]),function(a){return[a]})})},function(a){return 1===a.rowspan()},function(){return gb.getAt(a,b,0)})})}},cC=Y.detect(),Hg=function(a,b,c){a=t.get(a,b);a=parseFloat(a);return isNaN(a)?c:a},$_={getWidth:function(a){return Hg(a,"width",qa.get(a))},getHeight:function(a){var b;
if(cC.browser.isIE()||cC.browser.isEdge()){b=Hg(a,"padding-top",0);var c=Hg(a,"padding-bottom",0),d=Hg(a,"border-top-width",0),e=Hg(a,"border-bottom-width",0),g=a.dom().getBoundingClientRect().height;a=t.get(a,"box-sizing");b="border-box"===a?g:g-b-c-(d+e)}else b=Hg(a,"height",ia.get(a));return b}},dC=/(\d+(\.\d+)?)(\w|%)*/,Ci=/(\d+(\.\d+)?)%/,$q=/(\d+(\.\d+)?)px|em/,eC=function(a,b){t.set(a,"height",b+"px")},a0=function(a){return t.getRaw(a,"height").getOrThunk(function(){return $_.getHeight(a)+
"px"})},b0=function(a,b,c,d){var e=Ya.table(a).map(function(a){a=c(a);return Math.floor(b/100*a)}).getOr(b);d(a,e);return e},Ql=function(a){return t.getRaw(a,"width").fold(function(){return h.from(m.get(a,"width"))},function(a){return h.some(a)})},Ib={percentageBasedSizeRegex:f.constant(Ci),pixelBasedSizeRegex:f.constant($q),setPixelWidth:function(a,b){t.set(a,"width",b+"px")},setPercentageWidth:function(a,b){t.set(a,"width",b+"%")},setHeight:eC,getPixelWidth:function(a,b){return Ql(a).fold(function(){var b=
qa.get(a);return parseInt(b,10)},function(c){$q.test(c)?(c=$q.exec(c),c=parseInt(c[1],10)):Ci.test(c)?(c=Ci.exec(c),c=parseFloat(c[1])/100*b.pixelWidth()):(c=qa.get(a),c=parseInt(c,10));return c})},getPercentageWidth:function(a,b){return Ql(a).fold(function(){var c=qa.get(a);return parseInt(c,10)/b.pixelWidth()*100},function(c){Ci.test(c)?(c=Ci.exec(c),c=parseFloat(c[1])):(c=qa.get(a),c=parseInt(c,10)/b.pixelWidth()*100);return c})},getGenericWidth:function(a){return Ql(a).bind(function(a){return dC.test(a)?
(a=dC.exec(a),h.some({width:f.constant(a[1]),unit:f.constant(a[3])})):h.none()})},setGenericWidth:function(a,b,c){t.set(a,"width",b+c)},getHeight:function(a){var b;if(b=a0(a)){var c=ia.get,d=parseInt(b,10);b=aa.endsWith(b,"%")&&"table"!==A.name(a)?b0(a,d,c,eC):d}else b=ia.get(a);a=m.has(a,"rowspan")?parseInt(m.get(a,"rowspan"),10):1;return b/a},getRawWidth:Ql},fC=function(a,b){return m.has(a,b)&&1<parseInt(m.get(a,b),10)},Zc={hasColspan:function(a){return fC(a,"colspan")},hasRowspan:function(a){return fC(a,
"rowspan")},minWidth:f.constant(10),minHeight:f.constant(10),getInt:function(a,b){return parseInt(t.get(a,b),10)}},gC=function(a,b,c){return t.getRaw(a,b).fold(function(){return c(a)+"px"},function(a){return a})},c0=function(a){return gC(a,"width",Ib.getPixelWidth)},d0=function(a){return gC(a,"height",Ib.getHeight)},ar=function(a,b,c,d,e){a=Pl.columns(a);var g=k.map(a,function(a){return a.map(b.edge)});return k.map(a,function(a,b){return a.filter(f.not(Zc.hasColspan)).fold(function(){var a=od.deduce(g,
b);return d(a)},function(a){return c(a,e)})})},hC=function(a){return a.map(function(a){return a+"px"}).getOr("")},iC=function(a,b,c,d){a=Pl.rows(a);var e=k.map(a,function(a){return a.map(b.edge)});return k.map(a,function(a,b){return a.filter(f.not(Zc.hasRowspan)).fold(function(){var a=od.deduce(e,b);return d(a)},function(a){return c(a)})})},Di={getRawWidths:function(a,b){return ar(a,b,c0,hC)},getPixelWidths:function(a,b,c){return ar(a,b,Ib.getPixelWidth,function(a){return a.getOrThunk(c.minCellWidth)},
c)},getPercentageWidths:function(a,b,c){return ar(a,b,Ib.getPercentageWidth,function(a){return a.fold(function(){return c.minCellWidth()},function(a){return a/c.pixelWidth()*100})},c)},getPixelHeights:function(a,b){return iC(a,b,Ib.getHeight,function(a){return a.getOrThunk(Zc.minHeight)})},getRawHeights:function(a,b){return iC(a,b,d0,hC)}},Rl=ma.generate([{invalid:["raw"]},{pixels:["value"]},{percent:["value"]}]),jC=function(a,b,c){a=c.substring(0,c.length-a.length);var d=parseFloat(a);return a===
d.toString()?b(d):Rl.invalid(c)},Of=function(a){return aa.endsWith(a,"%")?jC("%",Rl.percent,a):aa.endsWith(a,"px")?jC("px",Rl.pixels,a):Rl.invalid(a)},e0=function(a,b){return k.map(a,function(a){return Of(a).fold(function(){return a},function(a){return a/b*100+"%"},function(a){return a+"%"})})},f0=function(a,b,c){var d=c/b;return k.map(a,function(a){return Of(a).fold(function(){return a},function(a){return a*d+"px"},function(a){return a/100*c+"px"})})},g0=function(a,b){var c=a.fold(function(){return f.constant("")},
function(a){return f.constant(a/b+"px")},function(a){return f.constant(a/b+"px")});return od.repeat(b,c)},h0=function(a,b,c){return a.fold(function(){return b},function(a){return f0(b,c,a)},function(a){return e0(b,c)})},kC=function(a,b){var c=Math.floor(a);return{value:c+b,remainder:a-c}},i0=function(a,b){return Of(a).fold(f.constant(a),function(a){return a+b+"px"},function(a){return a+b+"%"})},lC=function(a){if(0===a.length)return a;a=k.foldr(a,function(a,b){var e=Of(b).fold(function(){return{value:b,
remainder:0}},function(a){return kC(a,"px")},function(a){return kC(a,"%")});return{output:[e.value].concat(a.output),remainder:a.remainder+e.remainder}},{output:[],remainder:0});var b=a.output;return b.slice(0,b.length-1).concat([i0(b[b.length-1],Math.round(a.remainder))])},Ei={validate:Of,redistribute:function(a,b,c){c=Of(c);a=k.forall(a,function(a){return"0px"===a})?g0(c,a.length):h0(c,a,b);return lC(a)},sum:function(a,b){return 0===a.length?b:k.foldr(a,function(a,b){return Of(b).fold(f.constant(0),
f.identity,f.identity)+a},0)},toIntegers:lC},j0=function(a,b,c){k.each(b,function(b){var e=a.slice(b.column(),b.colspan()+b.column()),e=Ei.sum(e,Zc.minWidth());t.set(b.element(),"width",e+c)})},k0=function(a,b,c,d){k.each(c,function(b){var c=a.slice(b.row(),b.rowspan()+b.row()),c=Ei.sum(c,Zc.minHeight());t.set(b.element(),"height",c+d)});k.each(b,function(b,c){t.set(b.element(),"height",a[c])})},mC=function(a){return Ei.validate(a).fold(f.constant("px"),f.constant("px"),f.constant("%"))},l0={redistribute:function(a,
b,c,d){var e=We.fromTable(a),g=gb.generate(e),f=g.all(),h=gb.justCells(g);b.each(function(b){var c=mC(b),e=qa.get(a),f=Di.getRawWidths(g,d),e=Ei.redistribute(f,e,b);j0(e,h,c);t.set(a,"width",b)});c.each(function(b){var c=mC(b),d=ia.get(a),e=Di.getRawHeights(g,hi.height),d=Ei.redistribute(e,d,b);k0(d,f,h,c);t.set(a,"height",b)})}},m0=function(a){var b=m.has(a,"colspan")?parseInt(m.get(a,"colspan"),10):1,c=m.has(a,"rowspan")?parseInt(m.get(a,"rowspan"),10):1;return{element:f.constant(a),colspan:f.constant(b),
rowspan:f.constant(c)}},br=Gb.exactly(["cell","row","replace","gap"]),xb={modification:function(a,b){br(a);var c=wa(h.none()),d=void 0!==b?b:m0,e=function(b){var e;e=d(b);e=a.cell(e);c.get().isNone()&&c.set(h.some(e));g=h.some({item:b,replacement:e});return e},g=h.none();return{getOrInit:function(a,b){return g.fold(function(){return e(a)},function(c){return b(a,c.item)?c.replacement:e(a)})},cursor:c.get}},transform:function(a,b){return function(c){var d=wa(h.none());br(c);var e=[],g=function(a,b){return k.find(e,
function(c){return b(c.item,a)})},f=function(g){var f=c.replace(g,b,{scope:a});e.push({item:g,sub:f});d.get().isNone()&&d.set(h.some(f));return f};return{replaceOrInit:function(a,b){return g(a,b).fold(function(){return f(a)},function(c){return b(a,c.item)?c.sub:f(a)})},cursor:d.get}}},merging:function(a){br(a);var b=wa(h.none());return{combine:function(c){b.get().isNone()&&b.set(h.some(c));return function(){var b=a.cell({element:f.constant(c),colspan:f.constant(1),rowspan:f.constant(1)});t.remove(b,
"width");t.remove(c,"width");return b}},cursor:b.get}}},n0="body p div article aside figcaption figure footer header nav section ol ul table thead tfoot tbody caption tr td th h1 h2 h3 h4 h5 h6 blockquote pre address".split(" "),nC=function(a,b){var c=a.property().name(b);return k.contains(n0,c)},oC=function(a,b){return k.contains(["br","img","hr","input"],a.property().name(b))},Yb={isBlock:nC,isList:function(a,b){var c=a.property().name(b);return k.contains(["ol","ul"],c)},isFormatting:function(a,
b){var c=a.property().name(b);return k.contains("address pre p h1 h2 h3 h4 h5 h6".split(" "),c)},isHeading:function(a,b){var c=a.property().name(b);return k.contains("h1 h2 h3 h4 h5 h6".split(" "),c)},isContainer:function(a,b){return k.contains("div li td th blockquote body caption".split(" "),a.property().name(b))},isEmptyTag:oC,isFrame:function(a,b){return"iframe"===a.property().name(b)},isInline:function(a,b){return!(nC(a,b)||oC(a,b))&&"li"!==a.property().name(b)}},Xe=Sa(),fa={isBlock:function(a){return Yb.isBlock(Xe,
a)},isList:function(a){return Yb.isList(Xe,a)},isFormatting:function(a){return Yb.isFormatting(Xe,a)},isHeading:function(a){return Yb.isHeading(Xe,a)},isContainer:function(a){return Yb.isContainer(Xe,a)},isEmptyTag:function(a){return Yb.isEmptyTag(Xe,a)},isFrame:function(a){return Yb.isFrame(Xe,a)},isInline:function(a){return Yb.isInline(Xe,a)}},o0={merge:function(a){var b=function(a){return k.forall(a,function(a){return"br"===A.name(a)||A.isText(a)&&0===Da.get(a).trim().length})},c=function(a){return y.nextSibling(a).map(function(a){if(fa.isBlock(a))return!0;
if(fa.isEmptyTag(a))return"img"===A.name(a)?!1:!0}).getOr(!1)},d=function(a){return Cb.last(a).bind(function(b){var d=c(b);return y.parent(b).map(function(c){var e;(e=!0===d)||(e="li"===A.name(c)||sa.ancestor(c,fa.isList).isSome()||"br"===A.name(b)||fa.isBlock(c)&&!L.eq(a,c));return e?[]:[q.fromTag("br")]})}).getOr([])},e=function(){var c=k.bind(a,function(a){var c=y.children(a);return b(c)?[]:c.concat(d(a))});return 0===c.length?[q.fromTag("br")]:c}();B.empty(a[0]);K.append(a[0],e)}},pC=function(a,
b){return xa.rowcells(b,a.section())},qC=function(a,b){return a.cells()[b]},ta={addCell:function(a,b,c){var d=a.cells(),e=d.slice(0,b);b=d.slice(b);c=e.concat([c]).concat(b);return pC(a,c)},setCells:pC,mutateCell:function(a,b,c){a.cells()[b]=c},getCell:qC,getCellElement:function(a,b){return qC(a,b).element()},mapCells:function(a,b){var c=a.cells(),c=k.map(c,b);return xa.rowcells(c,a.section())},cellLength:function(a){return a.cells().length}},p0=function(a,b){return k.map(a,function(a){return ta.getCell(a,
b)})},rC=function(a,b){if(0===a.length)return 0;var c=a[0];return k.findIndex(a,function(a){return!b(c.element(),a.element())}).fold(function(){return a.length},function(a){return a})},q0={subgrid:function(a,b,c,d){var e=a[b].cells().slice(c),e=rC(e,d);a=p0(a,c).slice(b);d=rC(a,d);return{colspan:f.constant(e),rowspan:f.constant(d)}}},Sl={toDetails:function(a,b){var c=k.map(a,function(a,b){return k.map(a.cells(),function(a,b){return!1})});return k.map(a,function(d,e){var g=k.bind(d.cells(),function(d,
g){if(!1===c[e][g]){for(var f=q0.subgrid(a,e,g,b),h=f.rowspan(),k=f.colspan(),m=e;m<e+h;m++)for(var n=g;n<g+k;n++)c[m][n]=!0;return[xa.detailnew(d.element(),f.rowspan(),f.colspan(),d.isNew())]}return[]});return xa.rowdetails(g,d.section())})},toGrid:function(a,b,c){for(var d=[],e=0;e<a.grid().rows();e++){for(var g=[],f=0;f<a.grid().columns();f++){var h=gb.getAt(a,e,f).map(function(a){return xa.elementnew(a.element(),c)}).getOrThunk(function(){return xa.elementnew(b.gap(),!0)});g.push(h)}g=xa.rowcells(g,
a.all()[e].section());d.push(g)}return d}},Tl=function(a,b,c,d){c===d?m.remove(a,b):m.set(a,b,c)},r0={render:function(a,b){var c=[],d=[],e=function(b,e){var g=J.child(a,e).getOrThunk(function(){var b=q.fromTag(e,y.owner(a).dom());v.append(a,b);return b});B.empty(g);var f=k.map(b,function(a){a.isNew()&&c.push(a.element());var b=a.element();B.empty(b);k.each(a.cells(),function(a){a.isNew()&&d.push(a.element());Tl(a.element(),"colspan",a.colspan(),1);Tl(a.element(),"rowspan",a.rowspan(),1);v.append(b,
a.element())});return b});K.append(g,f)},g=function(b,c){0<b.length?e(b,c):J.child(a,c).bind(B.remove)},l=[],h=[],r=[];k.each(b,function(a){switch(a.section()){case "thead":l.push(a);break;case "tbody":h.push(a);break;case "tfoot":r.push(a)}});g(l,"thead");g(h,"tbody");g(r,"tfoot");return{newRows:f.constant(c),newCells:f.constant(d)}},copy:function(a){return k.map(a,function(a){var c=ec.shallow(a.element());k.each(a.cells(),function(a){var b=ec.deep(a.element());Tl(b,"colspan",a.colspan(),1);Tl(b,
"rowspan",a.rowspan(),1);v.append(c,b)});return c})}},sC={col:function(a,b,c,d,e){var g=q.fromTag("div");t.setAll(g,{position:"absolute",left:b-d/2+"px",top:c+"px",height:e+"px",width:d+"px"});m.setAll(g,{"data-column":a,role:"presentation"});return g},row:function(a,b,c,d,e){var g=q.fromTag("div");t.setAll(g,{position:"absolute",left:b+"px",top:c-e/2+"px",height:e+"px",width:d+"px"});m.setAll(g,{"data-row":a,role:"presentation"});return g}},Fc={resolve:yl.css("ephox-snooker").resolve},cr=Fc.resolve("resizer-bar"),
tC=Fc.resolve("resizer-rows"),uC=Fc.resolve("resizer-cols"),vC=function(a){a=X.descendants(a.parent(),"."+cr);k.each(a,B.remove)},wC=function(a,b,c){var d=a.origin();k.each(b,function(b,g){b.each(function(b){b=c(d,b);n.add(b,cr);v.append(a.parent(),b)})})},s0=function(a,b,c,d){wC(a,b,function(a,b){var f=sC.col(b.col(),b.x()-a.left(),c.top()-a.top(),7,d);n.add(f,uC);return f})},t0=function(a,b,c,d){wC(a,b,function(a,b){var f=sC.row(b.row(),c.left()-a.left(),b.y()-a.top(),d,7);n.add(f,tC);return f})},
xC=function(a,b){var c=X.descendants(a.parent(),"."+cr);k.each(c,b)},Ad={refresh:function(a,b,c,d){vC(a);var e=We.fromTable(b),e=gb.generate(e),g=Pl.rows(e),f=Pl.columns(e),e=Na.absolute(b);c=0<g.length?c.positions(g,b):[];t0(a,c,e,qa.getOuter(b));d=0<f.length?d.positions(f,b):[];s0(a,d,e,ia.getOuter(b))},hide:function(a){xC(a,function(a){t.set(a,"display","none")})},show:function(a){xC(a,function(a){t.set(a,"display","block")})},destroy:vC,isRowBar:function(a){return n.has(a,tC)},isColBar:function(a){return n.has(a,
uC)}},u0=function(a,b){var c=function(a){return ab.findMap(a,function(a){return y.parent(a.element()).map(function(a){var b=y.parent(a).isNone();return xa.elementnew(a,b)})}).getOrThunk(function(){return xa.elementnew(b.row(),!0)})};return k.map(a,function(a){var b=c(a.details());return xa.rowdatanew(b.element(),a.details(),a.section(),b.isNew())})},yC=function(a,b){var c=Sl.toDetails(a,L.eq);return u0(c,b)},Ul=function(a,b){var c=k.flatten(k.map(a.all(),function(a){return a.cells()}));return k.find(c,
function(a){return L.eq(b,a.element())})},ya={run:function(a,b,c,d,e){return function(g,l,p,k,m){var w=We.fromTable(l),n=gb.generate(w);return b(n,p).map(function(b){var c=Sl.toGrid(n,k,!1);b=a(c,b,L.eq,e(k));c=yC(b.grid(),k);return{grid:f.constant(c),cursor:b.cursor}}).fold(function(){return h.none()},function(a){var b=r0.render(l,a.grid());c(l,a.grid(),m);d(l);Ad.refresh(g,l,hi.height,m);return h.some({cursor:a.cursor,newRows:b.newRows,newCells:b.newCells})})}},toDetailList:yC,onCell:function(a,
b){return Ya.cell(b.element()).bind(function(b){return Ul(a,b)})},onCells:function(a,b){var c=k.map(b.selection(),function(b){return Ya.cell(b).bind(function(b){return Ul(a,b)})}),c=ab.cat(c);return 0<c.length?h.some(c):h.none()},onPaste:function(a,b){return Ya.cell(b.element()).bind(function(c){return Ul(a,c).map(function(a){return Z.merge(a,{generators:b.generators,clipboard:b.clipboard})})})},onPasteRows:function(a,b){var c=k.map(b.selection(),function(b){return Ya.cell(b).bind(function(b){return Ul(a,
b)})}),c=ab.cat(c);return 0<c.length?h.some(Z.merge({cells:c},{generators:b.generators,clipboard:b.clipboard})):h.none()},onMergable:function(a,b){return b.mergable()},onUnmergable:function(a,b){return b.unmergable()}},zC=function(a,b){return k.map(a,function(){return xa.elementnew(b.cell(),!0)})},v0=function(a,b,c){return a.concat(od.repeat(b,function(b){return ta.setCells(a[a.length-1],zC(a[a.length-1].cells(),c))}))},w0=function(a,b,c){return k.map(a,function(a){return ta.setCells(a,a.cells().concat(zC(od.range(0,
b),c)))})},Ig={measure:function(a,b,c){if(a.row()>=b.length||a.column()>ta.cellLength(b[0]))return H.error("invalid start address out of table bounds, row: "+a.row()+", column: "+a.column());b=b.slice(a.row());a=b[0].cells().slice(a.column());var d=ta.cellLength(c[0]);return H.value({rowDelta:f.constant(b.length-c.length),colDelta:f.constant(a.length-d)})},measureWidth:function(a,b){var c=ta.cellLength(a[0]),d=ta.cellLength(b[0]);return{rowDelta:f.constant(0),colDelta:f.constant(c-d)}},tailor:function(a,
b,c){var d=0>b.colDelta()?w0:f.identity,e=0>b.rowDelta()?v0:f.identity;a=d(a,Math.abs(b.colDelta()),c);return e(a,Math.abs(b.rowDelta()),c)}},x0=function(a,b){return k.foldl(a,function(a,d){return k.exists(a,function(a){return b(a.element(),d.element())})?a:a.concat([d])},[])},Vl={merge:function(a,b,c,d){if(0===a.length)return a;for(c=b.startRow();c<=b.finishRow();c++)for(var e=b.startCol();e<=b.finishCol();e++)ta.mutateCell(a[c],e,xa.elementnew(d(),!1));return a},unmerge:function(a,b,c,d){for(var e=
!0,g=0;g<a.length;g++)for(var f=0;f<ta.cellLength(a[0]);f++){var h=ta.getCellElement(a[g],f),h=c(h,b);!0===h&&!1===e?ta.mutateCell(a[g],f,xa.elementnew(d(),!0)):!0===h&&(e=!1)}return a},splitRows:function(a,b,c,d){if(0<b&&b<a.length){var e=a[b-1].cells(),e=x0(e,c);k.each(e,function(e){for(var f=h.none(),p=b;p<a.length;p++)for(var k=0;k<ta.cellLength(a[0]);k++){var m=a[p].cells()[k];c(m.element(),e.element())&&(f.isNone()&&(f=h.some(d())),f.each(function(b){ta.mutateCell(a[p],k,xa.elementnew(b,!0))}))}})}return a}},
dr={merge:function(a,b,c,d,e){return Ig.measure(a,b,c).map(function(g){g=Ig.tailor(b,g,d);for(var l=a.row(),h=a.column(),k=c.length,m=ta.cellLength(c[0]),k=l+k,m=h+m,w=l;w<k;w++)for(var n=h;n<m;n++){var q=g,s=w,t=n,u=e,v=ta.getCell(q[s],t),u=f.curry(u,v.element()),v=q[s];1<q.length&&1<ta.cellLength(v)&&(0<t&&u(ta.getCellElement(v,t-1))||t<v.length-1&&u(ta.getCellElement(v,t+1))||0<s&&u(ta.getCellElement(q[s-1],t))||s<q.length-1&&u(ta.getCellElement(q[s+1],t)))&&Vl.unmerge(g,ta.getCellElement(g[w],
n),e,d.cell);q=ta.getCellElement(c[w-l],n-h);q=d.replace(q);ta.mutateCell(g[w],n,xa.elementnew(q,!0))}return g})},insert:function(a,b,c,d,e){Vl.splitRows(b,a,e,d.cell);e=Ig.measureWidth(c,b);c=Ig.tailor(c,e,d);e=Ig.measureWidth(b,c);b=Ig.tailor(b,e,d);return b.slice(0,a).concat(c).concat(b.slice(a,b.length))}},bd={insertRowAt:function(a,b,c,d,e){var g=a.slice(0,b),f=a.slice(b);c=ta.mapCells(a[c],function(c,g){return 0<b&&b<a.length&&d(ta.getCellElement(a[b-1],g),ta.getCellElement(a[b],g))?ta.getCell(a[b],
g):xa.elementnew(e(c.element(),d),!0)});return g.concat([c]).concat(f)},insertColumnAt:function(a,b,c,d,e){return k.map(a,function(a){var f=0<b&&b<ta.cellLength(a)&&d(ta.getCellElement(a,b-1),ta.getCellElement(a,b))?ta.getCell(a,b):xa.elementnew(e(ta.getCellElement(a,c),d),!0);return ta.addCell(a,b,f)})},splitCellIntoColumns:function(a,b,c,d,e){var g=c+1;return k.map(a,function(a,f){var h=f===b?xa.elementnew(e(ta.getCellElement(a,c),d),!0):ta.getCell(a,c);return ta.addCell(a,g,h)})},splitCellIntoRows:function(a,
b,c,d,e){var g=b+1,f=a.slice(0,g),g=a.slice(g);a=ta.mapCells(a[b],function(a,b){return b===c?xa.elementnew(e(a.element(),d),!0):a});return f.concat([a]).concat(g)},deleteRowsAt:function(a,b,c){return a.slice(0,b).concat(a.slice(c+1))},deleteColumnsAt:function(a,b,c){a=k.map(a,function(a){var e=a.cells().slice(0,b).concat(a.cells().slice(c+1));return xa.rowcells(e,a.section())});return k.filter(a,function(a){return 0<a.cells().length})}},AC=function(a,b,c,d){var e=function(a){return k.exists(b,function(b){return c(a.element(),
b.element())})};return k.map(a,function(a){return ta.mapCells(a,function(a){return e(a)?xa.elementnew(d(a.element(),c),!0):a})})},BC=function(a,b,c,d){return void 0!==ta.getCellElement(a[b],c)&&0<b&&d(ta.getCellElement(a[b-1],c),ta.getCellElement(a[b],c))},CC=function(a,b,c){return 0<b&&c(ta.getCellElement(a,b-1),ta.getCellElement(a,b))},Wl={replaceColumn:function(a,b,c,d){var e=k.bind(a,function(d,e){return BC(a,e,b,c)||CC(d,b,c)?[]:[ta.getCell(d,b)]});return AC(a,e,c,d)},replaceRow:function(a,b,
c,d){var e=a[b],g=k.bind(e.cells(),function(d,g){return BC(a,b,g,c)||CC(e,g,c)?[]:[d]});return AC(a,g,c,d)}},Fi=function(a){return{fold:a}},Jg={none:function(){return Fi(function(a,b,c,d,e){return a()})},only:function(a){return Fi(function(b,c,d,e,g){return c(a)})},left:function(a,b){return Fi(function(c,d,e,g,f){return e(a,b)})},middle:function(a,b,c){return Fi(function(d,e,g,f,h){return f(a,b,c)})},right:function(a,b){return Fi(function(c,d,e,g,f){return f(a,b)})}},y0=function(a,b){return 0===a.length?
Jg.none():1===a.length?Jg.only(0):0===b?Jg.left(0,1):b===a.length-1?Jg.right(b-1,b):0<b&&b<a.length-1?Jg.middle(b-1,b,b+1):Jg.none()},z0={determine:function(a,b,c,d){var e=a.slice(0);a=y0(a,b);var g=function(a){return k.map(a,f.constant(0))};b=f.constant(g(e));var l=function(a,b){if(0<=c){var f=Math.max(d.minCellWidth(),e[b]-c);return g(e.slice(0,a)).concat([c,f-e[b]]).concat(g(e.slice(b+1)))}var f=Math.max(d.minCellWidth(),e[a]+c),l=e[a]-f;return g(e.slice(0,a)).concat([f-e[a],l]).concat(g(e.slice(b+
1)))};return a.fold(b,function(a){return d.singleColumnWidth(e[a],c)},l,function(a,b,c){return l(b,c)},function(a,b){if(0<=c)return g(e.slice(0,b)).concat([c]);var f=Math.max(d.minCellWidth(),e[b]+c);return g(e.slice(0,b)).concat([f-e[b]])})}},DC=function(a,b,c){for(var d=0;a<b;a++)d+=void 0!==c[a]?c[a]:0;return d},Xl={recalculateWidth:function(a,b){var c=gb.justCells(a);return k.map(c,function(a){var c=DC(a.column(),a.column()+a.colspan(),b);return{element:a.element,width:f.constant(c),colspan:a.colspan}})},
recalculateHeight:function(a,b){var c=gb.justCells(a);return k.map(c,function(a){var c=DC(a.row(),a.row()+a.rowspan(),b);return{element:a.element,height:f.constant(c),rowspan:a.rowspan}})},matchRowHeight:function(a,b){return k.map(a.all(),function(a,d){return{element:a.element,height:f.constant(b[d])}})}},A0=function(a,b){var c=parseFloat(a),d=qa.get(b);return{width:f.constant(c),pixelWidth:f.constant(d),getWidths:Di.getPercentageWidths,getCellDelta:function(a){return a/d*100},singleColumnWidth:function(a,
b){return[100-a]},minCellWidth:function(){return Zc.minWidth()/d*100},setElementWidth:Ib.setPercentageWidth,setTableWidth:function(a,b,d){Ib.setPercentageWidth(a,c+d)}}},er=function(a){a=parseInt(a,10);var b=f.identity;return{width:f.constant(a),pixelWidth:f.constant(a),getWidths:Di.getPixelWidths,getCellDelta:b,singleColumnWidth:function(a,b){return[Math.max(Zc.minWidth(),a+b)-a]},minCellWidth:Zc.minWidth,setElementWidth:Ib.setPixelWidth,setTableWidth:function(a,b,e){b=k.foldr(b,function(a,b){return a+
b},0);Ib.setPixelWidth(a,b)}}},EC={getTableSize:function(a){return Ib.getRawWidth(a).fold(function(){var b=qa.get(a);return er(b)},function(b){Ib.percentageBasedSizeRegex().test(b)?(b=Ib.percentageBasedSizeRegex().exec(b),b=A0(b[1],a)):Ib.pixelBasedSizeRegex().test(b)?(b=Ib.pixelBasedSizeRegex().exec(b),b=er(b[1])):(b=qa.get(a),b=er(b));return b})}},B0=function(a){return k.foldr(a,function(a,c){return a+c},0)},FC=function(a){a=We.fromTable(a);return gb.generate(a)},up={adjustWidth:function(a,b,c,
d){var e=EC.getTableSize(a);b=e.getCellDelta(b);var g=FC(a),f=e.getWidths(g,d,e);d=z0.determine(f,c,b,e);d=k.map(d,function(a,b){return a+f[b]});var h=Xl.recalculateWidth(g,d);k.each(h,function(a){e.setElementWidth(a.element(),a.width())});c===g.grid().columns()-1&&e.setTableWidth(a,d,b)},adjustHeight:function(a,b,c,d){var e=FC(a);d=Di.getPixelHeights(e,d);d=k.map(d,function(a,d){return c===d?Math.max(b+a,Zc.minHeight()):a});var g=Xl.recalculateHeight(e,d),e=Xl.matchRowHeight(e,d);k.each(e,function(a){Ib.setHeight(a.element(),
a.height())});k.each(g,function(a){Ib.setHeight(a.element(),a.height())});e=B0(d);Ib.setHeight(a,e)},adjustWidthTo:function(a,b,c){var d=EC.getTableSize(a);b=gb.generate(b);c=d.getWidths(b,c,d);b=Xl.recalculateWidth(b,c);k.each(b,function(a){d.setElementWidth(a.element(),a.width())});c=k.foldr(c,function(a,b){return b+a},0);0<b.length&&d.setElementWidth(a,c)}},GC=function(a){0===Ya.cells(a).length&&B.remove(a)},qe=C.immutable("grid","cursor"),Gi=function(a,b,c){return fr(a,b,c).orThunk(function(){return fr(a,
0,0)})},fr=function(a,b,c){return h.from(a[b]).bind(function(a){return h.from(a.cells()[c]).bind(function(a){return h.from(a.element())})})},vc=function(a,b,c){return qe(a,fr(a,b,c))},gr=function(a){return k.foldl(a,function(a,c){return k.exists(a,function(a){return a.row()===c.row()})?a:a.concat([c])},[]).sort(function(a,c){return a.row()-c.row()})},hr=function(a){return k.foldl(a,function(a,c){return k.exists(a,function(a){return a.column()===c.column()})?a:a.concat([c])},[]).sort(function(a,c){return a.column()-
c.column()})},HC=function(a,b,c){a=We.fromPastedRows(a,c);a=gb.generate(a);return Sl.toGrid(a,b,!0)},Ye=up.adjustWidthTo,Fb={insertRowBefore:ya.run(function(a,b,c,d){var e=b.row(),g=b.row();a=bd.insertRowAt(a,g,e,c,d.getOrInit);return vc(a,g,b.column())},ya.onCell,f.noop,f.noop,xb.modification),insertRowsBefore:ya.run(function(a,b,c,d){var e=b[0].row(),g=b[0].row(),f=gr(b);a=k.foldl(f,function(a,b){return bd.insertRowAt(a,g,e,c,d.getOrInit)},a);return vc(a,g,b[0].column())},ya.onCells,f.noop,f.noop,
xb.modification),insertRowAfter:ya.run(function(a,b,c,d){var e=b.row(),g=b.row()+b.rowspan();a=bd.insertRowAt(a,g,e,c,d.getOrInit);return vc(a,g,b.column())},ya.onCell,f.noop,f.noop,xb.modification),insertRowsAfter:ya.run(function(a,b,c,d){var e=gr(b),g=e[e.length-1].row(),f=e[e.length-1].row()+e[e.length-1].rowspan();a=k.foldl(e,function(a,b){return bd.insertRowAt(a,f,g,c,d.getOrInit)},a);return vc(a,f,b[0].column())},ya.onCells,f.noop,f.noop,xb.modification),insertColumnBefore:ya.run(function(a,
b,c,d){var e=b.column(),g=b.column();a=bd.insertColumnAt(a,g,e,c,d.getOrInit);return vc(a,b.row(),g)},ya.onCell,Ye,f.noop,xb.modification),insertColumnsBefore:ya.run(function(a,b,c,d){var e=hr(b),g=e[0].column(),f=e[0].column();a=k.foldl(e,function(a,b){return bd.insertColumnAt(a,f,g,c,d.getOrInit)},a);return vc(a,b[0].row(),f)},ya.onCells,Ye,f.noop,xb.modification),insertColumnAfter:ya.run(function(a,b,c,d){var e=b.column(),g=b.column()+b.colspan();a=bd.insertColumnAt(a,g,e,c,d.getOrInit);return vc(a,
b.row(),g)},ya.onCell,Ye,f.noop,xb.modification),insertColumnsAfter:ya.run(function(a,b,c,d){var e=b[b.length-1].column(),g=b[b.length-1].column()+b[b.length-1].colspan(),f=hr(b);a=k.foldl(f,function(a,b){return bd.insertColumnAt(a,g,e,c,d.getOrInit)},a);return vc(a,b[0].row(),g)},ya.onCells,Ye,f.noop,xb.modification),splitCellIntoColumns:ya.run(function(a,b,c,d){a=bd.splitCellIntoColumns(a,b.row(),b.column(),c,d.getOrInit);return vc(a,b.row(),b.column())},ya.onCell,Ye,f.noop,xb.modification),splitCellIntoRows:ya.run(function(a,
b,c,d){a=bd.splitCellIntoRows(a,b.row(),b.column(),c,d.getOrInit);return vc(a,b.row(),b.column())},ya.onCell,f.noop,f.noop,xb.modification),eraseColumns:ya.run(function(a,b,c,d){c=hr(b);a=bd.deleteColumnsAt(a,c[0].column(),c[c.length-1].column());b=Gi(a,b[0].row(),b[0].column());return qe(a,b)},ya.onCells,Ye,GC,xb.modification),eraseRows:ya.run(function(a,b,c,d){c=gr(b);a=bd.deleteRowsAt(a,c[0].row(),c[c.length-1].row());b=Gi(a,b[0].row(),b[0].column());return qe(a,b)},ya.onCells,f.noop,GC,xb.modification),
makeColumnHeader:ya.run(function(a,b,c,d){a=Wl.replaceColumn(a,b.column(),c,d.replaceOrInit);return vc(a,b.row(),b.column())},ya.onCell,f.noop,f.noop,xb.transform("row","th")),unmakeColumnHeader:ya.run(function(a,b,c,d){a=Wl.replaceColumn(a,b.column(),c,d.replaceOrInit);return vc(a,b.row(),b.column())},ya.onCell,f.noop,f.noop,xb.transform(null,"td")),makeRowHeader:ya.run(function(a,b,c,d){a=Wl.replaceRow(a,b.row(),c,d.replaceOrInit);return vc(a,b.row(),b.column())},ya.onCell,f.noop,f.noop,xb.transform("col",
"th")),unmakeRowHeader:ya.run(function(a,b,c,d){a=Wl.replaceRow(a,b.row(),c,d.replaceOrInit);return vc(a,b.row(),b.column())},ya.onCell,f.noop,f.noop,xb.transform(null,"td")),mergeCells:ya.run(function(a,b,c,d){d=b.cells();o0.merge(d);a=Vl.merge(a,b.bounds(),c,f.constant(d[0]));return qe(a,h.from(d[0]))},ya.onMergable,f.noop,f.noop,xb.merging),unmergeCells:ya.run(function(a,b,c,d){a=k.foldr(b,function(a,b){return Vl.unmerge(a,b,c,d.combine(b))},a);return qe(a,h.from(b[0]))},ya.onUnmergable,Ye,f.noop,
xb.merging),pasteCells:ya.run(function(a,b,c,d){d=function(a,b){var c=We.fromTable(a),c=gb.generate(c);return Sl.toGrid(c,b,!0)}(b.clipboard(),b.generators());var e=xa.address(b.row(),b.column());return dr.merge(e,a,d,b.generators(),c).fold(function(){return qe(a,h.some(b.element()))},function(a){var c=Gi(a,b.row(),b.column());return qe(a,c)})},ya.onPaste,Ye,f.noop,xb.modification),pasteRowsBefore:ya.run(function(a,b,c,d){var e=a[b.cells[0].row()];d=b.cells[0].row();e=HC(b.clipboard(),b.generators(),
e);a=dr.insert(d,a,e,b.generators(),c);b=Gi(a,b.cells[0].row(),b.cells[0].column());return qe(a,b)},ya.onPasteRows,f.noop,f.noop,xb.modification),pasteRowsAfter:ya.run(function(a,b,c,d){var e=a[b.cells[0].row()];d=b.cells[b.cells.length-1].row()+b.cells[b.cells.length-1].rowspan();e=HC(b.clipboard(),b.generators(),e);a=dr.insert(d,a,e,b.generators(),c);b=Gi(a,b.cells[0].row(),b.cells[0].column());return qe(a,b)},ya.onPasteRows,f.noop,f.noop,xb.modification)},ir=function(a){return function(b,c){return k.foldl(a,
function(a,e){return a.bind(function(a){return C0(c,function(a){return D0(e,b,c,a)})})},H.value(c))}},E0=ir([Fb.makeRowHeader,Fb.makeColumnHeader]),F0=ir([Fb.makeRowHeader]),G0=ir([Fb.makeColumnHeader]),D0=function(a,b,c,d){return pp.fromEngine(b).bind(function(e){e=f.constant(e);var g=f.noop;return Nb.execute(b.region(),a,g,e)(b.content(),c,{element:f.constant(d)})})},C0=function(a,b){return J.descendant(a,"td,th").bind(function(a){return b(a)}).fold(function(){return H.error("Could not run the operation on table")},
function(b){return H.value(a)})},Be={setCaption:function(a,b){var c=J.child(a,"caption").fold(function(){return q.fromTag("caption")},function(a){B.empty(a);return a});ua.set(c,b);v.prepend(a,c)},setSummary:function(a,b){m.set(a,"summary",b)},setDimensions:function(a,b){l0.redistribute(a,b.width(),b.height(),"rtl"===m.get(a,"dir")?Vj.rtl:Vj.ltr)},addBothHeaders:E0,addRowHeader:F0,addColumnHeader:G0},jr=ma.generate([{field:["name","textkey","f"]},{select:["name","textkey","values"]},{toggles:["name",
"textkey","options"]}]),H0=C.immutable("value","text"),I0=C.immutable("value","icon","textkey"),Bb={field:jr.field,select:jr.select,toggles:jr.toggles,pair:H0,toggle:I0,cata:function(a,b,c,d){return a.fold(b,c,d)}},J0=function(a,b){k.each(a,function(a){b.get(a.severity()).fold(function(){console.warn("unknown severity for accessibility rule: ",a.id(),a.severity())},function(d){ea.add(a.element(),[b.violation(),d.style()]);Ud.add(a.element(),d.attribute(),a.id())})});h.from(a[0]).each(function(a){var d=
a.element(),e=b.currentAttr();a=a.severity();m.set(d,e,a)})},K0=function(a,b,c,d){a=X.descendants(a,"["+b+"]");k.each(a,function(a){m.remove(a,b)});m.set(c,b,d)},L0=function(a,b){var c=X.descendants(a,"."+b.violation());k.each(c,function(a){n.remove(a,b.violation());m.remove(a,b.currentAttr());k.each(b.all(),function(b){n.remove(a,b.style());m.remove(a,b.attribute())})})},IC=function(a,b,c){var d=function(b){return 0===Ud.read(a,b.attribute()).length};b.get(c).each(function(b){d(b)&&n.remove(a,b.style())});
k.forall(b.all(),d)&&n.remove(a,b.violation())},M0=function(a,b,c,d){b.get(d).each(function(d){Ud.remove(a,d.attribute(),c);Ud.add(a,b.ignoreAttr(),c);d=b.currentAttr();m.remove(a,d)})},N0=function(a,b,c,d){b.get(d).each(function(d){Ud.remove(a,d.attribute(),c);d=b.currentAttr();m.remove(a,d)})},O0={manager:function(a){var b=a.content();return{start:function(b){J0(b,a)},moveTo:function(c,d){b.body().each(function(b){K0(b,a.currentAttr(),c,d)})},stop:function(){b.body().each(function(b){L0(b,a)})},
ignore:function(b,d,e){M0(b,a,d,e);IC(b,a,e)},solve:function(b,d,e){N0(b,a,d,e);IC(b,a,e)}}}},Kg=function(a){return function(b){k.each(b,a)}},P0=Kg(B.remove),Q0=Kg(function(a){m.set(a,"class","")}),R0=Kg(function(a){var b=y.children(a);K.before(a,b);B.remove(a)}),S0=Ua.zeroWidth(),T0=function(a){return A.isText(a)?(a=pb.get(a),""===a||a===S0):!1},U0=Kg(function(a){var b;A.isElement(a)?(b=y.childNodesCount(a),b=0===b||1===b&&y.child(a,0).exists(T0)):b=!1;b&&m.hasNone(a)&&B.remove(a)}),Ca={remove:P0,
clearClasses:Q0,removeClass:function(a){return Kg(function(b){n.remove(b,a)})},removeAttr:function(a){return Kg(function(b){m.remove(b,a)})},unwrap:R0,removeIfEmpty:U0},Hi=function(a){return"data-"+x.resolve(a)},JC=Hi("a11y-current-violation"),KC=x.resolve("a11y-violation"),LC=x.resolve("a11y-severity-info"),MC=x.resolve("a11y-severity-warn"),NC=x.resolve("a11y-severity-error"),Yl=Hi("a11y-ignore"),OC=Hi("a11y-error"),PC=Hi("a11y-warning"),QC=Hi("a11y-info"),kr=C.immutableBag(["style","attribute"],
[]),RC=kr({style:NC,attribute:OC}),SC=kr({style:MC,attribute:PC}),TC=kr({style:LC,attribute:QC}),V0=function(a){return"error"===a?h.some(RC):"warning"===a?h.some(SC):"info"===a?h.some(TC):h.none()},qo={configure:function(a){a.addSelector(a.output(),"["+Yl+"]",Ca.removeAttr(Yl));k.each([a.output(),a.input(),a.history(),a.internal()],function(b){k.each([KC,LC,MC,NC],function(c){a.add(b,c,Ca.removeClass(c))});k.each([JC,OC,PC,QC],function(c){a.addSelector(b,"["+c+"]",Ca.removeAttr(c))})})},ignoreAttr:f.constant(Yl),
manager:function(a){return O0.manager({content:f.constant(a),violation:f.constant(KC),currentAttr:f.constant(JC),ignoreAttr:f.constant(Yl),get:V0,all:f.constant([RC,SC,TC])})}},UC=x.resolve("highlight-common"),lr=x.resolve("highlight-placeholder"),ge={configure:function(a){a.add(a.output(),UC,Ca.unwrap);a.add(a.history(),lr,Ca.unwrap);a.add(a.output(),lr,Ca.unwrap);a.add(a.output(),"ephox-custom-highlight",Ca.unwrap);a.add(a.history(),"ephox-custom-highlight",Ca.unwrap)},highlightClass:f.constant(UC),
customHighlightClass:f.constant("ephox-custom-highlight"),placeholderClass:f.constant(lr)},VC=function(){return mr(function(a,b,c){return a()},"off")},WC=function(){return mr(function(a,b,c){return b()},"unchanged")},XC=function(a){return mr(function(b,c,d){return d(a)},"on")},mr=function(a,b){return{fold:a,map:function(b){return a(VC,WC,function(a){return XC(b(a))})},label:f.constant(b)}},oa={off:VC,unchanged:WC,on:XC},YC=C.immutable("content"),De={hasSelection:function(a){return a.getSelection().fold(function(){return YC(oa.off())},
function(){return YC(oa.on(!0))})},isAnnotation:function(a){return n.has(a,"ephox-cram-annotation-wrap")||n.has(a,ge.customHighlightClass())},isEmptyText:function(a){return A.isText(a)&&0===Da.get(a).length},isTextArea:function(a){return"textarea"===A.name(a)}},nr=function(a){return De.isTextArea(a)||"img"===A.name(a)},W0=function(a){return"false"===m.get(a,"contenteditable")},X0=function(a){a=a[a.length-1];var b=ka.getEnd(a);return S.point(a,b)},Kk={descend:function(a,b){var c=y.children(a);if(0===
c.length||nr(a))return S.point(a,b);if(b<c.length&&!nr(c[b]))return S.point(c[b],0);c=c[c.length-1];if(nr(c))return S.point(a,b);var d=A.isText(c)?Da.get(c).length:y.children(c).length;return S.point(c,d)},descendUntilLeaf:function(a,b){var c=S.point(a,b),d=function(a){return W0(a.element())?c:y.child(a.element(),a.offset()).fold(function(){return a},function(a){return d(S.point(a,0))})},e=y.children(a);return 0<e.length&&b<e.length?d(S.point(e[b],0)):0<e.length&&b===e.length?d(X0(e)):c}},or=function(a,
b){var c=Na.absolute(a),d=ia.get(a);return{element:f.constant(a),bottom:f.constant(c.top()+d),pos:f.constant(c),cleanup:b}},Y0=function(a,b){var c=Kk.descend(a,b),d=q.fromText(Ua.zeroWidth()),e=q.fromTag("span");v.append(e,d);v.before(c.element(),e);return or(e,function(){B.remove(e)})},$C=function(a,b){a.getSelection().each(function(c){ZC(a,function(d,e){Z0(a,b,c)},c)})},Z0=function(a,b,c){a.body().each(function(d){a.doc().each(function(a){t.reflow(d);var g=Ea.get(a).top(),f=Y0(c.startContainer(),
c.startOffset());b(a,g,f);f.cleanup()})})},aD=function(a,b,c){a.doc().each(function(a){var e=Ea.get(a).top();c(a,e,b)})},bD=function(a,b,c){var d=a.dom().defaultView.innerHeight,e=c.pos();c.bottom()>d+b?(b=e.top()-d+1.5*ia.get(c.element()),Ea.to(e.left(),b,a)):e.top()<b&&Ea.to(e.left(),e.top(),a)},cD=function(a,b,c,d){a.body().each(function(e){var g=ia.get(a.element()),f,h=e.dom();f=Math.max(h.offsetHeight,h.scrollHeight);h=d.pos();d.bottom()>f?t.set(e,"min-height",d.bottom()+"px"):t.remove(e,"min-height");
h.top()<c?Ea.to(h.left(),h.top(),b):h.top()>g+c&&(e=h.top()-g+(d.bottom()-h.top()),Ea.to(h.left(),e,b));e=wi.find(d.element());g=Ea.get().top();h=window.innerHeight+g;e.top()<g?Ea.intoView(d.element(),!0):e.top()>h&&Ea.intoView(d.element(),!1)})},$0=function(a){$C(a,bD)},a1=function(a,b){aD(a,or(b,f.noop),bD)},b1=function(a){$C(a,f.curry(cD,a))},c1=function(a,b){aD(a,or(b,f.noop),f.curry(cD,a))},ZC=function(a,b,c){var d=Kk.descend(c.startContainer(),c.startOffset()),e=Kk.descend(c.endContainer(),
c.endOffset());b(d.element(),e.element());a.setSelection(c.startContainer(),c.startOffset(),c.endContainer(),c.endOffset())},uf={cursorIntoView:function(a){(a.isInline()?$0:b1)(a)},elementIntoView:function(a,b){(a.isInline()?a1:c1)(a,b)},preserve:function(a,b){a.getSelection().each(function(c){ZC(a,b,c)})}},d1=Y.detect(),pr=function(a,b,c){a=q.fromTag(a);c=u.translate(c);return dD(a,b,c)},dD=function(a,b,c){c=Q.labelledField(a,b,c);n.add(c.element(),x.resolve("form-group"));n.add(c.element(),x.resolve(b));
"select"===A.name(a)&&(a=P.div(),n.add(a,x.resolve("select-wrap")),v.append(a,c.field()),v.append(c.element(),a));return c},hc={input:function(a,b){var c=pr("input",a,b);d1.browser.isSafari()&&m.setAll(c.field(),{autocomplete:"off",autocorrect:"off",autocapitalize:"off",spellcheck:"false"});m.setAll(c.field(),{type:"text"});return{element:c.element,input:c.field}},textarea:function(a,b){var c=pr("textarea",a,b);Q.multiline(c.field());return{element:c.element,textarea:c.field}},common:pr,commonWith:dD,
comboBox:function(a,b,c){var d=q.fromTag("input"),e=u.translate(b);b=P.div();var g=a+La.generate(""),e=q.fromHtml("<label>"+e+"</label>");m.set(e,"for",g);m.set(d,"id",g);n.add(b,x.resolve("form-group"));n.add(b,x.resolve(a));a=P.span();Q.hidden(a,!0);g=P.span();Q.hidden(g,!0);ea.add(g,["ephox-textbox-font",x.resolve("icon"),x.resolve("combo-drop-arrow-icon")]);n.add(a,x.resolve("combo-drop-arrow"));v.append(a,g);var l=P.div();n.add(l,x.resolve("label-wrap"));c.each(function(a){v.append(l,a)});v.append(l,
e);c=P.div();n.add(c,x.resolve("combo-wrap"));K.append(c,[d,a]);m.set(d,"type","text");K.append(b,[l,c]);return{element:f.constant(b),input:f.constant(d),dropArrow:f.constant(a)}}},eD=function(a){n.add(a,x.resolve("dialog-button-default"))},fD=function(a){n.add(a,x.resolve("dialog-button"))},qr=function(a,b,c){var d=ac(a+b);n.add(d.element(),x.resolve("dialog-"+b));a=qc(u.translate(c));d.content(a);m.set(d.element(),"tabindex","-1");Q.textButton(d.element(),a.element());return{enable:function(){d.enable();
ha.enable(d.element())},disable:function(){d.disable();ha.disable(d.element())},element:d.element,name:d.name,destroy:d.destroy,content:d.content,disabled:d.disabled,click:d.click,events:d.events}},e1=function(a){a=qr(a,"ok","button.ok");eD(a.element());return a},f1=function(a){a=qr(a,"cancel","button.cancel");fD(a.element());return a},Wc={button:qr,okCancel:function(a){var b=void 0!==a?a+"-":"";a=P.div();n.add(a,x.resolve(b+"dialog-controls"));var c=f1(b),d=e1(b);K.append(a,[c.element(),d.element()]);
b=D.create({ok:M([],d.events.click),cancel:M([],c.events.click)});return{element:f.constant(a),ok:f.constant(d),cancel:f.constant(c),destroy:function(){d.destroy();c.destroy()},events:b.registry}},makeNormal:fD,makeDefault:eD},rr=function(a,b){var c=a(),d=c.element();Q.button(d,b);m.set(d,"tabindex","-1");c.events.click.bind(function(){T.focus(d)});return c},gD=function(a,b){var c=rr(ac,b);c.content(Qd([a,x.resolve("icon")]));m.set(c.element(),"title",b);return c},wP={setDefaultAction:function(a){Wc.makeDefault(a)},
setNormalButton:function(a){Wc.makeNormal(a)},makeTextButton:function(a,b,c){b=rr(ac,b);ea.add(b.element(),a);b.content(qc(c));return b},makeIconToggle:function(a,b,c){var d=rr(id,c);d.content(Qd([b,x.resolve("icon")]));n.add(d.element(),x.resolve("choice-button"));m.set(d.element(),"data-value",a);m.set(d.element(),"title",c);return d},makeField:function(a,b,c){return hc.commonWith(a,b,c)},makeIconButton:gD,makeIconStepperButton:function(a,b){var c=gD(a,b);n.add(c.element(),x.resolve("stepper-button"));
return c}},sr={"a11y.widget.ok.label":"button.ok","a11y.widget.ok.text":"button.ok","a11y.widget.ignore.label":"a11y.widget.ignore","a11y.widget.ignore.text":"a11y.widget.ignore","a11y.widget.repair.label":"a11y.widget.repair","a11y.widget.repair.text":"a11y.widget.repair","a11y.widget.repair.available":"a11y.widget.available","a11y.i1.description":"a11y.image.alttext.empty","a11y.i1.repair.info":"a11y.image.alttext.set","a11y.i1.repair.errors.empty":"a11y.image.alttext.validation.empty","a11y.i1.repair.errors.duplicate":"a11y.image.alttext.validation.filenameduplicate",
"a11y.i2.description":"a11y.image.alttext.filenameduplicate","a11y.i2.repair.info":"a11y.image.alttext.set","a11y.i2.repair.errors.empty":"a11y.image.alttext.validation.empty","a11y.i2.repair.errors.duplicate":"a11y.image.alttext.validation.filenameduplicate","a11y.t1.description":"a11y.table.caption.empty","a11y.t1.repair.info":"a11y.table.caption.set","a11y.t1.repair.errors.empty":"a11y.table.caption.validation.empty","a11y.t1.repair.errors.duplicate":"a11y.table.caption.validation.summaryduplicate",
"a11y.t2.description":"a11y.table.summary.empty","a11y.t2.repair.info":"a11y.table.summary.set","a11y.t2.repair.errors.empty":"a11y.table.summary.validation.empty","a11y.t2.repair.errors.duplicate":"a11y.table.summary.validation.captionduplicate","a11y.t3.description":"a11y.table.caption.summaryduplicate","a11y.t3.repair.info":"a11y.table.caption.set","a11y.t3.repair.errors.empty":"a11y.table.caption.validation.empty","a11y.t3.repair.errors.duplicate":"a11y.table.caption.validation.summaryduplicate",
"a11y.t4a.description":"a11y.table.rowscells.none","a11y.t4b.description":"a11y.table.headers.none","a11y.t4b.repair.info":"a11y.table.headers.set","a11y.t4b.repair.row":"table.row-header","a11y.t4b.repair.column":"table.column-header","a11y.t4b.repair.errors.none":"a11y.table.headers.validation.none","a11y.t4c.description":"a11y.table.headerscope.none","a11y.t4c.repair.info":"a11y.table.headerscope.set","a11y.t4c.repair.scope.row":"a11y.table.headerscope.scope.row","a11y.t4c.repair.scope.col":"a11y.table.headerscope.scope.col",
"a11y.t4c.repair.scope.rowgroup":"a11y.table.headerscope.scope.rowgroup","a11y.t4c.repair.scope.colgroup":"a11y.table.headerscope.scope.colgroup","a11y.d1.description":"a11y.heading.paragraphmisuse","a11y.d1.repair.info":"a11y.heading.set","a11y.d1.repair.block.h1":"block.h1","a11y.d1.repair.block.h2":"block.h2","a11y.d1.repair.block.h3":"block.h3","a11y.d1.repair.block.h4":"block.h4","a11y.d1.repair.block.h5":"block.h5","a11y.d1.repair.block.h6":"block.h6","a11y.d2.description":"a11y.heading.nonsequential",
"a11y.d3.description":"a11y.link.adjacent","a11y.d4.description":"a11y.list.paragraphmisuse","a11y.d5a.description":"a11y.contrast.largetext","a11y.d5b.description":"a11y.contrast.smalltext"},zv={get:function(a){return h.from(sr[a])},translate:function(a){var b=Array.prototype.slice.call(arguments,1);return u.translate.apply(void 0,[void 0!==sr[a]?sr[a]:a].concat(b))}},yP=C.immutableBag(["rules"],[]),hD=C.immutableBag(["rule","element"],[]),g1=function(a,b){return k.bind(b,function(b){return b.check()(a)?
[hD({rule:b,element:a})]:[]})},h1=function(a,b){return k.bind(b,function(b){return k.map(b.check()(a),function(a){return hD({rule:b,element:a})})})},mP={audit:function(a,b){for(var c=[],d=k.partition(b,function(a){return a.walkable()}),e=d.pass,d=d.fail,g=document.createTreeWalker(a,Vb.getOrDie("NodeFilter").SHOW_ELEMENT,null,!1);g.nextNode();)c=c.concat(g1(g.currentNode,e));e=h1(a,d);c=c.concat(e);c.sort(function(a,b){var c=a.element(),d=b.element();return c===d?0:jk.documentPositionPreceding(c,
d)?1:-1});return c}},jc=C.immutableBag("id wcag severity desckey url walkable check".split(" "),[]),kc={error:f.constant("error"),warning:f.constant("warning"),info:f.constant("info")},Pf={match:function(a,b){return a.matches?a.matches(b):a.webkitMatchesSelector?a.webkitMatchesSelector(b):a.mozMatchesSelector?a.mozMatchesSelector(b):a.msMatchesSelector?a.msMatchesSelector(b):!1}},i1=function(a){return k.forall(a.childNodes,function(a){return/^(b|strong)$/i.test(a.nodeName)})},j1=jc({id:"D1",wcag:"1.3.1",
severity:kc.warning(),desckey:"a11y.d1.description",url:"http://www.w3.org/TR/UNDERSTANDING-WCAG20/content-structure-separation-programmatic.html",walkable:!0,check:function(a){return Pf.match(a,"p")?i1(a):!1}}),k1=jc({id:"D2",wcag:"1.3.1",severity:kc.error(),desckey:"a11y.d2.description",url:"http://www.w3.org/TR/UNDERSTANDING-WCAG20/content-structure-separation-programmatic.html",walkable:!1,check:function(a){a=a.querySelectorAll("h1, h2, h3, h4, h5, h6");var b,c=[];k.each(a,function(a){var e=parseInt(a.tagName.slice(1),
10);b&&1<e-b&&c.push(a);b=e});return c}}),l1=jc({id:"D3",wcag:"2.4.4",severity:kc.error(),desckey:"a11y.d3.description",url:"http://www.w3.org/TR/UNDERSTANDING-WCAG20/navigation-mechanisms-refs.html",walkable:!1,check:function(a){return k.filter(a.querySelectorAll("a + a"),function(a){return a.href===a.previousSibling.href})}}),iD=function(a){var b=a.nodeName;for(a=a.nextSibling;null!==a&&("#text"===a.nodeName||"BR"===a.nodeName);)a=a.nextSibling;return null!==a&&a.nodeName===b?a:!1},jD={check:function(a,
b){for(var c=a.querySelectorAll("p,h1,h2,h3,h4,h5,h6"),d=[],e=0;e<c.length;){var g,f=c[e];g=b;for(var h=iD(f),f=g(f)?[f]:[];!1!==h&&g(h);)f.push(h),h=iD(h);g=f;if(1<g.length){d.push(g[0]);g=g[g.length-1];h=-1;for(f=0;f<c.length;f++)if(c[f]===g){h=f;break}g=h;e=-1===g?e+1:g}else e++}return d}},m1=function(a){return a.textContent.match(/^\s{0,}([a-z1-9]|i+)\s{0,}[.)-]/gmi)},n1=jc({id:"D4o",wcag:"1.3.1",severity:kc.error(),desckey:"a11y.d4.description",url:"http://www.w3.org/TR/UNDERSTANDING-WCAG20/content-structure-separation-programmatic.html",
walkable:!1,check:function(a){return jD.check(a,m1)}}),o1=function(a){return a.textContent.match(/^\s{0,}(\*|-)\s{0,}/gmi)},p1=jc({id:"D4u",wcag:"1.3.1",severity:kc.error(),desckey:"a11y.d4.description",url:"http://www.w3.org/TR/UNDERSTANDING-WCAG20/content-structure-separation-programmatic.html",walkable:!1,check:function(a){return jD.check(a,o1)}}),kD=C.immutable("red","green","blue","alpha"),Ii={rgbaColour:kD,white:function(){return kD(255,255,255,1)}},Lg={calculateContrastRatio:function(a,b){var c=
Math.max(a,b)+0.05,d=Math.min(a,b)+0.05;return c/d},calculateLuminance:function(a){var b=function(a){return 0.03928>=a?a/12.92:Math.pow((a+0.055)/1.055,2.4)},c=a.red()/255,d=a.green()/255;a=a.blue()/255;c=b(c);d=b(d);b=b(a);return 0.2126*c+0.7152*d+0.0722*b},composeColour:function(a,b){var c=a.alpha()+b.alpha()*(1-a.alpha()),d=(a.red()*a.alpha()+b.red()*b.alpha()*(1-a.alpha()))/c,e=(a.green()*a.alpha()+b.green()*b.alpha()*(1-a.alpha()))/c,g=(a.blue()*a.alpha()+b.blue()*b.alpha()*(1-a.alpha()))/c;
return Ii.rgbaColour(Math.ceil(d),Math.ceil(e),Math.ceil(g),c)}},lD={parseColour:function(a){var b=function(a,b,c,f){a=parseInt(a,10);b=parseInt(b,10);c=parseInt(c,10);f=parseFloat(f);return h.some(Ii.rgbaColour(a,b,c,f))};if("transparent"===a)return b(0,0,0,0);var c=a.match(/^rgb\((\d+),\s*(\d+),\s*(\d+)\)/);if(null!==c)return b(c[1],c[2],c[3],1);a=a.match(/^rgba\((\d+),\s*(\d+),\s*(\d+),\s*(\d?(?:\.\d+)?)\)/);return null!==a?b(a[1],a[2],a[3],a[4]):h.none()}},mD=function(a,b){var c=t.get(a,"opacity");
return 1>c?Ii.rgbaColour(b.red(),b.green(),b.blue(),b.alpha()*c):b},nD={getBackgroundColor:function(a){var b=y.owner(a),c=ca.getBody(b);a=[a].concat(y.parents(a,function(a){return L.eq(a,c)}));a=k.foldl(a,function(a,b){if(1===a.alpha())return a;var c=t.get(b,"background-color");return lD.parseColour(c).bind(function(c){c=mD(b,c);return 0<c.alpha()?h.some(Lg.composeColour(a,c)):h.none()}).getOr(a)},Ii.rgbaColour(0,0,0,0));return 1>a.alpha()?Lg.composeColour(a,Ii.white()):a},getForegroundColor:function(a,
b){var c=t.get(a,"color");return lD.parseColour(c).bind(function(c){c=mD(a,c);c=1>c.alpha()?Lg.composeColour(c,b):c;return h.some(c)})}},oD=function(a){var b=nD.getBackgroundColor(a);return nD.getForegroundColor(a,b).map(function(a){var d=Lg.calculateLuminance(b);a=Lg.calculateLuminance(a);return Lg.calculateContrastRatio(a,d)})},pD={check:function(a,b){return oD(a).filter(function(a){return a<b}).isSome()},getRatio:oD},q1=function(a){a=t.get(a,"font-weight");return"bold"===a||"bolder"===a?!0:/^[0-9]*$/.test(a)?
700<=parseInt(a,10):!1},qD=function(a){var b=t.get(a,"font-size");a=/.*px/.test(b);b=b.match(/(\d*\.?\d*)(.*)/);return a&&null!==b?h.some(Number(b[1])):h.none()},rD={isLargeFont:function(a){var b=q1(a),c=y.owner(a),c=ca.getBody(c),d=qD(c);return qD(a).fold(f.never,function(a){return d.fold(function(){return 24<=a||b&&19<=a},function(c){return a>=1.5*c||b&&a>=1.2*c})})}},sD={hasNonEmptyTextChild:function(a){a=y.children(a);return 0<k.filter(a,function(a){return A.isText(a)&&0<A.value(a).trim().length}).length}},
r1=jc({id:"D5a",wcag:"1.4.3",severity:kc.error(),desckey:"a11y.d5a.description",url:"https://www.w3.org/TR/UNDERSTANDING-WCAG20/visual-audio-contrast-contrast.html",walkable:!0,check:function(a){a=q.fromDom(a);return sD.hasNonEmptyTextChild(a)&&rD.isLargeFont(a)&&pD.check(a,3)}}),s1=jc({id:"D5b",wcag:"1.4.3",severity:kc.error(),desckey:"a11y.d5b.description",url:"https://www.w3.org/TR/UNDERSTANDING-WCAG20/visual-audio-contrast-contrast.html",walkable:!0,check:function(a){a=q.fromDom(a);return sD.hasNonEmptyTextChild(a)&&
!rD.isLargeFont(a)&&pD.check(a,4.5)}}),t1=jc({id:"I1",wcag:"1.1.1",severity:kc.error(),desckey:"a11y.i1.description",url:"http://www.w3.org/TR/UNDERSTANDING-WCAG20/text-equiv-all.html",walkable:!0,check:function(a){return Pf.match(a,"img")&&(!1===a.hasAttribute("alt")||""===a.getAttribute("alt").trim())}}),u1=jc({id:"I2",wcag:"1.1.1",severity:kc.error(),desckey:"a11y.i2.description",url:"http://www.w3.org/TR/UNDERSTANDING-WCAG20/text-equiv-all.html",walkable:!0,check:function(a){return Pf.match(a,
"img[alt][src]")&&a.src.split("/").pop()===a.alt.split("/").pop()}}),v1=jc({id:"T1",wcag:"1.3.1",severity:kc.error(),desckey:"a11y.t1.description",url:"http://www.w3.org/TR/UNDERSTANDING-WCAG20/content-structure-separation-programmatic.html",walkable:!0,check:function(a){return Pf.match(a,"table")?(a=a.querySelector("caption"))?Pf.match(a,":empty"):!0:!1}}),w1=jc({id:"T2",wcag:"1.3.1",severity:kc.warning(),desckey:"a11y.t2.description",url:"http://www.w3.org/TR/UNDERSTANDING-WCAG20/content-structure-separation-programmatic.html",
walkable:!0,check:function(a){return Pf.match(a,"table")&&(a.querySelector("[rowspan]")||a.querySelector("[colspan]"))&&!a.hasAttribute("summary")}}),x1=jc({id:"T3",wcag:"1.3.1",severity:kc.error(),desckey:"a11y.t3.description",url:"http://www.w3.org/TR/UNDERSTANDING-WCAG20/content-structure-separation-programmatic.html",walkable:!0,check:function(a){return Pf.match(a,"table")&&a.hasAttribute("summary")&&0<a.querySelectorAll("caption").length&&a.getAttribute("summary")===a.querySelector("caption").innerHTML}}),
y1=jc({id:"T4b",wcag:"1.3.1",severity:kc.error(),desckey:"a11y.t4b.description",url:"http://www.w3.org/TR/UNDERSTANDING-WCAG20/content-structure-separation-programmatic.html",walkable:!0,check:function(a){return"table"===a.nodeName.toLowerCase()&&0===a.querySelectorAll("th").length}}),z1=jc({id:"T4c",wcag:"1.3.1",severity:kc.error(),desckey:"a11y.t4c.description",url:"http://www.w3.org/TR/UNDERSTANDING-WCAG20/content-structure-separation-programmatic.html",walkable:!0,check:function(a){var b;"th"===
a.nodeName.toLowerCase()?(b=!1===a.hasAttribute("scope"),a=!1===/row|col|rowgroup|colgroup/i.test(a.getAttribute("scope")),b=b||a):b=!1;return b}}),A1=jc({id:"T4a",wcag:"1.3.1",severity:kc.error(),desckey:"a11y.t4a.description",url:"http://www.w3.org/TR/UNDERSTANDING-WCAG20/content-structure-separation-programmatic.html",walkable:!0,check:function(a){return"table"===a.nodeName.toLowerCase()?0===a.querySelectorAll("tr").length||0===a.querySelectorAll("td").length:!1}}),tD=[j1,k1,l1,n1,p1,r1,s1,t1,
u1,v1,w1,x1,z1,A1,y1],uD={get:function(a){return k.find(tD,function(b){return b.name()===a})},all:f.constant(tD)},xP={get:function(a){return uD.get(a)},all:function(){return uD.all().slice()}},B1={resolve:lb.create("ephox-wheel").resolve},Ha={resolve:function(a,b){return B1.resolve(a.prefix()+"-"+b)},translate:function(a,b){var c=[(aa.startsWith(b,a.tnames())?"":a.tnames()+".")+b].concat(Array.prototype.slice.call(arguments,2));return a.translate.apply(null,c)}},tr=function(a,b,c,d,e){c=Ha.translate(e,
c);d=Ha.translate(e,d);return a.makeTextButton(b,c,d)},Td={defaultActionButton:function(a,b,c,d,e){b=tr(a,b,c,d,e);a.setDefaultAction(b.element());return b},normalButton:function(a,b,c,d,e){b=tr(a,b,c,d,e);a.setNormalButton(b.element());return b},button:tr,toggleButton:function(a,b,c,d,e){d=Ha.translate(e,d);return a.makeIconToggle(b,c,d)},field:function(a,b,c,d,e){d=Ha.translate(e,d);return a.makeField(b,c,d)},icon:function(a,b,c,d){c=Ha.translate(d,c);return a.makeIconStepperButton(b,c)}},C1=function(a,
b,c,d,e,g){var l=q.fromTag("input");a=Td.field(a,l,c,d,g);n.add(l,Ha.resolve(g,"input"));b=[ba.input(l,pa.input({execute:b}))];return{element:a.element,get:function(){var a={};a[c]=$.get(l);return a},update:function(a){a=e(a);$.set(l,a)},keylayouts:f.constant(b),destroy:f.noop}},D1=function(a,b,c,d,e){var g=q.fromTag("select");a=Td.field(a,g,b,c,e);d=k.map(d,function(a){var b=q.fromTag("option");$.set(b,a.value());ua.set(b,Ha.translate(e,a.text()));return b});K.append(g,d);d=[ba.single(g,[],h.none())];
return{element:a.element,get:function(){var a={};a[b]=$.get(g);return a},update:f.noop,keylayouts:f.constant(d),destroy:f.noop}},E1=function(a,b,c,d,e){var g=P.div();n.add(g,Ha.resolve(e,"flow"));var l=k.map(d,function(b){return Td.toggleButton(a,b.value(),b.icon(),b.textkey(),e)});d=k.map(l,function(a){return a.element()});K.append(g,d);d=[ba.flow(g,{item:f.constant(".ephox-pastry-statebutton")},pa.flow({trigger:function(a){return J.descendant(a,".ephox-pastry-button").map(function(a){a.dom().click();
return a})}}))];return{element:Td.field(a,g,b,c,e).element,get:function(){return k.foldr(l,function(a,b){var c={};c[m.get(b.element(),"data-value")]=b.selected();return Z.deepMerge(a,c)},{})},update:f.noop,keylayouts:f.constant(d),destroy:function(){k.each(l,function(a){a.destroy()})}}},pP={generate:function(a,b,c,d){b=b.getOr([]);var e=k.bind(b,function(a){return a.fold(function(a,b,c){return[a]},function(a,b,c){return[a]},function(a,b,c){return k.map(c,function(a){return a.value()})})}),g=0<e.length?
C.immutableBag(e,[]):f.constant({}),l=D.create({execute:s([])}),h=function(){l.trigger.execute()},r=k.map(b,function(b){return b.fold(function(b,c,e){return C1(a,h,b,c,e,d)},function(b,c,e){return D1(a,b,c,e,d)},function(b,c,e){return E1(a,b,c,e,d)})});k.each(r,function(a){a.update(c)});b=k.map(r,function(a){return a.element()});var m=k.bind(r,function(a){return a.keylayouts()});return{elements:f.constant(b),get:function(){var a=k.foldr(r,function(a,b){return Z.deepMerge(a,b.get())},{});return g(a)},
keylayouts:f.constant(m),destroy:function(){k.each(r,function(a){a.destroy()});k.each(m,function(a){a.destroy()})},events:l.registry}}},dc={title:f.constant("widget.title"),running:f.constant("widget.running"),previous:f.constant("widget.previous"),next:f.constant("widget.next"),issueCounter:f.constant("widget.issue.counter"),issueHelp:f.constant("widget.issue.help"),noIssue:f.constant("widget.issue.none"),severity:function(a){return"severity."+a},repairLabel:f.constant("widget.repair.label"),repairText:f.constant("widget.repair.text"),
repairAvailable:f.constant("widget.repair.available"),ignoreLabel:f.constant("widget.ignore.label"),ignoreText:f.constant("widget.ignore.text"),okLabel:f.constant("widget.ok.label"),okText:f.constant("widget.ok.text")},Wj=function(a,b,c){var d=P.div();n.add(d,Ha.resolve(c,a));b.each(function(a){v.append(d,q.fromText(Ha.translate(c,a)))});return d},vD=function(a){var b=wa(h.none());return{clear:function(){b.get().each(a);b.set(h.none())},isSet:function(){return b.get().isSome()},set:function(c){b.get().each(a);
b.set(h.some(c))}}},sc={destroyable:function(){return vD(function(a){a.destroy()})},unbindable:function(){return vD(function(a){a.unbind()})},api:function(){var a=wa(h.none()),b=function(){a.get().each(function(a){a.destroy()})};return{clear:function(){b();a.set(h.none())},isSet:function(){return a.get().isSome()},set:function(c){b();a.set(h.some(c))},run:function(b){a.get().each(b)}}},value:function(){var a=wa(h.none());return{clear:function(){a.set(h.none())},set:function(b){a.set(h.some(b))},isSet:function(){return a.get().isSome()},
on:function(b){a.get().each(b)}}}},wD=Y.detect(),Zl=wD.os.isOSX(),lc=Zl?"\u2318":"Ctrl",$l=Zl?"\u21e7":"Shift",xD={blockquote:[lc,$l,"9"],bold:[lc,"B"],italic:[lc,"I"],underline:[lc,"U"],link:[lc,"K"],help:[lc,"?"],fullscreen:[lc,$l,"F"],indent:[lc,"]"],outdent:[lc,"["],undo:[lc,"Z"],redo:Zl?[lc,$l,"Z"]:[lc,"Y"],search:[lc,"F"]},F1="fr nl sv ru de es it".split(" "),G1=Z.merge(xD,{indent:[lc,"M"],outdent:[lc,$l,"M"]}),yD=function(a){var b=fe.get();return k.contains(F1,b)?G1[a]:xD[a]},H1=function(a){a=
yD(a);return void 0===a?"":" "+a.join("")},I1=function(a){a=yD(a);return void 0===a?"":" ("+a.join(" ")+")"},J1=Zl?H1:I1,Wa={get:function(a,b){return wD.os.isiOS()?u.get(a):u.get(a)+J1(b)}},ro="a11y-check",BP=x.resolve("inverted-dialog-widget"),zD=function(a,b,c){var d=t.get(b,a);c=parseInt(d,10)+40*c;(0<c?t.set:t.remove)(b,a,c+"px")},K1={change:f.curry(zD,"margin-right")},Bv={ltr:{change:f.curry(zD,"margin-left")},rtl:K1},Yc={runTransaction:function(a,b,c,d){d=void 0!==d?d:f.noop;b.save();a.focus();
c=c();setTimeout(function(){a.domChanged();a.uiChanged()},0);d();b.save();return c}},ur=C.immutable("startContainer","startOffset","endContainer","endOffset"),vr=function(a,b){if(a.property().isText(b))return S.point(b,a.property().getText(b).length);var c=a.property().children(b);return 0<c.length?vr(a,c[c.length-1]):S.point(b,c.length)},AD=function(a,b,c){var d=a.property().children(b);return 0<d.length&&c<d.length?AD(a,d[c],0):0<d.length&&a.property().isElement(b)&&d.length===c?vr(a,d[d.length-
1]):S.point(b,c)},wr=function(a,b,c){return a.property().isText(b)&&0===a.property().getText(b).trim().length||a.property().isComment(b)?c(b).bind(function(b){return wr(a,b,c).orThunk(function(){return h.some(b)})}):h.none()},BD=function(a,b){var c=wr(a,b,a.query().nextSibling).getOr(b);if(a.property().isText(c))return S.point(c,0);var d=a.property().children(c);return 0<d.length?BD(a,d[0]):S.point(c,0)},CD=function(a,b){return a.property().isText(b)?a.property().getText(b).length:a.property().children(b).length},
DD=function(a,b){var c=wr(a,b,a.query().prevSibling).getOr(b);if(a.property().isText(c))return S.point(c,CD(a,c));var d=a.property().children(c);return 0<d.length?DD(a,d[d.length-1]):S.point(c,CD(a,c))},Ze={toLast:vr,toLeaf:AD,toLower:function(a,b){var c=a.property().isText(b)?a.property().getText(b).length:a.property().children(b).length;return S.point(b,c)},freefallLtr:BD,freefallRtl:DD},mc={toLeaf:function(a,b,c){return Ze.toLeaf(a,b,c)},freefallLtr:function(a,b){return Ze.freefallLtr(a,b)},freefallRtl:function(a,
b){return Ze.freefallRtl(a,b)}},xr=Sa(),Ja={toLeaf:function(a,b){return mc.toLeaf(xr,a,b)},freefallLtr:function(a){return mc.freefallLtr(xr,a)},freefallRtl:function(a){return mc.freefallRtl(xr,a)}},ED=function(a,b){return ca.inBody(a)&&0<=b&&b<y.children(a).length},am={multiple:function(a,b,c){return function(d,e){var g=a.getSelection(),f=g.map(function(a){return{left:y.parent(a.startContainer()).getOrDie(),right:y.parent(a.endContainer()).getOrDie()}}),h=Yc.runTransaction(a,b,function(){return k.bind(e,
function(a){return a.getSelection().bind(function(a){return c(a.startContainer(),a.startOffset(),a.endContainer(),a.endOffset()).map(function(a){return a.all()})}).getOr([])})},a.toEditing),g=g.filter(function(a){return ED(a.startContainer(),a.startOffset())&&ED(a.endContainer(),a.endOffset())}).getOrThunk(function(){var a=Ja.freefallLtr(0<h.length?h[0]:f.getOrDie().left),b=Ja.freefallRtl(0<h.length?h[h.length-1]:f.getOrDie().right);return ur(a.element(),a.offset(),b.element(),b.offset())});a.setSelection(g.startContainer(),
g.startOffset(),g.endContainer(),g.endOffset());return h}},single:function(a,b,c,d,e){return function(g,f){var h=d(a,g.startContainer(),g.startOffset()),k=d(a,g.endContainer(),g.endOffset()),m=Yc.runTransaction(a,b,function(){return c(h.element(),h.offset(),k.element(),k.offset())},a.toEditing);return e(a,m,h,k,g)}}},bm={multiple:function(a,b,c){return function(d,e){return Yc.runTransaction(a,b,function(){return k.bind(e,function(a){return a.getSelection().bind(function(a){return c(a.startContainer(),
a.startOffset(),a.endContainer(),a.endOffset()).map(function(a){return a.all()})}).getOr([])})},a.toEditing)}},single:function(a,b,c,d,e){return function(g,f){var h=d(a,g.startContainer(),g.startOffset()),k=d(a,g.endContainer(),g.endOffset()),m=Yc.runTransaction(a,b,function(){return c(h.element(),h.offset(),k.element(),k.offset())},a.toEditing);return e(a,m,h,k,g)}}},Ji=function(a,b,c,d,e){return b.map(function(b){a.setSelection(b.start(),b.soffset(),b.finish(),b.foffset());return b.all()}).getOr([])},
Ki=function(a,b,c){return S.point(b,c)},yr=ma.generate([{none:[]},{multiple:["elements","environments"]},{single:["selection","environment"]}]),tb={cata:function(a,b,c,d){return a.fold(b,c,d)},none:yr.none,multiple:yr.multiple,single:yr.single,toSelection:function(a){return a.fold(h.none,function(a,c){return h.from(c[0]).bind(function(a){return a.getSelection()})},h.some)}},zr=ma.generate([{before:["element"]},{on:["element","offset"]},{after:["element"]}]),da={before:zr.before,on:zr.on,after:zr.after,
cata:function(a,b,c,d){return a.fold(b,c,d)},getStart:function(a){return a.fold(f.identity,f.identity,f.identity)}},cm=ma.generate([{domRange:["rng"]},{relative:["startSitu","finishSitu"]},{exact:["start","soffset","finish","foffset"]}]),L1=C.immutable("start","soffset","finish","foffset"),M1=function(a){return a.match({domRange:function(a){return q.fromDom(a.startContainer)},relative:function(a,c){return da.getStart(a)},exact:function(a,c,d,e){return a}})},Wb={domRange:cm.domRange,relative:cm.relative,
exact:cm.exact,exactFromRange:function(a){return cm.exact(a.start(),a.soffset(),a.finish(),a.foffset())},range:L1,getWin:function(a){a=M1(a);return y.defaultView(a)}},FD=function(a,b,c,d){var e=y.owner(a).dom().createRange();e.setStart(a.dom(),b);e.setEnd(c.dom(),d);return e},Bc={after:function(a,b,c,d){var e=FD(a,b,c,d);a=L.eq(a,c)&&b===d;return e.collapsed&&!a},commonAncestorContainer:function(a,b,c,d){a=FD(a,b,c,d);return q.fromDom(a.commonAncestorContainer)}},N1={fromElements:function(a,b){var c=
(b||document).createDocumentFragment();k.each(a,function(a){c.appendChild(a.dom())});return q.fromDom(c)}},GD=function(a,b){a.selectNodeContents(b.dom())},O1=function(a,b){b.fold(function(b){a.setStartBefore(b.dom())},function(b,d){a.setStart(b.dom(),d)},function(b){a.setStartAfter(b.dom())})},P1=function(a,b){b.fold(function(b){a.setEndBefore(b.dom())},function(b,d){a.setEnd(b.dom(),d)},function(b){a.setEndAfter(b.dom())})},HD=function(a){a.deleteContents()},ID=function(a){return{left:f.constant(a.left),
top:f.constant(a.top),right:f.constant(a.right),bottom:f.constant(a.bottom),width:f.constant(a.width),height:f.constant(a.height)}},Ob={create:function(a){return a.document.createRange()},replaceWith:function(a,b){HD(a);a.insertNode(b.dom())},selectNodeContents:function(a,b){var c=a.document.createRange();GD(c,b);return c},selectNodeContentsUsing:GD,relativeToNative:function(a,b,c){a=a.document.createRange();O1(a,b);P1(a,c);return a},exactToNative:function(a,b,c,d,e){a=a.document.createRange();a.setStart(b.dom(),
c);a.setEnd(d.dom(),e);return a},deleteContents:HD,cloneFragment:function(a){a=a.cloneContents();return q.fromDom(a)},getFirstRect:function(a){var b=a.getClientRects();a=0<b.length?b[0]:a.getBoundingClientRect();return 0<a.width||0<a.height?h.some(a).map(ID):h.none()},getBounds:function(a){a=a.getBoundingClientRect();return 0<a.width||0<a.height?h.some(a).map(ID):h.none()},isWithin:function(a,b){return 1>b.compareBoundaryPoints(a.END_TO_START,a)&&-1<b.compareBoundaryPoints(a.START_TO_END,a)},toString:function(a){return a.toString()}},
Li=ma.generate([{ltr:["start","soffset","finish","foffset"]},{rtl:["start","soffset","finish","foffset"]}]),JD=function(a,b,c){return b(q.fromDom(c.startContainer),c.startOffset,q.fromDom(c.endContainer),c.endOffset)},Q1=function(a,b){return b.match({domRange:function(a){return{ltr:f.constant(a),rtl:h.none}},relative:function(b,d){return{ltr:hb.cached(function(){return Ob.relativeToNative(a,b,d)}),rtl:hb.cached(function(){return h.some(Ob.relativeToNative(a,d,b))})}},exact:function(b,d,e,g){return{ltr:hb.cached(function(){return Ob.exactToNative(a,
b,d,e,g)}),rtl:hb.cached(function(){return h.some(Ob.exactToNative(a,e,g,b,d))})}}})},R1=function(a,b){var c=b.ltr();return c.collapsed?b.rtl().filter(function(a){return!1===a.collapsed}).map(function(a){return Li.rtl(q.fromDom(a.endContainer),a.endOffset,q.fromDom(a.startContainer),a.startOffset)}).getOrThunk(function(){return JD(a,Li.ltr,c)}):JD(a,Li.ltr,c)},KD=function(a,b){var c=Q1(a,b);return R1(a,c)},Hd={ltr:Li.ltr,rtl:Li.rtl,diagnose:KD,asLtrRange:function(a,b){return KD(a,b).match({ltr:function(b,
d,e,g){var f=a.document.createRange();f.setStart(b.dom(),d);f.setEnd(e.dom(),g);return f},rtl:function(b,d,e,g){var f=a.document.createRange();f.setStart(e.dom(),g);f.setEnd(b.dom(),d);return f}})}},Ar={inRect:function(a,b,c){return b>=a.left&&b<=a.right&&c>=a.top&&c<=a.bottom},searchForPoint:function(a,b,c,d,e){if(0===e)return 0;if(b===d)return e-1;for(var g=1;g<e;g++){var f=a(g),h=Math.abs(b-f.left);if(!(c>f.bottom)){if(c<f.top||h>d)return g-1;d=h}}return 0}},S1=function(a,b,c,d,e){var g=function(c){var d=
a.dom().createRange();d.setStart(b.dom(),c);d.collapse(!0);return d},f=Da.get(b).length;c=Ar.searchForPoint(function(a){return g(a).getBoundingClientRect()},c,d,e.right,f);return g(c)},LD=function(a,b,c,d){var e=a.dom().createRange();e.selectNode(b.dom());e=e.getClientRects();return ab.findMap(e,function(a){return Ar.inRect(a,c,d)?h.some(a):h.none()}).map(function(e){return S1(a,b,c,d,e)})},MD=function(a,b,c,d){var e=a.dom().createRange();b=y.children(b);return ab.findMap(b,function(b){e.selectNode(b.dom());
return Ar.inRect(e.getBoundingClientRect(),c,d)?(A.isText(b)?LD:MD)(a,b,c,d):h.none()})},T1={locate:function(a,b,c,d){var e=a.dom().createRange();e.selectNode(b.dom());e=e.getBoundingClientRect();c=Math.max(e.left,Math.min(e.right,c));d=Math.max(e.top,Math.min(e.bottom,d));return(A.isText(b)?LD:MD)(a,b,c,d)}},ND=function(a,b,c){a=a.dom().createRange();a.selectNode(b.dom());a.collapse(c);return a},U1=function(a,b,c){var d=a.dom().createRange();d.selectNode(b.dom());var d=d.getBoundingClientRect(),
e=c-d.left<d.right-c?!0:!1;return(!0===e?Cb.first:Cb.last)(b).map(function(b){return ND(a,b,e)})},V1=function(a,b,c){var d=b.dom().getBoundingClientRect();return h.some(ND(a,b,c-d.left<d.right-c?!0:!1))},W1={search:function(a,b,c){return(0===y.children(b).length?V1:U1)(a,b,c)}},X1=function(a,b,c){return h.from(a.dom().caretPositionFromPoint(b,c)).bind(function(b){if(null===b.offsetNode)return h.none();var c=a.dom().createRange();c.setStart(b.offsetNode,b.offset);c.collapse();return h.some(c)})},Y1=
function(a,b,c){return h.from(a.dom().caretRangeFromPoint(b,c))},Z1=function(a,b,c,d){var e=a.dom().createRange();e.selectNode(b.dom());e=e.getBoundingClientRect();c=Math.max(e.left,Math.min(e.right,c));d=Math.max(e.top,Math.min(e.bottom,d));return T1.locate(a,b,c,d)},$1=function(a,b,c){return q.fromPoint(a,b,c).bind(function(d){var e=function(){return W1.search(a,d,b)};return 0===y.children(d).length?e():Z1(a,d,b,c).orThunk(e)})},a2=document.caretPositionFromPoint?X1:document.caretRangeFromPoint?
Y1:$1,b2={fromPoint:function(a,b,c){a=q.fromDom(a.document);return a2(a,b,c).map(function(a){return Wb.range(q.fromDom(a.startContainer),a.startOffset,q.fromDom(a.endContainer),a.endOffset)})}},c2=function(a,b,c,d){var e=Ob.create(a);a=(ra.is(b,d)?[b]:[]).concat(X.descendants(b,d));return k.filter(a,function(a){Ob.selectNodeContentsUsing(e,a);return Ob.isWithin(c,e)})},d2={find:function(a,b,c){b=Hd.asLtrRange(a,b);var d=q.fromDom(b.commonAncestorContainer);return A.isElement(d)?c2(a,d,b,c):[]}},Mi=
function(a,b){var c=A.name(a);return"input"===c?da.after(a):k.contains(["br","img"],c)?0===b?da.before(a):da.after(a):da.on(a,b)},OD=function(a,b){var c=a.fold(da.before,Mi,da.after),d=b.fold(da.before,Mi,da.after);return Wb.relative(c,d)},Br=function(a,b,c,d){a=Mi(a,b);c=Mi(c,d);return Wb.relative(a,c)},Cr={beforeSpecial:Mi,preprocess:function(a){return a.match({domRange:function(a){var c=q.fromDom(a.startContainer),d=q.fromDom(a.endContainer);return Br(c,a.startOffset,d,a.endOffset)},relative:OD,
exact:Br})},preprocessRelative:OD,preprocessExact:Br},Dr=function(a,b){h.from(a.getSelection()).each(function(a){a.removeAllRanges();a.addRange(b)})},PD=function(a,b){return Hd.diagnose(a,b).match({ltr:function(b,d,e,g){b=Ob.exactToNative(a,b,d,e,g);Dr(a,b)},rtl:function(b,d,e,g){var f=a.getSelection();f.setBaseAndExtent?f.setBaseAndExtent(b.dom(),d,e.dom(),g):f.extend?(f.collapse(b.dom(),d),f.extend(e.dom(),g)):(b=Ob.exactToNative(a,e,g,b,d),Dr(a,b))}})},QD=function(a){a=a.getSelection();if(0<a.rangeCount){var b=
q.fromDom(a.anchorNode),c=q.fromDom(a.focusNode);Bc.after(b,a.anchorOffset,c,a.focusOffset)?a=h.some(Wb.range(q.fromDom(a.anchorNode),a.anchorOffset,q.fromDom(a.focusNode),a.focusOffset)):0<a.rangeCount?(b=a.getRangeAt(0),a=a.getRangeAt(a.rangeCount-1),a=h.some(Wb.range(q.fromDom(b.startContainer),b.startOffset,q.fromDom(a.endContainer),a.endOffset))):a=h.none()}else a=h.none();return a},Fa={setExact:function(a,b,c,d,e){b=Cr.preprocessExact(b,c,d,e);PD(a,b)},getExact:QD,get:function(a){return QD(a).map(function(a){return Wb.exact(a.start(),
a.soffset(),a.finish(),a.foffset())})},setRelative:function(a,b,c){b=Cr.preprocessRelative(b,c);PD(a,b)},toNative:function(a){var b=Wb.getWin(a).dom(),c=function(a,c,g,f){return Ob.exactToNative(b,a,c,g,f)};a=Cr.preprocess(a);return Hd.diagnose(b,a).match({ltr:c,rtl:c})},setToElement:function(a,b){var c=Ob.selectNodeContents(a,b);Dr(a,c)},clear:function(a){a.getSelection().removeAllRanges()},clone:function(a,b){var c=Hd.asLtrRange(a,b);return Ob.cloneFragment(c)},replace:function(a,b,c){b=Hd.asLtrRange(a,
b);a=N1.fromElements(c,a.document);Ob.replaceWith(b,a)},deleteAt:function(a,b){var c=Hd.asLtrRange(a,b);Ob.deleteContents(c)},forElement:function(a,b){var c=Ob.selectNodeContents(a,b);return Wb.range(q.fromDom(c.startContainer),c.startOffset,q.fromDom(c.endContainer),c.endOffset)},getFirstRect:function(a,b){var c=Hd.asLtrRange(a,b);return Ob.getFirstRect(c)},getBounds:function(a,b){var c=Hd.asLtrRange(a,b);return Ob.getBounds(c)},getAtPoint:function(a,b,c){return b2.fromPoint(a,b,c)},findWithin:function(a,
b,c){return d2.find(a,b,c)},getAsString:function(a,b){var c=Hd.asLtrRange(a,b);return Ob.toString(c)},isCollapsed:function(a,b,c,d){return L.eq(a,c)&&b===d}},RD=function(a,b){return ca.inBody(a)?h.some(Math.max(0,Math.min(b,ka.getEnd(a)))):h.none()},Er={findDescendants:function(a,b,c,d,e){var g=y.defaultView(a);return RD(a,b).bind(function(b){return RD(c,d).map(function(d){d=Wb.exact(a,b,c,d);return Fa.findWithin(g.dom(),d,e)})}).getOr([])},findAncestors:function(a,b,c){return J.ancestor(a,c).fold(function(){return J.ancestor(b,
c).toArray()},function(a){return J.ancestor(b,c).fold(f.constant([a]),function(b){return L.eq(a,b)?[a]:[a,b]})})}},Fr=Y.detect().browser.isFirefox(),e2=function(a,b){return SD(a).filter(function(a){return L.eq(a.image(),b)}).isSome()},TD=function(a,b){var c=function(){a.setRelativeSelection(da.before(b),da.after(b))},d=Fr?function(){a.getSelection().fold(c,function(c){f2(a,c,b)})}:c;a.focus();d()},f2=function(a,b,c){2===Math.abs(b.startOffset()-b.endOffset())&&e2(b,c)||(b=y.nextSibling(c).filter(De.isEmptyText).getOrThunk(function(){var a=
q.fromText("");v.after(c,a);return a}),a.setRelativeSelection(da.before(c),da.after(b)))},Ni=function(a){return a.filter(function(a){return"img"===A.name(a)})},g2=function(a){var b=Ni(y.child(a.startContainer(),a.startOffset()));return Ni(y.nextSibling(a.startContainer())).orThunk(function(){return b}).bind(function(c){return Ni(y.prevSibling(a.endContainer())).bind(function(d){d=L.eq(c,d);var e=a.startContainer().dom().length===a.startOffset(),g=0===a.endOffset();return d&&(e||b.isSome())&&g?h.some(c):
h.none()})})},h2=function(a){if(!L.eq(a.startContainer(),a.endContainer()))return h.none();var b=Math.min(a.startOffset(),a.endOffset()),c=Math.max(a.startOffset(),a.endOffset()),d=Ni(y.child(a.startContainer(),b));return c===b+1?d.orThunk(function(){return Ni(h.some(a.startContainer()))}):Fr&&c===b+2?d.filter(function(a){return y.nextSibling(a).filter(De.isEmptyText).isSome()}):h.none()},SD=function(a){return a.collapsed()?h.none():h2(a).orThunk(function(){return g2(a)}).map(function(a){return{image:f.constant(a)}})},
ib={detect:SD,selectionContains:function(a){a=Er.findDescendants(a.startContainer(),a.startOffset(),a.endContainer(),a.endOffset(),"img");return 1===a.length?h.some(a[0]):h.none()},remove:function(a,b){a.setRelativeSelection(da.before(b),da.before(b));B.remove(b)},select:TD,reselect:Fr?TD:f.noop},UD=C.immutable("left","right"),VD=function(a,b,c){var d=a.property().children(b);return k.findIndex(d,f.curry(a.eq,c)).map(function(a){return{before:f.constant(d.slice(0,a)),after:f.constant(d.slice(a+1))}})},
Oi={breakToLeft:function(a,b,c){return VD(a,b,c).map(function(d){var e=a.create().clone(b);a.insert().appendAll(e,d.before().concat([c]));a.insert().appendAll(b,d.after());a.insert().before(b,e);return UD(e,b)})},breakToRight:function(a,b,c){return VD(a,b,c).map(function(c){var e=a.create().clone(b);a.insert().appendAll(e,c.after());a.insert().after(b,e);return UD(b,e)})},breakPath:function(a,b,c,d){var e=C.immutable("first","second","splits"),g=function(b,f,k){var m=e(b,h.none(),k);return c(b)?e(b,
f,k):a.property().parent(b).bind(function(e){return d(a,e,b).map(function(a){var b=[{first:a.left,second:a.right}],d=c(e)?e:a.left();return g(d,h.some(a.right()),k.concat(b))}).getOr(m)})};return g(b,h.none(),[])}},j2=function(a,b,c,d){c=b(a,c);return k.foldr(d,function(c,d){var f=b(a,d);return i2(a,c,f)},c)},i2=function(a,b,c){return b.bind(function(b){return c.filter(f.curry(a.eq,b))})},k2={oneAll:function(a,b,c){if(0<c.length){var d=c[0];c=c.slice(1);a=j2(a,b,d,c)}else a=h.none();return a}},l2=
function(a,b,c,d){var e=a.property().children(b);if(a.eq(b,c[0]))return h.some([c[0]]);if(a.eq(b,d[0]))return h.some([d[0]]);var g=function(c){c=k.reverse(c);var d=k.findIndex(c,f.curry(a.eq,b)).getOr(-1);return k.findIndex(e,f.curry(a.eq,d<c.length-1?c[d+1]:c[d]))};c=g(c);var l=g(d);return c.bind(function(a){return l.map(function(b){var c=Math.min(a,b);b=Math.max(a,b);return e.slice(c,b+1)})})},WD=function(a,b,c,d){var e=void 0!==d?d:f.constant(!1);b=[b].concat(a.up().all(b));c=[c].concat(a.up().all(c));
d=function(a){return k.findIndex(a,e).fold(function(){return a},function(b){return a.slice(0,b+1)})};b=d(b);var g=d(c);c=k.find(b,function(b){return k.exists(g,f.curry(a.eq,b))});return{firstpath:f.constant(b),secondpath:f.constant(g),shared:f.constant(c)}},Gr={subset:function(a,b,c){var d=WD(a,b,c);return d.shared().bind(function(b){return l2(a,b,d.firstpath(),d.secondpath())})},ancestors:WD},Ic={sharedOne:function(a,b,c){return k2.oneAll(a,b,c)},subset:function(a,b,c){return Gr.subset(a,b,c)},ancestors:function(a,
b,c,d){return Gr.ancestors(a,b,c,d)},breakToLeft:function(a,b,c){return Oi.breakToLeft(a,b,c)},breakToRight:function(a,b,c){return Oi.breakToRight(a,b,c)},breakPath:function(a,b,c,d){return Oi.breakPath(a,b,c,d)}},Mg=Sa(),be={sharedOne:function(a,b){return Ic.sharedOne(Mg,function(b,d){return a(d)},b)},subset:function(a,b){return Ic.subset(Mg,a,b)},ancestors:function(a,b,c){return Ic.ancestors(Mg,a,b,c)},breakToLeft:function(a,b){return Ic.breakToLeft(Mg,a,b)},breakToRight:function(a,b){return Ic.breakToRight(Mg,
a,b)},breakPath:function(a,b,c){return Ic.breakPath(Mg,a,b,function(a,b,g){return c(b,g)})}},dm=function(a,b,c,d,e){return d(a).fold(function(){return!0},function(a){return L.eq(b,a)&&e(b,c)})},wf={isLeft:function(a,b){return dm(a,b.startContainer(),b.startOffset(),Cb.first,ka.isStart)},isAtLeftEdge:function(a,b,c){return dm(a,b,c,Cb.first,ka.isStart)},isRight:function(a,b){return dm(a,b.startContainer(),b.startOffset(),Cb.last,ka.isEnd)},isAtRightEdge:function(a,b,c){return dm(a,b,c,Cb.last,ka.isEnd)}},
m2=function(a){a=A.name(a);return"td"===a||"th"===a},Hr=function(a){return"li"===A.name(a)||fa.isBlock(a)},XD=function(a){return"a"===A.name(a)},YD=function(a,b,c){return sa.closest(a,b,c).filter(f.not(c)).filter(f.not(m2))},yb={stopAt:function(a){return f.curry(L.eq,a)},isList:function(a){a=A.name(a);return"ol"===a||"ul"===a},isAnchor:XD,isHeading:function(a,b){var c=R.isArray(b)?b:[],c="h1 h2 h3 h4 h5 h6".split(" ").concat(c),d=A.name(a);return k.contains(c,d)},isCollapsed:function(a){return L.eq(a.startContainer(),
a.endContainer())&&a.startOffset()===a.endOffset()},isRightEdge:function(a,b,c){var d=C.immutable("startContainer","startOffset");return wf.isRight(a,d(b,c))},isBlockOrList:Hr,exitList:function(a){return 0===pc.descendants(a,function(a){return A.isText(a)&&0<Da.get(a).length&&0<aa.trim(Da.get(a)).length||A.isElement(a)&&k.contains("hr img input video iframe picture audio embed map object".split(" "),A.name(a))}).length},findBlock:function(a,b){return sa.closest(a,Hr,b)},findValidHardBreak:function(a,
b){return YD(a,Hr,b)},findAnchorBreak:function(a,b){return YD(a,XD,b)}},Ir={forceAddBr:function(a,b){if("br"!==A.name(a)){var c=q.fromTag("br");if(A.isText(a))v.after(a,c),L.eq(b,a)||B.remove(a);else if(yb.isBlockOrList(a))v.append(a,c);else{var d=q.fromText(Ua.zeroWidth());v.prepend(a,d);v.after(a,c)}}},rootBr:function(a,b){v.before(a,q.fromTag("br"))}},ZD=function(a,b,c){var d=[];k.each(y.children(a),function(a){b(a)&&(d=d.concat([a]));d=c(a)?d:d.concat(ZD(a,b,c))});return d},$D=function(a,b){var c=
y.children(a);return 0<c.length?(c=c[0],b(c)||"textarea"===A.name(c)?a:$D(c,b)):a},Jr={descendants:ZD,plunge:$D},em=function(a){if(!A.isText(a))return!1;a=Da.get(a).trim();return 0===a.length||0===a.replace(Ua.zeroWidth(),"").length},fm=function(a){if(A.isText(a))return em(a);var b=ua.get(a);a=y.children(a);return""===aa.trim(b)||k.forall(a,em)},Ng={vacant:function(a){a=X.descendants(a,"span,a");return k.filter(a,function(a){if("span"===A.name(a)){var c=m.has(a,"class")&&0<m.get(a,"class").length,
d=m.has(a,"id");a=!c&&!d&&!m.has(a,"style")&&fm(a)}else a=fm(a);return a})},isEmpty:fm,hasCursor:function(a,b){var c=void 0!==b?b:"hr br img input textarea iframe".split(" ");return A.isText(a)?!1===em(a):k.contains(c,A.name(a))},isVacantText:em,hasNoContent:fm},aE=function(a){var b="li"===A.name(a)?yb.isList:f.constant(!1);return 0<Jr.descendants(a,Ng.hasCursor,b).length?h.none():h.some(Jr.plunge(a,b))},bE=function(a){a=Ng.vacant(a);k.each(a,B.remove)},cE=function(a,b){var c=X.descendants(b,a),c=
k.filter(c,Ng.isEmpty);k.each(c,B.remove);Ng.isEmpty(b)&&B.remove(b)},re={vacate:bE,emptyStyles:function(a){!0===m.has(a,"style")&&0===m.get(a,"style").length&&m.remove(a,"style")},needsCursor:aE,postprocess:function(a,b,c){bE(a);aE(a).each(function(a){Ir.forceAddBr(a,b)})},lists:f.curry(cE,"ul,ol"),blockquotes:f.curry(cE,"blockquote")},n2=function(a,b){var c=be.breakToRight(a,b);c.each(function(a){a=a.right();k.each(["id"],f.curry(m.remove,a));re.emptyStyles(a)});return c},gm={breakPath:function(a,
b){return be.breakPath(a,b,n2)}},o2={breakQuote:function(a,b){var c=gm.breakPath(b,yb.stopAt(a));return c.second().map(function(a){var e=c.first();v.before(a,b);k.each([e,a],re.blockquotes);return{start:f.constant(da.on(b,0)),finish:f.constant(da.on(b,0))}})}},dE=function(a){return A.isElement(a)?a:y.parent(a).getOr(a)},eE={bothMatch:function(a,b,c){a=dE(a);b=dE(b);return c(a)&&c(b)}},hm={isRootOrCell:function(a){return function(b){return a(b)||ra.is(b,"td,th")}},isRootOrContainer:function(a){return function(b){return a(b)||
fa.isContainer(b)}}},fE=function(a,b){return 0===a.length?!0:0===b.length?!1:a[0]<b[0]?!0:b[0]<a[0]?!1:fE(a.slice(1),b.slice(1))},$e={head:function(a){return 0<a.length?h.some(a[0]):h.none()},tail:function(a){return 0<a.length?h.some(a.slice(1)):h.none()},toLast:function(a){return 0<a.length?h.some(a[a.length-1]):h.none()},box:function(a){return[a]},isBefore:fE},im=ma.generate([{quasi:["elements"]},{block:["element"]},{container:["element","children"]},{fixture:["element"]}]),gE=function(a){return a.fold(function(a){return"@q["+
k.map(a,function(a){return a.dom().outerHTML||'"'+a.dom().nodeValue+'"'}).join(",")+"]"},function(a){return"@"+a.dom().nodeName.toLowerCase()+"("+a.dom().innerHTML+")"},function(a,c){return"#"+a.dom().nodeName.toLowerCase()+"("+k.map(c,gE).join(",")+")"},function(a){return a.dom().nodeName.toLowerCase()})},hE=function(a){var b=function(a){return[a]};return a.fold(f.identity,b,b,b)},jm=f.constant(!0),af=f.constant(!1),Pb={cata:function(a,b,c,d,e){return a.fold(b,c,d,e)},quasi:im.quasi,block:im.block,
container:im.container,fixture:im.fixture,isQuasi:function(a){return a.fold(jm,af,af,af)},isFixture:function(a){return a.fold(af,af,af,jm)},isQuasiOrFixture:function(a){return a.fold(jm,af,af,jm)},toTarget:hE,toString:gE,toLastTarget:function(a){return $e.toLast(hE(a))}},Pi=function(a,b){return{fold:a,label:f.constant(b)}},Qf={before:function(a){return Pi(function(b,c,d,e,g){return b(a)},"before")},after:function(a){return Pi(function(b,c,d,e,g){return c(a)},"after")},rest:function(a){return Pi(function(b,
c,d,e,g){return d(a)},"rest")},last:function(a){return Pi(function(b,c,d,e,g){return e(a)},"last")},invalid:function(a,b){return Pi(function(c,d,e,g,f){return f(a,b)},"invalid")}},iE={subdivide:function(a,b,c){var d=a.property().getText(b);c=k.filter(Cl.splits(d,c),function(a){return 0<a.length});if(1>=c.length)return[S.range(b,0,d.length)];a.property().setText(b,c[0]);var d=ad.generate(c.slice(1),function(b,c){var d=a.create().text(b),d=S.range(d,c,c+b.length);return h.some(d)},c[0].length),e=k.map(d,
function(a){return a.element()});a.insert().afterAll(b,e);return[S.range(b,0,c[0].length)].concat(d)}},km=function(a,b){return{fold:a,label:f.constant(b)}},lm={none:function(){return km(function(a,b,c,d){return a()},"none")},start:function(a){return km(function(b,c,d,e){return c(a)},"start")},middle:function(a,b){return km(function(c,d,e,g){return e(a,b)},"middle")},end:function(a){return km(function(b,c,d,e){return e(a)},"end")}},p2={determine:function(a){return a.before().fold(function(){return a.after().fold(function(){return lm.none()},
function(a){return lm.start(a)})},function(b){return a.after().fold(function(){return lm.end(b)},function(a){return lm.middle(b,a)})})}},q2={group:function(a,b,c){b=k.bind(b,function(b){return Ue.from(a,b,c)});b=tz.splitbyAdv(b,function(a){return hl.cata(a,function(){return Bg.excludeWithout(a)},function(){return Bg.excludeWith(a)},function(){return Bg.include(a)})});return k.filter(b,function(a){return 0<a.length})}},r2={common:function(a,b,c){b=[b].concat(a.up().all(b));var d=[c].concat(a.up().all(c));
return k.find(b,function(b){return k.exists(d,function(c){return a.eq(c,b)})})}},DP="table tbody thead tfoot tr ul ol".split(" "),s2={range:function(a,b,c,d,e){return a.eq(b,d)?[b]:r2.common(a,b,d).fold(function(){return[]},function(g){var l=[g].concat(Ue.all(a,g,f.constant(!1)));g=k.findIndex(l,f.curry(a.eq,b));var h=k.findIndex(l,f.curry(a.eq,d));g=g.bind(function(a){return h.map(function(b){return a<b?l.slice(a+c,b+e):l.slice(b+e,a+c)})}).getOr([]);var r=CP(a);return k.filter(g,r.validateText)})}},
jE={range:function(a,b,c,d,e){return s2.range(a,b,c,d,e)},group:function(a,b,c){return q2.group(a,b,c)}},Kr=C.immutable("before","after"),kE=function(a,b,c){a=a.property().getText(b);return Cl.splits(a,c)},lE=function(a,b,c,d){if(!a.property().isText(b)||c===d)return b;if(c>d)return lE(a,b,d,c);var e=a.property().getText(b).length;if(0===c&&d===e)return b;d=kE(a,b,[c,d]);a.property().setText(b,d[0]);d=k.map(d.slice(1),function(b){return a.create().text(b)});e=d[0];a.insert().afterAll(b,d);return 0===
c?b:e},Qi={split:function(a,b,c){if(!a.property().isText(b)||0>=c)return Kr(h.none(),h.some(b));if(c>=a.property().getText(b).length)return Kr(h.some(b),h.none());c=kE(a,b,[c]);a.property().setText(b,c[0]);c=a.create().text(c[1]);a.insert().after(b,c);return Kr(h.some(b),h.some(c))},splitByPair:lE},t2=function(a,b,c,d,e){c=Qi.split(a,b,c).after().fold(function(){return S.delta(b,1)},function(a){return S.delta(a,0)});e=Qi.split(a,d,e).before().fold(function(){return S.delta(d,0)},function(a){return S.delta(a,
1)});return jE.range(a,c.element(),c.deltaOffset(),e.element(),e.deltaOffset())},u2=function(a,b,c,d,e){return[Qi.splitByPair(a,b,c,e)]},v2={nodes:function(a,b,c,d,e){return(a.eq(b,d)?u2:t2)(a,b,c,d,e)}},cd={split:function(a,b,c){return Qi.split(a,b,c)},splitByPair:function(a,b,c,d){return Qi.splitByPair(a,b,c,d)},range:function(a,b,c,d,e){return v2.nodes(a,b,c,d,e)},subdivide:function(a,b,c){return iE.subdivide(a,b,c)},position:function(a,b){return p2.determine(b)}},w2=function(a,b,c){var d=cd.split(a,
b,c);return cd.position(a,d).fold(function(){return Qf.invalid(b,c)},Qf.before,Qf.after,Qf.after)},x2=function(a,b,c){a=a.property().children(b);return c===a.length?Qf.last(b):c<a.length?Qf.rest(a[c]):Qf.invalid(b,c)},y2={atStartOf:function(a,b,c,d){var e=function(b){a.insert().before(b,d)};(a.property().isText(b)?w2:x2)(a,b,c).fold(e,function(b){a.insert().after(b,d)},e,function(b){a.insert().append(b,d)},f.noop)}},Lr={atStartOf:function(a,b,c,d){y2.atStartOf(a,b,c,d)}},z2=ma.generate([{none:[]},
{root:["target","block"]},{created:["target","block"]},{actual:["target","block"]}]),bf=Z.merge(z2,{toString:function(a){return a.fold(function(){return["block.none"]},function(a,c){return["block.root",a,c]},function(a,c){return["block.created",a,c]},function(a,c){return["block.actual",a,c]})},cata:function(a,b,c,d,e){return a.fold(b,c,d,e)},extract:function(a){var b=f.constant([]),c=function(a,b){return[b]};return a.fold(b,b,c,c)},extractEvenRoot:function(a){var b=f.constant([]),c=function(a,b){return[b]};
return a.fold(b,c,c,c)}}),mE=function(a,b){return function(c){return a.eq(c,b)||a.property().parent(c).map(function(c){return a.eq(c,b)}).getOr(!1)}},A2=function(a,b,c){return a.eq(b,c)?h.none():a.property().isBoundary(c)?a.query().nextSibling(c):a.property().isEmptyTag(c)?(b=mE(a,b),b(c)?a.query().nextSibling(c):Ic.breakPath(a,c,b,Ic.breakToRight).second()):h.none()},B2=function(a,b,c){if(a.eq(b,c))return h.none();if(a.property().isBoundary(c))return a.query().prevSibling(c);if(a.property().isEmptyTag(c)){var d=
mE(a,b);return d(c)?a.query().prevSibling(c):a.query().prevSibling(c).map(function(b){return Ic.breakPath(a,b,d,Ic.breakToRight).first()})}return h.none()},nE=function(a,b,c,d){return c(a,b,d).bind(function(c){var d=a.property().children(b);return k.findIndex(d,function(b){return a.eq(b,c)})})},Mr=function(a,b){return a.property().isEmptyTag(b)||a.property().isBoundary(b)},Nr={create:function(a,b,c){if(Mr(a,c)||Mr(a,mc.toLeaf(a,c,0).element()))return[c];var d=f.curry(Mr,a),e=f.curry(a.eq,b),g=na.seekLeft(a,
c,d,e).bind(function(c){return nE(a,b,A2,c)}).getOr(0),l=a.property().children(b);c=na.seekRight(a,c,d,e).bind(function(c){return nE(a,b,B2,c)}).getOr(l.length-1);return a.property().children(b).slice(g,c+1)}},C2=function(a,b,c,d){return c(b)||d(b)?h.some(b):a.up().predicate(b,c,d)},oE=function(a,b,c){c=c();a.insert().wrap(b,c);return bf.created(b,c)},D2=function(a,b,c,d){var e=d();a.insert().before(c[0],e);k.each(c,function(b){a.insert().append(e,b)});return bf.created(b,e)},E2=function(a,b,c,d,
e,g,f){if(g(c)){var h=a.property().children(b);if(d<h.length&&f(h[d]))return bf.actual(c,h[d])}return e(b).fold(function(){return bf.root(c,b)},function(e){if(g(c)){var f=e(),l=a.property().children(b),h=l[d];void 0!==h&&(e=Nr.create(a,b,h,e),a.insert().appendAll(f,0<e.length?e:[h]));Lr.atStartOf(a,b,Math.min(l.count,d),f);return bf.created(b,f)}a.property().isEmptyTag(c)?e=oE(a,c,e):(f=Nr.create(a,b,c),e=0<f.length?D2(a,c,f,e):oE(a,c,e));return e})},F2={single:function(a,b,c,d,e,g){return C2(a,c,
b,g).fold(bf.none,function(f){return g(f)?E2(a,f,c,d,e,g,b):bf.actual(c,f)})}},pE={resolve:lb.create("ephox-soldier").resolve},Og=pE.resolve("quasi-block"),qE=function(a){a=y.children(a);return 1===a.length&&"br"===A.name(a[0])},rE=function(a){return n.has(a,Og)&&!qE(a)},mm={condense:function(a){a=k.filter(a,function(a,c){if(!A.isElement(a))return!0;var d;n.has(a,Og)&&qE(a)&&(y.prevSibling(a).exists(rE)||y.nextSibling(a).exists(rE))?(B.remove(a),d=!1):d=!0;return d});return k.map(a,function(a){var c=
A.isElement(a)?[a].concat(X.descendants(a,"."+Og)):[];k.each(c,function(a){n.remove(a,Og)});return a})},register:function(a){return function(b){return a(b).map(function(a){return function(){var b=a();n.add(b,Og);return b}})}},style:f.constant(Og)},G2=Sa(),Rh={single:function(a,b,c,d,e){return F2.single(G2,a,b,c,mm.register(d),e)},isBlock:function(a){return k.contains("ol p div ul table caption td th h1 h2 h3 h4 h5 h6 body blockquote pre".split(" "),A.name(a))}},Ri={tag:function(){return"p"},nu:function(a){return f.curry(a.create().nu,
"p")}},Or=function(a){return 0<a.length?h.some(a[0]):h.none()},Si=ma.generate([{descend:["index"]},{leftOf:["rindex"]},{bounded:["lindex","rindex"]},{rightOf:["lindex"]},{all:[]}]),sE={derive:function(a,b){return a.bind(Or).fold(function(){return b.bind(Or).fold(function(){return Si.all()},function(a){return Si.leftOf(a)})},function(a){return b.bind(Or).fold(function(){return Si.rightOf(a)},function(b){return a===b?Si.descend(a):Si.bounded(a,b)})})},cata:function(a,b,c,d,e,g){return a.fold(b,c,d,
e,g)}},tE=function(a,b,c){return h.from(a.property().children(b)[c])},H2=function(a,b,c){var d=a.property().children(c),e=tE(a,c,0).bind(function(e){return tE(a,c,d.length-1).map(function(d){return Pr(a,b,c,e,d)})}).getOr([]);return Pb.container(c,e)},Pr=function(a,b,c,d,e){var g=uE(a,c,b,d);return Pb.toLastTarget(g).filter(function(b){return!a.eq(e,b)}).bind(function(d){return na.walk(a,d,na.sidestep,na.walkers().right()).bind(function(d){if(a.eq(c,d.item()))return h.none();d=Pr(a,b,c,d.item(),e);
return h.some([g].concat(d))})}).getOrThunk(function(){return[g]})},uE=function(a,b,c,d){if(Yb.isFormatting(a,d))return Pb.block(d);if(Yb.isEmptyTag(a,d))return Pb.fixture(d);if(Yb.isBlock(a,d)||Yb.isContainer(a,d))return H2(a,c,d);a=Nr.create(a,b,d);return Pb.quasi(0<a.length?a:[d])},vE=function(a,b,c){return uE(a,c,b,c)},Qr=function(a,b,c,d,e,g,f,k){c=c.map(function(c){return Ti(a,b,c,f.bind($e.tail),h.none())}).getOr([]);e=e.map(function(c){return Ti(a,b,c,h.none(),k.bind($e.tail))}).getOr([]);
d=0===d.length?[]:Pr(a,b,g,d[0],d[d.length-1]);return[Pb.container(g,c.concat(d).concat(e))]},Ti=function(a,b,c,d,e){var g=a.property().children(c);d.bind($e.head).getOr(0);e.bind($e.head).getOr(g.length-1);var f=sE.derive(d,e);return sE.cata(f,function(c){return Ti(a,b,g[c],d.bind($e.tail),e.bind($e.tail))},function(f){return Qr(a,b,h.none(),g.slice(0,f),h.from(g[f]),c,d,e)},function(f,l){return Qr(a,b,h.from(g[f]),g.slice(f+1,l),h.from(g[l]),c,d,e)},function(f){return Qr(a,b,h.from(g[f]),g.slice(f+
1),h.none(),c,d,e)},function(){return[vE(a,b,c)]})},wE={categorise:vE,within:function(a,b,c,d,e){return!0===$e.isBefore(d,e)?Ti(a,b,c,h.some(d),h.some(e)):Ti(a,b,c,h.some(e),h.some(d))}},Rr=function(a,b){if("li"!==A.name(a))return!1;var c=k.foldl(b,function(a,b){return 1<a.quasi||0<a.nonlist?a:Pb.cata(b,function(){return{quasi:1+a.quasi,nonlist:a.nonlist}},function(){return{quasi:a.quasi,nonlist:1+a.nonlist}},function(b){return{quasi:a.quasi,nonlist:(k.contains(["ul","ol"],A.name(b))?0:1)+a.nonlist}},
function(){return{quasi:1+a.quasi,nonlist:a.nonlist}})},{quasi:0,nonlist:0});return 1>=c.quasi&&0===c.nonlist},Sr=function(a,b,c){return 0>=c.length?!0:k.contains(a,A.name(b))?2<c.length?!1:1===c.length?Pb.isQuasiOrFixture(c[0]):Pb.isQuasiOrFixture(c[0])&&Pb.isQuasiOrFixture(c[1])&&!k.forall(c,Pb.isFixture):!1},dd={wrap:function(a){if(0===a.length||1===a.length&&A.isText(a[0])&&0===Da.get(a[0]).trim().length)return[];var b=q.fromTag(Ri.tag());n.add(b,mm.style());v.before(a[0],b);K.append(b,a);return[b]},
handleContainer:function(a,b,c,d){if(Sr(a,b,c))return[b];var e=Rr(b,c)?[b]:[];return e.concat(k.bind(c,function(a){return d(a,0<e.length)}))},hasSimpleContent:Sr,isNestedList:Rr,handleRoot:function(a,b){var c=wE.categorise(Sa(),f.curry(L.eq,b),b,b,b),d=function(){return h.some(Ri.nu(Sa()))},e=h.none;return Pb.cata(c,d,e,function(b,c){return Sr(a,b,c)||Rr(b,c)?e():d()},d)}},nm=["li","td","th","caption"],xE=function(a,b){return Pb.cata(a,function(a){return!0===b?[]:dd.wrap(a)},function(a){return[a]},
function(a,b){return dd.handleContainer(nm,a,b,xE)},function(a){return!0===b?[]:dd.wrap([a])})},I2={isRootContainer:hm.isRootOrContainer,extract:function(a){return xE(a,!1)},isCapable:function(a){return Rh.isBlock(a)||k.contains(nm,A.name(a))},canUse:function(a){return fa.isFormatting(a)||k.contains(nm,A.name(a))},quasi:f.curry(dd.handleRoot,nm),ascend:f.identity},yE=C.immutableBag(["start","soffset","finish","foffset","all"],[]),zE=C.immutableBag(["start","soffset","finish","foffset"],[]),Pg={block:yE,
inline:zE,blockFrom:function(a){return 0<a.length?h.some(yE({start:a[0],soffset:0,finish:a[a.length-1],foffset:ka.getEnd(a[a.length-1]),all:a})):h.none()},inlineFrom:function(a){return a.map(function(a){return zE({start:a.start(),soffset:a.soffset(),finish:a.finish(),foffset:a.foffset()})})}},om=function(a){return ra.is(a,"td,th")},AE=function(a){return L.eq(a.startContainer(),a.endContainer())&&a.startOffset()===a.endOffset()},BE=function(a,b){return a(b.startContainer())?h.some(Ja.freefallLtr(b.startContainer())):
h.none()},CE=function(a,b){return(a(b.endContainer())?h.some(Ja.freefallRtl(b.endContainer())):h.none()).map(function(a){return ra.is(a.element(),"br")&&!1===AE(b)?S.point(a.element(),1):a})},ed={is:om,isCollapsed:AE,isWholeCell:function(a){return om(a.startContainer())&&L.eq(a.startContainer(),a.endContainer())&&0===a.startOffset()&&a.endOffset()===ka.getEnd(a.endContainer())},left:f.curry(BE,om),leftWith:BE,right:f.curry(CE,om),rightWith:CE},DE=C.immutable("startContainer","startOffset","endContainer",
"endOffset"),Tr=C.immutable("start","soffset","finish","foffset"),sb={hare:DE,fussy:Tr,fromHare:function(a){return Tr(a.startContainer(),a.startOffset(),a.endContainer(),a.endOffset())},fromFussy:function(a){return DE(a.start(),a.soffset(),a.finish(),a.foffset())},fussyFromElements:function(a){if(0===a.length)return h.none();var b=a[0];a=a[a.length-1];b=y.firstChild(b).getOr(b);a=y.lastChild(a).getOr(a);return h.some(Tr(b,0,a,ka.getEnd(a)))}},EE=function(a,b){return b(a)?h.some([]):y.parent(a).bind(function(c){return y.findIndex(a).bind(function(a){return EE(c,
b).map(function(b){return b.concat([a])})})})},FE=function(a,b){return 0===b.length?h.some(a):y.child(a,b[0]).bind(function(a){return FE(a,b.slice(1))})},Qg={path:function(a,b){var c=f.curry(L.eq,a);return L.eq(a,b)?h.some([]):EE(b,c)},follow:FE},GE=C.immutable("ancestor","startPath","finishPath"),J2=function(a,b,c){return be.ancestors(b,c,a).shared().bind(function(a){return Qg.path(a,b).bind(function(b){return Qg.path(a,c).map(function(c){return GE(a,b,c)})})})},HE=function(a,b,c,d){var e=b.isRootContainer(a);
c=Rh.single(b.isCapable,c,d,b.quasi,e);c=bf.extractEvenRoot(c);return 1===c.length?h.some(c[0]).map(function(c){return b.ascend(c,a)}):h.none()},IE=function(a,b,c,d){if(!1===b(c))return S.point(c,d);d<ka.getEnd(c)||a.isCapable(c)&&a.canUse(c)?c=Ja.toLeaf(c,d):(a=q.fromTag("br"),v.append(c,a),c=S.point(a,0));return c},K2={branch:function(a,b,c,d,e,g){var l=b.isRootContainer(a),k=sb.hare(c,d,e,g),r=f.constant(!0===ed.isWholeCell(k)),m=ed.left(k).filter(r).getOrThunk(function(){return IE(b,l,c,d)}),
w=ed.right(k).filter(r).getOrThunk(function(){return IE(b,l,e,g)});return HE(a,b,m.element(),m.offset()).bind(function(c){return(L.eq(m.element(),w.element())&&m.offset()===w.offset()?h.some(c):HE(a,b,w.element(),w.offset())).bind(function(b){return L.eq(c,b)?h.some(GE(c,[],[])):J2(a,c,b)})})}},JE=function(a,b,c,d,e,g,f){a=b.isRootContainer(a)(c);b=!a&&ca.inBody(c)?S.point(c,d):a||!b.isCapable(c)||ca.inBody(c)?ca.inBody(e.element())&&k.contains(["td","th"],A.name(c))?e:!ca.inBody(c)&&ca.inBody(g.element())&&
"br"==A.name(g.element())?g:Ja.toLeaf(f,0):S.point(f,d);ka.getEnd(b.element());return b.offset()<=ka.getEnd(b.element())?h.some(b):h.none()},Rg={applyOn:function(a,b,c,d,e,g,f){var p=Bc.after(c,d,e,g),r=b.isRootContainer(a)(c)?Ja.toLeaf(c,d):S.point(c,d),m=b.isRootContainer(a)(e)?Ja.toLeaf(e,g):S.point(e,g),w=Ja.toLeaf(c,d),n=Ja.toLeaf(e,g),s=K2.branch(a,b,c,d,e,g).map(function(b){return wE.within(Sa(),a,b.ancestor(),b.startPath(),b.finishPath())}).getOr([]),s=k.bind(s,b.extract);k.each(s,function(a){0===
y.children(a).length&&fa.isFormatting(a)&&v.append(a,q.fromTag("br"))});var s=mm.condense(s),t=f(s);return 0===t.length?h.none():JE(a,b,c,d,r,w,t[p?t.length-1:0]).bind(function(c){return JE(a,b,e,g,m,n,t[p?0:t.length-1]).map(function(a){return Pg.block({start:c.element(),soffset:c.offset(),finish:a.element(),foffset:a.offset(),all:t})})})}},Ur=function(a){return function(b){return a}},KE={start:rd.onDirection("left","right"),end:rd.onDirection("right","left"),justify:Ur("justify"),center:Ur("center"),
"match-parent":Ur("match-parent")},L2={hasAlignment:function(a,b,c){A.isText(a)?a=!1:(b=t.get(a,b),a=(void 0!==KE[b]?KE[b](a):b)===c);return a}},M2=function(a,b,c){return L2.hasAlignment(c,a,b)},pm=function(a,b,c){var d=EP(b,c);return{run:function(a,b,c,f,h){return Rg.applyOn(a,I2,b,c,f,h,function(a){var b=k.forall(a,d.is)?d.remove:d.set;k.each(a,b);return a})},query:function(a,d){var l=f.curry(M2,b,c);return eE.bothMatch(a,d,l)},name:f.constant(a)}},N2=pm("JustifyLeft","text-align","left"),O2=pm("JustifyCenter",
"text-align","center"),P2=pm("JustifyRight","text-align","right"),Q2=pm("JustifyFull","text-align","justify"),pd={justifyleft:f.constant(N2),justifycentre:f.constant(O2),justifyright:f.constant(P2),justifyfull:f.constant(Q2)},LE=function(a){a=a.dom();if(Pd.isSupported(a)){var b=a.style.marginLeft;return"auto"===a.style.marginRight&&"auto"===b}return!1},Vr={isCentered:LE,divine:function(a){if(LE(a))return h.some("center");var b=t.getRaw(a,"float").getOrThunk(function(){return t.get(a,"float")});return void 0!==
b&&null!==b&&0<b.length?h.some(b):h.none()},getRaw:function(a){return t.getRaw(a,"float").getOr(null)},setCentered:function(a){t.setAll(a,{"margin-left":"auto","margin-right":"auto"})}},R2={left:{"float":"left",margin:""},right:{"float":"right",margin:""},none:{"float":"none",margin:""},center:{"float":"",margin:"0 auto"}},ME={none:{display:""},left:{display:""},right:{display:""},center:{display:"block"}},NE=function(a,b){return Vr.divine(a).is(b)},OE=function(a,b){var c="img"===A.name(a)&&void 0!==
ME[b]?ME[b]:{};h.from(R2[b]).each(function(b){t.setAll(a,b);t.setAll(a,c)})},qm=function(a,b){return{run:function(a){OE(a,b);return h.none()},query:function(a){return NE(a,b)},name:f.constant(a)}},S2=qm("JustifyLeft","left"),T2=qm("JustifyCenter","center"),U2=qm("JustifyRight","right"),V2=qm("FloatNone","none"),Rf={floatnone:f.constant(V2),floatleft:f.constant(S2),floatcentre:f.constant(T2),floatright:f.constant(U2),queryIsFloat:NE,runFloat:OE},PE=function(a){return Pb.cata(a,function(a){return dd.wrap(a)},
function(a){return[a]},function(a,c){return k.bind(c,PE)},function(a){return dd.wrap([a])})},QE={isRootContainer:hm.isRootOrContainer,extract:PE,isCapable:function(a){return Rh.isBlock(a)||"li"===A.name(a)},canUse:function(a){return fa.isFormatting(a)},quasi:function(a){return h.some(Ri.nu(Sa()))},ascend:f.identity},Wr=C.immutable("level","content","cruft","list","item"),rm={level:function(a,b){return Wr(a.level()+b,a.content(),a.cruft(),a.list(),a.item())},list:function(a,b){return Wr(a.level(),
a.content(),a.cruft(),b,a.item())}},RE=C.immutable("elements","elementsTransformed"),W2=C.immutable("original","replaced"),Sf=C.immutable("result","items"),sm={answer:RE,write:function(a,b){var c=[],d=function(b){var c=a.create().nu("p");a.insert().appendAll(c,b);return c},e=function(a){if(0===a.length)throw"Cannot get tail of an empty array";return a.slice(1)},g=function(b,c){for(var d=a.create().clone(c[0].list()),h=[],p=c[0];0<c.length&&c[0].level()>=b&&a.property().name(p.list())===a.property().name(c[0].list());){var m;
if(c[0].level()===b){m=b;var n=c,q=n[0],s;s=q;var t=s.item(),u=t,u=a.property().children(u);k.each(u,a.remove().remove);a.insert().appendAll(t,s.content());s=t;q=q.cruft();for(n=e(n);0<n.length&&n[0].level()>m;)n=g(m+1,n),t=[n.result()].filter(f.curry(l,["li"])),n=n.items(),a.insert().appendAll(s,t);a.insert().appendAll(s,q);m=Sf(s,n);n=[m.result()].filter(f.curry(l,["li"]));m=Sf(n,m.items())}else m=g(b+1,c),n=[m.result()].filter(f.curry(l,["ul","ol"])),q=a.create().nu("li"),a.styles().set(q,"list-style-type",
"none"),a.attrs().remove(d,"id"),a.insert().appendAll(q,n),m=Sf([q],m.items());c=m.items();h=h.concat(m.result())}a.insert().appendAll(d,h);return Sf(d,c)},l=function(b,c){return!(k.contains(b,a.property().name(c))&&0===a.property().children(c).length)},h=function(b){for(var h=[];0<b.length;){var k=b[0];if(0===k.level()){var p=b[0],m=p.content(),m=1===m.length&&Yb.isBlock(a,m[0])?m[0]:d(m);c.push(W2(p.item(),m));b=Sf(m,e(b));p=[b.result()].filter(f.curry(l,["p"]));0===p.length&&c.pop();b=b.items();
h=h.concat(p);m=b;for(b=[];0<m.length&&0<m[0].level();)m=g(1,m),p=[m.result()].filter(f.curry(l,["ul","ol"])),m=m.items(),b=b.concat(p);p=Sf(b,m);b=p.items();h=h.concat(p.result());0<k.cruft().length&&(h=h.concat([d(k.cruft())]))}else b=g(1,b),k=[b.result()].filter(f.curry(l,["ul","ol"])),b=b.items(),h=h.concat(k)}return Sf(h,b)}(b);if(0!==h.items().length)throw"Error - there should not be any remaining items to parse";return RE(h.result(),c)}},SE=function(a){return k.foldl(a,function(a,c){var d=
a.elements().concat(c.elements()),e=a.elementsTransformed().concat(c.elementsTransformed());return sm.answer(d,e)},sm.answer([],[]))},X2={combine:SE,write:function(a,b){var c=k.groupBy(b,function(b){return a.property().name(b.list())+"-"+a.styles().getRaw(b.list(),"list-style-type").getOr("default")}),c=k.map(c,function(b){return sm.write(a,b)});return SE(c)}},TE=Sa(),UE=function(a){return X2.write(TE,a)},Y2=function(a){return sm.write(TE,a)},VE=function(a,b,c){return J.closest(a,"li",c).bind(function(a){return b(a,
c).map(function(b){return{item:f.constant(a),outerlist:f.constant(b)}})})},WE=function(a,b){return k.findIndex(a,function(a){return L.eq(a,b)})},Z2=function(a,b,c){return WE(a,b).bind(function(b){return WE(a,c).map(function(c){var g=Math.min(b,c);c=Math.max(b,c);return a.slice(g,c+1)})})},Sg={context:function(a,b,c,d,e){return VE(a,c,e).bind(function(g){return VE(b,c,e).bind(function(c){if(L.eq(g.outerlist(),c.outerlist())){var e;if(L.eq(a,b))e=[g.item()];else{var k=g.outerlist();e=g.item();var m=
c.item(),k=X.descendants(k,"li");e=Z2(k,e,m).getOr([])}c=h.some({outerlist:g.outerlist,startItem:g.item,finishItem:c.item,writer:f.constant(d),selected:f.constant(e)})}else c=h.none();return c})})},immediate:function(a,b){return J.closest(a,"ul,ol",b)},topmost:function(a,b){var c=pc.ancestors(a,function(a){return fa.isList(a)||fa.isContainer(a)},b);return k.findIndex(c,function(a){return!k.contains(["li","ol","ul"],A.name(a))}).map(function(a){return 0<a&&a-1<c.length?h.some(c[a-1]):0<c.length?h.some(c[c.length-
1]):h.none()}).getOrThunk(function(){return 0<c.length?h.some(c[c.length-1]):h.none()})}},Xr={context:function(a,b,c){return Sg.context(a,b,Sg.topmost,Y2,c)},indent:function(a){return rm.level(a,1)},outdent:function(a){return rm.level(a,-1)}},$2=function(a,b){return k.exists(a,function(a){return L.eq(b,a)})},a3={operate:function(a,b,c){return k.map(b,function(b){return $2(c,b.item())?a(b):b})}},XE=C.immutable("before","on","after"),b3=function(a,b){var c=h.from(k.map(b,function(b){return a[b]})),
d=a.slice(0,b[0]),e=a.slice(b[b.length-1]+1);return XE(d,c,e)},c3={refine:function(a,b){var c=k.map(a,function(a,c){return b(a)?h.some(c):h.none()}),c=ab.cat(c),d=XE(a,h.none(),[]);return 0===c.length?d:b3(a,c)}},Yr=function(a,b){var c=X.children(a,"li,ol,ul");return k.bind(c,function(c){return"li"===A.name(c)?d3(a,c,b):Yr(c,b+1)})},d3=function(a,b,c){var d=e3(b),e=[Wr(c,d.before(),d.after(),a,b)];return d.on().fold(function(){return e},function(a){a=k.bind(a,function(a){return Yr(a,c+1)});return e.concat(a)})},
f3=function(a){return ra.is(a,"ol,ul")},e3=function(a){a=y.children(a);return c3.refine(a,f3)},g3={read:function(a){return Yr(a,1)}},h3=function(a){a=a.elementsTransformed();k.each(a,function(a){t.transfer(a.original(),a.replaced(),["text-align"]);m.transfer(a.original(),a.replaced(),["dir"])})},YE=function(a,b){return k.find(b,function(b){return L.eq(b.original(),a)}).map(function(a){return a.replaced()}).getOr(a)},ZE=function(a,b,c,d,e,g){if(0===a.selected().length)return h.none();var f=a.writer(),
k=a.outerlist();a=a.selected();var r=g3.read(k);g=g(a);g=a3.operate(g,r,a);f=f(g);h3(f);K.before(k,f.elements());B.remove(k);k=f.elementsTransformed();b=YE(b,k);d=YE(d,k);return h.some(Pg.block({start:b,soffset:c,finish:d,foffset:e,all:f.elements()}))},$E={execute:function(a,b,c,d,e,g){return ZE(a,b,c,d,e,f.constant(g))},executeUsing:ZE};Sa();var i3=function(a,b,c,d,e,g,f){return Rg.applyOn(a,QE,b,c,d,e,function(a){k.each(a,function(a){f.change(a,g)});return a})},aF=function(a,b){return function(c,
d,e,g,f,h){var k=hm.isRootOrCell(c);return Xr.context(d,g,c).fold(function(){return i3(k,d,e,g,f,a,h)},function(a){return $E.execute(a,d,e,g,f,b)})}},xo={indent:function(a,b,c,d,e,g){return aF(1,Xr.indent)(a,b,c,d,e,g)},outdent:function(a,b,c,d,e,g){return aF(-1,Xr.outdent)(a,b,c,d,e,g)}},j3={isRootContainer:function(a){return function(b){return a(b)||k.contains(["td","th","caption"],A.name(b))}},extract:function(a){return Pb.cata(a,function(a){return[a]},function(a){return[a]},function(a,c){return k.contains(["ol",
"ul"],A.name(a))?[a]:k.bind(c,Pb.toTarget)},function(a){return[a]})},isCapable:function(a){return fa.isBlock(a)},canUse:f.constant(!1),quasi:function(a){return h.some(Ri.nu(Sa()))},ascend:f.identity},Zr=function(a){return ed.is(a)?J.descendant(a,"blockquote"):J.closest(a,"blockquote")},bF=function(a){return be.sharedOne(Zr,a)},k3=function(a,b,c,d,e){return Rg.applyOn(a,j3,b,c,d,e,function(a){var b=q.fromTag("blockquote");0<a.length&&(v.before(a[0],b),K.append(b,a),b=X.descendants(b,"blockquote"),
k.map(b,B.unwrap));return a})},$r={toggle:function(a,b,c,d,e){return(L.eq(b,d)?Zr(b):bF([b,d])).fold(function(){return k3(a,b,c,d,e)},function(a){B.unwrap(a);return h.some(Pg.block({start:b,soffset:c,finish:d,foffset:e,all:[]}))})},queryPoint:Zr,queryAll:bF},cF=function(a,b,c,d){var e=a.content(),g=a.undo();tb.cata(a.selections().get(),f.noop,am.multiple(e,a.undo(),d),am.single(e,g,d,b,c))},dF=function(a){return function(b){var c=0<a?xo.indent:xo.outdent,d=so(b.region()),e=b.content();cF(b,Ki,Ji,
function(a,b,f,h){return c(e.isRoot,a,b,f,h,d)})}},tm=function(a){return function(b){var c=b.content(),d=function(b,d,f,h){return a.run(c.isRoot,b,d,f,h)};tb.cata(b.selections().get(),f.noop,bm.multiple(c,b.undo(),d),bm.single(c,b.undo(),d,Ki,Ji))}},um=function(a){return function(b){b.content().getSelection().bind(ib.detect).each(function(c){Yc.runTransaction(b.content(),b.undo(),function(){(a.query(c.image())?Rf.floatnone():a).run(c.image())},b.content().toEditing)})}},kd={indent:dF(1),outdent:dF(-1),
blockquote:function(a){var b=a.content();cF(a,Ki,Ji,function(a,d,e,g){return $r.toggle(b.isRoot,a,d,e,g)})},justifyLeft:tm(pd.justifyleft()),justifyCentre:tm(pd.justifycentre()),justifyRight:tm(pd.justifyright()),justifyFull:tm(pd.justifyfull()),imageNone:um(Rf.floatnone()),imageLeft:um(Rf.floatleft()),imageCentre:um(Rf.floatcentre()),imageRight:um(Rf.floatright()),breakQuote:function(a,b,c){o2.breakQuote(b,c).each(function(b){a.content().setRelativeSelection(b.start(),b.finish());a.content().uiChanged()})}},
l3=function(a,b){return k.find(a,function(a){return a.value()===b})},Yj={updateMenu:function(a,b,c,d,e){b.fold(a.clear,f.noop,function(b){(R.isString(b)?l3(d,b):h.some(b)).each(a.select);e.each(function(c){c=c(b);a.selectElement(c)})});c.fold(a.disable,f.noop,a.enable);b.fold(a.untick,f.noop,function(b){b=R.isString(b)?b:b.value();a.tick(b)})}},as=ma.generate([{table:["selection"]},{list:["selection"]}]),m3=function(a,b){return a.isRoot(b)||k.contains(["table","ul","ol"],A.name(b))},Xj={context:function(a){return a.getSelection().bind(function(b){if(b.collapsed())return J.closest(b.startContainer(),
"td,th,li",f.curry(m3,a)).map(function(a){return"li"===A.name(a)?as.list(b):as.table(b)});var c=Bc.commonAncestorContainer(b.startContainer(),b.startOffset(),b.endContainer(),b.endOffset());return J.closest(c,"ol,ul",a.isRoot).map(function(a){return as.list(b)})})},cata:function(a,b,c){return a.fold(b,c)}},Tg={attr:function(a,b,c){var d=f.curry(m.set,a,b,c);a=f.curry(m.remove,a,b);0<c.length?d():a()},css:function(a,b,c){var d=f.curry(t.set,a,b,c);a=f.curry(t.remove,a,b);0<c.length?d():a()}},bs={resolve:lb.create("ephox-salmon").resolve},
n3=bs.resolve("upload-image-in-progress"),o3="data-"+bs.resolve("image-blob"),p3="data-"+bs.resolve("image-upload"),Kb={uploadInProgress:f.constant(n3),blobId:f.constant(o3),trackedImage:f.constant(p3)},eF=C.immutableBag("url alt width height isLocal align".split(" "),["computedWidth","computedHeight"]),fF=function(a,b){var c=m.get(a,b);return void 0!==c?c:""},Ck={divine:function(a){var b=t.getRaw(a,"width").getOr(""),c=t.getRaw(a,"height").getOr(""),d=t.getRaw(a,"width").getOrThunk(function(){return qa.get(a)+
"px"}),e=t.getRaw(a,"height").getOrThunk(function(){return ia.get(a)+"px"}),g=Vr.divine(a);return eF({url:fF(a,"src"),alt:fF(a,"alt"),width:b,height:c,computedWidth:d,computedHeight:e,isLocal:m.has(a,Kb.blobId()),align:g})},properties:eF,apply:function(a,b){m.get(a,"src")===b.url()||b.isLocal()||m.set(a,"src",b.url());Tg.attr(a,"alt",b.alt());Tg.css(a,"width",b.width());Tg.css(a,"height",b.height());b.align().each(function(b){Rf.runFloat(a,b)})}},q3={hasNoContent:function(a){var b=y.children(a);return A.isElement(a)&&
(0===b.length||1===b.length&&"br"===A.name(b[0]))}},r3=C.immutable("alignment","quoted"),gF=function(a){return a.getSelection().fold(function(){return!1},function(a){var c=a.startContainer();a=a.endContainer();return L.eq(c,a)?$r.queryPoint(c).isSome():$r.queryAll([c,a]).isSome()})},s3=[pd.justifyleft(),pd.justifycentre(),pd.justifyright(),pd.justifyfull()],hF=function(a){return a.getSelection().bind(ib.detect).fold(function(){return a.getSelection().bind(function(a){return k.find(s3,function(c){return c.query(a.startContainer(),
a.endContainer())}).map(function(a){return a.name()})}).fold(oa.off,oa.on)},function(a){return Ck.divine(a.image()).align().fold(oa.off,function(a){return"center"===a?oa.on(pd.justifycentre().name()):"right"===a?oa.on(pd.justifyright().name()):oa.on(pd.justifyleft().name())})})},to={read:function(a){var b=hF(a);a=gF(a)?oa.on({}):oa.off();return r3(b,a)},alignment:hF,inQuote:gF,atQuoteBreak:function(a){return a.getSelection().bind(function(a){var c=a.startContainer();return a.collapsed()?J.closest(c,
"blockquote").bind(function(a){return q3.hasNoContent(c)?h.some({quote:f.constant(a),element:f.constant(c)}):h.none()}):h.none()})}},bk={multiple:function(a,b,c){return function(d,e){Yc.runTransaction(a,b,function(){k.each(e,c)},a.toEditing)}},single:function(a,b,c){return function(d,e){Yc.runTransaction(a,b,function(){c(e).each(function(b){a.setSelection(b.start(),b.soffset(),b.finish(),b.foffset())});a.backupSelection()},a.toEditing)}}},t3=Sa(),Zj={atStartOf:function(a,b,c){Lr.atStartOf(t3,a,b,
c)}},u3=pE.resolve("style-marker"),jb={styleSpan:f.constant(u3),overrideSpan:f.constant("data-ephox-override"),quasiBlock:mm.style},Ui=ma.generate([{none:[]},{tempIn:["selection"]},{tempOut:["selection"]},{rangedIn:["selection"]},{rangedOut:["selection"]}]),vo={detect:function(a,b){return a.getSelection().fold(function(){return Ui.none()},function(a){var d=b(a);return Fa.isCollapsed(a.startContainer(),a.startOffset(),a.endContainer(),a.endOffset())?d?Ui.tempIn(a):Ui.tempOut(a):d?Ui.rangedIn(a):Ui.rangedOut(a)})}},
iF=function(a,b){return q.fromTag(a,b.dom())},jF=function(a,b,c){c=q.fromTag("span",c.dom());t.set(c,a,b);return c},uo={nu:function(a,b,c,d){return"span"===c?jF(a,b,d):iF(c,d)},nuTag:iF,nuAttr:function(a,b,c){c=q.fromTag("span",c.dom());m.set(c,a,b);return c},nuStyle:jF},kF={"font-weight":{700:"bold",400:"normal"}},lF=function(a,b){if(a===b&&""!==b&&void 0!==b)return!0;var c=a.toLowerCase(),d=b.toLowerCase().split(" ");return k.contains(d,c)},mF=function(a,b){if(void 0!==kF[a]){var c=kF[a][b];return void 0!==
c?c:b}return b},nF=function(a,b){var c=v3(a,b);return mF(b,c)},v3=function(a,b){return(A.isText(a)?y.parent(a):h.some(a)).map(function(a){return t.get(a,b)}).getOr(void 0)},Jh={state:function(a,b,c){a=nF(a,b);return"text-decoration"===b?lF(c,a):c===a&&""!==a&&void 0!==a},rawState:function(a,b,c){a=mF(b,t.getRaw(a,b).getOr(void 0));return"text-decoration"===b?lF(c,a):c===a&&""!==a&&void 0!==a},value:nF},oF={"text-decoration":function(a,b,c,d){var e=function(d){return sa.closest(d,function(d){return A.isElement(d)&&
(k.contains(c,A.name(d))||t.getRaw(d,a).exists(function(a){return-1!==a.indexOf(b)}))}).toArray()};d=e(d.startContainer()).concat(e(d.endContainer()));return k.exists(d,function(d){return k.contains(c,A.name(d))?!0:t.getRaw(d,a).exists(function(a){return b===a})})}},w3={handle:function(a,b,c){return void 0!==oF[a]?h.some(oF[a]):h.none()}},pF=function(a,b){return b.getSelection().map(function(b){return a(b)}).getOr(!1)},cs=function(a,b,c,d){var e=ed.leftWith(fa.isBlock,d).getOrThunk(function(){return S.point(d.startContainer(),
d.startOffset())}),g=ed.rightWith(fa.isBlock,d).getOrThunk(function(){return S.point(d.endContainer(),d.endOffset())});return w3.handle(a,b).fold(function(){var c=Jh.state(e.element(),a,b),d=Jh.state(g.element(),a,b);return c||d},function(d){return d(a,b,c,sb.hare(e.element(),e.offset(),g.element(),g.offset()))})},x3=function(a,b,c,d){return pF(f.curry(cs,a,b,c),d)},qF=function(a,b){return y3(a,b)},z3=function(a,b){return pF(f.curry(qF,a),b)},y3=function(a,b){var c=ed.leftWith(fa.isBlock,b).fold(b.startContainer,
function(a){return a.element()}),d=ed.rightWith(fa.isBlock,b).fold(b.endContainer,function(a){return a.element()}),c=eb.closest(c,a),d=eb.closest(d,a);return c||d},rF=function(a,b){return k.contains(a,A.name(b))},A3=function(a,b){var c="["+a+"]";return J.closest(b.startContainer(),c).bind(function(d){return J.closest(b.endContainer(),c).bind(function(b){var c=m.get(d,a);b=m.get(b,a);c=c.toLowerCase();b=b.toLowerCase();return c===b?h.some(c):h.none()})})},vf={queriesStyle:function(a,b,c){return{queryOn:f.curry(cs,
a,b,c),query:f.curry(x3,a,b,c)}},queryStyleOn:cs,queriesTag:function(a){return{queryOn:f.curry(qF,a),query:f.curry(z3,a)}},relatedStyle:function(a,b,c,d){(c=rF(c,d))||(c="span"===A.name(d)&&Jh.state(d,a,b));return c},relatedTags:rF,queryAttrSame:function(a,b){return b.getSelection().bind(function(b){return A3(a,b)})},queryAttrIs:function(a,b,c){var d=b.toLowerCase(),e=function(b){return m.get(b,a).toLowerCase()===d};return J.closest(c.startContainer(),"["+a+"]").filter(e).orThunk(function(){return J.closest(c.endContainer(),
"["+a+"]").filter(e)}).isSome()}},vm=ma.generate([{none:["last","mode"]},{running:["next","mode"]},{split:["boundary","last","mode"]},{finished:["element","mode"]}]),ds=C.immutable("start","soffset","finish","foffset"),sF=function(a,b,c){c=mc.toLeaf(a,c,0);return es(a,c.element())?fs(a,b,c.element()).map(function(a){return S.point(a,0)}).getOr(c):c},Vi=function(a,b){return Yb.isFrame(a,b)||Yb.isBlock(a,b)||k.contains(["li"],a.property().name(b))},fs=function(a,b,c){return na.seekRight(a,c,function(b){return!es(a,
b)&&!Vi(a,b)},b)},es=function(a,b){return a.property().isText(b)?a.property().parent(b).exists(function(b){return k.contains("table tbody thead tfoot tr ul ol".split(" "),a.property().name(b))}):!1},B3=function(a,b,c,d,e){return na.walk(a,d,c,na.walkers().right()).fold(function(){return vm.none(d,na.sidestep)},function(b){return a.eq(b.item(),e)?vm.finished(e,b.mode()):Vi(a,b.item())?vm.split(b.item(),d,b.mode()):vm.running(b.item(),b.mode())})},uF=function(a,b,c,d){return fs(a,b,c).fold(function(){return h.none()},
function(e){return a.eq(e,d)?h.some(d):tF(a,c,e)?uF(a,b,e,d):Vi(a,e)?(e=sF(a,b,e),h.some(e.element())):h.some(e)})},tF=function(a,b,c){return a.property().parent(b).exists(function(b){return a.eq(b,c)})},gs=function(a,b,c,d,e,g){return B3(a,b,c,e,g).fold(function(a,b){return[{start:d,finish:a}]},function(c,e){return gs(a,b,e,d,c,g)},function(c,f,k){var m={start:d,finish:f};return(tF(a,e,c)?uF(a,b,c,g):function(){var d=sF(a,b,c);return a.eq(d.element(),c)?na.walk(a,c,na.advance,na.walkers().right()).map(function(a){return a.item()}):
h.some(d.element())}()).fold(function(){return[m]},function(c){return a.eq(c,g)?[m].concat({start:g,finish:g}):[m].concat(gs(a,b,na.advance,c,c,g))})},function(a,b){return[{start:d,finish:a}]})},hs=function(a,b,c){if(Vi(a,b)){var d=a.property().children(b);return 0<=c&&c<d.length?hs(a,d[c],0):c===d.length?hs(a,d[d.length-1],0):b}return b},vF=function(a,b,c,d){c=hs(a,c,d);return es(a,c)?fs(a,b,c).getOr(c):c},C3=function(a,b,c,d,e,g){var f=vF(a,b,c,d),h=vF(a,b,e,g);b=gs(a,b,na.sidestep,f,f,h);return k.map(b,
function(b,f){var l=a.eq(b.start,c)?d:0,h;a.eq(b.finish,e)?h=g:(h=b.finish,h=a.property().isText(h)?a.property().getText(h).length:a.property().children(h).length);return ds(b.start,l,b.finish,h)})},wF=function(a,b,c,d,e,g){a.eq(c,e)?Vi(a,c)?(d=mc.toLeaf(a,c,d),g=mc.toLeaf(a,c,g),a=a.eq(d.element(),c)&&a.eq(g.element(),c)?[ds(d.element(),d.offset(),g.element(),g.offset())]:wF(a,b,d.element(),d.offset(),g.element(),g.offset())):a=[ds(c,d,c,g)]:a=C3(a,b,c,d,e,g);return a},D3={collect:wF},se=ma.generate([{leftEdge:["element"]},
{between:["before","after"]},{rightEdge:["element"]}]),E3=function(a,b,c){c=cd.split(a,b,c);var d=function(){return se.rightEdge(b)};return cd.position(a,c).fold(d,function(){return se.leftEdge(b)},se.between,d)},xF=function(a,b,c,d){if(a.property().isText(b))return E3(a,b,c);if(a.property().isEmptyTag(b))return d(b);a=a.property().children(b);b=0===c?se.leftEdge(b):c===a.length?se.rightEdge(b):0<c&&c<a.length?se.between(a[c-1],a[c]):se.rightEdge(b);return b},yF={toLeft:function(a,b,c,d){return xF(a,
c,d,se.leftEdge).fold(function(a){return a},function(a,b){return b},function(c){return na.after(a,c,b).getOr(c)})},toRight:function(a,b,c,d){return xF(a,c,d,se.rightEdge).fold(function(c){return na.before(a,c,b).getOr(c)},function(a,b){return a},function(a){return a})}},F3=function(a,b,c,d){var e=a.property().children(b);b=function(b){return k.findIndex(e,f.curry(a.eq,b))};c=c.bind(b).getOr(0);d=d.bind(b).getOr(e.length-1);return-1<c&&-1<d?h.some(e.slice(c,d+1)):h.none()},zF=function(a,b,c,d){return Ic.breakPath(a,
b,function(b){return a.property().parent(b).fold(f.constant(!0),f.curry(a.eq,c))},d)},AF=function(a,b,c,d){a.eq(b,d)?d=h.none():(d=zF(a,d,b,Oi.breakToRight),d=h.some(d.first()));if(a.eq(b,c))c=h.none();else{var e=zF(a,c,b,Oi.breakToLeft);0<e.splits().length&&a.insert().prepend(e.splits()[0].second(),c);c=h.some(e.second().getOr(c))}return F3(a,b,c,d)},G3=function(a,b,c,d,e){return Gr.ancestors(a,c,d,b).shared().orThunk(function(){return Ic.sharedOne(a,function(c,d){return b(d)?h.some(d):a.up().predicate(d,
b)},[c,d])}).map(e)},H3=function(a,b,c,d,e){return G3(a,b,c,d,e).bind(function(b){return AF(a,b,c,d)})},BF={fracture:function(a,b,c,d,e){e=void 0!==e?e:f.identity;a.eq(c,d)?(b=e(c),a=a.eq(b,c)?h.some([c]):AF(a,b,c,c)):a=H3(a,b,c,d,e);return a}},CF=function(a,b,c,d,e,g,f){a.property().isText(c)&&a.eq(c,e)?(c=cd.splitByPair(a,c,d,g),a=BF.fracture(a,b,c,c,f)):(e=yF.toRight(a,b,e,g),c=yF.toLeft(a,b,c,d),a=BF.fracture(a,b,c,e,f));return a},DF={fractures:function(a,b,c,d,e,g,f){c=D3.collect(a,b,c,d,e,g);
return k.bind(c,function(c,d){return CF(a,b,c.start(),c.soffset(),c.finish(),c.foffset(),f).toArray()})},fracture:CF},EF=Sa(),I3={fractures:function(a,b,c,d,e,g){return DF.fractures(EF,a,b,c,d,e,g)},fracture:function(a,b,c,d,e,g){return DF.fracture(EF,a,b,c,d,e,g)}},Tf={makeTo:function(a,b,c){var d=y.owner(a).dom().createRange();d.setStart(a.dom(),b);d.setEndBefore(c.dom());return d},makeFrom:function(a,b,c){var d=y.owner(b).dom().createRange();d.setStartAfter(a.dom());d.setEnd(b.dom(),c);return d},
leftEdge:function(a,b,c,d){return Bc.after(a,b,c,d)?S.point(c,d):S.point(a,b)},rightEdge:function(a,b,c,d){return Bc.after(a,b,c,d)?S.point(a,b):S.point(c,d)}},J3=function(a,b){var c=W.keys(b).join(",");return(ra.is(a,c)?[a]:[]).concat(X.descendants(a,c))},FF={cleanse:function(a,b){var c=J3(a,b);k.each(c,function(a){var c=A.name(a);(void 0!==b[c]?b[c]:f.noop)(a)})}},K3={onlyEphox:function(a){var b=ea.get(a),b=k.filter(b,function(a,b,e){return!aa.startsWith(a,"ephox-")});k.each(b,function(b){n.remove(a,
b)})}},Wi=function(a){B.unwrap(a)},GF=function(a,b){t.remove(a,b);m.hasNone(a)&&Wi(a)},HF=function(a){var b={};k.each(a,function(a){b[a]=Uf(Wi)});return b},Uf=function(a){return function(b){m.has(b,"data-ignore")||a.apply(null,arguments)}},Ub={cssAndTag:function(a,b,c){a=HF(a);c=function(a){GF(a,b)};return Z.merge(a,{span:Uf(c),font:Uf(c)})},tagList:HF,removeStyle:Uf(GF),removeAttribute:Uf(function(a,b){m.remove(a,b);m.hasNone(a)&&Wi(a)}),unwrap:Uf(Wi),unstyle:Uf(function(a){K3.onlyEphox(a);m.remove(a,
"style");m.hasNone(a)&&Wi(a)}),normal:function(a){m.remove(a,"data-ignore")},ignore:function(a){m.set(a,"data-ignore","true")}},wm=function(a,b){var c=y.children(a);k.each(c,function(a){wm(a,b)});b(a)},IF=function(a){return A.isElement(a)&&!fa.isEmptyTag(a)&&!fa.isFrame(a)&&0===ua.get(a).replace(Ua.zeroWidth(),"").length},Ug={wipeout:wm,onEmpty:function(a){wm(a,function(a){!A.isElement(a)||fa.isFrame(a)||fa.isEmptyTag(a)||0!==ua.get(a).length?IF(a)&&B.unwrap(a):B.remove(a)})},onQuasiEmpty:function(a){wm(a,
function(a){IF(a)&&B.remove(a)})}},JF=function(a){return y.parent(a).fold(f.constant([]),function(a){return fa.isBlock(a)?[a]:[a].concat(JF(a))})},L3=function(a,b){var c=[b].concat(JF(b)),d=k.reverse(c);return k.findIndex(d,a).map(function(a){return 0===a?d[0]:d[a-1]}).getOr(b)},M3=function(a,b){k.each(b,function(b){FF.cleanse(b,a);Ug.onQuasiEmpty(b)})},wo={run:function(a,b,c,d){var e=ed.left(c).getOrThunk(function(){return Tf.leftEdge(c.startContainer(),c.startOffset(),c.endContainer(),c.endOffset())}),
g=ed.right(c).getOrThunk(function(){return Tf.rightEdge(c.startContainer(),c.startOffset(),c.endContainer(),c.endOffset())}),l=f.curry(L3,d.matches);b=I3.fractures(b,e.element(),e.offset(),g.element(),g.offset(),l);b=k.bind(b,function(b){var c;if(0<b.length){c=d.purifiers();var e=(0,d.nu)(a);Ub.ignore(e);h.from(b[0]).bind(y.prevSibling).each(Ug.onQuasiEmpty);h.from(b[b.length-1]).bind(y.nextSibling).each(Ug.onQuasiEmpty);k.each(b,Ug.onQuasiEmpty);b=k.filter(b,ca.inBody);0<b.length?(v.before(b[0],
e),K.append(e,b),M3(c,b),c=[e]):c=[]}else c=[];return c});k.each(b,Ub.normal);return sb.fussyFromElements(b)}},is=function(a,b){return L.eq(a,b)?[]:y.parent(a).fold(function(){return[]},function(c){c=is(c,b);var d=y.findIndex(a).toArray();return c.concat(d)})},N3=function(a,b){return a.slice(b+1)},O3=function(a,b){return a.slice(0,b)},js=function(a,b,c,d){if(0===d.length)b(c);else{var e=d[0],g=d.slice(1);y.child(c,e).each(function(c){js(a,b,c,g)});c=y.children(c);e=a(c,e);k.each(e,b)}},KF={one:function(a,
b){a(b)},across:function(a,b,c,d){c=is(c,b);d=is(d,b);b=y.children(b);var e=b.slice(0<c.length?c[0]+1:0,0<d.length?d[0]:b.length-1);k.each(e,a);0<c.length&&js(N3,a,b[c[0]],c.slice(1));0<d.length&&js(O3,a,b[d[0]],d.slice(1))}},Xi=Sa(),Vg={split:function(a,b){return cd.split(Xi,a,b)},splitByPair:function(a,b,c){return cd.splitByPair(Xi,a,b,c)},range:function(a,b,c,d){return cd.range(Xi,a,b,c,d)},subdivide:function(a,b){return cd.subdivide(Xi,a,b)},position:function(a){return cd.position(Xi,a)}},LF=
function(a,b,c){var d=y.children(b),e=0===c,g=c===d.length,f=0<d.length;return void 0!==a.empty&&k.contains(["br","hr","img"],A.name(b))?a.empty:void 0!==a.hasPrevious&&f&&0<c&&c<d.length?a.hasPrevious:void 0!==a.hasNext&&f&&c<d.length-1?a.hasNext:void 0!==a.rightEdge&&f&&g?a.rightEdge:void 0!==a.first&&e?a.first:a.otherwise},P3=function(a,b,c){return LF(a.before.element,b,c)(b,c)},Q3=function(a,b,c){return LF(a.after.element,b,c)(b,c)},MF=function(a,b,c){c=Vg.split(b,c);var d=function(a){return function(){var c=
Array.prototype.slice.call(arguments,0),c=[b].concat(c);return a.apply(null,c)}};return Vg.position(c).fold(d(a.none),d(a.start),d(a.middle),d(a.end))},R3=function(a,b,c){return MF(a.before.text,b,c)},S3=function(a,b,c){return MF(a.after.text,b,c)},xm={prefix:function(a,b,c){return(A.isText(b)?R3:P3)(a,b,c)},postfix:function(a,b,c,d,e){b=Tf.rightEdge(b,c,d,e);return(A.isText(b.element())?S3:Q3)(a,b.element(),b.offset())}},ks=function(a,b){return a.bind(function(a){return b.map(function(b){return{left:f.constant(a),
right:f.constant(b)}})})},T3=function(a,b,c,d){b=Vg.splitByPair(b,c,d);return ks(a.general.insertBefore(b),a.general.insertAfter(b))},U3=function(a,b,c,d){c=y.child(b,c);var e=d<y.children(b).length?y.child(b,d):y.lastChild(b);return c.bind(function(b){return e.bind(function(c){return ks(a.general.insertBefore(b),a.general.insertAfter(c))})})},Cx={both:function(a,b,c,d,e){if(L.eq(b,d)){var g=A.isText(b)?T3:U3;return c>e?g(a,b,e,c):g(a,b,c,e)}g=Tf.leftEdge(b,c,d,e);b=xm.postfix(a,b,c,d,e);a=xm.prefix(a,
g.element(),g.offset());return ks(a,b)}},ym=C.immutable("inside","outside"),NF=function(a,b){return y.parent(b).fold(f.constant(!0),function(b){return a(b)})},OF=function(a,b){var c=Sa(),d=[a].concat(c.up().all(a)),e=[b].concat(c.up().all(b));return k.find(d,function(a){return k.exists(e,f.curry(L.eq,a))})},PF=function(a){return ym(a.second(),h.some(a.first()))},QF=function(a){return ym(h.some(a.first()),a.second())},RF=function(a){return ym(h.some(a),h.none())},ls=function(a,b,c){c=f.curry(NF,c);
c=c(a)?a:sa.ancestor(a,c).getOr(a);return L.eq(a,c)?RF(a):SF(a,b,c)},SF=function(a,b,c){a=gm.breakPath(a,f.curry(L.eq,c));return b(a)},V3={bridge:function(a,b,c){var d=ls(a,PF,c),e=ls(b,QF,c);return d.inside().bind(function(a){return e.inside().bind(function(b){return OF(a,b).map(function(c){return{common:f.constant(c),left:f.constant(a),right:f.constant(b),wipeout:function(){d.outside().each(Ug.onEmpty);e.outside().each(Ug.onEmpty)}}})})})},edge:ym,isSplit:NF,getCommon:OF,onLeft:PF,onRight:QF,onInside:RF,
splitFrom:ls,breaker:SF},W3=C.immutable("start","soffset","finish","foffset"),zm=f.constant(!1),ms=function(a,b){return fa.isBlock(b)||fa.isContainer(b)||a(b)},TF=function(a,b){return y.parents(b,f.curry(ms,a))},UF=function(a,b,c){return A.isElement(c)&&!b(c)?(a=TF(a,c),!k.exists(a,b)):!1},VF=function(a,b){return function(c){return ms(a,c)||UF(a,b,c)}},X3=function(a,b,c,d,e,g){var f=Fa.isCollapsed(b,c,d,e);b=Ja.toLeaf(b,c);d=Ja.toLeaf(d,e);var h=yg.extractTo(b.element(),b.offset(),g,zm),k=yg.extractTo(d.element(),
d.offset(),g,zm),m=function(b){return f||b.offset()!==ka.getEnd(b.element())?b:Lb.after(b.element(),a).map(function(a){return S.point(a,0)}).getOr(b)},w=function(b){return f||0!==b.offset()?b:Lb.before(b.element(),a).map(function(a){return S.point(a,ka.getEnd(a))}).getOr(b)};return function(){return yg.find(h.element(),h.offset(),zm).map(m).bind(function(a){return yg.find(k.element(),k.offset(),zm).map(w).map(function(b){return W3(a.element(),a.offset(),b.element(),b.offset())})})}},Y3={applyTo:function(a,
b,c,d,e,g,f){var h=VF(a,f),k=X3(a,c,d,e,g,h);a=GP(Ua.zeroWidth(),Ua.zeroWidth());return Cx.both(a,c,d,e,g).bind(function(a){return V3.bridge(a.left().element(),a.right().element(),h).bind(function(c){b(c.common(),c.left(),c.right());a.left().cleanup();a.right().cleanup();c.wipeout();return k()})})},blockRoot:ms,pathToTop:TF,isEnclosingInline:UF,isStopper:VF},WF=function(a,b,c,d){var e=function(a){FF.cleanse(a,b)};return Y3.applyTo(a,function(a,b,c,d){if(L.eq(b,c))KF.one(e,b);else KF.across(e,a,b,
c)},c.startContainer(),c.startOffset(),c.endContainer(),c.endOffset(),d)},Vd={doRemoveAll:function(a,b,c){return WF(a,b,c,f.constant(!0))},doRemoveUpto:WF,allRules:f.constant({b:Ub.unwrap,strong:Ub.unwrap,i:Ub.unwrap,em:Ub.unwrap,u:Ub.unwrap,strike:Ub.unwrap,del:Ub.unwrap,s:Ub.unwrap,sup:Ub.unwrap,sub:Ub.unwrap,small:f.noop,span:Ub.unstyle,font:Ub.unstyle})},Z3=function(a,b,c){return{groups:0<a.current.length?a.groups.concat({parent:a.parent,children:a.current}):a.groups,current:[c],parent:b}},$3=
function(a,b,c){return{groups:a.groups,current:a.current.concat([c]),parent:b}},a4=function(a,b,c){return h.from(b.current[b.current.length-1]).bind(a.query().nextSibling).bind(function(b){return a.eq(b,c)?h.some($3):h.none()}).getOr(Z3)},b4={textnodes:function(a,b){var c=k.foldl(b,function(b,c){return a.property().parent(c).fold(function(){return b},function(g){return a4(a,b,c)(b,g,c)})},{groups:[],current:[],parent:null});return 0<c.current.length?c.groups.concat({parent:c.parent,children:c.current}):
c.groups}},YF=function(a,b,c,d,e,g){b=cd.range(a,b,c,d,e);return XF(a,b,g)},XF=function(a,b,c){if(0===b.length)return b;b=k.filter(b,function(b){return a.property().isText(b)&&0<a.property().getText(b).length});return k.map(b,function(b){var e=c();a.insert().before(b,e.element());e.wrap(b);return e.element()})},c4=function(a,b){return h.from(b[0]).map(function(c){var d=Ze.toLower(a,b[b.length-1]);return S.points(S.point(c,0),S.point(d.element(),d.offset()))})},Yi={wrapWith:YF,wrapper:XF,leaves:function(a,
b,c,d,e,g){b=Ze.toLeaf(a,b,c);d=Ze.toLeaf(a,d,e);g=YF(a,b.element(),b.offset(),d.element(),d.offset(),g);return c4(a,g)},reuse:function(a,b,c,d,e,g,f){b=Ze.toLeaf(a,b,c);d=Ze.toLeaf(a,d,e);d=cd.range(a,b.element(),b.offset(),d.element(),d.offset());d=b4.textnodes(a,d);var h=function(a){return a.parent},r=function(b){var c=f();a.insert().before(b.children[0],c.element());k.each(b.children,c.wrap);return c.element()};return k.map(d,function(b){return(a.property().children(b.parent).length===b.children.length&&
g(b.parent)?h:r)(b)})}},e4=function(a,b,c,d,e,g){a=d4(a,b,c,d,e,g);b=a.cursor();b=S.points(S.point(b.element(),b.offset()),S.point(b.element(),b.offset()));return h.some({range:f.constant(b),temporary:a.temporary,wrappers:a.wrappers})},f4=function(a,b,c,d,e){d=b.dom().ownerDocument;var g=a.create().nu("span",d);d=a.create().text(Ua.zeroWidth(),d);a.insert().append(g,d);(a.property().isEmptyTag(b)?a.property().parent(b):h.some(b)).each(function(b){Lr.atStartOf(a,b,c,g)});return{cursor:f.constant(S.point(d,
1)),wrappers:f.constant([g]),temporary:f.constant(!0)}},d4=function(a,b,c,d,e,g){return a.property().parent(b).bind(function(d){var e=S.point(b,c),k;if(k=ZF(a,g,d))if(k=1===a.property().children(d).length)k=a.property().isText(b)&&a.property().getText(b)===Ua.zeroWidth();return k?h.some({cursor:f.constant(e),temporary:f.constant(!1),wrappers:f.constant([d])}):h.none()}).getOrThunk(function(){return f4(a,b,c,d,e)})},ZF=function(a,b,c){return"span"===a.property().name(c)&&!1===b(c)},g4=function(a,b,
c,d,e,g){var l=b.dom().ownerDocument,k=Yi.reuse(a,b,c,d,e,f.curry(ZF,a,g),function(){return Dv(a,a.create().nu("span",l))});return h.from(k[k.length-1]).map(function(b){var c=a.property().children(b).length;b=S.points(S.point(k[0],0),S.point(b,c));return{wrappers:f.constant(k),temporary:f.constant(!1),range:f.constant(b)}})},h4={spans:function(a,b,c,d,e,g){g=void 0!==g?g:f.constant(!1);return(a.eq(b,d)&&c===e?e4:g4)(a,b,c,d,e,g)}},Wg={nu:function(a,b){return Dv(a,b)},wrapWith:function(a,b,c,d,e,g){return Yi.wrapWith(a,
b,c,d,e,g)},wrapper:function(a,b,c){return Yi.wrapper(a,b,c)},leaves:function(a,b,c,d,e,g){return Yi.leaves(a,b,c,d,e,g)},reuse:function(a,b,c,d,e,g,f){return Yi.reuse(a,b,c,d,e,g,f)},spans:function(a,b,c,d,e,g){return h4.spans(a,b,c,d,e,g)}},Xg=Sa(),Yg={nu:function(a){return Wg.nu(Xg,a)},wrapWith:function(a,b,c,d,e){return Wg.wrapWith(Xg,a,b,c,d,e)},wrapper:function(a,b){return Wg.wrapper(Xg,a,b)},leaves:function(a,b,c,d,e){return Wg.leaves(Xg,a,b,c,d,e)},reuse:function(a,b,c,d,e,g){return Wg.reuse(Xg,
a,b,c,d,e,g)},spans:function(a,b,c,d,e){return Wg.spans(Xg,a,b,c,d,e)}},i4=function(a,b,c){a.getSelection().each(function(d){var e=ed.left(d).getOrThunk(function(){return S.point(d.startContainer(),d.startOffset())}),g=ed.right(d).getOrThunk(function(){return S.point(d.endContainer(),d.endOffset())});Yg.spans(e.element(),e.offset(),g.element(),g.offset(),c).each(function(c){var d=c.temporary()?[jb.styleSpan()]:[],e=c.range();k.each(c.wrappers(),function(a){ea.add(a,d);b(a)});c=e.begin();e=e.end();
a.setSelection(c.element(),c.offset(),e.element(),e.offset())})})},$F=function(a,b,c,d){b(a);i4(a,d,c)},Wd={style:function(a,b,c,d){return $F(a,c,d,function(a){t.setAll(a,b)})},generic:$F},aG=ak("bold","strong",["strong","b"],"font-weight","bold","normal"),bG=ak("italic","em",["em","i"],"font-style","italic","normal"),cG=ak("underline","span",["u"],"text-decoration","underline","none"),dG=ak("strikethrough","span",["s","strike","del"],"text-decoration","line-through","none"),eG=function(a,b){var c=
function(a,c){return uo.nuAttr(b,a,c)},d=function(a){return"span"===A.name(a)&&m.has(a,b)},e={span:function(a){Ub.removeAttribute(a,b)}},g=function(a,c){return vf.queryAttrIs(b,a,c)},l=function(a,c){Wd.generic(c,f.noop,void 0,function(c){n.add(c,jb.styleSpan());m.set(c,b,a)});return h.none()};return{run:function(a,k,m){return vo.detect(k,f.curry(g,m)).fold(function(){return h.none()},function(a){var c=["[",b,'="',m,'"]'].join("");J.closest(a.startContainer(),c).each(B.unwrap);return h.none()},function(a){return l(m,
k)},function(b){return Vd.doRemoveAll(a,e,b)},function(b){return wo.run(k.doc(),a,b,{nu:f.curry(c,m),matches:d,purifiers:f.constant(e)})})},queryValue:function(a){return vf.queryAttrSame(b,a)},name:f.constant(a)}}("language","lang"),fG=Cv("superscript","sup",["sup","sub"],{"font-size":"smaller","vertical-align":"super"}),gG=Cv("subscript","sub",["sub","sup"],{"font-size":"smaller","vertical-align":"sub"}),hG={bold:aG,italic:bG,underline:cG,strikethrough:dG,language:eG,superscript:fG,subscript:gG},
nc={bold:f.constant(aG),italic:f.constant(bG),underline:f.constant(cG),strikethrough:f.constant(dG),language:f.constant(eG),superscript:f.constant(fG),subscript:f.constant(gG),search:function(a){return void 0!==hG[a]?h.some(hG[a]):h.none()}},Zg=function(a,b){return a.query(b)},Cf={bold:f.curry(Zg,nc.bold()),italic:f.curry(Zg,nc.italic()),underline:f.curry(Zg,nc.underline()),strikethrough:f.curry(Zg,nc.strikethrough()),superscript:f.curry(Zg,nc.superscript()),subscript:f.curry(Zg,nc.subscript()),language:f.curry(function(a,
b){return a.queryValue(b)},nc.language())},j4=C.immutable("bold","italic","underline","strikethrough","superscript","subscript"),$g=function(a,b){return a.on(function(a){return b(a)?oa.on(!0):oa.off()}).getOrThunk(oa.off)},IP={read:function(a){return j4($g(a,Cf.bold),$g(a,Cf.italic),$g(a,Cf.underline),$g(a,Cf.strikethrough),$g(a,Cf.superscript),$g(a,Cf.subscript))}},Am=C.immutable("startContainer","startOffset","endContainer","endOffset"),Bm=C.immutable("item","start","finish","text"),Cm=C.immutable("items",
"abort"),iG=function(a,b){var c=a.property().getText(b);return Bm(b,0,c.length,c)},ns=function(a,b,c){return Cm([],!0)},k4=function(a,b,c){return Cm([],!1)},l4=function(a,b,c){var d=a.property().getText(b);return c(d).fold(function(){return Cm([Bm(b,0,d.length,d)],!1)},function(a){a=a[0]===a[1]?[]:[Bm(b,a[0],a[1],d.substring(a[0],a[1]))];return Cm(a,!0)})},te={detail:iG,fromItem:function(a,b){return a.property().isText(b)?iG(a,b):Bm(b,0,0,"")},decide:function(a,b,c,d){return(a.property().isBoundary(b)?
ns:a.property().isEmptyTag(b)?ns:d(a,b)?ns:a.property().isText(b)?l4:k4)(a,b,c)}},jG={all:function(a,b){for(var c=b.term(),d=[],e=c.exec(a);e;){var g=e.index+b.prefix(e),e=e[0].length-b.prefix(e)-b.suffix(e);d.push({start:f.constant(g),finish:f.constant(g+e)});c.lastIndex=g+e;e=c.exec(a)}return d}},m4=function(a){a=Array.prototype.slice.call(a,0);a.sort(function(a,c){return a.start()<c.start()?-1:c.start()<a.start()?1:0});return a},n4={search:function(a,b){var c=k.bind(b,function(b){var c=jG.all(a,
b.pattern());return k.map(c,function(a){return Z.merge(b,{start:a.start,finish:a.finish})})});return m4(c)}},Dm={findall:function(a,b){return jG.all(a,b)},findmany:function(a,b){return n4.search(a,b)}},o4=RegExp(db.wordbreak()+"+","g"),kG=f.constant(0),lG=function(a){a=Dm.findall(a,db.custom(db.wordbreak(),kG,kG,h.none()));return h.from(a[a.length-1]).map(function(a){return a.start()})},os=function(a){a=a.search(o4);return-1<a?h.some(a):h.none()},Zi={firstWord:function(a){return os(a).map(function(b){return a.substring(0,
b+1)})},lastWord:function(a){return lG(a).map(function(b){return a.substring(b)})},leftBreak:lG,rightBreak:os,hasBreak:function(a){return os(a).isSome()}},mG=na.walkers(),nG=mG.left(),oG=mG.right(),p4={sibling:nG.sibling,first:nG.first,slicer:function(a){return Zi.leftBreak(a).map(function(b){return[b+1,a.length]})}},q4={sibling:oG.sibling,first:oG.first,slicer:function(a){return Zi.rightBreak(a).map(function(a){return[0,a]})}},ps=function(a,b,c,d,e){b=na.walk(a,b,c,d).map(function(b){var c=te.decide(a,
b.item(),d.slicer,e);b=c.abort()?[]:ps(a,b.item(),b.mode(),d,e);return c.items().concat(b)}).getOr([]);return k.filter(b,function(a){return 0<a.text().trim().length})},$i={creepLeft:function(a,b,c){return ps(a,b,na.sidestep,p4,c)},creepRight:function(a,b,c){return ps(a,b,na.sidestep,q4,c)},flatten:function(a,b){return a.property().isText(b)?[te.detail(a,b)]:k.map(a.down().predicate(b,a.property().isText),function(b){return te.detail(a,b)})},isEmpty:function(a,b){return a.property().isText(b)?!1:0===
a.down().predicate(b,a.property().isText).length}},qs=function(a,b){return a.up().closest(b,"[lang]",f.constant(!1)).map(function(b){return a.attrs().get(b,"lang")})},ah={nu:function(a){var b=[],c=[],d=[],e=a,g=function(a){a.each(function(a){b.push(a)})},l=function(a){a.each(function(a){b=b.slice(0,b.length-1)})},k=function(){0<d.length&&c.push({lang:f.constant(e),details:f.constant(d)})},r=function(a){k();d=[];e=a},m=function(c){return c.or(h.from(b[b.length-1])).getOr(a)};return{openInline:function(a,
b){var c=m(a);c!==e&&r(c);g(a)},closeInline:function(a,b){l(a)},addDetail:function(a){var b=m(h.none());b!==e&&r(b);d.push(a)},addEmpty:function(a){a=m(h.none());r(a)},openBoundary:function(a,b){g(a);var c=m(a);r(c)},closeBoundary:function(a,b){l(a);var c=m(a);r(c)},done:function(){k();return c.slice(0)}}},calculate:qs,softBounder:function(a){return function(b,c){var d=qs(b,c);return!a.equals(d)}},strictBounder:function(a,b){return function(c,d){var e=qs(c,d).getOr(a);return b!==e}}},pG=function(a,
b,c,d){var e=$i.creepLeft(a,b,d);d=$i.creepRight(a,c,d);b=0<e.length?e[e.length-1]:te.fromItem(a,b);a=0<d.length?d[d.length-1]:te.fromItem(a,c);e=0===e.length&&0===d.length;return{left:f.constant(b),right:f.constant(a),isEmpty:f.constant(e)}},aj={byBoundary:function(a,b){var c=f.constant(!1),c=pG(a,b,b,c),d=c.isEmpty()&&$i.isEmpty(a,b);return{left:c.left,right:c.right,isEmpty:f.constant(d)}},getEdges:pG,byLanguage:function(a,b){var c=ah.calculate(a,b),d=ah.softBounder(c),e=$i.creepLeft(a,b,d),d=$i.creepRight(a,
b,d),g=a.property().isText(b)?[te.detail(a,b)]:[],l=k.reverse(e).concat(g).concat(d);return{all:f.constant(l),left:f.constant(e),middle:f.constant(g),right:f.constant(d),lang:f.constant(c)}}},rs=function(a,b,c,d){d=a.property().getText(b);var e=k.filter(d.substring(0,c),function(a){return a!==Ua.zeroWidth()}).length,g=k.filter(d.substring(c,d.length),function(a){return a===Ua.zeroWidth()}).length;a=aj.byLanguage(a,b);b=0===e&&0===a.left().length;c=c+g===d.length&&0===a.right().length;return{all:a.all,
leftEdge:f.constant(b),rightEdge:f.constant(c),text:f.constant(d)}},Em={before:function(a,b,c,d,e){a=rs(a,b,c,e);a.rightEdge()?b=h.none():(a=a.all(),0===a.length?b=h.none():(a=a[a.length-1],b=h.some(Am(b,d,a.item(),a.finish()))));return b},after:function(a,b,c,d,e){a=rs(a,b,c,e);a.leftEdge()?b=h.none():(a=a.all(),0===a.length?b=h.none():(a=a[0],b=h.some(Am(a.item(),a.start(),b,d))));return b},both:function(a,b,c,d,e,g){return d===e?h.none():h.some(Am(b,d,b,e))},neither:function(a,b,c,d){a=rs(a,b,
c,d);a.leftEdge()||a.rightEdge()?a=h.none():(b=a.all(),0===b.length?a=h.none():(a=b[0],b=b[b.length-1],a=h.some(Am(a.item(),a.start(),b.item(),b.finish()))));return a}},qG=C.immutable("before","after"),r4={around:function(a,b){var c=a.substring(0,b),d=Zi.leftBreak(c).map(function(a){return a+1}),c=a.substring(b),c=Zi.rightBreak(c).map(function(a){return b+a}),e=qG(d,c),g=qG(h.some(b),h.some(b));return c.bind(function(a){return b===a?h.some(g):h.none()}).getOrThunk(function(){return d.bind(function(a){return b===
a?h.some(g):h.none()}).getOr(e)})}},s4={word:function(a,b,c,d){if(!a.property().isText(b))return h.none();var e=a.property().getText(b),g=r4.around(e,c);return g.before().fold(function(){return g.after().fold(function(){return Em.neither(a,b,c,d)},function(e){return Em.after(a,b,c,e,d)})},function(e){return g.after().fold(function(){return Em.before(a,b,c,e,d)},function(g){return Em.both(a,b,c,e,g,d)})})}},t4={word:function(a,b,c,d){return s4.word(a,b,c,d)}},u4=Sa(),v4={word:function(a,b,c){return t4.word(u4,
a,b,c)}},rG=function(a,b,c){v4.word(b,c,f.constant(!1)).each(function(b){a.setSelection(b.startContainer(),b.startOffset(),b.endContainer(),b.endOffset())})},Jc={expand:function(a){a.getSelection().each(function(b){Fa.isCollapsed(b.startContainer(),b.startOffset(),b.endContainer(),b.endOffset())&&rG(a,b.startContainer(),b.startOffset())})},expandWord:rG},tG=function(a,b){var c=sb.fussy(b,0,b,y.children(b).length-1);sG(a,b,c)},ss=function(a){a=y.children(a);return 1===a.length&&A.isText(a[0])&&Da.get(a[0])===
Ua.zeroWidth()},uG=function(a,b){return J.closest(a,"."+jb.styleSpan()).bind(function(a){return J.closest(b,"."+jb.styleSpan()).filter(function(b){return L.eq(a,b)})})},sG=function(a,b,c){n.remove(b,jb.styleSpan());return m.has(b,jb.overrideSpan())?vG(a,b,c):h.some(c)},vG=function(a,b,c){return w4(b).bind(function(d){if(d.isStyled(b)){d=d.nu;var e=y.owner(b),e=d(e);v.before(b,e);K.append(e,y.children(b));B.remove(b);d=L.eq(b,c.start())?e:c.start();e=L.eq(b,c.finish())?e:c.finish();d=h.some(sb.fussy(d,
c.soffset(),e,c.foffset()))}else d=x4(a,b,d.purifiers(),c,d.matches);return d})},w4=function(a){a=m.get(a,jb.overrideSpan());return nc.search(a).map(function(a){return a.deferred()})},x4=function(a,b,c,d,e){b=y.owner(b);var g=sb.fromFussy(d);Vd.doRemoveUpto(a,c,g,e);a="["+jb.overrideSpan()+"]";return ra.one(a,b).bind(function(a){B.unwrap(a);return ca.inBody(d.start())&&ca.inBody(d.finish())?h.some(d):h.none()})},cf={clean:function(a,b){ss(b)?B.unwrap(b):tG(a,b)},cleanNonEmpty:tG,isTransient:ss,withinTransient:uG,
handleOverride:vG,apply:function(a,b,c,d,e){var g=sb.fussy(b,c,d,e);return uG(b,d).bind(function(b){if(ss(b)){var c=y.children(b),c=L.eq(b,g.start())?sb.fussy(c[0],0,c[0],ka.getEnd(c[0])):g;B.unwrap(b);b=h.some(c)}else b=sG(a,b,g);return b})}},Fm=function(a,b){return a.action().run(a.isRoot(),a.environment())},df=function(a,b){var c=a.temp();cf.handleOverride(a.isRoot(),c,sb.fussy(c,0,c,ka.getEnd(c)));return h.some(b)},y4=function(a,b){v.after(a.temp(),a.cursor());return h.some(b)},wG=function(a,
b){v.before(a.temp(),a.cursor());return h.some(b)},ts=function(a,b){var c=a.original();Zj.atStartOf(c.startContainer(),c.startOffset(),a.cursor());return h.some(b)},xG=function(a,b){var c=a.action().deferred().isStyled(a.temp()),d=a.cursor(),e=h.some(sb.fussy(d,1,d,1));c?c=e:(a.environment().setSelection(d,0,d,0),c=Fm(a,b));return c},z4=function(a,b){var c=a.cursor(),d=a.action().deferred();Vd.doRemoveUpto(a.isRoot(),d.purifiers(),sb.hare(c,0,c,1),d.matches);return h.some(sb.fussy(c,1,c,1))},us=function(a,
b){var c=a.cursor();a.environment().setSelection(c,0,c,1);Fm(a,b);return h.some(sb.fussy(c,1,c,1))},yG=function(a,b){var c=a.environment(),d=a.original();c.setSelection(d.startContainer(),d.startOffset(),d.endContainer(),d.endOffset());Jc.expand(c);return c.getSelection().map(sb.fromHare)},A4=function(a,b){var c=a.action().deferred();return Vd.doRemoveUpto(a.isRoot(),c.purifiers(),sb.fromFussy(b),c.matches)},B4=function(a,b){var c;a.action().deferred().isStyled(a.temp())?c=h.some(b):(a.environment().setSelection(b.start(),
b.soffset(),b.finish(),b.foffset()),c=Fm(a,b));return c},C4=function(a,b){var c=a.temp(),d=y.children(c);B.unwrap(c);return 0===d.length?h.none():h.some(sb.fussy(d[0],0,d[d.length-1],ka.getEnd(d[d.length-1])))},D4=C.immutable("environment","isRoot","cursor","temp","action","original"),E4=function(a,b){return Cb.last(a).fold(f.constant(!0),function(a){return L.eq(a,b.startContainer())&&ka.getEnd(a)===b.startOffset()})},F4=function(a,b){return Cb.first(a).fold(f.constant(!0),function(a){return L.eq(a,
b.startContainer())&&0===b.startOffset()})},G4=function(a,b,c,d,e){var g=m.get(d,jb.overrideSpan()),f=e.collapsed(),p=q.fromText(Ua.zeroWidth()),r=E4(d,e),n=F4(d,e),g=a.name()===g,w=D4(c,b,p,d,a,e);a=g&&cf.isTransient(d)?[C4]:f&&g&&r?[y4,df,xG]:f&&g&&n?[wG,df,xG]:f&&g?[ts,df,z4]:f&&r?[ts,df,us]:f&&n?[wG,df,us]:f?[ts,df,us]:g?[df,yG,A4,B4]:[df,yG,Fm];return k.foldl(a,function(a,b){return a.bind(function(a){return b(w,a)})},h.some(sb.fromHare(e)))},bh=function(a,b,c){Jc.expand(c);var d=c.getSelection().bind(function(d){return cf.withinTransient(d.startContainer(),
d.endContainer()).filter(function(a){return m.has(a,jb.overrideSpan())}).fold(function(){return a.run(b,c)},function(g){return G4(a,b,c,g,d)})});return Pg.inlineFrom(d)},Ac={bold:f.curry(bh,nc.bold()),italic:f.curry(bh,nc.italic()),underline:f.curry(bh,nc.underline()),strikethrough:f.curry(bh,nc.strikethrough()),superscript:f.curry(bh,nc.superscript()),subscript:f.curry(bh,nc.subscript()),removeformat:function(a,b){Jc.expand(b);var c=b.getSelection().bind(function(b){var c=Vd.allRules();return Vd.doRemoveAll(a,
c,b)});return Pg.inlineFrom(c)},language:f.curry(function(a,b,c,d){c.getSelection().each(function(a){cf.apply(b,a.startContainer(),a.startOffset(),a.endContainer(),a.endOffset()).each(function(a){c.setSelection(a.start(),a.soffset(),a.finish(),a.foffset())})});Jc.expand(c);a=a.run(b,c,d);return Pg.inlineFrom(a)},nc.language()),cleanup:cf.clean,cleanupNonEmpty:cf.cleanNonEmpty,isTransient:cf.isTransient,withinTransient:cf.withinTransient},Gm={multiple:function(a,b,c,d){return function(e,g){Yc.runTransaction(a,
b,function(){k.each(g,c)},d)}},single:function(a,b,c,d){return function(e,g){Yc.runTransaction(a,b,function(){c(g)},d)}}},zG=function(a,b,c){c=a.create().nu(c);var d=a.property().children(b);a.insert().append(b,c);a.insert().appendAll(c,d);return c},AG=function(a,b,c){c=a.create().nu(c);var d=a.property().children(b);a.attrs().copyTo(b,c);a.insert().before(b,c);a.insert().appendAll(c,d);a.remove().remove(b);return c},Gv={blockTransform:function(a,b,c){return(Yb.isFormatting(a,b)?AG:zG)(a,b,c)},nest:zG,
replace:AG},JP={tags:f.constant("address article aside audio blockquote canvas dd div dl fieldset figcaption figure footer form h1 h2 h3 h4 h5 h6 header hgroup hr li noscript ol output p pre section table tbody td tfoot th thead tr ul video".split(" "))},H4=Sa(),I4=Ev(H4),BG={query:function(a,b,c){return I4.query(a,b,c)},apply:function(a,b,c,d,e,g){return Rg.applyOn(a,QE,b,c,d,e,function(a){return k.map(a,function(a){return Gv.blockTransform(Sa(),a,g)})})}},ch={background:function(a,b,c){Wd.style(a,
{"background-color":b},Jc.expand,c)},addClass:function(a,b,c){Wd.generic(a,Jc.expand,c,function(a){n.add(a,b)})},setClass:function(a,b,c){Wd.generic(a,Jc.expand,c,function(a){m.set(a,"class",b)})},foreground:function(a,b,c){Wd.style(a,{color:b},Jc.expand,c)},addAttribute:function(a){return function(b,c,d){Wd.generic(b,Jc.expand,d,function(b){m.set(b,a,c)})}},face:function(a,b,c){Wd.style(a,{"font-family":b},Jc.expand,c)},size:function(a,b,c){Wd.style(a,{"font-size":b},Jc.expand,c)}},CG=function(a,
b,c,d,e){b=function(b,d,e,g){return a(c.isRoot,b,d,e,g)};var g=am.multiple(c,d,b),f=am.single(c,d,b,Ki,Ji);return tb.cata(e.get(),h.none,function(a,b){return h.some(g(a,b))},function(a,b){return h.some(f(a,b))})},ig={transform:function(a,b,c,d,e){return CG(function(a,b,c,d,f){return BG.apply(a,b,c,d,f,e)},a,b,c,d)},setClass:function(a,b,c,d,e,g){var l=function(a){J.closest(a,e).each(function(a){Yc.runTransaction(b,c,function(){m.set(a,"class",g)},b.toEditing)})};tb.cata(d.get(),f.noop,function(a,
b){k.each(a,l)},function(a,b){l(a.startContainer())})},inline:function(a,b,c,d,e){a=function(a){ch.setClass(a,e,De.isAnnotation)};tb.cata(d.get(),f.noop,Gm.multiple(b,c,a,b.toEditing),Gm.single(b,c,a,b.toEditing))},list:function(a,b,c,d,e){return CG(a,b,c,d,e)}},DG=C.immutable("label","items"),dh=function(a,b){return 0<b.length?[DG(h.some(u.key(a)),b)]:[]},Iv={groups:function(a,b,c){var d=a.divisions(),e=k.map(d.blocks().concat(d.classblocks()),a.toCandidate),g=k.map(d.inlines(),a.toCandidate);a=
k.map(d.objects(),a.toCandidate);a=k.foldr(a,function(a,d){var e;eb.closest(b,d.base(),c)?(e=d.base(),e=k.contains(["a"],e)?Z.merge(a,{link:a.link.concat([d])}):k.contains(["embed","img","object"],e)?Z.merge(a,{media:a.media.concat([d])}):k.contains(["hr"],e)?Z.merge(a,{line:a.line.concat([d])}):k.contains(["li","ol","ul"],e)?Z.merge(a,{list:a.list.concat([d])}):k.contains(["table","tr","td","th"],e)?Z.merge(a,{table:a.table.concat([d])}):a):e=a;return e},{link:[],media:[],line:[],list:[],table:[]});
return k.flatten([[DG(h.none(),e)],dh("block.label.inline",g),dh("block.label.link",a.link),dh("block.label.media",a.media),dh("block.label.line",a.line),dh("block.label.list",a.list),dh("block.label.table",a.table)])}},Hm=function(a,b){return{fold:b}},Ce={inline:function(a,b,c){return Hm("inline",function(d,e,g,f){return d(a,b,c)})},object:function(a,b,c,d){return Hm("object",function(e,g,f,h){return g(a,b,c,d)})},block:function(a,b,c){return Hm("block",function(d,e,g,f){return g(a,b,c)})},classblock:function(a,
b,c,d){return Hm("classed block",function(e,g,f,h){return h(a,b,c,d)})},cata:function(a,b,c,d,e){return a.fold(b,c,d,e)}},EG="h1 h2 h3 h4 h5 h6 div p pre".split(" "),J4="a embed hr img li object ol table td tr ul th".split(" "),K4=function(a){var b=a.indexOf("."),c;-1<b?(c=a.substring(0,b),a=a.substring(b+1),c=-1===a.indexOf(".")?h.some({tag:f.constant(c),clazz:f.constant(a)}):h.none()):c=h.none();return c},M4=function(a,b){var c=a.tag(),d=a.clazz(),e=k.contains(EG,c),g=k.contains(J4,c);return(e?
H.value(Ce.classblock):g?H.value(Ce.object):H.error("Rule must have object tag or block tag.")).map(function(g){var f=L4(b,a,e);return g(c,d,f,b.style)})},N4=function(a){return a.text.getOrThunk(function(){return u.key("block."+a.name)})},O4=function(a,b){return a.text.getOrThunk(function(){return u.pkey("tag.inline.class",[b])})},L4=function(a,b,c){return a.text.getOrThunk(function(){var a=(c?"block":"tag")+"."+b.tag(),a=(u.has(a)?u.translate(a):b.tag())+" ("+b.clazz()+")";return u.raw(a)})},LP=
{decipher:function(a){return K4(a.name).fold(function(){return k.contains(EG,a.name)?H.value(Ce.block(a.name,N4(a),a.style)):H.error("Invalid style name: "+a.name)},function(b){return 0<b.tag().length?M4(b,a):H.value(Ce.inline(b.clazz(),O4(a,b.clazz()),a.style))})}},P4={text:function(a){var b=u.get(a.textkey()),c=u.cata(a.textkey(),f.constant(!0),f.constant(!1),f.constant(!1));a=c&&void 0===b?a.value():b;return{text:f.constant(a),unknown:f.constant(c)}}},Hv={format:function(a){var b=q.fromText(u.get(a.textkey())),
c=P.div();n.add(c,x.resolve("format-element-"+a.tag()));t.setAll(c,a.styles());a=P.div();n.add(a,x.resolve("format-element-container"));v.append(c,b);v.append(a,c);return a},face:function(a){var b=P4.text(a),c=b.text(),c=qc(c).element();t.setAll(c,{"font-family":a.value()});a=b.unknown()?[x.resolve("font-face-unknown")]:[];ea.add(c,a);return c}},OP=function(a,b){var c=q.fromTag(a);Q.hidden(c,!0);t.set(c,"list-style-type",b);for(var d=x.resolve("list-style-content"),e=0;3>e;e++){var g=q.fromTag("li"),
f=P.span();n.add(f,d);v.append(g,f);v.append(c,g)}return c},NP=function(a,b){var c=u.translate(b),d=q.fromText(c),e=P.span();n.add(e,x.resolve("list-style-default"));m.set(a,"data-"+x.resolve("list-style"),"default");m.set(a,"title",c);v.append(e,d);return e},ck="data-"+x.resolve("list-style"),Q4=ma.generate([{mine:["element"]},{other:["element"]},{paragraphs:["elements"]}]),Kh=Z.merge(Q4,{cata:function(a,b,c,d){return a.fold(b,c,d)}}),R4=function(a,b,c){var d=A.name(a);return t.getRaw(a,"list-style-type").fold(function(){return d===
b&&c.isNone()},function(a){return c.exists(function(c){return d===b&&a===c})})},RP={categorise:function(a,b,c){a=k.groupBy(a,function(a){return fa.isList(a)?!0:y.parent(a).map(function(a){return a.dom()}).getOr(!1)});return k.bind(a,function(a){return k.forall(a,fa.isList)?k.map(a,function(a){return R4(a,b,c)?Kh.mine(a):Kh.other(a)}):[Kh.paragraphs(a)]})}},FG=function(a){return Pb.cata(a,function(a){return dd.wrap(a)},function(a){return[a]},function(a,c){return fa.isList(a)?[a]:k.bind(c,FG)},function(a){return dd.wrap([a])})},
S4={isRootContainer:function(a){return function(b){return a(b)||fa.isContainer(b)&&"li"!==A.name(b)}},extract:FG,isCapable:fa.isBlock,canUse:f.constant(!1),quasi:function(){return h.some(Ri.nu(Sa()))},ascend:function(a,b){return J.closest(a,"ol,ul",b).getOr(a)}},T4=function(a,b,c){var d=q.fromTag(b);c.each(function(a){t.set(d,"list-style-type",a)});return rm.list(a,d)},Vf={hasSameType:function(a,b){return A.name(a)===b},hasSameStyle:function(a,b){var c=t.getRaw(a,"list-style-type");return b.fold(function(){return c.isNone()},
function(a){return c.is(a)})},convertOr:function(a,b,c,d,e){return function(g){return d(g.list(),a)&&e(g.list(),b)?c(g):T4(g,a,b)}},topContext:function(a,b,c){return Sg.context(a,b,Sg.topmost,UE,c)},context:function(a,b,c){return Sg.context(a,b,Sg.immediate,UE,c)},unapply:function(a){return rm.level(a,-1*a.level())}},bj=Sa(),U4=Ev(bj),V4=function(a){a=k.map(a,function(a){return J.ancestor(a,"ul,ol")});var b=k.map(a,function(a){return a.fold(function(){return"-"},function(a){return A.name(a)+"-"+t.get(a,
"list-style-type")})}),c=b[0];return k.forall(b.slice(1),function(a){return a===c})?a[0]:h.none()},GG=function(a,b,c){var d=QP(bj,a,c),e=function(a,b,c,e,g,f){return Rg.applyOn(a,S4,b,c,e,g,function(b){return d.apply(a,f,b)})},g=function(b,d,e){b=b.selected();return 0<b.length&&V4(b).map(function(b){return d(b,a)&&e(b,c)}).getOr(!1)};return{query:function(c){return U4.queryAll(c.startContainer(),c.endContainer(),["ul","ol"],b).exists(function(b){return b===a})},queryStyle:function(a){var d=c.fold(function(){return":not([style])"},
function(a){return'[style*="list-style-type: '+a+';"]'});return bj.up().closest(a.startContainer(),"ol, ul",b).exists(function(c){return bj.up().closest(a.endContainer(),"ol, ul",b).exists(function(a){return c.dom()===a.dom()?bj.is(c,d):!1})})},apply:function(b,d,h,k,m,n,q){var s=q?f.constant(!0):Vf.hasSameStyle,t=function(){return e(b,d,h,k,m,n)},u=function(b,e){return $E.executeUsing(b,d,h,k,m,function(){return Vf.convertOr(a,c,e,Vf.hasSameType,s)})},v=function(){return Vf.topContext(d,k,b).fold(t,
function(a){return u(a,Vf.unapply)})};return Vf.context(d,k,b).fold(t,function(a){return g(a,Vf.hasSameType,s)?v():u(a,f.identity)})}}},dk={unordered:f.curry(GG,"ul"),ordered:f.curry(GG,"ol")},HG=function(a,b,c,d){return function(c){var g=d(c.content().isRoot,h.none());return{toggle:!0,value:a,textkey:u.key(b),icon:!0,tooltip:Wa.get,updateGeneral:function(a){c.content().getSelection().each(function(b){(g.query(b)?oa.on(!0):oa.off()).fold(a.deselect,f.noop,a.select)})},action:function(){ig.list(function(a,
b,d,f,h){return g.apply(a,b,d,f,h,so(c.region()))},c.region(),c.content(),c.undo(),c.selections());c.content().focus()}}}},W4=HG("bullist","action.bullist","ul",dk.unordered),Jv={ol:HG("numlist","action.numlist","ol",dk.ordered),ul:W4},X4=C.immutable("tag"),IG=function(a,b){return k.contains(["label","li","td","th","normal"],b)?oa.on(a.backup(b)):oa.off()},Y4=function(a,b,c){return BG.query(a,b.selectors(),c).fold(function(){return a.collapsed()?IG(b,"normal"):oa.off()},function(a){return b.candidateByName(a).map(oa.on).getOrThunk(function(){return IG(b,
a)})})},UP={read:function(a,b){var c=a.getSelection().fold(function(){return oa.off()},function(c){return Y4(c,b,a.isRoot)});return X4(c)}},TP=Y.detect(),JG={resolve:lb.create("ephox-arcade").resolve},vs="data-"+JG.resolve("no-merge"),ws="data-"+JG.resolve("atomic"),Ee={doNotMerge:function(a){A.isElement(a)&&m.set(a,vs,"true")},shouldMerge:function(a){return A.isElement(a)&&!1===m.has(a,vs)},setAtomic:function(a){A.isElement(a)&&m.set(a,ws,"true")},isAtomic:function(a){return A.isElement(a)&&"true"===
m.get(a,ws)},clear:function(a){A.isElement(a)&&k.each([vs,ws],function(b){m.remove(a,b)})}},Z4=function(a){var b=y.owner(a);a=X.descendants(a,"td,th");k.each(a,function(a){re.needsCursor(a).each(function(d){v.append(a,q.fromTag("br",b.dom()))})})},$4=function(a){re.vacate(a);re.needsCursor(a).each(function(b){B.remove(a)})},a5=function(a){("table"===A.name(a)?Z4:$4)(a)},KG=function(a,b){var c=b5(a),c=k.filter(c,b);k.each(c,a5)},b5=function(a){var b=function(a){return[a]},c=y.prevSibling(a).map(b).getOr([]),
b=y.nextSibling(a).map(b).getOr([]);return c.concat([a]).concat(b)},eh={purge:function(a){KG(a,fa.isBlock)},purgeBy:KG},LG=function(a){eh.purgeBy(a,function(a){var c=fa.isBlock(a)||"li"===A.name(a)||"ol"===A.name(a)||"ul"===A.name(a);if("li"===A.name(a)){var d;if(!(d=0===X.descendants(a,"ul,ol").length)){d=X.descendants(a,"ul,ol,li");var e=X.descendants(a,"*");a=pc.descendants(a,A.isText);d=d.length===e.length&&0===a.length}a=d}else a=!0;return c&&a})},xs=function(a){var b=X.descendants(a,"ol,ul,li");
k.each([a].concat(b),LG)},MG=function(a){a=("li"===A.name(a)?[a]:[]).concat(X.descendants(a,"li"));k.each(a,function(a){var c=y.children(a);k.exists(c,function(a){return A.isText(a)?0<Da.get(a).length:!k.contains(["ol","ul"],A.name(a))})||t.set(a,"list-style-type","none")})},Im=function(a,b){var c=q.fromText(Ua.zeroWidth());a(b,c);return h.some({element:f.constant(c),cleanup:function(){B.remove(c)},splits:f.constant([])})},fh=function(a,b){return a.map(function(a){return{element:f.constant(a),cleanup:f.noop,
splits:f.constant(b)}})},NG=function(a,b){return Im(v.before,a)},OG=function(a,b){return Im(v.after,a)},ys=function(a,b){return Im(v.append,a)},zs=function(a,b){return Im(v.prepend,a)},PG=function(a,b){return function(c,d){return a(c,d).fold(function(){return b(c,d)},h.some)}},As=function(a,b,c){return fh(h.some(b),void 0!==b&&void 0!==c?[{first:b.dom(),second:c.dom()}]:[])},QG={before:{text:{none:h.none,start:PG(function(a,b){return fh(y.prevSibling(a),[])},NG),middle:As,end:As},element:{hasPrevious:function(a,
b){return fh(y.child(a,b-1),[])},rightEdge:ys,first:zs,empty:NG,otherwise:zs}},after:{text:{none:h.none,start:As,middle:function(a,b,c){return fh(h.some(c),void 0!==b&&void 0!==c?[{first:b.dom(),second:c.dom()}]:[])},end:PG(function(a,b){return fh(y.nextSibling(a),[])},OG)},element:{hasNext:function(a,b){return fh(y.child(a,b+1),[])},rightEdge:ys,first:zs,empty:OG,otherwise:ys}}},Jm={prefix:f.curry(xm.prefix,QG),postfix:f.curry(xm.postfix,QG)},c5=function(a){return!Gc.descendant(a,function(a){return Ng.hasCursor(a,
["hr","img"])})},d5=function(a){var b=ec.shallowAs(a,"p");v.before(a,b);B.remove(a);return h.some(b)},e5=function(a){var b=a.second(),b=(yb.isHeading(b)&&c5(b)?d5(b):h.none()).getOr(a.second());return{first:a.first,second:f.constant(b),mark:a.mark,splits:a.splits}},f5=function(a){return A.isElement(a)&&fa.isInline(a)&&0===y.children(a).length},g5=function(a){var b=q.fromText(Ua.zeroWidth());v.append(a,b);return b},RG=function(a,b,c){return Jm.prefix(b,c).bind(function(b){var c=gm.breakPath(b.element(),
yb.stopAt(a));return c.second().map(function(g){return{first:f.constant(a),second:f.constant(g),mark:f.constant(b),splits:f.constant(c.splits().concat(b.splits()))}})})},Id={soft:function(a,b,c){var d=function(c){var e;A.isText(c)?(e=Da.get(c),e=-1===e.indexOf("\u00a0")&&0===e.trim().length):e=!1;return e?y.nextSibling(c).forall(d):!ka.isStart(a,b)&&fa.isBlock(c)},e=q.fromTag("br");Zj.atStartOf(a,b,e);y.nextSibling(e).fold(function(){return ka.getEnd(a)===b&&yb.findBlock(a,c).bind(Cb.last).fold(f.constant(!1),
function(b){var c="br"===A.name(b)?e:a;return L.eq(c,b)})},d)&&v.after(e,q.fromTag("br"));return h.some({start:f.constant(da.after(e)),finish:f.constant(da.after(e))})},hard:function(a,b,c,d,e){return RG(a,b,c).map(e5).map(function(a){k.each([d.getOr(a.first()),a.second()],function(b){re.postprocess(b,a.mark().element(),e)});yb.findValidHardBreak(a.second(),e).orThunk(function(){Ir.rootBr(a.second(),a.mark().element())});var b=Jr.plunge(a.second(),f.constant(!1)),b=f5(b)?g5(b):b;a.mark().cleanup();
return{start:f.constant(da.on(b,0)),finish:f.constant(da.on(b,0))}})},rupture:RG},SG=function(a){return h.some({start:f.constant(a),finish:f.constant(a)})},h5={doubleBr:function(a){v.after(a,q.fromTag("br"));var b=q.fromTag("br");v.after(a,b);return SG(da.after(b))},emptyParagraph:function(a){var b=q.fromTag("p"),c=q.fromTag("br");v.append(b,c);v.after(a,b);return SG(da.on(c,0))}},TG=function(a){return[a]},i5={processes:function(a){return{stopper:yb.stopAt(a),wrapper:TG,post:f.noop}}},k5=function(a,
b,c){h.from(c[c.length-1]).each(function(a){K.append(a,j5(y.children(b)));B.empty(b)})},j5=function(a){return k.filter(a,function(a){return!(A.isElement(a)&&yb.isList(a)&&0===y.children(a).length)})},l5={processes:function(a){return{stopper:yb.stopAt(a),wrapper:TG,post:k5}}},m5=function(a,b){B.remove(a);return b},n5=function(a,b){var c=q.fromTag("p");K.append(c,b);v.before(a,c);B.remove(a);re.needsCursor(c).each(function(a){Ir.forceAddBr(a,q.fromTag("span"))});return[c]},o5=function(a){var b=y.children(a),
b=(Gc.descendant(a,fa.isFormatting)?m5:n5)(a,b);k.each(b,function(b){A.isElement(b)&&(m.transfer(a,b,["dir"]),t.transfer(a,b,["text-align"]))});return b},p5={processes:function(a){return{stopper:yb.stopAt(a),wrapper:o5,post:f.noop}}},q5={exit:function(a,b){return J.ancestor(a,"ol,ul",b).bind(function(c){var d=J.ancestor(c,"li,ol,ul",b).fold(function(){return p5.processes(c)},function(a){return"li"===A.name(a)?l5.processes(a):i5.processes(c)}),e=gm.breakPath(a,d.stopper);return e.second().map(function(b){var c=
e.first(),h=d.wrapper(a);K.before(b,h);d.post(c,b,h);k.each([c,b],re.lists);return{start:f.constant(da.on(h[0],0)),finish:f.constant(da.on(h[0],0))}})})}},r5=function(a,b,c,d){return J.closest(b,"li",d).fold(function(){return Id.hard(a,b,c,h.none(),d)},function(a){a=yb.exitList(a)?q5.exit(a,d):Id.hard(a,b,c,h.some(a),d);return a})},Bs={hard:function(a,b,c){return yb.findValidHardBreak(a,c).fold(function(){return yb.findAnchorBreak(a,c).fold(function(){return Id.soft(a,b,c)},function(d){return!0===
yb.isRightEdge(d,a,b)?h5.doubleBr(d):Id.hard(d,a,b,h.none(),c)})},function(d){return r5(d,a,b,c)})},soft:Id.soft,rupture:Id.rupture},Cs=C.immutable("cursor","blocks"),Km=function(a,b,c,d){return Jm.postfix(a,b,c,d).map(function(e){var g=Tf.leftEdge(a,b,c,d);Tf.makeTo(g.element(),g.offset(),e.element()).deleteContents();return{start:f.constant(da.before(e.element())),finish:f.constant(da.before(e.element())),mark:e.element}})},Ds=function(a,b,c,d){return Km(a,b,c,d).bind(function(a){return Id.soft(a.mark(),
0)})},s5=function(a,b,c,d,e){return Km(a,b,c,d).bind(function(a){return yb.findValidHardBreak(a.mark(),e).bind(function(b){return Id.hard(b,a.mark(),0,h.none(),e)})})},t5=function(a,b){k.each(a,function(a){re.postprocess(a,b)})},Es={soft:Ds,hard:function(a,b,c,d,e){var g=yb.findValidHardBreak(a,e),l=yb.findValidHardBreak(c,e),h=g.fold(function(){var e=Ds(a,b,c,d);return Cs(e,[])},function(g){return l.fold(function(){var e=Ds(a,b,c,d);return Cs(e,[g])},function(f){var l=L.eq(g,f)?s5(a,b,c,d,e):Km(a,
b,c,d);return Cs(l,[g,f])})});return h.cursor().map(function(a){var b=h.blocks(),c;c=a.start().fold(f.identity,f.identity,f.identity);t5(b,c);return a})},rupture:function(a,b,c,d,e){return Km(a,b,c,d).bind(function(a){return yb.findValidHardBreak(a.mark(),e).bind(function(b){return Id.rupture(b,a.mark(),0)})})}},Fs={point:function(a,b,c){return Bs.rupture(a,b,c)},range:function(a,b,c,d,e){return Es.rupture(a,b,c,d,e)},ruptureUntil:function(a,b,c,d,e,g){var f=Jm.postfix(a,b,c,d);return function(a,
b){return yb.findValidHardBreak(a,e).map(g).bind(function(c){return Id.rupture(c,a,b)})}(a,b).bind(function(a){return f.map(function(b){var c=document.createRange();c.setStartAfter(a.first().dom());c.setEndBefore(b.element().dom());c.deleteContents();b.cleanup();a.mark().cleanup();return a.first()})})}},cj=function(a,b,c){if(0<a.length){var d=a[0],e=a.slice(1);Zj.atStartOf(b,c,d);K.after(d,e);return h.some(a[a.length-1])}return h.none()},Kc={split:function(a,b,c){var d=void 0===c?eh.purge:c;return Fs.point(a.block(),
a.element(),a.offset()).bind(function(a){var c=b.all();K.before(a.second(),c);a.mark().cleanup();d(a.first());d(a.second());k.each([a.first(),a.second()],MG);a=k.reverse(c);return k.find(a,function(a){return y.parent(a).isSome()})})},merge:function(a,b){var c=y.children(b.single());return cj(c,a.element(),a.offset())},nest:function(a,b){return cj(b.all(),a.element(),a.offset())},insertMany:cj,insertChildren:function(a,b){var c=y.children(a);return cj(c,b.element(),b.offset())},fallback:function(a,
b,c){return cj(c,b.startContainer(),b.startOffset())}},UG={lists:function(a){return k.foldr(a,function(a,c){return a.bind(function(a){return J.closest(c,"ol,ul").map(function(b){return[b].concat(a)})})},h.some([])).map(function(a){return{all:f.constant(a)}})},from:function(a){return{all:f.constant(a)}}},VG=C.immutable("left","right"),u5=function(a){return fa.isFormatting(a)||fa.isContainer(a)},WG=function(a,b,c,d){return sa.closest(c,a,b)},XG=function(a){return function(b,c,d){return a(b,c.element(),
c.offset()).bind(function(c){return a(b,d.element(),d.offset()).map(function(a){return VG(c,a)})})}},YG=function(a){var b=XG(a);return function(a,d,e){return b(a,d,e).filter(function(a){return L.eq(a.left(),a.right())})}},Ia={inline:function(a,b,c){return sa.closest(b,u5,a).fold(function(){return h.some(b)},function(c){return a(c)?h.some(b):h.none()})},formatter:f.curry(WG,fa.isFormatting),container:function(a,b,c){return sa.closest(b,fa.isContainer,a).filter(f.not(a))},inListItem:function(a,b,c){return J.closest(b,
"li",a)},inList:function(a,b,c){return J.closest(b,"ol,ul",a)},inTableCell:function(a,b,c){return J.closest(b,"td,th",a)},setBlock:function(a,b){return Z.deepMerge(a,{block:f.constant(b)})},both:XG,shared:YG,sharedPredicate:function(a){a=f.curry(WG,a);return YG(a)},entireRoot:function(a,b,c){return sa.closest(b.element(),a).filter(function(c){var e=function(a){return L.eq(a,b.element())};return a(b.element())&&0===b.offset()||A.isText(b.element())&&0===b.offset()&&Cb.first(c).map(e).getOr(!1)}).filter(function(b){var e=
function(a){return L.eq(a,c.element())};return a(c.element())&&c.offset()>=y.children(b).length-1||A.isText(c.element())&&c.offset()>=Da.get(c.element()).length&&Cb.last(b).map(e).getOr(!1)}).map(function(a){return VG(a,a)})}},ZG=function(a){return function(b){return(1===b.length?h.some(b[0]):h.none()).filter(a).map(function(a){return{single:f.constant(a)}})}},$G=function(a){return function(b){return(k.exists(b,a)?h.some(b):h.none()).map(function(a){return{all:f.constant(a)}})}},zb={oneOf:ZG,anyOf:$G,
noneOf:function(a){return function(b){return(k.exists(b,a)?h.none():h.some(b)).map(function(a){return{all:f.constant(a)}})}},allOf:function(a){return function(b){return k.forall(b,a)?h.some({all:f.constant(b)}):h.none()}},any:h.some,oneOfTag:function(a){return ZG(function(b){return A.name(b)===a})},anyOfTag:function(a){return $G(function(b){return A.name(b)===a})},single:function(a){return h.some({single:f.constant(a)})},all:function(a){return h.some({all:f.constant(a)})}},Lm=function(a){return 1===
a.length?J.closest(a[0],"ol,ul").map(function(a){return{single:f.constant(a)}}):h.none()},aH=function(a){return Lm(a).bind(function(a){a=X.descendants(a.single(),"li");return 1===a.length?h.some({single:f.constant(a[0])}):h.none()})},bH=function(a){if(2!==a.length)return h.none();var b=a[0],c=a[1];return fa.isFormatting(b)||fa.isContainer(b)||k.contains(["ol","ul","li"],A.name(b))||!k.contains(["ol","ul"],A.name(c))?h.none():h.some({all:f.constant(a),list:f.constant(c),inline:f.constant(b)})},gh=
Ia.formatter,cH=Ia.inline,v5=Ia.container,dH=function(a){return fa.isFormatting(a)&&!1===Ee.shouldMerge(a)},w5=[Qa(Ia.inListItem,Lm,function(a,b){var c=X.children(b.single(),"li");return 1!==c.length||eb.descendant(c[0],"ol,ul")?Kc.split(a,UG.from(c),LG):Kc.insertChildren(c[0],a)},"List"),Qa(Ia.inListItem,UG.lists,function(a,b){return J.ancestor(a.block(),"ol,ul",a.isRoot).bind(function(c){c=Ia.setBlock(a,c);return Kc.split(c,b,xs)})},"Many lists"),Qa(cH,zb.anyOf(dH),Kc.nest,"Copying formatting (NO MERGE) into root"),
Qa(cH,zb.oneOf(fa.isFormatting),Kc.merge,"Copying formatting into root"),Qa(gh,zb.anyOfTag("hr"),Kc.split,"insert hr into formatter"),Qa(gh,zb.anyOf(dH),Kc.split,"Copying one formatting block (NO MERGE) into formatting"),Qa(gh,zb.oneOf(fa.isFormatting),Kc.merge,"Copying one formatting block into formatting"),Qa(gh,zb.anyOf(fa.isFormatting),Kc.split,"Copying many formatting blocks into formatting"),Qa(gh,zb.anyOfTag("table"),function(a){return function(b,c){var d=c.all(),e=a(b,c);h.from(d[d.length-
1]).each(function(a){y.nextSibling(a).fold(function(){var b=q.fromTag("p");v.append(b,q.fromTag("br"));v.after(a,b)},f.noop)});return e}}(Kc.split),"Copying tables into formatting"),Qa(gh,zb.anyOf(fa.isContainer),Kc.split,"Copying container blocks into formatting"),Qa(v5,zb.all,Kc.nest,"Fallback pasting into container.")],x5={context:function(a,b,c){return b.context(a.isRoot,c.startContainer(),c.startOffset()).map(function(b){return{block:f.constant(b),element:c.startContainer,offset:c.startOffset,
isRoot:a.isRoot}})},rules:f.constant(w5),fallback:Kc.fallback},y5={force:function(a){var b=Bc.after(a.startContainer(),a.startOffset(),a.endContainer(),a.endOffset()),c=b?S.point(a.endContainer(),a.endOffset()):S.point(a.startContainer(),a.startOffset());a=b?S.point(a.startContainer(),a.startOffset()):S.point(a.endContainer(),a.endOffset());return{left:f.constant(c),right:f.constant(a)}}},z5=function(a,b){var c=a.original(),d=Jm.postfix(c.left().element(),c.left().offset(),c.right().element(),c.right().offset()),
e=b.all();return d.bind(function(b){Tf.makeTo(c.left().element(),c.left().offset(),b.element()).deleteContents();b.cleanup();K.append(a.left(),e);b=y.children(a.right());K.append(a.left(),b);B.remove(a.right());return h.from(e[e.length-1])})},eH=function(a,b){if(!L.eq(a.left(),a.right()))throw"replaceContents used in the wrong context";var c=a.left();B.empty(c);var d=b.all();K.append(c,d);return Cb.last(c).or(h.from(d[d.length-1]))},Lc={replace:function(a,b){if(L.eq(a.left(),a.right())){var c=b.all();
a.replaceSelection(c);return h.from(c[c.length-1])}return z5(a,b)},replaceContents:eH,replaceContentsWrapP:function(a,b){var c=q.fromTag("p");K.append(c,b.all());return eH(a,{all:f.constant([c])})},replaceWhole:function(a,b){var c=b.all();K.after(a.left(),c);B.remove(a.left());return h.from(c[c.length-1])},merge:function(a,b){var c=y.children(b.single());a.replaceSelection(c);return h.from(c[c.length-1])},split:function(a,b,c){var d=void 0===c?eh.purge:c;c=a.original().left();var e=a.original().right();
return Fs.range(c.element(),c.offset(),e.element(),e.offset(),a.isRoot).bind(function(a){var c=b.all();K.before(a.second(),c);a.mark().cleanup();k.each([a.first(),a.second()],d);return h.from(c[c.length-1])})},fallback:function(a,b,c){a.replaceSelection(c);h.from(c[0]).each(eh.purge);h.from(c[c.length-1]).each(eh.purge);return h.from(c[c.length-1])}},fH=function(a,b,c,d){d=void 0!==d?d:f.identity;var e=b.original(),g=e.left(),e=e.right();return Fs.ruptureUntil(g.element(),g.offset(),e.element(),e.offset(),
b.isRoot,d).bind(function(b){var d=a(b,c);A5(b);return d},void 0!==d?d:f.identity)},A5=function(a){var b=X.ancestors(a,"ol,ul");0<b.length&&k.each([xs,MG],function(a){a(b[b.length-1])})},Mm=function(a,b,c){return fH(a,b,c,f.identity)},dj=Ia.formatter,B5=f.curry(Mm,function(a,b){v.after(a,b.single());return h.some(b.single())}),gH=f.curry(function(a,b,c){return fH(a,b,c,function(a){return y.parent(a).getOr(a)})},function(a,b){var c=b.all();K.after(a,c);return h.from(c[c.length-1])}),C5=f.curry(Mm,
function(a,b){var c=y.children(b.single());K.after(a,c);return h.some(c[c.length-1])}),D5=f.curry(Mm,function(a,b){var c=q.fromTag("li");K.append(c,[b.inline(),b.list()]);v.after(a,c);return h.some(b.list())}),E5=f.curry(Mm,function(a,b){var c=q.fromTag("li");K.append(c,[b.inline(),b.list()]);var d=J.ancestor(a,"ol,ul").fold(function(){var a=q.fromTag(A.name(b.list()));v.append(a,c);return a},function(){return c});v.after(a,d);return h.some(b.list())}),F5=[Qa(Ia.sharedPredicate(Ee.isAtomic),zb.all,
Lc.replaceWhole,"selection: inside something atomic, so just replace the whole thing"),Qa(Ia.shared(Ia.inListItem),aH,function(a,b){return Lc.merge(a,b)},"selection: same item, paste: singleton"),Qa(Ia.shared(Ia.inListItem),Lm,function(a,b){return Lc.split(a,{all:f.constant(y.children(b.single()))},xs)},"selection: same item, paste: single list"),Qa(Ia.shared(Ia.inListItem),bH,E5,"selection: same item, paste: partial"),Qa(Ia.shared(Ia.inListItem),zb.noneOf(fa.isBlock),Lc.replace,"selection: same item, paste: no blocks"),
Qa(Ia.shared(Ia.inListItem),zb.anyOf(fa.isHeading),function(a,b){var c=Lc.replace(a,b);k.each(b.all(),function(a){eh.purgeBy(a,Ng.isVacantText)});return c},"selection: same item, paste: heading"),Qa(Ia.shared(Ia.inListItem),zb.all,gH,"selection: same item, paste: any"),Qa(Ia.both(Ia.inListItem),aH,B5,"selection: diff items, paste: singleton"),Qa(Ia.both(Ia.inListItem),Lm,C5,"selection: diff items, paste: single list"),Qa(Ia.both(Ia.inListItem),bH,D5,"selection: diff items, paste: partial"),Qa(Ia.both(Ia.inListItem),
zb.all,gH,"selection: diff items, paste: any"),Qa(Ia.both(dj),zb.anyOfTag("hr"),Lc.split,"selection: inside formatter, paste: split"),Qa(Ia.both(dj),zb.noneOf(fa.isBlock),Lc.replace,"range no blocks"),Qa(Ia.both(dj),zb.anyOf(function(a){return fa.isFormatting(a)&&!1===Ee.shouldMerge(a)}),Lc.split,"range 1 formatting block"),Qa(Ia.both(dj),zb.oneOf(fa.isFormatting),Lc.merge,"range 1 formatting block"),Qa(Ia.both(dj),zb.all,Lc.split,"range anything > formatters"),Qa(Ia.entireRoot,zb.noneOf(fa.isBlock),
Lc.replaceContentsWrapP,"complete body, not inserting a block: wrap in P and replace the contents"),Qa(Ia.entireRoot,zb.all,Lc.replaceContents,"complete body: replace the contents")],G5={context:function(a,b,c){var d=y5.force(c);return b.context(a.isRoot,d.left(),d.right()).map(function(b){return{left:b.left,right:b.right,original:f.constant(d),isRoot:a.isRoot,replaceSelection:a.replaceSelection}})},rules:f.constant(F5),fallback:Lc.fallback},H5=function(a,b,c,d,e){return k.foldl(a,function(a,l){return a.orThunk(function(){return d(b,
l,c).bind(function(a){return l.profile(e).map(function(b){return{action:l.action,target:f.constant(b),ctx:f.constant(a)}})})})},h.none())},I5=function(a){var b=ka.getEnd(a);return S.point(a,b)},hH=function(a,b,c,d,e,g){var f={getSelection:a,replaceSelection:b,isRoot:c};return f.getSelection().bind(function(a){var b=a.collapsed()?x5:G5,c=a.collapsed()?e.concat(b.rules()):g.concat(b.rules()),c=H5(c,f,a,b.context,d).fold(function(){return b.fallback(f,a,d)},function(a){return a.action(a.ctx(),a.target())});
return c.isNone()&&0===d.length?h.some(S.point(a.startContainer(),a.startOffset())):c.map(I5)})},J5={insertAtCursor:function(a,b,c,d){return hH(a,b,c,d,[],[])},insertAtCursorWith:hH},K5=function(a){return A.isElement(a)&&"false"===m.get(a,"contenteditable")},L5=function(a){return a().map(function(a){a=k.map([a.startContainer(),a.endContainer()],function(a){return sa.closest(a,K5)});return ab.cat(a)}).getOr([])},iH=function(a,b,c){k.each(a,b);b=c();k.each(a,Ee.clear);return b},M5=function(a,b){return iH(a,
Ee.setAtomic,b)},jH=function(a,b,c,d,e,g){var f=L5(a);return M5(f,function(){return J5.insertAtCursorWith(a,b,c,d,e,g)})},kH=function(a,b,c,d){b=jH(a.getSelection,a.replaceSelection,a.isRoot,b,c,d);a.domChanged();a.undoChanged();return b},lH=function(a,b){return kH(a,b,[],[])},Cc={insertAtCursorWith:kH,insertAtCursor:lH,insertAtCursorWithInstructions:function(a,b,c){return iH(c,a,function(){return lH(b,c)})},insertAt:function(a,b,c){c=jH(function(){return h.some(b)},function(c){a.win().each(function(a){var g=
Wb.relative(da.on(b.startContainer(),b.startOffset()),da.on(b.endContainer(),b.endOffset()));Fa.replace(a,g,c)})},a.isRoot,c,[],[]);a.domChanged();a.undoChanged();return c}},ek={generate:function(a){return a+"-"+Math.floor(99999*Math.random()).toString()}},N5=C.immutable("ch","offset"),mH=function(a,b){return N5(a.charAt(b),b)},Nm={previous:function(a,b){for(var c=b.getOr(a.length)-1;0<=c;c--)if(a.charAt(c)!==Ua.zeroWidth())return h.some(mH(a,c));return h.none()},next:function(a,b){for(var c=b.getOr(0)+
1;c<a.length;c++)if(a.charAt(c)!==Ua.zeroWidth())return h.some(mH(a,c));return h.none()},rfind:function(a,b){b.lastIndex=-1;var c=k.reverse(a).join(""),d=c.match(b);return void 0!==d&&null!==d&&0<=d.index?h.some(c.length-1-d.index):h.none()},lfind:function(a,b){b.lastIndex=-1;var c=a.match(b);return void 0!==c&&null!==c&&0<=c.index?h.some(c.index):h.none()}},O5=na.walkers().left(),P5=na.walkers().right(),Q5=ma.generate([{abort:[]},{kontinue:[]},{finish:["info"]}]),Gs=ma.generate([{aborted:[]},{edge:["element"]},
{success:["info"]}]),nH=function(a,b){return Yb.isEmptyTag(a,b)||a.property().isBoundary(b)},Hs=function(a,b,c,d,e,g,f){var k=function(){return f.fold(Gs.aborted,Gs.edge)},m=function(d){return na.walk(a,b,c,g).fold(k,function(b){return Hs(a,b.item(),b.mode(),h.none(),e,g,d)})};if(nH(a,b))return k();if(a.property().isText(b)){var n=a.property().getText(b);return e(a,Q5,b,n,d).fold(k,function(){return m(h.some(b))},Gs.success)}return m(f)},R5=function(a,b,c,d){c=mc.toLeaf(a,b,c);return a.property().isText(b)?
h.none():Ol.left(a,c.element(),a.property().isText,d).map(function(b){return S.point(b,a.property().getText(b).length)})},S5=function(a,b,c,d){c=mc.toLeaf(a,b,c);return a.property().isText(b)?h.none():Ol.right(a,c.element(),a.property().isText,d).map(function(a){return S.point(a,0)})},oH=function(a,b,c){var d=f.curry(nH,a);return R5(a,b,c,d).orThunk(function(){return S5(a,b,c,d)}).getOr(S.point(b,c))},pH={repeatLeft:function(a,b,c,d){b=oH(a,b,c);return Hs(a,b.element(),na.sidestep,h.some(b.offset()),
d,O5,h.none())},repeatRight:function(a,b,c,d){b=oH(a,b,c);return Hs(a,b.element(),na.sidestep,h.some(b.offset()),d,P5,h.none())}},qH=Gb.exactly(["regex","attempt"]),rH=function(a,b,c,d){return pH.repeatLeft(a,b,c,d)},sH=function(a,b,c,d){return pH.repeatRight(a,b,c,d)},tH=function(a,b,c){var d=f.constant(!1);if(!a.property().isText(b))return h.none();var e=a.property().getText(b);return c<=e.length?h.some(S.point(b,c)):na.seekRight(a,b,a.property().isText,d).bind(function(b){return tH(a,b,c-e.length)})},
Wf={previousChar:function(a,b){return Nm.previous(a,b)},nextChar:function(a,b){return Nm.next(a,b)},repeatLeft:rH,repeatRight:sH,expandLeft:function(a,b,c,d){var e=qH(d);return rH(a,b,c,function(a,b,c,d,f){a=f.getOr(d.length);return Nm.rfind(d.substring(0,a),e.regex()).fold(function(){return b.kontinue()},function(a){return e.attempt(b,c,d,a)})})},expandRight:function(a,b,c,d){var e=qH(d);return sH(a,b,c,function(a,b,c,d,f){var h=f.getOr(0);return Nm.lfind(d.substring(h),e.regex()).fold(function(){return b.kontinue()},
function(a){return e.attempt(b,c,d,h+a)})})},scanRight:tH},ej=Sa(),Xf={previousChar:function(a,b){return Wf.previousChar(a,b)},nextChar:function(a,b){return Wf.nextChar(a,b)},repeatLeft:function(a,b,c){return Wf.repeatLeft(ej,a,b,c)},repeatRight:function(a,b,c){return Wf.repeatRight(ej,a,b,c)},expandLeft:function(a,b,c){return Wf.expandLeft(ej,a,b,c)},expandRight:function(a,b,c){return Wf.expandRight(ej,a,b,c)},scanRight:function(a,b){return Wf.scanRight(ej,a,b)}},hh=C.immutable("before","after"),
Ks=function(a,b,c,d,e){return Om(c).fold(function(){var c=uH(a,b,d);return hh(c,e)},function(g){var f=ih(c,g);return f.length>a?hh(d,e):Is(b).fold(function(){var b=Js(a-f.length,g,e+f);return hh(d,b)},function(c){var h=ih(c,b);return h.length>a-f.length?hh(d,e+f):Ks(a-f.length-h.length,c,g,h+d,e+f)})})},vH=function(a,b,c,d){return a.finish(S.point(b,d))},wH={regex:function(){return RegExp(db.wordchar(),"i")},attempt:vH},xH={regex:function(){return RegExp(db.wordbreak(),"i")},attempt:vH},yH=function(a,
b){return a.offset()===b.offset()&&L.eq(a.element(),b.element())},zH=function(a){return S.point(a,A.value(a).length)},AH=function(a){return S.point(a,0)},Ls=function(a,b,c){var d=function(b){b=c(b);return yH(a,b)?h.none():h.some(b)};return b(a.element(),a.offset(),wH).fold(h.none,d,function(a){return b(a.element(),a.offset(),xH).fold(h.none,d,function(a){return h.some(a)})})},Om=function(a){return Ls(a,Xf.expandRight,zH)},Is=function(a){return Ls(a,Xf.expandLeft,AH)},Pm=function(a,b,c,d,e,g){return a(c).fold(f.constant(d),
function(f){var h=e(c,f);return h.length+d.length>b||0===h.length?d:Pm(a,b,f,g(d,h),e,g)})},Js=function(a,b,c){return Pm(Om,a,b,c,ih,function(a,b){return a+b})},uH=function(a,b,c){return Pm(Is,a,b,c,function(a,b){return ih(b,a)},function(a,b){return b+a})},ih=function(a,b){var c=y.defaultView(a.element()).dom(),d=Wb.exact(a.element(),a.offset(),b.element(),b.offset());return Fa.getAsString(c,d)},BH={beforeAfter:hh,anchorText:function(a,b){var c=Ja.freefallLtr(b);return Om(c).map(function(b){var e=
ih(c,b);return e+Js(a-e.length,b,"")}).getOr("")},flankingText:function(a,b){if(0===a)return hh("","");var c=Ja.freefallLtr(b),d=Ja.freefallRtl(b);return Ks(a,c,d,"","")},extendRightAndLeft:Ks,extendWordsSearch:Pm,extendWordsRight:Js,extendWordsLeft:uH,wordSearch:Ls,wordLeft:Is,wordRight:Om,pointEq:yH,boundaryPointRight:zH,boundaryPointLeft:AH,textBetweenPoints:ih,wordSeeker:wH,wordBreakSeeker:xH},CH=function(a,b){try{return a()}catch(c){if(12===c.code)return b();throw c;}},Ms=function(a){return J.closest(a,
"a[href]")},Ns=function(a,b){return Ms(a).bind(function(a){return Ms(b).filter(function(b){return L.eq(a,b)})})},T5=C.immutable("element","innerText","before","after"),Os=function(a,b,c,d){b=Qe.isEmpty(b)&&Qe.isEmpty(c)&&Qe.isEmpty(d)?"#"+m.get(a,"id"):b;return T5(a,b,c,d)},Ps=function(a){return ra.is(a,"a[href]")?[a]:ra.all("a[href]",a)},Uc={contentLink:function(a){return a.getSelection().bind(function(a){return ib.detect(a).fold(function(){return Ns(a.startContainer(),a.endContainer())},function(a){return h.some(a.image()).bind(Ms)})})},
sharedLink:Ns,sharedLinkOrClosest:function(a,b){var c=Ja.toLeaf(a.startContainer(),a.startOffset()),d=Ja.toLeaf(a.endContainer(),a.endOffset());return Ns(c.element(),d.element()).orThunk(function(){return J.closest(c.element(),"a[href]",b)})},findLinkTargets:function(a){return ra.all("h1,h2,h3,h4,h5,h6,div[id],p[id],span[id],a[id],a[name]",a)},linkTargetText:function(a,b){var c=pb.get(b).length,d=ec.deep(b),c=BH.anchorText(c,d);if(fa.isBlock(b))return Os(b,c,"","");d=BH.flankingText(Math.max(0,a-
c.length),b);return Os(b,c,d.before(),d.after())},linkTargetFastText:function(a,b){var c=pb.get(b),c=c.length>a?c.substr(0,a):c;return Os(b,c,"","")},all:Ps,many:function(a){return k.bind(a,Ps)},hrefUrl:function(a,b){return CH(function(){return ra.all('a[href="'+b+'"]',a)},function(){var c=Ps(a);return k.filter(c,function(a){return m.get(a,"href")===b})})},anchorTarget:function(a,b){return CH(function(){return ra.one("#"+b+',a[name="'+b+'"]',a)},function(){var c=y.owner(a).dom();return h.from(c.getElementById(b)).map(q.fromDom).orThunk(function(){var c=
ra.all("[name]",a);return k.find(c,function(a){return m.get(a,"id")===b||m.get(a,"name")===b})})})},getAnchorValue:function(a,b){var c=a.replace(/[^0-9a-zA-Z$_.+*(),-]/g,"");return 0<c.length?c:b}},Qm=["li"],DH=function(a,b){return Pb.cata(a,function(a){return!0===b?[]:dd.wrap(a)},function(a){return[a]},function(a,b){return"table"===A.name(a)?[a]:dd.handleContainer(Qm,a,b,DH)},function(a){return!0===b?[]:dd.wrap([a])})},U5={isRootContainer:hm.isRootOrContainer,extract:function(a){return DH(a,!1)},
isCapable:function(a){return Rh.isBlock(a)||k.contains(Qm,A.name(a))},canUse:function(a){return fa.isFormatting(a)||k.contains(Qm,A.name(a))},quasi:f.curry(dd.handleRoot,Qm),ascend:f.identity},V5=function(a,b){return J.closest(b,'[dir="rtl"],[dir="ltr"]').filter(a).isSome()},EH=function(a,b,c){var d=WP(b,c),e=function(a){d.set(a);t.reflow(a)},g=function(a){return rd.getDirection(a)===c};return{run:function(a,b,c,d,g){return Rg.applyOn(a,U5,b,c,d,g,function(a){k.each(a,e);return a})},query:function(a,
b){return eE.bothMatch(a,b,f.curry(V5,g))},name:f.constant(a)}},W5=EH("DirRtl","dir","rtl"),X5=EH("DirLtr","dir","ltr"),Rm={dirLtr:f.constant(X5),dirRtl:f.constant(W5)},FH=function(a){return function(b){var c=function(c,e,g,f){return a.run(b.content().isRoot,c,e,g,f)};tb.cata(b.selections().get(),f.noop,bm.multiple(b.content(),b.undo(),c),bm.single(b.content(),b.undo(),c,Ki,Ji));setTimeout(function(){b.content().uiChanged()},0)}},Lv={dirRtl:FH(Rm.dirRtl()),dirLtr:FH(Rm.dirLtr())},Y5=C.immutable("ltrdir",
"rtldir"),GH=function(a,b){return b.query(a.startContainer(),a.endContainer())?oa.on(!0):oa.off()},Z5=function(a){return J.closest(a.startContainer(),"[dir]").isSome()||J.closest(a.endContainer(),"[dir]").isSome()},Kv={readDir:function(a){return a.getSelection().filter(Z5).map(function(a){return Y5(GH(a,Rm.dirLtr()),GH(a,Rm.dirRtl()))})}},$5=ma.generate([{blob:["id","imageresult","objurl"]},{url:["id","url","raw"]}]),oc=Z.merge($5,{cata:function(a,b,c){return a.fold(b,c)}}),fc={create:Mv,clone:function(a){var b;
b=Mv(a.width,a.height);Ov(b).drawImage(a,0,0);return b},resize:Nv,get2dContext:Ov,get3dContext:function(a){var b=null;try{b=a.getContext("webgl")||a.getContext("experimental-webgl")}catch(c){}b||(b=null);return b}},og={getWidth:function(a){return a.naturalWidth||a.width},getHeight:function(a){return a.naturalHeight||a.height}},a6=function(){function a(a,b){return function(){a.apply(b,arguments)}}function b(a){var b=this;null===this._state?this._deferreds.push(a):k(function(){var c=b._state?a.onFulfilled:
a.onRejected;if(null===c)(b._state?a.resolve:a.reject)(b._value);else{var d;try{d=c(b._value)}catch(e){a.reject(e);return}a.resolve(d)}})}function c(b){try{if(b===this)throw new TypeError("A promise cannot be resolved with itself.");if(b&&("object"===typeof b||"function"===typeof b)){var g=b.then;if("function"===typeof g){f(a(g,b),a(c,this),a(d,this));return}}this._state=!0;this._value=b;e.call(this)}catch(h){d.call(this,h)}}function d(a){this._state=!1;this._value=a;e.call(this)}function e(){for(var a=
0,c=this._deferreds.length;a<c;a++)b.call(this,this._deferreds[a]);this._deferreds=null}function g(a,b,c,d){this.onFulfilled="function"===typeof a?a:null;this.onRejected="function"===typeof b?b:null;this.resolve=c;this.reject=d}function f(a,b,c){var d=!1;try{a(function(a){d||(d=!0,b(a))},function(a){d||(d=!0,c(a))})}catch(e){d||(d=!0,c(e))}}var h=function(b){if("object"!==typeof this)throw new TypeError("Promises must be constructed via new");if("function"!==typeof b)throw new TypeError("not a function");
this._value=this._state=null;this._deferreds=[];f(b,a(c,this),a(d,this))},k=h.immediateFn||"function"===typeof setImmediate&&setImmediate||function(a){setTimeout(a,1)},m=Array.isArray||function(a){return"[object Array]"===Object.prototype.toString.call(a)};h.prototype["catch"]=function(a){return this.then(null,a)};h.prototype.then=function(a,c){var d=this;return new h(function(e,f){b.call(d,new g(a,c,e,f))})};h.all=function(){var a=Array.prototype.slice.call(1===arguments.length&&m(arguments[0])?
arguments[0]:arguments);return new h(function(b,c){function d(g,f){try{if(f&&("object"===typeof f||"function"===typeof f)){var l=f.then;if("function"===typeof l){l.call(f,function(a){d(g,a)},c);return}}a[g]=f;0===--e&&b(a)}catch(h){c(h)}}if(0===a.length)return b([]);for(var e=a.length,g=0;g<a.length;g++)d(g,a[g])})};h.resolve=function(a){return a&&"object"===typeof a&&a.constructor===h?a:new h(function(b){b(a)})};h.reject=function(a){return new h(function(b,c){c(a)})};h.race=function(a){return new h(function(b,
c){for(var d=0,e=a.length;d<e;d++)a[d].then(b,c)})};return h},Xd=window.Promise?window.Promise:a6(),fk={atob:function(a){return Vb.getOrDie("atob")(a)},requestAnimationFrame:function(a){Vb.getOrDie("requestAnimationFrame")(a)}},Dc={blobToImage:Rv,imageToBlob:function(a){a=a.src;return 0===a.indexOf("data:")?zo(a):Sv(a)},blobToArrayBuffer:function(a){return new Xd(function(b){var c=new yo;c.onloadend=function(){b(c.result)};c.readAsArrayBuffer(a)})},blobToDataUri:Uv,blobToBase64:function(a){return Uv(a).then(function(a){return a.split(",")[1]})},
dataUriToBlobSync:Tv,canvasToBlob:function(a,b,c){b=b||"image/png";return HTMLCanvasElement.prototype.toBlob?new Xd(function(d){a.toBlob(function(a){d(a)},b,c)}):zo(a.toDataURL(b,c))},canvasToDataURL:function(a,b,c){b=b||"image/png";return a.then(function(a){return a.toDataURL(b,c)})},blobToCanvas:function(a){return Rv(a).then(function(a){URL.revokeObjectURL(a.src);var c;c=fc.create(og.getWidth(a),og.getHeight(a));fc.get2dContext(c).drawImage(a,0,0);return c})},uriToBlob:function(a){return 0===a.indexOf("blob:")?
Sv(a):0===a.indexOf("data:")?zo(a):null}},$h={blobToImage:function(a){return Dc.blobToImage(a)},imageToBlob:function(a){return Dc.imageToBlob(a)},blobToDataUri:function(a){return Dc.blobToDataUri(a)},blobToBase64:function(a){return Dc.blobToBase64(a)},dataUriToBlobSync:function(a){return Dc.dataUriToBlobSync(a)},uriToBlob:function(a){return h.from(Dc.uriToBlob(a))}},vd={fromBlob:Vv,fromCanvas:function(a,b){return Dc.canvasToBlob(a,b).then(function(b){return Ao(Xd.resolve(a),b,a.toDataURL())})},fromImage:function(a){return Dc.imageToBlob(a).then(function(a){return Vv(a)})},
fromBlobAndUrlSync:function(a,b){return Ao(Dc.blobToCanvas(a),a,b)}},HH=function(a){return a.toBlob()},tc={blobToImageResult:function(a){return vd.fromBlob(a)},fromBlobAndUrlSync:function(a,b){return vd.fromBlobAndUrlSync(a,b)},imageToImageResult:function(a){return vd.fromImage(a)},imageResultToBlob:function(a,b,c){return void 0===b&&void 0===c?HH(a):a.toAdjustedBlob(b,c)},imageResultToOriginalBlob:HH,imageResultToDataURL:function(a){return a.toDataURL()}},Re={createObjectURL:function(a){return Vb.getOrDie("URL").createObjectURL(a)},
revokeObjectURL:function(a){Vb.getOrDie("URL").revokeObjectURL(a)}},fj=Y.detect(),JH=function(a){var b=Re.createObjectURL(a);return IH(a,b)},IH=function(a,b){return za.nu(function(c){$h.blobToDataUri(a).then(function(d){d=tc.fromBlobAndUrlSync(a,d);var e=La.generate("image");d=oc.blob(e,d,b);c(d)})})},b6=function(a){return 1===a.length&&k.contains(a,"Files")},c6=function(a){return!k.contains(a,"text/_moz_htmlcontext")},d6=function(a){return k.contains(a,"Files")},e6=function(a){return!0},ud={multiple:function(a){return 0===
a.length?za.pure([]):Nc.mapM(a,JH)},toFiles:function(a){return a.raw().target.files||a.raw().dataTransfer.files},isFiles:fj.browser.isChrome()||fj.browser.isSafari()||fj.browser.isOpera()?d6:fj.browser.isFirefox()?c6:fj.browser.isIE()?b6:e6,fromImages:function(a){a=k.map(a,function(a){var c=La.generate("image");return oc.url(c,m.get(a,"src"),a)});return za.pure(a)},single:JH,singleWithUrl:IH},Sw={register:function(a,b,c){var d=D.create({dragover:s(["original"]),dragleave:s(["original"]),drop:s(["original"]),
dragndropimages:s(["original","files"]),dragndrophtml:s(["original"])});b.each(function(b){var c=function(c){return function(){c(a,b)}};d.registry.dragover.bind(c(n.add));d.registry.dragleave.bind(c(n.remove));d.registry.drop.bind(c(n.remove))});var e=E.bind(a,"dragover",function(a){!0===c?(a.prevent(),d.trigger.dragover(a)):a.kill()}),g=E.bind(a,"dragleave",d.trigger.dragleave),f=E.bind(a,"drop",function(a){d.trigger.drop(a);a.kill();if(ud.isFiles(a.raw().dataTransfer.types)){var b=ud.toFiles(a);
d.trigger.dragndropimages(a,b)}else k.contains(a.raw().dataTransfer.types,"text/html")&&d.trigger.dragndrophtml(a)});return{destroy:function(){e.unbind();g.unbind();f.unbind()},events:d.registry}}},ZP={forImages:function(a,b,c){a=Sw.register(a,b,c);var d=D.create({dragover:s(["original"]),dragleave:s(["original"]),drop:s(["original"]),dragndropimages:s(["assets","x","y"]),dragndrophtml:s(["html","x","y"])});b=function(a,b){a.bind(function(a){b(a.original())})};a.events.dragndropimages.bind(function(a){var b=
a.original();ud.multiple(a.files()).get(function(a){d.trigger.dragndropimages(a,b.x(),b.y())})});a.events.dragndrophtml.bind(function(a){a=a.original();var b=a.raw().dataTransfer.getData("text/html");d.trigger.dragndrophtml(b,a.x(),a.y())});b(a.events.dragover,d.trigger.dragover);b(a.events.dragleave,d.trigger.dragleave);b(a.events.drop,d.trigger.drop);return{destroy:a.destroy,events:d.registry}}},f6=f.compose(za.pure,H.value),g6=f.compose(za.pure,H.error),Yd={pure:f6,failed:g6,foldSync:function(a,
b,c){return a.map(function(a){return a.fold(b,c)})},bindSync:function(a,b){return a.map(function(a){return a.bind(b)})},bindAsync:function(a,b){return a.bind(function(a){return a.bind(b)})},mapSync:function(a,b){return a.map(function(a){return a.map(b)})},handle:function(a,b,c){return a.get(function(a){return a.fold(b,c)})}},Lh={now:function(){return(new Date).getTime()}},$P=36E5,Sm=ma.generate([{bothErrors:["error1","error2"]},{firstError:["error1","value2"]},{secondError:["value1","error2"]},{bothValues:["value1",
"value2"]}]),KH={partition:function(a){var b=[],c=[];k.each(a,function(a){a.fold(function(a){b.push(a)},function(a){c.push(a)})});return{errors:b,values:c}},compare:function(a,b){return a.fold(function(a){return b.fold(function(b){return Sm.bothErrors(a,b)},function(b){return Sm.firstError(a,b)})},function(a){return b.fold(function(b){return Sm.secondError(a,b)},function(b){return Sm.bothValues(a,b)})})}},LH=function(a){return f.compose(H.error,k.flatten)(a)},Qs={consolidateObj:function(a,b){var c=
KH.partition(a);return 0<c.errors.length?LH(c.errors):H.value(Z.deepMerge.apply(void 0,[b].concat(c.values)))},consolidateArr:function(a){a=KH.partition(a);return 0<a.errors.length?LH(a.errors):H.value(a.values)}},Rs={narrow:function(a,b){var c={};k.each(b,function(b){void 0!==a[b]&&a.hasOwnProperty(b)&&(c[b]=a[b])});return c},exclude:function(a,b){var c={};W.each(a,function(a,e){k.contains(b,e)||(c[e]=a)});return c},indexOnKey:function(a,b){var c={};k.each(a,function(a){c[a[b]]=a});return c}},Ss=
function(a){return function(b){return b.hasOwnProperty(a)?h.from(b[a]):h.none()}},ef={readOpt:Ss,readOr:function(a,b){return function(c){return Ss(a)(c).getOr(b)}},readOptFrom:function(a,b){return Ss(b)(a)},hasKey:function(a,b){return a.hasOwnProperty(b)&&void 0!==a[b]&&null!==a[b]}},jh={wrap:function(a,b){var c={};c[a]=b;return c},wrapAll:function(a){var b={};k.each(a,function(a){b[a.key]=a.value});return b}},ie={narrow:function(a,b){return Rs.narrow(a,b)},exclude:function(a,b){return Rs.exclude(a,
b)},readOpt:function(a){return ef.readOpt(a)},readOr:function(a,b){return ef.readOr(a,b)},readOptFrom:function(a,b){return ef.readOptFrom(a,b)},wrap:function(a,b){return jh.wrap(a,b)},wrapAll:function(a){return jh.wrapAll(a)},indexOnKey:function(a,b){return Rs.indexOnKey(a,b)},hasKey:function(a,b){return ef.hasKey(a,b)},consolidate:function(a,b){return Qs.consolidateObj(a,b)}},$c={parse:function(a){return Vb.getOrDie("JSON").parse(a)},stringify:function(a,b,c){return Vb.getOrDie("JSON").stringify(a,
b,c)}},Tm={formatObj:function(a){return R.isObject(a)&&100<W.keys(a).length?" removed due to size":$c.stringify(a,null,2)},formatErrors:function(a){a=10<a.length?a.slice(0,10).concat([{path:[],getErrorInfo:function(){return"... (only showing first ten failures)"}}]):a;return k.map(a,function(a){return"Failed path: ("+a.path.join(" > ")+")\n"+a.getErrorInfo()})}},gj=function(a,b){return H.error([{path:a,getErrorInfo:b}])},hj={missingStrict:function(a,b,c){return gj(a,function(){return'Could not find valid *strict* value for "'+
b+'" in '+Tm.formatObj(c)})},missingKey:function(a,b){return gj(a,function(){return'Choice schema did not contain choice key: "'+b+'"'})},missingBranch:function(a,b,c){return gj(a,function(){return'The chosen schema: "'+c+'" did not exist in branches: '+Tm.formatObj(b)})},unsupportedFields:function(a,b){return gj(a,function(){return"There are unsupported fields: ["+b.join(", ")+"] specified"})},custom:function(a,b){return gj(a,function(){return b})},toString:function(a){return"Failed path: ("+a.path.join(" > ")+
")\n"+a.getErrorInfo()}},ff=ma.generate([{strict:[]},{defaultedThunk:["fallbackThunk"]},{asOption:[]},{asDefaultedOptionThunk:["fallbackThunk"]},{mergeWithThunk:["baseThunk"]}]),Zb={strict:ff.strict,asOption:ff.asOption,defaulted:function(a){return ff.defaultedThunk(f.constant(a))},defaultedThunk:ff.defaultedThunk,asDefaultedOption:function(a){return ff.asDefaultedOptionThunk(f.constant(a))},asDefaultedOptionThunk:ff.asDefaultedOptionThunk,mergeWith:function(a){return ff.mergeWithThunk(f.constant(a))},
mergeWithThunk:ff.mergeWithThunk},Yf=ma.generate([{setOf:["validator","valueType"]},{arrOf:["valueType"]},{objOf:["fields"]},{itemOf:["validator"]},{choiceOf:["key","branches"]},{thunk:["description"]},{func:["args","outputSchema"]}]),MH=ma.generate([{field:["name","presence","type"]},{state:["name"]}]),ij=ma.generate([{field:["key","okey","presence","prop"]},{state:["okey","instantiator"]}]),h6=function(a,b,c){return ef.readOptFrom(b,c).fold(function(){return hj.missingStrict(a,c,b)},H.value)},NH=
function(a,b,c){b=ef.readOptFrom(a,b).fold(function(){return c(a)},f.identity);return H.value(b)},i6=function(a,b,c){b=ef.readOptFrom(a,b).map(function(b){return!0===b?c(a):b});return H.value(b)},j6=function(a,b,c,d){return c.fold(function(c,g,l,k){var m=function(b){return k.extract(a.concat([c]),d,b).map(function(a){return jh.wrap(g,d(a))})},n=function(b){return b.fold(function(){var a=jh.wrap(g,d(h.none()));return H.value(a)},function(b){return k.extract(a.concat([c]),d,b).map(function(a){return jh.wrap(g,
d(h.some(a)))})})};return function(){return l.fold(function(){return h6(a,b,c).bind(m)},function(a){return NH(b,c,a).bind(m)},function(){return H.value(ef.readOptFrom(b,c)).bind(n)},function(a){return i6(b,c,a).bind(n)},function(a){var d=a(b);return NH(b,c,f.constant({})).map(function(a){return Z.deepMerge(d,a)}).bind(m)})}()},function(a,c){var f=c(b);return H.value(jh.wrap(a,d(f)))})},k6=function(a,b,c,d){c=k.map(c,function(c){return j6(a,b,c,d)});return Qs.consolidateObj(c,{})},Um=function(a){return{extract:function(b,
c,d){return a(d,c).fold(function(a){return hj.custom(b,a)},H.value)},toString:function(){return"val"},toDsl:function(){return Yf.itemOf(a)}}},l6=function(a){var b=W.keys(a);return k.filter(b,function(b){return ie.hasKey(a,b)})},Vm=function(a){return{extract:function(b,c,d){return k6(b,d,a,c)},toString:function(){return"obj{\n"+k.map(a,function(a){return a.fold(function(a,b,e,g){return a+" -> "+g.toString()},function(a,b){return"state("+a+")"})}).join("\n")+"}"},toDsl:function(){return Yf.objOf(k.map(a,
function(a){return a.fold(function(a,b,e,g){return MH.field(a,e,g)},function(a,b){return MH.state(a)})}))}}},Ts=function(a){return{extract:function(b,c,d){d=k.map(d,function(d,g){return a.extract(b.concat(["["+g+"]"]),c,d)});return Qs.consolidateArr(d)},toString:function(){return"array("+a.toString()+")"},toDsl:function(){return Yf.arrOf(a)}}},m6=Um(H.value),n6=f.compose(Ts,Vm),Ga={anyValue:f.constant(m6),value:Um,obj:Vm,objOnly:function(a){var b=Vm(a),c=k.foldr(a,function(a,b){return b.fold(function(b){return Z.deepMerge(a,
ie.wrap(b,!0))},f.constant(a))},{});return{extract:function(a,e,g){var f=R.isBoolean(g)?[]:l6(g),f=k.filter(f,function(a){return!ie.hasKey(c,a)});return 0===f.length?b.extract(a,e,g):hj.unsupportedFields(a,f)},toString:b.toString,toDsl:b.toDsl}},arr:Ts,setOf:function(a,b){var c=function(b,c){return Ts(Um(a)).extract(b,f.identity,c)};return{extract:function(a,e,g){var f=W.keys(g);return c(a,f).bind(function(c){c=k.map(c,function(a){return ij.field(a,a,Zb.strict(),b)});return Vm(c).extract(a,e,g)})},
toString:function(){return"setOf("+b.toString()+")"},toDsl:function(){return Yf.setOf(a,b)}}},arrOfObj:n6,state:ij.state,field:ij.field,output:function(a,b){return ij.state(a,f.constant(b))},snapshot:function(a){return ij.state(a,f.identity)},thunk:function(a,b){var c=hb.cached(function(){return b()});return{extract:function(a,b,g){return c().extract(a,b,g)},toString:function(){return c().toString()},toDsl:function(){return Yf.thunk(a)}}},func:function(a,b,c){return{extract:Um(function(b,e){return R.isFunction(b)?
H.value(function(){var g=Array.prototype.slice.call(arguments,0).slice(0,a.length),g=b.apply(null,g);return c(g,e)}):H.error("Not a function")}).extract,toString:function(){return"function"},toDsl:function(){return Yf.func(a,b)}}}},o6=function(a,b,c,d,e){return ie.readOptFrom(d,e).fold(function(){return hj.missingBranch(a,d,e)},function(d){return Ga.obj(d).extract(a.concat(["branch: "+e]),b,c)})},p6={choose:function(a,b){return{extract:function(c,d,e){return ie.readOptFrom(e,a).fold(function(){return hj.missingKey(c,
a)},function(a){return o6(c,d,e,b,a)})},toString:function(){return"chooseOn("+a+"). Possible values: "+W.keys(b)},toDsl:function(){return Yf.choiceOf(a,b)}}}},OH=Ga.value(H.value),q6=Ga.arr,r6=Ga.obj,s6=Ga.objOnly,t6=Ga.setOf,Us=function(a,b,c,d){return b.extract([a],c,d).fold(function(a){return H.error({input:d,errors:a})},H.value)},PH=function(a,b,c){return Us(a,b,f.constant,c)},QH=function(a,b,c){return Us(a,b,f.identity,c)},Wm=function(a){return a.fold(function(a){throw Error(RH(a));},f.identity)},
RH=function(a){return"Errors: \n"+Tm.formatErrors(a.errors)+"\n\nInput object: "+Tm.formatObj(a.input)},wc={anyValue:f.constant(OH),arrOfObj:function(a){return Ga.arrOfObj(a)},arrOf:q6,arrOfVal:function(){return Ga.arr(OH)},valueOf:function(a){return Ga.value(function(b){return a(b)})},setOf:t6,objOf:r6,objOfOnly:s6,asStruct:PH,asRaw:QH,asStructOrDie:function(a,b,c){return Wm(PH(a,b,c))},asRawOrDie:function(a,b,c){return Wm(QH(a,b,c))},getOrDie:Wm,formatError:RH,choose:function(a,b){return p6.choose(a,
b)},thunkOf:function(a,b){return Ga.thunk(a,b)},funcOrDie:function(a,b){return Ga.func(a,b,function(a,d){return Wm(Us("()",b,d,a))})}},Xm=ma.generate([{get:[]},{post:[]},{put:[]},{del:[]}]),Ym={get:Xm.get,post:Xm.post,put:Xm.put,del:Xm.del},u6={generate:function(a,b,c,d){a=a.bind(function(a){return a.match({file:function(){return h.none()},form:function(){return h.some("application/x-www-form-urlencoded; charset=UTF-8")},json:function(){return h.some("application/json")},plain:function(){return h.some("text/plain")},
html:function(){return h.some("text/html")}})});c=c.match({none:h.none,xhr:f.constant(h.some(!0))});var e=b.match({json:h.none,blob:f.constant(h.some("blob")),xml:f.constant(h.some("document")),html:f.constant(h.some("document")),text:f.constant(h.some("text"))});b=b.match({json:f.constant("application/json, text/javascript"),blob:f.constant("application/octet-stream"),text:f.constant("text/plain"),html:f.constant("text/html"),xml:f.constant("application/xml, text/xml")})+", */*; q=0.01";return{contentType:f.constant(a),
credentials:f.constant(c),responseType:f.constant(e),accept:f.constant(b),headers:f.constant(d)}}},v6={mutate:function(a,b){b.contentType().each(function(b){a.setRequestHeader("Content-Type",b)});var c=b.accept();a.setRequestHeader("Accept",c);b.credentials().each(function(b){a.withCredentials=b});b.responseType().each(function(b){a.responseType=b});c=b.headers();W.each(c,function(b,c){R.isString(c)||R.isString(b)?a.setRequestHeader(c,b):console.error("Request header data was not a string: ",c," -> ",
b)})}},SH={parse:function(a){try{var b=$c.parse(a);return H.value(b)}catch(c){return H.error("Response was not JSON")}}},TH=C.immutableBag(["message","status","responseText"],[]),UH={handle:function(a,b,c){var d=function(){return za.pure(c.response)};return b.match({json:function(){return SH.parse(c.response).fold(d,za.pure)},blob:function(){return h.from(c.response).map(aQ).getOr(za.pure("no response content"))},text:d,html:d,xml:d}).map(function(b){return TH({message:0===c.status?"Unknown HTTP error (possible cross-domain request)":
'Could not load url "'+a+'": '+c.statusText,status:c.status,responseText:b})})},nu:TH},w6=Object.assign||function(a){for(var b,c=1,d=arguments.length;c<d;c++){b=arguments[c];for(var e in b)Object.prototype.hasOwnProperty.call(b,e)&&(a[e]=b[e])}return a},Jd=function(a){return w6({},a,{bindFuture:function(b){return Jd(a.bind(function(c){return c.fold(function(){return a},function(a){return b(a)})}))},bindResult:function(b){return Jd(a.map(function(a){return a.bind(b)}))},mapResult:function(b){return Jd(a.map(function(a){return a.map(b)}))},
foldResult:function(b,c){return a.map(function(a){return a.fold(b,c)})},withTimeout:function(b,c){return Jd(za.nu(function(d){var e=!1,g=window.setTimeout(function(){e=!0;d(H.error(c()))},b);a.get(function(a){e||(window.clearTimeout(g),d(a))})}))}})},VH=function(a){return Jd(za.nu(a))},WH=function(a){return Jd(za.pure(H.value(a)))},jj={nu:VH,wrap:Jd,pure:WH,value:WH,error:function(a){return Jd(za.pure(H.error(a)))},fromResult:function(a){return Jd(za.pure(a))},fromFuture:function(a){return Jd(a.map(H.value))},
fromPromise:function(a){return VH(function(b){a.then(function(a){b(H.value(a))},function(a){b(H.error(a))})})}},x6={validate:function(a,b){var c=function(){return jj.pure(b.response)};return a.match({json:function(){return SH.parse(b.response).fold(function(a){return jj.error(UH.nu({message:a,status:b.status,responseText:b.responseText}))},jj.pure)},blob:c,text:c,html:c,xml:function(a,b){return function(){return b().bindResult(function(b){return null===b?H.error(a):H.value(b)})}}("Invalid XML document",
c)})}},Zm=function(a,b,c,d,e,g){var l=void 0!==g?g:{};return jj.nu(function(g){var h=b.match({get:f.constant("get"),put:f.constant("put"),post:f.constant("post"),del:f.constant("delete")}),k=new (Vb.getOrDie("XMLHttpRequest"));k.open(h,a,!0);h=u6.generate(c,d,e,l);v6.mutate(k,h);var m=function(){UH.handle(a,d,k).get(function(a){g(H.error(a))})};k.onload=function(){0!==k.status||aa.startsWith(a,"file:")?100>k.status||400<=k.status?m():x6.validate(d,k).get(g):m()};k.onerror=m;c.fold(function(){k.send()},
function(a){a=a.match({file:f.identity,form:f.identity,json:$c.stringify,plain:f.identity,html:f.identity});k.send(a)})}).toLazy()},wd={get:function(a,b,c,d){var e=Ym.get();return Zm(a,e,h.none(),b,c,d)},post:function(a,b,c,d,e){var g=Ym.post();return Zm(a,g,h.some(b),c,d,e)},put:function(a,b,c,d,e){var g=Ym.put();return Zm(a,g,h.some(b),c,d,e)},del:function(a,b,c,d){var e=Ym.del();return Zm(a,e,h.none(),b,c,d)}},kj=ma.generate([{file:["data"]},{form:["data"]},{json:["data"]},{plain:["data"]},{html:["data"]}]),
hy={file:kj.file,form:kj.form,json:kj.json,plain:kj.plain,html:kj.html},XH=ma.generate([{none:[]},{xhr:[]}]),nd={none:XH.none,xhr:XH.xhr},lj=ma.generate([{json:[]},{blob:[]},{text:[]},{html:[]},{xml:[]}]),xd={json:lj.json,blob:lj.blob,text:lj.text,html:lj.html,xml:lj.xml,cata:function(a,b,c,d,e,g){return a.match({json:b,blob:c,text:d,html:e,xml:g})}},YH=function(a,b){var c=-1===a.indexOf("?")?"?":"&";return b?a+c+"apiKey="+encodeURIComponent(b):a},ZH=function(a){return a.hasOwnProperty("tiny-api-key")?
a["tiny-api-key"]:a.hasOwnProperty("tinymce-api-key")?a["tinymce-api-key"]:a["textbox-api-key"]},$H=function(a){return a.fold(function(a){var c=a.responseText();a=R.isObject(c)?c:a.message();return H.error(a)},H.value)},hk={get:function(a,b,c,d){var e=ZH(d);return{execute:function(g){g=wc.asRawOrDie(a+".ajax.service.get",b,g);g=W.map(g,function(a){return R.isBoolean(a)?String(a):a});g=YH(aa.supplant(c,g),e);return wd.get(g,xd.json(),nd.xhr(),d).map($H)}}},post:function(a,b,c,d){var e=ZH(d);return{execute:function(g){g=
wc.asRawOrDie(a+".ajax.service.post",b,g);return wd.post(YH(c,e),hy.json(g),xd.json(),nd.xhr(),d).map($H)}}}},y6={partition:function(a,b,c,d,e){return!0===e?{known:{},unknown:c}:k.foldl(c,function(c,e){var f=d(e);return a(b,f).fold(function(){return{known:c.known,unknown:c.unknown.concat([e])}},function(a){a=ie.wrap(f,a);return{known:Z.merge(c.known,a),unknown:c.unknown}})},{known:{},unknown:[]})}},z6=function(a,b,c,d,e,g,f){a=a(d,f);return Yd.mapSync(a,function(a){a=e(a);W.each(a,function(a,c){b.set(c,
a,g)});return Z.deepMerge(c,a)})},ik={single:function(a,b,c,d,e){return(!0===e?h.none():b.get(d,c)).fold(function(){return a(c,e).map(function(a){return a.map(function(a){b.set(c,a,d);return a})})},function(a){return Yd.pure(a)})},multiple:function(a,b,c,d,e,g,f){c=y6.partition(b.get,g,c,e,f);return 0===c.unknown.length?Yd.pure(c.known):z6(a,b,c.known,c.unknown,d,g,f)}},aI=function(a,b){return Ga.field(a,a,Zb.strict(),b)},xc={strict:function(a){return Ga.field(a,a,Zb.strict(),Ga.anyValue())},strictOf:aI,
strictObjOf:function(a,b){return Ga.field(a,a,Zb.strict(),Ga.obj(b))},strictArrayOf:function(a,b){return aI(a,b)},strictArrayOfObj:function(a,b){return Ga.field(a,a,Zb.strict(),Ga.arrOfObj(b))},strictFunction:function(a){return Ga.field(a,a,Zb.strict(),Ga.value(function(a){return R.isFunction(a)?H.value(a):H.error("Not a function")}))},forbid:function(a,b){return Ga.field(a,a,Zb.asOption(),Ga.value(function(c){return H.error("The field: "+a+" is forbidden. "+b)}))},option:function(a){return Ga.field(a,
a,Zb.asOption(),Ga.anyValue())},optionOf:function(a,b){return Ga.field(a,a,Zb.asOption(),b)},optionObjOf:function(a,b){return Ga.field(a,a,Zb.asOption(),Ga.obj(b))},optionObjOfOnly:function(a,b){return Ga.field(a,a,Zb.asOption(),Ga.objOnly(b))},defaulted:function(a,b){return Ga.field(a,a,Zb.defaulted(b),Ga.anyValue())},defaultedOf:function(a,b,c){return Ga.field(a,a,Zb.defaulted(b),c)},defaultedObjOf:function(a,b,c){return Ga.field(a,a,Zb.defaulted(b),Ga.obj(c))},field:function(a,b,c,d){return Ga.field(a,
b,c,d)},state:function(a,b){return Ga.state(a,b)}},$m=function(a){return"["+a.join(", ")+"]"},bI=["iframely","oembed","fallback"],an=function(a,b,c){return k.exists(b,function(b){return k.contains(a,b)})?H.error("Embed response source: "+$m(a)+" contains more than one of "+$m(bI)):H.value(c)},A6={iframely:f.constant("iframely"),oembed:f.constant("oembed"),fallback:f.constant("fallback"),identify:function(a){return k.contains(a,"iframely")?an(a,["oembed","fallback","wikipedia"],"iframely"):k.contains(a,
"oembed")?an(a,["iframely","fallback","wikipedia"],"oembed"):k.contains(a,"fallback")?an(a,["iframely","oembed","wikipedia"],"fallback"):k.contains(a,"wikipedia")?an(a,["iframely","oembed","fallback"],"wikipedia"):H.error("Embed response source: "+$m(a)+" did not contain any of: "+$m(bI))}},B6=wc.objOf([xc.strict("url"),xc.strict("maxWidth"),xc.defaulted("fresh",!1)]),C6=wc.objOf([xc.strict("status_code"),xc.strict("sub_code"),xc.strict("msg")]),D6=wc.objOf([xc.strict("html"),xc.strictObjOf("rels",
[xc.field("source","source",Zb.strict(),wc.valueOf(function(a){return R.isArray(a)?A6.identify(a):H.error("Sources was not an array: "+a)}))])]),Bo={requestOne:f.constant(B6),responseFailed:f.constant(C6),responseSucceeded:f.constant(D6)},Wv={failed:function(a){return wc.asStruct("EmbedResponse.failure",Bo.responseFailed(),a).fold(function(a){return H.error({code:f.constant(600),subcode:f.constant(0),msg:f.constant(wc.formatError(a))})},function(a){return H.error({code:a.status_code,subcode:a.sub_code,
msg:a.msg})})},success:function(a){return wc.asStruct("EmbedResponse.success",Bo.responseSucceeded(),a).fold(function(a){return H.error({code:f.constant(601),subcode:f.constant(0),msg:f.constant(wc.formatError(a))})},function(a){return H.value({recommended:f.constant({html:a.html,source:a.rels().source})})})}},ue=x.resolve("embed-checking"),Co={configure:function(a){a.add(a.input(),ue,Ca.removeClass(ue));a.add(a.output(),ue,Ca.removeClass(ue));a.add(a.history(),ue,Ca.removeClass(ue));a.add(a.internal(),
ue,Ca.removeClass(ue))},embedChecking:f.constant(ue)},Vs=function(){return u.translate("link.anchor.top")},Ws=function(){return u.translate("link.anchor.bottom")},cI=function(){var a={};a[Vs().toLowerCase()]="top";a[Ws().toLowerCase()]="bottom";return a},Xa={bookmarkDataAttr:f.constant("data-textbox-bookmark"),bookmarkAnchorDataAttr:f.constant("data-textbox-bookmark-anchor"),bookmarkBookmarkDataAttr:f.constant("data-textbox-user-bookmark"),topAnchorUrl:Vs,bottomAnchorUrl:Ws,top:f.constant("top"),
bottom:f.constant("bottom"),reservedUrls:cI,isReservedUrl:function(a){return cI().hasOwnProperty(a.toLowerCase())},getReservedUrl:function(a){var b={};b.top=Vs();b.bottom=Ws();return b[a.toLowerCase()]}},E6={setCss:function(a,b,c){a.dom().styleSheet?a.dom().styleSheet.cssText=b:v.append(a,c)}},F6={escape:function(a){return a.replace(/[-[\]{}()*+?.,\\^$|#\s]/g,"\\$&")}},dI=function(a,b,c){b=RegExp("url\\(\\s*['\"]?"+F6.escape(b)+"(.*?)['\"]?\\s*\\)","g");return a.replace(b,'url("'+c+'$1")')},G6={replace:dI,
replaceMany:function(a,b){var c=a;W.each(b,function(a,b){console.log("rm css",c);c=dI(c,b,a)});return c}},eI={stylesheets:function(a){a=a.dom().styleSheets;return Array.prototype.slice.call(a)},inject:function(a,b,c){var d=q.fromTag("style",c.dom());m.set(d,"type","text/css");a=void 0===b?a:G6.replaceMany(a,b);E6.setCss(d,a,q.fromText(a,c.dom()));c=J.descendant(c,"head").getOrDie();v.append(c,d)}},H6=C.immutable("selector","style","raw"),Xs=function(a){return k.bind(a.cssRules,function(a){if(a.type===
CSSRule.IMPORT_RULE)return Xs(a.styleSheet);if(a.type===CSSRule.STYLE_RULE){var c=a.selectorText,d=a.style.cssText;if(void 0===d)throw"WARNING: Browser does not support cssText property";return[H6(c,d,a.style)]}return[]})},fI={extract:Xs,extractAll:function(a){return k.bind(a,Xs)}},gI={},hI={exports:gI};(function(a,b,c,d){(function(d){"object"===typeof b&&"undefined"!==typeof c?c.exports=d():"function"===typeof a&&a.amd?a([],d):("undefined"!==typeof window?window:"undefined"!==typeof global?global:
"undefined"!==typeof self?self:this).EphoxContactWrapper=d()})(function(){return function g(a,b,c){function f(k,m){if(!b[k]){if(!a[k]){var n="function"==typeof d&&d;if(!m&&n)return n(k,!0);if(h)return h(k,!0);n=Error("Cannot find module '"+k+"'");throw n.code="MODULE_NOT_FOUND",n;}n=b[k]={exports:{}};a[k][0].call(n.exports,function(b){var c=a[k][1][b];return f(c?c:b)},n,n.exports,g,a,b,c)}return b[k].exports}for(var h="function"==typeof d&&d,k=0;k<c.length;k++)f(c[k]);return f}({1:[function(a,b,c){a=
function(){var a;a=function(a){var b=a,c,d={a:0,b:0,c:0},g=[];c=function(c,f){var l,h,k,p,m,r;if(c.test(b))for(l=b.match(c),h=0,k=l.length;h<k;h+=1)d[f]+=1,p=l[h],m=b.indexOf(p),r=p.length,g.push({selector:a.substr(m,r),type:f,index:m,length:r}),b=b.replace(p,Array(r+1).join(" "))};(function(){var a=function(a){var c,d,g;if(a.test(b))for(a=b.match(a),c=0,d=a.length;c<d;c+=1)g=a[c],b=b.replace(g,Array(g.length+1).join("A"))};a(/\\[0-9A-Fa-f]{6}\s?/g);a(/\\[0-9A-Fa-f]{1,5}\s/g);a(/\\./g)})();(function(){var a=
/:not\(([^\)]*)\)/g;a.test(b)&&(b=b.replace(a,"     $1 "))})();(function(){var a=/{[^]*/gm,c,d,g;if(a.test(b))for(a=b.match(a),c=0,d=a.length;c<d;c+=1)g=a[c],b=b.replace(g,Array(g.length+1).join(" "))})();c(/(\[[^\]]+\])/g,"b");c(/(#[^\#\s\+>~\.\[:]+)/g,"a");c(/(\.[^\s\+>~\.\[:]+)/g,"b");c(/(::[^\s\+>~\.\[:]+|:first-line|:first-letter|:before|:after)/gi,"c");c(/(:[\w-]+\([^\)]*\))/gi,"b");c(/(:[^\s\+>~\.\[:]+)/g,"b");b=b.replace(/[\*\s\+>~]/g," ");b=b.replace(/[#\.]/g," ");c(/([^\s\+>~\.\[:]+)/g,
"c");g.sort(function(a,b){return a.index-b.index});return{selector:a,specificity:"0,"+d.a.toString()+","+d.b.toString()+","+d.c.toString(),specificityArray:[0,d.a,d.b,d.c],parts:g}};return{calculate:function(b){var c,d,g,f=[];b=b.split(",");d=0;for(g=b.length;d<g;d+=1)c=b[d],0<c.length&&f.push(a(c));return f},compare:function(b,c){var d,g,f;if("string"===typeof b){if(-1!==b.indexOf(","))throw"Invalid CSS selector";d=a(b).specificityArray}else if(Array.isArray(b)){if(4!==b.filter(function(a){return"number"===
typeof a}).length)throw"Invalid specificity array";d=b}else throw"Invalid CSS selector or specificity array";if("string"===typeof c){if(-1!==c.indexOf(","))throw"Invalid CSS selector";g=a(c).specificityArray}else if(Array.isArray(c)){if(4!==c.filter(function(a){return"number"===typeof a}).length)throw"Invalid specificity array";g=c}else throw"Invalid CSS selector or specificity array";for(f=0;4>f;f+=1){if(d[f]<g[f])return-1;if(d[f]>g[f])return 1}return 0}}}();"undefined"!==typeof c&&(c.calculate=
a.calculate,c.compare=a.compare)},{}],2:[function(a,b,c){a=a("specificity");b.exports={boltExport:a}},{specificity:1}]},{},[2])(2)})})(void 0,gI,hI,void 0);var I6=hI.exports.boltExport,iI=C.immutable("selector","raw"),J6=function(a,b){var c={};k.each(a,function(d){if(void 0!==a[d]){var e=b.dom().style;k.contains(e,d)||(c[d]=a[d])}});return c},K6=function(a,b){var c=k.bind(b,function(b){var c=X.descendants(a,b.selector());k.each(c,function(a){var c=J6(b.raw(),a);t.setAll(a,c)});return c});k.each(c,
function(a){m.remove(a,"class")})},jI=function(a,b){var c=function(a){return-1!==a.selector().indexOf(",")},d=k.bind(k.filter(a,c),function(a){var b=a.selector().split(",");return k.map(b,function(b){b=b.trim();return iI(b,a.raw())})}),d=k.filter(a,function(a){return!c(a)}).concat(d);d.sort(function(a,b){return I6.compare(a.selector(),b.selector())}).reverse();K6(b,d)},kI={inlineStyles:function(a,b,c){a=eI.stylesheets(a);a=fI.extractAll(a).map(function(a){var b=a.selector(),b=c.hasOwnProperty(b)?
c[b]:b;return iI(b,a.raw())});jI(a,b)},inlinePrefixedStyles:function(a,b,c){a=eI.stylesheets(a);a=fI.extractAll(a);a=k.filter(a,function(a){return aa.startsWith(a.selector(),c)});jI(a,b)}},Xv={inlineStyles:function(a,b,c){kI.inlineStyles(a,b,c)},inlinePrefixedStyles:function(a,b,c){kI.inlinePrefixedStyles(a,b,c)}},rk={handle:function(a,b){return E.bind(a,"click",b)}},PR={editor:function(a,b){var c=f.curry(u.translate,b?"aria.label.fullscreen":"aria.label.editor");return a.fold(f.identity,c)}},L6=
function(a,b){var c=setInterval(b,500);return function(){clearInterval(c)}},M6=function(a,b){var c=new window.MutationObserver(b);c.observe(y.owner(a).dom(),{attributes:!0,subtree:!0,childList:!0,attributeFilter:["style","class"]});return function(){c.disconnect()}},N6=void 0!==window.MutationObserver&&null!==window.MutationObserver?M6:L6,jw={onShow:function(a,b){if(Tb.isVisible(a))return fk.requestAnimationFrame(b),f.noop;var c=zc.adaptable(function(){Tb.isVisible(a)&&(d(),fk.requestAnimationFrame(b))},
100),d=N6(a,c.throttle);return d}},mj=[],Ys=function(a){return k.findIndex(mj,function(b){return L.eq(b.element,a)}).getOr(-1)},Zs={begin:function(a,b){if(-1===Ys(a)){var c=b();mj.push({element:a,unbind:c})}},query:function(a){a=Ys(a);return-1===a?h.none():h.some(mj[a])},end:function(a){a=Ys(a);if(-1!==a){var b=mj[a];mj.splice(a,1);b.unbind()}}},O6=function(a){return{element:a,handlers:[],lastWidth:qa.get(a),lastHeight:ia.get(a)}},ve=[],lI=function(a){return k.findIndex(ve,function(b){return L.eq(b.element,
a)}).getOr(-1)},mI=function(a){var b=qa.get(a.element),c=ia.get(a.element);if(b!==a.lastWidth||c!==a.lastHeight)a.lastWidth=b,a.lastHeight=c,k.each(a.handlers,f.apply)},nI=function(a){var b=a.element;Tb.isVisible(b)?mI(a):Zs.begin(b,function(){return jw.onShow(b,function(){Zs.end(b);mI(a)})})},$s=!1,P6=function(){$s=!1;k.each(ve,nI)},Q6=function(){$s||($s=!0,fk.requestAnimationFrame(P6))},bn=h.none(),R6=function(){bn.each(function(a){a.unbind();bn=h.none()})},Zv={bind:function(a,b){var c=k.find(ve,
function(b){return L.eq(b.element,a)}).getOr(void 0);void 0===c&&(c=O6(a),ve.push(c));c.handlers.push(b);bn.isNone()&&(bn=h.some(E.bind(q.fromDom(window),"resize",Q6)));setTimeout(function(){-1!==lI(c.element)&&nI(c)},100)},unbind:function(a,b){Zs.end(a);var c=lI(a);if(-1!==c){var d=k.indexOf(ve[c].handlers,b);-1!==d&&(ve[c].handlers.splice(d,1),0===ve[c].handlers.length&&ve.splice(c,1),0===ve.length&&R6())}}},$v=Y.detect(),S6=f.constant(!1),kk=Y.detect(),oI=kk.browser.isIE()||kk.browser.isEdge()?
2:5,bw=ma.generate([{above:["element"]},{below:["element"]}]),pI=function(a,b,c){return function(b){b=Fa.forElement(a,b);b=Wb.exactFromRange(b);return Fa.getBounds(a,b).fold(S6,c)}},eQ=function(a,b){return pI(a,b,function(a){return b>a.top()+a.height()-oI})},fQ=function(a,b){return pI(a,b,function(a){return b<a.top()+oI})},ub={clearAll:function(a){Fa.clear(a)},get:function(a){return Fa.getExact(a).map(function(a){return cc(a.start(),a.soffset(),a.finish(),a.foffset())})},set:function(a,b){Fa.setExact(a,
b.startContainer(),b.startOffset(),b.endContainer(),b.endOffset())},setExact:function(a,b,c,d,e){Fa.setExact(a,b,c,d,e)},setRelative:function(a,b,c){Fa.setRelative(a,b,c)},replace:function(a,b){Fa.get(a).each(function(c){Fa.replace(a,c,b)})}},gQ=function(a,b){var c=f.curry(L.eq,ca.body()),d=b.join(", ");return ab.findMap(a,function(a){return sa.ancestor(a,function(a){return ra.is(a,d)},c)}).isSome()},T6=function(a,b){return{toEditor:a,toOld:f.curry(T.focus,b),isSet:function(){return T.active().exists(function(a){return L.eq(a,
b)})}}},qI=function(a,b,c,d){d=d.dom().activeElement;return a()?{toEditor:b,toOld:f.noop,isSet:f.constant(!0)}:void 0===d||null===d?{toEditor:c,toOld:f.noop,isSet:f.constant(!1)}:T6(c,q.fromDom(d))},Do={build:qI,toEditor:function(a,b,c,d){qI(a,b,c,d).toEditor()}},U6=function(a){return eb.closest(a.startContainer(),'[contenteditable="false"]')&&eb.closest(a.endContainer(),'[contenteditable="false"]')},ew={focus:function(a,b,c){b=b().filter(U6);b.each(function(b){J.closest(b.endContainer(),'[contenteditable="false"]').each(function(b){ub.setRelative(a.win(),
da.after(b),da.after(b))})});a.focus();b.each(function(a){c(a.startContainer(),a.startOffset(),a.endContainer(),a.endOffset())})}},jQ=Y.detect(),V6=C.immutable("doc","href"),lQ=function(){var a=[],b=function(a,b){return function(e){return L.eq(a,e.doc())&&b===e.href()}};return{addLinkIfMissing:function(c,d){k.exists(a,b(c,d))||(a.push(V6(c,d)),hz.addStylesheet(d,c))}}}(),fw=function(){console.log("The previous selection could not be restored. Defaulting to the current selection.")},at=Y.detect(),
W6=at.deviceType.isiOS()?sQ:at.deviceType.isAndroid()?qQ:rQ,vQ={get:function(a){return W6(a,at)}},X6=function(a){a=a.dom();try{var b=a.contentWindow?a.contentWindow.document:a.contentDocument;return void 0!==b&&null!==b?h.some(q.fromDom(b)):h.none()}catch(c){return console.log("Error reading iframe: ",a),console.log("Error was: "+c),h.none()}},gw={doc:function(a){return X6(a).fold(function(){return a},function(a){return a})}},rI=function(a,b,c,d){return a(function(a){var g=E.bind(b,c,function(b){clearTimeout(f);
g.unbind();a(H.value(b))}),f=setTimeout(function(){g.unbind();a(H.error("Event "+c+" did not fire within "+d+"ms"))},d)})},hw={cWaitFor:function(a,b,c){return rI($d.nu,a,b,c)},waitFor:function(a,b,c){return rI(za.nu,a,b,c)}},uQ=Y.detect(),xQ={resolve:lb.create("ephox-hare").resolve},iw={write:function(a,b){if(!ca.inBody(a))throw"Internal error: attempted to write to an iframe that is not in the DOM";var c=gw.doc(a).dom();c.open("text/html","replace");c.writeln(b);c.close()}},yQ=function(a){var b=
a.base.fold(f.constant(""),function(a){return'<base href="'+a+'" />'}),c=a.editorStyles.concat(a.stylesheets),c=k.map(c,function(a){return'<link rel="stylesheet" type="text/css" href="'+a+'" />'});a=0===a.documentStyles.length?"":'<style type="text/css">'+a.documentStyles+"</style>";return"<!doctype html><html><head>"+b+c.join("")+a+'</head><body contenteditable="true" spellcheck="false"></body></html>'},sI=function(a){var b=h.none(),c=Qc.tap(function(a,b,c){t.trigger.nodeChanged(a,b,c)});a.loaded(function(a){var d=
function(){try{return null!==a.win().window&&void 0!==a.win().window}catch(b){return!1}};if(d()){b=h.some(a);if(Y.detect().browser.isFirefox()){try{a.execCommand("enableObjectResizing",!1,!1)}catch(e){}try{a.execCommand("enableInlineTableEditing",!1,!1)}catch(f){}}a.events.nodeChanged.bind(function(a){c.instance(a.node(),a.offset(),a.prev())});a.events.loiter.bind(function(a){t.trigger.loiter(a.node(),a.offset())});a.events.uiStale.bind(function(){t.trigger.uiStale()});a.events.domChanged.bind(function(){t.trigger.domChanged()});
a.events.interaction.bind(function(){t.trigger.interaction()});a.events.keyup.bind(function(a){t.trigger.keyup(a.original())});a.events.keydown.bind(function(a){t.trigger.keydown(a.original())});a.events.keypress.bind(function(a){t.trigger.keypress(a.original())});a.events.mousedown.bind(function(a){t.trigger.mousedown(a.original())});a.events.mouseup.bind(function(a){t.trigger.mouseup(a.original())});a.events.contextmenu.bind(function(a){t.trigger.contextmenu(a.original())});a.events.cut.bind(function(a){t.trigger.cut(a.original())});
a.events.copy.bind(function(a){t.trigger.copy(a.original())});a.events.paste.bind(function(a){t.trigger.paste(a.original())});a.events.focus.bind(function(a){t.trigger.focus(a.original())});a.events.focusOut.bind(function(a){t.trigger.focusOut(a.original())});a.events.dblclick.bind(function(a){t.trigger.dblclick(a.original())});a.events.selectionchange.bind(function(a){t.trigger.selectionchange(a.original())});a.futures.loaded.get(function(){d()&&t.trigger.loaded()});a.events.paint.bind(function(){t.trigger.paint()});
a.resetContent(g);d()&&t.trigger.ready(a);a.uiChanged()}else a.destroy()});var d=function(){return b.map(function(a){return a.win()})},e=function(){return b.map(function(a){return a.doc()})},g=a.initial(),l=function(){b.each(function(a){a.focus()})},k=function(){return b.bind(function(a){return a.getSelection()})},m=function(a,c,d,e){b.each(function(b){b.setSelection(a,c,d,e)})},n=function(a,c){b.each(function(b){b.setRelativeSelection(a,c)})},q=function(){var a=arguments;b.each(function(b){b.execCommand.apply(null,
a)})},t=D.create({nodeChanged:s(["node","offset","prev"]),loiter:s(["node","offset"]),uiStale:s([]),domChanged:s([]),interaction:s([]),keydown:s(["original"]),keyup:s(["original"]),keypress:s(["original"]),mousedown:s(["original"]),mouseup:s(["original"]),contextmenu:s(["original"]),cut:s(["original"]),copy:s(["original"]),paste:s(["original"]),focus:s(["original"]),focusOut:s(["original"]),dblclick:s(["original"]),selectionchange:s(["original"]),paint:s([]),ready:s(["api"]),loaded:s([])});return{element:a.element,
tabindex:a.tabindex,onCanvas:a.onCanvas,win:d,doc:e,body:function(){return b.map(function(a){return a.body()})},getContent:function(){return b.fold(function(){return g},function(a){return a.getContent()})},setContent:function(a){b.fold(function(){g=a},function(b){b.setContent(a)})},resetContent:function(a){b.fold(function(){g=a},function(b){b.resetContent(a)})},blur:function(){b.each(function(a){a.blur()})},focus:l,uiChanged:function(){b.each(function(a){a.uiChanged()})},domChanged:function(){b.each(function(a){a.domChanged()})},
clearSelection:function(){b.each(function(a){a.clearSelection()})},getSelection:k,setSelection:m,setRelativeSelection:n,backupSelection:function(){b.each(function(a){a.backupSelection()})},replaceSelection:function(a){b.each(function(b){b.replaceSelection(a)})},execCommand:q,lockdown:function(a){var b=c.control.isBlocked();c.control.block();a();b||c.control.unblock()},destroy:function(){a.destroy();b=h.none()},events:t.registry,background:function(){b.fold(f.identity,function(a){a.background()})},
foreground:function(){b.fold(f.identity,function(a){a.foreground()})},on:function(a){return d().bind(function(b){return e().map(function(c){return a({win:f.constant(b),doc:f.constant(c),execCommand:q,getSelection:k,setSelection:m,setRelativeSelection:n,focus:l})})})},translateScreen:function(a,c){return b.fold(function(){return Ba(a,c)},function(b){return b.translateScreen(a,c)})},translateLocation:function(a,c){return b.fold(function(){return Ba(a,c)},function(b){return b.translateLocation(a,c)})},
placeholder:a.placeholder,isInline:a.isInline}},tI={frame:function(a,b){var c=zQ(a,b);return sI(c)},detached:function(a,b,c){a=kQ(a,b,c);return sI(a)}},uI=Y.detect(),vI=uI.browser,Y6=!(vI.isEdge()&&16>vI.version.major||uI.deviceType.isiOS()),Z6=function(a,b){if(Y6&&void 0!==a&&R.isFunction(a.setData))try{return W.each(b,function(b,c){a.setData(c,b())}),!0}catch(c){return!1}else return!1},$6=function(a,b,c){var d=q.fromTag("div",a.doc().dom());t.setAll(d,{position:"absolute",top:"-1000px"});b=b["text/html"]();
ua.set(d,b);v.append(a.body(),d);var e=a.getSelection();Fa.setToElement(a.win(),d);setTimeout(function(){B.remove(d);e.each(function(b){a.setSelection(b.startContainer(),b.startOffset(),b.endContainer(),b.endOffset())});setTimeout(c,0)},0)},a7=function(a){return function(b,c,d){var e=b.raw();if(!R.isString(e.type)||"copy"!==e.type.toLowerCase()&&"cut"!==e.type.toLowerCase())throw"Can only set the clipboard with a cut or copy event";if(!R.isFunction(c["text/html"]))throw"Must provide at least text/html data function when setting clipboard contents";
Z6(e.clipboardData,c)?(b.kill(),setTimeout(d,0)):a(c,d)}},b7=Gb.exactly(["doc","win","body","getSelection","setSelection"]),bt={register:function(a){b7(a);a=f.curry($6,a);return{setClipboard:a7(a)}},textMime:f.constant("text/plain"),htmlMime:f.constant("text/html")},ct=Nu(A.isComment,"comment"),wI={get:function(a){return ct.get(a)},getOption:function(a){return ct.getOption(a)},set:function(a,b){ct.set(a,b)}},fi={intraFlag:f.constant({findClipboardTags:function(a){a=ab.cat(k.map(a,function(a){if(A.isComment(a)){var c=
wI.get(a);if(aa.contains(c,"x-textbox/html"))return h.some(a)}return h.none()}));return 0===a.length?h.none():h.some(a)}}),buildClipboardContents:function(a,b){var c={};c[bt.textMime()]=hb.cached(a);c[bt.htmlMime()]=hb.cached(function(){return"\x3c!-- x-textbox/html --\x3e"+b()});return c},ignoreSelectors:f.constant([".ephox-sloth-bin"])},xI=function(a,b){return{editable:f.constant(a),connect:b}},c7=function(a,b){var c=tI.frame(b,fi.ignoreSelectors());c.events.ready.bind(function(b){b=b.api().body();
a.updateUi(b)});return xI(c,function(a){v.append(a,c.element())})},d7=function(a,b){0!==b.stylesheets.length&&console.log("You have attempted to declare the following stylesheets for use in textbox.io: ["+b.stylesheets.join(", ")+"]. User stylesheets are not used in inline editing mode and have been ignored.");0!==b.documentStyles.length&&console.log('You have attempted to declare the following document styles for use in textbox.io: "'+b.documentStyles+'". Document styles are not used in inline editing mode and have been ignored.');
b.base.fold(f.noop,function(a){console.log('You have attempted to declare the following content base URL for use in textbox.io: "'+a+'". Content base URL is not used in inline editing mode and has been ignored.')});var c=tI.detached(b.inline,b.editorStyles,fi.ignoreSelectors());return xI(c,function(a){v.append(a,c.placeholder());a=J.ancestor(a,".ephox-polish-editor-container").getOr(a);t.set(c.placeholder(),"display","none");ea.add(a,[x.resolve("inline-editor-container"),x.resolve("layer-editor")])})},
CQ={build:function(a,b){return(void 0!==b.inline?d7:c7)(a,b)}},e7=Y.detect().browser.isFirefox(),f7=function(a){k.each(a,function(a){y.nextSibling(a).isNone()&&v.after(a,q.fromText(Ua.zeroWidth()))})},g7={configure:function(a){e7&&a.addSelector(a.input(),"a",f7)}},cn=x.resolve("content-eph-wrapper"),dn=x.resolve("content-eph-extra"),en=x.resolve("content-eph-general"),h7=function(a){k.each(a,function(a){n.remove(a,jb.styleSpan());m.remove(a,jb.overrideSpan());m.hasNone(a)?B.unwrap(a):Ac.isTransient(a)&&
B.remove(a)})},yI=function(a){k.each(a,function(a){n.remove(a,jb.quasiBlock())})},zI=function(a){k.each(a,function(a){var c=m.get(a,"style");m.set(a,"style",c.replace(/"/g,"'"))})},i7=function(a){return ka.isCursorPosition(a)||fa.isBlock(a)||ka.isCursorPosition(Ja.freefallLtr(a).element())},AI=function(a){k.each(a,function(a){if(0===k.filter(y.children(a),i7).length){var c=q.fromTag("br");v.append(a,c)}})},lk={configure:function(a){a.add(a.internal(),cn,Ca.unwrap);a.add(a.internal(),dn,Ca.remove);
a.add(a.internal(),en,Ca.remove);a.add(a.internal(),jb.quasiBlock(),yI);a.add(a.output(),cn,Ca.unwrap);a.add(a.output(),dn,Ca.remove);a.add(a.output(),en,Ca.remove);a.add(a.output(),jb.styleSpan(),h7);a.add(a.output(),jb.quasiBlock(),yI);a.addExcludedContext(["."+cn,"."+dn,"."+en,"."+jb.styleSpan()]);a.addSelector(a.input(),"[style]",zI);a.addSelector(a.input(),"td,th",AI);a.addSelector(a.paste(),"[style]",zI);a.addSelector(a.paste(),"td,th",AI);a.addSelector(a.output(),"strong,b,em,i,u,s,strike,del,sup,sub",
Ca.removeIfEmpty);g7.configure(a)},markerWrapper:f.constant(cn),markerExtra:f.constant(dn),markerGeneral:f.constant(en)},gf={resolve:lb.create("ephox-cereal").resolve},dt=gf.resolve("start-cursor"),et=gf.resolve("finish-cursor"),ft="data-"+gf.resolve("start-offset"),gt="data-"+gf.resolve("finish-offset"),BI=function(a){m.remove(a,ft);m.remove(a,gt);ea.remove(a,[dt,et]);""===m.get(a,"class")&&m.remove(a,"class")},CI=function(a,b){var c=m.get(a,b);return parseInt(c,10)},kh={setInfo:function(a,b,c,d){k.each([a,
b],BI);n.add(a,dt);n.add(b,et);m.set(a,ft,c);m.set(b,gt,d)},clearInfo:BI,findStart:function(a){return J.descendant(a,"."+dt).map(function(a){return{token:f.constant(a),offset:f.constant(CI(a,ft))}})},findFinish:function(a){return J.descendant(a,"."+et).map(function(a){return{token:f.constant(a),offset:f.constant(CI(a,gt))}})}},DI=C.immutable("token","remove","extra"),EI={use:function(a,b){return DI(b,f.noop,[])},extract:function(a){return{remove:f.curry(kh.clearInfo,a),element:f.constant(a)}}},FI=
gf.resolve("pre-cursor-token"),GI=gf.resolve("post-cursor-token"),HI=gf.resolve("first-cursor-token"),ht=function(){var a=q.fromTag("span");t.set(a,"display","none");return a},it=function(a){var b=f.curry(B.remove,a);return DI(a,b,[a])},j7=function(a){return n.has(a,FI)?y.nextSibling(a):n.has(a,GI)?y.prevSibling(a):n.has(a,HI)?y.parent(a):h.none()},Zf={pre:function(a,b){var c=ht();v.before(b,c);n.add(c,FI);return it(c)},post:function(a,b){var c=ht();v.after(b,c);n.add(c,GI);return it(c)},first:function(a,
b){var c=ht();v.prepend(a,c);n.add(c,HI);return it(c)},extract:function(a){return j7(a).map(function(b){return{remove:f.curry(B.remove,a),element:f.constant(b)}})}},k7=C.immutable("element","offset"),l7=C.immutable("startContainer","startOffset","endContainer","endOffset"),m7={point:k7,range:l7},jt=gf.resolve("text-node-wrapper"),II=function(a){return n.has(a,jt)},hf={create:function(){var a=q.fromTag("span");n.add(a,jt);return a},findIn:function(a){return X.children(a,"."+jt)},isWrap:II,inWrap:function(a){return y.parent(a).fold(f.constant(!1),
function(a){return II(a)})},child:function(a){return y.child(a,0).fold(function(){var b=q.fromText("");v.append(a,b);return b},f.identity)}},n7=function(a){a=hf.findIn(a);k.each(a,function(a){0===y.children(a).length&&v.before(a,q.fromText(""));B.unwrap(a)})},o7=function(a){y.parent(a).each(B.unwrap)},JI=function(a){(hf.inWrap(a)?o7:n7)(a)},kt=function(a,b,c,d){return L.eq(a,b)?Zf.first(a,b):A.isElement(b)&&!d(b)?EI.use(a,b):c(a,b)},lt={extractFrom:function(a,b){a.remove();b.remove();var c=function(a){var b=
hf.isWrap(a)?B.unwrap:JI,c=hf.isWrap(a)?hf.child(a):a;return{target:f.constant(c),unwrap:f.curry(b,a)}},d=c(a.element()),e=c(b.element()),g=m7.range(d.target(),a.offset(),e.target(),b.offset());return{remove:function(){d.unwrap();e.unwrap()},getSelection:function(){return g}}},same:function(a,b,c,d,e){a=kt(a,b,Zf.pre,e);kh.setInfo(a.token(),a.token(),c,d);return[a]},diff:function(a,b,c,d,e,g){var f=Bc.after(b,c,d,e),h=f?Zf.pre:Zf.post;b=kt(a,b,f?Zf.post:Zf.pre,g);a=kt(a,d,h,g);kh.setInfo(b.token(),
a.token(),c,e);return[b,a]}},p7=function(a){return y.parent(a).fold(f.constant(!1),function(a){return hf.isWrap(a)})},q7=function(a){return k.foldr(a,function(a,c){if(A.isText(c)&&!p7(c)){var d=hf.create();v.wrap(c,d);return[d].concat(a)}return a},[])},r7=function(a){var b=y.children(a),b=0<b.length?b:[a];return hf.isWrap(a)?[]:q7(b)},s7=function(a){a.remove()},KI={combine:function(a,b){var c=k.bind(a,r7),d=k.bind(b,function(a){return a.extra()});return{extra:f.constant(d),wrappers:f.constant(c),
remove:function(){k.each(b,function(a){kh.clearInfo(a.token())});k.each(a,JI);k.each(b,s7)}}}},t7=function(a,b,c,d,e,g){var f=[b,d];a=lt.diff(a,b,c,d,e,g);return KI.combine(f,a)},u7=function(a,b,c,d,e,g){d=[b];a=lt.same(a,b,c,e,g);return KI.combine(d,a)},LI=function(a,b){return b(a).map(function(a){var b=Zf.extract(a.token()).getOrThunk(function(){return EI.extract(a.token())});return{remove:b.remove,offset:a.offset,element:b.element}})},v7=function(a){return LI(a,kh.findStart).bind(function(b){return LI(a,
kh.findFinish).map(function(a){return{startMarker:f.constant(b),finishMarker:f.constant(a)}})})},MI={insertAt:function(a,b,c,d,e,g){return(L.eq(b,d)?u7:t7)(a,b,c,d,e,g)},extract:function(a){return v7(a).map(function(a){return lt.extractFrom(a.startMarker(),a.finishMarker())})}},NI={insert:function(a,b,c){a=MI.insertAt(a,b.startContainer(),b.startOffset(),b.endContainer(),b.endOffset(),c);return{extra:a.extra,wrappers:a.wrappers,remove:a.remove}},extract:function(a){return MI.extract(a).map(function(a){return{remove:a.remove,
getSelection:a.getSelection}})}},OI=function(a,b){k.each(a,function(a){n.add(a,b)})},w7=function(a){var b=function(){return y.parent(a).map(function(a){return"false"===m.get(a,"contenteditable")}).getOr(!1)};return"false"===m.get(a,"contenteditable")||b()},mk={extract:function(a){return NI.extract(a)},getSelection:function(a){var b=a.getSelection();a=b.startContainer();var c=b.startOffset(),d=b.endContainer(),b=b.endOffset(),e=ka.getEnd(a),g=ka.getEnd(d);return 0<=c&&c<=e&&0<=b&&b<=g?{start:f.constant(da.on(a,
c)),finish:f.constant(da.on(d,b))}:{start:f.constant(da.before(a)),finish:f.constant(da.before(a))}},remove:function(a){a.remove()},serialise:function(a,b){return Qg.path(a,b.startContainer()).bind(function(c){return Qg.path(a,b.endContainer()).bind(function(d){var e=ec.deep(a);return Qg.follow(e,c).bind(function(a){return Qg.follow(e,d).map(function(c){c={startContainer:f.constant(a),startOffset:f.constant(b.startOffset()),endContainer:f.constant(c),endOffset:f.constant(b.endOffset())};c=NI.insert(e,
c,w7);OI(c.wrappers(),lk.markerWrapper());OI(c.extra(),lk.markerExtra());return ll.writeChildren(e)})})})})}},fn=C.immutable("word","left","right"),PI=["'","\u2018","\u2019"],x7=k.bind(PI,function(a){return k.map(["twas"],function(b){return a+b})}),y7=function(a){var b=a.word();return fn(b.substring(1),h.some(b.charAt(0)),a.right())},gn=function(a){return k.contains(PI,a)},z7=function(a){var b=a.word(),c=k.exists(x7,function(a){return-1<b.indexOf(a)})?2:1,d=b.substring(0,c);return k.forall(d,gn)&&
!gn(b.charAt(c))?y7(a):a},QI=function(a){var b=a.word();2<=b.length&&gn(b.charAt(b.length-1))&&!gn(b.charAt(b.length-2))&&(b=a.word(),a=fn(b.substring(0,b.length-1),a.left(),h.some(b.charAt(b.length-1))));return z7(a)},A7={scope:QI,text:function(a){a=fn(a,h.none(),h.none());return QI(a).word()}},RI={words:function(a){var b=db.unsafetoken(db.wordchar()+"+"),b=Dm.findall(a,b),c=a.length;return k.map(b,function(b){var e=b.start(),g=b.finish();b=a.substring(e,g);e=0<e?h.some(a.charAt(e-1)):h.none();g=
g<c?h.some(a.charAt(g)):h.none();b=fn(b,e,g);return A7.scope(b)})}},SI={identify:function(a){return RI.words(a)},isWord:function(a,b){return!Zi.hasBreak(b)}},B7=Sa(),C7={identify:function(a){return SI.identify(a)},isWord:function(a){return SI.isWord(B7,a)}},D7=Y.detect(),TI=function(a,b){return A.isText(a)?h.some(S.point(a,b)):0===y.children(a).length?h.none():y.child(a,b).bind(function(a){return TI(a,0)})},E7=function(a,b,c){a.on(function(a){Jc.expandWord(a,b,c)})},F7=function(a,b){var c=a.dom().createRange();
c.setStart(b.startContainer().dom(),b.startOffset());c.setEnd(b.endContainer().dom(),b.endOffset());return C7.isWord(c.cloneContents().textContent)},kw={correct:function(a){a.getSelection().each(function(b){a.doc().each(function(c){F7(c,b)&&TI(b.startContainer(),b.startOffset()).each(function(c){(!b.collapsed()||a.isInline()&&D7.browser.isIE())&&E7(a,c.element(),c.offset())})})})},expand:function(a){a.on(Jc.expand);a.backupSelection()}},UI=function(a,b){return a.bind(function(a){return a.fold(function(){return za.pure(a)},
function(a){return b(a)})})},VI=function(a){return za.pure(H.value(a))},WI=function(a){return za.nu(function(b){a.then(function(a){b(H.value(a))},function(a){b(H.error(a))})})},XI=function(a){return a.map(H.value)},Pa={fromPromise:WI,error:function(a){return za.pure(H.error(a))},map:function(a,b){return a.map(function(a){return a.map(b)})},bind:UI,pure:VI,fromChain:function(a){return function(b){return k.foldl(a,function(a,b){return UI(a,function(a){return b(a)})},VI(b))}},fromFuture:XI,pToFR:function(a){return f.compose(WI,
a)},fToFR:function(a){return f.compose(XI,a)},effect:function(a){return function(b){b.each(function(){a()});return b}}},G7=function(a){return za.nu(function(b){if(!0===a.dom().complete)b(H.value(a));else{var c=Fo();YI(c,b);c.monitor(a)}})},YI=function(a,b){a.events.success.bind(function(a){b(H.value(a.image()))});a.events.fail.bind(function(a){b(H.error("Unable to download image "+a.url()))})},nk={load:function(a){return za.nu(function(b){var c=Fo();YI(c,b);c.go(a)})},loadIfNeeded:function(a){return!0===
a.dom().complete||void 0===m.get(a,"src")?Pa.pure(a):G7(a)}},jg={text:function(a){a=y.children(a);return ml.toText(a)},html:function(a,b){var c=q.fromTag("div");v.append(c,a);return Mh.output(b,c)},fragment:function(a,b){if(b.collapsed())return h.none();var c=Wb.exact(b.startContainer(),b.startOffset(),b.endContainer(),b.endOffset()),c=Fa.clone(a,c);return h.some(c)}},lw=Y.detect(),H7={get:f.constant({onShortcut:function(a,b){a.bind(b)}})},ZI="data-"+x.resolve("context-type"),Ho={nu:function(a,b,
c){a=bb.item(a,c.value(),c.text());m.set(a,ZI,b);m.setAll(a,c.attrs());return a},type:function(a){return m.get(a,ZI)}},EQ={derive:function(a,b,c){b=k.map(b,function(b){var e=c[b],g=e.primary(),f=e.submenus();return k.map(g,function(c){var e=Ho.nu(a,b,c),g=f.hasOwnProperty(c.value())&&void 0!==f[c.value()]?[ja.resolve("menu-more")]:[],k=void 0!==f[c.value()];Q.menuItem(e,h.some(c.text()),h.some(k));ea.add(e,g);return e})});b=k.filter(b,function(a){return 0<a.length});return k.flatten(EA.intersperseThunk(b,
function(){return[bb.separator(a)]}))}},FQ={derive:function(a,b,c){return k.bind(b,function(b){var e=c[b].secondary();return k.map(e,function(c){var e=bb.submenu(a,c.value(),[]);Q.menu(e,u.get(c.textkey()));c=k.map(c.items(),function(c){var e=Ho.nu(a,b,c);Q.menuItem(e,h.some(c.text()),h.none());return e});K.append(e,c);return e})})}},$I={mainmenu:function(a,b,c,d,e,g){return rf.main(a,b,c,d,e,g)},submenu:function(a,b,c,d,e){return rf.submenu(a,b,c,d,e)}},Lk={makeshift:function(a,b,c,d,e,g,h,k,m){return{recalculate:f.noop,
isAnchored:f.constant(!1),place:function(f){var n=b.uiOrigin(),q=a.isInline()?Ba(c,d):Ed.translate(n,c,d),s=Ve(q.left(),q.top(),e,g),q=h.isRtl()?rg.allRtl():rg.all(),n=mg.box(s,n);f=f.panel();Hk.simple(n,f,k,q,void 0===m?{}:m)}}},mainmenu:function(a,b,c,d){return $I.mainmenu(a.uiContainer,a.bounds,b,c,d)},submenu:function(a,b,c){return $I.submenu(a.uiContainer,a.bounds,b,c)}},aJ=C.immutable("target","x","y"),bJ=Y.detect().browser,HQ=function(a,b,c,d,e,g){c=b.doc().getOrDie();var l=DQ(c),p=Ba(0,0);
c=H7.get();var m=Mk.fromUnsafe(b),n=Sd.menuSystem(e.uiContainer,f.constant("context"),Gh.menu({focusActive:!0})),q=Sd.popup(n.sandbox,e.bounds,a,[],sf.editorContainer());g=P.div();var s=q.initMenu(g),u=function(a){l.update(a.target());var b=D(),c=[n.sandbox()];if(0<b.all.length){var d=a.x(),e=a.y();p=Ba(d,e);s.setContent(b.main);B(s);s.open();n.open(c,b.main);return h.some(a)}return h.none()},v=function(a){return b.getSelection().bind(function(a){return function(){return ib.selectionContains(a).fold(function(){return A.isText(a.startContainer())?
y.parent(a.startContainer()):h.some(a.startContainer())},function(a){return h.some(a)})}().map(function(a){var c=Na.viewport(a),c=b.translateScreen(c.left(),c.top());return aJ(a,c.left(),c.top())})})},x=function(a){var c;c=a.original();c=bJ.isIE()&&10>=bJ.version.major?Ba(c.raw().clientX,c.raw().clientY):Ba(c.x(),c.y());c=b.translateScreen(c.left(),c.top());return aJ(a.original().target(),c.left(),c.top())},B=function(c){var d;d=fg(14,10);d=Lk.makeshift(b,e,p.left(),p.top(),0,0,a.uiDirection(),d);
e.show(c,d)},C=function(){s.close();var a=D();k.each(a.all,function(a){t.set(a,"display","none")});m.end()};s.events.close.bind(function(){n.close()});var D=function(){var a=l.all();return{all:a,main:a[0],submenus:a.slice(1)}};b.events.contextmenu.bind(function(a){var b=x(a);u(b).orThunk(function(){return v().bind(u)}).each(function(b){a.original().raw().preventDefault()})});c.onShortcut(d.events.context,function(){v().each(u)});n.events.open.bind(function(){var a=l.structure(),b=D();n.update(a);
q.registerMenus(b.submenus);B(s);n.setActiveMenu(b.main)});n.events.itemAction.bind(function(a){l.handle(a.item())});n.events.itemHover.bind(function(a){m.begin()});n.events.reveal.bind(function(a){ha.expanded(a.trigger());q.expandMenu(a.menu(),a.trigger(),s);B(s)});n.events.collapse.bind(function(a){q.hideMenu(a.menu());a.item().each(ha.collapsed)});n.events.itemAction.bind(C);n.events.escaped.bind(function(){C();b.focus()});b.events.mousedown.bind(C);return{register:l.register,destroy:function(){s.destroy();
n.destroy()}}},mt={resolve:yl.css("ephox-darwin").resolve},cJ=mt.resolve("selected"),I7="."+cJ,dJ=mt.resolve("first-selected"),J7="."+dJ,eJ=mt.resolve("last-selected"),K7="."+eJ,sd={selected:f.constant(cJ),selectedSelector:f.constant(I7),firstSelected:f.constant(dJ),firstSelectedSelector:f.constant(J7),lastSelected:f.constant(eJ),lastSelectedSelector:f.constant(K7)},JQ=function(a){B.empty(a);v.append(a,q.fromTag("br"))},L7=Y.detect().browser,nt=function(a){var b=q.fromHtml("<p><br></p>");v.append(a,
b);return h.some(b)},M7=function(a){var b=y.children(a);return 0===b.length?nt(a):1===b.length&&"br"===A.name(b[0])?(B.empty(a),nt(a)):h.none()},N7=function(a){return 1===a.dom().childNodes.length?y.firstChild(a).fold(f.constant(!1),function(a){return"br"===A.name(a)&&m.has(a,"_moz_editor_bogus_node")}):!1},O7=function(a){return M7(a).orThunk(function(){var b;L7.isFirefox()&&N7(a)?(B.empty(a),b=nt(a)):b=h.none();return b})},MQ={listen:function(a){a.events.nodeChanged.bind(function(b){b=b.node();a.isRoot(b)&&
O7(b).each(function(b){a.setSelection(b,0,b,0)})})}},mw={onNodeChange:function(a,b){a.events.nodeChanged.bind(function(c){c.prev().each(function(c){Ac.withinTransient(c,c).each(function(c){b.atomic(function(){Ac.cleanup(a.isRoot,c)})})})});return{destroy:f.noop}},preLinebreak:function(a){a.getSelection().each(function(b){y.parent(b.startContainer()).each(function(b){Ac.withinTransient(b,b).each(function(b){Ac.cleanupNonEmpty(a.isRoot,b)})})})}},jf=function(a,b){var c=b.original(),c=c.raw().which||
c.raw().keycode;return k.contains(a,c)},P7=f.curry(jf,N.ENTER()),Q7=f.curry(jf,N.SPACE()),R7=f.curry(jf,N.BACKSPACE()),S7=f.curry(jf,N.DEL()),T7=f.curry(jf,N.LEFT().concat(N.UP())),U7=f.curry(jf,N.RIGHT().concat(N.DOWN())),V7=f.curry(jf,N.TAB()),W7=f.curry(jf,N.ESCAPE()),ld={isEnter:P7,isSpace:Q7,isBackspace:R7,isDel:S7,isTab:V7,isLeftOrUp:T7,isRightOrDown:U7,isEscape:W7},fJ=function(a,b,c,d){return yb.isCollapsed(a)?b(a.startContainer(),a.startOffset(),d):c(a.startContainer(),a.startOffset(),a.endContainer(),
a.endOffset(),d)},nw={soft:function(a,b){return fJ(a,Bs.soft,Es.soft,b)},hard:function(a,b){return fJ(a,Bs.hard,Es.hard,b)}},gJ=function(a,b){k.each(b,function(b){var d=b.find(a);b.cleaner(d,a)})},ok={filterHtml:function(a,b){var c=q.fromTag("div");c.dom().innerHTML=a;gJ(c,b);return ll.writeChildren(c)},filterDom:gJ},hJ=function(a,b){var c=S.point(a,b);return A.isText(a)?Xf.scanRight(a,b).getOr(c):c},qw={handle:function(a){var b=hJ(a.startContainer(),a.startOffset());a=hJ(a.endContainer(),a.endOffset());
return cc(b.element(),b.offset(),a.element(),a.offset())}},$f=ma.generate([{onText:["element","offset"]},{firstText:["parent","child","offset"]},{beforeText:["parent","child","offset"]},{afterText:["parent","child"," offset"]},{lastText:["parent","child","offset"]},{onElement:["element","offset"]}]),iJ={detect:function(a,b){if(A.isText(a))return $f.onText(a,b);var c=y.children(a);return 0===c.length?$f.onElement(a,b):0===b&&A.isText(c[0])?(0,$f.firstText)(a,c[0],0):b===c.length&&A.isText(c[c.length-
1])?(c=c[b-1],(0,$f.lastText)(a,c,ka.getEnd(c))):b<c.length&&A.isText(c[b])?(0,$f.beforeText)(a,c[b],0):0<b&&A.isText(c[b-1])?(c=c[b],(0,$f.afterText)(a,c,ka.getEnd(c))):$f.onElement(a,b)},cata:function(a,b,c,d,e,g,f){return a.fold(b,c,d,e,g,f)}},ag=C.immutable("startContainer","startOffset","endContainer","endOffset"),jJ=lb.create("ephox-newt").resolve("cursor-marker"),nj={generator:function(a){var b=null;return{getOrInit:function(){var c;null===b?(c=q.fromTag("span",a.dom()),n.add(c,jJ),t.set(c,
"display","none")):c=b;return b=c}}},isCursor:function(a){return n.has(a,jJ)}},kJ=function(a,b){return{element:f.constant(b),backup:f.identity,restore:function(){var b=ag(a,0,a,0);return h.some(b)}}},lJ=function(a){a=y.children(a);return k.filter(a,f.not(nj.isCursor))},X7=function(a,b){return{element:f.constant(b),backup:f.identity,restore:function(){var b=lJ(a).length,b=ag(a,b,a,b);return h.some(b)}}},Y7=function(a){return y.parent(a).bind(function(b){var c=lJ(b);return k.findIndex(c,function(b){return L.eq(a,
b)}).map(function(a){return ag(b,a,b,a)})})},Z7=function(a,b){return{backup:function(){v.before(a,b)},restore:function(){var a=y.nextSibling(b);B.remove(b);return a.map(function(a){return Y7(a).getOrThunk(function(){return ag(a,0,a,0)})})},element:f.constant(b)}},$7={place:function(a,b,c){var d=y.children(a);return 0===d.length?kJ(a,c):d.length===b?X7(a,c):0===b?kJ(a,c):b>d.length?ow(c):Z7(d[b],c)}},mJ=function(a,b){return b().fold(function(){return nj.generator(a).getOrInit()},function(a){return a.element()})},
hn=function(a){a=a();a.backup();return h.some(a)},nJ=function(a,b){var c=a.or(b),d=b.or(a);return c.bind(function(a){return d.map(function(b){return ag(a.startContainer(),a.startOffset(),b.startContainer(),b.startOffset())})})},a8=function(a,b,c){var d=h.none(),e=h.none();return{backup:function(){e=hn(b);d=hn(c)},restore:function(){return d.bind(function(a){return e.bind(function(b){var c=a.restore();b=b.restore();return nJ(c,b)})})},element:f.curry(mJ,a,function(){return e})}},b8=function(a,b,c){var d=
h.none(),e=h.none();return{backup:function(){d=hn(c);e=hn(b)},restore:function(){return e.bind(function(a){return d.bind(function(b){var c=a.restore();b=b.restore();return nJ(b,c)})})},element:f.curry(mJ,a,function(){return d})}},c8=function(a,b,c){return{backup:function(){v.before(b,c)},restore:function(){var b=Lb.after(c,a).map(function(a){return ag(a,0,a,0)});B.remove(c);return b},element:f.constant(c)}},oJ=function(a,b,c){return{backup:function(){v.after(b,c)},restore:function(){var b=Lb.before(c,
a).map(function(a){var b=ka.getEnd(a);return ag(a,b,a,b)});B.remove(c);return b},element:f.constant(c)}},pJ={place:function(a,b,c,d){b=Vg.split(b,c);return Vg.position(b).fold(function(){return ow(d)},function(b){return c8(a,b,d)},function(b,c){return oJ(a,b,d)},function(b){return oJ(a,b,d)})}},ot=C.immutable("result","cursor"),d8=function(a,b,c){var d=function(b,d,f){return pJ.place(a,d,f,c)};return iJ.cata(b,function(b,d){return pJ.place(a,b,d,c)},d,d,d,d,function(a,b){return $7.place(a,b,c)})},
pt=function(a,b,c,d){d=d.getOrInit();b=iJ.detect(b,c);return d8(a,b,d)},qJ=function(a,b){a.backup();var c=b(a.element());return ot(c,a.restore())},jn={inspect:pt,runOnRange:function(a,b,c,d,e,g){var f=y.owner(b),h=(Bc.after(b,c,d,e)?b8:a8)(f,function(){return pt(a,d,e,nj.generator(f))},function(){return pt(a,b,c,nj.generator(f))});return qJ(h,g)},runOnCursor:qJ},e8=function(a,b,c,d,e,g){return jn.runOnRange(a,b,e>c?c:e,b,e>c?e:c,g)},f8=jn.runOnRange,rJ={point:function(a,b,c,d){var e=y.owner(b),e=
nj.generator(e);a=jn.inspect(a,b,c,e);return jn.runOnCursor(a,d)},range:function(a,b,c){return(L.eq(b.startContainer(),b.endContainer())?e8:f8)(a,b.startContainer(),b.startOffset(),b.endContainer(),b.endOffset(),c)}},g8=function(a,b){return function(a,d){var e=d(b);return ot(e,h.none())}},h8=function(a,b){return function(c,d){return rJ.point(a,b.startContainer(),b.startOffset(),sJ(c,d))}},i8=function(a,b){return function(c,d){return rJ.range(a,b,sJ(c,d))}},sJ=function(a,b){return function(c){n.add(c,
a);return b(c)}},pw={diagnose:function(a,b){var c=b.startContainer(),d=b.endContainer(),e=y.owner(c);return ca.inBody(c)&&ca.inBody(d)&&T.active(e).forall(ca.inBody)?b.collapsed()?h8(a,b):i8(a,b):g8(a,b.startContainer())}},qt={fold:function(a,b){return k.foldr(a,function(a,d){return a.bind(function(a){return b(d).map(function(b){return[b].concat(a)})})},H.value([]))}},tJ=function(a,b,c){R.isObject(a)&&b.hasOwnProperty(a.label)&&c(b[a.label]);return a},uJ={mapGroup:function(a,b){return tJ(a,b,function(b){return a.label=
b})},mapMenu:function(a,b){return tJ(a,b,function(b){b.hasOwnProperty("label")&&(a.label=b.label);b.hasOwnProperty("menuLabel")&&(a.menuLabel=b.menuLabel)})}},j8=function(a,b,c){b=b.groups();return b.hasOwnProperty(a)?H.value(b[a]):Zd("Unknown group: "+a,["No binding for group: "+a+". Known groups: "+W.keys(b)])},k8=function(a,b,c){a=uJ.mapGroup(a,c.groupMappings());a=R.isArray(a)?{label:"",items:a}:a;return H.value(a)},l8=function(a,b,c,d,e,g){return qt.fold(b,function(a){return e.item(a,c,d,e,g)}).map(function(b){return{category:a,
items:b}})},m8=function(a,b,c,d,e){b=b.commands();return b.hasOwnProperty(a)?H.value(b[a]):Zd("Unknown command: "+a,["No binding for command: "+a+". Known commands: "+W.keys(b)])},o8=function(a,b,c,d,e){return(void 0===a.items?n8:d.menu)(a,b,c,d,e)},n8=function(a,b,c,d,e){return void 0===c?Zd("You need a custom action factory to use this",[]):c(a)},p8=function(a){return k.forall(a,R.isArray)?H.value(a):k.exists(a,R.isArray)?Zd("Menu items must be a single array or an array where every element is an array",
["Menu items must be a single array or an array where every element is an array.","Contents of menu: "+$c.stringify(a)]):H.value([a])},vJ=W.map({group:function(a,b,c,d,e){return(R.isString(a)?j8:k8)(a,b,e).bind(function(a){return l8(a.label,a.items,b,c,d,e)})},item:function(a,b,c,d,e){return(R.isString(a)?m8:o8)(a,b,c,d,e)},menu:function(a,b,c,d,e){if(void 0===a.label||void 0===a.items||0===a.items.length)return Zd("Invalid toolbar configuration for menu",["Invalid menu. Must be an object containing label and items.",
"Contents of menu: "+$c.stringify(a)]);var g=uJ.mapMenu(a,e.menuMappings()),f=g.menuLabel?g.menuLabel:g.label;return p8(g.items).bind(function(a){return qt.fold(a,function(a){return d.group(a,b,c,d,e)})}).map(function(b){return{name:a.id,textkey:a.label,menuTextkey:f,type:"submenu",contentFocus:!1,enabled:!0,icon:a.icon,commands:b}})}},function(a,b){return function(){if(5!==arguments.length)throw"Incorrect usage of builder ["+b+"]. Only given: "+arguments.length;return a.apply(null,arguments)}}),
q8=function(a){if(void 0===a.items)return Zd("Invalid group, no items specified",["Invalid group, no items specified: "+a.label]);var b=a.items;return R.isArray(b)?(a=k.bind(b,function(a,b){return R.isString(a)||R.isArray(a)||void 0!==a.label&&R.isArray(a.items)?[]:["Invalid group at index: "+b+". Must be either a string, an array or an object containing label and items. Was: "+$c.stringify(a)]}),0===a.length?H.value(b):Zd("Invalid toolbar configuration for groups",a)):Zd("Invalid toolbar, items is not an array",
["Invalid toolbar, items is not an array: "+$c.stringify(a)])},wJ={build:function(a,b,c,d){return q8(a).bind(function(a){return qt.fold(a,function(a){return vJ.group(a,b,c,vJ,d)})})}},xJ=function(){return Mb.isSmallTouch()},yJ=function(a,b){return{feature:a.feature,command:a.command,type:"pre_flatmenu",items:b}},Jo={shouldFlatten:xJ,never:f.constant("never_flatten"),streamline:function(a){var b=function(a){return k.bind(a,function(a){return"never_flatten"===a.flatten?[a]:"submenu"===a.type?k.bind(a.commands,
function(a){return b(a.items)}):"pre_flatmenu"===a.type?k.map(a.items,function(b){return{feature:a.feature,type:"general",command:b}}):[a]})};return k.map(a,function(a){var d=b(a.items);return{category:a.category,items:d}})},setup:function(a,b){var c=yJ(a,b);return Mb.choice([{predicate:xJ,value:f.constant(c)}],a)},make:yJ},Nh=function(a,b){return{predicate:a,value:f.constant(b)}},zJ=function(){return{make:function(a){var b=function(b){return Zd(u.translate("errors.toolbar.missing.custom.string",
b),['The required property "'+b+'" for a custom command must be a string. It was: '+$c.stringify(a[b]),"Properties specified in custom command: "+W.keys(a).join(", "),"Definition specified for custom command: "+$c.stringify(a)])};if(!R.isString(a.id))return b("id");if(!R.isString(a.text))return b("text");b={};b[a.id]=f.constant({general:function(){return{toggle:!1,value:Cl.cssSanitise(a.id),textkey:u.raw(a.text),action:a.action,updateGeneral:f.identity,icon:void 0!==a.icon?a.icon:!1}}});return H.value({feature:h.some(b),
command:a.id,type:"general"})}}}(),r8=function(a,b){return a.bind(function(a){return b.map(function(b){return a.concat(b)})})},AJ={groupMappings:f.constant({"group.undo":"category.undo","group.insert":"category.insert","group.style":"category.style","group.language":"category.language","group.emphasis":"category.emphasis","group.align":"category.align","group.listindent":"category.listindent","group.format":"category.format","group.tools":"category.tools","group.table":"category.table","group.image":"category.image"}),
menuMappings:f.constant({"menu.insert":{label:"insert.menu-button",menuLabel:"insert.menu"},"menu.font":{label:"font.menu-button",menuLabel:"font.menu"}})},Oh={config:function(a,b,c){return function(d,e,g,h){d=ZQ(d,e,g,h,c);d=Z.deepMerge(d,b);h={"table-tools":{feature:h.table().ui(),command:"cell",type:"palette"},"image-tools":{feature:h.imageEdit().ui(),command:"edit",type:"palette"},"crop-tools":{feature:h.imageEdit().ui(),command:"crop",type:"palette"}};d={forItems:f.constant({commands:f.constant(d),
groups:sw}),forContextual:f.constant({commands:f.constant(h),groups:sw})};h=wJ.build({items:a.items},d.forItems(),zJ.make,AJ);d=wJ.build({items:a.contextual},d.forContextual(),zJ.make,AJ);e=Y.detect().deviceType.isTouch()?Jo.streamline:f.identity;return r8(h,d).map(e)}}},s8=function(a){return function(a){setTimeout(function(){a()},0)}},t8={resume:function(a){a.focus();var b=q.fromDom(a.document.body);(T.active().exists(function(a){return k.contains(["input","textarea"],A.name(a))})?s8(b):f.apply)(function(){T.active().each(T.blur);
T.focus(b)})}},kn={safeParse:function(a,b){var c=parseInt(m.get(a,b),10);return isNaN(c)?0:c}},u8=function(a){return{left:a.left,top:a.top,width:2,height:a.height}},v8=function(a){if(a.collapsed){var b=q.fromDom(a.startContainer);return y.parent(b).bind(function(c){c=Wb.exact(b,a.startOffset,c,ka.getEnd(c));return Fa.getFirstRect(a.startContainer.ownerDocument.defaultView,c).map(function(a){return W.map(a,f.apply)}).map(u8).map(k.pure)}).getOr([])}return a.getClientRects()},Mo={getRectangles:function(a){a=
a.getSelection();return void 0!==a&&0<a.rangeCount?v8(a.getRangeAt(0)):[]}},td={resolve:lb.create("ephox-nomad").resolve},rt="data-"+td.resolve("last-outer-height"),w8=function(a){return{top:f.constant(a.top),bottom:f.constant(a.top+a.height)}},x8=function(a){a=Mo.getRectangles(a);return 0<a.length?h.some(a[0]).map(w8):h.none()},y8=function(a){a=kn.safeParse(a,rt);var b=window.innerHeight;return a>b?h.some(a-b):h.none()},cR={setup:function(a,b){var c=q.fromDom(b.document.body),d=E.bind(q.fromDom(a),
"resize",function(){y8(c).each(function(a){x8(b).each(function(c){c=c.top()>b.innerHeight||c.bottom()>b.innerHeight?Math.min(a,c.bottom()-b.innerHeight+50):0;0!==c&&b.scrollTo(b.pageXOffset,b.pageYOffset+c)})});m.set(c,rt,window.innerHeight)});m.set(c,rt,window.innerHeight);return{toEditing:function(){t8.resume(b)},destroy:function(){d.unbind()}}}},BJ=function(a){var b;b=new (Vb.getOrDie("Event"))("Event");b.initEvent(a,!0,!0);return b},tw={event:function(a,b){var c=BJ(b);a.target().dom().dispatchEvent(c)},
target:function(a,b){var c=BJ(b);a.dom().dispatchEvent(c)}},CJ=function(){var a=window.matchMedia("(orientation: portrait)").matches;return{isPortrait:f.constant(a)}},kg={get:CJ,onChange:function(){var a=D.create({ready:s(["orientation"]),change:s(["orientation"])}),b=q.fromDom(window),c=null,d=E.bind(b,"orientationchange",function(){clearInterval(c);var b=CJ();a.trigger.change(b);e(function(){a.trigger.ready(b)})}),e=function(a){clearInterval(c);var b=window.innerHeight,d=0;c=setInterval(function(){b!==
window.innerHeight?(clearInterval(c),a(h.some(window.innerHeight))):20<d&&(clearInterval(c),a(h.none()));d++},50)};return{onAdjustment:e,events:a.registry,destroy:function(){d.unbind()}}}},vw={tag:function(){var a=J.first("head").getOrDie(),b=J.first('meta[name="viewport"]').getOrThunk(function(){var b=q.fromTag("meta");m.set(b,"name","viewport");v.append(a,b);return b}),c=m.get(b,"content");return{maximize:function(){m.set(b,"content","width=device-width, initial-scale=1.0, user-scalable=no, maximum-scale=1.0")},
restore:function(){void 0!==c&&null!==c&&0<c.length?m.set(b,"content",c):m.remove(b,"content")}}}},DJ=function(a,b){var c=qa.get(a);return Math.floor(b/100*c)},EJ=function(a,b){return J.ancestor(b,"td,th").fold(function(){return DJ(a,98)},function(a){return DJ(a,94)})},z8=function(a,b,c){var d=b.dom().width;a=EJ(a,b);return Math.max(0,Math.min(a,d+c))},A8=function(a,b,c){return b.dom().height+c},Ko={compare:function(a,b){var c=b.deltaX()-a.deltaX(),d=b.deltaY()-a.deltaY(),e=b.deltaDistance()-a.deltaDistance();
return{deltaX:f.constant(c),deltaY:f.constant(d),deltaDistance:f.constant(e)}},extract:function(a){var b=a.raw().touches;if(2>b.length)return h.none();a=Math.abs(b[0].clientX-b[1].clientX);var b=Math.abs(b[0].clientY-b[1].clientY),c=Math.sqrt(Math.pow(a,2)+Math.pow(b,2));return h.some({deltaX:f.constant(a),deltaY:f.constant(b),deltaDistance:f.constant(c)})},resize:function(a,b,c){var d=t.getRaw(b,"height"),e=t.getRaw(b,"width"),g=0<c.deltaDistance()?1:-1,f=g*Math.abs(c.deltaX()),h=g*Math.abs(c.deltaY());
c=z8(a,b,f);g=A8(a,b,h);f=Math.abs(f)>Math.abs(h);t.setAll(b,{height:f?"auto":g+"px",width:f?c+"px":"auto",transform:"translate3d(0, 0, 0)"});qa.get(b)>EJ(a,b)&&(d.each(function(a){t.set(b,"height",a)}),e.each(function(a){t.set(b,"width",a)}))}},bR=6<=Y.detect().os.version.major,B8=function(a){return a.body().map(function(a){a=t.get(a,"background-color");return void 0!==a&&""!==a?"background-color:"+a+"!important":"background-color:rgb(255,255,255)!important;"}).getOr("background-color:rgb(255,255,255)!important;")},
yw={clobberStyles:function(a,b){var c=function(a){return function(b){var c=m.get(b,"style"),c=void 0===c?"no-styles":c.trim();c!==a&&(m.set(b,"data-ephox-mobile-fullscreen-style",c),m.set(b,"style",a))}},d=X.ancestors(a,"*"),e=k.bind(d,function(a){return X.siblings(a,"*")}),g=B8(b);k.each(e,c("display:none!important;"));k.each(d,c("position:absolute!important;top:0!important;left:0!important;margin:0!important;padding:0!important;width:100%!important;height:100%!important;overflow:visible!important;"+
g))},restoreStyles:function(){var a=X.all("[data-ephox-mobile-fullscreen-style]");k.each(a,function(a){var c=m.get(a,"data-ephox-mobile-fullscreen-style");"no-styles"!==c?m.set(a,"style",c):m.remove(a,"style");m.remove(a,"data-ephox-mobile-fullscreen-style")})}},FJ={input:function(a,b){var c=q.fromTag("input");t.setAll(c,{opacity:"0",position:"absolute",top:"-1000px",left:"-1000px"});v.append(a,c);T.focus(c);b(c);B.remove(c)}},To={refreshInput:function(a){var b=a.dom().selectionStart,c=a.dom().selectionEnd,
d=a.dom().selectionDirection;setTimeout(function(){a.dom().setSelectionRange(b,c,d);T.focus(a)},50)},refresh:function(a){var b=a.getSelection();if(0<b.rangeCount){var c=b.getRangeAt(0);a=a.document.createRange();a.setStart(c.startContainer,c.startOffset);a.setEnd(c.endContainer,c.endOffset);b.removeAllRanges();b.addRange(a)}}},Lo={resume:function(a,b){T.active().each(function(a){L.eq(a,b)||T.blur(a)});a.focus();T.focus(q.fromDom(a.document.body));To.refresh(a)}},mR={stubborn:function(a,b,c,d,e){var g=
pk(e),f=function(){Lo.resume(a,c)},h=E.bind(b,"keydown",function(a){k.contains(["input","textarea"],A.name(a.target()))||(g.closeMenus(void 0),f())});return{toReading:function(){FJ.input(ca.body(),T.blur)},toEditing:f,onToolbarTouch:function(a){},destroy:function(){h.unbind()}}},timid:function(a,b,c,d,e){return{toReading:function(){T.blur(c)},toEditing:function(){Lo.resume(a,c)},onToolbarTouch:function(){T.blur(c)},destroy:f.noop}}},C8={touch:function(a){var b=D.create({touchstart:s(["raw"]),touchend:s(["raw"])}),
c=!1,d=h.none(),e=E.capture(a,"touchstart",function(e){c=!0;d=h.some(E.capture(a,"touchend",function(a){c=!1;b.trigger.touchend(a.raw());g()}));b.trigger.touchstart(e.raw())}),g=function(){d.each(function(a){a.unbind()})};return{isTouching:function(){return c},events:b.registry,destroy:function(){e.unbind();g()}}}},D8=function(a){a=y.documentElement(a);var b=td.resolve("unfocused-selection");return J.descendant(a,"."+b).bind(function(a){return Na.absolute(a).top})},st={toFake:function(a,b,c){var d=
c.isActive,e=!1;return{on:function(){d()||(e=!1,c.update(),b.save(),Fa.clear(a))},off:function(){d()&&!e&&(c.clear(),b.restore())},fakeOut:function(){e=!0;FJ.input(ca.body(),b.restore)}}},inView:function(a,b,c){return function(){return D8(b).map(function(b){var c=a.dom().scrollTop;return 0<b()-c})}},onScroll:function(a,b,c,d){var e=function(){a()||g()},g=function(){d.events.touchend.unbind(e);d.isTouching()?d.events.touchend.bind(e):(c().getOr(!0)?b.off:b.fakeOut)()};return{start:function(){d.events.touchend.unbind(e);
b.on()},stop:g,destroy:function(){d.events.touchend.unbind(e)}}}},GJ=function(a,b){return ab.findMap([{width:320,height:480,keyboard:{portrait:300,landscape:240}},{width:320,height:568,keyboard:{portrait:300,landscape:240}},{width:375,height:667,keyboard:{portrait:305,landscape:240}},{width:414,height:736,keyboard:{portrait:320,landscape:240}},{width:768,height:1024,keyboard:{portrait:320,landscape:400}},{width:1024,height:1366,keyboard:{portrait:380,landscape:460}}],function(c){return a<=c.width&&
b<=c.height?h.some(c.keyboard):h.none()}).getOr({portrait:b/5,landscape:a/4})}(window.screen.width,window.screen.height),HJ=function(a){a=ia.get(a);var b=kg.get().isPortrait(),c=b?GJ.portrait:GJ.landscape;return a-((b?window.screen.height:window.screen.width)-window.innerHeight>c?0:c)},tt={getGreenzone:HJ,updatePadding:function(a,b){var c=HJ(b),c=ia.get(b)-c;t.set(a,"padding-bottom",c+"px")}},IJ=ma.generate([{fixed:["element","offsetY"]},{scroller:["element","offsetY"]}]),kf="data-"+td.resolve("position-y-fixed"),
ut="data-"+td.resolve("scrolling"),JJ="data-"+td.resolve("last-window-height"),KJ=function(a){return kn.safeParse(a,kf)},E8=function(a){var b=KJ(a);return("true"===m.get(a,ut)?IJ.scroller:IJ.fixed)(a,b)},F8=function(a){var b=m.get(a,"style");t.setAll(a,{position:"absolute",top:"0px"});m.set(a,kf,"0px");return{restore:function(){m.set(a,"style",b||"");m.remove(a,kf)}}},G8=function(a,b,c){var d=m.get(c,"style");gc.register(c);t.setAll(c,{position:"absolute",height:b+"px",width:"100%",top:a+"px"});m.set(c,
kf,a+"px");m.set(c,ut,"true");return{restore:function(){gc.deregister(c);m.set(c,"style",d||"");m.remove(c,kf);m.remove(c,ut)}}},LJ=function(a,b){var c=window.innerHeight;m.set(a,JJ,c+"px");return c-b},vt={findFixtures:function(a){a=X.descendants(a,"["+kf+"]");return k.map(a,E8)},takeover:function(a,b,c){var d=F8(c),e=ia.get(c),g=LJ(a,e),h=G8(e,g,a),e=function(){var b=window.innerHeight,c=kn.safeParse(a,JJ);return b>c},k=function(){var d=ia.get(c),e=LJ(a,d);m.set(a,kf,d+"px");t.set(a,"height",e+"px");
tt.updatePadding(b,a)};tt.updatePadding(b,a);return{setViewportOffset:function(b){m.set(a,kf,b+"px");k()},isExpanding:e,isShrinking:f.not(e),refresh:k,restore:function(){d.restore();h.restore()}}},getYFixedData:KJ},H8=function(a,b,c){return Math.abs(a-b)<=c?h.none():a<b?h.some(a+c):h.some(a-c)},ln=function(){var a=null;return{animate:function(b,c,d,e,g,f){clearInterval(a);var h=function(b){clearInterval(a);g(b)};a=setInterval(function(){var f=b();H8(f,c,d).fold(function(){clearInterval(a);g(c)},function(d){e(d,
h);var l=b();if(l!==d||Math.abs(l-c)>Math.abs(f-c))clearInterval(a),g(c)})},f)}}}(),mn="data-"+td.resolve("last-scroll-top"),MJ=function(a){a=t.getRaw(a,"top").getOr(0);return parseInt(a,10)},NJ=function(a){return parseInt(a.dom().scrollTop,10)},OJ=function(a,b){var c=b+vt.getYFixedData(a)+"px";t.set(a,"top",c)},oj={moveScrollAndTop:function(a,b,c){return za.nu(function(d){var e=f.curry(NJ,a);ln.animate(e,b,15,function(b){a.dom().scrollTop=b;t.set(a,"top",MJ(a)+15+"px")},function(e){a.dom().scrollTop=
b;t.set(a,"top",c+"px");d(b)},10)})},moveOnlyScroll:function(a,b){return za.nu(function(c){var d=f.curry(NJ,a);m.set(a,mn,d());var e=Math.abs(b-d()),e=Math.ceil(e/10);ln.animate(d,b,e,function(b,c){kn.safeParse(a,mn)!==a.dom().scrollTop?c(a.dom().scrollTop):(a.dom().scrollTop=b,m.set(a,mn,b))},function(d){a.dom().scrollTop=b;m.set(a,mn,b);c(b)},10)})},moveOnlyTop:function(a,b){return za.nu(function(c){var d=f.curry(MJ,a),e=function(b){t.set(a,"top",b+"px")},g=Math.abs(b-d()),g=Math.ceil(g/10);ln.animate(d,
b,g,e,function(a){e(b);c(b)},10)})},moveWindowScroll:function(a,b,c){return za.nu(function(d){var e=function(c){OJ(a,c);OJ(b,c);window.scrollTo(0,c)};ln.animate(function(){return window.pageYOffset},c,15,e,function(a){e(a);d(a)},10)})}},I8={scrollIntoView:function(a,b,c,d){var e=tt.getGreenzone(b);a=f.curry(To.refresh,a);c>e||d>e?oj.moveOnlyScroll(b,b.dom().scrollTop-e+d).get(a):0>c&&oj.moveOnlyScroll(b,b.dom().scrollTop+c).get(a)}},J8=function(a,b){return a.fold(function(a,d){t.set(a,"top",b+d+"px");
return za.pure(d)},function(a,d){var e=b+d,g=t.getRaw(a,"top").getOr(d),g=e-parseInt(g,10),g=a.dom().scrollTop+g;return oj.moveScrollAndTop(a,g,e)})},PJ={updatePositions:function(a,b){var c=vt.findFixtures(a),c=k.map(c,function(a){return J8(a,b)});return Nc.par(c)}},K8=function(a,b,c,d,e,g){var l=iR(function(c){return oj.moveWindowScroll(a,b,c)}),k=function(){var a=Mo.getRectangles(g);return h.from(a[0]).bind(function(a){var c=a.top-b.dom().scrollTop;return c>window.innerHeight+5||-5>c?h.some({top:f.constant(c),
bottom:f.constant(c+a.height)}):h.none()})},m=zc.adaptable(function(){l.idle(function(){PJ.updatePositions(c,d.pageYOffset).get(function(a){k().each(function(a){b.dom().scrollTop=b.dom().scrollTop+a.top()});l.start(0);e.refresh()})})},1E3),n=E.bind(q.fromDom(d),"scroll",function(){0>d.pageYOffset||m.throttle()});PJ.updatePositions(c,d.pageYOffset).get(f.identity);return{unbind:n.unbind}},lR={setup:function(a){var b=a.cWin(),c=a.ceBody(),d=a.socket(),e=a.toolstrip(),g=a.toolbar(),h=a.contentElement(),
k=a.cursor(),m=a.keyboardType();a=a.isScrolling();var n=vt.takeover(d,c,e),s=m(b,ca.body(),h,e,g),u=kg.onChange();u.events.ready.bind(n.refresh);u.onAdjustment(function(){n.refresh()});var v=E.bind(q.fromDom(window),"resize",function(){n.isExpanding()&&n.refresh()}),x=K8(e,d,ca.body(),window,n,b),y=hR(b,h),A=function(){y.clear()},e=st.toFake(b,k,y),c=st.inView(d,c,g),B=C8.touch(q.fromDom(b)),C=st.onScroll(a,e,c,B);return{toEditing:function(){s.toEditing();A()},toReading:function(){s.toReading()},
onToolbarTouch:function(a){s.onToolbarTouch(a)},refreshSelection:function(){y.isActive()&&y.update()},clearSelection:A,highlightSelection:function(){y.update()},scrollIntoView:function(a,c){I8.scrollIntoView(b,d,a,c)},updateToolbarPadding:f.noop,setViewportOffset:function(a){n.setViewportOffset(a);oj.moveOnlyTop(d,a).get(f.identity)},syncHeight:function(){t.set(h,"height",h.dom().contentWindow.document.body.scrollHeight+"px")},selectionScrollStarted:function(){C.start()},selectionScrollStopped:function(){C.stop()},
destroy:function(){n.restore();u.destroy();x.unbind();v.unbind();s.destroy();C.destroy();B.destroy();y.destroy()}}}},QJ="data-"+td.resolve("horizontal-scroll"),L8=function(a){var b;(b=0<a.dom().scrollTop)||(a.dom().scrollTop=1,b=0!==a.dom().scrollTop,a.dom().scrollTop=0);return b},M8=function(a){var b;(b=0<a.dom().scrollLeft)||(a.dom().scrollLeft=1,b=0!==a.dom().scrollLeft,a.dom().scrollLeft=0);return b},RJ={markAsHorizontal:function(a){m.set(a,QJ,"true")},hasScroll:function(a){return"true"===m.get(a,
QJ)?M8:L8}},N8=function(a){var b=a.dom().scrollTop,c=a.dom().scrollHeight;a=a.dom().offsetHeight;return 0>=b?h.some(1):b+a>=c?h.some(c-a-1):h.none()},SJ=function(a){N8(a).each(function(b){a.dom().scrollTop=b})},wt=function(a,b){return E.bind(a,"touchstart",function(a){b(a.target()).each(SJ)})},Aw={fixEdges:SJ,exclusive:function(a,b){return E.bind(a,"touchmove",function(a){J.closest(a.target(),b).filter(RJ.hasScroll).fold(function(){a.raw().preventDefault()},f.noop)})},link:function(a,b){return wt(a,
f.constant(b))},ancestor:function(a,b){return wt(a,function(a){return J.closest(a,b)})},indirect:wt,markAsHorizontal:function(a){RJ.markAsHorizontal(a)}},TJ=function(a){return q.fromText(Ua.zeroWidth(),a.dom())},UJ={runAndRemove:function(a,b){var c=y.owner(a),d=q.fromTag("span",c.dom());v.append(d,TJ(c));v.prepend(a,d);c=b(d);B.remove(d);return c},enrichAndRemove:function(a,b){var c=TJ(y.owner(a));v.append(a,c);var d=b(a);B.remove(c);return d}},nn=C.immutable("left","top","width","height","direction",
"absolute"),VJ=function(a){return{position:f.constant(wi.find(a)),width:f.constant(qa.getOuter(a)),height:f.constant(ia.getOuter(a))}},O8=function(a){var b=t.get(a,"display");return{backup:function(){t.set(a,"display","inline-block")},restore:function(){t.set(a,"display",b)}}},P8={backup:f.noop,restore:f.noop},Q8=function(a){return A.isElement(a.startContainer())?h.some(a.startContainer()):A.isElement(a.endContainer())?h.some(a.endContainer()):y.parent(a.startContainer()).filter(A.isElement).orThunk(function(){return y.parent(a.endContainer()).filter(A.isElement)})},
R8=function(a,b){return Q8(b).fold(function(){return nn(0,0,0,a.uiDirection(),Ba(0,0))},function(b){var d=wi.find(b),e=qa.get(b),g=ia.get(b),f=Na.absolute(b);return nn(d.left(),d.top(),e,g,a.directionAt(b),f)})},S8=function(a,b,c,d){return pw.diagnose(a,c)(d,function(a){var d=Tb.isVisible(a)||"br"===A.name(a)?P8:O8(a);d.backup();var f;if(ca.inBody(a)){var h=UJ.enrichAndRemove(a,VJ);f=h.position();var k=h.width(),h=h.height(),m=b.directionAt(a);a=Na.absolute(a);f=nn(f.left(),f.top(),k,h,m,a)}else f=
R8(b,c);d.restore();return f})},T8=function(a,b,c,d){var e=c.startContainer(),g=c.startOffset();if("body"===A.name(e)&&0===g){var f=c.startContainer();a=parseInt(t.get(f,"padding-top"),10);d=parseInt(t.get(f,"padding-left"),10);var k=UJ.runAndRemove(f,VJ),e=k.position(),g=k.width(),k=k.height();b=b.directionAt(f);f=Na.absolute(f);c=ot(nn(e.left()+d,e.top()+a,g,k,b,f),h.some(c))}else c=S8(a,b,c,d);return c},WJ=function(a,b,c,d,e){b.focus();a=T8(b.isRoot,a,c,e);a.cursor().each(function(a){b.setSelection(a.startContainer(),
a.startOffset(),a.endContainer(),a.endOffset())});a=a.result();d(a)},zw={insertAt:WJ,insertAtCursor:function(a,b,c,d){b.getSelection().each(function(e){WJ(a,b,e,c,d)})}},Cw=Y.detect(),XJ=function(){try{var a=Ke.localStorage;a.setItem("__storage_test__","__storage_test__");a.removeItem("__storage_test__");return h.some(a)}catch(b){return h.none()}}().fold(function(){var a={};return{getItem:function(b){return a[b]},setItem:function(b,c){a[b]=c}}},function(a){return{getItem:function(b){return a.getItem("ephox.textboxio."+
b)},setItem:function(b,c){a.setItem("ephox.textboxio."+b,c)}}}),YJ=function(a,b){return h.from(XJ.getItem(a)).map($c.parse).getOr(b)},ZJ=function(a){return YJ(a,{})},$J=function(a,b){var c=$c.stringify(b);XJ.setItem(a,c)},No={read:ZJ,readOr:YJ,write:$J,access:function(a){return{read:f.curry(ZJ,a),write:f.curry($J,a)}}},aK=function(a,b,c){b.body().each(function(d){c.save();var e=q.fromHtml(Ua.zeroWidth());a(d,e);b.setSelection(e,0,e,0)})},Dw={upOrLeft:function(a,b,c){a.setRelativeSelection(da.before(c),
da.before(c));a.getSelection().fold(function(){aK(v.prepend,a,b)},f.noop)},right:function(a,b,c){a.setRelativeSelection(da.after(c),da.after(c));a.getSelection().fold(function(){aK(v.append,a,b)},f.noop)}},Qh={left:function(a,b,c){return a(b)&&ka.isStart(b,c)?h.none():ka.isStart(b,c)?Lb.before(b,a).filter(function(a){return!eb.ancestor(a,"head")}):a(b)&&0<c?y.child(b,c-1):h.none()},right:function(a,b,c){return a(b)&&ka.isEnd(b,c)?h.none():ka.isEnd(b,c)?Lb.after(b,a).filter(function(b){return a(b)||
eb.ancestor(b,"body")}):a(b)&&c<ka.getEnd(b)?y.child(b,c+1):h.none()},isFirstInContent:function(a,b){return a.body().map(function(a){a=y.children(a);return L.eq(a[0],b)}).getOr(!1)},isLastInContent:function(a,b){return a.body().map(function(a){a=y.children(a);return 0<a.length&&L.eq(a[a.length-1],b)}).getOr(!1)}},U8=function(a,b){return a.getSelection().bind(function(c){return c.collapsed()?b(a.isRoot,c.startContainer(),c.startOffset()):h.none()})},V8=function(a,b){return k.foldr(a,function(a,d){return d(b).or(a)},
h.none())},Oo={get:function(a,b,c){return U8(a,c).bind(function(a){return V8(b,a)})}},lh={response:C.immutable("selection","kill")},mh=function(a){return function(b){return b===a}},W8=mh(38),X8=mh(40),Vc={ltr:{isBackward:mh(37),isForward:mh(39)},rtl:{isBackward:mh(39),isForward:mh(37)},isUp:W8,isDown:X8,isNavigation:function(a){return 37<=a&&40>=a}},lg={convertToRange:function(a,b){var c=Hd.asLtrRange(a,b);return{start:f.constant(q.fromDom(c.startContainer)),soffset:f.constant(c.startOffset),finish:f.constant(q.fromDom(c.endContainer)),
foffset:f.constant(c.endOffset)}},makeSitus:function(a,b,c,d){return{start:f.constant(da.on(a,b)),finish:f.constant(da.on(c,d))}}},bK=function(a,b){return b.column()>=a.startCol()&&b.column()+b.colspan()-1<=a.finishCol()&&b.row()>=a.startRow()&&b.row()+b.rowspan()-1<=a.finishRow()},cK={inSelection:function(a,b){var c=b.column(),d=b.column()+b.colspan()-1,e=b.row(),g=b.row()+b.rowspan()-1;return c<=a.finishCol()&&d>=a.startCol()&&e<=a.finishRow()&&g>=a.startRow()},isWithin:bK,isRectangular:function(a,
b){for(var c=!0,d=f.curry(bK,b),e=b.startRow();e<=b.finishRow();e++)for(var g=b.startCol();g<=b.finishCol();g++)c=c&&gb.getAt(a,e,g).exists(d);return c?h.some(b):h.none()}},dK=function(a,b,c){b=gb.findItem(a,b,L.eq);var d=gb.findItem(a,c,L.eq);return b.bind(function(a){return d.map(function(b){return xa.bounds(Math.min(a.row(),b.row()),Math.min(a.column(),b.column()),Math.max(a.row()+a.rowspan()-1,b.row()+b.rowspan()-1),Math.max(a.column()+a.colspan()-1,b.column()+b.colspan()-1))})})},eK={getAnyBox:dK,
getBox:function(a,b,c){return dK(a,b,c).bind(function(b){return cK.isRectangular(a,b)})}},pj={moveBy:function(a,b,c,d){return gb.findItem(a,b,L.eq).bind(function(b){var g=0<c?b.row()+b.rowspan()-1:b.row();b=0<d?b.column()+b.colspan()-1:b.column();return gb.getAt(a,g+c,b+d).map(function(a){return a.element()})})},intercepts:function(a,b,c){return eK.getAnyBox(a,b,c).map(function(b){b=gb.filterItems(a,f.curry(cK.inSelection,b));return k.map(b,function(a){return a.element()})})},parentCell:function(a,
b){return gb.findItem(a,b,function(a,b){return L.contains(b,a)}).bind(function(a){return a.element()})}},on=function(a){a=We.fromTable(a);return gb.generate(a)},nh={moveBy:function(a,b,c){return Ya.table(a).bind(function(d){d=on(d);return pj.moveBy(d,a,b,c)})},intercepts:function(a,b,c){a=on(a);return pj.intercepts(a,b,c)},nestedIntercepts:function(a,b,c,d,e){var g=on(a);b=L.eq(a,c)?b:pj.parentCell(g,b);a=L.eq(a,e)?d:pj.parentCell(g,d);return pj.intercepts(g,b,a)},getBox:function(a,b,c){a=on(a);return eK.getBox(a,
b,c)}},xt=function(a,b){return J.ancestor(a,"table")},qj=C.immutableBag(["boxes","start","finish"],[]),fK=function(a,b,c){var d=function(a){return function(b){return c(b)||L.eq(b,a)}};return L.eq(a,b)?h.some(qj({boxes:h.some([a]),start:a,finish:b})):xt(a,c).bind(function(e){return xt(b,c).bind(function(g){if(L.eq(e,g))return h.some(qj({boxes:nh.intercepts(e,a,b),start:a,finish:b}));if(L.contains(e,g)){var f=X.ancestors(b,"td,th",d(e)),f=0<f.length?f[f.length-1]:b;return h.some(qj({boxes:nh.nestedIntercepts(e,
a,e,b,g),start:a,finish:f}))}return L.contains(g,e)?(f=X.ancestors(a,"td,th",d(g)),f=0<f.length?f[f.length-1]:a,h.some(qj({boxes:nh.nestedIntercepts(g,a,e,b,g),start:a,finish:f}))):be.ancestors(a,b).shared().bind(function(f){return J.closest(f,"table",c).bind(function(c){var f=X.ancestors(b,"td,th",d(c)),f=0<f.length?f[f.length-1]:b,l=X.ancestors(a,"td,th",d(c)),l=0<l.length?l[l.length-1]:a;return h.some(qj({boxes:nh.nestedIntercepts(c,a,e,b,g),start:l,finish:f}))})})})})},Y8=function(a,b){return k.find(a,
function(a){return ra.is(a,b)})},Z8=function(a,b){return J.ancestor(a,"table").bind(function(c){return J.descendant(c,b).bind(function(b){return fK(b,a).bind(function(a){return a.boxes().map(function(b){return{boxes:f.constant(b),start:f.constant(a.start()),finish:f.constant(a.finish())}})})})})},Fe={identify:fK,retrieve:function(a,b){var c=X.descendants(a,b);return 0<c.length?h.some(c):h.none()},shiftSelection:function(a,b,c,d,e){return Y8(a,e).bind(function(a){return nh.moveBy(a,b,c).bind(function(a){return Z8(a,
d)})})},getEdges:function(a,b,c){return J.descendant(a,b).bind(function(b){return J.descendant(a,c).bind(function(a){return be.sharedOne(xt,[b,a]).map(function(c){return{first:f.constant(b),last:f.constant(a),table:f.constant(c)}})})})}},gK=function(a,b,c,d,e){return L.eq(c,d)?h.none():Fe.identify(c,d,b).bind(function(b){var d=b.boxes().getOr([]);return 0<d.length?(e(a,d,b.start(),b.finish()),h.some(lh.response(h.some(lg.makeSitus(c,0,c,ka.getEnd(c))),!0))):h.none()})},yt={sync:function(a,b,c,d,e,
g,f){return L.eq(c,e)&&d===g?h.none():J.closest(c,"td,th",b).bind(function(c){return J.closest(e,"td,th",b).bind(function(d){return gK(a,b,c,d,f)})})},detect:gK,update:function(a,b,c,d,e){return Fe.shiftSelection(d,a,b,e.firstSelectedSelector(),e.lastSelectedSelector()).map(function(a){e.clear(c);e.selectRange(c,a.boxes(),a.start(),a.finish());return a.boxes()})}},oh=C.immutableBag(["left","top","right","bottom"],[]),Mc={nu:oh,moveUp:function(a,b){return oh({left:a.left(),top:a.top()-b,right:a.right(),
bottom:a.bottom()-b})},moveDown:function(a,b){return oh({left:a.left(),top:a.top()+b,right:a.right(),bottom:a.bottom()+b})},moveBottomTo:function(a,b){var c=a.bottom()-a.top();return oh({left:a.left(),top:b-c,right:a.right(),bottom:b})},moveTopTo:function(a,b){var c=a.bottom()-a.top();return oh({left:a.left(),top:b,right:a.right(),bottom:b+c})},getTop:function(a){return a.top()},getBottom:function(a){return a.bottom()},translate:function(a,b,c){return oh({left:a.left()+b,top:a.top()+c,right:a.right()+
b,bottom:a.bottom()+c})},toString:function(a){return"("+a.left()+", "+a.top()+") -> ("+a.right()+", "+a.bottom()+")"}},pn=function(a){return Mc.nu({left:a.left,top:a.top,right:a.right,bottom:a.bottom})},qn={getBoxAt:function(a,b,c){return A.isElement(b)?h.some(a.getRect(b)).map(pn):A.isText(b)?(a=0<=c&&c<ka.getEnd(b)?a.getRangedRect(b,c,b,c+1):0<c?a.getRangedRect(b,c-1,b,c):h.none(),a.map(pn)):h.none()},getEntireBox:function(a,b){return A.isElement(b)?h.some(a.getRect(b)).map(pn):A.isText(b)?a.getRangedRect(b,
0,b,ka.getEnd(b)).map(pn):h.none()}},Kd=ma.generate([{none:[]},{retry:["caret"]}]),hK=function(a,b,c){return sa.closest(b,fa.isBlock).fold(f.constant(!1),function(b){return qn.getEntireBox(a,b).exists(function(a){return c.left()<a.left()||1>Math.abs(a.right()-c.left())||c.left()>a.right()})})},$8={point:Mc.getBottom,adjuster:function(a,b,c,d,e){var g=Mc.moveDown(e,5);return 1>Math.abs(c.bottom()-d.bottom())?Kd.retry(g):c.top()>e.bottom()?Kd.retry(g):c.top()===e.bottom()?Kd.retry(Mc.moveDown(e,1)):
hK(a,b,e)?Kd.retry(Mc.translate(g,5,0)):Kd.none()},move:Mc.moveDown,gather:Lb.after},a9=function(a,b,c){return a.elementFromPoint(b,c).filter(function(a){return"table"===A.name(a)}).isSome()},b9=function(a,b,c,d,e){return zt(a,b,c,b.move(d,5),e)},zt=function(a,b,c,d,e){return 0===e?h.some(d):a9(a,d.left(),b.point(d))?b9(a,b,c,d,e-1):a.situsFromPoint(d.left(),b.point(d)).bind(function(g){return g.start().fold(h.none,function(g,f){return qn.getEntireBox(a,g,f).bind(function(f){return b.adjuster(a,g,
f,c,d).fold(h.none,function(d){return zt(a,b,c,d,e-1)})}).orThunk(function(){return h.some(d)})},h.none)})},c9=function(a,b,c){return a.point(b)>c.getInnerHeight()?h.some(a.point(b)-c.getInnerHeight()):0>a.point(b)?h.some(-a.point(b)):h.none()},iK=function(a,b,c){var d=a.move(c,5),e=zt(b,a,c,d,100).getOr(d);return c9(a,e,b).fold(function(){return b.situsFromPoint(e.left(),a.point(e))},function(c){b.scrollBy(0,c);return b.situsFromPoint(e.left(),a.point(e)-c)})},rj={tryUp:f.curry(iK,{point:Mc.getTop,
adjuster:function(a,b,c,d,e){var g=Mc.moveUp(e,5);return 1>Math.abs(c.top()-d.top())?Kd.retry(g):c.bottom()<e.top()?Kd.retry(g):c.bottom()===e.top()?Kd.retry(Mc.moveUp(e,1)):hK(a,b,e)?Kd.retry(Mc.translate(g,5,0)):Kd.none()},move:Mc.moveUp,gather:Lb.before}),tryDown:f.curry(iK,$8),ieTryUp:function(a,b){return a.situsFromPoint(b.left(),b.top()-5)},ieTryDown:function(a,b){return a.situsFromPoint(b.left(),b.bottom()+5)},getJumpSize:f.constant(5)},rn=ma.generate([{none:["message"]},{success:[]},{failedUp:["cell"]},
{failedDown:["cell"]}]),d9=function(a){return J.closest(a,"tr")},ph={verify:function(a,b,c,d,e,g,f){return J.closest(d,"td,th",f).bind(function(c){return J.closest(b,"td,th",f).map(function(b){return L.eq(c,b)?L.eq(d,c)&&ka.getEnd(c)===e?g(b):rn.none("in same cell"):be.sharedOne(d9,[c,b]).fold(function(){var d=a.getRect(b),e=a.getRect(c);return e.right>d.left&&e.left<d.right?rn.success():g(b)},function(a){return g(b)})})}).getOr(rn.none("default"))},cata:function(a,b,c,d,e){return a.fold(b,c,d,e)},
adt:rn},e9=C.immutable("ancestor","descendants","element","index"),jK=C.immutable("parent","children","element","index"),sn=function(a,b){return k.findIndex(a,f.curry(L.eq,b))},f9={childOf:function(a,b){return sa.closest(a,function(a){return y.parent(a).exists(function(a){return L.eq(a,b)})})},indexOf:sn,indexInParent:function(a){return y.parent(a).bind(function(b){var c=y.children(b);return sn(c,a).map(function(d){return jK(b,c,a,d)})})},selectorsInParent:function(a,b){return y.parent(a).bind(function(c){var d=
X.children(c,b);return sn(d,a).map(function(b){return jK(c,d,a,b)})})},descendantsInAncestor:function(a,b,c){return J.closest(a,b).bind(function(b){var e=X.descendants(b,c);return sn(e,a).map(function(c){return e9(b,e,a,c)})})}},At=function(a){return"br"===A.name(a)},Bt=function(a,b,c){return b(a,c).bind(function(a){return A.isText(a)&&0===Da.get(a).trim().length?Bt(a,b,c):h.some(a)})},g9=function(a,b,c){return c.traverse(b).orThunk(function(){return Bt(b,c.gather,a)}).map(c.relative)},h9=function(a,
b){return y.child(a,b).filter(At).orThunk(function(){return y.child(a,b-1).filter(At)})},i9=function(a,b,c,d){return h9(b,c).bind(function(b){return d.traverse(b).fold(function(){return Bt(b,d.gather,a).map(d.relative)},function(a){return f9.indexInParent(a).map(function(a){return da.on(a.parent(),a.index())})})})},kK={tryBr:function(a,b,c,d){return(At(b)?g9(a,b,d):i9(a,b,c,d)).map(function(a){return{start:f.constant(a),finish:f.constant(a)}})},process:function(a){return ph.cata(a,function(a){return h.none()},
function(){return h.none()},function(a){return h.some(S.point(a,0))},function(a){return h.some(S.point(a,ka.getEnd(a)))})}},sj=Y.detect(),j9=function(a,b,c){return a.getSelection().bind(function(d){return kK.tryBr(b,d.finish(),d.foffset(),c).fold(function(){return h.some(S.point(d.finish(),d.foffset()))},function(e){e=a.fromSitus(e);e=ph.verify(a,d.finish(),d.foffset(),e.finish(),e.foffset(),c.failure,b);return kK.process(e)})})},Ct=function(a,b,c,d,e,g){return 0===g?h.none():k9(a,b,c,d,e).bind(function(f){var k=
a.fromSitus(f),k=ph.verify(a,c,d,k.finish(),k.foffset(),e.failure,b);return ph.cata(k,function(){return h.none()},function(){return h.some(f)},function(f){return L.eq(c,f)&&0===d?lK(a,c,d,Mc.moveUp,e):Ct(a,b,f,0,e,g-1)},function(f){return L.eq(c,f)&&d===ka.getEnd(f)?lK(a,c,d,Mc.moveDown,e):Ct(a,b,f,ka.getEnd(f),e,g-1)})})},lK=function(a,b,c,d,e){return qn.getBoxAt(a,b,c).bind(function(b){return mK(a,e,d(b,rj.getJumpSize()))})},mK=function(a,b,c){return sj.browser.isChrome()||sj.browser.isSafari()||
sj.browser.isFirefox()||sj.browser.isEdge()?b.otherRetry(a,c):sj.browser.isIE()?b.ieRetry(a,c):h.none()},k9=function(a,b,c,d,e){return qn.getBoxAt(a,c,d).bind(function(b){return mK(a,e,b)})},l9={handle:function(a,b,c){return j9(a,b,c).bind(function(d){return Ct(a,b,d.element(),d.offset(),c,20).map(a.fromSitus)})}},m9=Y.detect(),n9=function(a,b){return Gc.ancestor(a,function(a){return y.parent(a).exists(function(a){return L.eq(a,b)})})},nK=function(a,b,c,d,e){return J.closest(d,"td,th",b).bind(function(d){return J.closest(d,
"table",b).bind(function(l){return n9(e,l)?l9.handle(a,b,c).bind(function(a){return J.closest(a.finish(),"td,th",b).map(function(b){return{start:f.constant(d),finish:f.constant(b),range:f.constant(a)}})}):h.none()})})},oK=function(a,b,c,d,e,g){return m9.browser.isIE()?h.none():g(d,b).orThunk(function(){return nK(a,b,c,d,e).map(function(a){a=a.range();return lh.response(h.some(lg.makeSitus(a.start(),a.soffset(),a.finish(),a.foffset())),!0)})})},pK=function(a,b,c,d,e,g,f){return nK(a,c,d,e,g).bind(function(a){return yt.detect(b,
c,a.start(),a.finish(),f)})},o9=function(a,b){return J.closest(a,"tr",b).bind(function(a){return J.closest(a,"table",b).bind(function(d){var e=X.descendants(d,"tr");return L.eq(a,e[0])?Lb.seekLeft(d,function(a){return Cb.last(a).isSome()},b).map(function(a){var b=ka.getEnd(a);return lh.response(h.some(lg.makeSitus(a,b,a,b)),!0)}):h.none()})})},p9=function(a,b){return J.closest(a,"tr",b).bind(function(a){return J.closest(a,"table",b).bind(function(d){var e=X.descendants(d,"tr");return L.eq(a,e[e.length-
1])?Lb.seekRight(d,function(a){return Cb.first(a).isSome()},b).map(function(a){return lh.response(h.some(lg.makeSitus(a,0,a,0)),!0)}):h.none()})})},Fw=function(a,b){return J.closest(a,"td,th",b)},qK={traverse:y.nextSibling,gather:Lb.after,relative:da.before,otherRetry:rj.tryDown,ieRetry:rj.ieTryDown,failure:ph.adt.failedDown},rK={traverse:y.prevSibling,gather:Lb.before,relative:da.before,otherRetry:rj.tryUp,ieRetry:rj.ieTryUp,failure:ph.adt.failedUp},qh=C.immutable("rows","cols"),Iw={mouse:function(a,
b,c,d){a=Ew(a);b=tR(a,b,c,d);return{mousedown:b.mousedown,mouseover:b.mouseover,mouseup:b.mouseup}},keyboard:function(a,b,c,d){var e=Ew(a),g=function(){d.clear(b);return h.none()};return{keydown:function(a,k,m,n,q,s){var t=a.raw().which,u=!0===a.raw().shiftKey;return Fe.retrieve(b,d.selectedSelector()).fold(function(){return Vc.isDown(t)&&u?f.curry(pK,e,b,c,qK,n,k,d.selectRange):Vc.isUp(t)&&u?f.curry(pK,e,b,c,rK,n,k,d.selectRange):Vc.isDown(t)?f.curry(oK,e,c,qK,n,k,p9):Vc.isUp(t)?f.curry(oK,e,c,rK,
n,k,o9):h.none},function(a){var c=function(c){return function(){return ab.findMap(c,function(c){return yt.update(c.rows(),c.cols(),b,a,d)}).fold(function(){return Fe.getEdges(b,d.firstSelectedSelector(),d.lastSelectedSelector()).map(function(a){var c=Vc.isDown(t)||s.isForward(t)?da.after:da.before;e.setRelativeSelection(da.on(a.first(),0),c(a.table()));d.clear(b);return lh.response(h.none(),!0)})},function(a){return h.some(lh.response(h.none(),!0))})}};return Vc.isDown(t)&&u?c([qh(1,0)]):Vc.isUp(t)&&
u?c([qh(-1,0)]):s.isBackward(t)&&u?c([qh(0,-1),qh(-1,0)]):s.isForward(t)&&u?c([qh(0,1),qh(1,0)]):Vc.isNavigation(t)&&!1===u?g:h.none})()},keyup:function(a,e,g,f,k){return Fe.retrieve(b,d.selectedSelector()).fold(function(){var m=a.raw().which;return!0===a.raw().shiftKey===!1?h.none():Vc.isNavigation(m)?yt.sync(b,c,e,g,f,k,d.selectRange):h.none()},h.none)}}}},Gw={byClass:function(a){var b=me.addClass(a.selected()),c=me.removeClasses([a.selected(),a.lastSelected(),a.firstSelected()]),d=function(b){b=
X.descendants(b,a.selectedSelector());k.each(b,c)};return{clear:d,selectRange:function(c,g,f,h){d(c);k.each(g,b);n.add(f,a.firstSelected());n.add(h,a.lastSelected())},selectedSelector:a.selectedSelector,firstSelectedSelector:a.firstSelectedSelector,lastSelectedSelector:a.lastSelectedSelector}},byAttr:function(a){var b=function(b){m.remove(b,a.selected());m.remove(b,a.firstSelected());m.remove(b,a.lastSelected())},c=function(b){m.set(b,a.selected(),"1")},d=function(c){c=X.descendants(c,a.selectedSelector());
k.each(c,b)};return{clear:d,selectRange:function(b,g,f,h){d(b);k.each(g,c);m.set(f,a.firstSelected(),"1");m.set(h,a.lastSelected(),"1")},selectedSelector:a.selectedSelector,firstSelectedSelector:a.firstSelectedSelector,lastSelectedSelector:a.lastSelectedSelector}}},Jw={retrieve:function(a,b){return Fe.retrieve(a,b)},retrieveBox:function(a,b,c){return Fe.getEdges(a,b,c).bind(function(b){var c=function(b){return L.eq(a,b)},g=J.ancestor(b.first(),"thead,tfoot,tbody,table",c),f=J.ancestor(b.last(),"thead,tfoot,tbody,table",
c);return g.bind(function(a){return f.bind(function(c){return L.eq(a,c)?nh.getBox(b.table(),b.first(),b.last()):h.none()})})})}},wR={configure:function(a){k.each([a.output(),a.internal()],function(b){var c=[sd.lastSelected(),sd.firstSelected(),sd.selected()],d=k.map(c,function(a){return"."+a}).join(",");a.addSelector(b,d,function(a){k.each(a,me.removeClasses(c))})})}},sK=function(a){return 0===a.button},tn=function(a,b){return{bind:function(c,d){return Vp.bind(c,a,b,d)}}},sk={realClick:tn("click",
function(a){return 6===a.mozInputSource||0===a.mozInputSource?!1:void 0!==a.isTrusted&&!0!==a.isTrusted?!1:!0}),leftDown:tn("mousedown",sK),leftPressedOver:tn("mouseover",function(a){return void 0===a.buttons?!0:0!==(a.buttons&1)}),leftUp:tn("mouseup",sK)},Hw=Y.detect(),zR=function(a){var b=Y.detect(),c=b.os.isOSX(),d=b.os.isLinux(),e=c?N.META():N.CTRL(),b=[[e,"Z"]],g=c?[[N.META(),N.SHIFT(),"Z"]]:d?[[N.CTRL(),"Y"],[N.CTRL(),N.SHIFT(),"Z"]]:[[N.CTRL(),"Y"]],f=[[N.SHIFT(),121]],h=c?[[N.ALT(),121]]:
[[121]],k=[[e,N.SHIFT(),"F"]],d=c?[[N.META(),N.ALT(),"U"]]:d?[[N.CTRL(),N.ALT(),"U"]]:[[N.CTRL(),N.SHIFT(),"U"]],c=c?[[N.CTRL(),N.ALT(),"H"]]:[[N.CTRL(),N.SHIFT(),"H"]],m=[[e,"B"]],n=[[e,"I"]],q=[[e,"U"]],s=[[e,221],[e,"M"]],t=[[e,219],[e,N.SHIFT(),"M"]],u=[[e,N.SHIFT(),"9"]],v=[[e,"K"]],e=[[e,"F"]],x=C.immutable("keys","trigger");return{undo:x(b,a.trigger.undo),redo:x(g,a.trigger.redo),context:x(f,a.trigger.context),toolbar:x(h,a.trigger.toolbar),bold:x(m,a.trigger.bold),italic:x(n,a.trigger.italic),
underline:x(q,a.trigger.underline),indent:x(s,a.trigger.indent),outdent:x(t,a.trigger.outdent),link:x(v,a.trigger.link),search:x(e,a.trigger.search),blockquote:x(u,a.trigger.blockquote),help:x(c,a.trigger.help),fullscreen:x(k,a.trigger.fullscreen),plaincode:x(d,a.trigger.plaincode)}},AR=function(a,b){return function(c){return k.contains(a,c)?h.some(b):h.none()}},tK=function(a){return function(b){return b.key()===a}},q9=C.immutable("key","value"),CR={put:function(a,b,c,d,e){var g=void 0!==d?d:q9,f=
void 0!==e?e:tK(b);a=k.foldl(a,function(a,d){var e=f(d)?g(b,c):d;return{updated:f(d)||a.updated,acc:a.acc.concat([e])}},{updated:!1,acc:[]});return a.updated?a.acc:a.acc.concat([g(b,c)])},get:function(a,b,c){b=void 0!==c?c:tK(b);return k.find(a,b)}},DR=C.immutable("popup","anchor"),FR={bind:function(a,b){var c=Ea.get(a);return{unbind:E.bind(a,"scroll",function(d){d=Ea.get(a);d.top()===c.top()&&d.left()===c.left()||b(d);c=d}).unbind}}},LR={isDefaultEmpty:function(a){return/^\s*<p>\s*<br\s?\/?>\s*<\/p>\s*$/.test(a)}},
JR={fromInitial:function(a){return Kw({initial:a,undo:[],redo:[]})},fromState:function(a){return Kw(a)}},tk=C.immutable("raw","marked"),KR=function(a){return tk(a.raw,a.marked)},Lw=function(a,b){return{initial:b(a.initial),undo:k.map(a.undo,b),redo:k.map(a.redo,b)}},MR=function(a){return{raw:a.raw(),marked:a.marked()}},Dt=ma.generate([{aboveView:["item"]},{inView:["item"]},{belowView:["item"]}]),xf={aboveView:Dt.aboveView,inView:Dt.inView,belowView:Dt.belowView,cata:function(a,b,c,d){return a.fold(b,
c,d)}},kb={resolve:lb.create("ephox-hermes").resolve},uK=function(){var a=D.create({upload:s(["files"])}),b=q.fromTag("input");n.add(b,kb.resolve("hidden"));n.add(b,kb.resolve("file-selector-input"));m.setAll(b,{type:"file",accept:"image/*"});var c=E.bind(b,"change",function(b){b=ud.toFiles(b);a.trigger.upload(b)});return{element:f.constant(b),open:function(){$.set(b,"");b.dom().click()},destroy:function(){c.unbind()},events:a.registry}},Nw={single:uK,multiple:function(){var a=uK();m.set(a.element(),
"multiple","multiple");return{element:a.element,open:a.open,destroy:a.destroy,events:a.events}}},vK=function(a,b){return k.map(b,function(b){var d=q.fromTag("img"),e=E.bind(d,"load",function(){e.unbind();a.domChanged()});b=oc.cata(b,function(a,b,c){return c},function(a,b,c){return b});m.set(d,"src",b);return d})},So={insertMobileAssets:function(a,b,c,d){var e=vK(a,d);return a.undoOperation(function(){var g=Cc.insertAt(a,b,e);c.uploadAssets(d);return g})},insertAssets:function(a,b,c){var d=vK(a,c);
a.focus();a.undoOperation(function(){Cc.insertAtCursor(a,d).each(function(b){a.setSelection(b.element(),b.offset(),b.element(),b.offset())});b.uploadAssets(c)})}},Db={resolve:lb.create("ephox-gel").resolve},TR=Y.detect().deviceType.isiOS(),UR=Y.detect(),wK=function(a){a.dom().click();return h.some(a)},qb={setup:function(a,b,c){b=pa.dialog({execute:function(a){a.dom().click()},ok:b.ok().element(),cancel:b.cancel().element()});a=ba.dialog(a.element(),c,b);return{enter:a.enter,exit:a.exit,destroy:a.destroy}},
button:function(a){return(ra.is(a,".ephox-pastry-button")?h.some(a):J.descendant(a,".ephox-pastry-button")).bind(wK)},click:wK,radio:function(a,b,c){b={item:f.constant(b),selected:f.constant(c)};return ba.radio(a,b,pa.radio({one:function(a,b){b.dom().click()}}))}},WR={previewAsset:function(a,b){oc.cata(a,f.die("asset should not be a file type"),function(a,d,e){n.add(b,kb.resolve("fluid-image"));m.set(b,"src",d)})}},Qw=Y.detect(),yf=C.immutable("heading","renderer"),Qo={left:f.constant("left"),top:f.constant("top")},
r9=["image"],s9=function(a,b,c){return k.exists(r9,function(a){return R.isString(b.getType())&&-1<b.getType().indexOf(a)})},fS={validate:function(a){a=k.foldl(a,function(a,c){return!1===oc.cata(c,s9,f.constant(!0))?{assets:a.assets,failure:!0}:{assets:a.assets.concat([c]),failure:a.failure}},{assets:[],failure:!1});return{assets:f.constant(a.assets),failure:f.constant(a.failure)}}},we={multiple:function(a){return ud.multiple(a)},single:function(a){return ud.single(a)},singleWithUrl:function(a,b){return ud.singleWithUrl(a,
b)}},Et=[0,0.01,0.02,0.04,0.05,0.06,0.07,0.08,0.1,0.11,0.12,0.14,0.15,0.16,0.17,0.18,0.2,0.21,0.22,0.24,0.25,0.27,0.28,0.3,0.32,0.34,0.36,0.38,0.4,0.42,0.44,0.46,0.48,0.5,0.53,0.56,0.59,0.62,0.65,0.68,0.71,0.74,0.77,0.8,0.83,0.86,0.89,0.92,0.95,0.98,1,1.06,1.12,1.18,1.24,1.3,1.36,1.42,1.48,1.54,1.6,1.66,1.72,1.78,1.84,1.9,1.96,2,2.12,2.25,2.37,2.5,2.62,2.75,2.87,3,3.2,3.4,3.6,3.8,4,4.3,4.7,4.9,5,5.5,6,6.5,6.8,7,7.3,7.5,7.8,8,8.4,8.7,9,9.4,9.6,9.8,10],de={identity:function(){return[1,0,0,0,0,0,1,0,
0,0,0,0,1,0,0,0,0,0,1,0,0,0,0,0,1]},adjust:Ro,multiply:Ge,adjustContrast:function(a,b){var c;b=md(b,-1,1);b*=100;0>b?c=127+b/100*127:(c=b%1,c=0===c?Et[b]:Et[Math.floor(b)]*(1-c)+Et[Math.floor(b)+1]*c,c=127*c+127);return Ge(a,[c/127,0,0,0,0.5*(127-c),0,c/127,0,0,0.5*(127-c),0,0,c/127,0,0.5*(127-c),0,0,0,1,0,0,0,0,0,1])},adjustBrightness:function(a,b){b=md(255*b,-255,255);return Ge(a,[1,0,0,0,b,0,1,0,0,b,0,0,1,0,b,0,0,0,1,0,0,0,0,0,1])},adjustSaturation:function(a,b){var c;b=md(b,-1,1);c=1+(0<b?3*b:
b);return Ge(a,[0.3086*(1-c)+c,0.6094*(1-c),0.082*(1-c),0,0,0.3086*(1-c),0.6094*(1-c)+c,0.082*(1-c),0,0,0.3086*(1-c),0.6094*(1-c),0.082*(1-c)+c,0,0,0,0,0,1,0,0,0,0,0,1])},adjustHue:function(a,b){var c,d;b=md(b,-180,180)/180*Math.PI;c=Math.cos(b);d=Math.sin(b);return Ge(a,[0.213+0.787*c+-0.213*d,0.715+-0.715*c+-0.715*d,0.072+-0.072*c+0.928*d,0,0,0.213+-0.213*c+0.143*d,0.715+c*(1-0.715)+0.14*d,0.072+-0.072*c+-0.283*d,0,0,0.213+-0.213*c+-0.787*d,0.715+-0.715*c+0.715*d,0.072+0.928*c+0.072*d,0,0,0,0,0,
1,0,0,0,0,0,1])},adjustColors:function(a,b,c,d){b=md(b,0,2);c=md(c,0,2);d=md(d,0,2);return Ge(a,[b,0,0,0,0,0,c,0,0,0,0,0,d,0,0,0,0,0,1,0,0,0,0,0,1])},adjustSepia:function(a,b){b=md(b,0,1);return Ge(a,Ro([0.393,0.769,0.189,0,0,0.349,0.686,0.168,0,0,0.272,0.534,0.131,0,0,0,0,0,1,0,0,0,0,0,1],b))},adjustGrayscale:function(a,b){b=md(b,0,1);return Ge(a,Ro([0.33,0.34,0.33,0,0,0.33,0.34,0.33,0,0,0.33,0.34,0.33,0,0,0,0,0,1,0,0,0,0,0,1],b))}},fd={invert:function(a){return function(b){return uk(b,a)}}([-1,
0,0,0,255,0,-1,0,0,255,0,0,-1,0,255,0,0,0,1,0]),brightness:ng(de.adjustBrightness),hue:ng(de.adjustHue),saturate:ng(de.adjustSaturation),contrast:ng(de.adjustContrast),grayscale:ng(de.adjustGrayscale),sepia:ng(de.adjustSepia),colorize:function(a,b,c,d){return uk(a,de.adjustColors(de.identity(),b,c,d))},sharpen:Xw([0,-1,0,-1,5,-1,0,-1,0]),emboss:Xw([-2,-1,0,-1,1,1,0,1,2]),gamma:Ww(function(a,b){return 255*Math.pow(a/255,1-b)}),exposure:Ww(function(a,b){return 255*(1-Math.exp(-(a/255)*b))}),colorFilter:uk,
convoluteFilter:Vw},t9={scale:Yw},un={rotate:function(a,b){return a.toCanvas().then(function(c){var d=a.getType(),e=b,g=fc.create(c.width,c.height),f=fc.get2dContext(g),h=0,k=0,e=0>e?360+e:e;90!=e&&270!=e||fc.resize(g,g.height,g.width);if(90==e||180==e)h=g.width;if(270==e||180==e)k=g.height;f.translate(h,k);f.rotate(e*Math.PI/180);f.drawImage(c,0,0);return vd.fromCanvas(g,d)})},flip:function(a,b){return a.toCanvas().then(function(c){var d=a.getType(),e=fc.create(c.width,c.height),g=fc.get2dContext(e);
"v"==b?(g.scale(1,-1),g.drawImage(c,0,-e.height)):(g.scale(-1,1),g.drawImage(c,-e.width,0));return vd.fromCanvas(e,d)})},crop:function(a,b,c,d,e){return a.toCanvas().then(function(g){var f=a.getType(),h=fc.create(d,e);fc.get2dContext(h).drawImage(g,-b,-c);return vd.fromCanvas(h,f)})},resize:function(a,b,c){return a.toCanvas().then(function(d){return t9.scale(d,b,c).then(function(b){return vd.fromCanvas(b,a.getType())})})}},tj={invert:function(a){return fd.invert(a)},sharpen:function(a){return fd.sharpen(a)},
emboss:function(a){return fd.emboss(a)},brightness:function(a,b){return fd.brightness(a,b)},hue:function(a,b){return fd.hue(a,b)},saturate:function(a,b){return fd.saturate(a,b)},contrast:function(a,b){return fd.contrast(a,b)},grayscale:function(a,b){return fd.grayscale(a,b)},sepia:function(a,b){return fd.sepia(a,b)},colorize:function(a,b,c,d){return fd.colorize(a,b,c,d)},gamma:function(a,b){return fd.gamma(a,b)},exposure:function(a,b){return fd.exposure(a,b)},flip:function(a,b){return un.flip(a,b)},
crop:function(a,b,c,d,e){return un.crop(a,b,c,d,e)},resize:function(a,b,c){return un.resize(a,b,c)},rotate:function(a,b){return un.rotate(a,b)}},uj=function(a){return function(b){return tc.blobToImageResult(b).then(function(b){return a(b)}).then(tc.imageResultToOriginalBlob)}},xK=function(a){return uj(function(b){return tj.rotate(b,a)})},yK=function(a){return uj(function(b){return tj.flip(b,a)})},u9=function(a){return uj(function(b){return tj.resize(b,a.width(),a.height())})},v9=Pa.pToFR(uj(tj.sharpen)),
w9=Pa.pToFR(xK(-90)),x9=Pa.pToFR(xK(90)),y9=Pa.pToFR(yK("v")),z9=Pa.pToFR(yK("h")),pg={rotateLeft:w9,rotateRight:x9,flipVertical:y9,flipHorizontal:z9,sharpen:v9,crop:function(a,b,c,d){return Pa.pToFR(uj(function(e){return tj.crop(e,a,b,c,d)}))},resizeTo:function(a){return Pa.pToFR(u9(a))}},zK={mapIfSet:function(a,b){return Pa.map(a,function(a){return a.map(b)})},applyIfSet:function(a){return function(b){return Pa.bind(a,function(a){return a.fold(function(){return Pa.pure(b)},function(a){return a(b)})})}}},
di={partition:function(a){var b=[],c=[];k.each(a,function(a){a.fold(function(a){b.push(a)},function(a){c.push(a)})});return{errors:b,values:c}}},vn=C.immutable("width","height"),AK=ma.generate([{special:["size"]},{regular:["value","size","unit"]}]),wn={identify:function(a){a=a.match(/(\d*\.?\d*)(.*)/);return""!==a[1]?AK.regular(a[0],Number(a[1]),a[2]):AK.special(a[0])}},A9=function(a,b){return t.getRaw(a,b).orThunk(function(){return m.has(a,b)?h.some(m.get(a,b)):h.none()}).bind(function(a){return wn.identify(a).fold(function(a){return h.none()},
function(a,b,c){switch(c){case "":case "px":return h.some(parseInt(b,10));default:return h.none()}})})},Ft=function(a,b,c){b=b(a);return 0<b?h.some(b):A9(a,c)},B9={get:function(a){return Ft(a,qa.get,"width").bind(function(b){return Ft(a,ia.get,"height").map(function(a){return vn(b,a)})})},measure:Ft},BK=function(a,b){var c=b/a;return 1>c?c:1},C9=function(a,b){var c=BK(a.height,b.height);return 1===c?a:{width:a.width*c,height:b.height}},D9=function(a,b){var c=BK(a.width,b.width);return 1===c?a:{width:b.width,
height:a.height*c}},E9={max:function(a,b){var c=0<b.height?b.height:0,d=0<b.width?b.width:0;if(!(a.height>c||a.width>d)||c===d&&0===c)return a;var c={width:d,height:c},d=a.width>a.height,e=0<c.height,g=(e?a.height/c.height:0)>(0<c.width?a.width/c.width:0);return(c.width===c.height&&d||!e||!g?D9:C9)(a,c)}},F9=function(a){var b=Re.createObjectURL(a);return Pa.map(nk.load(b),function(a){a=CK(a);Re.revokeObjectURL(b);return a})},CK=function(a){return vn(a.dom().naturalWidth,a.dom().naturalHeight)},G9=
function(a){var b=CK(a);return B9.get(a).filter(function(a){return a.width()!==b.width()||a.height()!==b.height()})},Xo={restrain:function(a){var b=a.width(),c=a.height(),d=function(a){var d=E9.max({width:a.width(),height:a.height()},{width:b,height:c});return a.height()>c||a.width()>b?h.some(vn(d.width,d.height)):h.none()};return function(a){a=F9(a);return Pa.map(a,d)}},changedBounds:function(a){return Pa.map(nk.loadIfNeeded(a),G9)}},H9=function(a){a=zK.mapIfSet(a,pg.resizeTo);return zK.applyIfSet(a)},
DK=function(a,b,c){return Pa.fromChain([H9(a),c])(b)},Th={instance:function(a){var b=Xo.restrain(a);return function(a){return oc.cata(a,function(d,e,g){d=Pa.fromPromise(tc.imageResultToOriginalBlob(e));return Pa.bind(d,function(d){var e=b(d);return DK(e,d,function(b){if(b===d)return Pa.pure(a);Re.revokeObjectURL(g);return Pa.fromFuture(we.single(b))})})},function(){return Pa.pure(a)})}},processResults:function(a){return function(b){b=di.partition(b);0<b.errors.length&&(a("imageedit.generic"),console.error("Image resample errors: ",
b.errors));return b.values}},resampleAction:DK},EK=x.resolve("find-match"),FK=x.resolve("find-mark"),Gt="."+EK+", ."+FK,Xc={configure:function(a){a.addSelector(a.output(),Gt,Ca.unwrap);a.addSelector(a.history(),Gt,Ca.unwrap);a.addSelector(a.internal(),Gt,Ca.unwrap)},matchClass:f.constant(EK),markClass:f.constant(FK),findIdAttr:f.constant("data-ephox-find-id"),shiftKeyState:function(a){return{isShift:f.constant(a)}}},GK=function(a){return{fold:a}},xn={created:function(a){return GK(function(b,c){return c(a)})},
preserved:function(a){return GK(function(b,c){return b(a)})}},I9=C.immutable("id","match","current"),Ht={resolve:lb.create("data.ephox.cram").resolve},It={resolve:lb.create("ephox-cram").resolve},HK=Ht.resolve("highlight-id"),J9=It.resolve("highlight-wrap"),IK=function(a,b){return J.closest(a,"."+b)},JK=function(a){return h.from(m.get(a,HK))},lf={marker:IK,idMarker:function(a){return IK(a,J9)},id:JK,set:function(a,b){m.set(a,HK,b);n.add(a,b)},all:function(a){return JK(a).fold(f.constant([a]),function(b){var c=
y.owner(a);return X.descendants(c,"."+b)})}},Jt=C.immutable("word","pattern"),K9={separate:function(a,b,c){var d=k.bind(c,function(a){return[a.start(),a.finish()]}),e=ad.splits(b,d,function(b,c){return iE.subdivide(a,b.element(),c)});return k.map(c,function(b){var c=ad.sublist(e,b.start(),b.finish()),c=k.map(c,function(a){return a.element()}),d=k.map(c,a.property().getText).join("");return{elements:f.constant(c),word:b.word,exact:f.constant(d)}})}},L9=function(a,b){return ad.generate(b,function(b,
d){var e=d+a.property().getText(b).length;return h.from(S.range(b,d,e))})},Kt=function(a,b,c,d){b=jE.group(a,b,d);return k.bind(b,function(b){b=il.justText(b);var d=k.map(b,a.property().getText).join(""),d=Dm.findmany(d,c);b=L9(a,b);return K9.separate(a,b,d)})},Lt={safeWords:function(a,b,c,d){c=k.map(c,function(a){var b=db.safeword(a);return Jt(a,b)});return Kt(a,b,c,d)},safeToken:function(a,b,c,d){c=Jt(c,db.safetoken(c));return Kt(a,b,[c],d)},run:Kt},Mt={safeWords:function(a,b,c,d){return Lt.safeWords(a,
b,c,d)},safeToken:function(a,b,c,d){return Lt.safeToken(a,b,c,d)},run:function(a,b,c,d){return Lt.run(a,b,c,d)}},Nt=Sa(),yn={safeWords:function(a,b,c){return Mt.safeWords(Nt,a,b,c)},safeToken:function(a,b,c){return Mt.safeToken(Nt,a,b,c)},run:function(a,b,c){return Mt.run(Nt,a,b,c)}},M9=function(a,b){var c=k.bind(a.elements(),function(a){return lf.marker(a,b).fold(f.constant([]),function(a){return[a]})}),d=La.generate("ephox-cram");return I9(d,a,h.from(c[0]))},vj={matchWords:function(a,b,c){return yn.safeWords(a,
b,c)},matchToken:function(a,b,c){return yn.safeToken(a,b,c)},matchPattern:function(a,b,c){return yn.run(a,[b],c)},annotates:function(a,b){return k.map(a,function(a){return M9(a,b)})}},KK=function(a,b,c,d){b=vj.annotates(d,c.label());return k.bind(b,function(b){return b.current().fold(function(){var d=b.match();if(a.ignore(d))return[];d=Yg.wrapper(d.elements(),c.nu)||[];k.each(d,f.curry(c.tag,b));return[xn.created(d)]},function(a){return[xn.preserved(a)]})})},N9={highlight:function(a,b,c,d){c=vj.matchToken(b,
c,a.optimise);return KK(a,b,d,c)},highlightPattern:function(a,b,c,d){c=vj.matchPattern(b,c,a.optimise);return KK(a,b,d,c)}},O9=function(a,b){return A.isElement(b)&&fa.isBlock(b)?0>pb.get(b).toLowerCase().indexOf(a):!1},P9=function(a){return{ignore:f.constant(!1),optimise:f.curry(O9,a.toLowerCase())}},zn=f.constant("."+Xc.matchClass()),Q9={label:Xc.matchClass,tag:function(a,b){n.add(b,Xc.matchClass());m.set(b,Xc.findIdAttr(),a.id())},nu:function(){var a=q.fromTag("span");return{element:f.constant(a),
wrap:f.curry(v.append,a)}}},LK=function(a){k.each(a,function(a){n.add(a,Xc.markClass())})},R9=function(a){k.each(a,function(a){n.remove(a,Xc.markClass())})},Ot=function(a,b){var c=k.groupBy(X.descendants(a,b),function(a){return m.get(a,Xc.findIdAttr())});return 0===c.length?h.none():h.some(c)},MK=function(a,b){var c=db.custom(db.sanitise(a),f.constant(0),f.constant(0),b);return Jt(a,c)},Pt=function(a,b,c){return Ot(a,zn()).map(function(d){var e=k.filter(d,function(a){return n.has(a[0],Xc.markClass())}),
f=k.findIndex(d,function(a){return k.contains(e,a)}).getOr(-1);return{pos:c(d,k.flatten(e),f,b(f,d.length)),all:Ot(a,zn())}})},Qt=function(a,b,c,d){R9(b);LK(a[d]);return d},NK=function(a,b){return a===b-1?0:a+1},S9=function(a,b){return 0===a?b-1:a-1},OK=function(a,b){return Pt(a,NK,function(a,d,e,f){Qt(a,d,e,f);v.before(d[0],q.fromText(b));k.each(d,B.remove);return 0===f?f:e})},ee={search:function(a,b,c){if(0===b.length)return h.none();var d=P9(b),e=y.children(a);b=!0===c?MK(b,h.none()):MK(b,h.some("gi"));
N9.highlightPattern(d,e,b,Q9);return Ot(a,zn()).map(function(a){LK(a[0]);return{all:h.some(a),pos:0}})},clear:function(a){a=X.descendants(a,zn());k.each(a,B.unwrap);return h.none()},next:function(a){return Pt(a,NK,Qt)},prev:function(a){return Pt(a,S9,Qt)},replace:OK,replaceAll:function(a,b){for(;OK(a,b).isSome(););return h.none()}},ax=Y.detect().deviceType.isiOS(),Rt=function(a,b,c){a=a();var d=a.element();ea.add(d,b);Q.button(d,c);m.set(d,"tabindex","-1");a.events.click.bind(function(){T.focus(d)});
return a},St=function(a,b){var c=Rt(ac,a,b);m.set(c.element(),"title",b);return c},PK=function(a,b){var c=x.resolve("inline-main-button"),c=a.concat([c]),c=Rt(ac,c,b);c.content(qc(b));return c},QK=function(a,b,c){var d=x.resolve("inline-main-button-toggle");a=a.concat([d]);c=Rt(id,a,c);m.set(c.element(),"title",b);return c},Tt=function(a,b,c){var d=q.fromTag("input");ea.add(d,a);m.set(d,"placeholder",b);Q.label(d,c);return{element:f.constant(d),destroy:f.curry(B.remove,d)}},vk={counter:function(a){var b=
P.div(),c=P.span();n.add(b,x.resolve("search-widget-counter"));n.add(c,x.resolve("search-widget-counter-text"));v.append(b,c);return{element:f.constant(b),update:function(a,b){var f=u.translate("units.amount-of-total",a,b);B.empty(c);v.append(c,q.fromText(f));Tb.isVisible(c)&&Oa.speak(ca.body(),f)},hide:function(){t.set(b,"display","none")},show:function(){t.remove(b,"display")},destroy:function(){B.remove(c);B.remove(b)}}},dummyCounter:function(a){a=P.div();n.add(a,x.resolve("search-widget-counter"));
return{element:f.constant(a),update:f.noop,hide:f.noop,show:f.noop,destroy:f.curry(B.remove,a)}},searchField:function(){var a=P.span(),b=St([x.resolve("stepper-button"),x.resolve("search-widget-previous")],u.translate("search.previous")),c=St([x.resolve("stepper-button"),x.resolve("search-widget-next")],u.translate("search.next")),d=QK([x.resolve("search-widget-more")],u.translate("aria.label.more"),u.translate("aria.label.more"));n.add(a,x.resolve("search-widget-searchField"));var e=Tt([x.resolve("search-widget-input"),
x.resolve("search-widget-input-search")].concat(Mb.isSmallTouch()?[x.resolve("search-widget-small-touch")]:[]),u.translate("search.search"),u.translate("search.field.search")),g=D.create({next:M([],c.events.click),previous:M([],b.events.click),more:M([],d.events.click),input:s(["raw"])}),h=E.bind(e.element(),"input",g.trigger.input);K.append(a,[e.element(),b.element(),c.element(),d.element()]);return{events:g.registry,element:f.constant(a),input:e.element,focus:function(){T.focus(e.element())},previous:b.element,
next:c.element,moreToggle:d,destroy:function(){h.unbind();e.destroy();b.destroy();c.destroy();d.destroy()}}},replaceDrawer:function(){var a=Tt([x.resolve("search-widget-input"),x.resolve("search-widget-input-replace")],u.translate("search.replace"),u.translate("search.field.replace")),b=QK([x.resolve("search-widget-matchcase")],u.translate("search.matchcase"),u.translate("search.matchcase")),c=PK([x.resolve("search-widget-replace")],u.translate("search.replace")),d=PK([x.resolve("search-widget-replace-all")],
u.translate("search.replace-all")),e=q.fromTag("div");Q.group(e);var g=P.div();n.add(e,x.resolve("search-widget-replace-drawer"));n.add(g,x.resolve("search-widget-replace-drawer-tools"));K.append(g,[b.element(),c.element(),d.element()]);K.append(e,[a.element(),g]);return{element:f.constant(e),replace:c,replaceAll:d,input:a,matchcase:b,destroy:function(){b.destroy();c.destroy();d.destroy();B.remove(g);B.remove(e)}}},button:St,input:Tt},gS=3,cx=function(a,b,c){a?(ha.expanded(b),Q.hidden(c,!1)):(ha.collapsed(b),
Q.hidden(c,!0))},Uh={text:ma.generate([{text:["field","input"]}]).text,cata:function(a,b){return a.fold(b)}},kS=Y.detect(),jS=Gb.ensureWith("element addEscape open close execute type events destroy".split(" "),iA.isFunctionOrEvents(["toEditor"])),Uo="search",RK=x.resolve("toolbar-dialog"),T9=function(a){var b=hS(a);return Av(a,b,Uo,RK)},U9=function(a){var b=iS(a);return wk(a,b,RK)},mS=function(a){return(Y.detect().deviceType.isTouch()?U9:T9)(a)},V9=function(a){return De.isAnnotation(a)||m.has(a,jb.overrideSpan())},
SK=function(a,b,c,d,e,g){var h=function(b){a(b,e,V9)};tb.cata(d.get(),f.noop,Gm.multiple(b,c,h,g),Gm.single(b,c,h,g))},An=function(a,b,c,d,e){SK(a,b,c,d,e,b.toEditing)},He={face:f.curry(An,ch.face),background:f.curry(An,ch.background),foreground:f.curry(An,ch.foreground),size:f.curry(An,ch.size),partialSize:f.curry(function(a,b,c,d,e){SK(a,b,c,d,e,f.noop)},ch.size)},Xh={color:C.immutable("hex","classes","name"),face:C.immutable("textkey","value","icon"),size:C.immutable("name","size")},Jb=Xh.color,
Ld=Xh.face,Bn=Xh.size,W9=hb.cached(function(){return[Jb("#FFF",["color-highlight-dark","color-option-white"],u.key("color.white")),Jb("#000",["color-highlight-light"],u.key("color.black")),Jb("#444",["color-highlight-light"],u.key("color.gray")),Jb("#777",["color-highlight-light"],u.key("color.metal")),Jb("#CCC",["color-highlight-light"],u.key("color.smoke")),Jb("#FC1D00",["color-highlight-light"],u.key("color.red")),Jb("#C81500",["color-highlight-light"],u.key("color.darkred")),Jb("#FF8C00",["color-highlight-dark"],
u.key("color.darkorange")),Jb("#FEBE00",["color-highlight-dark"],u.key("color.orange")),Jb("#FFFC00",["color-highlight-dark"],u.key("color.yellow")),Jb("#22AE50",["color-highlight-dark"],u.key("color.green")),Jb("#006400",["color-highlight-dark"],u.key("color.darkgreen")),Jb("#3CB371",["color-highlight-dark"],u.key("color.mediumseagreen")),Jb("#8FCD4E",["color-highlight-dark"],u.key("color.lightgreen")),Jb("#00FF00 ",["color-highlight-dark"],u.key("color.lime")),Jb("#0000CD",["color-highlight-dark"],
u.key("color.mediumblue")),Jb("#002360",["color-highlight-light"],u.key("color.navy")),Jb("#0173C1",["color-highlight-light"],u.key("color.blue")),Jb("#14B2F2",["color-highlight-dark"],u.key("color.lightblue")),Jb("#ee82ee",["color-highlight-light"],u.key("color.violet"))]}),X9=hb.cached(function(){return[Ld(u.key("font.arial"),"Arial",h.none()),Ld(u.key("font.comicsans"),"Comic Sans MS",h.none()),Ld(u.key("font.consolas"),"Consolas",h.none()),Ld(u.key("font.courier"),"Courier",h.none()),Ld(u.key("font.georgia"),
"Georgia",h.none()),Ld(u.key("font.impact"),"Impact",h.none()),Ld(u.key("font.tahoma"),"Tahoma",h.none()),Ld(u.key("font.timesnr"),"Times New Roman",h.none()),Ld(u.key("font.trebuchet"),"Trebuchet MS",h.none()),Ld(u.key("font.verdana"),"Verdana",h.none())]}),Y9=[Bn("font.small","10px"),Bn("font.medium","13px"),Bn("font.large","18px"),Bn("font.huge","32px")],Ak={colors:W9,faces:X9,sizes:f.constant(Y9)},xk={configuration:f.constant({i18nNamespace:"font",cssNamespace:"font",property:"font-size",tag:"span",
specials:["xx-small x-small small medium large x-large xx-large".split(" ")]})},Z9=function(a){var b=!isNaN(parseFloat(a))&&isFinite(a)?"px":"";return a+b},$9=function(a,b,c){return(t.isValidValue(a,b,c)?h.some(c):h.none()).fold(function(){return H.error("Invalid value: "+c)},function(a){return a===c?H.value(c):H.error("Invalid value: "+c)})},TK=function(a){return wn.identify(a).fold(function(a){return a},function(a,c,d){return Math.round(10*c)/10+d})},Ie={identify:wn.identify,validate:function(a,
b,c){return(Qe.isEmpty(a)?H.error("can not use empty value for font size."):H.value(a)).map(Z9).map(TK).bind(f.curry($9,b,c))},sanitise:function(a){return TK(a)}},Cn={px:1,em:0.1,"%":10,pt:1,rem:"0.1"},a$=function(a,b,c){return k.find(b,function(a){return k.contains(a,c)}).bind(function(b){return k.indexOf(b,c).bind(function(c){return h.from(b[a(c,b)])})}).getOr(c)},UK=function(a,b,c,d){return Ie.identify(d).fold(function(b){return a$(a,c,b)},function(a,c,d){a=void 0!==Cn[d]?Cn[d]:1;var f=b*(void 0!==
Cn[d]?Cn[d]:1);return(R.isNumber(c)&&0<=c?Math.max(a,c+f):a)+d})},yk={decrease:f.curry(UK,function(a,b){return Math.max(0,a-1)},-1),increase:f.curry(UK,function(a,b){return Math.min(b.length-1,a+1)},1)},oS=C.immutableBag(["i18nNamespace","cssNamespace","property","tag","specials"],[]),b$=C.immutable("size"),d$=function(a){a=tb.toSelection(a.get()).fold(oa.off,function(a){var c=a.startContainer();a=k.contains(["td","th"],A.name(c))?Ja.freefallLtr(a.startContainer()).element():a.startContainer();a=
c$(a);return oa.on(a)});return b$(a)},e$=function(a){return a.body().bind(function(a){a=t.get(a,"font-size");return h.from(a)})},VK=function(a){var b=a.dom();if(!b.style)return!1;b=b.style.fontSize;return"body"!==A.name(a)&&!Qe.isEmpty(b)},c$=function(a){var b=A.isText(a)?y.parent(a).getOrDie():a;return(VK(b)?h.some(b):sa.ancestor(b,VK)).fold(function(){return t.get(b,"font-size")},function(a){return a.dom().style.fontSize})},zk={getState:function(a,b){return{readState:d$(a),fallbackSize:e$(b).getOr("12px")}}},
f$=function(a){a.dom().click();return h.some(a)},g$=function(a,b){var c=X.descendants(a,b);if(0<ia.get(a)&&0<c.length){var d=Na.absolute(c[0]).top();return k.findIndex(c.slice(1),function(a){return Na.absolute(a).top()!==d}).map(function(a){return{rows:f.constant(Math.ceil(c.length/(a+1))),columns:f.constant(a+1)}})}return h.none()},Wh={create:function(a){var b=void 0!==a?a:Ak.colors();a=rS();b=k.map(b,function(a){var b=qS(a);ea.add(b.element(),k.map(a.classes(),x.resolve));n.add(b.element(),x.resolve("palette-color-option"));
t.set(b.element(),"background-color",a.hex());return b});k.each(b,a.add);return a},makeLayout:function(a,b,c){c=pa.grid(Z.deepMerge({trigger:f$},c));var d=ba.grid(a,{cell:f.constant(b)},4,5,c);return{layout:f.constant(d),refresh:function(){d.clear();g$(a,b).each(function(a){d.resize(a.rows(),a.columns())})}}}},sS=".ephox-polish-palette-detail-action.ephox-polish-palette-color-option",tS=function(a){return eb.closest(a,".ephox-polish-palette-detail-active")},vS=Y.detect().deviceType.isTouch,FS={widget:function(a){var b=
a.content(),c=a.undo(),d=a.selections(),e=wS(),g=D.create({color:M(["value"],e.events.color),highlight:M(["value"],e.events.highlight),done:s([]),abort:M([],e.events.abort)});e.events.color.bind(function(a){g.trigger.done();He.foreground(b,c,d,a.value());b.toEditing()});e.events.highlight.bind(function(a){g.trigger.done();He.background(b,c,d,a.value());b.toEditing()});return{element:e.element,name:f.constant("color"),textkey:f.constant(u.key("font.coloroption")),menuTextkey:f.constant(u.key("aria.color.menu")),
open:e.open,close:e.close,updateWidget:function(b){Ah.isHighContrast(a.smartview())?b.disable():e.open()},ability:f.noop,destroy:e.destroy,events:g.registry,label:f.constant(!1)}}},zS=C.immutableBag(["i18nNamespace","cssNamespace","property","tag","specials"],[]),DS={widget:function(a,b){var c=ko.cata(b,f.constant(f.noop),f.constant(function(a){a.initLayout()})),d=a.content(),e=a.undo(),g=Bk(xk.configuration());c(g);var h=D.create({done:s([]),abort:M([],g.events.abort)});g.events.close.bind(function(a){h.trigger.done()});
g.events.action.bind(function(b){var c=T.active();d.focus();He.size(d,e,a.selections(),b.value());c.each(T.focus)});return{element:g.element,item:g.item,name:f.constant("size"),textkey:f.constant(u.key("font.size")),open:g.open,close:g.close,updateInlineWidget:function(){var b=zk.getState(a.selections(),d);g.updateInlineWidget(b.readState,b.fallbackSize)},ability:f.noop,destroy:g.destroy,events:h.registry}}},CS={widget:function(a,b){var c=a.content(),d=a.smartview().uiContainer(),e=a.ui().tabstrip.element,
g=a.undo(),h=Bk(xk.configuration()),k=h.input(),r=pk(e());m.set(k,"disabled","true");var n=E.bind(k,"touchstart",function(b){b.kill();r.closeMenus();var e=AS(c,d,f.curry(Bk,xk.configuration()));e.show();b=e.contents;var h=zk.getState(a.selections(),c),l=function(){e.destroy();c.cursorIntoView()};b.updateInlineWidget(h.readState,h.fallbackSize);b.events.close.bind(l);b.events.action.bind(function(b){He.size(c,g,a.selections(),b.value())});e.events.close.bind(e.destroy);e.events.ok.bind(function(b){He.size(c,
g,a.selections(),b.value());l()})}),q=D.create({done:s([]),abort:M([],h.events.abort)});h.events.close.bind(function(a){q.trigger.done()});h.events.action.bind(function(b){He.size(c,g,a.selections(),b.value())});return{element:h.element,item:h.item,name:f.constant("size"),textkey:f.constant(u.key("font.size")),open:h.open,close:h.close,updateInlineWidget:function(){var b=zk.getState(a.selections(),c);h.updateInlineWidget(b.readState,b.fallbackSize)},ability:f.noop,destroy:function(){n.unbind();h.destroy()},
events:q.registry}}},ES={face:function(a){return tb.toSelection(a.get()).map(function(a){var c=a.startContainer();a=k.contains(["td","th"],A.name(c))?Ja.freefallLtr(a.startContainer()).element():a.startContainer();a=Jh.value(a,"font-family");return Qe.isEmpty(a)?oa.off():oa.on(a)}).getOrThunk(oa.unchanged)}},HS=function(a){var b=a.baseUrl;!0===a.source&&(b+="/licenses");return b+"/textbox-licenses.html"},wj=function(a,b){return a.map(function(a){return a.bind(function(a){var c=a[b];return void 0===
c?H.error("Cannot find method "+b+" in help article list: "+W.keys(a)):R.isFunction(c)?H.value(c()):H.error("Key "+b+" in help article list was not a function")})})},JS=function(a){a=dl.addScript(a,f.identity);a.get(function(a){a.each(function(a){Je.weakMatch(a.version,"Help articles").each(function(a){console.warn(a)})})});return{about:wj(a,"about"),accessibility:wj(a,"accessibility"),a11ycheck:wj(a,"a11ycheck"),markdown:wj(a,"markdown"),shortcuts:wj(a,"shortcuts")}},Zh=function(a,b){return a?h.some(b):
h.none()},LS=function(a){var b={item:f.constant(".ephox-pastry-statebutton.ephox-pastry-tablayout-item")},c=pa.flow({trigger:qb.button,executeOnArrow:!0});return ba.flow(a.menu(),b,c)},MS=function(a){var b={stops:f.constant(["a"])},c=pa.page({trigger:qb.click,first:".ephox-pastry-tablayout-active"});return ba.page(a.body(),b,c)},QS=C.immutable("prefix","suffix","style"),fx={rehighlight:function(a,b,c){var d=y.children(c),e=k.bind(d,function(a){return A.isText(a)?Da.get(a):""}).join("");k.find(b,function(a){return aa.startsWith(e,
a.prefix())&&aa.endsWith(e,a.suffix())}).fold(function(){a.preserve(function(){B.unwrap(c)})},function(b){var e=q.fromTag("span");ea.add(e,[ge.highlightClass()]);t.setAll(e,b.style());a.preserve(function(){v.before(c,e);K.append(e,d);B.remove(c)})})},unhighlight:function(a,b){var c=y.children(b),d=q.fromTag("span");ea.add(d,[ge.placeholderClass()]);a.preserve(function(){v.before(b,d);B.unwrap(b);K.append(d,c)})}},h$=function(a,b){var c=k.map(b,function(a){var b=db.sanitise(a.prefix()),c=db.sanitise(a.suffix()),
b=db.unsafetoken(b+"[^"+c+"]*"+c);return{pattern:f.constant(b),style:a.style}});return Dm.findmany(a,c)},i$=Io("."+ge.highlightClass(),Ca.unwrap),RS={run:function(a,b){a.body().each(function(a){ok.filterDom(a,[i$]);a=pc.descendants(a,A.isText);k.each(a,function(a){var c=Da.get(a),c=h$(c,b),f=k.bind(c,function(a){return[a.start(),a.finish()]}),h=Vg.subdivide(a,f);k.each(c,function(a){var b=ad.sublist(h,a.start(),a.finish()),b=k.map(b,function(a){return a.element()});Yg.wrapper(b,function(){var b=q.fromTag("span");
ea.add(b,[ge.highlightClass()]);t.setAll(b,a.style());return Yg.nu(b)})})})})}},Ut=x.resolve("input-invalid-field"),j$=function(a){return parseInt(a,10).toString()===a&&0<=parseInt(a,10)},WK=function(a,b,c){var d=$.get(a);if(""===d)return H.value("");var e=q.fromTag("img");c=j$(d)?d+c.getOr(""):d;t.set(e,b,c);c=t.getRaw(e,b);t.remove(e,b);return c.fold(function(){return H.error(a)},H.value)},XK=function(a){n.remove(a,Ut)},Xb={highlight:function(a){Oa.shout(a,u.translate("aria.input.invalid"));n.add(a,
Ut)},dehighlight:XK,clearHighlights:function(a){a=X.descendants(a,"."+Ut);k.each(a,XK)},validate:function(a){return k.foldr(a,function(a,c){return a.bind(function(a){return WK(c.input(),c.property(),c.defaultUnit()).map(function(b){$.set(c.input(),b);var g={};g[c.property()]=f.constant(b);return Z.deepMerge(g,a)})})},H.value({}))},validateField:WK,rule:C.immutable("input","property","defaultUnit")},k$=C.immutable("size","unit"),Dn=function(a){return wn.identify(a).fold(h.none,function(a,c,d){return h.some(k$(c,
d))})},Vo={fromHeight:function(a,b){return Dn(b).map(function(b){return Math.ceil(a*b.size())+b.unit()})},fromWidth:function(a,b){return Dn(b).map(function(b){return Math.ceil(1/a*b.size())+b.unit()})},calculate:function(a,b){return Dn(a).fold(h.none,function(a){return Dn(b).fold(h.none,function(b){return h.some((a.size()+0)/(b.size()+0))})})}},YK=C.immutable("value","text","attrs"),ZK=function(a){return"img"===A.name(a)},hx={image:function(){var a=h.none(),b=D.create({properties:s(["image"]),contextmenu:s(["image"])});
return{situation:function(b){b=a.map(function(a){return[YK("imagemenu-properties",u.translate("image.context.properties"),{})]}).getOr([]);return{primary:f.constant(b),secondary:f.constant([]),submenus:f.constant([])}},handle:function(c){a.each(function(a){b.trigger.properties(a)})},assign:function(c){a=c.filter(ZK);a.each(b.trigger.contextmenu)},events:b.registry}},imageEdit:function(a,b,c){var d=h.none(),e=function(a){return k.map(a,function(a){return YK(a.spec.value,u.get(a.spec.textkey),{})})};
return{situation:function(a){a=d.map(e).getOr([]);return{primary:f.constant(a),secondary:f.constant([]),submenus:f.constant([])}},handle:function(a){d.each(function(b){var c=m.get(a,"data-ephox-gopher-item-value");k.find(b,function(a){return a.spec.value===c}).each(function(a){a.spec.action()})})},assign:function(e){d=a()?h.some(b):e.filter(ZK).map(f.constant(c))}}}},ix={generate:function(a,b,c){a=function(){e();d.trigger.close()};var d=D.create({ok:M(c,b.events.ok),close:s([])});b.events.close.bind(a);
var e=function(){b.destroy()};return{open:function(){b.open()},hide:a,destroy:e,represent:b.represent,events:d.registry}}},$K=ma.generate([{html:["key","value"]},{css:["key","value"]}]),aL=function(a,b){return k.flatten(W.mapToArray(a,function(a,d){var e=[];if(m.has(b,d)){var f=m.get(b,d);e.push($K.html(a,f))}t.getRaw(b,d).each(function(b){e.push($K.css(a,b))});return e}))},l$=function(a){t.remove(a,"width");t.remove(a,"height");m.remove(a,"width");m.remove(a,"height")},m$=f.curry(aL,{width:"width",
height:"height"}),n$=f.curry(aL,{width:"height",height:"width"}),bL=f.constant([]),he={keepCrop:bL,keepFlip:m$,keepRotate:n$,keepResample:bL,updater:function(a,b){var c=a(b);return Pa.effect(function(){l$(b);k.each(c,function(a){a.fold(function(a,c){m.set(b,a,c)},function(a,c){t.set(b,a,c)})})})}},o$={strictMode:!1,key:"source protocol authority userInfo user password host port relative path directory file query anchor".split(" "),q:{name:"queryKey",parser:/(?:^|&)([^&=]*)=?([^&]*)/g},parser:{strict:/^(?:([^:\/?#]+):)?(?:\/\/((?:(([^:@\/]*)(?::([^:@\/]*))?)?@)?([^:\/?#]*)(?::(\d*))?))?((((?:[^?#\/]*\/)*)([^?#]*))(?:\?([^#]*))?(?:#(.*))?)/,
loose:/^(?:(?![^:@\/]+:[^:@\/]*@)([^:\/?#.]+):)?(?:\/\/)?((?:(([^:@\/]*)(?::([^:@\/]*))?)?@)?([^:\/?#]*)(?::(\d*))?)(((\/(?:[^?#](?![^?#\/]*\.[^?#\/.]+(?:[?#]|$)))*\/?)?([^?#\/]*))(?:\?([^#]*))?(?:#(.*))?)/}},p$=function(a,b){for(var c=b.parser[b.strictMode?"strict":"loose"].exec(a),d={},e=14;e--;)d[b.key[e]]=c[e]||"";d[b.q.name]={};d[b.key[12]].replace(b.q.parser,function(a,c,e){c&&(d[b.q.name][c]=e)});return d},tg={parse:function(a,b){var c=Z.merge(o$,b);return p$(a,c)}},cL=function(a){return""!==
a.trim()&&!aa.contains(a,"://")},Wo={local:function(a){return aa.startsWith(a,"data:")||aa.startsWith(a,"blob:")||aa.startsWith(a,"file:")},relative:cL,sameDomain:function(a,b){if(""===a.trim()||""===b.trim())return!1;if(cL(a))return!0;var c=tg.parse(b),d=tg.parse(a);return c.host===d.host&&c.port===d.port}},q$=C.immutable("bannerMessage","consoleMessage"),dL=function(a){return R.isFunction(a.message)&&R.isFunction(a.status)&&R.isFunction(a.responseText)},eL=function(a){if(dL(a)){var b=a.responseText();
return a.message()+". Response code "+a.status()+(0===b.length?".":". Error details: "+b)}return a},Yo={diagnoseImageEdit:function(a,b){var c=18===a.code||"SecurityError"===a.name?"imageedit.needsproxy":dL(a)&&0===a.status()?"imageedit.proxyerror":"imageedit.generic",d;d=tg.parse(b);d=""===d.host||Wo.relative(b)?b:""===d.port?d.host:d.host+":"+d.port;c=ae.get(c,[d]);return q$(c,eL(a))},consoleMessage:eL},qg={make:function(a,b,c){return{type:"general",spec:{value:a,textkey:u.key(b),action:c,updateGeneral:f.noop,
icon:!0,hasPopup:!1,toggle:!1,tooltip:Wa.get}}}},r$=f.constant(4),Za={resolve:lb.create("ephox-clipper").resolve,CORNER_SIZE:r$},s$=function(a,b){var c={left:"0px",top:"0px",width:a.width()+"px",height:a.height()+"px"},d={left:t.get(b,"left"),top:t.get(b,"top"),width:t.get(b,"width"),height:t.get(b,"height")};return{outside:f.constant(c),inside:f.constant(d)}},t$={snap:function(a,b){var c=s$(a,b),d=q.fromTag("div");n.add(d,Za.resolve("dynamic-box"));t.setAll(d,c.outside());var e=q.fromTag("div");
n.add(e,Za.resolve("opaque-box"));t.setAll(e,c.outside());var f=function(a,b){return 2<Math.abs(parseInt(a,10)-parseInt(b,10))},h=function(a){return za.nu(function(b){b(a)})},k=function(a,b){return za.nu(function(c){var d=E.bind(a,"transitionend",function(){d.unbind();c(a)});t.setAll(a,b.inside())})};return{prepare:function(a,c,f){var g=m.get(c,"src"),h=t.get(c,"width");c=t.get(c,"height");t.setAll(b,{"background-image":"url("+g+")","background-size":h+" "+c,"background-position-x":-f.left()+"px",
"background-position-y":-f.top()+"px"});n.add(b,Za.resolve("snapshot"));n.add(d,Za.resolve("dynamic-box-animate"));K.append(a,[d,e]);t.reflow(a)},showcase:function(){var a=c.outside(),b=c.inside();return f(a.left,b.left)||f(a.top,b.top)||f(a.right,b.right)||f(a.bottom,b.bottom)?k(d,c):h(d)}}}},iT={slide:function(){return{animate:function(a,b,c,d){c=t$.snap(c.box(),c.shade());c.prepare(a,b,{left:d.cropX,top:d.cropY,width:d.cropWidth,height:d.cropHeight});return c.showcase().map(f.constant(d))}}},none:function(){return{animate:function(a,
b,c,d){return za.pure(d)}}}},gT=C.immutableBag("displayWidth displayHeight cropX cropY cropWidth cropHeight".split(" "),[]),dT={bind:function(a,b){return E.bind(a,"mousedown",function(a){a.kill();var d=a.target(),e=!1,f=E.bind(d,"mouseup",function(){e=!0;f.unbind()});setTimeout(function(){e||(f.unbind(),b(d))},250)})}},u$=C.immutableBag(["top","left","width","height"],[]),v$=C.immutableBag(["top","left"],[]),w$=C.immutableBag(["top","left"],[]),fb={boxModel:u$,delta:v$,cornersMovements:w$},Vt=function(a,
b){var c=Math.min(a.top()+a.height(),Math.max(0,a.top()+b.top())),d=c-a.top(),d=a.height()-d;return{top:c,height:d}},Wt=function(a,b,c){a=Math.min(a.width(),Math.max(b.left(),b.left()+b.width()+c.left()))-b.left()-b.width();return{width:b.width()+a}},Xt=function(a,b,c){return Math.min(a.height(),Math.max(b.top(),b.top()+c.top()+b.height()))-b.top()},Yt=function(a,b){var c=Math.min(a.width()+a.left(),Math.max(0,a.left()+b.left())),d=c-a.left(),d=a.width()-d;return{left:c,width:d}},Gk={nw:function(a,
b,c){a=Vt(b,c);b=Yt(b,c);return fb.boxModel({left:b.left,width:b.width,top:a.top,height:a.height})},n:function(a,b,c){a=Vt(b,c);return fb.boxModel({left:b.left(),width:b.width(),top:a.top,height:a.height})},ne:function(a,b,c){var d=Vt(b,c);a=Wt(a,b,c);return fb.boxModel({left:b.left(),width:a.width,top:d.top,height:d.height})},e:function(a,b,c){a=Wt(a,b,c);return fb.boxModel({left:b.left(),width:a.width,top:b.top(),height:b.height()})},se:function(a,b,c){var d=Wt(a,b,c);a=Xt(a,b,c);return fb.boxModel({left:b.left(),
width:d.width,top:b.top(),height:a})},s:function(a,b,c){a=Xt(a,b,c);return fb.boxModel({left:b.left(),width:b.width(),top:b.top(),height:a})},sw:function(a,b,c){a=Xt(a,b,c);c=Yt(b,c);return fb.boxModel({left:c.left,width:c.width,top:b.top(),height:a})},w:function(a,b,c){a=Yt(b,c);return fb.boxModel({left:a.left,width:a.width,top:b.top(),height:b.height()})},translate:function(a,b,c){var d=Math.min(Math.max(0,b.top()+c.top()),Math.max(0,a.height()-b.height()));a=Math.min(Math.max(0,b.left()+c.left()),
Math.max(0,a.width()-b.width()));return fb.boxModel({top:d,left:a,width:b.width(),height:b.height()})}},ai={navigation:function(a,b){var c=function(a,c,f){return function(){b(a,Ba(c,f));return h.some(!0)}};return[U.rule(U.and([U.isShift,U.inSet(N.LEFT())]),c(a,-1,0)),U.rule(U.and([U.isShift,U.inSet(N.RIGHT())]),c(a,1,0)),U.rule(U.and([U.isShift,U.inSet(N.UP())]),c(a,0,-1)),U.rule(U.and([U.isShift,U.inSet(N.DOWN())]),c(a,0,1)),U.basic(N.LEFT()[0],c(a,-10,0)),U.basic(N.RIGHT()[0],c(a,10,0)),U.basic(N.UP()[0],
c(a,0,-10)),U.basic(N.DOWN()[0],c(a,0,10))]}},fL=function(a,b,c){var d=b.top(),e=b.left()+b.width()/2-c.middle(),g=d+b.height()/2-c.middle(),h=b.left()+b.width()-c.right();a=b.left()-c.left();c=d+b.height()-c.bottom();b=fb.cornersMovements({top:d,left:e});var e=fb.cornersMovements({top:c,left:e}),k=fb.cornersMovements({top:g,left:h}),g=fb.cornersMovements({top:g,left:a}),m=fb.cornersMovements({top:d,left:a}),d=fb.cornersMovements({top:d,left:h}),h=fb.cornersMovements({top:c,left:h});a=fb.cornersMovements({top:c,
left:a});return{n:f.constant(b),s:f.constant(e),e:f.constant(k),w:f.constant(g),nw:f.constant(m),ne:f.constant(d),se:f.constant(h),sw:f.constant(a)}},jx={calculate:function(a,b){var c={middle:f.constant(2*Za.CORNER_SIZE()+1),right:f.constant(Za.CORNER_SIZE()),left:f.constant(0),bottom:f.constant(Za.CORNER_SIZE())};return fL(a,b,c)},limitBounds:function(a,b){var c={};if(20>b.width()){var d=20-b.width();c.width=b.width()+d;c.left=b.left()===a.left()?a.left():b.left()-d}20>b.height()&&(d=20-b.height(),
c.height=b.height()+d,c.top=b.top()===a.top()?a.top():b.top()-d);return Z.merge(b,W.map(c,f.constant))},doCalculate:fL},Zt=function(a){return fb.boxModel({left:parseInt(m.get(a,"data-"+Za.resolve("image-left")),10),top:parseInt(m.get(a,"data-"+Za.resolve("image-top")),10),width:parseInt(m.get(a,"data-"+Za.resolve("image-width")),10),height:parseInt(m.get(a,"data-"+Za.resolve("image-height")),10)})},gL=function(a,b){m.set(a,"data-"+Za.resolve("image-left"),b.left());m.set(a,"data-"+Za.resolve("image-top"),
b.top());m.set(a,"data-"+Za.resolve("image-width"),b.width());m.set(a,"data-"+Za.resolve("image-height"),b.height())},zf={readPosition:Zt,writePosition:gL,updatePosition:function(a,b){var c=Zt(a),d=b(c),c=jx.limitBounds(c,d);gL(a,c);return c},drawAtOrigin:function(a){var b=Zt(a);t.setAll(a,{left:b.left()+"px",top:b.top()+"px",width:b.width()+"px",height:b.height()+"px"})}},x$=Za.resolve("magic-box"),lx={magicBox:f.constant(x$)},Fk={grab:function(a,b){m.set(a,"aria-grabbed",b)},effect:function(a,b){m.set(a,
"aria-dropeffect",b)}},y$=C.immutable("help","ids"),hL=function(a){m.set(a,"role","gridcell")},Af={base:function(a,b){m.setAll(a,{role:"grid","aria-label":b})},row:function(a){m.set(a,"role","row")},cell:function(a,b){hL(a);m.set(a,"aria-label",b)},gridcell:hL,createHelp:function(a,b,c){var d=q.fromTag("div");Q.presentation(d);n.add(d,ui.resolve("aria-help"));for(var e=[],f=0;f<b;f++){var h=La.generate("ephox-aria"),k=q.fromTag("span");Q.presentation(k);m.set(k,"id",h);n.add(k,ui.resolve("aria-help"));
v.append(k,q.fromText(c.col(f+1)));v.append(d,k);e[f]=h}f=[];for(h=0;h<a;h++){var k=La.generate("ephox-aria"),r=q.fromTag("span");Q.presentation(r);m.set(r,"id",k);n.add(r,ui.resolve("aria-help"));v.append(r,q.fromText(c.row(h+1)));v.append(d,r);f[h]=[];for(r=0;r<b;r++)f[h][r]=e[r]+" "+k}return y$(d,f)}},Le=function(a,b,c){var d=q.fromTag("div");t.set(d,"position","absolute");var e=q.fromTag("div");v.append(d,e);c.each(function(a){Af.cell(d,a);Fk.grab(d,!1)});ea.add(d,[Za.resolve("corner-wrapper"),
Za.resolve("corner-wrapper-"+b)]);ea.add(e,[Za.resolve("corner-element"),Za.resolve("corner-"+a)]);m.set(d,"data-"+Za.resolve("direction"),b);return d},Me=function(a,b){t.setAll(a,{top:b.top()+"px",left:b.left()+"px"})},kx=function(a,b,c){var d=b()<c()?0:1;k.each(a,function(a){t.set(a,"opacity",String(d))})},$S={derive:function(a,b){return{leftRim:function(){var c=b.left(),d=a.height();return fb.boxModel({top:0,left:0,width:c,height:d})},topRim:function(){var c=b.left(),d=a.width()-b.left(),e=b.top();
return fb.boxModel({top:0,left:c,width:d,height:e})},rightRim:function(){return fb.boxModel({top:b.top(),left:b.left()+b.width(),width:a.width()-b.width()-b.left(),height:a.height()-b.top()})},bottomRim:function(){return fb.boxModel({top:b.top()+b.height(),left:b.left(),width:b.width(),height:a.height()-b.height()-b.top()})},shadeBox:f.constant(b)}}},Dk=function(a){var b=q.fromTag("div");ea.add(b,[Za.resolve("panel-"+a),Za.resolve("side-panel")]);return b},Ek=function(a,b){t.setAll(a,{top:b.top()+
"px",left:b.left()+"px",width:b.width()+"px",height:b.height()+"px"})},cT=function(a){var b=a.width();a=a.height();var c=Math.floor(0.125*a),d=Math.floor(0.125*b);return fb.boxModel({top:c,left:d,width:b-2*d,height:a-2*c})},fT=function(a){var b=Na.absolute(a),c=b.top(),d=ia.get(a),b=b.left();a=qa.get(a);return fb.boxModel({top:c,left:b,width:a,height:d})},jT=function(a,b){a.getSelection().bind(ib.detect).fold(function(){a.setRelativeSelection(da.before(b),da.before(b))},function(b){a.setRelativeSelection(da.on(b.image(),
0),da.on(b.image(),0))})},z$=Pa.fToFR(we.single),Zo={run:function(a,b,c){return Pa.fromChain([a.getImage,b,z$,c])},applyEdit:function(a){return function(b){var c=oc.cata(b,function(a,b,c){return c},function(a,b,c){return b});m.set(a,"src",c);b=Pa.pure(b);return a.dom().loaded?b:Pa.bind(hw.waitFor(a,"load",1E3),f.constant(b))}}},A$=function(a){return 20<=a},$t=function(a){return h.from(a).filter(A$)},lT={ensureMin:$t,parse:function(a,b){var c=$t(a).getOr(0),d=$t(b).getOr(0);return 0===c&&0===d?h.none():
h.some(vn(c,d))}},oT=function(a){return function(b){var c=b.image(),d=b.asset(),e=function(a){a.fold(function(){m.remove(c,"src")},function(a){oc.cata(a,function(a,b,d){m.set(c,"src",d)},f.noop)})};return nk.loadIfNeeded(c).bind(function(){return a(d).map(function(a){e(a);return a})})}},mT=f.constant(Pa.pure),nT=f.constant(function(a){return Pa.pure(a.asset())}),mx={edit:function(a,b,c,d){return{items:d,updatePalette:function(d){if(c())d.hide(h.none())&&a();else{var g=f.curry(d.show,h.some(u.translate("image.context.palette.show")));
d=f.curry(d.hide,h.some(u.translate("image.context.palette.hide")));(b().isSome()?g:d)()&&a()}}}},crop:function(a,b,c){return{items:c,updatePalette:function(c){(b()?c.show:c.hide)(h.none())&&a()}}}},iL=function(a){var b=aa.startsWith(a,"www.")?"http://"+a:a;return{text:f.constant(a),href:f.constant(b)}},au={preprocess:iL,auto:function(a){return iL(a).href()}},jL=function(a,b){a.setRelativeSelection(da.after(b),da.after(b))},B$=function(a,b,c,d){var e=a.getSelection();return e.bind(ib.detect).fold(function(){return e.fold(function(){return[]},
function(a){return(a.collapsed()?c:d)(a)})},function(a){return b(a.image())})},C$=function(a,b,c){var d=q.fromTag("a");En(d,Z.deepMerge(b,{text:f.constant(h.none())}));v.wrap(c,d);ib.select(a,c);return[d]},En=function(a,b){b.url().each(function(b){b=au.auto(b);Tg.attr(a,"href",b)});b.title().each(function(b){Tg.attr(a,"title",b)});b.target().each(function(b){Tg.attr(a,"target",b)});b.action().each(function(b){b(a)});return b.text().filter(b.textChanged).map(function(b){pb.set(a,b);return!0}).getOr(!1)},
D$=function(a){return function(b){return{link:b,properties:a}}},E$=function(a){return function(b){return{link:b,properties:Z.deepMerge(a,{text:f.constant(h.some(pb.get(b)))})}}},F$=function(a){return function(b){var c=Er.findDescendants(b.startContainer(),b.startOffset(),b.endContainer(),b.endOffset(),"a");b=0<c.length?c:Er.findAncestors(b.startContainer(),b.endContainer(),"a");return k.map(b,1===b.length?D$(a):E$(a))}},G$=function(a,b){return B$(a,f.curry(C$,a,b),function(c){var d=q.fromTag("a");
En(d,b);Cc.insertAtCursor(a,[d]);b.action().each(function(a){a(d)});jL(a,d);return[d]},function(c){a.execCommand("CreateLink",!1,au.auto(b.url().getOr("")));c=a.getSelection().map(F$(b)).getOr([]);k.each(c,function(c){En(c.link,c.properties);b.action().each(function(a){a(c.link)});jL(a,c.link)});return k.map(c,function(a){return a.link})})},bu={hyperlink:function(a,b){var c=b.text().fold(f.constant(b),function(a){return Z.deepMerge(b,{text:f.constant(h.some(0<a.trim().length?a:b.url().getOr(""))),
textChanged:0<a.trim().length?b.textChanged:f.constant(!0)})});a.focus();return c.url().fold(f.constant([]),function(b){return 0<b.trim().length?G$(a,c):[]})},setProperties:En},kL=function(a,b){var c=ka.getEnd(b);a.setSelection(b,c,b,c)},lL=function(a,b){a.setSelection(b,0,b,0)},Yx={selectEnd:kL,moveToEnd:function(a){a.getSelection().each(function(b){kL(a,b.endContainer());a.focus()})},selectStart:lL,moveToStart:function(a){a.getSelection().each(function(b){lL(a,b.endContainer());a.focus()})}},mL=
function(a,b){return a.getSelection().bind(function(c){return Uc.sharedLinkOrClosest(c,a.isRoot).map(function(a){b(a);return a})})},dp={insert:bu.hyperlink,update:function(a,b){return mL(a,function(c){b.text().fold(function(){bu.setProperties(c,b)},function(d){""===d&&""!==pb.get(c)?(a.setRelativeSelection(da.before(c),da.before(c)),B.remove(c)):bu.setProperties(c,b)&&(d=y.children(c).length,a.setRelativeSelection(da.on(c,d),da.on(c,d)))});a.focus()})},unlink:function(a){mL(a,function(b){a.setSelection(b,
0,b,y.children(b).length);var c=y.lastChild(b);B.unwrap(b);c.each(function(b){Yx.selectEnd(a,b)});a.focus()})}},H$=function(a){return a.getSelection().bind(function(b){return a.win().bind(function(a){var d=Wb.exact(b.startContainer(),b.startOffset(),b.endContainer(),b.endOffset());return Fa.getFirstRect(a,d).map(function(a){a={left:a.left(),top:a.top(),right:a.right(),bottom:a.bottom(),width:a.width(),height:a.height()};return{bounds:f.constant(a),element:f.constant(b.startContainer())}})})})},nL=
function(a,b,c){H$(b).fold(function(){zw.insertAtCursor(a,b,c)},function(d){var e=d.bounds();d=d.element();(A.isText(d)?y.parent(d):h.some(d)).each(function(d){d=a.directionAt(d);var h=b.translateScreen(e.left,e.top);c({left:h.left,top:h.top,height:f.constant(e.height),width:f.constant(e.width),direction:f.constant(d)})})})},oL=function(a,b,c,d,e,f,h){var k=e.direction();a=Lk.makeshift(a,b,e.left(),e.top(),e.width(),e.height(),k,f,h);b.show(c,a);c.setComponents([d.element()])},cp={show:function(a,
b,c,d,e){nL(a,b,function(a){oL(b,c,d,e,a,fg(0,0))})},showExpandable:function(a,b,c,d,e){nL(a,b,function(a){n.add(d.panel(),x.resolve("expandable-popup"));var f=fg(22,14),h={maxHeightFunction:Rd.expandable()};oL(b,c,d,e,a,f,h)})}},cu=function(a,b,c){a=ac(a);b=u.translate(b);a.content(qc(b));n.add(a.element(),x.resolve(c));Q.button(a.element(),b);return a},pL=function(a){var b=P.div();n.add(b,x.resolve("main-controls"));K.append(b,a);return b},qL=function(a,b,c){var d=cu("remove","link.dialog.remove",
"link-remove"),e=cu("cancel","button.cancel","link-cancel"),f=cu(a,b,c);a=D.create({remove:M([],d.events.click),cancel:M([],e.events.click),execute:M([],f.events.click)});b=[ba.button(d.element(),pa.button({execute:qb.click})),ba.button(e.element(),pa.button({execute:qb.click})),ba.button(f.element(),pa.button({execute:qb.click}))];return{remove:d,cancel:e,execute:f,keylayouts:b,events:a.registry,destroy:function(){d.destroy();e.destroy();f.destroy()}}},rx={insert:function(){var a=qL("insert","link.dialog.insert",
"link-insert"),b=pL([a.cancel.element(),a.execute.element()]);a.remove.disable();ha.disable(a.remove.element());return{dialogTitle:f.constant(u.key("link.dialog.aria.insert")),elements:f.constant([a.remove.element(),b]),events:f.constant(a.events),keylayouts:f.constant(a.keylayouts),execute:a.execute,destroy:a.destroy}},update:function(){var a=qL("update","link.dialog.update","link-update"),b=pL([a.cancel.element(),a.execute.element()]);return{dialogTitle:f.constant(u.key("link.dialog.aria.update")),
elements:f.constant([a.remove.element(),b]),events:f.constant(a.events),keylayouts:f.constant(a.keylayouts),execute:a.execute,destroy:a.destroy}}},rL=x.resolve("link-validation-highlight"),sL=x.resolve("link-validation-invalid"),tL=x.resolve("link-validation-unknown"),uL=f.constant([rL,sL,tL]),yd={configure:function(a){k.each(uL(),function(b){a.add(a.output(),b,Ca.removeClass(b));a.add(a.internal(),b,Ca.removeClass(b))})},allClasses:uL,invalidLink:f.constant(sL),unknownLink:f.constant(tL),validationHighlight:f.constant(rL)},
qx={enabled:function(a){var b=h.none(),c=C.immutable("value","text","attrs"),d=D.create({properties:s(["link"]),follow:s(["link"]),ignore:s(["link"]),remove:s(["link"])}),e={"linkmenu-properties":d.trigger.properties,"linkmenu-follow":d.trigger.follow,"linkmenu-ignore":d.trigger.ignore,"linkmenu-remove":d.trigger.remove};return{situation:function(a){a=b.map(function(a){var b=[c("linkmenu-properties",u.translate("link.context.edit"),{}),c("linkmenu-follow",u.translate("link.context.follow"),{}),c("linkmenu-remove",
u.translate("link.context.remove"),{})];a=n.has(a,yd.validationHighlight())?[c("linkmenu-ignore",u.translate("link.context.ignore"),{})]:[];return b.concat(a)}).getOr([]);return{primary:f.constant(a),secondary:f.constant([]),submenus:f.constant([])}},handle:function(a){b.each(function(b){var c=m.get(a,"data-ephox-gopher-item-value"),c=e[c];void 0!==c&&c(b)})},assign:function(c){b=c.bind(function(b){return J.closest(b,"a[href]",a.isRoot)})},events:d.registry}},disabled:function(){var a=nf.noSituation(),
b=D.create({properties:s(["link"])});return{situation:a,handle:f.noop,assign:f.noop,events:b.registry}}},Fn=x.resolve("link-eph-cursor"),I$=function(a){k.each(a,B.remove)},J$=function(a,b){vL("textbox-bookmark-top",a,v.prepend,b,Xa.top())},K$=function(a,b){vL("textbox-bookmark-bottom",a,v.append,b,Xa.bottom())},vL=function(a,b,c,d,e){var f=ek.generate(a);k.each(b,function(a){m.set(a,"href","#"+f)});a=X.children(d,"["+Xa.bookmarkDataAttr()+'="'+e+'"]');k.each(a,B.remove);0<b.length&&(b=q.fromTag("span"),
m.setAll(b,{id:f,style:"visibility: hidden;"}),m.set(b,Xa.bookmarkDataAttr(),e),c(d,b))},sg={configure:function(a){a.add(a.internal(),Fn,Ca.remove);a.add(a.output(),Fn,Ca.remove);a.addExcludedContext(["."+Fn]);a.addSelector(a.input(),"["+Xa.bookmarkDataAttr()+"]",I$);a.addSelector(a.output(),"["+Xa.bookmarkAnchorDataAttr()+'="'+Xa.top()+'"]',J$);a.addSelector(a.output(),"["+Xa.bookmarkAnchorDataAttr()+'="'+Xa.bottom()+'"]',K$)},properties:C.immutableBag("url textChanged text title target action invalid".split(" "),
[]),insertCursor:f.constant(Fn)},bi=x.resolve("inline-popup-expanded"),L$={setValue:function(a,b,c){a=a.dom();a.value=b;a.text=c}},wL=function(a,b){a.dom().add(b.dom())},bp={getValue:function(a){var b=a.dom();a=b.selectedIndex;b=b.options[a];return void 0===b||-1===a?h.none():h.from(b.value)},add:wL,addAll:function(a,b){k.each(b,function(b){wL(a,b)})},setSelected:function(a,b){a.dom().selectedIndex=b}},xL=[{value:"",i18nKey:"link.properties.target.none"},{value:"_blank",i18nKey:"link.properties.target.blank"},
{value:"_top",i18nKey:"link.properties.target.top"},{value:"_self",i18nKey:"link.properties.target.self"},{value:"_parent",i18nKey:"link.properties.target.parent"}],uT={target:function(a){var b=hc.common("select","target-container","link.properties.target.label"),c=k.map(xL,function(a){var b=q.fromTag("option");L$.setValue(b,a.value,u.translate(a.i18nKey));return b});bp.addAll(b.field(),c);var d=a.target().getOr("");a=k.findIndex(xL,function(a){return a.value===d}).getOr(0);bp.setSelected(b.field(),
a);return{element:b.element,input:b.field}}},tT=function(){var a=P.div();n.add(a,x.resolve("invalid-link-info"));var b=hc.comboBox("href-url-value","link.properties.url.label",h.some(a));m.setAll(b.input(),{placeholder:u.translate("link.properties.url.placeholder"),autocomplete:"off",autocorrect:"off"});var c=n.toggler(b.element(),x.resolve("link-is-invalid"));return{input:b.input,element:b.element,dropArrow:b.dropArrow,showInvalid:function(){c.on();var d=P.span();ea.add(d,[x.resolve("icon"),x.resolve("invalid-link-icon")]);
v.append(a,d);var e=u.translate("link.properties.url.invalid");m.set(d,"title",e);d=Oa.describe(b.input(),e);m.set(d,"aria-live","assertive");v.append(a,d)},hideInvalid:function(){c.off();B.empty(a);m.remove(b.input(),"aria-describedby")}}},wT=1E4,zT={stringArray:function(a){var b={};k.each(a,function(a){b[a]={}});return W.keys(b)}},ci={url:function(a){return h.some(a.url())},title:function(a){return a.title()},innerText:function(a){a=a.innerText();return 0<a.length?h.some(a):h.none()}},Qb={resolve:lb.create("ephox-tarot").resolve},
yL=function(a,b,c,d,e){var f=q.fromText(a.substr(0,b));b=q.fromText(a.substr(b,c));a=q.fromText(a.substr(d));e=q.fromTag(e);v.append(e,b);return[f,e,a]},zL=function(a,b,c,d){var e=P.tag(b);K.append(e,c);d.each(function(a){n.add(e,a)});return{item:f.constant(a),marked:f.constant(h.some(e))}},AL=function(a,b,c,d){var e=a.text,f=a.marker.start(),l=a.marker.length(),k=0;0<d-(f+l)||0===f||(k=l>d?f-10:f-Math.floor((d-l)/2));d=e.substr(k);f-=k;b=yL(d,f,l,f+l,b);e=d.length===e.length?h.none():h.some(Qb.resolve("anchor-before"));
return zL(a.item(),c,b,e)},du=function(a,b){var c=P.span();n.add(c,b);K.append(c,a);return c},BL=function(a,b,c){var d=[];b=du(b,Qb.resolve("content-innertext"));0<a.length&&(a=du([q.fromText(a)],Qb.resolve("anchor-before")),d.push(a));d.push(b);0<c.length&&(c=du([q.fromText(c)],Qb.resolve("anchor-after")),d.push(c));return d},eu={contentMarker:function(a,b,c,d){var e=a.item().before(),f=a.item().after(),l=a.text;if(0===e.length&&0===f.length&&l.length>d)return AL(a,b,c,d);d=a.marker.start();var k=
a.marker.length();b=yL(l,d,k,d+k,b);e=BL(e,b,f);return zL(a.item(),c,e,h.none())},contentUnderlined:BL,attrMarker:AL},CL=function(a,b,c){a=P.tag(b);v.append(a,q.fromText(c));return a},ox={anchors:function(a,b){var c=b.item(),d=c.url(),e=P.div();n.add(e,Qb.resolve("item-line"));var f=c.title().getOr(d);m.set(e,"title",f);c=b.marked().fold(function(){return CL(a,"div",f)},function(a){return a});n.add(c,Qb.resolve("item-main-label"));d=h.from(d).map(function(b){return b!==f?(b=CL(a,"div",b),n.add(b,
Qb.resolve("item-secondary-label")),[b]):[]}).getOr([]);d=[c].concat(d);K.append(e,d);return e},standard:function(a,b){var c=b.item(),d=P.div();n.add(d,Qb.resolve("item-line"));n.add(d,Qb.resolve("bookmark-icon"));var e=[c.before(),c.innerText(),c.after()].join("");m.set(d,"title",e);e=b.marked().fold(function(){var a=eu.contentUnderlined(c.before(),[q.fromText(c.innerText())],c.after()),b=P.div();K.append(b,a);return b},function(a){return a});n.add(e,Qb.resolve("item-main-label"));c.indent()&&n.add(e,
Qb.resolve("indent"));v.append(d,e);return d},error:function(a,b){var c=P.span(),d=q.fromText(b);v.append(c,d);return c}},M$=C.immutable("matchFunction","start","length"),N$=function(a,b,c){for(var d=0,e=[],f=function(a){e.push(a);d++},h=0,k=a.length;h<k&&d<b;h++)c(a[h]).each(f);return e},fu={unmarked:function(a,b){var c=b.getOr(1E3),c=a.length>c?a.slice(0,c):a;return k.map(c,function(a){return{item:f.constant(a),marked:h.none}})},match:function(a,b,c,d){var e=c.toLowerCase();c=d.getOr(1E3);return N$(a,
c,function(a){return ab.findMap(b,function(b){return b(a).bind(function(c){var d=c.toLowerCase().indexOf(e);if(-1<d){var k=e.length;return h.some({item:f.constant(a),marked:h.none,text:c,marker:M$(b,d,k)})}return h.none()})})})},mark:function(a,b,c,d){return k.map(a,function(a){return(a.marker.matchFunction()===ci.innerText?eu.contentMarker:eu.attrMarker)(a,b,c,d)})}},DL="data"+Qb.resolve("section-id"),gu={create:function(a,b){var c=q.fromTag("ol");m.set(c,DL,b.id());var d=function(){B.empty(c)};
return{element:f.constant(c),clear:d,add:function(a){var b=q.fromTag("li");v.append(b,a);v.append(c,b);return b},spec:f.constant(b),destroy:function(){d()}}},dataIdAttribute:f.constant(DL)},O$=C.immutableBag(["container","sections","translate"],["doc"]),P$=C.immutableBag("id highlight contentLimit items render extractors".split(" "),["itemLimit"]),hu="data"+Qb.resolve("item-id"),Ik={init:function(a,b){var c=b.doc().getOr(q.fromDom(document)),d=b.translate(),e={},g=Sh(),l=P.div();n.add(l,Qb.resolve("predictive-menu"));
Q.hidden(l,!0);var p=xT(a,l),r=function(){k.each(g.state(),function(a){t(a,[])})},t=function(a,b){B.remove(a.element());a.clear();var d={};0<b.length&&(k.each(b,function(b){var e=a.spec().render()(c,b),e=a.add(e),g=La.generate("");m.set(e,hu,g);d[g]={gui:f.constant(e),item:f.constant(b.item())}}),v.append(l,a.element()));e[a.spec().id()]=d},w=function(){return y.parent(l).isSome()},u=function(a){n.add(a,Qb.resolve("selected"));H().bind(N).each(function(a){Oa.speak(ca.body(),a.item().url())})},x=function(a){u(a);
Ea.intoViewIfNeeded(a,l)},A=function(a){n.remove(a,Qb.resolve("selected"))},C=function(){J.descendant(l,"li").each(x)},F=function(){ra.one("ol:last-child li:last-child",l).each(x)},H=function(){return J.descendant(l,"li."+Qb.resolve("selected"))},K=function(){return H().bind(N).map(function(a){a=a.item();Oa.speak(ca.body(),d("link.dialog.autocomplete.accept",a.url()));return a})},L=function(a){0===y.children(a).length?y.nextSibling(a).fold(C,L):y.firstChild(a).fold(F,x)},M=function(a){0===y.children(a).length?
y.prevSibling(a).fold(F,M):y.lastChild(a).fold(C,x)},N=function(a){a=J.closest(a,"["+hu+"]");var b=a.bind(function(a){return J.closest(a,"["+gu.dataIdAttribute()+"]")}),c=a.map(function(a){return m.get(a,hu)});return b.map(function(a){return m.get(a,gu.dataIdAttribute())}).bind(function(a){return h.from(e[a])}).bind(function(a){return c.bind(function(b){return h.from(a[b])})})},R=E.capture(l,"mouseover",function(a){a=a.target();eb.closest(a,"li."+Qb.resolve("selected"))||N(a).fold(f.noop,function(a){H().each(function(a){A(a)});
u(a.gui())})}),S=E.capture(l,"click",function(a){K().each(T.trigger.click)}),T=D.create({click:s(["item"])}),U=Gb.ensure(["url","indent"]);k.each(b.sections(),function(a){k.each(a.items(),U);a=gu.create(c,a);g.push(a)});r();return{element:f.constant(l),events:T.registry,reset:r,resetFull:function(){k.each(g.state(),function(a){a.clear();var b=a.spec().items(),c=a.spec().itemLimit(),b=fu.unmarked(b,c);t(a,b)})},filter:function(a){0<a.length?(e={},k.each(g.state(),function(b){var c=b.spec(),d=fu.match(c.items(),
c.extractors(),a,c.itemLimit()),c=fu.mark(d,c.highlight(),"div",c.contentLimit());t(b,c)})):r()},show:function(a){var c=b.container();v.append(c,l);p.position(a,mg.relativeOrigin(c));Oa.speak(ca.body(),d("link.dialog.autocomplete.open"))},isShowing:w,hide:function(){w()&&Oa.speak(ca.body(),d("link.dialog.autocomplete.close"));B.remove(l);H().each(A)},getSelectedData:K,moveup:function(){H().fold(F,function(a){A(a);y.prevSibling(a).fold(function(){y.parent(a).bind(y.prevSibling).fold(F,M)},x)})},movedown:function(){H().fold(C,
function(a){A(a);y.nextSibling(a).fold(function(){y.parent(a).bind(y.nextSibling).fold(C,L)},x)})},destroy:function(){R.unbind();S.unbind();k.each(g,function(a){a.destroy()});B.remove(l)}}},config:O$,section:P$},CT=Y.detect().browser.isSafari(),$o=50,BT=1E3,ap=C.immutable("url","indent","action","before","innerText","after"),AT=C.immutable("url","indent","title","action"),px=C.immutable("url","action"),nx=C.immutable("url","indent","action"),iu=ma.generate([{invalid:["invalidUrl"]},{unknown:["unknownUrl"]},
{valid:["validUrl"]}]),tx={cata:function(a,b,c,d){return a.fold(b,c,d)},derive:function(a,b){return"VALID"===b?iu.valid(a):"INVALID"===b?iu.invalid(a):iu.unknown(a)}},EL=[xc.strict("url"),xc.defaulted("fresh",!1)],Q$=wc.objOf(EL),R$=wc.objOf([xc.field("urls","urls",Zb.strict(),wc.arrOfObj(EL))]),sx={one:f.constant(Q$),many:f.constant(R$)},S$={recompose:function(a){var b="";""!==a.protocol&&(b+=a.protocol,b+=":");""!==a.authority&&(b=b+"//"+a.authority);b+=a.path;""!==a.query&&(b+="?",b+=a.query);
""!==a.anchor&&(b+="#",b+=a.anchor);return b}},Gn={remove:function(a){for(var b="";""!==a;)if(aa.startsWith(a,"../"))a=aa.removeLeading(a,"../");else if(aa.startsWith(a,"./"))a=aa.removeLeading(a,"./");else if(aa.startsWith(a,"/./"))a="/"+aa.removeLeading(a,"/./");else if("/."===a)a="/";else if(aa.startsWith(a,"/../"))a="/"+aa.removeLeading(a,"/../"),b=aa.removeTrailing(b,b.substring(b.lastIndexOf("/")));else if("/.."===a)a="/",b=aa.removeTrailing(b,b.substring(b.lastIndexOf("/")));else if("."===
a||".."===a)a="";else{var c=a.match(/(^\/?.*?)(\/|$)/)[1];a=aa.removeLeading(a,c);b+=c}return b}},T$={merge:function(a,b,c){if(""!==c&&""===a)return"/"+b;c=a.substring(a.lastIndexOf("/")+1);return aa.removeTrailing(a,c)+b}},U$={transform:function(a,b){var c={strictMode:!0},d=tg.parse(a,c),c=tg.parse(b,c),e={};""!==c.protocol?(e.protocol=c.protocol,e.authority=c.authority,e.path=Gn.remove(c.path),e.query=c.query):(""!==c.authority?(e.authority=c.authority,e.path=Gn.remove(c.path),e.query=c.query):
(""===c.path?(e.path=d.path,e.query=""!==c.query?c.query:d.query):(aa.startsWith(c.path,"/")?e.path=Gn.remove(c.path):(e.path=T$.merge(d.path,c.path,a.authority),e.path=Gn.remove(e.path)),e.query=c.query),e.authority=d.authority),e.protocol=d.protocol);e.anchor=c.anchor;return e}},ux={resolve:function(a,b){var c=U$.transform(a,b);return S$.recompose(c)}},KT=C.immutable("href","resolved"),MT={generate:function(a){var b={};k.each(a,function(a){var d=a.resolved();a=a.href();b.hasOwnProperty(d)?k.contains(b[d],
a)||b[d].push(a):b[d]=[a]});return b}},V$={rules:[{current:na.backtrack,next:na.sidestep,fallback:h.none()},{current:na.sidestep,next:na.sidestep,fallback:h.some(na.backtrack)},{current:na.advance,next:na.sidestep,fallback:h.some(na.sidestep)}],inclusion:function(a,b,c){return!a.property().parent(c).exists(function(c){return a.eq(c,b.item())})}},W$=na.walkers(),FL=function(a,b,c,d){return na.walk(a,b,c,W$.left(),d.rules).map(function(c){return a.property().isEmptyTag(c.item())||a.property().isBoundary(c.item())?
[]:(d.inclusion(a,c,b)?[c.item()]:[]).concat(FL(a,c.item(),c.mode(),d))}).getOr([])},GL=function(a,b,c){a=FL(b,c,na.sidestep,a);return k.reverse(a).concat([c])},HL={top:f.curry(GL,V$),all:f.curry(GL,{rules:void 0,inclusion:function(a,b,c){return a.property().isText(b.item())}})},IL=Sa(),JL={top:function(a){return HL.top(IL,a)},all:function(a){return HL.all(IL,a)}},X$=function(a,b){var c=ad.generate(b,function(b,c){return a.property().isText(b)?h.some(S.range(b,c,c+a.property().getText(b).length)):
h.none()}),d=k.foldr(c,function(b,c){return a.property().getText(c.element())+b},"");return{list:f.constant(c),text:f.constant(d)}},Y$=function(a,b,c,d){var e=ad.find(b.list(),function(b){return a.eq(b.element(),c)}).map(function(a){return a.start()+d});return{list:b.list,text:b.text,cursor:f.constant(e)}},Z$={from:function(a,b,c,d){b=X$(a,b);return Y$(a,b,c,d)}},$$=Sa(),aaa={from:function(a,b,c){return Z$.from($$,a,b,c)}},TT={get:function(a,b){var c=JL.all(a),d=aaa.from(c,a,b),c=d.cursor().orThunk(function(){return A.isText(a)?
h.none():h.some(d.text().length)});return{list:d.list,text:d.text,cursor:f.constant(c)}}},KL=function(a,b){return{patterns:function(){return[RegExp("^("+a+")(.+)","gi")]},useCursor:f.never,generate:function(a,d){var e=q.fromTag(b,a.dom());K.append(e,d.clones());return hd.species({elements:f.constant([e]),cursor:h.none,dictation:h.none,notify:h.none})},derive:function(a){var b=a[1];return h.some({startIndex:f.constant(a.index),finishIndex:f.constant(a.index+b.length)})}}},ug={normal:function(a,b){var c=
db.sanitise(a);return KL(c,b)},spaced:function(a,b){var c=db.sanitise(a)+"\\s";return KL(c,b)}},LL=function(a,b){return hd.species({elements:f.constant([q.fromText(b.text())]),cursor:h.none,dictation:h.none,notify:h.none})},baa=function(){return{patterns:function(){return[/([^.?!][.?!]\s+)([a-z][^\s]*)/g,/^()([a-z][^\s]*)/g]},useCursor:f.constant(!0),generate:LL,derive:function(a){var b=a.index,c=b+a[0].length;a=a[1]+aa.capitalize(a[2]);return h.some({startIndex:f.constant(b),finishIndex:f.constant(c),
text:f.constant(a)})}}}(),caa=function(){return{patterns:function(){return[RegExp("([A-Z][A-Z]+)([a-z]+)","g")]},useCursor:f.constant(!1),generate:LL,derive:function(a){var b=a[2],b=aa.capitalize(a[1].toLowerCase())+b.toLowerCase(),c=a.index;a=c+a[0].length;return h.some({startIndex:f.constant(c),finishIndex:f.constant(a),text:f.constant(b)})}}}(),vx={capitalise:f.constant(baa),normalise:f.constant(caa)},RT={patterns:function(){return[Dl.autolink()]},useCursor:f.constant(!0),generate:function(a,b){var c=
q.fromTag("a");m.set(c,"href",b.href());v.append(c,q.fromText(b.text()));return hd.species({elements:f.constant([c]),cursor:h.none,dictation:h.none,notify:f.curry(h.some,[c])})},derive:function(a){var b=au.preprocess(a[1]),c=b.text(),b=b.href();return-1<c.indexOf("@")?h.none():h.some({startIndex:f.constant(a.index),finishIndex:f.constant(a.index+c.length),text:f.constant(c),href:f.constant(b)})}},QT=f.constant(function(a,b){return b}),daa={cloneAfter:function(a,b,c){var d=b;return k.map(a,function(a){var b=
ec.deep(a);A.isText(a)&&L.eq(c.endContainer(),a)&&Da.get(a).length>c.endOffset()&&Da.set(b,Da.get(b).substring(0,c.endOffset()));if(A.isText(b)&&0<d){var f=Da.get(b);a=Math.min(f.length,d);f=f.substring(a);Da.set(b,f);d-=a}return b})}},ML={matcher:function(a,b,c){var d=function(a){return c.fold(function(){return a&&1<a.length},function(b){return a&&0<a.length&&a.index+a[0].length===b})};return ab.findMap(a,function(a){a:{if(!a.global)throw"Macro matching only works with global regexes: "+a;for(var c=
a.exec(b);c;){if(d(c)){a=h.some(c);break a}c=a.exec(b)}a=h.none()}return a})}},eaa=function(a,b,c,d){b=daa.cloneAfter(b,d.finishIndex(),c);var e=Z.merge(d,{clones:f.constant(b)});return{build:function(b){b=a.generate(b,e);return hd.applicator({elements:b.elements,cursor:b.cursor,range:f.constant(c),dictation:b.dictation,notify:b.notify})}}},faa=function(a,b,c){b=ad.get(a,b);var d=ad.get(a,c);return b.bind(function(a){return d.map(function(b){return{start:f.constant(a),finish:f.constant(b)}})})},gaa=
function(a,b){return b(a.start().element())&&b(a.finish().element())?h.some(a):h.none()},haa=function(a,b,c){return b.doc().bind(function(d){d=a.build(d);var e=d.elements();k.map(e,Ee.doNotMerge);var f=qw.handle(d.range());Cc.insertAt(b,f,e);k.map(e,Ee.clear);d.dictation().each(function(a){Oa.shout(ca.body(),a())});d.notify().each(function(){c.input(e)});return d.cursor()})},NL=function(a,b,c,d,e){return a.detect(b,c,d,e).map(function(a){return{execute:f.curry(haa,a)}})},fp={block:f.curry(NL,{detect:function(a,
b,c,d){var e=b.patterns(),f=a.cursor().filter(b.useCursor);return ML.matcher(e,a.text(),f).bind(b.derive).bind(function(a){var e=JL.top(c);if(0<e.length){var f=ur(e[0],0,c,d);return h.some(eaa(b,e,f,a))}return h.none()})}}),inline:f.curry(NL,{detect:function(a,b,c,d){c=b.patterns();d=a.cursor().filter(b.useCursor);return ML.matcher(c,a.text(),d).bind(b.derive).bind(function(c){return faa(a.list(),c.startIndex(),c.finishIndex()).bind(function(a){return gaa(a,b.allowed).map(function(a){var d=c.startIndex()-
a.start().start(),g=c.finishIndex()-a.finish().start(),h=ur(a.start().element(),d,a.finish().element(),g);return{build:function(a){a=b.generate(a,c);return hd.applicator({elements:a.elements,cursor:a.cursor,range:f.constant(h),dictation:a.dictation,notify:a.notify})}}})})})}})},WT=Y.detect().deviceType.isTouch(),wx=function(a){a.disable();ha.disable(a.element())},ZT=Y.detect().deviceType.isiOS(),bU=500,iaa=C.immutable("rawData","extra"),xx=function(a,b,c){if(R.isArray(b)&&R.isArray(c))return H.value(iaa(b,
c));console.error(a+" callback parameters were not both arrays",b,c);return H.error(a+" callback parameters were not both arrays")},gp=function(a,b){if(mi.isPrototypeOf(b))return H.value(q.fromDom(b));console.error(a+" was not a DOM element",b);return H.error(a+" was not a DOM element")},nU={configure:function(a){var b=[a.internal(),a.history(),a.output()];k.each(b,function(b){a.addSelector(b,"span.ephox-polish-incomplete-mention",Ca.unwrap)});a.addExcludedContext(["span.ephox-polish-incomplete-mention"])}},
zx=x.resolve("no-results"),yx=x.resolve("loading-results"),hp=ma.generate([{none:["item"]},{extra:["value","item"]},{fetched:["userId","name","rawname","info","item"]}]),ip=ma.generate([{nothing:[]},{refetch:["term"]},{open:["term"]}]),OL=function(){return RegExp(db.wordbreak(),"i")},jaa=function(a,b,c){return Xf.repeatLeft(a,c,function(a,b,c,f,h){return Xf.previousChar(f,h).fold(function(){return b.kontinue()},function(a){return b.finish(a.ch())})}).fold(h.none,h.none,h.some)},kaa=function(a,b,c,
d){return 0<=d&&"@"===c.charAt(d)?a.finish(S.point(b,d)):a.abort()},laa=function(a,b,c,d){return a.finish(S.point(b,d))},maa=function(a){return h.some(S.point(a,ka.getEnd(a)))},Bx={isValidMention:function(a,b){return jaa(a,na.sidestep,b).fold(f.constant(!0),function(a){return null===a.match(/[0-9a-zA-Z]/)})},scan:function(a,b){return Xf.expandLeft(a,b,{regex:OL,attempt:kaa}).fold(h.none,h.none,h.some).bind(function(c){return Xf.expandRight(a,b,{regex:OL,attempt:laa}).fold(h.none,maa,h.some).map(function(a){return{start:c.element,
soffset:c.offset,finish:a.element,foffset:a.offset}})})}},rh=ma.generate([{none:[]},{bare:["text","start","soffset","finish","foffset"]},{incomplete:["text","element"]},{complete:["text","element"]}]),PL=x.resolve("incomplete-mention"),naa=function(a,b){return Bx.scan(a,b).map(function(a){return rh.bare("",a.start(),a.soffset(),a.finish(),a.foffset())})},QL=function(a){if(A.isText(a))return Da.get(a);a=pc.descendants(a,A.isText);return k.map(a,Da.get).join("")},RL=function(a,b){return a.collapsed()?
J.closest(a.startContainer(),b.hoverSelector()).fold(function(){return J.closest(a.startContainer(),"."+PL).fold(function(){return naa(a.startContainer(),a.startOffset())},function(a){return h.some(rh.incomplete(QL(a),a))})},function(a){return h.some(rh.complete(QL(a),a))}).getOrThunk(rh.none):rh.none()},vg={detect:function(a,b){return a.getSelection().map(function(a){return RL(a,b)}).getOrThunk(rh.none)},detectOn:RL,withinIncomplete:function(a){return J.closest(a,"."+PL)}},SL=function(a){n.remove(a,
x.resolve("incomplete-mention"));B.unwrap(a)},kp={cancel:SL,create:function(){var a=q.fromTag("span");n.add(a,x.resolve("incomplete-mention"));var b=q.fromText("@");v.append(a,b);return a},validatePrevious:function(a,b){vg.withinIncomplete(a).filter(function(a){return vg.withinIncomplete(b).forall(function(b){return!L.eq(a,b)})}).each(SL)}},jp="."+x.resolve("mentions-business-card"),jU=function(a,b){return 1<arguments.length||/\{(\d+)\}/.test(a)?Dl.tokens.call(void 0,a,b):a},mU=1E6,pU=C.immutable("textkey",
"value","icon"),oaa=function(){return{configure:function(a){var b=[a.internal(),a.history(),a.output(),a.paste()];k.each(b,function(b){a.addSelector(b,"[unselectable]",Ca.removeAttr("unselectable"))})},clean:function(a){m.remove(a,"unselectable")},disableNativeResize:function(a){m.set(a,"unselectable","on")}}},paa=function(){return{configure:f.noop,clean:function(a){y.nextSibling(a).filter(De.isEmptyText).each(B.remove)},disableNativeResize:f.noop}},TL=Y.detect().browser,lp=TL.isIE()?oaa():TL.isFirefox()?
paa():{configure:f.noop,clean:f.noop,disableNativeResize:f.noop},UL="data-"+x.resolve("resize-direction"),Hn=function(a,b){var c=q.fromTag("span");ea.add(c,[x.resolve(a),x.resolve("resize-eph-resizer")]);m.set(c,UL,b);return c},xU={create:function(){var a=Hn("element-se-resize","1"),b=Hn("element-ne-resize","1"),c=Hn("element-sw-resize","-1"),d=Hn("element-nw-resize","-1"),e=q.fromTag("div");n.add(e,x.resolve("resize-wrapper"));K.append(e,[a,b,c,d]);var g=function(){B.remove(e)},h=D.create({click:s(["multiplier"])}),
k=E.bind(e,"mousedown",function(a){n.has(a.target(),x.resolve("resize-eph-resizer"))&&(a.prevent(),a=m.get(a.target(),UL),a=void 0!==a?parseInt(a,10):1,h.trigger.click(a))});return{refresh:function(g,h){v.append(g,e);var l;l=Na.absolute(h);var k=ia.get(h),m=qa.get(h);l={top:l.top,height:f.constant(k),left:l.left,width:f.constant(m)};k=qa.get(c)/2;t.setAll(c,{top:l.top()+l.height()-k+"px",left:l.left()-k+"px"});t.setAll(a,{top:l.top()+l.height()-k+"px",left:l.width()+l.left()-k+"px"});t.setAll(b,{top:l.top()-
k+"px",left:l.width()+l.left()-k+"px"});t.setAll(d,{top:l.top()-k+"px",left:l.left()-k+"px"})},clear:g,events:h.registry,destroy:function(){k.unbind();g()}}}},qaa=C.immutable("minRow","minCol","maxRow","maxCol"),raa=function(a,b){var c=a.grid().columns(),d=a.grid().rows(),e=c,f=0,h=0;W.each(a.access(),function(a){if(b(a)){var c=a.row(),k=c+a.rowspan()-1,m=a.column();a=m+a.colspan()-1;c<d?d=c:k>f&&(f=k);m<e?e=m:a>h&&(h=a)}});return qaa(d,e,f,h)},saa=function(a,b,c,d){for(var e=b.grid().columns(),f=
b.grid().rows(),h=0;h<f;h++)for(var k=!1,m=0;m<e;m++)if(!(h<c.minRow()||h>c.maxRow()||m<c.minCol()||m>c.maxCol()))if(gb.getAt(b,h,m).filter(d).isNone()){var n=k,s=a[h].element(),t=q.fromTag("td");v.append(t,q.fromTag("br"));(n?v.append:v.prepend)(s,t)}else k=!0},taa=function(a,b){var c=k.filter(Bi.firstLayer(a,"tr"),function(a){return 0===a.dom().childElementCount});k.each(c,B.remove);b.minCol()!==b.maxCol()&&b.minRow()!==b.maxRow()||k.each(Bi.firstLayer(a,"th,td"),function(a){m.remove(a,"rowspan");
m.remove(a,"colspan")});m.remove(a,"width");m.remove(a,"height");t.remove(a,"width");t.remove(a,"height")},uaa={extract:function(a,b){var c=function(a){return ra.is(a.element(),b)},d=We.fromTable(a),e=gb.generate(d),f=raa(e,c),h="th:not("+b+"),td:not("+b+")",m=Bi.filterFirstLayer(a,"th,td",function(a){return ra.is(a,h)});k.each(m,B.remove);saa(d,e,f,c);taa(a,f);return a}},CU=function(a){k.each(a,function(a){B.empty(a);v.append(a,q.fromTag("br"))})},vaa=function(a){return Ya.table(a[0]).map(ec.deep).map(function(a){return[uaa.extract(a,
sd.selectedSelector())]})},AU=function(a,b){var c=bt.register({doc:a.doc,win:a.win,body:a.body,getSelection:a.getSelection,setSelection:a.setSelection});return function(b,e,f,h){e=fi.buildClipboardContents(e,f);h();var k=Mk.fromSafe(a);k.begin();c.setClipboard(b,e,function(){k.end()})}},BU=function(a,b){return vaa(a).map(function(a){k.each(a,b);var d=f.curry(ml.toText,a);a=f.curry(ml.toHtml,a);return{text:d,html:a}})},DU=function(a,b,c){return jg.fragment(b.win(),a).map(function(a){var b=f.curry(jg.text,
a);a=f.curry(jg.html,a,c);return{text:b,html:a}})},Ec={resolve:lb.create("ephox-cement").resolve},waa=Ec.resolve("smartpaste-eph-bin"),Qx={binStyle:f.constant(waa)},VL=function(a,b){return tb.cata(b.get(),f.constant([]),f.identity,f.constant([a]))},ju={mergable:function(a,b){return tb.cata(b.get(),h.none,function(b,d){return 0===b.length?h.none():Jw.retrieveBox(a,sd.firstSelectedSelector(),sd.lastSelectedSelector()).bind(function(a){return 1<b.length?h.some({bounds:f.constant(a),cells:f.constant(b)}):
h.none()})},h.none)},unmergable:function(a,b){var c=function(a){return m.has(a,"rowspan")&&1<parseInt(m.get(a,"rowspan"),10)||m.has(a,"colspan")&&1<parseInt(m.get(a,"colspan"),10)},d=VL(a,b);return 0<d.length&&k.forall(d,c)?h.some(d):h.none()},selection:VL},WL=function(a){return{element:f.constant(a),mergable:h.none,unmergable:h.none,selection:f.constant([])}},xaa=C.immutable("element","clipboard","generators"),Rk={noMenu:WL,forMenu:function(a,b,c){return{element:f.constant(c),mergable:f.constant(ju.mergable(b,
a)),unmergable:f.constant(ju.unmergable(c,a)),selection:f.constant(ju.selection(c,a))}},notCell:function(a){return WL(a)},paste:xaa},yaa=function(a){return"table"===A.name(a)||0===pb.get(a).trim().length},XL=function(a){var b=k.filter(a,function(a){return"table"===A.name(a)});return 1!==b.length?h.none():k.forall(a,yaa)?h.from(b[0]):h.none()},nV={rules:function(a,b,c){var d=f.noop,e=function(e,g){return Ya.table(e,a.isRoot).bind(function(h){var k=a.doc().getOrDie(),k=YB.paste(k),k=Rk.paste(e,g,k),
m=f.constant(c);return Nb.execute(b,Fb.pasteCells,d,m)(a,h,k).map(function(a){return a.startContainer()})})},g=function(a){return XL(a).map(function(a){return{single:f.constant(a)}})};return{collapsed:f.constant(Qa(function(a,b,c){return Ya.cell(b,a)},g,function(a,b){return e(a.element(),b.single())},"custom arcade table insert collapsed")),ranged:f.constant(Qa(function(a,b,c){return Ya.cell(b.element(),a).map(function(a){return{left:f.constant(a),right:f.constant(a)}})},g,function(a,b){return e(a.left(),
b.single())},"custom arcade table insert ranged"))}},extractTable:XL},YL="officeStyles htmlStyles isWord proxyBin isInternal backgroundAssets".split(" "),ZL=C.immutableBag([],YL),rb={nu:ZL,merge:function(a,b){var c={};k.each(YL,function(d){b[d]().or(a[d]()).each(function(a){c[d]=a})});return ZL(c)}},xj=ma.generate([{error:["message"]},{paste:["elements","correlated"]},{cancel:[]},{incomplete:["elements","correlated","message"]}]),ku=function(a,b,c,d,e){return a.fold(b,c,d,e)},Ka={error:xj.error,paste:xj.paste,
cancel:xj.cancel,incomplete:xj.incomplete,cata:ku,carry:function(a,b){return ku(a,h.none,h.none,h.none,function(a,d,e){return ku(b,h.none,function(a,b){return h.some(xj.incomplete(a,b,e))},h.none,h.none)}).getOr(b)}},lu=C.immutableBag(["response","bundle"],[]),yj=function(a){return mu(function(b){var c=lu(a);b(c)})},mu=function(a){var b=function(b){a(b)},c=mu;return{get:b,map:function(a){return c(function(c){b(function(b){b=a(b);c(b)})})},bind:function(a){return c(function(c){b(function(b){a(b).get(c)})})}}},
va={call:function(a,b){a(lu(b))},sync:mu,pure:yj,pass:function(a){return yj({response:a.response(),bundle:a.bundle()})},done:lu,error:function(a){return yj({response:Ka.error(a),bundle:rb.nu({})})},initial:function(){return yj({response:Ka.paste([],[]),bundle:rb.nu({})})},cancel:function(){return yj({response:Ka.cancel(),bundle:rb.nu({})})}},$L=C.immutable("steps","input","label","capture"),zaa=function(a,b){return ab.findMap(a,function(a){return a.getAvailable(b).map(function(b){return $L(a.steps(),
b,a.label(),a.capture())})})},Aaa=function(a,b){var c=f.curry(va.pass,a),d=function(){return b().map(function(b){var c=rb.merge(a.bundle(),b.bundle());b=Ka.carry(a.response(),b.response());return va.done({response:b,bundle:c})})};return Ka.cata(a.response(),c,d,c,d)},Ex={choose:function(a,b,c){return zaa(a,c).getOrThunk(function(){var a=b.getAvailable(c);return $L(b.steps(),a,b.label(),b.capture())})},run:function(a,b){return k.foldl(a,function(a,d){return a.bind(function(a){return Aaa(a,function(){return d(b,
a)})})},va.initial())}},zj=La.generate("simple-adt"),aM={generate:function(a){var b=function(a){return 0===a.indexOf(zj)?a.substring(zj.length+1):a},c=W.keys(a),d={};k.each(c,function(a){d["as"+aa.capitalize(a)]=function(b){return b._simpleAdt===zj+"_"+a?h.some(b.data):h.none()}});var e={};W.each(a,function(a,b){e[b]=function(c){var d=zj+"_"+b,e;e=0===a.length?{}:C.immutableBag(a,[])(c);return{_simpleAdt:d,data:e,_simpleAdt_data:c}}});return Z.deepMerge(d,{constructors:e,match:function(a,d){var e=
W.keys(d);if(e.length!==c.length)throw Error("Partial match");return ab.findMap(e,function(b){return a._simpleAdt===zj+"_"+b?h.some(d[b]):h.none()}).getOrDie("Must find branch for constructor: "+b(a._simpleAdt))(a.data)}})}},ic=aM.generate({event:["nativeEvent"],html:["container"],word:["html","rtf"],text:["text"],images:["images"]}),In=aM.generate({getFromFlash:[],disabled:[],fromClipboard:["rtf"]}),Ok={getFromFlash:In.constructors.getFromFlash,disabled:In.constructors.disabled,fromClipboard:In.constructors.fromClipboard,
match:In.match},bM=function(a,b){var c=RegExp(b,"i");return ab.findMap(a,function(a){return null!==a.match(c)?h.some({type:a,flavor:b}):h.none()})},nu={isValidData:function(a){return void 0!==a&&void 0!==a.types&&null!==a.types},getPreferredFlavor:function(a,b){return ab.findMap(a,function(a){return bM(b,a)})},getFlavor:bM},ou=Y.detect().browser,Baa=!(ou.isIE()||ou.isEdge()&&16>ou.version.major),Caa=["^image/","file"],Daa=function(a){return aa.contains(a,"<html")&&(aa.contains(a,'xmlns:o="urn:schemas-microsoft-com:office:office"')||
aa.contains(a,'xmlns:x="urn:schemas-microsoft-com:office:excel"'))},Eaa=function(a){return 0<a.length},pu=function(a,b){return nu.getFlavor(a.types,b).map(function(b){return a.getData(b.type)}).filter(Eaa)},Aj=function(a){return Baa?h.some(a.clipboardData).filter(nu.isValidData):h.none()},GU={fromEvent:function(a){var b=function(a){return void 0===a.items?h.none():nu.getPreferredFlavor(Caa,a.types).map(function(b){return ic.constructors.images({images:a.items})})},c=function(a){return ab.findMap(a.types,
function(b){return"text/plain"===b?h.some(a.getData(b)).map(function(a){return ic.constructors.text({text:a})}):h.none()})};return{getWordData:function(){return Aj(a).bind(function(a){return pu(a,"html").filter(Daa).map(function(b){var c=pu(a,"rtf");return ic.constructors.word({html:b,rtf:c.fold(Ok.getFromFlash,function(a){return Ok.fromClipboard({rtf:a})})})})})},getImage:function(){return Aj(a).bind(b)},getText:function(){return Aj(a).fold(function(){var a=window.clipboardData;return void 0!==a?
h.some(ic.constructors.text({text:a.getData("text")})):h.none()},c)},getHtml:function(){return Aj(a).bind(function(a){return pu(a,"html").map(function(a){var b=q.fromTag("div");ua.set(b,a);return ic.constructors.html({container:b})})})},getOnlyText:function(){return Aj(a).bind(function(a){var b=a.types;return 1===b.length&&"text/plain"===b[0]?c(a):h.none()})},getNative:function(){return ic.constructors.event({nativeEvent:a})}}},fromHtml:function(a){return{getWordData:function(){return h.some(ic.constructors.word({html:a,
rtf:Ok.disabled()}))},getImage:h.none,getText:h.none,getNative:f.die("There is no native event"),getOnlyText:h.none}},fromText:function(a){return{getWordData:h.none,getImage:h.none,getText:function(){return h.some(ic.constructors.text({text:a}))},getNative:f.die("There is no native event"),getOnlyText:h.none}}},cM=Y.detect(),Faa=function(a,b,c){b.control.block();a.dom().execCommand("paste");c();b.control.unblock()},Gaa=function(a,b,c){setTimeout(c,1)},dM=cM.browser.isIE()&&10>=cM.browser.version.major,
Haa=dM?Faa:Gaa,Gx={willBlock:f.constant(dM),run:function(a,b,c){return Haa(a,b,c)}},Fx=function(a){return k.map(a,function(a){return a.asset()})},Bj=C.immutable("asset","image"),eM=function(a,b){return oc.cata(a,function(a,d,e,f){m.set(b,"src",e);return!0},f.never)},je={assetImage:Bj,createImages:function(a){var b=[],c=[],d=[];k.each(a,function(a){return oc.cata(a,function(f,h,k,n){f=q.fromTag("img");m.set(f,"src",k);b.push(f);c.push(a);d.push(Bj(a,f))},function(a,b,c){console.error("Internal error: Paste operation produced an image URL instead of a Data URI: ",
b)})});return Ka.paste(b,d)},findImages:function(a,b){var c=[],d=k.bind(a,function(a){return"img"===A.name(a)?[a]:X.descendants(a,"img")});k.each(b,function(a){oc.cata(a,function(b,f,h,n){k.each(d,function(b){m.get(b,"src")===h&&c.push(Bj(a,b))})},f.noop)});return c},updateSource:eM,updateSources:function(a,b){var c=[];k.each(a,function(a,e){var f=b[e];eM(a,f)&&c.push(Bj(a,f))});return c},browserBlobs:function(a){return Nc.mapM(a,function(a){return za.nu(function(c){var d=a.dom();tc.imageToImageResult(d).then(function(e){e.toBlob().then(function(f){f=
aa.startsWith(d.src,"blob:")?d.src:URL.createObjectURL(f);var h=La.generate("image");f=oc.blob(h,e,f);c(Bj(f,a))})})})})}},Iaa=function(a){return za.nu(function(b){var c=tc.imageResultToBlob(a,"image/png").then(function(a){we.single(a).map(h.some).get(b)});return c["catch"].call(c,function(a){console.warn(a);b(h.none())})})},HU=function(a){var b=function(){return za.pure(a)};return oc.cata(a.asset(),function(c,d,e){return/tiff$/.test(d.getType())?Iaa(d).map(function(b){return b.map(function(b){var c=
a.image();Re.revokeObjectURL(e);je.updateSource(b,c);return je.assetImage(b,c)}).getOr(a)}):b()},b)},mb={blocking:function(a){return function(b){return function(c,d){b.block();return a(c,d).map(function(a){b.unblock();return a})}}},normal:function(a){return f.constant(a)}},Jaa={convert:function(a){var b=ne.resolve("window.clipboardData.files"),c=void 0!==a.convertURL?a.convertURL:void 0!==a.msConvertURL?a.msConvertURL:void 0;return void 0!==b&&void 0!==c&&0<b.length?(b=Nc.mapM(b,function(b){var e=
Re.createObjectURL(b);c.apply(a,[b,"specified",e]);return we.singleWithUrl(b,e)}),h.some(b)):h.none()}},Kaa=function(){var a=h.none();return{convert:function(b){a=Jaa.convert(b)},listen:function(b){return a.fold(function(){return za.nu(function(a){a([])})},function(a){return a}).get(b)},clear:function(){a=h.none()}}},Laa=function(a,b){return $h.dataUriToBlobSync(b).map(function(b){return je.assetImage(we.single(b),a)})},Maa=function(a,b){return $h.uriToBlob(b).map(function(b){var d=za.nu(function(a){b.then(function(b){we.single(b).get(a)})});
return je.assetImage(d,a)})},Ix=function(a){var b=q.fromTag("div");K.append(b,a);return X.descendants(b,"img[src]")},fM=function(a){return 0===a.indexOf("data:")&&-1<a.indexOf("base64")},Jx=function(a){a=m.get(a,"src");return fM(a)||0===a.indexOf("blob:")},IU=function(a){return k.bind(Ix(a),function(a){var c=m.get(a,"src");return fM(c)?Laa(a,c).toArray():0===c.indexOf("blob:")?Maa(a,c).toArray():[]})},JU=function(a){a=k.filter(a,function(a){return"img"!==A.name(a)||!Jx(a)});return Ka.incomplete(a,
[],"errors.local.images.disallowed")},Naa={convert:function(a,b){if(0===a.length)throw"Zero length content passed to Hex conversion";for(var c=Array(a.length/2),d=0;d<a.length;d+=2){var e=a.substr(d,2);c[Math.floor(d/2)]=parseInt(e,16)}c=Qv(c);return Pv([c],{type:b})}},gM=ma.generate([{unsupported:["id","message"]},{supported:["id","contentType","blob"]}]),Cj={unsupported:gM.unsupported,supported:gM.supported,cata:function(a,b,c){return a.fold(b,c)}},hM=function(a,b,c,d,e){return-1===a||-1===b?h.none():
h.some({start:f.constant(a),end:f.constant(b),bower:c,regex:f.constant(d),idRef:f.constant(e)})},iM=function(a,b,c){return function(){return a.substring(b,c)}},qu=function(a,b){if(-1===b)return b;var c=0,d=a.length,e,f;do if(e=a.indexOf("{",b),f=a.indexOf("}",b),f>e&&-1!==e?(b=e+1,++c):(e>f||0>e)&&-1!==f&&(b=f+1,--c),b>d||-1===f)return-1;while(0<c);return b},Oaa=function(a,b,c,d){a=iM(a,c,d);return hM(c,d,a,/[^a-fA-F0-9]([a-fA-F0-9]+)\}$/,"i")},Paa=function(a,b,c,d){a=iM(a,c,d);return hM(c,d,a,/([a-fA-F0-9]{64,})(?:\}.*)/,
"s")},Qaa={identify:function(a,b){var c=a.indexOf("{\\pict{",b),d=qu(a,c),e=a.indexOf("{\\shp{",b),g=qu(a,e),k=f.curry(Paa,a,b,e,g),m=f.curry(Oaa,a,b,c,d);return-1===c&&-1===e?h.none():-1===c?k():-1===e?m():e<c&&g>d?m():c<e&&d>g?k():c<e?m():e<c?k():h.none()},endBracket:qu},jM=function(a,b){return Qaa.identify(a,b)},kM=function(a){return 0<=a.indexOf("\\pngblip")?H.value("image/png"):0<=a.indexOf("\\jpegblip")?H.value("image/jpeg"):H.error("errors.imageimport.unsupported")},lM=function(a,b){var c=
a.match(b);return c&&c[1]&&0===c[1].length%2?H.value(c[1]):H.error("errors.imageimport.invalid")},mM=function(a){a=a.match(/\\shplid(\d+)/);return null!==a?h.some(a[1]):h.none()},Raa=function(a){var b=a.bower(),c=a.regex();return mM(b).map(function(d){var e=a.idRef()+d;return kM(b).fold(function(a){return Cj.unsupported(e,a)},function(a){return lM(b,c).fold(function(a){return Cj.unsupported(e,a)},function(b){return Cj.supported(e,a,Naa.convert(b,a))})})})},Saa=function(a){for(var b=[],c=function(){return a.length},
d=function(a){var c=Raa(a);b=b.concat(c.toArray());return a.end()},e=0;e<a.length;)e=jM(a,e).fold(c,d);return b},Taa={nextBower:jM,extractId:mM,extractContentType:kM,extractHex:lM,images:function(a){a=a.replace(/\r/g,"").replace(/\n/g,"");return Saa(a)}},wg={images:function(a){return Taa.images(a)},toId:function(a){return Cj.cata(a,function(a,c){return a},function(a,c,d){return a})},toBlob:function(a){return Cj.cata(a,function(a,c){return H.error(c)},function(a,c,d){return H.value(d)})}},Uaa=function(a){return k.foldl(a,
function(a,c){var d=wg.toId(c);return k.findIndex(a,function(a){return wg.toId(a)===d}).fold(function(){return a.concat([c])},function(d){return wg.toBlob(a[d]).isValue()?a:a.slice(0,d).concat(a.slice(d+1)).concat([c])})},[])},KU={convert:function(a,b,c,d){var e=Uaa(b),f=[],h=!1;a=k.bind(a,function(a,b){var c=m.get(a,"data-image-id");m.remove(a,"rtf-data-image");m.remove(a,"data-image-id");return"unsupported"===c?(h=!0,m.set(a,"alt",d("errors.imageimport.unsupported")),[]):k.find(e,function(a){return wg.toId(a)===
c}).fold(function(){console.log("WARNING: unable to find data for image ",a.dom());return[]},function(b){return wg.toBlob(b).fold(function(b){h=!0;m.set(a,"alt",d(b));return[]},function(b){f.push(a);return[b]})})});we.multiple(a).get(function(a){a=je.updateSources(a,f);c(a,h)})}},nM=function(a){return X.descendants(a,"[rtf-data-image]")},ei={exists:function(a){return 0<nM(a).length},find:nM},oM=function(a){return a.officeStyles().getOr(!0)},pM=function(a){return a.htmlStyles().getOr(!1)},qM=function(a){return a.isWord().getOr(!1)},
Md={proxyBin:function(a){return{handle:function(b,c){return a.proxyBin().fold(function(){console.error(b);return va.pure({response:Ka.cancel(),bundle:rb.nu({})})},c)}}},backgroundAssets:function(a){return za.nu(function(b){a.backgroundAssets().fold(function(){b([])},function(a){a.listen(b)})})},merging:function(a){var b=qM(a);return b&&oM(a)||!b&&pM(a)},mergeOffice:oM,mergeNormal:pM,isWord:qM,isInternal:function(a){return a.isInternal().getOr(!1)}},rM=function(a,b){var c=MU(a,b);return function(a,
b){var f=b.bundle(),h=b.response();return va.sync(function(a){c.get(Md.isWord(f),function(c){c=c.fold(function(){return{response:Ka.cancel(),bundle:b.bundle()}},function(a){return{response:h,bundle:rb.nu({officeStyles:a.officeStyles,htmlStyles:a.htmlStyles})}});va.call(a,c)})})}},ru={fixed:function(a,b){return function(c,d){return va.pure({response:d.response(),bundle:rb.nu({officeStyles:a,htmlStyles:b})})}},fromConfig:rM,fromConfigIfExternal:function(a,b){return function(c,d){return Md.isInternal(d.bundle())?
va.pure({response:d.response(),bundle:rb.nu({officeStyles:!0,htmlStyles:!0})}):rM(a,b)(c,d)}}},Vaa={p:"p, li[data-converted-paragraph]"},Waa=f.noop,Yaa=function(a,b){var c=X.descendants(a,"li[data-converted-paragraph]");k.each(c,function(a){m.remove(a,"data-converted-paragraph")});c=X.descendants(a,"li");k.each(c,function(a){Xaa(a).fold(function(){t.remove(a,"margin-left")},function(b){t.set(a,"margin-left",b)});t.remove(a,"text-indent")});b&&(c=X.descendants(a,"ol,ul"),k.each(c,function(a){var b=
X.descendants(a,"li");t.getRaw(a,"margin-top").orThunk(function(){h.from(b[0]).each(function(b){t.set(a,"margin-top",t.get(b,"margin-top"))})});t.getRaw(a,"margin-bottom").orThunk(function(){h.from(b[b.length-1]).each(function(b){t.set(a,"margin-bottom",t.get(b,"margin-bottom"))})})}))},Xaa=function(a){var b=function(a){return parseFloat(a.match(/-?\d+\.?\d*/)[0])},c=function(a,b){var c=a.replace(/-?\d+\.?\d*/,"");return c===b.replace(/-?\d+\.?\d*/,"")?h.some(c):h.none()};return t.getRaw(a,"text-indent").bind(function(d){return t.getRaw(a,
"margin-left").map(function(e){return c(d,e).fold(function(){var c=t.get(a,"text-indent"),d=t.get(a,"margin-left");return b(c)+b(d)+"px"},function(a){return b(d)+b(e)+a})})})},Zaa={process:function(a,b,c){c=c.mergeInline();(c?Xv.inlineStyles:Waa)(a,b,Vaa);Yaa(b,c)}},$aa=function(a){var b=NU(q.fromDom(document.body));return function(c,d){b.play(c,function(b,c){Zaa.process(b,c,{mergeInline:f.constant(a)});return ua.get(c)},d)}},aba=function(a,b,c,d){var e=function(b){va.call(a,{response:b,bundle:rb.nu({})})};
$aa(b)(c,function(a){var b=Od.fromHtml(a);a=function(a){e(Ka.paste(b,a))};var c=q.fromTag("div");K.append(c,b);var f=ra.all("img[src]",c);0===f.length?a([]):d?je.browserBlobs(f).get(a):(k.each(f,B.remove),a=y.children(c),e(Ka.incomplete(a,[],"errors.local.images.disallowed")))})},bba={handle:function(a,b,c,d){var e=a.html();return va.sync(function(a){c.cleanDocument(e,b).get(function(c){c.fold(function(b){va.call(a,{response:Ka.error("errors.paste.process.failure"),bundle:rb.nu({})})},function(c){void 0===
c||null===c||0===c.length?va.call(a,{response:Ka.paste([],[]),bundle:rb.nu({})}):aba(a,b,c,d)})})})}},cba={handle:function(a){a=k.filter(a,function(a){return"file"===a.kind&&/image/.test(a.type)});var b=k.map(a,function(a){return a.getAsFile()});return va.sync(function(a){we.multiple(b).get(function(b){b=je.createImages(b);va.call(a,{response:b,bundle:rb.nu({})})})})}},dba=function(a,b){var c=A.value(a),d=q.fromTag("div"),e=c.indexOf("]>");d.dom().innerHTML=c.substr(e+2);return sa.descendant(d,function(a){return A.name(a)===
b})},eba=function(a){return A.isComment(a)?dba(a,"v:shape"):h.none()},fba=function(a){if("img"===A.name(a)){var b=m.get(a,"src");if(void 0!==b&&null!==b&&aa.startsWith(b,"file://"))return a=ec.shallow(a),b=b.split(/[\/\\]/),m.set(a,"data-image-id",b[b.length-1]),m.remove(a,"src"),m.set(a,"data-image-type","local"),n.add(a,"rtf-data-image"),h.some(a)}return h.none()},gba=function(a){return eba(a).map(function(a){var c=m.get(a,"o:spid"),c=void 0===c?m.get(a,"id"):c,d=q.fromTag("img");n.add(d,"rtf-data-image");
m.set(d,"data-image-id",c.substr(7));m.set(d,"data-image-type","code");t.setAll(d,{width:t.get(a,"width"),height:t.get(a,"height")});return d})},sM={validStyles:function(){return/^(mso-.*|tab-stops|tab-interval|language|text-underline|text-effect|text-line-through|font-color|horiz-align|list-image-[0-9]+|separator-image|table-border-color-(dark|light)|vert-align|vnd\..*)$/},specialInline:function(){return/^(font|em|strong|samp|acronym|cite|code|dfn|kbd|tt|b|i|u|s|sub|sup|ins|del|var|span)$/}},sh=
ma.generate([{starts:["value","f"]},{pattern:["regex","f"]},{contains:["value","f"]},{exact:["value","f"]},{all:[]},{not:["stringMatch"]}]),tM=function(a,b){return a.fold(function(a,d){return 0===d(b).indexOf(d(a))},function(a,d){return a.test(d(b))},function(a,d){return 0<=d(b).indexOf(d(a))},function(a,d){return d(b)===d(a)},function(){return!0},function(a){return!tM(a,b)})},F={starts:sh.starts,pattern:sh.pattern,contains:sh.contains,exact:sh.exact,all:sh.all,not:sh.not,cata:function(a,b,c,d,e,
f,h){return a.fold(b,c,d,e,f,h)},matches:tM,caseSensitive:function(a){return a},caseInsensitive:function(a){return a.toLowerCase()}},Jn={keyval:function(a,b,c,d){var e=d.name,g=void 0!==d.condition?d.condition:f.constant(!0);d=void 0!==d.value?d.value:F.all();return F.matches(e,c)&&F.matches(d,b)&&g(a)},name:function(a,b){var c=A.name(a),d=b.name,e=void 0!==b.condition?b.condition:f.constant(!0);return F.matches(d,c)&&e(a)}},hba=function(a,b,c){k.each(b,function(b){m.remove(a,b)});W.each(c,function(b,
c){m.set(a,c,b)})},uM={filter:function(a,b){var c={};k.each(a.dom().attributes,function(a){b(a.value,a.name)||(c[a.name]=a.value)});return c},clobber:function(a,b,c){c=k.map(a.dom().attributes,function(a){return a.name});W.size(b)!==c.length&&hba(a,c,b)},scan:f.constant({})},iba=function(a){var b={};a=void 0!==a&&null!==a?a.split(";"):[];k.each(a,function(a){a=a.split(":");2===a.length&&(b[aa.trim(a[0])]=aa.trim(a[1]))});return b},vM=function(a){var b=W.keys(a);return k.map(b,function(b){return b+
": "+a[b]}).join("; ")},mf={filter:function(a,b){var c=a.dom().style,d={};k.each(void 0===c||null===c?[]:c,function(c){var f;f=a.dom().style.getPropertyValue(c);b(f,c)||(d[c]=f)});return d},clobber:function(a,b,c){m.set(a,"style","");var d=W.size(b),e=W.size(c);0===d&&0===e?m.remove(a,"style"):0===d?m.set(a,"style",vM(c)):(W.each(b,function(b,c){t.set(a,c,b)}),b=m.get(a,"style"),c=0<e?vM(c)+"; ":"",m.set(a,"style",c+b))},scan:function(a,b,c){a=a.dom().getAttribute("style");var d=iba(a),e={};k.each(b,
function(a){var b=d[a];void 0===b||c(b,a)||(e[a]=b)});return e}},jba=["mso-list"],wM=function(a,b){var c=mf.scan(a,jba,b),d=mf.filter(a,b);mf.clobber(a,d,c)},xM=function(a,b){var c=uM.filter(a,b);uM.clobber(a,c,{})},Kn={style:wM,attribute:xM,styleDom:function(a,b){wM(q.fromDom(a),b)},attributeDom:function(a,b){xM(q.fromDom(a),b)},validateStyles:function(a){var b=mf.filter(a,f.constant(!1));mf.clobber(a,b,{})}},yM=function(a,b,c){a(c,function(a,e){return k.exists(b,function(b){return Jn.keyval(c,a,
e,b)})})},Ln={remover:function(a,b){var c=Z.merge({styles:[],attributes:[],classes:[],tags:[]},b),d=X.descendants(a,"*");k.each(d,function(a){yM(Kn.style,c.styles,a);yM(Kn.attribute,c.attributes,a);k.each(c.classes,function(b){var c=m.has(a,"class")?ea.get(a):[];k.each(c,function(c){F.matches(b.name,c)&&n.remove(a,c)})})});d=X.descendants(a,"*");k.each(d,function(a){k.exists(c.tags,f.curry(Jn.name,a))&&B.remove(a)})},unwrapper:function(a,b){var c=Z.merge({tags:[]},b),d=X.descendants(a,"*");k.each(d,
function(a){k.exists(c.tags,f.curry(Jn.name,a))&&B.unwrap(a)})},transformer:function(a,b){var c=Z.merge({tags:[]},b),d=X.descendants(a,"*");k.each(d,function(a){k.find(c.tags,f.curry(Jn.name,a)).each(function(b){b.mutate(a)})})},validator:function(a){a=X.descendants(a,"*");k.each(a,function(a){Kn.validateStyles(a)})}},Dj=function(a,b,c){var d,e,f,h=q.fromDom(a);switch(a.nodeType){case 1:b?d="endElement":(d="startElement",t.setAll(h,c||{}));e="HTML"!==a.scopeName&&a.scopeName&&a.tagName&&0>=a.tagName.indexOf(":")?
(a.scopeName+":"+a.tagName).toUpperCase():a.tagName;break;case 3:d="text";f=a.nodeValue;break;case 8:d="comment";f=a.nodeValue;break;default:console.log("WARNING: Unsupported node type encountered: "+a.nodeType)}return{getNode:function(){return a},tag:function(){return e},type:function(){return d},text:function(){return f}}},zM=function(a,b){return Dj(b.createElement(a),!0)},cb={START_ELEMENT_TYPE:"startElement",END_ELEMENT_TYPE:"endElement",TEXT_TYPE:"text",COMMENT_TYPE:"comment",FINISHED:zM("HTML",
window.document),token:Dj,createStartElement:function(a,b,c,d){var e=d.createElement(a);W.each(b,function(a,b){e.setAttribute(b,a)});return Dj(e,!1,c)},createEndElement:zM,createComment:function(a,b){return Dj(b.createComment(a),!1)},createText:function(a,b){return Dj(b.createTextNode(a))}},kba={create:function(a){var b=a.createDocumentFragment(),c=b;return{dom:c,receive:function(d){var e=function(a){a=a.getNode().cloneNode(!1);b.appendChild(a);b=a},f=function(c,d){var e=a.createTextNode(c.text());
b.appendChild(e)};switch(d.type()){case cb.START_ELEMENT_TYPE:e(d);break;case cb.TEXT_TYPE:f(d);break;case cb.END_ELEMENT_TYPE:b=b.parentNode;null===b&&(b=c);break;case cb.COMMENT_TYPE:break;default:throw{message:"Unsupported token type: "+d.type()};}},label:"SERIALISER"}}},lba={tokenise:function(a,b){var c;b=b||window.document;c=b.createElement("div");b.body.appendChild(c);c.style.position="absolute";c.style.left="-10000px";c.innerHTML=a;var d=c.firstChild||cb.FINISHED,e=[],f=!1;return{hasNext:function(){return void 0!==
d},next:function(){var a=d,h=f;!f&&d.firstChild?(e.push(d),d=d.firstChild):f||1!==d.nodeType?d.nextSibling?(d=d.nextSibling,f=!1):(d=e.pop(),f=!0):f=!0;a===cb.FINISHED||d||(b.body.removeChild(c),d=cb.FINISHED);a=a===cb.FINISHED?a:a?cb.token(a,h):void 0;return a}}}},AM=function(a,b,c){var d=c;for(c=b.length-1;0<=c;c--)d=b[c](d,{},a);return d},mba={build:AM,run:function(a,b,c){var d=kba.create(a);b=lba.tokenise(b,a);for(a=AM(a,c,d);b.hasNext();)c=b.next(),a.receive(c);return d.dom}},Ra={removal:function(a){return function(b){Ln.remover(b,
a)}},unwrapper:function(a){return function(b){Ln.unwrapper(b,a)}},transformer:function(a){return function(b){Ln.transformer(b,a)}},validate:function(){return function(a){Ln.validator(a)}},pipeline:function(a){return function(b){var c=ua.get(b),d=y.owner(b),c=mba.run(d.dom(),c,a);B.empty(b);b.dom().appendChild(c)}},isWordContent:function(a,b){return 0<=a.indexOf("<o:p>")||b.browser.isEdge()&&0<=a.indexOf('v:shapes="')||b.browser.isEdge()&&0<=a.indexOf("mso-")||0<=a.indexOf("mso-list")||0<=a.indexOf("p.MsoNormal, li.MsoNormal, div.MsoNormal")||
0<=a.indexOf("MsoListParagraphCxSpFirst")||0<=a.indexOf("<w:WordDocument>")},go:function(a,b,c){var d=q.fromTag("div",a.dom());d.dom().innerHTML=b;k.each(c,function(a){a(d)});return ua.get(d)}},su={getAttribute:function(a,b){var c=q.fromDom(a.getNode());return m.get(c,b)},getStyle:function(a,b){var c=q.fromDom(a.getNode());return t.get(c,b)},isWhitespace:function(a){return a.type()===cb.TEXT_TYPE&&/^[\s\u00A0]*$/.test(a.text())},getMsoList:function(a){a=q.fromDom(a.getNode());return mf.scan(a,["mso-list"],
f.constant(!1))["mso-list"]}},BM=function(a,b,c){return a===b||a&&b&&a.tag===b.tag&&a.type===b.type&&(c||a.variant===b.variant)},CM={guessFrom:function(a,b){return k.find(a,function(a){return"UL"===a.tag||b&&BM(a,b,!0)}).orThunk(function(){return k.head(a)})},eqListType:BM},tu=function(a,b){if(void 0===a||void 0===b)throw console.trace(),"brick";a.nextFilter.set(b)},DM=function(a,b,c){b.nextFilter.get()(a,b,c)},Mn={next:tu,go:DM,jump:function(a){return function(b,c,d){tu(c,a);DM(b,c,d)}},isNext:function(a,
b){return a.nextFilter.get()===b},setNext:function(a){return function(b,c,d){tu(c,a)}}},nba=C.immutable("level","token","type"),oba=function(a,b,c,d){a=c.getCurrentListType();c=c.getCurrentLevel()==d.level()?a:null;return CM.guessFrom(d.emblems(),c).filter(function(a){if(a="OL"===a.tag)a=!k.contains(["P"],b.tag())||/^MsoHeading/.test(su.getAttribute(b,"class"));return!a})},EM={predicate:function(a,b){return m.has(q.fromDom(b.getNode()),"data-list-level")},action:function(a){return function(b,c,d){b=
q.fromDom(d.getNode());var e=parseInt(m.get(b,"data-list-level"),10),g=m.get(b,"data-list-emblems"),g=JSON.parse(g);m.remove(b,"data-list-level");m.remove(b,"data-list-emblems");b={level:f.constant(e),emblems:f.constant(g)};b.level();c.originalToken.set(d);oba(c.listType.get(),d,c.emitter,b).each(c.listType.set);d=nba(b.level(),c.originalToken.get(),c.listType.get());c.emitter.openItem(d.level(),d.token(),d.type());Mn.next(c,a.inside())}}},Ox=function(a,b){return function(a,d,e){return b(a,d,e)}},
FM={inside:function(a){return Nx("Inside.List.Item",[mp(function(a,c){return c.type()===cb.END_ELEMENT_TYPE&&a.originalToken.get()&&c.tag()===a.originalToken.get().tag()},function(b,c,d){Mn.next(c,a.outside())},"Closing open tag")],function(a,c,d){a.emit(d)})},outside:function(a){return Nx("Outside.List.Item",[mp(EM.predicate,EM.action(a),"Data List ****"),mp(function(a,c){return c.type()===cb.TEXT_TYPE&&su.isWhitespace(c)},function(a,c,d){a.emit(d)},"Whitespace")],function(b,c,d){c.emitter.closeAllLists();
b.emit(d);Mn.next(c,a.outside())})}},pba=C.immutable("state","result"),qba=C.immutable("state","value"),Eb={state:C.immutable("level","type","types","items"),value:qba,result:pba},uu={start:function(a,b){var c=a.items().slice(0),d=void 0!==b&&"P"!==b?h.some(b):h.none();d.fold(function(){c.push("P")},function(a){c.push(a)});var e=Eb.state(a.level(),a.type(),a.types(),c);return Eb.value(e,d)},finish:function(a){var b=a.items().slice(0);if(0<b.length&&"P"!==b[b.length-1]){var c=b[b.length-1];b[b.length-
1]="P";a=Eb.state(a.level(),a.type(),a.types(),b);return Eb.value(a,h.some(c))}return Eb.value(a,h.none())}},vu=function(a,b,c){for(var d=[];b(a);){var e=c(a);a=e.state();d=d.concat(e.result())}return Eb.result(a,d)},GM={moveRight:function(a,b,c){return vu(a,function(a){return a.level()<b},c)},moveLeft:function(a,b,c){return vu(a,function(a){return a.level()>b},c)},moveUntil:vu},rba={from:function(a){var b={"list-style-type":"none"};a?(a=su.getStyle(a,"margin-left"),a=void 0!==a&&"0px"!==a?{"margin-left":a}:
{}):a=b;return a}},HM=function(a,b,c){var d=b.start&&1<b.start?{start:b.start}:{},e=a.level()+1,g=a.types().concat([b]);c=[f.curry(cb.createStartElement,b.tag,d,c)];a=Eb.state(e,b,g,a.items());return Eb.result(a,c)},IM=function(a){var b=a.types().slice(0),c=[f.curry(cb.createEndElement,b.pop().tag)],d=a.level()-1;a=Eb.state(d,b[b.length-1],b,a.items());return Eb.result(a,c)},sba=function(a,b){var c=IM(a),d=HM(c.state(),b,b.type?{"list-style-type":b.type}:{});return Eb.result(d.state(),c.result().concat(d.result()))},
th={open:HM,openItem:function(a,b,c){var d=rba.from(b);a=a.type()&&!CM.eqListType(a.type(),c)?sba(a,c):Eb.result(a,[]);d=[f.curry(cb.createStartElement,"LI",{},d)];c=uu.start(a.state(),b&&b.tag());var e=c.value().map(function(a){Kn.styleDom(b.getNode(),f.constant(!0));return[f.constant(b)]}).getOr([]);return Eb.result(c.state(),a.result().concat(d).concat(e))},close:IM,closeItem:function(a){var b=f.curry(cb.createEndElement,"LI");a=uu.finish(a);var c=a.value().fold(function(){return[b]},function(a){return[f.curry(cb.createEndElement,
a),b]});return Eb.result(a.state(),c)}},Nn=function(a){if(0===a.length)throw"Compose must have at least one element in the list";var b=a[a.length-1];a=k.bind(a,function(a){return a.result()});return Eb.result(b.state(),a)},tba=function(a){a=th.closeItem(a);var b=th.close(a.state());return Nn([a,b])},uba=function(a,b,c,d){return GM.moveRight(a,c,function(a){var f=a.level()===c-1&&b.type?{"list-style-type":b.type}:{};a=th.open(a,b,f);f=th.openItem(a.state(),a.state().level()==c?d:void 0,b);return Nn([a,
f])})},JM=function(a,b){return GM.moveLeft(a,b,tba)},vba=function(a,b,c,d){var e=1<c?uu.finish(a):Eb.value(a,h.none());a=e.value().map(function(a){return[f.curry(cb.createEndElement,a)]}).getOr([]);e.state().level();b=uba(e.state(),b,c,d);return Eb.result(b.state(),a.concat(b.result()))},Px={openItem:function(a,b,c,d){a=a.level()>b?JM(a,b):Eb.result(a,[]);a.state().level()===b?(b=a.state(),b=0<b.level()?th.closeItem(b):Eb.result(b,[]),c=th.openItem(b.state(),c,d),c=Nn([b,c])):c=vba(a.state(),d,b,
c);return Nn([a,c])},closeAllLists:JM},PU=["disc","circle","square"],wba={getCurrentListType:function(){return Ej().getCurrentListType()},getCurrentLevel:function(){return Ej().getCurrentLevel()},closeAllLists:function(){return Ej().closeAllLists.apply(void 0,arguments)},openItem:function(){return Ej().openItem.apply(void 0,arguments)}},Ej=function(){return{getCurrentListType:f.constant({}),getCurrentLevel:f.constant(1),closeAllLists:f.identity,openItem:f.identity}},LM={inside:function(){return xba},
outside:function(){return KM}},yba=function(){var a=!1,b="";return{check:function(c){return a&&c.type()===cb.TEXT_TYPE?(b+=c.text(),!0):c.type()===cb.START_ELEMENT_TYPE&&"STYLE"===c.tag()?a=!0:c.type()===cb.END_ELEMENT_TYPE&&"STYLE"===c.tag()?(a=!1,!0):!1}}}(),xba=FM.inside(LM),KM=FM.outside(LM),MM=function(a){var b=wa(a),c=wa(null),d=wa(null);return{reset:function(e){b.set(a);c.set(null);d.set(null);e=OU(e,e.document);Ej=f.constant(e)},nextFilter:b,originalToken:c,listType:d,emitter:wba}}(KM),zba=
Mx(function(a,b,c){yba.check(b)||Mn.go(a,MM,b)},MM.reset,"list.filters"),Aba=[{regex:/^\(?[dc][\.\)]$/,type:{tag:"OL",type:"lower-alpha"}},{regex:/^\(?[DC][\.\)]$/,type:{tag:"OL",type:"upper-alpha"}},{regex:/^\(?M*(CM|CD|D?C{0,3})(XC|XL|L?X{0,3})(IX|IV|V?I{0,3})[\.\)]$/,type:{tag:"OL",type:"upper-roman"}},{regex:/^\(?m*(cm|cd|d?c{0,3})(xc|xl|l?x{0,3})(ix|iv|v?i{0,3})[\.\)]$/,type:{tag:"OL",type:"lower-roman"}},{regex:/^\(?[0-9]+[\.\)]$/,type:{tag:"OL"}},{regex:/^([0-9]+\.)*[0-9]+\.?$/,type:{tag:"OL",
variant:"outline"}},{regex:/^\(?[a-z]+[\.\)]$/,type:{tag:"OL",type:"lower-alpha"}},{regex:/^\(?[A-Z]+[\.\)]$/,type:{tag:"OL",type:"upper-alpha"}}],NM={"\u2022":{tag:"UL",type:"disc"},"\u00b7":{tag:"UL",type:"disc"},"\u00a7":{tag:"UL",type:"square"}},OM={o:{tag:"UL",type:"circle"},"-":{tag:"UL",type:"disc"},"\u25cf":{tag:"UL",type:"disc"},"\ufffd":{tag:"UL",type:"circle"}},Bba=function(a,b){return void 0!==a.variant?a.variant:"("===b.charAt(0)?"()":")"===b.charAt(b.length-1)?")":"."},Cba=function(a){var b=
a.split(".");0===b.length?b=a:(a=b[b.length-1],b=0===a.length&&1<b.length?b[b.length-2]:a);b=parseInt(b,10);return isNaN(b)?{}:{start:b}},Dba={extract:function(a,b){var c=OM[a]?[OM[a]]:[],d=b&&NM[a]?[NM[a]]:b?[{tag:"UL",variant:a}]:[],e=k.bind(Aba,function(b){return b.regex.test(a)?[Z.merge(b.type,Cba(a),{variant:Bba(b.type,a)})]:[]}),c=c.concat(d).concat(e);return k.map(c,function(b){return void 0!==b.variant?b:Z.merge(b,{variant:a})})}},PM=function(a){return mf.scan(a,["mso-list"],f.constant(!1))["mso-list"]},
QM=function(a){return A.isElement(a)&&t.getRaw(a,"font-family").exists(function(a){return k.contains(["wingdings","symbol"],a.toLowerCase())})},Rb={getMsoList:PM,extractLevel:function(a){a=PM(a);return(a=/ level([0-9]+)/.exec(a))&&a[1]?h.some(parseInt(a[1],10)):h.none()},extractEmblem:function(a,b){var c=pb.get(a).trim(),c=Dba.extract(c,b);return 0<c.length?h.some(c):h.none()},extractSymSpan:function(a){return sa.child(a,QM)},extractMsoIgnore:function(a){return sa.descendant(a,function(a){return!!(A.isElement(a)?
mf.scan(a,["mso-list"],f.constant(!1)):[])["mso-list"]})},extractCommentSpan:function(a){return sa.child(a,A.isComment).bind(y.nextSibling).filter(function(a){return"span"===A.name(a)})},isSymbol:QM,deduceLevel:function(a){return t.getRaw(a,"margin-left").bind(function(a){a=parseInt(a,10);return isNaN(a)?h.none():h.some(Math.max(1,Math.ceil(a/18)))})}},RM=function(a){for(var b=[];null!==a.nextNode();)b.push(q.fromDom(a.currentNode));return b},Eba=function(a){try{return RM(a)}catch(b){return[]}},SM=
Y.detect().browser,Fba=SM.isIE()||SM.isEdge()?Eba:RM,Gba=f.constant(f.constant(!0)),Hba={find:function(a,b){var c=b.fold(Gba,function(a){return function(b){return a(b.nodeValue)}});c.acceptNode=c;c=document.createTreeWalker(a.dom(),Vb.getOrDie("NodeFilter").SHOW_COMMENT,c,!1);return Fba(c)}},Fj=function(a,b,c,d){m.set(a,"data-list-level",b);b=JSON.stringify(c);m.set(a,"data-list-emblems",b);b=Hba.find(a,h.none());k.each(b,B.remove);k.each(d,B.remove);m.remove(a,"style");m.remove(a,"class")},Iba=function(a){return Rb.extractLevel(a).bind(function(b){return Rb.extractSymSpan(a).bind(function(c){return Rb.extractEmblem(c,
!0).map(function(d){return{mutate:function(){Fj(a,b,d,[c])}}})})})},Jba=function(a){return Rb.extractLevel(a).bind(function(b){return Rb.extractCommentSpan(a).bind(function(c){return Rb.extractEmblem(c,Rb.isSymbol(c)).map(function(d){return{mutate:function(){Fj(a,b,d,[c])}}})})})},Kba=function(a){return Rb.extractLevel(a).bind(function(b){return Rb.extractCommentSpan(a).bind(function(c){return Rb.extractEmblem(c,Rb.isSymbol(c)).map(function(d){return{mutate:function(){Fj(a,b,d,[c])}}})})})},Lba=function(a){return"p"!==
A.name(a)?h.none():Rb.extractLevel(a).bind(function(b){return Rb.extractMsoIgnore(a).bind(function(c){return Rb.extractEmblem(c,!1).map(function(d){return{mutate:function(){Fj(a,b,d,[y.parent(c).getOr(c)])}}})})})},Mba=function(a){return"p"!==A.name(a)?h.none():Rb.extractMsoIgnore(a).bind(function(b){var c=y.parent(b).getOr(b),d=Rb.isSymbol(c);return Rb.extractEmblem(b,d).bind(function(b){return Rb.deduceLevel(a).map(function(d){return{mutate:function(){Fj(a,d,b,[c])}}})})})},Nba={find:function(a){return Iba(a).orThunk(function(){return Jba(a)}).orThunk(function(){return Kba(a)}).orThunk(function(){return Lba(a)}).orThunk(function(){return Mba(a)})}},
Oba=Ra.transformer({tags:[{name:F.pattern(/^(p|h\d+)$/,F.caseInsensitive),mutate:function(a){Nba.find(a).each(function(a){a.mutate()})}}]}),Pba=function(a){var b=a.dom().attributes,b=void 0!==b&&null!==b&&0<b.length;return"span"===A.name(a)?b:!0},On=function(a){return void 0===a.dom().attributes||null===a.dom().attributes?!0:0===a.dom().attributes.length||1===a.dom().attributes.length&&"style"===a.dom().attributes[0].name},TM=function(a,b){var c=q.fromTag(a);v.before(b,c);var d=b.dom().attributes;
k.each(d,function(a){c.dom().setAttribute(a.name,a.value)});d=y.children(b);K.append(c,d);B.remove(b);return c},UM=function(a){return y.prevSibling(a).bind(function(a){return A.isText(a)&&0===Da.get(a).trim().length?UM(a):"li"===A.name(a)?h.some(a):h.none()})},bg={changeTag:TM,addBrTag:function(a){0===ua.get(a).length&&v.append(a,q.fromTag("br"))},properlyNest:function(a){y.parent(a).each(function(b){b=A.name(b);k.contains(["ol","ul"],b)&&UM(a).fold(function(){var b=q.fromTag("li");t.set(b,"list-style-type",
"none");v.wrap(a,b)},function(b){v.append(b,a)})})},fontToSpan:function(a){var b=TM("span",a),c={"font-size":{1:"8pt",2:"10pt",3:"12pt",4:"14pt",5:"18pt",6:"24pt",7:"36pt"}};W.each({face:"font-family",size:"font-size",color:"color"},function(a,e){if(m.has(b,e)){var f=m.get(b,e);t.set(b,a,void 0!==c[a]&&void 0!==c[a][f]?c[a][f]:f);m.remove(b,e)}})}},Qba=Ra.unwrapper({tags:[{name:F.pattern(/^([OVWXP]|U[0-9]+|ST[0-9]+):/i,F.caseInsensitive)}]}),Rba=[Ra.pipeline([zba])],Sba=Ra.removal({attributes:[{name:F.pattern(/^v:/,
F.caseInsensitive)},{name:F.exact("href",F.caseInsensitive),value:F.contains("#_toc",F.caseInsensitive)},{name:F.exact("href",F.caseInsensitive),value:F.contains("#_mso",F.caseInsensitive)},{name:F.pattern(/^xmlns(:|$)/,F.caseInsensitive)},{name:F.exact("type",F.caseInsensitive),condition:function(a){return"ol"===A.name(a)||"ul"===A.name(a)}}]});Ra.removal({attributes:[{name:F.exact("id",F.caseInsensitive)},{name:F.exact("name",F.caseInsensitive)}]});var Tba=Ra.removal({tags:[{name:F.exact("script",
F.caseInsensitive)},{name:F.exact("link",F.caseInsensitive)},{name:F.exact("style",F.caseInsensitive),condition:function(a){return 0===ua.get(a).length}}],attributes:[{name:F.starts("on",F.caseInsensitive)},{name:F.exact('"',F.caseInsensitive)},{name:F.exact("lang",F.caseInsensitive)},{name:F.exact("language",F.caseInsensitive)}],styles:[{name:F.all(),value:F.pattern(/OLE_LINK/i,F.caseInsensitive)}]}),VM=Ra.removal({tags:[{name:F.exact("meta",F.caseInsensitive)}]}),Uba=Ra.removal({tags:[{name:F.exact("style",
F.caseInsensitive)}]}),Vba=Ra.removal({styles:[{name:F.not(F.pattern(/width|height|list-style-type/,F.caseInsensitive)),condition:function(a){return!n.has(a,"ephox-limbo-transform")}},{name:F.pattern(/width|height/,F.caseInsensitive),condition:function(a){return"img"!==A.name(a)}}]}),Wba=Ra.removal({classes:[{name:F.not(F.exact("rtf-data-image",F.caseInsensitive))}]}),Xba=Ra.removal({styles:[{name:F.pattern(sM.validStyles(),F.caseInsensitive)}]}),Yba=Ra.removal({classes:[{name:F.pattern(/mso/i,F.caseInsensitive)}]}),
Zba=Ra.unwrapper({tags:[{name:F.exact("img",F.caseInsensitive),condition:function(a){a=m.get(a,"src");return/^file:/.test(a)}},{name:F.exact("a",F.caseInsensitive),condition:On}]}),$ba=Ra.unwrapper({tags:[{name:F.exact("a",F.caseInsensitive),condition:On}]}),aca=Ra.removal({attributes:[{name:F.exact("style",F.caseInsensitive),value:F.exact("",F.caseInsensitive),debug:!0}]}),bca=Ra.removal({attributes:[{name:F.exact("class",F.caseInsensitive),value:F.exact("",F.caseInsensitive),debug:!0}]}),cca=Ra.unwrapper({tags:[{name:F.pattern(sM.specialInline(),
F.caseInsensitive),condition:f.not(function(a){return On(a)?Pba(a)&&Gc.descendant(a,function(a){var c=!On(a),d=!k.contains("font em strong samp acronym cite code dfn kbd tt b i u s sub sup ins del var span".split(" "),A.name(a));return A.isText(a)||c||d}):!0})}]}),dca=Ra.unwrapper({tags:[{name:F.exact("p",F.caseInsensitive),condition:function(a){return function(b){return y.parent(b).exists(function(b){return A.name(b)===a&&1===y.children(b).length})}}("li")}]}),eca=Ra.transformer({tags:[{name:F.exact("p",
F.caseInsensitive),mutate:bg.addBrTag}]}),fca=Ra.transformer({tags:[{name:F.pattern(/ol|ul/,F.caseInsensitive),mutate:bg.properlyNest}]}),gca=Ra.transformer({tags:[{name:F.exact("b",F.caseInsensitive),mutate:f.curry(bg.changeTag,"strong")},{name:F.exact("i",F.caseInsensitive),mutate:f.curry(bg.changeTag,"em")},{name:F.exact("u",F.caseInsensitive),mutate:function(a){a=bg.changeTag("span",a);n.add(a,"ephox-limbo-transform");t.set(a,"text-decoration","underline")}},{name:F.exact("s",F.caseInsensitive),
mutate:f.curry(bg.changeTag,"strike")},{name:F.exact("font",F.caseInsensitive),mutate:bg.fontToSpan,debug:!0}]}),hca=Ra.removal({classes:[{name:F.exact("ephox-limbo-transform",F.caseInsensitive)}]}),ica=Ra.removal({tags:[{name:F.exact("br",F.caseInsensitive),condition:me.hasClass("Apple-interchange-newline")}]}),jca=Ra.removal({styles:[{name:F.pattern(/^-/,F.caseInsensitive)},{name:F.all(),value:F.exact("initial",F.caseInsensitive)},{name:F.exact("background-color",F.caseInsensitive),value:F.exact("transparent",
F.caseInsensitive)},{name:F.exact("font-style",F.caseInsensitive),value:F.exact("normal",F.caseInsensitive)},{name:F.pattern(/font-variant.*/,F.caseInsensitive)},{name:F.exact("letter-spacing",F.caseInsensitive)},{name:F.exact("font-weight",F.caseInsensitive),value:F.pattern(/400|normal/,F.caseInsensitive)},{name:F.exact("orphans",F.caseInsensitive)},{name:F.exact("text-decoration",F.caseInsensitive),value:F.exact("none",F.caseInsensitive)},{name:F.exact("text-size-adjust",F.caseInsensitive)},{name:F.exact("text-indent",
F.caseInsensitive),value:F.exact("0px",F.caseInsensitive)},{name:F.exact("text-transform",F.caseInsensitive),value:F.exact("none",F.caseInsensitive)},{name:F.exact("white-space",F.caseInsensitive),value:F.exact("normal",F.caseInsensitive)},{name:F.exact("widows",F.caseInsensitive)},{name:F.exact("word-spacing",F.caseInsensitive),value:F.exact("0px",F.caseInsensitive)},{name:F.exact("text-align",F.caseInsensitive),value:F.pattern(/start|end/,F.caseInsensitive)},{name:F.exact("font-weight",F.caseInsensitive),
value:F.pattern(/700|bold/,F.caseInsensitive),condition:function(a){return/^h\d$/.test(A.name(a))}}]}),kca=function(){var a=function(a,b){return function(c){return a(c).filter(function(a){return A.isText(c)&&b(pb.get(a)," ")}).isSome()}},b=a(y.prevSibling,aa.endsWith),c=a(y.nextSibling,aa.startsWith);return Ra.transformer({tags:[{name:F.exact("span",F.caseInsensitive),condition:me.hasClass("Apple-converted-space"),mutate:function(a){"\u00a0"===pb.get(a)&&(b(a)||c(a)?B.unwrap(a):(v.prepend(a,q.fromText(" ")),
B.remove(a)))}}]})}(),lca=function(){var a=/^file:\/\/\/[^#]+(#[^#]+)$/;return Ra.transformer({tags:[{name:F.exact("a",F.caseInsensitive),condition:function(b){return(b=m.get(b,"href"))?a.test(b):!1},mutate:function(b){var c=m.get(b,"href");m.set(b,"href",c.replace(a,"$1"))}}]})}(),mca=Ra.removal({attributes:[{name:F.exact("href",F.caseInsensitive),value:F.starts("file:///",F.caseInsensitive),debug:!0}]}),WM=function(a){a=pc.descendants(a,A.isComment);k.each(a,function(a){var c=wI.get(a);(aa.contains(c,
"StartFragment")||aa.contains(c,"EndFragment"))&&B.remove(a)})},Pn=f.noop,Qn=function(a){return a.browser.isIE()&&11<=a.browser.version.major},nca=function(a,b,c){var d=q.fromDom(a.getNode());return b(d).fold(function(){return[a]},function(b){var d=a.type()===cb.END_ELEMENT_TYPE,f=[cb.token(b.dom(),d)];!d&&c&&f.push(cb.token(b.dom(),!0));return f})},oca=function(a,b){return Mx(function(c,d){var e=nca(d,a,b);k.each(e,c.emit)},f.noop,"image filters")},pca=function(a,b){return Qn(b)&&a?[dca]:[]},qca=
function(a,b,c){a=[Xba,Yba];c=[Vba,Wba];return b?a:c},rca=function(a,b,c){return Qn(c)||!a?[]:[Oba]},sca=function(a,b,c){if(!a)return[Pn];a=[Qba];c=Qn(c)?[]:Rba;return a.concat(c).concat([Sba])},tca={derive:function(a,b,c){var d,e;d=(e=c.browser.isFirefox()||c.browser.isEdge())?fba:gba;var f=!e;d=Qn(c)?Pn:Ra.pipeline([oca(d,f)]);d=[a?d:Pn];e=[e?Pn:Zba];return k.flatten([rca(a,b,c),d,[gca],sca(a,b,c),[fca],[Tba],[VM],e,qca(a,b,c),[lca,mca,$ba],[aca],[bca],[cca],[ica],!a&&b?[jca]:[],[kca],[eca],pca(a,
c),[hca],[WM],[Uba]])}},uca=Dl.link().source,vca=function(a){var b=C.immutable("word","pattern"),c=db.unsafetoken(uca),b=b("__INTERNAL__",c);return yn.run(a,[b])},wca=function(a){return!eb.closest(a,"a")},xca=function(a){return h.from(a[0]).filter(wca).map(function(b){var c=q.fromTag("a");v.before(b,c);K.append(c,a);m.set(c,"href",pb.get(c));return c})},yca=function(a){a=vca(a);k.each(a,function(a){var c=a.exact(),d;(d=0>c.indexOf("@"))||(c=c.indexOf("://"),d=3<=c&&9>=c);d&&xca(a.elements())})},zca=
function(a){k.each(a,function(a){A.isElement(a)&&t.getRaw(a,"position").isSome()&&t.remove(a,"position")})},Aca=function(a){var b=k.filter(a,function(a){return"li"===A.name(a)});if(0<b.length){var c=y.prevSiblings(b[0]),d=q.fromTag("ul");v.before(a[0],d);0<c.length&&(a=q.fromTag("li"),v.append(d,a),K.append(a,c));K.append(d,b)}},XM=function(a){var b=y.children(a);k.each([yca,zca,Aca],function(a){a(b)})},wu={go:function(a,b,c,d,e){XM(c);c=ua.get(c);b=tca.derive(e,d,b);return Ra.go(a,c,b)},preprocess:XM,
internal:function(a,b){var c=ua.get(b);return Ra.go(a,c,[WM,VM])}},Bca=Y.detect(),YM=function(a){try{var b=a(),c=void 0!==b&&null!==b&&0<b.length?Od.fromHtml(b):[];return H.value(c)}catch(d){return console.error(d),H.error("errors.paste.process.failure")}},ZM=function(a){return a.fold(function(a){return va.error(a)},function(a){return va.pure({response:Ka.paste(a,[]),bundle:rb.nu({})})})},$M=function(a,b,c,d){return YM(function(){return wu.go(a,Bca,b,c,d)})},Rn={direct:function(a,b,c){a=$M(a,b,c,
!1);return ZM(a)},internal:function(a,b){var c=YM(function(){return wu.internal(a,b)});return ZM(c)},transfer:function(a,b,c,d,e){return $M(a,b,d,c).fold(function(a){return va.error(a)},function(a){return va.sync(function(b){e.get(function(c){c=je.findImages(a,c);va.call(b,{response:Ka.paste(a,c),bundle:rb.nu({})})})})})}},aN={internal:function(a,b,c){b=b.findClipboardTags(y.children(c)).getOr([]);k.each(b,B.remove);b=za.nu(function(a){a([])});return Rn.transfer(a,c,!1,!0,b)},external:function(a,
b,c,d,e){return Rn.transfer(a,b,d,c,e)}},bN={encode:function(a){var b=q.fromTag("div");pb.set(b,a);return ua.get(b)},convert:function(a){a=a.trim().split(/\n{2,}|(?:\r\n){2,}/);a=k.map(a,function(a){return a.split(/\n|\r\n/).join("<br />")});return 1===a.length?a[0]:k.map(a,function(a){return"<p>"+a+"</p>"}).join("")}},Cca={handle:function(a){var b=ic.asText(a).getOrDie("Required text input for Text Handler");return va.sync(function(a){var d;0<b.text().length?(d=b.text(),d=bN.encode(d),d=bN.convert(d),
d=Od.fromHtml(d),d=Ka.paste(d,[])):d=Ka.cancel();va.call(a,{response:d,bundle:rb.nu({})})})}},gd={plain:function(a,b,c){return Cca.handle(a)},autolink:function(a,b){var c=function(a,c){var d=q.fromTag("div");K.append(d,a);wu.preprocess(d);d=y.children(d);return va.pure({response:Ka.paste(d,c),bundle:b.bundle()})},d=f.curry(va.pass,b);return Ka.cata(b.response(),d,c,d,c)},noImages:function(){return function(a,b){return va.error("errors.local.images.disallowed")}},images:function(){return function(a,
b){var c=ic.asImages(a).getOrDie("Must have image data for images handler");return cba.handle(c.images())}},handleInternal:function(a){return function(b,c){var d=ic.asHtml(b).getOrDie("Wrong input type for HTML handler"),d=a.findClipboardTags(y.children(d.container()));d.each(function(a){k.each(a,B.remove)});d=d.isSome();return va.pure({response:c.response(),bundle:rb.nu({isInternal:d})})}},nativeHtml:function(a,b,c){return function(c,e){var f=ic.asHtml(c).getOrDie("Wrong input type for HTML handler").container(),
h=y.owner(b),k=e.bundle();if(Md.isInternal(k))return Rn.internal(h,f);a(f);k=Md.merging(k);return Rn.direct(h,f,k)}},external:function(a,b){return function(c,d){var e=d.bundle();return Md.proxyBin(e).handle("There was no proxy bin setup. Ensure you have run proxyStep first.",function(c){var d=Md.merging(e),f=Md.isWord(e),h=Md.isInternal(e),k=Md.backgroundAssets(e),m=y.owner(a);return h?aN.internal(m,b,c):aN.external(m,c,d,f,k)})}},wordimport:function(a,b){return function(c,d){var e=ic.asWord(c).getOrDie("Wrong input type for Word Import handler"),
f=Md.mergeOffice(d.bundle());return bba.handle(e,f,a,b)}},setBundle:function(a){return function(b,c){var d=rb.merge(c.bundle(),rb.nu(a));return va.pure({response:c.response(),bundle:d})}},none:function(a,b){return va.cancel()}},Dca=function(a,b){return Gc.descendant(a,function(a){return m.has(a,"style")?-1<m.get(a,"style").indexOf("mso-"):!1})},Eca=function(a,b){var c=ua.get(a);return Ra.isWordContent(c,b)},RU={isWord:function(a,b){var c=a.browser;return(c.isIE()&&11<=c.version.major?Dca:Eca)(b,a)}},
np=Y.detect(),Fca=function(a,b){return a.isSupported()?b.getWordData():h.none()},Gca=function(a){return a.getNative()},Hca=function(a){return a.getImage()},Ica=function(a){return a.getHtml()},Jca=function(a){return a.getText()},Kca=function(a){return a.getOnlyText()},cg=function(a,b,c,d){return{_label:a,label:f.constant(a),getAvailable:b,steps:f.constant(c),capture:f.constant(d)}},Df={wordimport:function(a,b,c,d){return cg("Word Import pasting",f.curry(Fca,a),[mb.normal(gd.setBundle({isWord:!0})),
mb.normal(ru.fromConfig(b,c)),mb.normal(gd.wordimport(a,c.allowLocalImages)),mb.blocking(SU(c,Lx(d,c))),mb.normal(Nk())],!0)},image:function(a){return cg("Image pasting",Hca,[mb.normal(!1===a.allowLocalImages?gd.noImages(f.noop):gd.images()),mb.normal(Nk())],!0)},nativeHtml:function(a,b,c,d){return cg("Outside of Textbox.io pasting HTML5 API (could be internal)",Ica,[mb.normal(gd.handleInternal(b.intraFlag)),mb.normal(ru.fromConfigIfExternal(a,b)),mb.normal(gd.nativeHtml(c,d,b.intraFlag)),mb.normal(Hx(b)),
mb.normal(Nk())],!0)},text:function(){return cg("Plain text pasting",Jca,[mb.normal(gd.plain),mb.normal(gd.autolink)],!0)},onlyText:function(){return cg("Only plain text is available to paste",Kca,[mb.normal(gd.plain),mb.normal(gd.autolink)],!0)},none:function(){return cg("There is no valid way to paste",h.some,[mb.normal(gd.none)],!1)},fallback:function(a,b,c,d,e,f){return cg("Outside of Textbox.io pasting offscreen (could be internal)",Gca,[mb.normal(QU(d,c,e,b.intraFlag,Kaa)),mb.normal(ru.fromConfigIfExternal(a,
b)),mb.normal(gd.external(e,b.intraFlag)),mb.blocking(Lx(f,b)),mb.normal(Hx(b)),mb.normal(Nk())],!1)}},cN=function(a){return q.fromHtml("<p>"+a("cement.dialog.flash.press-escape")+"</p>")},xu={paste:function(a){var b=q.fromTag("div");n.add(b,Ec.resolve("flashbin-helpcopy"));var c;c=Y.detect().os.isOSX()?["\u2318"]:["Ctrl"];var d=q.fromHtml("<p>"+a("cement.dialog.flash.trigger-paste")+"</p>");c=q.fromHtml('<div><span class="ephox-polish-help-kbd">'+c+'</span> + <span class="ephox-polish-help-kbd">V</span></div>');
n.add(c,Ec.resolve("flashbin-helpcopy-kbd"));K.append(b,[d,c,cN(a)]);return b},noflash:function(a){var b=q.fromTag("div");n.add(b,Ec.resolve("flashbin-helpcopy"));var c=q.fromHtml("<p>"+a("cement.dialog.flash.missing")+"</p>");K.append(b,[c,cN(a)]);return b},indicator:function(a){var b=q.fromTag("div");n.add(b,Ec.resolve("flashbin-loading"));var c=q.fromTag("div");n.add(c,Ec.resolve("flashbin-loading-spinner"));var d=q.fromTag("p");a=a("loading.wait");ua.set(d,a);m.set(d,"aria-label",a);K.append(b,
[c,d]);return b}},UU=Y.detect(),WU=function(a,b,c,d){b=xu.noflash(d);v.append(a,b);return{reset:f.noop}},VU=function(a,b,c,d){var e=xu.paste(d),f=xu.indicator(d);K.append(a,[f,e,b.element()]);a=function(){t.set(e,"display","block");t.set(f,"display","none");c()};b.events.spin.bind(function(){t.set(e,"display","none");t.set(f,"display","block");t.remove(f,"height");t.remove(f,"padding");c()});b.events.reset.bind(a);b.events.hide.bind(function(){t.setAll(f,{height:"0",padding:"0"})});return{reset:a}},
ZU={responsive:function(){var a=wa(h.none()),b=D.create({crashed:s([]),timeout:s([])});return{start:function(c,d,e,f){var k=d,m=setInterval(function(){e()?clearInterval(m):0>=k?(b.trigger.timeout(),clearInterval(m)):f()&&(clearInterval(m),b.trigger.crashed());k--},c);a.set(h.some(m))},stop:function(){a.get().each(function(a){clearInterval(a)})},events:b.registry}}},Ne=ez.install("ephox.flash"),op=Y.detect(),$U=h.none(),dN=Y.detect(),Lca=dN.deviceType.isiOS()||dN.deviceType.isAndroid(),Mca=function(a){var b=
dl.addScript(a+"/wordimport.js",f.identity);b.get(function(a){a.fold(function(a){console.error("Unable to load word import: ",a)},f.noop)});return{isSupported:f.constant(!0),cleanDocument:function(a,d){return b.map(function(b){return b.map(function(b){return b.cleanDocument(a,d)})})}}},Nca=f.constant({isSupported:f.constant(!1),cleanDocument:f.constant(jj.error("not supported"))}),Oca=Lca?Nca:Mca,Pca="b i u sub sup strike".split(" "),Qca=function(a){k.each(y.children(a),function(a){A.isElement(a)&&
!a.dom().hasChildNodes()&&k.contains(Pca,A.name(a))&&B.remove(a)})},Rca=function(a,b){var c=y.children(a);k.each(c,function(a){if(b(a)){var c=y.children(a),f=q.fromTag("div",y.owner(a).dom());K.append(f,c);v.before(a,f);B.remove(a)}})},eV={consolidate:function(a,b){y.nextSibling(a).filter(b).each(function(b){var d=y.children(b);K.append(a,d);B.remove(b)});Rca(a,b);Qca(a)}},Sx=lb.create("ephox-sloth").resolve("bin"),Tx=Sx+La.generate(""),dV=rd.onDirection("-100000px","100000px"),Sca=function(a){return{startContainer:a.start,
startOffset:a.soffset,endContainer:a.finish,endOffset:a.foffset}},hV={set:function(a,b){Fa.setExact(a,b.startContainer(),b.startOffset(),b.endContainer(),b.endOffset())},get:function(a,b){return Fa.getExact(a).map(Sca)}},mV=function(a,b,c,d){var e=jV(a.uiContainer(),d),g=function(a){b.filterDom(b.paste(),a)},h=kV(u,fi.intraFlag(),c);return function(a){var b=Oca(h.baseUrl),c=f.curry(aV,e),d=gV(void 0!==h.pasteBinAttrs?h.pasteBinAttrs:{}),b=[Df.onlyText(),Df.wordimport(b,e,h,c),Df.nativeHtml(e,h,g,
a),Df.image(h)];a=Df.fallback(e,h,g,d,a,c);return Dx(b,a)}},qV=function(a){for(var b=[],c=a.start.charCodeAt(0);c<=a.finish.charCodeAt(0);c++)b.push(String.fromCharCode(c));return b},Oe=function(a){return{start:a,finish:a}},tV={controls:function(a){var b=a+"-";a=P.div();n.add(a,x.resolve(b+"dialog-controls"));var c=Wc.button(b,"cancel","button.cancel"),d=Wc.button(b,"insert","specialchar.insert");Wc.makeNormal(c.element());Wc.makeDefault(d.element());K.append(a,[c.element(),d.element()]);b=D.create({insert:M([],
d.events.click),cancel:M([],c.events.click)});return{element:f.constant(a),ok:f.constant(d),cancel:f.constant(c),destroy:function(){d.destroy();c.destroy()},events:b.registry}}},Tca=function(a){var b=q.fromTag("span");n.add(b,x.resolve("special-char"));var c=q.fromHtml(a);m.setAll(b,{"data-ephox-polish-special-char":a,"data-ephox-polish-char-code":a.charCodeAt(0).toString(16),tabindex:"-1"});Af.gridcell(b);v.append(b,c);return b},Uca=function(a,b){var c=q.fromTag("div");ea.add(c,[x.resolve("special-char-row"),
x.resolve("special-char-row-"+b)]);Af.row(c);k.each(a,function(a){v.append(c,Tca(a))});return c},Vca=function(a,b,c){var d=q.fromTag("div");n.add(d,x.resolve(b));Af.base(d,u.translate("specialchar.grid"));a=k.chunk(a,c);a=k.map(a,function(a){return Uca(a,b)});K.append(d,a);return d},sV={generateItems:function(a){var b=600>=Math.min(window.innerWidth,window.innerHeight)?12:16;return W.mapToArray(a,function(a,d){var e=Vca(a,d,b);return yf(u.translate("specialchar."+d),Uw(e))})}},uV=function(a){var b=
{item:f.constant(".ephox-pastry-statebutton.ephox-pastry-tablayout-item")},c=pa.flow({trigger:qb.button,executeOnArrow:!0});return ba.flow(a.menu(),b,c)},Wca=function(a){return eb.closest(a,".ephox-pastry-tablayout-active")},vV=function(a){var b={row:f.constant(".ephox-pastry-tablayout-active ."+x.resolve("special-char-row")),cell:f.constant(".ephox-pastry-tablayout-active ."+x.resolve("special-char"))},c=pa.tabular({trigger:qb.click,isValid:Wca});return ba.tabular(a.body(),b,c)},eN=ma.generate([{single:["element"]},
{range:["start","soffset","finish","foffset"]}]),Pk={cata:function(a,b,c){return a.fold(b,c)},single:eN.single,range:function(a,b,c,d){var e=Bc.after(c,d,a,b),f=e?S.point(a,b):S.point(c,d);a=e?S.point(c,d):S.point(a,b);return eN.range(f.element(),f.offset(),a.element(),a.offset())}},Ux=C.immutable("suggestions","ignore"),xg={clean:function(a){var b=RegExp(Ua.zeroWidth(),"g"),c=RegExp("[\u2018\u2019]","g");return a.replace(b,"").replace(c,"'")}},rp={configure:function(a){a.add(a.internal(),"ephox-cram-annotation-wrap",
Ca.unwrap);a.add(a.output(),"ephox-cram-annotation-wrap",Ca.unwrap);a.add(a.paste(),"ephox-cram-annotation-wrap",Ca.unwrap);a.addExcludedContext([".ephox-cram-annotation-wrap"])},isWrap:function(a){return n.has(a,"ephox-cram-annotation-wrap")}},Xca={en_au:"en_gb",en_bz:"en_gb",en_ca:"en_gb",en_gb:"en_gb",en_ie:"en_gb",en_in:"en_gb",en_jm:"en_gb",en_my:"en_gb",en_nz:"en_gb",en_ph:"en_gb",en_sg:"en_gb",en_tt:"en_gb",en_za:"en_gb",en_zw:"en_gb",en_us:"en_us",en:"en_us"},Yca="da de en en_au en_br en_gb en_uk en_us es fi fr it nb nl pt pt_pt sv".split(" "),
yc=function(a,b){return{value:f.constant(a),textkey:f.constant(b),icon:h.none}},Zca=function(a){var b=Z.merge(gq.localeList(),Xca);return gq.determineFrom(a,b).filter(function(a){return k.contains(Yca,a)})},ke={determine:function(a){return"no.language"===a?h.some(a):Zca(a)},get:function(){return[yc("no.language",u.key("spelling.none")),yc("en_us",u.key("parlance.en_us")),yc("en_gb",u.key("parlance.en_gb")),yc("da",u.key("parlance.da")),yc("de",u.key("parlance.de")),yc("es",u.key("parlance.es")),yc("fi",
u.key("parlance.fi")),yc("fr",u.key("parlance.fr")),yc("it",u.key("parlance.it")),yc("nb",u.key("parlance.no")),yc("nl",u.key("parlance.nl")),yc("pt",u.key("parlance.pt_br")),yc("pt_pt",u.key("parlance.pt_pt")),yc("sv",u.key("parlance.sv"))]},isOn:function(a){return"no.language"!==a},offKey:f.constant("no.language")},BV=function(a,b){var c=ke.get();return{"spelling.dropdown":{name:"spellcheck",textkey:u.key("spelling.menu"),menuTextkey:"",updateDropdown:function(b){b.populate([{label:h.none,items:f.constant(c)}]);
var e=a.profile().spelling().get();oa.on(e).fold(b.untick,f.noop,b.tick)},action:function(c){a.profile().spelling().setAndCache(c);aa.startsWith(c,"en")||a.profile().autocorrect().set(!1);b.trigger.change(c)},values:c,render:h.none(),describe:function(a){return u.key("spelling.menu")},hasPopup:!0,toggle:!1}}},CV=function(a,b,c){var d=a.profile().spelling().get()!==ke.offKey();return{"spelling.general":{toggle:!0,value:"spellcheck",textkey:u.key("cog.spellcheck"),action:function(){var e=(d=!d)?b:ke.offKey();
a.profile().spelling().setAndCache(e);c.trigger.change(e)},updateGeneral:function(a){d?a.select():a.deselect()},icon:!1}}},EV=C.immutable("area","f"),Gj={stop:f.constant("trampoline.stop"),run:function(a){for(;"trampoline.stop"!==a;)a=a()}},uh=ma.generate([{inline:["item","mode","lang"]},{text:["item","mode"]},{empty:["item","mode"]},{boundary:["item","mode","lang"]},{concluded:["item","mode"]}]),fN=function(a,b,c,d){var e=a.property().isElement(b)?h.from(a.attrs().get(b,"lang")):h.none();return a.property().isText(b)?
uh.text(b,c):d(b,c)?uh.concluded(b,c):a.property().isBoundary(b)?uh.boundary(b,c,e):a.property().isEmptyTag(b)?uh.empty(b,c):uh.inline(b,c,e)},$ca=function(a,b,c,d){return na.walk(a,b,c,na.walkers().right(),void 0).fold(function(){return uh.concluded(b,c)},function(b){return fN(a,b.item(),b.mode(),d)})},gN=function(a,b,c,d,e,f){return function(){return b.fold(function(b,h,k){(h===na.advance?d.openInline:d.closeInline)(k,b);return Hj(a,b,h,c,d,e,f)},function(b,h){var k=e(a,b);d.addDetail(k);return c(b,
h)?Gj.stop():Hj(a,b,h,c,d,e,f)},function(b,h){d.addEmpty(b);return Hj(a,b,h,c,d,e,f)},function(b,h,k){var m=f.assess(b);return xf.cata(m,function(k){return h!==na.backtrack?Hj(a,b,na.sidestep,c,d,e,f):Gj.stop()},function(m){(h===na.advance?d.openBoundary:d.closeBoundary)(k,b);return Hj(a,b,h,c,d,e,f)},function(a){return Gj.stop()})},function(a,b){return Gj.stop()})}},Hj=function(a,b,c,d,e,f,h){b=$ca(a,b,c,d);return gN(a,b,d,e,f,h)},ada={walk:function(a,b,c,d,e,f){var h=function(b,d){return a.eq(b,
c)&&(d!==na.advance||a.property().isText(b)||0===a.property().children(b).length)},k=ah.nu(d),m=fN(a,b,na.advance,h);Gj.run(function(){return gN(a,m,h,k,e,f)});return k.done()}},bda=C.immutableBag(["elements","lang","words"],[]),cda={fromWalking:function(a,b){var c=k.map(b,function(a){var b=a.details();a=a.lang();var c=k.map(b,function(a){return a.text()}).join(""),b=k.map(b,function(a){return a.item()}),c=RI.words(c);return bda({lang:a,words:c,elements:b})});return{zones:f.constant(c)}}},dda=function(a,
b,c,d,e,f){return(a.eq(b,c)?h.some(b):a.property().parent(b)).map(function(h){h=ah.calculate(a,h).getOr(d);return ada.walk(a,b,c,h,e,f)})},Sn=function(a,b,c,d,e,f){b=Ic.subset(a,b,c).bind(function(b){return 0===b.length?h.none():dda(a,b[0],b[b.length-1],d,e,f)}).getOr([]);return cda.fromWalking(a,b)},yu=function(a,b){return function(c,d){return c.eq(d,a.item())?a:c.eq(d,b.item())?b:te.detail(c,d)}},hN=function(){return{zones:f.constant([])}},dg={fromRange:function(a,b,c,d,e){b=aj.getEdges(a,b,c,f.constant(!1));
c=yu(b.left(),b.right());return Sn(a,b.left().item(),b.right().item(),d,c,e)},transformEdges:yu,fromBounded:function(a,b,c,d,e){return Sn(a,b,c,d,te.detail,e)},fromBoundedWith:Sn,fromInline:function(a,b,c,d){b=aj.byBoundary(a,b);var e=yu(b.left(),b.right());return b.isEmpty()?hN():Sn(a,b.left().item(),b.right().item(),c,e,d)},empty:hN},jN=function(a,b,c,d){return a.property().isBoundary(b)?dg.fromBounded(a,b,b,c,d):a.property().isEmptyTag(b)?iN():dg.fromInline(a,b,c,d)},iN=function(){return dg.empty()},
zu={single:jN,range:function(a,b,c,d,e,f,h){b=mc.toLeaf(a,b,c);d=mc.toLeaf(a,d,e);return a.eq(b.element(),d.element())?jN(a,b.element(),f,h):dg.fromRange(a,b.element(),d.element(),f,h)},empty:iN},kN=Sa(),Qk={single:function(a,b,c){return zu.single(kN,a,b,c)},range:function(a,b,c,d,e,f){return zu.range(kN,a,b,c,d,e,f)},empty:function(){return zu.empty()}},Vx={update:function(a,b,c){var d=W.keys(c);k.each(d,function(d){a.add(b,d,c[d])});return d},getInCache:function(a,b,c){return k.filter(c,function(c){return a.get(b,
c.word()).fold(f.constant(!1),function(a){return a.suggestions().isSome()})})}},eda=f.constant({assess:xf.inView})(),Au=function(a,b,c,d,e,f){a=dg.fromBoundedWith(a,b,c,d,f,eda).zones();1===a.length?(a=a[0],e=a.lang()===e?h.some(a):h.none()):e=h.none();return e},lN=function(a,b,c,d,e){return Au(a,b,c,d,e,te.detail)},Tn={fromRange:function(a,b,c,d,e){var f=ah.strictBounder(d,e);b=aj.getEdges(a,b,c,f);c=dg.transformEdges(b.left(),b.right());return Au(a,b.left().item(),b.right().item(),d,e,c)},fromBounded:lN,
fromInline:function(a,b,c,d){var e=ah.strictBounder(c,d),e=aj.getEdges(a,b,b,e),f=dg.transformEdges(e.left(),e.right());e.isEmpty()?(f=a.property().isText(b)?a.property().getText(b):a.property().children(b).length,e=mc.toLeaf(a,b,0),b=mc.toLeaf(a,b,f),a=lN(a,e.element(),b.element(),c,d)):a=Au(a,e.left().item(),e.right().item(),c,d,f);return a},empty:function(){return h.none()}},mN=function(a,b,c,d){return a.property().isBoundary(b)?Tn.fromBounded(a,b,b,c,d):a.property().isEmptyTag(b)?Tn.empty():Tn.fromInline(a,
b,c,d)},nN={single:mN,range:function(a,b,c,d,e,f,h){b=mc.toLeaf(a,b,c);d=mc.toLeaf(a,d,e);return a.eq(b.element(),d.element())?mN(a,b.element(),f,h):Tn.fromRange(a,b.element(),d.element(),f,h)}},oN=Sa(),fda={single:function(a,b,c){return nN.single(oN,a,b,c)},range:function(a,b,c,d,e,f){return nN.range(oN,a,b,c,d,e,f)}},pN=function(a){return k.bind(a,function(a){return pc.descendants(a,A.isText)})},Un={textnodes:pN,text:function(a){a=pN(a);return k.map(a,function(a){return A.isText(a)?Da.get(a):""}).join("")},
parsed:function(a,b,c){return h.from(a[0]).bind(function(a){return fda.single(a,b,c).bind(function(a){a=a.words();return 1===a.length?h.some(a[0]).map(function(a){return a.word()}):h.none()})})},children:function(a){return k.bind(a,y.children)}},gda={clobber:function(a,b){var c=Un.children(a),d=Un.textnodes(a);return h.from(a[0]).bind(function(e){K.before(e,c);e=h.from(d[0]);e.each(function(a){Da.set(a,b)});k.each(d.slice(1),function(a){Da.set(a,"")});k.each(a,B.remove);return e})}},qN=Ht.resolve("annotation"),
rN=Ht.resolve("lingo"),sN=It.resolve("annotation-wrap"),vh={nu:function(){var a=q.fromTag("span");n.add(a,sN);m.set(a,"aria-invalid","spelling");return Yg.nu(a)},set:function(a,b,c){m.set(a,qN,c);m.set(a,rN,b)},label:f.constant(sN),detail:function(a){return lf.id(a).bind(function(b){return h.from(m.get(a,qN)).bind(function(c){return h.from(m.get(a,rN)).map(function(a){return{id:f.constant(b),word:f.constant(c),lang:f.constant(a)}})})})}},hda=It.resolve("wrap-disabled"),ida=function(a){a=lf.all(a);
k.each(a,function(a){n.remove(a,hda)})},jda=C.immutable("id","elements","word","lang"),tN=function(a){return{fold:a}},uN={success:function(a){return tN(function(b,c){c(a)})},fail:function(a){return tN(function(b,c){b(a)})}},kda=function(a,b){var c={};k.each(a,function(a){lf.marker(a,b.label()).filter(ca.inBody).each(function(a){b.detail(a).each(function(b){var d=b.id(),f=b.word();b=b.lang();var h=lf.all(a);0<h.length&&(c[d]=jda(d,h,f,b))})})});return c},vN=function(a,b,c,d){var e=Un.text(a);a=Un.parsed(a,
c,d);return e===b&&a.is(b)},lda=function(a,b,c){a=W.values(a);return k.map(a,function(a){return a.lang()===c&&vN(a.elements(),a.word(),b,a.lang())?uN.success(a):uN.fail(a)})},mda={validate:vN,verify:function(a,b,c,d,e){b=kda(a.elements(),b);a=lda(b,e,a.lang());k.each(a,function(a){var b=function(a){return function(b){k.each(b.elements(),a)}};a.fold(b(d),b(c))})}},wN=function(a){return lf.marker(a,vh.label())},qp={highlight:function(a,b,c,d){mda.verify(b,vh,ida,B.unwrap,a.lang());c=vj.matchWords(b.elements(),
c,d);c=vj.annotates(c,vh.label());return k.bind(c,function(c){return c.current().fold(function(){var d=c.match(),f=d.word();if(a.ignore(d))return[];d=Yg.wrapper(d.elements(),vh.nu)||[];k.each(d,function(a){lf.set(a,c.id());vh.set(a,b.lang(),f)});return[xn.created(d)]},function(a){return[xn.preserved(a)]})})},marker:wN,substitute:function(a,b){var c=wN(a).fold(f.constant([]),lf.all);return gda.clobber(c,b)},detail:function(a){return vh.detail(a)}},HV={highlight:function(a,b,c,d){var e={operation:a,
internal:a,ignore:f.constant(!1),lang:f.constant(d)},g=k.map(c,function(a){return a.word()}),h=k.filter(g,function(a,b){return k.indexOf(g,a).filter(function(a){return a===b}).isSome()});e.operation(function(){qp.highlight(e,b,h,f.constant(!1))})}},nda=function(a,b){return za.nu(function(c){var d={words:k.map(a.words(),function(a){return xg.clean(a.word())})};b.correction(d,a.lang(),function(b){b=H.value({spelling:f.constant(b),zone:f.constant(a)});c(b)},function(a){a=H.error(Error(a));c(a)})})},
IV={scan:function(a,b,c){var d=function(b){var c=b.zone(),d=Vx.update(a,c.lang(),b.spelling());b=k.filter(c.words(),function(a){a=xg.clean(a.word());return k.contains(d,a)});return{zone:f.constant(c),words:f.constant(b)}};c=k.map(c,function(a){return nda(a,b)});return Nc.par(c).map(function(a){a=k.bind(a,function(a){return a.fold(f.constant([]),k.pure)});return k.map(a,d)})}},Xx={active:function(a){var b={};a.autocorrect().get(function(a){a.fold(function(a){console.log("Error loading autocorrect list")},
function(a){b=a.corrections})});return{lookup:function(a){return b.hasOwnProperty(a)?h.from(b[a]):h.none()}}},inactive:function(){return{lookup:h.none}}},oda=wc.objOf([xc.strict("language"),xc.strict("words")]),KV={many:f.constant(oda)},pda=function(a,b){return ie.readOptFrom(a,b).getOrThunk(function(){return gk()})},Bu=La.generate("good-spelling"),Wx={execute:function(a,b,c,d,e,g){var h=pda(b,c);g=ik.multiple(function(b,d){return a.execute({language:c,words:b})},h,d,function(a){return a.spell},f.identity,
e,void 0!==g?g:!1);b[c]=h;return Yd.mapSync(g,function(a){k.each(d,function(b){ie.hasKey(a,b)||h.set(b,Bu,e)});var b={};W.each(a,function(a,c){h.get(e,c).is(Bu)||(b[c]=a)});return b})},initCaches:function(a){var b={};W.each(void 0!==a?a:{},function(a,d){b[d]=gk(a)});return b},correct:f.constant(Bu)},qda={image:f.constant("1"),spelling:f.constant("1")},MV={corsQuery:function(a){return aa.ensureTrailing(a,"/")+"version"},spelling:function(a){var b=qda.spelling();return aa.ensureTrailing(a,"/")+b+"/"}},
Ij=function(a){a="data-"+x.resolve(a);return f.constant(a)},zd={value:Ij("spelling-value"),type:Ij("spelling-type"),overflow:Ij("spelling-overflow"),overflowMenu:Ij("spelling-overflow-menu"),spelling:Ij("spelling-suggestion")},xN=C.immutable("primary","secondary","submenus"),rda=C.immutable("value","text","attrs"),sda=C.immutable("textkey","value","items"),RV={generate:function(a,b){var c;if(a.length>b){c=a.slice(0,b);var d=rda(zd.overflow(),u.translate("spelling.context.more"),{}),e=a.slice(b),f=
{};f[zd.overflow()]=zd.overflowMenu();e=sda(u.key("spelling.context.morelabel"),zd.overflowMenu(),e);c=xN(c.concat([d]),[e],f)}else c=xN(a,[],{});return c}},SV=4,QV=C.immutable("value","text","attrs"),Zx=Y.detect().browser,WV=C.immutableBag("communication profile content undo viewable cache url autocorrectOn language fromServiceState multilingual".split(" "),[]),yN=function(a){a=q.fromTag(a);var b=q.fromText(Ua.zeroWidth());v.append(a,b);n.add(a,jb.styleSpan());return a},zN=function(a,b,c,d){var e=
yN(d);y.firstChild(e).each(function(d){a(c,e);b.setSelection(d,0,d,0)})},AN={nu:yN,insertAfter:f.curry(zN,v.after),insertBefore:f.curry(zN,v.before)},BN=Y.detect().browser,CN=function(a,b){return Ja.toLeaf(a,b)},tda=function(a){return A.isText(a)&&0!==Da.get(a).trim().length},uda=function(a){return A.isElement(a)&&"br"===A.name(a)},vda=function(a){return k.exists([tda,uda],function(b){return b(a)})},wda=function(a){return A.isText(a.element())&&0===a.offset()||A.isElement(a.element())},xda=function(a,
b,c){var d=f.constant(a);return(!1===b&&wda(a)?h.some(!0):h.none()).fold(d,function(b){return Lb.seekLeft(a.element(),vda,c).fold(d,function(a){return S.point(a,A.isText(a)?Da.get(a).length:0)})})},yda=function(a,b){return L.eq(a.element(),b.element())},le={context:function(a){return a.getSelection().bind(function(b){if(b.collapsed()){var c=b.startContainer();return Ya.cell(c,a.isRoot).bind(function(d){return Ya.table(c,a.isRoot).map(function(a){return{cell:f.constant(d),table:f.constant(a),start:f.constant(c),
selection:f.constant(b)}})})}return h.none()})},selectedCells:function(a,b){return tb.cata(a.get(),f.constant([]),f.identity,function(a){var d=BN.isFirefox()?CN(a.startContainer(),a.startOffset()):S.point(a.startContainer(),a.startOffset()),e=BN.isFirefox()?CN(a.endContainer(),a.endOffset()):S.point(a.endContainer(),a.endOffset());a=xda(e,a.collapsed()||yda(d,e),b);return be.sharedOne(function(a){return Ya.cell(a,b)},[d.element(),a.element()]).toArray()})}},Vn=function(a){return{fold:a}},wh={none:function(a){return Vn(function(b,
c,d,e){return b(a)})},first:function(a){return Vn(function(b,c,d,e){return c(a)})},middle:function(a,b){return Vn(function(c,d,e,f){return e(a,b)})},last:function(a){return Vn(function(b,c,d,e){return e(a)})}},DN=function(a,b){return Ya.table(a,b).bind(function(b){var d=Ya.cells(b);return k.findIndex(d,function(b){return L.eq(a,b)}).map(function(a){return{index:f.constant(a),all:f.constant(d)}})})},Wn={next:function(a,b){return DN(a,b).fold(function(){return wh.none(a)},function(b){return b.index()+
1<b.all().length?wh.middle(a,b.all()[b.index()+1]):wh.last(a)})},prev:function(a,b){return DN(a,b).fold(function(){return wh.none()},function(b){return 0<=b.index()-1?wh.middle(a,b.all()[b.index()-1]):wh.first(a)})}},ey={upOrLeft:function(a){le.context(a).each(function(b){Wn.prev(b.cell()).fold(f.noop,function(){Qh.isFirstInContent(a,b.table())&&wf.isLeft(b.cell(),b.selection())&&AN.insertBefore(a,b.table(),"p")},f.noop,f.noop)})},right:function(a){le.context(a).each(function(b){Wn.next(b.cell()).fold(f.noop,
f.noop,f.noop,function(){Qh.isLastInContent(a,b.table())&&wf.isRight(b.cell(),b.selection())&&AN.insertAfter(a,b.table(),"p")})})}},nW={widget:function(a){var b=a.content(),c=C.immutable("value","icon","textkey"),d=c("aligntop",h.some(ja.resolve("aligntop")),u.key("table.cell.alignment.top")),e=c("alignmiddle",h.some(ja.resolve("alignmiddle")),u.key("table.cell.alignment.middle")),c=c("alignbottom",h.some(ja.resolve("alignbottom")),u.key("table.cell.alignment.bottom")),g=[d,e,c],d=function(a){return function(b){t.set(b,
"vertical-align",a)}},l={aligntop:d("top"),alignmiddle:d("middle"),alignbottom:d("bottom")},m=h.none(),n=function(a,b){a.enable();return t.getRaw(b,"vertical-align").fold(function(){return oa.on("aligntop")},function(a){return"middle"===a?oa.on("alignmiddle"):"bottom"===a?oa.on("alignbottom"):oa.on("aligntop")})};return{name:"cell-alignment",textkey:u.key("table.cell.alignment.toolbar"),menuTextkey:u.key("table.cell.alignment.toolbar"),action:function(c){b.focus();var d=le.selectedCells(a.selections(),
b.isRoot);k.each(d,l[c]);b.uiChanged()},updateDropdown:function(c){var d=le.selectedCells(a.selections(),b.isRoot);0<d.length?d=n(c,d[0]):(c.disable(),d=oa.off());c.populate([{label:h.none,items:f.constant(g)}]);Yj.updateMenu(c,d,oa.on(),g,m)},values:g,render:m,tooltip:Wa.get,toggle:!1,describe:function(a){return u.translate("alignment.describe",u.get(a.textkey()))}}}},zda={cssNamespace:"table-cell-border",i18nNamespace:"table.cell.border",property:"border-width",tag:"table",specials:[["thin","medium",
"thick"]]},mW={widget:function(a){var b=a.content(),c=Bk(zda);c.initLayout();Q.dialog(c.element(),u.translate("aria.table.cell.border.size"));var d=D.create({done:s([]),abort:M([],c.events.abort)});c.events.close.bind(function(a){d.trigger.done();b.focus()});c.events.action.bind(function(c){var d=T.active();b.focus();var e=le.selectedCells(a.selections(),b.isRoot);k.each(e,function(a){t.set(a,"border-width",c.value());t.getRaw(a,"border-style").fold(function(){t.set(a,"border-style","solid")},f.noop)});
d.each(T.focus)});var e=function(){c.enter();c.open()};return{element:c.element,destroy:c.destroy,name:f.constant("cellborderwidth"),textkey:f.constant(u.key("table.cell.border.width")),tooltip:Wa.get,label:f.constant(!1),updateWidget:function(d){var h=le.selectedCells(a.selections(),b.isRoot);0<h.length?(d.enable(),d=t.getRaw(h[0],"border-width").fold(oa.off,oa.on),c.updateInlineWidget({size:f.constant(d)},"1px")):d.disable();e()},events:d.registry,open:e,close:c.close}}},Ada=C.immutableBag(["name",
"textkey","menuTextkey","action","shortname"],[]),cy={widget:function(a,b){var c=Ada(b),d=a.content(),e=P.div();Q.label(e,u.translate(c.textkey()));ea.add(e,[x.resolve("palette-cell-colors"),x.resolve(c.shortname()+"-cell-colors")]);var g=Wh.create();v.append(e,g.element());var h="."+x.resolve(c.shortname()+"-cell-colors")+" ."+x.resolve("palette-detail-action"),k=Wh.makeLayout(e,h,{}),h=k.layout(),m=D.create({done:s([]),abort:s([])});g.events.selected.bind(function(b){b=b.value();var e=le.selectedCells(a.selections(),
d.isRoot);c.action()(e,b);m.trigger.done()});var n=function(){q.enter();k.refresh()},q=lv(e,[h]);q.events.exit.bind(function(){m.trigger.abort()});return{element:f.constant(e),destroy:function(){q.destroy();g.destroy()},name:f.constant(c.name()),textkey:f.constant(u.key(c.textkey())),tooltip:Wa.get,label:f.constant(!1),updateWidget:function(b){Ah.isHighContrast(a.smartview())?b.disable():(0===le.selectedCells(a.selections(),d.isRoot).length?b.disable():b.enable(),n())},events:m.registry,open:n,close:function(){m.trigger.done()}}}},
Bda={styles:{"border-collapse":"collapse",width:"100%"},attributes:{border:"1"},percentages:!0},Cda=function(a,b,c,d,e){void 0===e&&(e=Bda);var f=q.fromTag("table");t.setAll(f,e.styles);m.setAll(f,e.attributes);var h=q.fromTag("tbody");v.append(f,h);for(var k=[],n=0;n<a;n++){for(var s=q.fromTag("tr"),u=0;u<b;u++){var x=n<c||u<d?q.fromTag("th"):q.fromTag("td");u<d&&m.set(x,"scope","row");n<c&&m.set(x,"scope","col");v.append(x,q.fromTag("br"));e.percentages&&t.set(x,"width",100/b+"%");v.append(s,x)}k.push(s)}K.append(h,
k);return f},Dda={insert:function(a,b,c,d,e){return a.getSelection().bind(function(f){var k=Cda(b,c,d,e);Cc.insertAtCursor(a,[k]).each(function(b){Ya.firstCell(k).each(function(b){a.setSelection(b,0,b,0)})});return h.some(k)})}},Eda=Fc.resolve("table-picker"),Fda=Fc.resolve("table-picker-row"),Gda=Fc.resolve("table-picker-cell"),Hda=Fc.resolve("table-picker-button"),Ef={table:f.constant(Eda),row:f.constant(Fda),cell:f.constant(Gda),button:f.constant(Hda)},Ida="."+Ef.cell(),Jda="."+Ef.row(),Pe={cells:function(a){return X.descendants(a,
Ida)},rows:function(a){return X.descendants(a,Jda)},grid:function(a,b,c){b=parseInt(m.get(a,b),10);a=parseInt(m.get(a,c),10);return xa.grid(b,a)},button:function(a){return J.child(a,"."+Ef.button()).getOr(a)}},EN=function(a,b,c,d,e){d=Math.max(d,Math.min(e,a.row()));a=Math.max(b,Math.min(c,a.column()));return xa.address(d,a)},FN=function(a,b){var c=EN(a,1,b.maxCols,1,b.maxRows),d;d=xa.address(c.row()+1,c.column()+1);d=EN(d,b.minCols,b.maxCols,b.minRows,b.maxRows);return{selection:f.constant(c),full:f.constant(d)}},
ay={resize:function(a,b){var c;c=xa.address(a.row()+1,a.column()+1);return FN(c,b)},grow:function(a,b,c,d){a=xa.address(a.rows()+c,a.columns()+b);return FN(a,d)}},GN=function(a,b,c,d){var e=a.width()/b.columns();a=a.height()/b.rows();return xa.address(Math.floor(d/a),Math.floor(c/e))},Kda={pickerCell:function(a,b,c,d){var e=d.x()-a.x();a=d.y()-a.y();return GN(b,c,e,a)},isRtl:f.constant(!1)},Lda={pickerCell:function(a,b,c,d){var e=a.x()+b.width()-d.x();a=d.y()-a.y();return GN(b,c,e,a)},isRtl:f.constant(!0)},
lW={widget:function(a,b){var c=a.content(),d=aW(a.region().uiDirection().isRtl()?Lda:Kda,b),e=a.undo(),g=D.create({done:M([],d.events.done),abort:M([],d.events.abort)});d.events.select.bind(function(a){e.save();Dda.insert(c,a.rows(),a.cols(),a.rowHeaders(),a.columnHeaders()).each(function(a){e.save();c.domChanged()});c.toEditing()});return{element:d.element,name:f.constant("table"),textkey:f.constant(u.key("insert.table")),menuTextkey:f.constant(u.key("table.menu")),tooltip:Wa.get,label:f.constant(!1),
updateWidget:function(a){c.getSelection().each(function(b){a.enable()});d.open()},destroy:d.destroy,events:g.registry,open:d.open,close:d.close}}},gi={set:function(a,b){a.dom().checked=b},find:function(a){return J.descendant(a,"input:checked")}},HN=C.immutableBag(["floatval","caption","summary","border","dimensions"],[]),Mda=function(a){a=m.get(a,"border");return"0"===a||"1"===a?h.some(a):h.none()},Nda=function(a){var b=t.getRaw(a,"width");a=t.getRaw(a,"height");return{width:f.constant(b),height:f.constant(a)}},
sp={divine:function(a,b){var c=Vr.divine(b),d=m.get(b,"summary"),e=J.child(b,"caption").map(function(b){b=ua.get(b);return a.filterHtml(a.output(),b)}).getOr(""),f=Mda(b),h=Nda(b);return HN({floatval:c,caption:e,summary:d,border:f,dimensions:h})},properties:HN,apply:function(a,b){b.floatval().each(function(b){Rf.runFloat(a,b)});b.border().each(function(b){m.set(a,"border",b)});""===b.summary()?m.remove(a,"summary"):Be.setSummary(a,b.summary());""===b.caption()?J.child(a,"caption").each(B.remove):
Be.setCaption(a,b.caption());Be.setDimensions(a,b.dimensions())}},tp={halve:function(a,b){Ib.getGenericWidth(a).each(function(c){var d=c.width()/2;Ib.setGenericWidth(a,d,c.unit());Ib.setGenericWidth(b,d,c.unit())})}},IN=function(a,b,c,d){return c.fold(h.none,h.none,function(a,b){return Cb.first(b).bind(function(a){return h.some(cc(a,0,a,0))})},function(c){return Ya.table(c,b.isRoot).bind(function(g){Nb.execute(a,Fb.insertRowAfter,f.noop,d)(b,g,Rk.noMenu(c));g=X.descendants(g,"tr");if(0<g.length)return J.descendant(g[g.length-
1],"td,th").map(function(a){return cc(a,0,a,0)})})})},dy={forward:function(a,b,c,d){return IN(a,b,Wn.next(c),d)},backward:function(a,b,c,d){return IN(a,b,Wn.prev(c),d)}},by={setBorderColor:function(a,b){t.set(a,"border-color",b);t.getRaw(a,"border-style").fold(function(){t.set(a,"border-style","solid")},f.noop)},setBackgroundColor:function(a,b){t.set(a,"background-color",b)},setHeight:function(a,b){t.set(a,"height",b)},setWidth:function(a,b){t.set(a,"width",b)},setType:function(a,b){var c=ec.copy(a);
v.after(a,c);B.remove(a)},setScope:function(a,b){m.set(a,"scope",b)},setStyle:function(a,b){m.set(a,"style",b)},setClass:function(a,b){m.set(a,"class",b)}},iW=Fc.resolve("resizer-bar-dragging"),Oda=C.immutable("withSpaces","withoutSpaces"),Pda=function(a){return k.filter(a.split(/[\.,-\/#!$%\^&\*;:{}=\-_`~()\s\u00A0]/gim),function(a){return 0<a.length}).length},fy={count:function(a){return{words:f.constant(Pda(a)),characters:f.constant(Oda(a.length,a.replace(/[\s\u00A0]/gim,"").length))}}},sW=C.immutableBag("accessibility formatting alignindent bookmark directionality table font parlance media imageEdit blocktag specialchar wordcount link linkvalidator horizontalrule help fileupload findreplace plaincodeview macros spelling".split(" "),
[]),Xn=0,Cu=h.none(),wp={add:function(){if(0===Xn){var a=q.fromTag("style");pb.set(a,'/*\n * Keep this file as small as possible.\n * It is injected into the JavaScript, so it needs to be tiny.\n */\n\n/*\n * Animate from 0 to 359, not 0 to 360, as they are the same and FireFox does nothing\n *\n * Until we have token replacement, manual shorthand for "ephox-polish-spin"\n */\n\n@keyframes e-p-spin {\n  0% {\n    transform: rotate(0deg);\n  }\n  100% {\n    transform: rotate(359deg);\n  }\n}\n\n/* Until we have token replacement, manual shorthand for "ephox-polish-spinner-img" */\n\n.e-p-s-img {\n  animation: e-p-spin 2s infinite linear;\n  width: 30px;\n  height: 30px;\n  margin: 2px 2em 0 auto;\n  background: url("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABAAAAAQBAMAAADt3eJSAAAAIVBMVEUAAABAQEBDQ0NERERERERCQkJDQ0NERERDQ0NEREREREQTPu3mAAAACnRSTlMAENDPwGBwsKB/qy4kVgAAAFpJREFUCNdjAAImAwYI4FoApooFuBYwWgAZXoZMCsJLgAyhpQwMUYpABmMiA4OYAAMCaC2GklqLwIxFMAmoYkmQYmGQdkOQgYpMBkIgA4tAVmggLAU7QwFIAADjyQ3FWhtneAAAAABJRU5ErkJggg==") 0/cover;\n}');
v.append(ca.body(),a);Cu=h.some(a)}Xn++},remove:function(){0!==Xn&&0===--Xn&&(Cu.each(B.remove),Cu=h.none())}},Qda=Ab.pencil(),Yn=Ab.code(),JN=function(a){var b=a.old().toTransfer();a.nu().fromTransfer(b)},Rda={transition:JN,factory:Tu,style:Ab.pencil},Sda={transition:JN,factory:Sk,style:Ab.code},Tk={goingToDesign:function(a){return a===Yn},alternate:function(a){return a===Yn?Qda:Yn},detect:function(a){return a===Yn?Rda:Sda}},wW=function(a,b){return{destroy:f.noop,init:f.noop,setMode:f.die("Code view is not enabled, cannot switch modes"),
events:D.create({change:s([]),ready:s([])}).registry}},vW=function(a,b){var c=b.css+"/codemirror.css",d=D.create({change:s([]),ready:s(["type"])}),e=h.none(),g=function(){var a=Nj.load.codeview(b.js).map(function(a){return a.map(f.constant(!0))});e=h.some(a)},k=function(a){a(H.value(!0))};return{init:function(a){e.fold(function(){dl.addStylesheet(c,k).get(function(b){b.fold(function(a){console.error(a);console.error("Code View disabled due to error loading stylesheet")},function(){g();d.trigger.ready(a)})})},
function(b){d.trigger.ready(a)})},setMode:function(b){return e.fold(function(){throw"Cannot set mode before Code View has loaded";},function(c){c.get(function(c){c.fold(function(a){d.trigger.ready(b);console.error(a);console.error("Code View non-functional due to error downloading script file")},function(){var c=Tk.detect(b),e=a.swap(c.factory,{},{});d.trigger.change();c.transition(e);e.destroy()})})})},events:d.registry,destroy:f.noop}},gy=Y.detect().browser,AW={build:function(a,b,c,d,e,g,h,k){var m=
yW(a,b,c,g,h,k),n=d.full(m);n.init();return{element:m.element,socket:m.socket,viewport:m.viewport,uiContainerParent:m.uiContainerParent,init:function(b,d,f){d=n.config(a.isInline(),d,e,f);vB.setup(m,g,a,b,c,d);n.loaded()},connect:m.connect,destroy:function(){m.destroy();n.destroy()},displayBanner:m.displayBanner,hideBanner:m.hideBanner,position:f.noop,refresh:m.refresh,tabstrip:m.toolbar,state:f.constant(k),dialogContainer:m.dialogContainer}},bounds:function(){return pe.view}},KN={blobTob64:function(a,
b){var c=m.get(b,Kb.blobId());m.remove(b,Kb.blobId());a.get(c).each(function(a){m.remove(b,Kb.blobId());m.set(b,"src",tc.imageResultToDataURL(a.imageresult()))})},b64ToBlob:function(a,b){var c=m.get(b,"src"),d=function(){return $h.dataUriToBlobSync(c).map(function(b){var d=tc.fromBlobAndUrlSync(b,c);b=Re.createObjectURL(b);var e=La.generate("image");return a.add(e,d,b)})};try{a.lookupByData(c).fold(d,h.some).each(function(a){m.set(b,Kb.blobId(),a.id());m.set(b,"src",a.objurl())})}catch(e){console.warn("Unable to read Base64 image data: ",
e,c)}}},NW={configure:function(a,b){var c=Kb.blobId(),d="img["+c+"]";a.addSelector(a.internal(),d,Ca.removeAttr(c));a.addSelector(a.output(),d,function(a){k.each(a,function(a){KN.blobTob64(b,a)})});a.addSelector(a.input(),"img:not(["+c+"])[src]",function(a){k.each(a,function(a){KN.b64ToBlob(b,a)})});c=Ca.removeClass(Kb.uploadInProgress());a.add(a.output(),Kb.uploadInProgress(),c);a.add(a.history(),Kb.uploadInProgress(),c);a.add(a.internal(),Kb.uploadInProgress(),c);d=Kb.trackedImage();c="img["+d+
"]";d=Ca.removeAttr(d);a.addSelector(a.output(),c,d);a.addSelector(a.internal(),c,d)},blobId:Kb.blobId},LN=C.immutable("image","blobInfo"),Du=ma.generate([{failure:["error"]},{success:["result","images","blob"]}]),Uda=function(a,b,c,d){k.each(b,function(a){m.set(a,"src",d.location);m.remove(a,Kb.blobId())});return Tda(a,c)},MN=function(a,b,c,d,e){var f=tc.imageResultToDataURL(c),f=a.lookupByData(f).getOrThunk(function(){return a.add(b,c,d)});m.set(e,Kb.blobId(),f.id());return LN(e,f)},Vda=function(a,
b){var c=m.get(b,Kb.blobId());return a.get(c).fold(function(){return H.error(c)},function(a){return H.value(LN(b,a))})},Tda=function(a,b){return a.get(b).fold(function(){return H.error("Internal error with blob cache")},function(c){a.remove(b);return H.value(c)})},ji={prepareForUpload:function(a,b,c){var d=a.isActive(b);a.register(c,b);n.add(c,Kb.uploadInProgress());return d?h.none():h.some(b)},handleUpload:function(a,b,c,d,e,g,h){var m=function(){console.error("Internal error with blob cache",e);
h(Du.failure({status:f.constant(666)}))};a.upload(g,e,function(a){var f=b.findById(d,e);k.each(f,me.removeClass(Kb.uploadInProgress()));a.fold(function(a){h(Du.failure(a))},function(a){Uda(c,f,e,a).fold(m,function(b){h(Du.success(a,f,b))})});b.report(e,f,a.isValue())})},registerAssets:function(a,b,c){return k.bind(c,function(c){return oc.cata(c,function(c,d,f){return J.descendant(b,'img[src="'+f+'"]').fold(function(){return[H.error("Image that was just inserted could not be found: "+f)]},function(b){return[H.value(MN(a,
c,d,f,b))]})},f.constant([]))})},registerBlob:MN,findBlobs:function(a,b,c){c=a.findAll(c);return a.inProgress()?[]:k.map(c,function(a){return Vda(b,a)})}},Wda=function(a){var b=function(b){var d=b.image(),e=b.blobInfo(),f=Xo.changedBounds(d),e=Pa.fromPromise(tc.imageResultToOriginalBlob(e.imageresult()));return Pa.bind(e,function(e){return Th.resampleAction(f,e,function(f){if(f===e)return Pa.pure(b);var g=he.updater(he.keepResample,d),h=Zo.applyEdit(d);f=we.single(f).bind(h).map(g);return Pa.map(f,
function(e){return oc.cata(e,function(b,c,e){return ji.registerBlob(a,b,c,e,d)},function(a,d,e){return b})})})})};return function(a){return Nc.mapM(a,b)}},Xda=function(a){a=k.map(a,H.value);return za.pure(a)},MW={instance:function(a,b){return!0===a.immediate?Xda:Wda(b)}},Yda={400:"errors.http.400",401:"errors.http.401",403:"errors.http.403",404:"errors.http.404",407:"errors.http.407",409:"errors.http.409",413:"errors.http.413",415:"errors.http.415",500:"errors.http.500",501:"errors.http.501"},Zda=
{get:function(a,b){return h.from(Yda[a]).map(function(a){return u.translate(a,b)})}},$da=function(a){return R.isString(a)&&""!==a.trim()?{description:f.constant(a),link:f.constant(null)}:ae.get("error.imageupload.generic")},aea=function(a,b){var c=b.status();return Zda.get(c,a).fold(function(){var b;b=0===c?"error.imageupload.cors":400<=c&&500>c?"error.code.images.not.found":"error.imageupload";return ae.get(b,[a])},function(a){return{description:f.constant(a),link:f.constant(null)}})},NN=function(a,
b){return{instance:function(){return hb.cached(function(c){c=b(c);a.show(c)})}}},ON={simple:function(a){return NN(a,$da)},url:function(a,b){return NN(a,f.curry(aea,b))}},bea=["jpg","png","gif","jpeg"],ii={failureObject:C.immutable("message","status","contents"),getFilename:function(a,b){var c;R.isString(a.name)&&!aa.endsWith(a.name,".tmp")?c=a.name:R.isString(a.type)&&aa.startsWith(a.type,"image/")?(c=a.type.substr(6),c=k.contains(bea,c)?b+"."+c:b):c=b;return c},buildData:function(a,b,c){var d;d=
new (Vb.getOrDie("FormData"));d.append(a,b,c);return d}},EW=C.immutable("id","filename","blob","base64"),PN={direct:function(a){return CW(a)},custom:function(a){return DW(a)}},Eu=C.immutable("uploader","errors"),LW={choose:function(a,b,c){return!0!==a.bundled?Eu({upload:f.die},{instance:f.constant(f.die("Image Upload is disabled"))}):R.isFunction(a.handler)?Eu(PN.custom(a.handler),ON.simple(b)):Eu(PN.direct(a.direct),ON.url(b,a.direct.url))}},GW=C.immutable("id","imageresult","objurl"),Uk=Kb.trackedImage(),
IW=function(a,b){return X.descendants(a,"img["+Uk+'="'+b+'"]')},JW=function(a){return X.descendants(a,"img:not(["+Uk+"])["+Kb.blobId()+"]")},cea={settings:function(a){var b={ui:{toolbar:{display:!1}},spelling:{bundled:!1},highlighter:{bundled:!1},table:{bundled:!1},resize:{bundled:!1},codeview:{bundled:!1},imageEdit:{bundled:!1},imageupload:{immediate:!0},linkvalidator:{bundled:!1}},b=Y.detect().deviceType.isTouch()?b:{};return Z.deepMerge(b,void 0===a.content||void 0===a.content.inline?{}:{fullscreen:{bundled:!1},
codeview:{bundled:!1},plaincodeview:{bundled:!1}})}},jy={isRtl:f.constant(!1)},iy={isRtl:f.constant(!0)},QW=function(a){return k.exists(["ar","fa","he"],function(b){return aa.startsWith(a,b)})},SW=function(a){return Z.deepMerge(ky.settings(a.resources),a,cea.settings(a))},QN=function(a,b,c){var d=Kz.withOptions(a,c);d.content&&d.content.tabindex&&(0,m.remove)(a,"tabindex",d.content.tabindex);c=yp(d,{});var e=ua.get(a),f=m.get(a,"style");b(a);ua.set(a,"");c.connect(a);Oc.set(c,e);return{polish:c,restore:function(b){ua.set(a,
b);null!==f&&void 0!==f&&0<f.length?m.set(a,"style",f):m.remove(a,"style");d.content&&d.content.tabindex&&(0,m.set)(a,"tabindex",d.content.tabindex);return a}}},RN={replace:function(a,b){return QN(a,b.content&&!0===b.content.autoresize?tY:sY,b)},takeover:function(a,b){return QN(a,f.noop,b)}},nb=function(a,b){var c={};if(void 0===a)return c;k.each(b,function(b){var e=b.name(),e=a[e];void 0!==e&&(b.validate(e),c[b.output()]=R.isArray(b.contents)?nb(e,b.contents):e)});return c},dea=function(a){return k.map(void 0===
a?[]:a,function(a){var c=nb,d=V.string("prefix"),e=V.string("suffix"),f;f=[V.string("color"),V.string("background-color")];f=Z.merge(V.object("style"),{contents:f});return c(a,[d,e,f])})},eea=function(a){if(void 0===a||void 0===a.url)return{service:!1};var b=SN(a,"autocorrect",!0);if(R.isString(a.url))return{url:a.url,service:!0,autocorrect:b};throw Error("Spelling URL was not a string");},fea=function(a){if(void 0===a||void 0===a.validation||void 0===a.validation.url)return{};if(R.isString(a.validation.url))return{url:a.validation.url,
bundled:!0};throw Error("Link Validation URL was not a string");},gea=function(a){if(void 0===a||void 0===a.embed||void 0===a.embed.url)return{};if(R.isString(a.embed.url))return{url:a.embed.url,bundled:!0};throw Error("Link Embed URL was not a string");},hea=function(a,b){var c=!1===b?!0:!1,d=(void 0===a?{}:a).upload;if(void 0!==d){if(void 0!==d.handler){if(!R.isFunction(d.handler))throw Error("Image Upload Handler was not a function");return{handler:d.handler,immediate:c,bundled:!0}}if(!R.isString(d.url))throw Error("Image Upload URL was not a string");
return{direct:nb(d,[V.string("url"),V.string("basePath"),V.bool("credentials")]),immediate:c,bundled:!0}}return{}},iea=function(a){a=nb((void 0===a?{}:a).editing,[V.bool("enabled"),V.string("proxy"),V.number("preferredWidth"),V.number("preferredHeight")]);void 0!==a.enabled&&(a.bundled=a.enabled);return a},SN=function(a,b,c){if(void 0===a[b])return c;V.bool(b).validate(a[b]);return a[b]},jea=function(a){return nb((void 0===a?{}:a).toolbar,[V.arrayOfToolbarGroups("items"),V.arrayOfStrings("contextual")])},
kea=function(a){if(void 0===a)return{};var b;if(void 0===a.toolbar)b={};else{var c=a.toolbar;if(void 0!==c.offset){c=c.offset;if(!R.isObject(c)||void 0===c.top||void 0===c.left)throw Error("toolbar offset must be an object with left and top");if(!R.isNumber(c.top)&&!R.isNumber(c.left))throw Error("toolbar offset values must be numbers");b={draggable:!0,offset:{left:c.left,top:c.top}}}else b={}}c=nb(a.toolbar,[V.bool("visible"),V.bool("draggable"),V.bool("sticky")]);b=Z.deepMerge(b,c);return void 0===
c.sticky&&!0===a.autoresize?Z.deepMerge(b,{sticky:!0}):b},lea=function(a){if(void 0!==a){if(!R.isString(a))throw Error("The basePath configuration value was not a string");return aa.removeTrailing(a,"/")}},mea=function(a){if(void 0===a||void 0===a.fonts)return[];a=a.fonts;if(!R.isArray(a))throw Error("The fonts configuration was not an array");var b=function(a,b){console.error(a+" :",b);throw Error(a);};return k.map(a,function(a){if(R.isString(a))return{value:a,text:a};R.isObject(a)||b("The fonts array value was not a string or object",
a);void 0!==a.value&&R.isString(a.value)||b("The fonts value configuration was not a string or is undefined",a);void 0===a.text||R.isString(a.text)||b("The fonts text configuration was not a string",a);void 0!==a.text&&R.isString(a.text)&&0===a.text.length&&b("The fonts text configuration was an empty string",a);return{value:a.value,text:a.text&&0<a.text.length?a.text:a.value}})},nea=function(a){a=nb(void 0===a?{}:a,[V.bool("enabled"),V.bool("showButton")]);return{bundled:void 0===a.enabled||!0===
a.enabled,showButton:void 0===a.showButton||!0===a.showButton}},oea=function(a){a=Z.merge({allowLocal:!0},void 0===a?{}:a);V.bool("allowLocal").validate(a.allowLocal);return a.allowLocal},pea=function(a){a=nb(void 0===a?{}:a,[V.arrayOfStrings("languages")]);return void 0===a.languages?{}:{languages:k.map(a.languages,function(a){return a.toLowerCase()})}},qea=function(a){return void 0===a.styles?{}:{styles:k.filter(a.styles,function(a){a=a.rule;return!(void 0===a||"."===a||0<=a.indexOf("..")||0<=a.indexOf("#"))})}},
rea=function(a,b,c){return"plain"===a?{forceText:!0}:{officeStyles:"retain"===a?"merge":void 0!==a?a:"prompt",htmlStyles:"retain"===a?"merge":void 0!==a?a:"merge",allowLocalImages:c,enableFlashImport:!1===b?!1:!0}},UW=function(a){var b=nb(a,[V.arrayOfStrings("plugins")]),b=void 0===b.plugins?[]:b.plugins,c=nb(a.ui,[V.string("locale","code")]),d=SN(a,"autosubmit",!0),e=nb(a.content,[V.arrayOfObjects("highlighters")]),f=nb(a.paste,[V.string("style"),V.bool("enableFlashImport")]),h=function(){var b=
nb(a.css,[V.arrayOfStrings("stylesheets"),V.string("documentStyles")]),c=nb(a.ui,[V.bool("autoresize")]);return Z.merge(b,c)}(),k=nb(a.css,[V.arrayOfObjects("styles")]),m=nb(a.ui,[V.bool("shortcuts","features")]),n=pea(a.ui),q=nea(a.codeview),s=eea(a.spelling),t=fea(a.links),u=gea(a.links),v=oea(a.images),x=iea(a.images),y=v?hea(a.images,x.bundled):{},A=jea(a.ui),B=kea(a.ui),C=lea(a.basePath),D=mea(a.ui),E=function(){var b=nb(a.ui,[V.string("aria-label","editorName"),V.bool("autoresize")]);return Z.merge(b,
{locale:c,toolbar:B})}();return{resources:C,plugins:b,spelling:s,linkvalidator:t,embed:u,font:{faces:D},ui:E,shortcuts:m,autosubmit:d,highlighter:{external:dea(e.highlighters)},smartclipboard:{paste:rea(f.style,f.enableFlashImport,v)},parlance:n,content:h,imageEdit:x,imageupload:y,blocktag:qea(k),fileupload:{allowLocalImages:v},platform:{toolbar:A,allowLocalImages:v},plaincodeview:{bundled:q.bundled},codeview:q}},VW=function(a){var b=void 0===a.privateapi?{}:a.privateapi;a=nb(b.ui,[V.func("bounds"),
V.bool("showHelpHint"),V.domElement("bottomBar"),V.domElement("sideBar")]);var c=nb(b.integration,[V.string("type"),V.string("version")]),d=void 0===b.content?{}:b.content,e=nb(d.macros,[V.arrayOfStrings("whitelist")]),d=nb(d,[V.string("base")]);void 0!==e.whitelist&&(e.whitelist=h.some(e.whitelist));var f=zy.paramList(),k=nb(b.mentions,f),m=W.keys(k).length,b=nb(b.output,[V.bool("prettyprint")]);if(0<m){if(m!==f.length)throw Error("Missing @mentions configuration. Expected "+f.length+" objects, was "+
m);k.bundled=!0}return{ui:a,help:{integration:c},macros:e,content:Z.merge(d,b),mentions:k}},sea={getPath:function(a){var b=document.getElementsByTagName("script");return k.find(b,function(b){return b.src.indexOf(a)===b.src.length-a.length}).map(function(b){return b.src.substring(0,b.src.indexOf(a))}).getOr(void 0)}},TN=hb.cached(function(){return h.from(sea.getPath("textboxio.js")).map(function(a){return aa.removeTrailing(a,"/")})}),UN=function(a){return a+"/resources"},ly={setupResources:function(a){return h.from(a.resources).orThunk(TN).map(UN).fold(f.die("basePath auto detection failed, please provide one in the editor configuration"),
function(b){b=ki.load(b);return Z.deepMerge(a,{resources:b})})},speculate:function(a){h.from(a).orThunk(TN).map(UN).each(ki.load)}},tea=function(a,b,c){var d=function(){var a=q.fromDom(document.activeElement);b(a)&&c()};return E.bind(a,"blur",function(){setTimeout(d,200)})},uea=function(a,b,c){return E.bind(a,"focusout",function(a){VN(a,b)&&c()})},vea=function(a,b,c){return E.bind(a,"blur",function(a){VN(a,b)&&c()})},wea=function(){var a=Y.detect();return a.browser.isFirefox()?tea:a.browser.isIE()?
uea:vea}(),VN=function(a,b){var c=h.from(a.raw().relatedTarget).getOr(document.activeElement);return b(q.fromDom(c))},xea={register:function(a,b,c,d){var e=E.bind(a,"focus",function(a){T.hasFocus(a.target())&&c()}),f=wea(a,function(c){return!Gc.closest(c,function(c){return L.eq(a,c)||L.eq(b(),c)||n.has(c,lx.magicBox())})},d);return{destroy:function(){e.unbind();f.unbind()}}}},Nd={replace:function(a,b){var c=Z.deepMerge(b,{content:{inline:a}}),d=m.clone(a),e=yp(c),f=WW(e.element,a,d),h=bo(f.toolbarHide,
f.toolbarShow,!1),k=function(){var a=f.isVisible();h.on();!1===a&&e.activateInline()},n=function(){e.deactivateInline();h.off()},q=xea.register(a,e.element,k,n),c=function(){T.active().each(function(b){L.eq(b,a)||n()})};c();e.events.connected.bind(c);e.events.loaded.bind(function(){e.content().getSelection().bind(k)});var s=e.focus,c={polish:Z.merge(e,{focus:function(){k();s()}}),restore:function(b){q.destroy();f.restore();ua.set(a,b);return a}};e.connect(ca.body());Oc.set(e,ua.get(a));return c}},
yea={attach:function(a,b,c,d){if(4!==arguments.length)throw"requires 4 parameters";var e=function(b){return function(){var c=q.fromTag("input");m.setAll(c,{type:"hidden",name:b});a.append(c.dom());return function(a){c.val(a)}}},f=c.getOrThunk(e(b)),h=d.getOrThunk(e(b+"_styles"));return function(a){f(a.content);h(a.styles)}}},WN=function(a,b){var c=a.onsubmit,d=XW(a);a.onsubmit=function(e){if(d.inprogress())return!0;b(function(){c&&!1===c.apply(a)||setTimeout(d.submit,0)});return!1};return{unbind:function(){a.onsubmit=
c}}},zea={attachToFormSubmit:WN,attachToParentFormSubmit:function(a,b,c,d,e){if(5!==arguments.length)throw"requires 5 parameters";var f=a.parents("form:first");if(0!==f.size()){var h=f.get(0),k=yea.attach(f,b,d,e);WN(h,function(a){k(c());a()})}}},Bea=function(a,b,c,d){t.set(b,"display","none");var e=yp(Kz.withOptions(b,d)),f=$.get(b);e.connect(a);Oc.set(e,f);var k=!0===d.autosubmit?Aea(b,e):h.none();return{polish:e,restore:function(d){k.each(function(a){a.unbind()});$.set(b,d);c.fold(function(){t.remove(b,
"display")},function(a){t.set(b,"display",a)});B.remove(a);return b}}},Aea=function(a,b){return J.closest(a,"form").map(function(c){return zea.attachToFormSubmit(c.dom(),function(c){b.uploadImages(function(){$.set(a,Oc.get(b));c()})})})},ZW={textarea:{replace:function(a,b){var c=t.getRaw(a,"display");return y.parent(a).fold(function(){return null},function(){var d=m.get(a,"id"),d=d?"ephox_"+d:La.generate("ephox"),e=q.fromTag("div");m.set(e,"id",d);t.copy(a,e);(b.content&&!0===b.content.autoresize?
rY:qY)(a,e);v.before(a,e);return Bea(e,a,c,b)})}},div:RN,body:{replace:function(a,b){var c=Z.deepMerge(b,{platform:{fillscreen:!0}}),c=RN.takeover(a,c);return{polish:c.polish,restore:c.restore}}}},$W={div:Nd,blockquote:Nd,article:Nd,aside:Nd,figure:Nd,figcaption:Nd,footer:Nd,header:Nd,nav:Nd,section:Nd},Jj=function(a){return{name:a.name,required:f.never,validate:a.validate}},Vk={bool:f.compose(Jj,V.bool),func:f.compose(Jj,V.func),number:f.compose(Jj,V.number),object:f.compose(Jj,V.object),string:f.compose(Jj,
V.string)},Fu=function(a){return H.error("ERROR: Textbox.io requires "+a+" or above")},Gu=Y.detect().os,Kj=Y.detect().browser,XN="CSS1Compat"===document.compatMode||"html"===document.doctype.name,Cea={supported:function(){if(Kj.isIE()&&11>Kj.version.major)return Fu("Internet Explorer 11");if(Gu.isOSX()&&Kj.isSafari()&&9>Kj.version.major)return Fu("Safari 9");if(Gu.isiOS()&&9>Gu.version.major)return Fu("iOS 9");if(!XN&&Kj.isIE())return H.error("ERROR: Document is not in standards mode. Textbox.io requires standards mode on Internet Explorer.");
XN||void 0===console.warn||console.warn("WARNING: Document is not in standards mode. Textbox.io may experience user interface issues.");return H.value(void 0)}},YN=function(a){Ke.textboxio=a},Dea=f.compose(YN,function(a){a=f.die(a);return{replace:a,replaceAll:a,inline:a,inlineAll:a,get:a,getActiveEditor:a,definePlugin:a,version:Hu,isSupported:f.never}});(function(){var a=Ke.install;Ke.install=function(b,c){Ke.install=a;Cea.supported().fold(Dea,function(){var a=null===c||void 0===c?{onprem:!0}:c,a=
null===b||void 0===b?a:Z.deepMerge(a,{resources:b});YN(aX(a));ly.speculate(a.resources)})}})()})();
install();

"use strict";

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/*
 *
 * More info at [www.dropzonejs.com](http://www.dropzonejs.com)
 *
 * Copyright (c) 2012, Matias Meno
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 */

// The Emitter class provides the ability to call `.on()` on Dropzone to listen
// to events.
// It is strongly based on component's emitter class, and I removed the
// functionality because of the dependency hell with different frameworks.
var Emitter = function () {
    function Emitter() {
        _classCallCheck(this, Emitter);
    }

    _createClass(Emitter, [{
        key: "on",

        // Add an event listener for given event
        value: function on(event, fn) {
            this._callbacks = this._callbacks || {};
            // Create namespace for this event
            if (!this._callbacks[event]) {
                this._callbacks[event] = [];
            }
            this._callbacks[event].push(fn);
            return this;
        }
    }, {
        key: "emit",
        value: function emit(event) {
            this._callbacks = this._callbacks || {};
            var callbacks = this._callbacks[event];

            if (callbacks) {
                for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
                    args[_key - 1] = arguments[_key];
                }

                for (var _iterator = callbacks, _isArray = true, _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
                    var _ref;

                    if (_isArray) {
                        if (_i >= _iterator.length) break;
                        _ref = _iterator[_i++];
                    } else {
                        _i = _iterator.next();
                        if (_i.done) break;
                        _ref = _i.value;
                    }

                    var callback = _ref;

                    callback.apply(this, args);
                }
            }

            return this;
        }

        // Remove event listener for given event. If fn is not provided, all event
        // listeners for that event will be removed. If neither is provided, all
        // event listeners will be removed.

    }, {
        key: "off",
        value: function off(event, fn) {
            if (!this._callbacks || arguments.length === 0) {
                this._callbacks = {};
                return this;
            }

            // specific event
            var callbacks = this._callbacks[event];
            if (!callbacks) {
                return this;
            }

            // remove all handlers
            if (arguments.length === 1) {
                delete this._callbacks[event];
                return this;
            }

            // remove specific handler
            for (var i = 0; i < callbacks.length; i++) {
                var callback = callbacks[i];
                if (callback === fn) {
                    callbacks.splice(i, 1);
                    break;
                }
            }

            return this;
        }
    }]);

    return Emitter;
}();

var Dropzone = function (_Emitter) {
    _inherits(Dropzone, _Emitter);

    _createClass(Dropzone, null, [{
        key: "initClass",
        value: function initClass() {

            // Exposing the emitter class, mainly for tests
            this.prototype.Emitter = Emitter;

            /*
             This is a list of all available events you can register on a dropzone object.
              You can register an event handler like this:
              dropzone.on("dragEnter", function() { });
              */
            this.prototype.events = ["drop", "dragstart", "dragend", "dragenter", "dragover", "dragleave", "addedfile", "addedfiles", "removedfile", "thumbnail", "error", "errormultiple", "processing", "processingmultiple", "uploadprogress", "totaluploadprogress", "sending", "sendingmultiple", "success", "successmultiple", "canceled", "canceledmultiple", "complete", "completemultiple", "reset", "maxfilesexceeded", "maxfilesreached", "queuecomplete"];

            this.prototype.defaultOptions = {
                /**
                 * Has to be specified on elements other than form (or when the form
                 * doesn't have an `action` attribute). You can also
                 * provide a function that will be called with `files` and
                 * must return the url (since `v3.12.0`)
                 */
                url: null,

                /**
                 * Can be changed to `"put"` if necessary. You can also provide a function
                 * that will be called with `files` and must return the method (since `v3.12.0`).
                 */
                method: "post",

                /**
                 * Will be set on the XHRequest.
                 */
                withCredentials: false,

                /**
                 * The timeout for the XHR requests in milliseconds (since `v4.4.0`).
                 */
                timeout: 30000,

                /**
                 * How many file uploads to process in parallel (See the
                 * Enqueuing file uploads* documentation section for more info)
                 */
                parallelUploads: 2,

                /**
                 * Whether to send multiple files in one request. If
                 * this it set to true, then the fallback file input element will
                 * have the `multiple` attribute as well. This option will
                 * also trigger additional events (like `processingmultiple`). See the events
                 * documentation section for more information.
                 */
                uploadMultiple: false,

                /**
                 * Whether you want files to be uploaded in chunks to your server. This can't be
                 * used in combination with `uploadMultiple`.
                 *
                 * See [chunksUploaded](#config-chunksUploaded) for the callback to finalise an upload.
                 */
                chunking: false,

                /**
                 * If `chunking` is enabled, this defines whether **every** file should be chunked,
                 * even if the file size is below chunkSize. This means, that the additional chunk
                 * form data will be submitted and the `chunksUploaded` callback will be invoked.
                 */
                forceChunking: false,

                /**
                 * If `chunking` is `true`, then this defines the chunk size in bytes.
                 */
                chunkSize: 2000000,

                /**
                 * If `true`, the individual chunks of a file are being uploaded simultaneously.
                 */
                parallelChunkUploads: false,

                /**
                 * Whether a chunk should be retried if it fails.
                 */
                retryChunks: false,

                /**
                 * If `retryChunks` is true, how many times should it be retried.
                 */
                retryChunksLimit: 3,

                /**
                 * If not `null` defines how many files this Dropzone handles. If it exceeds,
                 * the event `maxfilesexceeded` will be called. The dropzone element gets the
                 * class `dz-max-files-reached` accordingly so you can provide visual feedback.
                 */
                maxFilesize: 256,

                /**
                 * The name of the file param that gets transferred.
                 * **NOTE**: If you have the option  `uploadMultiple` set to `true`, then
                 * Dropzone will append `[]` to the name.
                 */
                paramName: "file",

                /**
                 * Whether thumbnails for images should be generated
                 */
                createImageThumbnails: true,

                /**
                 * In MB. When the filename exceeds this limit, the thumbnail will not be generated.
                 */
                maxThumbnailFilesize: 10,

                /**
                 * If `null`, the ratio of the image will be used to calculate it.
                 */
                thumbnailWidth: 120,

                /**
                 * The same as `thumbnailWidth`. If both are null, images will not be resized.
                 */
                thumbnailHeight: 120,

                /**
                 * How the images should be scaled down in case both, `thumbnailWidth` and `thumbnailHeight` are provided.
                 * Can be either `contain` or `crop`.
                 */
                thumbnailMethod: 'crop',

                /**
                 * If set, images will be resized to these dimensions before being **uploaded**.
                 * If only one, `resizeWidth` **or** `resizeHeight` is provided, the original aspect
                 * ratio of the file will be preserved.
                 *
                 * The `options.transformFile` function uses these options, so if the `transformFile` function
                 * is overridden, these options don't do anything.
                 */
                resizeWidth: null,

                /**
                 * See `resizeWidth`.
                 */
                resizeHeight: null,

                /**
                 * The mime type of the resized image (before it gets uploaded to the server).
                 * If `null` the original mime type will be used. To force jpeg, for example, use `image/jpeg`.
                 * See `resizeWidth` for more information.
                 */
                resizeMimeType: null,

                /**
                 * The quality of the resized images. See `resizeWidth`.
                 */
                resizeQuality: 0.8,

                /**
                 * How the images should be scaled down in case both, `resizeWidth` and `resizeHeight` are provided.
                 * Can be either `contain` or `crop`.
                 */
                resizeMethod: 'contain',

                /**
                 * The base that is used to calculate the filesize. You can change this to
                 * 1024 if you would rather display kibibytes, mebibytes, etc...
                 * 1024 is technically incorrect, because `1024 bytes` are `1 kibibyte` not `1 kilobyte`.
                 * You can change this to `1024` if you don't care about validity.
                 */
                filesizeBase: 1000,

                /**
                 * Can be used to limit the maximum number of files that will be handled by this Dropzone
                 */
                maxFiles: null,

                /**
                 * An optional object to send additional headers to the server. Eg:
                 * `{ "My-Awesome-Header": "header value" }`
                 */
                headers: null,

                /**
                 * If `true`, the dropzone element itself will be clickable, if `false`
                 * nothing will be clickable.
                 *
                 * You can also pass an HTML element, a CSS selector (for multiple elements)
                 * or an array of those. In that case, all of those elements will trigger an
                 * upload when clicked.
                 */
                clickable: true,

                /**
                 * Whether hidden files in directories should be ignored.
                 */
                ignoreHiddenFiles: true,

                /**
                 * The default implementation of `accept` checks the file's mime type or
                 * extension against this list. This is a comma separated list of mime
                 * types or file extensions.
                 *
                 * Eg.: `image/*,application/pdf,.psd`
                 *
                 * If the Dropzone is `clickable` this option will also be used as
                 * [`accept`](https://developer.mozilla.org/en-US/docs/HTML/Element/input#attr-accept)
                 * parameter on the hidden file input as well.
                 */
                acceptedFiles: null,

                /**
                 * **Deprecated!**
                 * Use acceptedFiles instead.
                 */
                acceptedMimeTypes: null,

                /**
                 * If false, files will be added to the queue but the queue will not be
                 * processed automatically.
                 * This can be useful if you need some additional user input before sending
                 * files (or if you want want all files sent at once).
                 * If you're ready to send the file simply call `myDropzone.processQueue()`.
                 *
                 * See the [enqueuing file uploads](#enqueuing-file-uploads) documentation
                 * section for more information.
                 */
                autoProcessQueue: true,

                /**
                 * If false, files added to the dropzone will not be queued by default.
                 * You'll have to call `enqueueFile(file)` manually.
                 */
                autoQueue: true,

                /**
                 * If `true`, this will add a link to every file preview to remove or cancel (if
                 * already uploading) the file. The `dictCancelUpload`, `dictCancelUploadConfirmation`
                 * and `dictRemoveFile` options are used for the wording.
                 */
                addRemoveLinks: false,

                /**
                 * Defines where to display the file previews – if `null` the
                 * Dropzone element itself is used. Can be a plain `HTMLElement` or a CSS
                 * selector. The element should have the `dropzone-previews` class so
                 * the previews are displayed properly.
                 */
                previewsContainer: null,

                /**
                 * This is the element the hidden input field (which is used when clicking on the
                 * dropzone to trigger file selection) will be appended to. This might
                 * be important in case you use frameworks to switch the content of your page.
                 */
                hiddenInputContainer: "body",

                /**
                 * If null, no capture type will be specified
                 * If camera, mobile devices will skip the file selection and choose camera
                 * If microphone, mobile devices will skip the file selection and choose the microphone
                 * If camcorder, mobile devices will skip the file selection and choose the camera in video mode
                 * On apple devices multiple must be set to false.  AcceptedFiles may need to
                 * be set to an appropriate mime type (e.g. "image/*", "audio/*", or "video/*").
                 */
                capture: null,

                /**
                 * **Deprecated**. Use `renameFile` instead.
                 */
                renameFilename: null,

                /**
                 * A function that is invoked before the file is uploaded to the server and renames the file.
                 * This function gets the `File` as argument and can use the `file.name`. The actual name of the
                 * file that gets used during the upload can be accessed through `file.upload.filename`.
                 */
                renameFile: null,

                /**
                 * If `true` the fallback will be forced. This is very useful to test your server
                 * implementations first and make sure that everything works as
                 * expected without dropzone if you experience problems, and to test
                 * how your fallbacks will look.
                 */
                forceFallback: false,

                /**
                 * The text used before any files are dropped.
                 */
                dictDefaultMessage: "Drop files here to upload",

                /**
                 * The text that replaces the default message text it the browser is not supported.
                 */
                dictFallbackMessage: "Your browser does not support drag'n'drop file uploads.",

                /**
                 * The text that will be added before the fallback form.
                 * If you provide a  fallback element yourself, or if this option is `null` this will
                 * be ignored.
                 */
                dictFallbackText: "Please use the fallback form below to upload your files like in the olden days.",

                /**
                 * If the filesize is too big.
                 * `{{filesize}}` and `{{maxFilesize}}` will be replaced with the respective configuration values.
                 */
                dictFileTooBig: "File is too big ({{filesize}}MiB). Max filesize: {{maxFilesize}}MiB.",

                /**
                 * If the file doesn't match the file type.
                 */
                dictInvalidFileType: "You can't upload files of this type.",

                /**
                 * If the server response was invalid.
                 * `{{statusCode}}` will be replaced with the servers status code.
                 */
                dictResponseError: "Server responded with {{statusCode}} code.",

                /**
                 * If `addRemoveLinks` is true, the text to be used for the cancel upload link.
                 */
                dictCancelUpload: "Cancel upload",

                /**
                 * The text that is displayed if an upload was manually canceled
                 */
                dictUploadCanceled: "Upload canceled.",

                /**
                 * If `addRemoveLinks` is true, the text to be used for confirmation when cancelling upload.
                 */
                dictCancelUploadConfirmation: "Are you sure you want to cancel this upload?",

                /**
                 * If `addRemoveLinks` is true, the text to be used to remove a file.
                 */
                dictRemoveFile: "Remove file",

                /**
                 * If this is not null, then the user will be prompted before removing a file.
                 */
                dictRemoveFileConfirmation: null,

                /**
                 * Displayed if `maxFiles` is st and exceeded.
                 * The string `{{maxFiles}}` will be replaced by the configuration value.
                 */
                dictMaxFilesExceeded: "You can not upload any more files.",

                /**
                 * Allows you to translate the different units. Starting with `tb` for terabytes and going down to
                 * `b` for bytes.
                 */
                dictFileSizeUnits: { tb: "TB", gb: "GB", mb: "MB", kb: "KB", b: "b" },
                /**
                 * Called when dropzone initialized
                 * You can add event listeners here
                 */
                init: function init() {},


                /**
                 * Can be an **object** of additional parameters to transfer to the server, **or** a `Function`
                 * that gets invoked with the `files`, `xhr` and, if it's a chunked upload, `chunk` arguments. In case
                 * of a function, this needs to return a map.
                 *
                 * The default implementation does nothing for normal uploads, but adds relevant information for
                 * chunked uploads.
                 *
                 * This is the same as adding hidden input fields in the form element.
                 */
                params: function params(files, xhr, chunk) {
                    if (chunk) {
                        return {
                            dzuuid: chunk.file.upload.uuid,
                            dzchunkindex: chunk.index,
                            dztotalfilesize: chunk.file.size,
                            dzchunksize: this.options.chunkSize,
                            dztotalchunkcount: chunk.file.upload.totalChunkCount,
                            dzchunkbyteoffset: chunk.index * this.options.chunkSize
                        };
                    }
                },


                /**
                 * A function that gets a [file](https://developer.mozilla.org/en-US/docs/DOM/File)
                 * and a `done` function as parameters.
                 *
                 * If the done function is invoked without arguments, the file is "accepted" and will
                 * be processed. If you pass an error message, the file is rejected, and the error
                 * message will be displayed.
                 * This function will not be called if the file is too big or doesn't match the mime types.
                 */
                accept: function accept(file, done) {
                    return done();
                },


                /**
                 * The callback that will be invoked when all chunks have been uploaded for a file.
                 * It gets the file for which the chunks have been uploaded as the first parameter,
                 * and the `done` function as second. `done()` needs to be invoked when everything
                 * needed to finish the upload process is done.
                 */
                chunksUploaded: function chunksUploaded(file, done) {
                    done();
                },

                /**
                 * Gets called when the browser is not supported.
                 * The default implementation shows the fallback input field and adds
                 * a text.
                 */
                fallback: function fallback() {
                    // This code should pass in IE7... :(
                    var messageElement = void 0;
                    this.element.className = this.element.className + " dz-browser-not-supported";

                    for (var _iterator2 = this.element.getElementsByTagName("div"), _isArray2 = true, _i2 = 0, _iterator2 = _isArray2 ? _iterator2 : _iterator2[Symbol.iterator]();;) {
                        var _ref2;

                        if (_isArray2) {
                            if (_i2 >= _iterator2.length) break;
                            _ref2 = _iterator2[_i2++];
                        } else {
                            _i2 = _iterator2.next();
                            if (_i2.done) break;
                            _ref2 = _i2.value;
                        }

                        var child = _ref2;

                        if (/(^| )dz-message($| )/.test(child.className)) {
                            messageElement = child;
                            child.className = "dz-message"; // Removes the 'dz-default' class
                            break;
                        }
                    }
                    if (!messageElement) {
                        messageElement = Dropzone.createElement("<div class=\"dz-message\"><span></span></div>");
                        this.element.appendChild(messageElement);
                    }

                    var span = messageElement.getElementsByTagName("span")[0];
                    if (span) {
                        if (span.textContent != null) {
                            span.textContent = this.options.dictFallbackMessage;
                        } else if (span.innerText != null) {
                            span.innerText = this.options.dictFallbackMessage;
                        }
                    }

                    return this.element.appendChild(this.getFallbackForm());
                },


                /**
                 * Gets called to calculate the thumbnail dimensions.
                 *
                 * It gets `file`, `width` and `height` (both may be `null`) as parameters and must return an object containing:
                 *
                 *  - `srcWidth` & `srcHeight` (required)
                 *  - `trgWidth` & `trgHeight` (required)
                 *  - `srcX` & `srcY` (optional, default `0`)
                 *  - `trgX` & `trgY` (optional, default `0`)
                 *
                 * Those values are going to be used by `ctx.drawImage()`.
                 */
                resize: function resize(file, width, height, resizeMethod) {
                    var info = {
                        srcX: 0,
                        srcY: 0,
                        srcWidth: file.width,
                        srcHeight: file.height
                    };

                    var srcRatio = file.width / file.height;

                    // Automatically calculate dimensions if not specified
                    if (width == null && height == null) {
                        width = info.srcWidth;
                        height = info.srcHeight;
                    } else if (width == null) {
                        width = height * srcRatio;
                    } else if (height == null) {
                        height = width / srcRatio;
                    }

                    // Make sure images aren't upscaled
                    width = Math.min(width, info.srcWidth);
                    height = Math.min(height, info.srcHeight);

                    var trgRatio = width / height;

                    if (info.srcWidth > width || info.srcHeight > height) {
                        // Image is bigger and needs rescaling
                        if (resizeMethod === 'crop') {
                            if (srcRatio > trgRatio) {
                                info.srcHeight = file.height;
                                info.srcWidth = info.srcHeight * trgRatio;
                            } else {
                                info.srcWidth = file.width;
                                info.srcHeight = info.srcWidth / trgRatio;
                            }
                        } else if (resizeMethod === 'contain') {
                            // Method 'contain'
                            if (srcRatio > trgRatio) {
                                height = width / srcRatio;
                            } else {
                                width = height * srcRatio;
                            }
                        } else {
                            throw new Error("Unknown resizeMethod '" + resizeMethod + "'");
                        }
                    }

                    info.srcX = (file.width - info.srcWidth) / 2;
                    info.srcY = (file.height - info.srcHeight) / 2;

                    info.trgWidth = width;
                    info.trgHeight = height;

                    return info;
                },


                /**
                 * Can be used to transform the file (for example, resize an image if necessary).
                 *
                 * The default implementation uses `resizeWidth` and `resizeHeight` (if provided) and resizes
                 * images according to those dimensions.
                 *
                 * Gets the `file` as the first parameter, and a `done()` function as the second, that needs
                 * to be invoked with the file when the transformation is done.
                 */
                transformFile: function transformFile(file, done) {
                    if ((this.options.resizeWidth || this.options.resizeHeight) && file.type.match(/image.*/)) {
                        return this.resizeImage(file, this.options.resizeWidth, this.options.resizeHeight, this.options.resizeMethod, done);
                    } else {
                        return done(file);
                    }
                },


                /**
                 * A string that contains the template used for each dropped
                 * file. Change it to fulfill your needs but make sure to properly
                 * provide all elements.
                 *
                 * If you want to use an actual HTML element instead of providing a String
                 * as a config option, you could create a div with the id `tpl`,
                 * put the template inside it and provide the element like this:
                 *
                 *     document
                 *       .querySelector('#tpl')
                 *       .innerHTML
                 *
                 */
                previewTemplate: "<div class=\"dz-preview dz-file-preview\">\n  <div class=\"dz-image\"><img data-dz-thumbnail /></div>\n  <div class=\"dz-details\">\n    <div class=\"dz-size\"><span data-dz-size></span></div>\n    <div class=\"dz-filename\"><span data-dz-name></span></div>\n  </div>\n  <div class=\"dz-progress\"><span class=\"dz-upload\" data-dz-uploadprogress></span></div>\n  <div class=\"dz-error-message\"><span data-dz-errormessage></span></div>\n  <div class=\"dz-success-mark\">\n    <svg width=\"54px\" height=\"54px\" viewBox=\"0 0 54 54\" version=\"1.1\" xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" xmlns:sketch=\"http://www.bohemiancoding.com/sketch/ns\">\n      <title>Check</title>\n      <defs></defs>\n      <g id=\"Page-1\" stroke=\"none\" stroke-width=\"1\" fill=\"none\" fill-rule=\"evenodd\" sketch:type=\"MSPage\">\n        <path d=\"M23.5,31.8431458 L17.5852419,25.9283877 C16.0248253,24.3679711 13.4910294,24.366835 11.9289322,25.9289322 C10.3700136,27.4878508 10.3665912,30.0234455 11.9283877,31.5852419 L20.4147581,40.0716123 C20.5133999,40.1702541 20.6159315,40.2626649 20.7218615,40.3488435 C22.2835669,41.8725651 24.794234,41.8626202 26.3461564,40.3106978 L43.3106978,23.3461564 C44.8771021,21.7797521 44.8758057,19.2483887 43.3137085,17.6862915 C41.7547899,16.1273729 39.2176035,16.1255422 37.6538436,17.6893022 L23.5,31.8431458 Z M27,53 C41.3594035,53 53,41.3594035 53,27 C53,12.6405965 41.3594035,1 27,1 C12.6405965,1 1,12.6405965 1,27 C1,41.3594035 12.6405965,53 27,53 Z\" id=\"Oval-2\" stroke-opacity=\"0.198794158\" stroke=\"#747474\" fill-opacity=\"0.816519475\" fill=\"#FFFFFF\" sketch:type=\"MSShapeGroup\"></path>\n      </g>\n    </svg>\n  </div>\n  <div class=\"dz-error-mark\">\n    <svg width=\"54px\" height=\"54px\" viewBox=\"0 0 54 54\" version=\"1.1\" xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" xmlns:sketch=\"http://www.bohemiancoding.com/sketch/ns\">\n      <title>Error</title>\n      <defs></defs>\n      <g id=\"Page-1\" stroke=\"none\" stroke-width=\"1\" fill=\"none\" fill-rule=\"evenodd\" sketch:type=\"MSPage\">\n        <g id=\"Check-+-Oval-2\" sketch:type=\"MSLayerGroup\" stroke=\"#747474\" stroke-opacity=\"0.198794158\" fill=\"#FFFFFF\" fill-opacity=\"0.816519475\">\n          <path d=\"M32.6568542,29 L38.3106978,23.3461564 C39.8771021,21.7797521 39.8758057,19.2483887 38.3137085,17.6862915 C36.7547899,16.1273729 34.2176035,16.1255422 32.6538436,17.6893022 L27,23.3431458 L21.3461564,17.6893022 C19.7823965,16.1255422 17.2452101,16.1273729 15.6862915,17.6862915 C14.1241943,19.2483887 14.1228979,21.7797521 15.6893022,23.3461564 L21.3431458,29 L15.6893022,34.6538436 C14.1228979,36.2202479 14.1241943,38.7516113 15.6862915,40.3137085 C17.2452101,41.8726271 19.7823965,41.8744578 21.3461564,40.3106978 L27,34.6568542 L32.6538436,40.3106978 C34.2176035,41.8744578 36.7547899,41.8726271 38.3137085,40.3137085 C39.8758057,38.7516113 39.8771021,36.2202479 38.3106978,34.6538436 L32.6568542,29 Z M27,53 C41.3594035,53 53,41.3594035 53,27 C53,12.6405965 41.3594035,1 27,1 C12.6405965,1 1,12.6405965 1,27 C1,41.3594035 12.6405965,53 27,53 Z\" id=\"Oval-2\" sketch:type=\"MSShapeGroup\"></path>\n        </g>\n      </g>\n    </svg>\n  </div>\n</div>",

                // END OPTIONS
                // (Required by the dropzone documentation parser)


                /*
                 Those functions register themselves to the events on init and handle all
                 the user interface specific stuff. Overwriting them won't break the upload
                 but can break the way it's displayed.
                 You can overwrite them if you don't like the default behavior. If you just
                 want to add an additional event handler, register it on the dropzone object
                 and don't overwrite those options.
                 */

                // Those are self explanatory and simply concern the DragnDrop.
                drop: function drop(e) {
                    return this.element.classList.remove("dz-drag-hover");
                },
                dragstart: function dragstart(e) {},
                dragend: function dragend(e) {
                    return this.element.classList.remove("dz-drag-hover");
                },
                dragenter: function dragenter(e) {
                    return this.element.classList.add("dz-drag-hover");
                },
                dragover: function dragover(e) {
                    return this.element.classList.add("dz-drag-hover");
                },
                dragleave: function dragleave(e) {
                    return this.element.classList.remove("dz-drag-hover");
                },
                paste: function paste(e) {},


                // Called whenever there are no files left in the dropzone anymore, and the
                // dropzone should be displayed as if in the initial state.
                reset: function reset() {
                    return this.element.classList.remove("dz-started");
                },


                // Called when a file is added to the queue
                // Receives `file`
                addedfile: function addedfile(file) {
                    var _this2 = this;

                    if (this.element === this.previewsContainer) {
                        this.element.classList.add("dz-started");
                    }

                    if (this.previewsContainer) {
                        file.previewElement = Dropzone.createElement(this.options.previewTemplate.trim());
                        file.previewTemplate = file.previewElement; // Backwards compatibility

                        this.previewsContainer.appendChild(file.previewElement);
                        for (var _iterator3 = file.previewElement.querySelectorAll("[data-dz-name]"), _isArray3 = true, _i3 = 0, _iterator3 = _isArray3 ? _iterator3 : _iterator3[Symbol.iterator]();;) {
                            var _ref3;

                            if (_isArray3) {
                                if (_i3 >= _iterator3.length) break;
                                _ref3 = _iterator3[_i3++];
                            } else {
                                _i3 = _iterator3.next();
                                if (_i3.done) break;
                                _ref3 = _i3.value;
                            }

                            var node = _ref3;

                            node.textContent = file.name;
                        }
                        for (var _iterator4 = file.previewElement.querySelectorAll("[data-dz-size]"), _isArray4 = true, _i4 = 0, _iterator4 = _isArray4 ? _iterator4 : _iterator4[Symbol.iterator]();;) {
                            if (_isArray4) {
                                if (_i4 >= _iterator4.length) break;
                                node = _iterator4[_i4++];
                            } else {
                                _i4 = _iterator4.next();
                                if (_i4.done) break;
                                node = _i4.value;
                            }

                            node.innerHTML = this.filesize(file.size);
                        }

                        if (this.options.addRemoveLinks) {
                            file._removeLink = Dropzone.createElement("<a class=\"dz-remove\" href=\"javascript:undefined;\" data-dz-remove>" + this.options.dictRemoveFile + "</a>");
                            file.previewElement.appendChild(file._removeLink);
                        }

                        var removeFileEvent = function removeFileEvent(e) {
                            e.preventDefault();
                            e.stopPropagation();
                            if (file.status === Dropzone.UPLOADING) {
                                return Dropzone.confirm(_this2.options.dictCancelUploadConfirmation, function () {
                                    return _this2.removeFile(file);
                                });
                            } else {
                                if (_this2.options.dictRemoveFileConfirmation) {
                                    return Dropzone.confirm(_this2.options.dictRemoveFileConfirmation, function () {
                                        return _this2.removeFile(file);
                                    });
                                } else {
                                    return _this2.removeFile(file);
                                }
                            }
                        };

                        for (var _iterator5 = file.previewElement.querySelectorAll("[data-dz-remove]"), _isArray5 = true, _i5 = 0, _iterator5 = _isArray5 ? _iterator5 : _iterator5[Symbol.iterator]();;) {
                            var _ref4;

                            if (_isArray5) {
                                if (_i5 >= _iterator5.length) break;
                                _ref4 = _iterator5[_i5++];
                            } else {
                                _i5 = _iterator5.next();
                                if (_i5.done) break;
                                _ref4 = _i5.value;
                            }

                            var removeLink = _ref4;

                            removeLink.addEventListener("click", removeFileEvent);
                        }
                    }
                },


                // Called whenever a file is removed.
                removedfile: function removedfile(file) {
                    if (file.previewElement != null && file.previewElement.parentNode != null) {
                        file.previewElement.parentNode.removeChild(file.previewElement);
                    }
                    return this._updateMaxFilesReachedClass();
                },


                // Called when a thumbnail has been generated
                // Receives `file` and `dataUrl`
                thumbnail: function thumbnail(file, dataUrl) {
                    if (file.previewElement) {
                        file.previewElement.classList.remove("dz-file-preview");
                        for (var _iterator6 = file.previewElement.querySelectorAll("[data-dz-thumbnail]"), _isArray6 = true, _i6 = 0, _iterator6 = _isArray6 ? _iterator6 : _iterator6[Symbol.iterator]();;) {
                            var _ref5;

                            if (_isArray6) {
                                if (_i6 >= _iterator6.length) break;
                                _ref5 = _iterator6[_i6++];
                            } else {
                                _i6 = _iterator6.next();
                                if (_i6.done) break;
                                _ref5 = _i6.value;
                            }

                            var thumbnailElement = _ref5;

                            thumbnailElement.alt = file.name;
                            thumbnailElement.src = dataUrl;
                        }

                        return setTimeout(function () {
                            return file.previewElement.classList.add("dz-image-preview");
                        }, 1);
                    }
                },


                // Called whenever an error occurs
                // Receives `file` and `message`
                error: function error(file, message) {
                    if (file.previewElement) {
                        file.previewElement.classList.add("dz-error");
                        if (typeof message !== "String" && message.error) {
                            message = message.error;
                        }
                        for (var _iterator7 = file.previewElement.querySelectorAll("[data-dz-errormessage]"), _isArray7 = true, _i7 = 0, _iterator7 = _isArray7 ? _iterator7 : _iterator7[Symbol.iterator]();;) {
                            var _ref6;

                            if (_isArray7) {
                                if (_i7 >= _iterator7.length) break;
                                _ref6 = _iterator7[_i7++];
                            } else {
                                _i7 = _iterator7.next();
                                if (_i7.done) break;
                                _ref6 = _i7.value;
                            }

                            var node = _ref6;

                            node.textContent = message;
                        }
                    }
                },
                errormultiple: function errormultiple() {},


                // Called when a file gets processed. Since there is a cue, not all added
                // files are processed immediately.
                // Receives `file`
                processing: function processing(file) {
                    if (file.previewElement) {
                        file.previewElement.classList.add("dz-processing");
                        if (file._removeLink) {
                            return file._removeLink.textContent = this.options.dictCancelUpload;
                        }
                    }
                },
                processingmultiple: function processingmultiple() {},


                // Called whenever the upload progress gets updated.
                // Receives `file`, `progress` (percentage 0-100) and `bytesSent`.
                // To get the total number of bytes of the file, use `file.size`
                uploadprogress: function uploadprogress(file, progress, bytesSent) {
                    if (file.previewElement) {
                        for (var _iterator8 = file.previewElement.querySelectorAll("[data-dz-uploadprogress]"), _isArray8 = true, _i8 = 0, _iterator8 = _isArray8 ? _iterator8 : _iterator8[Symbol.iterator]();;) {
                            var _ref7;

                            if (_isArray8) {
                                if (_i8 >= _iterator8.length) break;
                                _ref7 = _iterator8[_i8++];
                            } else {
                                _i8 = _iterator8.next();
                                if (_i8.done) break;
                                _ref7 = _i8.value;
                            }

                            var node = _ref7;

                            node.nodeName === 'PROGRESS' ? node.value = progress : node.style.width = progress + "%";
                        }
                    }
                },


                // Called whenever the total upload progress gets updated.
                // Called with totalUploadProgress (0-100), totalBytes and totalBytesSent
                totaluploadprogress: function totaluploadprogress() {},


                // Called just before the file is sent. Gets the `xhr` object as second
                // parameter, so you can modify it (for example to add a CSRF token) and a
                // `formData` object to add additional information.
                sending: function sending() {},
                sendingmultiple: function sendingmultiple() {},


                // When the complete upload is finished and successful
                // Receives `file`
                success: function success(file) {
                    if (file.previewElement) {
                        return file.previewElement.classList.add("dz-success");
                    }
                },
                successmultiple: function successmultiple() {},


                // When the upload is canceled.
                canceled: function canceled(file) {
                    return this.emit("error", file, this.options.dictUploadCanceled);
                },
                canceledmultiple: function canceledmultiple() {},


                // When the upload is finished, either with success or an error.
                // Receives `file`
                complete: function complete(file) {
                    if (file._removeLink) {
                        file._removeLink.textContent = this.options.dictRemoveFile;
                    }
                    if (file.previewElement) {
                        return file.previewElement.classList.add("dz-complete");
                    }
                },
                completemultiple: function completemultiple() {},
                maxfilesexceeded: function maxfilesexceeded() {},
                maxfilesreached: function maxfilesreached() {},
                queuecomplete: function queuecomplete() {},
                addedfiles: function addedfiles() {}
            };

            this.prototype._thumbnailQueue = [];
            this.prototype._processingThumbnail = false;
        }

        // global utility

    }, {
        key: "extend",
        value: function extend(target) {
            for (var _len2 = arguments.length, objects = Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
                objects[_key2 - 1] = arguments[_key2];
            }

            for (var _iterator9 = objects, _isArray9 = true, _i9 = 0, _iterator9 = _isArray9 ? _iterator9 : _iterator9[Symbol.iterator]();;) {
                var _ref8;

                if (_isArray9) {
                    if (_i9 >= _iterator9.length) break;
                    _ref8 = _iterator9[_i9++];
                } else {
                    _i9 = _iterator9.next();
                    if (_i9.done) break;
                    _ref8 = _i9.value;
                }

                var object = _ref8;

                for (var key in object) {
                    var val = object[key];
                    target[key] = val;
                }
            }
            return target;
        }
    }]);

    function Dropzone(el, options) {
        _classCallCheck(this, Dropzone);

        var _this = _possibleConstructorReturn(this, (Dropzone.__proto__ || Object.getPrototypeOf(Dropzone)).call(this));

        var fallback = void 0,
            left = void 0;
        _this.element = el;
        // For backwards compatibility since the version was in the prototype previously
        _this.version = Dropzone.version;

        _this.defaultOptions.previewTemplate = _this.defaultOptions.previewTemplate.replace(/\n*/g, "");

        _this.clickableElements = [];
        _this.listeners = [];
        _this.files = []; // All files

        if (typeof _this.element === "string") {
            _this.element = document.querySelector(_this.element);
        }

        // Not checking if instance of HTMLElement or Element since IE9 is extremely weird.
        if (!_this.element || _this.element.nodeType == null) {
            throw new Error("Invalid dropzone element.");
        }

        if (_this.element.dropzone) {
            throw new Error("Dropzone already attached.");
        }

        // Now add this dropzone to the instances.
        Dropzone.instances.push(_this);

        // Put the dropzone inside the element itself.
        _this.element.dropzone = _this;

        var elementOptions = (left = Dropzone.optionsForElement(_this.element)) != null ? left : {};

        _this.options = Dropzone.extend({}, _this.defaultOptions, elementOptions, options != null ? options : {});

        // If the browser failed, just call the fallback and leave
        if (_this.options.forceFallback || !Dropzone.isBrowserSupported()) {
            var _ret;

            return _ret = _this.options.fallback.call(_this), _possibleConstructorReturn(_this, _ret);
        }

        // @options.url = @element.getAttribute "action" unless @options.url?
        if (_this.options.url == null) {
            _this.options.url = _this.element.getAttribute("action");
        }

        if (!_this.options.url) {
            throw new Error("No URL provided.");
        }

        if (_this.options.acceptedFiles && _this.options.acceptedMimeTypes) {
            throw new Error("You can't provide both 'acceptedFiles' and 'acceptedMimeTypes'. 'acceptedMimeTypes' is deprecated.");
        }

        if (_this.options.uploadMultiple && _this.options.chunking) {
            throw new Error('You cannot set both: uploadMultiple and chunking.');
        }

        // Backwards compatibility
        if (_this.options.acceptedMimeTypes) {
            _this.options.acceptedFiles = _this.options.acceptedMimeTypes;
            delete _this.options.acceptedMimeTypes;
        }

        // Backwards compatibility
        if (_this.options.renameFilename != null) {
            _this.options.renameFile = function (file) {
                return _this.options.renameFilename.call(_this, file.name, file);
            };
        }

        _this.options.method = _this.options.method.toUpperCase();

        if ((fallback = _this.getExistingFallback()) && fallback.parentNode) {
            // Remove the fallback
            fallback.parentNode.removeChild(fallback);
        }

        // Display previews in the previewsContainer element or the Dropzone element unless explicitly set to false
        if (_this.options.previewsContainer !== false) {
            if (_this.options.previewsContainer) {
                _this.previewsContainer = Dropzone.getElement(_this.options.previewsContainer, "previewsContainer");
            } else {
                _this.previewsContainer = _this.element;
            }
        }

        if (_this.options.clickable) {
            if (_this.options.clickable === true) {
                _this.clickableElements = [_this.element];
            } else {
                _this.clickableElements = Dropzone.getElements(_this.options.clickable, "clickable");
            }
        }

        _this.init();
        return _this;
    }

    // Returns all files that have been accepted


    _createClass(Dropzone, [{
        key: "getAcceptedFiles",
        value: function getAcceptedFiles() {
            return this.files.filter(function (file) {
                return file.accepted;
            }).map(function (file) {
                return file;
            });
        }

        // Returns all files that have been rejected
        // Not sure when that's going to be useful, but added for completeness.

    }, {
        key: "getRejectedFiles",
        value: function getRejectedFiles() {
            return this.files.filter(function (file) {
                return !file.accepted;
            }).map(function (file) {
                return file;
            });
        }
    }, {
        key: "getFilesWithStatus",
        value: function getFilesWithStatus(status) {
            return this.files.filter(function (file) {
                return file.status === status;
            }).map(function (file) {
                return file;
            });
        }

        // Returns all files that are in the queue

    }, {
        key: "getQueuedFiles",
        value: function getQueuedFiles() {
            return this.getFilesWithStatus(Dropzone.QUEUED);
        }
    }, {
        key: "getUploadingFiles",
        value: function getUploadingFiles() {
            return this.getFilesWithStatus(Dropzone.UPLOADING);
        }
    }, {
        key: "getAddedFiles",
        value: function getAddedFiles() {
            return this.getFilesWithStatus(Dropzone.ADDED);
        }

        // Files that are either queued or uploading

    }, {
        key: "getActiveFiles",
        value: function getActiveFiles() {
            return this.files.filter(function (file) {
                return file.status === Dropzone.UPLOADING || file.status === Dropzone.QUEUED;
            }).map(function (file) {
                return file;
            });
        }

        // The function that gets called when Dropzone is initialized. You
        // can (and should) setup event listeners inside this function.

    }, {
        key: "init",
        value: function init() {
            var _this3 = this;

            // In case it isn't set already
            if (this.element.tagName === "form") {
                this.element.setAttribute("enctype", "multipart/form-data");
            }

            if (this.element.classList.contains("dropzone") && !this.element.querySelector(".dz-message")) {
                this.element.appendChild(Dropzone.createElement("<div class=\"dz-default dz-message\"><span>" + this.options.dictDefaultMessage + "</span></div>"));
            }

            if (this.clickableElements.length) {
                var setupHiddenFileInput = function setupHiddenFileInput() {
                    if (_this3.hiddenFileInput) {
                        _this3.hiddenFileInput.parentNode.removeChild(_this3.hiddenFileInput);
                    }
                    _this3.hiddenFileInput = document.createElement("input");
                    _this3.hiddenFileInput.setAttribute("type", "file");
                    if (_this3.options.maxFiles === null || _this3.options.maxFiles > 1) {
                        _this3.hiddenFileInput.setAttribute("multiple", "multiple");
                    }
                    _this3.hiddenFileInput.className = "dz-hidden-input";

                    if (_this3.options.acceptedFiles !== null) {
                        _this3.hiddenFileInput.setAttribute("accept", _this3.options.acceptedFiles);
                    }
                    if (_this3.options.capture !== null) {
                        _this3.hiddenFileInput.setAttribute("capture", _this3.options.capture);
                    }

                    // Not setting `display="none"` because some browsers don't accept clicks
                    // on elements that aren't displayed.
                    _this3.hiddenFileInput.style.visibility = "hidden";
                    _this3.hiddenFileInput.style.position = "absolute";
                    _this3.hiddenFileInput.style.top = "0";
                    _this3.hiddenFileInput.style.left = "0";
                    _this3.hiddenFileInput.style.height = "0";
                    _this3.hiddenFileInput.style.width = "0";
                    document.querySelector(_this3.options.hiddenInputContainer).appendChild(_this3.hiddenFileInput);
                    return _this3.hiddenFileInput.addEventListener("change", function () {
                        var files = _this3.hiddenFileInput.files;

                        if (files.length) {
                            for (var _iterator10 = files, _isArray10 = true, _i10 = 0, _iterator10 = _isArray10 ? _iterator10 : _iterator10[Symbol.iterator]();;) {
                                var _ref9;

                                if (_isArray10) {
                                    if (_i10 >= _iterator10.length) break;
                                    _ref9 = _iterator10[_i10++];
                                } else {
                                    _i10 = _iterator10.next();
                                    if (_i10.done) break;
                                    _ref9 = _i10.value;
                                }

                                var file = _ref9;

                                _this3.addFile(file);
                            }
                        }
                        _this3.emit("addedfiles", files);
                        return setupHiddenFileInput();
                    });
                };
                setupHiddenFileInput();
            }

            this.URL = window.URL !== null ? window.URL : window.webkitURL;

            // Setup all event listeners on the Dropzone object itself.
            // They're not in @setupEventListeners() because they shouldn't be removed
            // again when the dropzone gets disabled.
            for (var _iterator11 = this.events, _isArray11 = true, _i11 = 0, _iterator11 = _isArray11 ? _iterator11 : _iterator11[Symbol.iterator]();;) {
                var _ref10;

                if (_isArray11) {
                    if (_i11 >= _iterator11.length) break;
                    _ref10 = _iterator11[_i11++];
                } else {
                    _i11 = _iterator11.next();
                    if (_i11.done) break;
                    _ref10 = _i11.value;
                }

                var eventName = _ref10;

                this.on(eventName, this.options[eventName]);
            }

            this.on("uploadprogress", function () {
                return _this3.updateTotalUploadProgress();
            });

            this.on("removedfile", function () {
                return _this3.updateTotalUploadProgress();
            });

            this.on("canceled", function (file) {
                return _this3.emit("complete", file);
            });

            // Emit a `queuecomplete` event if all files finished uploading.
            this.on("complete", function (file) {
                if (_this3.getAddedFiles().length === 0 && _this3.getUploadingFiles().length === 0 && _this3.getQueuedFiles().length === 0) {
                    // This needs to be deferred so that `queuecomplete` really triggers after `complete`
                    return setTimeout(function () {
                        return _this3.emit("queuecomplete");
                    }, 0);
                }
            });

            var noPropagation = function noPropagation(e) {
                e.stopPropagation();
                if (e.preventDefault) {
                    return e.preventDefault();
                } else {
                    return e.returnValue = false;
                }
            };

            // Create the listeners
            this.listeners = [{
                element: this.element,
                events: {
                    "dragstart": function dragstart(e) {
                        return _this3.emit("dragstart", e);
                    },
                    "dragenter": function dragenter(e) {
                        noPropagation(e);
                        return _this3.emit("dragenter", e);
                    },
                    "dragover": function dragover(e) {
                        // Makes it possible to drag files from chrome's download bar
                        // http://stackoverflow.com/questions/19526430/drag-and-drop-file-uploads-from-chrome-downloads-bar
                        // Try is required to prevent bug in Internet Explorer 11 (SCRIPT65535 exception)
                        var efct = void 0;
                        try {
                            efct = e.dataTransfer.effectAllowed;
                        } catch (error) {}
                        e.dataTransfer.dropEffect = 'move' === efct || 'linkMove' === efct ? 'move' : 'copy';

                        noPropagation(e);
                        return _this3.emit("dragover", e);
                    },
                    "dragleave": function dragleave(e) {
                        return _this3.emit("dragleave", e);
                    },
                    "drop": function drop(e) {
                        noPropagation(e);
                        return _this3.drop(e);
                    },
                    "dragend": function dragend(e) {
                        return _this3.emit("dragend", e);
                    }

                    // This is disabled right now, because the browsers don't implement it properly.
                    // "paste": (e) =>
                    //   noPropagation e
                    //   @paste e
                } }];

            this.clickableElements.forEach(function (clickableElement) {
                return _this3.listeners.push({
                    element: clickableElement,
                    events: {
                        "click": function click(evt) {
                            // Only the actual dropzone or the message element should trigger file selection
                            if (clickableElement !== _this3.element || evt.target === _this3.element || Dropzone.elementInside(evt.target, _this3.element.querySelector(".dz-message"))) {
                                _this3.hiddenFileInput.click(); // Forward the click
                            }
                            return true;
                        }
                    }
                });
            });

            this.enable();

            return this.options.init.call(this);
        }

        // Not fully tested yet

    }, {
        key: "destroy",
        value: function destroy() {
            this.disable();
            this.removeAllFiles(true);
            if (this.hiddenFileInput != null ? this.hiddenFileInput.parentNode : undefined) {
                this.hiddenFileInput.parentNode.removeChild(this.hiddenFileInput);
                this.hiddenFileInput = null;
            }
            delete this.element.dropzone;
            return Dropzone.instances.splice(Dropzone.instances.indexOf(this), 1);
        }
    }, {
        key: "updateTotalUploadProgress",
        value: function updateTotalUploadProgress() {
            var totalUploadProgress = void 0;
            var totalBytesSent = 0;
            var totalBytes = 0;

            var activeFiles = this.getActiveFiles();

            if (activeFiles.length) {
                for (var _iterator12 = this.getActiveFiles(), _isArray12 = true, _i12 = 0, _iterator12 = _isArray12 ? _iterator12 : _iterator12[Symbol.iterator]();;) {
                    var _ref11;

                    if (_isArray12) {
                        if (_i12 >= _iterator12.length) break;
                        _ref11 = _iterator12[_i12++];
                    } else {
                        _i12 = _iterator12.next();
                        if (_i12.done) break;
                        _ref11 = _i12.value;
                    }

                    var file = _ref11;

                    totalBytesSent += file.upload.bytesSent;
                    totalBytes += file.upload.total;
                }
                totalUploadProgress = 100 * totalBytesSent / totalBytes;
            } else {
                totalUploadProgress = 100;
            }

            return this.emit("totaluploadprogress", totalUploadProgress, totalBytes, totalBytesSent);
        }

        // @options.paramName can be a function taking one parameter rather than a string.
        // A parameter name for a file is obtained simply by calling this with an index number.

    }, {
        key: "_getParamName",
        value: function _getParamName(n) {
            if (typeof this.options.paramName === "function") {
                return this.options.paramName(n);
            } else {
                return "" + this.options.paramName + (this.options.uploadMultiple ? "[" + n + "]" : "");
            }
        }

        // If @options.renameFile is a function,
        // the function will be used to rename the file.name before appending it to the formData

    }, {
        key: "_renameFile",
        value: function _renameFile(file) {
            if (typeof this.options.renameFile !== "function") {
                return file.name;
            }
            return this.options.renameFile(file);
        }

        // Returns a form that can be used as fallback if the browser does not support DragnDrop
        //
        // If the dropzone is already a form, only the input field and button are returned. Otherwise a complete form element is provided.
        // This code has to pass in IE7 :(

    }, {
        key: "getFallbackForm",
        value: function getFallbackForm() {
            var existingFallback = void 0,
                form = void 0;
            if (existingFallback = this.getExistingFallback()) {
                return existingFallback;
            }

            var fieldsString = "<div class=\"dz-fallback\">";
            if (this.options.dictFallbackText) {
                fieldsString += "<p>" + this.options.dictFallbackText + "</p>";
            }
            fieldsString += "<input type=\"file\" name=\"" + this._getParamName(0) + "\" " + (this.options.uploadMultiple ? 'multiple="multiple"' : undefined) + " /><input type=\"submit\" value=\"Upload!\"></div>";

            var fields = Dropzone.createElement(fieldsString);
            if (this.element.tagName !== "FORM") {
                form = Dropzone.createElement("<form action=\"" + this.options.url + "\" enctype=\"multipart/form-data\" method=\"" + this.options.method + "\"></form>");
                form.appendChild(fields);
            } else {
                // Make sure that the enctype and method attributes are set properly
                this.element.setAttribute("enctype", "multipart/form-data");
                this.element.setAttribute("method", this.options.method);
            }
            return form != null ? form : fields;
        }

        // Returns the fallback elements if they exist already
        //
        // This code has to pass in IE7 :(

    }, {
        key: "getExistingFallback",
        value: function getExistingFallback() {
            var getFallback = function getFallback(elements) {
                for (var _iterator13 = elements, _isArray13 = true, _i13 = 0, _iterator13 = _isArray13 ? _iterator13 : _iterator13[Symbol.iterator]();;) {
                    var _ref12;

                    if (_isArray13) {
                        if (_i13 >= _iterator13.length) break;
                        _ref12 = _iterator13[_i13++];
                    } else {
                        _i13 = _iterator13.next();
                        if (_i13.done) break;
                        _ref12 = _i13.value;
                    }

                    var el = _ref12;

                    if (/(^| )fallback($| )/.test(el.className)) {
                        return el;
                    }
                }
            };

            var _arr = ["div", "form"];
            for (var _i14 = 0; _i14 < _arr.length; _i14++) {
                var tagName = _arr[_i14];
                var fallback;
                if (fallback = getFallback(this.element.getElementsByTagName(tagName))) {
                    return fallback;
                }
            }
        }

        // Activates all listeners stored in @listeners

    }, {
        key: "setupEventListeners",
        value: function setupEventListeners() {
            return this.listeners.map(function (elementListeners) {
                return function () {
                    var result = [];
                    for (var event in elementListeners.events) {
                        var listener = elementListeners.events[event];
                        result.push(elementListeners.element.addEventListener(event, listener, false));
                    }
                    return result;
                }();
            });
        }

        // Deactivates all listeners stored in @listeners

    }, {
        key: "removeEventListeners",
        value: function removeEventListeners() {
            return this.listeners.map(function (elementListeners) {
                return function () {
                    var result = [];
                    for (var event in elementListeners.events) {
                        var listener = elementListeners.events[event];
                        result.push(elementListeners.element.removeEventListener(event, listener, false));
                    }
                    return result;
                }();
            });
        }

        // Removes all event listeners and cancels all files in the queue or being processed.

    }, {
        key: "disable",
        value: function disable() {
            var _this4 = this;

            this.clickableElements.forEach(function (element) {
                return element.classList.remove("dz-clickable");
            });
            this.removeEventListeners();
            this.disabled = true;

            return this.files.map(function (file) {
                return _this4.cancelUpload(file);
            });
        }
    }, {
        key: "enable",
        value: function enable() {
            delete this.disabled;
            this.clickableElements.forEach(function (element) {
                return element.classList.add("dz-clickable");
            });
            return this.setupEventListeners();
        }

        // Returns a nicely formatted filesize

    }, {
        key: "filesize",
        value: function filesize(size) {
            var selectedSize = 0;
            var selectedUnit = "b";

            if (size > 0) {
                var units = ['tb', 'gb', 'mb', 'kb', 'b'];

                for (var i = 0; i < units.length; i++) {
                    var unit = units[i];
                    var cutoff = Math.pow(this.options.filesizeBase, 4 - i) / 10;

                    if (size >= cutoff) {
                        selectedSize = size / Math.pow(this.options.filesizeBase, 4 - i);
                        selectedUnit = unit;
                        break;
                    }
                }

                selectedSize = Math.round(10 * selectedSize) / 10; // Cutting of digits
            }

            return "<strong>" + selectedSize + "</strong> " + this.options.dictFileSizeUnits[selectedUnit];
        }

        // Adds or removes the `dz-max-files-reached` class from the form.

    }, {
        key: "_updateMaxFilesReachedClass",
        value: function _updateMaxFilesReachedClass() {
            if (this.options.maxFiles != null && this.getAcceptedFiles().length >= this.options.maxFiles) {
                if (this.getAcceptedFiles().length === this.options.maxFiles) {
                    this.emit('maxfilesreached', this.files);
                }
                return this.element.classList.add("dz-max-files-reached");
            } else {
                return this.element.classList.remove("dz-max-files-reached");
            }
        }
    }, {
        key: "drop",
        value: function drop(e) {
            if (!e.dataTransfer) {
                return;
            }
            this.emit("drop", e);

            // Convert the FileList to an Array
            // This is necessary for IE11
            var files = [];
            for (var i = 0; i < e.dataTransfer.files.length; i++) {
                files[i] = e.dataTransfer.files[i];
            }

            this.emit("addedfiles", files);

            // Even if it's a folder, files.length will contain the folders.
            if (files.length) {
                var items = e.dataTransfer.items;

                if (items && items.length && items[0].webkitGetAsEntry != null) {
                    // The browser supports dropping of folders, so handle items instead of files
                    this._addFilesFromItems(items);
                } else {
                    this.handleFiles(files);
                }
            }
        }
    }, {
        key: "paste",
        value: function paste(e) {
            if (__guard__(e != null ? e.clipboardData : undefined, function (x) {
                return x.items;
            }) == null) {
                return;
            }

            this.emit("paste", e);
            var items = e.clipboardData.items;


            if (items.length) {
                return this._addFilesFromItems(items);
            }
        }
    }, {
        key: "handleFiles",
        value: function handleFiles(files) {
            for (var _iterator14 = files, _isArray14 = true, _i15 = 0, _iterator14 = _isArray14 ? _iterator14 : _iterator14[Symbol.iterator]();;) {
                var _ref13;

                if (_isArray14) {
                    if (_i15 >= _iterator14.length) break;
                    _ref13 = _iterator14[_i15++];
                } else {
                    _i15 = _iterator14.next();
                    if (_i15.done) break;
                    _ref13 = _i15.value;
                }

                var file = _ref13;

                this.addFile(file);
            }
        }

        // When a folder is dropped (or files are pasted), items must be handled
        // instead of files.

    }, {
        key: "_addFilesFromItems",
        value: function _addFilesFromItems(items) {
            var _this5 = this;

            return function () {
                var result = [];
                for (var _iterator15 = items, _isArray15 = true, _i16 = 0, _iterator15 = _isArray15 ? _iterator15 : _iterator15[Symbol.iterator]();;) {
                    var _ref14;

                    if (_isArray15) {
                        if (_i16 >= _iterator15.length) break;
                        _ref14 = _iterator15[_i16++];
                    } else {
                        _i16 = _iterator15.next();
                        if (_i16.done) break;
                        _ref14 = _i16.value;
                    }

                    var item = _ref14;

                    var entry;
                    if (item.webkitGetAsEntry != null && (entry = item.webkitGetAsEntry())) {
                        if (entry.isFile) {
                            result.push(_this5.addFile(item.getAsFile()));
                        } else if (entry.isDirectory) {
                            // Append all files from that directory to files
                            result.push(_this5._addFilesFromDirectory(entry, entry.name));
                        } else {
                            result.push(undefined);
                        }
                    } else if (item.getAsFile != null) {
                        if (item.kind == null || item.kind === "file") {
                            result.push(_this5.addFile(item.getAsFile()));
                        } else {
                            result.push(undefined);
                        }
                    } else {
                        result.push(undefined);
                    }
                }
                return result;
            }();
        }

        // Goes through the directory, and adds each file it finds recursively

    }, {
        key: "_addFilesFromDirectory",
        value: function _addFilesFromDirectory(directory, path) {
            var _this6 = this;

            var dirReader = directory.createReader();

            var errorHandler = function errorHandler(error) {
                return __guardMethod__(console, 'log', function (o) {
                    return o.log(error);
                });
            };

            var readEntries = function readEntries() {
                return dirReader.readEntries(function (entries) {
                    if (entries.length > 0) {
                        for (var _iterator16 = entries, _isArray16 = true, _i17 = 0, _iterator16 = _isArray16 ? _iterator16 : _iterator16[Symbol.iterator]();;) {
                            var _ref15;

                            if (_isArray16) {
                                if (_i17 >= _iterator16.length) break;
                                _ref15 = _iterator16[_i17++];
                            } else {
                                _i17 = _iterator16.next();
                                if (_i17.done) break;
                                _ref15 = _i17.value;
                            }

                            var entry = _ref15;

                            if (entry.isFile) {
                                entry.file(function (file) {
                                    if (_this6.options.ignoreHiddenFiles && file.name.substring(0, 1) === '.') {
                                        return;
                                    }
                                    file.fullPath = path + "/" + file.name;
                                    return _this6.addFile(file);
                                });
                            } else if (entry.isDirectory) {
                                _this6._addFilesFromDirectory(entry, path + "/" + entry.name);
                            }
                        }

                        // Recursively call readEntries() again, since browser only handle
                        // the first 100 entries.
                        // See: https://developer.mozilla.org/en-US/docs/Web/API/DirectoryReader#readEntries
                        readEntries();
                    }
                    return null;
                }, errorHandler);
            };

            return readEntries();
        }

        // If `done()` is called without argument the file is accepted
        // If you call it with an error message, the file is rejected
        // (This allows for asynchronous validation)
        //
        // This function checks the filesize, and if the file.type passes the
        // `acceptedFiles` check.

    }, {
        key: "accept",
        value: function accept(file, done) {
            if (this.options.maxFilesize && file.size > this.options.maxFilesize * 1024 * 1024) {
                return done(this.options.dictFileTooBig.replace("{{filesize}}", Math.round(file.size / 1024 / 10.24) / 100).replace("{{maxFilesize}}", this.options.maxFilesize));
            } else if (!Dropzone.isValidFile(file, this.options.acceptedFiles)) {
                return done(this.options.dictInvalidFileType);
            } else if (this.options.maxFiles != null && this.getAcceptedFiles().length >= this.options.maxFiles) {
                done(this.options.dictMaxFilesExceeded.replace("{{maxFiles}}", this.options.maxFiles));
                return this.emit("maxfilesexceeded", file);
            } else {
                return this.options.accept.call(this, file, done);
            }
        }
    }, {
        key: "addFile",
        value: function addFile(file) {
            var _this7 = this;

            file.upload = {
                uuid: Dropzone.uuidv4(),
                progress: 0,
                // Setting the total upload size to file.size for the beginning
                // It's actual different than the size to be transmitted.
                total: file.size,
                bytesSent: 0,
                filename: this._renameFile(file),
                chunked: this.options.chunking && (this.options.forceChunking || file.size > this.options.chunkSize),
                totalChunkCount: Math.ceil(file.size / this.options.chunkSize)
            };
            this.files.push(file);

            file.status = Dropzone.ADDED;

            this.emit("addedfile", file);

            this._enqueueThumbnail(file);

            return this.accept(file, function (error) {
                if (error) {
                    file.accepted = false;
                    _this7._errorProcessing([file], error); // Will set the file.status
                } else {
                    file.accepted = true;
                    if (_this7.options.autoQueue) {
                        _this7.enqueueFile(file);
                    } // Will set .accepted = true
                }
                return _this7._updateMaxFilesReachedClass();
            });
        }

        // Wrapper for enqueueFile

    }, {
        key: "enqueueFiles",
        value: function enqueueFiles(files) {
            for (var _iterator17 = files, _isArray17 = true, _i18 = 0, _iterator17 = _isArray17 ? _iterator17 : _iterator17[Symbol.iterator]();;) {
                var _ref16;

                if (_isArray17) {
                    if (_i18 >= _iterator17.length) break;
                    _ref16 = _iterator17[_i18++];
                } else {
                    _i18 = _iterator17.next();
                    if (_i18.done) break;
                    _ref16 = _i18.value;
                }

                var file = _ref16;

                this.enqueueFile(file);
            }
            return null;
        }
    }, {
        key: "enqueueFile",
        value: function enqueueFile(file) {
            var _this8 = this;

            if (file.status === Dropzone.ADDED && file.accepted === true) {
                file.status = Dropzone.QUEUED;
                if (this.options.autoProcessQueue) {
                    return setTimeout(function () {
                        return _this8.processQueue();
                    }, 0); // Deferring the call
                }
            } else {
                throw new Error("This file can't be queued because it has already been processed or was rejected.");
            }
        }
    }, {
        key: "_enqueueThumbnail",
        value: function _enqueueThumbnail(file) {
            var _this9 = this;

            if (this.options.createImageThumbnails && file.type.match(/image.*/) && file.size <= this.options.maxThumbnailFilesize * 1024 * 1024) {
                this._thumbnailQueue.push(file);
                return setTimeout(function () {
                    return _this9._processThumbnailQueue();
                }, 0); // Deferring the call
            }
        }
    }, {
        key: "_processThumbnailQueue",
        value: function _processThumbnailQueue() {
            var _this10 = this;

            if (this._processingThumbnail || this._thumbnailQueue.length === 0) {
                return;
            }

            this._processingThumbnail = true;
            var file = this._thumbnailQueue.shift();
            return this.createThumbnail(file, this.options.thumbnailWidth, this.options.thumbnailHeight, this.options.thumbnailMethod, true, function (dataUrl) {
                _this10.emit("thumbnail", file, dataUrl);
                _this10._processingThumbnail = false;
                return _this10._processThumbnailQueue();
            });
        }

        // Can be called by the user to remove a file

    }, {
        key: "removeFile",
        value: function removeFile(file) {
            if (file.status === Dropzone.UPLOADING) {
                this.cancelUpload(file);
            }
            this.files = without(this.files, file);

            this.emit("removedfile", file);
            if (this.files.length === 0) {
                return this.emit("reset");
            }
        }

        // Removes all files that aren't currently processed from the list

    }, {
        key: "removeAllFiles",
        value: function removeAllFiles(cancelIfNecessary) {
            // Create a copy of files since removeFile() changes the @files array.
            if (cancelIfNecessary == null) {
                cancelIfNecessary = false;
            }
            for (var _iterator18 = this.files.slice(), _isArray18 = true, _i19 = 0, _iterator18 = _isArray18 ? _iterator18 : _iterator18[Symbol.iterator]();;) {
                var _ref17;

                if (_isArray18) {
                    if (_i19 >= _iterator18.length) break;
                    _ref17 = _iterator18[_i19++];
                } else {
                    _i19 = _iterator18.next();
                    if (_i19.done) break;
                    _ref17 = _i19.value;
                }

                var file = _ref17;

                if (file.status !== Dropzone.UPLOADING || cancelIfNecessary) {
                    this.removeFile(file);
                }
            }
            return null;
        }

        // Resizes an image before it gets sent to the server. This function is the default behavior of
        // `options.transformFile` if `resizeWidth` or `resizeHeight` are set. The callback is invoked with
        // the resized blob.

    }, {
        key: "resizeImage",
        value: function resizeImage(file, width, height, resizeMethod, callback) {
            var _this11 = this;

            return this.createThumbnail(file, width, height, resizeMethod, false, function (dataUrl, canvas) {
                if (canvas == null) {
                    // The image has not been resized
                    return callback(file);
                } else {
                    var resizeMimeType = _this11.options.resizeMimeType;

                    if (resizeMimeType == null) {
                        resizeMimeType = file.type;
                    }
                    var resizedDataURL = canvas.toDataURL(resizeMimeType, _this11.options.resizeQuality);
                    if (resizeMimeType === 'image/jpeg' || resizeMimeType === 'image/jpg') {
                        // Now add the original EXIF information
                        resizedDataURL = ExifRestore.restore(file.dataURL, resizedDataURL);
                    }
                    return callback(Dropzone.dataURItoBlob(resizedDataURL));
                }
            });
        }
    }, {
        key: "createThumbnail",
        value: function createThumbnail(file, width, height, resizeMethod, fixOrientation, callback) {
            var _this12 = this;

            var fileReader = new FileReader();

            fileReader.onload = function () {

                file.dataURL = fileReader.result;

                // Don't bother creating a thumbnail for SVG images since they're vector
                if (file.type === "image/svg+xml") {
                    if (callback != null) {
                        callback(fileReader.result);
                    }
                    return;
                }

                return _this12.createThumbnailFromUrl(file, width, height, resizeMethod, fixOrientation, callback);
            };

            return fileReader.readAsDataURL(file);
        }
    }, {
        key: "createThumbnailFromUrl",
        value: function createThumbnailFromUrl(file, width, height, resizeMethod, fixOrientation, callback, crossOrigin) {
            var _this13 = this;

            // Not using `new Image` here because of a bug in latest Chrome versions.
            // See https://github.com/enyo/dropzone/pull/226
            var img = document.createElement("img");

            if (crossOrigin) {
                img.crossOrigin = crossOrigin;
            }

            img.onload = function () {
                var loadExif = function loadExif(callback) {
                    return callback(1);
                };
                if (typeof EXIF !== 'undefined' && EXIF !== null && fixOrientation) {
                    loadExif = function loadExif(callback) {
                        return EXIF.getData(img, function () {
                            return callback(EXIF.getTag(this, 'Orientation'));
                        });
                    };
                }

                return loadExif(function (orientation) {
                    file.width = img.width;
                    file.height = img.height;

                    var resizeInfo = _this13.options.resize.call(_this13, file, width, height, resizeMethod);

                    var canvas = document.createElement("canvas");
                    var ctx = canvas.getContext("2d");

                    canvas.width = resizeInfo.trgWidth;
                    canvas.height = resizeInfo.trgHeight;

                    if (orientation > 4) {
                        canvas.width = resizeInfo.trgHeight;
                        canvas.height = resizeInfo.trgWidth;
                    }

                    switch (orientation) {
                        case 2:
                            // horizontal flip
                            ctx.translate(canvas.width, 0);
                            ctx.scale(-1, 1);
                            break;
                        case 3:
                            // 180° rotate left
                            ctx.translate(canvas.width, canvas.height);
                            ctx.rotate(Math.PI);
                            break;
                        case 4:
                            // vertical flip
                            ctx.translate(0, canvas.height);
                            ctx.scale(1, -1);
                            break;
                        case 5:
                            // vertical flip + 90 rotate right
                            ctx.rotate(0.5 * Math.PI);
                            ctx.scale(1, -1);
                            break;
                        case 6:
                            // 90° rotate right
                            ctx.rotate(0.5 * Math.PI);
                            ctx.translate(0, -canvas.height);
                            break;
                        case 7:
                            // horizontal flip + 90 rotate right
                            ctx.rotate(0.5 * Math.PI);
                            ctx.translate(canvas.width, -canvas.height);
                            ctx.scale(-1, 1);
                            break;
                        case 8:
                            // 90° rotate left
                            ctx.rotate(-0.5 * Math.PI);
                            ctx.translate(-canvas.width, 0);
                            break;
                    }

                    // This is a bugfix for iOS' scaling bug.
                    drawImageIOSFix(ctx, img, resizeInfo.srcX != null ? resizeInfo.srcX : 0, resizeInfo.srcY != null ? resizeInfo.srcY : 0, resizeInfo.srcWidth, resizeInfo.srcHeight, resizeInfo.trgX != null ? resizeInfo.trgX : 0, resizeInfo.trgY != null ? resizeInfo.trgY : 0, resizeInfo.trgWidth, resizeInfo.trgHeight);

                    var thumbnail = canvas.toDataURL("image/png");

                    if (callback != null) {
                        return callback(thumbnail, canvas);
                    }
                });
            };

            if (callback != null) {
                img.onerror = callback;
            }

            return img.src = file.dataURL;
        }

        // Goes through the queue and processes files if there aren't too many already.

    }, {
        key: "processQueue",
        value: function processQueue() {
            var parallelUploads = this.options.parallelUploads;

            var processingLength = this.getUploadingFiles().length;
            var i = processingLength;

            // There are already at least as many files uploading than should be
            if (processingLength >= parallelUploads) {
                return;
            }

            var queuedFiles = this.getQueuedFiles();

            if (!(queuedFiles.length > 0)) {
                return;
            }

            if (this.options.uploadMultiple) {
                // The files should be uploaded in one request
                return this.processFiles(queuedFiles.slice(0, parallelUploads - processingLength));
            } else {
                while (i < parallelUploads) {
                    if (!queuedFiles.length) {
                        return;
                    } // Nothing left to process
                    this.processFile(queuedFiles.shift());
                    i++;
                }
            }
        }

        // Wrapper for `processFiles`

    }, {
        key: "processFile",
        value: function processFile(file) {
            return this.processFiles([file]);
        }

        // Loads the file, then calls finishedLoading()

    }, {
        key: "processFiles",
        value: function processFiles(files) {
            for (var _iterator19 = files, _isArray19 = true, _i20 = 0, _iterator19 = _isArray19 ? _iterator19 : _iterator19[Symbol.iterator]();;) {
                var _ref18;

                if (_isArray19) {
                    if (_i20 >= _iterator19.length) break;
                    _ref18 = _iterator19[_i20++];
                } else {
                    _i20 = _iterator19.next();
                    if (_i20.done) break;
                    _ref18 = _i20.value;
                }

                var file = _ref18;

                file.processing = true; // Backwards compatibility
                file.status = Dropzone.UPLOADING;

                this.emit("processing", file);
            }

            if (this.options.uploadMultiple) {
                this.emit("processingmultiple", files);
            }

            return this.uploadFiles(files);
        }
    }, {
        key: "_getFilesWithXhr",
        value: function _getFilesWithXhr(xhr) {
            var files = void 0;
            return files = this.files.filter(function (file) {
                return file.xhr === xhr;
            }).map(function (file) {
                return file;
            });
        }

        // Cancels the file upload and sets the status to CANCELED
        // **if** the file is actually being uploaded.
        // If it's still in the queue, the file is being removed from it and the status
        // set to CANCELED.

    }, {
        key: "cancelUpload",
        value: function cancelUpload(file) {
            if (file.status === Dropzone.UPLOADING) {
                var groupedFiles = this._getFilesWithXhr(file.xhr);
                for (var _iterator20 = groupedFiles, _isArray20 = true, _i21 = 0, _iterator20 = _isArray20 ? _iterator20 : _iterator20[Symbol.iterator]();;) {
                    var _ref19;

                    if (_isArray20) {
                        if (_i21 >= _iterator20.length) break;
                        _ref19 = _iterator20[_i21++];
                    } else {
                        _i21 = _iterator20.next();
                        if (_i21.done) break;
                        _ref19 = _i21.value;
                    }

                    var groupedFile = _ref19;

                    groupedFile.status = Dropzone.CANCELED;
                }
                if (typeof file.xhr !== 'undefined') {
                    file.xhr.abort();
                }
                for (var _iterator21 = groupedFiles, _isArray21 = true, _i22 = 0, _iterator21 = _isArray21 ? _iterator21 : _iterator21[Symbol.iterator]();;) {
                    var _ref20;

                    if (_isArray21) {
                        if (_i22 >= _iterator21.length) break;
                        _ref20 = _iterator21[_i22++];
                    } else {
                        _i22 = _iterator21.next();
                        if (_i22.done) break;
                        _ref20 = _i22.value;
                    }

                    var _groupedFile = _ref20;

                    this.emit("canceled", _groupedFile);
                }
                if (this.options.uploadMultiple) {
                    this.emit("canceledmultiple", groupedFiles);
                }
            } else if (file.status === Dropzone.ADDED || file.status === Dropzone.QUEUED) {
                file.status = Dropzone.CANCELED;
                this.emit("canceled", file);
                if (this.options.uploadMultiple) {
                    this.emit("canceledmultiple", [file]);
                }
            }

            if (this.options.autoProcessQueue) {
                return this.processQueue();
            }
        }
    }, {
        key: "resolveOption",
        value: function resolveOption(option) {
            if (typeof option === 'function') {
                for (var _len3 = arguments.length, args = Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
                    args[_key3 - 1] = arguments[_key3];
                }

                return option.apply(this, args);
            }
            return option;
        }
    }, {
        key: "uploadFile",
        value: function uploadFile(file) {
            return this.uploadFiles([file]);
        }
    }, {
        key: "uploadFiles",
        value: function uploadFiles(files) {
            var _this14 = this;

            this._transformFiles(files, function (transformedFiles) {
                if (files[0].upload.chunked) {
                    // This file should be sent in chunks!

                    // If the chunking option is set, we **know** that there can only be **one** file, since
                    // uploadMultiple is not allowed with this option.
                    var file = files[0];
                    var transformedFile = transformedFiles[0];
                    var startedChunkCount = 0;

                    file.upload.chunks = [];

                    var handleNextChunk = function handleNextChunk() {
                        var chunkIndex = 0;

                        // Find the next item in file.upload.chunks that is not defined yet.
                        while (file.upload.chunks[chunkIndex] !== undefined) {
                            chunkIndex++;
                        }

                        // This means, that all chunks have already been started.
                        if (chunkIndex >= file.upload.totalChunkCount) return;

                        startedChunkCount++;

                        var start = chunkIndex * _this14.options.chunkSize;
                        var end = Math.min(start + _this14.options.chunkSize, file.size);

                        var dataBlock = {
                            name: _this14._getParamName(0),
                            data: transformedFile.webkitSlice ? transformedFile.webkitSlice(start, end) : transformedFile.slice(start, end),
                            filename: file.upload.filename,
                            chunkIndex: chunkIndex
                        };

                        file.upload.chunks[chunkIndex] = {
                            file: file,
                            index: chunkIndex,
                            dataBlock: dataBlock, // In case we want to retry.
                            status: Dropzone.UPLOADING,
                            progress: 0,
                            retries: 0 // The number of times this block has been retried.
                        };

                        _this14._uploadData(files, [dataBlock]);
                    };

                    file.upload.finishedChunkUpload = function (chunk) {
                        var allFinished = true;
                        chunk.status = Dropzone.SUCCESS;

                        // Clear the data from the chunk
                        chunk.dataBlock = null;

                        for (var i = 0; i < file.upload.totalChunkCount; i++) {
                            if (file.upload.chunks[i] === undefined) {
                                return handleNextChunk();
                            }
                            if (file.upload.chunks[i].status !== Dropzone.SUCCESS) {
                                allFinished = false;
                            }
                        }

                        if (allFinished) {
                            _this14.options.chunksUploaded(file, function () {
                                _this14._finished(files, '', null);
                            });
                        }
                    };

                    if (_this14.options.parallelChunkUploads) {
                        for (var i = 0; i < file.upload.totalChunkCount; i++) {
                            handleNextChunk();
                        }
                    } else {
                        handleNextChunk();
                    }
                } else {
                    var dataBlocks = [];
                    for (var _i23 = 0; _i23 < files.length; _i23++) {
                        dataBlocks[_i23] = {
                            name: _this14._getParamName(_i23),
                            data: transformedFiles[_i23],
                            filename: files[_i23].upload.filename
                        };
                    }
                    _this14._uploadData(files, dataBlocks);
                }
            });
        }

        /// Returns the right chunk for given file and xhr

    }, {
        key: "_getChunk",
        value: function _getChunk(file, xhr) {
            for (var i = 0; i < file.upload.totalChunkCount; i++) {
                if (file.upload.chunks[i] !== undefined && file.upload.chunks[i].xhr === xhr) {
                    return file.upload.chunks[i];
                }
            }
        }

        // This function actually uploads the file(s) to the server.
        // If dataBlocks contains the actual data to upload (meaning, that this could either be transformed
        // files, or individual chunks for chunked upload).

    }, {
        key: "_uploadData",
        value: function _uploadData(files, dataBlocks) {
            var _this15 = this;

            var xhr = new XMLHttpRequest();

            // Put the xhr object in the file objects to be able to reference it later.
            for (var _iterator22 = files, _isArray22 = true, _i24 = 0, _iterator22 = _isArray22 ? _iterator22 : _iterator22[Symbol.iterator]();;) {
                var _ref21;

                if (_isArray22) {
                    if (_i24 >= _iterator22.length) break;
                    _ref21 = _iterator22[_i24++];
                } else {
                    _i24 = _iterator22.next();
                    if (_i24.done) break;
                    _ref21 = _i24.value;
                }

                var file = _ref21;

                file.xhr = xhr;
            }
            if (files[0].upload.chunked) {
                // Put the xhr object in the right chunk object, so it can be associated later, and found with _getChunk
                files[0].upload.chunks[dataBlocks[0].chunkIndex].xhr = xhr;
            }

            var method = this.resolveOption(this.options.method, files);
            var url = this.resolveOption(this.options.url, files);
            xhr.open(method, url, true);

            // Setting the timeout after open because of IE11 issue: https://gitlab.com/meno/dropzone/issues/8
            xhr.timeout = this.resolveOption(this.options.timeout, files);

            // Has to be after `.open()`. See https://github.com/enyo/dropzone/issues/179
            xhr.withCredentials = !!this.options.withCredentials;

            xhr.onload = function (e) {
                _this15._finishedUploading(files, xhr, e);
            };

            xhr.onerror = function () {
                _this15._handleUploadError(files, xhr);
            };

            // Some browsers do not have the .upload property
            var progressObj = xhr.upload != null ? xhr.upload : xhr;
            progressObj.onprogress = function (e) {
                return _this15._updateFilesUploadProgress(files, xhr, e);
            };

            var headers = {
                "Accept": "application/json",
                "Cache-Control": "no-cache",
                "X-Requested-With": "XMLHttpRequest"
            };

            if (this.options.headers) {
                Dropzone.extend(headers, this.options.headers);
            }

            for (var headerName in headers) {
                var headerValue = headers[headerName];
                if (headerValue) {
                    xhr.setRequestHeader(headerName, headerValue);
                }
            }

            var formData = new FormData();

            // Adding all @options parameters
            if (this.options.params) {
                var additionalParams = this.options.params;
                if (typeof additionalParams === 'function') {
                    additionalParams = additionalParams.call(this, files, xhr, files[0].upload.chunked ? this._getChunk(files[0], xhr) : null);
                }

                for (var key in additionalParams) {
                    var value = additionalParams[key];
                    formData.append(key, value);
                }
            }

            // Let the user add additional data if necessary
            for (var _iterator23 = files, _isArray23 = true, _i25 = 0, _iterator23 = _isArray23 ? _iterator23 : _iterator23[Symbol.iterator]();;) {
                var _ref22;

                if (_isArray23) {
                    if (_i25 >= _iterator23.length) break;
                    _ref22 = _iterator23[_i25++];
                } else {
                    _i25 = _iterator23.next();
                    if (_i25.done) break;
                    _ref22 = _i25.value;
                }

                var _file = _ref22;

                this.emit("sending", _file, xhr, formData);
            }
            if (this.options.uploadMultiple) {
                this.emit("sendingmultiple", files, xhr, formData);
            }

            this._addFormElementData(formData);

            // Finally add the files
            // Has to be last because some servers (eg: S3) expect the file to be the last parameter
            for (var i = 0; i < dataBlocks.length; i++) {
                var dataBlock = dataBlocks[i];
                formData.append(dataBlock.name, dataBlock.data, dataBlock.filename);
            }

            this.submitRequest(xhr, formData, files);
        }

        // Transforms all files with this.options.transformFile and invokes done with the transformed files when done.

    }, {
        key: "_transformFiles",
        value: function _transformFiles(files, done) {
            var _this16 = this;

            var transformedFiles = [];
            // Clumsy way of handling asynchronous calls, until I get to add a proper Future library.
            var doneCounter = 0;

            var _loop = function _loop(i) {
                _this16.options.transformFile.call(_this16, files[i], function (transformedFile) {
                    transformedFiles[i] = transformedFile;
                    if (++doneCounter === files.length) {
                        done(transformedFiles);
                    }
                });
            };

            for (var i = 0; i < files.length; i++) {
                _loop(i);
            }
        }

        // Takes care of adding other input elements of the form to the AJAX request

    }, {
        key: "_addFormElementData",
        value: function _addFormElementData(formData) {
            // Take care of other input elements
            if (this.element.tagName === "FORM") {
                for (var _iterator24 = this.element.querySelectorAll("input, textarea, select, button"), _isArray24 = true, _i26 = 0, _iterator24 = _isArray24 ? _iterator24 : _iterator24[Symbol.iterator]();;) {
                    var _ref23;

                    if (_isArray24) {
                        if (_i26 >= _iterator24.length) break;
                        _ref23 = _iterator24[_i26++];
                    } else {
                        _i26 = _iterator24.next();
                        if (_i26.done) break;
                        _ref23 = _i26.value;
                    }

                    var input = _ref23;

                    var inputName = input.getAttribute("name");
                    var inputType = input.getAttribute("type");
                    if (inputType) inputType = inputType.toLowerCase();

                    // If the input doesn't have a name, we can't use it.
                    if (typeof inputName === 'undefined' || inputName === null) continue;

                    if (input.tagName === "SELECT" && input.hasAttribute("multiple")) {
                        // Possibly multiple values
                        for (var _iterator25 = input.options, _isArray25 = true, _i27 = 0, _iterator25 = _isArray25 ? _iterator25 : _iterator25[Symbol.iterator]();;) {
                            var _ref24;

                            if (_isArray25) {
                                if (_i27 >= _iterator25.length) break;
                                _ref24 = _iterator25[_i27++];
                            } else {
                                _i27 = _iterator25.next();
                                if (_i27.done) break;
                                _ref24 = _i27.value;
                            }

                            var option = _ref24;

                            if (option.selected) {
                                formData.append(inputName, option.value);
                            }
                        }
                    } else if (!inputType || inputType !== "checkbox" && inputType !== "radio" || input.checked) {
                        formData.append(inputName, input.value);
                    }
                }
            }
        }

        // Invoked when there is new progress information about given files.
        // If e is not provided, it is assumed that the upload is finished.

    }, {
        key: "_updateFilesUploadProgress",
        value: function _updateFilesUploadProgress(files, xhr, e) {
            var progress = void 0;
            if (typeof e !== 'undefined') {
                progress = 100 * e.loaded / e.total;

                if (files[0].upload.chunked) {
                    var file = files[0];
                    // Since this is a chunked upload, we need to update the appropriate chunk progress.
                    var chunk = this._getChunk(file, xhr);
                    chunk.progress = progress;
                    chunk.total = e.total;
                    chunk.bytesSent = e.loaded;
                    var fileProgress = 0,
                        fileTotal = void 0,
                        fileBytesSent = void 0;
                    file.upload.progress = 0;
                    file.upload.total = 0;
                    file.upload.bytesSent = 0;
                    for (var i = 0; i < file.upload.totalChunkCount; i++) {
                        if (file.upload.chunks[i] !== undefined && file.upload.chunks[i].progress !== undefined) {
                            file.upload.progress += file.upload.chunks[i].progress;
                            file.upload.total += file.upload.chunks[i].total;
                            file.upload.bytesSent += file.upload.chunks[i].bytesSent;
                        }
                    }
                    file.upload.progress = file.upload.progress / file.upload.totalChunkCount;
                } else {
                    for (var _iterator26 = files, _isArray26 = true, _i28 = 0, _iterator26 = _isArray26 ? _iterator26 : _iterator26[Symbol.iterator]();;) {
                        var _ref25;

                        if (_isArray26) {
                            if (_i28 >= _iterator26.length) break;
                            _ref25 = _iterator26[_i28++];
                        } else {
                            _i28 = _iterator26.next();
                            if (_i28.done) break;
                            _ref25 = _i28.value;
                        }

                        var _file2 = _ref25;

                        _file2.upload.progress = progress;
                        _file2.upload.total = e.total;
                        _file2.upload.bytesSent = e.loaded;
                    }
                }
                for (var _iterator27 = files, _isArray27 = true, _i29 = 0, _iterator27 = _isArray27 ? _iterator27 : _iterator27[Symbol.iterator]();;) {
                    var _ref26;

                    if (_isArray27) {
                        if (_i29 >= _iterator27.length) break;
                        _ref26 = _iterator27[_i29++];
                    } else {
                        _i29 = _iterator27.next();
                        if (_i29.done) break;
                        _ref26 = _i29.value;
                    }

                    var _file3 = _ref26;

                    this.emit("uploadprogress", _file3, _file3.upload.progress, _file3.upload.bytesSent);
                }
            } else {
                // Called when the file finished uploading

                var allFilesFinished = true;

                progress = 100;

                for (var _iterator28 = files, _isArray28 = true, _i30 = 0, _iterator28 = _isArray28 ? _iterator28 : _iterator28[Symbol.iterator]();;) {
                    var _ref27;

                    if (_isArray28) {
                        if (_i30 >= _iterator28.length) break;
                        _ref27 = _iterator28[_i30++];
                    } else {
                        _i30 = _iterator28.next();
                        if (_i30.done) break;
                        _ref27 = _i30.value;
                    }

                    var _file4 = _ref27;

                    if (_file4.upload.progress !== 100 || _file4.upload.bytesSent !== _file4.upload.total) {
                        allFilesFinished = false;
                    }
                    _file4.upload.progress = progress;
                    _file4.upload.bytesSent = _file4.upload.total;
                }

                // Nothing to do, all files already at 100%
                if (allFilesFinished) {
                    return;
                }

                for (var _iterator29 = files, _isArray29 = true, _i31 = 0, _iterator29 = _isArray29 ? _iterator29 : _iterator29[Symbol.iterator]();;) {
                    var _ref28;

                    if (_isArray29) {
                        if (_i31 >= _iterator29.length) break;
                        _ref28 = _iterator29[_i31++];
                    } else {
                        _i31 = _iterator29.next();
                        if (_i31.done) break;
                        _ref28 = _i31.value;
                    }

                    var _file5 = _ref28;

                    this.emit("uploadprogress", _file5, progress, _file5.upload.bytesSent);
                }
            }
        }
    }, {
        key: "_finishedUploading",
        value: function _finishedUploading(files, xhr, e) {
            var response = void 0;

            if (files[0].status === Dropzone.CANCELED) {
                return;
            }

            if (xhr.readyState !== 4) {
                return;
            }

            if (xhr.responseType !== 'arraybuffer' && xhr.responseType !== 'blob') {
                response = xhr.responseText;

                if (xhr.getResponseHeader("content-type") && ~xhr.getResponseHeader("content-type").indexOf("application/json")) {
                    try {
                        response = JSON.parse(response);
                    } catch (error) {
                        e = error;
                        response = "Invalid JSON response from server.";
                    }
                }
            }

            this._updateFilesUploadProgress(files);

            if (!(200 <= xhr.status && xhr.status < 300)) {
                this._handleUploadError(files, xhr, response);
            } else {
                if (files[0].upload.chunked) {
                    files[0].upload.finishedChunkUpload(this._getChunk(files[0], xhr));
                } else {
                    this._finished(files, response, e);
                }
            }
        }
    }, {
        key: "_handleUploadError",
        value: function _handleUploadError(files, xhr, response) {
            if (files[0].status === Dropzone.CANCELED) {
                return;
            }

            if (files[0].upload.chunked && this.options.retryChunks) {
                var chunk = this._getChunk(files[0], xhr);
                if (chunk.retries++ < this.options.retryChunksLimit) {
                    this._uploadData(files, [chunk.dataBlock]);
                    return;
                } else {
                    console.warn('Retried this chunk too often. Giving up.');
                }
            }

            for (var _iterator30 = files, _isArray30 = true, _i32 = 0, _iterator30 = _isArray30 ? _iterator30 : _iterator30[Symbol.iterator]();;) {
                var _ref29;

                if (_isArray30) {
                    if (_i32 >= _iterator30.length) break;
                    _ref29 = _iterator30[_i32++];
                } else {
                    _i32 = _iterator30.next();
                    if (_i32.done) break;
                    _ref29 = _i32.value;
                }

                var file = _ref29;

                this._errorProcessing(files, response || this.options.dictResponseError.replace("{{statusCode}}", xhr.status), xhr);
            }
        }
    }, {
        key: "submitRequest",
        value: function submitRequest(xhr, formData, files) {
            xhr.send(formData);
        }

        // Called internally when processing is finished.
        // Individual callbacks have to be called in the appropriate sections.

    }, {
        key: "_finished",
        value: function _finished(files, responseText, e) {
            for (var _iterator31 = files, _isArray31 = true, _i33 = 0, _iterator31 = _isArray31 ? _iterator31 : _iterator31[Symbol.iterator]();;) {
                var _ref30;

                if (_isArray31) {
                    if (_i33 >= _iterator31.length) break;
                    _ref30 = _iterator31[_i33++];
                } else {
                    _i33 = _iterator31.next();
                    if (_i33.done) break;
                    _ref30 = _i33.value;
                }

                var file = _ref30;

                file.status = Dropzone.SUCCESS;
                this.emit("success", file, responseText, e);
                this.emit("complete", file);
            }
            if (this.options.uploadMultiple) {
                this.emit("successmultiple", files, responseText, e);
                this.emit("completemultiple", files);
            }

            if (this.options.autoProcessQueue) {
                return this.processQueue();
            }
        }

        // Called internally when processing is finished.
        // Individual callbacks have to be called in the appropriate sections.

    }, {
        key: "_errorProcessing",
        value: function _errorProcessing(files, message, xhr) {
            for (var _iterator32 = files, _isArray32 = true, _i34 = 0, _iterator32 = _isArray32 ? _iterator32 : _iterator32[Symbol.iterator]();;) {
                var _ref31;

                if (_isArray32) {
                    if (_i34 >= _iterator32.length) break;
                    _ref31 = _iterator32[_i34++];
                } else {
                    _i34 = _iterator32.next();
                    if (_i34.done) break;
                    _ref31 = _i34.value;
                }

                var file = _ref31;

                file.status = Dropzone.ERROR;
                this.emit("error", file, message, xhr);
                this.emit("complete", file);
            }
            if (this.options.uploadMultiple) {
                this.emit("errormultiple", files, message, xhr);
                this.emit("completemultiple", files);
            }

            if (this.options.autoProcessQueue) {
                return this.processQueue();
            }
        }
    }], [{
        key: "uuidv4",
        value: function uuidv4() {
            return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
                var r = Math.random() * 16 | 0,
                    v = c === 'x' ? r : r & 0x3 | 0x8;
                return v.toString(16);
            });
        }
    }]);

    return Dropzone;
}(Emitter);

Dropzone.initClass();

Dropzone.version = "5.4.0";

// This is a map of options for your different dropzones. Add configurations
// to this object for your different dropzone elemens.
//
// Example:
//
//     Dropzone.options.myDropzoneElementId = { maxFilesize: 1 };
//
// To disable autoDiscover for a specific element, you can set `false` as an option:
//
//     Dropzone.options.myDisabledElementId = false;
//
// And in html:
//
//     <form action="/upload" id="my-dropzone-element-id" class="dropzone"></form>
Dropzone.options = {};

// Returns the options for an element or undefined if none available.
Dropzone.optionsForElement = function (element) {
    // Get the `Dropzone.options.elementId` for this element if it exists
    if (element.getAttribute("id")) {
        return Dropzone.options[camelize(element.getAttribute("id"))];
    } else {
        return undefined;
    }
};

// Holds a list of all dropzone instances
Dropzone.instances = [];

// Returns the dropzone for given element if any
Dropzone.forElement = function (element) {
    if (typeof element === "string") {
        element = document.querySelector(element);
    }
    if ((element != null ? element.dropzone : undefined) == null) {
        throw new Error("No Dropzone found for given element. This is probably because you're trying to access it before Dropzone had the time to initialize. Use the `init` option to setup any additional observers on your Dropzone.");
    }
    return element.dropzone;
};

// Set to false if you don't want Dropzone to automatically find and attach to .dropzone elements.
Dropzone.autoDiscover = true;

// Looks for all .dropzone elements and creates a dropzone for them
Dropzone.discover = function () {
    var dropzones = void 0;
    if (document.querySelectorAll) {
        dropzones = document.querySelectorAll(".dropzone");
    } else {
        dropzones = [];
        // IE :(
        var checkElements = function checkElements(elements) {
            return function () {
                var result = [];
                for (var _iterator33 = elements, _isArray33 = true, _i35 = 0, _iterator33 = _isArray33 ? _iterator33 : _iterator33[Symbol.iterator]();;) {
                    var _ref32;

                    if (_isArray33) {
                        if (_i35 >= _iterator33.length) break;
                        _ref32 = _iterator33[_i35++];
                    } else {
                        _i35 = _iterator33.next();
                        if (_i35.done) break;
                        _ref32 = _i35.value;
                    }

                    var el = _ref32;

                    if (/(^| )dropzone($| )/.test(el.className)) {
                        result.push(dropzones.push(el));
                    } else {
                        result.push(undefined);
                    }
                }
                return result;
            }();
        };
        checkElements(document.getElementsByTagName("div"));
        checkElements(document.getElementsByTagName("form"));
    }

    return function () {
        var result = [];
        for (var _iterator34 = dropzones, _isArray34 = true, _i36 = 0, _iterator34 = _isArray34 ? _iterator34 : _iterator34[Symbol.iterator]();;) {
            var _ref33;

            if (_isArray34) {
                if (_i36 >= _iterator34.length) break;
                _ref33 = _iterator34[_i36++];
            } else {
                _i36 = _iterator34.next();
                if (_i36.done) break;
                _ref33 = _i36.value;
            }

            var dropzone = _ref33;

            // Create a dropzone unless auto discover has been disabled for specific element
            if (Dropzone.optionsForElement(dropzone) !== false) {
                result.push(new Dropzone(dropzone));
            } else {
                result.push(undefined);
            }
        }
        return result;
    }();
};

// Since the whole Drag'n'Drop API is pretty new, some browsers implement it,
// but not correctly.
// So I created a blacklist of userAgents. Yes, yes. Browser sniffing, I know.
// But what to do when browsers *theoretically* support an API, but crash
// when using it.
//
// This is a list of regular expressions tested against navigator.userAgent
//
// ** It should only be used on browser that *do* support the API, but
// incorrectly **
//
Dropzone.blacklistedBrowsers = [
// The mac os and windows phone version of opera 12 seems to have a problem with the File drag'n'drop API.
    /opera.*(Macintosh|Windows Phone).*version\/12/i];

// Checks if the browser is supported
Dropzone.isBrowserSupported = function () {
    var capableBrowser = true;

    if (window.File && window.FileReader && window.FileList && window.Blob && window.FormData && document.querySelector) {
        if (!("classList" in document.createElement("a"))) {
            capableBrowser = false;
        } else {
            // The browser supports the API, but may be blacklisted.
            for (var _iterator35 = Dropzone.blacklistedBrowsers, _isArray35 = true, _i37 = 0, _iterator35 = _isArray35 ? _iterator35 : _iterator35[Symbol.iterator]();;) {
                var _ref34;

                if (_isArray35) {
                    if (_i37 >= _iterator35.length) break;
                    _ref34 = _iterator35[_i37++];
                } else {
                    _i37 = _iterator35.next();
                    if (_i37.done) break;
                    _ref34 = _i37.value;
                }

                var regex = _ref34;

                if (regex.test(navigator.userAgent)) {
                    capableBrowser = false;
                    continue;
                }
            }
        }
    } else {
        capableBrowser = false;
    }

    return capableBrowser;
};

Dropzone.dataURItoBlob = function (dataURI) {
    // convert base64 to raw binary data held in a string
    // doesn't handle URLEncoded DataURIs - see SO answer #6850276 for code that does this
    var byteString = atob(dataURI.split(',')[1]);

    // separate out the mime component
    var mimeString = dataURI.split(',')[0].split(':')[1].split(';')[0];

    // write the bytes of the string to an ArrayBuffer
    var ab = new ArrayBuffer(byteString.length);
    var ia = new Uint8Array(ab);
    for (var i = 0, end = byteString.length, asc = 0 <= end; asc ? i <= end : i >= end; asc ? i++ : i--) {
        ia[i] = byteString.charCodeAt(i);
    }

    // write the ArrayBuffer to a blob
    return new Blob([ab], { type: mimeString });
};

// Returns an array without the rejected item
var without = function without(list, rejectedItem) {
    return list.filter(function (item) {
        return item !== rejectedItem;
    }).map(function (item) {
        return item;
    });
};

// abc-def_ghi -> abcDefGhi
var camelize = function camelize(str) {
    return str.replace(/[\-_](\w)/g, function (match) {
        return match.charAt(1).toUpperCase();
    });
};

// Creates an element from string
Dropzone.createElement = function (string) {
    var div = document.createElement("div");
    div.innerHTML = string;
    return div.childNodes[0];
};

// Tests if given element is inside (or simply is) the container
Dropzone.elementInside = function (element, container) {
    if (element === container) {
        return true;
    } // Coffeescript doesn't support do/while loops
    while (element = element.parentNode) {
        if (element === container) {
            return true;
        }
    }
    return false;
};

Dropzone.getElement = function (el, name) {
    var element = void 0;
    if (typeof el === "string") {
        element = document.querySelector(el);
    } else if (el.nodeType != null) {
        element = el;
    }
    if (element == null) {
        throw new Error("Invalid `" + name + "` option provided. Please provide a CSS selector or a plain HTML element.");
    }
    return element;
};

Dropzone.getElements = function (els, name) {
    var el = void 0,
        elements = void 0;
    if (els instanceof Array) {
        elements = [];
        try {
            for (var _iterator36 = els, _isArray36 = true, _i38 = 0, _iterator36 = _isArray36 ? _iterator36 : _iterator36[Symbol.iterator]();;) {
                if (_isArray36) {
                    if (_i38 >= _iterator36.length) break;
                    el = _iterator36[_i38++];
                } else {
                    _i38 = _iterator36.next();
                    if (_i38.done) break;
                    el = _i38.value;
                }

                elements.push(this.getElement(el, name));
            }
        } catch (e) {
            elements = null;
        }
    } else if (typeof els === "string") {
        elements = [];
        for (var _iterator37 = document.querySelectorAll(els), _isArray37 = true, _i39 = 0, _iterator37 = _isArray37 ? _iterator37 : _iterator37[Symbol.iterator]();;) {
            if (_isArray37) {
                if (_i39 >= _iterator37.length) break;
                el = _iterator37[_i39++];
            } else {
                _i39 = _iterator37.next();
                if (_i39.done) break;
                el = _i39.value;
            }

            elements.push(el);
        }
    } else if (els.nodeType != null) {
        elements = [els];
    }

    if (elements == null || !elements.length) {
        throw new Error("Invalid `" + name + "` option provided. Please provide a CSS selector, a plain HTML element or a list of those.");
    }

    return elements;
};

// Asks the user the question and calls accepted or rejected accordingly
//
// The default implementation just uses `window.confirm` and then calls the
// appropriate callback.
Dropzone.confirm = function (question, accepted, rejected) {
    if (window.confirm(question)) {
        return accepted();
    } else if (rejected != null) {
        return rejected();
    }
};

// Validates the mime type like this:
//
// https://developer.mozilla.org/en-US/docs/HTML/Element/input#attr-accept
Dropzone.isValidFile = function (file, acceptedFiles) {
    if (!acceptedFiles) {
        return true;
    } // If there are no accepted mime types, it's OK
    acceptedFiles = acceptedFiles.split(",");

    var mimeType = file.type;
    var baseMimeType = mimeType.replace(/\/.*$/, "");

    for (var _iterator38 = acceptedFiles, _isArray38 = true, _i40 = 0, _iterator38 = _isArray38 ? _iterator38 : _iterator38[Symbol.iterator]();;) {
        var _ref35;

        if (_isArray38) {
            if (_i40 >= _iterator38.length) break;
            _ref35 = _iterator38[_i40++];
        } else {
            _i40 = _iterator38.next();
            if (_i40.done) break;
            _ref35 = _i40.value;
        }

        var validType = _ref35;

        validType = validType.trim();
        if (validType.charAt(0) === ".") {
            if (file.name.toLowerCase().indexOf(validType.toLowerCase(), file.name.length - validType.length) !== -1) {
                return true;
            }
        } else if (/\/\*$/.test(validType)) {
            // This is something like a image/* mime type
            if (baseMimeType === validType.replace(/\/.*$/, "")) {
                return true;
            }
        } else {
            if (mimeType === validType) {
                return true;
            }
        }
    }

    return false;
};

// Augment jQuery
if (typeof jQuery !== 'undefined' && jQuery !== null) {
    jQuery.fn.dropzone = function (options) {
        return this.each(function () {
            return new Dropzone(this, options);
        });
    };
}

if (typeof module !== 'undefined' && module !== null) {
    module.exports = Dropzone;
} else {
    window.Dropzone = Dropzone;
}

// Dropzone file status codes
Dropzone.ADDED = "added";

Dropzone.QUEUED = "queued";
// For backwards compatibility. Now, if a file is accepted, it's either queued
// or uploading.
Dropzone.ACCEPTED = Dropzone.QUEUED;

Dropzone.UPLOADING = "uploading";
Dropzone.PROCESSING = Dropzone.UPLOADING; // alias

Dropzone.CANCELED = "canceled";
Dropzone.ERROR = "error";
Dropzone.SUCCESS = "success";

/*

 Bugfix for iOS 6 and 7
 Source: http://stackoverflow.com/questions/11929099/html5-canvas-drawimage-ratio-bug-ios
 based on the work of https://github.com/stomita/ios-imagefile-megapixel

 */

// Detecting vertical squash in loaded image.
// Fixes a bug which squash image vertically while drawing into canvas for some images.
// This is a bug in iOS6 devices. This function from https://github.com/stomita/ios-imagefile-megapixel
var detectVerticalSquash = function detectVerticalSquash(img) {
    var iw = img.naturalWidth;
    var ih = img.naturalHeight;
    var canvas = document.createElement("canvas");
    canvas.width = 1;
    canvas.height = ih;
    var ctx = canvas.getContext("2d");
    ctx.drawImage(img, 0, 0);

    var _ctx$getImageData = ctx.getImageData(1, 0, 1, ih),
        data = _ctx$getImageData.data;

    // search image edge pixel position in case it is squashed vertically.


    var sy = 0;
    var ey = ih;
    var py = ih;
    while (py > sy) {
        var alpha = data[(py - 1) * 4 + 3];

        if (alpha === 0) {
            ey = py;
        } else {
            sy = py;
        }

        py = ey + sy >> 1;
    }
    var ratio = py / ih;

    if (ratio === 0) {
        return 1;
    } else {
        return ratio;
    }
};

// A replacement for context.drawImage
// (args are for source and destination).
var drawImageIOSFix = function drawImageIOSFix(ctx, img, sx, sy, sw, sh, dx, dy, dw, dh) {
    var vertSquashRatio = detectVerticalSquash(img);
    return ctx.drawImage(img, sx, sy, sw, sh, dx, dy, dw, dh / vertSquashRatio);
};

// Based on MinifyJpeg
// Source: http://www.perry.cz/files/ExifRestorer.js
// http://elicon.blog57.fc2.com/blog-entry-206.html

var ExifRestore = function () {
    function ExifRestore() {
        _classCallCheck(this, ExifRestore);
    }

    _createClass(ExifRestore, null, [{
        key: "initClass",
        value: function initClass() {
            this.KEY_STR = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=';
        }
    }, {
        key: "encode64",
        value: function encode64(input) {
            var output = '';
            var chr1 = undefined;
            var chr2 = undefined;
            var chr3 = '';
            var enc1 = undefined;
            var enc2 = undefined;
            var enc3 = undefined;
            var enc4 = '';
            var i = 0;
            while (true) {
                chr1 = input[i++];
                chr2 = input[i++];
                chr3 = input[i++];
                enc1 = chr1 >> 2;
                enc2 = (chr1 & 3) << 4 | chr2 >> 4;
                enc3 = (chr2 & 15) << 2 | chr3 >> 6;
                enc4 = chr3 & 63;
                if (isNaN(chr2)) {
                    enc3 = enc4 = 64;
                } else if (isNaN(chr3)) {
                    enc4 = 64;
                }
                output = output + this.KEY_STR.charAt(enc1) + this.KEY_STR.charAt(enc2) + this.KEY_STR.charAt(enc3) + this.KEY_STR.charAt(enc4);
                chr1 = chr2 = chr3 = '';
                enc1 = enc2 = enc3 = enc4 = '';
                if (!(i < input.length)) {
                    break;
                }
            }
            return output;
        }
    }, {
        key: "restore",
        value: function restore(origFileBase64, resizedFileBase64) {
            if (!origFileBase64.match('data:image/jpeg;base64,')) {
                return resizedFileBase64;
            }
            var rawImage = this.decode64(origFileBase64.replace('data:image/jpeg;base64,', ''));
            var segments = this.slice2Segments(rawImage);
            var image = this.exifManipulation(resizedFileBase64, segments);
            return "data:image/jpeg;base64," + this.encode64(image);
        }
    }, {
        key: "exifManipulation",
        value: function exifManipulation(resizedFileBase64, segments) {
            var exifArray = this.getExifArray(segments);
            var newImageArray = this.insertExif(resizedFileBase64, exifArray);
            var aBuffer = new Uint8Array(newImageArray);
            return aBuffer;
        }
    }, {
        key: "getExifArray",
        value: function getExifArray(segments) {
            var seg = undefined;
            var x = 0;
            while (x < segments.length) {
                seg = segments[x];
                if (seg[0] === 255 & seg[1] === 225) {
                    return seg;
                }
                x++;
            }
            return [];
        }
    }, {
        key: "insertExif",
        value: function insertExif(resizedFileBase64, exifArray) {
            var imageData = resizedFileBase64.replace('data:image/jpeg;base64,', '');
            var buf = this.decode64(imageData);
            var separatePoint = buf.indexOf(255, 3);
            var mae = buf.slice(0, separatePoint);
            var ato = buf.slice(separatePoint);
            var array = mae;
            array = array.concat(exifArray);
            array = array.concat(ato);
            return array;
        }
    }, {
        key: "slice2Segments",
        value: function slice2Segments(rawImageArray) {
            var head = 0;
            var segments = [];
            while (true) {
                var length;
                if (rawImageArray[head] === 255 & rawImageArray[head + 1] === 218) {
                    break;
                }
                if (rawImageArray[head] === 255 & rawImageArray[head + 1] === 216) {
                    head += 2;
                } else {
                    length = rawImageArray[head + 2] * 256 + rawImageArray[head + 3];
                    var endPoint = head + length + 2;
                    var seg = rawImageArray.slice(head, endPoint);
                    segments.push(seg);
                    head = endPoint;
                }
                if (head > rawImageArray.length) {
                    break;
                }
            }
            return segments;
        }
    }, {
        key: "decode64",
        value: function decode64(input) {
            var output = '';
            var chr1 = undefined;
            var chr2 = undefined;
            var chr3 = '';
            var enc1 = undefined;
            var enc2 = undefined;
            var enc3 = undefined;
            var enc4 = '';
            var i = 0;
            var buf = [];
            // remove all characters that are not A-Z, a-z, 0-9, +, /, or =
            var base64test = /[^A-Za-z0-9\+\/\=]/g;
            if (base64test.exec(input)) {
                console.warn('There were invalid base64 characters in the input text.\nValid base64 characters are A-Z, a-z, 0-9, \'+\', \'/\',and \'=\'\nExpect errors in decoding.');
            }
            input = input.replace(/[^A-Za-z0-9\+\/\=]/g, '');
            while (true) {
                enc1 = this.KEY_STR.indexOf(input.charAt(i++));
                enc2 = this.KEY_STR.indexOf(input.charAt(i++));
                enc3 = this.KEY_STR.indexOf(input.charAt(i++));
                enc4 = this.KEY_STR.indexOf(input.charAt(i++));
                chr1 = enc1 << 2 | enc2 >> 4;
                chr2 = (enc2 & 15) << 4 | enc3 >> 2;
                chr3 = (enc3 & 3) << 6 | enc4;
                buf.push(chr1);
                if (enc3 !== 64) {
                    buf.push(chr2);
                }
                if (enc4 !== 64) {
                    buf.push(chr3);
                }
                chr1 = chr2 = chr3 = '';
                enc1 = enc2 = enc3 = enc4 = '';
                if (!(i < input.length)) {
                    break;
                }
            }
            return buf;
        }
    }]);

    return ExifRestore;
}();

ExifRestore.initClass();

/*
 * contentloaded.js
 *
 * Author: Diego Perini (diego.perini at gmail.com)
 * Summary: cross-browser wrapper for DOMContentLoaded
 * Updated: 20101020
 * License: MIT
 * Version: 1.2
 *
 * URL:
 * http://javascript.nwbox.com/ContentLoaded/
 * http://javascript.nwbox.com/ContentLoaded/MIT-LICENSE
 */

// @win window reference
// @fn function reference
var contentLoaded = function contentLoaded(win, fn) {
    var done = false;
    var top = true;
    var doc = win.document;
    var root = doc.documentElement;
    var add = doc.addEventListener ? "addEventListener" : "attachEvent";
    var rem = doc.addEventListener ? "removeEventListener" : "detachEvent";
    var pre = doc.addEventListener ? "" : "on";
    var init = function init(e) {
        if (e.type === "readystatechange" && doc.readyState !== "complete") {
            return;
        }
        (e.type === "load" ? win : doc)[rem](pre + e.type, init, false);
        if (!done && (done = true)) {
            return fn.call(win, e.type || e);
        }
    };

    var poll = function poll() {
        try {
            root.doScroll("left");
        } catch (e) {
            setTimeout(poll, 50);
            return;
        }
        return init("poll");
    };

    if (doc.readyState !== "complete") {
        if (doc.createEventObject && root.doScroll) {
            try {
                top = !win.frameElement;
            } catch (error) {}
            if (top) {
                poll();
            }
        }
        doc[add](pre + "DOMContentLoaded", init, false);
        doc[add](pre + "readystatechange", init, false);
        return win[add](pre + "load", init, false);
    }
};

// As a single function to be able to write tests.
Dropzone._autoDiscoverFunction = function () {
    if (Dropzone.autoDiscover) {
        return Dropzone.discover();
    }
};
contentLoaded(window, Dropzone._autoDiscoverFunction);

function __guard__(value, transform) {
    return typeof value !== 'undefined' && value !== null ? transform(value) : undefined;
}
function __guardMethod__(obj, methodName, transform) {
    if (typeof obj !== 'undefined' && obj !== null && typeof obj[methodName] === 'function') {
        return transform(obj, methodName);
    } else {
        return undefined;
    }
}

/*! Idle Timer - v1.0.1 - 2014-03-21
 * https://github.com/thorst/jquery-idletimer
 * Copyright (c) 2014 Paul Irish; Licensed MIT */
/*
 mousewheel (deprecated) -> IE6.0, Chrome, Opera, Safari
 DOMMouseScroll (deprecated) -> Firefox 1.0
 wheel (standard) -> Chrome 31, Firefox 17, IE9, Firefox Mobile 17.0

 //No need to use, use DOMMouseScroll
 MozMousePixelScroll -> Firefox 3.5, Firefox Mobile 1.0

 //Events
 WheelEvent -> see wheel
 MouseWheelEvent -> see mousewheel
 MouseScrollEvent -> Firefox 3.5, Firefox Mobile 1.0
 */
(function ($) {

    $.idleTimer = function (firstParam, elem) {
        var opts;
        if ( typeof firstParam === "object" ) {
            opts = firstParam;
            firstParam = null;
        } else if (typeof firstParam === "number") {
            opts = { timeout: firstParam };
            firstParam = null;
        }

        // element to watch
        elem = elem || document;

        // defaults that are to be stored as instance props on the elem
        opts = $.extend({
            idle: false,                // indicates if the user is idle
            timeout: 30000,             // the amount of time (ms) before the user is considered idle
            events: "mousemove keydown wheel DOMMouseScroll mousewheel mousedown touchstart touchmove MSPointerDown MSPointerMove" // define active events
        }, opts);

        var jqElem = $(elem),
            obj = jqElem.data("idleTimerObj") || {},

        /* (intentionally not documented)
         * Toggles the idle state and fires an appropriate event.
         * @return {void}
         */
            toggleIdleState = function (e) {

                var obj = $.data(elem, "idleTimerObj") || {};

                // toggle the state
                obj.idle = !obj.idle;

                // store toggle state date time
                obj.olddate = +new Date();

                // create a custom event, with state and name space
                var event = $.Event((obj.idle ? "idle" : "active") + ".idleTimer");

                // trigger event on object with elem and copy of obj
                $(elem).trigger(event, [elem, $.extend({}, obj), e]);
            },
            /**
             * Handle event triggers
             * @return {void}
             * @method event
             * @static
             */
            handleEvent = function (e) {

                var obj = $.data(elem, "idleTimerObj") || {};

                // this is already paused, ignore events for now
                if (obj.remaining != null) { return; }

                /*
                 mousemove is kinda buggy, it can be triggered when it should be idle.
                 Typically is happening between 115 - 150 milliseconds after idle triggered.
                 @psyafter & @kaellis report "always triggered if using modal (jQuery ui, with overlay)"
                 @thorst has similar issues on ios7 "after $.scrollTop() on text area"
                 */
                if (e.type === "mousemove") {
                    // if coord are same, it didn't move
                    if (e.pageX === obj.pageX && e.pageY === obj.pageY) {
                        return;
                    }
                    // if coord don't exist how could it move
                    if (typeof e.pageX === "undefined" && typeof e.pageY === "undefined") {
                        return;
                    }
                    // under 200 ms is hard to do, and you would have to stop, as continuous activity will bypass this
                    var elapsed = (+new Date()) - obj.olddate;
                    if (elapsed < 200) {
                        return;
                    }
                }

                // clear any existing timeout
                clearTimeout(obj.tId);

                // if the idle timer is enabled, flip
                if (obj.idle) {
                    toggleIdleState(e);
                }

                // store when user was last active
                obj.lastActive = +new Date();

                // update mouse coord
                obj.pageX = e.pageX;
                obj.pageY = e.pageY;

                // set a new timeout
                obj.tId = setTimeout(toggleIdleState, obj.timeout);

            },
            /**
             * Restore initial settings and restart timer
             * @return {void}
             * @method reset
             * @static
             */
            reset = function () {

                var obj = $.data(elem, "idleTimerObj") || {};

                // reset settings
                obj.idle = obj.idleBackup;
                obj.olddate = +new Date();
                obj.lastActive = obj.olddate;
                obj.remaining = null;

                // reset Timers
                clearTimeout(obj.tId);
                if (!obj.idle) {
                    obj.tId = setTimeout(toggleIdleState, obj.timeout);
                }

            },
            /**
             * Store remaining time, stop timer
             * You can pause from an idle OR active state
             * @return {void}
             * @method pause
             * @static
             */
            pause = function () {

                var obj = $.data(elem, "idleTimerObj") || {};

                // this is already paused
                if ( obj.remaining != null ) { return; }

                // define how much is left on the timer
                obj.remaining = obj.timeout - ((+new Date()) - obj.olddate);

                // clear any existing timeout
                clearTimeout(obj.tId);
            },
            /**
             * Start timer with remaining value
             * @return {void}
             * @method resume
             * @static
             */
            resume = function () {

                var obj = $.data(elem, "idleTimerObj") || {};

                // this isn't paused yet
                if ( obj.remaining == null ) { return; }

                // start timer
                if ( !obj.idle ) {
                    obj.tId = setTimeout(toggleIdleState, obj.remaining);
                }

                // clear remaining
                obj.remaining = null;
            },
            /**
             * Stops the idle timer. This removes appropriate event handlers
             * and cancels any pending timeouts.
             * @return {void}
             * @method destroy
             * @static
             */
            destroy = function () {

                var obj = $.data(elem, "idleTimerObj") || {};

                //clear any pending timeouts
                clearTimeout(obj.tId);

                //Remove data
                jqElem.removeData("idleTimerObj");

                //detach the event handlers
                jqElem.off("._idleTimer");
            },
            /**
             * Returns the time until becoming idle
             * @return {number}
             * @method remainingtime
             * @static
             */
            remainingtime = function () {

                var obj = $.data(elem, "idleTimerObj") || {};

                //If idle there is no time remaining
                if ( obj.idle ) { return 0; }

                //If its paused just return that
                if ( obj.remaining != null ) { return obj.remaining; }

                //Determine remaining, if negative idle didn't finish flipping, just return 0
                var remaining = obj.timeout - ((+new Date()) - obj.lastActive);
                if (remaining < 0) { remaining = 0; }

                //If this is paused return that number, else return current remaining
                return remaining;
            };


        // determine which function to call
        if (firstParam === null && typeof obj.idle !== "undefined") {
            // they think they want to init, but it already is, just reset
            reset();
            return jqElem;
        } else if (firstParam === null) {
            // they want to init
        } else if (firstParam !== null && typeof obj.idle === "undefined") {
            // they want to do something, but it isnt init
            // not sure the best way to handle this
            return false;
        } else if (firstParam === "destroy") {
            destroy();
            return jqElem;
        } else if (firstParam === "pause") {
            pause();
            return jqElem;
        } else if (firstParam === "resume") {
            resume();
            return jqElem;
        } else if (firstParam === "reset") {
            reset();
            return jqElem;
        } else if (firstParam === "getRemainingTime") {
            return remainingtime();
        } else if (firstParam === "getElapsedTime") {
            return (+new Date()) - obj.olddate;
        } else if (firstParam === "getLastActiveTime") {
            return obj.lastActive;
        } else if (firstParam === "isIdle") {
            return obj.idle;
        }

        /* (intentionally not documented)
         * Handles a user event indicating that the user isn't idle. namespaced with internal idleTimer
         * @param {Event} event A DOM2-normalized event object.
         * @return {void}
         */
        jqElem.on($.trim((opts.events + " ").split(" ").join("._idleTimer ")), function (e) {
            handleEvent(e);
        });


        // Internal Object Properties, This isn't all necessary, but we
        // explicitly define all keys here so we know what we are working with
        obj = $.extend({}, {
            olddate : +new Date(),          // the last time state changed
            lastActive: +new Date(),       // the last time timer was active
            idle : opts.idle,               // current state
            idleBackup : opts.idle,         // backup of idle parameter since it gets modified
            timeout : opts.timeout,         // the interval to change state
            remaining : null,               // how long until state changes
            tId : null,                     // the idle timer setTimeout
            pageX : null,                   // used to store the mouse coord
            pageY : null
        });

        // set a timeout to toggle state. May wish to omit this in some situations
        if (!obj.idle) {
            obj.tId = setTimeout(toggleIdleState, obj.timeout);
        }

        // store our instance on the object
        $.data(elem, "idleTimerObj", obj);

        return jqElem;
    };

    // This allows binding to element
    $.fn.idleTimer = function (firstParam) {
        if (this[0]) {
            return $.idleTimer(firstParam, this[0]);
        }

        return this;
    };

})(jQuery);
/*global define:false */
/*
 * =======================
 * jQuery Timer Plugin
 * =======================
 * Start/Stop/Resume a time in any HTML element
 */

(function(root, factory) {
    if (typeof define === 'function' && define.amd) {
        define(['jquery'], factory);
    } else {
        factory(root.jQuery);
    }
}(this, function($) {
    // PRIVATE
    var options = {
            seconds: 0,									// default seconds value to start timer from
            editable: false,							// this will let users make changes to the time
            restart: false,								// this will enable stop or continue after a timer callback
            duration: null,								// duration to run callback after
            // callback to run after elapsed duration
            callback: function() {
                alert('Time up!');
            },
            startTimer: function() {},
            pauseTimer: function() {},
            resumeTimer: function() {},
            resetTimer: function() {},
            removeTimer: function() {},
            repeat: false,								// this will repeat callback every n times duration is elapsed
            countdown: false,							// if true, this will render the timer as a countdown if duration > 0
            format: null,								// this sets the format in which the time will be printed
            updateFrequency: 1000,						// How often should timer display update (default 500ms)
            state: 'running'
        },
        display = 'html',	// to be used as $el.html in case of div and $el.val in case of input type text
    // Constants for various states of the timer
        TIMER_STOPPED = 'stopped',
        TIMER_RUNNING = 'running',
        TIMER_PAUSED = 'paused';

    /**
     * Common function to start or resume a timer interval
     */
    function startTimerInterval(timer) {
        var element = timer.element;
        $(element).data('intr', setInterval(incrementSeconds.bind(timer), timer.options.updateFrequency));
        $(element).data('isTimerRunning', true);
    }

    /**
     * Common function to stop timer interval
     */
    function stopTimerInterval(timer) {
        clearInterval($(timer.element).data('intr'));
        $(timer.element).data('isTimerRunning', false);
    }

    /**
     * Increment total seconds by subtracting startTime from the current unix timestamp in seconds
     * and call render to display pretty time
     */
    function incrementSeconds() {
        $(this.element).data('totalSeconds', getUnixSeconds() - $(this.element).data('startTime'));
        render(this);

        // Check if totalSeconds is equal to duration if any
        if ($(this.element).data('duration') &&
            $(this.element).data('totalSeconds') % $(this.element).data('duration') === 0) {

            // If 'repeat' is not requested then disable the duration
            if (!this.options.repeat) {
                $(this.element).data('duration', null);
                this.options.duration = null;
            }

            // If this is a countdown, then end it as duration has completed
            if (this.options.countdown) {
                stopTimerInterval(this);
                this.options.countdown = false;
                $(this.element).data('state', TIMER_STOPPED);
            }

            // Run the default callback
            this.options.callback();
        }
    }

    /**
     * Render pretty time
     */
    function render(timer) {
        var element = timer.element,
            sec = $(element).data('totalSeconds');

        if (timer.options.countdown && ($(element).data('duration') > 0)) {
            sec = $(element).data('duration') - $(element).data('totalSeconds');
        }

        $(element)[display](secondsToTime(sec, timer));
        $(element).data('seconds', sec);
    }

    /**
     * Method to make timer field editable
     * This method hard binds focus & blur events to pause & resume
     * and recognizes built-in pretty time (for eg 12 sec OR 3:34 min)
     * It won't recognize user created formats.
     * Users may not always want this hard bound. In such a case,
     * do not use the editable property. Instead bind custom functions
     * to blur and focus.
     */
    function makeEditable(timer) {
        var element = timer.element;
        $(element).on('focus', function() {
            pauseTimer(timer);
        });

        $(element).on('blur', function() {
            // eg. 12 sec 3:34 min 12:30 min
            var val = $(element)[display](), valArr;

            if (val.indexOf('sec') > 0) {
                // sec
                $(element).data('totalSeconds', Number(val.replace(/\ssec/g, '')));
            } else if (val.indexOf('min') > 0) {
                // min
                val = val.replace(/\smin/g, '');
                valArr = val.split(':');
                $(element).data('totalSeconds', Number(valArr[0] * 60) + Number(valArr[1]));
            } else if (val.match(/\d{1,2}:\d{2}:\d{2}/)) {
                // hrs
                valArr = val.split(':');
                $(element).data('totalSeconds', Number(valArr[0] * 3600) + Number(valArr[1] * 60) + Number(valArr[2]));
            }

            resumeTimer(timer);
        });
    }

    /**
     * Get the current unix timestamp in seconds
     * @return {Number} [unix timestamp in seconds]
     */
    function getUnixSeconds() {
        return Math.round(new Date().getTime() / 1000);
    }

    /**
     * Convert a number of seconds into an object of hours, minutes and seconds
     * @param  {Number} sec [Number of seconds]
     * @return {Object}     [An object with hours, minutes and seconds representation of the given seconds]
     */
    function sec2TimeObj(sec) {
        var hours = 0, minutes = Math.floor(sec / 60), seconds;

        // Hours
        if (sec >= 3600) {
            hours = Math.floor(sec / 3600);
        }

        // Minutes
        if (sec >= 3600) {
            minutes = Math.floor(sec % 3600 / 60);
        }
        // Prepend 0 to minutes under 10
        if (minutes < 10 && hours > 0) {
            minutes = '0' + minutes;
        }
        // Seconds
        seconds = sec % 60;
        // Prepend 0 to seconds under 10
        if (seconds < 10 && (minutes > 0 || hours > 0)) {
            seconds = '0' + seconds;
        }

        return {
            hours: hours,
            minutes: minutes,
            seconds: seconds
        };
    }

    /**
     * Convert the given seconds to an object made up of hours, minutes and seconds and return a pretty display
     * @param  {Number} sec [Second to display as pretty time]
     * @return {String}     [Pretty time]
     */
    function secondsToTime(sec, timer) {
        var time = '',
            timeObj = sec2TimeObj(sec);

        if (timer.options.format) {
            var formatDef = [
                {identifier: '%h', value: timeObj.hours, pad: false},
                {identifier: '%m', value: timeObj.minutes, pad: false},
                {identifier: '%s', value: timeObj.seconds, pad: false},
                {identifier: '%H', value: parseInt(timeObj.hours), pad: true},
                {identifier: '%M', value: parseInt(timeObj.minutes), pad: true},
                {identifier: '%S', value: parseInt(timeObj.seconds), pad: true}
            ];
            time = timer.options.format;

            formatDef.forEach(function(format) {
                time = time.replace(
                    new RegExp(format.identifier.replace(/([.*+?^=!:${}()|\[\]\/\\])/g, '\\$1'), 'g'),
                    (format.pad) ? ((format.value < 10) ? '0' + format.value : format.value) : format.value
                );
            });
        } else {
            if (timeObj.hours) {
                time = timeObj.hours + ':' + timeObj.minutes + ':' + timeObj.seconds;
            } else {
                if (timeObj.minutes) {
                    time = timeObj.minutes + ':' + timeObj.seconds + ' min';
                } else {
                    time = timeObj.seconds + ' sec';
                }
            }
        }
        return time;
    }

    /**
     * Convert a string time like 5m30s to seconds
     * If a number (eg 300) is provided, then return as is
     * @param  {Number|String} time [The human time to convert to seconds]
     * @return {Number}      [Number of seconds]
     */
    function timeToSeconds(time) {
        // In case the passed arg is a number, then use that as number of seconds
        if (!isNaN(Number(time))) {
            return time;
        }

        var hMatch = time.match(/\d{1,2}h/),
            mMatch = time.match(/\d{1,2}m/),
            sMatch = time.match(/\d{1,2}s/),
            seconds = 0;

        time = time.toLowerCase();

        // @todo: throw an error in case of faulty time value like 5m61s or 61m
        if (hMatch) {
            seconds += Number(hMatch[0].replace('h', '')) * 3600;
        }

        if (mMatch) {
            seconds += Number(mMatch[0].replace('m', '')) * 60;
        }

        if (sMatch) {
            seconds += Number(sMatch[0].replace('s', ''));
        }

        return seconds;
    }

    // TIMER INTERFACE
    function startTimer(timer) {
        var element = timer.element;
        if (!$(element).data('isTimerRunning')) {
            render(timer);
            startTimerInterval(timer);
            $(element).data('state', TIMER_RUNNING);
            timer.options.startTimer.bind(timer).call();
        }
    }

    function pauseTimer(timer) {
        var element = timer.element;
        if ($(element).data('isTimerRunning')) {
            stopTimerInterval(timer);
            $(element).data('state', TIMER_PAUSED);
            timer.options.pauseTimer.bind(timer).call();
        }
    }

    function resumeTimer(timer) {
        var element = timer.element;
        if (!$(element).data('isTimerRunning')) {
            $(element).data('startTime', getUnixSeconds() - $(element).data('totalSeconds'));
            startTimerInterval(timer);
            $(element).data('state', TIMER_RUNNING);
            timer.options.resumeTimer.bind(timer).call();
        }
    }

    function resetTimer(timer) {
        var element = timer.element;
        $(element).data('startTime', 0);
        $(element).data('totalSeconds', 0);
        $(element).data('seconds', 0);
        $(element).data('state', TIMER_STOPPED);
        $(element).data('duration', timer.options.duration);
        timer.options.resetTimer.bind(timer).call();
    }

    function removeTimer(timer) {
        var element = timer.element;
        stopTimerInterval(timer);
        timer.options.removeTimer.bind(timer).call();
        $(element).data('plugin_' + pluginName, null);
        $(element).data('seconds', null);
        $(element).data('state', null);
        $(element)[display]('');
    }

    // TIMER PROTOTYPE
    var Timer = function(element, userOptions) {
        var elementType;

        this.options = options = $.extend(this.options, options, userOptions);
        this.element = element;

        // Setup total seconds from options.seconds (if any)
        $(element).data('totalSeconds', options.seconds);

        // Setup start time if seconds were provided
        $(element).data('startTime', getUnixSeconds() - $(element).data('totalSeconds'));

        $(element).data('seconds', $(element).data('totalSeconds'));
        $(element).data('state', TIMER_STOPPED);

        // Check if this is a input/textarea element or not
        elementType = $(element).prop('tagName').toLowerCase();
        if (elementType === 'input' || elementType === 'textarea') {
            display = 'val';
        }

        if (this.options.duration) {
            $(element).data('duration', timeToSeconds(this.options.duration));
            this.options.duration = timeToSeconds(this.options.duration);
        }

        if (this.options.editable) {
            makeEditable(this);
        }

    };

    /**
     * Initialize the plugin with public methods
     */
    Timer.prototype = {
        start: function() {
            startTimer(this);
        },

        pause: function() {
            pauseTimer(this);
        },

        resume: function() {
            resumeTimer(this);
        },

        reset: function() {
            resetTimer(this);
        },

        remove: function() {
            removeTimer(this);
        }
    };

    // INITIALIZE THE PLUGIN
    var pluginName = 'timer';
    $.fn[pluginName] = function(options) {
        options = options || 'start';

        return this.each(function() {
            /**
             * Allow the plugin to be initialized on an element only once
             * This way we can call the plugin's internal function
             * without having to reinitialize the plugin all over again.
             */
            if (!($.data(this, 'plugin_' + pluginName) instanceof Timer)) {

                /**
                 * Create a new data attribute on the element to hold the plugin name
                 * This way we can know which plugin(s) is/are initialized on the element later
                 */
                $.data(this, 'plugin_' + pluginName, new Timer(this, options));

            }

            /**
             * Use the instance of this plugin derived from the data attribute for this element
             * to conduct whatever action requested as a string parameter.
             */
            var instance = $.data(this, 'plugin_' + pluginName);

            /**
             * Provision for calling a function from this plugin
             * without initializing it all over again
             */
            if (typeof options === 'string') {
                if (typeof instance[options] === 'function') {
                    /*
                     Pass in 'instance' to provide for the value of 'this' in the called function
                     */
                    instance[options].call(instance);
                }
            }

            /**
             * Allow passing custom options object
             */
            if (typeof options === 'object') {
                if (instance.options.state === TIMER_RUNNING) {
                    instance.start.call(instance);
                } else {
                    render(instance);
                }
            }
        });
    };

}));

var $jq = jQuery.noConflict();
// Disable caching of AJAX responses
$jq.ajaxSetup ({
    cache: false
});

//Pad function
$jq.strPad = function(i,l,s) {
	var o = i.toString();
	if (!s) { s = '0'; }
	while (o.length < l) {
		o = o + s;
	}
	return o;
};

//Numeric check
function IsNumeric(sText){

	var ValidChars = "0123456789.";
	var IsNumber=true;
	var Char;

	for (i = 0; i < sText.length && IsNumber == true; i++){ 
		Char = sText.charAt(i); 
		if(ValidChars.indexOf(Char) == -1){
			IsNumber = false;
		}
	}

	return IsNumber; 
}

//Display a notification
function hs_msg(msg,sticky,classname){
	sticky = sticky || false;
	classname = classname || false;
	
	if(classname) $("hs_msg").addClassName(classname);
	
	var space = 20;
	//Set body of message, use sizer span to get exact width
	$("hs_msg").update(msg);
	
	//Center
	var w = innerWinSize();
	var toppos = (w.height - 41) + "px";
	$("hs_msg").setStyle({top: toppos}); //26px high box plus 10 padding plus 5 space at bottom
	
	//Show
	new Effect.Parallel([
		new Effect.Move("hs_msg", { sync: true, x: 0, y: -30, mode: 'relative' }),
		new Effect.Appear("hs_msg", { sync: true })
	], { 
	  duration: 0.8
	});
	//Hide
	if(!sticky){
		setTimeout(function(){
			Effect.Fade('hs_msg', { duration: 1.0, onComplete: function(){
				//Reset position as effect moves it
				$("hs_msg").setStyle({top: toppos});
				//Reset class
				if(classname) $("hs_msg").removeClassName(classname);
			} });
		},3000);
	}
}

//Submit the login form
function login_form_sub(){
	new Ajax.Request("admin.php?pg=ajax_gateway&action=logincheck", 
		{method:"post", 
		 parameters: {username:$F("popup_username"), password:$F("popup_password")},
		 onSuccess: function(){
		 	Tips.hideAll();
		 	$jq.colorbox.close();
		 	did_re_login = true;
		 }
		}
	);
	return false;
}

//Preload images
function preloadImages(){
  var cache = [];
  // Arguments are image paths relative to the current page.
    var args_len = arguments.length;
    for (var i = args_len; i--;) {
      var cacheImage = document.createElement('img');
      cacheImage.src = arguments[i];
      cache.push(cacheImage);
    }
}

//Is a var defined
function hs_isdefined(variable)
{
    return (typeof(window[variable]) == "undefined")?  false: true;
}

function is_wysiwyg(){
	return ( !(typeof(textboxio) == "undefined") && textboxio.getActiveEditor() !== null );
}

function focus_note_body(field){
	if(! is_wysiwyg()){
		Field.focus(field);
	}
}

function get_note_body(field){
	if(is_wysiwyg()){
		// use jquery to see if it's empty. By default textbox has html tags that causes it not be blank.
		return ($jq("<div/>").html(textboxio.getActiveEditor().content.get()).text().trim() != "") ? textboxio.getActiveEditor().content.get() : "";
	}else{
		return ($F(field) ? $F(field) : "");
	}
}

function set_note_body(field,content){
	if(is_wysiwyg()){
		return textboxio.getActiveEditor().content.set(content)
	}else{
		return $(field).value = content;
	}
}

function append_wysiwyg_note_body(field, content)
{
	return textboxio.getActiveEditor().content.set(get_note_body(field) + content)
}

// send to page
function goPage(src){
	window.location.href = src;	
}

//By using idle timers we prevent periodically executed events from barraging a server when the user is
//not actually using the system. This is especially helpful with users who use multiple browser tabs.
//Note, can't use window blur/focus due to wysiwyg & other irregularities
function hs_PeriodicalExecuter(name,user_function,timeout){
	//Global variable name
	var varname = 'helpspot_pe_' + name;

    // Run the function every 300 seconds
	window[varname] = setInterval(user_function, timeout * 1000);
	
	$jq(document).bind("idle.idleTimer", function(){
		clearInterval(window[varname]);
	});
	
	$jq(document).bind("active.idleTimer", function(){
		user_function(); //Cause actions to occur as soon as back active for responsiveness
		window[varname] = setInterval(user_function, timeout * 1000);
	});

	//If idle for 180 seconds then stop execution
	$jq.idleTimer(cHD_IDLE_TIMEOUT * 1000);
}

//Alert box
function hs_alert(txt,options){
	var options = Object.extend({
	  errorlist:	true,
	  title: 		lg_js_notification
	}, arguments[1] || { });
	var body = "";
	
	if(options.errorlist){
		var list = txt.split(/(\r\n|[\r\n])/);
		body = '<ul class="alert-error-list">';
		for(i=0;i<list.length;i++){
			if(trim(list[i]) != "") body = body + '<li>'+list[i]+'</li>';
		}
		body = body + '</ul>';
	}else{
		body = txt.replace(/(\r\n|[\r\n])/g, "<br />");
	}

	$jq.colorbox({width: "500px",
				  opacity: 0.40, 
				  transition: 'none',
				  close: button_ok,
				  html: '<div class="alert-body">'+ body + '</div>' + '<div class="alert-title">'+options.title+'</div>',
				  onComplete: function(){
				  		$jq("#alert-button-ok").focus();
				  	}
				  });	
}

//Confirm box
function hs_confirm(txt,action,options){
	var options = Object.extend({
	  title: 		lg_js_confirmation
	}, arguments[1] || { });
	
	$jq.colorbox({width: "500px",
				  opacity: 0.40, 
				  transition: 'none',
				  close: button_cancel,
				  html: '<div class="alert-confirm-body">'+ txt + '</div>' + '<div class="alert-title">'+options.title+'</div>' + '<div class="alert-button-group">'+ button_ok +'</div>',
				  onComplete: function(){
				  	//Execute function or else go to URL
				  	if($jq.type(action) === "function"){
				  		$jq("#alert-button-ok").click(action);
				  	}else{
				  		$jq("#alert-button-ok").click(function(){goPage(action);});
				  	}
				  	
				  	//Add closing action
				  	$jq("#alert-button-ok").click(function(){
				  		$jq.colorbox.close();
				  	});
				  	
				  	//Focus
				  	$jq("#alert-button-ok").focus();
				  }
				  });	
				  
	return false;			  
}

function kbui(bookid){
	$jq.colorbox({width: "90%",
				  fixed: true,
				  height: "90%",
				  opacity: 0.80, 
				  close: button_close,
				  href: "admin.php?pg=ajax_gateway&action=kbui&xBook=" + bookid + "&rand=" + ajaxRandomString(),
				  onComplete: function(){
				  	kbui_gettoc(bookid);
				  }
				  });	
}

function kbui_showpage(pageid){
	$jq("#kbui-page").html(ajaxLoading());
	$jq("#kbui-page").load("admin.php?pg=ajax_gateway&action=kbui-page&xPage="+pageid+"&xPortal="+$jq("#xPortal").val());
}

function kbui_gettoc(bookid){
	$jq("#kbui-toc").load("admin.php?pg=ajax_gateway&action=kbui-toc&xBook="+bookid, function(){
		folderUI("kbui-toc");
		$jq("#kbui-toc .kbui:first").click(); //load first page in this toc
	});
}

function aKBL(link,text){
	//If the tPost box exists in forums use that
	if($jq("#tPost").length){
		var elem = "tPost";
		var docpath = document.postform.tPost;
	}else{
		var elem = "tBody";
		var docpath = document.requestform.tBody;	
	}

	if(elem == 'tBody' && editor_type == "wysiwyg" && is_wysiwyg("tBody")){
		// tinyMCE.execInstanceCommand(elem,"mceInsertContent",false,"<a href=\""+link+"\">"+text+"</a>");
		textboxio.getActiveEditor().content.insertHtmlAtCursor("<a href=\""+link+"\">"+text+"</a>");
	}else if(elem == 'tBody' && editor_type == "markdown"){
		insertAtCursor(docpath, "["+text+"]("+link+")");
	}else{		
		insertAtCursor(docpath, link);
	}

	$jq.colorbox.close();

	return false;
}

function folderUI(wrap){
	//Init array of open folders
	var sidebarOpenFolders = (getCookie("sidebarOpenFolders") ? getCookie("sidebarOpenFolders").split(",") : []);	

	//Click handler for folders
	$$("#" + wrap + " .folder").each(function(folder){
		$(folder).observe("click", function(event){
			Event.stop(event);
			$$("."+this.id).each(function(elem){
				elem.toggle();
			});
			$$("#"+this.id+" span.arrow").each(function(elem){
				elem.toggleClassName("arrow-open");
			});
			
			//Remember state
			var loc = indexInArray(sidebarOpenFolders, this.id);
			if(loc == -1){
				sidebarOpenFolders.push(this.id);
			}else{
				sidebarOpenFolders.splice(loc, 1);
			}
			
			//Save state cookie
			var cdate = new Date ( 2020, 1, 1 );
			setCookie("sidebarOpenFolders", sidebarOpenFolders.join(","), cdate);
		});
	});
}

//Get the url hash
function getHash(){
  var hash = window.location.hash;
  return hash.substring(1); //remove the #
}

//Open new page in original win from popup
function targetopener(mylink, closeme, closeonly){
	if (! (window.focus && window.opener))return true;
	window.opener.focus();
	if (! closeonly)window.opener.location.href=mylink.href;
	if (closeme)window.close();
	return false;
}

//Open a window
//ex: <a href="javascript:openWin(\'admin.php?pg=user.image\',\'ImagePick\',\'height=500, width=500, scrollbars=yes, toolbar=no, location=no, status=yes\');">
function openWin(loc,name,options){
	window.open(loc, "", options);
}

function showPopWin(page,winh,winw,name){
	openWin(page,name,'height='+winh+',width='+winw+',scrollbars=yes,toolbar=no,location=no,status=no,resizable=yes');
}

//close window with delay 
function hsCloseWin(time){
	if(!time){
		time = 2;
	}
	
	setTimeout("window.close()",time*1000);
}

//sets times when using quick time select function
function qtSet(id,from,to){
	times = Form.Element.getValue(id);
	if(times != ""){
		v = Form.Element.getValue(id).split("|");
		$(from).value = v[0];
		$('from_da').update(v[1]);
		$(to).value = v[2];
		$('to_da').update(v[3]);
		if(v[4]){
			//Only override grouping if it's currently a date type (and not a date agg type)
			if($jq("#graph_grouping").val().search("date_") != -1 && $jq("#graph_grouping").val().search("date_agg") == -1) setSelectToValue("graph_grouping",v[4]);
		}
	}
}

//js sidebar search
function sidebarReqSearch(evt){
	evt.keyCode == 13 ? sidebarSearchAction("request") : evt;
}

function sidebarSearchAction(type){
	if(type == "request"){
		var q = trim($jq("#sidebar-q").val());
        var doFullText = ($jq("#doFullTextSearch").get(0).checked) ? 1 : 0;
		
		//If just a number, use as reqid else search
		if(IsNumeric(q)){
			goPage("admin.php?pg=request&reqid=" + q);
		}else{
			//Handle different search types
			var name = q.split(" ");
			if(q.search(/@/) > 0){
				var url = "admin.php?pg=ajax_gateway&action=sidebarsearch&doFullText="+eq(doFullText)+"&search_type=2&q="+eq(q)+"&anyall=all&sEmail="+eq(q)+"&rand="+ajaxRandomString();
			}else if(name.length > 1){ 
				var url = "admin.php?pg=ajax_gateway&action=sidebarsearch&doFullText="+eq(doFullText)+"&search_type=3&q="+eq(q)+"&anyall=all&sFirstName="+eq(name[0])+"&sLastName="+eq(name[1])+"&rand="+ajaxRandomString();
			}else{
				var url = "admin.php?pg=ajax_gateway&action=sidebarsearch&doFullText="+eq(doFullText)+"&search_type=1&q="+eq(q)+"&anyall=any&sUserId="+eq(q)+"&sFirstName="+eq(q)+"&sLastName="+eq(q)+"&sEmail="+eq(q)+"&rand="+ajaxRandomString();
			}
			
			hs_overlay({width:'90%',href:url});
		}
	}
}

//shortcut flag
var shortcutsON = true;

function hs_shortcutsOff(){
	shortcutsON = false;
}

function hs_shortcutsOn(){
	shortcutsON = true;
}
	
//Set Cookie
function setCookie(name,value,expires,path,domain,secure) {
    var cookieString = name + "=" +escape(value) +
       ( (expires) ? ";expires=" + expires.toGMTString() : "") +
       ( (path) ? ";path=" + path : "") +
       ( (domain) ? ";domain=" + domain : "") +
       ( (secure) ? ";secure" : "");
    document.cookie = cookieString;
} 

//Get Cookie
function getCookie(cookie_name) {
		var str     = document.cookie;
        var cookies = str.split(';');
        for (var i = cookies.length; i--;)
        {
          var parts = cookies[i].split('=');
          if (parts[0].substring(0,1) == ' ')
              parts[0] = parts[0].substring(1,parts[0].length);

          if (unescape(parts[0]) == cookie_name){
             return unescape(parts[1]);
             }
        }
        return '';
}    

//Trim extra whitespace
function trim(stringToTrim) {
	return $jq.trim(stringToTrim);
	//return stringToTrim.replace(/^\s+|\s+$/g,"");
}

//Set focus to field
function setFieldFocus(fld){
	if(fld){
		fld.focus();
	}
}

//Clear the filler text in a form element
function clearFocusFill(obj,text,newclass){
	if(obj.value == text) obj.value = "";
	if(newclass) Element.addClassName(obj.id,newclass);
}

//Checkbox row highlight
function checkRowHighlight(rid){
	if(document.getElementById(rid+"_box")){
		if(document.getElementById(rid+"_box").checked){
			document.getElementById(rid).className = "boldcheckboxrow";
		}else{
			document.getElementById(rid).className = "";
		}			
	}
}

//Loop over checkbox sets and make lines bold. use with above.
function onloadRowHighlight(pre,len)
{
	for(var i=0; i<len; i++){
		checkRowHighlight(pre + i);
	}
}	

// Check any rows that are already checked. See #364
function checkRows() {
  $jq.each($jq(".checkbox-menu").find("input:checked"), function() {
    rowChecked(this.value);
  });
}
//Highlight a table row
function rowChecked(reqid){
	row = $("tr-"+reqid);
	box = $(reqid + "_checkbox");
	
	if(box.checked){
		Element.addClassName(box,"checkedfilterrow");
		Element.addClassName(row,"checkedfilterrow");				
	}else{
		Element.removeClassName(box,"checkedfilterrow");
		Element.removeClassName(row,"checkedfilterrow");
	}	
	
	//Make sure any hidden buttons are shown
	$("batch_action_buttons").removeClassName("thin-disabled");
}

//Sort ordering
function reorder_call(id,type){
	var url = "admin.php";
	var pars = "pg=ajax_gateway&action="+type+"&" + Sortable.serialize(id,{name:"sortorder"}) + "&rand=" + ajaxRandomString();
	
	var call = new Ajax.Request(
		url, 
		{
			method: 	 "get", 
			parameters:  pars, 
			onComplete:  function(){ $$("#tablesort .sortable").each(function(elem){ new Effect.Highlight(elem,{duration:1.0,startcolor:"#DCE8D8",keepBackgroundImage:true}); }); }
		});			
}

//Given a value go through a form and find the ID
//Fields is an array for form elements (usually) from Form.getInputs("rsform_1", "checkbox");
function findIDbyValue(fields,value){
	for(i=0;i < fields.length;i++){
		if(fields[i].value == value) return fields[i].id;
	}
}

//Find the currently checked value in a radio button group
function getCheckedRadioButton(buttonGroup){
	for(var i = 0; i < buttonGroup.length; i++){
		if(buttonGroup[i].checked){
			return i;	
		}
	}
}

//Find the currently selected option
function getSelectVal(elem){
	return $(elem).options[$(elem).selectedIndex].value;
}

// Return the index of a particular value
function hs_indexOf(thearray,item) {
    for (var i = 0; i < thearray.length; i++) {
        if (thearray[i] == item) {
            return i;
        }
    }
    return -1;
}

// Return true if the value is in an array
function hs_inArray (value)
// Returns true if the passed value is found in the
// array.  Returns false if it is not.
{
	var i;
	for (i=0; i < this.length; i++) {
		// Matches identical (===), not just similar (==).
		if (this[i] === value) {
			return true;
		}
	}
	return false;
};

//Find index in an array
function indexInArray(theArray, theValue){
	var arLength = theArray.length;

	for(var i=0; i < arLength; i++){
		if(theArray[i] == theValue){
			return i;
		}
	}

	return -1;
}

// Add ability to pad a string
function hs_pad(str, chr, num, prepend) {
  for (var i=str.length; i<num; i++) {
    str = ((prepend) ? (chr + str) : (str + chr));
  }
  return str;
}

// Use this function in textbox to prevent enter from submitting the form, optionally do a click event
function noenter(e, click_elem) {
	if (!e) var e = window.event
	if (e.keyCode) code = e.keyCode;
	else if (e.which) code = e.which;
	
	if(click_elem){
		if(code == 13){ 
			$(click_elem).onclick();
			return false;
		}else{
			return true;
		}
	}else{
		return !(code == 13); 
	}
}

//Set a select list to a value if the value exists
function setSelectToValue(selectname,v,reset){
	list = $(selectname).options;
	
	if(typeof(reset) != 'undefined') list.selectedIndex = 0;
	
	if(list && v){
		for(i=0; i < list.length; i++){
			if(list[i].value == v){
				list.selectedIndex = i;
				break;
			}
		}
	}
}

//Set a select list to a value if the text exists
function setSelectByText(selectname,v){
	list = $(selectname).options;
	
	if(list && v){
		for(i=0; i < list.length; i++){
			if(list[i].text == v){
				list.selectedIndex = i;
				break;
			}
		}
	}
}

function help_toggle(page,from){
	var from = (from ? from : "");
	new Ajax.Updater('help_toggle_body', "admin.php?pg=ajax_gateway&action=help&page="+page+"&from="+from+"&rand=" + ajaxRandomString());	
}

// CHECK RECORDSET ALL
function checkUncheckRsAll(id) {
	
	$$("input[name^=checktable]").each(function(elem){
		$(elem).checked = $(elem).checked ? false : true;
		$(elem).onclick();
	});
			
	return true;
}

// END CHECK ALL

//CHECK ONLY A GROUP OF REQUESTS
function checkUncheckRequestGroup(grouping){
	var boxes = $$('input.'+grouping);
	if($('groupcheckbox_'+grouping).checked){
		boxes.each(function(box){
			if(box.checked == false) box.click();
		});
	}else{
		boxes.each(function(box){
			if(box.checked == true) box.click();
		});	
	}	
}

//Create the stream view
streamViewCurrent = 0;
function streamViewInit(reqid, mode){
	streamViewCurrent = reqid;
	
	$("streamViewBody").update("<div class='streamview_loading'><img src='"+static_path+"/css/shared/general/big-loader.gif' /></div>");
	$("streamViewTop_reqid").update("");
	
	$jq.colorbox({
    width: "90%",
    height: "90%",
    opacity: 0.80,
    close: button_close,
    html: $jq("#streamView").html(),
    onLoad:function(){
      $jq("#cboxClose").hide();
    }
	});

  // Hide the prev next from sidebarsearch
  if (mode == "sidebarsearch") {
    $jq("#colorbox").find("#overflow-buttons .next, #overflow-buttons .prev").hide()
  }
}

function streamViewPrev(end){
	//Find the type we're dealing with (link- or takeit-)
	if($jq("[id^=link-]").length){
		var s = 'link-';
	}else{
		var s = 'takeit-';
	}
	
	//Find all links
	var prev = $jq("[id^="+s+"]").first().attr("id").replace(s,"");
	$jq("[id^="+s+"]").each(function(index){
		//At end
		if(prev == streamViewCurrent){
			hs_alert(end);
			return false;
		}
		
		//Loop over and find previous link
		if($jq(this).attr("id") == s+streamViewCurrent){
			showOverflow(prev.replace(s,""));
			return false;
		}
		prev = $jq(this).attr("id");
	});
}

function streamViewNext(end){
	//Find the type we're dealing with (link- or takeit-)
	if($jq("[id^=link-]").length){
		var s = 'link-';
	}else{
		var s = 'takeit-';
	}
	
	//Find all links
	var next = $jq("[id^="+s+"]").last().attr("id").replace(s,"");
	$jq($jq("[id^="+s+"]").get().reverse()).each(function(index){
		//At end
		if(next == streamViewCurrent){
			hs_alert(end);
			return false;
		}
		
		//Loop over and find next link
		if($jq(this).attr("id") == s+streamViewCurrent){
			showOverflow(next.replace(s,""));
			return false;
		}
		next = $jq(this).attr("id");
	});
}

//show the note menu
function showNoteItemMenu(){
	if($jq("#"+this.id+" .note-stream-item-menubtn").length){
		$jq("#"+this.id+" .note-stream-item-menubtn").fadeIn("fast");
		
		new Tip(this.id+"-menubtn", $jq("#"+this.id+" .note-stream-item-menu").html(), {
				title: "",
				style: "darkgrey",
				delay: 0,
				showOn: "click",
				hideOn: false,
				hideAfter: 1,
				stem: "topMiddle",
				width: "auto",
				hook: { target: "bottomMiddle", tip: "topMiddle" },
				offset: { x: 0, y: 0 }
			});	
	}
}

function hideNoteItemMenu(){
	if(!$jq(".prototip").is(":visible")) $jq("#"+this.id+" .note-stream-item-menubtn").hide();
}

function ms_select(id,value,fieldname){
	var field = id + "-hidden";
	
	//Set class
	$(id).toggleClassName('select-multiple-selected');
	
	//Insert hidden field
	if(!$(field)){
		$(id).insert({after:'<input type="hidden" id="'+field+'" name="'+fieldname+'[]" value="'+ value + '" />'});
	}else{
		$(field).remove();
	}
}

function ms_select_all(prefix){
	$$("."+prefix+"-select-multiple a").each(function(e){
		if(!$(e.id + "-hidden")){
			e.onclick();
		}
	});
}

function ms_remove_all(prefix){
	$$("."+prefix+"-select-multiple a").each(function(e){
		if($(e.id + "-hidden")){
			e.onclick();
		}
	});
}

function ms_expand(prefix){
	$$("."+prefix+"-select-multiple")[0].setStyle({height:"500px"});
}

function yes_no_btn(type,id,val){
	if(type == "yes"){
		$(id+"-yes").addClassName("btn-selected");
		$(id+"-yes").removeClassName("btn-yes-no");
		$(id+"-no").addClassName("btn-yes-no");
		$(id+"-no").removeClassName("btn-selected");		
	}else{
		$(id+"-no").addClassName("btn-selected");
		$(id+"-no").removeClassName("btn-yes-no");
		$(id+"-yes").addClassName("btn-yes-no");
		$(id+"-yes").removeClassName("btn-selected");		
	}
	
	$(id).setValue(val);
}

function addFolder(def,slist){
	hs_overlay({href:"admin.php?pg=ajax_gateway&action=addfolder&default="+eq(def)+"&folder="+eq(slist),onComplete:function(){$jq("#new_folder").focus();}});
}

function add_folder_action(val,slist){
		foldername = val;
		folders   = $(slist).options;
		folderlen = folders.length;
		newoption = folderlen;

		$(slist).options[newoption]= new Option(foldername,foldername);
		$(slist).selectedIndex = newoption;
		
		$jq.colorbox.close();
}

function ttm_tip(id, txt){
	new Tip(id, txt, {
			title: "",
			style: "darkgrey",
			className: "hstinytip",
			border: 0, 
			radius: 0,
			showOn: "mouseover",
			hideOn: "mouseout",
			stem: "topMiddle",
			width: "auto",
			hook: { target: "bottomMiddle", tip: "topMiddle" }
		});	
}

function ttm_tip_fat(id, txt){
	new Tip(id, txt, {
			title: "",
			style: "darkgrey",
			className: "hstinytip",
			border: 0, 
			radius: 0,
			showOn: "mouseover",
			hideOn: "mouseout",
			stem: "topMiddle",
			width: "160px",
			hook: { target: "bottomMiddle", tip: "topMiddle" }
		});	
}

function showOverflow(reqid, mode){
	Tips.hideAll();

  // Set a flag to hide the prev/next if on the sidebar search.
  if ($jq("#colorbox #rsgroup_1").length === 1) {
    mode = "sidebarsearch"
  }
  
	streamViewInit(reqid, mode);

	//Insert reqid link

	var menubody = new Template('<a href="admin.php?pg=request&reqid=#{id}" id="streamViewReqLink" class="reqid-link" style="display: inline-block;">#{id}</a>');
	$("streamViewTop_reqid").update(menubody.evaluate({id:reqid}));	
	
	var url = "admin.php?pg=request.static&from_streamview=1&reqid=" + reqid + "&rand=" + ajaxRandomString();

	if ($jq("#" + parseInt(reqid) + "_checkbox").is(':checked')) {
		$jq(".js-select-request").addClass("checked");
	} else {
		$jq(".js-select-request").removeClass("checked").prop( "checked", false );
	}
	$jq(".js-select-request").on("click", function(e){
		$jq(".js-select-request").toggleClass("checked");
		$jq("#" + parseInt(reqid) + "_checkbox").trigger("click");
		return false;
	});

	//Do ajax call and update content div
	new Ajax.Updater('streamViewBody',url,{
		evalScripts:true
	});
}

function showHistoryEmailAndHeaders(reqhistid, field, title){
	if(field == "emailheaders"){
		var url = "admin.php?pg=ajax_gateway&action=emailheaders&reqhisid=" + reqhistid + "&rand=" + ajaxRandomString();
	}else{
		var url = "admin.php?pg=ajax_gateway&action=emailsource&reqhisid=" + reqhistid + "&rand=" + ajaxRandomString();
	}
	
	//Show content overlay
	hs_overlay({href:url,title:title});	
}

function simplemenu_action(action, id, mode){
	mode = typeof(mode) != 'undefined' ? mode : "menu";
	
	//Hide tips & quick menu icon
	if(mode != "triage") document.fire("hs_overlay:closed");
	
	//Perform action
	if(action == 'unread'){
		
		$("replied_img_"+id).onclick();
		
		new Effect.Highlight("tr-"+id,{duration:2.0, startcolor:'#ffffff', endcolor:'#ffff99'});
		
	}else if(action == 'trash'){
		
		$("tr-"+id).addClassName("tablerow-trash");
		
		new Ajax.Request(
		'admin.php?pg=ajax_gateway&action=simplemenu_trash', 
		{
			method: 	 "post", 
			parameters:  {reqid:id},
			onComplete: function(r){ 
				if(r.responseText != ""){ 
					hs_alert(r.responseText);  
				}else{
					new Effect.Highlight("tr-"+id,{duration:1.0, startcolor:'#FDDFD7', endcolor:'#D8B3AE', afterFinish:function(){$("tr-"+id).remove();}});
				
					//Move triage to next
					if(mode == "triage") triage_next();					
				}
			}
		});			
		
	}else if(action == 'spam'){
	
		$("tr-"+id).addClassName("tablerow-spam");
		
		new Ajax.Request(
		'admin.php?pg=ajax_gateway&action=simplemenu_spam', 
		{
			method: 	 "post", 
			parameters:  {reqid:id},
			onComplete: function(r){ 
				if(r.responseText != ""){
					hs_alert(r.responseText); 
					$("tr-"+id).removeClassName("tablerow-spam");
				}else{
					new Effect.Highlight("tr-"+id,{duration:1.0, startcolor:'#FDDFD7', endcolor:'#D8B3AE', afterFinish:function(){$("tr-"+id).remove();}});
				
					//Move triage to next
					if(mode == "triage") triage_next();				
				}
			}
		});		
		
	}
}

person_status_update_details_flag=false;
function person_status_update_details(xperson,spage,ftype,sdetails){
	if(!person_status_update_details_flag){
		var pars = "xPersonStatus=" + eq(xperson) + "&sPage=" + eq(spage) + "&fType=" + eq(ftype) + "&sDetails=" + eq(sdetails) + "&rand=" + ajaxRandomString();
		
		var call = new Ajax.Request(
			'admin.php?pg=ajax_gateway&action=person_status_details', 
			{
				method: 	 "post", 
				parameters:  pars
			});	
			
		person_status_update_details_flag = true;
	}
}

function showPersonStatusWorkspace(id,reqid){

	var call = new Ajax.Request(
		'admin.php?pg=ajax_gateway&action=person_status_workspace', 
		{
			method: 	 "get", 
			parameters:  {rand:ajaxRandomString(),reqid:reqid}, 
			onComplete:  function(){ 

							new Tip(id, arguments[0].responseText,{
									title: reqid,
									style: "darkgrey",
									showOn: "click",
                                    hideOn: { element: 'closeButton', event: 'click' },
									hideOthers: true,
									stem: "leftMiddle",
									width: 250,
									hook: { target: "rightMiddle", tip: "leftMiddle" },
									offset: { x: -5, y: 0 }
								});	
								
							$(id).prototip.show();
						 }
		});				
				
}

//Overlay window
function hs_overlay(){
	//Accept 1 arg with just options array or 2 with a body element ID plus options
	if($jq.type(arguments[0]) == "string"){
		var body = arguments[0];
		var options = arguments[1];
	}else{
		var body = false;
		var options = arguments[0];	
	}
	
	var options = Object.extend({
	  width:	false,
	  height:	false,
	  maxWidth:	"90%",
	  maxHeight:"90%",
	  innerWidth: false,
	  innerHeight: false,
	  onHide: 	false,
	  onComplete: false,
	  close: button_close,
	  href: false,
	  iframe: false,
	  html: (body ? $jq("#"+body).html() : false),
	  title:	""
	}, options || { });

	Tips.hideAll();	

	$jq.colorbox({title: options.title,
				  width: (options.innerWidth ? false : options.width),
				  height: (options.innerHeight ? false : options.height),
				  minWidth: "550", //3.0.5 added patched version of colorbox which supports this
				  maxWidth: options.maxWidth,
				  maxHeight: options.maxHeight,
				  innerWidth: options.innerWidth,
				  innerHeight: options.innerHeight,				  
				  opacity: 0.80, 
				  close: options.close,
				  href: options.href,
				  html: options.html,
				  iframe: options.iframe,
				  onComplete: options.onComplete,
				  onCleanup: options.onHide});
				  
	return false;	
}

function insertTemplates(tpl){
	//text
	if($("ta_"+tpl)) $(tpl).value = $F("ta_"+tpl);
	
	//html
	if($("ta_"+tpl+"_html")) $(tpl+"_html").value = $F("ta_"+tpl+"_html");
	if($("ta_"+tpl+"_HTML")) $(tpl+"_HTML").value = $F("ta_"+tpl+"_HTML"); //AUTO RESPONSE TEMPLATE
	
	//subject
	if($("ta_"+tpl+"_subject")) $(tpl+"_subject").value = $F("ta_"+tpl+"_subject");

	//Show save message
	$(tpl+"_savemsg").show();

	$jq.colorbox.close();
}

function rsQuickMenu(elem,reqid){
	var menubody = new Template($("menubtn_tmpl").innerHTML);
	var html = menubody.evaluate({id:reqid});				

	new Tip(elem, html, {
			title: "",
			style: "darkgrey",
			delay: 0,
			showOn: "click",
			hideOn: false,
			hideAfter: 1,
			stem: "topMiddle",
			width: "auto",
			hook: { target: "bottomMiddle", tip: "topMiddle" },
			offset: { x: 0, y: 0 }
		});		
}

function hs_hover(element,newclass){
	if(Element.hasClassName(element,newclass)){
		//Swap to old class
		Element.removeClassName(element,newclass);
	}else{
		//Swap to new class
		Element.addClassName(element,newclass);		
	}
}

function getElementPosition(elemID) {
    var offsetTrail = document.getElementById(elemID);
    var offsetLeft = 0;
    var offsetTop = 0;
    while (offsetTrail) {
        offsetLeft += offsetTrail.offsetLeft;
        offsetTop += offsetTrail.offsetTop;
        offsetTrail = offsetTrail.offsetParent;
    }
    if (navigator.userAgent.indexOf("Mac") != -1 && 
        typeof document.body.leftMargin != "undefined") {
        offsetLeft += document.body.leftMargin;
        offsetTop += document.body.topMargin;
    }
    return {left:offsetLeft, top:offsetTop};
}

function innerWinSize() {
  var myWidth = 0, myHeight = 0;
  if( typeof( window.innerWidth ) == 'number' ) {
    //Non-IE
    myWidth = window.innerWidth;
    myHeight = window.innerHeight;
  } else if( document.documentElement &&
      ( document.documentElement.clientWidth || document.documentElement.clientHeight ) ) {
    //IE 6+ in 'standards compliant mode'
    myWidth = document.documentElement.clientWidth;
    myHeight = document.documentElement.clientHeight;
  } else if( document.body && ( document.body.clientWidth || document.body.clientHeight ) ) {
    //IE 4 compatible
    myWidth = document.body.clientWidth;
    myHeight = document.body.clientHeight;
  }
  return {width:myWidth, height:myHeight};
  
}

// Do lookup for ajax custom fields
function custom_ajax_field_lookup(div,url,calling,loading){
	//Get request fields
	query = getRequestFields();
	
	//Adding calling field id
	query.set('callingField', calling);
	
	var pars = "&url=" + eq(url)+ "&" + query.toQueryString() + "&rand=" + ajaxRandomString();
	
	$(div).innerHTML = loading;
	$(div).show();	
	
	var call = new Ajax.Request(
		'admin.php?pg=ajax_gateway&action=ajax_field_lookup', 
		{
			method: 	 "post",
			parameters:  pars, 
			onComplete:  function(){ 
							$(div).innerHTML = arguments[0].responseText;
						 }
		});	
}

//Popup a LL result
function ll_popup(base){
	Tips.hideAll();	

	$jq.colorbox({minWidth: "550",		  
				  opacity: 0.80, 
				  close: button_close,
				  html: $jq(base).next().html()});
}

function ll_popup_move(num,end){
	if($jq('#ll_popup_'+num).length){
		$jq('#ll_popup_'+num).click();
	}else{
		hs_alert(end);
	}
}

// Get field values for all fields on the request page
function getRequestFields(){
	var req = $H();
	
	//Find any requests fields that may be available and send along, if no customer ID present we"re in an automation rule so send back empty
	if($("sUserId")){
		var req = $H({sUserId: eq($F("sUserId")),
				   sFirstName: eq($F("sFirstName")),
				   sLastName: eq($F("sLastName")),
				   sEmail: eq($F("sEmail")),
				   sPhone: eq($F("sPhone")),
				   xStatus: eq($F("xStatus")),
				   xCategory: eq($F("xCategory")),
				   xPersonAssignedTo: eq($F("xPersonAssignedTo"))});
		
		//Check for custom fields and send in available
		var forms = document.getElementsByTagName("form");
		for(i=0;i < forms.length;i++){
			var f = Form.getElements(document.forms[i]);
			for(e=0;e < f.length;e++){
				if(f[e].id.indexOf("Custom") !== -1 && f[e].id.indexOf("_") === -1){ 
					req.set(f[e].id, eq($F(f[e].id)) );
				}
			}
		}
	}
	
	return req;
}

// HELPSPOT SPECIAL EFFECTS
HS_Effects = {	
	RsSetOrder: function(table,sortdiv){
		Element.hide(table);
		Element.show(sortdiv);
	},
	
	RsReturnSetOrder: function(url){
		goPage(url);
	}
};
// HELPSPOT SPECIAL EFFECTS

// AJAX HELPERS
function ajaxError(){

}

function ajaxLoadingImg(){
	return '<img src="'+static_path+'/css/shared/general/big-loader.gif" />';
}

function ajaxLoading(loading){
	return '<div class="inline_loading">'+ajaxLoadingImg()+'</div>';
}

//IE seems to cache ajax calls, this makes the URL unique
function ajaxRandomString(){
	var chars = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXTZabcdefghiklmnopqrstuvwxyz";
	var string_length = 8;
	var randomstring = '';
	for (var i=0; i<string_length; i++) {
			var rnum = Math.floor(Math.random() * chars.length);
			randomstring += chars.substring(rnum,rnum+1);
	}
	
	return randomstring.toString();
}

//Show spinning thingy for ajax requests
var login_box_active = false;
var ajaxHandlers = {
	onCreate: function(){
		//new Effect.Appear('ajaxFeedback');
	},
	
	onComplete: function(response, transport){
		//if(!Ajax || Ajax.activeRequestCount <= 0){
		//	Element.hide('ajaxFeedback');
		//}
		
		//Handle session timeout
		if(response.getHeader("X-HelpSpot-Session") == "Expired" && !$("colorbox").visible()){
			hs_overlay("login-box", {title:lg_sessionexpired,width: '500px'});
			login_box_active = true;
		}		
	}
};

//Add a selected item from a drop down list to a sortable div list
function addSortableColumn(select_list,sortable_vars,hidden_var,editable,colorselect){
	var listObj = $(select_list);
	
	if(listObj.options){
		if(listObj.selectedIndex != 0){
			var txt = listObj.options[listObj.selectedIndex].text;
			//Sometimes a width value is passed in as well
			if(listObj.options[listObj.selectedIndex].value.indexOf('@@@') !== -1){
				var item = listObj.options[listObj.selectedIndex].value.split('@@@');
				var val = item[0];
				var width = (item[1] ? item[1] : "");
				var hideflow = (item[2] == 'hideflow' ? 'overflowMsg' : 'setColumnWidth');
			}else{
				var val = $F(select_list);
			}		
			var selected = listObj.selectedIndex;
			
			listObj.selectedIndex = 0;
			listObj.options[selected] = null;
		}
	}else{
		var txt = listObj.value;
		var val = eq(listObj.value); //URL encode for placement in hidden field
	}

	var colorid = Math.floor(Math.random() * 2000000000);
	
	var newDiv = Builder.node("div", {id: select_list + "_" + val, className: "sortable_filter", style: "display:none;"}, 
								[Builder.node("img", {src: static_path+"/img/remove2.gif", onClick: "return confirmRemove(\'"+select_list+"_"+val+"\', confirmListDelete);", style: "vertical-align: middle;cursor:pointer;"}),
								" ",
								Builder.node("img", {src: static_path+"/img/drag.gif", className: "drag_handle", style: "vertical-align: middle;cursor:move;"}),
								" ",
								(editable ? Builder.node("img", {src: static_path+"/img/space.gif", width:"16px", height:"16px", style: "vertical-align: middle;cursor:pointer;"}) : ""),
								" - ",
								Builder.node("span", {id: select_list + "_" + val + "_text"}, txt),
								(typeof(width) !== "undefined" ? Builder.node("span", {id: "column_width_" + val, className: "hand filter_width_text", onclick: hideflow+"('column_width_"+val+"');"}, (width !== "" ? width : Builder.node("img", {src:static_path+"/img/fillcol.gif"}))) : "" ),
								(typeof(width) !== "undefined" ? Builder.node("input", {type: "hidden", name: "column_width_" + val + "_value", id: "column_width_" + val + "_value", value: width}) : ""),
								Builder.node("input", {type: "hidden", name: hidden_var, value: val}),
								(typeof(colorselect) !== "undefined" ? Builder.node("input", {"class": "jscolor jscolor-small jscolor-right", "data-jscolor": "{required:false,hash:true}", name: "sListItemsColors[]", value: "", id:colorid}) : ""), //jscolor accepts values in the class attribute
								]
							);
	$(sortable_vars).appendChild(newDiv);
	Effect.Appear(newDiv.id);	

	//Setup JSColor if needed
	if(typeof(colorselect) !== "undefined"){
		var picker = new jscolor(document.getElementById(colorid),{required:false,hash:true,value:null});
	}	
	
	//Destroy original sortable and rebuild with new element
	Sortable.destroy(sortable_vars);
	Sortable.create(sortable_vars ,{tag:"div", constraint: "vertical"});	
}

//Helper for column width JS
function insertColumnWidth(id){
	var value = trim($F(id + "_textbox"));
	
	$(id).update( (value == "" ? "<img src=\""+static_path+"/img/fillcol.gif\" />" : value) );
	$(id + "_value").value = value;
	
	$(id).prototip.remove();
}	

//Fix safari bug where hidden form fields are not sent in reordered order
//Only for use in onsubmit
function safari_order_fix(fieldname, form){
	//Safari only
	if(navigator.appVersion.match(/Konqueror|Safari|KHTML/)){
		
		//Prototype knows correct order of hidden fields so get and create new set of fields
		var inputs = Form.getInputs(form, "hidden", fieldname);
		for(i=0;i<inputs.length;i++){
			new_fields = Builder.node("input", {type: "hidden", name: fieldname, value: inputs[i].value});
			$(form).appendChild(new_fields);
		}			
		
		//Find and remove all hidden form fields
		for(i=0;i<inputs.length;i++){
			Element.remove(inputs[i]);
		}	
	}
	
	return false;
}

//Stop a form entry if a particular form elment is not empty
function stopFormEnter(field){
	if($F(field) == ''){
		return true;
	}else{
		document.getElementById(field).focus();
		return false;	
	}			
}

//Encode string
function eq(string){
	return encodeURIComponent(string);
}

function confirmRemove(id, text){
	return hs_confirm(text,function(){Element.remove(id);});
}

//Insert in text area at cursor position - Copyright Alex King - Licensed LGPL
function insertAtCursor(myField, myValue) {
//IE support
if (document.selection) {
myField.focus();
sel = document.selection.createRange();
sel.text = myValue;
}
//MOZILLA/NETSCAPE support
else if (myField.selectionStart || myField.selectionStart == '0') {
var startPos = myField.selectionStart;
var endPos = myField.selectionEnd;
myField.value = myField.value.substring(0, startPos)
+ myValue
+ myField.value.substring(endPos, myField.value.length);
} else {
myField.value += myValue;
}
}

//Auto grow text areas 
function resize_all_textareas(){
	//If iPhone or iPod Touch then don't resize
	if(!RegExp("iPhone").test(navigator.userAgent) && !RegExp("iPod").test(navigator.userAgent)){
		var textareas=document.getElementsByTagName('textarea');
		for (var i=0;i<textareas.length;i++)
		{	
			//Apply resizing to all areas except the tbody type
			if(textareas[i].id != 'tBody') new ResizeableTextarea(textareas[i]);
		}
	}
}

ResizeableTextarea = Class.create();
ResizeableTextarea.prototype = {
    initialize: function(element, options) {
        this.element = $(element);
        this.size = parseFloat(this.element.getStyle('height') || '100');
        this.options = Object.extend({
            inScreen: true,
            resizeStep: 10,
            minHeight: this.size
        }, options || {});
        Event.observe(this.element, "keyup", this.resize.bindAsEventListener(this));
        if ( !this.options.inScreen ) {
            this.element.style.overflow = 'hidden';
        }
        this.element.setAttribute("wrap","virtual");
        this.resize();
    },
    resize : function(){
        this.shrink();
        this.grow();
    },
    shrink : function(){
        if ( this.size <= this.options.minHeight ){
            return;
        }
        if ( this.element.scrollHeight <= this.element.clientHeight) {
            this.size -= this.options.resizeStep;
            this.element.style.height = this.size+'px';
            this.shrink();
        }
    },
    grow : function(){
        if ( this.element.scrollHeight > this.element.clientHeight ) {
            if ( this.options.inScreen && (20 + this.element.offsetTop + this.element.clientHeight) > document.body.clientHeight ) {
                return;
            }
            this.size += (this.element.scrollHeight - this.element.clientHeight) + this.options.resizeStep;
            this.element.style.height = this.size+'px';
            this.grow();
        }
    }
}

//Cleans the timestamp for date (not datetime) fields
//An identical function is used in the portal helper for date fields
function calendar_clean_date(calendar){
	var id = calendar.params.inputField.id;
	
	//Create javscript date object of selected time
	var odate = new Date($(id).value * 1000);

	//Create a new date starting at midnight
	var ndate = new Date(odate.getFullYear(),odate.getMonth(),odate.getDate(),12,0,0); 

	//Set the modified date back to the ID
	$(id).value = (ndate.getTime()/1000);
}

//Show/hide a date field
function showhide_datefield(rowid){
	if($F(rowid + "_2") == "is" || $F(rowid + "_2") == "less_than" || $F(rowid + "_2") == "greater_than"){
		if($(rowid + "_3")) $(rowid + "_3").show();
		if($("calendar_date_" + rowid)) $("calendar_date_" + rowid).show();
		if($("datepick_" + rowid)) $("datepick_" + rowid).show();	
		if($("datepick_datetime_" + rowid)) $("datepick_datetime_" + rowid).show();	
	}else{
		if($(rowid + "_3")) $(rowid + "_3").hide();
		if($("calendar_date_" + rowid)) $("calendar_date_" + rowid).hide();
		if($("datepick_" + rowid)) $("datepick_" + rowid).hide();	
		if($("datepick_datetime_" + rowid)) $("datepick_datetime_" + rowid).hide();	
	}
}

function showhide_thermostatfield(rowid) {
    if($F(rowid + "_2") == "is" || $F(rowid + "_2") == "less_than" || $F(rowid + "_2") == "greater_than"){
		if($(rowid + "_3")) $(rowid + "_3").show();
    } else {
		if($(rowid + "_3")) $(rowid + "_3").hide();
	}
}

var calc_row;
function do_min_calc() {
	var min;
	min=$("calc_days").value*60*24;
	min=min+($("calc_hours").value*60);
	$(calc_row).value=min;
}

//Note options table - request and merge request
var note_option_string = '<div id="@tabid"><table cellpadding="0" cellspacing="0" border="0" class="noteoptiontable"><tr class="noteoptionrow"><td class="noteoptiontab" width="145">@tabtext</td><td class="noteoptiontabexp" align="right">@tabexp</td></tr></table><table cellpadding="0" cellspacing="0" border="0" style="margin-bottom:0px;"><tr><td colspan="2" class="noteoptioninner" id="noteoptioninner_@tabid">@bodytext</td></tr></table></div>';

function validate_email(email) {
  // Regex from parsley.js
  var regExp = /^((([a-z]|\d|[!#\$%&'\*\+\-\/=\?\^_`{\|}~]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])+(\.([a-z]|\d|[!#\$%&'\*\+\-\/=\?\^_`{\|}~]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])+)*)|((\x22)((((\x20|\x09)*(\x0d\x0a))?(\x20|\x09)+)?(([\x01-\x08\x0b\x0c\x0e-\x1f\x7f]|\x21|[\x23-\x5b]|[\x5d-\x7e]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(\\([\x01-\x09\x0b\x0c\x0d-\x7f]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF]))))*(((\x20|\x09)*(\x0d\x0a))?(\x20|\x09)+)?(\x22)))@((([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.)+(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))$/i;
  if ( ! email) {
    return false;
  } else {
    return regExp.test(email);
  }
}

function showViewers(){

    $jq.get('admin.php?pg=ajax_gateway&action=get_request_viewers', function(data){
        var viewers = [];

        // list of all dom ID's with viewers
        $jq.each(data, function(index, value){
            viewers.push('viewing-'+index);
        });

        $jq.each(data, function(index, value){
            $jq("#viewing-"+index)
                .addClass('viewing-'+value.fType)
                .attr('onclick',"showPersonStatusWorkspace('viewing-"+index+"',"+index+");");
        });

        // Remove classes from any that weren't in result set
        $jq(".viewing").each(function(){
            if($jq.inArray(this.id,viewers) == -1){
                $jq(this).removeClass('viewing-2 viewing-1').removeAttr('onclick');
            }
        });
    },"json");
}

$jq(document).ready(function(){
	var checked = ($jq("input.canCheck:checked").length == $jq("input.canCheck").length);
	$jq(".check-all").prop("checked", checked);
	$jq(".check-all").on("change", function(e){
		$jq(".canCheck").prop("checked", $jq(this).prop("checked"));
	});
	$jq(".canCheck").on("change", function(e) {
		var checked = ($jq(".canCheck:checked").length == $jq(".canCheck").length);
		$jq(".check-all").prop("checked", checked);
	});
	$jq(".js-check-all").on("click", function(e){
		var checked = ! ($jq(".canCheck:checked").length == $jq(".canCheck").length);
		$jq(".canCheck").prop("checked", checked);
		return false;
	});
	$jq('.color-label').contrastColor();
});

// Request Pinning
$jq(document).on("click", "a.note-stream-item-pin", function(e){
	var id = $jq(this).data("id");
	$jq.ajax({
		method: "GET",
		url: "admin.php?pg=ajax_gateway&action=request_history_pin",
		data: { "xRequestHistory": id }
	})
		.done(function( msg ) {
			initRequestHistory();
		});
});

$jq.fn.contrastColor = function() {
	return this.each(function() {
		var bg = $jq(this).css('background-color');
		//use first opaque parent bg if element is transparent
		if(bg == 'transparent' || bg == 'rgba(0, 0, 0, 0)') {
			$jq(this).parents().each(function(){
				bg = $jq(this).css('background-color')
				if(bg != 'transparent' && bg != 'rgba(0, 0, 0, 0)') return false;
			});
			//exit if all parents are transparent
			if(bg == 'transparent' || bg == 'rgba(0, 0, 0, 0)') return false;
		}
		//get r,g,b and decide
		var rgb = bg.replace(/^(rgb|rgba)\(/,'').replace(/\)$/,'').replace(/\s/g,'').split(',');
		var yiq = ((rgb[0]*299)+(rgb[1]*587)+(rgb[2]*114))/1000;
		if(yiq >= 128) $jq(this).removeClass('light-color');
		else $jq(this).addClass('light-color');
	});
};

function addSlashes(str)
{
	return str.replace(/'/g, "\\'");
}
function removeSlashes(str)
{
	return str.replace(/\'/g, "'");
}
// script.aculo.us builder.js v1.9.0, Thu Dec 23 16:54:48 -0500 2010

// Copyright (c) 2005-2010 Thomas Fuchs (http://script.aculo.us, http://mir.aculo.us)
//
// script.aculo.us is freely distributable under the terms of an MIT-style license.
// For details, see the script.aculo.us web site: http://script.aculo.us/

var Builder = {
  NODEMAP: {
    AREA: 'map',
    CAPTION: 'table',
    COL: 'table',
    COLGROUP: 'table',
    LEGEND: 'fieldset',
    OPTGROUP: 'select',
    OPTION: 'select',
    PARAM: 'object',
    TBODY: 'table',
    TD: 'table',
    TFOOT: 'table',
    TH: 'table',
    THEAD: 'table',
    TR: 'table'
  },
  // note: For Firefox < 1.5, OPTION and OPTGROUP tags are currently broken,
  //       due to a Firefox bug
  node: function(elementName) {
    elementName = elementName.toUpperCase();

    // try innerHTML approach
    var parentTag = this.NODEMAP[elementName] || 'div';
    var parentElement = document.createElement(parentTag);
    try { // prevent IE "feature": http://dev.rubyonrails.org/ticket/2707
      parentElement.innerHTML = "<" + elementName + "></" + elementName + ">";
    } catch(e) {}
    var element = parentElement.firstChild || null;

    // see if browser added wrapping tags
    if(element && (element.tagName.toUpperCase() != elementName))
      element = element.getElementsByTagName(elementName)[0];

    // fallback to createElement approach
    if(!element) element = document.createElement(elementName);

    // abort if nothing could be created
    if(!element) return;

    // attributes (or text)
    if(arguments[1])
      if(this._isStringOrNumber(arguments[1]) ||
        (arguments[1] instanceof Array) ||
        arguments[1].tagName) {
          this._children(element, arguments[1]);
        } else {
          var attrs = this._attributes(arguments[1]);
          if(attrs.length) {
            try { // prevent IE "feature": http://dev.rubyonrails.org/ticket/2707
              parentElement.innerHTML = "<" +elementName + " " +
                attrs + "></" + elementName + ">";
            } catch(e) {}
            element = parentElement.firstChild || null;
            // workaround firefox 1.0.X bug
            if(!element) {
              element = document.createElement(elementName);
              for(attr in arguments[1])
                element[attr == 'class' ? 'className' : attr] = arguments[1][attr];
            }
            if(element.tagName.toUpperCase() != elementName)
              element = parentElement.getElementsByTagName(elementName)[0];
          }
        }

    // text, or array of children
    if(arguments[2])
      this._children(element, arguments[2]);

     return $(element);
  },
  _text: function(text) {
     return document.createTextNode(text);
  },

  ATTR_MAP: {
    'className': 'class',
    'htmlFor': 'for'
  },

  _attributes: function(attributes) {
    var attrs = [];
    for(attribute in attributes)
      attrs.push((attribute in this.ATTR_MAP ? this.ATTR_MAP[attribute] : attribute) +
          '="' + attributes[attribute].toString().escapeHTML().gsub(/"/,'&quot;') + '"');
    return attrs.join(" ");
  },
  _children: function(element, children) {
    if(children.tagName) {
      element.appendChild(children);
      return;
    }
    if(typeof children=='object') { // array can hold nodes and text
      children.flatten().each( function(e) {
        if(typeof e=='object')
          element.appendChild(e);
        else
          if(Builder._isStringOrNumber(e))
            element.appendChild(Builder._text(e));
      });
    } else
      if(Builder._isStringOrNumber(children))
        element.appendChild(Builder._text(children));
  },
  _isStringOrNumber: function(param) {
    return(typeof param=='string' || typeof param=='number');
  },
  build: function(html) {
    var element = this.node('div');
    $(element).update(html.strip());
    return element.down();
  },
  dump: function(scope) {
    if(typeof scope != 'object' && typeof scope != 'function') scope = window; //global scope

    var tags = ("A ABBR ACRONYM ADDRESS APPLET AREA B BASE BASEFONT BDO BIG BLOCKQUOTE BODY " +
      "BR BUTTON CAPTION CENTER CITE CODE COL COLGROUP DD DEL DFN DIR DIV DL DT EM FIELDSET " +
      "FONT FORM FRAME FRAMESET H1 H2 H3 H4 H5 H6 HEAD HR HTML I IFRAME IMG INPUT INS ISINDEX "+
      "KBD LABEL LEGEND LI LINK MAP MENU META NOFRAMES NOSCRIPT OBJECT OL OPTGROUP OPTION P "+
      "PARAM PRE Q S SAMP SCRIPT SELECT SMALL SPAN STRIKE STRONG STYLE SUB SUP TABLE TBODY TD "+
      "TEXTAREA TFOOT TH THEAD TITLE TR TT U UL VAR").split(/\s+/);

    tags.each( function(tag){
      scope[tag] = function() {
        return Builder.node.apply(Builder, [tag].concat($A(arguments)));
      };
    });
  }
};
// script.aculo.us effects.js v1.9.0, Thu Dec 23 16:54:48 -0500 2010

// Copyright (c) 2005-2010 Thomas Fuchs (http://script.aculo.us, http://mir.aculo.us)
// Contributors:
//  Justin Palmer (http://encytemedia.com/)
//  Mark Pilgrim (http://diveintomark.org/)
//  Martin Bialasinki
//
// script.aculo.us is freely distributable under the terms of an MIT-style license.
// For details, see the script.aculo.us web site: http://script.aculo.us/

// converts rgb() and #xxx to #xxxxxx format,
// returns self (or first argument) if not convertable
String.prototype.parseColor = function() {
  var color = '#';
  if (this.slice(0,4) == 'rgb(') {
    var cols = this.slice(4,this.length-1).split(',');
    var i=0; do { color += parseInt(cols[i]).toColorPart() } while (++i<3);
  } else {
    if (this.slice(0,1) == '#') {
      if (this.length==4) for(var i=1;i<4;i++) color += (this.charAt(i) + this.charAt(i)).toLowerCase();
      if (this.length==7) color = this.toLowerCase();
    }
  }
  return (color.length==7 ? color : (arguments[0] || this));
};

/*--------------------------------------------------------------------------*/

Element.collectTextNodes = function(element) {
  return $A($(element).childNodes).collect( function(node) {
    return (node.nodeType==3 ? node.nodeValue :
      (node.hasChildNodes() ? Element.collectTextNodes(node) : ''));
  }).flatten().join('');
};

Element.collectTextNodesIgnoreClass = function(element, className) {
  return $A($(element).childNodes).collect( function(node) {
    return (node.nodeType==3 ? node.nodeValue :
      ((node.hasChildNodes() && !Element.hasClassName(node,className)) ?
        Element.collectTextNodesIgnoreClass(node, className) : ''));
  }).flatten().join('');
};

Element.setContentZoom = function(element, percent) {
  element = $(element);
  element.setStyle({fontSize: (percent/100) + 'em'});
  if (Prototype.Browser.WebKit) window.scrollBy(0,0);
  return element;
};

Element.getInlineOpacity = function(element){
  return $(element).style.opacity || '';
};

Element.forceRerendering = function(element) {
  try {
    element = $(element);
    var n = document.createTextNode(' ');
    element.appendChild(n);
    element.removeChild(n);
  } catch(e) { }
};

/*--------------------------------------------------------------------------*/

var Effect = {
  _elementDoesNotExistError: {
    name: 'ElementDoesNotExistError',
    message: 'The specified DOM element does not exist, but is required for this effect to operate'
  },
  Transitions: {
    linear: Prototype.K,
    sinoidal: function(pos) {
      return (-Math.cos(pos*Math.PI)/2) + .5;
    },
    reverse: function(pos) {
      return 1-pos;
    },
    flicker: function(pos) {
      var pos = ((-Math.cos(pos*Math.PI)/4) + .75) + Math.random()/4;
      return pos > 1 ? 1 : pos;
    },
    wobble: function(pos) {
      return (-Math.cos(pos*Math.PI*(9*pos))/2) + .5;
    },
    pulse: function(pos, pulses) {
      return (-Math.cos((pos*((pulses||5)-.5)*2)*Math.PI)/2) + .5;
    },
    spring: function(pos) {
      return 1 - (Math.cos(pos * 4.5 * Math.PI) * Math.exp(-pos * 6));
    },
    none: function(pos) {
      return 0;
    },
    full: function(pos) {
      return 1;
    }
  },
  DefaultOptions: {
    duration:   1.0,   // seconds
    fps:        100,   // 100= assume 66fps max.
    sync:       false, // true for combining
    from:       0.0,
    to:         1.0,
    delay:      0.0,
    queue:      'parallel'
  },
  tagifyText: function(element) {
    var tagifyStyle = 'position:relative';
    if (Prototype.Browser.IE) tagifyStyle += ';zoom:1';

    element = $(element);
    $A(element.childNodes).each( function(child) {
      if (child.nodeType==3) {
        child.nodeValue.toArray().each( function(character) {
          element.insertBefore(
            new Element('span', {style: tagifyStyle}).update(
              character == ' ' ? String.fromCharCode(160) : character),
              child);
        });
        Element.remove(child);
      }
    });
  },
  multiple: function(element, effect) {
    var elements;
    if (((typeof element == 'object') ||
        Object.isFunction(element)) &&
       (element.length))
      elements = element;
    else
      elements = $(element).childNodes;

    var options = Object.extend({
      speed: 0.1,
      delay: 0.0
    }, arguments[2] || { });
    var masterDelay = options.delay;

    $A(elements).each( function(element, index) {
      new effect(element, Object.extend(options, { delay: index * options.speed + masterDelay }));
    });
  },
  PAIRS: {
    'slide':  ['SlideDown','SlideUp'],
    'blind':  ['BlindDown','BlindUp'],
    'appear': ['Appear','Fade']
  },
  toggle: function(element, effect, options) {
    element = $(element);
    effect  = (effect || 'appear').toLowerCase();
    
    return Effect[ Effect.PAIRS[ effect ][ element.visible() ? 1 : 0 ] ](element, Object.extend({
      queue: { position:'end', scope:(element.id || 'global'), limit: 1 }
    }, options || {}));
  }
};

Effect.DefaultOptions.transition = Effect.Transitions.sinoidal;

/* ------------- core effects ------------- */

Effect.ScopedQueue = Class.create(Enumerable, {
  initialize: function() {
    this.effects  = [];
    this.interval = null;
  },
  _each: function(iterator) {
    this.effects._each(iterator);
  },
  add: function(effect) {
    var timestamp = new Date().getTime();

    var position = Object.isString(effect.options.queue) ?
      effect.options.queue : effect.options.queue.position;

    switch(position) {
      case 'front':
        // move unstarted effects after this effect
        this.effects.findAll(function(e){ return e.state=='idle' }).each( function(e) {
            e.startOn  += effect.finishOn;
            e.finishOn += effect.finishOn;
          });
        break;
      case 'with-last':
        timestamp = this.effects.pluck('startOn').max() || timestamp;
        break;
      case 'end':
        // start effect after last queued effect has finished
        timestamp = this.effects.pluck('finishOn').max() || timestamp;
        break;
    }

    effect.startOn  += timestamp;
    effect.finishOn += timestamp;

    if (!effect.options.queue.limit || (this.effects.length < effect.options.queue.limit))
      this.effects.push(effect);

    if (!this.interval)
      this.interval = setInterval(this.loop.bind(this), 15);
  },
  remove: function(effect) {
    this.effects = this.effects.reject(function(e) { return e==effect });
    if (this.effects.length == 0) {
      clearInterval(this.interval);
      this.interval = null;
    }
  },
  loop: function() {
    var timePos = new Date().getTime();
    for(var i=0, len=this.effects.length;i<len;i++)
      this.effects[i] && this.effects[i].loop(timePos);
  }
});

Effect.Queues = {
  instances: $H(),
  get: function(queueName) {
    if (!Object.isString(queueName)) return queueName;

    return this.instances.get(queueName) ||
      this.instances.set(queueName, new Effect.ScopedQueue());
  }
};
Effect.Queue = Effect.Queues.get('global');

Effect.Base = Class.create({
  position: null,
  start: function(options) {
    if (options && options.transition === false) options.transition = Effect.Transitions.linear;
    this.options      = Object.extend(Object.extend({ },Effect.DefaultOptions), options || { });
    this.currentFrame = 0;
    this.state        = 'idle';
    this.startOn      = this.options.delay*1000;
    this.finishOn     = this.startOn+(this.options.duration*1000);
    this.fromToDelta  = this.options.to-this.options.from;
    this.totalTime    = this.finishOn-this.startOn;
    this.totalFrames  = this.options.fps*this.options.duration;

    this.render = (function() {
      function dispatch(effect, eventName) {
        if (effect.options[eventName + 'Internal'])
          effect.options[eventName + 'Internal'](effect);
        if (effect.options[eventName])
          effect.options[eventName](effect);
      }

      return function(pos) {
        if (this.state === "idle") {
          this.state = "running";
          dispatch(this, 'beforeSetup');
          if (this.setup) this.setup();
          dispatch(this, 'afterSetup');
        }
        if (this.state === "running") {
          pos = (this.options.transition(pos) * this.fromToDelta) + this.options.from;
          this.position = pos;
          dispatch(this, 'beforeUpdate');
          if (this.update) this.update(pos);
          dispatch(this, 'afterUpdate');
        }
      };
    })();

    this.event('beforeStart');
    if (!this.options.sync)
      Effect.Queues.get(Object.isString(this.options.queue) ?
        'global' : this.options.queue.scope).add(this);
  },
  loop: function(timePos) {
    if (timePos >= this.startOn) {
      if (timePos >= this.finishOn) {
        this.render(1.0);
        this.cancel();
        this.event('beforeFinish');
        if (this.finish) this.finish();
        this.event('afterFinish');
        return;
      }
      var pos   = (timePos - this.startOn) / this.totalTime,
          frame = (pos * this.totalFrames).round();
      if (frame > this.currentFrame) {
        this.render(pos);
        this.currentFrame = frame;
      }
    }
  },
  cancel: function() {
    if (!this.options.sync)
      Effect.Queues.get(Object.isString(this.options.queue) ?
        'global' : this.options.queue.scope).remove(this);
    this.state = 'finished';
  },
  event: function(eventName) {
    if (this.options[eventName + 'Internal']) this.options[eventName + 'Internal'](this);
    if (this.options[eventName]) this.options[eventName](this);
  },
  inspect: function() {
    var data = $H();
    for(property in this)
      if (!Object.isFunction(this[property])) data.set(property, this[property]);
    return '#<Effect:' + data.inspect() + ',options:' + $H(this.options).inspect() + '>';
  }
});

Effect.Parallel = Class.create(Effect.Base, {
  initialize: function(effects) {
    this.effects = effects || [];
    this.start(arguments[1]);
  },
  update: function(position) {
    this.effects.invoke('render', position);
  },
  finish: function(position) {
    this.effects.each( function(effect) {
      effect.render(1.0);
      effect.cancel();
      effect.event('beforeFinish');
      if (effect.finish) effect.finish(position);
      effect.event('afterFinish');
    });
  }
});

Effect.Tween = Class.create(Effect.Base, {
  initialize: function(object, from, to) {
    object = Object.isString(object) ? $(object) : object;
    var args = $A(arguments), method = args.last(),
      options = args.length == 5 ? args[3] : null;
    this.method = Object.isFunction(method) ? method.bind(object) :
      Object.isFunction(object[method]) ? object[method].bind(object) :
      function(value) { object[method] = value };
    this.start(Object.extend({ from: from, to: to }, options || { }));
  },
  update: function(position) {
    this.method(position);
  }
});

Effect.Event = Class.create(Effect.Base, {
  initialize: function() {
    this.start(Object.extend({ duration: 0 }, arguments[0] || { }));
  },
  update: Prototype.emptyFunction
});

Effect.Opacity = Class.create(Effect.Base, {
  initialize: function(element) {
    this.element = $(element);
    if (!this.element) throw(Effect._elementDoesNotExistError);
    // make this work on IE on elements without 'layout'
    if (Prototype.Browser.IE && (!this.element.currentStyle.hasLayout))
      this.element.setStyle({zoom: 1});
    var options = Object.extend({
      from: this.element.getOpacity() || 0.0,
      to:   1.0
    }, arguments[1] || { });
    this.start(options);
  },
  update: function(position) {
    this.element.setOpacity(position);
  }
});

Effect.Move = Class.create(Effect.Base, {
  initialize: function(element) {
    this.element = $(element);
    if (!this.element) throw(Effect._elementDoesNotExistError);
    var options = Object.extend({
      x:    0,
      y:    0,
      mode: 'relative'
    }, arguments[1] || { });
    this.start(options);
  },
  setup: function() {
    this.element.makePositioned();
    this.originalLeft = parseFloat(this.element.getStyle('left') || '0');
    this.originalTop  = parseFloat(this.element.getStyle('top')  || '0');
    if (this.options.mode == 'absolute') {
      this.options.x = this.options.x - this.originalLeft;
      this.options.y = this.options.y - this.originalTop;
    }
  },
  update: function(position) {
    this.element.setStyle({
      left: (this.options.x  * position + this.originalLeft).round() + 'px',
      top:  (this.options.y  * position + this.originalTop).round()  + 'px'
    });
  }
});

// for backwards compatibility
Effect.MoveBy = function(element, toTop, toLeft) {
  return new Effect.Move(element,
    Object.extend({ x: toLeft, y: toTop }, arguments[3] || { }));
};

Effect.Scale = Class.create(Effect.Base, {
  initialize: function(element, percent) {
    this.element = $(element);
    if (!this.element) throw(Effect._elementDoesNotExistError);
    var options = Object.extend({
      scaleX: true,
      scaleY: true,
      scaleContent: true,
      scaleFromCenter: false,
      scaleMode: 'box',        // 'box' or 'contents' or { } with provided values
      scaleFrom: 100.0,
      scaleTo:   percent
    }, arguments[2] || { });
    this.start(options);
  },
  setup: function() {
    this.restoreAfterFinish = this.options.restoreAfterFinish || false;
    this.elementPositioning = this.element.getStyle('position');

    this.originalStyle = { };
    ['top','left','width','height','fontSize'].each( function(k) {
      this.originalStyle[k] = this.element.style[k];
    }.bind(this));

    this.originalTop  = this.element.offsetTop;
    this.originalLeft = this.element.offsetLeft;

    var fontSize = this.element.getStyle('font-size') || '100%';
    ['em','px','%','pt'].each( function(fontSizeType) {
      if (fontSize.indexOf(fontSizeType)>0) {
        this.fontSize     = parseFloat(fontSize);
        this.fontSizeType = fontSizeType;
      }
    }.bind(this));

    this.factor = (this.options.scaleTo - this.options.scaleFrom)/100;

    this.dims = null;
    if (this.options.scaleMode=='box')
      this.dims = [this.element.offsetHeight, this.element.offsetWidth];
    if (/^content/.test(this.options.scaleMode))
      this.dims = [this.element.scrollHeight, this.element.scrollWidth];
    if (!this.dims)
      this.dims = [this.options.scaleMode.originalHeight,
                   this.options.scaleMode.originalWidth];
  },
  update: function(position) {
    var currentScale = (this.options.scaleFrom/100.0) + (this.factor * position);
    if (this.options.scaleContent && this.fontSize)
      this.element.setStyle({fontSize: this.fontSize * currentScale + this.fontSizeType });
    this.setDimensions(this.dims[0] * currentScale, this.dims[1] * currentScale);
  },
  finish: function(position) {
    if (this.restoreAfterFinish) this.element.setStyle(this.originalStyle);
  },
  setDimensions: function(height, width) {
    var d = { };
    if (this.options.scaleX) d.width = width.round() + 'px';
    if (this.options.scaleY) d.height = height.round() + 'px';
    if (this.options.scaleFromCenter) {
      var topd  = (height - this.dims[0])/2;
      var leftd = (width  - this.dims[1])/2;
      if (this.elementPositioning == 'absolute') {
        if (this.options.scaleY) d.top = this.originalTop-topd + 'px';
        if (this.options.scaleX) d.left = this.originalLeft-leftd + 'px';
      } else {
        if (this.options.scaleY) d.top = -topd + 'px';
        if (this.options.scaleX) d.left = -leftd + 'px';
      }
    }
    this.element.setStyle(d);
  }
});

Effect.Highlight = Class.create(Effect.Base, {
  initialize: function(element) {
    this.element = $(element);
    if (!this.element) throw(Effect._elementDoesNotExistError);
    var options = Object.extend({ startcolor: '#ffff99' }, arguments[1] || { });
    this.start(options);
  },
  setup: function() {
    // Prevent executing on elements not in the layout flow
    if (this.element.getStyle('display')=='none') { this.cancel(); return; }
    // Disable background image during the effect
    this.oldStyle = { };
    if (!this.options.keepBackgroundImage) {
      this.oldStyle.backgroundImage = this.element.getStyle('background-image');
      this.element.setStyle({backgroundImage: 'none'});
    }
    if (!this.options.endcolor)
      this.options.endcolor = this.element.getStyle('background-color').parseColor('#ffffff');
    if (!this.options.restorecolor)
      this.options.restorecolor = this.element.getStyle('background-color');
    // init color calculations
    this._base  = $R(0,2).map(function(i){ return parseInt(this.options.startcolor.slice(i*2+1,i*2+3),16) }.bind(this));
    this._delta = $R(0,2).map(function(i){ return parseInt(this.options.endcolor.slice(i*2+1,i*2+3),16)-this._base[i] }.bind(this));
  },
  update: function(position) {
    this.element.setStyle({backgroundColor: $R(0,2).inject('#',function(m,v,i){
      return m+((this._base[i]+(this._delta[i]*position)).round().toColorPart()); }.bind(this)) });
  },
  finish: function() {
    this.element.setStyle(Object.extend(this.oldStyle, {
      backgroundColor: this.options.restorecolor
    }));
  }
});

Effect.ScrollTo = function(element) {
  var options = arguments[1] || { },
  scrollOffsets = document.viewport.getScrollOffsets(),
  elementOffsets = $(element).cumulativeOffset();

  if (options.offset) elementOffsets[1] += options.offset;

  return new Effect.Tween(null,
    scrollOffsets.top,
    elementOffsets[1],
    options,
    function(p){ scrollTo(scrollOffsets.left, p.round()); }
  );
};

/* ------------- combination effects ------------- */

Effect.Fade = function(element) {
  element = $(element);
  var oldOpacity = element.getInlineOpacity();
  var options = Object.extend({
    from: element.getOpacity() || 1.0,
    to:   0.0,
    afterFinishInternal: function(effect) {
      if (effect.options.to!=0) return;
      effect.element.hide().setStyle({opacity: oldOpacity});
    }
  }, arguments[1] || { });
  return new Effect.Opacity(element,options);
};

Effect.Appear = function(element) {
  element = $(element);
  var options = Object.extend({
  from: (element.getStyle('display') == 'none' ? 0.0 : element.getOpacity() || 0.0),
  to:   1.0,
  // force Safari to render floated elements properly
  afterFinishInternal: function(effect) {
    effect.element.forceRerendering();
  },
  beforeSetup: function(effect) {
    effect.element.setOpacity(effect.options.from).show();
  }}, arguments[1] || { });
  return new Effect.Opacity(element,options);
};

Effect.Puff = function(element) {
  element = $(element);
  var oldStyle = {
    opacity: element.getInlineOpacity(),
    position: element.getStyle('position'),
    top:  element.style.top,
    left: element.style.left,
    width: element.style.width,
    height: element.style.height
  };
  return new Effect.Parallel(
   [ new Effect.Scale(element, 200,
      { sync: true, scaleFromCenter: true, scaleContent: true, restoreAfterFinish: true }),
     new Effect.Opacity(element, { sync: true, to: 0.0 } ) ],
     Object.extend({ duration: 1.0,
      beforeSetupInternal: function(effect) {
        Position.absolutize(effect.effects[0].element);
      },
      afterFinishInternal: function(effect) {
         effect.effects[0].element.hide().setStyle(oldStyle); }
     }, arguments[1] || { })
   );
};

Effect.BlindUp = function(element) {
  element = $(element);
  element.makeClipping();
  return new Effect.Scale(element, 0,
    Object.extend({ scaleContent: false,
      scaleX: false,
      restoreAfterFinish: true,
      afterFinishInternal: function(effect) {
        effect.element.hide().undoClipping();
      }
    }, arguments[1] || { })
  );
};

Effect.BlindDown = function(element) {
  element = $(element);
  var elementDimensions = element.getDimensions();
  return new Effect.Scale(element, 100, Object.extend({
    scaleContent: false,
    scaleX: false,
    scaleFrom: 0,
    scaleMode: {originalHeight: elementDimensions.height, originalWidth: elementDimensions.width},
    restoreAfterFinish: true,
    afterSetup: function(effect) {
      effect.element.makeClipping().setStyle({height: '0px'}).show();
    },
    afterFinishInternal: function(effect) {
      effect.element.undoClipping();
    }
  }, arguments[1] || { }));
};

Effect.SwitchOff = function(element) {
  element = $(element);
  var oldOpacity = element.getInlineOpacity();
  return new Effect.Appear(element, Object.extend({
    duration: 0.4,
    from: 0,
    transition: Effect.Transitions.flicker,
    afterFinishInternal: function(effect) {
      new Effect.Scale(effect.element, 1, {
        duration: 0.3, scaleFromCenter: true,
        scaleX: false, scaleContent: false, restoreAfterFinish: true,
        beforeSetup: function(effect) {
          effect.element.makePositioned().makeClipping();
        },
        afterFinishInternal: function(effect) {
          effect.element.hide().undoClipping().undoPositioned().setStyle({opacity: oldOpacity});
        }
      });
    }
  }, arguments[1] || { }));
};

Effect.DropOut = function(element) {
  element = $(element);
  var oldStyle = {
    top: element.getStyle('top'),
    left: element.getStyle('left'),
    opacity: element.getInlineOpacity() };
  return new Effect.Parallel(
    [ new Effect.Move(element, {x: 0, y: 100, sync: true }),
      new Effect.Opacity(element, { sync: true, to: 0.0 }) ],
    Object.extend(
      { duration: 0.5,
        beforeSetup: function(effect) {
          effect.effects[0].element.makePositioned();
        },
        afterFinishInternal: function(effect) {
          effect.effects[0].element.hide().undoPositioned().setStyle(oldStyle);
        }
      }, arguments[1] || { }));
};

Effect.Shake = function(element) {
  element = $(element);
  var options = Object.extend({
    distance: 20,
    duration: 0.5
  }, arguments[1] || {});
  var distance = parseFloat(options.distance);
  var split = parseFloat(options.duration) / 10.0;
  var oldStyle = {
    top: element.getStyle('top'),
    left: element.getStyle('left') };
    return new Effect.Move(element,
      { x:  distance, y: 0, duration: split, afterFinishInternal: function(effect) {
    new Effect.Move(effect.element,
      { x: -distance*2, y: 0, duration: split*2,  afterFinishInternal: function(effect) {
    new Effect.Move(effect.element,
      { x:  distance*2, y: 0, duration: split*2,  afterFinishInternal: function(effect) {
    new Effect.Move(effect.element,
      { x: -distance*2, y: 0, duration: split*2,  afterFinishInternal: function(effect) {
    new Effect.Move(effect.element,
      { x:  distance*2, y: 0, duration: split*2,  afterFinishInternal: function(effect) {
    new Effect.Move(effect.element,
      { x: -distance, y: 0, duration: split, afterFinishInternal: function(effect) {
        effect.element.undoPositioned().setStyle(oldStyle);
  }}); }}); }}); }}); }}); }});
};

Effect.SlideDown = function(element) {
  element = $(element).cleanWhitespace();
  // SlideDown need to have the content of the element wrapped in a container element with fixed height!
  var oldInnerBottom = element.down().getStyle('bottom');
  var elementDimensions = element.getDimensions();
  return new Effect.Scale(element, 100, Object.extend({
    scaleContent: false,
    scaleX: false,
    scaleFrom: window.opera ? 0 : 1,
    scaleMode: {originalHeight: elementDimensions.height, originalWidth: elementDimensions.width},
    restoreAfterFinish: true,
    afterSetup: function(effect) {
      effect.element.makePositioned();
      effect.element.down().makePositioned();
      if (window.opera) effect.element.setStyle({top: ''});
      effect.element.makeClipping().setStyle({height: '0px'}).show();
    },
    afterUpdateInternal: function(effect) {
      effect.element.down().setStyle({bottom:
        (effect.dims[0] - effect.element.clientHeight) + 'px' });
    },
    afterFinishInternal: function(effect) {
      effect.element.undoClipping().undoPositioned();
      effect.element.down().undoPositioned().setStyle({bottom: oldInnerBottom}); }
    }, arguments[1] || { })
  );
};

Effect.SlideUp = function(element) {
  element = $(element).cleanWhitespace();
  var oldInnerBottom = element.down().getStyle('bottom');
  var elementDimensions = element.getDimensions();
  return new Effect.Scale(element, window.opera ? 0 : 1,
   Object.extend({ scaleContent: false,
    scaleX: false,
    scaleMode: 'box',
    scaleFrom: 100,
    scaleMode: {originalHeight: elementDimensions.height, originalWidth: elementDimensions.width},
    restoreAfterFinish: true,
    afterSetup: function(effect) {
      effect.element.makePositioned();
      effect.element.down().makePositioned();
      if (window.opera) effect.element.setStyle({top: ''});
      effect.element.makeClipping().show();
    },
    afterUpdateInternal: function(effect) {
      effect.element.down().setStyle({bottom:
        (effect.dims[0] - effect.element.clientHeight) + 'px' });
    },
    afterFinishInternal: function(effect) {
      effect.element.hide().undoClipping().undoPositioned();
      effect.element.down().undoPositioned().setStyle({bottom: oldInnerBottom});
    }
   }, arguments[1] || { })
  );
};

// Bug in opera makes the TD containing this element expand for a instance after finish
Effect.Squish = function(element) {
  return new Effect.Scale(element, window.opera ? 1 : 0, {
    restoreAfterFinish: true,
    beforeSetup: function(effect) {
      effect.element.makeClipping();
    },
    afterFinishInternal: function(effect) {
      effect.element.hide().undoClipping();
    }
  });
};

Effect.Grow = function(element) {
  element = $(element);
  var options = Object.extend({
    direction: 'center',
    moveTransition: Effect.Transitions.sinoidal,
    scaleTransition: Effect.Transitions.sinoidal,
    opacityTransition: Effect.Transitions.full
  }, arguments[1] || { });
  var oldStyle = {
    top: element.style.top,
    left: element.style.left,
    height: element.style.height,
    width: element.style.width,
    opacity: element.getInlineOpacity() };

  var dims = element.getDimensions();
  var initialMoveX, initialMoveY;
  var moveX, moveY;

  switch (options.direction) {
    case 'top-left':
      initialMoveX = initialMoveY = moveX = moveY = 0;
      break;
    case 'top-right':
      initialMoveX = dims.width;
      initialMoveY = moveY = 0;
      moveX = -dims.width;
      break;
    case 'bottom-left':
      initialMoveX = moveX = 0;
      initialMoveY = dims.height;
      moveY = -dims.height;
      break;
    case 'bottom-right':
      initialMoveX = dims.width;
      initialMoveY = dims.height;
      moveX = -dims.width;
      moveY = -dims.height;
      break;
    case 'center':
      initialMoveX = dims.width / 2;
      initialMoveY = dims.height / 2;
      moveX = -dims.width / 2;
      moveY = -dims.height / 2;
      break;
  }

  return new Effect.Move(element, {
    x: initialMoveX,
    y: initialMoveY,
    duration: 0.01,
    beforeSetup: function(effect) {
      effect.element.hide().makeClipping().makePositioned();
    },
    afterFinishInternal: function(effect) {
      new Effect.Parallel(
        [ new Effect.Opacity(effect.element, { sync: true, to: 1.0, from: 0.0, transition: options.opacityTransition }),
          new Effect.Move(effect.element, { x: moveX, y: moveY, sync: true, transition: options.moveTransition }),
          new Effect.Scale(effect.element, 100, {
            scaleMode: { originalHeight: dims.height, originalWidth: dims.width },
            sync: true, scaleFrom: window.opera ? 1 : 0, transition: options.scaleTransition, restoreAfterFinish: true})
        ], Object.extend({
             beforeSetup: function(effect) {
               effect.effects[0].element.setStyle({height: '0px'}).show();
             },
             afterFinishInternal: function(effect) {
               effect.effects[0].element.undoClipping().undoPositioned().setStyle(oldStyle);
             }
           }, options)
      );
    }
  });
};

Effect.Shrink = function(element) {
  element = $(element);
  var options = Object.extend({
    direction: 'center',
    moveTransition: Effect.Transitions.sinoidal,
    scaleTransition: Effect.Transitions.sinoidal,
    opacityTransition: Effect.Transitions.none
  }, arguments[1] || { });
  var oldStyle = {
    top: element.style.top,
    left: element.style.left,
    height: element.style.height,
    width: element.style.width,
    opacity: element.getInlineOpacity() };

  var dims = element.getDimensions();
  var moveX, moveY;

  switch (options.direction) {
    case 'top-left':
      moveX = moveY = 0;
      break;
    case 'top-right':
      moveX = dims.width;
      moveY = 0;
      break;
    case 'bottom-left':
      moveX = 0;
      moveY = dims.height;
      break;
    case 'bottom-right':
      moveX = dims.width;
      moveY = dims.height;
      break;
    case 'center':
      moveX = dims.width / 2;
      moveY = dims.height / 2;
      break;
  }

  return new Effect.Parallel(
    [ new Effect.Opacity(element, { sync: true, to: 0.0, from: 1.0, transition: options.opacityTransition }),
      new Effect.Scale(element, window.opera ? 1 : 0, { sync: true, transition: options.scaleTransition, restoreAfterFinish: true}),
      new Effect.Move(element, { x: moveX, y: moveY, sync: true, transition: options.moveTransition })
    ], Object.extend({
         beforeStartInternal: function(effect) {
           effect.effects[0].element.makePositioned().makeClipping();
         },
         afterFinishInternal: function(effect) {
           effect.effects[0].element.hide().undoClipping().undoPositioned().setStyle(oldStyle); }
       }, options)
  );
};

Effect.Pulsate = function(element) {
  element = $(element);
  var options    = arguments[1] || { },
    oldOpacity = element.getInlineOpacity(),
    transition = options.transition || Effect.Transitions.linear,
    reverser   = function(pos){
      return 1 - transition((-Math.cos((pos*(options.pulses||5)*2)*Math.PI)/2) + .5);
    };

  return new Effect.Opacity(element,
    Object.extend(Object.extend({  duration: 2.0, from: 0,
      afterFinishInternal: function(effect) { effect.element.setStyle({opacity: oldOpacity}); }
    }, options), {transition: reverser}));
};

Effect.Fold = function(element) {
  element = $(element);
  var oldStyle = {
    top: element.style.top,
    left: element.style.left,
    width: element.style.width,
    height: element.style.height };
  element.makeClipping();
  return new Effect.Scale(element, 5, Object.extend({
    scaleContent: false,
    scaleX: false,
    afterFinishInternal: function(effect) {
    new Effect.Scale(element, 1, {
      scaleContent: false,
      scaleY: false,
      afterFinishInternal: function(effect) {
        effect.element.hide().undoClipping().setStyle(oldStyle);
      } });
  }}, arguments[1] || { }));
};

Effect.Morph = Class.create(Effect.Base, {
  initialize: function(element) {
    this.element = $(element);
    if (!this.element) throw(Effect._elementDoesNotExistError);
    var options = Object.extend({
      style: { }
    }, arguments[1] || { });

    if (!Object.isString(options.style)) this.style = $H(options.style);
    else {
      if (options.style.include(':'))
        this.style = options.style.parseStyle();
      else {
        this.element.addClassName(options.style);
        this.style = $H(this.element.getStyles());
        this.element.removeClassName(options.style);
        var css = this.element.getStyles();
        this.style = this.style.reject(function(style) {
          return style.value == css[style.key];
        });
        options.afterFinishInternal = function(effect) {
          effect.element.addClassName(effect.options.style);
          effect.transforms.each(function(transform) {
            effect.element.style[transform.style] = '';
          });
        };
      }
    }
    this.start(options);
  },

  setup: function(){
    function parseColor(color){
      if (!color || ['rgba(0, 0, 0, 0)','transparent'].include(color)) color = '#ffffff';
      color = color.parseColor();
      return $R(0,2).map(function(i){
        return parseInt( color.slice(i*2+1,i*2+3), 16 );
      });
    }
    this.transforms = this.style.map(function(pair){
      var property = pair[0], value = pair[1], unit = null;

      if (value.parseColor('#zzzzzz') != '#zzzzzz') {
        value = value.parseColor();
        unit  = 'color';
      } else if (property == 'opacity') {
        value = parseFloat(value);
        if (Prototype.Browser.IE && (!this.element.currentStyle.hasLayout))
          this.element.setStyle({zoom: 1});
      } else if (Element.CSS_LENGTH.test(value)) {
          var components = value.match(/^([\+\-]?[0-9\.]+)(.*)$/);
          value = parseFloat(components[1]);
          unit = (components.length == 3) ? components[2] : null;
      }

      var originalValue = this.element.getStyle(property);
      return {
        style: property.camelize(),
        originalValue: unit=='color' ? parseColor(originalValue) : parseFloat(originalValue || 0),
        targetValue: unit=='color' ? parseColor(value) : value,
        unit: unit
      };
    }.bind(this)).reject(function(transform){
      return (
        (transform.originalValue == transform.targetValue) ||
        (
          transform.unit != 'color' &&
          (isNaN(transform.originalValue) || isNaN(transform.targetValue))
        )
      );
    });
  },
  update: function(position) {
    var style = { }, transform, i = this.transforms.length;
    while(i--)
      style[(transform = this.transforms[i]).style] =
        transform.unit=='color' ? '#'+
          (Math.round(transform.originalValue[0]+
            (transform.targetValue[0]-transform.originalValue[0])*position)).toColorPart() +
          (Math.round(transform.originalValue[1]+
            (transform.targetValue[1]-transform.originalValue[1])*position)).toColorPart() +
          (Math.round(transform.originalValue[2]+
            (transform.targetValue[2]-transform.originalValue[2])*position)).toColorPart() :
        (transform.originalValue +
          (transform.targetValue - transform.originalValue) * position).toFixed(3) +
            (transform.unit === null ? '' : transform.unit);
    this.element.setStyle(style, true);
  }
});

Effect.Transform = Class.create({
  initialize: function(tracks){
    this.tracks  = [];
    this.options = arguments[1] || { };
    this.addTracks(tracks);
  },
  addTracks: function(tracks){
    tracks.each(function(track){
      track = $H(track);
      var data = track.values().first();
      this.tracks.push($H({
        ids:     track.keys().first(),
        effect:  Effect.Morph,
        options: { style: data }
      }));
    }.bind(this));
    return this;
  },
  play: function(){
    return new Effect.Parallel(
      this.tracks.map(function(track){
        var ids = track.get('ids'), effect = track.get('effect'), options = track.get('options');
        var elements = [$(ids) || $$(ids)].flatten();
        return elements.map(function(e){ return new effect(e, Object.extend({ sync:true }, options)) });
      }).flatten(),
      this.options
    );
  }
});

Element.CSS_PROPERTIES = $w(
  'backgroundColor backgroundPosition borderBottomColor borderBottomStyle ' +
  'borderBottomWidth borderLeftColor borderLeftStyle borderLeftWidth ' +
  'borderRightColor borderRightStyle borderRightWidth borderSpacing ' +
  'borderTopColor borderTopStyle borderTopWidth bottom clip color ' +
  'fontSize fontWeight height left letterSpacing lineHeight ' +
  'marginBottom marginLeft marginRight marginTop markerOffset maxHeight '+
  'maxWidth minHeight minWidth opacity outlineColor outlineOffset ' +
  'outlineWidth paddingBottom paddingLeft paddingRight paddingTop ' +
  'right textIndent top width wordSpacing zIndex');

Element.CSS_LENGTH = /^(([\+\-]?[0-9\.]+)(em|ex|px|in|cm|mm|pt|pc|\%))|0$/;

String.__parseStyleElement = document.createElement('div');
String.prototype.parseStyle = function(){
  var style, styleRules = $H();
  if (Prototype.Browser.WebKit)
    style = new Element('div',{style:this}).style;
  else {
    String.__parseStyleElement.innerHTML = '<div style="' + this + '"></div>';
    style = String.__parseStyleElement.childNodes[0].style;
  }

  Element.CSS_PROPERTIES.each(function(property){
    if (style[property]) styleRules.set(property, style[property]);
  });

  if (Prototype.Browser.IE && this.include('opacity'))
    styleRules.set('opacity', this.match(/opacity:\s*((?:0|1)?(?:\.\d*)?)/)[1]);

  return styleRules;
};

if (document.defaultView && document.defaultView.getComputedStyle) {
  Element.getStyles = function(element) {
    var css = document.defaultView.getComputedStyle($(element), null);
    return Element.CSS_PROPERTIES.inject({ }, function(styles, property) {
      styles[property] = css[property];
      return styles;
    });
  };
} else {
  Element.getStyles = function(element) {
    element = $(element);
    var css = element.currentStyle, styles;
    styles = Element.CSS_PROPERTIES.inject({ }, function(results, property) {
      results[property] = css[property];
      return results;
    });
    if (!styles.opacity) styles.opacity = element.getOpacity();
    return styles;
  };
}

Effect.Methods = {
  morph: function(element, style) {
    element = $(element);
    new Effect.Morph(element, Object.extend({ style: style }, arguments[2] || { }));
    return element;
  },
  visualEffect: function(element, effect, options) {
    element = $(element);
    var s = effect.dasherize().camelize(), klass = s.charAt(0).toUpperCase() + s.substring(1);
    new Effect[klass](element, options);
    return element;
  },
  highlight: function(element, options) {
    element = $(element);
    new Effect.Highlight(element, options);
    return element;
  }
};

$w('fade appear grow shrink fold blindUp blindDown slideUp slideDown '+
  'pulsate shake puff squish switchOff dropOut').each(
  function(effect) {
    Effect.Methods[effect] = function(element, options){
      element = $(element);
      Effect[effect.charAt(0).toUpperCase() + effect.substring(1)](element, options);
      return element;
    };
  }
);

$w('getInlineOpacity forceRerendering setContentZoom collectTextNodes collectTextNodesIgnoreClass getStyles').each(
  function(f) { Effect.Methods[f] = Element[f]; }
);

Element.addMethods(Effect.Methods);
// script.aculo.us dragdrop.js v1.9.0, Thu Dec 23 16:54:48 -0500 2010

// Copyright (c) 2005-2010 Thomas Fuchs (http://script.aculo.us, http://mir.aculo.us)
//
// script.aculo.us is freely distributable under the terms of an MIT-style license.
// For details, see the script.aculo.us web site: http://script.aculo.us/

if(Object.isUndefined(Effect))
  throw("dragdrop.js requires including script.aculo.us' effects.js library");

var Droppables = {
  drops: [],

  remove: function(element) {
    this.drops = this.drops.reject(function(d) { return d.element==$(element) });
  },

  add: function(element) {
    element = $(element);
    var options = Object.extend({
      greedy:     true,
      hoverclass: null,
      tree:       false
    }, arguments[1] || { });

    // cache containers
    if(options.containment) {
      options._containers = [];
      var containment = options.containment;
      if(Object.isArray(containment)) {
        containment.each( function(c) { options._containers.push($(c)) });
      } else {
        options._containers.push($(containment));
      }
    }

    if(options.accept) options.accept = [options.accept].flatten();

    Element.makePositioned(element); // fix IE
    options.element = element;

    this.drops.push(options);
  },

  findDeepestChild: function(drops) {
    deepest = drops[0];

    for (i = 1; i < drops.length; ++i)
      if (Element.isParent(drops[i].element, deepest.element))
        deepest = drops[i];

    return deepest;
  },

  isContained: function(element, drop) {
    var containmentNode;
    if(drop.tree) {
      containmentNode = element.treeNode;
    } else {
      containmentNode = element.parentNode;
    }
    return drop._containers.detect(function(c) { return containmentNode == c });
  },

  isAffected: function(point, element, drop) {
    return (
      (drop.element!=element) &&
      ((!drop._containers) ||
        this.isContained(element, drop)) &&
      ((!drop.accept) ||
        (Element.classNames(element).detect(
          function(v) { return drop.accept.include(v) } ) )) &&
      Position.within(drop.element, point[0], point[1]) );
  },

  deactivate: function(drop) {
    if(drop.hoverclass)
      Element.removeClassName(drop.element, drop.hoverclass);
    this.last_active = null;
  },

  activate: function(drop) {
    if(drop.hoverclass)
      Element.addClassName(drop.element, drop.hoverclass);
    this.last_active = drop;
  },

  show: function(point, element) {
    if(!this.drops.length) return;
    var drop, affected = [];

    this.drops.each( function(drop) {
      if(Droppables.isAffected(point, element, drop))
        affected.push(drop);
    });

    if(affected.length>0)
      drop = Droppables.findDeepestChild(affected);

    if(this.last_active && this.last_active != drop) this.deactivate(this.last_active);
    if (drop) {
      Position.within(drop.element, point[0], point[1]);
      if(drop.onHover)
        drop.onHover(element, drop.element, Position.overlap(drop.overlap, drop.element));

      if (drop != this.last_active) Droppables.activate(drop);
    }
  },

  fire: function(event, element) {
    if(!this.last_active) return;
    Position.prepare();

    if (this.isAffected([Event.pointerX(event), Event.pointerY(event)], element, this.last_active))
      if (this.last_active.onDrop) {
        this.last_active.onDrop(element, this.last_active.element, event);
        return true;
      }
  },

  reset: function() {
    if(this.last_active)
      this.deactivate(this.last_active);
  }
};

var Draggables = {
  drags: [],
  observers: [],

  register: function(draggable) {
    if(this.drags.length == 0) {
      this.eventMouseUp   = this.endDrag.bindAsEventListener(this);
      this.eventMouseMove = this.updateDrag.bindAsEventListener(this);
      this.eventKeypress  = this.keyPress.bindAsEventListener(this);

      Event.observe(document, "mouseup", this.eventMouseUp);
      Event.observe(document, "mousemove", this.eventMouseMove);
      Event.observe(document, "keypress", this.eventKeypress);
    }
    this.drags.push(draggable);
  },

  unregister: function(draggable) {
    this.drags = this.drags.reject(function(d) { return d==draggable });
    if(this.drags.length == 0) {
      Event.stopObserving(document, "mouseup", this.eventMouseUp);
      Event.stopObserving(document, "mousemove", this.eventMouseMove);
      Event.stopObserving(document, "keypress", this.eventKeypress);
    }
  },

  activate: function(draggable) {
    if(draggable.options.delay) {
      this._timeout = setTimeout(function() {
        Draggables._timeout = null;
        window.focus();
        Draggables.activeDraggable = draggable;
      }.bind(this), draggable.options.delay);
    } else {
      window.focus(); // allows keypress events if window isn't currently focused, fails for Safari
      this.activeDraggable = draggable;
    }
  },

  deactivate: function() {
    this.activeDraggable = null;
  },

  updateDrag: function(event) {
    if(!this.activeDraggable) return;
    var pointer = [Event.pointerX(event), Event.pointerY(event)];
    // Mozilla-based browsers fire successive mousemove events with
    // the same coordinates, prevent needless redrawing (moz bug?)
    if(this._lastPointer && (this._lastPointer.inspect() == pointer.inspect())) return;
    this._lastPointer = pointer;

    this.activeDraggable.updateDrag(event, pointer);
  },

  endDrag: function(event) {
    if(this._timeout) {
      clearTimeout(this._timeout);
      this._timeout = null;
    }
    if(!this.activeDraggable) return;
    this._lastPointer = null;
    this.activeDraggable.endDrag(event);
    this.activeDraggable = null;
  },

  keyPress: function(event) {
    if(this.activeDraggable)
      this.activeDraggable.keyPress(event);
  },

  addObserver: function(observer) {
    this.observers.push(observer);
    this._cacheObserverCallbacks();
  },

  removeObserver: function(element) {  // element instead of observer fixes mem leaks
    this.observers = this.observers.reject( function(o) { return o.element==element });
    this._cacheObserverCallbacks();
  },

  notify: function(eventName, draggable, event) {  // 'onStart', 'onEnd', 'onDrag'
    if(this[eventName+'Count'] > 0)
      this.observers.each( function(o) {
        if(o[eventName]) o[eventName](eventName, draggable, event);
      });
    if(draggable.options[eventName]) draggable.options[eventName](draggable, event);
  },

  _cacheObserverCallbacks: function() {
    ['onStart','onEnd','onDrag'].each( function(eventName) {
      Draggables[eventName+'Count'] = Draggables.observers.select(
        function(o) { return o[eventName]; }
      ).length;
    });
  }
};

/*--------------------------------------------------------------------------*/

var Draggable = Class.create({
  initialize: function(element) {
    var defaults = {
      handle: false,
      reverteffect: function(element, top_offset, left_offset) {
        var dur = Math.sqrt(Math.abs(top_offset^2)+Math.abs(left_offset^2))*0.02;
        new Effect.Move(element, { x: -left_offset, y: -top_offset, duration: dur,
          queue: {scope:'_draggable', position:'end'}
        });
      },
      endeffect: function(element) {
        var toOpacity = Object.isNumber(element._opacity) ? element._opacity : 1.0;
        new Effect.Opacity(element, {duration:0.2, from:0.7, to:toOpacity,
          queue: {scope:'_draggable', position:'end'},
          afterFinish: function(){
            Draggable._dragging[element] = false
          }
        });
      },
      zindex: 1000,
      revert: false,
      quiet: false,
      scroll: false,
      scrollSensitivity: 20,
      scrollSpeed: 15,
      snap: false,  // false, or xy or [x,y] or function(x,y){ return [x,y] }
      delay: 0
    };

    if(!arguments[1] || Object.isUndefined(arguments[1].endeffect))
      Object.extend(defaults, {
        starteffect: function(element) {
          element._opacity = Element.getOpacity(element);
          Draggable._dragging[element] = true;
          new Effect.Opacity(element, {duration:0.2, from:element._opacity, to:0.7});
        }
      });

    var options = Object.extend(defaults, arguments[1] || { });

    this.element = $(element);

    if(options.handle && Object.isString(options.handle))
      this.handle = this.element.down('.'+options.handle, 0);

    if(!this.handle) this.handle = $(options.handle);
    if(!this.handle) this.handle = this.element;

    if(options.scroll && !options.scroll.scrollTo && !options.scroll.outerHTML) {
      options.scroll = $(options.scroll);
      this._isScrollChild = Element.childOf(this.element, options.scroll);
    }

    Element.makePositioned(this.element); // fix IE

    this.options  = options;
    this.dragging = false;

    this.eventMouseDown = this.initDrag.bindAsEventListener(this);
    Event.observe(this.handle, "mousedown", this.eventMouseDown);

    Draggables.register(this);
  },

  destroy: function() {
    Event.stopObserving(this.handle, "mousedown", this.eventMouseDown);
    Draggables.unregister(this);
  },

  currentDelta: function() {
    return([
      parseInt(Element.getStyle(this.element,'left') || '0'),
      parseInt(Element.getStyle(this.element,'top') || '0')]);
  },

  initDrag: function(event) {
    if(!Object.isUndefined(Draggable._dragging[this.element]) &&
      Draggable._dragging[this.element]) return;
    if(Event.isLeftClick(event)) {
      // abort on form elements, fixes a Firefox issue
      var src = Event.element(event);
      if((tag_name = src.tagName.toUpperCase()) && (
        tag_name=='INPUT' ||
        tag_name=='SELECT' ||
        tag_name=='OPTION' ||
        tag_name=='BUTTON' ||
        tag_name=='TEXTAREA')) return;

      var pointer = [Event.pointerX(event), Event.pointerY(event)];
      var pos     = this.element.cumulativeOffset();
      this.offset = [0,1].map( function(i) { return (pointer[i] - pos[i]) });

      Draggables.activate(this);
      Event.stop(event);
    }
  },

  startDrag: function(event) {
    this.dragging = true;
    if(!this.delta)
      this.delta = this.currentDelta();

    if(this.options.zindex) {
      this.originalZ = parseInt(Element.getStyle(this.element,'z-index') || 0);
      this.element.style.zIndex = this.options.zindex;
    }

    if(this.options.ghosting) {
      this._clone = this.element.cloneNode(true);
      this._originallyAbsolute = (this.element.getStyle('position') == 'absolute');
      if (!this._originallyAbsolute)
        Position.absolutize(this.element);
      this.element.parentNode.insertBefore(this._clone, this.element);
    }

    if(this.options.scroll) {
      if (this.options.scroll == window) {
        var where = this._getWindowScroll(this.options.scroll);
        this.originalScrollLeft = where.left;
        this.originalScrollTop = where.top;
      } else {
        this.originalScrollLeft = this.options.scroll.scrollLeft;
        this.originalScrollTop = this.options.scroll.scrollTop;
      }
    }

    Draggables.notify('onStart', this, event);

    if(this.options.starteffect) this.options.starteffect(this.element);
  },

  updateDrag: function(event, pointer) {
    if(!this.dragging) this.startDrag(event);

    if(!this.options.quiet){
      Position.prepare();
      Droppables.show(pointer, this.element);
    }

    Draggables.notify('onDrag', this, event);

    this.draw(pointer);
    if(this.options.change) this.options.change(this);

    if(this.options.scroll) {
      this.stopScrolling();

      var p;
      if (this.options.scroll == window) {
        with(this._getWindowScroll(this.options.scroll)) { p = [ left, top, left+width, top+height ]; }
      } else {
        p = Position.page(this.options.scroll).toArray();
        p[0] += this.options.scroll.scrollLeft + Position.deltaX;
        p[1] += this.options.scroll.scrollTop + Position.deltaY;
        p.push(p[0]+this.options.scroll.offsetWidth);
        p.push(p[1]+this.options.scroll.offsetHeight);
      }
      var speed = [0,0];
      if(pointer[0] < (p[0]+this.options.scrollSensitivity)) speed[0] = pointer[0]-(p[0]+this.options.scrollSensitivity);
      if(pointer[1] < (p[1]+this.options.scrollSensitivity)) speed[1] = pointer[1]-(p[1]+this.options.scrollSensitivity);
      if(pointer[0] > (p[2]-this.options.scrollSensitivity)) speed[0] = pointer[0]-(p[2]-this.options.scrollSensitivity);
      if(pointer[1] > (p[3]-this.options.scrollSensitivity)) speed[1] = pointer[1]-(p[3]-this.options.scrollSensitivity);
      this.startScrolling(speed);
    }

    // fix AppleWebKit rendering
    if(Prototype.Browser.WebKit) window.scrollBy(0,0);

    Event.stop(event);
  },

  finishDrag: function(event, success) {
    this.dragging = false;

    if(this.options.quiet){
      Position.prepare();
      var pointer = [Event.pointerX(event), Event.pointerY(event)];
      Droppables.show(pointer, this.element);
    }

    if(this.options.ghosting) {
      if (!this._originallyAbsolute)
        Position.relativize(this.element);
      delete this._originallyAbsolute;
      Element.remove(this._clone);
      this._clone = null;
    }

    var dropped = false;
    if(success) {
      dropped = Droppables.fire(event, this.element);
      if (!dropped) dropped = false;
    }
    if(dropped && this.options.onDropped) this.options.onDropped(this.element);
    Draggables.notify('onEnd', this, event);

    var revert = this.options.revert;
    if(revert && Object.isFunction(revert)) revert = revert(this.element);

    var d = this.currentDelta();
    if(revert && this.options.reverteffect) {
      if (dropped == 0 || revert != 'failure')
        this.options.reverteffect(this.element,
          d[1]-this.delta[1], d[0]-this.delta[0]);
    } else {
      this.delta = d;
    }

    if(this.options.zindex)
      this.element.style.zIndex = this.originalZ;

    if(this.options.endeffect)
      this.options.endeffect(this.element);

    Draggables.deactivate(this);
    Droppables.reset();
  },

  keyPress: function(event) {
    if(event.keyCode!=Event.KEY_ESC) return;
    this.finishDrag(event, false);
    Event.stop(event);
  },

  endDrag: function(event) {
    if(!this.dragging) return;
    this.stopScrolling();
    this.finishDrag(event, true);
    Event.stop(event);
  },

  draw: function(point) {
    var pos = this.element.cumulativeOffset();
    if(this.options.ghosting) {
      var r   = Position.realOffset(this.element);
      pos[0] += r[0] - Position.deltaX; pos[1] += r[1] - Position.deltaY;
    }

    var d = this.currentDelta();
    pos[0] -= d[0]; pos[1] -= d[1];

    if(this.options.scroll && (this.options.scroll != window && this._isScrollChild)) {
      pos[0] -= this.options.scroll.scrollLeft-this.originalScrollLeft;
      pos[1] -= this.options.scroll.scrollTop-this.originalScrollTop;
    }

    var p = [0,1].map(function(i){
      return (point[i]-pos[i]-this.offset[i])
    }.bind(this));

    if(this.options.snap) {
      if(Object.isFunction(this.options.snap)) {
        p = this.options.snap(p[0],p[1],this);
      } else {
      if(Object.isArray(this.options.snap)) {
        p = p.map( function(v, i) {
          return (v/this.options.snap[i]).round()*this.options.snap[i] }.bind(this));
      } else {
        p = p.map( function(v) {
          return (v/this.options.snap).round()*this.options.snap }.bind(this));
      }
    }}

    var style = this.element.style;
    if((!this.options.constraint) || (this.options.constraint=='horizontal'))
      style.left = p[0] + "px";
    if((!this.options.constraint) || (this.options.constraint=='vertical'))
      style.top  = p[1] + "px";

    if(style.visibility=="hidden") style.visibility = ""; // fix gecko rendering
  },

  stopScrolling: function() {
    if(this.scrollInterval) {
      clearInterval(this.scrollInterval);
      this.scrollInterval = null;
      Draggables._lastScrollPointer = null;
    }
  },

  startScrolling: function(speed) {
    if(!(speed[0] || speed[1])) return;
    this.scrollSpeed = [speed[0]*this.options.scrollSpeed,speed[1]*this.options.scrollSpeed];
    this.lastScrolled = new Date();
    this.scrollInterval = setInterval(this.scroll.bind(this), 10);
  },

  scroll: function() {
    var current = new Date();
    var delta = current - this.lastScrolled;
    this.lastScrolled = current;
    if(this.options.scroll == window) {
      with (this._getWindowScroll(this.options.scroll)) {
        if (this.scrollSpeed[0] || this.scrollSpeed[1]) {
          var d = delta / 1000;
          this.options.scroll.scrollTo( left + d*this.scrollSpeed[0], top + d*this.scrollSpeed[1] );
        }
      }
    } else {
      this.options.scroll.scrollLeft += this.scrollSpeed[0] * delta / 1000;
      this.options.scroll.scrollTop  += this.scrollSpeed[1] * delta / 1000;
    }

    Position.prepare();
    Droppables.show(Draggables._lastPointer, this.element);
    Draggables.notify('onDrag', this);
    if (this._isScrollChild) {
      Draggables._lastScrollPointer = Draggables._lastScrollPointer || $A(Draggables._lastPointer);
      Draggables._lastScrollPointer[0] += this.scrollSpeed[0] * delta / 1000;
      Draggables._lastScrollPointer[1] += this.scrollSpeed[1] * delta / 1000;
      if (Draggables._lastScrollPointer[0] < 0)
        Draggables._lastScrollPointer[0] = 0;
      if (Draggables._lastScrollPointer[1] < 0)
        Draggables._lastScrollPointer[1] = 0;
      this.draw(Draggables._lastScrollPointer);
    }

    if(this.options.change) this.options.change(this);
  },

  _getWindowScroll: function(w) {
    var T, L, W, H;
    with (w.document) {
      if (w.document.documentElement && documentElement.scrollTop) {
        T = documentElement.scrollTop;
        L = documentElement.scrollLeft;
      } else if (w.document.body) {
        T = body.scrollTop;
        L = body.scrollLeft;
      }
      if (w.innerWidth) {
        W = w.innerWidth;
        H = w.innerHeight;
      } else if (w.document.documentElement && documentElement.clientWidth) {
        W = documentElement.clientWidth;
        H = documentElement.clientHeight;
      } else {
        W = body.offsetWidth;
        H = body.offsetHeight;
      }
    }
    return { top: T, left: L, width: W, height: H };
  }
});

Draggable._dragging = { };

/*--------------------------------------------------------------------------*/

var SortableObserver = Class.create({
  initialize: function(element, observer) {
    this.element   = $(element);
    this.observer  = observer;
    this.lastValue = Sortable.serialize(this.element);
  },

  onStart: function() {
    this.lastValue = Sortable.serialize(this.element);
  },

  onEnd: function() {
    Sortable.unmark();
    if(this.lastValue != Sortable.serialize(this.element))
      this.observer(this.element)
  }
});

var Sortable = {
  SERIALIZE_RULE: /^[^_\-](?:[A-Za-z0-9\-\_]*)[_](.*)$/,

  sortables: { },

  _findRootElement: function(element) {
    while (element.tagName.toUpperCase() != "BODY") {
      if(element.id && Sortable.sortables[element.id]) return element;
      element = element.parentNode;
    }
  },

  options: function(element) {
    element = Sortable._findRootElement($(element));
    if(!element) return;
    return Sortable.sortables[element.id];
  },

  destroy: function(element){
    element = $(element);
    var s = Sortable.sortables[element.id];

    if(s) {
      Draggables.removeObserver(s.element);
      s.droppables.each(function(d){ Droppables.remove(d) });
      s.draggables.invoke('destroy');

      delete Sortable.sortables[s.element.id];
    }
  },

  create: function(element) {
    element = $(element);
    var options = Object.extend({
      element:     element,
      tag:         'li',       // assumes li children, override with tag: 'tagname'
      dropOnEmpty: false,
      tree:        false,
      treeTag:     'ul',
      overlap:     'vertical', // one of 'vertical', 'horizontal'
      constraint:  'vertical', // one of 'vertical', 'horizontal', false
      containment: element,    // also takes array of elements (or id's); or false
      handle:      false,      // or a CSS class
      only:        false,
      delay:       0,
      hoverclass:  null,
      ghosting:    false,
      quiet:       false,
      scroll:      false,
      scrollSensitivity: 20,
      scrollSpeed: 15,
      format:      this.SERIALIZE_RULE,

      // these take arrays of elements or ids and can be
      // used for better initialization performance
      elements:    false,
      handles:     false,

      onChange:    Prototype.emptyFunction,
      onUpdate:    Prototype.emptyFunction
    }, arguments[1] || { });

    // clear any old sortable with same element
    this.destroy(element);

    // build options for the draggables
    var options_for_draggable = {
      revert:      true,
      quiet:       options.quiet,
      scroll:      options.scroll,
      scrollSpeed: options.scrollSpeed,
      scrollSensitivity: options.scrollSensitivity,
      delay:       options.delay,
      ghosting:    options.ghosting,
      constraint:  options.constraint,
      handle:      options.handle };

    if(options.starteffect)
      options_for_draggable.starteffect = options.starteffect;

    if(options.reverteffect)
      options_for_draggable.reverteffect = options.reverteffect;
    else
      if(options.ghosting) options_for_draggable.reverteffect = function(element) {
        element.style.top  = 0;
        element.style.left = 0;
      };

    if(options.endeffect)
      options_for_draggable.endeffect = options.endeffect;

    if(options.zindex)
      options_for_draggable.zindex = options.zindex;

    // build options for the droppables
    var options_for_droppable = {
      overlap:     options.overlap,
      containment: options.containment,
      tree:        options.tree,
      hoverclass:  options.hoverclass,
      onHover:     Sortable.onHover
    };

    var options_for_tree = {
      onHover:      Sortable.onEmptyHover,
      overlap:      options.overlap,
      containment:  options.containment,
      hoverclass:   options.hoverclass
    };

    // fix for gecko engine
    Element.cleanWhitespace(element);

    options.draggables = [];
    options.droppables = [];

    // drop on empty handling
    if(options.dropOnEmpty || options.tree) {
      Droppables.add(element, options_for_tree);
      options.droppables.push(element);
    }

    (options.elements || this.findElements(element, options) || []).each( function(e,i) {
      var handle = options.handles ? $(options.handles[i]) :
        (options.handle ? $(e).select('.' + options.handle)[0] : e);
      options.draggables.push(
        new Draggable(e, Object.extend(options_for_draggable, { handle: handle })));
      Droppables.add(e, options_for_droppable);
      if(options.tree) e.treeNode = element;
      options.droppables.push(e);
    });

    if(options.tree) {
      (Sortable.findTreeElements(element, options) || []).each( function(e) {
        Droppables.add(e, options_for_tree);
        e.treeNode = element;
        options.droppables.push(e);
      });
    }

    // keep reference
    this.sortables[element.identify()] = options;

    // for onupdate
    Draggables.addObserver(new SortableObserver(element, options.onUpdate));

  },

  // return all suitable-for-sortable elements in a guaranteed order
  findElements: function(element, options) {
    return Element.findChildren(
      element, options.only, options.tree ? true : false, options.tag);
  },

  findTreeElements: function(element, options) {
    return Element.findChildren(
      element, options.only, options.tree ? true : false, options.treeTag);
  },

  onHover: function(element, dropon, overlap) {
    if(Element.isParent(dropon, element)) return;

    if(overlap > .33 && overlap < .66 && Sortable.options(dropon).tree) {
      return;
    } else if(overlap>0.5) {
      Sortable.mark(dropon, 'before');
      if(dropon.previousSibling != element) {
        var oldParentNode = element.parentNode;
        element.style.visibility = "hidden"; // fix gecko rendering
        dropon.parentNode.insertBefore(element, dropon);
        if(dropon.parentNode!=oldParentNode)
          Sortable.options(oldParentNode).onChange(element);
        Sortable.options(dropon.parentNode).onChange(element);
      }
    } else {
      Sortable.mark(dropon, 'after');
      var nextElement = dropon.nextSibling || null;
      if(nextElement != element) {
        var oldParentNode = element.parentNode;
        element.style.visibility = "hidden"; // fix gecko rendering
        dropon.parentNode.insertBefore(element, nextElement);
        if(dropon.parentNode!=oldParentNode)
          Sortable.options(oldParentNode).onChange(element);
        Sortable.options(dropon.parentNode).onChange(element);
      }
    }
  },

  onEmptyHover: function(element, dropon, overlap) {
    var oldParentNode = element.parentNode;
    var droponOptions = Sortable.options(dropon);

    if(!Element.isParent(dropon, element)) {
      var index;

      var children = Sortable.findElements(dropon, {tag: droponOptions.tag, only: droponOptions.only});
      var child = null;

      if(children) {
        var offset = Element.offsetSize(dropon, droponOptions.overlap) * (1.0 - overlap);

        for (index = 0; index < children.length; index += 1) {
          if (offset - Element.offsetSize (children[index], droponOptions.overlap) >= 0) {
            offset -= Element.offsetSize (children[index], droponOptions.overlap);
          } else if (offset - (Element.offsetSize (children[index], droponOptions.overlap) / 2) >= 0) {
            child = index + 1 < children.length ? children[index + 1] : null;
            break;
          } else {
            child = children[index];
            break;
          }
        }
      }

      dropon.insertBefore(element, child);

      Sortable.options(oldParentNode).onChange(element);
      droponOptions.onChange(element);
    }
  },

  unmark: function() {
    if(Sortable._marker) Sortable._marker.hide();
  },

  mark: function(dropon, position) {
    // mark on ghosting only
    var sortable = Sortable.options(dropon.parentNode);
    if(sortable && !sortable.ghosting) return;

    if(!Sortable._marker) {
      Sortable._marker =
        ($('dropmarker') || Element.extend(document.createElement('DIV'))).
          hide().addClassName('dropmarker').setStyle({position:'absolute'});
      document.getElementsByTagName("body").item(0).appendChild(Sortable._marker);
    }
    var offsets = dropon.cumulativeOffset();
    Sortable._marker.setStyle({left: offsets[0]+'px', top: offsets[1] + 'px'});

    if(position=='after')
      if(sortable.overlap == 'horizontal')
        Sortable._marker.setStyle({left: (offsets[0]+dropon.clientWidth) + 'px'});
      else
        Sortable._marker.setStyle({top: (offsets[1]+dropon.clientHeight) + 'px'});

    Sortable._marker.show();
  },

  _tree: function(element, options, parent) {
    var children = Sortable.findElements(element, options) || [];

    for (var i = 0; i < children.length; ++i) {
      var match = children[i].id.match(options.format);

      if (!match) continue;

      var child = {
        id: encodeURIComponent(match ? match[1] : null),
        element: element,
        parent: parent,
        children: [],
        position: parent.children.length,
        container: $(children[i]).down(options.treeTag)
      };

      /* Get the element containing the children and recurse over it */
      if (child.container)
        this._tree(child.container, options, child);

      parent.children.push (child);
    }

    return parent;
  },

  tree: function(element) {
    element = $(element);
    var sortableOptions = this.options(element);
    var options = Object.extend({
      tag: sortableOptions.tag,
      treeTag: sortableOptions.treeTag,
      only: sortableOptions.only,
      name: element.id,
      format: sortableOptions.format
    }, arguments[1] || { });

    var root = {
      id: null,
      parent: null,
      children: [],
      container: element,
      position: 0
    };

    return Sortable._tree(element, options, root);
  },

  /* Construct a [i] index for a particular node */
  _constructIndex: function(node) {
    var index = '';
    do {
      if (node.id) index = '[' + node.position + ']' + index;
    } while ((node = node.parent) != null);
    return index;
  },

  sequence: function(element) {
    element = $(element);
    var options = Object.extend(this.options(element), arguments[1] || { });

    return $(this.findElements(element, options) || []).map( function(item) {
      return item.id.match(options.format) ? item.id.match(options.format)[1] : '';
    });
  },

  setSequence: function(element, new_sequence) {
    element = $(element);
    var options = Object.extend(this.options(element), arguments[2] || { });

    var nodeMap = { };
    this.findElements(element, options).each( function(n) {
        if (n.id.match(options.format))
            nodeMap[n.id.match(options.format)[1]] = [n, n.parentNode];
        n.parentNode.removeChild(n);
    });

    new_sequence.each(function(ident) {
      var n = nodeMap[ident];
      if (n) {
        n[1].appendChild(n[0]);
        delete nodeMap[ident];
      }
    });
  },

  serialize: function(element) {
    element = $(element);
    var options = Object.extend(Sortable.options(element), arguments[1] || { });
    var name = encodeURIComponent(
      (arguments[1] && arguments[1].name) ? arguments[1].name : element.id);

    if (options.tree) {
      return Sortable.tree(element, arguments[1]).children.map( function (item) {
        return [name + Sortable._constructIndex(item) + "[id]=" +
                encodeURIComponent(item.id)].concat(item.children.map(arguments.callee));
      }).flatten().join('&');
    } else {
      return Sortable.sequence(element, arguments[1]).map( function(item) {
        return name + "[]=" + encodeURIComponent(item);
      }).join('&');
    }
  }
};

// Returns true if child is contained within element
Element.isParent = function(child, element) {
  if (!child.parentNode || child == element) return false;
  if (child.parentNode == element) return true;
  return Element.isParent(child.parentNode, element);
};

Element.findChildren = function(element, only, recursive, tagName) {
  if(!element.hasChildNodes()) return null;
  tagName = tagName.toUpperCase();
  if(only) only = [only].flatten();
  var elements = [];
  $A(element.childNodes).each( function(e) {
    if(e.tagName && e.tagName.toUpperCase()==tagName &&
      (!only || (Element.classNames(e).detect(function(v) { return only.include(v) }))))
        elements.push(e);
    if(recursive) {
      var grandchildren = Element.findChildren(e, only, recursive, tagName);
      if(grandchildren) elements.push(grandchildren);
    }
  });

  return (elements.length>0 ? elements.flatten() : []);
};

Element.offsetSize = function (element, type) {
  return element['offset' + ((type=='vertical' || type=='height') ? 'Height' : 'Width')];
};
// script.aculo.us controls.js v1.9.0, Thu Dec 23 16:54:48 -0500 2010

// Copyright (c) 2005-2010 Thomas Fuchs (http://script.aculo.us, http://mir.aculo.us)
//           (c) 2005-2010 Ivan Krstic (http://blogs.law.harvard.edu/ivan)
//           (c) 2005-2010 Jon Tirsen (http://www.tirsen.com)
// Contributors:
//  Richard Livsey
//  Rahul Bhargava
//  Rob Wills
//
// script.aculo.us is freely distributable under the terms of an MIT-style license.
// For details, see the script.aculo.us web site: http://script.aculo.us/

// Autocompleter.Base handles all the autocompletion functionality
// that's independent of the data source for autocompletion. This
// includes drawing the autocompletion menu, observing keyboard
// and mouse events, and similar.
//
// Specific autocompleters need to provide, at the very least,
// a getUpdatedChoices function that will be invoked every time
// the text inside the monitored textbox changes. This method
// should get the text for which to provide autocompletion by
// invoking this.getToken(), NOT by directly accessing
// this.element.value. This is to allow incremental tokenized
// autocompletion. Specific auto-completion logic (AJAX, etc)
// belongs in getUpdatedChoices.
//
// Tokenized incremental autocompletion is enabled automatically
// when an autocompleter is instantiated with the 'tokens' option
// in the options parameter, e.g.:
// new Ajax.Autocompleter('id','upd', '/url/', { tokens: ',' });
// will incrementally autocomplete with a comma as the token.
// Additionally, ',' in the above example can be replaced with
// a token array, e.g. { tokens: [',', '\n'] } which
// enables autocompletion on multiple tokens. This is most
// useful when one of the tokens is \n (a newline), as it
// allows smart autocompletion after linebreaks.

if(typeof Effect == 'undefined')
  throw("controls.js requires including script.aculo.us' effects.js library");

var Autocompleter = { };
Autocompleter.Base = Class.create({
  baseInitialize: function(element, update, options) {
    element          = $(element);
    this.element     = element;
    this.update      = $(update);
    this.hasFocus    = false;
    this.changed     = false;
    this.active      = false;
    this.index       = 0;
    this.entryCount  = 0;
    this.oldElementValue = this.element.value;

    if(this.setOptions)
      this.setOptions(options);
    else
      this.options = options || { };

    this.options.paramName    = this.options.paramName || this.element.name;
    this.options.tokens       = this.options.tokens || [];
    this.options.frequency    = this.options.frequency || 0.4;
    this.options.minChars     = this.options.minChars || 1;
    this.options.onShow       = this.options.onShow ||
      function(element, update){
        if(!update.style.position || update.style.position=='absolute') {
          update.style.position = 'absolute';
          Position.clone(element, update, {
            setHeight: false,
            offsetTop: element.offsetHeight
          });
        }
        Effect.Appear(update,{duration:0.15});
      };
    this.options.onHide = this.options.onHide ||
      function(element, update){ new Effect.Fade(update,{duration:0.15}) };

    if(typeof(this.options.tokens) == 'string')
      this.options.tokens = new Array(this.options.tokens);
    // Force carriage returns as token delimiters anyway
    if (!this.options.tokens.include('\n'))
      this.options.tokens.push('\n');

    this.observer = null;

    this.element.setAttribute('autocomplete','off');

    Element.hide(this.update);

    Event.observe(this.element, 'blur', this.onBlur.bindAsEventListener(this));
    Event.observe(this.element, 'keydown', this.onKeyPress.bindAsEventListener(this));
  },

  show: function() {
    if(Element.getStyle(this.update, 'display')=='none') this.options.onShow(this.element, this.update);
    if(!this.iefix &&
      (Prototype.Browser.IE) &&
      (Element.getStyle(this.update, 'position')=='absolute')) {
      new Insertion.After(this.update,
       '<iframe id="' + this.update.id + '_iefix" '+
       'style="display:none;position:absolute;filter:progid:DXImageTransform.Microsoft.Alpha(opacity=0);" ' +
       'src="javascript:false;" frameborder="0" scrolling="no"></iframe>');
      this.iefix = $(this.update.id+'_iefix');
    }
    if(this.iefix) setTimeout(this.fixIEOverlapping.bind(this), 50);
  },

  fixIEOverlapping: function() {
    Position.clone(this.update, this.iefix, {setTop:(!this.update.style.height)});
    this.iefix.style.zIndex = 1;
    this.update.style.zIndex = 2;
    Element.show(this.iefix);
  },

  hide: function() {
    this.stopIndicator();
    if(Element.getStyle(this.update, 'display')!='none') this.options.onHide(this.element, this.update);
    if(this.iefix) Element.hide(this.iefix);
  },

  startIndicator: function() {
    if(this.options.indicator) Element.show(this.options.indicator);
  },

  stopIndicator: function() {
    if(this.options.indicator) Element.hide(this.options.indicator);
  },

  onKeyPress: function(event) {
    if(this.active)
      switch(event.keyCode) {
       case Event.KEY_TAB:
       case Event.KEY_RETURN:
         this.selectEntry();
         Event.stop(event);
       case Event.KEY_ESC:
         this.hide();
         this.active = false;
         Event.stop(event);
         return;
       case Event.KEY_LEFT:
       case Event.KEY_RIGHT:
         return;
       case Event.KEY_UP:
         this.markPrevious();
         this.render();
         Event.stop(event);
         return;
       case Event.KEY_DOWN:
         this.markNext();
         this.render();
         Event.stop(event);
         return;
      }
     else
       if(event.keyCode==Event.KEY_TAB || event.keyCode==Event.KEY_RETURN ||
         (Prototype.Browser.WebKit > 0 && event.keyCode == 0)) return;

    this.changed = true;
    this.hasFocus = true;

    if(this.observer) clearTimeout(this.observer);
      this.observer =
        setTimeout(this.onObserverEvent.bind(this), this.options.frequency*1000);
  },

  activate: function() {
    this.changed = false;
    this.hasFocus = true;
    this.getUpdatedChoices();
  },

  onHover: function(event) {
    var element = Event.findElement(event, 'LI');
    if(this.index != element.autocompleteIndex)
    {
        this.index = element.autocompleteIndex;
        this.render();
    }
    Event.stop(event);
  },

  onClick: function(event) {
    var element = Event.findElement(event, 'LI');
    this.index = element.autocompleteIndex;
    this.selectEntry();
    this.hide();
  },

  onBlur: function(event) {
    // needed to make click events working
    setTimeout(this.hide.bind(this), 250);
    this.hasFocus = false;
    this.active = false;
  },

  render: function() {
    if(this.entryCount > 0) {
      for (var i = 0; i < this.entryCount; i++)
        this.index==i ?
          Element.addClassName(this.getEntry(i),"selected") :
          Element.removeClassName(this.getEntry(i),"selected");
      if(this.hasFocus) {
        this.show();
        this.active = true;
      }
    } else {
      this.active = false;
      this.hide();
    }
  },

  markPrevious: function() {
    if(this.index > 0) this.index--;
      else this.index = this.entryCount-1;
    this.getEntry(this.index).scrollIntoView(true);
  },

  markNext: function() {
    if(this.index < this.entryCount-1) this.index++;
      else this.index = 0;
    this.getEntry(this.index).scrollIntoView(false);
  },

  getEntry: function(index) {
    return this.update.firstChild.childNodes[index];
  },

  getCurrentEntry: function() {
    return this.getEntry(this.index);
  },

  selectEntry: function() {
    this.active = false;
    this.updateElement(this.getCurrentEntry());
  },

  updateElement: function(selectedElement) {
    if (this.options.updateElement) {
      this.options.updateElement(selectedElement);
      return;
    }
    var value = '';
    if (this.options.select) {
      var nodes = $(selectedElement).select('.' + this.options.select) || [];
      if(nodes.length>0) value = Element.collectTextNodes(nodes[0], this.options.select);
    } else
      value = Element.collectTextNodesIgnoreClass(selectedElement, 'informal');

    var bounds = this.getTokenBounds();
    if (bounds[0] != -1) {
      var newValue = this.element.value.substr(0, bounds[0]);
      var whitespace = this.element.value.substr(bounds[0]).match(/^\s+/);
      if (whitespace)
        newValue += whitespace[0];
      this.element.value = newValue + value + this.element.value.substr(bounds[1]);
    } else {
      this.element.value = value;
    }
    this.oldElementValue = this.element.value;
    this.element.focus();

    if (this.options.afterUpdateElement)
      this.options.afterUpdateElement(this.element, selectedElement);
  },

  updateChoices: function(choices) {
    if(!this.changed && this.hasFocus) {
      this.update.innerHTML = choices;
      Element.cleanWhitespace(this.update);
      Element.cleanWhitespace(this.update.down());

      if(this.update.firstChild && this.update.down().childNodes) {
        this.entryCount =
          this.update.down().childNodes.length;
        for (var i = 0; i < this.entryCount; i++) {
          var entry = this.getEntry(i);
          entry.autocompleteIndex = i;
          this.addObservers(entry);
        }
      } else {
        this.entryCount = 0;
      }

      this.stopIndicator();
      this.index = 0;

      if(this.entryCount==1 && this.options.autoSelect) {
        this.selectEntry();
        this.hide();
      } else {
        this.render();
      }
    }
  },

  addObservers: function(element) {
    Event.observe(element, "mouseover", this.onHover.bindAsEventListener(this));
    Event.observe(element, "click", this.onClick.bindAsEventListener(this));
  },

  onObserverEvent: function() {
    this.changed = false;
    this.tokenBounds = null;
    if(this.getToken().length>=this.options.minChars) {
      this.getUpdatedChoices();
    } else {
      this.active = false;
      this.hide();
    }
    this.oldElementValue = this.element.value;
  },

  getToken: function() {
    var bounds = this.getTokenBounds();
    return this.element.value.substring(bounds[0], bounds[1]).strip();
  },

  getTokenBounds: function() {
    if (null != this.tokenBounds) return this.tokenBounds;
    var value = this.element.value;
    if (value.strip().empty()) return [-1, 0];
    var diff = arguments.callee.getFirstDifferencePos(value, this.oldElementValue);
    var offset = (diff == this.oldElementValue.length ? 1 : 0);
    var prevTokenPos = -1, nextTokenPos = value.length;
    var tp;
    for (var index = 0, l = this.options.tokens.length; index < l; ++index) {
      tp = value.lastIndexOf(this.options.tokens[index], diff + offset - 1);
      if (tp > prevTokenPos) prevTokenPos = tp;
      tp = value.indexOf(this.options.tokens[index], diff + offset);
      if (-1 != tp && tp < nextTokenPos) nextTokenPos = tp;
    }
    return (this.tokenBounds = [prevTokenPos + 1, nextTokenPos]);
  }
});

Autocompleter.Base.prototype.getTokenBounds.getFirstDifferencePos = function(newS, oldS) {
  var boundary = Math.min(newS.length, oldS.length);
  for (var index = 0; index < boundary; ++index)
    if (newS[index] != oldS[index])
      return index;
  return boundary;
};

Ajax.Autocompleter = Class.create(Autocompleter.Base, {
  initialize: function(element, update, url, options) {
    this.baseInitialize(element, update, options);
    this.options.asynchronous  = true;
    this.options.onComplete    = this.onComplete.bind(this);
    this.options.defaultParams = this.options.parameters || null;
    this.url                   = url;
  },

  getUpdatedChoices: function() {
    this.startIndicator();

    var entry = encodeURIComponent(this.options.paramName) + '=' +
      encodeURIComponent(this.getToken());

    this.options.parameters = this.options.callback ?
      this.options.callback(this.element, entry) : entry;

    if(this.options.defaultParams)
      this.options.parameters += '&' + this.options.defaultParams;

    new Ajax.Request(this.url, this.options);
  },

  onComplete: function(request) {
    this.updateChoices(request.responseText);
  }
});

// The local array autocompleter. Used when you'd prefer to
// inject an array of autocompletion options into the page, rather
// than sending out Ajax queries, which can be quite slow sometimes.
//
// The constructor takes four parameters. The first two are, as usual,
// the id of the monitored textbox, and id of the autocompletion menu.
// The third is the array you want to autocomplete from, and the fourth
// is the options block.
//
// Extra local autocompletion options:
// - choices - How many autocompletion choices to offer
//
// - partialSearch - If false, the autocompleter will match entered
//                    text only at the beginning of strings in the
//                    autocomplete array. Defaults to true, which will
//                    match text at the beginning of any *word* in the
//                    strings in the autocomplete array. If you want to
//                    search anywhere in the string, additionally set
//                    the option fullSearch to true (default: off).
//
// - fullSsearch - Search anywhere in autocomplete array strings.
//
// - partialChars - How many characters to enter before triggering
//                   a partial match (unlike minChars, which defines
//                   how many characters are required to do any match
//                   at all). Defaults to 2.
//
// - ignoreCase - Whether to ignore case when autocompleting.
//                 Defaults to true.
//
// It's possible to pass in a custom function as the 'selector'
// option, if you prefer to write your own autocompletion logic.
// In that case, the other options above will not apply unless
// you support them.

Autocompleter.Local = Class.create(Autocompleter.Base, {
  initialize: function(element, update, array, options) {
    this.baseInitialize(element, update, options);
    this.options.array = array;
  },

  getUpdatedChoices: function() {
    this.updateChoices(this.options.selector(this));
  },

  setOptions: function(options) {
    this.options = Object.extend({
      choices: 10,
      partialSearch: true,
      partialChars: 2,
      ignoreCase: true,
      fullSearch: false,
      selector: function(instance) {
        var ret       = []; // Beginning matches
        var partial   = []; // Inside matches
        var entry     = instance.getToken();
        var count     = 0;

        for (var i = 0; i < instance.options.array.length &&
          ret.length < instance.options.choices ; i++) {

          var elem = instance.options.array[i];
          var foundPos = instance.options.ignoreCase ?
            elem.toLowerCase().indexOf(entry.toLowerCase()) :
            elem.indexOf(entry);

          while (foundPos != -1) {
            if (foundPos == 0 && elem.length != entry.length) {
              ret.push("<li><strong>" + elem.substr(0, entry.length) + "</strong>" +
                elem.substr(entry.length) + "</li>");
              break;
            } else if (entry.length >= instance.options.partialChars &&
              instance.options.partialSearch && foundPos != -1) {
              if (instance.options.fullSearch || /\s/.test(elem.substr(foundPos-1,1))) {
                partial.push("<li>" + elem.substr(0, foundPos) + "<strong>" +
                  elem.substr(foundPos, entry.length) + "</strong>" + elem.substr(
                  foundPos + entry.length) + "</li>");
                break;
              }
            }

            foundPos = instance.options.ignoreCase ?
              elem.toLowerCase().indexOf(entry.toLowerCase(), foundPos + 1) :
              elem.indexOf(entry, foundPos + 1);

          }
        }
        if (partial.length)
          ret = ret.concat(partial.slice(0, instance.options.choices - ret.length));
        return "<ul>" + ret.join('') + "</ul>";
      }
    }, options || { });
  }
});

// AJAX in-place editor and collection editor
// Full rewrite by Christophe Porteneuve <tdd@tddsworld.com> (April 2007).

// Use this if you notice weird scrolling problems on some browsers,
// the DOM might be a bit confused when this gets called so do this
// waits 1 ms (with setTimeout) until it does the activation
Field.scrollFreeActivate = function(field) {
  setTimeout(function() {
    Field.activate(field);
  }, 1);
};

Ajax.InPlaceEditor = Class.create({
  initialize: function(element, url, options) {
    this.url = url;
    this.element = element = $(element);
    this.prepareOptions();
    this._controls = { };
    arguments.callee.dealWithDeprecatedOptions(options); // DEPRECATION LAYER!!!
    Object.extend(this.options, options || { });
    if (!this.options.formId && this.element.id) {
      this.options.formId = this.element.id + '-inplaceeditor';
      if ($(this.options.formId))
        this.options.formId = '';
    }
    if (this.options.externalControl)
      this.options.externalControl = $(this.options.externalControl);
    if (!this.options.externalControl)
      this.options.externalControlOnly = false;
    this._originalBackground = this.element.getStyle('background-color') || 'transparent';
    this.element.title = this.options.clickToEditText;
    this._boundCancelHandler = this.handleFormCancellation.bind(this);
    this._boundComplete = (this.options.onComplete || Prototype.emptyFunction).bind(this);
    this._boundFailureHandler = this.handleAJAXFailure.bind(this);
    this._boundSubmitHandler = this.handleFormSubmission.bind(this);
    this._boundWrapperHandler = this.wrapUp.bind(this);
    this.registerListeners();
  },
  checkForEscapeOrReturn: function(e) {
    if (!this._editing || e.ctrlKey || e.altKey || e.shiftKey) return;
    if (Event.KEY_ESC == e.keyCode)
      this.handleFormCancellation(e);
    else if (Event.KEY_RETURN == e.keyCode)
      this.handleFormSubmission(e);
  },
  createControl: function(mode, handler, extraClasses) {
    var control = this.options[mode + 'Control'];
    var text = this.options[mode + 'Text'];
    if ('button' == control) {
      var btn = document.createElement('input');
      btn.type = 'submit';
      btn.value = text;
      btn.className = 'editor_' + mode + '_button';
      if ('cancel' == mode)
        btn.onclick = this._boundCancelHandler;
      this._form.appendChild(btn);
      this._controls[mode] = btn;
    } else if ('link' == control) {
      var link = document.createElement('a');
      link.href = '#';
      link.appendChild(document.createTextNode(text));
      link.onclick = 'cancel' == mode ? this._boundCancelHandler : this._boundSubmitHandler;
      link.className = 'editor_' + mode + '_link';
      if (extraClasses)
        link.className += ' ' + extraClasses;
      this._form.appendChild(link);
      this._controls[mode] = link;
    }
  },
  createEditField: function() {
    var text = (this.options.loadTextURL ? this.options.loadingText : this.getText());
    var fld;
    if (1 >= this.options.rows && !/\r|\n/.test(this.getText())) {
      fld = document.createElement('input');
      fld.type = 'text';
      var size = this.options.size || this.options.cols || 0;
      if (0 < size) fld.size = size;
    } else {
      fld = document.createElement('textarea');
      fld.rows = (1 >= this.options.rows ? this.options.autoRows : this.options.rows);
      fld.cols = this.options.cols || 40;
    }
    fld.name = this.options.paramName;
    fld.value = text; // No HTML breaks conversion anymore
    fld.className = 'editor_field';
    if (this.options.submitOnBlur)
      fld.onblur = this._boundSubmitHandler;
    this._controls.editor = fld;
    if (this.options.loadTextURL)
      this.loadExternalText();
    this._form.appendChild(this._controls.editor);
  },
  createForm: function() {
    var ipe = this;
    function addText(mode, condition) {
      var text = ipe.options['text' + mode + 'Controls'];
      if (!text || condition === false) return;
      ipe._form.appendChild(document.createTextNode(text));
    };
    this._form = $(document.createElement('form'));
    this._form.id = this.options.formId;
    this._form.addClassName(this.options.formClassName);
    this._form.onsubmit = this._boundSubmitHandler;
    this.createEditField();
    if ('textarea' == this._controls.editor.tagName.toLowerCase())
      this._form.appendChild(document.createElement('br'));
    if (this.options.onFormCustomization)
      this.options.onFormCustomization(this, this._form);
    addText('Before', this.options.okControl || this.options.cancelControl);
    this.createControl('ok', this._boundSubmitHandler);
    addText('Between', this.options.okControl && this.options.cancelControl);
    this.createControl('cancel', this._boundCancelHandler, 'editor_cancel');
    addText('After', this.options.okControl || this.options.cancelControl);
  },
  destroy: function() {
    if (this._oldInnerHTML)
      this.element.innerHTML = this._oldInnerHTML;
    this.leaveEditMode();
    this.unregisterListeners();
  },
  enterEditMode: function(e) {
    if (this._saving || this._editing) return;
    this._editing = true;
    this.triggerCallback('onEnterEditMode');
    if (this.options.externalControl)
      this.options.externalControl.hide();
    this.element.hide();
    this.createForm();
    this.element.parentNode.insertBefore(this._form, this.element);
    if (!this.options.loadTextURL)
      this.postProcessEditField();
    if (e) Event.stop(e);
  },
  enterHover: function(e) {
    if (this.options.hoverClassName)
      this.element.addClassName(this.options.hoverClassName);
    if (this._saving) return;
    this.triggerCallback('onEnterHover');
  },
  getText: function() {
    return this.element.innerHTML.unescapeHTML();
  },
  handleAJAXFailure: function(transport) {
    this.triggerCallback('onFailure', transport);
    if (this._oldInnerHTML) {
      this.element.innerHTML = this._oldInnerHTML;
      this._oldInnerHTML = null;
    }
  },
  handleFormCancellation: function(e) {
    this.wrapUp();
    if (e) Event.stop(e);
  },
  handleFormSubmission: function(e) {
    var form = this._form;
    var value = $F(this._controls.editor);
    this.prepareSubmission();
    var params = this.options.callback(form, value) || '';
    if (Object.isString(params))
      params = params.toQueryParams();
    params.editorId = this.element.id;
    if (this.options.htmlResponse) {
      var options = Object.extend({ evalScripts: true }, this.options.ajaxOptions);
      Object.extend(options, {
        parameters: params,
        onComplete: this._boundWrapperHandler,
        onFailure: this._boundFailureHandler
      });
      new Ajax.Updater({ success: this.element }, this.url, options);
    } else {
      var options = Object.extend({ method: 'get' }, this.options.ajaxOptions);
      Object.extend(options, {
        parameters: params,
        onComplete: this._boundWrapperHandler,
        onFailure: this._boundFailureHandler
      });
      new Ajax.Request(this.url, options);
    }
    if (e) Event.stop(e);
  },
  leaveEditMode: function() {
    this.element.removeClassName(this.options.savingClassName);
    this.removeForm();
    this.leaveHover();
    this.element.style.backgroundColor = this._originalBackground;
    this.element.show();
    if (this.options.externalControl)
      this.options.externalControl.show();
    this._saving = false;
    this._editing = false;
    this._oldInnerHTML = null;
    this.triggerCallback('onLeaveEditMode');
  },
  leaveHover: function(e) {
    if (this.options.hoverClassName)
      this.element.removeClassName(this.options.hoverClassName);
    if (this._saving) return;
    this.triggerCallback('onLeaveHover');
  },
  loadExternalText: function() {
    this._form.addClassName(this.options.loadingClassName);
    this._controls.editor.disabled = true;
    var options = Object.extend({ method: 'get' }, this.options.ajaxOptions);
    Object.extend(options, {
      parameters: 'editorId=' + encodeURIComponent(this.element.id),
      onComplete: Prototype.emptyFunction,
      onSuccess: function(transport) {
        this._form.removeClassName(this.options.loadingClassName);
        var text = transport.responseText;
        if (this.options.stripLoadedTextTags)
          text = text.stripTags();
        this._controls.editor.value = text;
        this._controls.editor.disabled = false;
        this.postProcessEditField();
      }.bind(this),
      onFailure: this._boundFailureHandler
    });
    new Ajax.Request(this.options.loadTextURL, options);
  },
  postProcessEditField: function() {
    var fpc = this.options.fieldPostCreation;
    if (fpc)
      $(this._controls.editor)['focus' == fpc ? 'focus' : 'activate']();
  },
  prepareOptions: function() {
    this.options = Object.clone(Ajax.InPlaceEditor.DefaultOptions);
    Object.extend(this.options, Ajax.InPlaceEditor.DefaultCallbacks);
    [this._extraDefaultOptions].flatten().compact().each(function(defs) {
      Object.extend(this.options, defs);
    }.bind(this));
  },
  prepareSubmission: function() {
    this._saving = true;
    this.removeForm();
    this.leaveHover();
    this.showSaving();
  },
  registerListeners: function() {
    this._listeners = { };
    var listener;
    $H(Ajax.InPlaceEditor.Listeners).each(function(pair) {
      listener = this[pair.value].bind(this);
      this._listeners[pair.key] = listener;
      if (!this.options.externalControlOnly)
        this.element.observe(pair.key, listener);
      if (this.options.externalControl)
        this.options.externalControl.observe(pair.key, listener);
    }.bind(this));
  },
  removeForm: function() {
    if (!this._form) return;
    this._form.remove();
    this._form = null;
    this._controls = { };
  },
  showSaving: function() {
    this._oldInnerHTML = this.element.innerHTML;
    this.element.innerHTML = this.options.savingText;
    this.element.addClassName(this.options.savingClassName);
    this.element.style.backgroundColor = this._originalBackground;
    this.element.show();
  },
  triggerCallback: function(cbName, arg) {
    if ('function' == typeof this.options[cbName]) {
      this.options[cbName](this, arg);
    }
  },
  unregisterListeners: function() {
    $H(this._listeners).each(function(pair) {
      if (!this.options.externalControlOnly)
        this.element.stopObserving(pair.key, pair.value);
      if (this.options.externalControl)
        this.options.externalControl.stopObserving(pair.key, pair.value);
    }.bind(this));
  },
  wrapUp: function(transport) {
    this.leaveEditMode();
    // Can't use triggerCallback due to backward compatibility: requires
    // binding + direct element
    this._boundComplete(transport, this.element);
  }
});

Object.extend(Ajax.InPlaceEditor.prototype, {
  dispose: Ajax.InPlaceEditor.prototype.destroy
});

Ajax.InPlaceCollectionEditor = Class.create(Ajax.InPlaceEditor, {
  initialize: function($super, element, url, options) {
    this._extraDefaultOptions = Ajax.InPlaceCollectionEditor.DefaultOptions;
    $super(element, url, options);
  },

  createEditField: function() {
    var list = document.createElement('select');
    list.name = this.options.paramName;
    list.size = 1;
    this._controls.editor = list;
    this._collection = this.options.collection || [];
    if (this.options.loadCollectionURL)
      this.loadCollection();
    else
      this.checkForExternalText();
    this._form.appendChild(this._controls.editor);
  },

  loadCollection: function() {
    this._form.addClassName(this.options.loadingClassName);
    this.showLoadingText(this.options.loadingCollectionText);
    var options = Object.extend({ method: 'get' }, this.options.ajaxOptions);
    Object.extend(options, {
      parameters: 'editorId=' + encodeURIComponent(this.element.id),
      onComplete: Prototype.emptyFunction,
      onSuccess: function(transport) {
        var js = transport.responseText.strip();
        if (!/^\[.*\]$/.test(js)) // TODO: improve sanity check
          throw('Server returned an invalid collection representation.');
        this._collection = eval(js);
        this.checkForExternalText();
      }.bind(this),
      onFailure: this.onFailure
    });
    new Ajax.Request(this.options.loadCollectionURL, options);
  },

  showLoadingText: function(text) {
    this._controls.editor.disabled = true;
    var tempOption = this._controls.editor.firstChild;
    if (!tempOption) {
      tempOption = document.createElement('option');
      tempOption.value = '';
      this._controls.editor.appendChild(tempOption);
      tempOption.selected = true;
    }
    tempOption.update((text || '').stripScripts().stripTags());
  },

  checkForExternalText: function() {
    this._text = this.getText();
    if (this.options.loadTextURL)
      this.loadExternalText();
    else
      this.buildOptionList();
  },

  loadExternalText: function() {
    this.showLoadingText(this.options.loadingText);
    var options = Object.extend({ method: 'get' }, this.options.ajaxOptions);
    Object.extend(options, {
      parameters: 'editorId=' + encodeURIComponent(this.element.id),
      onComplete: Prototype.emptyFunction,
      onSuccess: function(transport) {
        this._text = transport.responseText.strip();
        this.buildOptionList();
      }.bind(this),
      onFailure: this.onFailure
    });
    new Ajax.Request(this.options.loadTextURL, options);
  },

  buildOptionList: function() {
    this._form.removeClassName(this.options.loadingClassName);
    this._collection = this._collection.map(function(entry) {
      return 2 === entry.length ? entry : [entry, entry].flatten();
    });
    var marker = ('value' in this.options) ? this.options.value : this._text;
    var textFound = this._collection.any(function(entry) {
      return entry[0] == marker;
    }.bind(this));
    this._controls.editor.update('');
    var option;
    this._collection.each(function(entry, index) {
      option = document.createElement('option');
      option.value = entry[0];
      option.selected = textFound ? entry[0] == marker : 0 == index;
      option.appendChild(document.createTextNode(entry[1]));
      this._controls.editor.appendChild(option);
    }.bind(this));
    this._controls.editor.disabled = false;
    Field.scrollFreeActivate(this._controls.editor);
  }
});

//**** DEPRECATION LAYER FOR InPlace[Collection]Editor! ****
//**** This only  exists for a while,  in order to  let ****
//**** users adapt to  the new API.  Read up on the new ****
//**** API and convert your code to it ASAP!            ****

Ajax.InPlaceEditor.prototype.initialize.dealWithDeprecatedOptions = function(options) {
  if (!options) return;
  function fallback(name, expr) {
    if (name in options || expr === undefined) return;
    options[name] = expr;
  };
  fallback('cancelControl', (options.cancelLink ? 'link' : (options.cancelButton ? 'button' :
    options.cancelLink == options.cancelButton == false ? false : undefined)));
  fallback('okControl', (options.okLink ? 'link' : (options.okButton ? 'button' :
    options.okLink == options.okButton == false ? false : undefined)));
  fallback('highlightColor', options.highlightcolor);
  fallback('highlightEndColor', options.highlightendcolor);
};

Object.extend(Ajax.InPlaceEditor, {
  DefaultOptions: {
    ajaxOptions: { },
    autoRows: 3,                                // Use when multi-line w/ rows == 1
    cancelControl: 'link',                      // 'link'|'button'|false
    cancelText: 'cancel',
    clickToEditText: 'Click to edit',
    externalControl: null,                      // id|elt
    externalControlOnly: false,
    fieldPostCreation: 'activate',              // 'activate'|'focus'|false
    formClassName: 'inplaceeditor-form',
    formId: null,                               // id|elt
    highlightColor: '#ffff99',
    highlightEndColor: '#ffffff',
    hoverClassName: '',
    htmlResponse: true,
    loadingClassName: 'inplaceeditor-loading',
    loadingText: 'Loading...',
    okControl: 'button',                        // 'link'|'button'|false
    okText: 'ok',
    paramName: 'value',
    rows: 1,                                    // If 1 and multi-line, uses autoRows
    savingClassName: 'inplaceeditor-saving',
    savingText: 'Saving...',
    size: 0,
    stripLoadedTextTags: false,
    submitOnBlur: false,
    textAfterControls: '',
    textBeforeControls: '',
    textBetweenControls: ''
  },
  DefaultCallbacks: {
    callback: function(form) {
      return Form.serialize(form);
    },
    onComplete: function(transport, element) {
      // For backward compatibility, this one is bound to the IPE, and passes
      // the element directly.  It was too often customized, so we don't break it.
      new Effect.Highlight(element, {
        startcolor: this.options.highlightColor, keepBackgroundImage: true });
    },
    onEnterEditMode: null,
    onEnterHover: function(ipe) {
      ipe.element.style.backgroundColor = ipe.options.highlightColor;
      if (ipe._effect)
        ipe._effect.cancel();
    },
    onFailure: function(transport, ipe) {
      alert('Error communication with the server: ' + transport.responseText.stripTags());
    },
    onFormCustomization: null, // Takes the IPE and its generated form, after editor, before controls.
    onLeaveEditMode: null,
    onLeaveHover: function(ipe) {
      ipe._effect = new Effect.Highlight(ipe.element, {
        startcolor: ipe.options.highlightColor, endcolor: ipe.options.highlightEndColor,
        restorecolor: ipe._originalBackground, keepBackgroundImage: true
      });
    }
  },
  Listeners: {
    click: 'enterEditMode',
    keydown: 'checkForEscapeOrReturn',
    mouseover: 'enterHover',
    mouseout: 'leaveHover'
  }
});

Ajax.InPlaceCollectionEditor.DefaultOptions = {
  loadingCollectionText: 'Loading options...'
};

// Delayed observer, like Form.Element.Observer,
// but waits for delay after last key input
// Ideal for live-search fields

Form.Element.DelayedObserver = Class.create({
  initialize: function(element, delay, callback) {
    this.delay     = delay || 0.5;
    this.element   = $(element);
    this.callback  = callback;
    this.timer     = null;
    this.lastValue = $F(this.element);
    Event.observe(this.element,'keyup',this.delayedListener.bindAsEventListener(this));
  },
  delayedListener: function(event) {
    if(this.lastValue == $F(this.element)) return;
    if(this.timer) clearTimeout(this.timer);
    this.timer = setTimeout(this.onTimerEvent.bind(this), this.delay * 1000);
    this.lastValue = $F(this.element);
  },
  onTimerEvent: function() {
    this.timer = null;
    this.callback(this.element, $F(this.element));
  }
});
// script.aculo.us slider.js v1.9.0, Thu Dec 23 16:54:48 -0500 2010

// Copyright (c) 2005-2010 Marty Haught, Thomas Fuchs
//
// script.aculo.us is freely distributable under the terms of an MIT-style license.
// For details, see the script.aculo.us web site: http://script.aculo.us/

if (!Control) var Control = { };

// options:
//  axis: 'vertical', or 'horizontal' (default)
//
// callbacks:
//  onChange(value)
//  onSlide(value)
Control.Slider = Class.create({
  initialize: function(handle, track, options) {
    var slider = this;

    if (Object.isArray(handle)) {
      this.handles = handle.collect( function(e) { return $(e) });
    } else {
      this.handles = [$(handle)];
    }

    this.track   = $(track);
    this.options = options || { };

    this.axis      = this.options.axis || 'horizontal';
    this.increment = this.options.increment || 1;
    this.step      = parseInt(this.options.step || '1');
    this.range     = this.options.range || $R(0,1);

    this.value     = 0; // assure backwards compat
    this.values    = this.handles.map( function() { return 0 });
    this.spans     = this.options.spans ? this.options.spans.map(function(s){ return $(s) }) : false;
    this.options.startSpan = $(this.options.startSpan || null);
    this.options.endSpan   = $(this.options.endSpan || null);

    this.restricted = this.options.restricted || false;

    this.maximum   = this.options.maximum || this.range.end;
    this.minimum   = this.options.minimum || this.range.start;

    // Will be used to align the handle onto the track, if necessary
    this.alignX = parseInt(this.options.alignX || '0');
    this.alignY = parseInt(this.options.alignY || '0');

    this.trackLength = this.maximumOffset() - this.minimumOffset();

    this.handleLength = this.isVertical() ?
      (this.handles[0].offsetHeight != 0 ?
        this.handles[0].offsetHeight : this.handles[0].style.height.replace(/px$/,"")) :
      (this.handles[0].offsetWidth != 0 ? this.handles[0].offsetWidth :
        this.handles[0].style.width.replace(/px$/,""));

    this.active   = false;
    this.dragging = false;
    this.disabled = false;

    if (this.options.disabled) this.setDisabled();

    // Allowed values array
    this.allowedValues = this.options.values ? this.options.values.sortBy(Prototype.K) : false;
    if (this.allowedValues) {
      this.minimum = this.allowedValues.min();
      this.maximum = this.allowedValues.max();
    }

    this.eventMouseDown = this.startDrag.bindAsEventListener(this);
    this.eventMouseUp   = this.endDrag.bindAsEventListener(this);
    this.eventMouseMove = this.update.bindAsEventListener(this);

    // Initialize handles in reverse (make sure first handle is active)
    this.handles.each( function(h,i) {
      i = slider.handles.length-1-i;
      slider.setValue(parseFloat(
        (Object.isArray(slider.options.sliderValue) ?
          slider.options.sliderValue[i] : slider.options.sliderValue) ||
         slider.range.start), i);
      h.makePositioned().observe("mousedown", slider.eventMouseDown);
    });

    this.track.observe("mousedown", this.eventMouseDown);
    document.observe("mouseup", this.eventMouseUp);
    document.observe("mousemove", this.eventMouseMove);

    this.initialized = true;
  },
  dispose: function() {
    var slider = this;
    Event.stopObserving(this.track, "mousedown", this.eventMouseDown);
    Event.stopObserving(document, "mouseup", this.eventMouseUp);
    Event.stopObserving(document, "mousemove", this.eventMouseMove);
    this.handles.each( function(h) {
      Event.stopObserving(h, "mousedown", slider.eventMouseDown);
    });
  },
  setDisabled: function(){
    this.disabled = true;
  },
  setEnabled: function(){
    this.disabled = false;
  },
  getNearestValue: function(value){
    if (this.allowedValues){
      if (value >= this.allowedValues.max()) return(this.allowedValues.max());
      if (value <= this.allowedValues.min()) return(this.allowedValues.min());

      var offset = Math.abs(this.allowedValues[0] - value);
      var newValue = this.allowedValues[0];
      this.allowedValues.each( function(v) {
        var currentOffset = Math.abs(v - value);
        if (currentOffset <= offset){
          newValue = v;
          offset = currentOffset;
        }
      });
      return newValue;
    }
    if (value > this.range.end) return this.range.end;
    if (value < this.range.start) return this.range.start;
    return value;
  },
  setValue: function(sliderValue, handleIdx){
    if (!this.active) {
      this.activeHandleIdx = handleIdx || 0;
      this.activeHandle    = this.handles[this.activeHandleIdx];
      this.updateStyles();
    }
    handleIdx = handleIdx || this.activeHandleIdx || 0;
    if (this.initialized && this.restricted) {
      if ((handleIdx>0) && (sliderValue<this.values[handleIdx-1]))
        sliderValue = this.values[handleIdx-1];
      if ((handleIdx < (this.handles.length-1)) && (sliderValue>this.values[handleIdx+1]))
        sliderValue = this.values[handleIdx+1];
    }
    sliderValue = this.getNearestValue(sliderValue);
    this.values[handleIdx] = sliderValue;
    this.value = this.values[0]; // assure backwards compat

    this.handles[handleIdx].style[this.isVertical() ? 'top' : 'left'] =
      this.translateToPx(sliderValue);

    this.drawSpans();
    if (!this.dragging || !this.event) this.updateFinished();
  },
  setValueBy: function(delta, handleIdx) {
    this.setValue(this.values[handleIdx || this.activeHandleIdx || 0] + delta,
      handleIdx || this.activeHandleIdx || 0);
  },
  translateToPx: function(value) {
    return Math.round(
      ((this.trackLength-this.handleLength)/(this.range.end-this.range.start)) *
      (value - this.range.start)) + "px";
  },
  translateToValue: function(offset) {
    return ((offset/(this.trackLength-this.handleLength) *
      (this.range.end-this.range.start)) + this.range.start);
  },
  getRange: function(range) {
    var v = this.values.sortBy(Prototype.K);
    range = range || 0;
    return $R(v[range],v[range+1]);
  },
  minimumOffset: function(){
    return(this.isVertical() ? this.alignY : this.alignX);
  },
  maximumOffset: function(){
    return(this.isVertical() ?
      (this.track.offsetHeight != 0 ? this.track.offsetHeight :
        this.track.style.height.replace(/px$/,"")) - this.alignY :
      (this.track.offsetWidth != 0 ? this.track.offsetWidth :
        this.track.style.width.replace(/px$/,"")) - this.alignX);
  },
  isVertical:  function(){
    return (this.axis == 'vertical');
  },
  drawSpans: function() {
    var slider = this;
    if (this.spans)
      $R(0, this.spans.length-1).each(function(r) { slider.setSpan(slider.spans[r], slider.getRange(r)) });
    if (this.options.startSpan)
      this.setSpan(this.options.startSpan,
        $R(0, this.values.length>1 ? this.getRange(0).min() : this.value ));
    if (this.options.endSpan)
      this.setSpan(this.options.endSpan,
        $R(this.values.length>1 ? this.getRange(this.spans.length-1).max() : this.value, this.maximum));
  },
  setSpan: function(span, range) {
    if (this.isVertical()) {
      span.style.top = this.translateToPx(range.start);
      span.style.height = this.translateToPx(range.end - range.start + this.range.start);
    } else {
      span.style.left = this.translateToPx(range.start);
      span.style.width = this.translateToPx(range.end - range.start + this.range.start);
    }
  },
  updateStyles: function() {
    this.handles.each( function(h){ Element.removeClassName(h, 'selected') });
    Element.addClassName(this.activeHandle, 'selected');
  },
  startDrag: function(event) {
    if (Event.isLeftClick(event)) {
      if (!this.disabled){
        this.active = true;

        var handle = Event.element(event);
        var pointer  = [Event.pointerX(event), Event.pointerY(event)];
        var track = handle;
        if (track==this.track) {
          var offsets  = this.track.cumulativeOffset();
          this.event = event;
          this.setValue(this.translateToValue(
           (this.isVertical() ? pointer[1]-offsets[1] : pointer[0]-offsets[0])-(this.handleLength/2)
          ));
          var offsets  = this.activeHandle.cumulativeOffset();
          this.offsetX = (pointer[0] - offsets[0]);
          this.offsetY = (pointer[1] - offsets[1]);
        } else {
          // find the handle (prevents issues with Safari)
          while((this.handles.indexOf(handle) == -1) && handle.parentNode)
            handle = handle.parentNode;

          if (this.handles.indexOf(handle)!=-1) {
            this.activeHandle    = handle;
            this.activeHandleIdx = this.handles.indexOf(this.activeHandle);
            this.updateStyles();

            var offsets  = this.activeHandle.cumulativeOffset();
            this.offsetX = (pointer[0] - offsets[0]);
            this.offsetY = (pointer[1] - offsets[1]);
          }
        }
      }
      Event.stop(event);
    }
  },
  update: function(event) {
   if (this.active) {
      if (!this.dragging) this.dragging = true;
      this.draw(event);
      if (Prototype.Browser.WebKit) window.scrollBy(0,0);
      Event.stop(event);
   }
  },
  draw: function(event) {
    var pointer = [Event.pointerX(event), Event.pointerY(event)];
    var offsets = this.track.cumulativeOffset();
    pointer[0] -= this.offsetX + offsets[0];
    pointer[1] -= this.offsetY + offsets[1];
    this.event = event;
    this.setValue(this.translateToValue( this.isVertical() ? pointer[1] : pointer[0] ));
    if (this.initialized && this.options.onSlide)
      this.options.onSlide(this.values.length>1 ? this.values : this.value, this);
  },
  endDrag: function(event) {
    if (this.active && this.dragging) {
      this.finishDrag(event, true);
      Event.stop(event);
    }
    this.active = false;
    this.dragging = false;
  },
  finishDrag: function(event, success) {
    this.active = false;
    this.dragging = false;
    this.updateFinished();
  },
  updateFinished: function() {
    if (this.initialized && this.options.onChange)
      this.options.onChange(this.values.length>1 ? this.values : this.value, this);
    this.event = null;
  }
});
var Prototip = {
  Version: '2.2.2'
};

Object.extend(Prototip, {
    REQUIRED_Prototype: "1.7",
    support: {
        canvas: !! document.createElement("canvas").getContext
    },
    insertScript: function (a) {
        try {
            document.write("<script type='text/javascript' src='" + a + "'><\/script>")
        } catch (b) {
            $$("head")[0].insert(new Element("script", {
                src: a,
                type: "text/javascript"
            }))
        }
    },
    start: function () {
        this.require("Prototype");
        var a = /prototip([\w\d-_.]+)?\.js(.*)/;
        this.path = (($$("script[src]").find(function (b) {
            return b.src.match(a)
        }) || {}).src || "").replace(a, "");
        Tips.paths = (function (b) {
            return {
                images: (/^(https?:\/\/|\/)/.test(b.images)) ? b.images : this.path + b.images,
                javascript: (/^(https?:\/\/|\/)/.test(b.javascript)) ? b.javascript : this.path + b.javascript
            }
        }.bind(this))(Tips.options.paths);
        if (!Prototip.Styles) {
            this.insertScript(Tips.paths.javascript + "styles.js")
        }
        if (!this.support.canvas) {
            if (document.documentMode >= 8 && !document.namespaces.ns_vml) {
                document.namespaces.add("ns_vml", "urn:schemas-microsoft-com:vml", "#default#VML")
            } else {
                document.observe("dom:loaded", function () {
                    var b = document.createStyleSheet();
                    b.cssText = "ns_vml\\:*{behavior:url(#default#VML)}"
                })
            }
        }
        Tips.initialize();
        Element.observe(window, "unload", this.unload)
    },
    require: function (a) {
        if ((typeof window[a] == "undefined") || (this.convertVersionString(window[a].Version) < this.convertVersionString(this["REQUIRED_" + a]))) {
            throw ("Prototip requires " + a + " >= " + this["REQUIRED_" + a])
        }
    },
    convertVersionString: function (a) {
        var b = a.replace(/_.*|\./g, "");
        b = parseInt(b + "0".times(4 - b.length));
        return a.indexOf("_") > -1 ? b - 1 : b
    },
    toggleInt: function (a) {
        return (a > 0) ? (-1 * a) : (a).abs()
    },
    unload: function () {
        Tips.removeAll()
    }
});
Object.extend(Tips, (function () {
    function a(b) {
        if (!b) {
            return
        }
        b.deactivate();
        if (b.tooltip) {
            b.wrapper.remove();
            if (Tips.fixIE) {
                b.iframeShim.remove()
            }
        }
        Tips.tips = Tips.tips.without(b)
    }
    return {
        tips: [],
        visible: [],
        initialize: function () {
            this.zIndexTop = this.zIndex
        },
        _inverse: {
            left: "right",
            right: "left",
            top: "bottom",
            bottom: "top",
            middle: "middle",
            horizontal: "vertical",
            vertical: "horizontal"
        },
        _stemTranslation: {
            width: "horizontal",
            height: "vertical"
        },
        inverseStem: function (b) {
            return !!arguments[1] ? this._inverse[b] : b
        },
        fixIE: (function (c) {
            var b = new RegExp("MSIE ([\\d.]+)").exec(c);
            return b ? (parseFloat(b[1]) < 7) : false
        })(navigator.userAgent),
        WebKit419: (Prototype.Browser.WebKit && !document.evaluate),
        add: function (b) {
            this.tips.push(b)
        },
        remove: function (d) {
            var g, e = [];
            for (var c = 0, b = this.tips.length; c < b; c++) {
                var f = this.tips[c];
                if (!g && f.element == $(d)) {
                    g = f
                } else {
                    if (!f.element.parentNode) {
                        e.push(f)
                    }
                }
            }
            a(g);
            for (var c = 0, b = e.length; c < b; c++) {
                var f = e[c];
                a(f)
            }
            d.prototip = null
        },
        removeAll: function () {
            for (var c = 0, b = this.tips.length; c < b; c++) {
                a(this.tips[c])
            }
        },
        raise: function (d) {
            if (d == this._highest) {
                return
            }
            if (this.visible.length === 0) {
                this.zIndexTop = this.options.zIndex;
                for (var c = 0, b = this.tips.length; c < b; c++) {
                    this.tips[c].wrapper.setStyle({
                        zIndex: this.options.zIndex
                    })
                }
            }
            d.wrapper.setStyle({
                zIndex: this.zIndexTop++
            });
            if (d.loader) {
                d.loader.setStyle({
                    zIndex: this.zIndexTop
                })
            }
            this._highest = d
        },
        addVisibile: function (b) {
            this.removeVisible(b);
            this.visible.push(b)
        },
        removeVisible: function (b) {
            this.visible = this.visible.without(b)
        },
        hideAll: function () {
            Tips.visible.invoke("hide")
        },
        isChrome: function(version) {
            if( ! version )
            {
                version = 32;
            }

            return (navigator.userAgent.toLowerCase().indexOf('chrome/'+version) > -1);
        },
        hook: function (c, g) {
            c = $(c), g = $(g);
            var l = Object.extend({
                offset: {
                    x: 0,
                    y: 0
                },
                position: false
            }, arguments[2] || {});
            var e = l.mouse || g.cumulativeOffset();
            e.left += l.offset.x;
            e.top += l.offset.y;
            var d = l.mouse ? [0, 0] : g.cumulativeScrollOffset(),
                b = document.viewport.getScrollOffsets(),
                h = l.mouse ? "mouseHook" : "target";
            e.left += (-1 * (d[0] - b[0]));

            // Only conditionally run if not Chrome 32
            // See GitHub bug #403
            if( ! this.isChrome('32') )
            {
                e.top += (-1 * (d[1] - b[1]));
            }

            if (l.mouse) {
                var f = [0, 0];
                f.width = 0;
                f.height = 0
            }
            var j = {
                element: c.getDimensions()
            }, k = {
                    element: Object.clone(e)
                };
            j[h] = l.mouse ? f : g.getDimensions();
            k[h] = Object.clone(e);
            for (var i in k) {
                switch (l[i]) {
                case "topRight":
                case "rightTop":
                    k[i].left += j[i].width;
                    break;
                case "topMiddle":
                    k[i].left += (j[i].width / 2);
                    break;
                case "rightMiddle":
                    k[i].left += j[i].width;
                    k[i].top += (j[i].height / 2);
                    break;
                case "bottomLeft":
                case "leftBottom":
                    k[i].top += j[i].height;
                    break;
                case "bottomRight":
                case "rightBottom":
                    k[i].left += j[i].width;
                    k[i].top += j[i].height;
                    break;
                case "bottomMiddle":
                    k[i].left += (j[i].width / 2);
                    k[i].top += j[i].height;
                    break;
                case "leftMiddle":
                    k[i].top += (j[i].height / 2);
                    break
                }
            }
            e.left += -1 * (k.element.left - k[h].left);
            e.top += -1 * (k.element.top - k[h].top);
            if (l.position) {
                c.setStyle({
                    left: e.left + "px",
                    top: e.top + "px"
                })
            }
            return e
        }
    }
})());
Tips.initialize();
var Tip = Class.create({
    initialize: function (c, e) {
        this.element = $(c);
        if (!this.element) {
            throw ("Prototip: Element not available, cannot create a tooltip.");
            return
        }
        Tips.remove(this.element);
        var a = (Object.isString(e) || Object.isElement(e)),
            b = a ? arguments[2] || [] : e;
        this.content = a ? e : null;
        if (b.style) {
            b = Object.extend(Object.clone(Prototip.Styles[b.style]), b)
        }
        this.options = Object.extend(Object.extend({
            ajax: false,
            border: 0,
            borderColor: "#000000",
            radius: 0,
            className: Tips.options.className,
            closeButton: Tips.options.closeButtons,
            delay: !(b.showOn && b.showOn == "click") ? 0.14 : false,
            hideAfter: false,
            hideOn: "mouseleave",
            hideOthers: false,
            hook: b.hook,
            offset: b.hook ? {
                x: 0,
                y: 0
            } : {
                x: 16,
                y: 16
            },
            fixed: (b.hook && !b.hook.mouse) ? true : false,
            showOn: "mousemove",
            stem: false,
            style: "default",
            target: this.element,
            title: false,
            viewport: (b.hook && !b.hook.mouse) ? false : true,
            width: false
        }, Prototip.Styles["default"]), b);
        this.target = $(this.options.target);
        this.radius = this.options.radius;
        this.border = (this.radius > this.options.border) ? this.radius : this.options.border;
        if (this.options.images) {
            this.images = this.options.images.include("://") ? this.options.images : Tips.paths.images + this.options.images
        } else {
            this.images = Tips.paths.images + "styles/" + (this.options.style || "") + "/"
        } if (!this.images.endsWith("/")) {
            this.images += "/"
        }
        if (Object.isString(this.options.stem)) {
            this.options.stem = {
                position: this.options.stem
            }
        }
        if (this.options.stem.position) {
            this.options.stem = Object.extend(Object.clone(Prototip.Styles[this.options.style].stem) || {}, this.options.stem);
            this.options.stem.position = [this.options.stem.position.match(/[a-z]+/)[0].toLowerCase(), this.options.stem.position.match(/[A-Z][a-z]+/)[0].toLowerCase()];
            this.options.stem.orientation = ["left", "right"].member(this.options.stem.position[0]) ? "horizontal" : "vertical";
            this.stemInverse = {
                horizontal: false,
                vertical: false
            }
        }
        if (this.options.ajax) {
            this.options.ajax.options = Object.extend({
                onComplete: Prototype.emptyFunction
            }, this.options.ajax.options || {})
        }
        if (this.options.hook.mouse) {
            var d = this.options.hook.tip.match(/[a-z]+/)[0].toLowerCase();
            this.mouseHook = Tips._inverse[d] + Tips._inverse[this.options.hook.tip.match(/[A-Z][a-z]+/)[0].toLowerCase()].capitalize()
        }
        this.fixSafari2 = (Tips.WebKit419 && this.radius);
        this.setup();
        Tips.add(this);
        this.activate();
        Prototip.extend(this)
    },
    setup: function () {
        this.wrapper = new Element("div", {
            className: "prototip"
        }).setStyle({
            zIndex: Tips.options.zIndex
        });
        if (this.fixSafari2) {
            this.wrapper.hide = function () {
                this.setStyle("left:-9500px;top:-9500px;visibility:hidden;");
                return this
            };
            this.wrapper.show = function () {
                this.setStyle("visibility:visible");
                return this
            };
            this.wrapper.visible = function () {
                return (this.getStyle("visibility") == "visible" && parseFloat(this.getStyle("top").replace("px", "")) > -9500)
            }
        }
        this.wrapper.hide();
        if (Tips.fixIE) {
            this.iframeShim = new Element("iframe", {
                className: "iframeShim",
                src: "javascript:false;",
                frameBorder: 0
            }).setStyle({
                display: "none",
                zIndex: Tips.options.zIndex - 1,
                opacity: 0
            })
        }
        if (this.options.ajax) {
            this.showDelayed = this.showDelayed.wrap(this.ajaxShow)
        }
        this.tip = new Element("div", {
            className: "content"
        });
        this.title = new Element("div", {
            className: "title"
        }).hide();
        if (this.options.closeButton || (this.options.hideOn.element && this.options.hideOn.element == "closeButton")) {
            this.closeButton = new Element("div", {
                className: "close"
            }).setPngBackground(this.images + "close.png")
        }
    },
    build: function () {
        if (document.loaded) {
            this._build();
            this._isBuilding = true;
            return true
        } else {
            if (!this._isBuilding) {
                document.observe("dom:loaded", this._build);
                return false
            }
        }
    },
    _build: function () {
        $(document.body).insert(this.wrapper);
        if (Tips.fixIE) {
            $(document.body).insert(this.iframeShim)
        }
        if (this.options.ajax) {
            $(document.body).insert(this.loader = new Element("div", {
                className: "prototipLoader"
            }).setPngBackground(this.images + "loader.gif").hide())
        }
        var g = "wrapper";
        if (this.options.stem.position) {
            this.stem = new Element("div", {
                className: "prototip_Stem"
            }).setStyle({
                height: this.options.stem[this.options.stem.orientation == "vertical" ? "height" : "width"] + "px"
            });
            var b = this.options.stem.orientation == "horizontal";
            this[g].insert(this.stemWrapper = new Element("div", {
                className: "prototip_StemWrapper clearfix"
            }).insert(this.stemBox = new Element("div", {
                className: "prototip_StemBox clearfix"
            })));
            this.stem.insert(this.stemImage = new Element("div", {
                className: "prototip_StemImage"
            }).setStyle({
                height: this.options.stem[b ? "width" : "height"] + "px",
                width: this.options.stem[b ? "height" : "width"] + "px"
            }));
            if (Tips.fixIE && !this.options.stem.position[1].toUpperCase().include("MIDDLE")) {
                this.stemImage.setStyle({
                    display: "inline"
                })
            }
            g = "stemBox"
        }
        if (this.border) {
            var d = this.border,
                f;
            this[g].insert(this.borderFrame = new Element("ul", {
                className: "borderFrame"
            }).insert(this.borderTop = new Element("li", {
                className: "borderTop borderRow"
            }).setStyle("height: " + d + "px").insert(new Element("div", {
                className: "prototip_CornerWrapper prototip_CornerWrapperTopLeft"
            }).insert(new Element("div", {
                className: "prototip_Corner"
            }))).insert(f = new Element("div", {
                className: "prototip_BetweenCorners"
            }).setStyle({
                height: d + "px"
            }).insert(new Element("div", {
                className: "prototip_Between"
            }).setStyle({
                margin: "0 " + d + "px",
                height: d + "px"
            }))).insert(new Element("div", {
                className: "prototip_CornerWrapper prototip_CornerWrapperTopRight"
            }).insert(new Element("div", {
                className: "prototip_Corner"
            })))).insert(this.borderMiddle = new Element("li", {
                className: "borderMiddle borderRow"
            }).insert(this.borderCenter = new Element("div", {
                className: "borderCenter"
            }).setStyle("padding: 0 " + d + "px"))).insert(this.borderBottom = new Element("li", {
                className: "borderBottom borderRow"
            }).setStyle("height: " + d + "px").insert(new Element("div", {
                className: "prototip_CornerWrapper prototip_CornerWrapperBottomLeft"
            }).insert(new Element("div", {
                className: "prototip_Corner"
            }))).insert(f.cloneNode(true)).insert(new Element("div", {
                className: "prototip_CornerWrapper prototip_CornerWrapperBottomRight"
            }).insert(new Element("div", {
                className: "prototip_Corner"
            })))));
            g = "borderCenter";
            var c = this.borderFrame.select(".prototip_Corner");
            $w("tl tr bl br").each(function (j, h) {
                if (this.radius > 0) {
                    Prototip.createCorner(c[h], j, {
                        backgroundColor: this.options.borderColor,
                        border: d,
                        radius: this.options.radius
                    })
                } else {
                    c[h].addClassName("prototip_Fill")
                }
                c[h].setStyle({
                    width: d + "px",
                    height: d + "px"
                }).addClassName("prototip_Corner" + j.capitalize())
            }.bind(this));
            this.borderFrame.select(".prototip_Between", ".borderMiddle", ".prototip_Fill").invoke("setStyle", {
                backgroundColor: this.options.borderColor
            })
        }
        this[g].insert(this.tooltip = new Element("div", {
            className: "tooltip " + this.options.className
        }).insert(this.toolbar = new Element("div", {
            className: "toolbar"
        }).insert(this.title)));
        if (this.options.width) {
            var e = this.options.width;
            if (Object.isNumber(e)) {
                e += "px"
            }
            this.tooltip.setStyle("width:" + e)
        }
        if (this.stem) {
            var a = {};
            a[this.options.stem.orientation == "horizontal" ? "top" : "bottom"] = this.stem;
            this.wrapper.insert(a);
            this.positionStem()
        }
        this.tooltip.insert(this.tip);
        if (!this.options.ajax) {
            this._update({
                title: this.options.title,
                content: this.content
            })
        }
    },
    _update: function (e) {
        var a = this.wrapper.getStyle("visibility");
        this.wrapper.setStyle("height:auto;width:auto;visibility:hidden").show();
        if (this.border) {
            this.borderTop.setStyle("height:0");
            this.borderTop.setStyle("height:0")
        }
        if (e.title) {
            this.title.show().update(e.title);
            this.toolbar.show()
        } else {
            if (!this.closeButton) {
                this.title.hide();
                this.toolbar.hide()
            }
        } if (Object.isElement(e.content)) {
            e.content.show()
        }
        if (Object.isString(e.content) || Object.isElement(e.content)) {
            this.tip.update(e.content)
        }
        this.tooltip.setStyle({
            width: this.tooltip.getWidth() + "px"
        });
        this.wrapper.setStyle("visibility:visible").show();
        this.tooltip.show();
        var c = this.tooltip.getDimensions(),
            b = {
                width: c.width + "px"
            }, d = [this.wrapper];
        if (Tips.fixIE) {
            d.push(this.iframeShim)
        }
        if (this.closeButton) {
            this.title.show().insert({
                top: this.closeButton
            });
            this.toolbar.show()
        }
        if (e.title || this.closeButton) {
            this.toolbar.setStyle("width: 100%")
        }
        b.height = null;
        this.wrapper.setStyle({
            visibility: a
        });
        this.tip.addClassName("clearfix");
        if (e.title || this.closeButton) {
            this.title.addClassName("clearfix")
        }
        if (this.border) {
            this.borderTop.setStyle("height:" + this.border + "px");
            this.borderTop.setStyle("height:" + this.border + "px");
            b = "width: " + (c.width + 2 * this.border) + "px";
            d.push(this.borderFrame)
        }
        d.invoke("setStyle", b);
        if (this.stem) {
            this.positionStem();
            if (this.options.stem.orientation == "horizontal") {
                this.wrapper.setStyle({
                    width: this.wrapper.getWidth() + this.options.stem.height + "px"
                })
            }
        }
        this.wrapper.hide()
    },
    activate: function () {
        this.eventShow = this.showDelayed.bindAsEventListener(this);
        this.eventHide = this.hide.bindAsEventListener(this);
        if (this.options.fixed && this.options.showOn == "mousemove") {
            this.options.showOn = "mouseover"
        }
        if (this.options.showOn && this.options.showOn == this.options.hideOn) {
            this.eventToggle = this.toggle.bindAsEventListener(this);
            this.element.observe(this.options.showOn, this.eventToggle)
        }
        if (this.closeButton) {
            this.closeButton.observe("mouseover", function (d) {
                d.setPngBackground(this.images + "close_hover.png")
            }.bind(this, this.closeButton)).observe("mouseout", function (d) {
                d.setPngBackground(this.images + "close.png")
            }.bind(this, this.closeButton))
        }
        var c = {
            element: this.eventToggle ? [] : [this.element],
            target: this.eventToggle ? [] : [this.target],
            tip: this.eventToggle ? [] : [this.wrapper],
            closeButton: [],
            none: []
        }, a = this.options.hideOn.element;
        this.hideElement = a || (!this.options.hideOn ? "none" : "element");
        this.hideTargets = c[this.hideElement];
        if (!this.hideTargets && a && Object.isString(a)) {
            this.hideTargets = this.tip.select(a)
        }
        $w("show hide").each(function (g) {
            var f = g.capitalize(),
                d = (this.options[g + "On"].event || this.options[g + "On"]);
            if (d == "mouseover") {
                d == "mouseenter"
            } else {
                if (d == "mouseout") {
                    d == "mouseleave"
                }
            }
            this[g + "Action"] = d
        }.bind(this));
        if (!this.eventToggle && this.options.showOn) {
            this.element.observe(this.options.showOn, this.eventShow)
        }
        if (this.hideTargets && this.options.hideOn) {
            this.hideTargets.invoke("observe", this.hideAction, this.eventHide)
        }
        if (!this.options.fixed && this.options.showOn == "click") {
            this.eventPosition = this.position.bindAsEventListener(this);
            this.element.observe("mousemove", this.eventPosition)
        }
        this.buttonEvent = this.hide.wrap(function (f, e) {
            var d = e.findElement(".close");
            if (d) {
                d.blur();
                e.stop();
                f(e)
            }
        }).bindAsEventListener(this);
        if (this.closeButton || (this.options.hideOn && (this.options.hideOn.element == ".close"))) {
            this.wrapper.observe("click", this.buttonEvent)
        }
        if (this.options.showOn != "click" && (this.hideElement != "element")) {
            this.eventCheckDelay = function () {
                this.clearTimer("show")
            }.bindAsEventListener(this);
            this.element.observe("mouseleave", this.eventCheckDelay)
        }
        if (this.options.hideOn || this.options.hideAfter) {
            var b = [this.element, this.wrapper];
            this.activityEnter = function () {
                Tips.raise(this);
                this.cancelHideAfter()
            }.bindAsEventListener(this);
            this.activityLeave = this.hideAfter.bindAsEventListener(this);
            b.invoke("observe", "mouseenter", this.activityEnter).invoke("observe", "mouseleave", this.activityLeave)
        }
        if (this.options.ajax && this.options.showOn != "click") {
            this.ajaxHideEvent = this.ajaxHide.bindAsEventListener(this);
            this.element.observe("mouseleave", this.ajaxHideEvent)
        }
    },
    deactivate: function () {
        if (this.options.showOn && this.options.showOn == this.options.hideOn) {
            this.element.stopObserving(this.options.showOn, this.eventToggle)
        } else {
            if (this.options.showOn) {
                this.element.stopObserving(this.options.showOn, this.eventShow)
            }
            if (this.hideTargets && this.options.hideOn) {
                this.hideTargets.invoke("stopObserving")
            }
        } if (this.eventPosition) {
            this.element.stopObserving("mousemove", this.eventPosition)
        }
        if (this.eventCheckDelay) {
            this.element.stopObserving("mouseout", this.eventCheckDelay)
        }
        this.wrapper.stopObserving();
        if (this.options.hideOn || this.options.hideAfter) {
            this.element.stopObserving("mouseenter", this.activityEnter).stopObserving("mouseleave", this.activityLeave)
        }
        if (this.ajaxHideEvent) {
            this.element.stopObserving("mouseleave", this.ajaxHideEvent)
        }
    },
    ajaxShow: function (c, b) {
        if (!this.tooltip) {
            if (!this.build()) {
                return
            }
        }
        this.position(b);
        if (this.ajaxContentLoading) {
            return
        } else {
            if (this.ajaxContentLoaded) {
                c(b);
                return
            }
        }
        this.ajaxContentLoading = true;
        var d = {
            fakePointer: {
                pointerX: 0,
                pointerY: 0
            }
        };
        if (b.pointer) {
            var e = b.pointer(),
                d = {
                    fakePointer: {
                        pointerX: e.x,
                        pointerY: e.y
                    }
                }
        } else {
            if (b.fakePointer) {
                d.fakePointer = b.fakePointer
            }
        }
        var a = Object.clone(this.options.ajax.options);
        a.onComplete = a.onComplete.wrap(function (g, f) {
            this._update({
                title: this.options.title,
                content: f.responseText
            });
            this.position(d);
            (function () {
                g(f);
                var h = (this.loader && this.loader.visible());
                if (this.loader) {
                    this.clearTimer("loader");
                    this.loader.remove();
                    this.loader = null
                }
                if (h) {
                    this.show()
                }
                this.ajaxContentLoaded = true;
                this.ajaxContentLoading = null
            }.bind(this)).delay(0.6)
        }.bind(this));
        this.loaderTimer = Element.show.delay(this.options.delay, this.loader);
        this.wrapper.hide();
        this.ajaxContentLoading = true;
        this.loader.show();
        this.ajaxTimer = (function () {
            new Ajax.Request(this.options.ajax.url, a)
        }.bind(this)).delay(this.options.delay);
        return false
    },
    ajaxHide: function () {
        this.clearTimer("loader")
    },
    showDelayed: function (a) {
        if (!this.tooltip) {
            if (!this.build()) {
                return
            }
        }
        this.position(a);
        if (this.wrapper.visible()) {
            return
        }
        this.clearTimer("show");
        this.showTimer = this.show.bind(this).delay(this.options.delay)
    },
    clearTimer: function (a) {
        if (this[a + "Timer"]) {
            clearTimeout(this[a + "Timer"])
        }
    },
    show: function () {
        if (this.wrapper.visible()) {
            return
        }
        if (Tips.fixIE) {
            this.iframeShim.show()
        }
        if (this.options.hideOthers) {
            Tips.hideAll()
        }
        Tips.addVisibile(this);
        this.tooltip.show();
        this.wrapper.show();
        if (this.stem) {
            this.stem.show()
        }
        this.element.fire("prototip:shown")
    },
    hideAfter: function (a) {
        if (this.options.ajax) {
            if (this.loader && this.options.showOn != "click") {
                this.loader.hide()
            }
        }
        if (!this.options.hideAfter) {
            return
        }
        this.cancelHideAfter();
        this.hideAfterTimer = this.hide.bind(this).delay(this.options.hideAfter)
    },
    cancelHideAfter: function () {
        if (this.options.hideAfter) {
            this.clearTimer("hideAfter")
        }
    },
    hide: function () {
        this.clearTimer("show");
        this.clearTimer("loader");
        if (!this.wrapper.visible()) {
            return
        }
        this.afterHide()
    },
    afterHide: function () {
        if (Tips.fixIE) {
            this.iframeShim.hide()
        }
        if (this.loader) {
            this.loader.hide()
        }
        this.wrapper.hide();
        (this.borderFrame || this.tooltip).show();
        Tips.removeVisible(this);
        this.element.fire("prototip:hidden")
    },
    toggle: function (a) {
        if (this.wrapper && this.wrapper.visible()) {
            this.hide(a)
        } else {
            this.showDelayed(a)
        }
    },
    positionStem: function () {
        var c = this.options.stem,
            b = arguments[0] || this.stemInverse,
            d = Tips.inverseStem(c.position[0], b[c.orientation]),
            f = Tips.inverseStem(c.position[1], b[Tips._inverse[c.orientation]]),
            a = this.radius || 0;
        this.stemImage.setPngBackground(this.images + d + f + ".png");
        if (c.orientation == "horizontal") {
            var e = (d == "left") ? c.height : 0;
            this.stemWrapper.setStyle("left: " + e + "px;");
            this.stemImage.setStyle({
                "float": d
            });
            this.stem.setStyle({
                left: 0,
                top: (f == "bottom" ? "100%" : f == "middle" ? "50%" : 0),
                marginTop: (f == "bottom" ? -1 * c.width : f == "middle" ? -0.5 * c.width : 0) + (f == "bottom" ? -1 * a : f == "top" ? a : 0) + "px"
            })
        } else {
            this.stemWrapper.setStyle(d == "top" ? "margin: 0; padding: " + c.height + "px 0 0 0;" : "padding: 0; margin: 0 0 " + c.height + "px 0;");
            this.stem.setStyle(d == "top" ? "top: 0; bottom: auto;" : "top: auto; bottom: 0;");
            this.stemImage.setStyle({
                margin: 0,
                "float": f != "middle" ? f : "none"
            });
            if (f == "middle") {
                this.stemImage.setStyle("margin: 0 auto;")
            } else {
                this.stemImage.setStyle("margin-" + f + ": " + a + "px;")
            } if (Tips.WebKit419) {
                if (d == "bottom") {
                    this.stem.setStyle({
                        position: "relative",
                        clear: "both",
                        top: "auto",
                        bottom: "auto",
                        "float": "left",
                        width: "100%",
                        margin: (-1 * c.height) + "px 0 0 0"
                    });
                    this.stem.style.display = "block"
                } else {
                    this.stem.setStyle({
                        position: "absolute",
                        "float": "none",
                        margin: 0
                    })
                }
            }
        }
        this.stemInverse = b
    },
    position: function (b) {
        if (!this.tooltip) {
            if (!this.build()) {
                return
            }
        }
        Tips.raise(this);
        if (Tips.fixIE) {
            var a = this.wrapper.getDimensions();
            if (!this.iframeShimDimensions || this.iframeShimDimensions.height != a.height || this.iframeShimDimensions.width != a.width) {
                this.iframeShim.setStyle({
                    width: a.width + "px",
                    height: a.height + "px"
                })
            }
            this.iframeShimDimensions = a
        }
        if (this.options.hook) {
            var j, h;
            if (this.mouseHook) {
                var k = document.viewport.getScrollOffsets(),
                    c = b.fakePointer || {};
                var g, i = 2;
                switch (this.mouseHook.toUpperCase()) {
                case "LEFTTOP":
                case "TOPLEFT":
                    g = {
                        x: 0 - i,
                        y: 0 - i
                    };
                    break;
                case "TOPMIDDLE":
                    g = {
                        x: 0,
                        y: 0 - i
                    };
                    break;
                case "TOPRIGHT":
                case "RIGHTTOP":
                    g = {
                        x: i,
                        y: 0 - i
                    };
                    break;
                case "RIGHTMIDDLE":
                    g = {
                        x: i,
                        y: 0
                    };
                    break;
                case "RIGHTBOTTOM":
                case "BOTTOMRIGHT":
                    g = {
                        x: i,
                        y: i
                    };
                    break;
                case "BOTTOMMIDDLE":
                    g = {
                        x: 0,
                        y: i
                    };
                    break;
                case "BOTTOMLEFT":
                case "LEFTBOTTOM":
                    g = {
                        x: 0 - i,
                        y: i
                    };
                    break;
                case "LEFTMIDDLE":
                    g = {
                        x: 0 - i,
                        y: 0
                    };
                    break
                }
                g.x += this.options.offset.x;
                g.y += this.options.offset.y;
                j = Object.extend({
                    offset: g
                }, {
                    element: this.options.hook.tip,
                    mouseHook: this.mouseHook,
                    mouse: {
                        top: c.pointerY || Event.pointerY(b) - k.top,
                        left: c.pointerX || Event.pointerX(b) - k.left
                    }
                });
                h = Tips.hook(this.wrapper, this.target, j);
                if (this.options.viewport) {
                    var n = this.getPositionWithinViewport(h),
                        m = n.stemInverse;
                    h = n.position;
                    h.left += m.vertical ? 2 * Prototip.toggleInt(g.x - this.options.offset.x) : 0;
                    h.top += m.vertical ? 2 * Prototip.toggleInt(g.y - this.options.offset.y) : 0;
                    if (this.stem && (this.stemInverse.horizontal != m.horizontal || this.stemInverse.vertical != m.vertical)) {
                        this.positionStem(m)
                    }
                }
                h = {
                    left: h.left + "px",
                    top: h.top + "px"
                };
                this.wrapper.setStyle(h)
            } else {
                j = Object.extend({
                    offset: this.options.offset
                }, {
                    element: this.options.hook.tip,
                    target: this.options.hook.target
                });
                h = Tips.hook(this.wrapper, this.target, Object.extend({
                    position: true
                }, j));
                h = {
                    left: h.left + "px",
                    top: h.top + "px"
                }
            } if (this.loader) {
                var e = Tips.hook(this.loader, this.target, Object.extend({
                    position: true
                }, j))
            }
            if (Tips.fixIE) {
                this.iframeShim.setStyle(h)
            }
        } else {
            var f = this.target.cumulativeOffset(),
                c = b.fakePointer || {}, h = {
                    left: ((this.options.fixed) ? f[0] : c.pointerX || Event.pointerX(b)) + this.options.offset.x,
                    top: ((this.options.fixed) ? f[1] : c.pointerY || Event.pointerY(b)) + this.options.offset.y
                };
            if (!this.options.fixed && this.element !== this.target) {
                var d = this.element.cumulativeOffset();
                h.left += -1 * (d[0] - f[0]);
                h.top += -1 * (d[1] - f[1])
            }
            if (!this.options.fixed && this.options.viewport) {
                var n = this.getPositionWithinViewport(h),
                    m = n.stemInverse;
                h = n.position;
                if (this.stem && (this.stemInverse.horizontal != m.horizontal || this.stemInverse.vertical != m.vertical)) {
                    this.positionStem(m)
                }
            }
            h = {
                left: h.left + "px",
                top: h.top + "px"
            };
            this.wrapper.setStyle(h);
            if (this.loader) {
                this.loader.setStyle(h)
            }
            if (Tips.fixIE) {
                this.iframeShim.setStyle(h)
            }
        }
    },
    getPositionWithinViewport: function (c) {
        var e = {
            horizontal: false,
            vertical: false
        }, d = this.wrapper.getDimensions(),
            b = document.viewport.getScrollOffsets(),
            a = document.viewport.getDimensions(),
            g = {
                left: "width",
                top: "height"
            };
        for (var f in g) {
            if ((c[f] + d[g[f]] - b[f]) > a[g[f]]) {
                c[f] = c[f] - (d[g[f]] + (2 * this.options.offset[f == "left" ? "x" : "y"]));
                if (this.stem) {
                    e[Tips._stemTranslation[g[f]]] = true
                }
            }
        }
        return {
            position: c,
            stemInverse: e
        }
    }
});
Object.extend(Prototip, {
    createCorner: function (d, g) {
        var j = arguments[2] || this.options,
            f = j.radius,
            c = j.border,
            e = {
                top: (g.charAt(0) == "t"),
                left: (g.charAt(1) == "l")
            };
        if (this.support.canvas) {
            var b = new Element("canvas", {
                className: "cornerCanvas" + g.capitalize(),
                width: c + "px",
                height: c + "px"
            });
            d.insert(b);
            var i = b.getContext("2d");
            i.fillStyle = j.backgroundColor;
            i.arc((e.left ? f : c - f), (e.top ? f : c - f), f, 0, Math.PI * 2, true);
            i.fill();
            i.fillRect((e.left ? f : 0), 0, c - f, c);
            i.fillRect(0, (e.top ? f : 0), c, c - f)
        } else {
            var h;
            d.insert(h = new Element("div").setStyle({
                width: c + "px",
                height: c + "px",
                margin: 0,
                padding: 0,
                display: "block",
                position: "relative",
                overflow: "hidden"
            }));
            var a = new Element("ns_vml:roundrect", {
                fillcolor: j.backgroundColor,
                strokeWeight: "1px",
                strokeColor: j.backgroundColor,
                arcSize: (f / c * 0.5).toFixed(2)
            }).setStyle({
                width: 2 * c - 1 + "px",
                height: 2 * c - 1 + "px",
                position: "absolute",
                left: (e.left ? 0 : (-1 * c)) + "px",
                top: (e.top ? 0 : (-1 * c)) + "px"
            });
            h.insert(a);
            a.outerHTML = a.outerHTML
        }
    }
});
Element.addMethods({
    setPngBackground: function (c, b) {
        c = $(c);
        var a = Object.extend({
            align: "top left",
            repeat: "no-repeat",
            sizingMethod: "scale",
            backgroundColor: ""
        }, arguments[2] || {});
        c.setStyle(Tips.fixIE ? {
            filter: "progid:DXImageTransform.Microsoft.AlphaImageLoader(src='" + b + "'', sizingMethod='" + a.sizingMethod + "')"
        } : {
            background: a.backgroundColor + " url(" + b + ") " + a.align + " " + a.repeat
        });
        return c
    }
});
Prototip.Methods = {
    hold: function (a) {
        if (a.element && !a.element.parentNode) {
            return true
        }
        return false
    },
    show: function () {
        if (Prototip.Methods.hold(this)) {
            return
        }
        Tips.raise(this);
        this.cancelHideAfter();
        var d = {};
        if (this.options.hook && !this.options.hook.mouse) {
            d.fakePointer = {
                pointerX: 0,
                pointerY: 0
            }
        } else {
            var a = this.target.cumulativeOffset(),
                c = this.target.cumulativeScrollOffset(),
                b = document.viewport.getScrollOffsets();
            a.left += (-1 * (c[0] - b[0]));
            a.top += (-1 * (c[1] - b[1]));
            d.fakePointer = {
                pointerX: a.left,
                pointerY: a.top
            }
        } if (this.options.ajax && !this.ajaxContentLoaded) {
            this.ajaxShow(this.showDelayed, d)
        } else {
            this.showDelayed(d)
        }
        this.hideAfter()
    }
};
Prototip.extend = function (a) {
    a.element.prototip = {};
    Object.extend(a.element.prototip, {
        show: Prototip.Methods.show.bind(a),
        hide: a.hide.bind(a),
        remove: Tips.remove.bind(Tips, a.element)
    })
};
Prototip.start();
/*
* Really easy field validation with Prototype
* http://tetlaw.id.au/view/javascript/really-easy-field-validation
* Andrew Tetlaw
* Version 1.5.4.1 (2007-01-05)
* 
* Copyright (c) 2007 Andrew Tetlaw
* Permission is hereby granted, free of charge, to any person
* obtaining a copy of this software and associated documentation
* files (the "Software"), to deal in the Software without
* restriction, including without limitation the rights to use, copy,
* modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be
* included in all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
* BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
* ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
* 
*/
var Validator = Class.create();

Validator.prototype = {
	initialize : function(className, error, test, options) {
		if(typeof test == 'function'){
			this.options = $H(options);
			this._test = test;
		} else {
			this.options = $H(test);
			this._test = function(){return true};
		}
		this.error = error || 'Validation failed.';
		this.className = className;
	},
	test : function(v, elm) {
		return (this._test(v,elm) && this.options.all(function(p){
			return Validator.methods[p.key] ? Validator.methods[p.key](v,elm,p.value) : true;
		}));
	}
}
Validator.methods = {
	pattern : function(v,elm,opt) {return Validation.get('IsEmpty').test(v) || opt.test(v)},
	minLength : function(v,elm,opt) {return v.length >= opt},
	maxLength : function(v,elm,opt) {return v.length <= opt},
	min : function(v,elm,opt) {return v >= parseFloat(opt)}, 
	max : function(v,elm,opt) {return v <= parseFloat(opt)},
	notOneOf : function(v,elm,opt) {return $A(opt).all(function(value) {
		return v != value;
	})},
	oneOf : function(v,elm,opt) {return $A(opt).any(function(value) {
		return v == value;
	})},
	is : function(v,elm,opt) {return v == opt},
	isNot : function(v,elm,opt) {return v != opt},
	equalToField : function(v,elm,opt) {return v == $F(opt)},
	notEqualToField : function(v,elm,opt) {return v != $F(opt)},
	include : function(v,elm,opt) {return $A(opt).all(function(value) {
		return Validation.get(value).test(v,elm);
	})}
}

var Validation = Class.create();

Validation.prototype = {
	initialize : function(form, options){
		this.options = Object.extend({
			onSubmit : true,
			stopOnFirst : false,
			immediate : false,
			focusOnError : true,
			useTitles : false,
			onFormValidate : function(result, form) {},
			onElementValidate : function(result, elm) {}
		}, options || {});
		this.form = $(form);
		if(this.options.onSubmit) Event.observe(this.form,'submit',this.onSubmit.bind(this),false);
		if(this.options.immediate) {
			var useTitles = this.options.useTitles;
			var callback = this.options.onElementValidate;
			Form.getElements(this.form).each(function(input) { // Thanks Mike!
				Event.observe(input, 'blur', function(ev) { Validation.validate(Event.element(ev),{useTitle : useTitles, onElementValidate : callback}); });
			});
		}
	},
	onSubmit :  function(ev){
		if(!this.validate()) Event.stop(ev);
	},
	validate : function() {
		var result = false;
		var useTitles = this.options.useTitles;
		var callback = this.options.onElementValidate;
		if(this.options.stopOnFirst) {
			result = Form.getElements(this.form).all(function(elm) { return Validation.validate(elm,{useTitle : useTitles, onElementValidate : callback}); });
		} else {
			result = Form.getElements(this.form).collect(function(elm) { return Validation.validate(elm,{useTitle : useTitles, onElementValidate : callback}); }).all();
		}
		if(!result && this.options.focusOnError) {
			Form.getElements(this.form).findAll(function(elm){return $(elm).hasClassName('validation-failed')}).first().focus()
		}
		this.options.onFormValidate(result, this.form);
		return result;
	},
	reset : function() {
		Form.getElements(this.form).each(Validation.reset);
	}
}

Object.extend(Validation, {
	validate : function(elm, options){
		options = Object.extend({
			useTitle : false,
			onElementValidate : function(result, elm) {}
		}, options || {});
		elm = $(elm);
		var cn = elm.classNames();
		return result = cn.all(function(value) {
			var test = Validation.test(value,elm,options.useTitle);
			options.onElementValidate(test, elm);
			return test;
		});
	},
	test : function(name, elm, useTitle) {
		var v = Validation.get(name);
		var prop = '__advice'+name.camelize();
		try {
		if(Validation.isVisible(elm) && !v.test($F(elm), elm)) {
			if(!elm[prop]) {
				var advice = Validation.getAdvice(name, elm);
				if(advice == null) {
					var errorMsg = useTitle ? ((elm && elm.title) ? elm.title : v.error) : v.error;
					advice = '<div class="validation-advice" id="advice-' + name + '-' + Validation.getElmID(elm) +'" style="display:none">' + errorMsg + '</div>'
					switch (elm.type.toLowerCase()) {
						case 'checkbox':
						case 'radio':
							var p = elm.parentNode;
							if(p) {
								new Insertion.Bottom(p, advice);
							} else {
								new Insertion.After(elm, advice);
							}
							break;
						default:
							new Insertion.After(elm, advice);
				    }
					advice = Validation.getAdvice(name, elm);
				}
				if(typeof Effect == 'undefined') {
					advice.style.display = 'block';
				} else {
					new Effect.Appear(advice, {duration : 1 });
				}
			}
			elm[prop] = true;
			elm.removeClassName('validation-passed');
			elm.addClassName('validation-failed');
			return false;
		} else {
			var advice = Validation.getAdvice(name, elm);
			if(advice != null) advice.hide();
			elm[prop] = '';
			elm.removeClassName('validation-failed');
			elm.addClassName('validation-passed');
			return true;
		}
		} catch(e) {
			throw(e)
		}
	},
	isVisible : function(elm) {
		while(elm.tagName != 'BODY') {
			if(!$(elm).visible()) return false;
			elm = elm.parentNode;
		}
		return true;
	},
	getAdvice : function(name, elm) {
		return $('advice-' + name + '-' + Validation.getElmID(elm)) || $('advice-' + Validation.getElmID(elm));
	},
	getElmID : function(elm) {
		return elm.id ? elm.id : elm.name;
	},
	reset : function(elm) {
		elm = $(elm);
		var cn = elm.classNames();
		cn.each(function(value) {
			var prop = '__advice'+value.camelize();
			if(elm[prop]) {
				var advice = Validation.getAdvice(value, elm);
				advice.hide();
				elm[prop] = '';
			}
			elm.removeClassName('validation-failed');
			elm.removeClassName('validation-passed');
		});
	},
	add : function(className, error, test, options) {
		var nv = {};
		nv[className] = new Validator(className, error, test, options);
		Object.extend(Validation.methods, nv);
	},
	addAllThese : function(validators) {
		var nv = {};
		$A(validators).each(function(value) {
				nv[value[0]] = new Validator(value[0], value[1], value[2], (value.length > 3 ? value[3] : {}));
			});
		Object.extend(Validation.methods, nv);
	},
	get : function(name) {
		return  Validation.methods[name] ? Validation.methods[name] : Validation.methods['_LikeNoIDIEverSaw_'];
	},
	methods : {
		'_LikeNoIDIEverSaw_' : new Validator('_LikeNoIDIEverSaw_','',{})
	}
});

Validation.add('IsEmpty', '', function(v) {
				return  ((v == null) || (v.length == 0)); // || /^\s+$/.test(v));
			});

Validation.addAllThese([
	['required', 'This is a required field.', function(v) {
				return !Validation.get('IsEmpty').test(v);
			}],
	['validate-number', 'Please enter a valid number in this field.', function(v) {
				return Validation.get('IsEmpty').test(v) || (!isNaN(v) && !/^\s+$/.test(v));
			}],
	['validate-digits', 'Please use numbers only in this field. please avoid spaces or other characters such as dots or commas.', function(v) {
				return Validation.get('IsEmpty').test(v) ||  !/[^\d]/.test(v);
			}],
	['validate-alpha', 'Please use letters only (a-z) in this field.', function (v) {
				return Validation.get('IsEmpty').test(v) ||  /^[a-zA-Z]+$/.test(v)
			}],
	['validate-alphanum', 'Please use only letters (a-z) or numbers (0-9) only in this field. No spaces or other characters are allowed.', function(v) {
				return Validation.get('IsEmpty').test(v) ||  !/\W/.test(v)
			}],
	['validate-date', 'Please enter a valid date.', function(v) {
				var test = new Date(v);
				return Validation.get('IsEmpty').test(v) || !isNaN(test);
			}],
	['validate-email', 'Please enter a valid email address. For example fred@domain.com .', function (v) {
				return Validation.get('IsEmpty').test(v) || /\w{1,}[@][\w\-]{1,}([.]([\w\-]{1,})){1,3}$/.test(v)
			}],
	['validate-url', 'Please enter a valid URL.', function (v) {
				return Validation.get('IsEmpty').test(v) || /^(http|https|ftp):\/\/(([A-Z0-9][A-Z0-9_-]*)(\.[A-Z0-9][A-Z0-9_-]*)+)(:(\d+))?\/?/i.test(v)
			}],
	['validate-date-au', 'Please use this date format: dd/mm/yyyy. For example 17/03/2006 for the 17th of March, 2006.', function(v) {
				if(Validation.get('IsEmpty').test(v)) return true;
				var regex = /^(\d{2})\/(\d{2})\/(\d{4})$/;
				if(!regex.test(v)) return false;
				var d = new Date(v.replace(regex, '$2/$1/$3'));
				return ( parseInt(RegExp.$2, 10) == (1+d.getMonth()) ) && 
							(parseInt(RegExp.$1, 10) == d.getDate()) && 
							(parseInt(RegExp.$3, 10) == d.getFullYear() );
			}],
	['validate-currency-dollar', 'Please enter a valid $ amount. For example $100.00 .', function(v) {
				// [$]1[##][,###]+[.##]
				// [$]1###+[.##]
				// [$]0.##
				// [$].##
				return Validation.get('IsEmpty').test(v) ||  /^\$?\-?([1-9]{1}[0-9]{0,2}(\,[0-9]{3})*(\.[0-9]{0,2})?|[1-9]{1}\d*(\.[0-9]{0,2})?|0(\.[0-9]{0,2})?|(\.[0-9]{1,2})?)$/.test(v)
			}],
	['validate-selection', 'Please make a selection', function(v,elm){
				return elm.options ? elm.selectedIndex > 0 : !Validation.get('IsEmpty').test(v);
			}],
	['validate-one-required', 'Please select one of the above options.', function (v,elm) {
				var p = elm.parentNode;
				var options = p.getElementsByTagName('INPUT');
				return $A(options).any(function(elm) {
					return $F(elm);
				});
			}]
]);
/**
 * @author Ryan Johnson <http://syntacticx.com/>
 * @copyright 2008 PersonalGrid Corporation <http://personalgrid.com/>
 * @package LivePipe UI
 * @license MIT
 * @url http://livepipe.net/core
 * @require prototype.js
 */

if(typeof(Control) == 'undefined')
    Control = {};
    
var $proc = function(proc){
    return typeof(proc) == 'function' ? proc : function(){return proc};
};

var $value = function(value){
    return typeof(value) == 'function' ? value() : value;
};

Object.Event = {
    extend: function(object){
        object._objectEventSetup = function(event_name){
            this._observers = this._observers || {};
            this._observers[event_name] = this._observers[event_name] || [];
        };
        object.observe = function(event_name,observer){
            if(typeof(event_name) == 'string' && typeof(observer) != 'undefined'){
                this._objectEventSetup(event_name);
                if(!this._observers[event_name].include(observer))
                    this._observers[event_name].push(observer);
            }else
                for(var e in event_name)
                    this.observe(e,event_name[e]);
        };
        object.stopObserving = function(event_name,observer){
            this._objectEventSetup(event_name);
            if(event_name && observer)
                this._observers[event_name] = this._observers[event_name].without(observer);
            else if(event_name)
                this._observers[event_name] = [];
            else
                this._observers = {};
        };
        object.observeOnce = function(event_name,outer_observer){
            var inner_observer = function(){
                outer_observer.apply(this,arguments);
                this.stopObserving(event_name,inner_observer);
            }.bind(this);
            this._objectEventSetup(event_name);
            this._observers[event_name].push(inner_observer);
        };
        object.notify = function(event_name){
            this._objectEventSetup(event_name);
            var collected_return_values = [];
            var args = $A(arguments).slice(1);
            try{
                for(var i = 0; i < this._observers[event_name].length; ++i)
                    collected_return_values.push(this._observers[event_name][i].apply(this._observers[event_name][i],args) || null);
            }catch(e){
                if(e == $break)
                    return false;
                else
                    throw e;
            }
            return collected_return_values;
        };
        if(object.prototype){
            object.prototype._objectEventSetup = object._objectEventSetup;
            object.prototype.observe = object.observe;
            object.prototype.stopObserving = object.stopObserving;
            object.prototype.observeOnce = object.observeOnce;
            object.prototype.notify = function(event_name){
                if(object.notify){
                    var args = $A(arguments).slice(1);
                    args.unshift(this);
                    args.unshift(event_name);
                    object.notify.apply(object,args);
                }
                this._objectEventSetup(event_name);
                var args = $A(arguments).slice(1);
                var collected_return_values = [];
                try{
                    if(this.options && this.options[event_name] && typeof(this.options[event_name]) == 'function')
                        collected_return_values.push(this.options[event_name].apply(this,args) || null);
                    var callbacks_copy = this._observers[event_name]; // since original array will be modified after observeOnce calls
                    for(var i = 0; i < callbacks_copy.length; ++i)
                        collected_return_values.push(callbacks_copy[i].apply(callbacks_copy[i],args) || null);
                }catch(e){
                    if(e == $break)
                        return false;
                    else
                        throw e;
                }
                return collected_return_values;
            };
        }
    }
};

/* Begin Core Extensions */

//Element.observeOnce
Element.addMethods({
    observeOnce: function(element,event_name,outer_callback){
        var inner_callback = function(){
            outer_callback.apply(this,arguments);
            Element.stopObserving(element,event_name,inner_callback);
        };
        Element.observe(element,event_name,inner_callback);
    }
});

//mouse:wheel
(function(){
    function wheel(event){
        var delta, element, custom_event;
        // normalize the delta
        if (event.wheelDelta) { // IE & Opera
            delta = event.wheelDelta / 120;
        } else if (event.detail) { // W3C
            delta =- event.detail / 3;
        }
        if (!delta) { return; }
        element = Event.extend(event).target;
        element = Element.extend(element.nodeType === Node.TEXT_NODE ? element.parentNode : element);
        custom_event = element.fire('mouse:wheel',{ delta: delta });
        if (custom_event.stopped) {
            Event.stop(event);
            return false;
        }
    }
    document.observe('mousewheel',wheel);
    document.observe('DOMMouseScroll',wheel);
})();

/* End Core Extensions */

//from PrototypeUI
var IframeShim = Class.create({
    initialize: function() {
        this.element = new Element('iframe',{
            style: 'position:absolute;filter:progid:DXImageTransform.Microsoft.Alpha(opacity=0);display:none',
            src: 'javascript:void(0);',
            frameborder: 0 
        });
        $(document.body).insert(this.element);
    },
    hide: function() {
        this.element.hide();
        return this;
    },
    show: function() {
        this.element.show();
        return this;
    },
    positionUnder: function(element) {
        var element = $(element);
        var offset = element.cumulativeOffset();
        var dimensions = element.getDimensions();
        this.element.setStyle({
            left: offset[0] + 'px',
            top: offset[1] + 'px',
            width: dimensions.width + 'px',
            height: dimensions.height + 'px',
            zIndex: element.getStyle('zIndex') - 1
        }).show();
        return this;
    },
    setBounds: function(bounds) {
        for(prop in bounds)
            bounds[prop] += 'px';
        this.element.setStyle(bounds);
        return this;
    },
    destroy: function() {
        if(this.element)
            this.element.remove();
        return this;
    }
});

/**
 * @author Ryan Johnson <http://syntacticx.com/>
 * @copyright 2008 PersonalGrid Corporation <http://personalgrid.com/>
 * @package LivePipe UI
 * @license MIT
 * @url http://livepipe.net/control/tabs
 * @require prototype.js, livepipe.js
 */

/*global window, document, Prototype, $, $A, $H, $break, Class, Element, Event, Control */

if(typeof(Prototype) == "undefined") {
    throw "Control.Tabs requires Prototype to be loaded."; }
if(typeof(Object.Event) == "undefined") {
    throw "Control.Tabs requires Object.Event to be loaded."; }

Control.Tabs = Class.create({
    initialize: function(tab_list_container,options){
        if(!$(tab_list_container)) {
            throw "Control.Tabs could not find the element: " + tab_list_container; }
        this.activeContainer = false;
        this.activeLink = false;
        this.containers = $H({});
        this.links = [];
        Control.Tabs.instances.push(this);
        this.options = {
            beforeChange: Prototype.emptyFunction,
            afterChange: Prototype.emptyFunction,
            hover: false,
            linkSelector: 'li a',
            setClassOnContainer: false,
            activeClassName: 'active',
            defaultTab: 'first',
            autoLinkExternal: true,
            targetRegExp: /#(.+)$/,
            showFunction: Element.show,
            hideFunction: Element.hide
        };
        Object.extend(this.options,options || {});
        (typeof(this.options.linkSelector) == 'string' ? 
            $(tab_list_container).select(this.options.linkSelector) : 
            this.options.linkSelector($(tab_list_container))
        ).findAll(function(link){
            return (/^#/).exec(link.href.replace(window.location.href.split('#')[0],''));
        }).each(function(link){
            this.addTab(link);
        }.bind(this));
        this.containers.values().each(Element.hide);
        if(this.options.defaultTab == 'first') {
            this.setActiveTab(this.links.first());
        } else if(this.options.defaultTab == 'last') {
            this.setActiveTab(this.links.last());
        } else {
            this.setActiveTab(this.options.defaultTab); }
        var targets = this.options.targetRegExp.exec(window.location);
        if(targets && targets[1]){
            targets[1].split(',').each(function(target){
                this.setActiveTab(this.links.find(function(link){
                    return link.key == target;
                }));
            }.bind(this));
        }
        if(this.options.autoLinkExternal){
            $A(document.getElementsByTagName('a')).each(function(a){
                if(!this.links.include(a)){
                    var clean_href = a.href.replace(window.location.href.split('#')[0],'');
                    if(clean_href.substring(0,1) == '#'){
                        if(this.containers.keys().include(clean_href.substring(1))){
                            $(a).observe('click',function(event,clean_href){
                                this.setActiveTab(clean_href.substring(1));
                            }.bindAsEventListener(this,clean_href));
                        }
                    }
                }
            }.bind(this));
        }
    },
    addTab: function(link){
        this.links.push(link);
        link.key = link.getAttribute('href').replace(window.location.href.split('#')[0],'').split('#').last().replace(/#/,'');
        var container = $(link.key);
        if(!container) {
            throw "Control.Tabs: #" + link.key + " was not found on the page."; }
        this.containers.set(link.key,container);
        link[this.options.hover ? 'onmouseover' : 'onclick'] = function(link){
            if(window.event) {
                Event.stop(window.event); }
            this.setActiveTab(link);
            return false;
        }.bind(this,link);
    },
    setActiveTab: function(link){
        if(!link && typeof(link) == 'undefined') {
            return; }
        if(typeof(link) == 'string'){
            this.setActiveTab(this.links.find(function(_link){
                return _link.key == link;
            }));
        }else if(typeof(link) == 'number'){
            this.setActiveTab(this.links[link]);
        }else{
            if(this.notify('beforeChange',this.activeContainer,this.containers.get(link.key)) === false) {
                return; }
            if(this.activeContainer) {
                this.options.hideFunction(this.activeContainer); }
            this.links.each(function(item){
                (this.options.setClassOnContainer ? $(item.parentNode) : item).removeClassName(this.options.activeClassName);
            }.bind(this));
            (this.options.setClassOnContainer ? $(link.parentNode) : link).addClassName(this.options.activeClassName);
            this.activeContainer = this.containers.get(link.key);
            this.activeLink = link;
            this.options.showFunction(this.containers.get(link.key));
            this.notify('afterChange',this.containers.get(link.key));
        }
    },
    next: function(){
        this.links.each(function(link,i){
            if(this.activeLink == link && this.links[i + 1]){
                this.setActiveTab(this.links[i + 1]);
                throw $break;
            }
        }.bind(this));
    },
    previous: function(){
        this.links.each(function(link,i){
            if(this.activeLink == link && this.links[i - 1]){
                this.setActiveTab(this.links[i - 1]);
                throw $break;
            }
        }.bind(this));
    },
    first: function(){
        this.setActiveTab(this.links.first());
    },
    last: function(){
        this.setActiveTab(this.links.last());
    }
});
Object.extend(Control.Tabs,{
    instances: [],
    findByTabId: function(id){
        return Control.Tabs.instances.find(function(tab){
            return tab.links.find(function(link){
                return link.key == id;
            });
        });
    }
});
Object.Event.extend(Control.Tabs);

/**
 * Copyright (c)2005-2007 Matt Kruse (javascripttoolbox.com)
 * 
 * Dual licensed under the MIT and GPL licenses. 
 * This basically means you can use this code however you want for
 * free, but don't claim to have written it yourself!
 * Donations always accepted: http://www.JavascriptToolbox.com/donate/
 * 
 * Please do not link to the .js files on javascripttoolbox.com from
 * your site. Copy the files locally to your server instead.
 * 
 */
// Global objects to keep track of DynamicOptionList objects created on the page
var dynamicOptionListCount=0;
var dynamicOptionListObjects = new Array();

// Init call to setup lists after page load. One call to this function sets up all lists.
function initDynamicOptionLists() {
	// init each DynamicOptionList object
	for (var i=0; i<dynamicOptionListObjects.length; i++) {
		var dol = dynamicOptionListObjects[i];

		// Find the form associated with this list
		if (dol.formName!=null) { 
			dol.form = document.forms[dol.formName];
		}
		else if (dol.formIndex!=null) {
			dol.form = document.forms[dol.formIndex];
		}
		else {
			// Form wasn't set manually, so go find it!
			// Search for the first form element name in the lists
			var name = dol.fieldNames[0][0];
			for (var f=0; f<document.forms.length; f++) {
				if (typeof(document.forms[f][name])!="undefined") {
					dol.form = document.forms[f];
					break;
				}
			}
			if (dol.form==null) {
				alert("ERROR: Couldn't find form element "+name+" in any form on the page! Init aborted"); return;
			}
		}

		// Form is found, now set the onchange attributes of each dependent select box
		for (var j=0; j<dol.fieldNames.length; j++) {
			// For each set of field names...
			for (var k=0; k<dol.fieldNames[j].length-1; k++) {
				// For each field in the set...
				var selObj = dol.form[dol.fieldNames[j][k]];
				if (typeof(selObj)=="undefined") { alert("Select box named "+dol.fieldNames[j][k]+" could not be found in the form. Init aborted"); return; }
				// Map the HTML options in the first select into the options we created
				if (k==0) {
					if (selObj.options!=null) {
						for (l=0; l<selObj.options.length; l++) {
							var sopt = selObj.options[l];
							var m = dol.findMatchingOptionInArray(dol.options,sopt.text,sopt.value,false);
							if (m!=null) {
								var reselectForNN6 = sopt.selected;
								var m2 = new Option(sopt.text, sopt.value, sopt.defaultSelected, sopt.selected);
								m2.selected = sopt.selected; // For some reason I need to do this to make NN4 happy
								m2.defaultSelected = sopt.defaultSelected;
								m2.DOLOption = m;
								selObj.options[l] = m2;
								selObj.options[l].selected = reselectForNN6; // Reselect this option for NN6 to be happy. Yuck.
							}
						}
					}
				}
				if (selObj.onchange==null) {
					// We only modify the onChange attribute if it's empty! Otherwise do it yourself in your source!
					selObj.onchange = new Function("dynamicOptionListObjects["+dol.index+"].change(this)");
				}
			}
		}
	}
	// Set the preselectd options on page load 
	resetDynamicOptionLists();
}

// This function populates lists with the preselected values. 
// It's pulled out into a separate function so it can be hooked into a 'reset' button on a form
// Optionally passed a form object which should be the only form reset
function resetDynamicOptionLists(theform) {
	// reset each DynamicOptionList object
	for (var i=0; i<dynamicOptionListObjects.length; i++) {
		var dol = dynamicOptionListObjects[i];
		if (typeof(theform)=="undefined" || theform==null || theform==dol.form) {
			for (var j=0; j<dol.fieldNames.length; j++) {
				dol.change(dol.form[dol.fieldNames[j][0]],true); // Second argument says to use preselected values rather than default values
			}
		}
	}
}

// An object to represent an Option() but just for data-holding
function DOLOption(text,value,defaultSelected,selected) {
	this.text = text;
	this.value = value;
	this.defaultSelected = defaultSelected;
	this.selected = selected;
	this.options = new Array(); // To hold sub-options
	return this;
}

// DynamicOptionList CONSTRUCTOR
function DynamicOptionList() {
	this.form = null;// The form this list belongs to
	this.options = new Array();// Holds the options of dependent lists
	this.longestString = new Array();// Longest string that is currently a potential option (for Netscape)
	this.numberOfOptions = new Array();// The total number of options that might be displayed, to build dummy options (for Netscape)
	this.currentNode = null;// The current node that has been selected with forValue() or forText()
	this.currentField = null;// The current field that is selected to be used for setValue()
	this.currentNodeDepth = 0;// How far down the tree the currentNode is
	this.fieldNames = new Array();// Lists of dependent fields which use this object
	this.formIndex = null;// The index of the form to associate with this list
	this.formName = null;// The name of the form to associate with this list
	this.fieldListIndexes = new Object();// Hold the field lists index where fields exist
	this.fieldIndexes = new Object();// Hold the index within the list where fields exist
	this.selectFirstOption = true;// Whether or not to select the first option by default if no options are default or preselected, otherwise set the selectedIndex = -1
	this.numberOfOptions = new Array();// Store the max number of options for a given option list
	this.longestString = new Array();// Store the longest possible string 
	this.values = new Object(); // Will hold the preselected values for fields, by field name
	
	// Method mappings
	this.forValue = DOL_forValue;
	this.forText = DOL_forText;
	this.forField = DOL_forField;
	this.forX = DOL_forX;
	this.addOptions = DOL_addOptions;
	this.addOptionsTextValue = DOL_addOptionsTextValue;
	this.setDefaultOptions = DOL_setDefaultOptions;
	this.setValues = DOL_setValues;
	this.setValue = DOL_setValues;
	this.setFormIndex = DOL_setFormIndex;
	this.setFormName = DOL_setFormName;
	this.printOptions = DOL_printOptions;
	this.addDependentFields = DOL_addDependentFields;
	this.change = DOL_change;
	this.child = DOL_child;
	this.selectChildOptions = DOL_selectChildOptions;
	this.populateChild = DOL_populateChild;
	this.change = DOL_change;
	this.addNewOptionToList = DOL_addNewOptionToList;
	this.findMatchingOptionInArray = DOL_findMatchingOptionInArray;

	// Optionally pass in the dependent field names
	if (arguments.length > 0) {
		// Process arguments and add dependency groups
		for (var i=0; i<arguments.length; i++) {
			this.fieldListIndexes[arguments[i].toString()] = this.fieldNames.length;
			this.fieldIndexes[arguments[i].toString()] = i;
		}
		this.fieldNames[this.fieldNames.length] = arguments;
	}
	
	// Add this object to the global array of dynamicoptionlist objects
	this.index = window.dynamicOptionListCount++;
	window["dynamicOptionListObjects"][this.index] = this;
}

// Given an array of Option objects, search for an existing option that matches value, text, or both
function DOL_findMatchingOptionInArray(a,text,value,exactMatchRequired) {
	if (a==null || typeof(a)=="undefined") { return null; }
	var value_match = null; // Whether or not a value has been matched
	var text_match = null; // Whether or not a text has been matched
	for (var i=0; i<a.length; i++) {
		var opt = a[i];
		// If both value and text match, return it right away
		if (opt.value==value && opt.text==text) { return opt; }
		if (!exactMatchRequired) {
			// If value matches, store it until we complete scanning the list
			if (value_match==null && value!=null && opt.value==value) {
				value_match = opt;
			}
			// If text matches, store it for later
			if (text_match==null && text!=null && opt.text==text) {
				text_match = opt;
			}
		}
	}
	return (value_match!=null)?value_match:text_match;
}

// Util function used by forValue and forText
function DOL_forX(s,type) {
	if (this.currentNode==null) { this.currentNodeDepth=0; }
	var useNode = (this.currentNode==null)?this:this.currentNode;
	var o = this.findMatchingOptionInArray(useNode["options"],(type=="text")?s:null,(type=="value")?s:null,false);
	if (o==null) {
		o = new DOLOption(null,null,false,false);
		o[type] = s;
		useNode.options[useNode.options.length] = o;
	}
	this.currentNode = o;
	this.currentNodeDepth++;
	return this;
}

// Set the portion of the list structure that is to be used by a later operation like addOptions
function DOL_forValue(s) { return this.forX(s,"value"); }

// Set the portion of the list structure that is to be used by a later operation like addOptions
function DOL_forText(s) { return this.forX(s,"text"); }

// Set the field to be used for setValue() calls
function DOL_forField(f) { this.currentField = f; return this; }

// Create and add an option to a list, avoiding duplicates
function DOL_addNewOptionToList(a, text, value, defaultSelected) {
	var o = new DOLOption(text,value,defaultSelected,false);
	// Add the option to the array
	if (a==null) { a = new Array(); }
	for (var i=0; i<a.length; i++) {
		if (a[i].text==o.text && a[i].value==o.value) {
			if (o.selected) { 
				a[i].selected=true;
			}
			if (o.defaultSelected) {
				a[i].defaultSelected = true;
			}
			return a;
		}
	}
	a[a.length] = o;
}

// Add sub-options to the currently-selected node, with the same text and value for each option
function DOL_addOptions() {
	if (this.currentNode==null) { this.currentNode = this; }
	if (this.currentNode["options"] == null) { this.currentNode["options"] = new Array(); }
	for (var i=0; i<arguments.length; i++) {
		var text = arguments[i];
		this.addNewOptionToList(this.currentNode.options,text,text,false);
		if (typeof(this.numberOfOptions[this.currentNodeDepth])=="undefined") {
			this.numberOfOptions[this.currentNodeDepth]=0;
		}
		if (this.currentNode.options.length > this.numberOfOptions[this.currentNodeDepth]) {
			this.numberOfOptions[this.currentNodeDepth] = this.currentNode.options.length;
		}
		if (typeof(this.longestString[this.currentNodeDepth])=="undefined" || (text.length > this.longestString[this.currentNodeDepth].length)) {
			this.longestString[this.currentNodeDepth] = text;
		}
	}
	this.currentNode = null;
	this.currentNodeDepth = 0;
}

// Add sub-options to the currently-selected node, specifying separate text and values for each option
function DOL_addOptionsTextValue() {
	if (this.currentNode==null) { this.currentNode = this; }
	if (this.currentNode["options"] == null) { this.currentNode["options"] = new Array(); }
	for (var i=0; i<arguments.length; i++) {
		var text = arguments[i++];
		var value = arguments[i];
		this.addNewOptionToList(this.currentNode.options,text,value,false);
		if (typeof(this.numberOfOptions[this.currentNodeDepth])=="undefined") {
			this.numberOfOptions[this.currentNodeDepth]=0;
		}
		if (this.currentNode.options.length > this.numberOfOptions[this.currentNodeDepth]) {
			this.numberOfOptions[this.currentNodeDepth] = this.currentNode.options.length;
		}
		if (typeof(this.longestString[this.currentNodeDepth])=="undefined" || (text.length > this.longestString[this.currentNodeDepth].length)) {
			this.longestString[this.currentNodeDepth] = text;
		}
	}
	this.currentNode = null;
	this.currentNodeDepth = 0;
}

// Find the first dependent list of a select box
// If it's the last list in a chain, return null because there are no children
function DOL_child(obj) {
	var listIndex = this.fieldListIndexes[obj.name];
	var index = this.fieldIndexes[obj.name];
	if (index < (this.fieldNames[listIndex].length-1)) {
		return this.form[this.fieldNames[listIndex][index+1]];
	}
	return null;
}

// Set the options which should be selected by default for a certain value in the parent
function DOL_setDefaultOptions() {
	if (this.currentNode==null) { this.currentNode = this; }
	for (var i=0; i<arguments.length; i++) {
		var o = this.findMatchingOptionInArray(this.currentNode.options,null,arguments[i],false);
		if (o!=null) {
			o.defaultSelected = true;
		}
	}
	this.currentNode = null;
}

// Set the options which should be selected when the page loads. This is different than the default value and ONLY applies when the page LOADS
function DOL_setValues() {
	if (this.currentField==null) { 
		alert("Can't call setValues() without using forField() first!");
		return;
	}
	if (typeof(this.values[this.currentField])=="undefined") {
		this.values[this.currentField] = new Object();
	}
	for (var i=0; i<arguments.length; i++) {
		this.values[this.currentField][arguments[i]] = true;
	}
	this.currentField = null;
}

// Manually set the form for the object using an index
function DOL_setFormIndex(i) {
	this.formIndex = i;
}

// Manually set the form for the object using a form name
function DOL_setFormName(n) {
	this.formName = n;
}

// Print blank <option> objects for Netscape4, since it refuses to grow or shrink select boxes for new options
function DOL_printOptions(name) {
	// Only need to write out "dummy" options for Netscape4
    if ((navigator.appName == 'Netscape') && (parseInt(navigator.appVersion) <= 4)){
		var index = this.fieldIndexes[name];
		var ret = "";
		if (typeof(this.numberOfOptions[index])!="undefined") {
			for (var i=0; i<this.numberOfOptions[index]; i++) { 
				ret += "<OPTION>";
			}
		}
		ret += "<OPTION>";
		if (typeof(this.longestString[index])!="undefined") {
			for (var i=0; i<this.longestString[index].length; i++) {
				ret += "_";
			}
		}
		document.writeln(ret);
	}
}

// Add a list of field names which use this option-mapping object.
// A single mapping object may be used by multiple sets of fields
function DOL_addDependentFields() {
	for (var i=0; i<arguments.length; i++) {
		this.fieldListIndexes[arguments[i].toString()] = this.fieldNames.length;
		this.fieldIndexes[arguments[i].toString()] = i;
	}
	this.fieldNames[this.fieldNames.length] = arguments;
}

// Called when a parent select box is changed. It populates its direct child, then calls change on the child object to continue the population.
function DOL_change(obj, usePreselected) {
	if (usePreselected==null || typeof(usePreselected)=="undefined") { usePreselected = false; }
	var changedListIndex = this.fieldListIndexes[obj.name];
	var changedIndex = this.fieldIndexes[obj.name];
	var child = this.child(obj);
	if (child == null) { return; } // No child, no need to continue
	if (obj.type == "select-one") {
		// Treat single-select differently so we don't have to scan the entire select list, which could potentially speed things up
		if (child.options!=null) {
			child.options.length=0; // Erase all the options from the child so we can re-populate
		}
		if (obj.options!=null && obj.options.length>0 && obj.selectedIndex>=0) {
			var o = obj.options[obj.selectedIndex];
			this.populateChild(o.DOLOption,child,usePreselected);
			this.selectChildOptions(child,usePreselected);
		}
	}
	else if (obj.type == "select-multiple") {
		// For each selected value in the parent, find the options to fill in for this list
		// Loop through the child list and keep track of options that are currently selected
		var currentlySelectedOptions = new Array();
		if (!usePreselected) {
			for (var i=0; i<child.options.length; i++) {
				var co = child.options[i];
				if (co.selected) {
					this.addNewOptionToList(currentlySelectedOptions, co.text, co.value, co.defaultSelected);
				}
			}
		}
		child.options.length=0;
		if (obj.options!=null) {
			var obj_o = obj.options;
			// For each selected option in the parent...
			for (var i=0; i<obj_o.length; i++) {
				if (obj_o[i].selected) {
					// if option is selected, add its children to the list
 					this.populateChild(obj_o[i].DOLOption,child,usePreselected);
				}
			}
			// Now go through and re-select any options which were selected before
			var atLeastOneSelected = false;
			if (!usePreselected) {
				for (var i=0; i<child.options.length; i++) {
					var m = this.findMatchingOptionInArray(currentlySelectedOptions,child.options[i].text,child.options[i].value,true);
					if (m!=null) {
						child.options[i].selected = true;
						atLeastOneSelected = true;
					}
				}
			}
			if (!atLeastOneSelected) {	
				this.selectChildOptions(child,usePreselected);
			}
		}
	}
	// Change all the way down the chain
	this.change(child,usePreselected);
}
function DOL_populateChild(dolOption,childSelectObj,usePreselected) {
	// If this opton has sub-options, populate the child list with them
	if (dolOption!=null && dolOption.options!=null) {
		for (var j=0; j<dolOption.options.length; j++) {
			var srcOpt = dolOption.options[j];
			if (childSelectObj.options==null) { childSelectObj.options = new Array(); }
			// Put option into select list
			var duplicate = false;
			var preSelectedExists = false;
			for (var k=0; k<childSelectObj.options.length; k++) {
				var csi = childSelectObj.options[k];
				if (csi.text==srcOpt.text && csi.value==srcOpt.value) {
					duplicate = true;
					break;
				}
			}
			if (!duplicate) {
				var newopt = new Option(srcOpt.text, srcOpt.value, false, false);
				newopt.selected = false; // Again, we have to do these two statements for NN4 to work
				newopt.defaultSelected = false;
				newopt.DOLOption = srcOpt;
				childSelectObj.options[childSelectObj.options.length] = newopt;
			}
		}
	}
}

// Once a child select is populated, go back over it to select options which should be selected
function DOL_selectChildOptions(obj,usePreselected) {
	// Look to see if any options are preselected=true. If so, then set then selected if usePreselected=true, otherwise set defaults
	var values = this.values[obj.name];
	var preselectedExists = false;
	if (usePreselected && values!=null && typeof(values)!="undefined") {
		for (var i=0; i<obj.options.length; i++) {
			var v = obj.options[i].value;
			if (v!=null && values[v]!=null && typeof(values[v])!="undefined") {
				preselectedExists = true;
				break;
			}
		}
	}
	// Go back over all the options to do the selection
	var atLeastOneSelected = false;
	for (var i=0; i<obj.options.length; i++) {
		var o = obj.options[i];
		//if(o.value == 'pages'){alert(preselectedExists)}
		if (preselectedExists && o.value!=null && values[o.value]!=null && typeof(values[o.value])!="undefined") {
			o.selected = true;
			atLeastOneSelected = true;
		}
		else if (!preselectedExists && o.DOLOption!=null && o.DOLOption.defaultSelected) {
			o.selected = true;
			atLeastOneSelected = true;
		}
		else {
			//o.selected = false;
		}
	}
	// If nothing else was selected, select the first one by default
	if (this.selectFirstOption && !atLeastOneSelected && obj.options.length>0) {
		obj.options[0].selected = true;
	}
	else if (!atLeastOneSelected &&  obj.type=="select-one") {
		obj.selectedIndex = -1;
	}
}
/**
 * jscolor - JavaScript Color Picker
 *
 * @link    http://jscolor.com
 * @license For open source use: GPLv3
 *          For commercial use: JSColor Commercial License
 * @author  Jan Odvarko
 *
 * See usage examples at http://jscolor.com/examples/
 */"use strict";window.jscolor||(window.jscolor=function(){var e={register:function(){e.attachDOMReadyEvent(e.init),e.attachEvent(document,"mousedown",e.onDocumentMouseDown),e.attachEvent(document,"touchstart",e.onDocumentTouchStart),e.attachEvent(window,"resize",e.onWindowResize)},init:function(){e.jscolor.lookupClass&&e.jscolor.installByClassName(e.jscolor.lookupClass)},tryInstallOnElements:function(t,n){var r=new RegExp("(^|\\s)("+n+")(\\s*(\\{[^}]*\\})|\\s|$)","i");for(var i=0;i<t.length;i+=1){if(t[i].type!==undefined&&t[i].type.toLowerCase()=="color"&&e.isColorAttrSupported)continue;var s;if(!t[i].jscolor&&t[i].className&&(s=t[i].className.match(r))){var o=t[i],u=null,a=e.getDataAttr(o,"jscolor");a!==null?u=a:s[4]&&(u=s[4]);var f={};if(u)try{f=(new Function("return ("+u+")"))()}catch(l){e.warn("Error parsing jscolor options: "+l+":\n"+u)}o.jscolor=new e.jscolor(o,f)}}},isColorAttrSupported:function(){var e=document.createElement("input");if(e.setAttribute){e.setAttribute("type","color");if(e.type.toLowerCase()=="color")return!0}return!1}(),isCanvasSupported:function(){var e=document.createElement("canvas");return!!e.getContext&&!!e.getContext("2d")}(),fetchElement:function(e){return typeof e=="string"?document.getElementById(e):e},isElementType:function(e,t){return e.nodeName.toLowerCase()===t.toLowerCase()},getDataAttr:function(e,t){var n="data-"+t,r=e.getAttribute(n);return r!==null?r:null},attachEvent:function(e,t,n){e.addEventListener?e.addEventListener(t,n,!1):e.attachEvent&&e.attachEvent("on"+t,n)},detachEvent:function(e,t,n){e.removeEventListener?e.removeEventListener(t,n,!1):e.detachEvent&&e.detachEvent("on"+t,n)},_attachedGroupEvents:{},attachGroupEvent:function(t,n,r,i){e._attachedGroupEvents.hasOwnProperty(t)||(e._attachedGroupEvents[t]=[]),e._attachedGroupEvents[t].push([n,r,i]),e.attachEvent(n,r,i)},detachGroupEvents:function(t){if(e._attachedGroupEvents.hasOwnProperty(t)){for(var n=0;n<e._attachedGroupEvents[t].length;n+=1){var r=e._attachedGroupEvents[t][n];e.detachEvent(r[0],r[1],r[2])}delete e._attachedGroupEvents[t]}},attachDOMReadyEvent:function(e){var t=!1,n=function(){t||(t=!0,e())};if(document.readyState==="complete"){setTimeout(n,1);return}if(document.addEventListener)document.addEventListener("DOMContentLoaded",n,!1),window.addEventListener("load",n,!1);else if(document.attachEvent){document.attachEvent("onreadystatechange",function(){document.readyState==="complete"&&(document.detachEvent("onreadystatechange",arguments.callee),n())}),window.attachEvent("onload",n);if(document.documentElement.doScroll&&window==window.top){var r=function(){if(!document.body)return;try{document.documentElement.doScroll("left"),n()}catch(e){setTimeout(r,1)}};r()}}},warn:function(e){window.console&&window.console.warn&&window.console.warn(e)},preventDefault:function(e){e.preventDefault&&e.preventDefault(),e.returnValue=!1},captureTarget:function(t){t.setCapture&&(e._capturedTarget=t,e._capturedTarget.setCapture())},releaseTarget:function(){e._capturedTarget&&(e._capturedTarget.releaseCapture(),e._capturedTarget=null)},fireEvent:function(e,t){if(!e)return;if(document.createEvent){var n=document.createEvent("HTMLEvents");n.initEvent(t,!0,!0),e.dispatchEvent(n)}else if(document.createEventObject){var n=document.createEventObject();e.fireEvent("on"+t,n)}else e["on"+t]&&e["on"+t]()},classNameToList:function(e){return e.replace(/^\s+|\s+$/g,"").split(/\s+/)},hasClass:function(e,t){return t?-1!=(" "+e.className.replace(/\s+/g," ")+" ").indexOf(" "+t+" "):!1},setClass:function(t,n){var r=e.classNameToList(n);for(var i=0;i<r.length;i+=1)e.hasClass(t,r[i])||(t.className+=(t.className?" ":"")+r[i])},unsetClass:function(t,n){var r=e.classNameToList(n);for(var i=0;i<r.length;i+=1){var s=new RegExp("^\\s*"+r[i]+"\\s*|"+"\\s*"+r[i]+"\\s*$|"+"\\s+"+r[i]+"(\\s+)","g");t.className=t.className.replace(s,"$1")}},getStyle:function(e){return window.getComputedStyle?window.getComputedStyle(e):e.currentStyle},setStyle:function(){var e=document.createElement("div"),t=function(t){for(var n=0;n<t.length;n+=1)if(t[n]in e.style)return t[n]},n={borderRadius:t(["borderRadius","MozBorderRadius","webkitBorderRadius"]),boxShadow:t(["boxShadow","MozBoxShadow","webkitBoxShadow"])};return function(e,t,r){switch(t.toLowerCase()){case"opacity":var i=Math.round(parseFloat(r)*100);e.style.opacity=r,e.style.filter="alpha(opacity="+i+")";break;default:e.style[n[t]]=r}}}(),setBorderRadius:function(t,n){e.setStyle(t,"borderRadius",n||"0")},setBoxShadow:function(t,n){e.setStyle(t,"boxShadow",n||"none")},getElementPos:function(t,n){var r=0,i=0,s=t.getBoundingClientRect();r=s.left,i=s.top;if(!n){var o=e.getViewPos();r+=o[0],i+=o[1]}return[r,i]},getElementSize:function(e){return[e.offsetWidth,e.offsetHeight]},getAbsPointerPos:function(e){e||(e=window.event);var t=0,n=0;return typeof e.changedTouches!="undefined"&&e.changedTouches.length?(t=e.changedTouches[0].clientX,n=e.changedTouches[0].clientY):typeof e.clientX=="number"&&(t=e.clientX,n=e.clientY),{x:t,y:n}},getRelPointerPos:function(e){e||(e=window.event);var t=e.target||e.srcElement,n=t.getBoundingClientRect(),r=0,i=0,s=0,o=0;return typeof e.changedTouches!="undefined"&&e.changedTouches.length?(s=e.changedTouches[0].clientX,o=e.changedTouches[0].clientY):typeof e.clientX=="number"&&(s=e.clientX,o=e.clientY),r=s-n.left,i=o-n.top,{x:r,y:i}},getViewPos:function(){var e=document.documentElement;return[(window.pageXOffset||e.scrollLeft)-(e.clientLeft||0),(window.pageYOffset||e.scrollTop)-(e.clientTop||0)]},getViewSize:function(){var e=document.documentElement;return[window.innerWidth||e.clientWidth,window.innerHeight||e.clientHeight]},redrawPosition:function(){if(e.picker&&e.picker.owner){var t=e.picker.owner,n,r;t.fixed?(n=e.getElementPos(t.targetElement,!0),r=[0,0]):(n=e.getElementPos(t.targetElement),r=e.getViewPos());var i=e.getElementSize(t.targetElement),s=e.getViewSize(),o=e.getPickerOuterDims(t),u,a,f;switch(t.position.toLowerCase()){case"left":u=1,a=0,f=-1;break;case"right":u=1,a=0,f=1;break;case"top":u=0,a=1,f=-1;break;default:u=0,a=1,f=1}var l=(i[a]+o[a])/2;if(!t.smartPosition)var c=[n[u],n[a]+i[a]-l+l*f];else var c=[-r[u]+n[u]+o[u]>s[u]?-r[u]+n[u]+i[u]/2>s[u]/2&&n[u]+i[u]-o[u]>=0?n[u]+i[u]-o[u]:n[u]:n[u],-r[a]+n[a]+i[a]+o[a]-l+l*f>s[a]?-r[a]+n[a]+i[a]/2>s[a]/2&&n[a]+i[a]-l-l*f>=0?n[a]+i[a]-l-l*f:n[a]+i[a]-l+l*f:n[a]+i[a]-l+l*f>=0?n[a]+i[a]-l+l*f:n[a]+i[a]-l-l*f];var h=c[u],p=c[a],d=t.fixed?"fixed":"absolute",v=(c[0]+o[0]>n[0]||c[0]<n[0]+i[0])&&c[1]+o[1]<n[1]+i[1];e._drawPosition(t,h,p,d,v)}},_drawPosition:function(t,n,r,i,s){var o=s?0:t.shadowBlur;e.picker.wrap.style.position=i,e.picker.wrap.style.left=n+"px",e.picker.wrap.style.top=r+"px",e.setBoxShadow(e.picker.boxS,t.shadow?new e.BoxShadow(0,o,t.shadowBlur,0,t.shadowColor):null)},getPickerDims:function(t){var n=!!e.getSliderComponent(t),r=[2*t.insetWidth+2*t.padding+t.width+(n?2*t.insetWidth+e.getPadToSliderPadding(t)+t.sliderSize:0),2*t.insetWidth+2*t.padding+t.height+(t.closable?2*t.insetWidth+t.padding+t.buttonHeight:0)];return r},getPickerOuterDims:function(t){var n=e.getPickerDims(t);return[n[0]+2*t.borderWidth,n[1]+2*t.borderWidth]},getPadToSliderPadding:function(e){return Math.max(e.padding,1.5*(2*e.pointerBorderWidth+e.pointerThickness))},getPadYComponent:function(e){switch(e.mode.charAt(1).toLowerCase()){case"v":return"v"}return"s"},getSliderComponent:function(e){if(e.mode.length>2)switch(e.mode.charAt(2).toLowerCase()){case"s":return"s";case"v":return"v"}return null},onDocumentMouseDown:function(t){t||(t=window.event);var n=t.target||t.srcElement;n._jscLinkedInstance?n._jscLinkedInstance.showOnClick&&n._jscLinkedInstance.show():n._jscControlName?e.onControlPointerStart(t,n,n._jscControlName,"mouse"):e.picker&&e.picker.owner&&e.picker.owner.hide()},onDocumentTouchStart:function(t){t||(t=window.event);var n=t.target||t.srcElement;n._jscLinkedInstance?n._jscLinkedInstance.showOnClick&&n._jscLinkedInstance.show():n._jscControlName?e.onControlPointerStart(t,n,n._jscControlName,"touch"):e.picker&&e.picker.owner&&e.picker.owner.hide()},onWindowResize:function(t){e.redrawPosition()},onParentScroll:function(t){e.picker&&e.picker.owner&&e.picker.owner.hide()},_pointerMoveEvent:{mouse:"mousemove",touch:"touchmove"},_pointerEndEvent:{mouse:"mouseup",touch:"touchend"},_pointerOrigin:null,_capturedTarget:null,onControlPointerStart:function(t,n,r,i){var s=n._jscInstance;e.preventDefault(t),e.captureTarget(n);var o=function(s,o){e.attachGroupEvent("drag",s,e._pointerMoveEvent[i],e.onDocumentPointerMove(t,n,r,i,o)),e.attachGroupEvent("drag",s,e._pointerEndEvent[i],e.onDocumentPointerEnd(t,n,r,i))};o(document,[0,0]);if(window.parent&&window.frameElement){var u=window.frameElement.getBoundingClientRect(),a=[-u.left,-u.top];o(window.parent.window.document,a)}var f=e.getAbsPointerPos(t),l=e.getRelPointerPos(t);e._pointerOrigin={x:f.x-l.x,y:f.y-l.y};switch(r){case"pad":switch(e.getSliderComponent(s)){case"s":s.hsv[1]===0&&s.fromHSV(null,100,null);break;case"v":s.hsv[2]===0&&s.fromHSV(null,null,100)}e.setPad(s,t,0,0);break;case"sld":e.setSld(s,t,0)}e.dispatchFineChange(s)},onDocumentPointerMove:function(t,n,r,i,s){return function(t){var i=n._jscInstance;switch(r){case"pad":t||(t=window.event),e.setPad(i,t,s[0],s[1]),e.dispatchFineChange(i);break;case"sld":t||(t=window.event),e.setSld(i,t,s[1]),e.dispatchFineChange(i)}}},onDocumentPointerEnd:function(t,n,r,i){return function(t){var r=n._jscInstance;e.detachGroupEvents("drag"),e.releaseTarget(),e.dispatchChange(r)}},dispatchChange:function(t){t.valueElement&&e.isElementType(t.valueElement,"input")&&e.fireEvent(t.valueElement,"change")},dispatchFineChange:function(e){if(e.onFineChange){var t;typeof e.onFineChange=="string"?t=new Function(e.onFineChange):t=e.onFineChange,t.call(e)}},setPad:function(t,n,r,i){var s=e.getAbsPointerPos(n),o=r+s.x-e._pointerOrigin.x-t.padding-t.insetWidth,u=i+s.y-e._pointerOrigin.y-t.padding-t.insetWidth,a=o*(360/(t.width-1)),f=100-u*(100/(t.height-1));switch(e.getPadYComponent(t)){case"s":t.fromHSV(a,f,null,e.leaveSld);break;case"v":t.fromHSV(a,null,f,e.leaveSld)}},setSld:function(t,n,r){var i=e.getAbsPointerPos(n),s=r+i.y-e._pointerOrigin.y-t.padding-t.insetWidth,o=100-s*(100/(t.height-1));switch(e.getSliderComponent(t)){case"s":t.fromHSV(null,o,null,e.leavePad);break;case"v":t.fromHSV(null,null,o,e.leavePad)}},_vmlNS:"jsc_vml_",_vmlCSS:"jsc_vml_css_",_vmlReady:!1,initVML:function(){if(!e._vmlReady){var t=document;t.namespaces[e._vmlNS]||t.namespaces.add(e._vmlNS,"urn:schemas-microsoft-com:vml");if(!t.styleSheets[e._vmlCSS]){var n=["shape","shapetype","group","background","path","formulas","handles","fill","stroke","shadow","textbox","textpath","imagedata","line","polyline","curve","rect","roundrect","oval","arc","image"],r=t.createStyleSheet();r.owningElement.id=e._vmlCSS;for(var i=0;i<n.length;i+=1)r.addRule(e._vmlNS+"\\:"+n[i],"behavior:url(#default#VML);")}e._vmlReady=!0}},createPalette:function(){var t={elm:null,draw:null};if(e.isCanvasSupported){var n=document.createElement("canvas"),r=n.getContext("2d"),i=function(e,t,i){n.width=e,n.height=t,r.clearRect(0,0,n.width,n.height);var s=r.createLinearGradient(0,0,n.width,0);s.addColorStop(0,"#F00"),s.addColorStop(1/6,"#FF0"),s.addColorStop(2/6,"#0F0"),s.addColorStop(.5,"#0FF"),s.addColorStop(4/6,"#00F"),s.addColorStop(5/6,"#F0F"),s.addColorStop(1,"#F00"),r.fillStyle=s,r.fillRect(0,0,n.width,n.height);var o=r.createLinearGradient(0,0,0,n.height);switch(i.toLowerCase()){case"s":o.addColorStop(0,"rgba(255,255,255,0)"),o.addColorStop(1,"rgba(255,255,255,1)");break;case"v":o.addColorStop(0,"rgba(0,0,0,0)"),o.addColorStop(1,"rgba(0,0,0,1)")}r.fillStyle=o,r.fillRect(0,0,n.width,n.height)};t.elm=n,t.draw=i}else{e.initVML();var s=document.createElement("div");s.style.position="relative",s.style.overflow="hidden";var o=document.createElement(e._vmlNS+":fill");o.type="gradient",o.method="linear",o.angle="90",o.colors="16.67% #F0F, 33.33% #00F, 50% #0FF, 66.67% #0F0, 83.33% #FF0";var u=document.createElement(e._vmlNS+":rect");u.style.position="absolute",u.style.left="-1px",u.style.top="-1px",u.stroked=!1,u.appendChild(o),s.appendChild(u);var a=document.createElement(e._vmlNS+":fill");a.type="gradient",a.method="linear",a.angle="180",a.opacity="0";var f=document.createElement(e._vmlNS+":rect");f.style.position="absolute",f.style.left="-1px",f.style.top="-1px",f.stroked=!1,f.appendChild(a),s.appendChild(f);var i=function(e,t,n){s.style.width=e+"px",s.style.height=t+"px",u.style.width=f.style.width=e+1+"px",u.style.height=f.style.height=t+1+"px",o.color="#F00",o.color2="#F00";switch(n.toLowerCase()){case"s":a.color=a.color2="#FFF";break;case"v":a.color=a.color2="#000"}};t.elm=s,t.draw=i}return t},createSliderGradient:function(){var t={elm:null,draw:null};if(e.isCanvasSupported){var n=document.createElement("canvas"),r=n.getContext("2d"),i=function(e,t,i,s){n.width=e,n.height=t,r.clearRect(0,0,n.width,n.height);var o=r.createLinearGradient(0,0,0,n.height);o.addColorStop(0,i),o.addColorStop(1,s),r.fillStyle=o,r.fillRect(0,0,n.width,n.height)};t.elm=n,t.draw=i}else{e.initVML();var s=document.createElement("div");s.style.position="relative",s.style.overflow="hidden";var o=document.createElement(e._vmlNS+":fill");o.type="gradient",o.method="linear",o.angle="180";var u=document.createElement(e._vmlNS+":rect");u.style.position="absolute",u.style.left="-1px",u.style.top="-1px",u.stroked=!1,u.appendChild(o),s.appendChild(u);var i=function(e,t,n,r){s.style.width=e+"px",s.style.height=t+"px",u.style.width=e+1+"px",u.style.height=t+1+"px",o.color=n,o.color2=r};t.elm=s,t.draw=i}return t},leaveValue:1,leaveStyle:2,leavePad:4,leaveSld:8,BoxShadow:function(){var e=function(e,t,n,r,i,s){this.hShadow=e,this.vShadow=t,this.blur=n,this.spread=r,this.color=i,this.inset=!!s};return e.prototype.toString=function(){var e=[Math.round(this.hShadow)+"px",Math.round(this.vShadow)+"px",Math.round(this.blur)+"px",Math.round(this.spread)+"px",this.color];return this.inset&&e.push("inset"),e.join(" ")},e}(),jscolor:function(t,n){function i(e,t,n){e/=255,t/=255,n/=255;var r=Math.min(Math.min(e,t),n),i=Math.max(Math.max(e,t),n),s=i-r;if(s===0)return[null,0,100*i];var o=e===r?3+(n-t)/s:t===r?5+(e-n)/s:1+(t-e)/s;return[60*(o===6?0:o),100*(s/i),100*i]}function s(e,t,n){var r=255*(n/100);if(e===null)return[r,r,r];e/=60,t/=100;var i=Math.floor(e),s=i%2?e-i:1-(e-i),o=r*(1-t),u=r*(1-t*s);switch(i){case 6:case 0:return[r,u,o];case 1:return[u,r,o];case 2:return[o,r,u];case 3:return[o,u,r];case 4:return[u,o,r];case 5:return[r,o,u]}}function o(){e.unsetClass(d.targetElement,d.activeClass),e.picker.wrap.parentNode.removeChild(e.picker.wrap),delete e.picker.owner}function u(){function l(){var e=d.insetColor.split(/\s+/),n=e.length<2?e[0]:e[1]+" "+e[0]+" "+e[0]+" "+e[1];t.btn.style.borderColor=n}d._processParentElementsInDOM(),e.picker||(e.picker={owner:null,wrap:document.createElement("div"),box:document.createElement("div"),boxS:document.createElement("div"),boxB:document.createElement("div"),pad:document.createElement("div"),padB:document.createElement("div"),padM:document.createElement("div"),padPal:e.createPalette(),cross:document.createElement("div"),crossBY:document.createElement("div"),crossBX:document.createElement("div"),crossLY:document.createElement("div"),crossLX:document.createElement("div"),sld:document.createElement("div"),sldB:document.createElement("div"),sldM:document.createElement("div"),sldGrad:e.createSliderGradient(),sldPtrS:document.createElement("div"),sldPtrIB:document.createElement("div"),sldPtrMB:document.createElement("div"),sldPtrOB:document.createElement("div"),btn:document.createElement("div"),btnT:document.createElement("span")},e.picker.pad.appendChild(e.picker.padPal.elm),e.picker.padB.appendChild(e.picker.pad),e.picker.cross.appendChild(e.picker.crossBY),e.picker.cross.appendChild(e.picker.crossBX),e.picker.cross.appendChild(e.picker.crossLY),e.picker.cross.appendChild(e.picker.crossLX),e.picker.padB.appendChild(e.picker.cross),e.picker.box.appendChild(e.picker.padB),e.picker.box.appendChild(e.picker.padM),e.picker.sld.appendChild(e.picker.sldGrad.elm),e.picker.sldB.appendChild(e.picker.sld),e.picker.sldB.appendChild(e.picker.sldPtrOB),e.picker.sldPtrOB.appendChild(e.picker.sldPtrMB),e.picker.sldPtrMB.appendChild(e.picker.sldPtrIB),e.picker.sldPtrIB.appendChild(e.picker.sldPtrS),e.picker.box.appendChild(e.picker.sldB),e.picker.box.appendChild(e.picker.sldM),e.picker.btn.appendChild(e.picker.btnT),e.picker.box.appendChild(e.picker.btn),e.picker.boxB.appendChild(e.picker.box),e.picker.wrap.appendChild(e.picker.boxS),e.picker.wrap.appendChild(e.picker.boxB));var t=e.picker,n=!!e.getSliderComponent(d),r=e.getPickerDims(d),i=2*d.pointerBorderWidth+d.pointerThickness+2*d.crossSize,s=e.getPadToSliderPadding(d),o=Math.min(d.borderRadius,Math.round(d.padding*Math.PI)),u="crosshair";t.wrap.style.clear="both",t.wrap.style.width=r[0]+2*d.borderWidth+"px",t.wrap.style.height=r[1]+2*d.borderWidth+"px",t.wrap.style.zIndex=d.zIndex,t.box.style.width=r[0]+"px",t.box.style.height=r[1]+"px",t.boxS.style.position="absolute",t.boxS.style.left="0",t.boxS.style.top="0",t.boxS.style.width="100%",t.boxS.style.height="100%",e.setBorderRadius(t.boxS,o+"px"),t.boxB.style.position="relative",t.boxB.style.border=d.borderWidth+"px solid",t.boxB.style.borderColor=d.borderColor,t.boxB.style.background=d.backgroundColor,e.setBorderRadius(t.boxB,o+"px"),t.padM.style.background=t.sldM.style.background="#FFF",e.setStyle(t.padM,"opacity","0"),e.setStyle(t.sldM,"opacity","0"),t.pad.style.position="relative",t.pad.style.width=d.width+"px",t.pad.style.height=d.height+"px",t.padPal.draw(d.width,d.height,e.getPadYComponent(d)),t.padB.style.position="absolute",t.padB.style.left=d.padding+"px",t.padB.style.top=d.padding+"px",t.padB.style.border=d.insetWidth+"px solid",t.padB.style.borderColor=d.insetColor,t.padM._jscInstance=d,t.padM._jscControlName="pad",t.padM.style.position="absolute",t.padM.style.left="0",t.padM.style.top="0",t.padM.style.width=d.padding+2*d.insetWidth+d.width+s/2+"px",t.padM.style.height=r[1]+"px",t.padM.style.cursor=u,t.cross.style.position="absolute",t.cross.style.left=t.cross.style.top="0",t.cross.style.width=t.cross.style.height=i+"px",t.crossBY.style.position=t.crossBX.style.position="absolute",t.crossBY.style.background=t.crossBX.style.background=d.pointerBorderColor,t.crossBY.style.width=t.crossBX.style.height=2*d.pointerBorderWidth+d.pointerThickness+"px",t.crossBY.style.height=t.crossBX.style.width=i+"px",t.crossBY.style.left=t.crossBX.style.top=Math.floor(i/2)-Math.floor(d.pointerThickness/2)-d.pointerBorderWidth+"px",t.crossBY.style.top=t.crossBX.style.left="0",t.crossLY.style.position=t.crossLX.style.position="absolute",t.crossLY.style.background=t.crossLX.style.background=d.pointerColor,t.crossLY.style.height=t.crossLX.style.width=i-2*d.pointerBorderWidth+"px",t.crossLY.style.width=t.crossLX.style.height=d.pointerThickness+"px",t.crossLY.style.left=t.crossLX.style.top=Math.floor(i/2)-Math.floor(d.pointerThickness/2)+"px",t.crossLY.style.top=t.crossLX.style.left=d.pointerBorderWidth+"px",t.sld.style.overflow="hidden",t.sld.style.width=d.sliderSize+"px",t.sld.style.height=d.height+"px",t.sldGrad.draw(d.sliderSize,d.height,"#000","#000"),t.sldB.style.display=n?"block":"none",t.sldB.style.position="absolute",t.sldB.style.right=d.padding+"px",t.sldB.style.top=d.padding+"px",t.sldB.style.border=d.insetWidth+"px solid",t.sldB.style.borderColor=d.insetColor,t.sldM._jscInstance=d,t.sldM._jscControlName="sld",t.sldM.style.display=n?"block":"none",t.sldM.style.position="absolute",t.sldM.style.right="0",t.sldM.style.top="0",t.sldM.style.width=d.sliderSize+s/2+d.padding+2*d.insetWidth+"px",t.sldM.style.height=r[1]+"px",t.sldM.style.cursor="default",t.sldPtrIB.style.border=t.sldPtrOB.style.border=d.pointerBorderWidth+"px solid "+d.pointerBorderColor,t.sldPtrOB.style.position="absolute",t.sldPtrOB.style.left=-(2*d.pointerBorderWidth+d.pointerThickness)+"px",t.sldPtrOB.style.top="0",t.sldPtrMB.style.border=d.pointerThickness+"px solid "+d.pointerColor,t.sldPtrS.style.width=d.sliderSize+"px",t.sldPtrS.style.height=m+"px",t.btn.style.display=d.closable?"block":"none",t.btn.style.position="absolute",t.btn.style.left=d.padding+"px",t.btn.style.bottom=d.padding+"px",t.btn.style.padding="0 15px",t.btn.style.height=d.buttonHeight+"px",t.btn.style.border=d.insetWidth+"px solid",l(),t.btn.style.color=d.buttonColor,t.btn.style.font="12px sans-serif",t.btn.style.textAlign="center";try{t.btn.style.cursor="pointer"}catch(c){t.btn.style.cursor="hand"}t.btn.onmousedown=function(){d.hide()},t.btnT.style.lineHeight=d.buttonHeight+"px",t.btnT.innerHTML="",t.btnT.appendChild(document.createTextNode(d.closeText)),a(),f(),e.picker.owner&&e.picker.owner!==d&&e.unsetClass(e.picker.owner.targetElement,d.activeClass),e.picker.owner=d,e.isElementType(v,"body")?e.redrawPosition():e._drawPosition(d,0,0,"relative",!1),t.wrap.parentNode!=v&&v.appendChild(t.wrap),e.setClass(d.targetElement,d.activeClass)}function a(){switch(e.getPadYComponent(d)){case"s":var t=1;break;case"v":var t=2}var n=Math.round(d.hsv[0]/360*(d.width-1)),r=Math.round((1-d.hsv[t]/100)*(d.height-1)),i=2*d.pointerBorderWidth+d.pointerThickness+2*d.crossSize,o=-Math.floor(i/2);e.picker.cross.style.left=n+o+"px",e.picker.cross.style.top=r+o+"px";switch(e.getSliderComponent(d)){case"s":var u=s(d.hsv[0],100,d.hsv[2]),a=s(d.hsv[0],0,d.hsv[2]),f="rgb("+Math.round(u[0])+","+Math.round(u[1])+","+Math.round(u[2])+")",l="rgb("+Math.round(a[0])+","+Math.round(a[1])+","+Math.round(a[2])+")";e.picker.sldGrad.draw(d.sliderSize,d.height,f,l);break;case"v":var c=s(d.hsv[0],d.hsv[1],100),f="rgb("+Math.round(c[0])+","+Math.round(c[1])+","+Math.round(c[2])+")",l="#000";e.picker.sldGrad.draw(d.sliderSize,d.height,f,l)}}function f(){var t=e.getSliderComponent(d);if(t){switch(t){case"s":var n=1;break;case"v":var n=2}var r=Math.round((1-d.hsv[n]/100)*(d.height-1));e.picker.sldPtrOB.style.top=r-(2*d.pointerBorderWidth+d.pointerThickness)-Math.floor(m/2)+"px"}}function l(){return e.picker&&e.picker.owner===d}function c(){d.importColor()}this.value=null,this.valueElement=t,this.styleElement=t,this.required=!0,this.refine=!0,this.hash=!1,this.uppercase=!0,this.onFineChange=null,this.activeClass="jscolor-active",this.minS=0,this.maxS=100,this.minV=0,this.maxV=100,this.hsv=[0,0,100],this.rgb=[255,255,255],this.width=181,this.height=101,this.showOnClick=!0,this.mode="HSV",this.position="bottom",this.smartPosition=!0,this.sliderSize=16,this.crossSize=8,this.closable=!1,this.closeText="Close",this.buttonColor="#000000",this.buttonHeight=18,this.padding=12,this.backgroundColor="#FFFFFF",this.borderWidth=1,this.borderColor="#BBBBBB",this.borderRadius=8,this.insetWidth=1,this.insetColor="#BBBBBB",this.shadow=!0,this.shadowBlur=15,this.shadowColor="rgba(0,0,0,0.2)",this.pointerColor="#4C4C4C",this.pointerBorderColor="#FFFFFF",this.pointerBorderWidth=1,this.pointerThickness=2,this.zIndex=1e3,this.container=null;for(var r in n)n.hasOwnProperty(r)&&(this[r]=n[r]);this.hide=function(){l()&&o()},this.show=function(){u()},this.redraw=function(){l()&&u()},this.importColor=function(){this.valueElement?e.isElementType(this.valueElement,"input")?this.refine?!this.required&&/^\s*$/.test(this.valueElement.value)?(this.valueElement.value="",this.styleElement&&(this.styleElement.style.backgroundImage=this.styleElement._jscOrigStyle.backgroundImage,this.styleElement.style.backgroundColor=this.styleElement._jscOrigStyle.backgroundColor,this.styleElement.style.color=this.styleElement._jscOrigStyle.color),this.exportColor(e.leaveValue|e.leaveStyle)):this.fromString(this.valueElement.value)||this.exportColor():this.fromString(this.valueElement.value,e.leaveValue)||(this.styleElement&&(this.styleElement.style.backgroundImage=this.styleElement._jscOrigStyle.backgroundImage,this.styleElement.style.backgroundColor=this.styleElement._jscOrigStyle.backgroundColor,this.styleElement.style.color=this.styleElement._jscOrigStyle.color),this.exportColor(e.leaveValue|e.leaveStyle)):this.exportColor():this.exportColor()},this.exportColor=function(t){if(!(t&e.leaveValue)&&this.valueElement){var n=this.toString();this.uppercase&&(n=n.toUpperCase()),this.hash&&(n="#"+n),e.isElementType(this.valueElement,"input")?this.valueElement.value=n:this.valueElement.innerHTML=n}t&e.leaveStyle||this.styleElement&&(this.styleElement.style.backgroundImage="none",this.styleElement.style.backgroundColor="#"+this.toString(),this.styleElement.style.color=this.isLight()?"#000":"#FFF"),!(t&e.leavePad)&&l()&&a(),!(t&e.leaveSld)&&l()&&f()},this.fromHSV=function(e,t,n,r){if(e!==null){if(isNaN(e))return!1;e=Math.max(0,Math.min(360,e))}if(t!==null){if(isNaN(t))return!1;t=Math.max(0,Math.min(100,this.maxS,t),this.minS)}if(n!==null){if(isNaN(n))return!1;n=Math.max(0,Math.min(100,this.maxV,n),this.minV)}this.rgb=s(e===null?this.hsv[0]:this.hsv[0]=e,t===null?this.hsv[1]:this.hsv[1]=t,n===null?this.hsv[2]:this.hsv[2]=n),this.exportColor(r)},this.fromRGB=function(e,t,n,r){if(e!==null){if(isNaN(e))return!1;e=Math.max(0,Math.min(255,e))}if(t!==null){if(isNaN(t))return!1;t=Math.max(0,Math.min(255,t))}if(n!==null){if(isNaN(n))return!1;n=Math.max(0,Math.min(255,n))}var o=i(e===null?this.rgb[0]:e,t===null?this.rgb[1]:t,n===null?this.rgb[2]:n);o[0]!==null&&(this.hsv[0]=Math.max(0,Math.min(360,o[0]))),o[2]!==0&&(this.hsv[1]=o[1]===null?null:Math.max(0,this.minS,Math.min(100,this.maxS,o[1]))),this.hsv[2]=o[2]===null?null:Math.max(0,this.minV,Math.min(100,this.maxV,o[2]));var u=s(this.hsv[0],this.hsv[1],this.hsv[2]);this.rgb[0]=u[0],this.rgb[1]=u[1],this.rgb[2]=u[2],this.exportColor(r)},this.fromString=function(e,t){var n;if(n=e.match(/^\W*([0-9A-F]{3}([0-9A-F]{3})?)\W*$/i))return n[1].length===6?this.fromRGB(parseInt(n[1].substr(0,2),16),parseInt(n[1].substr(2,2),16),parseInt(n[1].substr(4,2),16),t):this.fromRGB(parseInt(n[1].charAt(0)+n[1].charAt(0),16),parseInt(n[1].charAt(1)+n[1].charAt(1),16),parseInt(n[1].charAt(2)+n[1].charAt(2),16),t),!0;if(n=e.match(/^\W*rgba?\(([^)]*)\)\W*$/i)){var r=n[1].split(","),i=/^\s*(\d*)(\.\d+)?\s*$/,s,o,u;if(r.length>=3&&(s=r[0].match(i))&&(o=r[1].match(i))&&(u=r[2].match(i))){var a=parseFloat((s[1]||"0")+(s[2]||"")),f=parseFloat((o[1]||"0")+(o[2]||"")),l=parseFloat((u[1]||"0")+(u[2]||""));return this.fromRGB(a,f,l,t),!0}}return!1},this.toString=function(){return(256|Math.round(this.rgb[0])).toString(16).substr(1)+(256|Math.round(this.rgb[1])).toString(16).substr(1)+(256|Math.round(this.rgb[2])).toString(16).substr(1)},this.toHEXString=function(){return"#"+this.toString().toUpperCase()},this.toRGBString=function(){return"rgb("+Math.round(this.rgb[0])+","+Math.round(this.rgb[1])+","+Math.round(this.rgb[2])+")"},this.isLight=function(){return.213*this.rgb[0]+.715*this.rgb[1]+.072*this.rgb[2]>127.5},this._processParentElementsInDOM=function(){if(this._linkedElementsProcessed)return;this._linkedElementsProcessed=!0;var t=this.targetElement;do{var n=e.getStyle(t);n&&n.position.toLowerCase()==="fixed"&&(this.fixed=!0),t!==this.targetElement&&(t._jscEventsAttached||(e.attachEvent(t,"scroll",e.onParentScroll),t._jscEventsAttached=!0))}while((t=t.parentNode)&&!e.isElementType(t,"body"))};if(typeof t=="string"){var h=t,p=document.getElementById(h);p?this.targetElement=p:e.warn("Could not find target element with ID '"+h+"'")}else t?this.targetElement=t:e.warn("Invalid target element: '"+t+"'");if(this.targetElement._jscLinkedInstance){e.warn("Cannot link jscolor twice to the same element. Skipping.");return}this.targetElement._jscLinkedInstance=this,this.valueElement=e.fetchElement(this.valueElement),this.styleElement=e.fetchElement(this.styleElement);var d=this,v=this.container?e.fetchElement(this.container):document.getElementsByTagName("body")[0],m=3;if(e.isElementType(this.targetElement,"button"))if(this.targetElement.onclick){var g=this.targetElement.onclick;this.targetElement.onclick=function(e){return g.call(this,e),!1}}else this.targetElement.onclick=function(){return!1};if(this.valueElement&&e.isElementType(this.valueElement,"input")){var y=function(){d.fromString(d.valueElement.value,e.leaveValue),e.dispatchFineChange(d)};e.attachEvent(this.valueElement,"keyup",y),e.attachEvent(this.valueElement,"input",y),e.attachEvent(this.valueElement,"blur",c),this.valueElement.setAttribute("autocomplete","off")}this.styleElement&&(this.styleElement._jscOrigStyle={backgroundImage:this.styleElement.style.backgroundImage,backgroundColor:this.styleElement.style.backgroundColor,color:this.styleElement.style.color}),this.value?this.fromString(this.value)||this.exportColor():this.importColor()}};return e.jscolor.lookupClass="jscolor",e.jscolor.installByClassName=function(t){var n=document.getElementsByTagName("input"),r=document.getElementsByTagName("button");e.tryInstallOnElements(n,t),e.tryInstallOnElements(r,t)},e.register(),e.jscolor}());
/**
 * SyntaxHighlighter
 * http://alexgorbatchev.com/SyntaxHighlighter
 *
 * SyntaxHighlighter is donationware. If you are using it, please donate.
 * http://alexgorbatchev.com/SyntaxHighlighter/donate.html
 *
 * @version
 * 3.0.83 (July 02 2010)
 * 
 * @copyright
 * Copyright (C) 2004-2010 Alex Gorbatchev.
 *
 * @license
 * Dual licensed under the MIT and GPL licenses.
 */
eval(function(p,a,c,k,e,d){e=function(c){return(c<a?'':e(parseInt(c/a)))+((c=c%a)>35?String.fromCharCode(c+29):c.toString(36))};if(!''.replace(/^/,String)){while(c--){d[e(c)]=k[c]||e(c)}k=[function(e){return d[e]}];e=function(){return'\\w+'};c=1};while(c--){if(k[c]){p=p.replace(new RegExp('\\b'+e(c)+'\\b','g'),k[c])}}return p}('K M;I(M)1S 2U("2a\'t 4k M 4K 2g 3l 4G 4H");(6(){6 r(f,e){I(!M.1R(f))1S 3m("3s 15 4R");K a=f.1w;f=M(f.1m,t(f)+(e||""));I(a)f.1w={1m:a.1m,19:a.19?a.19.1a(0):N};H f}6 t(f){H(f.1J?"g":"")+(f.4s?"i":"")+(f.4p?"m":"")+(f.4v?"x":"")+(f.3n?"y":"")}6 B(f,e,a,b){K c=u.L,d,h,g;v=R;5K{O(;c--;){g=u[c];I(a&g.3r&&(!g.2p||g.2p.W(b))){g.2q.12=e;I((h=g.2q.X(f))&&h.P===e){d={3k:g.2b.W(b,h,a),1C:h};1N}}}}5v(i){1S i}5q{v=11}H d}6 p(f,e,a){I(3b.Z.1i)H f.1i(e,a);O(a=a||0;a<f.L;a++)I(f[a]===e)H a;H-1}M=6(f,e){K a=[],b=M.1B,c=0,d,h;I(M.1R(f)){I(e!==1d)1S 3m("2a\'t 5r 5I 5F 5B 5C 15 5E 5p");H r(f)}I(v)1S 2U("2a\'t W 3l M 59 5m 5g 5x 5i");e=e||"";O(d={2N:11,19:[],2K:6(g){H e.1i(g)>-1},3d:6(g){e+=g}};c<f.L;)I(h=B(f,c,b,d)){a.U(h.3k);c+=h.1C[0].L||1}Y I(h=n.X.W(z[b],f.1a(c))){a.U(h[0]);c+=h[0].L}Y{h=f.3a(c);I(h==="[")b=M.2I;Y I(h==="]")b=M.1B;a.U(h);c++}a=15(a.1K(""),n.Q.W(e,w,""));a.1w={1m:f,19:d.2N?d.19:N};H a};M.3v="1.5.0";M.2I=1;M.1B=2;K C=/\\$(?:(\\d\\d?|[$&`\'])|{([$\\w]+)})/g,w=/[^5h]+|([\\s\\S])(?=[\\s\\S]*\\1)/g,A=/^(?:[?*+]|{\\d+(?:,\\d*)?})\\??/,v=11,u=[],n={X:15.Z.X,1A:15.Z.1A,1C:1r.Z.1C,Q:1r.Z.Q,1e:1r.Z.1e},x=n.X.W(/()??/,"")[1]===1d,D=6(){K f=/^/g;n.1A.W(f,"");H!f.12}(),y=6(){K f=/x/g;n.Q.W("x",f,"");H!f.12}(),E=15.Z.3n!==1d,z={};z[M.2I]=/^(?:\\\\(?:[0-3][0-7]{0,2}|[4-7][0-7]?|x[\\29-26-f]{2}|u[\\29-26-f]{4}|c[A-3o-z]|[\\s\\S]))/;z[M.1B]=/^(?:\\\\(?:0(?:[0-3][0-7]{0,2}|[4-7][0-7]?)?|[1-9]\\d*|x[\\29-26-f]{2}|u[\\29-26-f]{4}|c[A-3o-z]|[\\s\\S])|\\(\\?[:=!]|[?*+]\\?|{\\d+(?:,\\d*)?}\\??)/;M.1h=6(f,e,a,b){u.U({2q:r(f,"g"+(E?"y":"")),2b:e,3r:a||M.1B,2p:b||N})};M.2n=6(f,e){K a=f+"/"+(e||"");H M.2n[a]||(M.2n[a]=M(f,e))};M.3c=6(f){H r(f,"g")};M.5l=6(f){H f.Q(/[-[\\]{}()*+?.,\\\\^$|#\\s]/g,"\\\\$&")};M.5e=6(f,e,a,b){e=r(e,"g"+(b&&E?"y":""));e.12=a=a||0;f=e.X(f);H b?f&&f.P===a?f:N:f};M.3q=6(){M.1h=6(){1S 2U("2a\'t 55 1h 54 3q")}};M.1R=6(f){H 53.Z.1q.W(f)==="[2m 15]"};M.3p=6(f,e,a,b){O(K c=r(e,"g"),d=-1,h;h=c.X(f);){a.W(b,h,++d,f,c);c.12===h.P&&c.12++}I(e.1J)e.12=0};M.57=6(f,e){H 6 a(b,c){K d=e[c].1I?e[c]:{1I:e[c]},h=r(d.1I,"g"),g=[],i;O(i=0;i<b.L;i++)M.3p(b[i],h,6(k){g.U(d.3j?k[d.3j]||"":k[0])});H c===e.L-1||!g.L?g:a(g,c+1)}([f],0)};15.Z.1p=6(f,e){H J.X(e[0])};15.Z.W=6(f,e){H J.X(e)};15.Z.X=6(f){K e=n.X.1p(J,14),a;I(e){I(!x&&e.L>1&&p(e,"")>-1){a=15(J.1m,n.Q.W(t(J),"g",""));n.Q.W(f.1a(e.P),a,6(){O(K c=1;c<14.L-2;c++)I(14[c]===1d)e[c]=1d})}I(J.1w&&J.1w.19)O(K b=1;b<e.L;b++)I(a=J.1w.19[b-1])e[a]=e[b];!D&&J.1J&&!e[0].L&&J.12>e.P&&J.12--}H e};I(!D)15.Z.1A=6(f){(f=n.X.W(J,f))&&J.1J&&!f[0].L&&J.12>f.P&&J.12--;H!!f};1r.Z.1C=6(f){M.1R(f)||(f=15(f));I(f.1J){K e=n.1C.1p(J,14);f.12=0;H e}H f.X(J)};1r.Z.Q=6(f,e){K a=M.1R(f),b,c;I(a&&1j e.58()==="3f"&&e.1i("${")===-1&&y)H n.Q.1p(J,14);I(a){I(f.1w)b=f.1w.19}Y f+="";I(1j e==="6")c=n.Q.W(J,f,6(){I(b){14[0]=1f 1r(14[0]);O(K d=0;d<b.L;d++)I(b[d])14[0][b[d]]=14[d+1]}I(a&&f.1J)f.12=14[14.L-2]+14[0].L;H e.1p(N,14)});Y{c=J+"";c=n.Q.W(c,f,6(){K d=14;H n.Q.W(e,C,6(h,g,i){I(g)5b(g){24"$":H"$";24"&":H d[0];24"`":H d[d.L-1].1a(0,d[d.L-2]);24"\'":H d[d.L-1].1a(d[d.L-2]+d[0].L);5a:i="";g=+g;I(!g)H h;O(;g>d.L-3;){i=1r.Z.1a.W(g,-1)+i;g=1Q.3i(g/10)}H(g?d[g]||"":"$")+i}Y{g=+i;I(g<=d.L-3)H d[g];g=b?p(b,i):-1;H g>-1?d[g+1]:h}})})}I(a&&f.1J)f.12=0;H c};1r.Z.1e=6(f,e){I(!M.1R(f))H n.1e.1p(J,14);K a=J+"",b=[],c=0,d,h;I(e===1d||+e<0)e=5D;Y{e=1Q.3i(+e);I(!e)H[]}O(f=M.3c(f);d=f.X(a);){I(f.12>c){b.U(a.1a(c,d.P));d.L>1&&d.P<a.L&&3b.Z.U.1p(b,d.1a(1));h=d[0].L;c=f.12;I(b.L>=e)1N}f.12===d.P&&f.12++}I(c===a.L){I(!n.1A.W(f,"")||h)b.U("")}Y b.U(a.1a(c));H b.L>e?b.1a(0,e):b};M.1h(/\\(\\?#[^)]*\\)/,6(f){H n.1A.W(A,f.2S.1a(f.P+f[0].L))?"":"(?:)"});M.1h(/\\((?!\\?)/,6(){J.19.U(N);H"("});M.1h(/\\(\\?<([$\\w]+)>/,6(f){J.19.U(f[1]);J.2N=R;H"("});M.1h(/\\\\k<([\\w$]+)>/,6(f){K e=p(J.19,f[1]);H e>-1?"\\\\"+(e+1)+(3R(f.2S.3a(f.P+f[0].L))?"":"(?:)"):f[0]});M.1h(/\\[\\^?]/,6(f){H f[0]==="[]"?"\\\\b\\\\B":"[\\\\s\\\\S]"});M.1h(/^\\(\\?([5A]+)\\)/,6(f){J.3d(f[1]);H""});M.1h(/(?:\\s+|#.*)+/,6(f){H n.1A.W(A,f.2S.1a(f.P+f[0].L))?"":"(?:)"},M.1B,6(){H J.2K("x")});M.1h(/\\./,6(){H"[\\\\s\\\\S]"},M.1B,6(){H J.2K("s")})})();1j 2e!="1d"&&(2e.M=M);K 1v=6(){6 r(a,b){a.1l.1i(b)!=-1||(a.1l+=" "+b)}6 t(a){H a.1i("3e")==0?a:"3e"+a}6 B(a){H e.1Y.2A[t(a)]}6 p(a,b,c){I(a==N)H N;K d=c!=R?a.3G:[a.2G],h={"#":"1c",".":"1l"}[b.1o(0,1)]||"3h",g,i;g=h!="3h"?b.1o(1):b.5u();I((a[h]||"").1i(g)!=-1)H a;O(a=0;d&&a<d.L&&i==N;a++)i=p(d[a],b,c);H i}6 C(a,b){K c={},d;O(d 2g a)c[d]=a[d];O(d 2g b)c[d]=b[d];H c}6 w(a,b,c,d){6 h(g){g=g||1P.5y;I(!g.1F){g.1F=g.52;g.3N=6(){J.5w=11}}c.W(d||1P,g)}a.3g?a.3g("4U"+b,h):a.4y(b,h,11)}6 A(a,b){K c=e.1Y.2j,d=N;I(c==N){c={};O(K h 2g e.1U){K g=e.1U[h];d=g.4x;I(d!=N){g.1V=h.4w();O(g=0;g<d.L;g++)c[d[g]]=h}}e.1Y.2j=c}d=e.1U[c[a]];d==N&&b!=11&&1P.1X(e.13.1x.1X+(e.13.1x.3E+a));H d}6 v(a,b){O(K c=a.1e("\\n"),d=0;d<c.L;d++)c[d]=b(c[d],d);H c.1K("\\n")}6 u(a,b){I(a==N||a.L==0||a=="\\n")H a;a=a.Q(/</g,"&1y;");a=a.Q(/ {2,}/g,6(c){O(K d="",h=0;h<c.L-1;h++)d+=e.13.1W;H d+" "});I(b!=N)a=v(a,6(c){I(c.L==0)H"";K d="";c=c.Q(/^(&2s;| )+/,6(h){d=h;H""});I(c.L==0)H d;H d+\'<17 1g="\'+b+\'">\'+c+"</17>"});H a}6 n(a,b){a.1e("\\n");O(K c="",d=0;d<50;d++)c+="                    ";H a=v(a,6(h){I(h.1i("\\t")==-1)H h;O(K g=0;(g=h.1i("\\t"))!=-1;)h=h.1o(0,g)+c.1o(0,b-g%b)+h.1o(g+1,h.L);H h})}6 x(a){H a.Q(/^\\s+|\\s+$/g,"")}6 D(a,b){I(a.P<b.P)H-1;Y I(a.P>b.P)H 1;Y I(a.L<b.L)H-1;Y I(a.L>b.L)H 1;H 0}6 y(a,b){6 c(k){H k[0]}O(K d=N,h=[],g=b.2D?b.2D:c;(d=b.1I.X(a))!=N;){K i=g(d,b);I(1j i=="3f")i=[1f e.2L(i,d.P,b.23)];h=h.1O(i)}H h}6 E(a){K b=/(.*)((&1G;|&1y;).*)/;H a.Q(e.3A.3M,6(c){K d="",h=N;I(h=b.X(c)){c=h[1];d=h[2]}H\'<a 2h="\'+c+\'">\'+c+"</a>"+d})}6 z(){O(K a=1E.36("1k"),b=[],c=0;c<a.L;c++)a[c].3s=="20"&&b.U(a[c]);H b}6 f(a){a=a.1F;K b=p(a,".20",R);a=p(a,".3O",R);K c=1E.4i("3t");I(!(!a||!b||p(a,"3t"))){B(b.1c);r(b,"1m");O(K d=a.3G,h=[],g=0;g<d.L;g++)h.U(d[g].4z||d[g].4A);h=h.1K("\\r");c.39(1E.4D(h));a.39(c);c.2C();c.4C();w(c,"4u",6(){c.2G.4E(c);b.1l=b.1l.Q("1m","")})}}I(1j 3F!="1d"&&1j M=="1d")M=3F("M").M;K e={2v:{"1g-27":"","2i-1s":1,"2z-1s-2t":11,1M:N,1t:N,"42-45":R,"43-22":4,1u:R,16:R,"3V-17":R,2l:11,"41-40":R,2k:11,"1z-1k":11},13:{1W:"&2s;",2M:R,46:11,44:11,34:"4n",1x:{21:"4o 1m",2P:"?",1X:"1v\\n\\n",3E:"4r\'t 4t 1D O: ",4g:"4m 4B\'t 51 O 1z-1k 4F: ",37:\'<!4T 1z 4S "-//4V//3H 4W 1.0 4Z//4Y" "1Z://2y.3L.3K/4X/3I/3H/3I-4P.4J"><1z 4I="1Z://2y.3L.3K/4L/5L"><3J><4N 1Z-4M="5G-5M" 6K="2O/1z; 6J=6I-8" /><1t>6L 1v</1t></3J><3B 1L="25-6M:6Q,6P,6O,6N-6F;6y-2f:#6x;2f:#6w;25-22:6v;2O-3D:3C;"><T 1L="2O-3D:3C;3w-32:1.6z;"><T 1L="25-22:6A-6E;">1v</T><T 1L="25-22:.6C;3w-6B:6R;"><T>3v 3.0.76 (72 73 3x)</T><T><a 2h="1Z://3u.2w/1v" 1F="38" 1L="2f:#3y">1Z://3u.2w/1v</a></T><T>70 17 6U 71.</T><T>6T 6X-3x 6Y 6D.</T></T><T>6t 61 60 J 1k, 5Z <a 2h="6u://2y.62.2w/63-66/65?64=5X-5W&5P=5O" 1L="2f:#3y">5R</a> 5V <2R/>5U 5T 5S!</T></T></3B></1z>\'}},1Y:{2j:N,2A:{}},1U:{},3A:{6n:/\\/\\*[\\s\\S]*?\\*\\//2c,6m:/\\/\\/.*$/2c,6l:/#.*$/2c,6k:/"([^\\\\"\\n]|\\\\.)*"/g,6o:/\'([^\\\\\'\\n]|\\\\.)*\'/g,6p:1f M(\'"([^\\\\\\\\"]|\\\\\\\\.)*"\',"3z"),6s:1f M("\'([^\\\\\\\\\']|\\\\\\\\.)*\'","3z"),6q:/(&1y;|<)!--[\\s\\S]*?--(&1G;|>)/2c,3M:/\\w+:\\/\\/[\\w-.\\/?%&=:@;]*/g,6a:{18:/(&1y;|<)\\?=?/g,1b:/\\?(&1G;|>)/g},69:{18:/(&1y;|<)%=?/g,1b:/%(&1G;|>)/g},6d:{18:/(&1y;|<)\\s*1k.*?(&1G;|>)/2T,1b:/(&1y;|<)\\/\\s*1k\\s*(&1G;|>)/2T}},16:{1H:6(a){6 b(i,k){H e.16.2o(i,k,e.13.1x[k])}O(K c=\'<T 1g="16">\',d=e.16.2x,h=d.2X,g=0;g<h.L;g++)c+=(d[h[g]].1H||b)(a,h[g]);c+="</T>";H c},2o:6(a,b,c){H\'<2W><a 2h="#" 1g="6e 6h\'+b+" "+b+\'">\'+c+"</a></2W>"},2b:6(a){K b=a.1F,c=b.1l||"";b=B(p(b,".20",R).1c);K d=6(h){H(h=15(h+"6f(\\\\w+)").X(c))?h[1]:N}("6g");b&&d&&e.16.2x[d].2B(b);a.3N()},2x:{2X:["21","2P"],21:{1H:6(a){I(a.V("2l")!=R)H"";K b=a.V("1t");H e.16.2o(a,"21",b?b:e.13.1x.21)},2B:6(a){a=1E.6j(t(a.1c));a.1l=a.1l.Q("47","")}},2P:{2B:6(){K a="68=0";a+=", 18="+(31.30-33)/2+", 32="+(31.2Z-2Y)/2+", 30=33, 2Z=2Y";a=a.Q(/^,/,"");a=1P.6Z("","38",a);a.2C();K b=a.1E;b.6W(e.13.1x.37);b.6V();a.2C()}}}},35:6(a,b){K c;I(b)c=[b];Y{c=1E.36(e.13.34);O(K d=[],h=0;h<c.L;h++)d.U(c[h]);c=d}c=c;d=[];I(e.13.2M)c=c.1O(z());I(c.L===0)H d;O(h=0;h<c.L;h++){O(K g=c[h],i=a,k=c[h].1l,j=3W 0,l={},m=1f M("^\\\\[(?<2V>(.*?))\\\\]$"),s=1f M("(?<27>[\\\\w-]+)\\\\s*:\\\\s*(?<1T>[\\\\w-%#]+|\\\\[.*?\\\\]|\\".*?\\"|\'.*?\')\\\\s*;?","g");(j=s.X(k))!=N;){K o=j.1T.Q(/^[\'"]|[\'"]$/g,"");I(o!=N&&m.1A(o)){o=m.X(o);o=o.2V.L>0?o.2V.1e(/\\s*,\\s*/):[]}l[j.27]=o}g={1F:g,1n:C(i,l)};g.1n.1D!=N&&d.U(g)}H d},1M:6(a,b){K c=J.35(a,b),d=N,h=e.13;I(c.L!==0)O(K g=0;g<c.L;g++){b=c[g];K i=b.1F,k=b.1n,j=k.1D,l;I(j!=N){I(k["1z-1k"]=="R"||e.2v["1z-1k"]==R){d=1f e.4l(j);j="4O"}Y I(d=A(j))d=1f d;Y 6H;l=i.3X;I(h.2M){l=l;K m=x(l),s=11;I(m.1i("<![6G[")==0){m=m.4h(9);s=R}K o=m.L;I(m.1i("]]\\>")==o-3){m=m.4h(0,o-3);s=R}l=s?m:l}I((i.1t||"")!="")k.1t=i.1t;k.1D=j;d.2Q(k);b=d.2F(l);I((i.1c||"")!="")b.1c=i.1c;i.2G.74(b,i)}}},2E:6(a){w(1P,"4k",6(){e.1M(a)})}};e.2E=e.2E;e.1M=e.1M;e.2L=6(a,b,c){J.1T=a;J.P=b;J.L=a.L;J.23=c;J.1V=N};e.2L.Z.1q=6(){H J.1T};e.4l=6(a){6 b(j,l){O(K m=0;m<j.L;m++)j[m].P+=l}K c=A(a),d,h=1f e.1U.5Y,g=J,i="2F 1H 2Q".1e(" ");I(c!=N){d=1f c;O(K k=0;k<i.L;k++)(6(){K j=i[k];g[j]=6(){H h[j].1p(h,14)}})();d.28==N?1P.1X(e.13.1x.1X+(e.13.1x.4g+a)):h.2J.U({1I:d.28.17,2D:6(j){O(K l=j.17,m=[],s=d.2J,o=j.P+j.18.L,F=d.28,q,G=0;G<s.L;G++){q=y(l,s[G]);b(q,o);m=m.1O(q)}I(F.18!=N&&j.18!=N){q=y(j.18,F.18);b(q,j.P);m=m.1O(q)}I(F.1b!=N&&j.1b!=N){q=y(j.1b,F.1b);b(q,j.P+j[0].5Q(j.1b));m=m.1O(q)}O(j=0;j<m.L;j++)m[j].1V=c.1V;H m}})}};e.4j=6(){};e.4j.Z={V:6(a,b){K c=J.1n[a];c=c==N?b:c;K d={"R":R,"11":11}[c];H d==N?c:d},3Y:6(a){H 1E.4i(a)},4c:6(a,b){K c=[];I(a!=N)O(K d=0;d<a.L;d++)I(1j a[d]=="2m")c=c.1O(y(b,a[d]));H J.4e(c.6b(D))},4e:6(a){O(K b=0;b<a.L;b++)I(a[b]!==N)O(K c=a[b],d=c.P+c.L,h=b+1;h<a.L&&a[b]!==N;h++){K g=a[h];I(g!==N)I(g.P>d)1N;Y I(g.P==c.P&&g.L>c.L)a[b]=N;Y I(g.P>=c.P&&g.P<d)a[h]=N}H a},4d:6(a){K b=[],c=2u(J.V("2i-1s"));v(a,6(d,h){b.U(h+c)});H b},3U:6(a){K b=J.V("1M",[]);I(1j b!="2m"&&b.U==N)b=[b];a:{a=a.1q();K c=3W 0;O(c=c=1Q.6c(c||0,0);c<b.L;c++)I(b[c]==a){b=c;1N a}b=-1}H b!=-1},2r:6(a,b,c){a=["1s","6i"+b,"P"+a,"6r"+(b%2==0?1:2).1q()];J.3U(b)&&a.U("67");b==0&&a.U("1N");H\'<T 1g="\'+a.1K(" ")+\'">\'+c+"</T>"},3Q:6(a,b){K c="",d=a.1e("\\n").L,h=2u(J.V("2i-1s")),g=J.V("2z-1s-2t");I(g==R)g=(h+d-1).1q().L;Y I(3R(g)==R)g=0;O(K i=0;i<d;i++){K k=b?b[i]:h+i,j;I(k==0)j=e.13.1W;Y{j=g;O(K l=k.1q();l.L<j;)l="0"+l;j=l}a=j;c+=J.2r(i,k,a)}H c},49:6(a,b){a=x(a);K c=a.1e("\\n");J.V("2z-1s-2t");K d=2u(J.V("2i-1s"));a="";O(K h=J.V("1D"),g=0;g<c.L;g++){K i=c[g],k=/^(&2s;|\\s)+/.X(i),j=N,l=b?b[g]:d+g;I(k!=N){j=k[0].1q();i=i.1o(j.L);j=j.Q(" ",e.13.1W)}i=x(i);I(i.L==0)i=e.13.1W;a+=J.2r(g,l,(j!=N?\'<17 1g="\'+h+\' 5N">\'+j+"</17>":"")+i)}H a},4f:6(a){H a?"<4a>"+a+"</4a>":""},4b:6(a,b){6 c(l){H(l=l?l.1V||g:g)?l+" ":""}O(K d=0,h="",g=J.V("1D",""),i=0;i<b.L;i++){K k=b[i],j;I(!(k===N||k.L===0)){j=c(k);h+=u(a.1o(d,k.P-d),j+"48")+u(k.1T,j+k.23);d=k.P+k.L+(k.75||0)}}h+=u(a.1o(d),c()+"48");H h},1H:6(a){K b="",c=["20"],d;I(J.V("2k")==R)J.1n.16=J.1n.1u=11;1l="20";J.V("2l")==R&&c.U("47");I((1u=J.V("1u"))==11)c.U("6S");c.U(J.V("1g-27"));c.U(J.V("1D"));a=a.Q(/^[ ]*[\\n]+|[\\n]*[ ]*$/g,"").Q(/\\r/g," ");b=J.V("43-22");I(J.V("42-45")==R)a=n(a,b);Y{O(K h="",g=0;g<b;g++)h+=" ";a=a.Q(/\\t/g,h)}a=a;a:{b=a=a;h=/<2R\\s*\\/?>|&1y;2R\\s*\\/?&1G;/2T;I(e.13.46==R)b=b.Q(h,"\\n");I(e.13.44==R)b=b.Q(h,"");b=b.1e("\\n");h=/^\\s*/;g=4Q;O(K i=0;i<b.L&&g>0;i++){K k=b[i];I(x(k).L!=0){k=h.X(k);I(k==N){a=a;1N a}g=1Q.4q(k[0].L,g)}}I(g>0)O(i=0;i<b.L;i++)b[i]=b[i].1o(g);a=b.1K("\\n")}I(1u)d=J.4d(a);b=J.4c(J.2J,a);b=J.4b(a,b);b=J.49(b,d);I(J.V("41-40"))b=E(b);1j 2H!="1d"&&2H.3S&&2H.3S.1C(/5s/)&&c.U("5t");H b=\'<T 1c="\'+t(J.1c)+\'" 1g="\'+c.1K(" ")+\'">\'+(J.V("16")?e.16.1H(J):"")+\'<3Z 5z="0" 5H="0" 5J="0">\'+J.4f(J.V("1t"))+"<3T><3P>"+(1u?\'<2d 1g="1u">\'+J.3Q(a)+"</2d>":"")+\'<2d 1g="17"><T 1g="3O">\'+b+"</T></2d></3P></3T></3Z></T>"},2F:6(a){I(a===N)a="";J.17=a;K b=J.3Y("T");b.3X=J.1H(a);J.V("16")&&w(p(b,".16"),"5c",e.16.2b);J.V("3V-17")&&w(p(b,".17"),"56",f);H b},2Q:6(a){J.1c=""+1Q.5d(1Q.5n()*5k).1q();e.1Y.2A[t(J.1c)]=J;J.1n=C(e.2v,a||{});I(J.V("2k")==R)J.1n.16=J.1n.1u=11},5j:6(a){a=a.Q(/^\\s+|\\s+$/g,"").Q(/\\s+/g,"|");H"\\\\b(?:"+a+")\\\\b"},5f:6(a){J.28={18:{1I:a.18,23:"1k"},1b:{1I:a.1b,23:"1k"},17:1f M("(?<18>"+a.18.1m+")(?<17>.*?)(?<1b>"+a.1b.1m+")","5o")}}};H e}();1j 2e!="1d"&&(2e.1v=1v);',62,441,'||||||function|||||||||||||||||||||||||||||||||||||return|if|this|var|length|XRegExp|null|for|index|replace|true||div|push|getParam|call|exec|else|prototype||false|lastIndex|config|arguments|RegExp|toolbar|code|left|captureNames|slice|right|id|undefined|split|new|class|addToken|indexOf|typeof|script|className|source|params|substr|apply|toString|String|line|title|gutter|SyntaxHighlighter|_xregexp|strings|lt|html|test|OUTSIDE_CLASS|match|brush|document|target|gt|getHtml|regex|global|join|style|highlight|break|concat|window|Math|isRegExp|throw|value|brushes|brushName|space|alert|vars|http|syntaxhighlighter|expandSource|size|css|case|font|Fa|name|htmlScript|dA|can|handler|gm|td|exports|color|in|href|first|discoveredBrushes|light|collapse|object|cache|getButtonHtml|trigger|pattern|getLineHtml|nbsp|numbers|parseInt|defaults|com|items|www|pad|highlighters|execute|focus|func|all|getDiv|parentNode|navigator|INSIDE_CLASS|regexList|hasFlag|Match|useScriptTags|hasNamedCapture|text|help|init|br|input|gi|Error|values|span|list|250|height|width|screen|top|500|tagName|findElements|getElementsByTagName|aboutDialog|_blank|appendChild|charAt|Array|copyAsGlobal|setFlag|highlighter_|string|attachEvent|nodeName|floor|backref|output|the|TypeError|sticky|Za|iterate|freezeTokens|scope|type|textarea|alexgorbatchev|version|margin|2010|005896|gs|regexLib|body|center|align|noBrush|require|childNodes|DTD|xhtml1|head|org|w3|url|preventDefault|container|tr|getLineNumbersHtml|isNaN|userAgent|tbody|isLineHighlighted|quick|void|innerHTML|create|table|links|auto|smart|tab|stripBrs|tabs|bloggerMode|collapsed|plain|getCodeLinesHtml|caption|getMatchesHtml|findMatches|figureOutLineNumbers|removeNestedMatches|getTitleHtml|brushNotHtmlScript|substring|createElement|Highlighter|load|HtmlScript|Brush|pre|expand|multiline|min|Can|ignoreCase|find|blur|extended|toLowerCase|aliases|addEventListener|innerText|textContent|wasn|select|createTextNode|removeChild|option|same|frame|xmlns|dtd|twice|1999|equiv|meta|htmlscript|transitional|1E3|expected|PUBLIC|DOCTYPE|on|W3C|XHTML|TR|EN|Transitional||configured|srcElement|Object|after|run|dblclick|matchChain|valueOf|constructor|default|switch|click|round|execAt|forHtmlScript|token|gimy|functions|getKeywords|1E6|escape|within|random|sgi|another|finally|supply|MSIE|ie|toUpperCase|catch|returnValue|definition|event|border|imsx|constructing|one|Infinity|from|when|Content|cellpadding|flags|cellspacing|try|xhtml|Type|spaces|2930402|hosted_button_id|lastIndexOf|donate|active|development|keep|to|xclick|_s|Xml|please|like|you|paypal|cgi|cmd|webscr|bin|highlighted|scrollbars|aspScriptTags|phpScriptTags|sort|max|scriptScriptTags|toolbar_item|_|command|command_|number|getElementById|doubleQuotedString|singleLinePerlComments|singleLineCComments|multiLineCComments|singleQuotedString|multiLineDoubleQuotedString|xmlComments|alt|multiLineSingleQuotedString|If|https|1em|000|fff|background|5em|xx|bottom|75em|Gorbatchev|large|serif|CDATA|continue|utf|charset|content|About|family|sans|Helvetica|Arial|Geneva|3em|nogutter|Copyright|syntax|close|write|2004|Alex|open|JavaScript|highlighter|July|02|replaceChild|offset|83'.split('|'),0,{}))

/**
 * SyntaxHighlighter
 * http://alexgorbatchev.com/SyntaxHighlighter
 *
 * SyntaxHighlighter is donationware. If you are using it, please donate.
 * http://alexgorbatchev.com/SyntaxHighlighter/donate.html
 *
 * @version
 * 3.0.83 (July 02 2010)
 * 
 * @copyright
 * Copyright (C) 2004-2010 Alex Gorbatchev.
 *
 * @license
 * Dual licensed under the MIT and GPL licenses.
 */
;(function()
{
	// CommonJS
	typeof(require) != 'undefined' ? SyntaxHighlighter = require('shCore').SyntaxHighlighter : null;

	function Brush()
	{
		function getKeywordsCSS(str)
		{
			return '\\b([a-z_]|)' + str.replace(/ /g, '(?=:)\\b|\\b([a-z_\\*]|\\*|)') + '(?=:)\\b';
		};
	
		function getValuesCSS(str)
		{
			return '\\b' + str.replace(/ /g, '(?!-)(?!:)\\b|\\b()') + '\:\\b';
		};

		var keywords =	'ascent azimuth background-attachment background-color background-image background-position ' +
						'background-repeat background baseline bbox border-collapse border-color border-spacing border-style border-top ' +
						'border-right border-bottom border-left border-top-color border-right-color border-bottom-color border-left-color ' +
						'border-top-style border-right-style border-bottom-style border-left-style border-top-width border-right-width ' +
						'border-bottom-width border-left-width border-width border bottom cap-height caption-side centerline clear clip color ' +
						'content counter-increment counter-reset cue-after cue-before cue cursor definition-src descent direction display ' +
						'elevation empty-cells float font-size-adjust font-family font-size font-stretch font-style font-variant font-weight font ' +
						'height left letter-spacing line-height list-style-image list-style-position list-style-type list-style margin-top ' +
						'margin-right margin-bottom margin-left margin marker-offset marks mathline max-height max-width min-height min-width orphans ' +
						'outline-color outline-style outline-width outline overflow padding-top padding-right padding-bottom padding-left padding page ' +
						'page-break-after page-break-before page-break-inside pause pause-after pause-before pitch pitch-range play-during position ' +
						'quotes right richness size slope src speak-header speak-numeral speak-punctuation speak speech-rate stemh stemv stress ' +
						'table-layout text-align top text-decoration text-indent text-shadow text-transform unicode-bidi unicode-range units-per-em ' +
						'vertical-align visibility voice-family volume white-space widows width widths word-spacing x-height z-index';

		var values =	'above absolute all always aqua armenian attr aural auto avoid baseline behind below bidi-override black blink block blue bold bolder '+
						'both bottom braille capitalize caption center center-left center-right circle close-quote code collapse compact condensed '+
						'continuous counter counters crop cross crosshair cursive dashed decimal decimal-leading-zero default digits disc dotted double '+
						'embed embossed e-resize expanded extra-condensed extra-expanded fantasy far-left far-right fast faster fixed format fuchsia '+
						'gray green groove handheld hebrew help hidden hide high higher icon inline-table inline inset inside invert italic '+
						'justify landscape large larger left-side left leftwards level lighter lime line-through list-item local loud lower-alpha '+
						'lowercase lower-greek lower-latin lower-roman lower low ltr marker maroon medium message-box middle mix move narrower '+
						'navy ne-resize no-close-quote none no-open-quote no-repeat normal nowrap n-resize nw-resize oblique olive once open-quote outset '+
						'outside overline pointer portrait pre print projection purple red relative repeat repeat-x repeat-y rgb ridge right right-side '+
						'rightwards rtl run-in screen scroll semi-condensed semi-expanded separate se-resize show silent silver slower slow '+
						'small small-caps small-caption smaller soft solid speech spell-out square s-resize static status-bar sub super sw-resize '+
						'table-caption table-cell table-column table-column-group table-footer-group table-header-group table-row table-row-group teal '+
						'text-bottom text-top thick thin top transparent tty tv ultra-condensed ultra-expanded underline upper-alpha uppercase upper-latin '+
						'upper-roman url visible wait white wider w-resize x-fast x-high x-large x-loud x-low x-slow x-small x-soft xx-large xx-small yellow';

		var fonts =		'[mM]onospace [tT]ahoma [vV]erdana [aA]rial [hH]elvetica [sS]ans-serif [sS]erif [cC]ourier mono sans serif';
	
		this.regexList = [
			{ regex: SyntaxHighlighter.regexLib.multiLineCComments,		css: 'comments' },	// multiline comments
			{ regex: SyntaxHighlighter.regexLib.doubleQuotedString,		css: 'string' },	// double quoted strings
			{ regex: SyntaxHighlighter.regexLib.singleQuotedString,		css: 'string' },	// single quoted strings
			{ regex: /\#[a-fA-F0-9]{3,6}/g,								css: 'value' },		// html colors
			{ regex: /(-?\d+)(\.\d+)?(px|em|pt|\:|\%|)/g,				css: 'value' },		// sizes
			{ regex: /!important/g,										css: 'color3' },	// !important
			{ regex: new RegExp(getKeywordsCSS(keywords), 'gm'),		css: 'keyword' },	// keywords
			{ regex: new RegExp(getValuesCSS(values), 'g'),				css: 'value' },		// values
			{ regex: new RegExp(this.getKeywords(fonts), 'g'),			css: 'color1' }		// fonts
			];

		this.forHtmlScript({ 
			left: /(&lt;|<)\s*style.*?(&gt;|>)/gi, 
			right: /(&lt;|<)\/\s*style\s*(&gt;|>)/gi 
			});
	};

	Brush.prototype	= new SyntaxHighlighter.Highlighter();
	Brush.aliases	= ['css'];

	SyntaxHighlighter.brushes.CSS = Brush;

	// CommonJS
	typeof(exports) != 'undefined' ? exports.Brush = Brush : null;
})();

/**
 * SyntaxHighlighter
 * http://alexgorbatchev.com/SyntaxHighlighter
 *
 * SyntaxHighlighter is donationware. If you are using it, please donate.
 * http://alexgorbatchev.com/SyntaxHighlighter/donate.html
 *
 * @version
 * 3.0.83 (July 02 2010)
 * 
 * @copyright
 * Copyright (C) 2004-2010 Alex Gorbatchev.
 *
 * @license
 * Dual licensed under the MIT and GPL licenses.
 */
;(function()
{
	// CommonJS
	typeof(require) != 'undefined' ? SyntaxHighlighter = require('shCore').SyntaxHighlighter : null;

	function Brush()
	{
		var keywords =	'break case catch continue ' +
						'default delete do else false  ' +
						'for function if in instanceof ' +
						'new null return super switch ' +
						'this throw true try typeof var while with'
						;

		var r = SyntaxHighlighter.regexLib;
		
		this.regexList = [
			{ regex: r.multiLineDoubleQuotedString,					css: 'string' },			// double quoted strings
			{ regex: r.multiLineSingleQuotedString,					css: 'string' },			// single quoted strings
			{ regex: r.singleLineCComments,							css: 'comments' },			// one line comments
			{ regex: r.multiLineCComments,							css: 'comments' },			// multiline comments
			{ regex: /\s*#.*/gm,									css: 'preprocessor' },		// preprocessor tags like #region and #endregion
			{ regex: new RegExp(this.getKeywords(keywords), 'gm'),	css: 'keyword' }			// keywords
			];
	
		this.forHtmlScript(r.scriptScriptTags);
	};

	Brush.prototype	= new SyntaxHighlighter.Highlighter();
	Brush.aliases	= ['js', 'jscript', 'javascript'];

	SyntaxHighlighter.brushes.JScript = Brush;

	// CommonJS
	typeof(exports) != 'undefined' ? exports.Brush = Brush : null;
})();

/**
 * SyntaxHighlighter
 * http://alexgorbatchev.com/SyntaxHighlighter
 *
 * SyntaxHighlighter is donationware. If you are using it, please donate.
 * http://alexgorbatchev.com/SyntaxHighlighter/donate.html
 *
 * @version
 * 3.0.83 (July 02 2010)
 * 
 * @copyright
 * Copyright (C) 2004-2010 Alex Gorbatchev.
 *
 * @license
 * Dual licensed under the MIT and GPL licenses.
 */
;(function()
{
	// CommonJS
	typeof(require) != 'undefined' ? SyntaxHighlighter = require('shCore').SyntaxHighlighter : null;

	function Brush()
	{
		var funcs	=	'abs acos acosh addcslashes addslashes ' +
						'array_change_key_case array_chunk array_combine array_count_values array_diff '+
						'array_diff_assoc array_diff_key array_diff_uassoc array_diff_ukey array_fill '+
						'array_filter array_flip array_intersect array_intersect_assoc array_intersect_key '+
						'array_intersect_uassoc array_intersect_ukey array_key_exists array_keys array_map '+
						'array_merge array_merge_recursive array_multisort array_pad array_pop array_product '+
						'array_push array_rand array_reduce array_reverse array_search array_shift '+
						'array_slice array_splice array_sum array_udiff array_udiff_assoc '+
						'array_udiff_uassoc array_uintersect array_uintersect_assoc '+
						'array_uintersect_uassoc array_unique array_unshift array_values array_walk '+
						'array_walk_recursive atan atan2 atanh base64_decode base64_encode base_convert '+
						'basename bcadd bccomp bcdiv bcmod bcmul bindec bindtextdomain bzclose bzcompress '+
						'bzdecompress bzerrno bzerror bzerrstr bzflush bzopen bzread bzwrite ceil chdir '+
						'checkdate checkdnsrr chgrp chmod chop chown chr chroot chunk_split class_exists '+
						'closedir closelog copy cos cosh count count_chars date decbin dechex decoct '+
						'deg2rad delete ebcdic2ascii echo empty end ereg ereg_replace eregi eregi_replace error_log '+
						'error_reporting escapeshellarg escapeshellcmd eval exec exit exp explode extension_loaded '+
						'feof fflush fgetc fgetcsv fgets fgetss file_exists file_get_contents file_put_contents '+
						'fileatime filectime filegroup fileinode filemtime fileowner fileperms filesize filetype '+
						'floatval flock floor flush fmod fnmatch fopen fpassthru fprintf fputcsv fputs fread fscanf '+
						'fseek fsockopen fstat ftell ftok getallheaders getcwd getdate getenv gethostbyaddr gethostbyname '+
						'gethostbynamel getimagesize getlastmod getmxrr getmygid getmyinode getmypid getmyuid getopt '+
						'getprotobyname getprotobynumber getrandmax getrusage getservbyname getservbyport gettext '+
						'gettimeofday gettype glob gmdate gmmktime ini_alter ini_get ini_get_all ini_restore ini_set '+
						'interface_exists intval ip2long is_a is_array is_bool is_callable is_dir is_double '+
						'is_executable is_file is_finite is_float is_infinite is_int is_integer is_link is_long '+
						'is_nan is_null is_numeric is_object is_readable is_real is_resource is_scalar is_soap_fault '+
						'is_string is_subclass_of is_uploaded_file is_writable is_writeable mkdir mktime nl2br '+
						'parse_ini_file parse_str parse_url passthru pathinfo print readlink realpath rewind rewinddir rmdir '+
						'round str_ireplace str_pad str_repeat str_replace str_rot13 str_shuffle str_split '+
						'str_word_count strcasecmp strchr strcmp strcoll strcspn strftime strip_tags stripcslashes '+
						'stripos stripslashes stristr strlen strnatcasecmp strnatcmp strncasecmp strncmp strpbrk '+
						'strpos strptime strrchr strrev strripos strrpos strspn strstr strtok strtolower strtotime '+
						'strtoupper strtr strval substr substr_compare';

		var keywords =	'abstract and array as break case catch cfunction class clone const continue declare default die do ' +
						'else elseif enddeclare endfor endforeach endif endswitch endwhile extends final for foreach ' +
						'function include include_once global goto if implements interface instanceof namespace new ' +
						'old_function or private protected public return require require_once static switch ' +
						'throw try use var while xor ';
		
		var constants	= '__FILE__ __LINE__ __METHOD__ __FUNCTION__ __CLASS__';

		this.regexList = [
			{ regex: SyntaxHighlighter.regexLib.singleLineCComments,	css: 'comments' },			// one line comments
			{ regex: SyntaxHighlighter.regexLib.multiLineCComments,		css: 'comments' },			// multiline comments
			{ regex: SyntaxHighlighter.regexLib.doubleQuotedString,		css: 'string' },			// double quoted strings
			{ regex: SyntaxHighlighter.regexLib.singleQuotedString,		css: 'string' },			// single quoted strings
			{ regex: /\$\w+/g,											css: 'variable' },			// variables
			{ regex: new RegExp(this.getKeywords(funcs), 'gmi'),		css: 'functions' },			// common functions
			{ regex: new RegExp(this.getKeywords(constants), 'gmi'),	css: 'constants' },			// constants
			{ regex: new RegExp(this.getKeywords(keywords), 'gm'),		css: 'keyword' }			// keyword
			];

		this.forHtmlScript(SyntaxHighlighter.regexLib.phpScriptTags);
	};

	Brush.prototype	= new SyntaxHighlighter.Highlighter();
	Brush.aliases	= ['php'];

	SyntaxHighlighter.brushes.Php = Brush;

	// CommonJS
	typeof(exports) != 'undefined' ? exports.Brush = Brush : null;
})();

/**
 * SyntaxHighlighter
 * http://alexgorbatchev.com/SyntaxHighlighter
 *
 * SyntaxHighlighter is donationware. If you are using it, please donate.
 * http://alexgorbatchev.com/SyntaxHighlighter/donate.html
 *
 * @version
 * 3.0.83 (July 02 2010)
 * 
 * @copyright
 * Copyright (C) 2004-2010 Alex Gorbatchev.
 *
 * @license
 * Dual licensed under the MIT and GPL licenses.
 */
;(function()
{
	// CommonJS
	typeof(require) != 'undefined' ? SyntaxHighlighter = require('shCore').SyntaxHighlighter : null;

	function Brush()
	{
		function process(match, regexInfo)
		{
			var constructor = SyntaxHighlighter.Match,
				code = match[0],
				tag = new XRegExp('(&lt;|<)[\\s\\/\\?]*(?<name>[:\\w-\\.]+)', 'xg').exec(code),
				result = []
				;
		
			if (match.attributes != null) 
			{
				var attributes,
					regex = new XRegExp('(?<name> [\\w:\\-\\.]+)' +
										'\\s*=\\s*' +
										'(?<value> ".*?"|\'.*?\'|\\w+)',
										'xg');

				while ((attributes = regex.exec(code)) != null) 
				{
					result.push(new constructor(attributes.name, match.index + attributes.index, 'color1'));
					result.push(new constructor(attributes.value, match.index + attributes.index + attributes[0].indexOf(attributes.value), 'string'));
				}
			}

			if (tag != null)
				result.push(
					new constructor(tag.name, match.index + tag[0].indexOf(tag.name), 'keyword')
				);

			return result;
		}
	
		this.regexList = [
			{ regex: new XRegExp('(\\&lt;|<)\\!\\[[\\w\\s]*?\\[(.|\\s)*?\\]\\](\\&gt;|>)', 'gm'),			css: 'color2' },	// <![ ... [ ... ]]>
			{ regex: SyntaxHighlighter.regexLib.xmlComments,												css: 'comments' },	// <!-- ... -->
			{ regex: new XRegExp('(&lt;|<)[\\s\\/\\?]*(\\w+)(?<attributes>.*?)[\\s\\/\\?]*(&gt;|>)', 'sg'), func: process }
		];
	};

	Brush.prototype	= new SyntaxHighlighter.Highlighter();
	Brush.aliases	= ['xml', 'xhtml', 'xslt', 'html'];

	SyntaxHighlighter.brushes.Xml = Brush;

	// CommonJS
	typeof(exports) != 'undefined' ? exports.Brush = Brush : null;
})();

/**
 *                                                        ____   _____
 *  Dynarch Calendar -- JSCal2, version 1.9               \  /_  /   /
 *  Built at 2011/03/13 10:28 GMT                          \  / /   /
 *                                                          \/ /_  /
 *  (c) Dynarch.com 2009                                     \  / /
 *  All rights reserved.                                       / /
 *  Visit www.dynarch.com/projects/calendar for details        \/
 *
 */
Calendar=function(){function bm(a){typeof a=="string"&&(a=document.getElementById(a));return a}function bk(a,b,c){for(c=0;c<a.length;++c)b(a[c])}function bj(){var a=document.documentElement,b=document.body;return{x:a.scrollLeft||b.scrollLeft,y:a.scrollTop||b.scrollTop,w:a.clientWidth||window.innerWidth||b.clientWidth,h:a.clientHeight||window.innerHeight||b.clientHeight}}function bi(a){var b=0,c=0,d=/^div$/i.test(a.tagName),e,f;d&&a.scrollLeft&&(b=a.scrollLeft),d&&a.scrollTop&&(c=a.scrollTop),e={x:a.offsetLeft-b,y:a.offsetTop-c},a.offsetParent&&(f=bi(a.offsetParent),e.x+=f.x,e.y+=f.y);return e}function bh(a,b){var c=e?a.clientX+document.body.scrollLeft:a.pageX,d=e?a.clientY+document.body.scrollTop:a.pageY;b&&(c-=b.x,d-=b.y);return{x:c,y:d}}function bg(a,b){var c=a.style;b!=null&&(c.display=b?"":"none");return c.display!="none"}function bf(a,b){b===""?e?a.style.filter="":a.style.opacity="":b!=null?e?a.style.filter="alpha(opacity="+b*100+")":a.style.opacity=b:e?/alpha\(opacity=([0-9.])+\)/.test(a.style.opacity)&&(b=parseFloat(RegExp.$1)/100):b=parseFloat(a.style.opacity);return b}function bd(a,b,c){function h(){var b=a.len;a.onUpdate(c/b,d),c==b&&g(),++c}function g(){b&&(clearInterval(b),b=null),a.onStop(c/a.len,d)}function f(){b&&g(),c=0,b=setInterval(h,1e3/a.fps)}function d(a,b,c,d){return d?c+a*(b-c):b+a*(c-b)}a=U(a,{fps:50,len:15,onUpdate:bl,onStop:bl}),e&&(a.len=Math.round(a.len/2)),f();return{start:f,stop:g,update:h,args:a,map:d}}function bc(a,b){if(!b(a))for(var c=a.firstChild;c;c=c.nextSibling)c.nodeType==1&&bc(c,b)}function bb(a,b){var c=ba(arguments,2);return b==undefined?function(){return a.apply(this,c.concat(ba(arguments)))}:function(){return a.apply(b,c.concat(ba(arguments)))}}function ba(a,b){b==null&&(b=0);var c,d,e;try{c=Array.prototype.slice.call(a,b)}catch(f){c=Array(a.length-b);for(d=b,e=0;d<a.length;++d,++e)c[e]=a[d]}return c}function _(a,b,c){var d=null;document.createElementNS?d=document.createElementNS("http://www.w3.org/1999/xhtml",a):d=document.createElement(a),b&&(d.className=b),c&&c.appendChild(d);return d}function $(a,b,c){if(b instanceof Array)for(var d=b.length;--d>=0;)$(a,b[d],c);else Y(b,c,a?c:null);return a}function Z(a,b){return Y(a,b,b)}function Y(a,b,c){if(a){var d=a.className.replace(/^\s+|\s+$/,"").split(/\x20/),e=[],f;for(f=d.length;f>0;)d[--f]!=b&&e.push(d[f]);c&&e.push(c),a.className=e.join(" ")}return c}function X(a){a=a||window.event,e?(a.cancelBubble=!0,a.returnValue=!1):(a.preventDefault(),a.stopPropagation());return!1}function W(a,b,c,d){if(a instanceof Array)for(var f=a.length;--f>=0;)W(a[f],b,c);else if(typeof b=="object")for(var f in b)b.hasOwnProperty(f)&&W(a,f,b[f],c);else a.removeEventListener?a.removeEventListener(b,c,e?!0:!!d):a.detachEvent?a.detachEvent("on"+b,c):a["on"+b]=null}function V(a,b,c,d){if(a instanceof Array)for(var f=a.length;--f>=0;)V(a[f],b,c,d);else if(typeof b=="object")for(var f in b)b.hasOwnProperty(f)&&V(a,f,b[f],c);else a.addEventListener?a.addEventListener(b,c,e?!0:!!d):a.attachEvent?a.attachEvent("on"+b,c):a["on"+b]=c}function U(a,b,c,d){d={};for(c in b)b.hasOwnProperty(c)&&(d[c]=b[c]);for(c in a)a.hasOwnProperty(c)&&(d[c]=a[c]);return d}function T(a){if(/\S/.test(a)){a=a.toLowerCase();function b(b){for(var c=b.length;--c>=0;)if(b[c].toLowerCase().indexOf(a)==0)return c+1}return b(L("smn"))||b(L("mn"))}}function S(a){if(a){if(typeof a=="number")return P(a);if(!(a instanceof Date)){var b=a.split(/-/);return new Date(parseInt(b[0],10),parseInt(b[1],10)-1,parseInt(b[2],10),12,0,0,0)}}return a}function R(a,b){var c=a.getMonth(),d=a.getDate(),e=a.getFullYear(),f=M(a),g=a.getDay(),h=a.getHours(),i=h>=12,j=i?h-12:h,k=N(a),l=a.getMinutes(),m=a.getSeconds(),n=/%./g,o;j===0&&(j=12),o={"%a":L("sdn")[g],"%A":L("dn")[g],"%b":L("smn")[c],"%B":L("mn")[c],"%C":1+Math.floor(e/100),"%d":d<10?"0"+d:d,"%e":d,"%H":h<10?"0"+h:h,"%I":j<10?"0"+j:j,"%j":k<10?"00"+k:k<100?"0"+k:k,"%k":h,"%l":j,"%m":c<9?"0"+(1+c):1+c,"%o":1+c,"%M":l<10?"0"+l:l,"%n":"\n","%p":i?"PM":"AM","%P":i?"pm":"am","%s":Math.floor(a.getTime()/1e3),"%S":m<10?"0"+m:m,"%t":"\t","%U":f<10?"0"+f:f,"%W":f<10?"0"+f:f,"%V":f<10?"0"+f:f,"%u":g+1,"%w":g,"%y":(""+e).substr(2,2),"%Y":e,"%%":"%"};return b.replace(n,function(a){return o.hasOwnProperty(a)?o[a]:a})}function Q(a,b,c){var d=a.getFullYear(),e=a.getMonth(),f=a.getDate(),g=b.getFullYear(),h=b.getMonth(),i=b.getDate();return d<g?-3:d>g?3:e<h?-2:e>h?2:c?0:f<i?-1:f>i?1:0}function P(a,b,c,d,e){if(!(a instanceof Date)){a=parseInt(a,10);var f=Math.floor(a/1e4);a=a%1e4;var g=Math.floor(a/100);a=a%100,a=new Date(f,g-1,a,b==null?12:b,c==null?0:c,d==null?0:d,e==null?0:e)}return a}function O(a){if(a instanceof Date)return 1e4*a.getFullYear()+100*(a.getMonth()+1)+a.getDate();if(typeof a=="string")return parseInt(a,10);return a}function N(a){a=new Date(a.getFullYear(),a.getMonth(),a.getDate(),12,0,0);var b=new Date(a.getFullYear(),0,1,12,0,0),c=a-b;return Math.floor(c/864e5)}function M(a){a=new Date(a.getFullYear(),a.getMonth(),a.getDate(),12,0,0);var b=a.getDay();a.setDate(a.getDate()-(b+6)%7+3);var c=a.valueOf();a.setMonth(0),a.setDate(4);return Math.round((c-a.valueOf())/6048e5)+1}function L(a,b){var c=i.__.data[a];b&&typeof c=="string"&&(c=K(c,b));return c}function K(a,b){return a.replace(/\$\{([^:\}]+)(:[^\}]+)?\}/g,function(a,c,d){var e=b[c],f;d&&(f=d.substr(1).split(/\s*\|\s*/),e=(e>=f.length?f[f.length-1]:f[e]).replace(/##?/g,function(a){return a.length==2?"#":e}));return e})}function J(b){if(!this._menuAnim){b=b||window.event;var c=b.target||b.srcElement,d=c.getAttribute("dyc-btn"),e=b.keyCode,f=b.charCode||e,g=H[e];if("year"==d&&e==13){var h=new Date(this.date);h.setDate(1),h.setFullYear(this._getInputYear()),this.moveTo(h,!0),z(this,!1);return X(b)}if(this._menuVisible){if(e==27){z(this,!1);return X(b)}}else{b.ctrlKey||(g=null),g==null&&!b.ctrlKey&&(g=I[e]),e==36&&(g=0);if(g!=null){y(this,g);return X(b)}f=String.fromCharCode(f).toLowerCase();var i=this.els.yearInput,j=this.selection;if(f==" "){z(this,!0),this.focus(),i.focus(),i.select();return X(b)}if(f>="0"&&f<="9"){z(this,!0),this.focus(),i.value=f,i.focus();return X(b)}var k=L("mn"),l=b.shiftKey?-1:this.date.getMonth(),m=0,n;while(++m<12){n=k[(l+m)%12].toLowerCase();if(n.indexOf(f)==0){var h=new Date(this.date);h.setDate(1),h.setMonth((l+m)%12),this.moveTo(h,!0);return X(b)}}if(e>=37&&e<=40){var h=this._lastHoverDate;if(!h&&!j.isEmpty()){h=e<39?j.getFirstDate():j.getLastDate();if(h<this._firstDateVisible||h>this._lastDateVisible)h=null}if(!h)h=e<39?this._lastDateVisible:this._firstDateVisible;else{var o=h;h=P(h);var l=100;while(l-->0){switch(e){case 37:h.setDate(h.getDate()-1);break;case 38:h.setDate(h.getDate()-7);break;case 39:h.setDate(h.getDate()+1);break;case 40:h.setDate(h.getDate()+7)}if(!this.isDisabled(h))break}h=O(h),(h<this._firstDateVisible||h>this._lastDateVisible)&&this.moveTo(h)}Y(this._getDateDiv(o),Z(this._getDateDiv(h),"DynarchCalendar-hover-date")),this._lastHoverDate=h;return X(b)}if(e==13&&this._lastHoverDate){j.type==a.SEL_MULTIPLE&&(b.shiftKey||b.ctrlKey)?(b.shiftKey&&this._selRangeStart&&(j.clear(!0),j.selectRange(this._selRangeStart,this._lastHoverDate)),b.ctrlKey&&j.set(this._selRangeStart=this._lastHoverDate,!0)):j.reset(this._selRangeStart=this._lastHoverDate);return X(b)}e==27&&!this.args.cont&&this.hide()}}}function G(){this.refresh();var a=this.inputField,b=this.selection;if(a){var c=b.print(this.dateFormat);/input|textarea/i.test(a.tagName)?a.value=c:a.innerHTML=c}this.callHooks("onSelect",this,b)}function F(a){a=a||window.event;var b=C(a);if(b){var c=b.getAttribute("dyc-btn"),d=b.getAttribute("dyc-type"),e=a.wheelDelta?a.wheelDelta/120:-a.detail/3;e=e<0?-1:e>0?1:0,this.args.reverseWheel&&(e=-e);if(/^(time-(hour|min))/.test(d)){switch(RegExp.$1){case"time-hour":this.setHours(this.getHours()+e);break;case"time-min":this.setMinutes(this.getMinutes()+this.args.minuteStep*e)}X(a)}else/Y/i.test(c)&&(e*=2),y(this,-e),X(a)}}function E(a,b){b=b||window.event;var c=C(b);if(c){var d=c.getAttribute("dyc-type");if(d&&!c.getAttribute("disabled"))if(!a||!this._bodyAnim||d!="date"){var e=c.getAttribute("dyc-cls");e=e?D(e,0):"DynarchCalendar-hover-"+d,(d!="date"||this.selection.type)&&$(a,c,e),d=="date"&&($(a,c.parentNode.parentNode,"DynarchCalendar-hover-week"),this._showTooltip(c.getAttribute("dyc-date"))),/^time-hour/.test(d)&&$(a,this.els.timeHour,"DynarchCalendar-hover-time"),/^time-min/.test(d)&&$(a,this.els.timeMinute,"DynarchCalendar-hover-time"),Y(this._getDateDiv(this._lastHoverDate),"DynarchCalendar-hover-date"),this._lastHoverDate=null}}a||this._showTooltip()}function D(a,b){return"DynarchCalendar-"+a.split(/,/)[b]}function C(a){var b=a.target||a.srcElement,c=b;while(b&&b.getAttribute&&!b.getAttribute("dyc-type"))b=b.parentNode;return b.getAttribute&&b||c}function B(a){a=a||window.event;var b=this.els.topCont.style,c=bh(a,this._mouseDiff);b.left=c.x+"px",b.top=c.y+"px"}function A(b,c){c=c||window.event;var d=C(c);if(d&&!d.getAttribute("disabled")){var f=d.getAttribute("dyc-btn"),g=d.getAttribute("dyc-type"),h=d.getAttribute("dyc-date"),i=this.selection,j,k={mouseover:X,mousemove:X,mouseup:function(a){var b=d.getAttribute("dyc-cls");b&&Y(d,D(b,1)),clearTimeout(j),W(document,k,!0),k=null}};if(b){setTimeout(bb(this.focus,this),1);var l=d.getAttribute("dyc-cls");l&&Z(d,D(l,1));if("menu"==f)this.toggleMenu();else if(d&&/^[+-][MY]$/.test(f))if(y(this,f)){var m=bb(function(){y(this,f,!0)?j=setTimeout(m,40):(k.mouseup(),y(this,f))},this);j=setTimeout(m,350),V(document,k,!0)}else k.mouseup();else if("year"==f)this.els.yearInput.focus(),this.els.yearInput.select();else if(g=="time-am")V(document,k,!0);else if(/^time/.test(g)){var m=bb(function(a){w.call(this,a),j=setTimeout(m,100)},this,g);w.call(this,g),j=setTimeout(m,350),V(document,k,!0)}else h&&i.type&&(i.type==a.SEL_MULTIPLE?c.shiftKey&&this._selRangeStart?i.selectRange(this._selRangeStart,h):(!c.ctrlKey&&!i.isSelected(h)&&i.clear(!0),i.set(h,!0),this._selRangeStart=h):(i.set(h),this.moveTo(P(h),2)),d=this._getDateDiv(h),E.call(this,!0,{target:d})),V(document,k,!0);e&&k&&/dbl/i.test(c.type)&&k.mouseup(),!this.args.fixed&&/^(DynarchCalendar-(topBar|bottomBar|weekend|weekNumber|menu(-sep)?))?$/.test(d.className)&&!this.args.cont&&(k.mousemove=bb(B,this),this._mouseDiff=bh(c,bi(this.els.topCont)),V(document,k,!0))}else if("today"==f)!this._menuVisible&&i.type==a.SEL_SINGLE&&i.set(new Date),this.moveTo(new Date,!0),z(this,!1);else if(/^m([0-9]+)/.test(f)){var h=new Date(this.date);h.setDate(1),h.setMonth(RegExp.$1),h.setFullYear(this._getInputYear()),this.moveTo(h,!0),z(this,!1)}else g=="time-am"&&this.setHours(this.getHours()+12);e||X(c)}}function z(a,b){a._menuVisible=b,$(b,a.els.title,"DynarchCalendar-pressed-title");var c=a.els.menu;f&&(c.style.height=a.els.main.offsetHeight+"px");if(!a.args.animation)bg(c,b),a.focused&&a.focus();else{a._menuAnim&&a._menuAnim.stop();var d=a.els.main.offsetHeight;f&&(c.style.width=a.els.topBar.offsetWidth+"px"),b&&(c.firstChild.style.marginTop=-d+"px",a.args.opacity>0&&bf(c,0),bg(c,!0)),a._menuAnim=bd({onUpdate:function(e,f){c.firstChild.style.marginTop=f(be.accel_b(e),-d,0,!b)+"px",a.args.opacity>0&&bf(c,f(be.accel_b(e),0,.85,!b))},onStop:function(){a.args.opacity>0&&bf(c,.85),c.firstChild.style.marginTop="",a._menuAnim=null,b||(bg(c,!1),a.focused&&a.focus())}})}}function y(a,b,c){this._bodyAnim&&this._bodyAnim.stop();var d;if(b!=0){d=new Date(a.date),d.setDate(1);switch(b){case"-Y":case-2:d.setFullYear(d.getFullYear()-1);break;case"+Y":case 2:d.setFullYear(d.getFullYear()+1);break;case"-M":case-1:d.setMonth(d.getMonth()-1);break;case"+M":case 1:d.setMonth(d.getMonth()+1)}}else d=new Date;return a.moveTo(d,!c)}function w(a){switch(a){case"time-hour+":this.setHours(this.getHours()+1);break;case"time-hour-":this.setHours(this.getHours()-1);break;case"time-min+":this.setMinutes(this.getMinutes()+this.args.minuteStep);break;case"time-min-":this.setMinutes(this.getMinutes()-this.args.minuteStep);break;default:return}}function v(){this._bluringTimeout=setTimeout(bb(u,this),50)}function u(){this.focused=!1,Y(this.els.main,"DynarchCalendar-focused"),this._menuVisible&&z(this,!1),this.args.cont||this.hide(),this.callHooks("onBlur",this)}function t(){this._bluringTimeout&&clearTimeout(this._bluringTimeout),this.focused=!0,Z(this.els.main,"DynarchCalendar-focused"),this.callHooks("onFocus",this)}function s(a){var b=_("div"),c=a.els={},d={mousedown:bb(A,a,!0),mouseup:bb(A,a,!1),mouseover:bb(E,a,!0),mouseout:bb(E,a,!1),keypress:bb(J,a)};a.args.noScroll||(d[g?"DOMMouseScroll":"mousewheel"]=bb(F,a)),e&&(d.dblclick=d.mousedown,d.keydown=d.keypress),b.innerHTML=m(a),bc(b.firstChild,function(a){var b=r[a.className];b&&(c[b]=a),e&&a.setAttribute("unselectable","on")}),V(c.main,d),V([c.focusLink,c.yearInput],a._focusEvents={focus:bb(t,a),blur:bb(v,a)}),a.moveTo(a.date,!1),a.setTime(null,!0);return c.topCont}function q(a){function d(){c.showTime&&(b.push("<td>"),p(a,b),b.push("</td>"))}var b=[],c=a.args;b.push("<table",j," style='width:100%'><tr>"),c.timePos=="left"&&d(),c.bottomBar&&(b.push("<td>"),b.push("<table",j,"><tr><td>","<div dyc-btn='today' dyc-cls='hover-bottomBar-today,pressed-bottomBar-today' dyc-type='bottomBar-today' ","class='DynarchCalendar-bottomBar-today'>",L("today"),"</div>","</td></tr></table>"),b.push("</td>")),c.timePos=="right"&&d(),b.push("</tr></table>");return b.join("")}function p(a,b){b.push("<table class='DynarchCalendar-time'"+j+"><tr>","<td rowspan='2'><div dyc-type='time-hour' dyc-cls='hover-time,pressed-time' class='DynarchCalendar-time-hour'></div></td>","<td dyc-type='time-hour+' dyc-cls='hover-time,pressed-time' class='DynarchCalendar-time-up'></td>","<td rowspan='2' class='DynarchCalendar-time-sep'></td>","<td rowspan='2'><div dyc-type='time-min' dyc-cls='hover-time,pressed-time' class='DynarchCalendar-time-minute'></div></td>","<td dyc-type='time-min+' dyc-cls='hover-time,pressed-time' class='DynarchCalendar-time-up'></td>"),a.args.showTime==12&&b.push("<td rowspan='2' class='DynarchCalendar-time-sep'></td>","<td rowspan='2'><div class='DynarchCalendar-time-am' dyc-type='time-am' dyc-cls='hover-time,pressed-time'></div></td>"),b.push("</tr><tr>","<td dyc-type='time-hour-' dyc-cls='hover-time,pressed-time' class='DynarchCalendar-time-down'></td>","<td dyc-type='time-min-' dyc-cls='hover-time,pressed-time' class='DynarchCalendar-time-down'></td>","</tr></table>")}function o(a){var b=["<table height='100%'",j,"><tr><td>","<table style='margin-top: 1.5em'",j,">","<tr><td colspan='3'><input dyc-btn='year' class='DynarchCalendar-menu-year' size='6' value='",a.date.getFullYear(),"' /></td></tr>","<tr><td><div dyc-type='menubtn' dyc-cls='hover-navBtn,pressed-navBtn' dyc-btn='today'>",L("goToday"),"</div></td></tr>","</table>","<p class='DynarchCalendar-menu-sep'>&nbsp;</p>","<table class='DynarchCalendar-menu-mtable'",j,">"],c=L("smn"),d=0,e=b.length,f;while(d<12){b[e++]="<tr>";for(f=4;--f>0;)b[e++]="<td><div dyc-type='menubtn' dyc-cls='hover-navBtn,pressed-navBtn' dyc-btn='m"+d+"' class='DynarchCalendar-menu-month'>"+c[d++]+"</div></td>";b[e++]="</tr>"}b[e++]="</table></td></tr></table>";return b.join("")}function n(a){return"<div unselectable='on'>"+R(a.date,a.args.titleFormat)+"</div>"}function m(a){var b=["<table class='DynarchCalendar-topCont'",j,"><tr><td>","<div class='DynarchCalendar'>",e?"<a class='DynarchCalendar-focusLink' href='#'></a>":"<button class='DynarchCalendar-focusLink'></button>","<div class='DynarchCalendar-topBar'>","<div dyc-type='nav' dyc-btn='-Y' dyc-cls='hover-navBtn,pressed-navBtn' ","class='DynarchCalendar-navBtn DynarchCalendar-prevYear'><div></div></div>","<div dyc-type='nav' dyc-btn='+Y' dyc-cls='hover-navBtn,pressed-navBtn' ","class='DynarchCalendar-navBtn DynarchCalendar-nextYear'><div></div></div>","<div dyc-type='nav' dyc-btn='-M' dyc-cls='hover-navBtn,pressed-navBtn' ","class='DynarchCalendar-navBtn DynarchCalendar-prevMonth'><div></div></div>","<div dyc-type='nav' dyc-btn='+M' dyc-cls='hover-navBtn,pressed-navBtn' ","class='DynarchCalendar-navBtn DynarchCalendar-nextMonth'><div></div></div>","<table class='DynarchCalendar-titleCont'",j,"><tr><td>","<div dyc-type='title' dyc-btn='menu' dyc-cls='hover-title,pressed-title' class='DynarchCalendar-title'>",n(a),"</div></td></tr></table>","<div class='DynarchCalendar-dayNames'>",k(a),"</div>","</div>","<div class='DynarchCalendar-body'></div>"];(a.args.bottomBar||a.args.showTime)&&b.push("<div class='DynarchCalendar-bottomBar'>",q(a),"</div>"),b.push("<div class='DynarchCalendar-menu' style='display: none'>",o(a),"</div>","<div class='DynarchCalendar-tooltip'></div>","</div>","</td></tr></table>");return b.join("")}function l(a,b,c){b=b||a.date,c=c||a.fdow,b=new Date(b.getFullYear(),b.getMonth(),b.getDate(),12,0,0,0);var d=b.getMonth(),e=[],f=0,g=a.args.weekNumbers;b.setDate(1);var h=(b.getDay()-c)%7;h<0&&(h+=7),b.setDate(0-h),b.setDate(b.getDate()+1);var i=new Date,k=i.getDate(),l=i.getMonth(),m=i.getFullYear();e[f++]="<table class='DynarchCalendar-bodyTable'"+j+">";for(var n=0;n<6;++n){e[f++]="<tr class='DynarchCalendar-week",n==0&&(e[f++]=" DynarchCalendar-first-row"),n==5&&(e[f++]=" DynarchCalendar-last-row"),e[f++]="'>",g&&(e[f++]="<td class='DynarchCalendar-first-col'><div class='DynarchCalendar-weekNumber'>"+M(b)+"</div></td>");for(var o=0;o<7;++o){var p=b.getDate(),q=b.getMonth(),r=b.getFullYear(),s=1e4*r+100*(q+1)+p,t=a.selection.isSelected(s),u=a.isDisabled(b);e[f++]="<td class='",o==0&&!g&&(e[f++]=" DynarchCalendar-first-col"),o==0&&n==0&&(a._firstDateVisible=s),o==6&&(e[f++]=" DynarchCalendar-last-col",n==5&&(a._lastDateVisible=s)),t&&(e[f++]=" DynarchCalendar-td-selected"),e[f++]="'><div dyc-type='date' unselectable='on' dyc-date='"+s+"' ",u&&(e[f++]="disabled='1' "),e[f++]="class='DynarchCalendar-day",L("weekend").indexOf(b.getDay())>=0&&(e[f++]=" DynarchCalendar-weekend"),q!=d&&(e[f++]=" DynarchCalendar-day-othermonth"),p==k&&q==l&&r==m&&(e[f++]=" DynarchCalendar-day-today"),u&&(e[f++]=" DynarchCalendar-day-disabled"),t&&(e[f++]=" DynarchCalendar-day-selected"),u=a.args.dateInfo(b),u&&u.klass&&(e[f++]=" "+u.klass),e[f++]="'>"+p+"</div></td>",b=new Date(r,q,p+1,12,0,0,0)}e[f++]="</tr>"}e[f++]="</table>";return e.join("")}function k(a){var b=["<table",j,"><tr>"],c=0;a.args.weekNumbers&&b.push("<td><div class='DynarchCalendar-weekNumber'>",L("wk"),"</div></td>");while(c<7){var d=(c++ +a.fdow)%7;b.push("<td><div",L("weekend").indexOf(d)>=0?" class='DynarchCalendar-weekend'>":">",L("sdn")[d],"</div></td>")}b.push("</tr></table>");return b.join("")}function a(b){b=b||{},this.args=b=U(b,{animation:!f,cont:null,bottomBar:!0,date:!0,fdow:L("fdow"),min:null,max:null,reverseWheel:!1,selection:[],selectionType:a.SEL_SINGLE,weekNumbers:!1,align:"Bl/ / /T/r",inputField:null,trigger:null,dateFormat:"%Y-%m-%d",fixed:!1,opacity:e?1:3,titleFormat:"%b %Y",showTime:!1,timePos:"right",time:!0,minuteStep:5,noScroll:!1,disabled:bl,checkRange:!1,dateInfo:bl,onChange:bl,onSelect:bl,onTimeChange:bl,onFocus:bl,onBlur:bl}),this.handlers={};var c=this,d=new Date;b.min=S(b.min),b.max=S(b.max),b.date===!0&&(b.date=d),b.time===!0&&(b.time=d.getHours()*100+Math.floor(d.getMinutes()/b.minuteStep)*b.minuteStep),this.date=S(b.date),this.time=b.time,this.fdow=b.fdow,bk("onChange onSelect onTimeChange onFocus onBlur".split(/\s+/),function(a){var d=b[a];d instanceof Array||(d=[d]),c.handlers[a]=d}),this.selection=new a.Selection(b.selection,b.selectionType,G,this);var g=s(this);b.cont&&bm(b.cont).appendChild(g),b.trigger&&this.manageFields(b.trigger,b.inputField,b.dateFormat)}var b=navigator.userAgent,c=/opera/i.test(b),d=/Konqueror|Safari|KHTML/i.test(b),e=/msie/i.test(b)&&!c&&!/mac_powerpc/i.test(b),f=e&&/msie 6/i.test(b),g=/gecko/i.test(b)&&!d&&!c&&!e,h=a.prototype,i=a.I18N={};a.SEL_NONE=0,a.SEL_SINGLE=1,a.SEL_MULTIPLE=2,a.SEL_WEEK=3,a.dateToInt=O,a.intToDate=P,a.printDate=R,a.formatString=K,a.i18n=L,a.LANG=function(a,b,c){i.__=i[a]={name:b,data:c}},a.setup=function(b){return new a(b)},h.moveTo=function(a,b){var c=this;a=S(a);var d=Q(a,c.date,!0),e,f=c.args,g=f.min&&Q(a,f.min),h=f.max&&Q(a,f.max);f.animation||(b=!1),$(g!=null&&g<=1,[c.els.navPrevMonth,c.els.navPrevYear],"DynarchCalendar-navDisabled"),$(h!=null&&h>=-1,[c.els.navNextMonth,c.els.navNextYear],"DynarchCalendar-navDisabled"),g<-1&&(a=f.min,e=1,d=0),h>1&&(a=f.max,e=2,d=0),c.date=a,c.refresh(!!b),c.callHooks("onChange",c,a,b);if(b&&(d!=0||b!=2)){c._bodyAnim&&c._bodyAnim.stop();var i=c.els.body,j=_("div","DynarchCalendar-animBody-"+x[d],i),k=i.firstChild,m=bf(k)||.7,n=e?be.brakes:d==0?be.shake:be.accel_ab2,o=d*d>4,p=o?k.offsetTop:k.offsetLeft,q=j.style,r=o?i.offsetHeight:i.offsetWidth;d<0?r+=p:d>0?r=p-r:(r=Math.round(r/7),e==2&&(r=-r));if(!e&&d!=0){var s=j.cloneNode(!0),t=s.style,u=2*r;s.appendChild(k.cloneNode(!0)),t[o?"marginTop":"marginLeft"]=r+"px",i.appendChild(s)}k.style.visibility="hidden",j.innerHTML=l(c),c._bodyAnim=bd({onUpdate:function(a,b){var f=n(a);if(s)var g=b(f,r,u)+"px";if(e)q[o?"marginTop":"marginLeft"]=b(f,r,0)+"px";else{if(o||d==0)q.marginTop=b(d==0?n(a*a):f,0,r)+"px",d!=0&&(t.marginTop=g);if(!o||d==0)q.marginLeft=b(f,0,r)+"px",d!=0&&(t.marginLeft=g)}c.args.opacity>2&&s&&(bf(s,1-f),bf(j,f))},onStop:function(b){i.innerHTML=l(c,a),c._bodyAnim=null}})}c._lastHoverDate=null;return g>=-1&&h<=1},h.isDisabled=function(a){var b=this.args;return b.min&&Q(a,b.min)<0||b.max&&Q(a,b.max)>0||b.disabled(a)},h.toggleMenu=function(){z(this,!this._menuVisible)},h.refresh=function(a){var b=this.els;a||(b.body.innerHTML=l(this)),b.title.innerHTML=n(this),b.yearInput.value=this.date.getFullYear()},h.redraw=function(){var a=this,b=a.els;a.refresh(),b.dayNames.innerHTML=k(a),b.menu.innerHTML=o(a),b.bottomBar&&(b.bottomBar.innerHTML=q(a)),bc(b.topCont,function(c){var d=r[c.className];d&&(b[d]=c),c.className=="DynarchCalendar-menu-year"?(V(c,a._focusEvents),b.yearInput=c):e&&c.setAttribute("unselectable","on")}),a.setTime(null,!0)},h.setLanguage=function(b){var c=a.setLanguage(b);c&&(this.fdow=c.data.fdow,this.redraw())},a.setLanguage=function(a){var b=i[a];b&&(i.__=b);return b},h.focus=function(){try{this.els[this._menuVisible?"yearInput":"focusLink"].focus()}catch(a){}t.call(this)},h.blur=function(){this.els.focusLink.blur(),this.els.yearInput.blur(),u.call(this)},h.showAt=function(a,b,c){this._showAnim&&this._showAnim.stop(),c=c&&this.args.animation;var d=this.els.topCont,e=this,f=this.els.body.firstChild,g=f.offsetHeight,h=d.style;h.position="absolute",h.left=a+"px",h.top=b+"px",h.zIndex=1e4,h.display="",c&&(f.style.marginTop=-g+"px",this.args.opacity>1&&bf(d,0),this._showAnim=bd({onUpdate:function(a,b){f.style.marginTop=-b(be.accel_b(a),g,0)+"px",e.args.opacity>1&&bf(d,a)},onStop:function(){e.args.opacity>1&&bf(d,""),e._showAnim=null}}))},h.hide=function(){var a=this.els.topCont,b=this,c=this.els.body.firstChild,d=c.offsetHeight,e=bi(a).y;this.args.animation?(this._showAnim&&this._showAnim.stop(),this._showAnim=bd({onUpdate:function(f,g){b.args.opacity>1&&bf(a,1-f),c.style.marginTop=-g(be.accel_b(f),0,d)+"px",a.style.top=g(be.accel_ab(f),e,e-10)+"px"},onStop:function(){a.style.display="none",c.style.marginTop="",b.args.opacity>1&&bf(a,""),b._showAnim=null}})):a.style.display="none",this.inputField=null},h.popup=function(a,b){function h(b){var c={x:i.x,y:i.y};if(!b)return c;/B/.test(b)&&(c.y+=a.offsetHeight),/b/.test(b)&&(c.y+=a.offsetHeight-f.y),/T/.test(b)&&(c.y-=f.y),/l/.test(b)&&(c.x-=f.x-a.offsetWidth),/L/.test(b)&&(c.x-=f.x),/R/.test(b)&&(c.x+=a.offsetWidth),/c/i.test(b)&&(c.x+=(a.offsetWidth-f.x)/2),/m/i.test(b)&&(c.y+=(a.offsetHeight-f.y)/2);return c}a=bm(a),b||(b=this.args.align),b=b.split(/\x2f/);var c=bi(a),d=this.els.topCont,e=d.style,f,g=bj();e.visibility="hidden",e.display="",this.showAt(0,0),document.body.appendChild(d),f={x:d.offsetWidth,y:d.offsetHeight};var i=c;i=h(b[0]),i.y<g.y&&(i.y=c.y,i=h(b[1])),i.x+f.x>g.x+g.w&&(i.x=c.x,i=h(b[2])),i.y+f.y>g.y+g.h&&(i.y=c.y,i=h(b[3])),i.x<g.x&&(i.x=c.x,i=h(b[4])),this.showAt(i.x,i.y,!0),e.visibility="",this.focus()},h.manageFields=function(b,c,d){var e=this;c=bm(c),b=bm(b),/^button$/i.test(b.tagName)&&b.setAttribute("type","button"),V(b,"click",function(){e.inputField=c,e.dateFormat=d;if(e.selection.type==a.SEL_SINGLE){var f,g,h,i;f=/input|textarea/i.test(c.tagName)?c.value:c.innerText||c.textContent,f&&(g=/(^|[^%])%[bBmo]/.exec(d),h=/(^|[^%])%[de]/.exec(d),g&&h&&(i=g.index<h.index),f=Calendar.parseDate(f,i),f&&(e.selection.set(f,!1,!0),e.args.showTime&&(e.setHours(f.getHours()),e.setMinutes(f.getMinutes())),e.moveTo(f)))}e.popup(b)})},h.callHooks=function(a){var b=ba(arguments,1),c=this.handlers[a],d=0;for(;d<c.length;++d)c[d].apply(this,b)},h.addEventListener=function(a,b){this.handlers[a].push(b)},h.removeEventListener=function(a,b){var c=this.handlers[a],d=c.length;while(--d>=0)c[d]===b&&c.splice(d,1)},h.getTime=function(){return this.time},h.setTime=function(a,b){if(this.args.showTime){a=a!=null?a:this.time,this.time=a;var c=this.getHours(),d=this.getMinutes(),e=c<12;this.args.showTime==12&&(c==0&&(c=12),c>12&&(c-=12),this.els.timeAM.innerHTML=L(e?"AM":"PM")),c<10&&(c="0"+c),d<10&&(d="0"+d),this.els.timeHour.innerHTML=c,this.els.timeMinute.innerHTML=d,b||this.callHooks("onTimeChange",this,a)}},h.getHours=function(){return Math.floor(this.time/100)},h.getMinutes=function(){return this.time%100},h.setHours=function(a){a<0&&(a+=24),this.setTime(100*(a%24)+this.time%100)},h.setMinutes=function(a){a<0&&(a+=60),a=Math.floor(a/this.args.minuteStep)*this.args.minuteStep,this.setTime(100*this.getHours()+a%60)},h._getInputYear=function(){var a=parseInt(this.els.yearInput.value,10);isNaN(a)&&(a=this.date.getFullYear());return a},h._showTooltip=function(a){var b="",c,d=this.els.tooltip;a&&(a=P(a),c=this.args.dateInfo(a),c&&c.tooltip&&(b="<div class='DynarchCalendar-tooltipCont'>"+R(a,c.tooltip)+"</div>")),d.innerHTML=b};var j=" align='center' cellspacing='0' cellpadding='0'",r={"DynarchCalendar-topCont":"topCont","DynarchCalendar-focusLink":"focusLink",DynarchCalendar:"main","DynarchCalendar-topBar":"topBar","DynarchCalendar-title":"title","DynarchCalendar-dayNames":"dayNames","DynarchCalendar-body":"body","DynarchCalendar-menu":"menu","DynarchCalendar-menu-year":"yearInput","DynarchCalendar-bottomBar":"bottomBar","DynarchCalendar-tooltip":"tooltip","DynarchCalendar-time-hour":"timeHour","DynarchCalendar-time-minute":"timeMinute","DynarchCalendar-time-am":"timeAM","DynarchCalendar-navBtn DynarchCalendar-prevYear":"navPrevYear","DynarchCalendar-navBtn DynarchCalendar-nextYear":"navNextYear","DynarchCalendar-navBtn DynarchCalendar-prevMonth":"navPrevMonth","DynarchCalendar-navBtn DynarchCalendar-nextMonth":"navNextMonth"},x={"-3":"backYear","-2":"back",0:"now",2:"fwd",3:"fwdYear"},H={37:-1,38:-2,39:1,40:2},I={33:-1,34:1};h._getDateDiv=function(a){var b=null;if(a)try{bc(this.els.body,function(c){if(c.getAttribute("dyc-date")==a)throw b=c})}catch(c){}return b},(a.Selection=function(a,b,c,d){this.type=b,this.sel=a instanceof Array?a:[a],this.onChange=bb(c,d),this.cal=d}).prototype={get:function(){return this.type==a.SEL_SINGLE?this.sel[0]:this.sel},isEmpty:function(){return this.sel.length==0},set:function(b,c,d){var e=this.type==a.SEL_SINGLE;b instanceof Array?(this.sel=b,this.normalize(),d||this.onChange(this)):(b=O(b),e||!this.isSelected(b)?(e?this.sel=[b]:this.sel.splice(this.findInsertPos(b),0,b),this.normalize(),d||this.onChange(this)):c&&this.unselect(b,d))},reset:function(){this.sel=[],this.set.apply(this,arguments)},countDays:function(){var a=0,b=this.sel,c=b.length,d,e,f;while(--c>=0)d=b[c],d instanceof Array&&(e=P(d[0]),f=P(d[1]),a+=Math.round(Math.abs(f.getTime()-e.getTime())/864e5)),++a;return a},unselect:function(a,b){a=O(a);var c=!1;for(var d=this.sel,e=d.length,f;--e>=0;){f=d[e];if(f instanceof Array){if(a>=f[0]&&a<=f[1]){var g=P(a),h=g.getDate();if(a==f[0])g.setDate(h+1),f[0]=O(g),c=!0;else if(a==f[1])g.setDate(h-1),f[1]=O(g),c=!0;else{var i=new Date(g);i.setDate(h+1),g.setDate(h-1),d.splice(e+1,0,[O(i),f[1]]),f[1]=O(g),c=!0}}}else a==f&&(d.splice(e,1),c=!0)}c&&(this.normalize(),b||this.onChange(this))},normalize:function(){this.sel=this.sel.sort(function(a,b){a instanceof Array&&(a=a[0]),b instanceof Array&&(b=b[0]);return a-b});for(var a=this.sel,b=a.length,c,d;--b>=0;){c=a[b];if(c instanceof Array){if(c[0]>c[1]){a.splice(b,1);continue}c[0]==c[1]&&(c=a[b]=c[0])}if(d){var e=d,f=c instanceof Array?c[1]:c;f=P(f),f.setDate(f.getDate()+1),f=O(f);if(f>=e){var g=a[b+1];c instanceof Array&&g instanceof Array?(c[1]=g[1],a.splice(b+1,1)):c instanceof Array?(c[1]=d,a.splice(b+1,1)):g instanceof Array?(g[0]=c,a.splice(b,1)):(a[b]=[c,g],a.splice(b+1,1))}}d=c instanceof Array?c[0]:c}},findInsertPos:function(a){for(var b=this.sel,c=b.length,d;--c>=0;){d=b[c],d instanceof Array&&(d=d[0]);if(d<=a)break}return c+1},clear:function(a){this.sel=[],a||this.onChange(this)},selectRange:function(b,c){b=O(b),c=O(c);if(b>c){var d=b;b=c,c=d}var e=this.cal.args.checkRange;if(!e)return this._do_selectRange(b,c);try{bk((new a.Selection([[b,c]],a.SEL_MULTIPLE,bl)).getDates(),bb(function(a){if(this.isDisabled(a)){e instanceof Function&&e(a,this);throw"OUT"}},this.cal)),this._do_selectRange(b,c)}catch(f){}},_do_selectRange:function(a,b){this.sel.push([a,b]),this.normalize(),this.onChange(this)},isSelected:function(a){for(var b=this.sel.length,c;--b>=0;){c=this.sel[b];if(c instanceof Array&&a>=c[0]&&a<=c[1]||a==c)return!0}return!1},getFirstDate:function(){var a=this.sel[0];a&&a instanceof Array&&(a=a[0]);return a},getLastDate:function(){if(this.sel.length>0){var a=this.sel[this.sel.length-1];a&&a instanceof Array&&(a=a[1]);return a}},print:function(a,b){var c=[],d=0,e,f=this.cal.getHours(),g=this.cal.getMinutes();b||(b=" -> ");while(d<this.sel.length)e=this.sel[d++],e instanceof Array?c.push(R(P(e[0],f,g),a)+b+R(P(e[1],f,g),a)):c.push(R(P(e,f,g),a));return c},getDates:function(a){var b=[],c=0,d,e;while(c<this.sel.length){e=this.sel[c++];if(e instanceof Array){d=P(e[0]),e=e[1];while(O(d)<e)b.push(a?R(d,a):new Date(d)),d.setDate(d.getDate()+1)}else d=P(e);b.push(a?R(d,a):d)}return b}},a.isUnicodeLetter=function(a){return a.toUpperCase()!=a.toLowerCase()},a.parseDate=function(b,c,d){if(!/\S/.test(b))return"";b=b.replace(/^\s+/,"").replace(/\s+$/,""),d=d||new Date;var e=null,f=null,g=null,h=null,i=null,j=null,k=b.match(/([0-9]{1,2}):([0-9]{1,2})(:[0-9]{1,2})?\s*(am|pm)?/i);k&&(h=parseInt(k[1],10),i=parseInt(k[2],10),j=k[3]?parseInt(k[3].substr(1),10):0,b=b.substring(0,k.index)+b.substr(k.index+k[0].length),k[4]&&(k[4].toLowerCase()=="pm"&&h<12?h+=12:k[4].toLowerCase()=="am"&&h>=12&&(h-=12)));var l=function(){function k(a){d.push(a)}function j(){var a="";while(g()&&/[0-9]/.test(g()))a+=f();if(h(g()))return i(a);return parseInt(a,10)}function i(a){while(g()&&h(g()))a+=f();return a}function g(){return b.charAt(c)}function f(){return b.charAt(c++)}var c=0,d=[],e,h=a.isUnicodeLetter;while(c<b.length)e=g(),h(e)?k(i("")):/[0-9]/.test(e)?k(j()):f();return d}(),m=[];for(var n=0;n<l.length;++n){var o=l[n];/^[0-9]{4}$/.test(o)?(e=parseInt(o,10),f==null&&g==null&&c==null&&(c=!0)):/^[0-9]{1,2}$/.test(o)?(o=parseInt(o,10),o<60?o<0||o>12?o>=1&&o<=31&&(g=o):m.push(o):e=o):f==null&&(f=T(o))}m.length<2?m.length==1&&(g==null?g=m.shift():f==null&&(f=m.shift())):c?(f==null&&(f=m.shift()),g==null&&(g=m.shift())):(g==null&&(g=m.shift()),f==null&&(f=m.shift())),e==null&&(e=m.length>0?m.shift():d.getFullYear()),e<30?e+=2e3:e<99&&(e+=1900),f==null&&(f=d.getMonth()+1);return e!=null&&f!=null&&g!=null?new Date(e,f-1,g,h,i,j):null};var be={elastic_b:function(a){return 1-Math.cos(-a*5.5*Math.PI)/Math.pow(2,7*a)},magnetic:function(a){return 1-Math.cos(a*a*a*10.5*Math.PI)/Math.exp(4*a)},accel_b:function(a){a=1-a;return 1-a*a*a*a},accel_a:function(a){return a*a*a},accel_ab:function(a){a=1-a;return 1-Math.sin(a*a*Math.PI/2)},accel_ab2:function(a){return(a/=.5)<1?.5*a*a:-0.5*(--a*(a-2)-1)},brakes:function(a){a=1-a;return 1-Math.sin(a*a*Math.PI)},shake:function(a){return a<.5?-Math.cos(a*11*Math.PI)*a*a:(a=1-a,Math.cos(a*11*Math.PI)*a*a)}},bl=new Function;return a}()
Calendar.isUnicodeLetter = (function(){
        var rx = new RegExp("[\\u0041-\\u005A\\u0061-\\u007A\\u00AA\\u00B5\\u00BA\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02C1\\u02C6-\\u02D1\\u02E0-\\u02E4\\u02EC\\u02EE\\u0370-\\u0374\\u0376\\u0377\\u037A-\\u037D\\u0386\\u0388-\\u038A\\u038C\\u038E-\\u03A1\\u03A3-\\u03F5\\u03F7-\\u0481\\u048A-\\u0523\\u0531-\\u0556\\u0559\\u0561-\\u0587\\u05D0-\\u05EA\\u05F0-\\u05F2\\u0621-\\u064A\\u066E\\u066F\\u0671-\\u06D3\\u06D5\\u06E5\\u06E6\\u06EE\\u06EF\\u06FA-\\u06FC\\u06FF\\u0710\\u0712-\\u072F\\u074D-\\u07A5\\u07B1\\u07CA-\\u07EA\\u07F4\\u07F5\\u07FA\\u0904-\\u0939\\u093D\\u0950\\u0958-\\u0961\\u0971\\u0972\\u097B-\\u097F\\u0985-\\u098C\\u098F\\u0990\\u0993-\\u09A8\\u09AA-\\u09B0\\u09B2\\u09B6-\\u09B9\\u09BD\\u09CE\\u09DC\\u09DD\\u09DF-\\u09E1\\u09F0\\u09F1\\u0A05-\\u0A0A\\u0A0F\\u0A10\\u0A13-\\u0A28\\u0A2A-\\u0A30\\u0A32\\u0A33\\u0A35\\u0A36\\u0A38\\u0A39\\u0A59-\\u0A5C\\u0A5E\\u0A72-\\u0A74\\u0A85-\\u0A8D\\u0A8F-\\u0A91\\u0A93-\\u0AA8\\u0AAA-\\u0AB0\\u0AB2\\u0AB3\\u0AB5-\\u0AB9\\u0ABD\\u0AD0\\u0AE0\\u0AE1\\u0B05-\\u0B0C\\u0B0F\\u0B10\\u0B13-\\u0B28\\u0B2A-\\u0B30\\u0B32\\u0B33\\u0B35-\\u0B39\\u0B3D\\u0B5C\\u0B5D\\u0B5F-\\u0B61\\u0B71\\u0B83\\u0B85-\\u0B8A\\u0B8E-\\u0B90\\u0B92-\\u0B95\\u0B99\\u0B9A\\u0B9C\\u0B9E\\u0B9F\\u0BA3\\u0BA4\\u0BA8-\\u0BAA\\u0BAE-\\u0BB9\\u0BD0\\u0C05-\\u0C0C\\u0C0E-\\u0C10\\u0C12-\\u0C28\\u0C2A-\\u0C33\\u0C35-\\u0C39\\u0C3D\\u0C58\\u0C59\\u0C60\\u0C61\\u0C85-\\u0C8C\\u0C8E-\\u0C90\\u0C92-\\u0CA8\\u0CAA-\\u0CB3\\u0CB5-\\u0CB9\\u0CBD\\u0CDE\\u0CE0\\u0CE1\\u0D05-\\u0D0C\\u0D0E-\\u0D10\\u0D12-\\u0D28\\u0D2A-\\u0D39\\u0D3D\\u0D60\\u0D61\\u0D7A-\\u0D7F\\u0D85-\\u0D96\\u0D9A-\\u0DB1\\u0DB3-\\u0DBB\\u0DBD\\u0DC0-\\u0DC6\\u0E01-\\u0E30\\u0E32\\u0E33\\u0E40-\\u0E46\\u0E81\\u0E82\\u0E84\\u0E87\\u0E88\\u0E8A\\u0E8D\\u0E94-\\u0E97\\u0E99-\\u0E9F\\u0EA1-\\u0EA3\\u0EA5\\u0EA7\\u0EAA\\u0EAB\\u0EAD-\\u0EB0\\u0EB2\\u0EB3\\u0EBD\\u0EC0-\\u0EC4\\u0EC6\\u0EDC\\u0EDD\\u0F00\\u0F40-\\u0F47\\u0F49-\\u0F6C\\u0F88-\\u0F8B\\u1000-\\u102A\\u103F\\u1050-\\u1055\\u105A-\\u105D\\u1061\\u1065\\u1066\\u106E-\\u1070\\u1075-\\u1081\\u108E\\u10A0-\\u10C5\\u10D0-\\u10FA\\u10FC\\u1100-\\u1159\\u115F-\\u11A2\\u11A8-\\u11F9\\u1200-\\u1248\\u124A-\\u124D\\u1250-\\u1256\\u1258\\u125A-\\u125D\\u1260-\\u1288\\u128A-\\u128D\\u1290-\\u12B0\\u12B2-\\u12B5\\u12B8-\\u12BE\\u12C0\\u12C2-\\u12C5\\u12C8-\\u12D6\\u12D8-\\u1310\\u1312-\\u1315\\u1318-\\u135A\\u1380-\\u138F\\u13A0-\\u13F4\\u1401-\\u166C\\u166F-\\u1676\\u1681-\\u169A\\u16A0-\\u16EA\\u1700-\\u170C\\u170E-\\u1711\\u1720-\\u1731\\u1740-\\u1751\\u1760-\\u176C\\u176E-\\u1770\\u1780-\\u17B3\\u17D7\\u17DC\\u1820-\\u1877\\u1880-\\u18A8\\u18AA\\u1900-\\u191C\\u1950-\\u196D\\u1970-\\u1974\\u1980-\\u19A9\\u19C1-\\u19C7\\u1A00-\\u1A16\\u1B05-\\u1B33\\u1B45-\\u1B4B\\u1B83-\\u1BA0\\u1BAE\\u1BAF\\u1C00-\\u1C23\\u1C4D-\\u1C4F\\u1C5A-\\u1C7D\\u1D00-\\u1DBF\\u1E00-\\u1F15\\u1F18-\\u1F1D\\u1F20-\\u1F45\\u1F48-\\u1F4D\\u1F50-\\u1F57\\u1F59\\u1F5B\\u1F5D\\u1F5F-\\u1F7D\\u1F80-\\u1FB4\\u1FB6-\\u1FBC\\u1FBE\\u1FC2-\\u1FC4\\u1FC6-\\u1FCC\\u1FD0-\\u1FD3\\u1FD6-\\u1FDB\\u1FE0-\\u1FEC\\u1FF2-\\u1FF4\\u1FF6-\\u1FFC\\u2071\\u207F\\u2090-\\u2094\\u2102\\u2107\\u210A-\\u2113\\u2115\\u2119-\\u211D\\u2124\\u2126\\u2128\\u212A-\\u212D\\u212F-\\u2139\\u213C-\\u213F\\u2145-\\u2149\\u214E\\u2183\\u2184\\u2C00-\\u2C2E\\u2C30-\\u2C5E\\u2C60-\\u2C6F\\u2C71-\\u2C7D\\u2C80-\\u2CE4\\u2D00-\\u2D25\\u2D30-\\u2D65\\u2D6F\\u2D80-\\u2D96\\u2DA0-\\u2DA6\\u2DA8-\\u2DAE\\u2DB0-\\u2DB6\\u2DB8-\\u2DBE\\u2DC0-\\u2DC6\\u2DC8-\\u2DCE\\u2DD0-\\u2DD6\\u2DD8-\\u2DDE\\u2E2F\\u3005\\u3006\\u3031-\\u3035\\u303B\\u303C\\u3041-\\u3096\\u309D-\\u309F\\u30A1-\\u30FA\\u30FC-\\u30FF\\u3105-\\u312D\\u3131-\\u318E\\u31A0-\\u31B7\\u31F0-\\u31FF\\u3400\\u4DB5\\u4E00\\u9FC3\\uA000-\\uA48C\\uA500-\\uA60C\\uA610-\\uA61F\\uA62A\\uA62B\\uA640-\\uA65F\\uA662-\\uA66E\\uA67F-\\uA697\\uA717-\\uA71F\\uA722-\\uA788\\uA78B\\uA78C\\uA7FB-\\uA801\\uA803-\\uA805\\uA807-\\uA80A\\uA80C-\\uA822\\uA840-\\uA873\\uA882-\\uA8B3\\uA90A-\\uA925\\uA930-\\uA946\\uAA00-\\uAA28\\uAA40-\\uAA42\\uAA44-\\uAA4B\\uAC00\\uD7A3\\uF900-\\uFA2D\\uFA30-\\uFA6A\\uFA70-\\uFAD9\\uFB00-\\uFB06\\uFB13-\\uFB17\\uFB1D\\uFB1F-\\uFB28\\uFB2A-\\uFB36\\uFB38-\\uFB3C\\uFB3E\\uFB40\\uFB41\\uFB43\\uFB44\\uFB46-\\uFBB1\\uFBD3-\\uFD3D\\uFD50-\\uFD8F\\uFD92-\\uFDC7\\uFDF0-\\uFDFB\\uFE70-\\uFE74\\uFE76-\\uFEFC\\uFF21-\\uFF3A\\uFF41-\\uFF5A\\uFF66-\\uFFBE\\uFFC2-\\uFFC7\\uFFCA-\\uFFCF\\uFFD2-\\uFFD7\\uFFDA-\\uFFDC]", "i");
        return function(ch) {
                return rx.test(ch);
        };
})();

//# sourceMappingURL=helpspot.js.map
